"""
Python model phase2.py
Translated using PySD version 0.8.2
"""
from __future__ import division
import numpy as np
from pysd.py_backend import utils
import xarray as xr

from pysd.py_backend.functions import cache
from pysd.py_backend import functions

equality_threshold = 1e-4

# functions names
_subscript_dict = {
    'bottom': ['layer4'],
    'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
    'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
    'hfc type': [
        'hfc134a', 'hfc23', 'hfc32', 'hfc125', 'hfc143a', 'hfc152a', 'hfc227ea', 'hfc245ca',
        'hfc4310mee'
    ],
    'households vehicles':
    ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels'],
    'layers': ['layer1', 'layer2', 'layer3', 'layer4'],
    'materials': [
        'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
        'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components',
        'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
        'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime',
        'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment',
        'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel',
        'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics',
        'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules',
        'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium',
        'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc'
    ],
    'rcp scenario': ['rcp26', 'rcp45', 'rcp60', 'rcp85'],
    'res elec': [
        'hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore',
        'wind offshore', 'solar pv', 'csp'
    ],
    'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat'],
    'scenarios': ['scen1', 'scen2', 'scen3', 'scen4', 'bau', 'user defined'],
    'sectors': [
        'agriculture hunting forestry and fishing', 'mining and quarrying',
        'food beverages and tobacco', 'textiles and textile products',
        'leather leather and footwear', 'wood and products of woood and cork',
        'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
        'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral',
        'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment',
        'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply',
        'construction',
        'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
        'wholesale trade and commissions trade except of motor vehicles and motorcycles',
        'retail trade except of motor vehicles and motorcycles repair of household goods',
        'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
        'other supporting and auxiliary transport activities activities of travel agencies',
        'post and telecommunications', 'financial intermedation', 'real estate activities',
        'renting od meq and other business activities',
        'public admin and defence compulsory social security', 'education',
        'health and social work', 'other community social and persona services',
        'private households with employed persons'
    ],
    'sectors1': [
        'agriculture hunting forestry and fishing', 'mining and quarrying',
        'food beverages and tobacco', 'textiles and textile products',
        'leather leather and footwear', 'wood and products of woood and cork',
        'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
        'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral',
        'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment',
        'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply',
        'construction',
        'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
        'wholesale trade and commissions trade except of motor vehicles and motorcycles',
        'retail trade except of motor vehicles and motorcycles repair of household goods',
        'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
        'other supporting and auxiliary transport activities activities of travel agencies',
        'post and telecommunications', 'financial intermedation', 'real estate activities',
        'renting od meq and other business activities',
        'public admin and defence compulsory social security', 'education',
        'health and social work', 'other community social and persona services',
        'private households with employed persons'
    ],
    'vehiclet': [
        'hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq',
        'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec'
    ],
    'water color': ['blue water', 'green water', 'gray water'],
    'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']
}

_namespace = {
    'TIME':
    'time',
    'Time':
    'time',
    'hadcrut4 temperature':
    'hadcrut4_temperature',
    'co2 mauna loa':
    'co2_mauna_loa',
    'giss nasa temperature':
    'giss_nasa_temperature',
    'other forcings history':
    'other_forcings_history',
    'mp rf total':
    'mp_rf_total',
    'other forcings rcp scenario subrcp26':
    'other_forcings_rcp_scenario_subrcp26',
    'other forcings rcp scenario subrcp45':
    'other_forcings_rcp_scenario_subrcp45',
    'other forcings rcp scenario subrcp60':
    'other_forcings_rcp_scenario_subrcp60',
    'other forcings rcp scenario subrcp85':
    'other_forcings_rcp_scenario_subrcp85',
    'global sf6 emissions rcp subrcp26':
    'global_sf6_emissions_rcp_subrcp26',
    'global sf6 emissions rcp subrcp45':
    'global_sf6_emissions_rcp_subrcp45',
    'global sf6 emissions rcp subrcp60':
    'global_sf6_emissions_rcp_subrcp60',
    'global sf6 emissions rcp subrcp85':
    'global_sf6_emissions_rcp_subrcp85',
    'global ch4 anthro emissions rcp subrcp26':
    'global_ch4_anthro_emissions_rcp_subrcp26',
    'global ch4 anthro emissions rcp subrcp45':
    'global_ch4_anthro_emissions_rcp_subrcp45',
    'global ch4 anthro emissions rcp subrcp60':
    'global_ch4_anthro_emissions_rcp_subrcp60',
    'global ch4 anthro emissions rcp subrcp85':
    'global_ch4_anthro_emissions_rcp_subrcp85',
    'global hfc emissions rcp x2x6 subhfc134a':
    'global_hfc_emissions_rcp_x2x6_subhfc134a',
    'global hfc emissions rcp x2x6 subhfc23':
    'global_hfc_emissions_rcp_x2x6_subhfc23',
    'global hfc emissions rcp x2x6 subhfc32':
    'global_hfc_emissions_rcp_x2x6_subhfc32',
    'global hfc emissions rcp x2x6 subhfc125':
    'global_hfc_emissions_rcp_x2x6_subhfc125',
    'global hfc emissions rcp x2x6 subhfc143a':
    'global_hfc_emissions_rcp_x2x6_subhfc143a',
    'global hfc emissions rcp x2x6 subhfc152a':
    'global_hfc_emissions_rcp_x2x6_subhfc152a',
    'global hfc emissions rcp x2x6 subhfc227ea':
    'global_hfc_emissions_rcp_x2x6_subhfc227ea',
    'global hfc emissions rcp x2x6 subhfc245ca':
    'global_hfc_emissions_rcp_x2x6_subhfc245ca',
    'global hfc emissions rcp x2x6 subhfc4310mee':
    'global_hfc_emissions_rcp_x2x6_subhfc4310mee',
    'global hfc emissions rcp x4x5 subhfc134a':
    'global_hfc_emissions_rcp_x4x5_subhfc134a',
    'global hfc emissions rcp x4x5 subhfc23':
    'global_hfc_emissions_rcp_x4x5_subhfc23',
    'global hfc emissions rcp x4x5 subhfc32':
    'global_hfc_emissions_rcp_x4x5_subhfc32',
    'global hfc emissions rcp x4x5 subhfc125':
    'global_hfc_emissions_rcp_x4x5_subhfc125',
    'global hfc emissions rcp x4x5 subhfc143a':
    'global_hfc_emissions_rcp_x4x5_subhfc143a',
    'global hfc emissions rcp x4x5 subhfc152a':
    'global_hfc_emissions_rcp_x4x5_subhfc152a',
    'global hfc emissions rcp x4x5 subhfc227ea':
    'global_hfc_emissions_rcp_x4x5_subhfc227ea',
    'global hfc emissions rcp x4x5 subhfc245ca':
    'global_hfc_emissions_rcp_x4x5_subhfc245ca',
    'global hfc emissions rcp x4x5 subhfc4310mee':
    'global_hfc_emissions_rcp_x4x5_subhfc4310mee',
    'global hfc emissions rcp x6x0 subhfc134a':
    'global_hfc_emissions_rcp_x6x0_subhfc134a',
    'global hfc emissions rcp x6x0 subhfc23':
    'global_hfc_emissions_rcp_x6x0_subhfc23',
    'global hfc emissions rcp x6x0 subhfc32':
    'global_hfc_emissions_rcp_x6x0_subhfc32',
    'global hfc emissions rcp x6x0 subhfc125':
    'global_hfc_emissions_rcp_x6x0_subhfc125',
    'global hfc emissions rcp x6x0 subhfc143a':
    'global_hfc_emissions_rcp_x6x0_subhfc143a',
    'global hfc emissions rcp x6x0 subhfc152a':
    'global_hfc_emissions_rcp_x6x0_subhfc152a',
    'global hfc emissions rcp x6x0 subhfc227ea':
    'global_hfc_emissions_rcp_x6x0_subhfc227ea',
    'global hfc emissions rcp x6x0 subhfc245ca':
    'global_hfc_emissions_rcp_x6x0_subhfc245ca',
    'global hfc emissions rcp x6x0 subhfc4310mee':
    'global_hfc_emissions_rcp_x6x0_subhfc4310mee',
    'global pfc emissions rcp subrcp26':
    'global_pfc_emissions_rcp_subrcp26',
    'global pfc emissions rcp subrcp45':
    'global_pfc_emissions_rcp_subrcp45',
    'global pfc emissions rcp subrcp60':
    'global_pfc_emissions_rcp_subrcp60',
    'global pfc emissions rcp subrcp85':
    'global_pfc_emissions_rcp_subrcp85',
    'global n2o anthro emissions rcp subrcp26':
    'global_n2o_anthro_emissions_rcp_subrcp26',
    'global n2o anthro emissions rcp subrcp45':
    'global_n2o_anthro_emissions_rcp_subrcp45',
    'global n2o anthro emissions rcp subrcp60':
    'global_n2o_anthro_emissions_rcp_subrcp60',
    'global n2o anthro emissions rcp subrcp85':
    'global_n2o_anthro_emissions_rcp_subrcp85',
    'global hfc emissions rcp x8x5 subhfc134a':
    'global_hfc_emissions_rcp_x8x5_subhfc134a',
    'global hfc emissions rcp x8x5 subhfc23':
    'global_hfc_emissions_rcp_x8x5_subhfc23',
    'global hfc emissions rcp x8x5 subhfc32':
    'global_hfc_emissions_rcp_x8x5_subhfc32',
    'global hfc emissions rcp x8x5 subhfc125':
    'global_hfc_emissions_rcp_x8x5_subhfc125',
    'global hfc emissions rcp x8x5 subhfc143a':
    'global_hfc_emissions_rcp_x8x5_subhfc143a',
    'global hfc emissions rcp x8x5 subhfc152a':
    'global_hfc_emissions_rcp_x8x5_subhfc152a',
    'global hfc emissions rcp x8x5 subhfc227ea':
    'global_hfc_emissions_rcp_x8x5_subhfc227ea',
    'global hfc emissions rcp x8x5 subhfc245ca':
    'global_hfc_emissions_rcp_x8x5_subhfc245ca',
    'global hfc emissions rcp x8x5 subhfc4310mee':
    'global_hfc_emissions_rcp_x8x5_subhfc4310mee',
    'co2x landxdashxuse change emissions exogenous':
    'co2x_landxdashxuse_change_emissions_exogenous',
    'historic water use by type sectors subagriculture hunting forestry and fishing subblue water':
    'historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subblue_water',
    'historic water use by type sectors subagriculture hunting forestry and fishing subgreen water':
    'historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subgreen_water',
    'historic water use by type sectors subagriculture hunting forestry and fishing subgray water':
    'historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subgray_water',
    'historic water use by type sectors submining and quarrying subblue water':
    'historic_water_use_by_type_sectors_submining_and_quarrying_subblue_water',
    'historic water use by type sectors submining and quarrying subgreen water':
    'historic_water_use_by_type_sectors_submining_and_quarrying_subgreen_water',
    'historic water use by type sectors submining and quarrying subgray water':
    'historic_water_use_by_type_sectors_submining_and_quarrying_subgray_water',
    'historic water use by type sectors subfood beverages and tobacco subblue water':
    'historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subblue_water',
    'historic water use by type sectors subfood beverages and tobacco subgreen water':
    'historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subgreen_water',
    'historic water use by type sectors subfood beverages and tobacco subgray water':
    'historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subgray_water',
    'historic water use by type sectors subtextiles and textile products subblue water':
    'historic_water_use_by_type_sectors_subtextiles_and_textile_products_subblue_water',
    'historic water use by type sectors subtextiles and textile products subgreen water':
    'historic_water_use_by_type_sectors_subtextiles_and_textile_products_subgreen_water',
    'historic water use by type sectors subtextiles and textile products subgray water':
    'historic_water_use_by_type_sectors_subtextiles_and_textile_products_subgray_water',
    'historic water use by type sectors subleather leather and footwear subblue water':
    'historic_water_use_by_type_sectors_subleather_leather_and_footwear_subblue_water',
    'historic water use by type sectors subleather leather and footwear subgreen water':
    'historic_water_use_by_type_sectors_subleather_leather_and_footwear_subgreen_water',
    'historic water use by type sectors subleather leather and footwear subgray water':
    'historic_water_use_by_type_sectors_subleather_leather_and_footwear_subgray_water',
    'historic water use by type sectors subwood and products of woood and cork subblue water':
    'historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subblue_water',
    'historic water use by type sectors subwood and products of woood and cork subgreen water':
    'historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subgreen_water',
    'historic water use by type sectors subwood and products of woood and cork subgray water':
    'historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subgray_water',
    'historic water use by type sectors subpulp paper printing and publishing subblue water':
    'historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subblue_water',
    'historic water use by type sectors subpulp paper printing and publishing subgreen water':
    'historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subgreen_water',
    'historic water use by type sectors subpulp paper printing and publishing subgray water':
    'historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subgray_water',
    'historic water use by type sectors subcoke refined petroleum and nuclear fuel subblue water':
    'historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subblue_water',
    'historic water use by type sectors subcoke refined petroleum and nuclear fuel subgreen water':
    'historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subgreen_water',
    'historic water use by type sectors subcoke refined petroleum and nuclear fuel subgray water':
    'historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subgray_water',
    'historic water use by type sectors subchemicals and chemical products subblue water':
    'historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subblue_water',
    'historic water use by type sectors subchemicals and chemical products subgreen water':
    'historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subgreen_water',
    'historic water use by type sectors subchemicals and chemical products subgray water':
    'historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subgray_water',
    'historic water use by type sectors subrubber and plastics subblue water':
    'historic_water_use_by_type_sectors_subrubber_and_plastics_subblue_water',
    'historic water use by type sectors subrubber and plastics subgreen water':
    'historic_water_use_by_type_sectors_subrubber_and_plastics_subgreen_water',
    'historic water use by type sectors subrubber and plastics subgray water':
    'historic_water_use_by_type_sectors_subrubber_and_plastics_subgray_water',
    'historic water use by type sectors subother non metalic mineral subblue water':
    'historic_water_use_by_type_sectors_subother_non_metalic_mineral_subblue_water',
    'historic water use by type sectors subother non metalic mineral subgreen water':
    'historic_water_use_by_type_sectors_subother_non_metalic_mineral_subgreen_water',
    'historic water use by type sectors subother non metalic mineral subgray water':
    'historic_water_use_by_type_sectors_subother_non_metalic_mineral_subgray_water',
    'historic water use by type sectors subbasic metals and fabricated metal subblue water':
    'historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subblue_water',
    'historic water use by type sectors subbasic metals and fabricated metal subgreen water':
    'historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subgreen_water',
    'historic water use by type sectors subbasic metals and fabricated metal subgray water':
    'historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subgray_water',
    'historic water use by type sectors submachinery nec subblue water':
    'historic_water_use_by_type_sectors_submachinery_nec_subblue_water',
    'historic water use by type sectors submachinery nec subgreen water':
    'historic_water_use_by_type_sectors_submachinery_nec_subgreen_water',
    'historic water use by type sectors submachinery nec subgray water':
    'historic_water_use_by_type_sectors_submachinery_nec_subgray_water',
    'historic water use by type sectors subelectrical and optical equipment subblue water':
    'historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subblue_water',
    'historic water use by type sectors subelectrical and optical equipment subgreen water':
    'historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subgreen_water',
    'historic water use by type sectors subelectrical and optical equipment subgray water':
    'historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subgray_water',
    'historic water use by type sectors subtransport equipment subblue water':
    'historic_water_use_by_type_sectors_subtransport_equipment_subblue_water',
    'historic water use by type sectors subtransport equipment subgreen water':
    'historic_water_use_by_type_sectors_subtransport_equipment_subgreen_water',
    'historic water use by type sectors subtransport equipment subgray water':
    'historic_water_use_by_type_sectors_subtransport_equipment_subgray_water',
    'historic water use by type sectors submanufacturing nec recycling subblue water':
    'historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subblue_water',
    'historic water use by type sectors submanufacturing nec recycling subgreen water':
    'historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subgreen_water',
    'historic water use by type sectors submanufacturing nec recycling subgray water':
    'historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subgray_water',
    'historic water use by type sectors subelectricity gas and water supply subblue water':
    'historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subblue_water',
    'historic water use by type sectors subelectricity gas and water supply subgreen water':
    'historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subgreen_water',
    'historic water use by type sectors subelectricity gas and water supply subgray water':
    'historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subgray_water',
    'historic water use by type sectors subconstruction subblue water':
    'historic_water_use_by_type_sectors_subconstruction_subblue_water',
    'historic water use by type sectors subconstruction subgreen water':
    'historic_water_use_by_type_sectors_subconstruction_subgreen_water',
    'historic water use by type sectors subconstruction subgray water':
    'historic_water_use_by_type_sectors_subconstruction_subgray_water',
    'historic water use by type sectors subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel subblue water':
    'historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subblue_water',
    'historic water use by type sectors subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel subgreen water':
    'historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subgreen_water',
    'historic water use by type sectors subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel subgray water':
    'historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subgray_water',
    'historic water use by type sectors subwholesale trade and commissions trade except of motor vehicles and motorcycles subblue water':
    'historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subblue_water',
    'historic water use by type sectors subwholesale trade and commissions trade except of motor vehicles and motorcycles subgreen water':
    'historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subgreen_water',
    'historic water use by type sectors subwholesale trade and commissions trade except of motor vehicles and motorcycles subgray water':
    'historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subgray_water',
    'historic water use by type sectors subretail trade except of motor vehicles and motorcycles repair of household goods subblue water':
    'historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subblue_water',
    'historic water use by type sectors subretail trade except of motor vehicles and motorcycles repair of household goods subgreen water':
    'historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subgreen_water',
    'historic water use by type sectors subretail trade except of motor vehicles and motorcycles repair of household goods subgray water':
    'historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subgray_water',
    'historic water use by type sectors subhotels and restaurants subblue water':
    'historic_water_use_by_type_sectors_subhotels_and_restaurants_subblue_water',
    'historic water use by type sectors subhotels and restaurants subgreen water':
    'historic_water_use_by_type_sectors_subhotels_and_restaurants_subgreen_water',
    'historic water use by type sectors subhotels and restaurants subgray water':
    'historic_water_use_by_type_sectors_subhotels_and_restaurants_subgray_water',
    'historic water use by type sectors subinland transport subblue water':
    'historic_water_use_by_type_sectors_subinland_transport_subblue_water',
    'historic water use by type sectors subinland transport subgreen water':
    'historic_water_use_by_type_sectors_subinland_transport_subgreen_water',
    'historic water use by type sectors subinland transport subgray water':
    'historic_water_use_by_type_sectors_subinland_transport_subgray_water',
    'historic water use by type sectors subwater transport subblue water':
    'historic_water_use_by_type_sectors_subwater_transport_subblue_water',
    'historic water use by type sectors subwater transport subgreen water':
    'historic_water_use_by_type_sectors_subwater_transport_subgreen_water',
    'historic water use by type sectors subwater transport subgray water':
    'historic_water_use_by_type_sectors_subwater_transport_subgray_water',
    'historic water use by type sectors subair transport subblue water':
    'historic_water_use_by_type_sectors_subair_transport_subblue_water',
    'historic water use by type sectors subair transport subgreen water':
    'historic_water_use_by_type_sectors_subair_transport_subgreen_water',
    'historic water use by type sectors subair transport subgray water':
    'historic_water_use_by_type_sectors_subair_transport_subgray_water',
    'historic water use by type sectors subother supporting and auxiliary transport activities activities of travel agencies subblue water':
    'historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subblue_water',
    'historic water use by type sectors subother supporting and auxiliary transport activities activities of travel agencies subgreen water':
    'historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subgreen_water',
    'historic water use by type sectors subother supporting and auxiliary transport activities activities of travel agencies subgray water':
    'historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subgray_water',
    'historic water use by type sectors subpost and telecommunications subblue water':
    'historic_water_use_by_type_sectors_subpost_and_telecommunications_subblue_water',
    'historic water use by type sectors subpost and telecommunications subgreen water':
    'historic_water_use_by_type_sectors_subpost_and_telecommunications_subgreen_water',
    'historic water use by type sectors subpost and telecommunications subgray water':
    'historic_water_use_by_type_sectors_subpost_and_telecommunications_subgray_water',
    'historic water use by type sectors subfinancial intermedation subblue water':
    'historic_water_use_by_type_sectors_subfinancial_intermedation_subblue_water',
    'historic water use by type sectors subfinancial intermedation subgreen water':
    'historic_water_use_by_type_sectors_subfinancial_intermedation_subgreen_water',
    'historic water use by type sectors subfinancial intermedation subgray water':
    'historic_water_use_by_type_sectors_subfinancial_intermedation_subgray_water',
    'historic water use by type sectors subreal estate activities subblue water':
    'historic_water_use_by_type_sectors_subreal_estate_activities_subblue_water',
    'historic water use by type sectors subreal estate activities subgreen water':
    'historic_water_use_by_type_sectors_subreal_estate_activities_subgreen_water',
    'historic water use by type sectors subreal estate activities subgray water':
    'historic_water_use_by_type_sectors_subreal_estate_activities_subgray_water',
    'historic water use by type sectors subrenting od meq and other business activities subblue water':
    'historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subblue_water',
    'historic water use by type sectors subrenting od meq and other business activities subgreen water':
    'historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subgreen_water',
    'historic water use by type sectors subrenting od meq and other business activities subgray water':
    'historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subgray_water',
    'historic water use by type sectors subpublic admin and defence compulsory social security subblue water':
    'historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subblue_water',
    'historic water use by type sectors subpublic admin and defence compulsory social security subgreen water':
    'historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subgreen_water',
    'historic water use by type sectors subpublic admin and defence compulsory social security subgray water':
    'historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subgray_water',
    'historic water use by type sectors subeducation subblue water':
    'historic_water_use_by_type_sectors_subeducation_subblue_water',
    'historic water use by type sectors subeducation subgreen water':
    'historic_water_use_by_type_sectors_subeducation_subgreen_water',
    'historic water use by type sectors subeducation subgray water':
    'historic_water_use_by_type_sectors_subeducation_subgray_water',
    'historic water use by type sectors subhealth and social work subblue water':
    'historic_water_use_by_type_sectors_subhealth_and_social_work_subblue_water',
    'historic water use by type sectors subhealth and social work subgreen water':
    'historic_water_use_by_type_sectors_subhealth_and_social_work_subgreen_water',
    'historic water use by type sectors subhealth and social work subgray water':
    'historic_water_use_by_type_sectors_subhealth_and_social_work_subgray_water',
    'historic water use by type sectors subother community social and persona services subblue water':
    'historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subblue_water',
    'historic water use by type sectors subother community social and persona services subgreen water':
    'historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subgreen_water',
    'historic water use by type sectors subother community social and persona services subgray water':
    'historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subgray_water',
    'historic water use by type sectors subprivate households with employed persons subblue water':
    'historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subblue_water',
    'historic water use by type sectors subprivate households with employed persons subgreen water':
    'historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subgreen_water',
    'historic water use by type sectors subprivate households with employed persons subgray water':
    'historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subgray_water',
    'historic water use by type for households subblue water':
    'historic_water_use_by_type_for_households_subblue_water',
    'historic water use by type for households subgreen water':
    'historic_water_use_by_type_for_households_subgreen_water',
    'historic water use by type for households subgray water':
    'historic_water_use_by_type_for_households_subgray_water',
    'gdppc annual growth ssp2':
    'gdppc_annual_growth_ssp2',
    'pop ssp2':
    'pop_ssp2',
    'pop ssp4':
    'pop_ssp4',
    'pop ssp1':
    'pop_ssp1',
    'pop ssp5':
    'pop_ssp5',
    'pop ssp3':
    'pop_ssp3',
    'p timeseries gdppc growth rate':
    'p_timeseries_gdppc_growth_rate',
    'historic gdppc':
    'historic_gdppc',
    'total ghg emissions olt medeas d3x2':
    'total_ghg_emissions_olt_medeas_d3x2',
    'total ghg emissions bauxdashxcat medeas d3x2':
    'total_ghg_emissions_bauxdashxcat_medeas_d3x2',
    'total ghg emissions mlt2030x medeas d3x2':
    'total_ghg_emissions_mlt2030x_medeas_d3x2',
    'total ghg emissions mlt2020x medeas d3x2':
    'total_ghg_emissions_mlt2020x_medeas_d3x2',
    'table hist capacity phs':
    'table_hist_capacity_phs',
    'table max extraction aspo oil ej x0x x1x x0x':
    'table_max_extraction_aspo_oil_ej_x0x_x1x_x0x',
    'table max extraction aspo oil ej x0x x1x':
    'table_max_extraction_aspo_oil_ej_x0x_x1x',
    'table max extraction aspo oil ej x0x':
    'table_max_extraction_aspo_oil_ej_x0x',
    'table max extraction aspo oil ej x0x x0x':
    'table_max_extraction_aspo_oil_ej_x0x_x0x',
    'historical extraction minerals rest subadhesive':
    'historical_extraction_minerals_rest_subadhesive',
    'historical extraction minerals rest subaluminium':
    'historical_extraction_minerals_rest_subaluminium',
    'historical extraction minerals rest subaluminium mirrors':
    'historical_extraction_minerals_rest_subaluminium_mirrors',
    'historical extraction minerals rest subcadmium':
    'historical_extraction_minerals_rest_subcadmium',
    'historical extraction minerals rest subcarbon fiber':
    'historical_extraction_minerals_rest_subcarbon_fiber',
    'historical extraction minerals rest subcement':
    'historical_extraction_minerals_rest_subcement',
    'historical extraction minerals rest subchromium':
    'historical_extraction_minerals_rest_subchromium',
    'historical extraction minerals rest subcopper':
    'historical_extraction_minerals_rest_subcopper',
    'historical extraction minerals rest subdiesel':
    'historical_extraction_minerals_rest_subdiesel',
    'historical extraction minerals rest subdy':
    'historical_extraction_minerals_rest_subdy',
    'historical extraction minerals rest subelectricxdivxelectronic components':
    'historical_extraction_minerals_rest_subelectricxdivxelectronic_components',
    'historical extraction minerals rest subevacuation lines':
    'historical_extraction_minerals_rest_subevacuation_lines',
    'historical extraction minerals rest subfiberglass':
    'historical_extraction_minerals_rest_subfiberglass',
    'historical extraction minerals rest subfoam glass':
    'historical_extraction_minerals_rest_subfoam_glass',
    'historical extraction minerals rest subgalium':
    'historical_extraction_minerals_rest_subgalium',
    'historical extraction minerals rest subglass':
    'historical_extraction_minerals_rest_subglass',
    'historical extraction minerals rest subglass reinforcing plastic':
    'historical_extraction_minerals_rest_subglass_reinforcing_plastic',
    'historical extraction minerals rest subgravel':
    'historical_extraction_minerals_rest_subgravel',
    'historical extraction minerals rest subindium':
    'historical_extraction_minerals_rest_subindium',
    'historical extraction minerals rest subiron':
    'historical_extraction_minerals_rest_subiron',
    'historical extraction minerals rest subkno3 mined':
    'historical_extraction_minerals_rest_subkno3_mined',
    'historical extraction minerals rest subasphalt':
    'historical_extraction_minerals_rest_subasphalt',
    'historical extraction minerals rest sublime':
    'historical_extraction_minerals_rest_sublime',
    'historical extraction minerals rest sublimestone':
    'historical_extraction_minerals_rest_sublimestone',
    'historical extraction minerals rest sublithium':
    'historical_extraction_minerals_rest_sublithium',
    'historical extraction minerals rest sublubricant':
    'historical_extraction_minerals_rest_sublubricant',
    'historical extraction minerals rest submagnesium':
    'historical_extraction_minerals_rest_submagnesium',
    'historical extraction minerals rest submanganese':
    'historical_extraction_minerals_rest_submanganese',
    'historical extraction minerals rest subheavy equipment':
    'historical_extraction_minerals_rest_subheavy_equipment',
    'historical extraction minerals rest subconcrete':
    'historical_extraction_minerals_rest_subconcrete',
    'historical extraction minerals rest submolybdenum':
    'historical_extraction_minerals_rest_submolybdenum',
    'historical extraction minerals rest subnano3 mined':
    'historical_extraction_minerals_rest_subnano3_mined',
    'historical extraction minerals rest subnano3 synthetic':
    'historical_extraction_minerals_rest_subnano3_synthetic',
    'historical extraction minerals rest subneodymium':
    'historical_extraction_minerals_rest_subneodymium',
    'historical extraction minerals rest subnickel':
    'historical_extraction_minerals_rest_subnickel',
    'historical extraction minerals rest subover grid x15xpercxx':
    'historical_extraction_minerals_rest_subover_grid_x15xpercxx',
    'historical extraction minerals rest subover grid x5xpercxx':
    'historical_extraction_minerals_rest_subover_grid_x5xpercxx',
    'historical extraction minerals rest subpaint':
    'historical_extraction_minerals_rest_subpaint',
    'historical extraction minerals rest sublead':
    'historical_extraction_minerals_rest_sublead',
    'historical extraction minerals rest subplastics':
    'historical_extraction_minerals_rest_subplastics',
    'historical extraction minerals rest subpolypropylene':
    'historical_extraction_minerals_rest_subpolypropylene',
    'historical extraction minerals rest subrock':
    'historical_extraction_minerals_rest_subrock',
    'historical extraction minerals rest subrock wool':
    'historical_extraction_minerals_rest_subrock_wool',
    'historical extraction minerals rest subsand':
    'historical_extraction_minerals_rest_subsand',
    'historical extraction minerals rest subsilicon sand':
    'historical_extraction_minerals_rest_subsilicon_sand',
    'historical extraction minerals rest subsilicon wafer modules':
    'historical_extraction_minerals_rest_subsilicon_wafer_modules',
    'historical extraction minerals rest subsilver':
    'historical_extraction_minerals_rest_subsilver',
    'historical extraction minerals rest subsite preparation':
    'historical_extraction_minerals_rest_subsite_preparation',
    'historical extraction minerals rest subtin':
    'historical_extraction_minerals_rest_subtin',
    'historical extraction minerals rest subsoda ash':
    'historical_extraction_minerals_rest_subsoda_ash',
    'historical extraction minerals rest substeel':
    'historical_extraction_minerals_rest_substeel',
    'historical extraction minerals rest subsynthetic oil':
    'historical_extraction_minerals_rest_subsynthetic_oil',
    'historical extraction minerals rest subtellurium':
    'historical_extraction_minerals_rest_subtellurium',
    'historical extraction minerals rest subtitanium':
    'historical_extraction_minerals_rest_subtitanium',
    'historical extraction minerals rest subtitanium dioxide':
    'historical_extraction_minerals_rest_subtitanium_dioxide',
    'historical extraction minerals rest subvanadium':
    'historical_extraction_minerals_rest_subvanadium',
    'historical extraction minerals rest subwires':
    'historical_extraction_minerals_rest_subwires',
    'historical extraction minerals rest subzinc':
    'historical_extraction_minerals_rest_subzinc',
    'start production biofuels':
    'start_production_biofuels',
    'historic res capacity for heatxdashxnc subsolarxdashxheat':
    'historic_res_capacity_for_heatxdashxnc_subsolarxdashxheat',
    'historic res capacity for heatxdashxnc subgeotxdashxheat':
    'historic_res_capacity_for_heatxdashxnc_subgeotxdashxheat',
    'historic res capacity for heatxdashxnc subsolid bioexdashxheat':
    'historic_res_capacity_for_heatxdashxnc_subsolid_bioexdashxheat',
    'historic share of transformation losses vs extraction subliquids':
    'historic_share_of_transformation_losses_vs_extraction_subliquids',
    'historic share of transformation losses vs extraction subsolids':
    'historic_share_of_transformation_losses_vs_extraction_subsolids',
    'ratio gain gas vs lose solids in tranf processes':
    'ratio_gain_gas_vs_lose_solids_in_tranf_processes',
    'historic share of losses vs extraction subliquids':
    'historic_share_of_losses_vs_extraction_subliquids',
    'historic share of losses vs extraction subsolids':
    'historic_share_of_losses_vs_extraction_subsolids',
    'historic share of losses vs extraction subgases':
    'historic_share_of_losses_vs_extraction_subgases',
    'historic demand by sector subagriculture hunting forestry and fishing':
    'historic_demand_by_sector_subagriculture_hunting_forestry_and_fishing',
    'historic demand by sector submining and quarrying':
    'historic_demand_by_sector_submining_and_quarrying',
    'historic demand by sector subfood beverages and tobacco':
    'historic_demand_by_sector_subfood_beverages_and_tobacco',
    'historic demand by sector subtextiles and textile products':
    'historic_demand_by_sector_subtextiles_and_textile_products',
    'historic demand by sector subleather leather and footwear':
    'historic_demand_by_sector_subleather_leather_and_footwear',
    'historic demand by sector subwood and products of woood and cork':
    'historic_demand_by_sector_subwood_and_products_of_woood_and_cork',
    'historic demand by sector subpulp paper printing and publishing':
    'historic_demand_by_sector_subpulp_paper_printing_and_publishing',
    'historic demand by sector subcoke refined petroleum and nuclear fuel':
    'historic_demand_by_sector_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic demand by sector subchemicals and chemical products':
    'historic_demand_by_sector_subchemicals_and_chemical_products',
    'historic demand by sector subrubber and plastics':
    'historic_demand_by_sector_subrubber_and_plastics',
    'historic demand by sector subother non metalic mineral':
    'historic_demand_by_sector_subother_non_metalic_mineral',
    'historic demand by sector subbasic metals and fabricated metal':
    'historic_demand_by_sector_subbasic_metals_and_fabricated_metal',
    'historic demand by sector submachinery nec':
    'historic_demand_by_sector_submachinery_nec',
    'historic demand by sector subelectrical and optical equipment':
    'historic_demand_by_sector_subelectrical_and_optical_equipment',
    'historic demand by sector subtransport equipment':
    'historic_demand_by_sector_subtransport_equipment',
    'historic demand by sector submanufacturing nec recycling':
    'historic_demand_by_sector_submanufacturing_nec_recycling',
    'historic demand by sector subelectricity gas and water supply':
    'historic_demand_by_sector_subelectricity_gas_and_water_supply',
    'historic demand by sector subconstruction':
    'historic_demand_by_sector_subconstruction',
    'historic demand by sector subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_demand_by_sector_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic demand by sector subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_demand_by_sector_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic demand by sector subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_demand_by_sector_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic demand by sector subhotels and restaurants':
    'historic_demand_by_sector_subhotels_and_restaurants',
    'historic demand by sector subinland transport':
    'historic_demand_by_sector_subinland_transport',
    'historic demand by sector subwater transport':
    'historic_demand_by_sector_subwater_transport',
    'historic demand by sector subair transport':
    'historic_demand_by_sector_subair_transport',
    'historic demand by sector subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_demand_by_sector_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic demand by sector subpost and telecommunications':
    'historic_demand_by_sector_subpost_and_telecommunications',
    'historic demand by sector subfinancial intermedation':
    'historic_demand_by_sector_subfinancial_intermedation',
    'historic demand by sector subreal estate activities':
    'historic_demand_by_sector_subreal_estate_activities',
    'historic demand by sector subrenting od meq and other business activities':
    'historic_demand_by_sector_subrenting_od_meq_and_other_business_activities',
    'historic demand by sector subpublic admin and defence compulsory social security':
    'historic_demand_by_sector_subpublic_admin_and_defence_compulsory_social_security',
    'historic demand by sector subeducation':
    'historic_demand_by_sector_subeducation',
    'historic demand by sector subhealth and social work':
    'historic_demand_by_sector_subhealth_and_social_work',
    'historic demand by sector subother community social and persona services':
    'historic_demand_by_sector_subother_community_social_and_persona_services',
    'historic demand by sector subprivate households with employed persons':
    'historic_demand_by_sector_subprivate_households_with_employed_persons',
    'historic final energy intensity h subelectricity':
    'historic_final_energy_intensity_h_subelectricity',
    'historic final energy intensity h subheat':
    'historic_final_energy_intensity_h_subheat',
    'historic final energy intensity h subliquids':
    'historic_final_energy_intensity_h_subliquids',
    'historic final energy intensity h subsolids':
    'historic_final_energy_intensity_h_subsolids',
    'historic final energy intensity h subgases':
    'historic_final_energy_intensity_h_subgases',
    'historic res capacity for heatxdashxcom subsolarxdashxheat':
    'historic_res_capacity_for_heatxdashxcom_subsolarxdashxheat',
    'historic res capacity for heatxdashxcom subgeotxdashxheat':
    'historic_res_capacity_for_heatxdashxcom_subgeotxdashxheat',
    'historic res capacity for heatxdashxcom subsolid bioexdashxheat':
    'historic_res_capacity_for_heatxdashxcom_subsolid_bioexdashxheat',
    'share inventories next step subagriculture hunting forestry and fishing':
    'share_inventories_next_step_subagriculture_hunting_forestry_and_fishing',
    'share inventories next step submining and quarrying':
    'share_inventories_next_step_submining_and_quarrying',
    'share inventories next step subfood beverages and tobacco':
    'share_inventories_next_step_subfood_beverages_and_tobacco',
    'share inventories next step subtextiles and textile products':
    'share_inventories_next_step_subtextiles_and_textile_products',
    'share inventories next step subleather leather and footwear':
    'share_inventories_next_step_subleather_leather_and_footwear',
    'share inventories next step subwood and products of woood and cork':
    'share_inventories_next_step_subwood_and_products_of_woood_and_cork',
    'share inventories next step subpulp paper printing and publishing':
    'share_inventories_next_step_subpulp_paper_printing_and_publishing',
    'share inventories next step subcoke refined petroleum and nuclear fuel':
    'share_inventories_next_step_subcoke_refined_petroleum_and_nuclear_fuel',
    'share inventories next step subchemicals and chemical products':
    'share_inventories_next_step_subchemicals_and_chemical_products',
    'share inventories next step subrubber and plastics':
    'share_inventories_next_step_subrubber_and_plastics',
    'share inventories next step subother non metalic mineral':
    'share_inventories_next_step_subother_non_metalic_mineral',
    'share inventories next step subbasic metals and fabricated metal':
    'share_inventories_next_step_subbasic_metals_and_fabricated_metal',
    'share inventories next step submachinery nec':
    'share_inventories_next_step_submachinery_nec',
    'share inventories next step subelectrical and optical equipment':
    'share_inventories_next_step_subelectrical_and_optical_equipment',
    'share inventories next step subtransport equipment':
    'share_inventories_next_step_subtransport_equipment',
    'share inventories next step submanufacturing nec recycling':
    'share_inventories_next_step_submanufacturing_nec_recycling',
    'share inventories next step subelectricity gas and water supply':
    'share_inventories_next_step_subelectricity_gas_and_water_supply',
    'share inventories next step subconstruction':
    'share_inventories_next_step_subconstruction',
    'share inventories next step subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'share_inventories_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'share inventories next step subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'share_inventories_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'share inventories next step subretail trade except of motor vehicles and motorcycles repair of household goods':
    'share_inventories_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'share inventories next step subhotels and restaurants':
    'share_inventories_next_step_subhotels_and_restaurants',
    'share inventories next step subinland transport':
    'share_inventories_next_step_subinland_transport',
    'share inventories next step subwater transport':
    'share_inventories_next_step_subwater_transport',
    'share inventories next step subair transport':
    'share_inventories_next_step_subair_transport',
    'share inventories next step subother supporting and auxiliary transport activities activities of travel agencies':
    'share_inventories_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'share inventories next step subpost and telecommunications':
    'share_inventories_next_step_subpost_and_telecommunications',
    'share inventories next step subfinancial intermedation':
    'share_inventories_next_step_subfinancial_intermedation',
    'share inventories next step subreal estate activities':
    'share_inventories_next_step_subreal_estate_activities',
    'share inventories next step subrenting od meq and other business activities':
    'share_inventories_next_step_subrenting_od_meq_and_other_business_activities',
    'share inventories next step subpublic admin and defence compulsory social security':
    'share_inventories_next_step_subpublic_admin_and_defence_compulsory_social_security',
    'share inventories next step subeducation':
    'share_inventories_next_step_subeducation',
    'share inventories next step subhealth and social work':
    'share_inventories_next_step_subhealth_and_social_work',
    'share inventories next step subother community social and persona services':
    'share_inventories_next_step_subother_community_social_and_persona_services',
    'share inventories next step subprivate households with employed persons':
    'share_inventories_next_step_subprivate_households_with_employed_persons',
    'share consum goverments subagriculture hunting forestry and fishing':
    'share_consum_goverments_subagriculture_hunting_forestry_and_fishing',
    'share consum goverments submining and quarrying':
    'share_consum_goverments_submining_and_quarrying',
    'share consum goverments subfood beverages and tobacco':
    'share_consum_goverments_subfood_beverages_and_tobacco',
    'share consum goverments subtextiles and textile products':
    'share_consum_goverments_subtextiles_and_textile_products',
    'share consum goverments subleather leather and footwear':
    'share_consum_goverments_subleather_leather_and_footwear',
    'share consum goverments subwood and products of woood and cork':
    'share_consum_goverments_subwood_and_products_of_woood_and_cork',
    'share consum goverments subpulp paper printing and publishing':
    'share_consum_goverments_subpulp_paper_printing_and_publishing',
    'share consum goverments subcoke refined petroleum and nuclear fuel':
    'share_consum_goverments_subcoke_refined_petroleum_and_nuclear_fuel',
    'share consum goverments subchemicals and chemical products':
    'share_consum_goverments_subchemicals_and_chemical_products',
    'share consum goverments subrubber and plastics':
    'share_consum_goverments_subrubber_and_plastics',
    'share consum goverments subother non metalic mineral':
    'share_consum_goverments_subother_non_metalic_mineral',
    'share consum goverments subbasic metals and fabricated metal':
    'share_consum_goverments_subbasic_metals_and_fabricated_metal',
    'share consum goverments submachinery nec':
    'share_consum_goverments_submachinery_nec',
    'share consum goverments subelectrical and optical equipment':
    'share_consum_goverments_subelectrical_and_optical_equipment',
    'share consum goverments subtransport equipment':
    'share_consum_goverments_subtransport_equipment',
    'share consum goverments submanufacturing nec recycling':
    'share_consum_goverments_submanufacturing_nec_recycling',
    'share consum goverments subelectricity gas and water supply':
    'share_consum_goverments_subelectricity_gas_and_water_supply',
    'share consum goverments subconstruction':
    'share_consum_goverments_subconstruction',
    'share consum goverments subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'share_consum_goverments_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'share consum goverments subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'share_consum_goverments_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'share consum goverments subretail trade except of motor vehicles and motorcycles repair of household goods':
    'share_consum_goverments_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'share consum goverments subhotels and restaurants':
    'share_consum_goverments_subhotels_and_restaurants',
    'share consum goverments subinland transport':
    'share_consum_goverments_subinland_transport',
    'share consum goverments subwater transport':
    'share_consum_goverments_subwater_transport',
    'share consum goverments subair transport':
    'share_consum_goverments_subair_transport',
    'share consum goverments subother supporting and auxiliary transport activities activities of travel agencies':
    'share_consum_goverments_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'share consum goverments subpost and telecommunications':
    'share_consum_goverments_subpost_and_telecommunications',
    'share consum goverments subfinancial intermedation':
    'share_consum_goverments_subfinancial_intermedation',
    'share consum goverments subreal estate activities':
    'share_consum_goverments_subreal_estate_activities',
    'share consum goverments subrenting od meq and other business activities':
    'share_consum_goverments_subrenting_od_meq_and_other_business_activities',
    'share consum goverments subpublic admin and defence compulsory social security':
    'share_consum_goverments_subpublic_admin_and_defence_compulsory_social_security',
    'share consum goverments subeducation':
    'share_consum_goverments_subeducation',
    'share consum goverments subhealth and social work':
    'share_consum_goverments_subhealth_and_social_work',
    'share consum goverments subother community social and persona services':
    'share_consum_goverments_subother_community_social_and_persona_services',
    'share consum goverments subprivate households with employed persons':
    'share_consum_goverments_subprivate_households_with_employed_persons',
    'share inventories subagriculture hunting forestry and fishing':
    'share_inventories_subagriculture_hunting_forestry_and_fishing',
    'share inventories submining and quarrying':
    'share_inventories_submining_and_quarrying',
    'share inventories subfood beverages and tobacco':
    'share_inventories_subfood_beverages_and_tobacco',
    'share inventories subtextiles and textile products':
    'share_inventories_subtextiles_and_textile_products',
    'share inventories subleather leather and footwear':
    'share_inventories_subleather_leather_and_footwear',
    'share inventories subwood and products of woood and cork':
    'share_inventories_subwood_and_products_of_woood_and_cork',
    'share inventories subpulp paper printing and publishing':
    'share_inventories_subpulp_paper_printing_and_publishing',
    'share inventories subcoke refined petroleum and nuclear fuel':
    'share_inventories_subcoke_refined_petroleum_and_nuclear_fuel',
    'share inventories subchemicals and chemical products':
    'share_inventories_subchemicals_and_chemical_products',
    'share inventories subrubber and plastics':
    'share_inventories_subrubber_and_plastics',
    'share inventories subother non metalic mineral':
    'share_inventories_subother_non_metalic_mineral',
    'share inventories subbasic metals and fabricated metal':
    'share_inventories_subbasic_metals_and_fabricated_metal',
    'share inventories submachinery nec':
    'share_inventories_submachinery_nec',
    'share inventories subelectrical and optical equipment':
    'share_inventories_subelectrical_and_optical_equipment',
    'share inventories subtransport equipment':
    'share_inventories_subtransport_equipment',
    'share inventories submanufacturing nec recycling':
    'share_inventories_submanufacturing_nec_recycling',
    'share inventories subelectricity gas and water supply':
    'share_inventories_subelectricity_gas_and_water_supply',
    'share inventories subconstruction':
    'share_inventories_subconstruction',
    'share inventories subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'share_inventories_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'share inventories subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'share_inventories_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'share inventories subretail trade except of motor vehicles and motorcycles repair of household goods':
    'share_inventories_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'share inventories subhotels and restaurants':
    'share_inventories_subhotels_and_restaurants',
    'share inventories subinland transport':
    'share_inventories_subinland_transport',
    'share inventories subwater transport':
    'share_inventories_subwater_transport',
    'share inventories subair transport':
    'share_inventories_subair_transport',
    'share inventories subother supporting and auxiliary transport activities activities of travel agencies':
    'share_inventories_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'share inventories subpost and telecommunications':
    'share_inventories_subpost_and_telecommunications',
    'share inventories subfinancial intermedation':
    'share_inventories_subfinancial_intermedation',
    'share inventories subreal estate activities':
    'share_inventories_subreal_estate_activities',
    'share inventories subrenting od meq and other business activities':
    'share_inventories_subrenting_od_meq_and_other_business_activities',
    'share inventories subpublic admin and defence compulsory social security':
    'share_inventories_subpublic_admin_and_defence_compulsory_social_security',
    'share inventories subeducation':
    'share_inventories_subeducation',
    'share inventories subhealth and social work':
    'share_inventories_subhealth_and_social_work',
    'share inventories subother community social and persona services':
    'share_inventories_subother_community_social_and_persona_services',
    'share inventories subprivate households with employed persons':
    'share_inventories_subprivate_households_with_employed_persons',
    'share consum goverments next step subagriculture hunting forestry and fishing':
    'share_consum_goverments_next_step_subagriculture_hunting_forestry_and_fishing',
    'share consum goverments next step submining and quarrying':
    'share_consum_goverments_next_step_submining_and_quarrying',
    'share consum goverments next step subfood beverages and tobacco':
    'share_consum_goverments_next_step_subfood_beverages_and_tobacco',
    'share consum goverments next step subtextiles and textile products':
    'share_consum_goverments_next_step_subtextiles_and_textile_products',
    'share consum goverments next step subleather leather and footwear':
    'share_consum_goverments_next_step_subleather_leather_and_footwear',
    'share consum goverments next step subwood and products of woood and cork':
    'share_consum_goverments_next_step_subwood_and_products_of_woood_and_cork',
    'share consum goverments next step subpulp paper printing and publishing':
    'share_consum_goverments_next_step_subpulp_paper_printing_and_publishing',
    'share consum goverments next step subcoke refined petroleum and nuclear fuel':
    'share_consum_goverments_next_step_subcoke_refined_petroleum_and_nuclear_fuel',
    'share consum goverments next step subchemicals and chemical products':
    'share_consum_goverments_next_step_subchemicals_and_chemical_products',
    'share consum goverments next step subrubber and plastics':
    'share_consum_goverments_next_step_subrubber_and_plastics',
    'share consum goverments next step subother non metalic mineral':
    'share_consum_goverments_next_step_subother_non_metalic_mineral',
    'share consum goverments next step subbasic metals and fabricated metal':
    'share_consum_goverments_next_step_subbasic_metals_and_fabricated_metal',
    'share consum goverments next step submachinery nec':
    'share_consum_goverments_next_step_submachinery_nec',
    'share consum goverments next step subelectrical and optical equipment':
    'share_consum_goverments_next_step_subelectrical_and_optical_equipment',
    'share consum goverments next step subtransport equipment':
    'share_consum_goverments_next_step_subtransport_equipment',
    'share consum goverments next step submanufacturing nec recycling':
    'share_consum_goverments_next_step_submanufacturing_nec_recycling',
    'share consum goverments next step subelectricity gas and water supply':
    'share_consum_goverments_next_step_subelectricity_gas_and_water_supply',
    'share consum goverments next step subconstruction':
    'share_consum_goverments_next_step_subconstruction',
    'share consum goverments next step subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'share_consum_goverments_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'share consum goverments next step subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'share_consum_goverments_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'share consum goverments next step subretail trade except of motor vehicles and motorcycles repair of household goods':
    'share_consum_goverments_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'share consum goverments next step subhotels and restaurants':
    'share_consum_goverments_next_step_subhotels_and_restaurants',
    'share consum goverments next step subinland transport':
    'share_consum_goverments_next_step_subinland_transport',
    'share consum goverments next step subwater transport':
    'share_consum_goverments_next_step_subwater_transport',
    'share consum goverments next step subair transport':
    'share_consum_goverments_next_step_subair_transport',
    'share consum goverments next step subother supporting and auxiliary transport activities activities of travel agencies':
    'share_consum_goverments_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'share consum goverments next step subpost and telecommunications':
    'share_consum_goverments_next_step_subpost_and_telecommunications',
    'share consum goverments next step subfinancial intermedation':
    'share_consum_goverments_next_step_subfinancial_intermedation',
    'share consum goverments next step subreal estate activities':
    'share_consum_goverments_next_step_subreal_estate_activities',
    'share consum goverments next step subrenting od meq and other business activities':
    'share_consum_goverments_next_step_subrenting_od_meq_and_other_business_activities',
    'share consum goverments next step subpublic admin and defence compulsory social security':
    'share_consum_goverments_next_step_subpublic_admin_and_defence_compulsory_social_security',
    'share consum goverments next step subeducation':
    'share_consum_goverments_next_step_subeducation',
    'share consum goverments next step subhealth and social work':
    'share_consum_goverments_next_step_subhealth_and_social_work',
    'share consum goverments next step subother community social and persona services':
    'share_consum_goverments_next_step_subother_community_social_and_persona_services',
    'share consum goverments next step subprivate households with employed persons':
    'share_consum_goverments_next_step_subprivate_households_with_employed_persons',
    'historic gfcf subagriculture hunting forestry and fishing':
    'historic_gfcf_subagriculture_hunting_forestry_and_fishing',
    'historic gfcf submining and quarrying':
    'historic_gfcf_submining_and_quarrying',
    'historic gfcf subfood beverages and tobacco':
    'historic_gfcf_subfood_beverages_and_tobacco',
    'historic gfcf subtextiles and textile products':
    'historic_gfcf_subtextiles_and_textile_products',
    'historic gfcf subleather leather and footwear':
    'historic_gfcf_subleather_leather_and_footwear',
    'historic gfcf subwood and products of woood and cork':
    'historic_gfcf_subwood_and_products_of_woood_and_cork',
    'historic gfcf subpulp paper printing and publishing':
    'historic_gfcf_subpulp_paper_printing_and_publishing',
    'historic gfcf subcoke refined petroleum and nuclear fuel':
    'historic_gfcf_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic gfcf subchemicals and chemical products':
    'historic_gfcf_subchemicals_and_chemical_products',
    'historic gfcf subrubber and plastics':
    'historic_gfcf_subrubber_and_plastics',
    'historic gfcf subother non metalic mineral':
    'historic_gfcf_subother_non_metalic_mineral',
    'historic gfcf subbasic metals and fabricated metal':
    'historic_gfcf_subbasic_metals_and_fabricated_metal',
    'historic gfcf submachinery nec':
    'historic_gfcf_submachinery_nec',
    'historic gfcf subelectrical and optical equipment':
    'historic_gfcf_subelectrical_and_optical_equipment',
    'historic gfcf subtransport equipment':
    'historic_gfcf_subtransport_equipment',
    'historic gfcf submanufacturing nec recycling':
    'historic_gfcf_submanufacturing_nec_recycling',
    'historic gfcf subelectricity gas and water supply':
    'historic_gfcf_subelectricity_gas_and_water_supply',
    'historic gfcf subconstruction':
    'historic_gfcf_subconstruction',
    'historic gfcf subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_gfcf_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic gfcf subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_gfcf_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic gfcf subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_gfcf_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic gfcf subhotels and restaurants':
    'historic_gfcf_subhotels_and_restaurants',
    'historic gfcf subinland transport':
    'historic_gfcf_subinland_transport',
    'historic gfcf subwater transport':
    'historic_gfcf_subwater_transport',
    'historic gfcf subair transport':
    'historic_gfcf_subair_transport',
    'historic gfcf subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_gfcf_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic gfcf subpost and telecommunications':
    'historic_gfcf_subpost_and_telecommunications',
    'historic gfcf subfinancial intermedation':
    'historic_gfcf_subfinancial_intermedation',
    'historic gfcf subreal estate activities':
    'historic_gfcf_subreal_estate_activities',
    'historic gfcf subrenting od meq and other business activities':
    'historic_gfcf_subrenting_od_meq_and_other_business_activities',
    'historic gfcf subpublic admin and defence compulsory social security':
    'historic_gfcf_subpublic_admin_and_defence_compulsory_social_security',
    'historic gfcf subeducation':
    'historic_gfcf_subeducation',
    'historic gfcf subhealth and social work':
    'historic_gfcf_subhealth_and_social_work',
    'historic gfcf subother community social and persona services':
    'historic_gfcf_subother_community_social_and_persona_services',
    'historic gfcf subprivate households with employed persons':
    'historic_gfcf_subprivate_households_with_employed_persons',
    'historic hd subagriculture hunting forestry and fishing':
    'historic_hd_subagriculture_hunting_forestry_and_fishing',
    'historic hd submining and quarrying':
    'historic_hd_submining_and_quarrying',
    'historic hd subfood beverages and tobacco':
    'historic_hd_subfood_beverages_and_tobacco',
    'historic hd subtextiles and textile products':
    'historic_hd_subtextiles_and_textile_products',
    'historic hd subleather leather and footwear':
    'historic_hd_subleather_leather_and_footwear',
    'historic hd subwood and products of woood and cork':
    'historic_hd_subwood_and_products_of_woood_and_cork',
    'historic hd subpulp paper printing and publishing':
    'historic_hd_subpulp_paper_printing_and_publishing',
    'historic hd subcoke refined petroleum and nuclear fuel':
    'historic_hd_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic hd subchemicals and chemical products':
    'historic_hd_subchemicals_and_chemical_products',
    'historic hd subrubber and plastics':
    'historic_hd_subrubber_and_plastics',
    'historic hd subother non metalic mineral':
    'historic_hd_subother_non_metalic_mineral',
    'historic hd subbasic metals and fabricated metal':
    'historic_hd_subbasic_metals_and_fabricated_metal',
    'historic hd submachinery nec':
    'historic_hd_submachinery_nec',
    'historic hd subelectrical and optical equipment':
    'historic_hd_subelectrical_and_optical_equipment',
    'historic hd subtransport equipment':
    'historic_hd_subtransport_equipment',
    'historic hd submanufacturing nec recycling':
    'historic_hd_submanufacturing_nec_recycling',
    'historic hd subelectricity gas and water supply':
    'historic_hd_subelectricity_gas_and_water_supply',
    'historic hd subconstruction':
    'historic_hd_subconstruction',
    'historic hd subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_hd_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic hd subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_hd_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic hd subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_hd_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic hd subhotels and restaurants':
    'historic_hd_subhotels_and_restaurants',
    'historic hd subinland transport':
    'historic_hd_subinland_transport',
    'historic hd subwater transport':
    'historic_hd_subwater_transport',
    'historic hd subair transport':
    'historic_hd_subair_transport',
    'historic hd subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_hd_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic hd subpost and telecommunications':
    'historic_hd_subpost_and_telecommunications',
    'historic hd subfinancial intermedation':
    'historic_hd_subfinancial_intermedation',
    'historic hd subreal estate activities':
    'historic_hd_subreal_estate_activities',
    'historic hd subrenting od meq and other business activities':
    'historic_hd_subrenting_od_meq_and_other_business_activities',
    'historic hd subpublic admin and defence compulsory social security':
    'historic_hd_subpublic_admin_and_defence_compulsory_social_security',
    'historic hd subeducation':
    'historic_hd_subeducation',
    'historic hd subhealth and social work':
    'historic_hd_subhealth_and_social_work',
    'historic hd subother community social and persona services':
    'historic_hd_subother_community_social_and_persona_services',
    'historic hd subprivate households with employed persons':
    'historic_hd_subprivate_households_with_employed_persons',
    'share cc next step subagriculture hunting forestry and fishing':
    'share_cc_next_step_subagriculture_hunting_forestry_and_fishing',
    'share cc next step submining and quarrying':
    'share_cc_next_step_submining_and_quarrying',
    'share cc next step subfood beverages and tobacco':
    'share_cc_next_step_subfood_beverages_and_tobacco',
    'share cc next step subtextiles and textile products':
    'share_cc_next_step_subtextiles_and_textile_products',
    'share cc next step subleather leather and footwear':
    'share_cc_next_step_subleather_leather_and_footwear',
    'share cc next step subwood and products of woood and cork':
    'share_cc_next_step_subwood_and_products_of_woood_and_cork',
    'share cc next step subpulp paper printing and publishing':
    'share_cc_next_step_subpulp_paper_printing_and_publishing',
    'share cc next step subcoke refined petroleum and nuclear fuel':
    'share_cc_next_step_subcoke_refined_petroleum_and_nuclear_fuel',
    'share cc next step subchemicals and chemical products':
    'share_cc_next_step_subchemicals_and_chemical_products',
    'share cc next step subrubber and plastics':
    'share_cc_next_step_subrubber_and_plastics',
    'share cc next step subother non metalic mineral':
    'share_cc_next_step_subother_non_metalic_mineral',
    'share cc next step subbasic metals and fabricated metal':
    'share_cc_next_step_subbasic_metals_and_fabricated_metal',
    'share cc next step submachinery nec':
    'share_cc_next_step_submachinery_nec',
    'share cc next step subelectrical and optical equipment':
    'share_cc_next_step_subelectrical_and_optical_equipment',
    'share cc next step subtransport equipment':
    'share_cc_next_step_subtransport_equipment',
    'share cc next step submanufacturing nec recycling':
    'share_cc_next_step_submanufacturing_nec_recycling',
    'share cc next step subelectricity gas and water supply':
    'share_cc_next_step_subelectricity_gas_and_water_supply',
    'share cc next step subconstruction':
    'share_cc_next_step_subconstruction',
    'share cc next step subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'share_cc_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'share cc next step subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'share_cc_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'share cc next step subretail trade except of motor vehicles and motorcycles repair of household goods':
    'share_cc_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'share cc next step subhotels and restaurants':
    'share_cc_next_step_subhotels_and_restaurants',
    'share cc next step subinland transport':
    'share_cc_next_step_subinland_transport',
    'share cc next step subwater transport':
    'share_cc_next_step_subwater_transport',
    'share cc next step subair transport':
    'share_cc_next_step_subair_transport',
    'share cc next step subother supporting and auxiliary transport activities activities of travel agencies':
    'share_cc_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'share cc next step subpost and telecommunications':
    'share_cc_next_step_subpost_and_telecommunications',
    'share cc next step subfinancial intermedation':
    'share_cc_next_step_subfinancial_intermedation',
    'share cc next step subreal estate activities':
    'share_cc_next_step_subreal_estate_activities',
    'share cc next step subrenting od meq and other business activities':
    'share_cc_next_step_subrenting_od_meq_and_other_business_activities',
    'share cc next step subpublic admin and defence compulsory social security':
    'share_cc_next_step_subpublic_admin_and_defence_compulsory_social_security',
    'share cc next step subeducation':
    'share_cc_next_step_subeducation',
    'share cc next step subhealth and social work':
    'share_cc_next_step_subhealth_and_social_work',
    'share cc next step subother community social and persona services':
    'share_cc_next_step_subother_community_social_and_persona_services',
    'share cc next step subprivate households with employed persons':
    'share_cc_next_step_subprivate_households_with_employed_persons',
    'historic labour share':
    'historic_labour_share',
    'share cc sectoral subagriculture hunting forestry and fishing':
    'share_cc_sectoral_subagriculture_hunting_forestry_and_fishing',
    'share cc sectoral submining and quarrying':
    'share_cc_sectoral_submining_and_quarrying',
    'share cc sectoral subfood beverages and tobacco':
    'share_cc_sectoral_subfood_beverages_and_tobacco',
    'share cc sectoral subtextiles and textile products':
    'share_cc_sectoral_subtextiles_and_textile_products',
    'share cc sectoral subleather leather and footwear':
    'share_cc_sectoral_subleather_leather_and_footwear',
    'share cc sectoral subwood and products of woood and cork':
    'share_cc_sectoral_subwood_and_products_of_woood_and_cork',
    'share cc sectoral subpulp paper printing and publishing':
    'share_cc_sectoral_subpulp_paper_printing_and_publishing',
    'share cc sectoral subcoke refined petroleum and nuclear fuel':
    'share_cc_sectoral_subcoke_refined_petroleum_and_nuclear_fuel',
    'share cc sectoral subchemicals and chemical products':
    'share_cc_sectoral_subchemicals_and_chemical_products',
    'share cc sectoral subrubber and plastics':
    'share_cc_sectoral_subrubber_and_plastics',
    'share cc sectoral subother non metalic mineral':
    'share_cc_sectoral_subother_non_metalic_mineral',
    'share cc sectoral subbasic metals and fabricated metal':
    'share_cc_sectoral_subbasic_metals_and_fabricated_metal',
    'share cc sectoral submachinery nec':
    'share_cc_sectoral_submachinery_nec',
    'share cc sectoral subelectrical and optical equipment':
    'share_cc_sectoral_subelectrical_and_optical_equipment',
    'share cc sectoral subtransport equipment':
    'share_cc_sectoral_subtransport_equipment',
    'share cc sectoral submanufacturing nec recycling':
    'share_cc_sectoral_submanufacturing_nec_recycling',
    'share cc sectoral subelectricity gas and water supply':
    'share_cc_sectoral_subelectricity_gas_and_water_supply',
    'share cc sectoral subconstruction':
    'share_cc_sectoral_subconstruction',
    'share cc sectoral subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'share_cc_sectoral_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'share cc sectoral subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'share_cc_sectoral_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'share cc sectoral subretail trade except of motor vehicles and motorcycles repair of household goods':
    'share_cc_sectoral_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'share cc sectoral subhotels and restaurants':
    'share_cc_sectoral_subhotels_and_restaurants',
    'share cc sectoral subinland transport':
    'share_cc_sectoral_subinland_transport',
    'share cc sectoral subwater transport':
    'share_cc_sectoral_subwater_transport',
    'share cc sectoral subair transport':
    'share_cc_sectoral_subair_transport',
    'share cc sectoral subother supporting and auxiliary transport activities activities of travel agencies':
    'share_cc_sectoral_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'share cc sectoral subpost and telecommunications':
    'share_cc_sectoral_subpost_and_telecommunications',
    'share cc sectoral subfinancial intermedation':
    'share_cc_sectoral_subfinancial_intermedation',
    'share cc sectoral subreal estate activities':
    'share_cc_sectoral_subreal_estate_activities',
    'share cc sectoral subrenting od meq and other business activities':
    'share_cc_sectoral_subrenting_od_meq_and_other_business_activities',
    'share cc sectoral subpublic admin and defence compulsory social security':
    'share_cc_sectoral_subpublic_admin_and_defence_compulsory_social_security',
    'share cc sectoral subeducation':
    'share_cc_sectoral_subeducation',
    'share cc sectoral subhealth and social work':
    'share_cc_sectoral_subhealth_and_social_work',
    'share cc sectoral subother community social and persona services':
    'share_cc_sectoral_subother_community_social_and_persona_services',
    'share cc sectoral subprivate households with employed persons':
    'share_cc_sectoral_subprivate_households_with_employed_persons',
    'historic pes waste ej':
    'historic_pes_waste_ej',
    'table hist capacity res elec subhydro':
    'table_hist_capacity_res_elec_subhydro',
    'table hist capacity res elec subgeotxdashxelec':
    'table_hist_capacity_res_elec_subgeotxdashxelec',
    'table hist capacity res elec subsolid bioexdashxelec':
    'table_hist_capacity_res_elec_subsolid_bioexdashxelec',
    'table hist capacity res elec suboceanic':
    'table_hist_capacity_res_elec_suboceanic',
    'table hist capacity res elec subwind onshore':
    'table_hist_capacity_res_elec_subwind_onshore',
    'table hist capacity res elec subwind offshore':
    'table_hist_capacity_res_elec_subwind_offshore',
    'table hist capacity res elec subsolar pv':
    'table_hist_capacity_res_elec_subsolar_pv',
    'table hist capacity res elec subcsp':
    'table_hist_capacity_res_elec_subcsp',
    'losses in charcoal plants ej':
    'losses_in_charcoal_plants_ej',
    'historic share chp plants gas':
    'historic_share_chp_plants_gas',
    'historic share liquids for heat plants':
    'historic_share_liquids_for_heat_plants',
    'historic share chp plants oil':
    'historic_share_chp_plants_oil',
    'historic biogas pes':
    'historic_biogas_pes',
    'historic pes peat ej':
    'historic_pes_peat_ej',
    'efficiency heat oil chp plants':
    'efficiency_heat_oil_chp_plants',
    'efficiency elec coal chp plants':
    'efficiency_elec_coal_chp_plants',
    'efficiency elec gas chp plants':
    'efficiency_elec_gas_chp_plants',
    'efficiency elec oil chp plants':
    'efficiency_elec_oil_chp_plants',
    'efficiency heat gas chp plants':
    'efficiency_heat_gas_chp_plants',
    'share heatxdashxcom chp plants nre vs nre tot heatxdashxcom generation':
    'share_heatxdashxcom_chp_plants_nre_vs_nre_tot_heatxdashxcom_generation',
    'efficiency heat coal chp plants':
    'efficiency_heat_coal_chp_plants',
    'historic nonxdashxenergy use subliquids':
    'historic_nonxdashxenergy_use_subliquids',
    'historic nonxdashxenergy use subsolids':
    'historic_nonxdashxenergy_use_subsolids',
    'historic nonxdashxenergy use subgases':
    'historic_nonxdashxenergy_use_subgases',
    'historic nonxdashxenergy use subelectricity':
    'historic_nonxdashxenergy_use_subelectricity',
    'historic nonxdashxenergy use subheat':
    'historic_nonxdashxenergy_use_subheat',
    'efficiency liquids for heat plants':
    'efficiency_liquids_for_heat_plants',
    'share gasxdivxxcoalxplusxgasx for heat plants':
    'share_gasxdivxxcoalxplusxgasx_for_heat_plants',
    'efficiency coal for heat plants':
    'efficiency_coal_for_heat_plants',
    'efficiency gases for heat plants':
    'efficiency_gases_for_heat_plants',
    'invest cost res elec subhydro':
    'invest_cost_res_elec_subhydro',
    'invest cost res elec subgeotxdashxelec':
    'invest_cost_res_elec_subgeotxdashxelec',
    'invest cost res elec subsolid bioexdashxelec':
    'invest_cost_res_elec_subsolid_bioexdashxelec',
    'invest cost res elec suboceanic':
    'invest_cost_res_elec_suboceanic',
    'invest cost res elec subwind onshore':
    'invest_cost_res_elec_subwind_onshore',
    'invest cost res elec subwind offshore':
    'invest_cost_res_elec_subwind_offshore',
    'invest cost res elec subsolar pv':
    'invest_cost_res_elec_subsolar_pv',
    'invest cost res elec subcsp':
    'invest_cost_res_elec_subcsp',
    'historic final energy intensity by sector and fuel subelectricity subagriculture hunting forestry and fishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculture_hunting_forestry_and_fishing',
    'historic final energy intensity by sector and fuel subelectricity submining and quarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submining_and_quarrying',
    'historic final energy intensity by sector and fuel subelectricity subfood beverages and tobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfood_beverages_and_tobacco',
    'historic final energy intensity by sector and fuel subelectricity subtextiles and textile products':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextiles_and_textile_products',
    'historic final energy intensity by sector and fuel subelectricity subleather leather and footwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleather_leather_and_footwear',
    'historic final energy intensity by sector and fuel subelectricity subwood and products of woood and cork':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwood_and_products_of_woood_and_cork',
    'historic final energy intensity by sector and fuel subelectricity subpulp paper printing and publishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulp_paper_printing_and_publishing',
    'historic final energy intensity by sector and fuel subelectricity subcoke refined petroleum and nuclear fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic final energy intensity by sector and fuel subelectricity subchemicals and chemical products':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicals_and_chemical_products',
    'historic final energy intensity by sector and fuel subelectricity subrubber and plastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubber_and_plastics',
    'historic final energy intensity by sector and fuel subelectricity subother non metalic mineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_non_metalic_mineral',
    'historic final energy intensity by sector and fuel subelectricity subbasic metals and fabricated metal':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasic_metals_and_fabricated_metal',
    'historic final energy intensity by sector and fuel subelectricity submachinery nec':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinery_nec',
    'historic final energy intensity by sector and fuel subelectricity subelectrical and optical equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectrical_and_optical_equipment',
    'historic final energy intensity by sector and fuel subelectricity subtransport equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransport_equipment',
    'historic final energy intensity by sector and fuel subelectricity submanufacturing nec recycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturing_nec_recycling',
    'historic final energy intensity by sector and fuel subelectricity subelectricity gas and water supply':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricity_gas_and_water_supply',
    'historic final energy intensity by sector and fuel subelectricity subconstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction',
    'historic final energy intensity by sector and fuel subelectricity subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic final energy intensity by sector and fuel subelectricity subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic final energy intensity by sector and fuel subelectricity subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic final energy intensity by sector and fuel subelectricity subhotels and restaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotels_and_restaurants',
    'historic final energy intensity by sector and fuel subelectricity subinland transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinland_transport',
    'historic final energy intensity by sector and fuel subelectricity subwater transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwater_transport',
    'historic final energy intensity by sector and fuel subelectricity subair transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subair_transport',
    'historic final energy intensity by sector and fuel subelectricity subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic final energy intensity by sector and fuel subelectricity subpost and telecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpost_and_telecommunications',
    'historic final energy intensity by sector and fuel subelectricity subfinancial intermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancial_intermedation',
    'historic final energy intensity by sector and fuel subelectricity subreal estate activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subreal_estate_activities',
    'historic final energy intensity by sector and fuel subelectricity subrenting od meq and other business activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrenting_od_meq_and_other_business_activities',
    'historic final energy intensity by sector and fuel subelectricity subpublic admin and defence compulsory social security':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublic_admin_and_defence_compulsory_social_security',
    'historic final energy intensity by sector and fuel subelectricity subeducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation',
    'historic final energy intensity by sector and fuel subelectricity subhealth and social work':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealth_and_social_work',
    'historic final energy intensity by sector and fuel subelectricity subother community social and persona services':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_community_social_and_persona_services',
    'historic final energy intensity by sector and fuel subelectricity subprivate households with employed persons':
    'historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivate_households_with_employed_persons',
    'historic final energy intensity by sector and fuel subheat subagriculture hunting forestry and fishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculture_hunting_forestry_and_fishing',
    'historic final energy intensity by sector and fuel subheat submining and quarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_submining_and_quarrying',
    'historic final energy intensity by sector and fuel subheat subfood beverages and tobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subfood_beverages_and_tobacco',
    'historic final energy intensity by sector and fuel subheat subtextiles and textile products':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextiles_and_textile_products',
    'historic final energy intensity by sector and fuel subheat subleather leather and footwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subleather_leather_and_footwear',
    'historic final energy intensity by sector and fuel subheat subwood and products of woood and cork':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwood_and_products_of_woood_and_cork',
    'historic final energy intensity by sector and fuel subheat subpulp paper printing and publishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulp_paper_printing_and_publishing',
    'historic final energy intensity by sector and fuel subheat subcoke refined petroleum and nuclear fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic final energy intensity by sector and fuel subheat subchemicals and chemical products':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicals_and_chemical_products',
    'historic final energy intensity by sector and fuel subheat subrubber and plastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubber_and_plastics',
    'historic final energy intensity by sector and fuel subheat subother non metalic mineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_non_metalic_mineral',
    'historic final energy intensity by sector and fuel subheat subbasic metals and fabricated metal':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasic_metals_and_fabricated_metal',
    'historic final energy intensity by sector and fuel subheat submachinery nec':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinery_nec',
    'historic final energy intensity by sector and fuel subheat subelectrical and optical equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectrical_and_optical_equipment',
    'historic final energy intensity by sector and fuel subheat subtransport equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransport_equipment',
    'historic final energy intensity by sector and fuel subheat submanufacturing nec recycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturing_nec_recycling',
    'historic final energy intensity by sector and fuel subheat subelectricity gas and water supply':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricity_gas_and_water_supply',
    'historic final energy intensity by sector and fuel subheat subconstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction',
    'historic final energy intensity by sector and fuel subheat subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic final energy intensity by sector and fuel subheat subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic final energy intensity by sector and fuel subheat subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic final energy intensity by sector and fuel subheat subhotels and restaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotels_and_restaurants',
    'historic final energy intensity by sector and fuel subheat subinland transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subinland_transport',
    'historic final energy intensity by sector and fuel subheat subwater transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subwater_transport',
    'historic final energy intensity by sector and fuel subheat subair transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subair_transport',
    'historic final energy intensity by sector and fuel subheat subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic final energy intensity by sector and fuel subheat subpost and telecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpost_and_telecommunications',
    'historic final energy intensity by sector and fuel subheat subfinancial intermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancial_intermedation',
    'historic final energy intensity by sector and fuel subheat subreal estate activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subreal_estate_activities',
    'historic final energy intensity by sector and fuel subheat subrenting od meq and other business activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subrenting_od_meq_and_other_business_activities',
    'historic final energy intensity by sector and fuel subheat subpublic admin and defence compulsory social security':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublic_admin_and_defence_compulsory_social_security',
    'historic final energy intensity by sector and fuel subheat subeducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation',
    'historic final energy intensity by sector and fuel subheat subhealth and social work':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealth_and_social_work',
    'historic final energy intensity by sector and fuel subheat subother community social and persona services':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_community_social_and_persona_services',
    'historic final energy intensity by sector and fuel subheat subprivate households with employed persons':
    'historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivate_households_with_employed_persons',
    'historic final energy intensity by sector and fuel subliquids subagriculture hunting forestry and fishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculture_hunting_forestry_and_fishing',
    'historic final energy intensity by sector and fuel subliquids submining and quarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_submining_and_quarrying',
    'historic final energy intensity by sector and fuel subliquids subfood beverages and tobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfood_beverages_and_tobacco',
    'historic final energy intensity by sector and fuel subliquids subtextiles and textile products':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextiles_and_textile_products',
    'historic final energy intensity by sector and fuel subliquids subleather leather and footwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleather_leather_and_footwear',
    'historic final energy intensity by sector and fuel subliquids subwood and products of woood and cork':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwood_and_products_of_woood_and_cork',
    'historic final energy intensity by sector and fuel subliquids subpulp paper printing and publishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulp_paper_printing_and_publishing',
    'historic final energy intensity by sector and fuel subliquids subcoke refined petroleum and nuclear fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic final energy intensity by sector and fuel subliquids subchemicals and chemical products':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicals_and_chemical_products',
    'historic final energy intensity by sector and fuel subliquids subrubber and plastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubber_and_plastics',
    'historic final energy intensity by sector and fuel subliquids subother non metalic mineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_non_metalic_mineral',
    'historic final energy intensity by sector and fuel subliquids subbasic metals and fabricated metal':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasic_metals_and_fabricated_metal',
    'historic final energy intensity by sector and fuel subliquids submachinery nec':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinery_nec',
    'historic final energy intensity by sector and fuel subliquids subelectrical and optical equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectrical_and_optical_equipment',
    'historic final energy intensity by sector and fuel subliquids subtransport equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransport_equipment',
    'historic final energy intensity by sector and fuel subliquids submanufacturing nec recycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturing_nec_recycling',
    'historic final energy intensity by sector and fuel subliquids subelectricity gas and water supply':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricity_gas_and_water_supply',
    'historic final energy intensity by sector and fuel subliquids subconstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction',
    'historic final energy intensity by sector and fuel subliquids subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic final energy intensity by sector and fuel subliquids subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic final energy intensity by sector and fuel subliquids subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic final energy intensity by sector and fuel subliquids subhotels and restaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotels_and_restaurants',
    'historic final energy intensity by sector and fuel subliquids subinland transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinland_transport',
    'historic final energy intensity by sector and fuel subliquids subwater transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwater_transport',
    'historic final energy intensity by sector and fuel subliquids subair transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subair_transport',
    'historic final energy intensity by sector and fuel subliquids subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic final energy intensity by sector and fuel subliquids subpost and telecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpost_and_telecommunications',
    'historic final energy intensity by sector and fuel subliquids subfinancial intermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancial_intermedation',
    'historic final energy intensity by sector and fuel subliquids subreal estate activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subreal_estate_activities',
    'historic final energy intensity by sector and fuel subliquids subrenting od meq and other business activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrenting_od_meq_and_other_business_activities',
    'historic final energy intensity by sector and fuel subliquids subpublic admin and defence compulsory social security':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublic_admin_and_defence_compulsory_social_security',
    'historic final energy intensity by sector and fuel subliquids subeducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation',
    'historic final energy intensity by sector and fuel subliquids subhealth and social work':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealth_and_social_work',
    'historic final energy intensity by sector and fuel subliquids subother community social and persona services':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_community_social_and_persona_services',
    'historic final energy intensity by sector and fuel subliquids subprivate households with employed persons':
    'historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivate_households_with_employed_persons',
    'historic final energy intensity by sector and fuel subgases subagriculture hunting forestry and fishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculture_hunting_forestry_and_fishing',
    'historic final energy intensity by sector and fuel subgases submining and quarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_submining_and_quarrying',
    'historic final energy intensity by sector and fuel subgases subfood beverages and tobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subfood_beverages_and_tobacco',
    'historic final energy intensity by sector and fuel subgases subtextiles and textile products':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextiles_and_textile_products',
    'historic final energy intensity by sector and fuel subgases subleather leather and footwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subleather_leather_and_footwear',
    'historic final energy intensity by sector and fuel subgases subwood and products of woood and cork':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwood_and_products_of_woood_and_cork',
    'historic final energy intensity by sector and fuel subgases subpulp paper printing and publishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulp_paper_printing_and_publishing',
    'historic final energy intensity by sector and fuel subgases subcoke refined petroleum and nuclear fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic final energy intensity by sector and fuel subgases subchemicals and chemical products':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicals_and_chemical_products',
    'historic final energy intensity by sector and fuel subgases subrubber and plastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubber_and_plastics',
    'historic final energy intensity by sector and fuel subgases subother non metalic mineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_non_metalic_mineral',
    'historic final energy intensity by sector and fuel subgases subbasic metals and fabricated metal':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasic_metals_and_fabricated_metal',
    'historic final energy intensity by sector and fuel subgases submachinery nec':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinery_nec',
    'historic final energy intensity by sector and fuel subgases subelectrical and optical equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectrical_and_optical_equipment',
    'historic final energy intensity by sector and fuel subgases subtransport equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransport_equipment',
    'historic final energy intensity by sector and fuel subgases submanufacturing nec recycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturing_nec_recycling',
    'historic final energy intensity by sector and fuel subgases subelectricity gas and water supply':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricity_gas_and_water_supply',
    'historic final energy intensity by sector and fuel subgases subconstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction',
    'historic final energy intensity by sector and fuel subgases subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic final energy intensity by sector and fuel subgases subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic final energy intensity by sector and fuel subgases subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic final energy intensity by sector and fuel subgases subhotels and restaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotels_and_restaurants',
    'historic final energy intensity by sector and fuel subgases subinland transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subinland_transport',
    'historic final energy intensity by sector and fuel subgases subwater transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subwater_transport',
    'historic final energy intensity by sector and fuel subgases subair transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subair_transport',
    'historic final energy intensity by sector and fuel subgases subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic final energy intensity by sector and fuel subgases subpost and telecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpost_and_telecommunications',
    'historic final energy intensity by sector and fuel subgases subfinancial intermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancial_intermedation',
    'historic final energy intensity by sector and fuel subgases subreal estate activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subreal_estate_activities',
    'historic final energy intensity by sector and fuel subgases subrenting od meq and other business activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subrenting_od_meq_and_other_business_activities',
    'historic final energy intensity by sector and fuel subgases subpublic admin and defence compulsory social security':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublic_admin_and_defence_compulsory_social_security',
    'historic final energy intensity by sector and fuel subgases subeducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation',
    'historic final energy intensity by sector and fuel subgases subhealth and social work':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealth_and_social_work',
    'historic final energy intensity by sector and fuel subgases subother community social and persona services':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_community_social_and_persona_services',
    'historic final energy intensity by sector and fuel subgases subprivate households with employed persons':
    'historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivate_households_with_employed_persons',
    'historic final energy intensity by sector and fuel subsolids subagriculture hunting forestry and fishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculture_hunting_forestry_and_fishing',
    'historic final energy intensity by sector and fuel subsolids submining and quarrying':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_submining_and_quarrying',
    'historic final energy intensity by sector and fuel subsolids subfood beverages and tobacco':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfood_beverages_and_tobacco',
    'historic final energy intensity by sector and fuel subsolids subtextiles and textile products':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextiles_and_textile_products',
    'historic final energy intensity by sector and fuel subsolids subleather leather and footwear':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleather_leather_and_footwear',
    'historic final energy intensity by sector and fuel subsolids subwood and products of woood and cork':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwood_and_products_of_woood_and_cork',
    'historic final energy intensity by sector and fuel subsolids subpulp paper printing and publishing':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulp_paper_printing_and_publishing',
    'historic final energy intensity by sector and fuel subsolids subcoke refined petroleum and nuclear fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcoke_refined_petroleum_and_nuclear_fuel',
    'historic final energy intensity by sector and fuel subsolids subchemicals and chemical products':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicals_and_chemical_products',
    'historic final energy intensity by sector and fuel subsolids subrubber and plastics':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubber_and_plastics',
    'historic final energy intensity by sector and fuel subsolids subother non metalic mineral':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_non_metalic_mineral',
    'historic final energy intensity by sector and fuel subsolids subbasic metals and fabricated metal':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasic_metals_and_fabricated_metal',
    'historic final energy intensity by sector and fuel subsolids submachinery nec':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinery_nec',
    'historic final energy intensity by sector and fuel subsolids subelectrical and optical equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectrical_and_optical_equipment',
    'historic final energy intensity by sector and fuel subsolids subtransport equipment':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransport_equipment',
    'historic final energy intensity by sector and fuel subsolids submanufacturing nec recycling':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturing_nec_recycling',
    'historic final energy intensity by sector and fuel subsolids subelectricity gas and water supply':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricity_gas_and_water_supply',
    'historic final energy intensity by sector and fuel subsolids subconstruction':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction',
    'historic final energy intensity by sector and fuel subsolids subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel',
    'historic final energy intensity by sector and fuel subsolids subwholesale trade and commissions trade except of motor vehicles and motorcycles':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles',
    'historic final energy intensity by sector and fuel subsolids subretail trade except of motor vehicles and motorcycles repair of household goods':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods',
    'historic final energy intensity by sector and fuel subsolids subhotels and restaurants':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotels_and_restaurants',
    'historic final energy intensity by sector and fuel subsolids subinland transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinland_transport',
    'historic final energy intensity by sector and fuel subsolids subwater transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwater_transport',
    'historic final energy intensity by sector and fuel subsolids subair transport':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subair_transport',
    'historic final energy intensity by sector and fuel subsolids subother supporting and auxiliary transport activities activities of travel agencies':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies',
    'historic final energy intensity by sector and fuel subsolids subpost and telecommunications':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpost_and_telecommunications',
    'historic final energy intensity by sector and fuel subsolids subfinancial intermedation':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancial_intermedation',
    'historic final energy intensity by sector and fuel subsolids subreal estate activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subreal_estate_activities',
    'historic final energy intensity by sector and fuel subsolids subrenting od meq and other business activities':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrenting_od_meq_and_other_business_activities',
    'historic final energy intensity by sector and fuel subsolids subpublic admin and defence compulsory social security':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublic_admin_and_defence_compulsory_social_security',
    'historic final energy intensity by sector and fuel subsolids subeducation':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation',
    'historic final energy intensity by sector and fuel subsolids subhealth and social work':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealth_and_social_work',
    'historic final energy intensity by sector and fuel subsolids subother community social and persona services':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_community_social_and_persona_services',
    'historic final energy intensity by sector and fuel subsolids subprivate households with employed persons':
    'historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivate_households_with_employed_persons',
    'cpxdashxini res elec subhydro':
    'cpxdashxini_res_elec_subhydro',
    'table max extraction uranium user defined':
    'table_max_extraction_uranium_user_defined',
    'userxdashxdefined extraction growth unconv gas':
    'userxdashxdefined_extraction_growth_unconv_gas',
    'userxdashxdefined extraction growth unconv oil':
    'userxdashxdefined_extraction_growth_unconv_oil',
    'historic unconv oil':
    'historic_unconv_oil',
    'historic unconv gas':
    'historic_unconv_gas',
    'historic pop':
    'historic_pop',
    'balancing costs ref':
    'balancing_costs_ref',
    'historic gtl production':
    'historic_gtl_production',
    'historic ctl production':
    'historic_ctl_production',
    'historic produc biofuels 2gen':
    'historic_produc_biofuels_2gen',
    'table max extraction tot agg oil laherrere x2006x':
    'table_max_extraction_tot_agg_oil_laherrere_x2006x',
    'table max extraction maggio12 high conv oil ej':
    'table_max_extraction_maggio12_high_conv_oil_ej',
    'table max extraction maggio12 low con oil ej':
    'table_max_extraction_maggio12_low_con_oil_ej',
    'table max extraction maggio12middle conv oil ej':
    'table_max_extraction_maggio12middle_conv_oil_ej',
    'table max extraction unconv oil user defined':
    'table_max_extraction_unconv_oil_user_defined',
    'table max extraction uranium zittel12':
    'table_max_extraction_uranium_zittel12',
    'table max extraction unconv oil low mohr15':
    'table_max_extraction_unconv_oil_low_mohr15',
    'table max extraction unconv oil high mohr15':
    'table_max_extraction_unconv_oil_high_mohr15',
    'table max extraction tot agg oil user defined':
    'table_max_extraction_tot_agg_oil_user_defined',
    'table max conv oil extraction user defined':
    'table_max_conv_oil_extraction_user_defined',
    'table max extraction uranium ewg13 ej':
    'table_max_extraction_uranium_ewg13_ej',
    'hist share oil elec':
    'hist_share_oil_elec',
    'hist share gasxdivxxcoal xplusxgasx elec':
    'hist_share_gasxdivxxcoal_xplusxgasx_elec',
    'historic nuclear generation twh':
    'historic_nuclear_generation_twh',
    'invest cost nuclear':
    'invest_cost_nuclear',
    'tpes de castro phd xdashx scen xmadcoalx':
    'tpes_de_castro_phd_xdashx_scen_xmadcoalx',
    'gtc historic emissions rcps':
    'gtc_historic_emissions_rcps',
    'afforestation program x2020x':
    'afforestation_program_x2020x',
    'historic efficiency gas for electricity':
    'historic_efficiency_gas_for_electricity',
    'p timeseries pop growth rate':
    'p_timeseries_pop_growth_rate',
    'time dmnl':
    'time_dmnl',
    'table max extraction aspo oil ej':
    'table_max_extraction_aspo_oil_ej',
    'table max extraction gas laherrere2010':
    'table_max_extraction_gas_laherrere2010',
    'table max extraction gas mohr high2012':
    'table_max_extraction_gas_mohr_high2012',
    'table max extraction gas mohr bg2012':
    'table_max_extraction_gas_mohr_bg2012',
    'tnes de castro phd xdashx scen i':
    'tnes_de_castro_phd_xdashx_scen_i',
    'net oil extraction de castro phd xdashx scen i':
    'net_oil_extraction_de_castro_phd_xdashx_scen_i',
    'pe oil extraction de castro phd xdashx scen ii':
    'pe_oil_extraction_de_castro_phd_xdashx_scen_ii',
    'net oil extraction de castro phd xdashx scen iii':
    'net_oil_extraction_de_castro_phd_xdashx_scen_iii',
    'pe coal extraction de castro phd xdashx scen ii':
    'pe_coal_extraction_de_castro_phd_xdashx_scen_ii',
    'net gas extraction de castro phd xdashx scen i':
    'net_gas_extraction_de_castro_phd_xdashx_scen_i',
    'pe gas extraction de castro phd xdashx scen ii':
    'pe_gas_extraction_de_castro_phd_xdashx_scen_ii',
    'net gas extraction de castro phd xdashx scen iii':
    'net_gas_extraction_de_castro_phd_xdashx_scen_iii',
    'net coal extraction de castro phd xdashx scen iii':
    'net_coal_extraction_de_castro_phd_xdashx_scen_iii',
    'net coal extraction de castro phd xdashx scen i':
    'net_coal_extraction_de_castro_phd_xdashx_scen_i',
    'tnes de castro phd xdashx scen iii':
    'tnes_de_castro_phd_xdashx_scen_iii',
    'primary coal extraction de castro phd xdashx scen xmadcoalx':
    'primary_coal_extraction_de_castro_phd_xdashx_scen_xmadcoalx',
    'tpes de castro phd xdashx scen ii':
    'tpes_de_castro_phd_xdashx_scen_ii',
    'table max extraction conv gas low mohr15':
    'table_max_extraction_conv_gas_low_mohr15',
    'table max extraction conv gas user defined':
    'table_max_extraction_conv_gas_user_defined',
    'table max extraction total gas bg mohr12':
    'table_max_extraction_total_gas_bg_mohr12',
    'table max extraction total gas laherrere10':
    'table_max_extraction_total_gas_laherrere10',
    'table max extraction total gas user defined':
    'table_max_extraction_total_gas_user_defined',
    'table max extraction unconv gas bg mohr15':
    'table_max_extraction_unconv_gas_bg_mohr15',
    'table max extraction unconv gas high mohr15':
    'table_max_extraction_unconv_gas_high_mohr15',
    'table max extraction unconv gas low mohr15':
    'table_max_extraction_unconv_gas_low_mohr15',
    'table max extraction unconv gas user defined':
    'table_max_extraction_unconv_gas_user_defined',
    'table max extraction coal bg mohr15':
    'table_max_extraction_coal_bg_mohr15',
    'table max extraction coal high mohr15':
    'table_max_extraction_coal_high_mohr15',
    'table max extraction coal low mohr15':
    'table_max_extraction_coal_low_mohr15',
    'table max extraction coal mohr2012 ej':
    'table_max_extraction_coal_mohr2012_ej',
    'table max extraction coal user defined':
    'table_max_extraction_coal_user_defined',
    'table max extraction conv gas bg mohr15':
    'table_max_extraction_conv_gas_bg_mohr15',
    'table max extraction conv gas high mohr15':
    'table_max_extraction_conv_gas_high_mohr15',
    'table max extraction unconv oil bg mohr15':
    'table_max_extraction_unconv_oil_bg_mohr15',
    'ratio land productivity 2gen vs marg':
    'ratio_land_productivity_2gen_vs_marg',
    'land productivity biofuels marg ej mha':
    'land_productivity_biofuels_marg_ej_mha',
    'correction factor all ghgs':
    'correction_factor_all_ghgs',
    'cumulative co2e ghg emissions':
    'cumulative_co2e_ghg_emissions',
    'total ce all ghg':
    'total_ce_all_ghg',
    'total co2e all ghg':
    'total_co2e_all_ghg',
    'eroi fc system from x2015x':
    'eroi_fc_system_from_x2015x',
    'share in target year oil for heat':
    'share_in_target_year_oil_for_heat',
    'p share oil for heat':
    'p_share_oil_for_heat',
    'target year policy phasexdashxout oil for heat':
    'target_year_policy_phasexdashxout_oil_for_heat',
    'b lineal regr phasexdashxout oil for heat':
    'b_lineal_regr_phasexdashxout_oil_for_heat',
    'start year policy phasexdashxout oil for heat':
    'start_year_policy_phasexdashxout_oil_for_heat',
    'start year policy phasexdashxout oil for elec':
    'start_year_policy_phasexdashxout_oil_for_elec',
    'p share oil for elec':
    'p_share_oil_for_elec',
    'share in target year oil for elec':
    'share_in_target_year_oil_for_elec',
    'target year policy phasexdashxout oil for elec':
    'target_year_policy_phasexdashxout_oil_for_elec',
    'b lineal regr phasexdashxout oil for elec':
    'b_lineal_regr_phasexdashxout_oil_for_elec',
    'fed heatxdashxnc ej':
    'fed_heatxdashxnc_ej',
    'leontief matrix':
    'leontief_matrix',
    'pressure to change energy technology h':
    'pressure_to_change_energy_technology_h',
    'pressure to change energy technology':
    'pressure_to_change_energy_technology',
    'ia matrix':
    'ia_matrix',
    'pe traditional biomass consum ej':
    'pe_traditional_biomass_consum_ej',
    'scarcity resources counter':
    'scarcity_resources_counter',
    'scarcity resources flag':
    'scarcity_resources_flag',
    'percent res vs tpes':
    'percent_res_vs_tpes',
    'year init scarcity final fuels':
    'year_init_scarcity_final_fuels',
    'year init scarcity reserves':
    'year_init_scarcity_reserves',
    'percent tot monet invest reselec vs gdp':
    'percent_tot_monet_invest_reselec_vs_gdp',
    'scarcity final fuels counter':
    'scarcity_final_fuels_counter',
    'percent exdashxlosses cc':
    'percent_exdashxlosses_cc',
    'scarcity reserves counter':
    'scarcity_reserves_counter',
    'year final scarcity reserves':
    'year_final_scarcity_reserves',
    'year final scarcity resources':
    'year_final_scarcity_resources',
    'scarcity fuels flag':
    'scarcity_fuels_flag',
    'scarcity reserves flag':
    'scarcity_reserves_flag',
    'year init scarcity resources':
    'year_init_scarcity_resources',
    'percent remaining potential tot res heat':
    'percent_remaining_potential_tot_res_heat',
    'abundance final fuels':
    'abundance_final_fuels',
    'scarcity final fuels':
    'scarcity_final_fuels',
    'percent remaining potential tot res elec':
    'percent_remaining_potential_tot_res_elec',
    'year final scarcity final fuels':
    'year_final_scarcity_final_fuels',
    'share blue water use vs ar':
    'share_blue_water_use_vs_ar',
    'ar water':
    'ar_water',
    'share total water use vs ar':
    'share_total_water_use_vs_ar',
    'renewable water resources':
    'renewable_water_resources',
    'share blue water use vs renewable water resources':
    'share_blue_water_use_vs_renewable_water_resources',
    'dam3 per km3':
    'dam3_per_km3',
    'share total water use vs renewable water resources':
    'share_total_water_use_vs_renewable_water_resources',
    'total pe solid bioe potential heatxplusxelec ej':
    'total_pe_solid_bioe_potential_heatxplusxelec_ej',
    'land module activatedxquestx':
    'land_module_activatedxquestx',
    'activate_eroi fc system feedbackxquestx':
    'activate_eroi_fc_system_feedbackxquestx',
    'cp res for heat':
    'cp_res_for_heat',
    'potential fes res for heatxdashxcom ej':
    'potential_fes_res_for_heatxdashxcom_ej',
    'potential fes res for heatxdashxcom twh':
    'potential_fes_res_for_heatxdashxcom_twh',
    'real generation res elec twh':
    'real_generation_res_elec_twh',
    'potential fes res for heatxdashxnc twh':
    'potential_fes_res_for_heatxdashxnc_twh',
    'replacement capacity res elec':
    'replacement_capacity_res_elec',
    'replacement res for heatxdashxnc tw':
    'replacement_res_for_heatxdashxnc_tw',
    'replacement res for heatxdashxcom tw':
    'replacement_res_for_heatxdashxcom_tw',
    'max eroi fc':
    'max_eroi_fc',
    'eroi fc system from x2015x x1x':
    'eroi_fc_system_from_x2015x_x1x',
    'evol final energy intensity by sector and fuel delayed 1yr':
    'evol_final_energy_intensity_by_sector_and_fuel_delayed_1yr',
    'evol final energy intensity h delayed x1x yr':
    'evol_final_energy_intensity_h_delayed_x1x_yr',
    'maximun yearly aceleratuin of intensity improvement h':
    'maximun_yearly_aceleratuin_of_intensity_improvement_h',
    'maximun yearly aceleratuin of intensity improvement':
    'maximun_yearly_aceleratuin_of_intensity_improvement',
    'initial energy intensity by fuel and sector x1995x':
    'initial_energy_intensity_by_fuel_and_sector_x1995x',
    'share max of change vs historical mean h':
    'share_max_of_change_vs_historical_mean_h',
    'efficiency energy aceleration h':
    'efficiency_energy_aceleration_h',
    'share max of change vs historical mean rate':
    'share_max_of_change_vs_historical_mean_rate',
    'efficiency energy aceleration':
    'efficiency_energy_aceleration',
    'evol final energy intensity h':
    'evol_final_energy_intensity_h',
    'inertial rate energy intensity top down':
    'inertial_rate_energy_intensity_top_down',
    'historical mean rate energy intensity':
    'historical_mean_rate_energy_intensity',
    'inertial rate energy intensity h top down':
    'inertial_rate_energy_intensity_h_top_down',
    'p rr ti rest':
    'p_rr_ti_rest',
    'p rr li rest':
    'p_rr_li_rest',
    'p rr v rest':
    'p_rr_v_rest',
    'p rr mg rest':
    'p_rr_mg_rest',
    'p rr zn rest':
    'p_rr_zn_rest',
    'p rr minerals rest':
    'p_rr_minerals_rest',
    'p rr cd rest':
    'p_rr_cd_rest',
    'improvement recycling rates minerals rest':
    'improvement_recycling_rates_minerals_rest',
    'b lineal regr rr alt techn':
    'b_lineal_regr_rr_alt_techn',
    'b lineal regr rr rest':
    'b_lineal_regr_rr_rest',
    'by mineral rr rest 1yr':
    'by_mineral_rr_rest_1yr',
    'p rr cu rest':
    'p_rr_cu_rest',
    'a lineal regr rr alt techn':
    'a_lineal_regr_rr_alt_techn',
    'a lineal regr rr rest':
    'a_lineal_regr_rr_rest',
    'start year p rr minerals rest':
    'start_year_p_rr_minerals_rest',
    'common rr minerals variation rest':
    'common_rr_minerals_variation_rest',
    'p rr in rest':
    'p_rr_in_rest',
    'p rr te rest':
    'p_rr_te_rest',
    'p rr mo rest':
    'p_rr_mo_rest',
    'by mineral rr rest':
    'by_mineral_rr_rest',
    'p rr ag rest':
    'p_rr_ag_rest',
    'p rr mn rest':
    'p_rr_mn_rest',
    'p rr al rest':
    'p_rr_al_rest',
    'target year p rr minerals rest':
    'target_year_p_rr_minerals_rest',
    'p rr ga rest':
    'p_rr_ga_rest',
    'p rr sn rest':
    'p_rr_sn_rest',
    'p rr pb rest':
    'p_rr_pb_rest',
    'p rr cr rest':
    'p_rr_cr_rest',
    'p rr ni rest':
    'p_rr_ni_rest',
    'p rr fe rest':
    'p_rr_fe_rest',
    'by mineral rr variation rest':
    'by_mineral_rr_variation_rest',
    'current recycling rates minerals alt techn':
    'current_recycling_rates_minerals_alt_techn',
    'p rr mg alt techn':
    'p_rr_mg_alt_techn',
    'p rr minerals alt techn':
    'p_rr_minerals_alt_techn',
    'p rr mn alt techn':
    'p_rr_mn_alt_techn',
    'p rr mo alt techn':
    'p_rr_mo_alt_techn',
    'p rr ni alt techn':
    'p_rr_ni_alt_techn',
    'p rr pb alt techn':
    'p_rr_pb_alt_techn',
    'by mineral rr alt techn':
    'by_mineral_rr_alt_techn',
    'by mineral rr alt techn 1yr':
    'by_mineral_rr_alt_techn_1yr',
    'p rr al alt techn':
    'p_rr_al_alt_techn',
    'p rr cd alt techn':
    'p_rr_cd_alt_techn',
    'p rr cu alt techn':
    'p_rr_cu_alt_techn',
    'p rr fe alt techn':
    'p_rr_fe_alt_techn',
    'p rr ga alt techn':
    'p_rr_ga_alt_techn',
    'p rr in alt techn':
    'p_rr_in_alt_techn',
    'p rr li alt techn':
    'p_rr_li_alt_techn',
    'recycling rates minerals alt techn':
    'recycling_rates_minerals_alt_techn',
    'p rr v alt techn':
    'p_rr_v_alt_techn',
    'p rr zn alt techn':
    'p_rr_zn_alt_techn',
    'p rr ti alt techn':
    'p_rr_ti_alt_techn',
    'p rr cr alt techn':
    'p_rr_cr_alt_techn',
    'p rr ag alt techn':
    'p_rr_ag_alt_techn',
    'p rr sn alt techn':
    'p_rr_sn_alt_techn',
    'p rr te alt techn':
    'p_rr_te_alt_techn',
    'improvement recycling rates minerals alt techn':
    'improvement_recycling_rates_minerals_alt_techn',
    'by mineral rr variation alt techn':
    'by_mineral_rr_variation_alt_techn',
    'start year p rr minerals alt techn':
    'start_year_p_rr_minerals_alt_techn',
    'target year p rr minerals alt techn':
    'target_year_p_rr_minerals_alt_techn',
    'temp change x1x5c':
    'temp_change_x1x5c',
    'density':
    'density',
    'sec per day':
    'sec_per_day',
    'sec per yr':
    'sec_per_yr',
    'init atmos uocean temp':
    'init_atmos_uocean_temp',
    'watt per j s':
    'watt_per_j_s',
    'lower layer volume vu':
    'lower_layer_volume_vu',
    'volumetric heat capacity':
    'volumetric_heat_capacity',
    'mass heat cap':
    'mass_heat_cap',
    'heat diffusion covar':
    'heat_diffusion_covar',
    'x2x co2x forcing':
    'x2x_co2x_forcing',
    'climate sensitivity to 2x co2':
    'climate_sensitivity_to_2x_co2',
    'temperature change':
    'temperature_change',
    'land thickness':
    'land_thickness',
    'heat transfer':
    'heat_transfer',
    'climate feedback param':
    'climate_feedback_param',
    'area':
    'area',
    'heat transfer coeff':
    'heat_transfer_coeff',
    'eddy diff coeff':
    'eddy_diff_coeff',
    'upper layer volume vu':
    'upper_layer_volume_vu',
    'heat in deep ocean':
    'heat_in_deep_ocean',
    'land area fraction':
    'land_area_fraction',
    'relative deep ocean temp':
    'relative_deep_ocean_temp',
    'co2 rad force coeffcroads':
    'co2_rad_force_coeffcroads',
    'temp change x2x':
    'temp_change_x2x',
    'feedback cooling':
    'feedback_cooling',
    'heat transfer rate':
    'heat_transfer_rate',
    'atm and upper ocean heat cap':
    'atm_and_upper_ocean_heat_cap',
    'deep ocean heat cap':
    'deep_ocean_heat_cap',
    'init deep ocean temp':
    'init_deep_ocean_temp',
    'heat in atmosphere and upper ocean':
    'heat_in_atmosphere_and_upper_ocean',
    'ch4 n2o unit adj':
    'ch4_n2o_unit_adj',
    'adjustment for ch4ref and n2oref':
    'adjustment_for_ch4ref_and_n2oref',
    'ch4 radiative efficiency coefficient':
    'ch4_radiative_efficiency_coefficient',
    'total radiative forcing':
    'total_radiative_forcing',
    'last historical rf year':
    'last_historical_rf_year',
    'ch4 reference conc':
    'ch4_reference_conc',
    'rf from f gases':
    'rf_from_f_gases',
    'wellxdashxmixed ghg forcing':
    'wellxdashxmixed_ghg_forcing',
    'mineral aerosols and land rf':
    'mineral_aerosols_and_land_rf',
    'time to commit rf':
    'time_to_commit_rf',
    'ch4 n20 inter exp':
    'ch4_n20_inter_exp',
    'ch4 n20 inter exp x2x':
    'ch4_n20_inter_exp_x2x',
    'ch4 n2o inter coef x2x':
    'ch4_n2o_inter_coef_x2x',
    'adjusted other forcings':
    'adjusted_other_forcings',
    'adjustment for ch4 and n2oref':
    'adjustment_for_ch4_and_n2oref',
    'adjustment for ch4ref and n2o':
    'adjustment_for_ch4ref_and_n2o',
    'n2o reference conc':
    'n2o_reference_conc',
    'delay effective radiative forcing':
    'delay_effective_radiative_forcing',
    'effective radiative forcing':
    'effective_radiative_forcing',
    'co2 rad force':
    'co2_rad_force',
    'co2 radiative forcing':
    'co2_radiative_forcing',
    'ch4 n2o interaction coeffient':
    'ch4_n2o_interaction_coeffient',
    'n2o radiative efficiency coefficient':
    'n2o_radiative_efficiency_coefficient',
    'ch4 n2o inter coef x3x':
    'ch4_n2o_inter_coef_x3x',
    'n2o radiative forcing':
    'n2o_radiative_forcing',
    'ch4 radiative forcing':
    'ch4_radiative_forcing',
    'other ghg rad forcing xnon co2x':
    'other_ghg_rad_forcing_xnon_co2x',
    'ch4 and n2o radiative forcing':
    'ch4_and_n2o_radiative_forcing',
    'hfc rf total':
    'hfc_rf_total',
    'time const for hfc':
    'time_const_for_hfc',
    'time const for n2o':
    'time_const_for_n2o',
    'time const for pfc':
    'time_const_for_pfc',
    'sensitivity of methane emissions to permafrost and clathrate':
    'sensitivity_of_methane_emissions_to_permafrost_and_clathrate',
    'reference ch4 time constant':
    'reference_ch4_time_constant',
    'stratospheric ch4 path share':
    'stratospheric_ch4_path_share',
    'g per ton':
    'g_per_ton',
    'init pfc in atm con':
    'init_pfc_in_atm_con',
    'inital hfc con':
    'inital_hfc_con',
    'sf6':
    'sf6',
    'sf6 atm conc':
    'sf6_atm_conc',
    'sf6 molar mass':
    'sf6_molar_mass',
    'sf6 radiative efficiency':
    'sf6_radiative_efficiency',
    'sf6 rf':
    'sf6_rf',
    'sf6 uptake':
    'sf6_uptake',
    'initial ch4':
    'initial_ch4',
    'initial ch4 conc':
    'initial_ch4_conc',
    'pfc radiative efficiency':
    'pfc_radiative_efficiency',
    'ch4 emissions from permafrost and clathrate':
    'ch4_emissions_from_permafrost_and_clathrate',
    'hfc in atm':
    'hfc_in_atm',
    'hfc molar mass':
    'hfc_molar_mass',
    'hfc radiative efficiency':
    'hfc_radiative_efficiency',
    'ch4 fractional uptake':
    'ch4_fractional_uptake',
    'hfc uptake':
    'hfc_uptake',
    'total c from permafrost':
    'total_c_from_permafrost',
    'ch4 molar mass':
    'ch4_molar_mass',
    'n2oxdashxn molar mass':
    'n2oxdashxn_molar_mass',
    'ch4 uptake':
    'ch4_uptake',
    'total ch4 released':
    'total_ch4_released',
    'natural pfc emissions':
    'natural_pfc_emissions',
    'choose rcp':
    'choose_rcp',
    'pfc atm conc':
    'pfc_atm_conc',
    'ppb ch4 per mton ch4':
    'ppb_ch4_per_mton_ch4',
    'ppb n2o per mtonn':
    'ppb_n2o_per_mtonn',
    'ppt hfc per tons hfc':
    'ppt_hfc_per_tons_hfc',
    'ppt per mol':
    'ppt_per_mol',
    'ppt per ppb':
    'ppt_per_ppb',
    'ppt pfc per tons pfc':
    'ppt_pfc_per_tons_pfc',
    'c from ch4 oxidation':
    'c_from_ch4_oxidation',
    'reference sensitivity of ch4 from permafrost and clathrate to temperature':
    'reference_sensitivity_of_ch4_from_permafrost_and_clathrate_to_temperature',
    'global ch4 emissions':
    'global_ch4_emissions',
    'preindustrial ch4':
    'preindustrial_ch4',
    'preindustrial hfc conc':
    'preindustrial_hfc_conc',
    'pfc in atm':
    'pfc_in_atm',
    'initial sf6 con':
    'initial_sf6_con',
    'pfc rf':
    'pfc_rf',
    'pfc uptake':
    'pfc_uptake',
    'init pfc in atm':
    'init_pfc_in_atm',
    'n2o atm conc':
    'n2o_atm_conc',
    'n2o in atm':
    'n2o_in_atm',
    'n2o uptake':
    'n2o_uptake',
    'hfc atm conc':
    'hfc_atm_conc',
    'ton per mton':
    'ton_per_mton',
    'natural n2o emissions':
    'natural_n2o_emissions',
    'cf4 molar mass':
    'cf4_molar_mass',
    'ch4 atm conc':
    'ch4_atm_conc',
    'preindustrial sf6 conc':
    'preindustrial_sf6_conc',
    'time const for ch4':
    'time_const_for_ch4',
    'initial hfc':
    'initial_hfc',
    'flux c from permafrost release':
    'flux_c_from_permafrost_release',
    'global n2o emissions':
    'global_n2o_emissions',
    'tropospheric ch4 path share':
    'tropospheric_ch4_path_share',
    'preindustrial pfc':
    'preindustrial_pfc',
    'ppt sf6 per tons sf6':
    'ppt_sf6_per_tons_sf6',
    'hfc rf':
    'hfc_rf',
    'initial n2o':
    'initial_n2o',
    'initial n2o conc':
    'initial_n2o_conc',
    'temperature threshold for methane emissions from permafrost and clathrate':
    'temperature_threshold_for_methane_emissions_from_permafrost_and_clathrate',
    'time const for sf6':
    'time_const_for_sf6',
    'preindustrial pfc conc':
    'preindustrial_pfc_conc',
    'initial sf6':
    'initial_sf6',
    'reference sensitivity of c from permafrost and clathrate to temperature':
    'reference_sensitivity_of_c_from_permafrost_and_clathrate_to_temperature',
    'global total pfc emissions':
    'global_total_pfc_emissions',
    'ch4 in atm':
    'ch4_in_atm',
    'global anthropogenic ch4 emissions':
    'global_anthropogenic_ch4_emissions',
    'sensitivity of pco2 dic to temperature':
    'sensitivity_of_pco2_dic_to_temperature',
    'sensitivity of pco2 dic to temperature mean':
    'sensitivity_of_pco2_dic_to_temperature_mean',
    'mean depth of adjacent layers':
    'mean_depth_of_adjacent_layers',
    'diffusion flux':
    'diffusion_flux',
    'init c in biomass':
    'init_c_in_biomass',
    'init c in deep ocean per meter':
    'init_c_in_deep_ocean_per_meter',
    'init c in humus':
    'init_c_in_humus',
    'flux atm to biomass':
    'flux_atm_to_biomass',
    'flux atm to ocean':
    'flux_atm_to_ocean',
    'flux biomass to atmosphere':
    'flux_biomass_to_atmosphere',
    'flux biomass to ch4':
    'flux_biomass_to_ch4',
    'flux biomass to humus':
    'flux_biomass_to_humus',
    'flux biosphere to ch4':
    'flux_biosphere_to_ch4',
    'flux humus to atmosphere':
    'flux_humus_to_atmosphere',
    'co2 ppm concentrations':
    'co2_ppm_concentrations',
    'total c anthro emissions':
    'total_c_anthro_emissions',
    'effect of temp on dic pco2':
    'effect_of_temp_on_dic_pco2',
    'effect of warming on c flux to biomass':
    'effect_of_warming_on_c_flux_to_biomass',
    'effect of warming on ch4 release from biological activity':
    'effect_of_warming_on_ch4_release_from_biological_activity',
    'c in mixed layer per meter':
    'c_in_mixed_layer_per_meter',
    'biomass res time':
    'biomass_res_time',
    'biostim coeff':
    'biostim_coeff',
    'mtons per gtons':
    'mtons_per_gtons',
    'biostim coeff mean':
    'biostim_coeff_mean',
    'buff c coeff':
    'buff_c_coeff',
    'natural ch4 emissions':
    'natural_ch4_emissions',
    'humification fraction':
    'humification_fraction',
    'humus res time':
    'humus_res_time',
    'layer depth':
    'layer_depth',
    'ref buffer factor':
    'ref_buffer_factor',
    'reference temperature change for effect of warming on ch4 from respiration':
    'reference_temperature_change_for_effect_of_warming_on_ch4_from_respiration',
    'c in humus':
    'c_in_humus',
    'c in mixed layer':
    'c_in_mixed_layer',
    'init c in atmos':
    'init_c_in_atmos',
    'strength of temp effect on land c flux mean':
    'strength_of_temp_effect_on_land_c_flux_mean',
    'pre industrial value ppm':
    'pre_industrial_value_ppm',
    'biostim coeff index':
    'biostim_coeff_index',
    'init c in mixed ocean per meter':
    'init_c_in_mixed_ocean_per_meter',
    'init co2 in atmos ppm':
    'init_co2_in_atmos_ppm',
    'init npp':
    'init_npp',
    'buffer factor':
    'buffer_factor',
    'c in atmosphere':
    'c_in_atmosphere',
    'mixed depth':
    'mixed_depth',
    'mixing time':
    'mixing_time',
    'c in deep ocean per meter':
    'c_in_deep_ocean_per_meter',
    'elf concentrations logistic':
    'elf_concentrations_logistic',
    'strength of temp effect on c flux to land':
    'strength_of_temp_effect_on_c_flux_to_land',
    'ch4 generation rate from humus':
    'ch4_generation_rate_from_humus',
    'preind c in mixed layer':
    'preind_c_in_mixed_layer',
    'preind ocean c per meter':
    'preind_ocean_c_per_meter',
    'preindustrial c':
    'preindustrial_c',
    'equilibrium c per meter in mixed layer':
    'equilibrium_c_per_meter_in_mixed_layer',
    'sensitivity of c uptake to temperature':
    'sensitivity_of_c_uptake_to_temperature',
    'ch4 generation rate from biomass':
    'ch4_generation_rate_from_biomass',
    'ch4 per c':
    'ch4_per_c',
    'flux humus to ch4':
    'flux_humus_to_ch4',
    'sensitivity of methane emissions to temperature':
    'sensitivity_of_methane_emissions_to_temperature',
    'eddy diff coeff index':
    'eddy_diff_coeff_index',
    'c in biomass':
    'c_in_biomass',
    'gtc per ppm':
    'gtc_per_ppm',
    'c in deep ocean':
    'c_in_deep_ocean',
    'equil c in mixed layer':
    'equil_c_in_mixed_layer',
    'layer time constant':
    'layer_time_constant',
    'eddy diff mean':
    'eddy_diff_mean',
    'common rr minerals variation alt techn':
    'common_rr_minerals_variation_alt_techn',
    'choose targets mineral recycling rates':
    'choose_targets_mineral_recycling_rates',
    'total water use per capita':
    'total_water_use_per_capita',
    'total water use':
    'total_water_use',
    'water use per type per capita':
    'water_use_per_type_per_capita',
    'variation water intensity by sector':
    'variation_water_intensity_by_sector',
    'historic water intensities by sector delayed 1yr':
    'historic_water_intensities_by_sector_delayed_1yr',
    'historic water intensities for households delayed 1yr':
    'historic_water_intensities_for_households_delayed_1yr',
    'variation water intensity households':
    'variation_water_intensity_households',
    'water intensity for households':
    'water_intensity_for_households',
    'total water use by type':
    'total_water_use_by_type',
    'total water for oxandxm required by res elec dam3':
    'total_water_for_oxandxm_required_by_res_elec_dam3',
    'initial water intensity for households':
    'initial_water_intensity_for_households',
    'water intensity by sector':
    'water_intensity_by_sector',
    'water use by sector':
    'water_use_by_sector',
    'water use by households':
    'water_use_by_households',
    'modern bioe in households':
    'modern_bioe_in_households',
    'tfec res ej':
    'tfec_res_ej',
    'pe traditional biomass ej delayed 1yr':
    'pe_traditional_biomass_ej_delayed_1yr',
    'pe traditional biomass demand ej':
    'pe_traditional_biomass_demand_ej',
    'population dependent on trad biomass':
    'population_dependent_on_trad_biomass',
    'mt to dam3':
    'mt_to_dam3',
    'total water for oxandxm required by res elec':
    'total_water_for_oxandxm_required_by_res_elec',
    'initial water intensity by sector':
    'initial_water_intensity_by_sector',
    'initial water use by households':
    'initial_water_use_by_households',
    'initial water use by sector':
    'initial_water_use_by_sector',
    'max potential npp bioe conventional for heatxplusxelec':
    'max_potential_npp_bioe_conventional_for_heatxplusxelec',
    'total transport fed by fuel':
    'total_transport_fed_by_fuel',
    'diff demand':
    'diff_demand',
    'variation lc':
    'variation_lc',
    'variation cc':
    'variation_cc',
    'demand by sector fd adjusted':
    'demand_by_sector_fd_adjusted',
    'total demand adjusted':
    'total_demand_adjusted',
    'desired gdp':
    'desired_gdp',
    'demand by sector':
    'demand_by_sector',
    'aux4':
    'aux4',
    'x1x year':
    'x1x_year',
    'eptb dynamic':
    'eptb_dynamic',
    'eroist system delayed':
    'eroist_system_delayed',
    'eroist system delayed 1yr':
    'eroist_system_delayed_1yr',
    'pipeline transport constant x2x6x ej in x2014':
    'pipeline_transport_constant_x2x6x_ej_in_x2014',
    'total distribution losses':
    'total_distribution_losses',
    'ia matrix x2002x':
    'ia_matrix_x2002x',
    'ia matrix x2003x':
    'ia_matrix_x2003x',
    'ia matrix x2004x':
    'ia_matrix_x2004x',
    'ia matrix x2005x':
    'ia_matrix_x2005x',
    'ia matrix x2006x':
    'ia_matrix_x2006x',
    'ia matrix x2007x':
    'ia_matrix_x2007x',
    'ia matrix x2008x':
    'ia_matrix_x2008x',
    'ia matrix for python':
    'ia_matrix_for_python',
    'ia matrix x1995x':
    'ia_matrix_x1995x',
    'ia matrix x1996x':
    'ia_matrix_x1996x',
    'ia matrix x1997x':
    'ia_matrix_x1997x',
    'ia matrix x1998x':
    'ia_matrix_x1998x',
    'ia matrix x1999x':
    'ia_matrix_x1999x',
    'ia matrix x2000x':
    'ia_matrix_x2000x',
    'ia matrix x2001x':
    'ia_matrix_x2001x',
    'ia matrix x2009x':
    'ia_matrix_x2009x',
    'leontief matrix for python':
    'leontief_matrix_for_python',
    'leontief matrix x1996x':
    'leontief_matrix_x1996x',
    'leontief matrix x1997x':
    'leontief_matrix_x1997x',
    'leontief matrix x1998x':
    'leontief_matrix_x1998x',
    'leontief matrix x1999x':
    'leontief_matrix_x1999x',
    'leontief matrix x2000x':
    'leontief_matrix_x2000x',
    'leontief matrix x2001x':
    'leontief_matrix_x2001x',
    'leontief matrix x2002x':
    'leontief_matrix_x2002x',
    'leontief matrix x2003x':
    'leontief_matrix_x2003x',
    'leontief matrix x2004x':
    'leontief_matrix_x2004x',
    'leontief matrix x2005x':
    'leontief_matrix_x2005x',
    'leontief matrix x2006x':
    'leontief_matrix_x2006x',
    'leontief matrix x2007x':
    'leontief_matrix_x2007x',
    'leontief matrix x2008x':
    'leontief_matrix_x2008x',
    'leontief matrix x2009x':
    'leontief_matrix_x2009x',
    'leontief matrix x1995x':
    'leontief_matrix_x1995x',
    'replacement batteries':
    'replacement_batteries',
    'share of electric light vehicles':
    'share_of_electric_light_vehicles',
    'share elecxplusxhyb light vehicles':
    'share_elecxplusxhyb_light_vehicles',
    'aux p inland transp':
    'aux_p_inland_transp',
    't fin inlandt':
    't_fin_inlandt',
    'h elec initial growth':
    'h_elec_initial_growth',
    't fin h veh':
    't_fin_h_veh',
    'h gas initial growth':
    'h_gas_initial_growth',
    't ini h veh':
    't_ini_h_veh',
    'h hyb initial growth':
    'h_hyb_initial_growth',
    'policy 2wheels':
    'policy_2wheels',
    'h 2w initial growth':
    'h_2w_initial_growth',
    'var percents h vehicles':
    'var_percents_h_vehicles',
    'increase households energy final demand for transp':
    'increase_households_energy_final_demand_for_transp',
    'start year p common rr minerals alt techn':
    'start_year_p_common_rr_minerals_alt_techn',
    'start year p common rr minerals rest':
    'start_year_p_common_rr_minerals_rest',
    'select pop ssps':
    'select_pop_ssps',
    'dynamic low range fec good standard of living':
    'dynamic_low_range_fec_good_standard_of_living',
    'cp ev batteries for elec storage':
    'cp_ev_batteries_for_elec_storage',
    'dynamic threshold xhigh developmentx':
    'dynamic_threshold_xhigh_developmentx',
    'max cp ev batteries for elec storage':
    'max_cp_ev_batteries_for_elec_storage',
    'tfec intensity until x2009x':
    'tfec_intensity_until_x2009x',
    'desired annual gdp growth rate':
    'desired_annual_gdp_growth_rate',
    'cumulative tfec intensity change from x2009x':
    'cumulative_tfec_intensity_change_from_x2009x',
    'lifetime ev batteries':
    'lifetime_ev_batteries',
    'dynamic high range fec good standard of living':
    'dynamic_high_range_fec_good_standard_of_living',
    'tfec per capita':
    'tfec_per_capita',
    'aux15':
    'aux15',
    'cp ev batteries for transp':
    'cp_ev_batteries_for_transp',
    'p customized cte gdppc variation':
    'p_customized_cte_gdppc_variation',
    'annual growth rate electricity generation res elec tot':
    'annual_growth_rate_electricity_generation_res_elec_tot',
    'annual growth rate res for heat':
    'annual_growth_rate_res_for_heat',
    'p customized year pop evolution':
    'p_customized_year_pop_evolution',
    'implementatio policy to change final energy':
    'implementatio_policy_to_change_final_energy',
    'implementatio policy to change final energy h':
    'implementatio_policy_to_change_final_energy_h',
    'implementation policy to improve energy intensity effciency':
    'implementation_policy_to_improve_energy_intensity_effciency',
    'select gdppc evolution input':
    'select_gdppc_evolution_input',
    'select population evolution input':
    'select_population_evolution_input',
    'p customized year gdppc evolution':
    'p_customized_year_gdppc_evolution',
    'implementation policy to improve energy intensity effciency h':
    'implementation_policy_to_improve_energy_intensity_effciency_h',
    'fe tot generation all res elec twh delayed 1yr':
    'fe_tot_generation_all_res_elec_twh_delayed_1yr',
    'p customized cte pop variation':
    'p_customized_cte_pop_variation',
    'fes res for heat delayed 1yr':
    'fes_res_for_heat_delayed_1yr',
    'exp slow evol change energy':
    'exp_slow_evol_change_energy',
    'year to finish policy improve efficiency':
    'year_to_finish_policy_improve_efficiency',
    'exp slow evolution improve efficiency':
    'exp_slow_evolution_improve_efficiency',
    'exp rapid evolution improve efficiency':
    'exp_rapid_evolution_improve_efficiency',
    'exp rapid evol change energy':
    'exp_rapid_evol_change_energy',
    'year to finish policy change energy':
    'year_to_finish_policy_change_energy',
    'year to finish policy improve efficiency h':
    'year_to_finish_policy_improve_efficiency_h',
    'year to finish policy change energy h':
    'year_to_finish_policy_change_energy_h',
    'exp rapid evolution change energy h':
    'exp_rapid_evolution_change_energy_h',
    'pressure to improve energy intensity efficiency h':
    'pressure_to_improve_energy_intensity_efficiency_h',
    'exp slow evolution change energy h':
    'exp_slow_evolution_change_energy_h',
    'exp rapid evolution improve efficiency h':
    'exp_rapid_evolution_improve_efficiency_h',
    'exp slow evolution improve efficiency h':
    'exp_slow_evolution_improve_efficiency_h',
    'desired gdp delayed 1yr':
    'desired_gdp_delayed_1yr',
    'diff annual gdp growth rate':
    'diff_annual_gdp_growth_rate',
    'gdppc initial year':
    'gdppc_initial_year',
    'desired gdppc':
    'desired_gdppc',
    'tpes intensity until x2009x':
    'tpes_intensity_until_x2009x',
    'sharexdashxe losses cc until x2015':
    'sharexdashxe_losses_cc_until_x2015',
    'aux13':
    'aux13',
    'aux14':
    'aux14',
    'share exdashxlosses cc from x2015':
    'share_exdashxlosses_cc_from_x2015',
    'elf':
    'elf',
    'cumulative tpes intensity change from x2009x':
    'cumulative_tpes_intensity_change_from_x2009x',
    'cumulative tfec intensity change from x2009x without eroi':
    'cumulative_tfec_intensity_change_from_x2009x_without_eroi',
    'tfec intensity until x2009x without eroi':
    'tfec_intensity_until_x2009x_without_eroi',
    'tfei sectors':
    'tfei_sectors',
    'annual tfes intensity growth rate without eroi':
    'annual_tfes_intensity_growth_rate_without_eroi',
    'final energy intensity by sector and fuel':
    'final_energy_intensity_by_sector_and_fuel',
    'energy intensity of households':
    'energy_intensity_of_households',
    'tfes intensity without eroi delayed 1yr':
    'tfes_intensity_without_eroi_delayed_1yr',
    'households total final energy demand':
    'households_total_final_energy_demand',
    'required tfed sectors':
    'required_tfed_sectors',
    'ratio fed households vs sectors':
    'ratio_fed_households_vs_sectors',
    'tfes intensity ej tdollar without eroi':
    'tfes_intensity_ej_tdollar_without_eroi',
    'ghg emissions x2050x mlt2030':
    'ghg_emissions_x2050x_mlt2030',
    'ghg emissions x2050x mlt2020':
    'ghg_emissions_x2050x_mlt2020',
    'start year p growth res elec':
    'start_year_p_growth_res_elec',
    'start year p growth res heat':
    'start_year_p_growth_res_heat',
    'labour share growth':
    'labour_share_growth',
    'year initial labour share':
    'year_initial_labour_share',
    'year final labour share':
    'year_final_labour_share',
    'p labour share x2050x':
    'p_labour_share_x2050x',
    'initial labour share':
    'initial_labour_share',
    'low range fec good standard of living':
    'low_range_fec_good_standard_of_living',
    'threshold fec xhigh developmentx':
    'threshold_fec_xhigh_developmentx',
    'high range fec good standard of living':
    'high_range_fec_good_standard_of_living',
    'decrease of intensity due to energy a technology change top down':
    'decrease_of_intensity_due_to_energy_a_technology_change_top_down',
    'maximun yearly aceleration of intensity improvement pct':
    'maximun_yearly_aceleration_of_intensity_improvement_pct',
    'policy change energy speed':
    'policy_change_energy_speed',
    'policy to improve efficiency speed':
    'policy_to_improve_efficiency_speed',
    'minimum fraction':
    'minimum_fraction',
    'increase of intensity due to energy a technology change top down':
    'increase_of_intensity_due_to_energy_a_technology_change_top_down',
    'year policy change energy':
    'year_policy_change_energy',
    'max yearly change':
    'max_yearly_change',
    'efficiency rate of substitution':
    'efficiency_rate_of_substitution',
    'activate bottom up method':
    'activate_bottom_up_method',
    'year policy to improve efficiency':
    'year_policy_to_improve_efficiency',
    'decrease of intensity due to energy a technology change h top down':
    'decrease_of_intensity_due_to_energy_a_technology_change_h_top_down',
    'increase of intensity due to energy a technology change h top down':
    'increase_of_intensity_due_to_energy_a_technology_change_h_top_down',
    'total co2e xgwpxequalsx100x yearsx':
    'total_co2e_xgwpxequalsx100x_yearsx',
    'net tfec per capita':
    'net_tfec_per_capita',
    'gwp x100x years ch4':
    'gwp_x100x_years_ch4',
    'activate energy scarcity feedbackxquestx':
    'activate_energy_scarcity_feedbackxquestx',
    'max potential phs twh':
    'max_potential_phs_twh',
    'replacement rate phs':
    'replacement_rate_phs',
    'energy scarcity feedback shortage coeff':
    'energy_scarcity_feedback_shortage_coeff',
    'real fe elec stored ev batteries twh':
    'real_fe_elec_stored_ev_batteries_twh',
    'share tot fei res elec var':
    'share_tot_fei_res_elec_var',
    'cp ev batteries required':
    'cp_ev_batteries_required',
    'output ev bateries for storage over lifetime':
    'output_ev_bateries_for_storage_over_lifetime',
    'discarded batteries':
    'discarded_batteries',
    'esoi elec storage':
    'esoi_elec_storage',
    'esoi ev batteries':
    'esoi_ev_batteries',
    'rt elec storage efficiency':
    'rt_elec_storage_efficiency',
    'xxabundancexx storage':
    'xxabundancexx_storage',
    'used ev batteries for elec storage':
    'used_ev_batteries_for_elec_storage',
    'max cp ev batteries':
    'max_cp_ev_batteries',
    'total capacity elec storage tw':
    'total_capacity_elec_storage_tw',
    'demand ev batteries for elec storage':
    'demand_ev_batteries_for_elec_storage',
    'ced per tw over lifetime phs':
    'ced_per_tw_over_lifetime_phs',
    'remaining potential constraint on new phs capacity':
    'remaining_potential_constraint_on_new_phs_capacity',
    'new required phs capacity':
    'new_required_phs_capacity',
    'share dyn fei for res vs tfec':
    'share_dyn_fei_for_res_vs_tfec',
    'fei ev batteries':
    'fei_ev_batteries',
    'output phs over lifetime':
    'output_phs_over_lifetime',
    'esoi static phs':
    'esoi_static_phs',
    'total dyn fei res':
    'total_dyn_fei_res',
    'adapt growth res elec after allocation':
    'adapt_growth_res_elec_after_allocation',
    'esoi phs':
    'esoi_phs',
    'p phs growth':
    'p_phs_growth',
    'constraint elec storage availability':
    'constraint_elec_storage_availability',
    'eroist system':
    'eroist_system',
    'initial instal cap phs':
    'initial_instal_cap_phs',
    'cedtot over lifetime phs':
    'cedtot_over_lifetime_phs',
    'installed capacity phs tw':
    'installed_capacity_phs_tw',
    'phs capacity under construction':
    'phs_capacity_under_construction',
    'past phs capacity growth':
    'past_phs_capacity_growth',
    'new phs capacity under planning':
    'new_phs_capacity_under_planning',
    'wear phs':
    'wear_phs',
    'replacement capacity phs':
    'replacement_capacity_phs',
    'real fe elec stored phs twh':
    'real_fe_elec_stored_phs_twh',
    'phs planned capacity':
    'phs_planned_capacity',
    'final energy invested phs':
    'final_energy_invested_phs',
    'remaining potential phs':
    'remaining_potential_phs',
    'initial capacity in construction phs':
    'initial_capacity_in_construction_phs',
    'required capacity phs':
    'required_capacity_phs',
    'total fei over lifetime res elec dispatch':
    'total_fei_over_lifetime_res_elec_dispatch',
    'min lifetime ev batteries':
    'min_lifetime_ev_batteries',
    'total final energy invested res elec var':
    'total_final_energy_invested_res_elec_var',
    'materials required for ev batteries mt':
    'materials_required_for_ev_batteries_mt',
    'newxplusxreplaced batteries tw':
    'newxplusxreplaced_batteries_tw',
    'energy intensity construction ev batteries mjxdivxmw':
    'energy_intensity_construction_ev_batteries_mjxdivxmw',
    'new batteries':
    'new_batteries',
    'batteries evxplusxhibxplusx2we':
    'batteries_evxplusxhibxplusx2we',
    'kw per mw':
    'kw_per_mw',
    'net stored energy ev battery over lifetime':
    'net_stored_energy_ev_battery_over_lifetime',
    'grid correction factor ev batteries':
    'grid_correction_factor_ev_batteries',
    'share energy requirements for decom ev batteries':
    'share_energy_requirements_for_decom_ev_batteries',
    'working hours per year':
    'working_hours_per_year',
    'days per year':
    'days_per_year',
    'total energy required for material consumption for res elec':
    'total_energy_required_for_material_consumption_for_res_elec',
    'total energy required for material consumption per res elec':
    'total_energy_required_for_material_consumption_per_res_elec',
    'mw in x1x year to mj':
    'mw_in_x1x_year_to_mj',
    'working hours per day':
    'working_hours_per_day',
    'annual work hours for res':
    'annual_work_hours_for_res',
    'hours work per gj res delivered':
    'hours_work_per_gj_res_delivered',
    'total energy required for total material consumption for ev batteries':
    'total_energy_required_for_total_material_consumption_for_ev_batteries',
    'tfe required for total material consumption for alt techn':
    'tfe_required_for_total_material_consumption_for_alt_techn',
    'energy required for material consumption per res elec':
    'energy_required_for_material_consumption_per_res_elec',
    'total energy required per material for alt techn':
    'total_energy_required_per_material_for_alt_techn',
    'total jobs res elec':
    'total_jobs_res_elec',
    'total jobs biofuels':
    'total_jobs_biofuels',
    'total jobs res':
    'total_jobs_res',
    'total dxplusxi jobs res heat per techn':
    'total_dxplusxi_jobs_res_heat_per_techn',
    'total jobs res heat':
    'total_jobs_res_heat',
    'total dxplusxi jobs res elec per techn':
    'total_dxplusxi_jobs_res_elec_per_techn',
    'ratio total vs d jobs res heat':
    'ratio_total_vs_d_jobs_res_heat',
    'ratio total vs d jobs res elec':
    'ratio_total_vs_d_jobs_res_elec',
    'employment factor biofuels':
    'employment_factor_biofuels',
    'installed capacity res elec delayed 1yr':
    'installed_capacity_res_elec_delayed_1yr',
    'jobs oxandxm res heat per techn':
    'jobs_oxandxm_res_heat_per_techn',
    'd jobs new installed res heat per techn':
    'd_jobs_new_installed_res_heat_per_techn',
    'd jobs fuel supply solids bioe':
    'd_jobs_fuel_supply_solids_bioe',
    'new capacity installed growth rate res elec':
    'new_capacity_installed_growth_rate_res_elec',
    'employment factor fuel supply solids bioe':
    'employment_factor_fuel_supply_solids_bioe',
    'exogenous growth gtl':
    'exogenous_growth_gtl',
    'replacement gtl':
    'replacement_gtl',
    'crash programme gtlxquestx':
    'crash_programme_gtlxquestx',
    'exogenous growth ctl':
    'exogenous_growth_ctl',
    'replacement ctl':
    'replacement_ctl',
    'aux12':
    'aux12',
    'coal to leave underground':
    'coal_to_leave_underground',
    'rurr coal start year plg':
    'rurr_coal_start_year_plg',
    'rurr conv oil until start year plg':
    'rurr_conv_oil_until_start_year_plg',
    'conv gas to leave underground':
    'conv_gas_to_leave_underground',
    'conv oil to leave underground':
    'conv_oil_to_leave_underground',
    'rurr unconv oil until start year plg':
    'rurr_unconv_oil_until_start_year_plg',
    'rurr unconv gas until start year plg':
    'rurr_unconv_gas_until_start_year_plg',
    'unconv gas to leave underground':
    'unconv_gas_to_leave_underground',
    'tot agg gas to leave underground':
    'tot_agg_gas_to_leave_underground',
    'unconv oil to leave underground':
    'unconv_oil_to_leave_underground',
    'aux11':
    'aux11',
    'tot agg oil to leave underground':
    'tot_agg_oil_to_leave_underground',
    'aux6':
    'aux6',
    'aux7':
    'aux7',
    'aux8':
    'aux8',
    'aux9':
    'aux9',
    'rurr conv gas until start year plg':
    'rurr_conv_gas_until_start_year_plg',
    'aux10':
    'aux10',
    'rurr tot gas until start year plg':
    'rurr_tot_gas_until_start_year_plg',
    'rurr tot oil until start year plg':
    'rurr_tot_oil_until_start_year_plg',
    'required fed by fuel before heat correction':
    'required_fed_by_fuel_before_heat_correction',
    'share elec demand covered by res':
    'share_elec_demand_covered_by_res',
    'rurr conv gas':
    'rurr_conv_gas',
    'rurr unconv gas':
    'rurr_unconv_gas',
    'rurr conv oil':
    'rurr_conv_oil',
    'variation share transmxandxdistr losses elec':
    'variation_share_transmxandxdistr_losses_elec',
    'share demand solids in transport':
    'share_demand_solids_in_transport',
    'share demand gas in transport':
    'share_demand_gas_in_transport',
    'share demand by fuel in transport':
    'share_demand_by_fuel_in_transport',
    'share demand electricity in transport':
    'share_demand_electricity_in_transport',
    'share demand heat in transport':
    'share_demand_heat_in_transport',
    'share demand liquids in transport':
    'share_demand_liquids_in_transport',
    'co2 emissions per value added':
    'co2_emissions_per_value_added',
    'liquids per x bus':
    'liquids_per_x_bus',
    'liquids per x hv':
    'liquids_per_x_hv',
    'liquids per x lv':
    'liquids_per_x_lv',
    'energy per x train':
    'energy_per_x_train',
    'adjust energy for transport to inland transport':
    'adjust_energy_for_transport_to_inland_transport',
    'effects shortage elec on evxdivxhib':
    'effects_shortage_elec_on_evxdivxhib',
    'hist var percent h':
    'hist_var_percent_h',
    'required number standard batteries':
    'required_number_standard_batteries',
    'transport tfed energy intensity':
    'transport_tfed_energy_intensity',
    'total number light vehicles':
    'total_number_light_vehicles',
    'transport tfed':
    'transport_tfed',
    'transport households final energy demand':
    'transport_households_final_energy_demand',
    'real fe consumption liquids ej':
    'real_fe_consumption_liquids_ej',
    'co2 emissions unconv gas':
    'co2_emissions_unconv_gas',
    'phasexdashxout oil for electricityxquestx':
    'phasexdashxout_oil_for_electricityxquestx',
    'max solar on land mha':
    'max_solar_on_land_mha',
    'max solar pv on land mha':
    'max_solar_pv_on_land_mha',
    'max solar pv on land twe':
    'max_solar_pv_on_land_twe',
    'total ch4 emissions fossil fuels':
    'total_ch4_emissions_fossil_fuels',
    'max csp on land mha':
    'max_csp_on_land_mha',
    'max csp twe':
    'max_csp_twe',
    'gch4 per mj gtl':
    'gch4_per_mj_gtl',
    'ch4 emissions gtl':
    'ch4_emissions_gtl',
    'ch4 emissions ctl':
    'ch4_emissions_ctl',
    'phasexdashxout oil for heatxdashxcomxquestx':
    'phasexdashxout_oil_for_heatxdashxcomxquestx',
    'gch4 per mj ctl':
    'gch4_per_mj_ctl',
    'g per gt':
    'g_per_gt',
    'bioe co2 emissions':
    'bioe_co2_emissions',
    'maximum 2w':
    'maximum_2w',
    'p h vehicle':
    'p_h_vehicle',
    'solid biofuels emissions relevant ej':
    'solid_biofuels_emissions_relevant_ej',
    'eroistx1995x':
    'eroistx1995x',
    'eroist system until x2015x':
    'eroist_system_until_x2015x',
    'remaining potential elec storage by res techn':
    'remaining_potential_elec_storage_by_res_techn',
    'cp hydro x2015x':
    'cp_hydro_x2015x',
    'esoi phs full potential':
    'esoi_phs_full_potential',
    'max capacity elec storage':
    'max_capacity_elec_storage',
    'cc impacts feedback shortage coeff':
    'cc_impacts_feedback_shortage_coeff',
    'eolxdashxrr minerals alt techn res vsx total economy':
    'eolxdashxrr_minerals_alt_techn_res_vsx_total_economy',
    'crash programme ctlxquestx':
    'crash_programme_ctlxquestx',
    'recycling rates minerals rest':
    'recycling_rates_minerals_rest',
    'p elec':
    'p_elec',
    'percents h vehicles':
    'percents_h_vehicles',
    't hist h transp':
    't_hist_h_transp',
    'bat number 2w':
    'bat_number_2w',
    'bateries ratio 2w e':
    'bateries_ratio_2w_e',
    'bat number ev':
    'bat_number_ev',
    'bat number hib':
    'bat_number_hib',
    'bateries ratio bus e':
    'bateries_ratio_bus_e',
    'bateries ratio hib bus':
    'bateries_ratio_hib_bus',
    'bateries ratio hib hv':
    'bateries_ratio_hib_hv',
    'bateries ratio hib lv':
    'bateries_ratio_hib_lv',
    'total number hybrid light vehicles':
    'total_number_hybrid_light_vehicles',
    'total number elec light vehicles':
    'total_number_elec_light_vehicles',
    'total number gas light vehicles':
    'total_number_gas_light_vehicles',
    'ev batteries tw':
    'ev_batteries_tw',
    'var percent t vehicles':
    'var_percent_t_vehicles',
    'p train elec':
    'p_train_elec',
    'p hv gas':
    'p_hv_gas',
    't ini inlandt':
    't_ini_inlandt',
    'p inlandt':
    'p_inlandt',
    'activate policy inlandt':
    'activate_policy_inlandt',
    'p lv elec':
    'p_lv_elec',
    'p lv hyb':
    'p_lv_hyb',
    'p bus elec':
    'p_bus_elec',
    'p bus gas':
    'p_bus_gas',
    'p bus hyb':
    'p_bus_hyb',
    'percents bus':
    'percents_bus',
    'percents train':
    'percents_train',
    'p hv hyb':
    'p_hv_hyb',
    'adapt var inlandt':
    'adapt_var_inlandt',
    'percents lv':
    'percents_lv',
    'var i inland elec':
    'var_i_inland_elec',
    'var i inlandt gas':
    'var_i_inlandt_gas',
    'var i inlandt liq':
    'var_i_inlandt_liq',
    'shares available t':
    'shares_available_t',
    'vehicles inlandt':
    'vehicles_inlandt',
    'percent t vehicles':
    'percent_t_vehicles',
    'efects shortage inlandt':
    'efects_shortage_inlandt',
    'p lv gas':
    'p_lv_gas',
    'percents hv':
    'percents_hv',
    'initial xt inland':
    'initial_xt_inland',
    'nx bus inlandt':
    'nx_bus_inlandt',
    'nx hv inland t':
    'nx_hv_inland_t',
    'nx lv inland t':
    'nx_lv_inland_t',
    'n vehicles inland t x0x':
    'n_vehicles_inland_t_x0x',
    'energy initial inland transport':
    'energy_initial_inland_transport',
    'energy per x t':
    'energy_per_x_t',
    'nx train inland t':
    'nx_train_inland_t',
    'a2 coef th':
    'a2_coef_th',
    'h gas adapt growth':
    'h_gas_adapt_growth',
    'h hyb adapt growth':
    'h_hyb_adapt_growth',
    'activate policy h transp':
    'activate_policy_h_transp',
    'initial energy intensity of households transport x2009x':
    'initial_energy_intensity_of_households_transport_x2009x',
    'h ev adapt growth':
    'h_ev_adapt_growth',
    'var ih e2':
    'var_ih_e2',
    'var ih gas2':
    'var_ih_gas2',
    'a1 coef th':
    'a1_coef_th',
    'var ih liq2':
    'var_ih_liq2',
    'percent h vehicles initial':
    'percent_h_vehicles_initial',
    'energy intensity of households transport':
    'energy_intensity_of_households_transport',
    'h 2we adapt growth':
    'h_2we_adapt_growth',
    'liq 4w':
    'liq_4w',
    'percent 2w liq':
    'percent_2w_liq',
    'percent 4w liq':
    'percent_4w_liq',
    'electricity 2we':
    'electricity_2we',
    'n vehicles h':
    'n_vehicles_h',
    'demand h':
    'demand_h',
    'ratio n veh demand h':
    'ratio_n_veh_demand_h',
    'percents 2w h vehicles':
    'percents_2w_h_vehicles',
    'percents 4w h vehicles':
    'percents_4w_h_vehicles',
    'p 2we':
    'p_2we',
    'max percent x2x wheels':
    'max_percent_x2x_wheels',
    'max percent x4x wheels':
    'max_percent_x4x_wheels',
    'aux hist h':
    'aux_hist_h',
    'p gas':
    'p_gas',
    'p share x2x wheelers':
    'p_share_x2x_wheelers',
    'p hyb':
    'p_hyb',
    'initial 2w percent':
    'initial_2w_percent',
    'share available 2w':
    'share_available_2w',
    'rate 4w to 2w':
    'rate_4w_to_2w',
    'share feh over fed by final fuel':
    'share_feh_over_fed_by_final_fuel',
    'share feh over fed oil':
    'share_feh_over_fed_oil',
    'share feh over fed coal':
    'share_feh_over_fed_coal',
    'share feh over fed natx gas':
    'share_feh_over_fed_natx_gas',
    'pes oil ej delayed':
    'pes_oil_ej_delayed',
    'energy required for material consumption for ev batteries':
    'energy_required_for_material_consumption_for_ev_batteries',
    'households final energy demand':
    'households_final_energy_demand',
    'change total intensity to rest':
    'change_total_intensity_to_rest',
    'activate transport h bottom up method':
    'activate_transport_h_bottom_up_method',
    'variation energy intensity of households transport':
    'variation_energy_intensity_of_households_transport',
    'number vehicles h':
    'number_vehicles_h',
    'effects shortage elec on ev':
    'effects_shortage_elec_on_ev',
    'effects shortage gas h veh':
    'effects_shortage_gas_h_veh',
    'xax extraction projection minerals':
    'xax_extraction_projection_minerals',
    'xbx extraction projection minerals':
    'xbx_extraction_projection_minerals',
    'variation minerals extraction rest':
    'variation_minerals_extraction_rest',
    'total recycled materials for other mt':
    'total_recycled_materials_for_other_mt',
    'share minerals consumption alt techn vs total economy':
    'share_minerals_consumption_alt_techn_vs_total_economy',
    'labor share ctexquestx':
    'labor_share_ctexquestx',
    'initial minerals extraction rest':
    'initial_minerals_extraction_rest',
    'minerals extraction projection rest cte rr':
    'minerals_extraction_projection_rest_cte_rr',
    'materials to extract rest mt':
    'materials_to_extract_rest_mt',
    'minerals extraction projection rest with rr':
    'minerals_extraction_projection_rest_with_rr',
    'minerals consumption estimation rest cte rr':
    'minerals_consumption_estimation_rest_cte_rr',
    'total materials required for res elec xplusx ev batteries mt':
    'total_materials_required_for_res_elec_xplusx_ev_batteries_mt',
    'variation nonxdashxenergy use':
    'variation_nonxdashxenergy_use',
    'number 2w':
    'number_2w',
    'number 4w':
    'number_4w',
    'number all':
    'number_all',
    'sum 4w shares':
    'sum_4w_shares',
    'share available 4w':
    'share_available_4w',
    'percent 4w':
    'percent_4w',
    'saving ratio 2we':
    'saving_ratio_2we',
    'percent all':
    'percent_all',
    'percent 2w':
    'percent_2w',
    'share fed coal vs nre heatxdashxnc':
    'share_fed_coal_vs_nre_heatxdashxnc',
    'share fed gas vs nre heatxdashxnc':
    'share_fed_gas_vs_nre_heatxdashxnc',
    'fed nre for heatxdashxnc':
    'fed_nre_for_heatxdashxnc',
    'share fed liquids vs nre heatxdashxnc':
    'share_fed_liquids_vs_nre_heatxdashxnc',
    'required fed by fuel':
    'required_fed_by_fuel',
    'bioe gen land marg available':
    'bioe_gen_land_marg_available',
    'potential peavail cellulosic biofuel ej':
    'potential_peavail_cellulosic_biofuel_ej',
    'potential peavail total biofuels':
    'potential_peavail_total_biofuels',
    'biofuels 3gen land compet available':
    'biofuels_3gen_land_compet_available',
    'fes total biofuels production ej':
    'fes_total_biofuels_production_ej',
    'fes total biofuels production ej x2x':
    'fes_total_biofuels_production_ej_x2x',
    'fes total biofuels production mbxdivxd':
    'fes_total_biofuels_production_mbxdivxd',
    'peavail biofuels 2gen land compet ej':
    'peavail_biofuels_2gen_land_compet_ej',
    'peavail biofuels 3gen land compet ej':
    'peavail_biofuels_3gen_land_compet_ej',
    'peavail biofuels land marg ej':
    'peavail_biofuels_land_marg_ej',
    'pe biofuels land marg ej':
    'pe_biofuels_land_marg_ej',
    'pe biofuels prod x2genxplusx3gen ej':
    'pe_biofuels_prod_x2genxplusx3gen_ej',
    'pe biomass for biofuels production ej':
    'pe_biomass_for_biofuels_production_ej',
    'pe cellulosic biofuel ej':
    'pe_cellulosic_biofuel_ej',
    'max peavail biofuels potential':
    'max_peavail_biofuels_potential',
    'start year biofuels land marg':
    'start_year_biofuels_land_marg',
    'land required biofuels land marg':
    'land_required_biofuels_land_marg',
    'max peavail potential biofuels marginal lands':
    'max_peavail_potential_biofuels_marginal_lands',
    'share biofuels overcapacity':
    'share_biofuels_overcapacity',
    'total land requirements renew mha':
    'total_land_requirements_renew_mha',
    'land occupation ratio biofuels marg land':
    'land_occupation_ratio_biofuels_marg_land',
    'peavail cellulosic biofuel ej':
    'peavail_cellulosic_biofuel_ej',
    'potential peavail biofuels land marg ej':
    'potential_peavail_biofuels_land_marg_ej',
    'bioe potential npp marginal lands':
    'bioe_potential_npp_marginal_lands',
    'additional pe production of bioenergy for biofuels':
    'additional_pe_production_of_bioenergy_for_biofuels',
    'oil liquids saved by biofuels ej':
    'oil_liquids_saved_by_biofuels_ej',
    'efficiency bioe residues to cellulosic liquids':
    'efficiency_bioe_residues_to_cellulosic_liquids',
    'conv efficiency from npp to biofuels':
    'conv_efficiency_from_npp_to_biofuels',
    'pes heat res':
    'pes_heat_res',
    'ped nre liquids':
    'ped_nre_liquids',
    'potential fes ctlxplusxgtl ej':
    'potential_fes_ctlxplusxgtl_ej',
    'tfec res delayed 1yr':
    'tfec_res_delayed_1yr',
    'pe supply from res nonxdashxelec without trad bioe ej':
    'pe_supply_from_res_nonxdashxelec_without_trad_bioe_ej',
    'pes gases':
    'pes_gases',
    'fes ctlxplusxgtl ej':
    'fes_ctlxplusxgtl_ej',
    'activate elf all scenxquestx':
    'activate_elf_all_scenxquestx',
    'tpes res delayed 1yr':
    'tpes_res_delayed_1yr',
    'annual tpes res growth rate':
    'annual_tpes_res_growth_rate',
    'ped natx gas ej':
    'ped_natx_gas_ej',
    'annual share res vs tfec growth rate':
    'annual_share_res_vs_tfec_growth_rate',
    'annual share res vs tpes growth rate':
    'annual_share_res_vs_tpes_growth_rate',
    'share res vs tpes delayed 1yr':
    'share_res_vs_tpes_delayed_1yr',
    'gtl production':
    'gtl_production',
    'pes biogas for tfc':
    'pes_biogas_for_tfc',
    'share ctlxplusxgtl overcapacity':
    'share_ctlxplusxgtl_overcapacity',
    'share res vs tfec delayed 1yr':
    'share_res_vs_tfec_delayed_1yr',
    'share exdashxlosses cc':
    'share_exdashxlosses_cc',
    'ped natx gas for gtl ej':
    'ped_natx_gas_for_gtl_ej',
    'ped coal for ctl ej':
    'ped_coal_for_ctl_ej',
    'annual tfec res growth rate':
    'annual_tfec_res_growth_rate',
    'ped total oil ej':
    'ped_total_oil_ej',
    'ctl production':
    'ctl_production',
    'ch4 emissions coal without ctl':
    'ch4_emissions_coal_without_ctl',
    'pes tot res for heat':
    'pes_tot_res_for_heat',
    'total fe real supply res for heatxdashxnc ej':
    'total_fe_real_supply_res_for_heatxdashxnc_ej',
    'real extraction conv oil emissions relevant ej':
    'real_extraction_conv_oil_emissions_relevant_ej',
    'co2 emissions coal without ctl':
    'co2_emissions_coal_without_ctl',
    'real extraction conv gas emissions relevant ej':
    'real_extraction_conv_gas_emissions_relevant_ej',
    'pes res for heatxdashxnc by techn':
    'pes_res_for_heatxdashxnc_by_techn',
    'fes nre for heat':
    'fes_nre_for_heat',
    'installed capacity res heatxdashxnc tw':
    'installed_capacity_res_heatxdashxnc_tw',
    'fed heatxdashxcom plants fossil fuels ej':
    'fed_heatxdashxcom_plants_fossil_fuels_ej',
    'real extraction unconv gas emissions relevant ej':
    'real_extraction_unconv_gas_emissions_relevant_ej',
    'real extraction unconv oil emissions relevant ej':
    'real_extraction_unconv_oil_emissions_relevant_ej',
    'available pe potential solid bioe for elec ej':
    'available_pe_potential_solid_bioe_for_elec_ej',
    'remaining potential res for heat':
    'remaining_potential_res_for_heat',
    'extraction coal emissions relevant ej':
    'extraction_coal_emissions_relevant_ej',
    'wear res capacity for heatxdashxnc tw':
    'wear_res_capacity_for_heatxdashxnc_tw',
    'fe real generation res heatxdashxnc ej':
    'fe_real_generation_res_heatxdashxnc_ej',
    'total fe real supply res for heatxdashxcom ej':
    'total_fe_real_supply_res_for_heatxdashxcom_ej',
    'past res growth for heatxdashxcom x0':
    'past_res_growth_for_heatxdashxcom_x0',
    'efficiency conversion bioe plants to heat x0x':
    'efficiency_conversion_bioe_plants_to_heat_x0x',
    'fed heatxdashxcom after priorities ej':
    'fed_heatxdashxcom_after_priorities_ej',
    'fes heat from biow':
    'fes_heat_from_biow',
    'abundance res heatxdashxnc':
    'abundance_res_heatxdashxnc',
    'efficiency geothermal for heat x0x':
    'efficiency_geothermal_for_heat_x0x',
    'fes heatxdashxcom nuclear chp plants ej':
    'fes_heatxdashxcom_nuclear_chp_plants_ej',
    'fe real supply res for heatxdashxnc tot ej':
    'fe_real_supply_res_for_heatxdashxnc_tot_ej',
    'initial value res for heatxdashxnc':
    'initial_value_res_for_heatxdashxnc',
    'past res growth for heatxdashxnc':
    'past_res_growth_for_heatxdashxnc',
    'p solid bioe for heat x0x':
    'p_solid_bioe_for_heat_x0x',
    'fed heat fossil fuels chp plants ej':
    'fed_heat_fossil_fuels_chp_plants_ej',
    'replacement res for heatxdashxnc':
    'replacement_res_for_heatxdashxnc',
    'efficiency solar panels for heat x0x':
    'efficiency_solar_panels_for_heat_x0x',
    'p res for heat x0x':
    'p_res_for_heat_x0x',
    'p solar for heat x0x':
    'p_solar_for_heat_x0x',
    'abundance res heatxdashxcom':
    'abundance_res_heatxdashxcom',
    'abundance res heatxdashxnc2':
    'abundance_res_heatxdashxnc2',
    'potential fes tot res for heatxdashxnc ej':
    'potential_fes_tot_res_for_heatxdashxnc_ej',
    'life time res for heat x0x':
    'life_time_res_for_heat_x0x',
    'efficiency res heat x0x':
    'efficiency_res_heat_x0x',
    'replacement res for heat x0x':
    'replacement_res_for_heat_x0x',
    'p geothermal for heat x0x':
    'p_geothermal_for_heat_x0x',
    'fes res for heat ej':
    'fes_res_for_heat_ej',
    'fe real supply res for heatxdashxcom tot ej':
    'fe_real_supply_res_for_heatxdashxcom_tot_ej',
    'losses solar for heat x0x':
    'losses_solar_for_heat_x0x',
    'res heatxdashxnc tot overcapacity':
    'res_heatxdashxnc_tot_overcapacity',
    'potential fes res for heatxdashxnc ej':
    'potential_fes_res_for_heatxdashxnc_ej',
    'other solids required':
    'other_solids_required',
    'other gases required':
    'other_gases_required',
    'transformation ff losses ej':
    'transformation_ff_losses_ej',
    'fes elec fossil fuel chp plants ej':
    'fes_elec_fossil_fuel_chp_plants_ej',
    'total fe elec demand after priorities twh':
    'total_fe_elec_demand_after_priorities_twh',
    'demand elec nre twh':
    'demand_elec_nre_twh',
    'nonxdashxenergy use demand by final fuel ej':
    'nonxdashxenergy_use_demand_by_final_fuel_ej',
    'share solids for final energy':
    'share_solids_for_final_energy',
    'real fe consumption gases ej':
    'real_fe_consumption_gases_ej',
    'total real nonxdashxenergy use consumption ej':
    'total_real_nonxdashxenergy_use_consumption_ej',
    'share gases for final energy':
    'share_gases_for_final_energy',
    'ratio fed for heatxdashxnc vs fed for heatxdashxcom':
    'ratio_fed_for_heatxdashxnc_vs_fed_for_heatxdashxcom',
    'real fe consumption by fuel before heat correction':
    'real_fe_consumption_by_fuel_before_heat_correction',
    'required tfed before heat dem corr':
    'required_tfed_before_heat_dem_corr',
    'cum materials to extract for ev batteries':
    'cum_materials_to_extract_for_ev_batteries',
    'share cum dem materials to extract alt techn vs total':
    'share_cum_dem_materials_to_extract_alt_techn_vs_total',
    'total recycled materials for ev batteries mt':
    'total_recycled_materials_for_ev_batteries_mt',
    'share res vs tfec':
    'share_res_vs_tfec',
    'cum materials to extract for alt techn from x2015x':
    'cum_materials_to_extract_for_alt_techn_from_x2015x',
    'total materials to extract for ev batteries from x2015x mt':
    'total_materials_to_extract_for_ev_batteries_from_x2015x_mt',
    'materials per new capacity installed xdashx ev batteries':
    'materials_per_new_capacity_installed_xdashx_ev_batteries',
    'total materials to extract for ev batteries mt':
    'total_materials_to_extract_for_ev_batteries_mt',
    'total materials to extract mt':
    'total_materials_to_extract_mt',
    'cum materials requirements for ev batteries':
    'cum_materials_requirements_for_ev_batteries',
    'xstaticx eroigrid res elec':
    'xstaticx_eroigrid_res_elec',
    't per x':
    't_per_x',
    'total materials required for ev batteries':
    'total_materials_required_for_ev_batteries',
    'tfec from res per capita':
    'tfec_from_res_per_capita',
    'share materials cum demand to extract vs reserves for res elec':
    'share_materials_cum_demand_to_extract_vs_reserves_for_res_elec',
    'rt storage efficiency ev batteries':
    'rt_storage_efficiency_ev_batteries',
    'cum materials to extract for ev batteries from x2015x':
    'cum_materials_to_extract_for_ev_batteries_from_x2015x',
    'total cumulative demand materials to extract from x2015x':
    'total_cumulative_demand_materials_to_extract_from_x2015x',
    'initial cumulated material requirements for ev batteries x1995x':
    'initial_cumulated_material_requirements_for_ev_batteries_x1995x',
    'share materials cum demand to extract vs resources for res elec':
    'share_materials_cum_demand_to_extract_vs_resources_for_res_elec',
    'fe tot generation all res elec ej':
    'fe_tot_generation_all_res_elec_ej',
    'carbon footprint tonnescxdivxperson':
    'carbon_footprint_tonnescxdivxperson',
    'potential max hdi':
    'potential_max_hdi',
    'kw per battery ev':
    'kw_per_battery_ev',
    'pes fossil fuel extraction delayed':
    'pes_fossil_fuel_extraction_delayed',
    'energy distr losses ff ej':
    'energy_distr_losses_ff_ej',
    'pes fossil fuel extraction':
    'pes_fossil_fuel_extraction',
    'initial energy intensity x1995x h':
    'initial_energy_intensity_x1995x_h',
    'min energy intensity vs intial h':
    'min_energy_intensity_vs_intial_h',
    'minimum fraction h':
    'minimum_fraction_h',
    'policy to improve efficiency speed h':
    'policy_to_improve_efficiency_speed_h',
    'max yearly change h':
    'max_yearly_change_h',
    'available improvement efficiency h':
    'available_improvement_efficiency_h',
    'year policy change energy h':
    'year_policy_change_energy_h',
    'efficiency rate of substitution h':
    'efficiency_rate_of_substitution_h',
    'energy cost pressure h':
    'energy_cost_pressure_h',
    'abundance of energy h':
    'abundance_of_energy_h',
    'global energy intensity h':
    'global_energy_intensity_h',
    'maximun yearly aceleration of intensity improvement pct h':
    'maximun_yearly_aceleration_of_intensity_improvement_pct_h',
    'historical mean rate energy intensity h':
    'historical_mean_rate_energy_intensity_h',
    'policy change energy speed h':
    'policy_change_energy_speed_h',
    'initial global energy intensity x2009x h':
    'initial_global_energy_intensity_x2009x_h',
    'year policy to improve efficiency h':
    'year_policy_to_improve_efficiency_h',
    'share conv vs total oil extraction':
    'share_conv_vs_total_oil_extraction',
    'pes natx gas without gtl':
    'pes_natx_gas_without_gtl',
    'co2 emissions conv gas without gtl':
    'co2_emissions_conv_gas_without_gtl',
    'co2 emissions conv oil':
    'co2_emissions_conv_oil',
    'co2 emissions gtl':
    'co2_emissions_gtl',
    'ch4 emissions conv gas without gtl':
    'ch4_emissions_conv_gas_without_gtl',
    'co2 emissions unconv oil':
    'co2_emissions_unconv_oil',
    'share conv vs total gas extraction':
    'share_conv_vs_total_gas_extraction',
    'ch4 emissions unconv gas':
    'ch4_emissions_unconv_gas',
    'fe heat demand consum':
    'fe_heat_demand_consum',
    'share electricity vs tfes':
    'share_electricity_vs_tfes',
    'co2 emissions peat':
    'co2_emissions_peat',
    'required fed by gas':
    'required_fed_by_gas',
    'required tfed':
    'required_tfed',
    'total fe elec consumption ej':
    'total_fe_elec_consumption_ej',
    'total fe elec consumption twh':
    'total_fe_elec_consumption_twh',
    'ped gases':
    'ped_gases',
    'total fe heat consumption ej':
    'total_fe_heat_consumption_ej',
    'share solids vs tfes':
    'share_solids_vs_tfes',
    'share gases vs tfes':
    'share_gases_vs_tfes',
    'required fed solids':
    'required_fed_solids',
    'share heat vs tfes':
    'share_heat_vs_tfes',
    'ped solids':
    'ped_solids',
    'scarcity conv oil':
    'scarcity_conv_oil',
    'share liquids vs tfes':
    'share_liquids_vs_tfes',
    'gfcf not covered':
    'gfcf_not_covered',
    'lc not covered':
    'lc_not_covered',
    'cc total not covered':
    'cc_total_not_covered',
    'household demand not covered':
    'household_demand_not_covered',
    'real total output inland transport':
    'real_total_output_inland_transport',
    'tfes intensity ej tdollar':
    'tfes_intensity_ej_tdollar',
    'tfes intensity ej tdollar delayed 1yr':
    'tfes_intensity_ej_tdollar_delayed_1yr',
    'tpes intensity ej tdollar delayed 1yr':
    'tpes_intensity_ej_tdollar_delayed_1yr',
    'annual tpes intensity growth rate':
    'annual_tpes_intensity_growth_rate',
    'annual tfes intensity growth rate':
    'annual_tfes_intensity_growth_rate',
    'total d jobs res elec per techn':
    'total_d_jobs_res_elec_per_techn',
    'total d jobs res heat per techn':
    'total_d_jobs_res_heat_per_techn',
    'employment factors new res elec':
    'employment_factors_new_res_elec',
    'employment factors new res heat':
    'employment_factors_new_res_heat',
    'employment factors oxandxm res elec':
    'employment_factors_oxandxm_res_elec',
    'employment factors oxandxm res heat':
    'employment_factors_oxandxm_res_heat',
    'd jobs new installed res elec per techn':
    'd_jobs_new_installed_res_elec_per_techn',
    'jobs oxandxm res elec per techn':
    'jobs_oxandxm_res_elec_per_techn',
    'nx0 vehicles per xinland t':
    'nx0_vehicles_per_xinland_t',
    'initial percent t vehicles':
    'initial_percent_t_vehicles',
    'saving ratios v':
    'saving_ratios_v',
    'share gases dem for heatxdashxnc':
    'share_gases_dem_for_heatxdashxnc',
    'increase scarcity conv gas':
    'increase_scarcity_conv_gas',
    'scarcity conv gas stock':
    'scarcity_conv_gas_stock',
    'pes natx gas for heatxdashxnc plants':
    'pes_natx_gas_for_heatxdashxnc_plants',
    'real growth ctl':
    'real_growth_ctl',
    'ctl potential production':
    'ctl_potential_production',
    'abundance liquids gtl':
    'abundance_liquids_gtl',
    'gtl potential production':
    'gtl_potential_production',
    'abundance liquids ctl':
    'abundance_liquids_ctl',
    'wear gtl':
    'wear_gtl',
    'lifetime gtl':
    'lifetime_gtl',
    'lifetime ctl':
    'lifetime_ctl',
    'wear ctl':
    'wear_ctl',
    'abundance unconv oil stock':
    'abundance_unconv_oil_stock',
    'increase abundance unconv oil':
    'increase_abundance_unconv_oil',
    'abundance unconv oil':
    'abundance_unconv_oil',
    'abundance unconv oil delayed 1yr':
    'abundance_unconv_oil_delayed_1yr',
    'abundance unconv oil2':
    'abundance_unconv_oil2',
    'real extraction unconv oil ej':
    'real_extraction_unconv_oil_ej',
    'scarcity conv oil stock':
    'scarcity_conv_oil_stock',
    'share variable res elec generation vs total gen':
    'share_variable_res_elec_generation_vs_total_gen',
    'extra monet invest to cope with variable elec res':
    'extra_monet_invest_to_cope_with_variable_elec_res',
    'increase scarcity conv oil':
    'increase_scarcity_conv_oil',
    'share extra monet invest to cope with variable elec res':
    'share_extra_monet_invest_to_cope_with_variable_elec_res',
    'initial share variable res elec gen vs total':
    'initial_share_variable_res_elec_gen_vs_total',
    'pes oil for heatxdashxnc plants':
    'pes_oil_for_heatxdashxnc_plants',
    'cumulated invest e grid':
    'cumulated_invest_e_grid',
    'total monet invest res for elec tdolar':
    'total_monet_invest_res_for_elec_tdolar',
    'cp exogenous res elec var reduction':
    'cp_exogenous_res_elec_var_reduction',
    'cp exogenous res elec dispatch reduction':
    'cp_exogenous_res_elec_dispatch_reduction',
    'increase variable res share elec vs total generation':
    'increase_variable_res_share_elec_vs_total_generation',
    'available improvement efficiency':
    'available_improvement_efficiency',
    'pes nre heatxdashxnc':
    'pes_nre_heatxdashxnc',
    'tpes heat':
    'tpes_heat',
    'pes nre heat':
    'pes_nre_heat',
    'share liquids dem for heatxdashxnc':
    'share_liquids_dem_for_heatxdashxnc',
    'pes nre heatxdashxcom':
    'pes_nre_heatxdashxcom',
    'check tpe':
    'check_tpe',
    'fed by fuel for heatxdashxnc':
    'fed_by_fuel_for_heatxdashxnc',
    'share fed heatxdashxcom vs total heat':
    'share_fed_heatxdashxcom_vs_total_heat',
    'average elec consumption per capita':
    'average_elec_consumption_per_capita',
    'fed solid bioe for heatxdashxnc':
    'fed_solid_bioe_for_heatxdashxnc',
    'total fed heat ej':
    'total_fed_heat_ej',
    'heatxdashxnc distribution losses':
    'heatxdashxnc_distribution_losses',
    'total fed heatxdashxnc ej':
    'total_fed_heatxdashxnc_ej',
    'total fed nre heatxdashxnc':
    'total_fed_nre_heatxdashxnc',
    'deactivate heat dem correctionxquestx':
    'deactivate_heat_dem_correctionxquestx',
    'share feh over fed solid bioe':
    'share_feh_over_fed_solid_bioe',
    'abundance heat':
    'abundance_heat',
    'pes coal for heatxdashxnc plants':
    'pes_coal_for_heatxdashxnc_plants',
    'required fed sectors by fuel':
    'required_fed_sectors_by_fuel',
    'share trad biomass vs solids in households':
    'share_trad_biomass_vs_solids_in_households',
    'share global pop dependent on trad biomass':
    'share_global_pop_dependent_on_trad_biomass',
    'pes oil for heatxdashxcom plants':
    'pes_oil_for_heatxdashxcom_plants',
    'share coal dem for heatxdashxnc':
    'share_coal_dem_for_heatxdashxnc',
    'res heatxdashxcom tot overcapacity':
    'res_heatxdashxcom_tot_overcapacity',
    'pes natx gas for heatxdashxcom plants':
    'pes_natx_gas_for_heatxdashxcom_plants',
    'pes coal for heatxdashxcom plants':
    'pes_coal_for_heatxdashxcom_plants',
    'required heatxdashxcom':
    'required_heatxdashxcom',
    'variation labour share':
    'variation_labour_share',
    'variation household demand':
    'variation_household_demand',
    'labour share':
    'labour_share',
    'capital share':
    'capital_share',
    'growth capital share':
    'growth_capital_share',
    'include materials for overgridsxquestx':
    'include_materials_for_overgridsxquestx',
    'all minerals virginxquestx':
    'all_minerals_virginxquestx',
    'materials for new res elec per capacity installed':
    'materials_for_new_res_elec_per_capacity_installed',
    'cedtot per material res elec var':
    'cedtot_per_material_res_elec_var',
    'cedtot per tw res elec var':
    'cedtot_per_tw_res_elec_var',
    'cedtot per tw per material res elec var':
    'cedtot_per_tw_per_material_res_elec_var',
    'demand by sector fd':
    'demand_by_sector_fd',
    'sum variation':
    'sum_variation',
    'cc sectoral next step':
    'cc_sectoral_next_step',
    'gross fixed capital formation':
    'gross_fixed_capital_formation',
    'pct gfcf vs gfcfxplusxhd':
    'pct_gfcf_vs_gfcfxplusxhd',
    'household demand':
    'household_demand',
    'lc next step':
    'lc_next_step',
    'household demand total':
    'household_demand_total',
    'quality of electricity x2015x':
    'quality_of_electricity_x2015x',
    'dynamic quality of electricity':
    'dynamic_quality_of_electricity',
    'quality of electricity':
    'quality_of_electricity',
    'cp exogenous res elec reduction':
    'cp_exogenous_res_elec_reduction',
    'res elec variablesxquestx':
    'res_elec_variablesxquestx',
    'aux3':
    'aux3',
    'fei res elec var':
    'fei_res_elec_var',
    'xstaticx eroi res elec':
    'xstaticx_eroi_res_elec',
    'materials required for new res elec mt':
    'materials_required_for_new_res_elec_mt',
    'fei over lifetime res elec var':
    'fei_over_lifetime_res_elec_var',
    'ced decom res elec capacity':
    'ced_decom_res_elec_capacity',
    'ced oxandxm per material res elec var':
    'ced_oxandxm_per_material_res_elec_var',
    'cedtot new cap res elec var':
    'cedtot_new_cap_res_elec_var',
    'cedtot oxandxm res elec var':
    'cedtot_oxandxm_res_elec_var',
    'aux1':
    'aux1',
    'b lineal regr':
    'b_lineal_regr',
    'a lineal regr':
    'a_lineal_regr',
    'esoi phs depleted potential':
    'esoi_phs_depleted_potential',
    'cc total':
    'cc_total',
    'initial household demand':
    'initial_household_demand',
    'total demand':
    'total_demand',
    'demand not covered by sector fd':
    'demand_not_covered_by_sector_fd',
    'initial demand by sectot':
    'initial_demand_by_sectot',
    'beta x1x lab':
    'beta_x1x_lab',
    'initial lc total':
    'initial_lc_total',
    'initial gfcf':
    'initial_gfcf',
    'beta x0x cap':
    'beta_x0x_cap',
    'initial cc total':
    'initial_cc_total',
    'beta x2x cap':
    'beta_x2x_cap',
    'demand not covered total fd':
    'demand_not_covered_total_fd',
    'beta x1x cap':
    'beta_x1x_cap',
    'lc':
    'lc',
    'bet x0x lab':
    'bet_x0x_lab',
    'tped by fuel':
    'tped_by_fuel',
    'abundance tpe':
    'abundance_tpe',
    'beta x2x lab':
    'beta_x2x_lab',
    'initial global energy intensity by sector x2009x':
    'initial_global_energy_intensity_by_sector_x2009x',
    'available pe potential solid bioe for heat ej':
    'available_pe_potential_solid_bioe_for_heat_ej',
    'max pe potential res for heat':
    'max_pe_potential_res_for_heat',
    'pes res for heatxdashxcom by techn':
    'pes_res_for_heatxdashxcom_by_techn',
    'p res for heat':
    'p_res_for_heat',
    'fe real generation res heatxdashxcom ej':
    'fe_real_generation_res_heatxdashxcom_ej',
    'potential fes tot res for heatxdashxcom ej':
    'potential_fes_tot_res_for_heatxdashxcom_ej',
    'available potential fe solid bioe for elec ej':
    'available_potential_fe_solid_bioe_for_elec_ej',
    'max bioe twe':
    'max_bioe_twe',
    'remaining potential tot res heat':
    'remaining_potential_tot_res_heat',
    'potential fes heatxdashxcom nuclear chp plants ej':
    'potential_fes_heatxdashxcom_nuclear_chp_plants_ej',
    'share of heat production in chp plants vs total nucelar elec generation':
    'share_of_heat_production_in_chp_plants_vs_total_nucelar_elec_generation',
    'pes waste for elec plants':
    'pes_waste_for_elec_plants',
    'losses chp biogas':
    'losses_chp_biogas',
    'losses chp waste':
    'losses_chp_waste',
    'share pes biogas for heat':
    'share_pes_biogas_for_heat',
    'pe losses res for elec':
    'pe_losses_res_for_elec',
    'pes tot waste for elec':
    'pes_tot_waste_for_elec',
    'pes tot waste for heatxdashxcom':
    'pes_tot_waste_for_heatxdashxcom',
    'abundance res heatxdashxcom2':
    'abundance_res_heatxdashxcom2',
    'max pe potential biogas for heat':
    'max_pe_potential_biogas_for_heat',
    'share efficiency biogas for elec in chp plants':
    'share_efficiency_biogas_for_elec_in_chp_plants',
    'share efficiency waste for elec in chp plants':
    'share_efficiency_waste_for_elec_in_chp_plants',
    'max potential pe nonxdashxelectric res':
    'max_potential_pe_nonxdashxelectric_res',
    'pes tot biogas for elec':
    'pes_tot_biogas_for_elec',
    'pes tot biogas for heatxdashxcom':
    'pes_tot_biogas_for_heatxdashxcom',
    'pe losses nre elec generation':
    'pe_losses_nre_elec_generation',
    'elec gen related losses ej':
    'elec_gen_related_losses_ej',
    'pe losses biogas for elec':
    'pe_losses_biogas_for_elec',
    'pe losses waste for elec':
    'pe_losses_waste_for_elec',
    'max pe potential tot res heat ej':
    'max_pe_potential_tot_res_heat_ej',
    'share pes biogas for elec':
    'share_pes_biogas_for_elec',
    'p hydro growth':
    'p_hydro_growth',
    'fe real tot generation res elec twh':
    'fe_real_tot_generation_res_elec_twh',
    'p solar pv growth':
    'p_solar_pv_growth',
    'remaining potential constraint on new res heat capacity':
    'remaining_potential_constraint_on_new_res_heat_capacity',
    'total fe elec generation twh':
    'total_fe_elec_generation_twh',
    'fe tot generation all res elec twh':
    'fe_tot_generation_all_res_elec_twh',
    'p oceanic growth':
    'p_oceanic_growth',
    'pe elec generation from res ej':
    'pe_elec_generation_from_res_ej',
    'p res elec growth':
    'p_res_elec_growth',
    'share res electricity generation':
    'share_res_electricity_generation',
    'p solid bioexdashxelec growth':
    'p_solid_bioexdashxelec_growth',
    'p csp growth':
    'p_csp_growth',
    'remaining potential tot res elec':
    'remaining_potential_tot_res_elec',
    'p wind offshore growth':
    'p_wind_offshore_growth',
    'p wind onshore growth':
    'p_wind_onshore_growth',
    'elec generation dispatch from res twh':
    'elec_generation_dispatch_from_res_twh',
    'fes elec from res with priority twh':
    'fes_elec_from_res_with_priority_twh',
    'abundance res elec':
    'abundance_res_elec',
    'fes elec from biow':
    'fes_elec_from_biow',
    'p geot growth':
    'p_geot_growth',
    'max potential tot res elec twh':
    'max_potential_tot_res_elec_twh',
    'p solid bioe for heat':
    'p_solid_bioe_for_heat',
    'pe real generation res elec':
    'pe_real_generation_res_elec',
    'efficiency res heat':
    'efficiency_res_heat',
    'constant final energy intensity sectorsxandxhouseholds tdxquestx':
    'constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx',
    'share oil dem for heatxdashxcom':
    'share_oil_dem_for_heatxdashxcom',
    'fe elec generation from total oil twh':
    'fe_elec_generation_from_total_oil_twh',
    'total fe heat generation ej':
    'total_fe_heat_generation_ej',
    'share coal dem for elec':
    'share_coal_dem_for_elec',
    'pe losses oil for elec ej':
    'pe_losses_oil_for_elec_ej',
    'share oil dem for elec':
    'share_oil_dem_for_elec',
    'share coal dem for heatxdashxcom':
    'share_coal_dem_for_heatxdashxcom',
    'fe elec generation from conv gas twh':
    'fe_elec_generation_from_conv_gas_twh',
    'share res heat generation':
    'share_res_heat_generation',
    'share natx gas dem for elec':
    'share_natx_gas_dem_for_elec',
    'pe losses conv gas for elec ej':
    'pe_losses_conv_gas_for_elec_ej',
    'pe losses coal for elec ej':
    'pe_losses_coal_for_elec_ej',
    'fe elec generation from coal twh':
    'fe_elec_generation_from_coal_twh',
    'share natx gas dem for heatxdashxcom':
    'share_natx_gas_dem_for_heatxdashxcom',
    'year scarcity heat':
    'year_scarcity_heat',
    'fes heatxdashxcom from biogas in chp plants':
    'fes_heatxdashxcom_from_biogas_in_chp_plants',
    'efficiency biogas for heat plants':
    'efficiency_biogas_for_heat_plants',
    'fes elec from biogas in chp plants':
    'fes_elec_from_biogas_in_chp_plants',
    'fes elec from biogas ej':
    'fes_elec_from_biogas_ej',
    'pes biogas for heatxdashxcom plants':
    'pes_biogas_for_heatxdashxcom_plants',
    'fes elec from biogas in elec plants':
    'fes_elec_from_biogas_in_elec_plants',
    'efficiency biogas for elec plants':
    'efficiency_biogas_for_elec_plants',
    'fes biogas for heatxdashxcom plants':
    'fes_biogas_for_heatxdashxcom_plants',
    'pes biogas for elec plants':
    'pes_biogas_for_elec_plants',
    'share pes biogas tfc':
    'share_pes_biogas_tfc',
    'potential pes biogas for tfc':
    'potential_pes_biogas_for_tfc',
    'fes elec from biogas twh':
    'fes_elec_from_biogas_twh',
    'pes biogas for chp':
    'pes_biogas_for_chp',
    'share pes biogas for heatxdashxcom plants':
    'share_pes_biogas_for_heatxdashxcom_plants',
    'efficiency biogas for elec chp plants':
    'efficiency_biogas_for_elec_chp_plants',
    'fes heatxdashxcom from biogas ej':
    'fes_heatxdashxcom_from_biogas_ej',
    'efficiency biogas for heat chp plants':
    'efficiency_biogas_for_heat_chp_plants',
    'share pes biogas for elec plants':
    'share_pes_biogas_for_elec_plants',
    'share pes biogas for chp':
    'share_pes_biogas_for_chp',
    'demand elec plants fossil fuels twh':
    'demand_elec_plants_fossil_fuels_twh',
    'total extraction nre ej':
    'total_extraction_nre_ej',
    'energy cost pressure':
    'energy_cost_pressure',
    'pressure to improve energy intensity efficiency':
    'pressure_to_improve_energy_intensity_efficiency',
    'abundance of energy':
    'abundance_of_energy',
    'constrain rr improv for alt techn per mineral':
    'constrain_rr_improv_for_alt_techn_per_mineral',
    'constrain rr improv for rest per mineral':
    'constrain_rr_improv_for_rest_per_mineral',
    'ced oxandxm over lifetime per material res elec var':
    'ced_oxandxm_over_lifetime_per_material_res_elec_var',
    'energy cons per unit of material cons for res elec':
    'energy_cons_per_unit_of_material_cons_for_res_elec',
    'ced new cap per material res elec var':
    'ced_new_cap_per_material_res_elec_var',
    'energy required for material consumption for new res elec':
    'energy_required_for_material_consumption_for_new_res_elec',
    'energy required for material consumption for oxandxm res elec':
    'energy_required_for_material_consumption_for_oxandxm_res_elec',
    'share other cumulative demand to extract vs resources materials':
    'share_other_cumulative_demand_to_extract_vs_resources_materials',
    'materials to extract rest from x2015x mt':
    'materials_to_extract_rest_from_x2015x_mt',
    'total recycled materials for res elec mt':
    'total_recycled_materials_for_res_elec_mt',
    'cum materials to extract rest':
    'cum_materials_to_extract_rest',
    'cum materials to extract rest from x2015x':
    'cum_materials_to_extract_rest_from_x2015x',
    'share other cumulative demand to extract vs reserves materials':
    'share_other_cumulative_demand_to_extract_vs_reserves_materials',
    'materials availability xannual extractionx':
    'materials_availability_xannual_extractionx',
    'mt per tonne':
    'mt_per_tonne',
    'initial cumulated material requirements for rest x1995x':
    'initial_cumulated_material_requirements_for_rest_x1995x',
    'p common rr minerals variation rest':
    'p_common_rr_minerals_variation_rest',
    'cum materials to extract for res elec from x2015x':
    'cum_materials_to_extract_for_res_elec_from_x2015x',
    'total materials to extract for res elec mt':
    'total_materials_to_extract_for_res_elec_mt',
    'cum materials to extract for res elec':
    'cum_materials_to_extract_for_res_elec',
    'total materials to extract for res elec from x2015x mt':
    'total_materials_to_extract_for_res_elec_from_x2015x_mt',
    'max recycling rates minerals':
    'max_recycling_rates_minerals',
    'p common rr minerals variation alt techn':
    'p_common_rr_minerals_variation_alt_techn',
    'current recycling rates minerals':
    'current_recycling_rates_minerals',
    'historic improvement recycling rates minerals':
    'historic_improvement_recycling_rates_minerals',
    'min energy intensity vs intial':
    'min_energy_intensity_vs_intial',
    'global energy intensity by sector':
    'global_energy_intensity_by_sector',
    'initial energy cons per unit of material cons xrecycledx':
    'initial_energy_cons_per_unit_of_material_cons_xrecycledx',
    'materials required for oxandxm res elec mt':
    'materials_required_for_oxandxm_res_elec_mt',
    'share tot cum dem vs reserves materials':
    'share_tot_cum_dem_vs_reserves_materials',
    'share tot cum dem vs resources materials':
    'share_tot_cum_dem_vs_resources_materials',
    'maximum annual extraction materials':
    'maximum_annual_extraction_materials',
    'materials availability xresourcesx':
    'materials_availability_xresourcesx',
    'materials availability xreservesx':
    'materials_availability_xreservesx',
    'tpes ej':
    'tpes_ej',
    'fes heatxdashxcom from waste ej':
    'fes_heatxdashxcom_from_waste_ej',
    'fes waste for heatxdashxcom plants':
    'fes_waste_for_heatxdashxcom_plants',
    'fes elec from waste in chp plants':
    'fes_elec_from_waste_in_chp_plants',
    'fes elec from waste in elec plants':
    'fes_elec_from_waste_in_elec_plants',
    'fes heatxdashxcom from waste in chp plants':
    'fes_heatxdashxcom_from_waste_in_chp_plants',
    'pes waste for heatxdashxcom plants':
    'pes_waste_for_heatxdashxcom_plants',
    'share pes waste for heatxdashxcom plants':
    'share_pes_waste_for_heatxdashxcom_plants',
    'share pes waste tfc':
    'share_pes_waste_tfc',
    'share pes waste for chp':
    'share_pes_waste_for_chp',
    'share pes waste for elec plants':
    'share_pes_waste_for_elec_plants',
    'waste change':
    'waste_change',
    'fes elec from waste ej':
    'fes_elec_from_waste_ej',
    'fes elec from waste twh':
    'fes_elec_from_waste_twh',
    'efficiency waste for elec plants':
    'efficiency_waste_for_elec_plants',
    'pes waste ej':
    'pes_waste_ej',
    'pes waste for chp plants':
    'pes_waste_for_chp_plants',
    'efficiency waste for elec chp plants':
    'efficiency_waste_for_elec_chp_plants',
    'initial pes waste':
    'initial_pes_waste',
    'pes waste for tfc':
    'pes_waste_for_tfc',
    'efficiency waste for heat chp plants':
    'efficiency_waste_for_heat_chp_plants',
    'p waste change':
    'p_waste_change',
    'efficiency waste for heat plants':
    'efficiency_waste_for_heat_plants',
    'max waste':
    'max_waste',
    'pes solids bioe xandx waste ej':
    'pes_solids_bioe_xandx_waste_ej',
    'past waste growth':
    'past_waste_growth',
    'ch4 emissions oil':
    'ch4_emissions_oil',
    'gch4 per mj coal':
    'gch4_per_mj_coal',
    'gch4 per mj oil':
    'gch4_per_mj_oil',
    'fe elec generation from fossil fuels twh':
    'fe_elec_generation_from_fossil_fuels_twh',
    'pe demand uranium ej':
    'pe_demand_uranium_ej',
    'fes elec fossil fuel chp plants twh':
    'fes_elec_fossil_fuel_chp_plants_twh',
    'fes heatxdashxcom fossil fuels chp plants ej':
    'fes_heatxdashxcom_fossil_fuels_chp_plants_ej',
    'potential generation nuclear elec twh':
    'potential_generation_nuclear_elec_twh',
    'threshold remaining potential new capacity':
    'threshold_remaining_potential_new_capacity',
    'installed capacity res elec tw':
    'installed_capacity_res_elec_tw',
    'total time planxplusxconstr res elec':
    'total_time_planxplusxconstr_res_elec',
    'new required capacity res elec':
    'new_required_capacity_res_elec',
    'initial capacity in construction res elec':
    'initial_capacity_in_construction_res_elec',
    'remaining potential constraint on new res elec capacity':
    'remaining_potential_constraint_on_new_res_elec_capacity',
    'required capacity res elec tw':
    'required_capacity_res_elec_tw',
    'res elec planned capacity tw':
    'res_elec_planned_capacity_tw',
    'initial energy cons per unit of material cons xrecycledx xdashx data':
    'initial_energy_cons_per_unit_of_material_cons_xrecycledx_xdashx_data',
    'cp phs':
    'cp_phs',
    'max potential phs twe':
    'max_potential_phs_twe',
    'max capacity potential phs':
    'max_capacity_potential_phs',
    'materials per new res elec capacity installed xdashx hvdcs':
    'materials_per_new_res_elec_capacity_installed_xdashx_hvdcs',
    'materials per new res elec capacity installed xdashx material overgrid high power':
    'materials_per_new_res_elec_capacity_installed_xdashx_material_overgrid_high_power',
    'real growth gtl':
    'real_growth_gtl',
    'scarcity conv gas delayed 1yr':
    'scarcity_conv_gas_delayed_1yr',
    'scarcity conv oil delayed 1yr':
    'scarcity_conv_oil_delayed_1yr',
    'pes natx gas':
    'pes_natx_gas',
    'fe elec generation from unconv gas twh':
    'fe_elec_generation_from_unconv_gas_twh',
    'exponent availability conv oil':
    'exponent_availability_conv_oil',
    'pe losses uncon gas for elec ej':
    'pe_losses_uncon_gas_for_elec_ej',
    'abundance total natx gas':
    'abundance_total_natx_gas',
    'exponent availability conv gas':
    'exponent_availability_conv_gas',
    'scarcity conv gas':
    'scarcity_conv_gas',
    'year scarcity total natx gas':
    'year_scarcity_total_natx_gas',
    'share unconv gas vs tot agg in x2050x':
    'share_unconv_gas_vs_tot_agg_in_x2050x',
    'share unconv gas vs tot agg in x2050x laherrere2010':
    'share_unconv_gas_vs_tot_agg_in_x2050x_laherrere2010',
    'share unconv gas vs tot agg in x2050x mohr12 bg':
    'share_unconv_gas_vs_tot_agg_in_x2050x_mohr12_bg',
    'share unconv gas vs tot agg in x2050x user defined':
    'share_unconv_gas_vs_tot_agg_in_x2050x_user_defined',
    'cumulated tot agg gas extraction to x1995x':
    'cumulated_tot_agg_gas_extraction_to_x1995x',
    'share conv gas vs tot agg':
    'share_conv_gas_vs_tot_agg',
    'share rurr tot agg gas to leave underground':
    'share_rurr_tot_agg_gas_to_leave_underground',
    'rurr tot agg gas':
    'rurr_tot_agg_gas',
    'flow tot agg gas left in ground':
    'flow_tot_agg_gas_left_in_ground',
    'tot rurr tot agg gas':
    'tot_rurr_tot_agg_gas',
    'urr total agg gas unlimited':
    'urr_total_agg_gas_unlimited',
    'start policy leave in ground tot agg gas':
    'start_policy_leave_in_ground_tot_agg_gas',
    'extraction unconv gas xdashx tot agg':
    'extraction_unconv_gas_xdashx_tot_agg',
    'total agg gas left in ground':
    'total_agg_gas_left_in_ground',
    'evolution share unconv gas vs tot agg':
    'evolution_share_unconv_gas_vs_tot_agg',
    'real extraction conv gas ej':
    'real_extraction_conv_gas_ej',
    'urr tot agg gas':
    'urr_tot_agg_gas',
    'cumulated tot agg gas extraction':
    'cumulated_tot_agg_gas_extraction',
    'extraction conv gas xdashx tot agg':
    'extraction_conv_gas_xdashx_tot_agg',
    'real extraction unconv gas ej':
    'real_extraction_unconv_gas_ej',
    'extraction tot agg gas ej':
    'extraction_tot_agg_gas_ej',
    'real extraction conv oil mbxdivxd':
    'real_extraction_conv_oil_mbxdivxd',
    'evolution share unconv oil vs tot agg':
    'evolution_share_unconv_oil_vs_tot_agg',
    'share unconv oil vs tot agg in x2050x':
    'share_unconv_oil_vs_tot_agg_in_x2050x',
    'share unconv oil vs tot agg in x2050x laherrere2006':
    'share_unconv_oil_vs_tot_agg_in_x2050x_laherrere2006',
    'share unconv oil vs tot agg in x2050x user defined':
    'share_unconv_oil_vs_tot_agg_in_x2050x_user_defined',
    'extraction conv oil xdashx tot agg':
    'extraction_conv_oil_xdashx_tot_agg',
    'extraction unconv oil xdashx tot agg':
    'extraction_unconv_oil_xdashx_tot_agg',
    'share conv oil vs tot agg':
    'share_conv_oil_vs_tot_agg',
    'rurr tot agg oil':
    'rurr_tot_agg_oil',
    'pes oil ej':
    'pes_oil_ej',
    'extraction tot agg oil ej':
    'extraction_tot_agg_oil_ej',
    'real extraction conv oil ej':
    'real_extraction_conv_oil_ej',
    'gch4 per mj conv gas':
    'gch4_per_mj_conv_gas',
    'total agg oil left in ground':
    'total_agg_oil_left_in_ground',
    'cumulated tot agg oil extraction':
    'cumulated_tot_agg_oil_extraction',
    'cumulated tot agg extraction to x1995x':
    'cumulated_tot_agg_extraction_to_x1995x',
    'urr tot agg oil':
    'urr_tot_agg_oil',
    'urr tot agg oil unlimited':
    'urr_tot_agg_oil_unlimited',
    'tot rurr tot agg oil':
    'tot_rurr_tot_agg_oil',
    'start policy leave in ground tot agg oil':
    'start_policy_leave_in_ground_tot_agg_oil',
    'flow tot agg oil left in ground':
    'flow_tot_agg_oil_left_in_ground',
    'g per mt':
    'g_per_mt',
    'share rurr tot agg oil to leave underground':
    'share_rurr_tot_agg_oil_to_leave_underground',
    'gch4 per mj unconv gas':
    'gch4_per_mj_unconv_gas',
    'tot rurr unconv gas':
    'tot_rurr_unconv_gas',
    'start policy leave in ground coal':
    'start_policy_leave_in_ground_coal',
    'total unconv gas left in ground':
    'total_unconv_gas_left_in_ground',
    'flow coal left in ground':
    'flow_coal_left_in_ground',
    'flow conv gas left in ground':
    'flow_conv_gas_left_in_ground',
    'flow conv oil left in ground':
    'flow_conv_oil_left_in_ground',
    'flow unconv oil left in ground':
    'flow_unconv_oil_left_in_ground',
    'share rurr unconv gas to leave underground':
    'share_rurr_unconv_gas_to_leave_underground',
    'share rurr unconv oil to leave underground':
    'share_rurr_unconv_oil_to_leave_underground',
    'tot rurr coal':
    'tot_rurr_coal',
    'tot rurr conv gas':
    'tot_rurr_conv_gas',
    'tot rurr conv oil':
    'tot_rurr_conv_oil',
    'share rurr conv oil to leave underground':
    'share_rurr_conv_oil_to_leave_underground',
    'tot rurr unconv oil':
    'tot_rurr_unconv_oil',
    'total unconv oil left in ground':
    'total_unconv_oil_left_in_ground',
    'total coal left in ground':
    'total_coal_left_in_ground',
    'total conv gas left in ground':
    'total_conv_gas_left_in_ground',
    'total conv oil left in ground':
    'total_conv_oil_left_in_ground',
    'extraction conv oil ej':
    'extraction_conv_oil_ej',
    'extraction coal ej':
    'extraction_coal_ej',
    'rurr coal':
    'rurr_coal',
    'start policy leave in ground unconv gas':
    'start_policy_leave_in_ground_unconv_gas',
    'flow unconv gas left in ground':
    'flow_unconv_gas_left_in_ground',
    'start policy leave in ground unconv oil':
    'start_policy_leave_in_ground_unconv_oil',
    'start policy leave in ground conv oil':
    'start_policy_leave_in_ground_conv_oil',
    'share rurr coal to leave underground':
    'share_rurr_coal_to_leave_underground',
    'pes oil mbxdivxd':
    'pes_oil_mbxdivxd',
    'current mineral reserves mt':
    'current_mineral_reserves_mt',
    'share rurr conv gas to leave underground':
    'share_rurr_conv_gas_to_leave_underground',
    'start policy leave in ground conv gas':
    'start_policy_leave_in_ground_conv_gas',
    'extraction conv gas ej':
    'extraction_conv_gas_ej',
    'check gases':
    'check_gases',
    'abundance gases':
    'abundance_gases',
    'demand conv gas':
    'demand_conv_gas',
    'ped natx gas without gtl':
    'ped_natx_gas_without_gtl',
    'abundance solids':
    'abundance_solids',
    'gdp':
    'gdp',
    'demand elec gas and coal twh':
    'demand_elec_gas_and_coal_twh',
    'fe demand coal elec plants twh':
    'fe_demand_coal_elec_plants_twh',
    'fe demand gas elec plants twh':
    'fe_demand_gas_elec_plants_twh',
    'fed heat gasxplusxcoal ej':
    'fed_heat_gasxplusxcoal_ej',
    'pe demand oil elec plants ej':
    'pe_demand_oil_elec_plants_ej',
    'fe demand oil elec plants twh':
    'fe_demand_oil_elec_plants_twh',
    'share coal for elec':
    'share_coal_for_elec',
    'current mineral resources mt':
    'current_mineral_resources_mt',
    'output elec over lifetime res elec for allocation2':
    'output_elec_over_lifetime_res_elec_for_allocation2',
    'demand storage capacity':
    'demand_storage_capacity',
    'share capacity storagexdivxres elec var':
    'share_capacity_storagexdivxres_elec_var',
    'staticxdivxdynamic quality of electricityxquestx':
    'staticxdivxdynamic_quality_of_electricityxquestx',
    'xstaticx eroigrid totxdashxeffective for allocation res elec':
    'xstaticx_eroigrid_totxdashxeffective_for_allocation_res_elec',
    'ratio eroigrid vs eroi xstaticx':
    'ratio_eroigrid_vs_eroi_xstaticx',
    'ratio eroi per techn vs eroitot xstaticx':
    'ratio_eroi_per_techn_vs_eroitot_xstaticx',
    'total installed capacity res elec var':
    'total_installed_capacity_res_elec_var',
    'share res elec generation curtailedxandxstored':
    'share_res_elec_generation_curtailedxandxstored',
    'rt storage efficiency phs':
    'rt_storage_efficiency_phs',
    'ratio xequalsx x1':
    'ratio_xequalsx_x1',
    'fei over lifetime res elec':
    'fei_over_lifetime_res_elec',
    'real cp res elec':
    'real_cp_res_elec',
    'fei over lifetime res elec for allocation':
    'fei_over_lifetime_res_elec_for_allocation',
    'remaining potential res elec after intermitt':
    'remaining_potential_res_elec_after_intermitt',
    'remaining potential res elec switch':
    'remaining_potential_res_elec_switch',
    'output elec over lifetime res elec for allocation':
    'output_elec_over_lifetime_res_elec_for_allocation',
    'potential res elec after intermitt twh':
    'potential_res_elec_after_intermitt_twh',
    'replacement rate res elec':
    'replacement_rate_res_elec',
    'xstaticx eroitot res elec':
    'xstaticx_eroitot_res_elec',
    'xstaticx eroitotxdashxeffective for allocation res elec':
    'xstaticx_eroitotxdashxeffective_for_allocation_res_elec',
    'new capacity installed onshore wind tw':
    'new_capacity_installed_onshore_wind_tw',
    'ced oxandxm over lifetime per water res elec var':
    'ced_oxandxm_over_lifetime_per_water_res_elec_var',
    'selfxdashxelectricity consumption res elec':
    'selfxdashxelectricity_consumption_res_elec',
    'share energy requirements for decom res elec':
    'share_energy_requirements_for_decom_res_elec',
    'ced oxandxm over lifetime res elec var':
    'ced_oxandxm_over_lifetime_res_elec_var',
    'grid correction factor res elec':
    'grid_correction_factor_res_elec',
    'materials for oxandxm per capacity installed xdashx csp':
    'materials_for_oxandxm_per_capacity_installed_xdashx_csp',
    'total water for oxandxm required by res elec per techn':
    'total_water_for_oxandxm_required_by_res_elec_per_techn',
    'water for oxandxm required for res elec':
    'water_for_oxandxm_required_for_res_elec',
    'materials for oxandxm per capacity installed res elec':
    'materials_for_oxandxm_per_capacity_installed_res_elec',
    'total energy requirements oxandxm for water consumption res elec':
    'total_energy_requirements_oxandxm_for_water_consumption_res_elec',
    'energy requirements for oxandxm for water consumption res elec':
    'energy_requirements_for_oxandxm_for_water_consumption_res_elec',
    'water for oxandxm xdashx res elec':
    'water_for_oxandxm_xdashx_res_elec',
    'energy requirements per unit of water consumption':
    'energy_requirements_per_unit_of_water_consumption',
    'unlimited nrexquestx':
    'unlimited_nrexquestx',
    'extraction uranium ej':
    'extraction_uranium_ej',
    'urr uranium':
    'urr_uranium',
    'urr conv oil':
    'urr_conv_oil',
    'growth biogas':
    'growth_biogas',
    'past biogas growth':
    'past_biogas_growth',
    'pes biogas ej':
    'pes_biogas_ej',
    'biogas evol':
    'biogas_evol',
    'max biogas ej':
    'max_biogas_ej',
    'fei over lifetime res elec dispatch':
    'fei_over_lifetime_res_elec_dispatch',
    'activate eroi allocation rule':
    'activate_eroi_allocation_rule',
    'eroi allocation rule per res elec':
    'eroi_allocation_rule_per_res_elec',
    'xdynamicx eroi res elec var':
    'xdynamicx_eroi_res_elec_var',
    'eroixdashxini res elec dispatch':
    'eroixdashxini_res_elec_dispatch',
    'res elec capacity under construction tw':
    'res_elec_capacity_under_construction_tw',
    'real generation solar pv ej':
    'real_generation_solar_pv_ej',
    'materials required for new csp mt':
    'materials_required_for_new_csp_mt',
    'materials required for new pv mt':
    'materials_required_for_new_pv_mt',
    'materials required for new wind offshore mt':
    'materials_required_for_new_wind_offshore_mt',
    'materials required for new wind onshore mt':
    'materials_required_for_new_wind_onshore_mt',
    'materials required for oxandxm csp mt':
    'materials_required_for_oxandxm_csp_mt',
    'materials required for oxandxm pv mt':
    'materials_required_for_oxandxm_pv_mt',
    'materials required for oxandxm wind offshore mt':
    'materials_required_for_oxandxm_wind_offshore_mt',
    'materials required for oxandxm wind onshore mt':
    'materials_required_for_oxandxm_wind_onshore_mt',
    'real generation res elec ej':
    'real_generation_res_elec_ej',
    'energy ownxdashxuse constant x830x mtoe in x2013':
    'energy_ownxdashxuse_constant_x830x_mtoe_in_x2013',
    'cedtot solar pv':
    'cedtot_solar_pv',
    'total materials required for new res elec mt':
    'total_materials_required_for_new_res_elec_mt',
    'total materials required for oxandxm res elec mt':
    'total_materials_required_for_oxandxm_res_elec_mt',
    'eroixequalsx1':
    'eroixequalsx1',
    'electrical distribution losses twh':
    'electrical_distribution_losses_twh',
    'max share transmxandxdistr elec losses':
    'max_share_transmxandxdistr_elec_losses',
    'heatxdashxcom distribution losses':
    'heatxdashxcom_distribution_losses',
    'potential generation res elec twh':
    'potential_generation_res_elec_twh',
    'output elec over lifetime res elec':
    'output_elec_over_lifetime_res_elec',
    'variation share transmxandxdistr elec losses':
    'variation_share_transmxandxdistr_elec_losses',
    'remaining share transmxandxdistr elec losses':
    'remaining_share_transmxandxdistr_elec_losses',
    'share transmxandxdistr elec losses':
    'share_transmxandxdistr_elec_losses',
    'total gen losses demand for elec plants ej':
    'total_gen_losses_demand_for_elec_plants_ej',
    'pe losses bioe for elec ej':
    'pe_losses_bioe_for_elec_ej',
    'water for oxandxm xdashx wind offshore':
    'water_for_oxandxm_xdashx_wind_offshore',
    'total materials required for res elec mt':
    'total_materials_required_for_res_elec_mt',
    'share energy for material consumption for alt techn vs tfec':
    'share_energy_for_material_consumption_for_alt_techn_vs_tfec',
    'water for oxandxm xdashx pv':
    'water_for_oxandxm_xdashx_pv',
    'water for oxandxm xdashx wind onshore':
    'water_for_oxandxm_xdashx_wind_onshore',
    'materials for oxandxm per capacity installed xdashx wind offshore':
    'materials_for_oxandxm_per_capacity_installed_xdashx_wind_offshore',
    'materials for oxandxm per capacity installed xdashx wind onshore':
    'materials_for_oxandxm_per_capacity_installed_xdashx_wind_onshore',
    'materials for oxandxm per capacity installed xdashx pv':
    'materials_for_oxandxm_per_capacity_installed_xdashx_pv',
    'water for oxandxm xdashx csp':
    'water_for_oxandxm_xdashx_csp',
    'materials per capacity installed xdashx pv x0':
    'materials_per_capacity_installed_xdashx_pv_x0',
    'cum materials requirements for res elec':
    'cum_materials_requirements_for_res_elec',
    'mj per ej':
    'mj_per_ej',
    'initial energy cons per unit of material cons xvirginx':
    'initial_energy_cons_per_unit_of_material_cons_xvirginx',
    'materials per new capacity installed xdashx csp':
    'materials_per_new_capacity_installed_xdashx_csp',
    'materials per new capacity installed xdashx wind offshore':
    'materials_per_new_capacity_installed_xdashx_wind_offshore',
    'materials per new capacity installed xdashx wind onshore':
    'materials_per_new_capacity_installed_xdashx_wind_onshore',
    'initial cumulated material requirements for res elec x1995x':
    'initial_cumulated_material_requirements_for_res_elec_x1995x',
    'materials per new capacity installed xdashx pv':
    'materials_per_new_capacity_installed_xdashx_pv',
    'kg per mt':
    'kg_per_mt',
    'm per t':
    'm_per_t',
    'elec generation variable from res twh':
    'elec_generation_variable_from_res_twh',
    'new required capacity nuclear':
    'new_required_capacity_nuclear',
    'replacement res for heatxdashxcom':
    'replacement_res_for_heatxdashxcom',
    'losses solar for heat':
    'losses_solar_for_heat',
    'cpxdashxini res for heat':
    'cpxdashxini_res_for_heat',
    'efficiency geothermal for heat':
    'efficiency_geothermal_for_heat',
    'efficiency solar panels for heat':
    'efficiency_solar_panels_for_heat',
    'max fe potential res for heat':
    'max_fe_potential_res_for_heat',
    'p geothermal for heat':
    'p_geothermal_for_heat',
    'past res growth for heatxdashxcom':
    'past_res_growth_for_heatxdashxcom',
    'installed capacity res heatxdashxcom tw':
    'installed_capacity_res_heatxdashxcom_tw',
    'initial value res for heatxdashxcom':
    'initial_value_res_for_heatxdashxcom',
    'p solar for heat':
    'p_solar_for_heat',
    'life time res for heat':
    'life_time_res_for_heat',
    'wear res capacity for heatxdashxcom tw':
    'wear_res_capacity_for_heatxdashxcom_tw',
    'potential fe gen elec fossil fuel chp plants ej':
    'potential_fe_gen_elec_fossil_fuel_chp_plants_ej',
    'fed heat liquids chp plants ej':
    'fed_heat_liquids_chp_plants_ej',
    'fed heat coal chp plants ej':
    'fed_heat_coal_chp_plants_ej',
    'remaining potential csp':
    'remaining_potential_csp',
    'pe csp for elec generation ej':
    'pe_csp_for_elec_generation_ej',
    'invest res for elec':
    'invest_res_for_elec',
    'power density csp':
    'power_density_csp',
    'max potential res elec twe':
    'max_potential_res_elec_twe',
    'surface csp mha':
    'surface_csp_mha',
    'fe elec generation from csp twh':
    'fe_elec_generation_from_csp_twh',
    'invest csp tdolar':
    'invest_csp_tdolar',
    'cp limit nuclear':
    'cp_limit_nuclear',
    'cp nuclear':
    'cp_nuclear',
    'installed capacity nuclear tw':
    'installed_capacity_nuclear_tw',
    'nuclear capacity phasexdashxout':
    'nuclear_capacity_phasexdashxout',
    'min cp nuclear':
    'min_cp_nuclear',
    'replacement nuclear capacity':
    'replacement_nuclear_capacity',
    'min cp baseload res':
    'min_cp_baseload_res',
    'abundance uranium':
    'abundance_uranium',
    'nuclear overcapacity':
    'nuclear_overcapacity',
    'p nuclear scen3xdashx4':
    'p_nuclear_scen3xdashx4',
    'start year nuclear growth scen3xdashx4':
    'start_year_nuclear_growth_scen3xdashx4',
    'p nuclear elec gen':
    'p_nuclear_elec_gen',
    'required capacity nuclear tw':
    'required_capacity_nuclear_tw',
    'time planification nuclear':
    'time_planification_nuclear',
    'time construction nuclear':
    'time_construction_nuclear',
    'initial capacity installed nuclear':
    'initial_capacity_installed_nuclear',
    'new nuclear capacity under planning':
    'new_nuclear_capacity_under_planning',
    'selection of nuclear scenario':
    'selection_of_nuclear_scenario',
    'initial gen nuclear':
    'initial_gen_nuclear',
    'initial capacity in construction nuclear':
    'initial_capacity_in_construction_nuclear',
    'initial required capacity nuclear':
    'initial_required_capacity_nuclear',
    'planned nuclear capacity tw':
    'planned_nuclear_capacity_tw',
    'wear nuclear':
    'wear_nuclear',
    'effects shortage uranium':
    'effects_shortage_uranium',
    'tpes intensity ej tdollar':
    'tpes_intensity_ej_tdollar',
    'real ped intensity of electricity':
    'real_ped_intensity_of_electricity',
    'annual gdp growth rate':
    'annual_gdp_growth_rate',
    'gdp delayed 1yr':
    'gdp_delayed_1yr',
    'share tot monet invest elec res vs gdp':
    'share_tot_monet_invest_elec_res_vs_gdp',
    'gdppc':
    'gdppc',
    'unlimited coalxquestx':
    'unlimited_coalxquestx',
    'real tfec':
    'real_tfec',
    'unlimited uraniumxquestx':
    'unlimited_uraniumxquestx',
    'urr coal unlimited':
    'urr_coal_unlimited',
    'urr uranium unlimited':
    'urr_uranium_unlimited',
    'urr conv gas unlimited':
    'urr_conv_gas_unlimited',
    'unlimited gasxquestx':
    'unlimited_gasxquestx',
    'unlimited oilxquestx':
    'unlimited_oilxquestx',
    'urr conv oil unlimited':
    'urr_conv_oil_unlimited',
    'b logistic':
    'b_logistic',
    'a logistic':
    'a_logistic',
    'activate elf by scenxquestx':
    'activate_elf_by_scenxquestx',
    'total demand liquids mbxdivxd':
    'total_demand_liquids_mbxdivxd',
    'ped coal without ctl':
    'ped_coal_without_ctl',
    'surface hydro mha':
    'surface_hydro_mha',
    'remaining potential hydro':
    'remaining_potential_hydro',
    'grid reinforcement costs tdollar':
    'grid_reinforcement_costs_tdollar',
    'remaining potential onshore wind':
    'remaining_potential_onshore_wind',
    'remaining potential offshore wind':
    'remaining_potential_offshore_wind',
    'remaining potential bioe':
    'remaining_potential_bioe',
    'remaining potential oceanic':
    'remaining_potential_oceanic',
    'remaining potential geotxdashxelec':
    'remaining_potential_geotxdashxelec',
    'remaining potential solarxdashxelec pv':
    'remaining_potential_solarxdashxelec_pv',
    'initial nonxdashxenergy use':
    'initial_nonxdashxenergy_use',
    'real total output':
    'real_total_output',
    'real demand':
    'real_demand',
    'max potential res elec twh':
    'max_potential_res_elec_twh',
    'pe onshore wind for elec generation ej':
    'pe_onshore_wind_for_elec_generation_ej',
    'fe elec generation from solar pv twh':
    'fe_elec_generation_from_solar_pv_twh',
    'power density solar pv':
    'power_density_solar_pv',
    'pe hydro for elec generation ej':
    'pe_hydro_for_elec_generation_ej',
    'invest biow tdolar':
    'invest_biow_tdolar',
    'fe elec generation from hydro twh':
    'fe_elec_generation_from_hydro_twh',
    'invest geotxdashxelec tdolar':
    'invest_geotxdashxelec_tdolar',
    'invest hydro tdolar':
    'invest_hydro_tdolar',
    'fe elec generation from onshore wind twh':
    'fe_elec_generation_from_onshore_wind_twh',
    'invest oceanic tdolar':
    'invest_oceanic_tdolar',
    'invest onshore wind tdolar':
    'invest_onshore_wind_tdolar',
    'invest offshore wind tdolar':
    'invest_offshore_wind_tdolar',
    'fe elec generation from geotxdashxelec twh':
    'fe_elec_generation_from_geotxdashxelec_twh',
    'pe geotxdashxelec for elec generation ej':
    'pe_geotxdashxelec_for_elec_generation_ej',
    'pe solar pv for elec generation ej':
    'pe_solar_pv_for_elec_generation_ej',
    'fe elec generation from bioe twh':
    'fe_elec_generation_from_bioe_twh',
    'pe oceanic for elec generation ej':
    'pe_oceanic_for_elec_generation_ej',
    'pe bioe for elec generation ej':
    'pe_bioe_for_elec_generation_ej',
    'fe elec generation from offshore wind twh':
    'fe_elec_generation_from_offshore_wind_twh',
    'fe elec generation from oceanic twh':
    'fe_elec_generation_from_oceanic_twh',
    'pe offshore wind for elec generation ej':
    'pe_offshore_wind_for_elec_generation_ej',
    'invest solar tdolar':
    'invest_solar_tdolar',
    'initial value land compet biofuels 2gen mha':
    'initial_value_land_compet_biofuels_2gen_mha',
    'initial value land compet biofuels 2gen ktoe':
    'initial_value_land_compet_biofuels_2gen_ktoe',
    'abundance coal':
    'abundance_coal',
    'abundance liquids':
    'abundance_liquids',
    'check liquids':
    'check_liquids',
    'share heat distribution losses':
    'share_heat_distribution_losses',
    'potential tot res elec after intermitt':
    'potential_tot_res_elec_after_intermitt',
    'pe biow for elec generation mtoe':
    'pe_biow_for_elec_generation_mtoe',
    'remaining potential tot res elec after intermitt':
    'remaining_potential_tot_res_elec_after_intermitt',
    'power density res elec twexdivxmha':
    'power_density_res_elec_twexdivxmha',
    'time 95pc ts potential res elec':
    'time_95pc_ts_potential_res_elec',
    'surface res elec':
    'surface_res_elec',
    'max geotxdashxelec twe':
    'max_geotxdashxelec_twe',
    'max pe geotxdashxelec twth':
    'max_pe_geotxdashxelec_twth',
    'efficiency conversion geot pe to elec':
    'efficiency_conversion_geot_pe_to_elec',
    'geot pe potential for heat ej':
    'geot_pe_potential_for_heat_ej',
    'initial instal cap res elec':
    'initial_instal_cap_res_elec',
    'abundance res elec2':
    'abundance_res_elec2',
    'potential tot generation res elec twh':
    'potential_tot_generation_res_elec_twh',
    'res elec tot overcapacity':
    'res_elec_tot_overcapacity',
    'wear res elec':
    'wear_res_elec',
    'g per t':
    'g_per_t',
    'time construction res elec':
    'time_construction_res_elec',
    'time planification res elec':
    'time_planification_res_elec',
    'new res elec capacity under planning':
    'new_res_elec_capacity_under_planning',
    'lifetime res elec':
    'lifetime_res_elec',
    'past res elec capacity growth':
    'past_res_elec_capacity_growth',
    'mt per gt':
    'mt_per_gt',
    'gtl efficiency':
    'gtl_efficiency',
    'ctl efficiency':
    'ctl_efficiency',
    'ctlxplusxgtl gb':
    'ctlxplusxgtl_gb',
    'mbxdivxd per ejxdivxyear':
    'mbxdivxd_per_ejxdivxyear',
    'hist growth ctl':
    'hist_growth_ctl',
    'gboe per ej':
    'gboe_per_ej',
    'hist growth gtl':
    'hist_growth_gtl',
    'urr uranium user defined':
    'urr_uranium_user_defined',
    'replacement rate nuclear':
    'replacement_rate_nuclear',
    'demand gas for oil refinery gains':
    'demand_gas_for_oil_refinery_gains',
    'share res vs tpes':
    'share_res_vs_tpes',
    'tpes mtoe':
    'tpes_mtoe',
    'fe elec generation from nre twh':
    'fe_elec_generation_from_nre_twh',
    'pepc consumption people depending on trad biomass':
    'pepc_consumption_people_depending_on_trad_biomass',
    'share gas for oil refinery gains':
    'share_gas_for_oil_refinery_gains',
    'people relying trad biomass ref':
    'people_relying_trad_biomass_ref',
    'pe consumption trad biomass ref':
    'pe_consumption_trad_biomass_ref',
    'tpefpc threshold high development':
    'tpefpc_threshold_high_development',
    'tped acceptable standard living':
    'tped_acceptable_standard_living',
    'pop not dependent on trad biomass':
    'pop_not_dependent_on_trad_biomass',
    'carbon footprint tco2xdivxperson':
    'carbon_footprint_tco2xdivxperson',
    'average tpespc xwithout trad biomassx':
    'average_tpespc_xwithout_trad_biomassx',
    'tpes xwithout trad biomassx':
    'tpes_xwithout_trad_biomassx',
    'max npp potential bioe residues for heat and elec':
    'max_npp_potential_bioe_residues_for_heat_and_elec',
    'max npp potential bioe residues for cellulosic biofuels':
    'max_npp_potential_bioe_residues_for_cellulosic_biofuels',
    'total cumulative emissions gtco2':
    'total_cumulative_emissions_gtco2',
    'carbon emissions gtc':
    'carbon_emissions_gtc',
    'gco2 per mj unconv gas':
    'gco2_per_mj_unconv_gas',
    'gco2 per mj unconv oil':
    'gco2_per_mj_unconv_oil',
    'co2 emissions ctl':
    'co2_emissions_ctl',
    'gco2 per mj conv gas':
    'gco2_per_mj_conv_gas',
    'gco2 per mj ctl':
    'gco2_per_mj_ctl',
    'gco2 per mj gtl':
    'gco2_per_mj_gtl',
    'gco2 per mj coal':
    'gco2_per_mj_coal',
    'gco2 per mj conv oil':
    'gco2_per_mj_conv_oil',
    'gco2 per mj shale oil':
    'gco2_per_mj_shale_oil',
    'dollars to tdollars':
    'dollars_to_tdollars',
    'land compet 2gen vs total land compet':
    'land_compet_2gen_vs_total_land_compet',
    'gj per ej':
    'gj_per_ej',
    'land shifted to biofuels 3gen':
    'land_shifted_to_biofuels_3gen',
    'average tpes per capita':
    'average_tpes_per_capita',
    'geot pe potential for heat twth':
    'geot_pe_potential_for_heat_twth',
    'pe losses uranium for elec ej':
    'pe_losses_uranium_for_elec_ej',
    'share res for elec vs tpe res':
    'share_res_for_elec_vs_tpe_res',
    'extraction coal mtoe':
    'extraction_coal_mtoe',
    'max extraction coal mtoe':
    'max_extraction_coal_mtoe',
    'tpe from res ej':
    'tpe_from_res_ej',
    'tpe from res mtoe':
    'tpe_from_res_mtoe',
    'electrical distribution losses ej':
    'electrical_distribution_losses_ej',
    'efficiency conversion bioe plants to heat':
    'efficiency_conversion_bioe_plants_to_heat',
    'efficiency improvement biofuels 3gen':
    'efficiency_improvement_biofuels_3gen',
    'potential peavail biofuels 2gen land compet ej':
    'potential_peavail_biofuels_2gen_land_compet_ej',
    'annual additional historic land use biofuels 2gen':
    'annual_additional_historic_land_use_biofuels_2gen',
    'max npp potential bioe residues':
    'max_npp_potential_bioe_residues',
    'grid reinforcement costs':
    'grid_reinforcement_costs',
    'historic land compet available for biofuels 2gen':
    'historic_land_compet_available_for_biofuels_2gen',
    'land productivity biofuels 2gen ej mha':
    'land_productivity_biofuels_2gen_ej_mha',
    'max peavail potential biofuels x2xdashx3gen':
    'max_peavail_potential_biofuels_x2xdashx3gen',
    'land compet biofuels 2gen mha':
    'land_compet_biofuels_2gen_mha',
    'efficiency conversion bioe to elec':
    'efficiency_conversion_bioe_to_elec',
    'potential peavail biofuels prod 3gen ej':
    'potential_peavail_biofuels_prod_3gen_ej',
    'max peavail potential bioe residues for cellulosic biofuels':
    'max_peavail_potential_bioe_residues_for_cellulosic_biofuels',
    'p biofuels 3gen':
    'p_biofuels_3gen',
    'constrain gas exogenous growthxquestx delayed x1yr':
    'constrain_gas_exogenous_growthxquestx_delayed_x1yr',
    'constrain gas exogenous growthxquestx':
    'constrain_gas_exogenous_growthxquestx',
    'constrain liquids exogenous growthxquestx delayed x1yr':
    'constrain_liquids_exogenous_growthxquestx_delayed_x1yr',
    'check liquids delayed 1yr':
    'check_liquids_delayed_1yr',
    'check gas delayed 1yr':
    'check_gas_delayed_1yr',
    'constrain liquids exogenous growthxquestx':
    'constrain_liquids_exogenous_growthxquestx',
    'share variable res elec vs total generation delayed 1yr':
    'share_variable_res_elec_vs_total_generation_delayed_1yr',
    'overcapacity vsx intermittent res penetration x0':
    'overcapacity_vsx_intermittent_res_penetration_x0',
    'total electrical losses ej':
    'total_electrical_losses_ej',
    'gen losses vs pe for elec':
    'gen_losses_vs_pe_for_elec',
    'initial gtl production':
    'initial_gtl_production',
    'initial ctl production':
    'initial_ctl_production',
    'pe demand gas elec plants ej':
    'pe_demand_gas_elec_plants_ej',
    'pe demand coal elec plants ej':
    'pe_demand_coal_elec_plants_ej',
    'max efficiency gas power plants':
    'max_efficiency_gas_power_plants',
    'total pe for electricity consumption ej':
    'total_pe_for_electricity_consumption_ej',
    'initial efficiency gas for electricity':
    'initial_efficiency_gas_for_electricity',
    'remaining efficiency improv gas for electricity':
    'remaining_efficiency_improv_gas_for_electricity',
    'efficiency improv gas for electricity':
    'efficiency_improv_gas_for_electricity',
    'efficiency gas for electricity':
    'efficiency_gas_for_electricity',
    'percent to share':
    'percent_to_share',
    'additional pe production of ctlxplusxgtl for liquids':
    'additional_pe_production_of_ctlxplusxgtl_for_liquids',
    'carbon budget':
    'carbon_budget',
    'cumulative emissions to x1995x':
    'cumulative_emissions_to_x1995x',
    'fe solar potential for heat':
    'fe_solar_potential_for_heat',
    'start year bioe residues for heatxplusxelec':
    'start_year_bioe_residues_for_heatxplusxelec',
    'co2 fossil fuel emissions':
    'co2_fossil_fuel_emissions',
    't per gt':
    't_per_gt',
    'share land compet biofuels':
    'share_land_compet_biofuels',
    'share land total res vs arable':
    'share_land_total_res_vs_arable',
    'share land res land compet vs arable':
    'share_land_res_land_compet_vs_arable',
    'year scarcity oil':
    'year_scarcity_oil',
    'future share gasxdivxxcoalxplusxgasx for elec':
    'future_share_gasxdivxxcoalxplusxgasx_for_elec',
    'cumulated conv gas extraction':
    'cumulated_conv_gas_extraction',
    'cumulated conv gas extraction to x1995x':
    'cumulated_conv_gas_extraction_to_x1995x',
    'cumulated unconv gas extraction':
    'cumulated_unconv_gas_extraction',
    'cumulated unconv gas extraction to x1995x':
    'cumulated_unconv_gas_extraction_to_x1995x',
    'rurr uranium':
    'rurr_uranium',
    'cumulated uranium extraction to x1995x':
    'cumulated_uranium_extraction_to_x1995x',
    'cumulated conv oil extraction':
    'cumulated_conv_oil_extraction',
    'cumulated conv oil extraction to x1995x':
    'cumulated_conv_oil_extraction_to_x1995x',
    'cumulated unconv oil extraction':
    'cumulated_unconv_oil_extraction',
    'cumulated coal extraction':
    'cumulated_coal_extraction',
    'cumulated coal extraction to x1995x':
    'cumulated_coal_extraction_to_x1995x',
    'cumulated unconv oil extraction to x1995x':
    'cumulated_unconv_oil_extraction_to_x1995x',
    'cumulated uranium extraction':
    'cumulated_uranium_extraction',
    'oil refinery gains ej':
    'oil_refinery_gains_ej',
    'oil refinery gains share':
    'oil_refinery_gains_share',
    'efficiency gas for oil refinery gains':
    'efficiency_gas_for_oil_refinery_gains',
    'pes liquids ej':
    'pes_liquids_ej',
    'one year':
    'one_year',
    'land compet required dedicated crops for biofuels':
    'land_compet_required_dedicated_crops_for_biofuels',
    'new biofuels 2gen land compet':
    'new_biofuels_2gen_land_compet',
    'annual shift from 2gen to 3gen':
    'annual_shift_from_2gen_to_3gen',
    'land compet biofuels 3gen mha':
    'land_compet_biofuels_3gen_mha',
    'p bioe residues for heatxplusxelec':
    'p_bioe_residues_for_heatxplusxelec',
    'potential pe cellulosic biofuel ej':
    'potential_pe_cellulosic_biofuel_ej',
    'cellulosic biofuels available':
    'cellulosic_biofuels_available',
    'bioe residues for heatxplusxelec available':
    'bioe_residues_for_heatxplusxelec_available',
    'pe bioe residues for heatxplusxelec ej':
    'pe_bioe_residues_for_heatxplusxelec_ej',
    'p cellulosic biofuels':
    'p_cellulosic_biofuels',
    'share cellulosic biofuels vs bioe residues':
    'share_cellulosic_biofuels_vs_bioe_residues',
    'start year cellulosic biofuels':
    'start_year_cellulosic_biofuels',
    'share land total res vs urban surface':
    'share_land_total_res_vs_urban_surface',
    'max land compet biofuels 2gen':
    'max_land_compet_biofuels_2gen',
    'p biofuels 2gen':
    'p_biofuels_2gen',
    'past biofuels 2gen':
    'past_biofuels_2gen',
    'ej per ktoe':
    'ej_per_ktoe',
    'abundance total oil':
    'abundance_total_oil',
    'cp nuclear initial':
    'cp_nuclear_initial',
    'extraction coal without ctl ej':
    'extraction_coal_without_ctl_ej',
    'extraction coal for ctl ej':
    'extraction_coal_for_ctl_ej',
    'other liquids supply ej':
    'other_liquids_supply_ej',
    'max offshore wind twe':
    'max_offshore_wind_twe',
    'urr unconv oil high mohr15':
    'urr_unconv_oil_high_mohr15',
    'urr unconv oil low mohr15':
    'urr_unconv_oil_low_mohr15',
    'urr unconv oil user defined':
    'urr_unconv_oil_user_defined',
    'urr uranium zittel12':
    'urr_uranium_zittel12',
    'urr conv oil maggio12 high':
    'urr_conv_oil_maggio12_high',
    'urr conv oil maggio12 low':
    'urr_conv_oil_maggio12_low',
    'separate conv and unconv gasxquestx':
    'separate_conv_and_unconv_gasxquestx',
    'urr tot agg oil laherrere x2006x':
    'urr_tot_agg_oil_laherrere_x2006x',
    'urr tot agg oil user defined':
    'urr_tot_agg_oil_user_defined',
    'choose extraction curve tot agg oil':
    'choose_extraction_curve_tot_agg_oil',
    'urr conv oil maggio12 middle':
    'urr_conv_oil_maggio12_middle',
    'urr conv oil user defined':
    'urr_conv_oil_user_defined',
    'choose extraction tot agg gas curve':
    'choose_extraction_tot_agg_gas_curve',
    'urr unconv oil bg mohr15':
    'urr_unconv_oil_bg_mohr15',
    'separate conv and unconv oilxquestx':
    'separate_conv_and_unconv_oilxquestx',
    'urr uranium ewg13':
    'urr_uranium_ewg13',
    'kt uranium per ej':
    'kt_uranium_per_ej',
    'life time nuclear':
    'life_time_nuclear',
    'surface solar pv mha':
    'surface_solar_pv_mha',
    'cumulated uranium extraction kt':
    'cumulated_uranium_extraction_kt',
    'extraction uranium kt':
    'extraction_uranium_kt',
    'efficiency uranium for electricity':
    'efficiency_uranium_for_electricity',
    'fe nuclear elec generation twh':
    'fe_nuclear_elec_generation_twh',
    'kwh per twh':
    'kwh_per_twh',
    'total fe elec demand ej':
    'total_fe_elec_demand_ej',
    'gco2e per kwh biomass':
    'gco2e_per_kwh_biomass',
    'mtoe per ej':
    'mtoe_per_ej',
    'share transmxandxdistr elec losses initial':
    'share_transmxandxdistr_elec_losses_initial',
    'ej per twh':
    'ej_per_twh',
    'surface onshore wind mha':
    'surface_onshore_wind_mha',
    'twe per twh':
    'twe_per_twh',
    'initial population':
    'initial_population',
    'population':
    'population',
    've objetive ue2020 extrap':
    've_objetive_ue2020_extrap',
    've objetive ue2020':
    've_objetive_ue2020',
    'choose extraction uranium curve':
    'choose_extraction_uranium_curve',
    'year scarcity tpe':
    'year_scarcity_tpe',
    'year scarcity uranium':
    'year_scarcity_uranium',
    'year scarcity coal':
    'year_scarcity_coal',
    'year scarcity liquids':
    'year_scarcity_liquids',
    'effects shortage gas':
    'effects_shortage_gas',
    'year scarcity gases':
    'year_scarcity_gases',
    'max percent of change':
    'max_percent_of_change',
    'choose extraction coal curve':
    'choose_extraction_coal_curve',
    'selection constraint extraction unconv gas':
    'selection_constraint_extraction_unconv_gas',
    'year scarcity elec':
    'year_scarcity_elec',
    'extraction unconv gas delayed':
    'extraction_unconv_gas_delayed',
    'max unconv gas growth extraction ej':
    'max_unconv_gas_growth_extraction_ej',
    'p constraint growth extraction unconv gas':
    'p_constraint_growth_extraction_unconv_gas',
    'choose extraction curve unconv gas':
    'choose_extraction_curve_unconv_gas',
    'choose extraction curve unconv oil':
    'choose_extraction_curve_unconv_oil',
    'biofuels land compet available':
    'biofuels_land_compet_available',
    'urr oil aspo':
    'urr_oil_aspo',
    'urr gas mohr high2013':
    'urr_gas_mohr_high2013',
    'urr gas leherrere2010':
    'urr_gas_leherrere2010',
    'urr gas mohr bg2012':
    'urr_gas_mohr_bg2012',
    'res to fossil accounting':
    'res_to_fossil_accounting',
    'new c gtc':
    'new_c_gtc',
    'year adjust':
    'year_adjust',
    'urban surface x2008x':
    'urban_surface_x2008x',
    'c per co2':
    'c_per_co2',
    'activate affores program':
    'activate_affores_program',
    'global arable land':
    'global_arable_land',
    'choose extraction curve conv oil':
    'choose_extraction_curve_conv_oil',
    'choose extraction conv gas curve':
    'choose_extraction_conv_gas_curve',
    'cumulated total monet invest res for elec':
    'cumulated_total_monet_invest_res_for_elec',
    'efficiency coal for electricity':
    'efficiency_coal_for_electricity',
    'efficiency liquids for electricity':
    'efficiency_liquids_for_electricity',
    'additional land compet available for biofuels':
    'additional_land_compet_available_for_biofuels',
    'max hydro twe':
    'max_hydro_twe',
    'max oceanic twe':
    'max_oceanic_twe',
    'max onshore wind twe':
    'max_onshore_wind_twe',
    'p ctl':
    'p_ctl',
    'p gtl':
    'p_gtl',
    'start year 3gen':
    'start_year_3gen',
    'share variable res elec generation vs total':
    'share_variable_res_elec_generation_vs_total',
    'total cumulative emissions gtc':
    'total_cumulative_emissions_gtc',
    'p nuclear scen x1x':
    'p_nuclear_scen_x1x',
    'p nuclear x2xdashx3':
    'p_nuclear_x2xdashx3',
    'gco2e per gtco2e x4x':
    'gco2e_per_gtco2e_x4x',
    'gco2e per gtco2e x3x':
    'gco2e_per_gtco2e_x3x',
    'gco2e per gtco2e x0x':
    'gco2e_per_gtco2e_x0x',
    'gco2e per gtco2e x1x':
    'gco2e_per_gtco2e_x1x',
    'twh per gco2e x0x':
    'twh_per_gco2e_x0x',
    'fed heatxdashxcom ej':
    'fed_heatxdashxcom_ej',
    'evol final energy intensity by sector and fuel':
    'evol_final_energy_intensity_by_sector_and_fuel',
    'total fe elec demand twh':
    'total_fe_elec_demand_twh',
    'real demand by sector':
    'real_demand_by_sector',
    'required total output by sector':
    'required_total_output_by_sector',
    'rate change intensity bottom up':
    'rate_change_intensity_bottom_up',
    'rurr unconv oil ej':
    'rurr_unconv_oil_ej',
    'share liquids for final energy':
    'share_liquids_for_final_energy',
    'ped liquids ej':
    'ped_liquids_ej',
    'fed heat liquids plants ej':
    'fed_heat_liquids_plants_ej',
    'real final energy by sector and fuel':
    'real_final_energy_by_sector_and_fuel',
    'hist var inlandt':
    'hist_var_inlandt',
    'inland transport variation intensity':
    'inland_transport_variation_intensity',
    'fed heatxdashxcom nre ej':
    'fed_heatxdashxcom_nre_ej',
    'real fe consumption by fuel':
    'real_fe_consumption_by_fuel',
    'other liquids required ej':
    'other_liquids_required_ej',
    'fe elec demand consum ej':
    'fe_elec_demand_consum_ej',
    'required fed by liquids ej':
    'required_fed_by_liquids_ej',
    'real gfcf':
    'real_gfcf',
    'real household demand':
    'real_household_demand',
    'demand conv oil ej':
    'demand_conv_oil_ej',
    'max unconv oil growth extraction ej':
    'max_unconv_oil_growth_extraction_ej',
    'fe demand elec consum twh':
    'fe_demand_elec_consum_twh',
    'total fed heatxdashxcom ej':
    'total_fed_heatxdashxcom_ej',
    'real total output by fuel and sector':
    'real_total_output_by_fuel_and_sector',
    'required final energy by sector and fuel':
    'required_final_energy_by_sector_and_fuel',
    'urr unconv oil':
    'urr_unconv_oil',
    'fed heat coal plants ej':
    'fed_heat_coal_plants_ej',
    'urr coal':
    'urr_coal',
    'urr conv gas':
    'urr_conv_gas',
    'real demand by sector delayed':
    'real_demand_by_sector_delayed',
    'required final energy other transport':
    'required_final_energy_other_transport',
    'real total output by sector':
    'real_total_output_by_sector',
    'required final energy air transport':
    'required_final_energy_air_transport',
    'required final energy inland transport':
    'required_final_energy_inland_transport',
    'required final energy water transport':
    'required_final_energy_water_transport',
    'urr coal bg mohr15':
    'urr_coal_bg_mohr15',
    'urr coal user defined ej':
    'urr_coal_user_defined_ej',
    'urr conv gas bg mohr15':
    'urr_conv_gas_bg_mohr15',
    'urr conv gas high mohr15':
    'urr_conv_gas_high_mohr15',
    'urr conv gas low mohr15':
    'urr_conv_gas_low_mohr15',
    'urr conv gas user defined':
    'urr_conv_gas_user_defined',
    'urr total gas laherrere10':
    'urr_total_gas_laherrere10',
    'urr total gas mohr12 bg':
    'urr_total_gas_mohr12_bg',
    'urr total gas user defined':
    'urr_total_gas_user_defined',
    'urr unconv gas':
    'urr_unconv_gas',
    'urr unconv gas bg mohr15':
    'urr_unconv_gas_bg_mohr15',
    'urr unconv gas low mohr15':
    'urr_unconv_gas_low_mohr15',
    'urr coal high15':
    'urr_coal_high15',
    'urr coal low mohr15':
    'urr_coal_low_mohr15',
    'urr unconv gas high mohr15':
    'urr_unconv_gas_high_mohr15',
    'urr unconv gas user defined':
    'urr_unconv_gas_user_defined',
    'urr coal mohr2012 ej':
    'urr_coal_mohr2012_ej',
    'selection constraint extraction unconv oil':
    'selection_constraint_extraction_unconv_oil',
    'abundance electricity':
    'abundance_electricity',
    'p constraint growth extraction unconv oil':
    'p_constraint_growth_extraction_unconv_oil',
    'extraction unconv oil delayed':
    'extraction_unconv_oil_delayed',
    'final time':
    'final_time',
    'initial time':
    'initial_time',
    'saveper':
    'saveper',
    'time step':
    'time_step',
    'new cellulosic biofuels':
    'new_cellulosic_biofuels',
    'new bioe residues for heatxplusxelec':
    'new_bioe_residues_for_heatxplusxelec',
    'new biofuels land marg':
    'new_biofuels_land_marg',
    'share liquids fot heat plants':
    'share_liquids_fot_heat_plants',
    'a lineal regr phasexdashxout oil for heat':
    'a_lineal_regr_phasexdashxout_oil_for_heat',
    'share oil for elec':
    'share_oil_for_elec',
    'a lineal regr phasexdashxout oil for elec':
    'a_lineal_regr_phasexdashxout_oil_for_elec',
    'co2x soilxandxluc emissions':
    'co2x_soilxandxluc_emissions',
    'cp res elec':
    'cp_res_elec',
    'evol final energy intensity by sector and fuel x2x':
    'evol_final_energy_intensity_by_sector_and_fuel_x2x',
    'energy intensity of households rest':
    'energy_intensity_of_households_rest',
    'other forcings rcp':
    'other_forcings_rcp',
    'halocarbon rf':
    'halocarbon_rf',
    'other forcings':
    'other_forcings',
    'global ch4 anthro emissions':
    'global_ch4_anthro_emissions',
    'global hfc emissions':
    'global_hfc_emissions',
    'global pfc emissions':
    'global_pfc_emissions',
    'global sf6 emissions':
    'global_sf6_emissions',
    'global n2o anthro emissions':
    'global_n2o_anthro_emissions',
    'historic water by type intensities by sector':
    'historic_water_by_type_intensities_by_sector',
    'historic water by type intensities for households':
    'historic_water_by_type_intensities_for_households',
    'total co2 emissions gtco2':
    'total_co2_emissions_gtco2',
    'adapt growth res for heat x0x':
    'adapt_growth_res_for_heat_x0x',
    'adapt growth res for heatxdashxcom':
    'adapt_growth_res_for_heatxdashxcom',
    'adapt growth res for heatxdashxnc':
    'adapt_growth_res_for_heatxdashxnc',
    'adapt growth waste':
    'adapt_growth_waste',
    'adapt growth biogas':
    'adapt_growth_biogas',
    'adapt growth phs':
    'adapt_growth_phs',
    'adapt growth res elec':
    'adapt_growth_res_elec',
    'adapt co2 emissions unconv gas':
    'adapt_co2_emissions_unconv_gas',
    'adapt emissions shale oil':
    'adapt_emissions_shale_oil',
    'adapt growth biofuels 2gen':
    'adapt_growth_biofuels_2gen',
    'annual gdppc growth rate':
    'annual_gdppc_growth_rate',
    'variation pop ssp4':
    'variation_pop_ssp4',
    'variation pop ssp5':
    'variation_pop_ssp5',
    'variation pop ssp1':
    'variation_pop_ssp1',
    'variation pop ssp2':
    'variation_pop_ssp2',
    'variation pop ssp3':
    'variation_pop_ssp3',
    'annual population growth rate':
    'annual_population_growth_rate',
    'variation pop ssps':
    'variation_pop_ssps',
    'desired variation gdppc':
    'desired_variation_gdppc',
    'pop variation':
    'pop_variation',
    'variation historic gdppc':
    'variation_historic_gdppc',
    'growth labour share':
    'growth_labour_share',
    'historic new required capacity phs':
    'historic_new_required_capacity_phs',
    'initial required capacity phs':
    'initial_required_capacity_phs',
    'fed coal for heatxdashxnc':
    'fed_coal_for_heatxdashxnc',
    'historical variation minerals extraction rest':
    'historical_variation_minerals_extraction_rest',
    'max unconv gas growth extraction':
    'max_unconv_gas_growth_extraction',
    'new res capacity for heatxdashxnc tw':
    'new_res_capacity_for_heatxdashxnc_tw',
    'annual variation nonxdashxenergy use':
    'annual_variation_nonxdashxenergy_use',
    'pes solids bioe ej':
    'pes_solids_bioe_ej',
    'real fe consumption solids ej':
    'real_fe_consumption_solids_ej',
    'variation demand flow fd':
    'variation_demand_flow_fd',
    'historic variation demand':
    'historic_variation_demand',
    'historic rate final energy intensity h':
    'historic_rate_final_energy_intensity_h',
    'fed natx gas for heatxdashxnc':
    'fed_natx_gas_for_heatxdashxnc',
    'fed oil for heatxdashxnc':
    'fed_oil_for_heatxdashxnc',
    'pes solids':
    'pes_solids',
    'ped gas heatxdashxnc':
    'ped_gas_heatxdashxnc',
    'ped coal heatxdashxnc':
    'ped_coal_heatxdashxnc',
    'ped liquids heatxdashxnc':
    'ped_liquids_heatxdashxnc',
    'ped coal ej':
    'ped_coal_ej',
    'fes heat from coal':
    'fes_heat_from_coal',
    'fes heat from natx gas':
    'fes_heat_from_natx_gas',
    'fes heat from oil':
    'fes_heat_from_oil',
    'new res capacity for heatxdashxcom tw':
    'new_res_capacity_for_heatxdashxcom_tw',
    'variation historic demand':
    'variation_historic_demand',
    'variation gfcf':
    'variation_gfcf',
    'variation historic gfcf':
    'variation_historic_gfcf',
    'share consum goverment and inventories':
    'share_consum_goverment_and_inventories',
    'share consum goverments and inventories next step':
    'share_consum_goverments_and_inventories_next_step',
    'cc sectoral':
    'cc_sectoral',
    'variation cc sectoral':
    'variation_cc_sectoral',
    'cedtot per tw over lifetime res elec dispatch':
    'cedtot_per_tw_over_lifetime_res_elec_dispatch',
    'historic rate final energy intensity':
    'historic_rate_final_energy_intensity',
    'fed heatxdashxcom by nre chp plants ej':
    'fed_heatxdashxcom_by_nre_chp_plants_ej',
    'pe supply res nonxdashxelec ej':
    'pe_supply_res_nonxdashxelec_ej',
    'new pes biogas':
    'new_pes_biogas',
    'new waste supply ej':
    'new_waste_supply_ej',
    'historic new required capacity res elec':
    'historic_new_required_capacity_res_elec',
    'share unconv gas vs tot agg':
    'share_unconv_gas_vs_tot_agg',
    'max extraction tot agg gas ej':
    'max_extraction_tot_agg_gas_ej',
    'share unconv oil vs tot agg':
    'share_unconv_oil_vs_tot_agg',
    'max extraction tot agg oil ej':
    'max_extraction_tot_agg_oil_ej',
    'extraction unconv gas ej':
    'extraction_unconv_gas_ej',
    'max extraction conv oil ej':
    'max_extraction_conv_oil_ej',
    'share chp plants oil':
    'share_chp_plants_oil',
    'share chp plants coal':
    'share_chp_plants_coal',
    'share coalxcoalxplusxgasx for heat plants':
    'share_coalxcoalxplusxgasx_for_heat_plants',
    'cp baseload reduction':
    'cp_baseload_reduction',
    'pes peat ej':
    'pes_peat_ej',
    'total gen losses demand for heat plants ej':
    'total_gen_losses_demand_for_heat_plants_ej',
    'total gen losses demand for chp plants ej':
    'total_gen_losses_demand_for_chp_plants_ej',
    'potential fe gen elec liquids chp plants ej':
    'potential_fe_gen_elec_liquids_chp_plants_ej',
    'potential fe gen elec coal chp plants ej':
    'potential_fe_gen_elec_coal_chp_plants_ej',
    'potential fe gen elec gas chp plants ej':
    'potential_fe_gen_elec_gas_chp_plants_ej',
    'fed heat gas chp plants ej':
    'fed_heat_gas_chp_plants_ej',
    'ped oil for chp plants ej':
    'ped_oil_for_chp_plants_ej',
    'ped gas for chp plants ej':
    'ped_gas_for_chp_plants_ej',
    'ped coal for chp plants ej':
    'ped_coal_for_chp_plants_ej',
    'invest nuclear tdolar':
    'invest_nuclear_tdolar',
    'nuclear capacity under construction':
    'nuclear_capacity_under_construction',
    'invest res elec tdolar':
    'invest_res_elec_tdolar',
    'power density res elec twxdivxmha':
    'power_density_res_elec_twxdivxmha',
    'cpxdashxini res elec subgeotxdashxelec':
    'cpxdashxini_res_elec_subgeotxdashxelec',
    'cpxdashxini res elec subsolid bioexdashxelec':
    'cpxdashxini_res_elec_subsolid_bioexdashxelec',
    'cpxdashxini res elec suboceanic':
    'cpxdashxini_res_elec_suboceanic',
    'cpxdashxini res elec subwind onshore':
    'cpxdashxini_res_elec_subwind_onshore',
    'cpxdashxini res elec subwind offshore':
    'cpxdashxini_res_elec_subwind_offshore',
    'cpxdashxini res elec subsolar pv':
    'cpxdashxini_res_elec_subsolar_pv',
    'cpxdashxini res elec subcsp':
    'cpxdashxini_res_elec_subcsp',
    'initial required capacity res elec':
    'initial_required_capacity_res_elec',
    'afforestation program x2020x gtco2':
    'afforestation_program_x2020x_gtco2',
    'max extraction uranium ej':
    'max_extraction_uranium_ej',
    'gtco2 historic emissions rcps':
    'gtco2_historic_emissions_rcps',
    'variation historic pop':
    'variation_historic_pop',
    'tpes de castro phd xdashx scen xmadcoalx ej':
    'tpes_de_castro_phd_xdashx_scen_xmadcoalx_ej',
    'tpes de castro phd xdashx scen ii ej':
    'tpes_de_castro_phd_xdashx_scen_ii_ej',
    'balancing costs':
    'balancing_costs',
    'variation ctl':
    'variation_ctl',
    'variation gtl':
    'variation_gtl',
    'improvement efficiency gas for electricity':
    'improvement_efficiency_gas_for_electricity',
    'annual additional historic product biofuels 2gen':
    'annual_additional_historic_product_biofuels_2gen',
    'share gasxdivxxcoal xplusxgasx for elec':
    'share_gasxdivxxcoal_xplusxgasx_for_elec',
    'max unconv oil growth extraction':
    'max_unconv_oil_growth_extraction',
    'extraction unconv oil ej':
    'extraction_unconv_oil_ej',
    'max extraction unconv gas':
    'max_extraction_unconv_gas',
    'max extraction unconv oil':
    'max_extraction_unconv_oil',
    'max extraction coal ej':
    'max_extraction_coal_ej',
    'max extraction conv gas ej':
    'max_extraction_conv_gas_ej',
    'fed heat gas plants ej':
    'fed_heat_gas_plants_ej',
    'ped gases for heat plants ej':
    'ped_gases_for_heat_plants_ej',
    'ped oil for heat plants ej':
    'ped_oil_for_heat_plants_ej',
    'ped coal for heat plants ej':
    'ped_coal_for_heat_plants_ej'
}


def hadcrut4_temperature(x):
    """
    hadcrut4 temperature



    lookup


    """
    return functions.lookup(x, [
        1850, 1851, 1852, 1853, 1854, 1855, 1856, 1857, 1858, 1859, 1860, 1861, 1862, 1863, 1864,
        1865, 1866, 1867, 1868, 1869, 1870, 1871, 1872, 1873, 1874, 1875, 1876, 1877, 1878, 1879,
        1880, 1881, 1882, 1883, 1884, 1885, 1886, 1887, 1888, 1889, 1890, 1891, 1892, 1893, 1894,
        1895, 1896, 1897, 1898, 1899, 1900, 1901, 1902, 1903, 1904, 1905, 1906, 1907, 1908, 1909,
        1910, 1911, 1912, 1913, 1914, 1915, 1916, 1917, 1918, 1919, 1920, 1921, 1922, 1923, 1924,
        1925, 1926, 1927, 1928, 1929, 1930, 1931, 1932, 1933, 1934, 1935, 1936, 1937, 1938, 1939,
        1940, 1941, 1942, 1943, 1944, 1945, 1946, 1947, 1948, 1949, 1950, 1951, 1952, 1953, 1954,
        1955, 1956, 1957, 1958, 1959, 1960, 1961, 1962, 1963, 1964, 1965, 1966, 1967, 1968, 1969,
        1970, 1971, 1972, 1973, 1974, 1975, 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984,
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016
    ], [
        -0.064, 0.089, 0.086, 0.04099999999999998, 0.059, 0.03699999999999998,
        -0.04299999999999998, -0.15000000000000002, -0.15600000000000003, 0.02300000000000002,
        -0.03899999999999998, -0.10299999999999998, -0.21600000000000003, 0.02899999999999997,
        -0.184, 0.035999999999999976, 0.061, -0.01200000000000001, 0.07599999999999998,
        0.04299999999999998, 0.03199999999999997, -0.02400000000000002, 0.07999999999999999,
        0.006000000000000005, -0.065, -0.08700000000000002, -0.07200000000000001,
        0.23399999999999999, 0.346, 0.07799999999999999, 0.08099999999999999, 0.10300000000000001,
        0.096, 0.01200000000000001, -0.10099999999999998, -0.08000000000000002,
        -0.057999999999999996, -0.11199999999999999, -0.0020000000000000018, 0.137,
        -0.10699999999999998, -0.025000000000000022, -0.14600000000000002, -0.16299999999999998,
        -0.09500000000000003, -0.08000000000000002, 0.126, 0.10200000000000001,
        -0.10299999999999998, 0.02100000000000002, 0.11199999999999999, 0.04899999999999999,
        -0.09400000000000003, -0.172, -0.20900000000000002, -0.067, 0.026000000000000023,
        -0.15600000000000003, -0.201, -0.21300000000000002, -0.181, -0.23300000000000004, -0.127,
        -0.11499999999999999, 0.065, 0.163, -0.07200000000000001, -0.15300000000000002,
        -0.02100000000000002, 0.03799999999999998, 0.069, 0.123, 0.009000000000000008,
        0.03799999999999998, 0.018000000000000016, 0.096, 0.20500000000000002, 0.10200000000000001,
        0.10400000000000001, -0.03799999999999998, 0.176, 0.22699999999999998, 0.176,
        0.04299999999999998, 0.183, 0.138, 0.16999999999999998, 0.288, 0.307, 0.263, 0.328, 0.33,
        0.286, 0.31, 0.45899999999999996, 0.337, 0.238, 0.272, 0.273, 0.238, 0.138, 0.259,
        0.34099999999999997, 0.40700000000000003, 0.181, 0.12, 0.043999999999999984, 0.306, 0.356,
        0.327, 0.261, 0.35, 0.326, 0.359, 0.087, 0.16999999999999998, 0.241, 0.236, 0.198,
        0.34099999999999997, 0.283, 0.123, 0.243, 0.372, 0.097, 0.163, 0.07, 0.356, 0.247, 0.368,
        0.403, 0.45, 0.321, 0.503, 0.297, 0.28, 0.356, 0.501, 0.509, 0.428, 0.606,
        0.5640000000000001, 0.413, 0.45499999999999996, 0.516, 0.631, 0.49, 0.6990000000000001,
        0.8460000000000001, 0.616, 0.603, 0.749, 0.8069999999999999, 0.8180000000000001, 0.758,
        0.8540000000000001, 0.815, 0.802, 0.704, 0.8160000000000001, 0.8660000000000001, 0.731,
        0.7789999999999999, 0.8220000000000001, 0.885, 1.07, 1.083
    ])


def co2_mauna_loa(x):
    """
    co2 mauna loa



    lookup


    """
    return functions.lookup(x, [
        1959, 1960, 1961, 1962, 1963, 1964, 1965, 1966, 1967, 1968, 1969, 1970, 1971, 1972, 1973,
        1974, 1975, 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984, 1985, 1986, 1987, 1988,
        1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003,
        2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        315.97, 316.91, 317.64, 318.45, 318.99, 319.62, 320.04, 321.38, 322.16, 323.04, 324.62,
        325.68, 326.32, 327.45, 329.68, 330.18, 331.11, 332.04, 333.83, 335.4, 336.84, 338.75,
        340.11, 341.45, 343.05, 344.65, 346.12, 347.42, 349.19, 351.57, 353.12, 354.39, 355.61,
        356.45, 357.1, 358.83, 360.82, 362.61, 363.73, 366.7, 368.38, 369.55, 371.14, 373.28,
        375.8, 377.52, 379.8, 381.9, 383.79, 385.6, 387.43, 389.9, 391.65, 393.85, 396.52, 398.65,
        400.83, 404.21
    ])


def giss_nasa_temperature(x):
    """
    giss nasa temperature



    lookup


    """
    return functions.lookup(x, [
        1880, 1881, 1882, 1883, 1884, 1885, 1886, 1887, 1888, 1889, 1890, 1891, 1892, 1893, 1894,
        1895, 1896, 1897, 1898, 1899, 1900, 1901, 1902, 1903, 1904, 1905, 1906, 1907, 1908, 1909,
        1910, 1911, 1912, 1913, 1914, 1915, 1916, 1917, 1918, 1919, 1920, 1921, 1922, 1923, 1924,
        1925, 1926, 1927, 1928, 1929, 1930, 1931, 1932, 1933, 1934, 1935, 1936, 1937, 1938, 1939,
        1940, 1941, 1942, 1943, 1944, 1945, 1946, 1947, 1948, 1949, 1950, 1951, 1952, 1953, 1954,
        1955, 1956, 1957, 1958, 1959, 1960, 1961, 1962, 1963, 1964, 1965, 1966, 1967, 1968, 1969,
        1970, 1971, 1972, 1973, 1974, 1975, 1976, 1977, 1978, 1979, 1980, 1981, 1982, 1983, 1984,
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016
    ], [
        0.024999999999999994, 0.115, 0.10500000000000001, 0.024999999999999994,
        -0.07499999999999998, -0.095, -0.10500000000000001, -0.13499999999999998, 0.035,
        0.10500000000000001, -0.155, -0.024999999999999994, -0.05500000000000002,
        -0.10500000000000001, -0.10500000000000001, -0.014999999999999986, 0.095, 0.095,
        -0.06499999999999997, 0.024999999999999994, 0.125, 0.065, -0.07499999999999998, -0.165,
        -0.275, -0.07499999999999998, -0.014999999999999986, -0.18499999999999997, -0.225, -0.265,
        -0.215, -0.205, -0.145, -0.13499999999999998, 0.05499999999999999, 0.095,
        -0.11500000000000002, -0.225, -0.06499999999999997, -0.05500000000000002,
        -0.04500000000000001, 0.035, -0.05500000000000002, -0.024999999999999994, -0.035,
        0.0050000000000000044, 0.125, 0.015000000000000013, 0.024999999999999994,
        -0.13499999999999998, 0.07500000000000001, 0.115, 0.05499999999999999,
        -0.07499999999999998, 0.08499999999999999, 0.015000000000000013, 0.065, 0.185, 0.185,
        0.195, 0.335, 0.405, 0.275, 0.29500000000000004, 0.42500000000000004, 0.305, 0.155, 0.185,
        0.115, 0.115, 0.04500000000000001, 0.165, 0.23500000000000001, 0.29500000000000004,
        0.08499999999999999, 0.08499999999999999, 0.024999999999999994, 0.275, 0.29500000000000004,
        0.255, 0.20500000000000002, 0.275, 0.265, 0.29500000000000004, 0.024999999999999994, 0.115,
        0.175, 0.20500000000000002, 0.14500000000000002, 0.29500000000000004, 0.255, 0.135,
        0.23500000000000001, 0.385, 0.14500000000000002, 0.20500000000000002, 0.115, 0.395,
        0.28500000000000003, 0.385, 0.495, 0.555, 0.355, 0.535, 0.385, 0.345, 0.405, 0.555, 0.635,
        0.515, 0.665, 0.635, 0.445, 0.46499999999999997, 0.535, 0.665, 0.555, 0.695, 0.845, 0.625,
        0.625, 0.755, 0.845, 0.835, 0.755, 0.895, 0.845, 0.865, 0.745, 0.865, 0.9249999999999999,
        0.8049999999999999, 0.845, 0.865, 0.955, 1.095, 1.225
    ])


def other_forcings_history(x):
    """
    other forcings history



    lookup


    """
    return functions.lookup(x, [1990, 1991, 1992, 1993, 1994, 1995, 2000, 2005, 2010], [
        -0.49598, -1.98053, -3.41805, -1.58304, -0.92833, -0.7171, -0.981, -0.966, -0.896
    ])


def mp_rf_total(x):
    """
    mp rf total



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2015, 2020, 2025, 2030, 2035, 2040, 2045, 2050, 2055,
        2060, 2065, 2070, 2075, 2080, 2085, 2090, 2095, 2100
    ], [
        0.287364, 0.2981974, 0.30549670000000007, 0.3116818, 0.31517199999999995, 0.3179608,
        0.32052250000000004, 0.3220829, 0.32290989999999997, 0.3241693, 0.32495349999999995,
        0.3257068999999999, 0.32634300000000005, 0.3262145999999999, 0.32629450000000004,
        0.32596909999999996, 0.32495520000000006, 0.3240775999999999, 0.3231530999999999,
        0.32227409999999995, 0.32141790000000003, 0.316705, 0.3112558, 0.30478420000000006,
        0.297133, 0.2870987, 0.26998350000000004, 0.24620709999999996, 0.221897, 0.20014, 0.1816,
        0.16609, 0.153, 0.14193, 0.13232, 0.124, 0.11653, 0.10983, 0.10374
    ])


def other_forcings_rcp_scenario_subrcp26(x):
    """
    other forcings rcp scenario subrcp26



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
        -0.71688, -0.57694, -0.48852, -0.42674, -0.41276, -0.43195, -0.41792, -0.38228, -0.35317,
        -0.32284
    ])


def other_forcings_rcp_scenario_subrcp45(x):
    """
    other forcings rcp scenario subrcp45



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
        -0.71329, -0.60463, -0.51849, -0.43097, -0.34356, -0.29633, -0.25669, -0.22573, -0.22653,
        -0.22386
    ])


def other_forcings_rcp_scenario_subrcp60(x):
    """
    other forcings rcp scenario subrcp60



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
        -0.75125, -0.67085, -0.5727, -0.57466, -0.52115, -0.50936, -0.3863, -0.32024, -0.32243,
        -0.32754
    ])


def other_forcings_rcp_scenario_subrcp85(x):
    """
    other forcings rcp scenario subrcp85



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
        -0.69615, -0.6482, -0.57292, -0.45195, -0.34149, -0.27418, -0.22593, -0.18895, -0.12128,
        -0.08835
    ])


def global_sf6_emissions_rcp_subrcp26(x):
    """
    global sf6 emissions rcp subrcp26



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            5520.77, 5538.0, 6341.0, 6623.1, 2244.2000000000003, 1952.3, 1567.6000000000001,
            632.1999999999999, 498.5, 418.3, 315.8, 185.20000000000002, 44.2
        ])


def global_sf6_emissions_rcp_subrcp45(x):
    """
    global sf6 emissions rcp subrcp45



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            5520.77, 5538.0, 6341.0, 5655.400000000001, 2502.7, 2910.8999999999996, 3337.1, 3781.6,
            4293.0, 4894.8, 5586.9, 6010.6, 6434.3
        ])


def global_sf6_emissions_rcp_subrcp60(x):
    """
    global sf6 emissions rcp subrcp60



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            5520.77, 5538.0, 6341.0, 7770.3, 9425.4, 9602.5, 9687.300000000001, 9779.5, 9928.0,
            10082.800000000001, 9996.1, 9874.9, 9497.1
        ])


def global_sf6_emissions_rcp_subrcp85(x):
    """
    global sf6 emissions rcp subrcp85



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            5520.77, 5538.0, 6341.0, 7044.3, 8081.200000000001, 9988.300000000001,
            11260.300000000001, 11965.3, 14059.699999999999, 13492.0, 14875.8, 16052.7, 16922.3
        ])


def global_ch4_anthro_emissions_rcp_subrcp26(x):
    """
    global ch4 anthro emissions rcp subrcp26



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            226.47982000000002, 229.35242, 231.19312000000002, 242.67688, 208.7452, 197.1581,
            187.833, 158.7792, 152.24050000000003, 146.4999, 140.8769, 135.1028, 130.4424
        ])


def global_ch4_anthro_emissions_rcp_subrcp45(x):
    """
    global ch4 anthro emissions rcp subrcp45



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            226.47982000000002, 229.24151999999998, 232.26422999999997, 238.28171999999998,
            250.3293, 255.85819999999995, 257.0696, 253.9631, 245.01260000000002, 233.6735,
            219.95030000000003, 213.96869999999998, 207.987
        ])


def global_ch4_anthro_emissions_rcp_subrcp60(x):
    """
    global ch4 anthro emissions rcp subrcp60



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            226.47982000000002, 229.27952, 240.16632, 247.16665, 237.8823, 247.3007, 255.6114,
            261.3207, 268.9658, 276.2532, 269.48830000000004, 228.5847, 210.30579999999998
        ])


def global_ch4_anthro_emissions_rcp_subrcp85(x):
    """
    global ch4 anthro emissions rcp subrcp85



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [
        226.47982000000002, 229.27931999999998, 250.01143, 269.93085, 312.293, 355.548,
        408.2289999999999, 464.02099999999996, 503.03, 527.515, 547.7719999999999, 559.518, 558.015
    ])


def global_hfc_emissions_rcp_x2x6_subhfc134a(x):
    """
    global hfc emissions rcp x2x6 subhfc134a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            1010, 75039, 120933, 157043, 173879, 194186, 212829, 198351, 226357, 255846, 265700,
            256848, 235392
        ])


def global_hfc_emissions_rcp_x2x6_subhfc23(x):
    """
    global hfc emissions rcp x2x6 subhfc23



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [6622, 10395, 10812, 9522, 1152, 691, 413, 246, 138, 73, 37, 0, 0])


def global_hfc_emissions_rcp_x2x6_subhfc32(x):
    """
    global hfc emissions rcp x2x6 subhfc32



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [0, 4000, 10994, 17942, 60642, 72035, 80712, 74607, 84086, 93880, 96271, 91204, 81227])


def global_hfc_emissions_rcp_x2x6_subhfc125(x):
    """
    global hfc emissions rcp x2x6 subhfc125



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            186, 8538, 13759, 26279, 89349, 108899, 124961, 119137, 138797, 159182, 166569, 161556,
            148214
        ])


def global_hfc_emissions_rcp_x2x6_subhfc143a(x):
    """
    global hfc emissions rcp x2x6 subhfc143a



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [889, 6234, 12448, 21616, 64374, 75555, 83724, 77089, 86593, 95507, 95810, 88763, 77457])


def global_hfc_emissions_rcp_x2x6_subhfc152a(x):
    """
    global hfc emissions rcp x2x6 subhfc152a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [12, 15200, 23010, 34530, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def global_hfc_emissions_rcp_x2x6_subhfc227ea(x):
    """
    global hfc emissions rcp x2x6 subhfc227ea



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [0, 1951, 4890, 8979, 1608, 795, 886, 296, 142, 132, 120, 107, 91])


def global_hfc_emissions_rcp_x2x6_subhfc245ca(x):
    """
    global hfc emissions rcp x2x6 subhfc245ca



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [0, 17926, 26888, 51053, 32321, 3174, 0, 0, 0, 0, 0, 0, 0])


def global_hfc_emissions_rcp_x2x6_subhfc4310mee(x):
    """
    global hfc emissions rcp x2x6 subhfc4310mee



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [1.037, 105.5, 206.6, 305.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])


def global_hfc_emissions_rcp_x4x5_subhfc134a(x):
    """
    global hfc emissions rcp x4x5 subhfc134a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            1010, 75039, 120933, 142731, 189436, 208276, 229429, 252971, 243443, 258052, 300616,
            353004, 409075
        ])


def global_hfc_emissions_rcp_x4x5_subhfc23(x):
    """
    global hfc emissions rcp x4x5 subhfc23



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [6622, 10395, 10812, 9522, 1152, 691, 413, 246, 138, 73, 37, 0, 0])


def global_hfc_emissions_rcp_x4x5_subhfc32(x):
    """
    global hfc emissions rcp x4x5 subhfc32



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [0, 4000, 10994, 17942, 60642, 72035, 80712, 74607, 84086, 93880, 96271, 91204, 81227])


def global_hfc_emissions_rcp_x4x5_subhfc125(x):
    """
    global hfc emissions rcp x4x5 subhfc125



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [186, 8538, 13759, 17127, 31282, 31762, 31380, 30710, 26222, 23990, 23536, 23520, 23006])


def global_hfc_emissions_rcp_x4x5_subhfc143a(x):
    """
    global hfc emissions rcp x4x5 subhfc143a



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [889, 6234, 12448, 14163, 17666, 20459, 23292, 26118, 25539, 28046, 33790, 39928, 46270])


def global_hfc_emissions_rcp_x4x5_subhfc152a(x):
    """
    global hfc emissions rcp x4x5 subhfc152a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [12, 15200, 23010, 34530, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def global_hfc_emissions_rcp_x4x5_subhfc227ea(x):
    """
    global hfc emissions rcp x4x5 subhfc227ea



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [0, 1951, 4890, 6212, 600, 258, 257, 89, 31, 22, 18, 16, 14])


def global_hfc_emissions_rcp_x4x5_subhfc245ca(x):
    """
    global hfc emissions rcp x4x5 subhfc245ca



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [0, 17926, 26888, 48596, 92011, 95765, 96916, 95465, 86276, 80878, 79272, 81080, 82888])


def global_hfc_emissions_rcp_x4x5_subhfc4310mee(x):
    """
    global hfc emissions rcp x4x5 subhfc4310mee



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [1.037, 105.5, 206.6, 305.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])


def global_hfc_emissions_rcp_x6x0_subhfc134a(x):
    """
    global hfc emissions rcp x6x0 subhfc134a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            1010, 75039, 120933, 146301, 153891, 159119, 163629, 167105, 171969, 175922, 177072,
            174566, 170500
        ])


def global_hfc_emissions_rcp_x6x0_subhfc23(x):
    """
    global hfc emissions rcp x6x0 subhfc23



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [6622, 10395, 10812, 9522, 1152, 691, 413, 246, 138, 73, 37, 0, 0])


def global_hfc_emissions_rcp_x6x0_subhfc32(x):
    """
    global hfc emissions rcp x6x0 subhfc32



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [0, 4000, 10994, 17942, 60642, 72035, 80712, 74607, 84086, 93880, 96271, 91204, 81227])


def global_hfc_emissions_rcp_x6x0_subhfc125(x):
    """
    global hfc emissions rcp x6x0 subhfc125



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [186, 8538, 13759, 8121, 8889, 9418, 10006, 10385, 10938, 11420, 11739, 11779, 11699])


def global_hfc_emissions_rcp_x6x0_subhfc143a(x):
    """
    global hfc emissions rcp x6x0 subhfc143a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [889, 6234, 12448, 4514, 5554, 6059, 6639, 7119, 7758, 8342, 8768, 8999, 9099])


def global_hfc_emissions_rcp_x6x0_subhfc152a(x):
    """
    global hfc emissions rcp x6x0 subhfc152a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [12, 15200, 23010, 34530, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def global_hfc_emissions_rcp_x6x0_subhfc227ea(x):
    """
    global hfc emissions rcp x6x0 subhfc227ea



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [0, 1951, 4890, 8820, 1516, 701, 707, 207, 85, 66, 51, 38, 27])


def global_hfc_emissions_rcp_x6x0_subhfc245ca(x):
    """
    global hfc emissions rcp x6x0 subhfc245ca



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [0, 17926, 26888, 63231, 65706, 67329, 68510, 68946, 70024, 70671, 70024, 67758, 65062])


def global_hfc_emissions_rcp_x6x0_subhfc4310mee(x):
    """
    global hfc emissions rcp x6x0 subhfc4310mee



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [1.037, 105.5, 206.6, 305.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])


def global_pfc_emissions_rcp_subrcp26(x):
    """
    global pfc emissions rcp subrcp26



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11997.68, 12000.1, 11605.9, 1491.67, 1221.6799999999998, 669.49, 505.22, 263.85,
            217.60000000000002, 213.77, 144.4, 119.57, 109.21000000000001
        ])


def global_pfc_emissions_rcp_subrcp45(x):
    """
    global pfc emissions rcp subrcp45



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11997.68, 12000.1, 11605.9, 10608.599999999999, 8441.5, 8445.6, 8663.4, 9135.7, 8904.3,
            9003.9, 9465.199999999999, 10581.6, 11396.4
        ])


def global_pfc_emissions_rcp_subrcp60(x):
    """
    global pfc emissions rcp subrcp60



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11997.68, 12000.1, 11605.9, 22209.7, 22873.5, 23524.4, 23833.300000000003,
            23712.800000000003, 23725.3, 23459.5, 22934.6, 21940.4, 20799.0
        ])


def global_pfc_emissions_rcp_subrcp85(x):
    """
    global pfc emissions rcp subrcp85



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11997.68, 12000.1, 11605.9, 10634.4, 11774.6, 12414.8, 12139.6, 12490.8, 11773.5,
            11840.9, 11592.800000000001, 11093.199999999999, 10765.4
        ])


def global_n2o_anthro_emissions_rcp_subrcp26(x):
    """
    global n2o anthro emissions rcp subrcp26



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            7.585454545454545, 7.4569090909090905, 7.684090909090909, 7.8380018181818185, 7.3633,
            7.305002727272727, 7.1772018181818185, 6.246602727272728, 5.718102727272727,
            5.741100909090909, 5.613401818181818, 5.460699999999999, 5.282301818181819
        ])


def global_n2o_anthro_emissions_rcp_subrcp45(x):
    """
    global n2o anthro emissions rcp subrcp45



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            7.585454545454545, 7.4569090909090905, 7.684090909090909, 7.868197272727272,
            8.235302727272728, 8.57179909090909, 8.688699090909092, 8.5883, 8.5064,
            8.35929818181818, 8.146797272727273, 8.129297272727271, 8.111701818181817
        ])


def global_n2o_anthro_emissions_rcp_subrcp60(x):
    """
    global n2o anthro emissions rcp subrcp60



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            7.585454545454545, 7.4569090909090905, 7.684090909090909, 8.18539909090909,
            7.929600000000001, 8.8179, 9.72830090909091, 10.537099999999999, 11.361101818181817,
            12.031199090909091, 12.362502727272727, 12.359702727272728, 12.270000909090909
        ])


def global_n2o_anthro_emissions_rcp_subrcp85(x):
    """
    global n2o anthro emissions rcp subrcp85



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            7.585454545454545, 7.4569090909090905, 7.684090909090909, 8.152702727272727,
            9.555299090909092, 10.778899090909091, 12.028099999999998, 12.795898181818181,
            13.421602727272727, 13.94020090909091, 14.5586, 15.292702727272726, 15.77560090909091
        ])


def global_hfc_emissions_rcp_x8x5_subhfc134a(x):
    """
    global hfc emissions rcp x8x5 subhfc134a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            1010, 75039, 120933, 152513, 265080, 333876, 406963, 464903, 508058, 554713, 605051,
            653132, 703459
        ])


def global_hfc_emissions_rcp_x8x5_subhfc23(x):
    """
    global hfc emissions rcp x8x5 subhfc23



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [6622, 10395, 10812, 9522, 1152, 691, 413, 246, 138, 73, 37, 0, 0])


def global_hfc_emissions_rcp_x8x5_subhfc32(x):
    """
    global hfc emissions rcp x8x5 subhfc32



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [0, 4000, 10994, 17942, 60642, 72035, 80712, 74607, 84086, 93880, 96271, 91204, 81227])


def global_hfc_emissions_rcp_x8x5_subhfc125(x):
    """
    global hfc emissions rcp x8x5 subhfc125



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            186, 8538, 13759, 34598, 66696, 78956, 100250, 116135, 128080, 140714, 144726, 147547,
            148304
        ])


def global_hfc_emissions_rcp_x8x5_subhfc143a(x):
    """
    global hfc emissions rcp x8x5 subhfc143a



    lookup


    """
    return functions.lookup(x, [
        1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100
    ], [889, 6234, 12448, 26537, 35067, 38184, 45837, 50207, 52424, 54619, 52707, 50177, 46842])


def global_hfc_emissions_rcp_x8x5_subhfc152a(x):
    """
    global hfc emissions rcp x8x5 subhfc152a



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [12, 15200, 23010, 34530, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def global_hfc_emissions_rcp_x8x5_subhfc227ea(x):
    """
    global hfc emissions rcp x8x5 subhfc227ea



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [0, 1951, 4890, 11541, 1159, 606, 764, 320, 148, 129, 113, 102, 91])


def global_hfc_emissions_rcp_x8x5_subhfc245ca(x):
    """
    global hfc emissions rcp x8x5 subhfc245ca



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            0, 17926, 26888, 85605, 148223, 193765, 181171, 161710, 149109, 136491, 128236, 120245,
            131805
        ])


def global_hfc_emissions_rcp_x8x5_subhfc4310mee(x):
    """
    global hfc emissions rcp x8x5 subhfc4310mee



    lookup


    """
    return functions.lookup(
        x, [1990, 2000, 2005, 2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090,
            2100], [1.037, 105.5, 206.6, 305.2, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0])


def co2x_landxdashxuse_change_emissions_exogenous(x):
    """
    co2x landxdashxuse change emissions exogenous



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3, 3.3,
        3.3, 3.3, 3.3
    ])


def historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subblue_water(
        x):
    """
    historic water use by type sectors subagriculture hunting forestry and fishing subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        875813247.2645526, 904894037.0876915, 924992750.6496964, 915744665.9767051,
        944131164.2051525, 930727079.578415, 931938668.0864745, 932344746.4425533,
        947493154.282249, 996930676.2433419, 1021318186.6275508, 1046320348.2075572,
        1087846995.467034, 1083882800.3577504, 1089805524.1143212
    ])


def historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subgreen_water(
        x):
    """
    historic water use by type sectors subagriculture hunting forestry and fishing subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        5977864528.807236, 6326270468.179815, 6354728879.821534, 6327130943.334498,
        6425843571.417871, 6435228369.735994, 6691165889.684004, 6589938054.514836,
        6947913764.896528, 7281996193.570124, 7460347849.916722, 7555367969.278706,
        7762014616.151556, 8227010892.160533, 8086731839.894531
    ])


def historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subgray_water(
        x):
    """
    historic water use by type sectors subagriculture hunting forestry and fishing subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        651138685.02772, 691880700.1998262, 699269186.6198692, 708256791.9608455,
        717409269.4949044, 720460108.3121521, 722912474.1009471, 715695871.758256,
        742812021.0725459, 801433146.1337559, 811655677.0531788, 816310961.6172218,
        869449582.2730234, 909023540.0702205, 901373091.6981382
    ])


def historic_water_use_by_type_sectors_submining_and_quarrying_subblue_water(x):
    """
    historic water use by type sectors submining and quarrying subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_submining_and_quarrying_subgreen_water(x):
    """
    historic water use by type sectors submining and quarrying subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_submining_and_quarrying_subgray_water(x):
    """
    historic water use by type sectors submining and quarrying subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subblue_water(x):
    """
    historic water use by type sectors subfood beverages and tobacco subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        6043346.010340913, 6107439.559879856, 6231315.252456846, 6339487.778081499,
        6488077.177998222, 6674527.489274913, 6781761.790437164, 6776312.793257868,
        7035610.110967136, 7218477.619853792, 7606922.768967351, 8030604.742224609,
        8450094.227811394, 8536196.648654146, 8480932.359374203
    ])


def historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subgreen_water(x):
    """
    historic water use by type sectors subfood beverages and tobacco subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subgray_water(x):
    """
    historic water use by type sectors subfood beverages and tobacco subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        51655857.26464701, 52862894.43220483, 53336700.83082201, 54331507.947123885,
        56042281.21474628, 58336484.49223001, 59904004.36978328, 60705637.62379336,
        63747311.465051435, 66466864.99645599, 70909464.18281865, 76668931.17553678,
        81981026.17319132, 83634337.0023973, 83154340.2282583
    ])


def historic_water_use_by_type_sectors_subtextiles_and_textile_products_subblue_water(x):
    """
    historic water use by type sectors subtextiles and textile products subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2856810.350862667, 2854776.157266739, 2901687.0169803756, 2880559.77255534,
        2957620.829265346, 3073680.5060626315, 3013728.994760872, 2946964.85093716,
        2928631.5051592784, 2916380.1663147463, 2980441.0619652485, 3119208.5928101963,
        3184617.64981006, 3210798.2505277228, 2999361.432855909
    ])


def historic_water_use_by_type_sectors_subtextiles_and_textile_products_subgreen_water(x):
    """
    historic water use by type sectors subtextiles and textile products subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subtextiles_and_textile_products_subgray_water(x):
    """
    historic water use by type sectors subtextiles and textile products subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        25563592.7501675, 25711087.323870335, 25971948.364553403, 26021484.500666298,
        27190028.760726564, 29238393.955100752, 29542595.943647943, 29370724.629816152,
        29829798.94473654, 30298651.17885314, 31383177.898448173, 34160667.329453066,
        35965725.93976969, 37100882.90252638, 35400467.43615428
    ])


def historic_water_use_by_type_sectors_subleather_leather_and_footwear_subblue_water(x):
    """
    historic water use by type sectors subleather leather and footwear subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subleather_leather_and_footwear_subgreen_water(x):
    """
    historic water use by type sectors subleather leather and footwear subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subleather_leather_and_footwear_subgray_water(x):
    """
    historic water use by type sectors subleather leather and footwear subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subblue_water(x):
    """
    historic water use by type sectors subwood and products of woood and cork subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subgreen_water(x):
    """
    historic water use by type sectors subwood and products of woood and cork subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subgray_water(x):
    """
    historic water use by type sectors subwood and products of woood and cork subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subblue_water(x):
    """
    historic water use by type sectors subpulp paper printing and publishing subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        5511624.242226528, 5610128.6319932565, 5894966.26947698, 6000030.516985005,
        6222288.677616086, 6436607.354004963, 6460144.200315665, 6524269.319553448,
        6767396.801711854, 7286483.748585307, 7787246.9237831365, 8120940.267707023,
        8869715.657402454, 9148944.748788334, 8944824.384443942
    ])


def historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subgreen_water(x):
    """
    historic water use by type sectors subpulp paper printing and publishing subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subgray_water(x):
    """
    historic water use by type sectors subpulp paper printing and publishing subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        48897115.23912518, 50260682.37775563, 52816750.33501838, 54155133.34918979,
        55703573.025912896, 57688494.002147324, 59068727.81800082, 60206387.40918563,
        64309826.15065473, 71263540.40019777, 78230874.08060494, 83112055.0121726,
        93210711.4945201, 97617633.19816479, 97163113.10806447
    ])


def historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subblue_water(x):
    """
    historic water use by type sectors subcoke refined petroleum and nuclear fuel subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subgreen_water(
        x):
    """
    historic water use by type sectors subcoke refined petroleum and nuclear fuel subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subgray_water(x):
    """
    historic water use by type sectors subcoke refined petroleum and nuclear fuel subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subblue_water(x):
    """
    historic water use by type sectors subchemicals and chemical products subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        7734732.538293498, 8110356.463576764, 8617195.393249623, 8801239.349758193,
        9279683.973082485, 9609181.829644354, 9829776.68557161, 10258003.28702495,
        10981765.021278488, 11593938.030353935, 12137059.46891245, 13409295.443929115,
        15145125.92085762, 15327167.438895136, 15460495.429674448
    ])


def historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subgreen_water(x):
    """
    historic water use by type sectors subchemicals and chemical products subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subgray_water(x):
    """
    historic water use by type sectors subchemicals and chemical products subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        71068432.36246447, 75395851.84929058, 80334154.65319662, 82242025.84663436,
        87528854.33887208, 90074912.93474054, 92784808.07015377, 97084569.01069757,
        106104833.58081052, 113534645.4027672, 120748635.46432692, 135833134.2780234,
        155394042.57533154, 159381258.95667988, 166237032.513937
    ])


def historic_water_use_by_type_sectors_subrubber_and_plastics_subblue_water(x):
    """
    historic water use by type sectors subrubber and plastics subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subrubber_and_plastics_subgreen_water(x):
    """
    historic water use by type sectors subrubber and plastics subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subrubber_and_plastics_subgray_water(x):
    """
    historic water use by type sectors subrubber and plastics subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subother_non_metalic_mineral_subblue_water(x):
    """
    historic water use by type sectors subother non metalic mineral subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1654566.634922395, 1590162.5759721335, 1640599.8033098953, 1660898.6521802363,
        1743683.4098575562, 1838831.196324295, 1838622.3294739733, 1861957.6590920282,
        1947023.5698884313, 2083364.0880357167, 2217032.3652505176, 2405708.799250856,
        2590473.7161120963, 2569340.0914561283, 2249144.2706229854
    ])


def historic_water_use_by_type_sectors_subother_non_metalic_mineral_subgreen_water(x):
    """
    historic water use by type sectors subother non metalic mineral subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subother_non_metalic_mineral_subgray_water(x):
    """
    historic water use by type sectors subother non metalic mineral subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        17130547.550654273, 15345608.327496383, 15651683.18433788, 15790368.092607673,
        16672621.245390099, 17785698.253186226, 17969133.34450238, 18224313.985377964,
        19290777.950024813, 20928856.582539964, 22586528.338676754, 24950712.573254433,
        27761861.569189362, 27998207.41698554, 24748256.38490064
    ])


def historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subblue_water(x):
    """
    historic water use by type sectors subbasic metals and fabricated metal subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        7855774.10285661, 8350044.151949255, 8906036.686630167, 9087008.246610556,
        9229458.25560122, 9602324.748112699, 9653785.120265555, 9840745.03145729,
        10632836.936686257, 11443296.575238016, 12285425.072026506, 14112373.721374692,
        15594910.642490087, 16024172.185162164, 16902295.385492112
    ])


def historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subgreen_water(x):
    """
    historic water use by type sectors subbasic metals and fabricated metal subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subgray_water(x):
    """
    historic water use by type sectors subbasic metals and fabricated metal subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        78961833.92970943, 85226477.45238905, 91545664.33299291, 92503264.48035632,
        94198400.7286549, 98610019.14240205, 101395939.04939361, 104347100.2117023,
        115246353.53713316, 124513779.27993722, 134812696.04307225, 158261330.94802743,
        176740416.86429477, 182974315.60517973, 199831153.0006033
    ])


def historic_water_use_by_type_sectors_submachinery_nec_subblue_water(x):
    """
    historic water use by type sectors submachinery nec subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_submachinery_nec_subgreen_water(x):
    """
    historic water use by type sectors submachinery nec subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_submachinery_nec_subgray_water(x):
    """
    historic water use by type sectors submachinery nec subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subblue_water(x):
    """
    historic water use by type sectors subelectrical and optical equipment subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subgreen_water(x):
    """
    historic water use by type sectors subelectrical and optical equipment subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subgray_water(x):
    """
    historic water use by type sectors subelectrical and optical equipment subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subtransport_equipment_subblue_water(x):
    """
    historic water use by type sectors subtransport equipment subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subtransport_equipment_subgreen_water(x):
    """
    historic water use by type sectors subtransport equipment subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subtransport_equipment_subgray_water(x):
    """
    historic water use by type sectors subtransport equipment subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subblue_water(x):
    """
    historic water use by type sectors submanufacturing nec recycling subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subgreen_water(x):
    """
    historic water use by type sectors submanufacturing nec recycling subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subgray_water(x):
    """
    historic water use by type sectors submanufacturing nec recycling subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subblue_water(x):
    """
    historic water use by type sectors subelectricity gas and water supply subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        607162225.3108122, 615597597.9583583, 623203677.641867, 625679543.8123907,
        627354545.2912184, 641460276.1566416, 626668036.3627105, 643153598.0795102,
        646502573.9967462, 688040368.768368, 717875854.2415878, 743508087.9977753,
        753149076.7904673, 783692162.3416656, 796153484.495191
    ])


def historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subgreen_water(x):
    """
    historic water use by type sectors subelectricity gas and water supply subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subgray_water(x):
    """
    historic water use by type sectors subelectricity gas and water supply subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        710.306571369756, 754.2628326728772, 742.0653879267878, 749.1084056240832,
        892.0169591297997, 1023.429239248635, 1112.8693605831934, 953.3207382598795,
        987.97183427495, 1005.9798005761447, 1106.5754122679002, 1127.603871083194,
        1203.2707308215072, 1308.7833790691557, 1188.776385567763
    ])


def historic_water_use_by_type_sectors_subconstruction_subblue_water(x):
    """
    historic water use by type sectors subconstruction subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subconstruction_subgreen_water(x):
    """
    historic water use by type sectors subconstruction subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subconstruction_subgray_water(x):
    """
    historic water use by type sectors subconstruction subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subblue_water(
        x):
    """
    historic water use by type sectors subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subgreen_water(
        x):
    """
    historic water use by type sectors subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subgray_water(
        x):
    """
    historic water use by type sectors subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subblue_water(
        x):
    """
    historic water use by type sectors subwholesale trade and commissions trade except of motor vehicles and motorcycles subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subgreen_water(
        x):
    """
    historic water use by type sectors subwholesale trade and commissions trade except of motor vehicles and motorcycles subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subgray_water(
        x):
    """
    historic water use by type sectors subwholesale trade and commissions trade except of motor vehicles and motorcycles subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subblue_water(
        x):
    """
    historic water use by type sectors subretail trade except of motor vehicles and motorcycles repair of household goods subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subgreen_water(
        x):
    """
    historic water use by type sectors subretail trade except of motor vehicles and motorcycles repair of household goods subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subgray_water(
        x):
    """
    historic water use by type sectors subretail trade except of motor vehicles and motorcycles repair of household goods subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subhotels_and_restaurants_subblue_water(x):
    """
    historic water use by type sectors subhotels and restaurants subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subhotels_and_restaurants_subgreen_water(x):
    """
    historic water use by type sectors subhotels and restaurants subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subhotels_and_restaurants_subgray_water(x):
    """
    historic water use by type sectors subhotels and restaurants subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subinland_transport_subblue_water(x):
    """
    historic water use by type sectors subinland transport subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subinland_transport_subgreen_water(x):
    """
    historic water use by type sectors subinland transport subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subinland_transport_subgray_water(x):
    """
    historic water use by type sectors subinland transport subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwater_transport_subblue_water(x):
    """
    historic water use by type sectors subwater transport subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwater_transport_subgreen_water(x):
    """
    historic water use by type sectors subwater transport subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subwater_transport_subgray_water(x):
    """
    historic water use by type sectors subwater transport subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subair_transport_subblue_water(x):
    """
    historic water use by type sectors subair transport subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subair_transport_subgreen_water(x):
    """
    historic water use by type sectors subair transport subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subair_transport_subgray_water(x):
    """
    historic water use by type sectors subair transport subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subblue_water(
        x):
    """
    historic water use by type sectors subother supporting and auxiliary transport activities activities of travel agencies subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subgreen_water(
        x):
    """
    historic water use by type sectors subother supporting and auxiliary transport activities activities of travel agencies subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subgray_water(
        x):
    """
    historic water use by type sectors subother supporting and auxiliary transport activities activities of travel agencies subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpost_and_telecommunications_subblue_water(x):
    """
    historic water use by type sectors subpost and telecommunications subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpost_and_telecommunications_subgreen_water(x):
    """
    historic water use by type sectors subpost and telecommunications subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpost_and_telecommunications_subgray_water(x):
    """
    historic water use by type sectors subpost and telecommunications subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subfinancial_intermedation_subblue_water(x):
    """
    historic water use by type sectors subfinancial intermedation subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subfinancial_intermedation_subgreen_water(x):
    """
    historic water use by type sectors subfinancial intermedation subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subfinancial_intermedation_subgray_water(x):
    """
    historic water use by type sectors subfinancial intermedation subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subreal_estate_activities_subblue_water(x):
    """
    historic water use by type sectors subreal estate activities subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subreal_estate_activities_subgreen_water(x):
    """
    historic water use by type sectors subreal estate activities subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subreal_estate_activities_subgray_water(x):
    """
    historic water use by type sectors subreal estate activities subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subblue_water(
        x):
    """
    historic water use by type sectors subrenting od meq and other business activities subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subgreen_water(
        x):
    """
    historic water use by type sectors subrenting od meq and other business activities subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subgray_water(
        x):
    """
    historic water use by type sectors subrenting od meq and other business activities subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subblue_water(
        x):
    """
    historic water use by type sectors subpublic admin and defence compulsory social security subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subgreen_water(
        x):
    """
    historic water use by type sectors subpublic admin and defence compulsory social security subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subgray_water(
        x):
    """
    historic water use by type sectors subpublic admin and defence compulsory social security subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subeducation_subblue_water(x):
    """
    historic water use by type sectors subeducation subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        76.6143535870801, 79.34623374471475, 82.74315426919267, 86.60018236958769,
        88.81640750628509, 91.5554932218082, 94.33025511832513, 94.70105391409032,
        94.3337661223288, 95.21263900327992, 97.5347049156145, 99.25377697935946,
        101.70574369564017, 103.40059652299276, 102.36765147374021
    ])


def historic_water_use_by_type_sectors_subeducation_subgreen_water(x):
    """
    historic water use by type sectors subeducation subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subeducation_subgray_water(x):
    """
    historic water use by type sectors subeducation subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        496.9685861399711, 515.9863152658087, 542.7051409128488, 567.8331594647725,
        580.9641029914214, 599.3933841293607, 618.3976816357925, 614.250476965647,
        610.3358923461076, 618.1373130639456, 635.5348895403408, 650.8709307587583,
        669.8483452440975, 685.5124161992544, 682.9142555743476
    ])


def historic_water_use_by_type_sectors_subhealth_and_social_work_subblue_water(x):
    """
    historic water use by type sectors subhealth and social work subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        76.30793913643964, 79.33184648263418, 82.11238440547405, 84.46557983761329,
        85.424442590247, 87.71292129723375, 90.96604819244752, 93.66510642204236, 96.9050496971886,
        100.92182736591627, 105.6690725067325, 109.8840478410611, 114.28047474812212,
        118.80052338649232, 120.75875434878486
    ])


def historic_water_use_by_type_sectors_subhealth_and_social_work_subgreen_water(x):
    """
    historic water use by type sectors subhealth and social work subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subhealth_and_social_work_subgray_water(x):
    """
    historic water use by type sectors subhealth and social work subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        498.00510336560285, 523.0619302940098, 545.9183135386813, 561.0344368859685,
        561.7726508912538, 574.7043789472302, 592.1948291578987, 600.1201990965885,
        619.6680710031076, 651.2485894719183, 685.2735760904864, 719.7429175278176,
        754.7382987308483, 788.5680762455889, 800.4534980971242
    ])


def historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subblue_water(
        x):
    """
    historic water use by type sectors subother community social and persona services subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subgreen_water(
        x):
    """
    historic water use by type sectors subother community social and persona services subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subgray_water(
        x):
    """
    historic water use by type sectors subother community social and persona services subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subblue_water(
        x):
    """
    historic water use by type sectors subprivate households with employed persons subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subgreen_water(
        x):
    """
    historic water use by type sectors subprivate households with employed persons subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subgray_water(
        x):
    """
    historic water use by type sectors subprivate households with employed persons subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_for_households_subblue_water(x):
    """
    historic water use by type for households subblue water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        39538356.485179946, 40003162.75187955, 40466184.14733441, 40924633.120817594,
        41376268.09075577, 41823191.05993492, 42260849.06568138, 42686651.51102832,
        43121548.65589165, 43560417.95149626, 44006108.64518016, 44454875.508882836,
        44909551.291070096, 45369429.80659038, 45822308.62191979
    ])


def historic_water_use_by_type_for_households_subgreen_water(x):
    """
    historic water use by type for households subgreen water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_water_use_by_type_for_households_subgray_water(x):
    """
    historic water use by type for households subgray water



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        264130629.8114975, 267488985.0987483, 270826457.46761274, 274129799.01614493,
        277383979.13562506, 280593543.68102324, 283734749.1645494, 286787722.45889896,
        289890646.75197935, 293020768.3420327, 296195891.04128563, 299401950.43797207,
        302651145.94000876, 305928199.5909348, 309178818.896767
    ])


def gdppc_annual_growth_ssp2(x):
    """
    gdppc annual growth ssp2



    lookup


    """
    return functions.lookup(x, [
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        0.03857969744680689, 0.03857969744680689, 0.03857969744680689, 0.03857969744680689,
        0.03857969744680689, 0.03857969744680689, 0.035236454832179964, 0.035236454832179964,
        0.035236454832179964, 0.035236454832179964, 0.035236454832179964, 0.035236454832179964,
        0.035236454832179964, 0.035236454832179964, 0.035236454832179964, 0.035236454832179964,
        0.026075635629570826, 0.026075635629570826, 0.026075635629570826, 0.026075635629570826,
        0.026075635629570826, 0.026075635629570826, 0.026075635629570826, 0.026075635629570826,
        0.026075635629570826, 0.026075635629570826, 0.022743789939873027, 0.022743789939873027,
        0.022743789939873027, 0.022743789939873027, 0.022743789939873027, 0.022743789939873027,
        0.022743789939873027, 0.022743789939873027, 0.022743789939873027, 0.022743789939873027,
        0.021843416743548616, 0.021843416743548616, 0.021843416743548616, 0.021843416743548616,
        0.021843416743548616, 0.021843416743548616, 0.021843416743548616, 0.021843416743548616,
        0.021843416743548616, 0.021843416743548616, 0.022653169341066315, 0.022653169341066315,
        0.022653169341066315, 0.022653169341066315, 0.022653169341066315, 0.022653169341066315,
        0.022653169341066315, 0.022653169341066315, 0.022653169341066315, 0.022653169341066315,
        0.022024412912428737, 0.022024412912428737, 0.022024412912428737, 0.022024412912428737,
        0.022024412912428737, 0.022024412912428737, 0.022024412912428737, 0.022024412912428737,
        0.022024412912428737, 0.022024412912428737, 0.021875392173132502, 0.021875392173132502,
        0.021875392173132502, 0.021875392173132502, 0.021875392173132502, 0.021875392173132502,
        0.021875392173132502, 0.021875392173132502, 0.021875392173132502, 0.021875392173132502,
        0.021284366520702536, 0.021284366520702536, 0.021284366520702536, 0.021284366520702536,
        0.021284366520702536, 0.021284366520702536, 0.021284366520702536, 0.021284366520702536,
        0.021284366520702536, 0.021284366520702536
    ])


def pop_ssp2(x):
    """
    pop ssp2



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6867.39, 7611.25, 8261.99, 8787.12, 9169.11, 9384.7, 9456.88,
                                9407.26, 9253.95, 9032.42, 9032.42
                            ])


def pop_ssp4(x):
    """
    pop ssp4



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6895.882, 7626.353, 8258.565, 8765.68, 9146.527, 9377.45, 9472.529,
                                9471.44, 9401.586, 9292.446, 9292.446
                            ])


def pop_ssp1(x):
    """
    pop ssp1



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6921.798, 7576.105, 8061.938, 8388.763, 8530.5, 8492.176, 8298.95,
                                7967.387, 7510.454, 6957.989, 6957.989
                            ])


def pop_ssp5(x):
    """
    pop ssp5



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [6894, 7552, 8054, 8403, 8579, 8589, 8457, 8200, 7831, 7375, 7375])


def pop_ssp3(x):
    """
    pop ssp3



    lookup


    """
    return functions.lookup(x, [2010, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100, 2110],
                            [
                                6879.59, 7697.854, 8514.307, 9257.22, 9957.131, 10574.362,
                                11117.376, 11633.415, 12134.327, 12620.136, 12620.136
                            ])


def p_timeseries_gdppc_growth_rate(x):
    """
    p timeseries gdppc growth rate



    lookup


    """
    return functions.lookup(x, [
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019,
        0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019, 0.019
    ])


def historic_gdppc(x):
    """
    historic gdppc



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        5106.7947854704435, 5174.896039944818, 5262.160141079583, 5328.423151357811,
        5486.243275343644, 5614.624756138972, 5622.1612721180445, 5617.043532323626,
        5717.34533140038, 5872.719107872231, 6001.98989859595, 6160.0297122424945,
        6314.980459059734, 6352.441542803503, 6110.378125282639, 6298.007948978156,
        6420.647080849909, 6498.888786922519, 6588.576150940048, 6694.810605835951,
        6797.225563489298, 6881.608808351995
    ])


def total_ghg_emissions_olt_medeas_d3x2(x):
    """
    total ghg emissions olt medeas d3x2



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 47.42876601959408, 47.09505948151982,
        46.76135294344556, 46.4276464053713, 46.09393986729704, 45.76023332922278,
        45.42652679114852, 45.09282025307426, 44.759113715, 44.05724267735, 43.355371639699996,
        42.65350060205, 41.9516295644, 41.24975852675, 40.5478874891, 39.84601645145,
        39.144145413800004, 38.44227437615, 37.7404033385, 36.99498139958053, 36.24955946066106,
        35.50413752174159, 34.75871558282212, 34.01329364390265, 33.26787170498318,
        32.52244976606371, 31.77702782714424, 31.03160588822477, 30.2861839493053,
        29.55797931044127, 28.82977467157724, 28.101570032713212, 27.37336539384918,
        26.645160754985152, 25.91695611612112, 25.188751477257092, 24.46054683839306,
        23.73234219952903, 23.004137560665, 22.37502643707183, 21.745915313478662,
        21.11680418988549, 20.48769306629232, 19.85858194269915, 19.22947081910598,
        18.600359695512807, 17.971248571919638, 17.34213744832647, 16.7130263247333,
        16.21015507735497, 15.707283829976639, 15.204412582598309, 14.701541335219979,
        14.198670087841649, 13.695798840463318, 13.19292759308499, 12.69005634570666,
        12.18718509832833, 11.68431385095, 11.265439517733332, 10.846565184516665, 10.4276908513,
        10.008816518083332, 9.589942184866665, 9.171067851649997, 8.75219351843333,
        8.333319185216663, 7.914444851999997, 7.49557051878333, 7.215631669871664,
        6.935692820959998, 6.655753972048332, 6.375815123136666, 6.095876274225001,
        5.815937425313334, 5.535998576401668, 5.256059727490002, 4.976120878578336,
        4.69618202966667, 4.523135116700003, 4.350088203733336, 4.177041290766669,
        4.0039943778000024, 3.8309474648333355, 3.657900551866668, 3.484853638900001,
        3.311806725933334, 3.0, 3.0
    ])


def total_ghg_emissions_bauxdashxcat_medeas_d3x2(x):
    """
    total ghg emissions bauxdashxcat medeas d3x2



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 52.46089897560433, 54.21177696260433,
        55.96265494960433, 57.71353293660433, 59.464410923604326, 61.21528891060433,
        62.96616689760433, 64.71704488460433, 66.46792287160433, 68.00663966160434,
        69.54535645160432, 71.08407324160433, 72.62279003160432, 74.16150682160433,
        75.70022361160433, 77.23894040160432, 78.77765719160433, 80.31637398160433,
        81.85509077160432, 83.05051741060433, 84.24594404960432, 85.44137068860432,
        86.63679732760431, 87.83222396660433, 89.02765060560434, 90.22307724460433,
        91.41850388360433, 92.61393052260432, 93.80935716160432, 95.46841207750431,
        97.12746699340433, 98.78652190930431, 100.44557682520431, 102.10463174110431,
        103.7636866570043, 105.42274157290431, 107.0817964888043, 108.74085140470432,
        110.3999063206043, 111.83580096570431, 113.27169561080431, 114.70759025590432,
        116.14348490100431, 117.57937954610432, 119.01527419120433, 120.45116883630432,
        121.88706348140433, 123.32295812650432, 124.75885277160434, 126.37919332160433,
        127.99953387160433, 129.61987442160432, 131.24021497160436, 132.86055552160434,
        134.48089607160435, 136.10123662160433, 137.72157717160434, 139.34191772160435,
        140.96225827160433, 141.74471066160433, 142.52716305160436, 143.30961544160434,
        144.09206783160434, 144.8745202216043, 145.65697261160435, 146.43942500160435,
        147.22187739160432, 148.00432978160435, 148.78678217160433, 150.03883602160434,
        151.29088987160432, 152.54294372160433, 153.7949975716043, 155.04705142160435,
        156.29910527160436, 157.55115912160434, 158.80321297160435, 160.05526682160433,
        161.30732067160434, 162.69081727160432, 164.07431387160435, 165.45781047160435,
        166.84130707160432, 168.22480367160432, 169.60830027160432, 170.99179687160435,
        172.37529347160432, 173.7587900716043, 175.14228667160432
    ])


def total_ghg_emissions_mlt2030x_medeas_d3x2(x):
    """
    total ghg emissions mlt2030x medeas d3x2



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 52.460899, 54.211777, 55.9626549, 57.7135329,
        59.4644109, 61.2152889, 62.9661669, 64.7170449, 66.4679229, 68.0066397, 69.5453565,
        71.0840732, 72.62279, 74.1615068, 75.7002236, 77.2389404, 78.7776572, 80.316374,
        81.8550908, 72.851030812, 64.83741742268, 57.70530150618521, 51.35771834050483,
        45.7083693230493, 40.680448697513874, 36.205599340787344, 32.22298341330074,
        28.678455237837657, 25.523825161675514, 22.716204393891207, 20.217421910563175,
        17.993505500401227, 16.01421989535709, 14.252655706867811, 12.684863579112353,
        11.289528585409993, 10.047680441014894, 8.942435592503255, 7.958767677327897,
        7.083303232821828, 6.304139877211427, 5.61068449071817, 4.993509196739171,
        4.444223185097862, 3.9553586347370975, 3.5202691849160166, 3.1330395745752546,
        2.7884052213719768, 2.4816806470210593, 2.2086957758487427, 1.965739240505381,
        1.749507924049789, 1.5570620524043122, 1.3857852266398378, 1.2333488517094557,
        1.0976804780214156, 0.9769356254390599, 0.8694727066407633, 0.7738307089102793,
        0.6887093309301485, 0.6129513045278322, 0.5455266610297707, 0.4855187283164959,
        0.43211166820168134, 0.3845793846994964, 0.34227565238255175, 0.30462533062047104,
        0.2711165442522192, 0.2412937243844751, 0.21475141470218284, 0.19112875908494273,
        0.17010459558559904, 0.15139309007118315, 0.134739850163353, 0.11991846664538416,
        0.10672743531439191, 0.0949874174298088, 0.08453880151252983, 0.07523953334615155,
        0.06696318467807488, 0.05959723436348664, 0.05304153858350311, 0.04720696933931777,
        0.04201420271199281, 0.0373926404136736, 0.033279449968169506, 0.02961871047167086,
        0.026360652319787067, 0.02346098056461049
    ])


def total_ghg_emissions_mlt2020x_medeas_d3x2(x):
    """
    total ghg emissions mlt2020x medeas d3x2



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        35.759413148286896, 35.9654102916019, 35.24924015949415, 35.39397812433697,
        35.518166161269896, 36.24446809126249, 36.95717178883806, 37.17157881794709,
        37.401771576560655, 37.62916715402579, 38.88923353257618, 39.1672436132021,
        39.70029697776367, 40.962653787287586, 42.43164699720583, 43.4585485141285,
        44.73882999735216, 45.98817373194996, 46.41034844291112, 45.85923565238791,
        47.554580971711395, 48.42974943441521, 52.46089897560433, 54.21177696260433,
        55.96265494960433, 57.71353293660433, 59.464410923604326, 61.21528891060433,
        62.96616689760433, 64.71704488460433, 66.46792287160433, 63.54333426525374,
        60.747427557582576, 58.07454074504894, 55.51926095226679, 53.07641347036705,
        50.7410512776709, 48.508445021453376, 46.37407344050943, 44.33361420912701,
        42.382935183925426, 40.51808603583271, 38.73529025025607, 37.030937479244805,
        35.401576230158035, 33.843906876031085, 32.35477497348572, 30.931164874652346,
        29.570193620167643, 28.269105100880267, 27.025264476441535, 25.836152839478107,
        24.69936211454107, 23.612590181501265, 22.573636213515208, 21.580396220120537,
        20.630858786435233, 19.723100999832084, 18.85528455583947, 18.025652035382535,
        17.232523345825705, 16.474292318609372, 15.74942345659056, 15.056448824500574,
        14.39396507622255, 13.760630612868757, 13.155162865902533, 12.57633569980282,
        12.022976929011497, 11.493965944134992, 10.988231442593053, 10.504749259118958,
        10.042540291717724, 9.600668518882145, 9.17823910405133, 8.774396583473072,
        8.388323133800256, 8.019236915913044, 7.66639049161287, 7.329069309981904, 7.0065902603427,
        6.698300288887622, 6.403575076176566, 6.121817772824797, 5.852457790820506,
        5.594949648024404, 5.34877186351133, 5.113425901516831, 4.888435161850091,
        4.673344014728687, 4.4677168780806245, 4.271137335445077, 4.083207292685493,
        3.903546171807332, 3.731790140247809, 3.5675913740769056, 3.4106173536175217,
        3.2605501900583507, 3.117085981695783, 2.979934198501169, 2.8488170937671176,
        2.7234691416413646, 2.6036364994091445, 2.4890764934351424, 2.379557127723996,
        2.27485661410414, 2.174762923083558, 2.0790733544678814, 1.9875941268712947,
        1.9001399852889578, 1.8165338259362436
    ])


def table_hist_capacity_phs(x):
    """
    table hist capacity phs



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.0916300043623243, 0.0933800056375377, 0.0915700044762995, 0.095060004532393,
        0.0963800032561458, 0.0979700041995104, 0.099380003724601, 0.0988500056353863,
        0.0984100053188484, 0.102080005003214, 0.103850004442362, 0.10771000355781,
        0.112910003772387, 0.117070004867921, 0.12167000599687, 0.123550007094676,
        0.130110007937634, 0.132230006731795, 0.1332792609425959, 0.13455522053152233,
        0.1379890354996751
    ])


def table_max_extraction_aspo_oil_ej_x0x_x1x_x0x(x):
    """
    table max extraction aspo oil ej x0x x1x x0x



    lookup


    """
    return functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0.0, 29.9783, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])


def table_max_extraction_aspo_oil_ej_x0x_x1x(x):
    """
    table max extraction aspo oil ej x0x x1x



    lookup


    """
    return functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0.0, 29.9783, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])


def table_max_extraction_aspo_oil_ej_x0x(x):
    """
    table max extraction aspo oil ej x0x



    lookup


    """
    return functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0.0, 29.9783, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])


def table_max_extraction_aspo_oil_ej_x0x_x0x(x):
    """
    table max extraction aspo oil ej x0x x0x



    lookup


    """
    return functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0.0, 29.9783, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])


def historical_extraction_minerals_rest_subadhesive(x):
    """
    historical extraction minerals rest subadhesive



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subaluminium(x):
    """
    historical extraction minerals rest subaluminium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        19100000, 19393200, 20694690, 21392970, 22089700, 23084960, 23982470, 24378590, 25872770,
        27662610, 29751750, 31838150, 33607000, 37851990, 38789070, 36941740, 40260150, 43814260,
        45233790, 46864630, 49355110, 55928650, 55520790
    ])


def historical_extraction_minerals_rest_subaluminium_mirrors(x):
    """
    historical extraction minerals rest subaluminium mirrors



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subcadmium(x):
    """
    historical extraction minerals rest subcadmium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        18100, 18500, 18900, 20000, 19600, 19100, 19700, 18200, 15800, 16890, 18790, 19390, 19280,
        20370, 19550, 18690, 20930, 22010, 22780, 21770, 22020, 22680, 22320
    ])


def historical_extraction_minerals_rest_subcarbon_fiber(x):
    """
    historical extraction minerals rest subcarbon fiber



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subcement(x):
    """
    historical extraction minerals rest subcement



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subchromium(x):
    """
    historical extraction minerals rest subchromium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        9570000, 11999960, 12189960, 12499930, 12699880, 13499820, 14399830, 12099800, 13499670,
        15499420, 17499230, 19298990, 19598220, 21496500, 23794450, 19289490, 23683310, 23281570,
        25578710, 28777200, 26364160, 30350300, 30335910
    ])


def historical_extraction_minerals_rest_subcopper(x):
    """
    historical extraction minerals rest subcopper



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        9430000, 9992170, 10994390, 11392880, 12190320, 12586080, 13182150, 13677330, 13573660,
        13569730, 14561920, 14949450, 15032000, 15309050, 15285180, 15750700, 15707320, 15891470,
        16672810, 18037090, 18126070, 18615350, 18700270
    ])


def historical_extraction_minerals_rest_subdiesel(x):
    """
    historical extraction minerals rest subdiesel



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subdy(x):
    """
    historical extraction minerals rest subdy



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subelectricxdivxelectronic_components(x):
    """
    historical extraction minerals rest subelectricxdivxelectronic components



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subevacuation_lines(x):
    """
    historical extraction minerals rest subevacuation lines



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subfiberglass(x):
    """
    historical extraction minerals rest subfiberglass



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subfoam_glass(x):
    """
    historical extraction minerals rest subfoam glass



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subgalium(x):
    """
    historical extraction minerals rest subgalium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        35, 35, 63, 54, 60, 75, 100, 75, 64, 69, 60, 69, 80, 80, 111, 69, 172, 282, 373, 340, 415,
        440, 345
    ])


def historical_extraction_minerals_rest_subglass(x):
    """
    historical extraction minerals rest subglass



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subglass_reinforcing_plastic(x):
    """
    historical extraction minerals rest subglass reinforcing plastic



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subgravel(x):
    """
    historical extraction minerals rest subgravel



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subindium(x):
    """
    historical extraction minerals rest subindium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        145, 239, 200, 230, 230, 215, 335, 345, 335, 370, 395, 490, 570, 533, 530, 466, 479, 522,
        622, 629, 564, 369, 155
    ])


def historical_extraction_minerals_rest_subiron(x):
    """
    historical extraction minerals rest subiron



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        1000000000, 999928230, 1019944840, 1039927180, 1019893920, 993844440, 1059821110,
        1059791350, 1079735210, 1159623830, 1339511300, 1539358150, 1799023770, 1998410780,
        2217726110, 2236160950, 2584196110, 2933714900, 2922833710, 3102160850, 3408008320,
        3663619920, 3568372660
    ])


def historical_extraction_minerals_rest_subkno3_mined(x):
    """
    historical extraction minerals rest subkno3 mined



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subasphalt(x):
    """
    historical extraction minerals rest subasphalt



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_sublime(x):
    """
    historical extraction minerals rest sublime



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_sublimestone(x):
    """
    historical extraction minerals rest sublimestone



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_sublithium(x):
    """
    historical extraction minerals rest sublithium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        6100, 6300, 11000, 14000, 15000, 14000, 14000, 15100, 14200, 15100, 20200, 20600, 23500,
        25800, 25400, 18800, 28100, 34100, 35000, 34000, 31700, 31500, 35000
    ])


def historical_extraction_minerals_rest_sublubricant(x):
    """
    historical extraction minerals rest sublubricant



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_submagnesium(x):
    """
    historical extraction minerals rest submagnesium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        2460000, 2640000, 2639990, 3029990, 3089990, 3089980, 3089980, 3169980, 3319970, 3459940,
        4269920, 4209830, 4059670, 4389210, 5428700, 5507890, 5756730, 5926210, 6345590, 6905580,
        8413980, 27692050, 27689810
    ])


def historical_extraction_minerals_rest_submanganese(x):
    """
    historical extraction minerals rest submanganese



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        7190000, 7579970, 7729960, 7679940, 7039890, 6989840, 7279850, 7599810, 8099700, 8199470,
        9349300, 10499090, 11898380, 12596810, 13294950, 10790450, 13884820, 15983240, 15780650,
        16879270, 17767420, 17454820, 15941730
    ])


def historical_extraction_minerals_rest_subheavy_equipment(x):
    """
    historical extraction minerals rest subheavy equipment



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subconcrete(x):
    """
    historical extraction minerals rest subconcrete



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_submolybdenum(x):
    """
    historical extraction minerals rest submolybdenum



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        104000, 126000, 128000, 139990, 134990, 121980, 128980, 129980, 122970, 124950, 140930,
        184910, 183840, 204680, 217500, 220040, 240480, 262320, 257060, 255930, 277740, 230480,
        221170
    ])


def historical_extraction_minerals_rest_subnano3_mined(x):
    """
    historical extraction minerals rest subnano3 mined



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subnano3_synthetic(x):
    """
    historical extraction minerals rest subnano3 synthetic



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subneodymium(x):
    """
    historical extraction minerals rest subneodymium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subnickel(x):
    """
    historical extraction minerals rest subnickel



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        906000, 1039680, 1079770, 1119700, 1139590, 1119400, 1249270, 1329150, 1338990, 1398710,
        1398340, 1487810, 1576940, 1655660, 1564350, 1391710, 1578370, 1927720, 2206300, 2614500,
        2427300, 2249800, 2208450
    ])


def historical_extraction_minerals_rest_subover_grid_x15xpercxx(x):
    """
    historical extraction minerals rest subover grid x15xpercxx



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subover_grid_x5xpercxx(x):
    """
    historical extraction minerals rest subover grid x5xpercxx



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subpaint(x):
    """
    historical extraction minerals rest subpaint



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_sublead(x):
    """
    historical extraction minerals rest sublead



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        2800000, 2710000, 2920000, 3010000, 3100000, 3019990, 3099990, 3099990, 2909990, 2949980,
        3149970, 3269960, 3469940, 3769880, 3839810, 3859630, 4139410, 4699350, 5169250, 5489190,
        4868700, 4948180, 4817650
    ])


def historical_extraction_minerals_rest_subplastics(x):
    """
    historical extraction minerals rest subplastics



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subpolypropylene(x):
    """
    historical extraction minerals rest subpolypropylene



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subrock(x):
    """
    historical extraction minerals rest subrock



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subrock_wool(x):
    """
    historical extraction minerals rest subrock wool



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subsand(x):
    """
    historical extraction minerals rest subsand



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subsilicon_sand(x):
    """
    historical extraction minerals rest subsilicon sand



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subsilicon_wafer_modules(x):
    """
    historical extraction minerals rest subsilicon wafer modules



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subsilver(x):
    """
    historical extraction minerals rest subsilver



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        13900, 14600, 15200, 16390, 16390, 17680, 17690, 18680, 19970, 18750, 19630, 19220, 20060,
        20530, 20870, 20970, 21780, 21860, 23830, 24190, 23910, 21060, 21790
    ])


def historical_extraction_minerals_rest_subsite_preparation(x):
    """
    historical extraction minerals rest subsite preparation



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subtin(x):
    """
    historical extraction minerals rest subtin



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        184000, 188970, 195960, 210950, 205900, 197850, 237860, 221830, 248720, 206510, 263350,
        289200, 300610, 317350, 294830, 251880, 252040, 229800, 223630, 276260, 257510, 249220,
        228660
    ])


def historical_extraction_minerals_rest_subsoda_ash(x):
    """
    historical extraction minerals rest subsoda ash



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_substeel(x):
    """
    historical extraction minerals rest substeel



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subsynthetic_oil(x):
    """
    historical extraction minerals rest subsynthetic oil



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subtellurium(x):
    """
    historical extraction minerals rest subtellurium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subtitanium(x):
    """
    historical extraction minerals rest subtitanium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        3600000, 3814000, 4015000, 4055000, 4260000, 4500000, 4700000, 5000000, 4800000, 5299990,
        4999990, 5199990, 5799980, 6289960, 6389940, 5799880, 6399810, 6699790, 7229760, 7399740,
        6039590, 6939440, 6599270
    ])


def historical_extraction_minerals_rest_subtitanium_dioxide(x):
    """
    historical extraction minerals rest subtitanium dioxide



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subvanadium(x):
    """
    historical extraction minerals rest subvanadium



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        33900, 33700, 35100, 37100, 41800, 42800, 43000, 58000, 60000, 41000, 40200, 58200, 55700,
        58500, 55500, 53490, 57590, 62380, 73980, 78980, 82670, 77760, 75940
    ])


def historical_extraction_minerals_rest_subwires(x):
    """
    historical extraction minerals rest subwires



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historical_extraction_minerals_rest_subzinc(x):
    """
    historical extraction minerals rest subzinc



    lookup


    """
    return functions.lookup(x, [
        1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008,
        2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        6810000, 7119990, 7439990, 7459980, 7549970, 8039950, 8729950, 8849940, 8359900, 9009830,
        9599770, 9799700, 9999480, 10898960, 11598360, 11196900, 11995070, 12794550, 13493710,
        13393260, 13289410, 12785320, 11881060
    ])


def start_production_biofuels(x):
    """
    start production biofuels



    lookup


    """
    return functions.lookup(x, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14], [
        846.5825944871467, 1810.3039684919459, 2851.45794782365, 1759.0104688962037,
        3204.9937558820748, 6015.641852769073, 9035.178358448335, 11752.602921107493,
        5450.94157715125, 7699.6114069983305, 1558.1122306438265, 1115.666955235938,
        4982.002823415249, 6947.042164021972, 0.0
    ])


def historic_res_capacity_for_heatxdashxnc_subsolarxdashxheat(x):
    """
    historic res capacity for heatxdashxnc subsolarxdashxheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.03252055114522492, 0.037030040661937866, 0.04216484232697614, 0.04801166557416679,
        0.05466924347374749, 0.06225, 0.0685, 0.0756, 0.0849, 0.09559999999999999,
        0.10940000000000001, 0.12459999999999999, 0.1454, 0.1707, 0.2036, 0.24230000000000002,
        0.2863, 0.33080000000000004, 0.3757, 0.41
    ])


def historic_res_capacity_for_heatxdashxnc_subgeotxdashxheat(x):
    """
    historic res capacity for heatxdashxnc subgeotxdashxheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.0171805111069092, 0.01850341046214121, 0.01992817306772608, 0.021462642393940985,
        0.02311526585827444, 0.024895141329361572, 0.026812067211722412, 0.028876596387025034,
        0.031100094308825963, 0.03349480157060556, 0.036073901291542185, 0.03885159169099093,
        0.04184316425119723, 0.04506508789853942, 0.04853509966672695, 0.05227230234106492,
        0.05629726962132692, 0.060632159382169086, 0.0653008356545961, 0.07032899999999999
    ])


def historic_res_capacity_for_heatxdashxnc_subsolid_bioexdashxheat(x):
    """
    historic res capacity for heatxdashxnc subsolid bioexdashxheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.3731539881219632, 0.3882404414710075, 0.40393683356356724, 0.42026782395760703,
        0.43725906918628854, 0.4549372630652439, 0.4733301786294544, 0.4924667117656194,
        0.5123769266085623, 0.533092102772994, 0.554644784494836, 0.5770688317593051,
        0.6003994734960852, 0.6246733629251563, 0.6499286351402311, 0.6762049670202656,
        0.7035436395631648, 0.7319876027396146, 0.7615815429689244, 0.7923719533228895
    ])


def historic_share_of_transformation_losses_vs_extraction_subliquids(x):
    """
    historic share of transformation losses vs extraction subliquids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.01605706386987077, 0.013375704567402669, 0.01129379949353825, 0.01014342848294583,
        0.011490351286760049, 0.009944630113635408, 0.009105457515074949, 0.010502419167205769,
        0.010548036524274081, 0.009593662119669003, 0.010125399533825435, 0.011405448684794902,
        0.008476172840581088, 0.009926004318436333, 0.01357902103740488, 0.008188000564044163,
        0.009297613425131876, 0.020291121242449806, 0.015035909876251948, 0.01611299762406862
    ])


def historic_share_of_transformation_losses_vs_extraction_subsolids(x):
    """
    historic share of transformation losses vs extraction subsolids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.12808887489468548, 0.12419446569792036, 0.1252050437569366, 0.11935691213136902,
        0.12016837210710733, 0.12425103898164969, 0.11836657177817682, 0.11980088894829675,
        0.12262630759437118, 0.12004068583025358, 0.12182831042908908, 0.1260774488688789,
        0.12589138460016347, 0.12208371603473156, 0.12167722490071634, 0.1253090144654484,
        0.1249926742022905, 0.12795495498588333, 0.1330103435651417, 0.13218471066181914
    ])


def ratio_gain_gas_vs_lose_solids_in_tranf_processes(x):
    """
    ratio gain gas vs lose solids in tranf processes



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        -0.4655436785765454, -0.47767082024331714, -0.4613465490489168, -0.4638943144481499,
        -0.4594885439207112, -0.4471825571722795, -0.4450852126345265, -0.4481817224089238,
        -0.43264382276329605, -0.43190874162897963, -0.4483483475515704, -0.44555528830304947,
        -0.44797554896001957, -0.45012744230484714, -0.43332372797703317, -0.44871221929050276,
        -0.4493572878390711, -0.42914870047197806, -0.4229488604826579, -0.419986716594219
    ])


def historic_share_of_losses_vs_extraction_subliquids(x):
    """
    historic share of losses vs extraction subliquids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0012739789373926223, 0.0017486800026021427, 0.0013106160395248033, 0.0015007993499810536,
        0.0015448879729707073, 0.0017210004587693457, 0.0016050651209882875, 0.001608462807127648,
        0.0016644010289518145, 0.0016083865954213087, 0.0017099881829806055, 0.0018255697673978206,
        0.0015685355302594128, 0.0018991216520622383, 0.0017500263224406436
    ])


def historic_share_of_losses_vs_extraction_subsolids(x):
    """
    historic share of losses vs extraction subsolids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0038436874766076485, 0.003489742533787759, 0.0027225260873578864, 0.0026049521185396614,
        0.0026362712696692743, 0.001098585769199882, 0.0006980968889870701, 0.0006737200345284238,
        0.0008583882245816353, 0.0009023672127944747, 0.0009568153334068469, 0.0008955097842159121,
        0.0010495767208806045, 0.0008108439599839434, 0.0006342725505040347
    ])


def historic_share_of_losses_vs_extraction_subgases(x):
    """
    historic share of losses vs extraction subgases



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.009422896256783414, 0.010421118505311026, 0.008944163046561274, 0.008947414438666463,
        0.008670592726980444, 0.008904855075100917, 0.007845516574254089, 0.007085866070235719,
        0.006639624981043146, 0.007052502263863183, 0.00934137005114715, 0.009487119248233027,
        0.007689650174055921, 0.008058662926939446, 0.007826223652392038
    ])


def historic_demand_by_sector_subagriculture_hunting_forestry_and_fishing(x):
    """
    historic demand by sector subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        821277.3516440002, 835480.3083830001, 852339.710031, 867561.326752, 888737.4700559999,
        889964.025429, 898181.2951590001, 913098.1065810001, 929168.457933, 940175.14705,
        938211.6497899999, 950724.4604299999, 953269.03026, 1000267.1132700001, 1025427.57278
    ])


def historic_demand_by_sector_submining_and_quarrying(x):
    """
    historic demand by sector submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        106956.558, 101326.61625099998, 111658.141394, 89949.42214700002, 97189.104822,
        113437.34111929999, 118329.25110650001, 115389.89345049998, 114746.10045029999,
        123733.9133598, 140981.9255457, 164113.0972093, 158758.56715390002, 171002.8987663,
        125310.6947264
    ])


def historic_demand_by_sector_subfood_beverages_and_tobacco(x):
    """
    historic demand by sector subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1761460.089629, 1768733.999874, 1810212.34125, 1861146.10074, 1921397.32094, 1967617.76054,
        1982124.37565, 1977751.3650200001, 2016883.86587, 2061797.69482, 2122361.6047400003,
        2188629.83772, 2273095.40383, 2285147.92467, 2264271.11142
    ])


def historic_demand_by_sector_subtextiles_and_textile_products(x):
    """
    historic demand by sector subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        500387.93122930004, 496732.7349816, 502418.7662404, 505669.1392506, 516506.8589314,
        543756.6590378999, 544729.2685248, 549154.9920234001, 546981.0669373, 562149.7308112,
        581835.5048554, 616268.9163088, 643652.2674367, 647357.3118739999, 614923.9903501001
    ])


def historic_demand_by_sector_subleather_leather_and_footwear(x):
    """
    historic demand by sector subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        102249.58580294, 102739.65463260999, 102069.93810505999, 101012.66603034, 100658.93459827,
        106023.70085458002, 107789.0621533, 105896.79562564, 106633.31462076001,
        109273.86423253999, 110422.78371366, 115698.4457478, 121106.74319760001,
        124745.49981770002, 116785.2113091
    ])


def historic_demand_by_sector_subwood_and_products_of_woood_and_cork(x):
    """
    historic demand by sector subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        58713.0731415, 56674.16311640001, 62268.855379600005, 58158.672443799995, 57872.8769209,
        58320.3595168, 53786.765614899996, 52005.7191848, 50443.2999891, 50103.892673500006,
        51304.7731625, 51145.0856719, 49934.6358646, 44747.9546273, 39162.763552
    ])


def historic_demand_by_sector_subpulp_paper_printing_and_publishing(x):
    """
    historic demand by sector subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        261905.83585600002, 265565.90128099994, 279257.067462, 279803.98620299995, 291755.846006,
        308452.43508, 304893.28129199997, 298083.50359, 297931.183945, 308944.077062,
        313511.24393999996, 320463.29508, 324466.0879200001, 321599.71193000005, 295319.46288600005
    ])


def historic_demand_by_sector_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    historic demand by sector subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        224714.8110007, 236897.91061300001, 243059.31450900002, 236224.16534899996,
        247056.70687700002, 256171.347335, 263672.271245, 255718.812736, 253104.36730500002,
        262416.302171, 274366.25466000004, 288918.56216400006, 284291.709385, 311399.94481899997,
        312224.22007199994
    ])


def historic_demand_by_sector_subchemicals_and_chemical_products(x):
    """
    historic demand by sector subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        401281.55396000005, 413943.55226, 442258.04358, 446961.19391, 475205.06678, 488320.40037,
        497658.60743000003, 516406.78763999994, 526275.95114, 547978.1543599999, 543997.78205,
        561054.6723000001, 593443.6471099999, 584461.54249, 585171.71828
    ])


def historic_demand_by_sector_subrubber_and_plastics(x):
    """
    historic demand by sector subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        113840.58768099999, 113945.8747367, 121180.6431085, 119625.4361529, 124489.8262263,
        128815.7121217, 126288.11073310001, 124339.4912779, 125704.6160781, 134120.678324,
        136741.1405684, 144708.426336, 147552.14972770002, 148650.51235210002, 138539.8729385
    ])


def historic_demand_by_sector_subother_non_metalic_mineral(x):
    """
    historic demand by sector subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        64807.204056099996, 64640.82182840001, 70114.2305891, 66353.5733206, 70736.5639863,
        77060.69853259998, 77541.35509950001, 69471.22983340001, 66587.0727681, 66825.3387091,
        65313.7523083, 66810.5609691, 67500.15473740001, 65682.5872663, 55997.3959377
    ])


def historic_demand_by_sector_subbasic_metals_and_fabricated_metal(x):
    """
    historic demand by sector subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        259983.9362259, 271267.73239910003, 284850.0447308, 267256.15758, 270201.303311,
        293898.83040549996, 282533.8759788, 267336.84316630004, 274795.9357562, 275358.1269384,
        281170.5760759, 304571.7293547, 295717.7282419, 290339.8805475, 243740.63525680001
    ])


def historic_demand_by_sector_submachinery_nec(x):
    """
    historic demand by sector submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        777405.4233899999, 801501.20794, 832540.52744, 835703.45149, 819924.72297, 902042.814358,
        879577.6255900001, 849484.3911969999, 920199.3619949999, 987875.3180239999, 1066212.241217,
        1141738.3607909998, 1221521.2313239998, 1257063.9259199998, 1056073.346949
    ])


def historic_demand_by_sector_subelectrical_and_optical_equipment(x):
    """
    historic demand by sector subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        895155.41074, 976698.0374099999, 1094417.91334, 1124876.11626, 1244519.2935199998,
        1454204.57718, 1406121.56144, 1335929.5425900002, 1438594.87697, 1580086.59112,
        1690569.5772499999, 1851443.85137, 1990232.9053299997, 2060699.9134699998, 1892591.02717
    ])


def historic_demand_by_sector_subtransport_equipment(x):
    """
    historic demand by sector subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1109416.0565499999, 1135802.47235, 1226890.2660299998, 1276699.31241, 1373687.11738,
        1384100.43887, 1397812.84453, 1420318.7065599998, 1501915.9688499998, 1596650.4719800001,
        1691087.47325, 1786616.31244, 1931330.63335, 1876568.46472, 1617711.04039
    ])


def historic_demand_by_sector_submanufacturing_nec_recycling(x):
    """
    historic demand by sector submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        323158.75545460003, 335321.0626389, 355650.8314714999, 365768.3166445, 378992.4530009999,
        396661.51933479996, 395347.88420100004, 396660.0778211, 390690.83852200006, 396934.2083292,
        413810.3586759001, 433585.8377551, 438903.5490424001, 441841.0405447, 408741.12856210006
    ])


def historic_demand_by_sector_subelectricity_gas_and_water_supply(x):
    """
    historic demand by sector subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        440415.90923, 458865.99158999993, 459269.43363, 471152.12711999996, 458875.22546000005,
        478247.15768, 488833.95616, 498522.19292, 519711.2229899999, 534402.90054, 564518.89212,
        573584.0885200001, 591804.6821, 635631.9996700002, 608763.13434
    ])


def historic_demand_by_sector_subconstruction(x):
    """
    historic demand by sector subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3373315.0363289, 3434879.9374897, 3485285.9841774, 3529125.8090693, 3614579.819173,
        3641791.5051483, 3654702.3182296003, 3656187.6076439996, 3760508.5251406003, 3902283.11202,
        4076951.045161, 4285140.697028999, 4449914.955079299, 4452435.567324, 4336102.287169401
    ])


def historic_demand_by_sector_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic demand by sector subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        352813.949813, 364987.526195, 374988.41228399996, 390505.4327470001, 415087.193983,
        426132.658587, 420831.65746899997, 443827.2860779999, 449920.669003, 469539.119757,
        487527.73227100004, 506476.85412399995, 519155.96607500006, 517059.485185, 470685.328304
    ])


def historic_demand_by_sector_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic demand by sector subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1263743.815819, 1306432.08967, 1379786.87546, 1434839.818197, 1506643.2353400001,
        1541085.6627800001, 1571980.7765, 1612734.9313400001, 1679155.55503, 1784506.21438,
        1855189.96935, 1904291.2269, 2045268.72965, 2126597.04328, 2061884.04247
    ])


def historic_demand_by_sector_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic demand by sector subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1598464.786877, 1648586.60758, 1685964.6484180002, 1722990.11125, 1789523.4932309997,
        1850190.8611700002, 1891414.40524, 1923779.9265100001, 1982685.37776, 2043144.18267,
        2131827.02697, 2203047.24762, 2225281.3843300003, 2260433.2593400003, 2157301.38706
    ])


def historic_demand_by_sector_subhotels_and_restaurants(x):
    """
    historic demand by sector subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1135913.8165600002, 1162855.3384500002, 1172469.28884, 1214919.2694700002, 1258149.66992,
        1302468.91283, 1317318.16966, 1330213.84362, 1355117.6058699999, 1391521.7102100002,
        1442156.8142900004, 1491022.5754500001, 1533115.23302, 1544786.7201400003, 1489192.91292
    ])


def historic_demand_by_sector_subinland_transport(x):
    """
    historic demand by sector subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        579016.9834, 584911.4735, 600650.3032, 620252.4103999999, 664271.8027000001,
        656405.2264999999, 655483.6779, 668734.9833999999, 686551.086, 709341.2559999999,
        726012.1481, 745568.8351999999, 766743.0387, 795106.8757, 775058.13165
    ])


def historic_demand_by_sector_subwater_transport(x):
    """
    historic demand by sector subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        49225.027030000005, 52377.94612, 56106.39053, 57596.62124, 63663.95427000001, 69701.91572,
        75626.58343, 74689.24618, 81826.50412000001, 89424.92079999999, 91514.71831, 97218.65327,
        100111.31591, 112992.41015, 105631.90736
    ])


def historic_demand_by_sector_subair_transport(x):
    """
    historic demand by sector subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        143095.69405000002, 154668.3083, 160504.03436000002, 163488.79459, 167379.92006,
        178105.67977999998, 167901.45130000002, 166335.519676, 171770.97738, 186349.149928,
        189198.01622999998, 198871.62333, 207776.60457999996, 206423.82469, 188968.02946
    ])


def historic_demand_by_sector_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic demand by sector subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        165486.95716999998, 165384.27064, 175843.47261000003, 179745.55862000003, 186129.49729,
        180018.40734000003, 185577.71883, 180602.60449000003, 184531.88577, 194745.12383,
        205545.08441, 213653.95099999997, 227410.73386, 240132.33006, 238220.93145
    ])


def historic_demand_by_sector_subpost_and_telecommunications(x):
    """
    historic demand by sector subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        336812.13262999995, 367656.51979, 403243.33547, 441682.99098, 507596.44377, 579504.20999,
        646763.1806800001, 689667.7482999999, 739765.3866699999, 799182.3432199999,
        840815.0947499999, 888244.47724, 941335.00411, 997668.7938999999, 1005773.2609799999
    ])


def historic_demand_by_sector_subfinancial_intermedation(x):
    """
    historic demand by sector subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1035860.0685400001, 1057752.5364589998, 1137527.325717, 1176056.461432, 1248309.3130800002,
        1371926.4410299999, 1393974.3315, 1399563.49432, 1445953.6470100002, 1569038.7126999998,
        1664819.1774399998, 1796878.4608, 1952596.1780100001, 1987962.3170099999,
        1971211.2656500002
    ])


def historic_demand_by_sector_subreal_estate_activities(x):
    """
    historic demand by sector subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2560435.9584529996, 2654649.5703189997, 2701297.424652, 2787885.699962999, 2885024.72414,
        2953149.4055139995, 3034015.1805149997, 3088175.290212, 3175968.3918589, 3268139.760941,
        3351133.155774, 3420737.17127, 3479278.0990759996, 3513168.39938, 3544579.12064
    ])


def historic_demand_by_sector_subrenting_od_meq_and_other_business_activities(x):
    """
    historic demand by sector subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        715097.6881200001, 797501.2814, 835585.9882, 927374.2024399999, 1009120.5656999999,
        1088315.6816, 1127993.1044, 1140984.21059, 1165672.8135, 1214971.802, 1258994.5296,
        1331339.7985, 1419165.8394, 1497408.79, 1435702.263
    ])


def historic_demand_by_sector_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    historic demand by sector subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3163684.928, 3243750.0253999997, 3285804.0601, 3371778.8820999996, 3522683.9758,
        3622324.8474, 3719665.8919999995, 3839981.7000999996, 3947516.9477999997, 4051795.136,
        4081013.6649999996, 4137274.7199999993, 4201002.810999999, 4305412.6680000005,
        4451215.852999999
    ])


def historic_demand_by_sector_subeducation(x):
    """
    historic demand by sector subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1176056.855178, 1187789.1011589998, 1208351.7021030001, 1249417.9983253998, 1293705.199234,
        1273106.580141, 1290728.985381, 1302962.424367, 1328156.208041, 1350439.9591238,
        1354468.59733, 1371169.29075, 1397140.7155360002, 1414788.936597, 1427681.240051
    ])


def historic_demand_by_sector_subhealth_and_social_work(x):
    """
    historic demand by sector subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2116985.945408, 2176783.2387836003, 2219207.0180761, 2281499.2523729997,
        2380721.4962919997, 2433973.862812, 2511162.247971, 2600103.371875, 2687871.9925499996,
        2778897.983662, 2904651.5214279997, 2988410.438662, 3089709.4131940003, 3157580.3929769997,
        3215178.3753789994
    ])


def historic_demand_by_sector_subother_community_social_and_persona_services(x):
    """
    historic demand by sector subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1121312.875, 1129908.562, 1134725.674, 1168176.164, 1225040.244, 1283243.0548,
        1301613.3548, 1340765.8257, 1364030.6045, 1401915.9587000003, 1411694.3254000002,
        1450671.2885999999, 1486387.1959000002, 1509352.3303999999, 1504855.5695999998
    ])


def historic_demand_by_sector_subprivate_households_with_employed_persons(x):
    """
    historic demand by sector subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        48106.64623337812, 48926.64200190818, 49488.43596377072, 51425.36695231778,
        50869.11033087904, 52155.816324571155, 52146.134999738344, 52317.25933618104,
        53559.402520043535, 54794.767373428214, 55162.71588258705, 56378.119990721425,
        58766.5993487199, 59780.91423515376, 59073.326054017445
    ])


def historic_final_energy_intensity_h_subelectricity(x):
    """
    historic final energy intensity h subelectricity



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.6319452492878225, 0.6474447373221129, 0.6391440903441314, 0.6425429079164128,
        0.6367531217492081, 0.6381846304893362, 0.6397246612461346, 0.6454008888282491,
        0.6483867554557563, 0.6454292743121216, 0.6541601234210128, 0.6498359966439917,
        0.6468084213534645, 0.6527043158711583, 0.6755273041086106
    ])


def historic_final_energy_intensity_h_subheat(x):
    """
    historic final energy intensity h subheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.2855157381546875, 0.25404242100647506, 0.24406615678652396, 0.23610742913107505,
        0.22273851188620555, 0.2209255171420526, 0.2280921535653369, 0.21266637246609502,
        0.20891825736332598, 0.20146970103522027, 0.17173640930783207, 0.17010396189688415,
        0.16529326408979034, 0.16134248582086183, 0.16062564751664934
    ])


def historic_final_energy_intensity_h_subliquids(x):
    """
    historic final energy intensity h subliquids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.568424004123364, 1.5730823826140428, 1.5751815947459, 1.5215015338090632,
        1.5231645132539757, 1.5686384037861798, 1.5744559913368583, 1.5255667193507039,
        1.5531450950363177, 1.600019681580165, 1.645749849370925, 1.6358124056952272,
        1.6167993069329223, 1.5976487464510023, 1.5959097248509646
    ])


def historic_final_energy_intensity_h_subsolids(x):
    """
    historic final energy intensity h subsolids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.95104724970324, 1.91801080181749, 1.8449354246520582, 1.7516917434535115,
        1.6942081331809153, 1.6505256393456411, 1.6289997302953656, 1.5913544671283866,
        1.5780902111352826, 1.540341358331322, 1.5113938358165822, 1.4752141383258566,
        1.4422144923500455, 1.4416220292300719, 1.4683310666706337
    ])


def historic_final_energy_intensity_h_subgases(x):
    """
    historic final energy intensity h subgases



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8069129864373691, 0.8648994945858922, 0.8062956712422991, 0.7601404056051215,
        0.7625889623573991, 0.7699018890737395, 0.7585472995515781, 0.7588815049134393,
        0.778744234679196, 0.7536943195133183, 0.7279680365530793, 0.697784790932038,
        0.6896973704625412, 0.6939916695225792, 0.6975686054927074
    ])


def historic_res_capacity_for_heatxdashxcom_subsolarxdashxheat(x):
    """
    historic res capacity for heatxdashxcom subsolarxdashxheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        2.0303326826418786e-06, 2.0303326826418786e-06, 6.4293868283659495e-06,
        5.414220487045009e-06, 7.782941950127201e-06, 8.121330730567515e-06, 9.136497071888453e-06,
        1.2520384876291584e-05, 1.7257827802455968e-05, 1.7257827802455968e-05,
        1.8611382924217222e-05, 1.658105024157534e-05, 2.0980104387299415e-05,
        2.3348825850381605e-05, 4.7036040481203515e-05, 6.497064584454012e-05,
        9.339530340152642e-05, 0.00014246167656537182, 0.00018137638631600782,
        0.0002493925311845108
    ])


def historic_res_capacity_for_heatxdashxcom_subgeotxdashxheat(x):
    """
    historic res capacity for heatxdashxcom subgeotxdashxheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.0019992724712776778, 0.0019333400358171797, 0.001957511273499612, 0.0019656481257887483,
        0.0020795640578366507, 0.002191445776812269, 0.0022711390654088053, 0.00233635354331438,
        0.002309190815819764, 0.002360764100181787, 0.002849214556714914, 0.002853282982859482,
        0.002944702911519773, 0.00304868709739123, 0.0030231996041914363, 0.003124551279028173,
        0.003072140377518739, 0.003306194540423882, 0.003420230132064272, 0.0035817705819221166
    ])


def historic_res_capacity_for_heatxdashxcom_subsolid_bioexdashxheat(x):
    """
    historic res capacity for heatxdashxcom subsolid bioexdashxheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.01167364283024201, 0.014963216653269409, 0.014075786414101827, 0.014111998996576254,
        0.013306506859960276, 0.01325437596189726, 0.014136859473511416, 0.014877727054012784,
        0.016775811784095436, 0.017262303413099544, 0.01806468799009863, 0.01981614664193105,
        0.020802194334247032, 0.02147558347735297, 0.022805745832552054, 0.02708885085880959,
        0.0262413115379895, 0.029897640815394526, 0.030840372931826027, 0.03108885086200959
    ])


def share_inventories_next_step_subagriculture_hunting_forestry_and_fishing(x):
    """
    share inventories next step subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03461669019581704, 0.041942233336400334, 0.009276799405214435, 0.016075123961072323,
            0.024217754183502624, 0.032446087618470246, 0.013212424725283112, 0.03376795965480616,
            0.03382604411506391, 0.018401487557593552, 0.01982139598204595, 0.018904899276004725,
            0.015809658030545876, 0.010160482589476171
        ])


def share_inventories_next_step_submining_and_quarrying(x):
    """
    share inventories next step submining and quarrying



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.09426322868948797, 0.1452938030085441, -0.010522918740413148, 0.015323039580696841,
            0.062066986325035677, 0.04610341694033021, -0.03526634160335441, 0.04212319356415536,
            0.022861810664424075, 0.04151711701584518, 0.10319681541565758, 0.021374515787298973,
            0.0025377049344237237, 0.020923144714220698
        ])


def share_inventories_next_step_subfood_beverages_and_tobacco(x):
    """
    share inventories next step subfood beverages and tobacco



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.011229125465680464, 0.014950252179427848, 0.0015067456009418114,
            0.008219784022741013, 0.013112354704970405, 0.015008429524128383, 0.00496232604036188,
            0.010115436166276024, 0.014090630265515118, 0.010472018505405785, 0.011288293056324823,
            0.010285984899975899, 0.011670533759363205, 0.003484397941663512
        ])


def share_inventories_next_step_subtextiles_and_textile_products(x):
    """
    share inventories next step subtextiles and textile products



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.025922783205493752, 0.025535222531598935, 0.0008444416652216993,
            0.020417391981624455, 0.028992295980142094, 0.025311362536731913, 0.007727580121531834,
            0.002825711516221771, 0.012079303124812086, 0.005519571894805784, 0.013120023882477527,
            0.005518243280871071, 0.007934972086945095, -0.01617110107273339
        ])


def share_inventories_next_step_subleather_leather_and_footwear(x):
    """
    share inventories next step subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.02741801605303332, 0.03308742478636399, 0.004191804519572037, 0.014882491116945982,
        0.019108017204367233, 0.024126896069485666, -0.0019191225645622243, -0.0011903123376724623,
        0.000523595101187629, 0.0014322759731371915, 0.0007313941121081056, -0.0067765566832306955,
        -0.00849822559971483, -0.02726512256395504
    ])


def share_inventories_next_step_subwood_and_products_of_woood_and_cork(x):
    """
    share inventories next step subwood and products of woood and cork



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.05099801463435518, 0.060516673656964955, -0.0006817607991020593, 0.03308155567618923,
            0.04742896859549011, 0.011396171400018095, 0.0012185991078174092,
            -0.0032429105557199417, 0.014559681116070878, 0.010593103068180825,
            -0.012816239554372989, -0.07238100443548619, -0.10795313752850011, -0.1616632516649013
        ])


def share_inventories_next_step_subpulp_paper_printing_and_publishing(x):
    """
    share inventories next step subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0017209535478593398, 0.011116633244823542, -0.019023876579578658,
            -0.013690682653597472, 0.01797167429902852, 0.009140803917325045, -0.03402770659174538,
            -0.03710591437128926, -0.03741405276295467, -0.03459212455574808, -0.04266723275308837,
            -0.05746675136230981, -0.06279328074894397, -0.08641675950037707
        ])


def share_inventories_next_step_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    share inventories next step subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.022397163766748884, 0.029657526659950657, -0.009844842065857869, -0.0012157576444566558,
        0.017875454252156166, 0.017945093269225307, -0.016425040672843304, 0.0015011190602734984,
        0.012571300154402403, 0.024019248679712973, 0.01245074623470567, -0.011533595218422517,
        0.008062090060610764, -0.01979745228789293
    ])


def share_inventories_next_step_subchemicals_and_chemical_products(x):
    """
    share inventories next step subchemicals and chemical products



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.05005938101189353, 0.06341341306758376, 0.020305971801720977, 0.045376681578992654,
            0.04901889821081202, 0.04136089819946551, 0.009874150034516383, 0.013029559844310389,
            0.031023231245148577, 0.027035191107908304, 0.03542343907150098, 0.03570302269336227,
            0.014621691554917347, 0.002697712399088707
        ])


def share_inventories_next_step_subrubber_and_plastics(x):
    """
    share inventories next step subrubber and plastics



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.05155601037400166, 0.05613935712413145, 0.01993782490332162, 0.014493329733790127,
            0.0326650525055877, 0.022302165133758694, 0.007727464783111726, 0.008855703431832364,
            0.02600911316279692, 0.00811772737440893, 0.02960706717970953, 0.0038264700381658127,
            0.005101709762047021, -0.05411230890422361
        ])


def share_inventories_next_step_subother_non_metalic_mineral(x):
    """
    share inventories next step subother non metalic mineral



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.09206994019660208, 0.10746874545566801, 0.029834516830541346, 0.0323522570370145,
            0.04737016753690253, 0.0459722775727321, -0.06198036957638333, -0.028632375936390057,
            -0.0045939428954693065, 0.007506379019318679, 0.0529179511250499, 0.040814956805921516,
            0.023955408358392986, -0.049341810520510544
        ])


def share_inventories_next_step_subbasic_metals_and_fabricated_metal(x):
    """
    share inventories next step subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0031870963507300964, 0.009123680856206623, -0.05776728641089819, -0.063023093491151,
            0.028766376131321228, -0.0012881757231380966, -0.05179315666335895,
            -0.02408351048492784, 0.007466442421217998, -7.513578161285693e-05,
            0.039232899341370225, -0.04691660213435318, -0.05954158611418101, -0.1578288206210243
        ])


def share_inventories_next_step_submachinery_nec(x):
    """
    share inventories next step submachinery nec



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.010574262541391523, 0.00696599493820793, -0.002172547925658203,
            0.0016757710330077133, 0.018576378785219894, 0.013271560872378692,
            -0.011732858194081435, 0.01588751699230089, 0.009124612525020096, 0.006060567821491422,
            0.014060643446260344, 0.006903952206265926, 0.004552560360692592, -0.00599537770581172
        ])


def share_inventories_next_step_subelectrical_and_optical_equipment(x):
    """
    share inventories next step subelectrical and optical equipment



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.023168508723542067, 0.028354704013657167, 0.015847142402905943, 0.02265004660576361,
            0.04296331477731733, 0.0016590409136539952, -0.00859960846267911, 0.011252897017189633,
            0.03358385692165521, 0.02280449294652064, 0.029076321142639805, 0.01526034762999966,
            0.013992211001478148, -0.009340748606651865
        ])


def share_inventories_next_step_subtransport_equipment(x):
    """
    share inventories next step subtransport equipment



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.008363827805679104, 0.010886046103550569, 0.0032035392047634697,
            0.021106266218248022, 0.011928490546162383, 0.021600092686336735, 0.011439507854791203,
            0.012926172570669917, 0.010692503650363025, 0.0198993047564401, 0.02190095810026589,
            0.015357859751103917, 0.006712683942431885, -0.006722001475231583
        ])


def share_inventories_next_step_submanufacturing_nec_recycling(x):
    """
    share inventories next step submanufacturing nec recycling



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.01857734778416911, 0.022404736598060047, 0.007320140860101643, 0.017631024963925525,
            0.03481081306590101, 0.030668103927010547, 0.014946590623808492, 0.013840143578630438,
            0.021493995531179255, 0.02802975748870618, 0.022896789367939235, 0.007827848982984867,
            0.01601343571723777, -0.006185433819430002
        ])


def share_inventories_next_step_subelectricity_gas_and_water_supply(x):
    """
    share inventories next step subelectricity gas and water supply



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.00042413317519049144, 0.0013558288107230248, 0.0005431237710083078,
            0.0007282829655163663, 0.0006328974362719102, -0.0002694355176032213,
            -0.0010487328296012264, -0.00033330844580074255, -0.0016884595669077243,
            -0.0005938883617179873, 0.00023858153100637895, -0.0009011798421525196,
            -0.00120160160343804, -0.0021056969906526287
        ])


def share_inventories_next_step_subconstruction(x):
    """
    share inventories next step subconstruction



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            -0.00014896785893889322, 0.00032862049346872265, 0.00012345146463194415,
            0.0004437915830468549, 0.0002729355150054159, 0.00034811231920423496,
            2.6341616277743705e-05, -5.133599583922819e-05, -0.00020292513825069224,
            0.00020208833289227384, 0.0003465351793535172, 0.0009605838860180702,
            0.0011973075229034687, 0.0010350447943260576
        ])


def share_inventories_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    share inventories next step subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0023929746013658833, 0.0031142324982446606, 0.0015004391254643849, 0.0028785617029874824,
        0.006521608339560344, 0.006391678364164279, -0.000475560711611828, 0.001976871260373391,
        0.004240190894063026, 0.0031622262651984615, 0.005417379052287837, 0.004613781515618692,
        0.004953710498287375, -0.004117596796533778
    ])


def share_inventories_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    share inventories next step subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.015368793493944032, 0.0176144036678834, 0.0076430350349353925, 0.010562158065515002,
        0.016008740199098148, 0.007243281960070458, 0.0009669664677655769, 0.0035568537305010403,
        0.0022383440684109767, 0.003896461990106976, 0.005195678507696852, 0.003923266504628535,
        0.008047525531025905, -0.0021458011259934246
    ])


def share_inventories_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    share inventories next step subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.004783819038525881, 0.006436223327803524, 0.0026383681892997285,
            0.005164286657848896, 0.010710865249582027, 0.00984858735790179, 0.0015008944423482583,
            0.003897921771497273, 0.006130297658989541, 0.004498001375669275, 0.006691703056267292,
            0.0054425175554355726, 0.004830282847275033, -0.000741128990872675
        ])


def share_inventories_next_step_subhotels_and_restaurants(x):
    """
    share inventories next step subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        -0.00020737488320897337, 0.0001446685483487721, -0.0003572996584286368,
        0.0006549819387167176, 0.0020955879814969908, 0.002911897359610583, 0.0006233249367963001,
        0.0007016828988724827, 0.0005359206216678118, 0.0004919458570455678, 0.0006475599403373204,
        0.0002856755321237399, 0.00023288616823906658, 0.0004124112092339731
    ])


def share_inventories_next_step_subinland_transport(x):
    """
    share inventories next step subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.010176833024630351, 0.012536329641196792, 0.00430214531254968, 0.005348367920419633,
        0.006554338122717555, 0.005378382435539208, 0.0019515962711634627, 0.0027197909056981668,
        0.004730984658814206, 0.003957970135238292, 0.006311517029460731, 0.003440626216147738,
        0.0028761249209255206, 0.0002411375900309864
    ])


def share_inventories_next_step_subwater_transport(x):
    """
    share inventories next step subwater transport



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03445012707955338, 0.033535203070921914, 0.013131889748329966, 0.027919145462781503,
            0.041024487927804686, 0.08466919183155805, 0.00307740567425285, 0.003488829237790001,
            0.0029639888705386475, 0.003599625678616155, 0.05066574298576478,
            0.0014308799030149516, 0.08611950649678217, 0.08684434873201745
        ])


def share_inventories_next_step_subair_transport(x):
    """
    share inventories next step subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0021245645834738853, 0.0025850705351703157, 0.0008091388791002279, 0.0023436047159025035,
        0.004759184665233702, 0.010872926861949048, -0.00044423394440304637, 0.0012555125044372616,
        0.0004702718420441391, 0.001191508687522141, 0.00350381933999573, 0.0010309466286302909,
        0.002977872931688678, 0.0005918793793829298
    ])


def share_inventories_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    share inventories next step subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.015631051792266138, 0.02986296802524229, 0.012259844509753592, 0.012338783661043004,
            0.011534383792643545, 0.030505284447352746, 0.004886689383534703, 0.004357898672331982,
            0.005428457355999516, 0.005817028918166772, 0.00976790454953955, 0.005881481394028688,
            0.010246379983008607, 0.0019734244893533683
        ])


def share_inventories_next_step_subpost_and_telecommunications(x):
    """
    share inventories next step subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0009488551439241703, 0.0012396105428931219, 0.0007444440848184912, 0.0014685834763999532,
        0.0020441730699772512, 0.0032325119339692335, -0.005068250485275013,
        0.00043111420964910284, 0.0009169560942092407, 0.000570352331915007, 0.0007087983726690691,
        0.0005448397305536379, 0.0009565343787786686, 0.000645581668543594
    ])


def share_inventories_next_step_subfinancial_intermedation(x):
    """
    share inventories next step subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        9.11231272700768e-05, 7.542788209146378e-05, -1.4400616429060148e-05,
        8.229749543927034e-05, 0.0003937912440804006, 0.0009093943635503735, -0.007524568226264509,
        -0.004274426163508445, 8.49992603244964e-05, 9.833482351623146e-05, 0.00013797126261373458,
        8.384317855566425e-05, 0.0006442841441413284, 0.0012264529642908699
    ])


def share_inventories_next_step_subreal_estate_activities(x):
    """
    share inventories next step subreal estate activities



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            2.91907526576806e-05, 1.9879281529659025e-05, 1.4105622407877741e-05,
            8.646807700213472e-05, 2.161401447580686e-05, 7.110088024127258e-06,
            7.479695881646118e-06, 2.0296672084406522e-06, 9.156903678863279e-06,
            2.6108742903649806e-05, 3.0340352036303263e-05, 2.039824756142603e-05,
            7.322588921311033e-05, -2.8778440691593814e-05
        ])


def share_inventories_next_step_subrenting_od_meq_and_other_business_activities(x):
    """
    share inventories next step subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.000831602174777396, 0.0033847710947051517, 0.0005761788915349635, 0.0031079738205755606,
        0.0021995063936603304, 0.0022211587023244217, 0.000463289136776625, 0.003827049793333797,
        0.0032723048332935715, 0.00282669028048174, 0.0033238445248807004, 0.0019278144414444816,
        0.01864576873493577, 0.0074378011898418245
    ])


def share_inventories_next_step_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    share inventories next step subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        -0.006890673086698083, -0.005679982938310693, -0.0072323360613766275,
        -0.006114828678354021, -0.005356116532156387, -0.0038007332944622442,
        -0.005674782512487631, -0.004853486698943161, -0.004676214705836499, -0.004558338326465761,
        -0.004537527544218771, -0.0053082216326086635, -0.005544836892740801, -0.003922579936947399
    ])


def share_inventories_next_step_subeducation(x):
    """
    share inventories next step subeducation



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            1.8730226585082882e-05, 1.3231663407411775e-05, 4.799534989921149e-06,
            3.0446684471332545e-05, 1.098827169556429e-05, 2.5728856619887022e-05,
            1.3380636827243842e-05, 4.505158402132235e-05, 2.248247898388548e-06,
            8.725195270895369e-05, -2.615975302391741e-05, 8.26970101974835e-06,
            1.5833949800231638e-05, 4.7541740478130375e-05
        ])


def share_inventories_next_step_subhealth_and_social_work(x):
    """
    share inventories next step subhealth and social work



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            8.428876000649878e-07, 3.87479673142649e-06, 4.426989397210959e-06,
            1.484503418608409e-05, 2.0269244774470538e-05, 1.2951043297292982e-05,
            -3.1727632790426594e-05, 1.9634547384056006e-05, 1.5997263037852878e-05,
            1.0177684924305919e-05, 1.965940194824921e-05, 1.4489386545164095e-05,
            2.3448960211648786e-05, -9.502931854099198e-06
        ])


def share_inventories_next_step_subother_community_social_and_persona_services(x):
    """
    share inventories next step subother community social and persona services



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            -0.010629458350807685, -0.008848671736319593, -0.013448818323945873,
            -0.008946039163755016, -0.004793431904417115, -0.0016249396890407504,
            -0.0022020738024543162, -0.0018956770408739025, -0.001785715673228679,
            -0.0018879388774512668, -0.0017953987374448959, -0.0024802205711734492,
            -0.003448648466737081, -0.0020721433092937003
        ])


def share_inventories_next_step_subprivate_households_with_employed_persons(x):
    """
    share inventories next step subprivate households with employed persons



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            9.814582614953874e-05, 9.588248461668407e-06, -7.821361593256881e-05,
            -2.411516128394616e-05, -3.631893686050125e-06, -5.08923087015618e-05,
            -0.00022539143199814182, -0.0002681430771119126, 6.398985647852799e-05,
            4.7633349046714305e-05, 0.0015237584902465407, 0.000990345581418567,
            0.0009393292444326495, 0.0008607609964860263
        ])


def share_consum_goverments_subagriculture_hunting_forestry_and_fishing(x):
    """
    share consum goverments subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.005268868173873272, 0.0067709613778314465, 0.006395238231716023, 0.007562221479560522,
        0.006909511984020509, 0.007982988184916011, 0.008446717428753593, 0.009326405167881661,
        0.009029494951501007, 0.009305637069275475, 0.009311394291421764, 0.009911989953154088,
        0.010233756778334888, 0.008994169837883667, 0.00869869421963916
    ])


def share_consum_goverments_submining_and_quarrying(x):
    """
    share consum goverments submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.00465464399106785, 0.00436164392290795, 0.003995603763685553, 0.004066047799643348,
        0.006393419006564868, 0.006003153311605073, 0.0042157315738525596, 0.005643824953173819,
        0.005273239069785034, 0.005338694073945094, 0.005231106875192582, 0.004151188915356074,
        0.004717994395060901, 0.00450208701463089, 0.004934708656353205
    ])


def share_consum_goverments_subfood_beverages_and_tobacco(x):
    """
    share consum goverments subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.004599538103475525, 0.0038220671963571573, 0.002903073070627261, 0.002936792709517417,
        0.003142168948714033, 0.0031340511473669625, 0.0030303441468097964, 0.003102005746783019,
        0.003076968042145073, 0.002915823853670982, 0.002931906554520569, 0.002970467773012117,
        0.003045614006493215, 0.002961515281763346, 0.002885066309883363
    ])


def share_consum_goverments_subtextiles_and_textile_products(x):
    """
    share consum goverments subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.003187221954138558, 0.003551578898993538, 0.00295957655229884, 0.0026490453856551233,
        0.0028527421747088517, 0.0025817118313251836, 0.0027787410874748823, 0.0028558425631741735,
        0.003004220437100366, 0.0030660841863479906, 0.0034604581246728532, 0.00402674114226547,
        0.004420561138285466, 0.004407476902887508, 0.005253312524302093
    ])


def share_consum_goverments_subleather_leather_and_footwear(x):
    """
    share consum goverments subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0017657285218539121, 0.0015559542279136721, 0.0012399353066103794, 0.0013267840090444238,
        0.00133344367825553, 0.0012313162901100574, 0.0011201067862366928, 0.0008200600923468703,
        0.0007351843303268899, 0.0007872038350995223, 0.0008524970828855757, 0.000980302019330771,
        0.0012534804090331476, 0.0013308059227996672, 0.001514645887241861
    ])


def share_consum_goverments_subwood_and_products_of_woood_and_cork(x):
    """
    share consum goverments subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.008220627948340928, 0.009005435491861913, 0.00784410468158404, 0.010385380453511185,
        0.005983313227598484, 0.0029442159037194747, 0.003191210291931943, 0.0038803930252921484,
        0.0033353083568353945, 0.0031038702125085094, 0.002934685814185623, 0.0032009994283761284,
        0.0031885473327917984, 0.003374415685759153, 0.003641151621250122
    ])


def share_consum_goverments_subpulp_paper_printing_and_publishing(x):
    """
    share consum goverments subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.011608853579237062, 0.012152022471383788, 0.010944306361792914, 0.013144143333725558,
        0.01481962044356459, 0.014280346332352903, 0.014584228557471577, 0.01813378209427802,
        0.01800422812064625, 0.017428493050279235, 0.01703528662283678, 0.018253358153044425,
        0.01977514458023117, 0.016156351225622194, 0.016038861285036368
    ])


def share_consum_goverments_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    share consum goverments subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.014933772211345458, 0.014073012258205416, 0.014138264180255291, 0.01638462176078289,
        0.012559974344452331, 0.009745231174255206, 0.008722834180249866, 0.009133945504476285,
        0.007599833303872037, 0.006750850405801407, 0.005867347651754397, 0.006685369003406569,
        0.007433579067682456, 0.006607524613390546, 0.007784103038001168
    ])


def share_consum_goverments_subchemicals_and_chemical_products(x):
    """
    share consum goverments subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.09588197518751453, 0.1019113905016282, 0.0973261077437338, 0.10451610930994258,
        0.1038905105421696, 0.10684729730821506, 0.11441273827060026, 0.11710461296677933,
        0.11802421308716918, 0.1131680314380918, 0.11811035287289917, 0.1159991266683548,
        0.12291936960693232, 0.13104795855975498, 0.13239560556296268
    ])


def share_consum_goverments_subrubber_and_plastics(x):
    """
    share consum goverments subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.006867200230823096, 0.007733117166691025, 0.007382076353556275, 0.008098756846008068,
        0.008121760071879654, 0.006928724883784163, 0.006611896916921303, 0.006412599503225718,
        0.00627859918453306, 0.006463852038577615, 0.006987327120633945, 0.007095522534506072,
        0.007271046894131369, 0.006823697974194504, 0.0071450764967817045
    ])


def share_consum_goverments_subother_non_metalic_mineral(x):
    """
    share consum goverments subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.005150147500748169, 0.005601972248454674, 0.004984833707272183, 0.0050045677027149025,
        0.005220994619862051, 0.004906031702269911, 0.0046695726368900506, 0.004765867407184146,
        0.004575710980134348, 0.004925408480648355, 0.005451801457053173, 0.005476256518325244,
        0.005940164753101489, 0.006012657637842877, 0.006158849250484725
    ])


def share_consum_goverments_subbasic_metals_and_fabricated_metal(x):
    """
    share consum goverments subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.003714588731977942, 0.003695607624002559, 0.003264025079857985, 0.003456144989750174,
        0.003909675072085389, 0.003567486126274761, 0.004087484008773003, 0.004580826890509096,
        0.003954657469767477, 0.003832560933406138, 0.003872948283557392, 0.0037354425586724055,
        0.004369197638847987, 0.004504712881791057, 0.005212766425513586
    ])


def share_consum_goverments_submachinery_nec(x):
    """
    share consum goverments submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.003462102165770177, 0.0036617822542566983, 0.003605389168536691, 0.003975012421065428,
        0.003576070848771421, 0.0024004574567129543, 0.002133422389799059, 0.0020732528087047206,
        0.0015195162676146234, 0.0016070917766987168, 0.0017110936542217888, 0.0017042538525656047,
        0.0018089274613794315, 0.00179493990200113, 0.0020036796744404423
    ])


def share_consum_goverments_subelectrical_and_optical_equipment(x):
    """
    share consum goverments subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.005879390033122016, 0.006689280053562139, 0.006260917348372466, 0.006383402577586877,
        0.007626339783897834, 0.008952172345135323, 0.009938402470472538, 0.010584245313257168,
        0.010485426607225824, 0.009528021492371893, 0.009159007241327075, 0.009390892943976928,
        0.009368214117085202, 0.008491380955383702, 0.010526372953268005
    ])


def share_consum_goverments_subtransport_equipment(x):
    """
    share consum goverments subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.002818274426028272, 0.0030134399979940317, 0.0030013782829294688, 0.003324582114787668,
        0.0033937364928424093, 0.004179287165548907, 0.004060256365656964, 0.004599388270976094,
        0.004521827080112946, 0.0039045057195699682, 0.003377303061102904, 0.0032156831100202668,
        0.002999720037553041, 0.003025284132570713, 0.0031225730516014757
    ])


def share_consum_goverments_submanufacturing_nec_recycling(x):
    """
    share consum goverments submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.02071206794500831, 0.02145589854505419, 0.0214355818274276, 0.02514402773966533,
        0.01774772016365786, 0.013419020602064787, 0.012203537170182726, 0.011759010701610579,
        0.010195116719573927, 0.010562251657894164, 0.010833741123216334, 0.01030096052750391,
        0.010593246762629493, 0.011621782561596393, 0.012719313855909485
    ])


def share_consum_goverments_subelectricity_gas_and_water_supply(x):
    """
    share consum goverments subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.011073682393823454, 0.014062257866705289, 0.013761686141498856, 0.014297671627160625,
        0.012947249209290532, 0.013116308794034108, 0.012724684366983808, 0.01399302698870909,
        0.013964725368533456, 0.014932092232165415, 0.016923049048231383, 0.020237318698904688,
        0.020307088408552488, 0.018222111231047673, 0.019718462441084224
    ])


def share_consum_goverments_subconstruction(x):
    """
    share consum goverments subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0038967149105365593, 0.0037926289235951103, 0.003584887454493617, 0.004030136291386803,
        0.0043782604871680005, 0.004424182707116258, 0.004527167347521452, 0.004951394442164713,
        0.0046428441481454205, 0.003398133507831463, 0.0023546145866514584, 0.001689633412741827,
        0.0016539914075433919, 0.0017930949205848523, 0.0020141898925777795
    ])


def share_consum_goverments_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    share consum goverments subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0038892410595649297, 0.005403324383601953, 0.005555193258671485, 0.005806921261116362,
        0.0055187734847195, 0.0060804999283362755, 0.006576770190355463, 0.006599236892085225,
        0.006951525047583767, 0.006930155684757487, 0.0071569467110036466, 0.006965090647827174,
        0.006885708406717167, 0.006969574494336234, 0.008003455543375147
    ])


def share_consum_goverments_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    share consum goverments subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.008215710233384077, 0.01006648267750522, 0.009307515695653029, 0.009444958125729505,
        0.009364679486856759, 0.009765600552568655, 0.010207317570209486, 0.010771962374229452,
        0.01067527004648461, 0.010462899400149748, 0.010587576110538116, 0.0108347645089915,
        0.01077759596108046, 0.010732709364063026, 0.012048869135356076
    ])


def share_consum_goverments_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    share consum goverments subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.004884652051206438, 0.006598553542763822, 0.006508686887531563, 0.006705691997046858,
        0.007025347276833513, 0.00720794339648111, 0.007317693553382741, 0.007707318698817354,
        0.007486817205849609, 0.007496077432961913, 0.007788715871380901, 0.007790913253695477,
        0.00796523132976134, 0.008075132023729639, 0.009141731015561386
    ])


def share_consum_goverments_subhotels_and_restaurants(x):
    """
    share consum goverments subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.014358500409294464, 0.009986641171961503, 0.004972707818870327, 0.005180644227287415,
        0.003157808005666468, 0.0026105687179988736, 0.0023619220258709814, 0.0022802596097973695,
        0.002236912343894969, 0.0032285975612425005, 0.004006717191046085, 0.004771145868031531,
        0.005057546903846365, 0.004275366439841901, 0.004318083939434814
    ])


def share_consum_goverments_subinland_transport(x):
    """
    share consum goverments subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.015458390956758244, 0.020988629145090622, 0.020208389033241396, 0.024190375963752966,
        0.028979965613103086, 0.0340573400355154, 0.03787454338990796, 0.038778817683728796,
        0.042856040285980994, 0.04333737779943819, 0.040877931695314014, 0.040017654428911954,
        0.044561875720359245, 0.042253292012375945, 0.04548291871340436
    ])


def share_consum_goverments_subwater_transport(x):
    """
    share consum goverments subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.006638978375793083, 0.007670674964602831, 0.006643243068732121, 0.0070420634278164475,
        0.005625692184953877, 0.005825483070381239, 0.005755526565640597, 0.0059064747947359724,
        0.031040299562048543, 0.0444328959360957, 0.045343068051017205, 0.0419767170469466,
        0.05198184793293863, 0.050442763300947255, 0.05576891724508192
    ])


def share_consum_goverments_subair_transport(x):
    """
    share consum goverments subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.005646296734251731, 0.0056558638910256965, 0.003568218719757792, 0.003452988698190144,
        0.003113214535012367, 0.002600142907132616, 0.00352760744719066, 0.0035150489272458216,
        0.008303301417705423, 0.01003440101939009, 0.007348632019004971, 0.007740245562564343,
        0.009810656999234713, 0.009168958102788652, 0.00981130620506604
    ])


def share_consum_goverments_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    share consum goverments subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.09733431731089942, 0.09901239662412917, 0.09351013009433082, 0.09307715377473842,
        0.090464559595115, 0.08605821054032661, 0.10652486260006906, 0.11145975472969767,
        0.11357295739188283, 0.11017723359651423, 0.10491172806150008, 0.1011831510665581,
        0.09842241225860136, 0.09838464480853919, 0.09731960940160281
    ])


def share_consum_goverments_subpost_and_telecommunications(x):
    """
    share consum goverments subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.007425900250296456, 0.007218033564359982, 0.006671966436504588, 0.00755011709325932,
        0.007798984111468217, 0.006194892527289748, 0.005508562494627751, 0.00575293539502172,
        0.005392135225406816, 0.004821274684917958, 0.005148559804682313, 0.00553343108337951,
        0.006297209998691717, 0.006491788496943986, 0.007810889098747097
    ])


def share_consum_goverments_subfinancial_intermedation(x):
    """
    share consum goverments subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03758331475685865, 0.027581273496790935, 0.025235830692599766, 0.02508840686447435,
        0.024038702335690176, 0.02474875619024354, 0.031203030082480394, 0.03890041017856948,
        0.04298777082414324, 0.04647391196264396, 0.04820648818054139, 0.054020887398685435,
        0.05660365990915811, 0.057191672612287296, 0.05349723889956909
    ])


def share_consum_goverments_subreal_estate_activities(x):
    """
    share consum goverments subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0053409330371467, 0.00480747520979442, 0.004356879732159147, 0.005051056791957753,
        0.0042817328034103035, 0.003082759572882315, 0.002882345202542986, 0.0028984962506404615,
        0.0027998697728837665, 0.0029336909071597957, 0.0029228584018284724, 0.0032377155699126986,
        0.0032471376757725566, 0.003313038453281683, 0.0033056974047413435
    ])


def share_consum_goverments_subrenting_od_meq_and_other_business_activities(x):
    """
    share consum goverments subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.10777177059907846, 0.12766213970374193, 0.128477992110974, 0.13977631646313407,
        0.14717767633342763, 0.15478196524040605, 0.16957094795516728, 0.17978959576830966,
        0.1755571954925755, 0.16458717780184334, 0.1432232593236837, 0.13416821175274135,
        0.14146388986144026, 0.13442398718655846, 0.13699391236524086
    ])


def share_consum_goverments_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    share consum goverments subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8825512222435824, 0.9019740353263537, 0.9055209153005452, 0.9070353979129336,
        0.9101561542351737, 0.9071215692757418, 0.9039291693459441, 0.9009660123926903,
        0.8991346577949707, 0.8948821888313753, 0.8935084759144025, 0.8922590472793164,
        0.8935680286075393, 0.895349946046101, 0.892671897122667
    ])


def share_consum_goverments_subeducation(x):
    """
    share consum goverments subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.7095164543500799, 0.7040856404423688, 0.6993589354235593, 0.698581612534672,
        0.6987685915889159, 0.6958775202480544, 0.6893004419028962, 0.6836379033975156,
        0.6765593042142081, 0.6662693916312913, 0.6630464757694154, 0.660201578395071,
        0.6615962513449365, 0.659828413873094, 0.6593581001080975
    ])


def share_consum_goverments_subhealth_and_social_work(x):
    """
    share consum goverments subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.4588957626796008, 0.44707649924014653, 0.43785031864326934, 0.43967250874908564,
        0.44259141677895925, 0.44421483587784627, 0.440685782407788, 0.43726784569342825,
        0.4372577649745116, 0.43796753502845864, 0.4378164095136612, 0.43992755579739684,
        0.4425537217710332, 0.443198533634356, 0.4408943873395209
    ])


def share_consum_goverments_subother_community_social_and_persona_services(x):
    """
    share consum goverments subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.13593187360842532, 0.0910574921371381, 0.0873395017587308, 0.08692860129270708,
        0.08999187621790489, 0.090740783333636, 0.08823471238711048, 0.0863032736828504,
        0.08522731060173952, 0.08453682210016202, 0.08167532299694542, 0.0807001909529624,
        0.08463448174674901, 0.0850306568023039, 0.08931954847715241
    ])


def share_consum_goverments_subprivate_households_with_employed_persons(x):
    """
    share consum goverments subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0010228200228570537, 0.0011290416169956153, 0.0009518818302216298, 0.0009739207509484318,
        0.003226765298868626, 0.0045618940852030915, 0.006250676296558422, 0.007856379237276827,
        0.009536129903780428, 0.009469554208776931, 0.01007163264373241, 0.010290112726275327,
        0.010477541610775753, 0.009767631985404314, 0.012031510285201963
    ])


def share_inventories_subagriculture_hunting_forestry_and_fishing(x):
    """
    share inventories subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.039806446548852466, 0.03461669019581704, 0.041942233336400334, 0.009276799405214435,
        0.016075123961072323, 0.024217754183502624, 0.032446087618470246, 0.013212424725283112,
        0.03376795965480616, 0.03382604411506391, 0.018401487557593552, 0.01982139598204595,
        0.018904899276004725, 0.015809658030545876, 0.010160482589476171
    ])


def share_inventories_submining_and_quarrying(x):
    """
    share inventories submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.14492615777706683, 0.09426322868948797, 0.1452938030085441, -0.010522918740413148,
        0.015323039580696841, 0.062066986325035677, 0.04610341694033021, -0.03526634160335441,
        0.04212319356415536, 0.022861810664424075, 0.04151711701584518, 0.10319681541565758,
        0.021374515787298973, 0.0025377049344237237, 0.020923144714220698
    ])


def share_inventories_subfood_beverages_and_tobacco(x):
    """
    share inventories subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.015018588928445093, 0.011229125465680464, 0.014950252179427848, 0.0015067456009418114,
        0.008219784022741013, 0.013112354704970405, 0.015008429524128383, 0.00496232604036188,
        0.010115436166276024, 0.014090630265515118, 0.010472018505405785, 0.011288293056324823,
        0.010285984899975899, 0.011670533759363205, 0.003484397941663512
    ])


def share_inventories_subtextiles_and_textile_products(x):
    """
    share inventories subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03700800887524616, 0.025922783205493752, 0.025535222531598935, 0.0008444416652216993,
        0.020417391981624455, 0.028992295980142094, 0.025311362536731913, 0.007727580121531834,
        0.002825711516221771, 0.012079303124812086, 0.005519571894805784, 0.013120023882477527,
        0.005518243280871071, 0.007934972086945095, -0.01617110107273339
    ])


def share_inventories_subleather_leather_and_footwear(x):
    """
    share inventories subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03913219959356502, 0.02741801605303332, 0.03308742478636399, 0.004191804519572037,
        0.014882491116945982, 0.019108017204367233, 0.024126896069485666, -0.0019191225645622243,
        -0.0011903123376724623, 0.000523595101187629, 0.0014322759731371915, 0.0007313941121081056,
        -0.0067765566832306955, -0.00849822559971483, -0.02726512256395504
    ])


def share_inventories_subwood_and_products_of_woood_and_cork(x):
    """
    share inventories subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.07544310598978189, 0.05099801463435518, 0.060516673656964955, -0.0006817607991020593,
        0.03308155567618923, 0.04742896859549011, 0.011396171400018095, 0.0012185991078174092,
        -0.0032429105557199417, 0.014559681116070878, 0.010593103068180825, -0.012816239554372989,
        -0.07238100443548619, -0.10795313752850011, -0.1616632516649013
    ])


def share_inventories_subpulp_paper_printing_and_publishing(x):
    """
    share inventories subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.014348233928113558, 0.0017209535478593398, 0.011116633244823542, -0.019023876579578658,
        -0.013690682653597472, 0.01797167429902852, 0.009140803917325045, -0.03402770659174538,
        -0.03710591437128926, -0.03741405276295467, -0.03459212455574808, -0.04266723275308837,
        -0.05746675136230981, -0.06279328074894397, -0.08641675950037707
    ])


def share_inventories_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    share inventories subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.009662414285604149, 0.022397163766748884, 0.029657526659950657, -0.009844842065857869,
        -0.0012157576444566558, 0.017875454252156166, 0.017945093269225307, -0.016425040672843304,
        0.0015011190602734984, 0.012571300154402403, 0.024019248679712973, 0.01245074623470567,
        -0.011533595218422517, 0.008062090060610764, -0.01979745228789293
    ])


def share_inventories_subchemicals_and_chemical_products(x):
    """
    share inventories subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.05693764085223191, 0.05005938101189353, 0.06341341306758376, 0.020305971801720977,
        0.045376681578992654, 0.04901889821081202, 0.04136089819946551, 0.009874150034516383,
        0.013029559844310389, 0.031023231245148577, 0.027035191107908304, 0.03542343907150098,
        0.03570302269336227, 0.014621691554917347, 0.002697712399088707
    ])


def share_inventories_subrubber_and_plastics(x):
    """
    share inventories subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0778141986127367, 0.05155601037400166, 0.05613935712413145, 0.01993782490332162,
        0.014493329733790127, 0.0326650525055877, 0.022302165133758694, 0.007727464783111726,
        0.008855703431832364, 0.02600911316279692, 0.00811772737440893, 0.02960706717970953,
        0.0038264700381658127, 0.005101709762047021, -0.05411230890422361
    ])


def share_inventories_subother_non_metalic_mineral(x):
    """
    share inventories subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.12020984415955097, 0.09206994019660208, 0.10746874545566801, 0.029834516830541346,
        0.0323522570370145, 0.04737016753690253, 0.0459722775727321, -0.06198036957638333,
        -0.028632375936390057, -0.0045939428954693065, 0.007506379019318679, 0.0529179511250499,
        0.040814956805921516, 0.023955408358392986, -0.049341810520510544
    ])


def share_inventories_subbasic_metals_and_fabricated_metal(x):
    """
    share inventories subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03598798077997525, 0.0031870963507300964, 0.009123680856206623, -0.05776728641089819,
        -0.063023093491151, 0.028766376131321228, -0.0012881757231380966, -0.05179315666335895,
        -0.02408351048492784, 0.007466442421217998, -7.513578161285693e-05, 0.039232899341370225,
        -0.04691660213435318, -0.05954158611418101, -0.1578288206210243
    ])


def share_inventories_submachinery_nec(x):
    """
    share inventories submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.01812631038583679, 0.010574262541391523, 0.00696599493820793, -0.002172547925658203,
        0.0016757710330077133, 0.018576378785219894, 0.013271560872378692, -0.011732858194081435,
        0.01588751699230089, 0.009124612525020096, 0.006060567821491422, 0.014060643446260344,
        0.006903952206265926, 0.004552560360692592, -0.00599537770581172
    ])


def share_inventories_subelectrical_and_optical_equipment(x):
    """
    share inventories subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03195972862002789, 0.023168508723542067, 0.028354704013657167, 0.015847142402905943,
        0.02265004660576361, 0.04296331477731733, 0.0016590409136539952, -0.00859960846267911,
        0.011252897017189633, 0.03358385692165521, 0.02280449294652064, 0.029076321142639805,
        0.01526034762999966, 0.013992211001478148, -0.009340748606651865
    ])


def share_inventories_subtransport_equipment(x):
    """
    share inventories subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.01872732946069781, 0.008363827805679104, 0.010886046103550569, 0.0032035392047634697,
        0.021106266218248022, 0.011928490546162383, 0.021600092686336735, 0.011439507854791203,
        0.012926172570669917, 0.010692503650363025, 0.0198993047564401, 0.02190095810026589,
        0.015357859751103917, 0.006712683942431885, -0.006722001475231583
    ])


def share_inventories_submanufacturing_nec_recycling(x):
    """
    share inventories submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.02326623640267202, 0.01857734778416911, 0.022404736598060047, 0.007320140860101643,
        0.017631024963925525, 0.03481081306590101, 0.030668103927010547, 0.014946590623808492,
        0.013840143578630438, 0.021493995531179255, 0.02802975748870618, 0.022896789367939235,
        0.007827848982984867, 0.01601343571723777, -0.006185433819430002
    ])


def share_inventories_subelectricity_gas_and_water_supply(x):
    """
    share inventories subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0009962481163932318, 0.00042413317519049144, 0.0013558288107230248,
        0.0005431237710083078, 0.0007282829655163663, 0.0006328974362719102,
        -0.0002694355176032213, -0.0010487328296012264, -0.00033330844580074255,
        -0.0016884595669077243, -0.0005938883617179873, 0.00023858153100637895,
        -0.0009011798421525196, -0.00120160160343804, -0.0021056969906526287
    ])


def share_inventories_subconstruction(x):
    """
    share inventories subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.00020072745436100465, -0.00014896785893889322, 0.00032862049346872265,
        0.00012345146463194415, 0.0004437915830468549, 0.0002729355150054159,
        0.00034811231920423496, 2.6341616277743705e-05, -5.133599583922819e-05,
        -0.00020292513825069224, 0.00020208833289227384, 0.0003465351793535172,
        0.0009605838860180702, 0.0011973075229034687, 0.0010350447943260576
    ])


def share_inventories_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    share inventories subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.004810299878730776, 0.0023929746013658833, 0.0031142324982446606, 0.0015004391254643849,
        0.0028785617029874824, 0.006521608339560344, 0.006391678364164279, -0.000475560711611828,
        0.001976871260373391, 0.004240190894063026, 0.0031622262651984615, 0.005417379052287837,
        0.004613781515618692, 0.004953710498287375, -0.004117596796533778
    ])


def share_inventories_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    share inventories subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.021671426326427642, 0.015368793493944032, 0.0176144036678834, 0.0076430350349353925,
        0.010562158065515002, 0.016008740199098148, 0.007243281960070458, 0.0009669664677655769,
        0.0035568537305010403, 0.0022383440684109767, 0.003896461990106976, 0.005195678507696852,
        0.003923266504628535, 0.008047525531025905, -0.0021458011259934246
    ])


def share_inventories_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    share inventories subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.008215506595961384, 0.004783819038525881, 0.006436223327803524, 0.0026383681892997285,
        0.005164286657848896, 0.010710865249582027, 0.00984858735790179, 0.0015008944423482583,
        0.003897921771497273, 0.006130297658989541, 0.004498001375669275, 0.006691703056267292,
        0.0054425175554355726, 0.004830282847275033, -0.000741128990872675
    ])


def share_inventories_subhotels_and_restaurants(x):
    """
    share inventories subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        -0.00014589121778786507, -0.00020737488320897337, 0.0001446685483487721,
        -0.0003572996584286368, 0.0006549819387167176, 0.0020955879814969908, 0.002911897359610583,
        0.0006233249367963001, 0.0007016828988724827, 0.0005359206216678118, 0.0004919458570455678,
        0.0006475599403373204, 0.0002856755321237399, 0.00023288616823906658, 0.0004124112092339731
    ])


def share_inventories_subinland_transport(x):
    """
    share inventories subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.01399853187795113, 0.010176833024630351, 0.012536329641196792, 0.00430214531254968,
        0.005348367920419633, 0.006554338122717555, 0.005378382435539208, 0.0019515962711634627,
        0.0027197909056981668, 0.004730984658814206, 0.003957970135238292, 0.006311517029460731,
        0.003440626216147738, 0.0028761249209255206, 0.0002411375900309864
    ])


def share_inventories_subwater_transport(x):
    """
    share inventories subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.04320323072050123, 0.03445012707955338, 0.033535203070921914, 0.013131889748329966,
        0.027919145462781503, 0.041024487927804686, 0.08466919183155805, 0.00307740567425285,
        0.003488829237790001, 0.0029639888705386475, 0.003599625678616155, 0.05066574298576478,
        0.0014308799030149516, 0.08611950649678217, 0.08684434873201745
    ])


def share_inventories_subair_transport(x):
    """
    share inventories subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0031032243349323895, 0.0021245645834738853, 0.0025850705351703157, 0.0008091388791002279,
        0.0023436047159025035, 0.004759184665233702, 0.010872926861949048, -0.00044423394440304637,
        0.0012555125044372616, 0.0004702718420441391, 0.001191508687522141, 0.00350381933999573,
        0.0010309466286302909, 0.002977872931688678, 0.0005918793793829298
    ])


def share_inventories_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    share inventories subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.02284442934144017, 0.015631051792266138, 0.02986296802524229, 0.012259844509753592,
        0.012338783661043004, 0.011534383792643545, 0.030505284447352746, 0.004886689383534703,
        0.004357898672331982, 0.005428457355999516, 0.005817028918166772, 0.00976790454953955,
        0.005881481394028688, 0.010246379983008607, 0.0019734244893533683
    ])


def share_inventories_subpost_and_telecommunications(x):
    """
    share inventories subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0011057850769560624, 0.0009488551439241703, 0.0012396105428931219, 0.0007444440848184912,
        0.0014685834763999532, 0.0020441730699772512, 0.0032325119339692335, -0.005068250485275013,
        0.00043111420964910284, 0.0009169560942092407, 0.000570352331915007, 0.0007087983726690691,
        0.0005448397305536379, 0.0009565343787786686, 0.000645581668543594
    ])


def share_inventories_subfinancial_intermedation(x):
    """
    share inventories subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.00018628940902411898, 9.11231272700768e-05, 7.542788209146378e-05,
        -1.4400616429060148e-05, 8.229749543927034e-05, 0.0003937912440804006,
        0.0009093943635503735, -0.007524568226264509, -0.004274426163508445, 8.49992603244964e-05,
        9.833482351623146e-05, 0.00013797126261373458, 8.384317855566425e-05,
        0.0006442841441413284, 0.0012264529642908699
    ])


def share_inventories_subreal_estate_activities(x):
    """
    share inventories subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3.55572466866178e-05, 2.91907526576806e-05, 1.9879281529659025e-05, 1.4105622407877741e-05,
        8.646807700213472e-05, 2.161401447580686e-05, 7.110088024127258e-06, 7.479695881646118e-06,
        2.0296672084406522e-06, 9.156903678863279e-06, 2.6108742903649806e-05,
        3.0340352036303263e-05, 2.039824756142603e-05, 7.322588921311033e-05,
        -2.8778440691593814e-05
    ])


def share_inventories_subrenting_od_meq_and_other_business_activities(x):
    """
    share inventories subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0011964287316454426, 0.000831602174777396, 0.0033847710947051517, 0.0005761788915349635,
        0.0031079738205755606, 0.0021995063936603304, 0.0022211587023244217, 0.000463289136776625,
        0.003827049793333797, 0.0032723048332935715, 0.00282669028048174, 0.0033238445248807004,
        0.0019278144414444816, 0.01864576873493577, 0.0074378011898418245
    ])


def share_inventories_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    share inventories subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        -0.006668793979221436, -0.006890673086698083, -0.005679982938310693,
        -0.0072323360613766275, -0.006114828678354021, -0.005356116532156387,
        -0.0038007332944622442, -0.005674782512487631, -0.004853486698943161,
        -0.004676214705836499, -0.004558338326465761, -0.004537527544218771,
        -0.0053082216326086635, -0.005544836892740801, -0.003922579936947399
    ])


def share_inventories_subeducation(x):
    """
    share inventories subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.144403809132422e-05, 1.8730226585082882e-05, 1.3231663407411775e-05,
        4.799534989921149e-06, 3.0446684471332545e-05, 1.098827169556429e-05,
        2.5728856619887022e-05, 1.3380636827243842e-05, 4.505158402132235e-05,
        2.248247898388548e-06, 8.725195270895369e-05, -2.615975302391741e-05, 8.26970101974835e-06,
        1.5833949800231638e-05, 4.7541740478130375e-05
    ])


def share_inventories_subhealth_and_social_work(x):
    """
    share inventories subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        5.356250958867206e-06, 8.428876000649878e-07, 3.87479673142649e-06, 4.426989397210959e-06,
        1.484503418608409e-05, 2.0269244774470538e-05, 1.2951043297292982e-05,
        -3.1727632790426594e-05, 1.9634547384056006e-05, 1.5997263037852878e-05,
        1.0177684924305919e-05, 1.965940194824921e-05, 1.4489386545164095e-05,
        2.3448960211648786e-05, -9.502931854099198e-06
    ])


def share_inventories_subother_community_social_and_persona_services(x):
    """
    share inventories subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        -0.0090127726393938, -0.010629458350807685, -0.008848671736319593, -0.013448818323945873,
        -0.008946039163755016, -0.004793431904417115, -0.0016249396890407504,
        -0.0022020738024543162, -0.0018956770408739025, -0.001785715673228679,
        -0.0018879388774512668, -0.0017953987374448959, -0.0024802205711734492,
        -0.003448648466737081, -0.0020721433092937003
    ])


def share_inventories_subprivate_households_with_employed_persons(x):
    """
    share inventories subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        7.853351243135922e-05, 9.814582614953874e-05, 9.588248461668407e-06,
        -7.821361593256881e-05, -2.411516128394616e-05, -3.631893686050125e-06,
        -5.08923087015618e-05, -0.00022539143199814182, -0.0002681430771119126,
        6.398985647852799e-05, 4.7633349046714305e-05, 0.0015237584902465407, 0.000990345581418567,
        0.0009393292444326495, 0.0008607609964860263
    ])


def share_consum_goverments_next_step_subagriculture_hunting_forestry_and_fishing(x):
    """
    share consum goverments next step subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0067709613778314465, 0.006395238231716023, 0.007562221479560522,
            0.006909511984020509, 0.007982988184916011, 0.008446717428753593, 0.009326405167881661,
            0.009029494951501007, 0.009305637069275475, 0.009311394291421764, 0.009911989953154088,
            0.010233756778334888, 0.008994169837883667, 0.00869869421963916
        ])


def share_consum_goverments_next_step_submining_and_quarrying(x):
    """
    share consum goverments next step submining and quarrying



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.00436164392290795, 0.003995603763685553, 0.004066047799643348, 0.006393419006564868,
            0.006003153311605073, 0.0042157315738525596, 0.005643824953173819,
            0.005273239069785034, 0.005338694073945094, 0.005231106875192582, 0.004151188915356074,
            0.004717994395060901, 0.00450208701463089, 0.004934708656353205
        ])


def share_consum_goverments_next_step_subfood_beverages_and_tobacco(x):
    """
    share consum goverments next step subfood beverages and tobacco



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0038220671963571573, 0.002903073070627261, 0.002936792709517417,
            0.003142168948714033, 0.0031340511473669625, 0.0030303441468097964,
            0.003102005746783019, 0.003076968042145073, 0.002915823853670982, 0.002931906554520569,
            0.002970467773012117, 0.003045614006493215, 0.002961515281763346, 0.002885066309883363
        ])


def share_consum_goverments_next_step_subtextiles_and_textile_products(x):
    """
    share consum goverments next step subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.003551578898993538, 0.00295957655229884, 0.0026490453856551233, 0.0028527421747088517,
        0.0025817118313251836, 0.0027787410874748823, 0.0028558425631741735, 0.003004220437100366,
        0.0030660841863479906, 0.0034604581246728532, 0.00402674114226547, 0.004420561138285466,
        0.004407476902887508, 0.005253312524302093
    ])


def share_consum_goverments_next_step_subleather_leather_and_footwear(x):
    """
    share consum goverments next step subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0015559542279136721, 0.0012399353066103794, 0.0013267840090444238, 0.00133344367825553,
        0.0012313162901100574, 0.0011201067862366928, 0.0008200600923468703, 0.0007351843303268899,
        0.0007872038350995223, 0.0008524970828855757, 0.000980302019330771, 0.0012534804090331476,
        0.0013308059227996672, 0.001514645887241861
    ])


def share_consum_goverments_next_step_subwood_and_products_of_woood_and_cork(x):
    """
    share consum goverments next step subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.009005435491861913, 0.00784410468158404, 0.010385380453511185, 0.005983313227598484,
        0.0029442159037194747, 0.003191210291931943, 0.0038803930252921484, 0.0033353083568353945,
        0.0031038702125085094, 0.002934685814185623, 0.0032009994283761284, 0.0031885473327917984,
        0.003374415685759153, 0.003641151621250122
    ])


def share_consum_goverments_next_step_subpulp_paper_printing_and_publishing(x):
    """
    share consum goverments next step subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.012152022471383788, 0.010944306361792914, 0.013144143333725558, 0.01481962044356459,
            0.014280346332352903, 0.014584228557471577, 0.01813378209427802, 0.01800422812064625,
            0.017428493050279235, 0.01703528662283678, 0.018253358153044425, 0.01977514458023117,
            0.016156351225622194, 0.016038861285036368
        ])


def share_consum_goverments_next_step_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    share consum goverments next step subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.014073012258205416, 0.014138264180255291, 0.01638462176078289, 0.012559974344452331,
            0.009745231174255206, 0.008722834180249866, 0.009133945504476285, 0.007599833303872037,
            0.006750850405801407, 0.005867347651754397, 0.006685369003406569, 0.007433579067682456,
            0.006607524613390546, 0.007784103038001168
        ])


def share_consum_goverments_next_step_subchemicals_and_chemical_products(x):
    """
    share consum goverments next step subchemicals and chemical products



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.1019113905016282, 0.0973261077437338, 0.10451610930994258, 0.1038905105421696,
            0.10684729730821506, 0.11441273827060026, 0.11710461296677933, 0.11802421308716918,
            0.1131680314380918, 0.11811035287289917, 0.1159991266683548, 0.12291936960693232,
            0.13104795855975498, 0.13239560556296268
        ])


def share_consum_goverments_next_step_subrubber_and_plastics(x):
    """
    share consum goverments next step subrubber and plastics



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007733117166691025, 0.007382076353556275, 0.008098756846008068, 0.008121760071879654,
            0.006928724883784163, 0.006611896916921303, 0.006412599503225718, 0.00627859918453306,
            0.006463852038577615, 0.006987327120633945, 0.007095522534506072, 0.007271046894131369,
            0.006823697974194504, 0.0071450764967817045
        ])


def share_consum_goverments_next_step_subother_non_metalic_mineral(x):
    """
    share consum goverments next step subother non metalic mineral



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.005601972248454674, 0.004984833707272183, 0.0050045677027149025,
            0.005220994619862051, 0.004906031702269911, 0.0046695726368900506,
            0.004765867407184146, 0.004575710980134348, 0.004925408480648355, 0.005451801457053173,
            0.005476256518325244, 0.005940164753101489, 0.006012657637842877, 0.006158849250484725
        ])


def share_consum_goverments_next_step_subbasic_metals_and_fabricated_metal(x):
    """
    share consum goverments next step subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.003695607624002559, 0.003264025079857985, 0.003456144989750174, 0.003909675072085389,
            0.003567486126274761, 0.004087484008773003, 0.004580826890509096, 0.003954657469767477,
            0.003832560933406138, 0.003872948283557392, 0.0037354425586724055,
            0.004369197638847987, 0.004504712881791057, 0.005212766425513586
        ])


def share_consum_goverments_next_step_submachinery_nec(x):
    """
    share consum goverments next step submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0036617822542566983, 0.003605389168536691, 0.003975012421065428, 0.003576070848771421,
        0.0024004574567129543, 0.002133422389799059, 0.0020732528087047206, 0.0015195162676146234,
        0.0016070917766987168, 0.0017110936542217888, 0.0017042538525656047, 0.0018089274613794315,
        0.00179493990200113, 0.0020036796744404423
    ])


def share_consum_goverments_next_step_subelectrical_and_optical_equipment(x):
    """
    share consum goverments next step subelectrical and optical equipment



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.006689280053562139, 0.006260917348372466, 0.006383402577586877, 0.007626339783897834,
            0.008952172345135323, 0.009938402470472538, 0.010584245313257168, 0.010485426607225824,
            0.009528021492371893, 0.009159007241327075, 0.009390892943976928, 0.009368214117085202,
            0.008491380955383702, 0.010526372953268005
        ])


def share_consum_goverments_next_step_subtransport_equipment(x):
    """
    share consum goverments next step subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0030134399979940317, 0.0030013782829294688, 0.003324582114787668, 0.0033937364928424093,
        0.004179287165548907, 0.004060256365656964, 0.004599388270976094, 0.004521827080112946,
        0.0039045057195699682, 0.003377303061102904, 0.0032156831100202668, 0.002999720037553041,
        0.003025284132570713, 0.0031225730516014757
    ])


def share_consum_goverments_next_step_submanufacturing_nec_recycling(x):
    """
    share consum goverments next step submanufacturing nec recycling



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02145589854505419, 0.0214355818274276, 0.02514402773966533, 0.01774772016365786,
            0.013419020602064787, 0.012203537170182726, 0.011759010701610579, 0.010195116719573927,
            0.010562251657894164, 0.010833741123216334, 0.01030096052750391, 0.010593246762629493,
            0.011621782561596393, 0.012719313855909485
        ])


def share_consum_goverments_next_step_subelectricity_gas_and_water_supply(x):
    """
    share consum goverments next step subelectricity gas and water supply



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.014062257866705289, 0.013761686141498856, 0.014297671627160625, 0.012947249209290532,
            0.013116308794034108, 0.012724684366983808, 0.01399302698870909, 0.013964725368533456,
            0.014932092232165415, 0.016923049048231383, 0.020237318698904688, 0.020307088408552488,
            0.018222111231047673, 0.019718462441084224
        ])


def share_consum_goverments_next_step_subconstruction(x):
    """
    share consum goverments next step subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0037926289235951103, 0.003584887454493617, 0.004030136291386803, 0.0043782604871680005,
        0.004424182707116258, 0.004527167347521452, 0.004951394442164713, 0.0046428441481454205,
        0.003398133507831463, 0.0023546145866514584, 0.001689633412741827, 0.0016539914075433919,
        0.0017930949205848523, 0.0020141898925777795
    ])


def share_consum_goverments_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    share consum goverments next step subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.005403324383601953, 0.005555193258671485, 0.005806921261116362, 0.0055187734847195,
            0.0060804999283362755, 0.006576770190355463, 0.006599236892085225,
            0.006951525047583767, 0.006930155684757487, 0.0071569467110036466,
            0.006965090647827174, 0.006885708406717167, 0.006969574494336234, 0.008003455543375147
        ])


def share_consum_goverments_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    share consum goverments next step subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.01006648267750522, 0.009307515695653029, 0.009444958125729505, 0.009364679486856759,
            0.009765600552568655, 0.010207317570209486, 0.010771962374229452, 0.01067527004648461,
            0.010462899400149748, 0.010587576110538116, 0.0108347645089915, 0.01077759596108046,
            0.010732709364063026, 0.012048869135356076
        ])


def share_consum_goverments_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    share consum goverments next step subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.006598553542763822, 0.006508686887531563, 0.006705691997046858, 0.007025347276833513,
            0.00720794339648111, 0.007317693553382741, 0.007707318698817354, 0.007486817205849609,
            0.007496077432961913, 0.007788715871380901, 0.007790913253695477, 0.00796523132976134,
            0.008075132023729639, 0.009141731015561386
        ])


def share_consum_goverments_next_step_subhotels_and_restaurants(x):
    """
    share consum goverments next step subhotels and restaurants



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.009986641171961503, 0.004972707818870327, 0.005180644227287415, 0.003157808005666468,
            0.0026105687179988736, 0.0023619220258709814, 0.0022802596097973695,
            0.002236912343894969, 0.0032285975612425005, 0.004006717191046085,
            0.004771145868031531, 0.005057546903846365, 0.004275366439841901, 0.004318083939434814
        ])


def share_consum_goverments_next_step_subinland_transport(x):
    """
    share consum goverments next step subinland transport



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.020988629145090622, 0.020208389033241396, 0.024190375963752966, 0.028979965613103086,
            0.0340573400355154, 0.03787454338990796, 0.038778817683728796, 0.042856040285980994,
            0.04333737779943819, 0.040877931695314014, 0.040017654428911954, 0.044561875720359245,
            0.042253292012375945, 0.04548291871340436
        ])


def share_consum_goverments_next_step_subwater_transport(x):
    """
    share consum goverments next step subwater transport



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007670674964602831, 0.006643243068732121, 0.0070420634278164475,
            0.005625692184953877, 0.005825483070381239, 0.005755526565640597,
            0.0059064747947359724, 0.031040299562048543, 0.0444328959360957, 0.045343068051017205,
            0.0419767170469466, 0.05198184793293863, 0.050442763300947255, 0.05576891724508192
        ])


def share_consum_goverments_next_step_subair_transport(x):
    """
    share consum goverments next step subair transport



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0056558638910256965, 0.003568218719757792, 0.003452988698190144,
            0.003113214535012367, 0.002600142907132616, 0.00352760744719066, 0.0035150489272458216,
            0.008303301417705423, 0.01003440101939009, 0.007348632019004971, 0.007740245562564343,
            0.009810656999234713, 0.009168958102788652, 0.00981130620506604
        ])


def share_consum_goverments_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    share consum goverments next step subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.09901239662412917, 0.09351013009433082, 0.09307715377473842, 0.090464559595115,
            0.08605821054032661, 0.10652486260006906, 0.11145975472969767, 0.11357295739188283,
            0.11017723359651423, 0.10491172806150008, 0.1011831510665581, 0.09842241225860136,
            0.09838464480853919, 0.09731960940160281
        ])


def share_consum_goverments_next_step_subpost_and_telecommunications(x):
    """
    share consum goverments next step subpost and telecommunications



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007218033564359982, 0.006671966436504588, 0.00755011709325932, 0.007798984111468217,
            0.006194892527289748, 0.005508562494627751, 0.00575293539502172, 0.005392135225406816,
            0.004821274684917958, 0.005148559804682313, 0.00553343108337951, 0.006297209998691717,
            0.006491788496943986, 0.007810889098747097
        ])


def share_consum_goverments_next_step_subfinancial_intermedation(x):
    """
    share consum goverments next step subfinancial intermedation



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.027581273496790935, 0.025235830692599766, 0.02508840686447435, 0.024038702335690176,
            0.02474875619024354, 0.031203030082480394, 0.03890041017856948, 0.04298777082414324,
            0.04647391196264396, 0.04820648818054139, 0.054020887398685435, 0.05660365990915811,
            0.057191672612287296, 0.05349723889956909
        ])


def share_consum_goverments_next_step_subreal_estate_activities(x):
    """
    share consum goverments next step subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.00480747520979442, 0.004356879732159147, 0.005051056791957753, 0.0042817328034103035,
        0.003082759572882315, 0.002882345202542986, 0.0028984962506404615, 0.0027998697728837665,
        0.0029336909071597957, 0.0029228584018284724, 0.0032377155699126986, 0.0032471376757725566,
        0.003313038453281683, 0.0033056974047413435
    ])


def share_consum_goverments_next_step_subrenting_od_meq_and_other_business_activities(x):
    """
    share consum goverments next step subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.12766213970374193, 0.128477992110974, 0.13977631646313407, 0.14717767633342763,
            0.15478196524040605, 0.16957094795516728, 0.17978959576830966, 0.1755571954925755,
            0.16458717780184334, 0.1432232593236837, 0.13416821175274135, 0.14146388986144026,
            0.13442398718655846, 0.13699391236524086
        ])


def share_consum_goverments_next_step_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    share consum goverments next step subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.9019740353263537, 0.9055209153005452, 0.9070353979129336, 0.9101561542351737,
            0.9071215692757418, 0.9039291693459441, 0.9009660123926903, 0.8991346577949707,
            0.8948821888313753, 0.8935084759144025, 0.8922590472793164, 0.8935680286075393,
            0.895349946046101, 0.892671897122667
        ])


def share_consum_goverments_next_step_subeducation(x):
    """
    share consum goverments next step subeducation



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.7040856404423688, 0.6993589354235593, 0.698581612534672, 0.6987685915889159,
            0.6958775202480544, 0.6893004419028962, 0.6836379033975156, 0.6765593042142081,
            0.6662693916312913, 0.6630464757694154, 0.660201578395071, 0.6615962513449365,
            0.659828413873094, 0.6593581001080975
        ])


def share_consum_goverments_next_step_subhealth_and_social_work(x):
    """
    share consum goverments next step subhealth and social work



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.44707649924014653, 0.43785031864326934, 0.43967250874908564, 0.44259141677895925,
            0.44421483587784627, 0.440685782407788, 0.43726784569342825, 0.4372577649745116,
            0.43796753502845864, 0.4378164095136612, 0.43992755579739684, 0.4425537217710332,
            0.443198533634356, 0.4408943873395209
        ])


def share_consum_goverments_next_step_subother_community_social_and_persona_services(x):
    """
    share consum goverments next step subother community social and persona services



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0910574921371381, 0.0873395017587308, 0.08692860129270708, 0.08999187621790489,
            0.090740783333636, 0.08823471238711048, 0.0863032736828504, 0.08522731060173952,
            0.08453682210016202, 0.08167532299694542, 0.0807001909529624, 0.08463448174674901,
            0.0850306568023039, 0.08931954847715241
        ])


def share_consum_goverments_next_step_subprivate_households_with_employed_persons(x):
    """
    share consum goverments next step subprivate households with employed persons



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0011290416169956153, 0.0009518818302216298, 0.0009739207509484318,
            0.003226765298868626, 0.0045618940852030915, 0.006250676296558422,
            0.007856379237276827, 0.009536129903780428, 0.009469554208776931, 0.01007163264373241,
            0.010290112726275327, 0.010477541610775753, 0.009767631985404314, 0.012031510285201963
        ])


def historic_gfcf_subagriculture_hunting_forestry_and_fishing(x):
    """
    historic gfcf subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        35732.832, 37614.691, 40803.641, 41916.227, 38520.117, 38440.754, 36423.711, 38511.609,
        37743.941, 40307.848, 42430.488, 40835.578, 39981.016, 41155.035, 41396.871
    ])


def historic_gfcf_submining_and_quarrying(x):
    """
    historic gfcf submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        49168.223, 50558.738, 59227.895, 57727.891, 57437.492, 57307.668, 65567.406, 67555.18,
        59670.992, 70731.68, 87666.406, 107134.18, 120398.63, 130683.09, 87922.734
    ])


def historic_gfcf_subfood_beverages_and_tobacco(x):
    """
    historic gfcf subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3153.3279, 3405.948, 3856.0354, 3880.1582, 3853.2202, 3988.8564, 3816.6501, 3554.2256,
        3559.5305, 3899.083, 4242.4302, 4362.2729, 4699.4136, 4752.7988, 4192.0625
    ])


def historic_gfcf_subtextiles_and_textile_products(x):
    """
    historic gfcf subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        9805.4414, 10735.85, 11137.633, 10800.096, 10876.121, 11490.052, 11577.753, 12166.8,
        11730.841, 11857.032, 13205.372, 12958.692, 11822.783, 12343.811, 10536.79
    ])


def historic_gfcf_subleather_leather_and_footwear(x):
    """
    historic gfcf subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        654.27881, 757.39868, 813.76819, 817.67688, 752.20032, 643.75079, 576.78583, 519.88055,
        542.40131, 604.32587, 622.45618, 615.94867, 678.13239, 644.13391, 604.7887
    ])


def historic_gfcf_subwood_and_products_of_woood_and_cork(x):
    """
    historic gfcf subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        26489.537, 26581.762, 28397.006, 27623.027, 26920.006, 25842.48, 23475.318, 21862.025,
        20325.885, 18978.408, 20301.361, 20253.908, 20320.295, 18429.826, 15459.762
    ])


def historic_gfcf_subpulp_paper_printing_and_publishing(x):
    """
    historic gfcf subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        36798.738, 37032.848, 39961.035, 41669.242, 46405.641, 49903.871, 51752.203, 50975.605,
        50689.711, 54906.703, 56410.535, 57824.684, 59567.02, 58348.961, 52958.492
    ])


def historic_gfcf_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    historic gfcf subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1328.0853, 1227.5522, 1353.3188, 1539.4121, 1476.5179, 1365.4154, 1631.8383, 1701.6514,
        1283.2863, 1285.8644, 1511.526, 1526.3302, 1572.2338, 1773.6797, 1679.7283
    ])


def historic_gfcf_subchemicals_and_chemical_products(x):
    """
    historic gfcf subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        10869.645, 12038.465, 13229.726, 13637.507, 12738.396, 11845.654, 11435.997, 10917.669,
        11387.182, 10602.439, 11093.072, 11520.979, 14185.478, 14120.531, 12864.762
    ])


def historic_gfcf_subrubber_and_plastics(x):
    """
    historic gfcf subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        12706.039, 14192.511, 14736.633, 15018.491, 15287.824, 14431.255, 14019.702, 14085.24,
        13984.364, 14517.224, 15653.112, 16725.594, 18017.344, 16867.807, 15186.176
    ])


def historic_gfcf_subother_non_metalic_mineral(x):
    """
    historic gfcf subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        5833.2832, 6141.4766, 6658.7974, 6528.9243, 6939.7749, 6940.7358, 6785.4204, 6367.7344,
        6292.3218, 6195.3579, 6629.5493, 6526.0938, 7284.7769, 6779.4404, 5801.4595
    ])


def historic_gfcf_subbasic_metals_and_fabricated_metal(x):
    """
    historic gfcf subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        151450.72, 168036.17, 175682.53, 172156.44, 176539.11, 174882.11, 172961.69, 172230.45,
        172920.44, 163433.64, 171458.94, 180851.8, 191195.25, 189079.64, 169590.94
    ])


def historic_gfcf_submachinery_nec(x):
    """
    historic gfcf submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        591070.69, 624630.06, 650666.31, 661738.56, 649393.38, 705485.75, 686770.25, 676821.63,
        711485.5, 772953.69, 844631.38, 901317.69, 980461.25, 1010400.5, 851105.94
    ])


def historic_gfcf_subelectrical_and_optical_equipment(x):
    """
    historic gfcf subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        592198.44, 655007.69, 741965.0, 772863.31, 844308.38, 974987.0, 968576.31, 909083.63,
        945232.75, 1011340.2, 1107939.9, 1204886.8, 1314391.4, 1354865.9, 1251572.4
    ])


def historic_gfcf_subtransport_equipment(x):
    """
    historic gfcf subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        525742.88, 560220.13, 599235.56, 630740.38, 681251.81, 683685.88, 656420.25, 667179.38,
        713015.38, 768573.88, 828957.69, 900477.81, 995201.56, 985637.94, 849517.56
    ])


def historic_gfcf_submanufacturing_nec_recycling(x):
    """
    historic gfcf submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        85966.852, 93761.969, 99366.813, 103879.14, 107439.22, 111866.84, 110534.02, 111077.36,
        113451.95, 118669.55, 131024.1, 131647.16, 136376.63, 142813.61, 132990.0
    ])


def historic_gfcf_subelectricity_gas_and_water_supply(x):
    """
    historic gfcf subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        9163.3457, 9879.9082, 9898.4268, 10042.622, 10664.927, 11776.001, 12365.013, 12512.324,
        13303.008, 13730.786, 16534.152, 16654.773, 17834.195, 18707.898, 16229.776
    ])


def historic_gfcf_subconstruction(x):
    """
    historic gfcf subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3315735.5, 3380354.5, 3427133.5, 3469614.0, 3552832.5, 3576725.5, 3590266.5, 3589436.8,
        3693404.0, 3835621.0, 4009451.3, 4215311.0, 4373414.0, 4370240.5, 4246355.0
    ])


def historic_gfcf_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic gfcf subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        20114.217, 22476.516, 22885.188, 25268.574, 25426.08, 26552.805, 26028.451, 25794.117,
        25518.584, 27205.137, 29690.898, 30987.475, 33514.207, 33058.988, 27617.543
    ])


def historic_gfcf_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic gfcf subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        272784.38, 295626.84, 311371.94, 327195.78, 341950.22, 355568.5, 355673.53, 358418.03,
        358038.75, 385270.97, 408631.41, 413487.31, 427566.13, 435917.31, 387395.69
    ])


def historic_gfcf_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic gfcf subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        134780.13, 145710.55, 152762.44, 161182.28, 164131.47, 166019.34, 166866.67, 170615.55,
        174059.25, 190358.11, 216744.02, 224155.45, 228615.17, 227170.06, 195714.02
    ])


def historic_gfcf_subhotels_and_restaurants(x):
    """
    historic gfcf subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        6338.0215, 6149.7041, 5875.1206, 4984.3564, 5163.3916, 5408.5967, 5032.4741, 4891.4541,
        4406.771, 4312.0898, 4053.8296, 4194.439, 4313.0947, 4425.0718, 4036.8662
    ])


def historic_gfcf_subinland_transport(x):
    """
    historic gfcf subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        49677.738, 50398.684, 53456.406, 54578.48, 58112.934, 52657.016, 50783.926, 52491.391,
        51862.168, 57095.516, 64329.469, 70106.383, 72946.836, 72193.539, 61845.871
    ])


def historic_gfcf_subwater_transport(x):
    """
    historic gfcf subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2462.5908, 2531.9485, 2810.0728, 4047.5605, 4551.7358, 4567.0215, 4585.7559, 4775.0659,
        4308.2896, 4417.4858, 4655.2271, 4511.1509, 4315.3672, 4326.1084, 4421.8511
    ])


def historic_gfcf_subair_transport(x):
    """
    historic gfcf subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        4389.7974, 4811.0552, 5240.1558, 5297.9785, 5140.0283, 4839.623, 4492.3311, 4079.4124,
        3658.7437, 4157.6909, 4297.7334, 4729.563, 5259.9507, 5141.3306, 4125.6865
    ])


def historic_gfcf_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic gfcf subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        7163.0146, 6735.1499, 6212.1035, 6414.583, 6624.3755, 6811.5542, 6294.2832, 6947.418,
        7070.0679, 7531.814, 8541.0635, 9228.2764, 9921.543, 9895.9375, 8696.0947
    ])


def historic_gfcf_subpost_and_telecommunications(x):
    """
    historic gfcf subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        9400.0742, 10358.336, 12133.059, 13841.907, 17040.002, 20493.102, 22652.508, 22411.086,
        24883.102, 28778.445, 31190.512, 33387.305, 36844.961, 38329.137, 36114.047
    ])


def historic_gfcf_subfinancial_intermedation(x):
    """
    historic gfcf subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3753.4692, 4015.5376, 4656.2686, 4870.4028, 5495.1387, 5409.3198, 5890.9595, 6514.6733,
        6840.2881, 8639.1191, 9947.8135, 11000.363, 12140.073, 11535.349, 9802.5811
    ])


def historic_gfcf_subreal_estate_activities(x):
    """
    historic gfcf subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        105648.82, 112505.58, 121700.46, 130935.03, 138473.86, 140215.89, 148289.28, 163227.39,
        184642.38, 208041.58, 231293.97, 233717.48, 227724.55, 208201.86, 192484.63
    ])


def historic_gfcf_subrenting_od_meq_and_other_business_activities(x):
    """
    historic gfcf subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        322190.16, 361983.34, 384212.78, 429207.44, 473706.81, 517232.19, 537414.25, 532837.94,
        542599.75, 577060.56, 617376.75, 667582.25, 703895.94, 725018.06, 672112.25
    ])


def historic_gfcf_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    historic gfcf subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        16466.713, 8284.8584, 8695.0361, 8756.8301, 9585.9248, 9790.8984, 9414.084, 9308.3301,
        9819.3438, 10995.426, 12019.087, 12665.439, 13487.202, 12984.448, 10894.959
    ])


def historic_gfcf_subeducation(x):
    """
    historic gfcf subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        877.41577, 1190.7976, 1051.7266, 1118.6437, 1211.5232, 1361.8879, 1197.3674, 1297.8099,
        1128.9475, 1177.98, 1207.3063, 1207.5222, 1313.7346, 1230.2579, 1138.8306
    ])


def historic_gfcf_subhealth_and_social_work(x):
    """
    historic gfcf subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2020.5743, 2411.981, 2387.1941, 2317.7742, 2457.7114, 2582.917, 2985.6318, 3204.249,
        3562.0264, 4106.2969, 4548.6128, 4686.6963, 4528.8252, 3930.343, 3988.499
    ])


def historic_gfcf_subother_community_social_and_persona_services(x):
    """
    historic gfcf subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        19066.77, 23366.191, 23901.691, 25248.57, 25473.307, 27629.49, 27269.873, 26582.359,
        26523.111, 28863.715, 30638.781, 32234.186, 33909.887, 33997.836, 28864.693
    ])


def historic_gfcf_subprivate_households_with_employed_persons(x):
    """
    historic gfcf subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        133.84763, 131.45456, 128.82707, 147.96854, 122.28409, 104.01704, 80.335487, 71.17453,
        73.229172, 78.620636, 90.77932, 88.329956, 97.497345, 95.166161, 68.236107
    ])


def historic_hd_subagriculture_hunting_forestry_and_fishing(x):
    """
    historic hd subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        748525.1845, 763287.0495, 770336.1225, 811036.2165, 829790.0459, 822865.7691, 825028.4316,
        854006.3347, 851658.472, 859315.9644, 869780.6132, 881620.6251, 885511.0359, 934301.625,
        964691.9819
    ])


def historic_hd_submining_and_quarrying(x):
    """
    historic hd submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        41789.6873, 40774.55363, 35760.8687, 32802.32296, 37641.00965, 48407.97747, 46807.61795,
        51252.85249, 49636.55263, 49512.87455, 46724.86493, 39361.70374, 34217.52762, 39115.98394,
        34147.69516
    ])


def historic_hd_subfood_beverages_and_tobacco(x):
    """
    historic hd subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1723750.214, 1738706.496, 1774037.996, 1848995.869, 1895713.275, 1931662.187, 1942552.633,
        1958247.896, 1986716.788, 2022834.744, 2089671.199, 2153060.415, 2238091.994, 2246958.729,
        2245656.855
    ])


def historic_hd_subtextiles_and_textile_products(x):
    """
    historic hd subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        470469.2814, 471356.0045, 476964.8114, 493102.4947, 493611.554, 515098.03, 517850.0139,
        531176.2526, 532061.3635, 541778.7234, 563405.2326, 592743.206, 625432.3504, 627023.5263,
        611100.8105
    ])


def historic_hd_subleather_leather_and_footwear(x):
    """
    historic hd subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        97413.51078, 99005.48025, 97752.3784, 99637.54181, 98274.45556, 103223.4987, 104490.9256,
        105493.3023, 106139.4451, 108526.3023, 109548.0365, 114884.4565, 121097.4926, 124995.4693,
        119187.6975
    ])


def historic_hd_subwood_and_products_of_woood_and_cork(x):
    """
    historic hd subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        27311.38121, 26691.7558, 29615.10196, 29971.29581, 28692.07457, 29540.09729, 29526.83953,
        29878.51743, 30112.75414, 30240.47199, 30309.37202, 31382.94995, 33069.44101, 30997.81253,
        29891.58369
    ])


def historic_hd_subpulp_paper_printing_and_publishing(x):
    """
    historic hd subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        218308.7852, 224848.8639, 233135.3592, 239779.917, 245020.8308, 248600.3498, 245907.4753,
        251845.6153, 252932.4609, 260211.7944, 262604.975, 270462.3618, 277128.7161, 278249.174,
        263144.934
    ])


def historic_hd_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    historic hd subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        217859.5983, 227030.6499, 231061.0208, 233139.8992, 242777.5242, 247730.3037, 255008.8399,
        255881.6315, 249517.5902, 256059.9905, 264654.8552, 281863.4531, 283885.0762, 305058.1479,
        314295.3504
    ])


def historic_hd_subchemicals_and_chemical_products(x):
    """
    historic hd subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        329088.216, 338997.7663, 357939.9716, 377533.0605, 391534.1448, 400362.1034, 408700.5194,
        439916.4235, 445918.3201, 458362.0534, 453945.8561, 464577.3553, 485124.7181, 485202.7031,
        493254.1673
    ])


def historic_hd_subrubber_and_plastics(x):
    """
    historic hd subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        91494.36847, 92997.61224, 98746.44195, 101253.0568, 106386.6536, 109284.1565, 108616.9065,
        108496.0829, 109817.8004, 115248.1582, 119022.5462, 122671.6583, 127897.3432, 130009.9874,
        129860.5313
    ])


def historic_hd_subother_non_metalic_mineral(x):
    """
    historic hd subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        50849.6903, 52185.75254, 55570.83701, 57512.95127, 61138.98637, 66091.5223, 66829.09701,
        67078.25726, 61896.61387, 60607.83051, 57837.85562, 56383.1174, 57059.3999, 56934.76676,
        52614.06982
    ])


def historic_hd_subbasic_metals_and_fabricated_metal(x):
    """
    historic hd subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        98211.18593, 101364.5069, 105638.8761, 109614.7046, 109634.716, 109513.8361, 108781.2866,
        107727.9884, 107406.8228, 108813.2145, 108643.8029, 110632.9872, 117104.5, 117239.6397,
        111348.4293
    ])


def historic_hd_submachinery_nec(x):
    """
    historic hd submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        169551.7844, 165460.9408, 173073.1117, 172458.5657, 166225.228, 177635.06, 179257.497,
        180868.4452, 192695.9211, 204320.0422, 213294.6206, 222421.2828, 230417.0138, 238684.2123,
        209182.9328
    ])


def historic_hd_subelectrical_and_optical_equipment(x):
    """
    historic hd subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        269085.0789, 292528.3037, 314568.9572, 327006.1972, 362531.3665, 403721.8382, 421237.8362,
        424194.5776, 462089.486, 500625.8901, 528593.1563, 575337.1644, 626824.9313, 659502.0775,
        638774.7252
    ])


def historic_hd_subtransport_equipment(x):
    """
    historic hd subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        559770.1377, 562660.0135, 610616.3602, 637624.4844, 658779.9693, 678119.7767, 705524.229,
        730358.9824, 762695.1596, 804770.2701, 822767.0034, 841264.7015, 900674.5172, 872656.5609,
        774016.3155
    ])


def historic_hd_submanufacturing_nec_recycling(x):
    """
    historic hd submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        222979.9294, 228135.1029, 240692.1728, 250014.8123, 258144.9556, 265663.7602, 267864.6516,
        274989.6719, 267848.5325, 265540.4372, 266704.1404, 287544.6036, 294441.8347, 286817.0569,
        273080.4631
    ])


def historic_hd_subelectricity_gas_and_water_supply(x):
    """
    historic hd subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        425936.7741, 442338.7712, 442427.9943, 454117.2328, 441934.9356, 459895.6379, 470380.3946,
        479556.851, 499323.8146, 513594.6788, 538766.6204, 545184.665, 562485.9796, 606105.3211,
        581811.356
    ])


def historic_hd_subconstruction(x):
    """
    historic hd subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        43757.57239, 42009.8992, 44512.78978, 44853.27532, 44317.62707, 47960.07991, 46618.12233,
        48551.27075, 49838.11959, 54193.50436, 57076.19252, 61104.42813, 64866.31738, 68880.49312,
        76525.49367
    ])


def historic_hd_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic hd subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        329630.4134, 339665.4583, 348852.2901, 362383.2948, 386175.4877, 394209.6837, 389345.6728,
        415315.3145, 420385.015, 437089.0681, 452805.9513, 469217.9448, 479671.7303, 477835.4496,
        441238.7686
    ])


def historic_hd_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic hd subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        953189.7518, 977575.7887, 1031268.424, 1083125.505, 1134670.38, 1145796.696, 1188875.24,
        1235385.121, 1297218.855, 1376569.796, 1419687.917, 1460277.285, 1587635.385, 1650741.741,
        1654069.375
    ])


def historic_hd_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic hd subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1442744.515, 1484111.231, 1511377.547, 1545708.108, 1603578.387, 1651018.305, 1692079.184,
        1735449.801, 1786053.772, 1824945.424, 1888889.851, 1946985.91, 1966830.2, 2004091.37,
        1943464.737
    ])


def historic_hd_subhotels_and_restaurants(x):
    """
    historic hd subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1113431.496, 1145333.762, 1160594.202, 1204074.939, 1248189.218, 1290930.693, 1305338.397,
        1321460.001, 1346728.693, 1381971.212, 1431615.207, 1478748.724, 1520610.363, 1533397.36,
        1478111.427
    ])


def historic_hd_subinland_transport(x):
    """
    historic hd subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        512283.1868, 516283.7531, 527525.772, 548001.3754, 583355.5247, 577090.4927, 576348.165,
        589005.7397, 603398.7816, 618148.8674, 629131.2697, 640920.8658, 656990.6185, 687030.627,
        677773.459
    ])


def historic_hd_subwater_transport(x):
    """
    historic hd subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        44308.95214, 47639.79652, 51042.05014, 52387.1092, 56976.62146, 61869.36149, 64202.31502,
        69243.18102, 74692.81662, 80768.97233, 82380.51438, 83700.92717, 90448.73024, 93235.80175,
        86145.54496
    ])


def historic_hd_subair_transport(x):
    """
    historic hd subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        137453.8779, 148653.8674, 154276.2508, 157494.006, 161326.5298, 171955.3187, 160991.2496,
        161745.3217, 166470.3069, 180233.9222, 183284.5052, 191905.9349, 200264.0223, 198775.0988,
        182876.4735
    ])


def historic_hd_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic hd subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        138435.9275, 139688.8976, 147937.0151, 154397.118, 160370.3872, 155638.3897, 153853.6935,
        152642.7157, 155699.8146, 164699.6652, 174244.2692, 180720.5432, 193769.3659, 204150.5715,
        205871.1577
    ])


def historic_hd_subpost_and_telecommunications(x):
    """
    historic hd subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        324538.4833, 354295.5739, 387919.9858, 424177.5174, 485852.2574, 554236.5348, 618457.2676,
        666784.4572, 710574.4463, 765818.0055, 804816.0351, 849312.5464, 898049.3822, 951908.6976,
        961153.9218
    ])


def historic_hd_subfinancial_intermedation(x):
    """
    historic hd subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        992982.5746, 1024466.451, 1104078.809, 1141697.612, 1212703.706, 1332023.396, 1343319.477,
        1349136.338, 1383135.657, 1487346.859, 1574452.568, 1688561.211, 1829768.303, 1861451.265,
        1853536.727
    ])


def historic_hd_subreal_estate_activities(x):
    """
    historic hd subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2441020.979, 2529304.337, 2567774.037, 2642829.576, 2733948.497, 2803765.837, 2876959.249,
        2915973.737, 2982427.268, 3050480.543, 3109956.804, 3175840.531, 3240184.883, 3293070.023,
        3340479.192
    ])


def historic_hd_subrenting_od_meq_and_other_business_activities(x):
    """
    historic hd subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        314984.6207, 333044.0176, 341190.5309, 368007.479, 383757.4154, 400238.0943, 396798.5427,
        402480.575, 413969.7256, 433966.7039, 457741.692, 480708.902, 511773.291, 543182.732,
        556229.075
    ])


def historic_hd_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    historic hd subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        376202.178, 332038.488, 320408.035, 329085.09, 328446.16, 346046.543, 362074.766,
        392771.431, 407507.525, 433867.474, 441176.919, 451861.479, 455933.663, 461450.031,
        484305.844
    ])


def historic_hd_subeducation(x):
    """
    historic hd subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        340722.53, 350270.806, 362212.427, 375472.918, 388453.727, 385804.453, 399798.349,
        410892.68, 428390.985, 449502.133, 455067.481, 464749.508, 471472.367, 480018.337,
        485121.345
    ])


def historic_hd_subhealth_and_social_work(x):
    """
    historic hd subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1143478.152, 1201180.793, 1245130.725, 1276058.878, 1324541.543, 1350134.311, 1401510.594,
        1460040.018, 1508964.291, 1557680.132, 1628369.246, 1668980.892, 1717773.42, 1754141.008,
        1793666.33
    ])


def historic_hd_subother_community_social_and_persona_services(x):
    """
    historic hd subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        959930.083, 1015666.047, 1021758.423, 1057090.263, 1100282.525, 1145322.223, 1161611.045,
        1201423.452, 1223840.595, 1257042.147, 1268420.147, 1303972.186, 1330364.267, 1352218.5,
        1344696.133
    ])


def historic_hd_subprivate_households_with_employed_persons(x):
    """
    historic hd subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        47919.81618, 48735.14528, 49312.02724, 51231.33634, 50583.91028, 51814.0594, 51742.50474,
        51846.85244, 52989.78551, 54193.75841, 54513.73037, 55623.74619, 57995.17327, 59045.67614,
        58243.5006
    ])


def share_cc_next_step_subagriculture_hunting_forestry_and_fishing(x):
    """
    share cc next step subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.025078382628402493, 0.02464951185425293, 0.024412482955470757, 0.022658014973602505,
            0.02491994295641962, 0.02507741479510443, 0.02302318244876147, 0.02341619403613124,
            0.023769823644028576, 0.02040672872055882, 0.018639964524329856, 0.019132109467672614,
            0.019439319871382504, 0.01655073055008412
        ])


def share_cc_next_step_submining_and_quarrying(x):
    """
    share cc next step submining and quarrying



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.018101740661219327, 0.018332290256034372, 0.01703113414914862, 0.0165157932633882,
            0.017486811309906522, 0.01801189939841712, 0.017721589288219524, 0.016965563499255994,
            0.017484206427070492, 0.01752032040491682, 0.018382928551712276, 0.01856406183756792,
            0.01917091605291903, 0.019668699343532815
        ])


def share_cc_next_step_subfood_beverages_and_tobacco(x):
    """
    share cc next step subfood beverages and tobacco



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02876498203026079, 0.02814531333997819, 0.027531214667705785, 0.02874565191986845,
            0.02788345999760771, 0.02802430949192551, 0.026795521737131983, 0.02600865546252975,
            0.0238629158502632, 0.02330770485908382, 0.023629186745957292, 0.02356268466844763,
            0.023143221409336112, 0.023638735612115706
        ])


def share_cc_next_step_subtextiles_and_textile_products(x):
    """
    share cc next step subtextiles and textile products



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007825063986088282, 0.007651438290957927, 0.0074916774298229, 0.0065129555117770625,
            0.006447661322563113, 0.00606430460937917, 0.005883481200538257, 0.005318456094464006,
            0.005508273168393984, 0.005124370728697409, 0.005360769244134405, 0.005676448056192027,
            0.005897454007932367, 0.005910777608613948
        ])


def share_cc_next_step_subleather_leather_and_footwear(x):
    """
    share cc next step subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.001143608757867767, 0.0010819449533878667, 0.0010508849725526389, 0.0009647149390711795,
        0.0009573791517385931, 0.0011008752744973995, 0.00102773596644666, 0.0009999533655333415,
        0.0008843018128007417, 0.0009008914767220458, 0.0009874610391665518, 0.001067974870512249,
        0.001128960946128135, 0.0011516930123566763
    ])


def share_cc_next_step_subwood_and_products_of_woood_and_cork(x):
    """
    share cc next step subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.003168228187107202, 0.0033681335538219806, 0.0032993092220650784, 0.003328976784436948,
        0.0031833806038531506, 0.003120972846879468, 0.0029739924497252965, 0.003047707007696271,
        0.0030294902999746915, 0.0029438212130334035, 0.002889431931536702, 0.0029644037920589375,
        0.002754986281307624, 0.0024892388370459616
    ])


def share_cc_next_step_subpulp_paper_printing_and_publishing(x):
    """
    share cc next step subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.015985164553250452, 0.015469435942986349, 0.015498594516144475, 0.016674768573807,
            0.016252638998880308, 0.015313901877892746, 0.01488650915592016, 0.01395986569583055,
            0.014257477691022605, 0.014401523761225027, 0.012986096954139215, 0.0124551897341896,
            0.011948904045551118, 0.011602358740517354
        ])


def share_cc_next_step_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    share cc next step subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.012746459305556141, 0.013633527821310106, 0.01359401984732537, 0.011047742103384642,
            0.01184315652889293, 0.012897411493201982, 0.010670476406875505, 0.01269276226211265,
            0.01461178252401957, 0.014107919688276434, 0.01221847799976767, 0.011732689862703299,
            0.011403344506130163, 0.012538492970489885
        ])


def share_cc_next_step_subchemicals_and_chemical_products(x):
    """
    share cc next step subchemicals and chemical products



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.027479288796231974, 0.02786846743794753, 0.02644383447466471, 0.02724529339079687,
            0.02624198821499827, 0.026181380294071407, 0.027621052672975967, 0.027145511225085295,
            0.027529056528481182, 0.02654123196808494, 0.027374754999941013, 0.02853635359715732,
            0.026591622864745416, 0.026767398009174638
        ])


def share_cc_next_step_subrubber_and_plastics(x):
    """
    share cc next step subrubber and plastics



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.007931836698194594, 0.007929950133355458, 0.008081105585862648, 0.008032743025468464,
            0.007568253072198993, 0.007055808764641412, 0.007271392284354727, 0.007325513751262952,
            0.007354678780301329, 0.00721820304143248, 0.007005938695842652, 0.007367759456408463,
            0.006715065354087769, 0.005981566587978499
        ])


def share_cc_next_step_subother_non_metalic_mineral(x):
    """
    share cc next step subother non metalic mineral



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.009083085220471966, 0.009359182092185262, 0.008776038855268316, 0.00880951349188728,
            0.008519134385357204, 0.008295573751607464, 0.00811510007249048, 0.008044587776070526,
            0.008372147189461823, 0.00843120193208617, 0.008877056370860618, 0.009110566920957066,
            0.008719093392781017, 0.007857115260071003
        ])


def share_cc_next_step_subbasic_metals_and_fabricated_metal(x):
    """
    share cc next step subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02409241566665906, 0.024628932833239057, 0.022608432090721487, 0.021563891804170753,
            0.02274463504803309, 0.02031556538542781, 0.019502770478668816, 0.020279731161448163,
            0.022241883350801843, 0.023391355493617486, 0.024932615017941605, 0.026498741926502475,
            0.026394927346192133, 0.02322394279397999
        ])


def share_cc_next_step_submachinery_nec(x):
    """
    share cc next step submachinery nec



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.013719072372734927, 0.01393565671152066, 0.014096065919260808, 0.012396757811239846,
            0.013555513896937202, 0.012458152299629186, 0.011661210055884422, 0.012266401752284125,
            0.013881604122856654, 0.015158824782683553, 0.016585895022378887, 0.01761172852089743,
            0.01970770333839225, 0.017516772760582607
        ])


def share_cc_next_step_subelectrical_and_optical_equipment(x):
    """
    share cc next step subelectrical and optical equipment



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.0240724320246828, 0.026879516079187753, 0.027491298893010396, 0.030775962613424947,
            0.03820207983468018, 0.027047047895312745, 0.027906514442787748, 0.03409009944486845,
            0.038641933859760995, 0.04129830740111195, 0.043431988576497264, 0.04706087316905279,
            0.04779006980275797, 0.045176727210357176
        ])


def share_cc_next_step_subtransport_equipment(x):
    """
    share cc next step subtransport equipment



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.016071504276576817, 0.016269187914275323, 0.017384791192130742, 0.018521083745308402,
            0.017510845619079896, 0.01750544095862379, 0.019580343726275647, 0.017519592188039654,
            0.017505942568556376, 0.018029153530819162, 0.018884900139347384, 0.02068709274361638,
            0.018734791973192437, 0.017331831349728804
        ])


def share_cc_next_step_submanufacturing_nec_recycling(x):
    """
    share cc next step submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.004246578649526875, 0.004325414800652897, 0.004465400376368324, 0.0042640218851806785,
        0.004382284365538603, 0.004177586302183998, 0.0044474981883766305, 0.0044519849138500095,
        0.005381816335691264, 0.006430083963705986, 0.006026154324366072, 0.0058666091075879,
        0.005829000303710761, 0.005502366671535831
    ])


def share_cc_next_step_subelectricity_gas_and_water_supply(x):
    """
    share cc next step subelectricity gas and water supply



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.04702297200735588, 0.043770893328872754, 0.043527249921124174, 0.042248197518757455,
            0.04144001516883796, 0.03937928827045648, 0.03954100227559819, 0.03992084800226018,
            0.04122838142863999, 0.04124366752005908, 0.03908474464960975, 0.038426723351002466,
            0.03972970117195154, 0.039340682954113484
        ])


def share_cc_next_step_subconstruction(x):
    """
    share cc next step subconstruction



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.029711743968391505, 0.02784947827139627, 0.028342394795020702, 0.027970268160605542,
            0.025480331282401503, 0.025832810595568145, 0.025271920012836506, 0.02490199946395865,
            0.026099648636237482, 0.026303515770198044, 0.027156725944773412, 0.026060042489757677,
            0.025878577266468014, 0.025758496017789966
        ])


def share_cc_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    share cc next step subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.00903374834872022, 0.009173380221024536, 0.009522368180830508, 0.009262104773787541,
            0.008679969036621738, 0.008024579623124155, 0.007668346382009341, 0.007234417099582751,
            0.007793426044455792, 0.0077206884013198435, 0.00787785920922557, 0.007519391350279477,
            0.006816306866714831, 0.00579608344911891
        ])


def share_cc_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    share cc next step subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.06872416518698367, 0.07397466130108132, 0.07759140573182152, 0.0790311572976072,
            0.07895106157594772, 0.08358153667209856, 0.08046215105242577, 0.08057597818003052,
            0.08366266458872629, 0.08459099239634021, 0.08197165939884506, 0.08438846586604647,
            0.0837852429770754, 0.08680986189833101
        ])


def share_cc_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    share cc next step subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.043218689413549206, 0.04418452831060235, 0.04431907265956516, 0.04453363331100283,
            0.04423144754730959, 0.04719982136870136, 0.04802528865820533, 0.046326157202964964,
            0.043729995596854115, 0.04365355366137524, 0.04447043011858734, 0.04115812502638437,
            0.037186956214448466, 0.03600442411878235
        ])


def share_cc_next_step_subhotels_and_restaurants(x):
    """
    share cc next step subhotels and restaurants



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.017674065016558355, 0.018149583857362074, 0.018880821695584853, 0.019187506494702862,
            0.01848919255326852, 0.018801788372633084, 0.019375033321432044, 0.01894584982699591,
            0.018775261769920393, 0.018685354294381163, 0.019167548759844704, 0.01817645780586637,
            0.017594543932584812, 0.01729384088477041
        ])


def share_cc_next_step_subinland_transport(x):
    """
    share cc next step subinland transport



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.01757328059423465, 0.017533477095097345, 0.01708335113402725, 0.016899900875868307,
            0.017573466333478917, 0.0184060737777463, 0.017573394156884907, 0.01752725282216478,
            0.017674037253620118, 0.01775614429616209, 0.01790638924580825, 0.01823776170799223,
            0.018692217055157924, 0.017259316371031952
        ])


def share_cc_next_step_subwater_transport(x):
    """
    share cc next step subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.0019363175510912095, 0.002174967262959381, 0.002445572534255821, 0.0025031485102358655,
        0.0027778550421596786, 0.003017307520741118, 0.003093341832727418, 0.0033244710209141787,
        0.0040514983108206415, 0.004701125070493769, 0.004946960268778851, 0.005842127228333952,
        0.006485775212538603, 0.006004459143157512
    ])


def share_cc_next_step_subair_transport(x):
    """
    share cc next step subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.004725321799396147, 0.004985152824590062, 0.004397007883431417, 0.003784892111621302,
        0.003282607584440268, 0.0016726602837590257, 0.0021694690555328824, 0.0030221235882390467,
        0.0035400602398324727, 0.0034155250192160567, 0.0036206051079420323, 0.0033596968339394905,
        0.002817936173472771, 0.0023199328516766635
    ])


def share_cc_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    share cc next step subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.006899084212642317, 0.0074114575956901255, 0.0071853424807637525,
            0.006714489302199572, 0.005691227831434559, 0.006036424779551169, 0.006585132573263568,
            0.006975233725391538, 0.007547036142033878, 0.008121570535510506, 0.008817806913968915,
            0.00905327322223547, 0.00899968325699662, 0.007461986150000285
        ])


def share_cc_next_step_subpost_and_telecommunications(x):
    """
    share cc next step subpost and telecommunications



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03160549275661185, 0.0322243006026475, 0.03388806015063383, 0.03591291947337805,
            0.03791737172900439, 0.0413838152640967, 0.04456758387526782, 0.04571097051257681,
            0.046742194825149755, 0.04604476002740335, 0.045529587353981896, 0.04443512758226394,
            0.046731061459096244, 0.050155005310443236
        ])


def share_cc_next_step_subfinancial_intermedation(x):
    """
    share cc next step subfinancial intermedation



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.07263386823827737, 0.07524361994554456, 0.07469807199082727, 0.07674454224960045,
            0.07811355669530828, 0.08063039557217784, 0.0847256373038466, 0.08363368144151041,
            0.07945056175518507, 0.07873728904029753, 0.08477781706234779, 0.08274482610250618,
            0.07932373057747709, 0.08359235909013855
        ])


def share_cc_next_step_subreal_estate_activities(x):
    """
    share cc next step subreal estate activities



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.23937803446193515, 0.23358810685060818, 0.2301581651449089, 0.227948775874283,
            0.2256644107843945, 0.22877553857089394, 0.22367874012980918, 0.22277622638960762,
            0.21590454968628062, 0.2127113153994352, 0.20839896270047154, 0.2057033512412547,
            0.2094927971640791, 0.21569041217322732
        ])


def share_cc_next_step_subrenting_od_meq_and_other_business_activities(x):
    """
    share cc next step subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.06905001380524613, 0.06821043531529368, 0.07288181835394007, 0.06817971257110544,
            0.06450830258564538, 0.06454304884231093, 0.06721206423359273, 0.0667362552587201,
            0.06682921530945202, 0.06977231435038858, 0.06861132671855419, 0.07189063738602502,
            0.07522583494958558, 0.06706870740084765
        ])


def share_cc_next_step_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    share cc next step subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.03783224147960273, 0.03701298705578099, 0.03567177066473178, 0.038192407838542546,
            0.038012695658412206, 0.03826439070447431, 0.03718222874310394, 0.03554916941396557,
            0.030954803248098826, 0.029375582837865413, 0.029483320594616752, 0.026471578950167513,
            0.026470257950723577, 0.0318911564104782
        ])


def share_cc_next_step_subeducation(x):
    """
    share cc next step subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008
    ], [
        0.005875669601914743, 0.0056678531809458655, 0.006082054444355262, 0.006127937515178124,
        0.005803650449733586, 0.005767438285737494, 0.0059783876907612225, 0.006127107081716946,
        0.005054519325957908, 0.005130009752766508, 0.004847058955568252, 0.0051597539362335815,
        0.005669951603168591, 0.005755993947517754
    ])


def share_cc_next_step_subhealth_and_social_work(x):
    """
    share cc next step subhealth and social work



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.029160160403648494, 0.02821173656516283, 0.026615770620869734, 0.0273539727857798,
            0.02662583643780871, 0.028142691231517622, 0.029297619075475182, 0.02941131460970212,
            0.0301810591244524, 0.031490932282237226, 0.031232041065638115, 0.02959403415633184,
            0.02988511373451547, 0.03422943890008811
        ])


def share_cc_next_step_subother_community_social_and_persona_services(x):
    """
    share cc next step subother community social and persona services



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            0.02841304632967128, 0.027082293603525917, 0.027431107643009052, 0.029323662796849222,
            0.029033478768738467, 0.027866881374139507, 0.028477811794828094, 0.027741634257114605,
            0.02643605715958021, 0.025304071077875273, 0.023855398102559003, 0.02385049848422552,
            0.023811135179053207, 0.024622633133622788
        ])


def share_cc_next_step_subprivate_households_with_employed_persons(x):
    """
    share cc next step subprivate households with employed persons



    lookup


    """
    return functions.lookup(
        x, [1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008], [
            2.224100933757743e-05, 2.3182796720525097e-05, 2.2308822775885723e-05,
            2.2884702082035344e-05, 2.4357628368450084e-05, 2.586345147694192e-05,
            2.6477256965928314e-05, 2.6730466120345426e-05, 2.769540121679224e-05,
            2.994530061908033e-05, 3.0137690959046645e-05, 3.283954762959073e-05,
            3.3795758345553865e-05, 3.676247669895163e-05
        ])


def historic_labour_share(x):
    """
    historic labour share



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013
    ], [
        -0.03570221794602446, -0.007511835079606288, -4.9226177098571355e-05,
        -0.0036856044367975516, -0.0018807592046075072, 0.0033245164448738596,
        -0.00916060631432225, -0.0035427091662072474, -0.012338255775489415, -0.00752841386661518,
        -0.0031033781483257306, -0.005033300111722316, -0.00010456283432869196,
        -0.005033300111722316, -0.009419751237859098, -0.009419751237859098, -0.009419751237859098,
        -0.009419751237859098, -0.009419751237859098
    ])


def share_cc_sectoral_subagriculture_hunting_forestry_and_fishing(x):
    """
    share cc sectoral subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.02287729187012649, 0.025078382628402493, 0.02464951185425293, 0.024412482955470757,
        0.022658014973602505, 0.02491994295641962, 0.02507741479510443, 0.02302318244876147,
        0.02341619403613124, 0.023769823644028576, 0.02040672872055882, 0.018639964524329856,
        0.019132109467672614, 0.019439319871382504, 0.01655073055008412
    ])


def share_cc_sectoral_submining_and_quarrying(x):
    """
    share cc sectoral submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.019074894806125776, 0.018101740661219327, 0.018332290256034372, 0.01703113414914862,
        0.0165157932633882, 0.017486811309906522, 0.01801189939841712, 0.017721589288219524,
        0.016965563499255994, 0.017484206427070492, 0.01752032040491682, 0.018382928551712276,
        0.01856406183756792, 0.01917091605291903, 0.019668699343532815
    ])


def share_cc_sectoral_subfood_beverages_and_tobacco(x):
    """
    share cc sectoral subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0321389816671635, 0.02876498203026079, 0.02814531333997819, 0.027531214667705785,
        0.02874565191986845, 0.02788345999760771, 0.02802430949192551, 0.026795521737131983,
        0.02600865546252975, 0.0238629158502632, 0.02330770485908382, 0.023629186745957292,
        0.02356268466844763, 0.023143221409336112, 0.023638735612115706
    ])


def share_cc_sectoral_subtextiles_and_textile_products(x):
    """
    share cc sectoral subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.008385489965815875, 0.007825063986088282, 0.007651438290957927, 0.0074916774298229,
        0.0065129555117770625, 0.006447661322563113, 0.00606430460937917, 0.005883481200538257,
        0.005318456094464006, 0.005508273168393984, 0.005124370728697409, 0.005360769244134405,
        0.005676448056192027, 0.005897454007932367, 0.005910777608613948
    ])


def share_cc_sectoral_subleather_leather_and_footwear(x):
    """
    share cc sectoral subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0012340222421751328, 0.001143608757867767, 0.0010819449533878667, 0.0010508849725526389,
        0.0009647149390711795, 0.0009573791517385931, 0.0011008752744973995, 0.00102773596644666,
        0.0009999533655333415, 0.0008843018128007417, 0.0009008914767220458, 0.0009874610391665518,
        0.001067974870512249, 0.001128960946128135, 0.0011516930123566763
    ])


def share_cc_sectoral_subwood_and_products_of_woood_and_cork(x):
    """
    share cc sectoral subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0036959103200494992, 0.003168228187107202, 0.0033681335538219806, 0.0032993092220650784,
        0.003328976784436948, 0.0031833806038531506, 0.003120972846879468, 0.0029739924497252965,
        0.003047707007696271, 0.0030294902999746915, 0.0029438212130334035, 0.002889431931536702,
        0.0029644037920589375, 0.002754986281307624, 0.0024892388370459616
    ])


def share_cc_sectoral_subpulp_paper_printing_and_publishing(x):
    """
    share cc sectoral subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.016163481867476657, 0.015985164553250452, 0.015469435942986349, 0.015498594516144475,
        0.016674768573807, 0.016252638998880308, 0.015313901877892746, 0.01488650915592016,
        0.01395986569583055, 0.014257477691022605, 0.014401523761225027, 0.012986096954139215,
        0.0124551897341896, 0.011948904045551118, 0.011602358740517354
    ])


def share_cc_sectoral_subcoke_refined_petroleum_and_nuclear_fuel(x):
    """
    share cc sectoral subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.012911021621846729, 0.012746459305556141, 0.013633527821310106, 0.01359401984732537,
        0.011047742103384642, 0.01184315652889293, 0.012897411493201982, 0.010670476406875505,
        0.01269276226211265, 0.01461178252401957, 0.014107919688276434, 0.01221847799976767,
        0.011732689862703299, 0.011403344506130163, 0.012538492970489885
    ])


def share_cc_sectoral_subchemicals_and_chemical_products(x):
    """
    share cc sectoral subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.02819783641479054, 0.027479288796231974, 0.02786846743794753, 0.02644383447466471,
        0.02724529339079687, 0.02624198821499827, 0.026181380294071407, 0.027621052672975967,
        0.027145511225085295, 0.027529056528481182, 0.02654123196808494, 0.027374754999941013,
        0.02853635359715732, 0.026591622864745416, 0.026767398009174638
    ])


def share_cc_sectoral_subrubber_and_plastics(x):
    """
    share cc sectoral subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.007458520759641382, 0.007931836698194594, 0.007929950133355458, 0.008081105585862648,
        0.008032743025468464, 0.007568253072198993, 0.007055808764641412, 0.007271392284354727,
        0.007325513751262952, 0.007354678780301329, 0.00721820304143248, 0.007005938695842652,
        0.007367759456408463, 0.006715065354087769, 0.005981566587978499
    ])


def share_cc_sectoral_subother_non_metalic_mineral(x):
    """
    share cc sectoral subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.009909953771210725, 0.009083085220471966, 0.009359182092185262, 0.008776038855268316,
        0.00880951349188728, 0.008519134385357204, 0.008295573751607464, 0.00811510007249048,
        0.008044587776070526, 0.008372147189461823, 0.00843120193208617, 0.008877056370860618,
        0.009110566920957066, 0.008719093392781017, 0.007857115260071003
    ])


def share_cc_sectoral_subbasic_metals_and_fabricated_metal(x):
    """
    share cc sectoral subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.025876033237234416, 0.02409241566665906, 0.024628932833239057, 0.022608432090721487,
        0.021563891804170753, 0.02274463504803309, 0.02031556538542781, 0.019502770478668816,
        0.020279731161448163, 0.022241883350801843, 0.023391355493617486, 0.024932615017941605,
        0.026498741926502475, 0.026394927346192133, 0.02322394279397999
    ])


def share_cc_sectoral_submachinery_nec(x):
    """
    share cc sectoral submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.014059658337226175, 0.013719072372734927, 0.01393565671152066, 0.014096065919260808,
        0.012396757811239846, 0.013555513896937202, 0.012458152299629186, 0.011661210055884422,
        0.012266401752284125, 0.013881604122856654, 0.015158824782683553, 0.016585895022378887,
        0.01761172852089743, 0.01970770333839225, 0.017516772760582607
    ])


def share_cc_sectoral_subelectrical_and_optical_equipment(x):
    """
    share cc sectoral subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.02211615042750115, 0.0240724320246828, 0.026879516079187753, 0.027491298893010396,
        0.030775962613424947, 0.03820207983468018, 0.027047047895312745, 0.027906514442787748,
        0.03409009944486845, 0.038641933859760995, 0.04129830740111195, 0.043431988576497264,
        0.04706087316905279, 0.04779006980275797, 0.045176727210357176
    ])


def share_cc_sectoral_subtransport_equipment(x):
    """
    share cc sectoral subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.01611580684921055, 0.016071504276576817, 0.016269187914275323, 0.017384791192130742,
        0.018521083745308402, 0.017510845619079896, 0.01750544095862379, 0.019580343726275647,
        0.017519592188039654, 0.017505942568556376, 0.018029153530819162, 0.018884900139347384,
        0.02068709274361638, 0.018734791973192437, 0.017331831349728804
    ])


def share_cc_sectoral_submanufacturing_nec_recycling(x):
    """
    share cc sectoral submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.004136770188653854, 0.004246578649526875, 0.004325414800652897, 0.004465400376368324,
        0.0042640218851806785, 0.004382284365538603, 0.004177586302183998, 0.0044474981883766305,
        0.0044519849138500095, 0.005381816335691264, 0.006430083963705986, 0.006026154324366072,
        0.0058666091075879, 0.005829000303710761, 0.005502366671535831
    ])


def share_cc_sectoral_subelectricity_gas_and_water_supply(x):
    """
    share cc sectoral subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.047066365445528786, 0.04702297200735588, 0.043770893328872754, 0.043527249921124174,
        0.042248197518757455, 0.04144001516883796, 0.03937928827045648, 0.03954100227559819,
        0.03992084800226018, 0.04122838142863999, 0.04124366752005908, 0.03908474464960975,
        0.038426723351002466, 0.03972970117195154, 0.039340682954113484
    ])


def share_cc_sectoral_subconstruction(x):
    """
    share cc sectoral subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.032283292539891505, 0.029711743968391505, 0.02784947827139627, 0.028342394795020702,
        0.027970268160605542, 0.025480331282401503, 0.025832810595568145, 0.025271920012836506,
        0.02490199946395865, 0.026099648636237482, 0.026303515770198044, 0.027156725944773412,
        0.026060042489757677, 0.025878577266468014, 0.025758496017789966
    ])


def share_cc_sectoral_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    share cc sectoral subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.009551079848249643, 0.00903374834872022, 0.009173380221024536, 0.009522368180830508,
        0.009262104773787541, 0.008679969036621738, 0.008024579623124155, 0.007668346382009341,
        0.007234417099582751, 0.007793426044455792, 0.0077206884013198435, 0.00787785920922557,
        0.007519391350279477, 0.006816306866714831, 0.00579608344911891
    ])


def share_cc_sectoral_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    share cc sectoral subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.06748267878929118, 0.06872416518698367, 0.07397466130108132, 0.07759140573182152,
        0.0790311572976072, 0.07895106157594772, 0.08358153667209856, 0.08046215105242577,
        0.08057597818003052, 0.08366266458872629, 0.08459099239634021, 0.08197165939884506,
        0.08438846586604647, 0.0837852429770754, 0.08680986189833101
    ])


def share_cc_sectoral_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    share cc sectoral subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03997245896306118, 0.043218689413549206, 0.04418452831060235, 0.04431907265956516,
        0.04453363331100283, 0.04423144754730959, 0.04719982136870136, 0.04802528865820533,
        0.046326157202964964, 0.043729995596854115, 0.04365355366137524, 0.04447043011858734,
        0.04115812502638437, 0.037186956214448466, 0.03600442411878235
    ])


def share_cc_sectoral_subhotels_and_restaurants(x):
    """
    share cc sectoral subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.01617662855110026, 0.017674065016558355, 0.018149583857362074, 0.018880821695584853,
        0.019187506494702862, 0.01848919255326852, 0.018801788372633084, 0.019375033321432044,
        0.01894584982699591, 0.018775261769920393, 0.018685354294381163, 0.019167548759844704,
        0.01817645780586637, 0.017594543932584812, 0.01729384088477041
    ])


def share_cc_sectoral_subinland_transport(x):
    """
    share cc sectoral subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.021253625676560958, 0.01757328059423465, 0.017533477095097345, 0.01708335113402725,
        0.016899900875868307, 0.017573466333478917, 0.0184060737777463, 0.017573394156884907,
        0.01752725282216478, 0.017674037253620118, 0.01775614429616209, 0.01790638924580825,
        0.01823776170799223, 0.018692217055157924, 0.017259316371031952
    ])


def share_cc_sectoral_subwater_transport(x):
    """
    share cc sectoral subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.00237930634975559, 0.0019363175510912095, 0.002174967262959381, 0.002445572534255821,
        0.0025031485102358655, 0.0027778550421596786, 0.003017307520741118, 0.003093341832727418,
        0.0033244710209141787, 0.0040514983108206415, 0.004701125070493769, 0.004946960268778851,
        0.005842127228333952, 0.006485775212538603, 0.006004459143157512
    ])


def share_cc_sectoral_subair_transport(x):
    """
    share cc sectoral subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.004166442318428751, 0.004725321799396147, 0.004985152824590062, 0.004397007883431417,
        0.003784892111621302, 0.003282607584440268, 0.0016726602837590257, 0.0021694690555328824,
        0.0030221235882390467, 0.0035400602398324727, 0.0034155250192160567, 0.0036206051079420323,
        0.0033596968339394905, 0.002817936173472771, 0.0023199328516766635
    ])


def share_cc_sectoral_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    share cc sectoral subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.006778853572000129, 0.006899084212642317, 0.0074114575956901255, 0.0071853424807637525,
        0.006714489302199572, 0.005691227831434559, 0.006036424779551169, 0.006585132573263568,
        0.006975233725391538, 0.007547036142033878, 0.008121570535510506, 0.008817806913968915,
        0.00905327322223547, 0.00899968325699662, 0.007461986150000285
    ])


def share_cc_sectoral_subpost_and_telecommunications(x):
    """
    share cc sectoral subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.029057579895015385, 0.03160549275661185, 0.0322243006026475, 0.03388806015063383,
        0.03591291947337805, 0.03791737172900439, 0.0413838152640967, 0.04456758387526782,
        0.04571097051257681, 0.046742194825149755, 0.04604476002740335, 0.045529587353981896,
        0.04443512758226394, 0.046731061459096244, 0.050155005310443236
    ])


def share_cc_sectoral_subfinancial_intermedation(x):
    """
    share cc sectoral subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.07273569540483278, 0.07263386823827737, 0.07524361994554456, 0.07469807199082727,
        0.07674454224960045, 0.07811355669530828, 0.08063039557217784, 0.0847256373038466,
        0.08363368144151041, 0.07945056175518507, 0.07873728904029753, 0.08477781706234779,
        0.08274482610250618, 0.07932373057747709, 0.08359235909013855
    ])


def share_cc_sectoral_subreal_estate_activities(x):
    """
    share cc sectoral subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.24111520591885524, 0.23937803446193515, 0.23358810685060818, 0.2301581651449089,
        0.227948775874283, 0.2256644107843945, 0.22877553857089394, 0.22367874012980918,
        0.22277622638960762, 0.21590454968628062, 0.2127113153994352, 0.20839896270047154,
        0.2057033512412547, 0.2094927971640791, 0.21569041217322732
    ])


def share_cc_sectoral_subrenting_od_meq_and_other_business_activities(x):
    """
    share cc sectoral subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.06459602118620193, 0.06905001380524613, 0.06821043531529368, 0.07288181835394007,
        0.06817971257110544, 0.06450830258564538, 0.06454304884231093, 0.06721206423359273,
        0.0667362552587201, 0.06682921530945202, 0.06977231435038858, 0.06861132671855419,
        0.07189063738602502, 0.07522583494958558, 0.06706870740084765
    ])


def share_cc_sectoral_subpublic_admin_and_defence_compulsory_social_security(x):
    """
    share cc sectoral subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03788162091421682, 0.03783224147960273, 0.03701298705578099, 0.03567177066473178,
        0.038192407838542546, 0.038012695658412206, 0.03826439070447431, 0.03718222874310394,
        0.03554916941396557, 0.030954803248098826, 0.029375582837865413, 0.029483320594616752,
        0.026471578950167513, 0.026470257950723577, 0.0318911564104782
    ])


def share_cc_sectoral_subeducation(x):
    """
    share cc sectoral subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.006956181200851506, 0.005875669601914743, 0.0056678531809458655, 0.006082054444355262,
        0.006127937515178124, 0.005803650449733586, 0.005767438285737494, 0.0059783876907612225,
        0.006127107081716946, 0.005054519325957908, 0.005130009752766508, 0.004847058955568252,
        0.0051597539362335815, 0.005669951603168591, 0.005755993947517754
    ])


def share_cc_sectoral_subhealth_and_social_work(x):
    """
    share cc sectoral subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.027445053332828068, 0.029160160403648494, 0.02821173656516283, 0.026615770620869734,
        0.0273539727857798, 0.02662583643780871, 0.028142691231517622, 0.029297619075475182,
        0.02941131460970212, 0.0301810591244524, 0.031490932282237226, 0.031232041065638115,
        0.02959403415633184, 0.02988511373451547, 0.03422943890008811
    ])


def share_cc_sectoral_subother_community_social_and_persona_services(x):
    """
    share cc sectoral subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.028727661644533248, 0.02841304632967128, 0.027082293603525917, 0.027431107643009052,
        0.029323662796849222, 0.029033478768738467, 0.027866881374139507, 0.028477811794828094,
        0.027741634257114605, 0.02643605715958021, 0.025304071077875273, 0.023855398102559003,
        0.02385049848422552, 0.023811135179053207, 0.024622633133622788
    ])


def share_cc_sectoral_subprivate_households_with_employed_persons(x):
    """
    share cc sectoral subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.242410254852871e-05, 2.224100933757743e-05, 2.3182796720525097e-05,
        2.2308822775885723e-05, 2.2884702082035344e-05, 2.4357628368450084e-05,
        2.586345147694192e-05, 2.6477256965928314e-05, 2.6730466120345426e-05,
        2.769540121679224e-05, 2.994530061908033e-05, 3.0137690959046645e-05,
        3.283954762959073e-05, 3.3795758345553865e-05, 3.676247669895163e-05
    ])


def historic_pes_waste_ej(x):
    """
    historic pes waste ej



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.94403359, 0.9254440700000001, 0.96264396, 0.97404579, 1.0843876399999999, 1.20351759,
        1.1960921400000002, 1.24529466, 1.24459098, 1.29713973, 1.34430356, 1.4583756600000002,
        1.48302204, 1.51246124, 1.57780579, 1.8249681400000002, 1.96140481, 2.0159873999999998,
        2.07606288, 2.11879269
    ])


def table_hist_capacity_res_elec_subhydro(x):
    """
    table hist capacity res elec subhydro



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.624490030142469, 0.634070032104449, 0.648500038817058, 0.657040027472228,
        0.674990036363238, 0.688220036952962, 0.698510036448025, 0.71313002660836,
        0.731790042351271, 0.750370028846775, 0.770490041148841, 0.792870048368696,
        0.82019003092096, 0.852160046683773, 0.8859300529104, 0.919750045926543, 0.946530044441139,
        0.976190046399544, 1.0204726000000002, 1.0567436000000001, 1.0915964000000002
    ])


def table_hist_capacity_res_elec_subgeotxdashxelec(x):
    """
    table hist capacity res elec subgeotxdashxelec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.007384619419926325, 0.007564460131224811, 0.007748680578241711, 0.007937387422504466,
        0.008130689923113103, 0.008328700000000001, 0.0080097, 0.0082019, 0.008357799999999999,
        0.0083085, 0.008689700000000002, 0.008921799999999999, 0.0091435, 0.0094635, 0.0099034,
        0.0101248, 0.010015399999999999, 0.010482, 0.010787, 0.0114575, 0.0118479
    ])


def table_hist_capacity_res_elec_subsolid_bioexdashxelec(x):
    """
    table hist capacity res elec subsolid bioexdashxelec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.02075556130242758, 0.022343248327393452, 0.024052384734166706, 0.025892260736819418,
        0.02787287720003564, 0.030005, 0.0318786, 0.033744300000000005, 0.0367185, 0.0392421,
        0.0394749, 0.0430149, 0.0451547, 0.0488584, 0.054362, 0.0588526, 0.062469699999999996,
        0.06715460000000001, 0.07378739999999999, 0.078254, 0.08402219999999999
    ])


def table_hist_capacity_res_elec_suboceanic(x):
    """
    table hist capacity res elec suboceanic



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.00020760889860613118, 0.000218011155677108, 0.00022893461850032943,
        0.0002404054021231664, 0.0002524509300890995, 0.00026510000000000005,
        0.00026510000000000005, 0.00026510000000000005, 0.00026510000000000005,
        0.00026510000000000005, 0.00026510000000000005, 0.00026510000000000005, 0.0002674,
        0.0002674, 0.00026910000000000004, 0.00027069999999999997, 0.0005251,
        0.0005277000000000001, 0.0005265, 0.0005273, 0.0005334
    ])


def table_hist_capacity_res_elec_subwind_onshore(x):
    """
    table hist capacity res elec subwind onshore



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.00455706227539308, 0.005947976066351176, 0.007763426775385634, 0.010132992235415647,
        0.013225800231482635, 0.0172626, 0.0238921, 0.0306214, 0.038160599999999996,
        0.046517699999999995, 0.057833, 0.0723973, 0.09245880000000001, 0.1182237, 0.147983,
        0.1795401, 0.21826189999999998, 0.26633959999999995, 0.2961245, 0.341731, 0.4036673
    ])


def table_hist_capacity_res_elec_subwind_offshore(x):
    """
    table hist capacity res elec subwind offshore



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        4.77710420476937e-06, 8.101151096781961e-06, 1.3738165691962337e-05, 2.329757763125582e-05,
        3.9508704120663585e-05, 6.7e-05, 7.7e-05, 0.000237, 0.0005072, 0.0005974,
        0.0006843999999999999, 0.0008834, 0.0010939, 0.0014419, 0.0021587, 0.0031435, 0.0037905,
        0.005374100000000001, 0.0075502, 0.0084937, 0.011636700000000002
    ])


def table_hist_capacity_res_elec_subsolar_pv(x):
    """
    table hist capacity res elec subsolar pv



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        3.921043766748761e-05, 7.173827876311545e-05, 0.00013125027278544562,
        0.00024013168984911632, 0.0004393379704745763, 0.0008037999999999999, 0.0010863, 0.0014352,
        0.0019645, 0.0030528, 0.0045056, 0.006083399999999999, 0.0086421, 0.0145796, 0.0224319,
        0.0387925, 0.0693767, 0.098947, 0.1353989, 0.1718955, 0.2192966
    ])


def table_hist_capacity_res_elec_subcsp(x):
    """
    table hist capacity res elec subcsp



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.00035380000000000003, 0.00035380000000000003, 0.00035380000000000003,
        0.00035380000000000003, 0.00035380000000000003, 0.00035380000000000003,
        0.00035380000000000003, 0.00035380000000000003, 0.00035380000000000003,
        0.00035380000000000003, 0.00035380000000000003, 0.000341, 0.000424, 0.00048039999999999997,
        0.0008077000000000001, 0.00116082, 0.00167582, 0.00253307, 0.00328092,
        0.004338020000000001, 0.00469302
    ])


def losses_in_charcoal_plants_ej(x):
    """
    losses in charcoal plants ej



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        2.1227642400000004, 2.1262310400000004, 2.1231064699999997, 2.1130467299999998, 2.1735678,
        2.18822825, 2.2560478, 2.3403727099999996, 2.4201021199999997, 2.54406673, 2.6353136,
        2.7016235600000007, 2.83974232, 2.86682404, 2.78902384, 2.92604662, 3.0703874300000003,
        3.5232924299999997, 3.4736966899999997, 3.4703146300000003
    ])


def historic_share_chp_plants_gas(x):
    """
    historic share chp plants gas



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.542607444618554, 0.5269313193785189, 0.5382532583200468, 0.5297249478665147,
        0.5460827573768788, 0.5224061688803091, 0.5411877199764002, 0.5577070041319593,
        0.562342979935354, 0.5613705825714098, 0.5857040300373348, 0.5890026793248733,
        0.5983201717080272, 0.5900688532905982, 0.5816026263018895, 0.5969505315378193,
        0.6059246889618498, 0.6191394159419781, 0.6468415798082748, 0.6518959474484941
    ])


def historic_share_liquids_for_heat_plants(x):
    """
    historic share liquids for heat plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.18497727967085997, 0.17191646447570985, 0.1330532764569175, 0.15161781088000842,
        0.13531842227897417, 0.12023891330325187, 0.12010733048107938, 0.10499688074615252,
        0.10170168137771297, 0.08909757858316623, 0.08211320924276294, 0.07546615577930145,
        0.07135237951220018, 0.06937841924029578, 0.07213434112645382, 0.07775801992149381,
        0.06902303071848384, 0.06498032662251463, 0.06478078155386392, 0.06752954750266457
    ])


def historic_share_chp_plants_oil(x):
    """
    historic share chp plants oil



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.09862928248379321, 0.11787701564444898, 0.10608252906539808, 0.10858090585253756,
        0.08882230533385095, 0.07562621902637809, 0.0734600656866448, 0.06506856593351103,
        0.06329937880802863, 0.07127638319113186, 0.07390231230724596, 0.07582583414512277,
        0.06819575385574247, 0.06266601654129239, 0.06998905828978902, 0.05073704681719738,
        0.05157446046900763, 0.04861084829791663, 0.033462519012298, 0.032844869257742916
    ])


def historic_biogas_pes(x):
    """
    historic biogas pes



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.13135264000000002, 0.14522501999999998, 0.16244876, 0.17106013, 0.25641764, 0.28492731,
        0.33951272, 0.39872446, 0.41243782, 0.45342303, 0.49843029, 0.53353154, 0.6900384399999999,
        0.79629398, 0.90816046, 0.8367312299999999, 0.98802405, 1.0866051, 1.19932464, 1.26714942
    ])


def historic_pes_peat_ej(x):
    """
    historic pes peat ej



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.22106789999999998, 0.23877885999999998, 0.22058298, 0.19908862, 0.17753217000000002,
        0.16850551, 0.18371079999999998, 0.19475629, 0.19557075, 0.17127541000000002, 0.15939893,
        0.18059229999999998, 0.18713522, 0.17082508, 0.15967739, 0.1799199, 0.1704865, 0.15029599,
        0.13919226, 0.13753595999999998
    ])


def efficiency_heat_oil_chp_plants(x):
    """
    efficiency heat oil chp plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.3583416873580516, 0.3864604322008381, 0.3427001131428596, 0.3426092425053806,
        0.2559985362733478, 0.27189087245527377, 0.30978899425880957, 0.2622427885462967,
        0.24249576844589016, 0.2814351729755998, 0.29350979736335797, 0.3169617343470768,
        0.3231319029553005, 0.32732942509102475, 0.3520069392931584, 0.31166079425216436,
        0.2791179408861372, 0.26613174359788516, 0.27197080023843734, 0.2580529463443723
    ])


def efficiency_elec_coal_chp_plants(x):
    """
    efficiency elec coal chp plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.2720349999945255, 0.26306888518464144, 0.27545884097647594, 0.2789107682797072,
        0.27789150432842147, 0.27881150074145433, 0.27529653704292867, 0.2751896064517859,
        0.28277916069267756, 0.2832216768494038, 0.2842686028283118, 0.28068849559240433,
        0.28915271943220666, 0.30241506318388284, 0.30167536009026713, 0.2979348686134844,
        0.2926626262217007, 0.29278659243706345, 0.29829300985708207, 0.30761772840986873
    ])


def efficiency_elec_gas_chp_plants(x):
    """
    efficiency elec gas chp plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.26782295944824347, 0.2666572241731881, 0.26830320759173404, 0.2738131863521139,
        0.2780196471160715, 0.3055205882931438, 0.30845101322725493, 0.3086933636483027,
        0.316047593938253, 0.31760571304542445, 0.31705352904069145, 0.3240026232130847,
        0.3279642160865447, 0.32862834898119897, 0.3336097590206972, 0.3351515677147325,
        0.3243356439411598, 0.32476042746440104, 0.31797643498814804, 0.33154405356432026
    ])


def efficiency_elec_oil_chp_plants(x):
    """
    efficiency elec oil chp plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.3605926738806412, 0.3348333610288783, 0.3466476498304344, 0.3531299339587378,
        0.29737398934546155, 0.31012580512156696, 0.32513162171457316, 0.3418424504153948,
        0.3283762635470366, 0.31172847731284714, 0.30438653224080414, 0.29767818828797493,
        0.29991121932367, 0.2993764831229446, 0.300032598019261, 0.29242527982890654,
        0.2893579004394849, 0.29758405532679544, 0.3107288107229254, 0.32993276951104356
    ])


def efficiency_heat_gas_chp_plants(x):
    """
    efficiency heat gas chp plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.34818197812029744, 0.30658370588876055, 0.29071150494278963, 0.27992388078634095,
        0.2689321316194951, 0.2916261332435117, 0.29242955417479993, 0.289381470807263,
        0.2814273774503354, 0.27495237777396736, 0.26980106264673176, 0.2871855759202209,
        0.27965735655873114, 0.2711157820652094, 0.26852977690961327, 0.26853864364145363,
        0.26936552781846224, 0.26897430848954873, 0.27004646141898336, 0.2843726348639607
    ])


def share_heatxdashxcom_chp_plants_nre_vs_nre_tot_heatxdashxcom_generation(x):
    """
    share heatxdashxcom chp plants nre vs nre tot heatxdashxcom generation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.47494035235993737, 0.48784484337985057, 0.48319453732010703, 0.4689138621366646,
        0.45504898204107314, 0.4787622080599851, 0.4767869618571136, 0.47874893888354547,
        0.47529703503778065, 0.477544401524363, 0.4599838057623242, 0.4659217428645132,
        0.4655488531999236, 0.46823780955770167, 0.4600254701073876, 0.45474576456496724,
        0.44887567948184176, 0.4410346921325672, 0.44994734894394395, 0.4547662603919192
    ])


def efficiency_heat_coal_chp_plants(x):
    """
    efficiency heat coal chp plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.31110748223005136, 0.27146068584931377, 0.2800501070449751, 0.2908126586324929,
        0.2831297492346078, 0.30852271504871565, 0.30629958279337455, 0.2992513065181298,
        0.2780742636093701, 0.2829681290854686, 0.24566650306280705, 0.2406045127607476,
        0.2438696178536268, 0.25122846960658485, 0.2689306835894877, 0.27212307003403136,
        0.26753189523710946, 0.2645720057578548, 0.24924363461171214, 0.25692298720979917
    ])


def historic_nonxdashxenergy_use_subliquids(x):
    """
    historic nonxdashxenergy use subliquids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        17.86474451, 18.642216100000002, 19.669279510000003, 19.482172170000002, 20.12592009,
        20.309394920000003, 20.619444329999997, 21.34137057, 22.0610557, 23.32966305,
        23.355337600000002, 24.137153670000004, 24.616445300000002, 23.63162834, 23.839961539999997
    ])


def historic_nonxdashxenergy_use_subsolids(x):
    """
    historic nonxdashxenergy use subsolids



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.86732782, 0.9033509599999999, 0.81759914, 0.74229678, 0.68570774, 1.03101072, 1.01775427,
        1.08015287, 1.18593028, 1.29226541, 1.4364099399999999, 1.6726974399999999,
        1.7306662800000001, 1.74080181, 1.78348197
    ])


def historic_nonxdashxenergy_use_subgases(x):
    """
    historic nonxdashxenergy use subgases



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3.74572862, 3.75295893, 4.03143292, 4.05466539, 4.17667242, 4.47201851, 4.3420209199999995,
        4.3756594699999996, 4.45160149, 4.78613611, 5.14473235, 5.00779145, 5.3063370899999995,
        5.338537059999999, 5.4843512
    ])


def historic_nonxdashxenergy_use_subelectricity(x):
    """
    historic nonxdashxenergy use subelectricity



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.001995, 0.001996, 0.001997, 0.001998, 0.001999, 0.002, 0.002001, 0.002002, 0.002003,
        0.002004, 0.002005, 0.002006, 0.002007, 0.002008, 0.002009
    ])


def historic_nonxdashxenergy_use_subheat(x):
    """
    historic nonxdashxenergy use subheat



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.001995, 0.001996, 0.001997, 0.001998, 0.001999, 0.002, 0.002001, 0.002002, 0.002003,
        0.002004, 0.002005, 0.002006, 0.002007, 0.002008, 0.002009
    ])


def efficiency_liquids_for_heat_plants(x):
    """
    efficiency liquids for heat plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.923745042449277, 0.8363533334993, 0.8318533392156451, 0.8343775097644487,
        0.8449037487785805, 0.8134700348415264, 0.8315091058175587, 0.8036477473316663,
        0.8199626156663135, 0.8201391471763312, 0.830453125951232, 0.8510030575113863,
        0.8534313700543216, 0.8527114298309938, 0.8308203086018318, 0.8312887451419698,
        0.831416361961128, 0.8371918262743113, 0.8177248627892834, 0.7866970642224647
    ])


def share_gasxdivxxcoalxplusxgasx_for_heat_plants(x):
    """
    share gasxdivxxcoalxplusxgasx for heat plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.6400078403045726, 0.625494609565936, 0.633011936961359, 0.6486922923750881,
        0.6469276462210707, 0.6138097737209008, 0.606109214138782, 0.5986175242634422,
        0.595281391596198, 0.586586261482702, 0.5447112921280477, 0.52717673519315,
        0.5032201982790275, 0.49668145545839515, 0.48668197474167624, 0.4775030096269512,
        0.47070644665043565, 0.44995202983813604, 0.3903674184032734, 0.382003505923756
    ])


def efficiency_coal_for_heat_plants(x):
    """
    efficiency coal for heat plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.8347276195018529, 0.7939170408846971, 0.8075329657040736, 0.7985649421904794,
        0.8053657541377044, 0.733731426164544, 0.7535839368782363, 0.7799860116948372,
        0.69841143160776, 0.7051976957278331, 0.7227781675786922, 0.7195061580692983,
        0.7293959273179971, 0.7378841063284353, 0.7364423437396291, 0.7004937762675217,
        0.7112918920827271, 0.665985886065352, 0.7079178145750791, 0.7145485392606492
    ])


def efficiency_gases_for_heat_plants(x):
    """
    efficiency gases for heat plants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.9002705165704274, 0.798015478374507, 0.8265508524517888, 0.8513851809893292,
        0.8366292510284139, 0.8071606916492732, 0.8075260241858225, 0.8116671100413593,
        0.7803729904585283, 0.8291140201011076, 0.8331308600977195, 0.8581487641371625,
        0.8638647108245096, 0.8652630305943033, 0.8470379015860756, 0.8354220854903446,
        0.8235722289925171, 0.7992649323212949, 0.7349857411765203, 0.7302838157453234
    ])


def invest_cost_res_elec_subhydro(x):
    """
    invest cost res elec subhydro



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            2.214812479617349, 2.214812479617349, 2.344999239047722, 2.4170522882922056,
            2.5259506468094357, 2.6168357430155447, 2.697076638765082, 2.697076638765082,
            2.697076638765082, 2.697076638765082, 2.697076638765082, 2.697076638765082
        ])


def invest_cost_res_elec_subgeotxdashxelec(x):
    """
    invest cost res elec subgeotxdashxelec



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            20.381187520382646, 10.190593760191323, 8.9042830742472, 7.519718230242417,
            5.9361887161648, 4.947096858354169, 4.254405044026525, 4.254405044026525,
            4.254405044026525, 4.254405044026525, 4.254405044026525, 4.254405044026525
        ])


def invest_cost_res_elec_subsolid_bioexdashxelec(x):
    """
    invest cost res elec subsolid bioexdashxelec



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            4.614670290248939, 2.3073351451244695, 2.0076599630394605, 1.9937406239808673,
            1.9462511142515488, 1.923325144037395, 1.904493097075769, 1.904493097075769,
            1.904493097075769, 1.904493097075769, 1.904493097075769, 1.904493097075769
        ])


def invest_cost_res_elec_suboceanic(x):
    """
    invest cost res elec suboceanic



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11.816700076095225, 5.908350038047613, 3.1867098597673653, 2.297509729318404,
            1.7669372757908466, 1.4754499402108923, 1.3141493640613107, 1.3141493640613107,
            1.3141493640613107, 1.3141493640613107, 1.3141493640613107, 1.3141493640613107
        ])


def invest_cost_res_elec_subwind_onshore(x):
    """
    invest cost res elec subwind onshore



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            2.4727296445265785, 1.2363648222632893, 1.0275747363843895, 0.8171470812044787,
            0.7794829872812261, 0.7418188933579735, 0.7319934775519076, 0.7319934775519076,
            0.7319934775519076, 0.7319934775519076, 0.7319934775519076, 0.7319934775519076
        ])


def invest_cost_res_elec_subwind_offshore(x):
    """
    invest cost res elec subwind offshore



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            4.74895097293184, 2.37447548646592, 1.801326231112077, 1.2609283617784537,
            1.1954255897380146, 1.088983585172301, 1.0685139689096637, 1.0685139689096637,
            1.0685139689096637, 1.0685139689096637, 1.0685139689096637, 1.0685139689096637
        ])


def invest_cost_res_elec_subsolar_pv(x):
    """
    invest cost res elec subsolar pv



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            6.134334601587128, 3.067167300793564, 2.137027937819328, 1.4541615392977496,
            0.8408918360691379, 0.6427459506468093, 0.6230951190346776, 0.6230951190346776,
            0.6230951190346776, 0.6230951190346776, 0.6230951190346776, 0.6230951190346776
        ])


def invest_cost_res_elec_subcsp(x):
    """
    invest cost res elec subcsp



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            11.872377432329598, 5.936188716164799, 4.56554321121861, 4.129949777149689,
            3.4904789651049017, 3.438895532123056, 3.406144146102837, 3.406144146102837,
            3.406144146102837, 3.406144146102837, 3.406144146102837, 3.406144146102837
        ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculture_hunting_forestry_and_fishing(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.7241971461530861, 0.708925072756311, 0.6998144878200208, 0.6785107560564287,
        0.6552600882169791, 0.6315664469531413, 0.6474383043815678, 0.6462555924915394,
        0.6362194771534165, 0.6269526899631803, 0.6287197169177513, 0.649448507837442,
        0.6773391674440454, 0.6565191729265971, 0.6750516890263
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submining_and_quarrying(x):
    """
    historic final energy intensity by sector and fuel subelectricity submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.5634910007468006, 1.5112702035759604, 1.5517158938879816, 1.575438640288508,
        1.4696896640657946, 1.3928937451492078, 1.5000345353002407, 1.5238317127411405,
        1.4923776372944797, 1.4713179539606795, 1.4377547101284724, 1.335297206200341,
        1.3719554839627304, 1.204262170936298, 1.3283417503967039
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfood_beverages_and_tobacco(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.3872589683473662, 0.38991463229558976, 0.39101143513263503, 0.39505465054488725,
        0.3933419255112612, 0.3999444285581746, 0.4126345462420507, 0.43200313279180014,
        0.4299559197047555, 0.4285299876839093, 0.42113855610917716, 0.41406466418672366,
        0.4119767790339999, 0.41251967360302194, 0.41162621691153284
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextiles_and_textile_products(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8098595818123617, 0.7809054314995713, 0.8129769521621621, 0.8081881969977484,
        0.784174020547582, 0.7971238777782144, 0.7883099437214302, 0.7795792702076949,
        0.7823568161223905, 0.8096815563070188, 0.7777888964711069, 0.7702650014841644,
        0.7564683224583868, 0.7000523728228109, 0.7024078973512976
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleather_leather_and_footwear(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.35139111305783044, 0.33316073109134436, 0.33808935659610473, 0.3301911458791342,
        0.32742257172175443, 0.3383774266000901, 0.3320929088833885, 0.33770250309350436,
        0.3328079754567813, 0.3515074970928847, 0.32595095501665117, 0.3104903316024953,
        0.29027356586922193, 0.259192861625059, 0.25757048727115656
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwood_and_products_of_woood_and_cork(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8729045156287489, 0.8803194465366069, 0.8585216005427833, 0.8732729973240205,
        0.8661234191346798, 0.8644024652870175, 0.8579514779557791, 0.8304499897127946,
        0.816855955036405, 0.8115243628395454, 0.6828909225936747, 0.6526672333479513,
        0.6585095983831069, 0.6816773281639819, 0.6777837932313371
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulp_paper_printing_and_publishing(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.2263768378705506, 1.2144155296983012, 1.197934329105796, 1.184390669935025,
        1.1712966433885488, 1.169997471480713, 1.1966292361936643, 1.1999943104244262,
        1.2203385654272842, 1.2225810264380397, 1.1766825924595492, 1.1650724058722093,
        1.1133423207226474, 1.0869767100370342, 1.0676209273986548
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcoke_refined_petroleum_and_nuclear_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.9165557410684781, 0.9358000116019126, 0.9243841164204407, 0.8953306635622691,
        0.8664440465758191, 0.9135066449439855, 0.9412770882258273, 0.9360752874602274,
        0.8827707875290411, 0.8565953567127532, 0.7948033487166073, 0.7413171643399493,
        0.7637328942332169, 0.6064451932608534, 0.7069123870027182
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicals_and_chemical_products(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.803552955165638, 1.8308315315621162, 1.7496041241839269, 1.6452475708012788,
        1.6101830348922872, 1.666450341608775, 1.6611052910033846, 1.6204107698126002,
        1.644341474418466, 1.6483042729285202, 1.6666473381010591, 1.6106763308171923,
        1.558536120192618, 1.5213063144943828, 1.514599748285027
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubber_and_plastics(x):
    """
    historic final energy intensity by sector and fuel subelectricity subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.7516755152937544, 1.824574493937183, 1.7920601389627309, 1.8067289091982912,
        1.8795825516807279, 1.8435548709997704, 1.7842275613004739, 1.845080435538212,
        1.6730108764548752, 1.6424496581101529, 1.7862606136200592, 1.706408159426469,
        1.6649464780881742, 1.6942052186029162, 1.7361101430096244
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_non_metalic_mineral(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.6010029593093376, 1.5932256091723684, 1.5699087462270658, 1.5672520333522502,
        1.5615298108178137, 1.576358038509453, 1.6431319189067426, 1.6958319558465424,
        1.7255318447745254, 1.7461744142710622, 1.7654612366537779, 1.791817576209471,
        1.770131164494374, 1.784319158537765, 1.9519753682288368
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasic_metals_and_fabricated_metal(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.9776155903879076, 1.9392195301196662, 1.9215450030205303, 1.9122783844372382,
        1.929118205864779, 1.9815693703488377, 2.009944479295412, 2.075131535173117,
        2.1513151629091998, 2.218317981076714, 2.199945744072929, 2.18378513331959,
        2.2290229109356408, 2.230707958542277, 2.19480244602311
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinery_nec(x):
    """
    historic final energy intensity by sector and fuel subelectricity submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.38497878559139365, 0.3561548057898775, 0.3415291275971809, 0.3427851610972068,
        0.3437487022843571, 0.3419753064945743, 0.351691064073765, 0.3675386907038594,
        0.3554827336970448, 0.3381230832880707, 0.3132625954950186, 0.2968054254027654,
        0.28813036805786, 0.2886452636675849, 0.32409542919292395
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectrical_and_optical_equipment(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.3881121835801443, 0.37575844467622455, 0.35774634718219034, 0.34943354468767224,
        0.34381343331070224, 0.3112718982059209, 0.30223301921643114, 0.2984296037773112,
        0.2794400085359985, 0.2601078257061289, 0.24895235432425467, 0.23047562719826115,
        0.2196316918765295, 0.2080015981423694, 0.21658242383320356
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransport_equipment(x):
    """
    historic final energy intensity by sector and fuel subelectricity subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.29157175769801413, 0.2932733502579648, 0.2865009021457406, 0.28318178927285437,
        0.2708520350396004, 0.3056591045379025, 0.30542284485819216, 0.2939825969509754,
        0.2861779021729641, 0.2932579238288578, 0.2660253656864477, 0.2523719403305147,
        0.23683425921684204, 0.2378185951613391, 0.26199150808256366
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturing_nec_recycling(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8569169761465573, 0.8694348704950727, 0.8818025593648565, 0.9096502751261356,
        0.8725065440725464, 0.8044003060524386, 0.7687321308791215, 0.8306392887445839,
        0.7500852931749243, 0.7215141505939332, 0.7922587022642156, 0.7822512856775996,
        0.7153254959608057, 0.814964939710505, 0.8170739106271032
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricity_gas_and_water_supply(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.6240585151440134, 2.5827547952978684, 2.6980829723166853, 2.6792657225987164,
        2.6552380668911906, 2.6264867297696766, 2.333247621523986, 2.5314112882795596,
        2.679668653346034, 2.6745633298685774, 2.6329365375141487, 2.7085624351709545,
        2.6564387303091666, 2.581028066820715, 2.8574728837212615
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(x):
    """
    historic final energy intensity by sector and fuel subelectricity subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.10596936611626993, 0.11752002197397564, 0.10637076833546745, 0.11940080040308412,
        0.1244365089556424, 0.11581265842963812, 0.09767988013188698, 0.10694419521117998,
        0.07564017908634847, 0.07964715523270356, 0.07981840107155418, 0.08144162509328723,
        0.08640696882604974, 0.0957116405787128, 0.10473826892065874
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.29023907436985996, 0.29546092453914086, 0.29982259435817693, 0.2985275103903131,
        0.29392740007325285, 0.27164651402310047, 0.29919715620305803, 0.2914474478372432,
        0.30414212771064614, 0.29680981845439275, 0.27965689364716184, 0.2851604486781958,
        0.2876977938199097, 0.3028584015902629, 0.3180725318361754
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.19819158488103156, 0.1988984695251443, 0.1902452549651085, 0.19114253279138624,
        0.20230726292600698, 0.20364517363751936, 0.20459780243338735, 0.21415082222077816,
        0.2131461017021144, 0.2073036112635289, 0.20399321708074358, 0.20639263406148797,
        0.20941044634889908, 0.22317011916600307, 0.19985183045602822
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.49852024433364905, 0.49076619915037845, 0.4876463096311946, 0.48853974638903147,
        0.5091177800892925, 0.4873674900783807, 0.48477778575155234, 0.49413782382381477,
        0.5021773483646942, 0.5071092493982671, 0.47008904095609366, 0.46117943887494817,
        0.48198387270778725, 0.5049321158097608, 0.47628496738666654
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotels_and_restaurants(x):
    """
    historic final energy intensity by sector and fuel subelectricity subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.6316259743579479, 0.6228922759356975, 0.6409051875756983, 0.6349520690961216,
        0.6313261976802199, 0.651029329488104, 0.6736644901263359, 0.7027213918199248,
        0.7192748013416302, 0.6926817749424463, 0.6975865146363074, 0.7203197000945538,
        0.7392532435237646, 0.7956355255567737, 0.8162340685124673
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinland_transport(x):
    """
    historic final energy intensity by sector and fuel subelectricity subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.4483452062377024, 0.45286590216598044, 0.4451464125518518, 0.4250024788940665,
        0.41169122950481013, 0.4262861053691009, 0.4192292284879077, 0.4322862683192847,
        0.44932075861220566, 0.44433341086508094, 0.42814418035587165, 0.4182680008927667,
        0.417896236209432, 0.40666224043170296, 0.426985238182552
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwater_transport(x):
    """
    historic final energy intensity by sector and fuel subelectricity subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.796435777226835e-05
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subair_transport(x):
    """
    historic final energy intensity by sector and fuel subelectricity subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.3351250982416472, 0.3172017860190052, 0.31166535903931797, 0.3122641457070816,
        0.31651534903017065, 0.31346968682389725, 0.3367378703234646, 0.3585816344470207,
        0.362608758443261, 0.3258887564484779, 0.32369825391559337, 0.3429122781162016,
        0.3442326706594128, 0.36049764467512, 0.385387817272461
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpost_and_telecommunications(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.2142453248427915, 0.21499985706883995, 0.22267555471748215, 0.21380570808085655,
        0.21358389938913094, 0.22572066945040173, 0.23543921322671155, 0.25414285942284653,
        0.27141218788839566, 0.26605874009786207, 0.2603283205745559, 0.26302026795246414,
        0.25757215250376087, 0.2678675035433738, 0.260025663572746
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancial_intermedation(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.1357129444444695, 0.1335025986264839, 0.13347355787905524, 0.13530342504336876,
        0.1332398516261878, 0.13077685809779627, 0.11743851754411798, 0.1160974931940181,
        0.12211180403401836, 0.12151574900706145, 0.1160599929351302, 0.10825026748583594,
        0.1273302306890166, 0.13839558030319435, 0.1314090560744755
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subreal_estate_activities(x):
    """
    historic final energy intensity by sector and fuel subelectricity subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.31331706620363864, 0.3265824867186806, 0.3271085731423654, 0.3272119796580536,
        0.3049079343857554, 0.3466851022546009, 0.34590615723305, 0.3448615879622379,
        0.34449325697322003, 0.4120247607578594, 0.4612870638020768, 0.4424942059188907,
        0.3033982102598411, 0.23581865395159607, 0.27833046224886804
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrenting_od_meq_and_other_business_activities(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.18155751478579876, 0.18895369439380297, 0.1917845876674018, 0.19816334008250425,
        0.19857039813316948, 0.1913312708049842, 0.18326721091232706, 0.17244465134086473,
        0.17894389359752372, 0.17355814915740955, 0.17303117423193054, 0.1690768138532979,
        0.18005979606460235, 0.18671243674251325, 0.19378581229435143
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublic_admin_and_defence_compulsory_social_security(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.40839951722917894, 0.41521087071191465, 0.428197889891699, 0.4285825726564223,
        0.4201005975115219, 0.41268027268550234, 0.41211831677424504, 0.4760328358825686,
        0.4927290785345975, 0.4784253547244492, 0.45089277838492575, 0.46473727616096494,
        0.5449616826567257, 0.5187732225130455, 0.5261113823348827
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(x):
    """
    historic final energy intensity by sector and fuel subelectricity subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.47589911961923304, 0.48810317329631847, 0.5010530845658214, 0.5051103590478777,
        0.4922273153673258, 0.5185140868132246, 0.5384892892118762, 0.5537962899548067,
        0.5845769195084838, 0.58478878910022, 0.5987198528777099, 0.6291527202343252,
        0.6913116423440042, 0.7187108804786225, 0.7413783491439477
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealth_and_social_work(x):
    """
    historic final energy intensity by sector and fuel subelectricity subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.3360550520069014, 0.3319971823881577, 0.3409749985532885, 0.3457400169586935,
        0.3412237146366023, 0.3511642658000085, 0.36708612847876365, 0.36523506999109473,
        0.35815461591948927, 0.3366687390387799, 0.33902602916666025, 0.34971618344751637,
        0.35870597911304736, 0.37563059560620543, 0.3927018157309736
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_community_social_and_persona_services(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.3878879751275172, 0.3708698504162547, 0.38061467555067985, 0.39955675996821305,
        0.37762190742153284, 0.39091172102920724, 0.41155004608034385, 0.40927380105050193,
        0.4264253858524306, 0.40805678697158126, 0.4090799499066392, 0.41687604647378884,
        0.4272759179642891, 0.4509644858352208, 0.4522371698072469
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivate_households_with_employed_persons(
        x):
    """
    historic final energy intensity by sector and fuel subelectricity subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculture_hunting_forestry_and_fishing(
        x):
    """
    historic final energy intensity by sector and fuel subheat subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.1779432025330486, 0.13870566072139295, 0.12082321976272868, 0.10766561537513031,
        0.09597024560745618, 0.09182242973574563, 0.08539721418532935, 0.0787292046287861,
        0.07030452612026813, 0.06269616690801379, 0.06307201187077975, 0.06065281467706288,
        0.058431192052661673, 0.05410842896927197, 0.0510942878679188
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_submining_and_quarrying(x):
    """
    historic final energy intensity by sector and fuel subheat submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.39615757379217825, 0.32878335690737376, 0.3362121913754084, 0.34030321699873817,
        0.3253708905799867, 0.30352427099269813, 0.3190183924402979, 0.31766176005583435,
        0.29892007500937756, 0.2821306927832654, 0.31331686569242123, 0.3033988711082321,
        0.24857012656222557, 0.20262862281637783, 0.2166198242522173
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subfood_beverages_and_tobacco(x):
    """
    historic final energy intensity by sector and fuel subheat subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.1649647280255494, 0.13923080611190244, 0.1263507918573515, 0.1277449510541924,
        0.12716106154203174, 0.11618948111680057, 0.1142084176253826, 0.1168006772309306,
        0.12312181692947147, 0.12095713287918461, 0.11552633351364591, 0.12292787322827238,
        0.11748827453727254, 0.11982507444312444, 0.12010630655390092
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextiles_and_textile_products(x):
    """
    historic final energy intensity by sector and fuel subheat subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.22086369963658975, 0.17406359415856623, 0.16224183065657455, 0.16693572325962322,
        0.14221222136468667, 0.16381674657242878, 0.16831927283258077, 0.16035462473979567,
        0.1699009243369815, 0.19637915352632893, 0.20538678486804926, 0.2042770960192051,
        0.20725324806843504, 0.19506434139198559, 0.18757549668212872
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subleather_leather_and_footwear(x):
    """
    historic final energy intensity by sector and fuel subheat subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.11953127415721884, 0.09474854527802631, 0.08764839600586773, 0.09160695304828317,
        0.07675379152382777, 0.08074204323896955, 0.0765580679520774, 0.07607364176815419,
        0.08070889556854588, 0.08007746298837058, 0.08548371486588444, 0.08390383293604126,
        0.0784175115061452, 0.07111850342724534, 0.07220969773204038
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subwood_and_products_of_woood_and_cork(
        x):
    """
    historic final energy intensity by sector and fuel subheat subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8793952550681494, 0.6058544020694085, 0.5334020352587955, 0.5153793131116994,
        0.5871102874575236, 0.6203211143932192, 0.6313759985796766, 0.6439431104796545,
        0.6336153779824492, 0.6116323831845747, 0.16268003128165623, 0.1751541793275517,
        0.17069373931579726, 0.17493073288251104, 0.17875505194786806
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulp_paper_printing_and_publishing(
        x):
    """
    historic final energy intensity by sector and fuel subheat subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.10098146124858108, 0.11673687052005599, 0.1065235889017262, 0.10299216552866995,
        0.1043649214113851, 0.09959438674903104, 0.10092439867341914, 0.1148304535785617,
        0.16569911801129128, 0.18942512330308361, 0.2771436820991434, 0.30306283049683014,
        0.27831023548626566, 0.26327814535913896, 0.27018420717455527
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subcoke_refined_petroleum_and_nuclear_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subheat subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8306255389091878, 0.6867102923723846, 0.6398357709177271, 0.6697612735545528,
        0.670925349487634, 0.69573506267167, 0.6784280946749888, 0.6479712802547783,
        0.6406490162492339, 0.6332790793123346, 0.7910253303240071, 0.8147667435369287,
        0.8170829812904382, 0.6381195823715806, 0.7219641210498305
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicals_and_chemical_products(
        x):
    """
    historic final energy intensity by sector and fuel subheat subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8584367275321372, 0.7397852128919687, 0.66724787357296, 0.6203922862106979,
        0.6560905475232538, 0.6819427269570311, 0.6776761148129726, 0.686752258681959,
        0.7212402945801206, 0.7175417810760435, 0.7353749926714853, 0.7432752979283427,
        0.708328418290724, 0.6675915872266838, 0.6697430685859749
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubber_and_plastics(x):
    """
    historic final energy intensity by sector and fuel subheat subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.45484858353019414, 0.4169535131947727, 0.34233489349153934, 0.2979234015680134,
        0.30308181830473974, 0.3345339758495406, 0.35123840932746825, 0.35016180267711977,
        0.3665131168273066, 0.3592264431339666, 0.35219533350061194, 0.29932860571513153,
        0.27516433128575574, 0.2608095184633447, 0.24392179773689737
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_non_metalic_mineral(x):
    """
    historic final energy intensity by sector and fuel subheat subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.11474364843841015, 0.1015156829539818, 0.2177311622717772, 0.19260112324974013,
        0.18604035907517102, 0.1777772888849531, 0.1742377266923505, 0.16987213626105857,
        0.17492400397096394, 0.16052398993714187, 0.14392398218341645, 0.1410041738280214,
        0.12626025452818337, 0.12437860569413856, 0.1287548346990869
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasic_metals_and_fabricated_metal(
        x):
    """
    historic final energy intensity by sector and fuel subheat subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.44447170498746297, 0.3740641211103763, 0.3443018955107859, 0.3497366773559753,
        0.34266848220142215, 0.33541431019446977, 0.34253539572158626, 0.3375977700887783,
        0.3578912584020226, 0.3332142470071909, 0.2401490143563816, 0.2341118588902698,
        0.22060202401252918, 0.2110469999353281, 0.22324946339634189
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinery_nec(x):
    """
    historic final energy intensity by sector and fuel subheat submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.2663694279283176, 0.22834274038650992, 0.19136133527590732, 0.1567241483829092,
        0.13627395112125568, 0.12207404682352825, 0.13086844378406912, 0.12703448169356632,
        0.13194301068066228, 0.11608788997022071, 0.0448765141282129, 0.04099402725968257,
        0.03927945818919396, 0.04207330253739464, 0.04949099593229013
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectrical_and_optical_equipment(
        x):
    """
    historic final energy intensity by sector and fuel subheat subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.10697241151915922, 0.08110770964139015, 0.06775288279050015, 0.05798327242155274,
        0.04813151588123405, 0.04356663913750887, 0.04610646694846743, 0.049813041729922375,
        0.05209170202061897, 0.0462007411792655, 0.030935037153437074, 0.026442576907990945,
        0.021417439255133085, 0.01861581660746257, 0.01916486673716104
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransport_equipment(x):
    """
    historic final energy intensity by sector and fuel subheat subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03230247009715763, 0.03365905354505978, 0.027996316297390002, 0.02328166049397064,
        0.016752929925473788, 0.020291977858137314, 0.0213013464890133, 0.0228524453077512,
        0.033935283609928715, 0.031736018731842015, 0.05254221452168104, 0.04936251156866958,
        0.04186236037475353, 0.04464775304925383, 0.04994551000737595
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturing_nec_recycling(x):
    """
    historic final energy intensity by sector and fuel subheat submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.2524742276675683, 0.2770783096876561, 0.2892463743676505, 0.2726238912320256,
        0.20248756829133865, 0.23533342968400625, 0.23530529422681773, 0.247149639979618,
        0.19238247594215038, 0.198232738346037, 0.36604614345951936, 0.29344140426572635,
        0.2457290709739713, 0.20145663662856353, 0.195326355095624
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricity_gas_and_water_supply(
        x):
    """
    historic final energy intensity by sector and fuel subheat subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.17656446185316851, 0.19723181996517808, 0.18347204565651665, 0.19266562232704065,
        0.1947154183161916, 0.1618948285114104, 0.1630513827742791, 0.21828024089209305,
        0.22847274873040754, 0.22888674862368502, 0.25089241522740796, 0.3033529090412146,
        0.257287023612773, 0.21861331518289215, 0.23420824672911003
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction(x):
    """
    historic final energy intensity by sector and fuel subheat subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.08379636901874359, 0.05912016941951932, 0.026342470308196652, 0.021830112391916626,
        0.0177824602263644, 0.016762388622089782, 0.017243465798204186, 0.01683030360991656,
        0.017512769640440632, 0.01567768728733379, 0.012056202327930386, 0.011630321576189033,
        0.01092204974650565, 0.009948286060340346, 0.009952056661525526
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subheat subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03437430257370856, 0.03248580789445889, 0.0270797425335092, 0.02779519490656119,
        0.03217994573339533, 0.026495515235646395, 0.026497911781729094, 0.024298545909817323,
        0.03498663904464091, 0.035730772189885705, 0.03929017979841124, 0.03797602390213039,
        0.033945298554452795, 0.03266934670788762, 0.03173735550499495
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic final energy intensity by sector and fuel subheat subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.042371359971517655, 0.05596603289438831, 0.04526361343825246, 0.04894881026177591,
        0.058810711388934206, 0.04360433804948141, 0.036269972299407226, 0.03565722771814744,
        0.03291479330176479, 0.037072826274117766, 0.04706699112577181, 0.044500239569127034,
        0.04317299982273536, 0.04155025444429505, 0.039743093028138735
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic final energy intensity by sector and fuel subheat subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.038548253304305724, 0.04543492604717159, 0.03797619948818973, 0.038324294809383125,
        0.04647061847730034, 0.03601597769139513, 0.03304018345041006, 0.031854062061745764,
        0.03309240552870057, 0.035111994469669665, 0.040624476649071505, 0.04005480654467303,
        0.036903519610464, 0.0361959235250515, 0.03583695395061817
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotels_and_restaurants(x):
    """
    historic final energy intensity by sector and fuel subheat subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.05421856516261138, 0.06721093729720135, 0.05909171500281141, 0.06368378192849475,
        0.05173149970655956, 0.041638870297647956, 0.04098533755316226, 0.04087420088691245,
        0.04173290319937963, 0.04143908662406254, 0.05174077473653161, 0.05231967402658624,
        0.05148103706063099, 0.05301062747112252, 0.0511994977092531
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subinland_transport(x):
    """
    historic final energy intensity by sector and fuel subheat subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subwater_transport(x):
    """
    historic final energy intensity by sector and fuel subheat subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subair_transport(x):
    """
    historic final energy intensity by sector and fuel subheat subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic final energy intensity by sector and fuel subheat subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.24766113572697843, 0.1433899454448171, 0.12316507729787073, 0.13796832107982962,
        0.12429463594134163, 0.08049699793469801, 0.07487965639843151, 0.08137504711026855,
        0.08083071239759364, 0.05079584375909763, 0.07694141579440167, 0.08051152446751678,
        0.07359313523684147, 0.08391793791851684, 0.08941672292834295
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subpost_and_telecommunications(x):
    """
    historic final energy intensity by sector and fuel subheat subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03630472699066584, 0.053799886422011715, 0.04635085848000384, 0.04920255698920714,
        0.04471989075620264, 0.028167882760934248, 0.026121487945967616, 0.02833649680492858,
        0.02612764861482731, 0.024237634423599087, 0.028316188776702115, 0.028717708371679678,
        0.02677222898860627, 0.02554407567267204, 0.02464708011310464
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancial_intermedation(x):
    """
    historic final energy intensity by sector and fuel subheat subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.030423145843399658, 0.01665834393959276, 0.013724581645353938, 0.013857964113641258,
        0.016876689956649222, 0.014085984707775272, 0.01454129496610741, 0.015620240851815416,
        0.01145640746873261, 0.011402731070671244, 0.01498877870550671, 0.013780569855714568,
        0.013772255699667259, 0.014489989788855498, 0.013042316407139366
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subreal_estate_activities(x):
    """
    historic final energy intensity by sector and fuel subheat subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.06748503295992885, 0.052712807858196084, 0.04472599605940692, 0.046509173529932196,
        0.0504066297130954, 0.03849224135054137, 0.0360052279616092, 0.03715113372307803,
        0.032936513579305195, 0.032487350660944145, 0.043093058601061336, 0.04439623389273437,
        0.04488968655828661, 0.04391661441880346, 0.04894407742230376
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subrenting_od_meq_and_other_business_activities(
        x):
    """
    historic final energy intensity by sector and fuel subheat subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.02293190808390416, 0.02262243635032486, 0.02096783126347895, 0.021200116426919404,
        0.022831201389171465, 0.017908038621137316, 0.016612837616866733, 0.015609073846054325,
        0.017273430847816965, 0.016751498672525022, 0.021446576044583145, 0.019672326657890368,
        0.019251023600707035, 0.017985852686757987, 0.018232748381631827
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublic_admin_and_defence_compulsory_social_security(
        x):
    """
    historic final energy intensity by sector and fuel subheat subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.05140637380793385, 0.07822085710082032, 0.07133029723763389, 0.06891092689533442,
        0.06304826454808282, 0.05149168732541779, 0.0418730835245787, 0.04009143398935635,
        0.04430766757577621, 0.04199334476524284, 0.05229449894692725, 0.051383289960517796,
        0.049446733721309644, 0.045065768676422174, 0.042066680726921296
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation(x):
    """
    historic final energy intensity by sector and fuel subheat subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.07013227324556913, 0.08104441544299763, 0.08272705449847896, 0.07810516179875657,
        0.06825900240482347, 0.06472244536514574, 0.0637069998901736, 0.06583975893681501,
        0.07683234666824508, 0.07429560990252872, 0.086218901772325, 0.08224517387718157,
        0.07588073999609178, 0.07548265200675518, 0.06914624772308771
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealth_and_social_work(x):
    """
    historic final energy intensity by sector and fuel subheat subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.04920998564494497, 0.05141091949811415, 0.04964417414865848, 0.047972326347685064,
        0.04261668454896623, 0.038718065794993886, 0.036951395118389394, 0.033341191049112706,
        0.04012472109111849, 0.03942842881355761, 0.048905735335520575, 0.04624978583898568,
        0.04175277696184828, 0.04141778786473526, 0.0392434145450208
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_community_social_and_persona_services(
        x):
    """
    historic final energy intensity by sector and fuel subheat subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.11996458634785939, 0.08592983866536506, 0.07838672881777277, 0.08568639893345602,
        0.07897091899630367, 0.0683711232726671, 0.0631457274135446, 0.060414224408576064,
        0.08316301095814016, 0.07813626750086931, 0.09398520749492556, 0.08550533473728855,
        0.0850786293713881, 0.08126627081667989, 0.07499198502816463
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivate_households_with_employed_persons(
        x):
    """
    historic final energy intensity by sector and fuel subheat subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculture_hunting_forestry_and_fishing(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3.329337118908045, 3.330675684050808, 3.3056876138324642, 3.1199531685254867,
        3.0241068932783115, 2.913297719495583, 2.9134123728860097, 2.8418511504522566,
        2.864071301035634, 2.9121439587818303, 2.8766896473028623, 2.8180790690079416,
        2.7650218349970377, 2.716188095273943, 2.6291186340338735
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_submining_and_quarrying(x):
    """
    historic final energy intensity by sector and fuel subliquids submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.341362582961613, 1.3031888788382189, 1.430442507336978, 1.3586221542833017,
        1.2683375830030548, 1.1904792850612582, 1.3060180798221765, 1.3125273787385903,
        1.2362762669247982, 1.2020515629065311, 1.243794594549829, 1.1279633790764654,
        1.1888499764514466, 1.1056716925600512, 1.1957022090658815
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfood_beverages_and_tobacco(x):
    """
    historic final energy intensity by sector and fuel subliquids subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.5141106312905513, 0.603598529324621, 0.5875630312879945, 0.5580747247388643,
        0.5415400295426829, 0.5264542344804909, 0.5176174853298501, 0.5357280165325976,
        0.5053332256371244, 0.5050467405535006, 0.4624651436890585, 0.43955110647063045,
        0.4211821670459373, 0.43365936067900906, 0.43695490528792386
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextiles_and_textile_products(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8806921388281855, 0.8756214954360507, 0.8493598989798039, 0.849401584399211,
        0.8177730902051842, 0.7683661837902472, 0.7385580752297171, 0.7266538185593405,
        0.685886679576849, 0.6750174885763368, 0.5772313619964377, 0.4942478870481967,
        0.4380942904533427, 0.3984498823683011, 0.39044188652557815
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleather_leather_and_footwear(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.543912206399191, 0.5340187940829365, 0.5116496072212218, 0.5283172564053502,
        0.52680124791453, 0.5160920116238285, 0.4839876013888461, 0.4903718807301217,
        0.47278823455177815, 0.48650732038204203, 0.4456619609063785, 0.38300531633542345,
        0.34493136665614327, 0.33242252290944396, 0.32373966589948894
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwood_and_products_of_woood_and_cork(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8531844182955877, 0.8718623558860619, 0.8991039249103806, 0.8887123996273485,
        0.8400586168150834, 0.7360387172883827, 0.7629195554611713, 0.7178919576542717,
        0.674971768143922, 0.6969505117942627, 0.628846884172979, 0.5627407956041065,
        0.5205881484627026, 0.5945847854329732, 0.5734254205861374
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulp_paper_printing_and_publishing(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.7725248579617263, 0.7829920825601699, 0.71588470498281, 0.6658319712890726,
        0.6109643718413529, 0.6275512605419433, 0.6274433630557608, 0.6317304071588181,
        0.6232452688814072, 0.6055655728152876, 0.5941347693276107, 0.5273776248355037,
        0.4675760180751636, 0.44067615705964414, 0.4127060249099261
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcoke_refined_petroleum_and_nuclear_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        11.7231108444282, 11.564060149591148, 11.340352092537033, 11.563164910084271,
        10.864766657178281, 10.78734421352214, 10.422426170714616, 10.13479762834825,
        10.646299993259092, 9.876892067841245, 9.28312948240861, 8.218957060695121,
        8.251613356505768, 6.446731934830551, 7.192514001861613
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicals_and_chemical_products(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.6141045942860075, 1.6909110512942793, 1.572229862535497, 1.5122922448899447,
        1.444844708751819, 1.6245028626437592, 1.6194445026822364, 1.5954316796075378,
        1.5077553991312074, 1.4466087946389843, 1.4173573214290192, 1.3253081564925795,
        1.1517646687971657, 1.1532075599936515, 1.147877898612611
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubber_and_plastics(x):
    """
    historic final energy intensity by sector and fuel subliquids subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.07496675931583, 2.0471013607839823, 2.1060098875889803, 2.0250070773864786,
        1.9256772798330721, 1.7744313808070888, 1.8326844748437345, 1.8302390102895039,
        1.7403618599700157, 1.7490444809705308, 1.6296472792826477, 1.5410729153480742,
        1.523797656583321, 1.4965650486610333, 1.5970725518602833
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_non_metalic_mineral(x):
    """
    historic final energy intensity by sector and fuel subliquids subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.1960551224028317, 2.209369279652865, 2.201003613289866, 2.2289873773480102,
        2.182166433525797, 2.125796639185664, 2.135941163866843, 2.1895788315277644,
        2.0588368623674085, 2.149389932401747, 2.044124977828056, 1.9443217660087966,
        1.7825558242556516, 1.796657802289093, 1.784985510455616
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasic_metals_and_fabricated_metal(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.7241195062411848, 0.7375635494333631, 0.7032144025710197, 0.6719289119030403,
        0.6670717802584506, 0.6116986317199548, 0.5740454082643064, 0.5817006667890227,
        0.5871598157245603, 0.5681759516759873, 0.5279323510489545, 0.48096034933659615,
        0.4313959142384465, 0.42793579021142536, 0.3892597105315791
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinery_nec(x):
    """
    historic final energy intensity by sector and fuel subliquids submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.42382675349627796, 0.43122964165918515, 0.4024212807620231, 0.3864226855321285,
        0.3651561958518691, 0.32735529906108074, 0.3280616894183554, 0.33543565446199,
        0.3015467633221359, 0.26741820406262334, 0.24156298078554952, 0.21311040005375667,
        0.1874255971936674, 0.18467487948687933, 0.19970663635058034
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectrical_and_optical_equipment(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.4780198382356848, 0.4656733928129168, 0.42951861678564707, 0.40692686532943895,
        0.3501942085262995, 0.2948534837509099, 0.3024074848311568, 0.2965072485549675,
        0.2596846206533788, 0.24143762875863975, 0.2025018460671438, 0.17409890423093224,
        0.15556729617604012, 0.15031456794290005, 0.16111787204701797
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransport_equipment(x):
    """
    historic final energy intensity by sector and fuel subliquids subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.28330009685739516, 0.28893704651021057, 0.2773930693280514, 0.28170554050566016,
        0.2640356328569094, 0.26617313675337534, 0.26283685026058545, 0.2564223272621323,
        0.230220430382433, 0.20440510127256015, 0.18441103774116221, 0.164381507678766,
        0.13916350423569543, 0.13915089157188737, 0.14861014064157796
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturing_nec_recycling(
        x):
    """
    historic final energy intensity by sector and fuel subliquids submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.1439231260394123, 2.1822559556740875, 2.1474962473193537, 2.07879094938377,
        1.972911085545422, 1.8619188110702514, 1.9425922821998154, 1.9697176408084243,
        2.0376165598787592, 1.9143895352629738, 1.9587050999351112, 1.828934743240394,
        1.7881466556469001, 1.7799236844600743, 1.536870590861347
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricity_gas_and_water_supply(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.6479000510051925, 0.7888352005533726, 0.4910380502554513, 0.762775895251622,
        0.8909714193880099, 0.7699091727081196, 0.6930243309961277, 0.7477250122028472,
        0.5764056024097187, 0.6500826340495188, 0.596504106990596, 0.6436601191645978,
        0.6877840796316987, 0.5217922438419683, 0.4799829085953009
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction(x):
    """
    historic final energy intensity by sector and fuel subliquids subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.7068140570746776, 0.7227705762434552, 0.723014885894517, 0.7203116642657159,
        0.7079479952465305, 0.6676336719201289, 0.6906288420159953, 0.707442981637406,
        0.6912606246613766, 0.7418413135750408, 0.7067213863020272, 0.6910126127831341,
        0.6757742886564271, 0.6529588055898928, 0.6673692259969749
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.6516105473732899, 0.7067331641687331, 0.7052237313922668, 0.7032266073227602,
        0.6584109402587034, 0.5688049730923108, 0.5965215562557896, 0.5980643302533083,
        0.6041771342169036, 0.5388948338236673, 0.4964532411883088, 0.4691391675535737,
        0.4521656701888638, 0.4559478455247383, 0.4802210060653841
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.7995096107996469, 0.8252303745364935, 0.7798495117539251, 0.7444435424859365,
        0.72262496148166, 0.6588557039169136, 0.6603278340733744, 0.6678129315618117,
        0.6274976663337006, 0.5761831477921462, 0.5227289399539211, 0.4870441351630275,
        0.4405275466176488, 0.4212832080799799, 0.41068203325280145
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.446224555441523, 1.4643807252621894, 1.4226249680023841, 1.401696186693699,
        1.3626051781347446, 1.2573436676006837, 1.26578256600391, 1.2930398860925554,
        1.2494822721142014, 1.152885617016206, 1.018537732900704, 0.9540939035898561,
        0.9361325265437349, 0.9200090487527909, 0.9301466718050068
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotels_and_restaurants(x):
    """
    historic final energy intensity by sector and fuel subliquids subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.4196391790703387, 1.3406818361044612, 1.2874351683880576, 1.2413950506121258,
        1.2026812023761946, 1.1181780600825535, 1.1472675222149598, 1.1918494467876615,
        1.172418278082716, 1.1060507580712087, 1.0084245382646344, 0.9600751466958125,
        0.9295922641309979, 0.9201921195598188, 0.9410191119656305
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinland_transport(x):
    """
    historic final energy intensity by sector and fuel subliquids subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        5.784594651904558, 5.841882003967407, 5.839617640577593, 5.7234514917421215,
        5.657425050322958, 5.805799163252883, 5.663783687522083, 5.750384611283826,
        5.571938823049814, 5.823024459391582, 5.905534741704541, 5.866070650980922,
        5.986263477165527, 5.783156576760606, 6.039976824724781
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwater_transport(x):
    """
    historic final energy intensity by sector and fuel subliquids subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        37.934160872041296, 36.99229763627306, 35.857425977728354, 32.96034180361318,
        31.445947900920526, 29.763612221163157, 26.048593193983724, 25.861329225695492,
        25.45709435265915, 25.418232855341213, 25.38070613356822, 25.818633973914036,
        25.508559284206285, 23.604923335035554, 23.628451979670505
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subair_transport(x):
    """
    historic final energy intensity by sector and fuel subliquids subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        26.715731790170462, 26.56880019698801, 26.328612139720242, 26.223829302523733,
        26.197997404918336, 26.067380154584512, 26.7898761094474, 27.715378037620063,
        26.232774917101356, 26.350842653034938, 27.148378588319208, 25.86595493563425,
        25.117966478688643, 24.98087409172215, 25.085688402244163
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.1229444350330606, 2.1858031309710717, 2.1320401162060647, 2.055422526573931,
        2.374243138577253, 2.370472430671425, 2.283326869757632, 2.380527825919046,
        2.310300684442889, 2.3597546052719696, 2.478585956426773, 2.4043233217469235,
        2.516730364247273, 2.433282842599178, 2.4998329347889965
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpost_and_telecommunications(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.768866469814047, 0.7045417318922991, 0.7194975287130991, 0.686119642007903,
        0.7092963917543954, 0.654541940164107, 0.6347548762949851, 0.5634405956580067,
        0.5496258714368878, 0.5691231126948875, 0.5039055491247418, 0.4743621645845341,
        0.4544160511189765, 0.4363370772707986, 0.42985268631370466
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancial_intermedation(x):
    """
    historic final energy intensity by sector and fuel subliquids subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.3902641535503575, 0.39105057361588785, 0.36928625271596555, 0.3590196832786975,
        0.3311330705068184, 0.2928685985146136, 0.2917779755835705, 0.29868082636364557,
        0.2902083684045119, 0.26600477549860785, 0.23489360829412367, 0.21865593642713602,
        0.2028196283561895, 0.19794825830942292, 0.18964237442788068
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subreal_estate_activities(x):
    """
    historic final energy intensity by sector and fuel subliquids subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.1409964245790585, 0.14439320564947894, 0.14063164520472649, 0.13509166535454392,
        0.1288958348609235, 0.11690815269898912, 0.11608242310919194, 0.11967779600337494,
        0.12119155409114757, 0.11743942535448604, 0.12178078876475229, 0.12048154605461525,
        0.11657170991065724, 0.09798634547528803, 0.09797353805266568
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrenting_od_meq_and_other_business_activities(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8331509374789908, 0.829491361635125, 0.8049435201879306, 0.7970443962992084,
        0.7720927405758682, 0.7200246935712061, 0.709344111552235, 0.7133550252479071,
        0.6943767987662407, 0.6622543272614511, 0.595351967333654, 0.5644181014297519,
        0.5320028562634156, 0.5176487865353288, 0.5129037303887946
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublic_admin_and_defence_compulsory_social_security(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.188054173776091, 1.2133749217514826, 1.2293234237135575, 1.2591481738533437,
        1.23382876392151, 1.1449859659635278, 1.177309625404061, 1.0967878132962667,
        1.0769636298652006, 1.0096199030034558, 0.9373540473198093, 0.8892619382789926,
        0.8470477962264747, 0.8214546226563212, 0.8399946748583467
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation(x):
    """
    historic final energy intensity by sector and fuel subliquids subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.8557400891731884, 0.8946873392113956, 0.8930235708093455, 0.863600839879686,
        0.8440303344467589, 0.8331950040741682, 0.8489841047121085, 0.8783471403593619,
        0.9017617765416281, 0.8783858725603914, 0.8578588702110131, 0.84812637305127,
        0.8251310688591592, 0.8420228859112069, 0.8500830847935721
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealth_and_social_work(x):
    """
    historic final energy intensity by sector and fuel subliquids subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.0899935382020292, 1.0794120603730522, 1.0621939904932913, 1.08098506648033,
        1.0309042029489681, 0.9656209501278881, 0.9759940916694444, 1.0121111492679689,
        0.990627177098384, 0.9236746612908852, 0.8266047105195358, 0.7931182690161734,
        0.753621853534988, 0.7451840021094017, 0.7538561418115993
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_community_social_and_persona_services(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.1493987832688457, 1.1737860075312971, 1.1936830991238718, 1.166987638888675,
        1.1358084209938066, 1.0397880554918142, 1.0555258409763975, 1.0882492817835265,
        1.0919568464681755, 1.0506014179378114, 0.9811287055745656, 0.947410230772165,
        0.9201585543705517, 0.9257677200705718, 0.9278880445636408
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivate_households_with_employed_persons(
        x):
    """
    historic final energy intensity by sector and fuel subliquids subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculture_hunting_forestry_and_fishing(
        x):
    """
    historic final energy intensity by sector and fuel subgases subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.2150356857860538, 0.23808543700535903, 0.2249828547013219, 0.21602014550226922,
        0.25889017338562903, 0.3259500278002751, 0.30655078994607454, 0.29941530067408895,
        0.2930062742460066, 0.2847165229116941, 0.28312899573704253, 0.2654783961694343,
        0.27608551847790697, 0.26682611690254887, 0.2602340811751907
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_submining_and_quarrying(x):
    """
    historic final energy intensity by sector and fuel subgases submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        4.69078137659984, 4.674680491324511, 5.587114925579444, 6.1914402689816, 5.643124554829877,
        4.7890750283167876, 5.175069902516305, 5.305256393623547, 5.391187388350845,
        5.160359018472073, 4.822047105186946, 4.677087534518218, 4.783909980508121,
        4.2016196787038345, 4.66277570791695
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subfood_beverages_and_tobacco(x):
    """
    historic final energy intensity by sector and fuel subgases subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.2630712135501656, 0.27482351735400984, 0.27176046941280213, 0.2497142239689453,
        0.22989966178122906, 0.3600879841515737, 0.3413860103813444, 0.3547360068253931,
        0.36246315037251453, 0.3521274036529059, 0.32192202882420695, 0.3346373730931401,
        0.33396681864562816, 0.34726851971209743, 0.3339346560772236
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextiles_and_textile_products(
        x):
    """
    historic final energy intensity by sector and fuel subgases subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.22170152798157228, 0.2237974799341098, 0.22194372593124323, 0.2088333392472178,
        0.2078195342464195, 0.37449650739686546, 0.3586887970392119, 0.35716516554601063,
        0.3522560996360123, 0.3372593373609412, 0.2703351189299814, 0.2807123319032572,
        0.23938080094397174, 0.22992445335417921, 0.23320030953411783
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subleather_leather_and_footwear(x):
    """
    historic final energy intensity by sector and fuel subgases subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.09354931025043114, 0.08569200202644255, 0.07304451191760464, 0.07136564883324445,
        0.07142556681890348, 0.13274807570637964, 0.12685016213423064, 0.1330872415275701,
        0.1352147717183019, 0.13128698330208274, 0.11008525329131127, 0.11462176247199041,
        0.08361741992091336, 0.08401051515893582, 0.08239602053061558
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subwood_and_products_of_woood_and_cork(
        x):
    """
    historic final energy intensity by sector and fuel subgases subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.16492387728107027, 0.16491848452458757, 0.17214935197061923, 0.15540987441108312,
        0.1865545041278503, 0.29532803353441145, 0.2736965354434537, 0.27654305363553544,
        0.25044299075958315, 0.2658742371032976, 0.24614252504824943, 0.2469193165307909,
        0.2211512839987132, 0.24082680160621636, 0.24657379451326117
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulp_paper_printing_and_publishing(
        x):
    """
    historic final energy intensity by sector and fuel subgases subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.4924434997862261, 0.5035167282353104, 0.4653216859317899, 0.416591668347514,
        0.4086323731686502, 0.613317658719408, 0.5591768596748578, 0.5785423929231583,
        0.5518005251519551, 0.5041909188261511, 0.46497936329519063, 0.48430099601694454,
        0.465593045195173, 0.4675388580899925, 0.4605113243815074
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subcoke_refined_petroleum_and_nuclear_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subgases subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3.001313091203896, 3.0136097520997716, 3.0187830548072707, 3.0860044083105795,
        2.7827190716347494, 2.8832010213385604, 2.5713728213822455, 2.55786837768559,
        2.58934072303863, 2.3350796608400692, 2.3657057558749415, 2.127615858724935,
        2.2773583161358864, 1.801941232901524, 2.0365279736233033
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicals_and_chemical_products(
        x):
    """
    historic final energy intensity by sector and fuel subgases subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.4720283824991145, 1.5052003901403945, 1.227394683360337, 1.0698578280829782,
        0.9930600957911819, 1.7935253915292202, 1.6220540746157186, 1.6494410859171633,
        1.5954078706124257, 1.4787828774506468, 1.4343100769756831, 1.3652166052244967,
        1.372267493452264, 1.443397904881445, 1.4393643503734865
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubber_and_plastics(x):
    """
    historic final energy intensity by sector and fuel subgases subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.0293817265575258, 0.9634590176256144, 0.8084872172503001, 0.7447988463130435,
        0.7574406019185975, 1.1443065304809965, 1.1112397447609332, 1.1054866360049462,
        1.0595606692803112, 0.8905151550364396, 0.916813834866961, 0.9561836159085504,
        1.026366866964444, 1.020582712688694, 0.988506771535562
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_non_metalic_mineral(x):
    """
    historic final energy intensity by sector and fuel subgases subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.3434388215425517, 1.3321895549088403, 1.2478116049949717, 1.1401006559973619,
        1.0459696896128106, 1.6702388734857427, 1.6313622184199437, 1.616916337047098,
        1.6330751267939065, 1.6658242098361133, 1.7203220587904575, 1.7494487873793174,
        1.5969640730489476, 1.7302326094995535, 1.6461143264900595
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasic_metals_and_fabricated_metal(
        x):
    """
    historic final energy intensity by sector and fuel subgases subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.3108065204731356, 1.2970726797702823, 1.223246411855429, 1.1573621173483866,
        1.0881253181024007, 1.3578255707345974, 1.2821081524399458, 1.3184894219974421,
        1.2981874351653735, 1.3748165507613535, 1.344758899655073, 1.2838653063511276,
        1.2020320290082778, 1.2277708817077104, 1.1200065077483736
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinery_nec(x):
    """
    historic final energy intensity by sector and fuel subgases submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.12021541503389496, 0.125859005200925, 0.11693052615703729, 0.10233769702234845,
        0.10751502036095155, 0.16047071251418615, 0.16435763359568387, 0.17573359055047366,
        0.13896859663809286, 0.12862940828249575, 0.13033325871434107, 0.12927142408537162,
        0.12075807835585903, 0.11131003163056657, 0.1228715703315172
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectrical_and_optical_equipment(
        x):
    """
    historic final energy intensity by sector and fuel subgases subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.11143834233419135, 0.10446346973068762, 0.08630328151060919, 0.07603249406013288,
        0.07347651364949342, 0.10598282339115185, 0.10454045158487307, 0.10568435481581273,
        0.09458777586901591, 0.08087214110403276, 0.0779417468457911, 0.0773382557605845,
        0.07139255213764506, 0.06749995759716539, 0.0696880791985734
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransport_equipment(x):
    """
    historic final energy intensity by sector and fuel subgases subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.09171080390030475, 0.09674881322946037, 0.09240595736273877, 0.08242188677387538,
        0.07992416080483881, 0.1317976929305965, 0.12381969027745805, 0.12022025094727935,
        0.11739312755701244, 0.11216561087233788, 0.10393196609250903, 0.11603835704046012,
        0.0995192919370611, 0.10302801213369138, 0.11017171508322544
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturing_nec_recycling(x):
    """
    historic final energy intensity by sector and fuel subgases submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.6361904215163183, 0.5768012848405862, 0.49606560248822923, 0.4791899670272098,
        0.4655241319213859, 0.7072263003043257, 0.6779642867657234, 0.7087227274542818,
        0.7253802448834674, 0.6980047177438519, 0.6557236985272097, 0.7780231182652662,
        0.6918367741023013, 0.7164942889097418, 0.6922034067991938
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricity_gas_and_water_supply(
        x):
    """
    historic final energy intensity by sector and fuel subgases subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        3.922396867876367, 3.8908805002198408, 3.960153185544289, 3.83014763381815,
        3.9460710609040603, 3.774894877799922, 3.626795439041342, 3.6828147873277035,
        3.6692242539163367, 3.316258634594196, 3.1406659378887456, 3.125622734727022,
        3.0467002692300236, 2.719112027570859, 2.731191712620855
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction(x):
    """
    historic final energy intensity by sector and fuel subgases subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0576678281130595, 0.05759797955978964, 0.054849216142609805, 0.0552265188459029,
        0.05300736053494253, 0.05613113180406631, 0.05618862193148195, 0.06099338784248549,
        0.05319371454310903, 0.05768052438354068, 0.0628364091240574, 0.0717340226442291,
        0.06750868575973777, 0.06593784904217394, 0.06379826136672472
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subgases subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.12338711663998722, 0.14156085039019767, 0.13373724297590503, 0.13413682440414731,
        0.12139209363720632, 0.10245712863372658, 0.11672220331164067, 0.10247692561851296,
        0.1206959374120446, 0.11796345098172378, 0.10455470303966023, 0.11496503227562352,
        0.10979747716211825, 0.1152407626679457, 0.11355533230389225
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic final energy intensity by sector and fuel subgases subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.08311735701445912, 0.08264550858150513, 0.07434285189320597, 0.06999926533913718,
        0.07061226806353163, 0.0716951075551002, 0.07334576145974414, 0.05856954203692449,
        0.06790223122416063, 0.06816169525522552, 0.07017975751497046, 0.07055662220598093,
        0.06856935642669208, 0.06884983984819669, 0.059109336003401826
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic final energy intensity by sector and fuel subgases subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.18343784780840566, 0.17571147382553004, 0.18011471116892394, 0.17822724063783765,
        0.18225544144230094, 0.18442893092936644, 0.19182988574399967, 0.16835349960126303,
        0.1890527790350421, 0.2018756033970964, 0.177562069213027, 0.18024065768526512,
        0.1817884994152512, 0.18541308729004072, 0.17702047126196366
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotels_and_restaurants(x):
    """
    historic final energy intensity by sector and fuel subgases subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.2296887595143569, 0.24545601899441238, 0.23698316507221934, 0.22727014618673852,
        0.21806857343434236, 0.21680620576486417, 0.23817525049187382, 0.1975478906533931,
        0.23029168731264182, 0.27893354750435984, 0.28591236839346135, 0.31385282829780764,
        0.3331125790232874, 0.36885491612934235, 0.4056754915204717
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subinland_transport(x):
    """
    historic final energy intensity by sector and fuel subgases subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.768689072259116, 1.7917046309320366, 1.6435054662707695, 1.7138042008227812,
        1.6384909990256893, 1.5335878589335115, 1.5632852095573495, 1.6604083471917965,
        1.7233067613550759, 1.706074719516254, 1.7138893365378558, 1.662875867498881,
        1.6927180417809842, 1.7030920439173627, 1.5852983930085915
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subwater_transport(x):
    """
    historic final energy intensity by sector and fuel subgases subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        4.574847164421825e-06, 3.5083317099837725e-06, 2.2543503767533515e-06,
        3.17598044517727e-06, 3.0609406155923296e-06, 4.179549733011051e-06, 4.791593519426679e-06,
        5.593053392243618e-06, 6.2620557715434685e-06, 5.3582774971173225e-06,
        5.569080572749993e-06, 4.6301136778513e-06, 2.8986228116751754e-06, 2.3681513939620776e-06,
        2.6750270062826567e-06
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subair_transport(x):
    """
    historic final energy intensity by sector and fuel subgases subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.98484033210949e-05, 1.7923726095086716e-05, 1.605546819730422e-05,
        1.4401849284183056e-05, 1.2420497571094495e-05, 1.7167074083988788e-05,
        2.034013678030099e-05, 2.1669727520137526e-05, 2.1755735666445098e-05,
        2.460157375607659e-05, 2.572173276240859e-05, 2.4062924673815406e-05,
        1.828053757134078e-05, 1.700543257260789e-05, 1.8955822151764034e-05
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic final energy intensity by sector and fuel subgases subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.15920033013924226, 0.1462653412657254, 0.13750798057984082, 0.12679160517105934,
        0.11254947017031008, 0.0992373868312057, 0.10003573898651195, 0.09807307747323457,
        0.098822003010246, 0.0958235680501819, 0.1026264908600997, 0.11059539187753116,
        0.10686343674581243, 0.10442853400674075, 0.10464609635555286
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subpost_and_telecommunications(x):
    """
    historic final energy intensity by sector and fuel subgases subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.557114158189985, 0.5472730387659961, 0.5197418530500941, 0.48040480838419714,
        0.463796507686431, 0.19502066451585254, 0.2289077295900135, 0.17540867590327613,
        0.1928580587552419, 0.19487285803924426, 0.18958196691492568, 0.17019557521741102,
        0.16233097499278262, 0.175412440542782, 0.19572299833922094
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancial_intermedation(x):
    """
    historic final energy intensity by sector and fuel subgases subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.04635997696533132, 0.046744865231611685, 0.0447351069202782, 0.04268199059411815,
        0.04019711180265761, 0.04058553299336325, 0.04038838748973608, 0.03749500409530591,
        0.041477197913039505, 0.04384957478665145, 0.04314727817008194, 0.03987437403323019,
        0.04170324486339083, 0.04301901300787914, 0.04127577120538779
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subreal_estate_activities(x):
    """
    historic final energy intensity by sector and fuel subgases subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.08985636789862816, 0.08216238775043112, 0.07728302303128676, 0.07620240719739438,
        0.08203065255604013, 0.08084037631457545, 0.07381013913588647, 0.07785242082615067,
        0.07619088399937314, 0.07697874791338906, 0.07792588444866497, 0.07842749110707001,
        0.08563504721208058, 0.07975016800990146, 0.07296546325240949
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subrenting_od_meq_and_other_business_activities(
        x):
    """
    historic final energy intensity by sector and fuel subgases subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.09772309728227796, 0.11115803062770635, 0.10580241841803352, 0.10681667576834784,
        0.10522864842044777, 0.09467934401641757, 0.10213827710360436, 0.07833873328067115,
        0.09223915867588768, 0.09732601885051016, 0.09079701149562185, 0.08828570175789931,
        0.09002758585564463, 0.09515527299978106, 0.10356363499920908
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublic_admin_and_defence_compulsory_social_security(
        x):
    """
    historic final energy intensity by sector and fuel subgases subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.5373951654266507, 1.5773355485211076, 1.6069214033563568, 1.5581521796463662,
        1.5924836438541974, 0.8946457984749805, 0.7860677446553317, 0.995986989631449,
        0.9270969437849531, 0.920655350885897, 0.8808926606088301, 0.8257451964081314,
        0.8606127743772815, 0.8316784911656734, 0.7942589878807084
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation(x):
    """
    historic final energy intensity by sector and fuel subgases subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.19922448291807177, 0.23531238535474305, 0.2231415545653652, 0.22317933842060514,
        0.21438305202709548, 0.20330385315308613, 0.20638467532306168, 0.20166091895972965,
        0.21881425132286253, 0.23146364374082085, 0.22149602406457594, 0.21929110510209326,
        0.21676984237117738, 0.2211725111125796, 0.21082169303149165
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealth_and_social_work(x):
    """
    historic final energy intensity by sector and fuel subgases subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.16141074562287921, 0.1732020915342724, 0.1657377513264254, 0.15836996028351982,
        0.14672777699115497, 0.1409050090986358, 0.1482148290757661, 0.11775767279203651,
        0.13644975758106215, 0.14151464690529184, 0.1413414886206319, 0.1476061657474311,
        0.14650608522130928, 0.15076300318769348, 0.1530470898697172
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_community_social_and_persona_services(
        x):
    """
    historic final energy intensity by sector and fuel subgases subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.24354639533712438, 0.22410397697909837, 0.2181196495176965, 0.21745540669238772,
        0.20644271381163165, 0.20161633674608678, 0.19625973853525425, 0.1554216693170553,
        0.1772699128910812, 0.1885584053327525, 0.19696970570613992, 0.19897657587795384,
        0.20464005931279272, 0.20326798159486942, 0.1974065126411919
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivate_households_with_employed_persons(
        x):
    """
    historic final energy intensity by sector and fuel subgases subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculture_hunting_forestry_and_fishing(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subagriculture hunting forestry and fishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.47086767577643307, 0.4689734098478973, 0.46425486815666095, 0.44534418746398285,
        0.3761325320228421, 0.3551956503752216, 0.3365821032904595, 0.34446052697705376,
        0.3369613067779433, 0.3777420374226403, 0.37787484702445184, 0.3437689486231854,
        0.33878148476263686, 0.2577659589029426, 0.254453630249139
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_submining_and_quarrying(x):
    """
    historic final energy intensity by sector and fuel subsolids submining and quarrying



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.9506886637716337, 1.086630598111202, 1.01667597588606, 1.0660191521961055,
        0.7868691739470693, 0.6526677911564813, 0.7204889529796701, 0.7089010456405578,
        0.7794095503774782, 0.9704793933712182, 0.9823892491919919, 0.9278313661875002,
        1.0833070867907952, 1.1106331881813756, 1.1870484516524065
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfood_beverages_and_tobacco(x):
    """
    historic final energy intensity by sector and fuel subsolids subfood beverages and tobacco



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.047944772544057, 1.0360568758730864, 1.014018726524927, 1.037066032050261,
        0.9815967965758055, 0.8843014622198658, 0.8913773668345029, 0.9258706928918075,
        0.9257269945592518, 0.9338086140710012, 0.9319479467345817, 0.9313754300283074,
        0.9057922750003461, 0.9381713242043601, 0.9704476184444321
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextiles_and_textile_products(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subtextiles and textile products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.9919552339550126, 0.847724044256255, 0.8186147769677025, 0.81062296508303,
        0.679675887864844, 0.5982756067962458, 0.576790088518787, 0.5340863948470222,
        0.5622607319189838, 0.5959511163236807, 0.5801706727339709, 0.5566949002161229,
        0.624951990189146, 0.5381622642204315, 0.49724291289232403
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleather_leather_and_footwear(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subleather leather and footwear



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.36017316114149805, 0.30773968477634617, 0.2849058765356348, 0.28389198839865315,
        0.2758287894515175, 0.2705123915880374, 0.28642317546372925, 0.2592775000171222,
        0.25206032689756724, 0.2709501807246148, 0.2663224865185713, 0.2287097852786095,
        0.2384944850405983, 0.21190362815894584, 0.20248687709043212
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwood_and_products_of_woood_and_cork(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subwood and products of woood and cork



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.543584669525891, 1.5713183395016395, 1.646331221104433, 1.795836426642832,
        1.7003984320277263, 1.6450077961022413, 1.5214839862171476, 1.460250568308033,
        1.5076450931321594, 1.490623203594703, 1.4882085700158616, 1.507689366543973,
        1.4345046259573466, 1.340937777405623, 1.4033225078918963
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulp_paper_printing_and_publishing(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subpulp paper printing and publishing



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.4159664231910054, 1.304235051194609, 1.281397330192007, 1.2932604822767144,
        1.7279845074651576, 1.79209690458253, 1.6591041409759142, 1.6661724225835628,
        1.6987801938092408, 1.7486162038976443, 1.7029419945596787, 1.7735393640433785,
        1.6853550268307096, 1.672671289783725, 1.6711461492308324
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcoke_refined_petroleum_and_nuclear_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subcoke refined petroleum and nuclear fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.5635188463251101, 0.5851743616352155, 0.644313184666882, 0.5956834437835982,
        0.49924952325982, 0.43173984930216935, 0.43175921769540554, 0.4496436343012353,
        0.5259551145590667, 0.5010401981173834, 0.5140242524355785, 0.4598391323689899,
        0.5220070922514527, 0.4924544615127271, 0.5813028667048447
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicals_and_chemical_products(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subchemicals and chemical products



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.6903382545784063, 1.71695887393953, 1.469956704294383, 1.4895419256445153,
        1.2626857708375467, 1.1688347660484393, 1.0871156670944135, 0.9774906791790546,
        0.9984063908017585, 0.8767153852524797, 0.8983008834549645, 0.9166930108086413,
        0.893563172950253, 1.0391958069307874, 1.0175599074126027
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubber_and_plastics(x):
    """
    historic final energy intensity by sector and fuel subsolids subrubber and plastics



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.737440360502975, 2.653168017704157, 2.3746478495781074, 2.301560235678491,
        2.1343633621673237, 2.2543433691103147, 2.362199596156166, 2.2395715789595965,
        2.2315149801129435, 2.5658474287552555, 2.3535271983433366, 2.1753693103690255,
        1.9161742444311836, 2.0402872988069793, 2.339955454982643
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_non_metalic_mineral(x):
    """
    historic final energy intensity by sector and fuel subsolids subother non metalic mineral



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        7.997229698191975, 7.898914325545515, 7.359390241247763, 7.592310249488967,
        6.762910078485197, 6.149652094708481, 5.861348640263628, 5.853044728617025,
        6.3261226753336075, 7.468790661464871, 7.079585118618744, 6.69047655316014,
        6.401360089133524, 7.701962886349043, 8.312149498212824
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasic_metals_and_fabricated_metal(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subbasic metals and fabricated metal



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.8148348986360774, 2.7453111311363547, 2.62872708237162, 2.659907145169566,
        2.5387101246366677, 2.4665722660034266, 2.469370038397535, 2.500267643907507,
        2.576380342176518, 2.6695802148310936, 2.7472797340606054, 2.5883819547704148,
        2.5827382350380548, 2.5709281525712395, 2.740962862203819
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinery_nec(x):
    """
    historic final energy intensity by sector and fuel subsolids submachinery nec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.34965201090783354, 0.3485096842068839, 0.279054906117444, 0.2722305779495829,
        0.21136922436306088, 0.15806235724746237, 0.1484902355772645, 0.14741882829007508,
        0.1384083188884344, 0.13466797467401953, 0.13256471647883097, 0.12398225671433248,
        0.11652380828759189, 0.13134038990693306, 0.16728426817745956
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectrical_and_optical_equipment(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subelectrical and optical equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.08366552295776288, 0.07699981553840246, 0.06369621210588272, 0.058340254239730174,
        0.042325918786184684, 0.0320595402306469, 0.02807478003124616, 0.027376385420920114,
        0.02927390039219211, 0.024577044028479374, 0.025339117161324965, 0.025543833065601596,
        0.02342428699557163, 0.023911288618159554, 0.026689358813394036
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransport_equipment(x):
    """
    historic final energy intensity by sector and fuel subsolids subtransport equipment



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.11490861193939238, 0.1081115484591803, 0.09782318939953644, 0.09946326835018271,
        0.08621479823797218, 0.06788637710209482, 0.058334866072842516, 0.05555683566291642,
        0.048682890764563565, 0.059841856401587394, 0.06601679929344707, 0.06934989494004637,
        0.06723161358908392, 0.07580187120558012, 0.09639910951746274
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturing_nec_recycling(x):
    """
    historic final energy intensity by sector and fuel subsolids submanufacturing nec recycling



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        2.1260014275788457, 2.057377428407371, 2.125381313616282, 1.9796899098853875,
        1.8710438855509501, 1.4687557873678239, 1.4589131498308834, 1.4739066155637164,
        1.700472171062823, 1.130894749465082, 1.2406698372753777, 1.1490739423944445,
        1.2665482132346975, 1.2983931028176303, 1.0499525592289427
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricity_gas_and_water_supply(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subelectricity gas and water supply



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        1.3084750756044377, 0.8104891430027502, 1.4617037707221665, 1.4740996259925323,
        1.4532688376916307, 0.7903411399721437, 0.6739217324429169, 0.6471981342381012,
        0.5778798448053956, 1.1868744116093666, 1.2377147803870046, 1.493035799268893,
        1.4076289630531469, 1.9610455757353344, 2.473662882947921
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction(x):
    """
    historic final energy intensity by sector and fuel subsolids subconstruction



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0806173098110524, 0.08594746821812554, 0.08227770261546485, 0.08790092227513241,
        0.082592651911312, 0.07803037174132649, 0.07631604424840002, 0.07855546769792085,
        0.08064242615211846, 0.09077341072803267, 0.0935716874675732, 0.09678117644493296,
        0.09358490192627616, 0.09520594589281828, 0.09338286988860646
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subsale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.025721291024246726, 0.016083306734592066, 0.014490815894267404, 0.011709581795349921,
        0.011843814344314, 0.006437423982575871, 0.006038090062378964, 0.006924385237484932,
        0.005840258138144695, 0.005757907828806607, 0.006912255105327055, 0.006751246171271431,
        0.006345110988541257, 0.016012871441109883, 0.014688616766985556
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subwholesale trade and commissions trade except of motor vehicles and motorcycles



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.03537462347797025, 0.027669882015807064, 0.019705301927951002, 0.017788346538106767,
        0.017375946586304866, 0.01627434845823217, 0.01727836710245665, 0.017746088363663474,
        0.01741101783195977, 0.01560001702399118, 0.01671682497154882, 0.015361347240106649,
        0.015576581908921505, 0.023605606963863403, 0.02326698711777169
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subretail trade except of motor vehicles and motorcycles repair of household goods



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.037316453987752074, 0.03091046696966835, 0.024031385122695918, 0.019608921509661207,
        0.02168496086862959, 0.015667218303222808, 0.014353440929141027, 0.01681562607212218,
        0.01606014700682523, 0.017763624047269853, 0.017737844896048568, 0.01563874466158938,
        0.015034232682292148, 0.02157103204247284, 0.019494781005226815
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotels_and_restaurants(x):
    """
    historic final energy intensity by sector and fuel subsolids subhotels and restaurants



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.4722702682067137, 0.47460825688986324, 0.47383107688548043, 0.3703955333570004,
        0.37102085338660346, 0.35307411559939295, 0.33593866092970665, 0.3338866063826925,
        0.3216559203857432, 0.3142122903875573, 0.2967407036504295, 0.2814820577560224,
        0.27497346761202845, 0.30930676588570116, 0.4039358176250789
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinland_transport(x):
    """
    historic final energy intensity by sector and fuel subsolids subinland transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.19289283581954494, 0.16574545130667276, 0.19524776114800166, 0.18703942896412112,
        0.16542357718750053, 0.14393693859557874, 0.1314425924974032, 0.13094386215497808,
        0.1269476862589329, 0.10145820290830995, 0.09604043673468537, 0.08083495467828675,
        0.07216014172557603, 0.06770309824135796, 0.06913765881252633
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwater_transport(x):
    """
    historic final energy intensity by sector and fuel subsolids subwater transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.020257549859789746, 0.019448944791747077, 0.019280458523735264, 0.01762807010271867,
        0.016233261604574407, 0.015221325496242827, 0.017148001015014576, 0.009232557504386006,
        0.008250926485561576, 0.008534341220736591, 0.011991602081475282, 0.010329416483131851,
        0.010122791173485413, 0.008017104407259123, 0.009792350532875372
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subair_transport(x):
    """
    historic final energy intensity by sector and fuel subsolids subair transport



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0, 3.7320077908385635e-09, 2.2663659904102352e-08, 1.8129530502928767e-09,
        8.158887940594626e-10, 1.247536137308539e-08, 1.8922947444433446e-08,
        5.832519874703696e-08, 4.1394275468854056e-08, 0.0, 9.204725637811062e-08, 0.0, 0.0, 0.0,
        1.4927273666771165e-06
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subother supporting and auxiliary transport activities activities of travel agencies



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.0493350265929109, 0.029880114079265312, 0.021769099877484484, 0.020264698559728334,
        0.017857791999022384, 0.014372833892743186, 0.011835549572757174, 0.01408806635254586,
        0.01675283806156665, 0.01713560079288289, 0.023635420648333905, 0.025905630959936757,
        0.028382091125843972, 0.035127368310808114, 0.03583322404994238
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpost_and_telecommunications(x):
    """
    historic final energy intensity by sector and fuel subsolids subpost and telecommunications



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.008802154580489769, 0.006849775384173932, 0.006373370813379648, 0.006583077487266303,
        0.008034321437860639, 0.0073363880967828735, 0.007681985254192262, 0.008404733845272091,
        0.007886118279475168, 0.007305669706869622, 0.006415787721319419, 0.005497382571308583,
        0.004051243085745235, 0.005356114156355983, 0.00632167139261299
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancial_intermedation(x):
    """
    historic final energy intensity by sector and fuel subsolids subfinancial intermedation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.014962966037717462, 0.006999623262715352, 0.0036871410062793044, 0.0033891821903891508,
        0.0034949956176403957, 0.0034969511236112604, 0.0034045480124904604, 0.004572338604714762,
        0.003151566761297772, 0.002744336513450184, 0.003656388017117741, 0.003261585586787987,
        0.002924517223465735, 0.004112842705433558, 0.0035234857215375435
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subreal_estate_activities(x):
    """
    historic final energy intensity by sector and fuel subsolids subreal estate activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.05228868249614063, 0.03642181330610362, 0.022236844264576505, 0.020547510389879235,
        0.02261782112835534, 0.021273067203708175, 0.019078580107547097, 0.0203027669092925,
        0.01799919562097267, 0.017683685455269195, 0.018537828954760974, 0.01700557316702225,
        0.012207164859604358, 0.014836353679504581, 0.014944020301411355
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrenting_od_meq_and_other_business_activities(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subrenting od meq and other business activities



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.029830788986078677, 0.029845876171776614, 0.02113408923374507, 0.019246162349765873,
        0.01892142091938055, 0.017096721854260245, 0.01609687839592735, 0.015985049597349442,
        0.017093125327429747, 0.01745848677226531, 0.016628019140136795, 0.015934870158277582,
        0.015999333840099075, 0.024502755541621134, 0.029755588523413663
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublic_admin_and_defence_compulsory_social_security(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subpublic admin and defence compulsory social security



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.07497030892925732, 0.07517673240762333, 0.05697302605983134, 0.0518955977723396,
        0.05837898098001085, 0.059572159062379205, 0.05560159332740479, 0.06700976660980495,
        0.07550049342595831, 0.08251023861209508, 0.08367537095400508, 0.07903576498544398,
        0.07918587821401651, 0.09242157665491013, 0.08212842751731904
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation(x):
    """
    historic final energy intensity by sector and fuel subsolids subeducation



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.1919927279978261, 0.19636605203264995, 0.13655203248504122, 0.13531732732379906,
        0.12183360291423968, 0.11352346481802379, 0.10736468368656304, 0.09991975953695116,
        0.0969676728724115, 0.08726517762316098, 0.07728313027204972, 0.07702355401757097,
        0.06538195677412448, 0.11079695373357354, 0.1145460059306778
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealth_and_social_work(x):
    """
    historic final energy intensity by sector and fuel subsolids subhealth and social work



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.05863354785334826, 0.04942708233077913, 0.040557906871152904, 0.03822182114509443,
        0.034823706400789224, 0.030804019895711035, 0.02692162160782073, 0.02454673774799809,
        0.02701696026679188, 0.0301557121060672, 0.033701894426510906, 0.040421025987261654,
        0.047543910250616266, 0.07589495099367287, 0.0789741172107411
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_community_social_and_persona_services(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subother community social and persona services



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [
        0.06819527064185982, 0.04535426311241609, 0.0293864371017948, 0.026153967022636167,
        0.04590856884905141, 0.04269116062911706, 0.033670880404642486, 0.04238727073336363,
        0.0704906307634303, 0.06313010917804748, 0.05951784163038456, 0.055528082109709026,
        0.052090285210014645, 0.08120230568637982, 0.08137642990063773
    ])


def historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivate_households_with_employed_persons(
        x):
    """
    historic final energy intensity by sector and fuel subsolids subprivate households with employed persons



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def cpxdashxini_res_elec_subhydro(x):
    """
    cpxdashxini res elec subhydro



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            0.42302360615086493, 0.42302360615086493, 0.42302360615086493, 0.42302360615086493,
            0.35794305135842425, 0.332, 0.332, 0.332, 0.332, 0.332, 0.332, 0.332
        ])


def table_max_extraction_uranium_user_defined(x):
    """
    table max extraction uranium user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def userxdashxdefined_extraction_growth_unconv_gas(x):
    """
    userxdashxdefined extraction growth unconv gas



    lookup


    """
    return functions.lookup(x, [
        2015, 2020, 2025, 2030, 2035, 2040, 2045, 2050, 2055, 2060, 2065, 2070, 2075, 2080, 2085,
        2090, 2095, 2100
    ], [
        0.19, 0.18, 0.17, 0.16, 0.15, 0.14, 0.13, 0.12, 0.11, 0.1, 0.09, 0.08, 0.07, 0.06, 0.05,
        0.04, 0.03, 0.02
    ])


def userxdashxdefined_extraction_growth_unconv_oil(x):
    """
    userxdashxdefined extraction growth unconv oil



    lookup


    """
    return functions.lookup(x, [
        2015, 2020, 2025, 2030, 2035, 2040, 2045, 2050, 2055, 2060, 2065, 2070, 2075, 2080, 2085,
        2090, 2095, 2100
    ], [0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0])


def historic_unconv_oil(x):
    """
    historic unconv oil



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
    ], [
        0.8042547447027982, 0.8410083449405368, 0.882574107072071, 0.9251795264521996,
        0.9725192749523464, 1.0602575075341525, 1.1056776788632727, 1.2903463301151419,
        1.4172025337438179, 1.4737016614421428, 2.6459843451365654, 2.7048717825313453,
        3.129671545010238, 3.4697200909205494, 3.9555142957287224, 3.9560890176996315,
        4.200681334685477, 4.568217289793923, 5.212758598196126, 5.557878988219476,
        6.383971244410107, 7.882661254058065, 10.105037981577686
    ])


def historic_unconv_gas(x):
    """
    historic unconv gas



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012
    ], [
        0.38917863237784056, 0.5687418653410531, 0.7882043095790647, 1.0097858800546946,
        1.1442239446187303, 1.2779344700159663, 1.3548112305831623, 1.4597823428573315,
        1.640969796322726, 1.6869119403665676, 1.7427730974322593, 1.9657612858624163,
        2.0760554776445383, 2.2113588514111604, 2.448984753192105, 2.6410048917620728,
        3.011593523318122, 3.6728311149610593, 4.893861654707023, 5.979963799647268,
        8.601989803490744, 11.335682793297288, 13.461287001815634
    ])


def historic_pop(x):
    """
    historic pop



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014, 2015, 2016
    ], [
        5709757338.0, 5792568347.0, 5875398158.0, 5957237460.0, 6038067278.0, 6118075293.0,
        6197638117.0, 6276824417.882873, 6356259573.898927, 6436346997.991372, 6517020798.243966,
        6598421257.484986, 6680423046.990929, 6763745673.295534, 6847214549.110201,
        6930656699.362227, 7012843635.357816, 7097400665.077233, 7182860114.610083,
        7268986175.739162, 7355220411.68203, 7442135578.0
    ])


def balancing_costs_ref(x):
    """
    balancing costs ref



    lookup


    """
    return functions.lookup(x, [
        0.0, 0.1, 0.2, 0.30000000000000004, 0.4, 0.5, 0.6, 0.7, 0.7999999999999999,
        0.8999999999999999, 0.9999999999999999
    ], [
        0.0, 1.516461136385635, 3.03292227277127, 4.549383409156905, 4.549383409156905,
        4.549383409156905, 4.549383409156905, 4.549383409156905, 4.549383409156905,
        4.549383409156905, 4.549383409156905
    ])


def historic_gtl_production(x):
    """
    historic gtl production



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.06346222, 0.058381449999999994, 0.06496829, 0.06506953, 0.06767736, 0.064206, 0.05606117,
        0.04747241000000001, 0.025892259999999997, 0.033874730000000006, 0.03802368, 0.06179034,
        0.058625540000000004, 0.042497019999999996, 0.08330459, 0.10185614, 0.10121208, 0.13211313,
        0.15293263, 0.14861201999999998, 0.15202329, 0.14950447, 0.24690301, 0.29162875,
        0.31618224, 0.0
    ])


def historic_ctl_production(x):
    """
    historic ctl production



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        0.16499057, 0.16768147, 0.21825412, 0.24450376000000001, 0.24462182999999998, 0.26293196,
        0.27309483, 0.27772338, 0.26207404, 0.26722632, 0.30062303999999995, 0.15679415,
        0.16186449, 0.16539379999999998, 0.13935132, 0.10021173, 0.14281665, 0.11917675999999999,
        0.09999644, 0.11247093, 0.14480621, 0.2200387, 0.22243912, 0.2406164, 0.27127102000000003,
        0.0
    ])


def historic_produc_biofuels_2gen(x):
    """
    historic produc biofuels 2gen



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        7096.883524626101, 7971.694528739449, 7707.278339633647, 7846.760877745217,
        8705.079406958428, 9052.844108405054, 9171.927311788611, 10472.882598545473,
        9937.08737060958, 9619.477252460889, 9178.36595626787, 10024.948550755016,
        11835.252519246962, 14686.710467070612, 16445.720935966816, 19650.71469184889,
        25666.356544617964, 34701.5349030663, 46454.13782417379, 51905.07940132504,
        59604.69080832337, 61162.8030389672, 62278.46999420314, 67260.47281761839,
        74207.51498164036, 74847.36659353007
    ])


def table_max_extraction_tot_agg_oil_laherrere_x2006x(x):
    """
    table max extraction tot agg oil laherrere x2006x



    lookup


    """
    return functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200, 13800, 14400, 15000, 15600, 16200, 16800
    ], [
        0.0, 54.326237263, 77.92436275200001, 98.78275612, 115.24967286600001, 130.06932745699999,
        140.497709169, 151.475382009, 163.45282174999997, 171.0, 177.460153, 182.146242,
        187.686421656, 189.07350399999999, 189.324922862, 189.324922862, 189.324922862,
        189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862,
        189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862, 189.324922862
    ])


def table_max_extraction_maggio12_high_conv_oil_ej(x):
    """
    table max extraction maggio12 high conv oil ej



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500, 15000, 15500, 16000, 16500, 17000
    ], [
        0.0, 15.6384, 43.7316, 66.2483, 87.3472, 103.416, 119.408, 131.57, 140.057, 150.257,
        156.932, 162.683, 167.462, 171.239, 173.998, 175.733, 176.452, 176.452, 176.452, 176.452,
        176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452, 176.452,
        176.452, 176.452, 176.452, 176.452, 176.452
    ])


def table_max_extraction_maggio12_low_con_oil_ej(x):
    """
    table max extraction maggio12 low con oil ej



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500
    ], [
        0.0, 31.19257370050454, 55.93036702223348, 78.9742737866738, 98.99541997370072,
        112.90238993902808, 129.7869030062217, 139.05477860121135, 149.65885530140235,
        155.85226158136214, 160.213586322676, 162.5352378599971, 162.89303662558495,
        162.89303662558495, 162.89303662558495, 162.89303662558495, 162.89303662558495,
        162.89303662558495, 162.89303662558495, 162.89303662558495, 162.89303662558495,
        162.89303662558495, 162.89303662558495, 162.89303662558495, 162.89303662558495,
        162.89303662558495
    ])


def table_max_extraction_maggio12middle_conv_oil_ej(x):
    """
    table max extraction maggio12middle conv oil ej



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_extraction_unconv_oil_user_defined(x):
    """
    table max extraction unconv oil user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_extraction_uranium_zittel12(x):
    """
    table max extraction uranium zittel12



    lookup


    """
    return functions.lookup(
        x, [0, 300, 600, 900, 1200, 1500, 1800, 2100, 2400, 2700, 3000, 3300, 3600, 3900], [
            0.0, 10.03061064, 23.76488412, 33.486863160000006, 39.968210400000004, 44.0, 44.0,
            44.0, 44.0, 44.0, 44.0, 44.0, 44.0, 44.0
        ])


def table_max_extraction_unconv_oil_low_mohr15(x):
    """
    table max extraction unconv oil low mohr15



    lookup


    """
    return functions.lookup(
        x, [0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000], [
            0.0, 43.7805, 76.0897, 88.8414, 94.8748, 97.1327, 97.1327, 97.1327, 97.1327, 97.1327,
            97.1327, 97.1327, 97.1327
        ])


def table_max_extraction_unconv_oil_high_mohr15(x):
    """
    table max extraction unconv oil high mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000,
        15000, 16000, 17000, 18000, 19000, 20000, 21000, 22000
    ], [
        0.0, 24.283, 74.1838, 112.563, 141.537, 163.822, 181.819, 196.629, 208.171, 221.49,
        231.626, 241.856, 246.539, 246.539, 246.539, 246.539, 246.539, 246.539, 246.539, 246.539,
        246.539, 246.539, 246.539
    ])


def table_max_extraction_tot_agg_oil_user_defined(x):
    """
    table max extraction tot agg oil user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_conv_oil_extraction_user_defined(x):
    """
    table max conv oil extraction user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_extraction_uranium_ewg13_ej(x):
    """
    table max extraction uranium ewg13 ej



    lookup


    """
    return functions.lookup(
        x, [0, 300, 600, 900, 1200, 1500, 1800, 2100, 2400, 2700, 3000, 3300, 3600, 3900], [
            0.0, 17.2759, 31.6577, 41.2069, 48.093, 51.4386, 52.2, 52.2, 52.2, 52.2, 52.2, 52.2,
            52.2, 52.2
        ])


def hist_share_oil_elec(x):
    """
    hist share oil elec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010
    ], [
        0.08179045518574861, 0.07839861446722847, 0.07794585253287176, 0.07934861054352757,
        0.07492787387164873, 0.07004498230243206, 0.06754355131189571, 0.06449449910798712,
        0.06163245529496515, 0.05952908873934008, 0.05601852360426078, 0.04972132065506773,
        0.04758264742818795, 0.04532315701504618, 0.041656283956667574, 0.037650367975256026
    ])


def hist_share_gasxdivxxcoal_xplusxgasx_elec(x):
    """
    hist share gasxdivxxcoal xplusxgasx elec



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        0.22413023824294928, 0.2217251120457687, 0.23386523401297685, 0.24495146040651222,
        0.25808112961507557, 0.2589533680662218, 0.2683927992823794, 0.2734259444963716,
        0.26962043633910304, 0.2788310171125217, 0.2808384825803754, 0.28373606863189066,
        0.28937632319005824, 0.299165260107698, 0.3072979704663346, 0.3104639543550776,
        0.30204108495613075, 0.31079123544643, 0.2980520456011999, 0.3032674271146121
    ])


def historic_nuclear_generation_twh(x):
    """
    historic nuclear generation twh



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015
    ], [
        1908, 1996, 2015, 2081, 2125, 2210, 2291, 2271, 2316, 2393, 2449, 2516, 2545, 2517, 2617,
        2639, 2659, 2597, 2602, 2568, 2620, 2507, 2345, 2364, 2408, 0
    ])


def invest_cost_nuclear(x):
    """
    invest cost nuclear



    lookup


    """
    return functions.lookup(
        x, [1995, 2007, 2015, 2020, 2030, 2040, 2050, 2060, 2070, 2080, 2090, 2100], [
            5.536708628135624, 5.536708628135624, 5.536708628135624, 5.536708628135624,
            5.536708628135624, 5.536708628135624, 5.536708628135624, 5.536708628135624,
            5.536708628135624, 5.536708628135624, 5.536708628135624, 5.536708628135624
        ])


def tpes_de_castro_phd_xdashx_scen_xmadcoalx(x):
    """
    tpes de castro phd xdashx scen xmadcoalx



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7727.25, 7904.71, 8090.31, 8272.83, 8453.11, 8632.09, 8810.72, 8989.94, 9170.63, 9353.55,
        9539.38, 9728.58, 9921.49, 10118.2, 10318.7, 10522.5, 10729.3, 10938.4, 11148.8, 11359.5,
        11569.5, 11786.3, 11999.6, 12211.4, 12420.8, 12626.9, 12829.2, 13027.3, 13221.0, 13410.5,
        13596.1, 13774.0, 13924.7, 14047.6, 14141.5, 14208.3, 14251.8, 14277.8, 14292.7, 14303.7,
        14317.3, 14343.2, 14382.9, 14441.4, 14521.6, 14625.6, 14754.7, 14909.2, 15089.5, 15295.1,
        15525.7, 15768.7, 16021.4, 16282.5, 16551.3, 16827.5, 17111.3, 17402.7, 17701.8, 18008.4,
        18322.2, 18642.6, 18968.8, 19299.8, 19634.2, 19970.7, 20218.4, 20503.9, 20792.3, 21082.0,
        21370.6, 21655.5, 21933.9, 22202.8, 22459.9, 22702.5, 22928.4, 23135.7, 23322.3, 23486.7,
        23627.4, 23743.2, 23833.1, 23896.3, 23932.3, 23940.6, 23921.3, 23874.3, 23810.4, 23774.2,
        23753.0, 23746.1, 23752.7, 23771.8, 23802.3, 23843.3, 23893.7, 23952.6, 24018.9, 24091.6,
        24169.7, 24252.2, 24338.1, 24426.4, 24516.1, 24606.1, 24695.4, 24783.0, 24868.0, 24949.3,
        25026.0, 25097.2, 25162.0, 25219.5, 25268.9, 25309.5
    ])


def gtc_historic_emissions_rcps(x):
    """
    gtc historic emissions rcps



    lookup


    """
    return functions.lookup(x, [1990, 1995, 2000, 2005], [6.144, 6.4395, 6.735, 7.971])


def afforestation_program_x2020x(x):
    """
    afforestation program x2020x



    lookup


    """
    return functions.lookup(x, [
        1980, 2020, 2025, 2030, 2035, 2040, 2045, 2050, 2055, 2060, 2065, 2070, 2075, 2080, 2085,
        2090, 2095, 2100, 2105, 2110, 2115, 2120
    ], [
        0.0, 0.0, 138.2727041328326, 299.56044274475425, 452.9353423630819, 597.8798562521115,
        733.8764376947656, 860.4075399609283, 976.9556163242087, 1083.0031200675294,
        1178.0325044589117, 1261.5262227738276, 1332.966728293337, 1391.8364742817357,
        1437.6179140219465, 1469.7935007894412, 1487.8456878541037, 1491.256928496994,
        1479.5096759861335, 1452.086383602582, 1408.469504618086, 1348.1414923118427
    ])


def historic_efficiency_gas_for_electricity(x):
    """
    historic efficiency gas for electricity



    lookup


    """
    return functions.lookup(x, [
        1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009,
        2010, 2011, 2012, 2013, 2014
    ], [
        37.813, 38.008, 38.708, 38.843, 39.84, 39.903, 39.938, 40.541, 41.535, 41.946, 42.082,
        42.187, 42.885, 43.196, 43.382, 43.393, 43.951, 43.855, 44.466, 44.268
    ])


def p_timeseries_pop_growth_rate(x):
    """
    p timeseries pop growth rate



    lookup


    """
    return functions.lookup(x, [
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100, 2101
    ], [
        0.0105, 0.0102, 0.01, 0.0098, 0.0096, 0.0093, 0.0091, 0.0089, 0.0087, 0.0085, 0.0082,
        0.008, 0.0078, 0.0076, 0.0074, 0.0073, 0.0071, 0.0069, 0.0067, 0.0065, 0.0064, 0.0062,
        0.006, 0.0058, 0.0057, 0.0055, 0.0053, 0.0052, 0.005, 0.0048, 0.0047, 0.0045, 0.0044,
        0.0042, 0.004, 0.0039, 0.0037, 0.0036, 0.0035, 0.0033, 0.0032, 0.0031, 0.0029, 0.0028,
        0.0027, 0.0026, 0.0025, 0.0024, 0.0023, 0.0022, 0.0021, 0.002, 0.002, 0.0019, 0.0018,
        0.0017, 0.0017, 0.0016, 0.0015, 0.0015, 0.0014, 0.0013, 0.0013, 0.0012, 0.0012, 0.0011,
        0.0011, 0.001, 0.001, 0.0009, 0.0009, 0.0009, 0.0008, 0.0008, 0.0008, 0.0008, 0.0007,
        0.0007, 0.0007, 0.0006, 0.0006, 0.0006, 0.0006, 0.0005, 0.0005, 0.0, 0.0
    ])


def time_dmnl(x):
    """
    time dmnl



    lookup


    """
    return functions.lookup(x, [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
        2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016, 2017, 2018, 2019,
        2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029, 2030, 2031, 2032, 2033, 2034,
        2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044, 2045, 2046, 2047, 2048, 2049,
        2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059, 2060, 2061, 2062, 2063, 2064,
        2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074, 2075, 2076, 2077, 2078, 2079,
        2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089, 2090, 2091, 2092, 2093, 2094,
        2095, 2096, 2097, 2098, 2099, 2100
    ])


def table_max_extraction_aspo_oil_ej(x):
    """
    table max extraction aspo oil ej



    lookup


    """
    return functions.lookup(x, [
        0, 600, 1200, 1800, 2400, 3000, 3600, 4200, 4800, 5400, 6000, 6600, 7200, 7800, 8400, 9000,
        9600, 10200, 10800, 11400, 12000, 12600, 13200
    ], [
        0.0, 29.9783, 46.7403, 59.4953, 71.3603, 84.9357, 96.0997, 107.465, 118.46, 127.537,
        137.018, 145.888, 152.005, 156.288, 162.47, 166.659, 171.044, 171.044, 171.044, 171.044,
        171.044, 171.044, 171.044
    ])


def table_max_extraction_gas_laherrere2010(x):
    """
    table max extraction gas laherrere2010



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000
    ], [
        0.0, 23.8621, 44.631, 62.3067, 77.7729, 93.8891, 107.821, 119.752, 129.033, 137.87, 144.16,
        148.917, 151.78, 152.895, 152.895, 152.895, 152.895, 152.895, 152.895, 152.895, 152.895,
        152.895, 152.895, 152.895, 152.895, 152.895, 152.895
    ])


def table_max_extraction_gas_mohr_high2012(x):
    """
    table max extraction gas mohr high2012



    lookup


    """
    return functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500, 24000, 25500, 27000, 28500
    ], [
        0.0, 36.4477, 58.3656, 72.8198, 98.5, 130.439, 159.979, 177.268, 190.383, 194.052, 194.197,
        194.197, 194.197, 194.197, 194.197, 194.197, 194.197, 194.197, 194.197, 194.197
    ])


def table_max_extraction_gas_mohr_bg2012(x):
    """
    table max extraction gas mohr bg2012



    lookup


    """
    return functions.lookup(x, [
        0, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000,
        15000, 16000, 17000, 18000, 19000, 20000
    ], [
        0.0, 27.4739, 42.1041, 51.2745, 58.1215, 69.3726, 79.5334, 99.1636, 115.575, 128.798,
        138.387, 147.465, 150.753, 157.008, 159.247, 159.247, 159.247, 159.247, 159.247, 159.247,
        159.247
    ])


def tnes_de_castro_phd_xdashx_scen_i(x):
    """
    tnes de castro phd xdashx scen i



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7173.13, 7368.39, 7560.95, 7750.95, 7938.54, 8123.85, 8306.94, 8487.87, 8666.6, 8843.01,
        9016.91, 9188.0, 9355.9, 9520.11, 9680.06, 9835.12, 9984.57, 10127.7, 10263.7, 10391.9,
        10511.6, 10622.1, 10723.1, 10814.1, 10894.8, 10965.4, 11025.8, 11076.5, 11117.8, 11150.3,
        11174.8, 11187.2, 11171.3, 11125.6, 11048.7, 10941.3, 10806.3, 10648.5, 10473.6, 10288.0,
        10098.3, 9914.3, 9737.27, 9572.49, 9422.99, 9290.79, 9176.91, 9081.61, 9004.54, 8944.92,
        8901.71, 8868.15, 8842.36, 8822.52, 8807.44, 8796.45, 8789.34, 8786.33, 8787.93, 8794.91,
        8800.9, 8805.38, 8813.47, 8826.89, 8847.13, 8844.22, 8829.3, 8822.62, 8829.47, 8853.12,
        8895.31, 8956.7, 9037.27, 9136.56, 9253.94, 9388.68, 9540.13, 9707.65, 9890.73, 10089.0,
        10302.0, 10529.6, 10771.5, 11027.7, 11298.0, 11582.2, 11880.5, 12192.5, 12518.4, 12857.9,
        13211.0, 13577.5, 13957.4, 14350.4, 14756.5, 15175.4, 15606.9, 16051.0, 16507.3, 16975.7,
        17456.0, 17947.9, 18451.3, 18965.9, 19491.6, 20028.0, 20575.1, 21132.7, 21700.6, 22278.8,
        22867.1, 23465.6, 24074.2, 24693.1, 25322.3, 25962.1
    ])


def net_oil_extraction_de_castro_phd_xdashx_scen_i(x):
    """
    net oil extraction de castro phd xdashx scen i



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2586.21, 2657.24, 2727.03, 2795.64, 2863.08, 2929.39, 2994.56, 3058.58, 3121.38, 3182.87,
        3242.92, 3301.35, 3357.93, 3412.41, 3464.47, 3513.79, 3559.99, 3602.72, 3641.58, 3676.21,
        3706.25, 3731.41, 3751.42, 3766.07, 3775.24, 3778.86, 3776.96, 3769.61, 3756.99, 3739.31,
        3716.84, 3688.03, 3646.76, 3592.8, 3525.96, 3446.96, 3357.36, 3259.3, 3155.32, 3048.04,
        2940.0, 2833.9, 2731.45, 2633.99, 2542.41, 2457.18, 2378.41, 2305.92, 2239.3, 2178.03,
        2121.51, 2067.41, 2015.01, 1963.69, 1913.01, 1862.72, 1812.75, 1763.12, 1713.95, 1665.41,
        1617.7, 1569.24, 1521.33, 1474.43, 1428.88, 1378.91, 1326.44, 1276.28, 1229.49, 1186.64,
        1147.86, 1113.07, 1082.06, 1054.53, 1030.2, 1008.79, 990.086, 973.882, 960.031, 948.419,
        938.964, 931.608, 926.306, 923.025, 921.731, 922.385, 924.939, 929.328, 935.469, 943.259,
        952.569, 963.248, 975.123, 987.996, 1001.65, 1015.86, 1030.38, 1044.96, 1059.33, 1073.23,
        1086.43, 1098.65, 1109.67, 1119.27, 1127.24, 1133.4, 1137.59, 1139.68, 1139.56, 1137.17,
        1132.46, 1125.41, 1116.04, 1104.39, 1090.55, 1074.6
    ])


def pe_oil_extraction_de_castro_phd_xdashx_scen_ii(x):
    """
    pe oil extraction de castro phd xdashx scen ii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2853.6, 2932.69, 3010.55, 3087.22, 3162.74, 3237.16, 3310.47, 3382.67, 3453.7, 3523.47,
        3591.83, 3658.6, 3723.55, 3786.39, 3846.79, 3904.4, 3958.81, 4009.62, 4056.42, 4098.78,
        4136.27, 4169.57, 4198.24, 4222.15, 4241.19, 4255.33, 4264.62, 4269.17, 4269.15, 4264.78,
        4256.34, 4242.11, 4215.12, 4174.81, 4120.76, 4053.66, 3975.16, 3887.67, 3794.08, 3697.45,
        3600.71, 3506.78, 3417.66, 3334.64, 3258.44, 3189.17, 3126.44, 3069.5, 3017.29, 2968.65,
        2922.37, 2873.74, 2821.48, 2764.72, 2703.17, 2637.04, 2566.93, 2493.69, 2418.33, 2341.94,
        2265.58, 2190.28, 2116.99, 2046.54, 1979.65, 1909.2, 1839.5, 1776.01, 1719.36, 1669.55,
        1626.19, 1588.74, 1556.59, 1529.08, 1505.57, 1485.41, 1467.99, 1452.74, 1439.12, 1426.63,
        1414.83, 1403.34, 1391.81, 1379.96, 1367.57, 1354.44, 1340.44, 1325.47, 1309.44, 1293.54,
        1278.69, 1264.85, 1251.89, 1239.63, 1227.89, 1216.49, 1205.3, 1194.17, 1182.97, 1171.6,
        1159.95, 1147.95, 1135.52, 1122.6, 1109.14, 1095.1, 1080.45, 1065.16, 1049.24, 1032.67,
        1015.47, 997.651, 979.238, 960.264, 940.765, 920.786
    ])


def net_oil_extraction_de_castro_phd_xdashx_scen_iii(x):
    """
    net oil extraction de castro phd xdashx scen iii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2586.21, 2657.24, 2727.03, 2795.64, 2863.08, 2929.39, 2994.56, 3058.58, 3121.38, 3182.87,
        3242.92, 3301.35, 3357.93, 3412.41, 3464.47, 3513.79, 3559.99, 3602.72, 3641.58, 3676.21,
        3706.25, 3731.41, 3751.42, 3766.07, 3775.24, 3778.86, 3776.96, 3769.56, 3756.02, 3735.73,
        3708.45, 3672.29, 3621.03, 3554.53, 3472.85, 3377.15, 3269.46, 3152.43, 3029.07, 2902.39,
        2775.22, 2649.98, 2528.6, 2412.5, 2302.57, 2199.24, 2102.59, 2012.4, 1928.26, 1849.64,
        1775.94, 1705.9, 1639.0, 1574.79, 1512.89, 1453.04, 1395.04, 1338.77, 1284.12, 1231.05,
        1179.51, 1129.49, 1080.95, 1033.89, 988.283, 939.916, 888.983, 838.921, 790.425, 743.889,
        699.498, 657.297, 617.242, 579.24, 543.177, 508.93, 476.381, 445.423, 415.961, 387.915,
        361.216, 335.808, 311.643, 288.683, 266.895, 246.251, 226.726, 208.297, 190.943, 174.638,
        159.36, 145.082, 131.775, 119.409, 107.95, 97.3636, 87.6112, 78.6537, 70.4503, 62.9592,
        56.1382, 49.9448, 44.3368, 39.2727, 34.7122, 30.6158, 26.9459, 23.6663, 20.743, 18.1435,
        15.8376, 13.797, 11.9953, 10.4082, 9.01332, 7.79015
    ])


def pe_coal_extraction_de_castro_phd_xdashx_scen_ii(x):
    """
    pe coal extraction de castro phd xdashx scen ii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2081.44, 2135.51, 2188.27, 2239.78, 2290.12, 2339.37, 2387.58, 2434.81, 2481.08, 2526.43,
        2570.82, 2614.21, 2656.54, 2697.73, 2737.64, 2776.16, 2813.12, 2848.39, 2881.79, 2913.19,
        2942.43, 2970.14, 2996.14, 3020.47, 3043.2, 3064.45, 3084.36, 3103.16, 3121.08, 3138.41,
        3155.49, 3171.11, 3180.04, 3181.73, 3175.59, 3161.8, 3141.21, 3115.28, 3085.91, 3055.2,
        3025.33, 2998.72, 2977.04, 2961.66, 2953.52, 2953.18, 2960.78, 2976.17, 2998.94, 3028.5,
        3064.18, 3101.34, 3138.56, 3174.51, 3208.26, 3239.24, 3267.14, 3291.87, 3313.47, 3332.03,
        3347.68, 3360.5, 3370.53, 3377.77, 3382.15, 3369.61, 3346.49, 3321.23, 3294.99, 3268.1,
        3240.42, 3211.63, 3181.37, 3149.3, 3115.17, 3078.78, 3040.04, 2998.94, 2955.52, 2909.85,
        2862.1, 2812.42, 2761.01, 2708.07, 2653.84, 2598.53, 2542.37, 2485.58, 2428.5, 2374.08,
        2323.46, 2276.45, 2232.72, 2191.9, 2153.67, 2117.71, 2083.74, 2051.52, 2020.82, 1991.45,
        1963.21, 1935.96, 1909.56, 1883.85, 1858.74, 1834.13, 1809.91, 1786.01, 1762.35, 1738.88,
        1715.54, 1692.29, 1669.09, 1645.91, 1622.73, 1599.53
    ])


def net_gas_extraction_de_castro_phd_xdashx_scen_i(x):
    """
    net gas extraction de castro phd xdashx scen i



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1378.15, 1422.43, 1466.37, 1509.97, 1553.27, 1596.27, 1639.0, 1681.45, 1723.6, 1765.43,
        1806.88, 1847.87, 1888.31, 1928.08, 1967.04, 2005.02, 2041.85, 2077.35, 2111.34, 2143.61,
        2174.01, 2202.36, 2228.55, 2252.45, 2274.0, 2293.18, 2310.0, 2324.5, 2336.78, 2346.97,
        2355.23, 2360.56, 2358.93, 2349.91, 2333.02, 2308.36, 2276.56, 2238.66, 2196.02, 2150.19,
        2102.72, 2055.42, 2009.48, 1965.96, 1925.6, 1888.94, 1856.22, 1827.53, 1802.76, 1781.69,
        1764.05, 1748.05, 1733.15, 1718.83, 1704.69, 1690.47, 1676.02, 1661.3, 1646.32, 1631.17,
        1615.96, 1598.98, 1581.36, 1563.41, 1545.42, 1521.06, 1491.54, 1462.02, 1433.79, 1407.58,
        1383.71, 1362.23, 1342.98, 1325.69, 1310.03, 1295.68, 1282.28, 1269.52, 1257.12, 1244.82,
        1232.39, 1219.62, 1206.36, 1192.45, 1177.78, 1162.23, 1145.74, 1128.25, 1109.71, 1090.12,
        1069.47, 1047.78, 1025.09, 1001.45, 976.929, 951.611, 925.587, 898.96, 871.837, 844.332,
        816.56, 788.637, 760.675, 732.784, 705.067, 677.624, 650.543, 623.906, 597.787, 572.25,
        547.351, 523.135, 499.642, 476.902, 454.936, 433.76
    ])


def pe_gas_extraction_de_castro_phd_xdashx_scen_ii(x):
    """
    pe gas extraction de castro phd xdashx scen ii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1511.25, 1560.28, 1609.0, 1657.46, 1705.67, 1753.67, 1801.48, 1849.09, 1896.51, 1943.7,
        1990.62, 2037.18, 2083.29, 2128.84, 2173.66, 2217.6, 2260.46, 2302.05, 2342.18, 2380.64,
        2417.18, 2452.25, 2485.65, 2517.3, 2547.21, 2575.39, 2601.9, 2626.83, 2650.34, 2672.6,
        2693.85, 2713.04, 2725.69, 2731.16, 2728.82, 2718.75, 2701.64, 2678.7, 2651.58, 2622.12,
        2592.25, 2564.02, 2538.98, 2518.29, 2502.75, 2492.79, 2488.48, 2489.57, 2495.59, 2505.9,
        2519.76, 2533.25, 2544.98, 2553.75, 2558.7, 2559.31, 2555.33, 2546.69, 2533.45, 2515.76,
        2493.79, 2467.71, 2437.67, 2403.82, 2366.26, 2315.78, 2256.66, 2195.62, 2133.79, 2071.61,
        2009.2, 1946.54, 1883.58, 1820.32, 1756.81, 1693.17, 1629.56, 1566.17, 1503.21, 1440.92,
        1379.52, 1319.21, 1260.2, 1202.66, 1146.75, 1092.58, 1040.26, 989.855, 941.438, 895.994,
        853.858, 814.762, 778.406, 744.501, 712.786, 683.028, 655.029, 628.61, 603.618, 579.92,
        557.398, 535.948, 515.482, 495.919, 477.189, 459.233, 441.995, 425.428, 409.491, 394.147,
        379.364, 365.113, 351.369, 338.109, 325.314, 312.966
    ])


def net_gas_extraction_de_castro_phd_xdashx_scen_iii(x):
    """
    net gas extraction de castro phd xdashx scen iii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1378.15, 1422.43, 1466.37, 1509.97, 1553.27, 1596.27, 1639.0, 1681.45, 1723.6, 1765.43,
        1806.88, 1847.87, 1888.31, 1928.08, 1967.04, 2005.02, 2041.85, 2077.35, 2111.34, 2143.61,
        2174.01, 2202.36, 2228.55, 2252.45, 2274.0, 2293.18, 2310.0, 2324.47, 2336.18, 2344.72,
        2349.88, 2350.38, 2342.04, 2324.36, 2296.96, 2260.1, 2214.64, 2161.88, 2103.47, 2041.17,
        1976.72, 1911.72, 1847.53, 1785.2, 1725.49, 1668.87, 1615.53, 1565.49, 1518.59, 1474.56,
        1433.08, 1393.24, 1354.7, 1317.14, 1280.29, 1243.98, 1208.07, 1172.48, 1137.16, 1102.09,
        1067.27, 1032.71, 998.441, 964.476, 930.845, 893.625, 852.817, 811.701, 771.01, 731.211,
        692.579, 655.254, 619.284, 584.664, 551.358, 519.318, 488.491, 458.831, 430.297, 402.855,
        376.484, 351.166, 326.892, 303.655, 281.454, 260.289, 240.158, 221.061, 202.995, 185.952,
        169.922, 154.893, 140.845, 127.756, 115.6, 104.347, 93.9612, 84.4072, 75.6455, 67.6349,
        60.3331, 53.697, 47.6834, 42.2492, 37.3523, 32.9516, 29.0072, 25.481, 22.3367, 19.5399,
        17.0583, 14.8618, 12.922, 11.2131, 9.71091, 8.39349
    ])


def net_coal_extraction_de_castro_phd_xdashx_scen_iii(x):
    """
    net coal extraction de castro phd xdashx scen iii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1945.63, 1995.96, 2045.03, 2092.89, 2139.62, 2185.28, 2229.93, 2273.61, 2316.36, 2358.19,
        2399.06, 2438.96, 2477.8, 2515.5, 2551.95, 2587.03, 2620.58, 2652.48, 2682.56, 2710.7,
        2736.76, 2760.66, 2782.31, 2801.7, 2818.82, 2833.73, 2846.52, 2857.31, 2865.65, 2871.1,
        2873.47, 2871.23, 2859.31, 2837.16, 2804.26, 2760.86, 2707.91, 2646.88, 2579.6, 2508.07,
        2434.29, 2360.07, 2286.98, 2216.23, 2148.71, 2084.94, 2025.17, 1969.39, 1917.4, 1868.88,
        1823.41, 1779.85, 1737.75, 1696.69, 1656.32, 1616.38, 1576.69, 1537.11, 1497.56, 1458.0,
        1418.41, 1378.78, 1339.14, 1299.51, 1259.91, 1215.03, 1164.71, 1113.39, 1062.05, 1011.37,
        961.747, 913.409, 866.46, 820.93, 776.805, 734.051, 692.63, 652.506, 613.653, 576.054,
        539.706, 504.611, 470.782, 438.236, 406.994, 377.077, 348.507, 321.3, 295.471, 271.027,
        247.969, 226.292, 205.981, 187.015, 169.365, 152.995, 137.864, 123.924, 111.123, 99.4056,
        88.714, 78.9881, 70.1672, 62.1905, 54.9978, 48.5302, 42.7303, 37.5432, 32.916, 28.7988,
        25.1446, 21.9093, 19.0516, 16.5334, 14.3195, 12.3777
    ])


def net_coal_extraction_de_castro_phd_xdashx_scen_i(x):
    """
    net coal extraction de castro phd xdashx scen i



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        1378.15, 1422.43, 1466.37, 1509.97, 1553.27, 1596.27, 1639.0, 1681.45, 1723.6, 1765.43,
        1806.88, 1847.87, 1888.31, 1928.08, 1967.04, 2005.02, 2041.85, 2077.35, 2111.34, 2143.61,
        2174.01, 2202.36, 2228.55, 2252.45, 2274.0, 2293.18, 2310.0, 2324.5, 2336.78, 2346.97,
        2355.23, 2360.56, 2358.93, 2349.91, 2333.02, 2308.36, 2276.56, 2238.66, 2196.02, 2150.19,
        2102.72, 2055.42, 2009.48, 1965.96, 1925.6, 1888.94, 1856.22, 1827.53, 1802.76, 1781.69,
        1764.05, 1748.05, 1733.15, 1718.83, 1704.69, 1690.47, 1676.02, 1661.3, 1646.32, 1631.17,
        1615.96, 1598.98, 1581.36, 1563.41, 1545.42, 1521.06, 1491.54, 1462.02, 1433.79, 1407.58,
        1383.71, 1362.23, 1342.98, 1325.69, 1310.03, 1295.68, 1282.28, 1269.52, 1257.12, 1244.82,
        1232.39, 1219.62, 1206.36, 1192.45, 1177.78, 1162.23, 1145.74, 1128.25, 1109.71, 1090.12,
        1069.47, 1047.78, 1025.09, 1001.45, 976.929, 951.611, 925.587, 898.96, 871.837, 844.332,
        816.56, 788.637, 760.675, 732.784, 705.067, 677.624, 650.543, 623.906, 597.787, 572.25,
        547.351, 523.135, 499.642, 476.902, 454.936, 433.76
    ])


def tnes_de_castro_phd_xdashx_scen_iii(x):
    """
    tnes de castro phd xdashx scen iii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7173.13, 7368.39, 7560.95, 7750.95, 7938.54, 8123.85, 8306.94, 8487.87, 8666.6, 8843.01,
        9016.91, 9188.0, 9355.9, 9520.11, 9680.06, 9835.12, 9984.57, 10127.7, 10263.7, 10391.9,
        10511.6, 10622.1, 10723.1, 10814.1, 10894.8, 10965.4, 11025.8, 11075.0, 11109.9, 11131.5,
        11139.1, 11127.4, 11079.5, 10993.9, 10869.3, 10707.1, 10511.3, 10287.5, 10042.6, 9783.87,
        9518.5, 9252.88, 8992.4, 8741.21, 8502.22, 8277.14, 8066.69, 7870.72, 7688.5, 7518.87,
        7360.42, 7209.45, 7064.52, 6924.25, 6787.58, 6653.67, 6521.96, 6392.02, 6263.61, 6136.56,
        6010.8, 5886.31, 5763.1, 5641.19, 5520.62, 5379.15, 5222.66, 5064.42, 4907.35, 4753.31,
        4603.4, 4458.15, 4317.76, 4182.18, 4051.27, 3924.82, 3802.6, 3684.43, 3570.15, 3459.62,
        3352.78, 3249.57, 3149.98, 3054.03, 2961.73, 2873.14, 2788.3, 2707.25, 2630.04, 2556.68,
        2487.21, 2421.6, 2359.84, 2301.9, 2247.7, 2197.16, 2150.2, 2106.69, 2066.5, 2029.49,
        1995.52, 1964.42, 1936.03, 1910.19, 1886.73, 1865.5, 1846.32, 1829.04, 1813.51, 1799.59,
        1787.14, 1776.03, 1766.14, 1757.35, 1749.56, 1742.67
    ])


def primary_coal_extraction_de_castro_phd_xdashx_scen_xmadcoalx(x):
    """
    primary coal extraction de castro phd xdashx scen xmadcoalx



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        2058.92, 2092.27, 2125.72, 2159.47, 2193.76, 2228.81, 2264.86, 2302.13, 2340.84, 2381.18,
        2423.34, 2467.47, 2513.71, 2562.15, 2612.87, 2665.92, 2721.31, 2779.03, 2839.06, 2901.33,
        2965.74, 3032.57, 3101.67, 3172.97, 3246.4, 3321.9, 3399.43, 3478.96, 3560.51, 3644.12,
        3729.86, 3816.11, 3897.75, 3974.81, 4046.62, 4112.97, 4174.06, 4230.44, 4282.94, 4332.57,
        4380.4, 4427.69, 4475.38, 4524.25, 4574.94, 4627.89, 4683.41, 4741.63, 4802.57, 4866.15,
        4932.23, 4997.04, 5059.9, 5120.05, 5176.96, 5230.21, 5279.47, 5324.46, 5364.9, 5400.52,
        5431.05, 5456.18, 5475.61, 5489.04, 5496.15, 5485.53, 5462.41, 5433.0, 5397.99, 5357.58,
        5311.76, 5260.45, 5203.57, 5141.1, 5073.11, 4999.72, 4921.11, 4837.56, 4749.39, 4656.95,
        4560.64, 4460.9, 4358.16, 4252.9, 4145.55, 4036.59, 3926.47, 3815.6, 3704.47, 3595.32,
        3489.22, 3386.16, 3286.01, 3188.63, 3093.87, 3001.58, 2911.63, 2823.91, 2738.31, 2654.74,
        2573.11, 2493.35, 2415.41, 2339.22, 2264.73, 2191.92, 2120.74, 2051.16, 1983.17, 1916.73,
        1851.84, 1788.49, 1726.66, 1666.35, 1607.54, 1550.24
    ])


def tpes_de_castro_phd_xdashx_scen_ii(x):
    """
    tpes de castro phd xdashx scen ii



    lookup


    """
    return functions.lookup(x, [
        1985, 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994, 1995, 1996, 1997, 1998, 1999,
        2000, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
        2015, 2016, 2017, 2018, 2019, 2020, 2021, 2022, 2023, 2024, 2025, 2026, 2027, 2028, 2029,
        2030, 2031, 2032, 2033, 2034, 2035, 2036, 2037, 2038, 2039, 2040, 2041, 2042, 2043, 2044,
        2045, 2046, 2047, 2048, 2049, 2050, 2051, 2052, 2053, 2054, 2055, 2056, 2057, 2058, 2059,
        2060, 2061, 2062, 2063, 2064, 2065, 2066, 2067, 2068, 2069, 2070, 2071, 2072, 2073, 2074,
        2075, 2076, 2077, 2078, 2079, 2080, 2081, 2082, 2083, 2084, 2085, 2086, 2087, 2088, 2089,
        2090, 2091, 2092, 2093, 2094, 2095, 2096, 2097, 2098, 2099, 2100
    ], [
        7749.13, 7949.4, 8160.47, 8369.05, 8575.33, 8779.44, 8981.51, 9181.61, 9379.72, 9575.75,
        9769.52, 9960.74, 10149.0, 10333.8, 10514.5, 10690.6, 10861.1, 11025.4, 11182.6, 11331.9,
        11472.6, 11613.1, 11744.0, 11868.2, 11985.7, 12096.8, 12202.0, 12302.1, 12398.0, 12490.6,
        12581.3, 12667.2, 12730.0, 12767.9, 12779.1, 12764.7, 12728.4, 12676.0, 12615.0, 12553.4,
        12499.7, 12465.2, 12453.3, 12470.3, 12519.7, 12603.9, 12723.6, 12878.5, 13067.2, 13287.8,
        13538.2, 13805.0, 14082.5, 14367.0, 14656.3, 14949.0, 15244.7, 15543.8, 15846.7, 16154.3,
        16467.1, 16785.7, 17110.0, 17439.8, 17774.5, 18113.2, 18354.1, 18638.3, 18929.8, 19227.6,
        19529.3, 19832.1, 20132.6, 20427.7, 20714.2, 20989.2, 21250.3, 21494.8, 21720.8, 21926.3,
        22109.7, 22269.4, 22404.4, 22513.5, 22596.2, 22651.7, 22679.9, 22680.6, 22664.5, 22677.2,
        22706.2, 22750.8, 22809.7, 22881.7, 22965.5, 23059.8, 23163.4, 23275.1, 23393.7, 23518.0,
        23647.0, 23779.6, 23914.5, 24050.7, 24187.2, 24322.8, 24456.4, 24587.0, 24713.6, 24835.1,
        24950.6, 25059.1, 25159.7, 25251.6, 25333.9, 25405.8
    ])


def table_max_extraction_conv_gas_low_mohr15(x):
    """
    table max extraction conv gas low mohr15



    lookup


    """
    return functions.lookup(x, [0, 1100, 2200, 3300, 4400, 5500, 6600, 7700, 8800, 9900, 11000], [
        0.0, 49.9516317103108, 93.2254158071833, 118.25322742687901, 122.34766617760398,
        124.03503097023999, 124.03503097023999, 124.03503097023999, 124.03503097023999,
        124.03503097023999, 124.03503097023999
    ])


def table_max_extraction_conv_gas_user_defined(x):
    """
    table max extraction conv gas user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_extraction_total_gas_bg_mohr12(x):
    """
    table max extraction total gas bg mohr12



    lookup


    """
    return functions.lookup(x, [
        0, 1000, 2000, 3000, 4000, 5000, 6000, 7000, 8000, 9000, 10000, 11000, 12000, 13000, 14000,
        15000, 16000, 17000, 18000, 19000, 20000
    ], [
        0.0, 27.4739, 42.1041, 51.2745, 58.1215, 69.3726, 79.5334, 99.1636, 115.575, 128.798,
        138.387, 147.465, 150.753, 157.008, 159.247, 159.247, 159.247, 159.247, 159.247, 159.247,
        159.247
    ])


def table_max_extraction_total_gas_laherrere10(x):
    """
    table max extraction total gas laherrere10



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000
    ], [
        0.0, 23.86208767023, 44.63101020582, 62.30666199473999, 77.77291011605999, 93.88909467,
        107.8214336676, 119.75242469669998, 129.03255377280001, 137.87016844320001, 144.16042095,
        148.91718678119997, 151.78032891450002, 152.89453583099998, 152.89453583099998,
        152.89453583099998, 152.89453583099998, 152.89453583099998, 152.89453583099998,
        152.89453583099998, 152.89453583099998, 152.89453583099998, 152.89453583099998,
        152.89453583099998, 152.89453583099998, 152.89453583099998, 152.89453583099998
    ])


def table_max_extraction_total_gas_user_defined(x):
    """
    table max extraction total gas user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_extraction_unconv_gas_bg_mohr15(x):
    """
    table max extraction unconv gas bg mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 36.281, 59.2482, 79.7156, 92.6562, 104.84, 114.916, 121.97, 128.037, 132.221, 134.519,
        134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974,
        134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974, 134.974
    ])


def table_max_extraction_unconv_gas_high_mohr15(x):
    """
    table max extraction unconv gas high mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500, 24000
    ], [
        0.0, 83.3997, 135.51, 170.831, 191.804, 202.817, 205.615, 205.615, 205.615, 205.615,
        205.615, 205.615, 205.615, 205.615, 205.615, 205.615, 205.615
    ])


def table_max_extraction_unconv_gas_low_mohr15(x):
    """
    table max extraction unconv gas low mohr15



    lookup


    """
    return functions.lookup(x, [0, 400, 800, 1200, 1600, 2000, 2400, 2800],
                            [0.0, 0.0, 0.0, 0.0, 35.9832, 35.9832, 35.9832, 35.9832])


def table_max_extraction_unconv_gas_user_defined(x):
    """
    table max extraction unconv gas user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_extraction_coal_bg_mohr15(x):
    """
    table max extraction coal bg mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500
    ], [
        0.0, 45.8067598, 101.45721979999999, 129.918137083, 129.918137083, 129.918137083,
        140.66041522999998, 172.65528881379996, 210.75409942999997, 245.94171830000005,
        245.94171830000005, 245.94171830000005, 245.94171830000005, 245.94171830000005,
        245.94171830000005, 245.94171830000005
    ])


def table_max_extraction_coal_high_mohr15(x):
    """
    table max extraction coal high mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500, 24000, 25500, 27000, 28500, 30000, 31500
    ], [
        0.0, 49.7354913, 91.20222852385001, 93.95328151, 120.98636085000001, 142.6042858061,
        157.275271969, 170.16619920999997, 187.20167642, 198.71934311599998, 213.61074611700002,
        228.50740735999997, 252.196010948, 264.94099527149996, 272.123730091, 274.87091482169996,
        274.87091482169996, 274.87091482169996, 274.87091482169996, 274.87091482169996,
        274.87091482169996, 274.87091482169996
    ])


def table_max_extraction_coal_low_mohr15(x):
    """
    table max extraction coal low mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 10.7961939, 41.4887626012, 69.58980656000001, 94.81256219, 106.66908508867104,
        119.606988, 140.56757997980003, 158.59254590300003, 179.93148888900004, 202.8910104403,
        214.784730211, 224.11729660400007, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001, 224.4862273680001, 224.4862273680001,
        224.4862273680001, 224.4862273680001
    ])


def table_max_extraction_coal_mohr2012_ej(x):
    """
    table max extraction coal mohr2012 ej



    lookup


    """
    return functions.lookup(x, [
        0, 2000, 4000, 6000, 8000, 10000, 12000, 14000, 16000, 18000, 20000, 22000, 24000, 26000,
        28000
    ], [
        0.0, 50.9743, 81.21, 91.3165, 96.3823, 109.46, 123.422, 148.171, 204.223, 216.291, 216.291,
        216.291, 216.291, 216.291, 216.291
    ])


def table_max_extraction_coal_user_defined(x):
    """
    table max extraction coal user defined



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500, 11000, 11500, 12000, 12500, 13000, 13500, 14000,
        14500
    ], [
        0.0, 29.4861, 56.0597, 77.0, 94.7098, 111.198, 124.131, 136.26, 147.073, 154.041, 160.5,
        165.566, 167.496, 169.176, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474,
        169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474, 169.474
    ])


def table_max_extraction_conv_gas_bg_mohr15(x):
    """
    table max extraction conv gas bg mohr15



    lookup


    """
    return functions.lookup(
        x, [0, 1200, 2400, 3600, 4800, 6000, 7200, 8400, 9600, 10800, 12000, 13200], [
            0.0, 53.7105, 94.1488, 124.386, 132.465, 134.77, 134.77, 134.77, 134.77, 134.77,
            134.77, 134.77
        ])


def table_max_extraction_conv_gas_high_mohr15(x):
    """
    table max extraction conv gas high mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 1500, 3000, 4500, 6000, 7500, 9000, 10500, 12000, 13500, 15000, 16500, 18000, 19500,
        21000, 22500
    ], [
        0.0, 85.11063547168709, 136.8937293509019, 179.4938495500182, 203.86285809423433,
        216.27987364278962, 218.76844181844524, 218.76844181844524, 218.76844181844524,
        218.76844181844524, 218.76844181844524, 218.76844181844524, 218.76844181844524,
        218.76844181844524, 218.76844181844524, 218.76844181844524
    ])


def table_max_extraction_unconv_oil_bg_mohr15(x):
    """
    table max extraction unconv oil bg mohr15



    lookup


    """
    return functions.lookup(x, [
        0, 500, 1000, 1500, 2000, 2500, 3000, 3500, 4000, 4500, 5000, 5500, 6000, 6500, 7000, 7500,
        8000, 8500, 9000, 9500, 10000, 10500
    ], [
        0.0, 11.1694, 20.7262, 46.1364, 67.4689, 93.0394, 111.958, 122.983, 129.802, 134.906,
        138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735, 138.735,
        138.735, 138.735
    ])


@cache('run')
def ratio_land_productivity_2gen_vs_marg():
    """
    ratio land productivity 2gen vs marg



    component


    """
    
    ret = land_productivity_biofuels_2gen_ej_mha() / land_productivity_biofuels_marg_ej_mha()
    #test_nans(ret, 0)
    return ret


@cache('run')
def land_productivity_biofuels_marg_ej_mha():
    """
    land productivity biofuels marg ej mha



    constant


    """
    
    ret = 0.0106
    #test_nans(ret, 1)
    return ret


@cache('run')
def correction_factor_all_ghgs():
    """
    correction factor all ghgs



    constant


    """
    
    ret = 1.22
    #test_nans(ret, 2)
    return ret


@cache('step')
def cumulative_co2e_ghg_emissions():
    """
    cumulative co2e ghg emissions



    component


    """
    
    ret = integ_cumulative_co2e_ghg_emissions()
    #test_nans(ret, 3)
    return ret


@cache('step')
def total_ce_all_ghg():
    """
    total ce all ghg



    component


    """
    
    ret = total_co2e_xgwpxequalsx100x_yearsx() * correction_factor_all_ghgs() * c_per_co2()
    #test_nans(ret, 4)
    return ret


@cache('step')
def total_co2e_all_ghg():
    """
    total co2e all ghg



    component


    """
    
    ret = total_co2e_xgwpxequalsx100x_yearsx() * correction_factor_all_ghgs()
    #test_nans(ret, 5)
    return ret


@cache('run')
def activate_eroi_fc_system_feedbackxquestx():
    """

    """
    ret = 0
    return ret


@cache('step')
def eroi_fc_system_from_x2015x():
    """
    eroi fc system from x2015x



    component


    """
    
    ret = functions.if_then_else(activate_eroi_fc_system_feedbackxquestx() == 0, 1, eroi_fc_system_from_x2015x_x1x())
    #test_nans(ret, 6)
    return ret


@cache('run')
def share_in_target_year_oil_for_heat():
    """
    share in target year oil for heat



    constant


    """
    
    ret = 0
    #test_nans(ret, 7)
    return ret


@cache('step')
def p_share_oil_for_heat():
    """
    p share oil for heat



    component


    """
    
    ret = np.maximum(0,
                      a_lineal_regr_phasexdashxout_oil_for_heat() * time() +
                      b_lineal_regr_phasexdashxout_oil_for_heat())
    #test_nans(ret, 8)
    return ret


@cache('run')
def target_year_policy_phasexdashxout_oil_for_heat():
    """
    target year policy phasexdashxout oil for heat



    constant


    """
    
    ret = 2030
    #test_nans(ret, 9)
    return ret


@cache('step')
def b_lineal_regr_phasexdashxout_oil_for_heat():
    """
    b lineal regr phasexdashxout oil for heat



    component


    """
    
    ret = share_in_target_year_oil_for_heat() - a_lineal_regr_phasexdashxout_oil_for_heat(
    ) * target_year_policy_phasexdashxout_oil_for_heat()
    #test_nans(ret, 10)
    return ret


@cache('run')
def start_year_policy_phasexdashxout_oil_for_heat():
    """
    start year policy phasexdashxout oil for heat



    constant


    """
    
    ret = 2020
    #test_nans(ret, 11)
    return ret


@cache('run')
def start_year_policy_phasexdashxout_oil_for_elec():
    """
    start year policy phasexdashxout oil for elec



    constant


    """
    
    ret = 2020
    #test_nans(ret, 12)
    return ret


@cache('step')
def p_share_oil_for_elec():
    """
    p share oil for elec



    component


    """
    
    ret = np.maximum(0,
                      a_lineal_regr_phasexdashxout_oil_for_elec() * time() +
                      b_lineal_regr_phasexdashxout_oil_for_elec())
    #test_nans(ret, 13)
    return ret


@cache('run')
def share_in_target_year_oil_for_elec():
    """
    share in target year oil for elec



    constant


    """
    
    ret = 0
    #test_nans(ret, 14)
    return ret


@cache('run')
def target_year_policy_phasexdashxout_oil_for_elec():
    """
    target year policy phasexdashxout oil for elec



    constant


    """
    
    ret = 2030
    #test_nans(ret, 15)
    return ret


@cache('step')
def b_lineal_regr_phasexdashxout_oil_for_elec():
    """
    b lineal regr phasexdashxout oil for elec



    component


    """
    
    ret = share_in_target_year_oil_for_elec() - a_lineal_regr_phasexdashxout_oil_for_elec(
    ) * target_year_policy_phasexdashxout_oil_for_elec()
    #test_nans(ret, 16)
    return ret


@cache('step')
def fed_heatxdashxnc_ej():
    """
    fed heatxdashxnc ej



    component


    """
    
    ret = fe_heat_demand_consum() - required_heatxdashxcom()
    #test_nans(ret, 17)
    return ret


@cache('step')
def leontief_matrix():
    """
    leontief matrix



    component


    """
    
    ret = leontief_matrix_for_python()
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 18)
    return ret


@cache('step')
def pressure_to_change_energy_technology_h():
    """
    pressure to change energy technology h



    component


    """
    
    ret = np.minimum(np.maximum(implementatio_policy_to_change_final_energy_h(), 0), 1)
    #test_nans(ret, 19)
    return ret


@cache('step')
def pressure_to_change_energy_technology():
    """
    pressure to change energy technology



    component


    """
    
    ret = np.minimum(np.maximum(implementatio_policy_to_change_final_energy(), 0), 1)
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 20)
    return ret


@cache('step')
def ia_matrix():
    """
    ia matrix



    component


    """
    
    ret = ia_matrix_for_python()
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 21)
    return ret


@cache('step')
def pe_traditional_biomass_consum_ej():
    """
    pe traditional biomass consum ej



    component


    """
    
    ret = pe_traditional_biomass_demand_ej()
    #test_nans(ret, 22)
    return ret


@cache('step')
def scarcity_resources_counter():
    """
    scarcity resources counter



    component


    """
    
    ret = integ_scarcity_resources_counter()
    #test_nans(ret, 23)
    return ret


@cache('step')
def scarcity_resources_flag():
    """
    scarcity resources flag



    component


    """
    
    ret = functions.if_then_else(scarcity_resources_counter() > 1, 1, 0)
    #test_nans(ret, 24)
    return ret


@cache('step')
def percent_res_vs_tpes():
    """
    percent res vs tpes



    component


    """
    
    ret = share_res_vs_tpes() * 100
    #test_nans(ret, 25)
    return ret


@cache('step')
def year_init_scarcity_final_fuels():
    """
    year init scarcity final fuels



    component


    """
    
    ret = integ_year_init_scarcity_final_fuels()
    #test_nans(ret, 26)
    return ret


@cache('step')
def year_init_scarcity_reserves():
    """
    year init scarcity reserves



    component


    """
    
    ret = integ_year_init_scarcity_reserves()
    #test_nans(ret, 27)
    return ret


@cache('step')
def percent_tot_monet_invest_reselec_vs_gdp():
    """
    percent tot monet invest reselec vs gdp



    component


    """
    
    ret = share_tot_monet_invest_elec_res_vs_gdp() * 100
    #test_nans(ret, 28)
    return ret


@cache('step')
def scarcity_final_fuels_counter():
    """
    scarcity final fuels counter



    component


    """
    
    ret = integ_scarcity_final_fuels_counter()
    #test_nans(ret, 29)
    return ret


@cache('step')
def percent_exdashxlosses_cc():
    """
    percent exdashxlosses cc



    component


    """
    
    ret = share_exdashxlosses_cc() * 100
    #test_nans(ret, 30)
    return ret


@cache('step')
def scarcity_reserves_counter():
    """
    scarcity reserves counter



    component


    """
    
    ret = integ_scarcity_reserves_counter()
    #test_nans(ret, 31)
    return ret


@cache('step')
def year_final_scarcity_reserves():
    """
    year final scarcity reserves



    component


    """
    
    ret = functions.if_then_else(scarcity_reserves_counter() > 0,
                                  year_init_scarcity_reserves() + scarcity_reserves_counter() - 1,
                                  0)
    #test_nans(ret, 32)
    return ret


@cache('step')
def year_final_scarcity_resources():
    """
    year final scarcity resources



    component


    """
    
    ret = functions.if_then_else(
        scarcity_resources_counter() > 0,
        year_init_scarcity_resources() + scarcity_resources_counter() - 1, 0)
    #test_nans(ret, 33)
    return ret


@cache('step')
def scarcity_fuels_flag():
    """
    scarcity fuels flag



    component


    """
    
    ret = functions.if_then_else(scarcity_final_fuels_counter() > 1, 1, 0)
    #test_nans(ret, 34)
    return ret


@cache('step')
def scarcity_reserves_flag():
    """
    scarcity reserves flag



    component


    """
    
    ret = functions.if_then_else(scarcity_reserves_counter() > 1, 1, 0)
    #test_nans(ret, 35)
    return ret


@cache('step')
def year_init_scarcity_resources():
    """
    year init scarcity resources



    component


    """
    
    ret = integ_year_init_scarcity_resources()
    #test_nans(ret, 36)
    return ret


@cache('step')
def percent_remaining_potential_tot_res_heat():
    """
    percent remaining potential tot res heat



    component


    """
    
    ret = remaining_potential_tot_res_heat() * 100
    #test_nans(ret, 37)
    return ret


@cache('step')
def abundance_final_fuels():
    """
    abundance final fuels



    component


    """
    
    ret = xr.DataArray(data=[
        abundance_electricity(),
        abundance_heat(),
        abundance_liquids(),
        abundance_gases(),
        abundance_solids()],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 38)
    return ret


@cache('step')
def scarcity_final_fuels():
    """
    scarcity final fuels



    component


    """
    
    ret = functions.if_then_else(abundance_final_fuels() < 0.999, 1, 0)
    #test_nans(ret, 39)
    return ret


@cache('step')
def percent_remaining_potential_tot_res_elec():
    """
    percent remaining potential tot res elec



    component


    """
    
    ret = remaining_potential_tot_res_elec() * 100
    #test_nans(ret, 40)
    return ret


@cache('step')
def year_final_scarcity_final_fuels():
    """
    year final scarcity final fuels



    component


    """
    
    ret = functions.if_then_else(
        scarcity_final_fuels_counter() > 0,
        year_init_scarcity_final_fuels() + scarcity_final_fuels_counter() - 1, 0)
    #test_nans(ret, 41)
    return ret


@cache('step')
def share_blue_water_use_vs_ar():
    """
    share blue water use vs ar



    component


    """
    
    ret = total_water_use_by_type().loc['blue water'].values / (ar_water() * dam3_per_km3())
    #test_nans(ret, 42)
    return ret


@cache('run')
def ar_water():
    """
    ar water



    constant


    """
    
    ret = 7091.0
    #test_nans(ret, 43)
    return ret


@cache('step')
def share_total_water_use_vs_ar():
    """
    share total water use vs ar



    component


    """
    
    ret = total_water_use() / (ar_water() * dam3_per_km3())
    #test_nans(ret, 44)
    return ret


@cache('run')
def renewable_water_resources():
    """
    renewable water resources



    constant


    """
    
    ret = 43659.0
    #test_nans(ret, 45)
    return ret


@cache('step')
def share_blue_water_use_vs_renewable_water_resources():
    """
    share blue water use vs renewable water resources



    component


    """
    
    ret = total_water_use_by_type().loc['blue water'].values / (renewable_water_resources() * dam3_per_km3())
    #test_nans(ret, 46)
    return ret


@cache('run')
def dam3_per_km3():
    """
    dam3 per km3



    constant


    """
    
    ret = 1000000.0
    #test_nans(ret, 47)
    return ret


@cache('step')
def share_total_water_use_vs_renewable_water_resources():
    """
    share total water use vs renewable water resources



    component


    """
    
    ret = total_water_use() / (renewable_water_resources() * dam3_per_km3())
    #test_nans(ret, 48)
    return ret


@cache('step')
def total_pe_solid_bioe_potential_heatxplusxelec_ej():
    """
    total pe solid bioe potential heatxplusxelec ej



    component


    """
    
    ret = max_potential_npp_bioe_conventional_for_heatxplusxelec(
    ) + pe_bioe_residues_for_heatxplusxelec_ej()
    #test_nans(ret, 49)
    return ret


@cache('run')
def land_module_activatedxquestx():
    """
    land module activatedxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 50)
    return ret


@cache('run')
def cp_res_for_heat():
    """
    cp res for heat



    component


    """
    
    ret = cpxdashxini_res_for_heat()
    #test_nans(ret, 51)
    return ret


@cache('step')
def potential_fes_res_for_heatxdashxcom_ej():
    """
    potential fes res for heatxdashxcom ej



    component


    """
    
    ret = potential_fes_res_for_heatxdashxcom_twh() * ej_per_twh()
    #test_nans(ret, 52)
    return ret


@cache('step')
def potential_fes_res_for_heatxdashxcom_twh():
    """
    potential fes res for heatxdashxcom twh



    component


    """
    
    ret = installed_capacity_res_heatxdashxcom_tw() * efficiency_res_heat() * cp_res_for_heat(
    ) / twe_per_twh()
    #test_nans(ret, 53)
    return ret


@cache('step')
def real_generation_res_elec_twh():
    """
    real generation res elec twh



    component


    """
    
    ret = potential_generation_res_elec_twh() * (1 - res_elec_tot_overcapacity())
    #test_nans(ret, 54)
    return ret


@cache('step')
def potential_fes_res_for_heatxdashxnc_twh():
    """
    potential fes res for heatxdashxnc twh



    component


    """
    
    ret = installed_capacity_res_heatxdashxnc_tw() * efficiency_res_heat() * cp_res_for_heat(
    ) / twe_per_twh()
    #test_nans(ret, 55)
    return ret


@cache('step')
def replacement_capacity_res_elec():
    """
    replacement capacity res elec



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015, 0,
        replacement_rate_res_elec() * wear_res_elec() *
        (1 - res_elec_tot_overcapacity())) * constraint_elec_storage_availability()
    #test_nans(ret, 56)
    return ret


@cache('step')
def replacement_res_for_heatxdashxnc_tw():
    """
    replacement res for heatxdashxnc tw



    component


    """
    
    ret = wear_res_capacity_for_heatxdashxnc_tw() * replacement_res_for_heatxdashxnc() * (
        1 - res_heatxdashxnc_tot_overcapacity())
    #test_nans(ret, 57)
    return ret


@cache('step')
def replacement_res_for_heatxdashxcom_tw():
    """
    replacement res for heatxdashxcom tw



    component


    """
    
    ret = wear_res_capacity_for_heatxdashxcom_tw() * replacement_res_for_heatxdashxcom() * (
        1 - res_heatxdashxcom_tot_overcapacity())
    #test_nans(ret, 58)
    return ret


@cache('run')
def max_eroi_fc():
    """
    max eroi fc



    constant


    """
    
    ret = 5
    #test_nans(ret, 59)
    return ret


@cache('step')
def eroi_fc_system_from_x2015x_x1x():
    """
    eroi fc system from x2015x x1x



    component


    """
    
    ret = functions.if_then_else(
        time() < 2016, 1,
        functions.if_then_else(
            eroist_system_delayed_1yr() > 1,
            np.minimum(max_eroi_fc(), (eroist_system_delayed_1yr() /
                                       (eroist_system_delayed_1yr() - 1)) *
                       ((eroist_system_until_x2015x() - 1) / eroist_system_until_x2015x())),
            max_eroi_fc()))
    #test_nans(ret, 60)
    return ret


@cache('step')
def evol_final_energy_intensity_by_sector_and_fuel_delayed_1yr():
    """
    evol final energy intensity by sector and fuel delayed 1yr



    component


    """
    a = [x() for x in delays_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step]

    ret = xr.DataArray(data=np.reshape(a, evol_final_energy_intensity_by_sector_and_fuel().T.shape).T,
    dims=evol_final_energy_intensity_by_sector_and_fuel().dims, coords=evol_final_energy_intensity_by_sector_and_fuel().coords)
    #ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 61)
    return ret


@cache('step')
def evol_final_energy_intensity_h_delayed_x1x_yr():
    """
    evol final energy intensity h delayed x1x yr



    component


    """
    
    ret = xr.DataArray(data=[x() for x in delays_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step],
        dims=evol_final_energy_intensity_h().dims,
        coords=evol_final_energy_intensity_h().coords)
    #test_nans(ret, 62)
    return ret


@cache('step')
def maximun_yearly_aceleratuin_of_intensity_improvement_h():
    """
    maximun yearly aceleratuin of intensity improvement h



    component


    """
    
    ret = -maximun_yearly_aceleration_of_intensity_improvement_pct_h(
    ) * evol_final_energy_intensity_h_delayed_x1x_yr()
    #test_nans(ret, 63)
    return ret


@cache('step')
def maximun_yearly_aceleratuin_of_intensity_improvement():
    """
    maximun yearly aceleratuin of intensity improvement



    component


    """
    
    ret = -maximun_yearly_aceleration_of_intensity_improvement_pct(
    ) * evol_final_energy_intensity_by_sector_and_fuel_delayed_1yr()
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 64)
    return ret


@cache('run')
def initial_energy_intensity_by_fuel_and_sector_x1995x():
    """
    initial energy intensity by fuel and sector x1995x



    constant


    """
    
    ret = xr.DataArray(data=[[0.72419715,0.1779432,3.32933712,0.21503569,0.47086768],[1.563491,0.39615757,1.34136258,4.69078138,0.95068866],[0.38725897,0.16496473,0.51411063,0.26307121,1.04794477],[0.80985958,0.2208637,0.88069214,0.22170153,0.99195523],[0.35139111,0.11953127,0.54391221,0.09354931,0.36017316],[0.87290452,0.87939526,0.85318442,0.16492388,1.54358467],[1.22637684,0.10098146,0.77252486,0.4924435,1.41596642],[0.91655574,0.83062554,11.72311084,3.00131309,0.56351885],[1.80355296,0.85843673,1.61410459,1.47202838,1.69033825],[1.75167552,0.45484858,2.07496676,1.02938173,2.73744036],[1.60100296,0.11474365,2.19605512,1.34343882,7.9972297],[1.97761559,0.4444717,0.72411951,1.31080652,2.8148349],[0.38497879,0.26636943,0.42382675,0.12021542,0.34965201],[0.38811218,0.10697241,0.47801984,0.11143834,0.08366552],[0.29157176,0.03230247,0.2833001,0.0917108,0.11490861],[0.85691698,0.25247423,2.14392313,0.63619042,2.12600143],[2.62405852,0.17656446,0.64790005,3.92239687,1.30847508],[0.10596937,0.08379637,0.70681406,0.05766783,0.08061731],[0.29023907,0.0343743,0.65161055,0.12338712,0.02572129],[0.19819158,0.04237136,0.79950961,0.08311736,0.03537462],[0.49852024,0.03854825,1.44622456,0.18343785,0.03731645],[0.63162597,0.05421857,1.41963918,0.22968876,0.47227027],[0.44834521,0.0,5.78459465,1.76868907,0.19289284],[0.0,0.0,37.9341609,4.57484716e-06,0.0202575499],[0.0,0.0,26.7157318,1.98484033e-05,0.0],[0.3351251,0.24766114,2.12294444,0.15920033,0.04933503],[0.21424532,0.03630473,0.76886647,0.55711416,0.00880215],[0.13571294,0.03042315,0.39026415,0.04635998,0.01496297],[0.31331707,0.06748503,0.14099642,0.08985637,0.05228868],[0.18155751,0.02293191,0.83315094,0.0977231,0.02983079],[0.40839952,0.05140637,1.18805417,1.53739517,0.07497031],[0.47589912,0.07013227,0.85574009,0.19922448,0.19199273],[0.33605505,0.04920999,1.08999354,0.16141075,0.05863355],[0.38788798,0.11996459,1.14939878,0.2435464,0.06819527],[0.001,0.001,0.001,0.001,0.001]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 65)
    return ret


@cache('run')
def share_max_of_change_vs_historical_mean_h():
    """
    share max of change vs historical mean h



    constant


    """
    
    ret = 0.5
    #test_nans(ret, 66)
    return ret


@cache('step')
def efficiency_energy_aceleration_h():
    """
    efficiency energy aceleration h



    component


    """
    
    ret = functions.if_then_else(maximun_yearly_aceleratuin_of_intensity_improvement_h() *
                                  pressure_to_improve_energy_intensity_efficiency_h() < 0,
                                  maximun_yearly_aceleratuin_of_intensity_improvement_h() *
                                  pressure_to_improve_energy_intensity_efficiency_h(),
                                  functions.if_then_else(
                                      (maximun_yearly_aceleratuin_of_intensity_improvement_h() *
                                       pressure_to_improve_energy_intensity_efficiency_h()) < abs(
                                           historical_mean_rate_energy_intensity_h() *
                                           share_max_of_change_vs_historical_mean_h()),
                                      maximun_yearly_aceleratuin_of_intensity_improvement_h() *
                                      pressure_to_improve_energy_intensity_efficiency_h(),
                                      abs(historical_mean_rate_energy_intensity_h() *
                                          share_max_of_change_vs_historical_mean_h())))
    #test_nans(ret, 67)
    return ret


@cache('run')
def share_max_of_change_vs_historical_mean_rate():
    """
    share max of change vs historical mean rate



    constant


    """
    
    ret = 0.5
    #test_nans(ret, 68)
    return ret


@cache('step')
def efficiency_energy_aceleration():
    """
    efficiency energy aceleration



    component


    """
    
    ret = functions.if_then_else(maximun_yearly_aceleratuin_of_intensity_improvement() *
                                  pressure_to_improve_energy_intensity_efficiency() < 0,
                                  maximun_yearly_aceleratuin_of_intensity_improvement() *
                                  pressure_to_improve_energy_intensity_efficiency(),
                                  functions.if_then_else(
                                      (maximun_yearly_aceleratuin_of_intensity_improvement() *
                                       pressure_to_improve_energy_intensity_efficiency()) < abs(
                                           historical_mean_rate_energy_intensity() *
                                           share_max_of_change_vs_historical_mean_rate()),
                                      maximun_yearly_aceleratuin_of_intensity_improvement() *
                                      pressure_to_improve_energy_intensity_efficiency(),
                                      abs(historical_mean_rate_energy_intensity() *
                                          share_max_of_change_vs_historical_mean_rate())))
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 69)
    return ret


@cache('step')
def evol_final_energy_intensity_h():
    """
    evol final energy intensity h



    component


    """
    
    ret = integ_evol_final_energy_intensity_h()
    #test_nans(ret, 70)
    return ret


@cache('step')
def inertial_rate_energy_intensity_top_down():
    """
    inertial rate energy intensity top down



    component


    """
    
    ret = functions.if_then_else(
        time() > 2008,
        functions.if_then_else(
            functions.logical_or(abs(activate_bottom_up_method() - 0) < equality_threshold,rate_change_intensity_bottom_up() == 0),
            functions.if_then_else(
                (historical_mean_rate_energy_intensity() + efficiency_energy_aceleration()) < 0,
                evol_final_energy_intensity_by_sector_and_fuel() *
                (historical_mean_rate_energy_intensity() + efficiency_energy_aceleration()) *
                available_improvement_efficiency(),
                initial_energy_intensity_by_fuel_and_sector_x1995x() *
                (historical_mean_rate_energy_intensity() + efficiency_energy_aceleration())), 0),
        historic_rate_final_energy_intensity())
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 71)
    return ret


@cache('run')
def historical_mean_rate_energy_intensity():
    """
    historical mean rate energy intensity



    constant


    """
    
    ret = xr.DataArray(data=[[-0.00594331,-0.12011212,-0.01702233,0.00649193,-0.04199447],[-0.01292442,-0.04518246,-0.01034553,-0.00468514,0.01253506],[0.00400537,-0.03138628,-0.01180247,0.00788062,-0.00678344],[-0.00992662,-0.01949561,-0.04541047,-0.01309037,-0.06194968],[-0.02023598,-0.04979546,-0.03162812,-0.02367399,-0.04577093],[-0.01776254,-0.12490597,-0.02823126,0.01183321,-0.00733869],[-0.00950341,0.05313765,-0.04045002,-0.01206145,0.00749482],[-0.01886605,-0.01847262,-0.03188427,-0.02840818,-0.00295949],[-0.01281968,-0.02254765,-0.02264326,-0.01543831,-0.04863373],[-0.00156569,-0.04884816,-0.01991446,-0.01145573,-0.01683848],[0.01540775,-0.01630938,-0.01400116,0.0066328,-0.00025385],[0.00733569,-0.05521067,-0.03842037,-0.01299638,-0.00274809],[-0.01479467,-0.13612799,-0.05007249,-0.00896218,-0.08810427],[-0.04147397,-0.14823739,-0.07540414,-0.04217459,-0.12148873],[-0.00904026,0.02069871,-0.03899174,0.0007079,-0.03058529],[-0.00569454,-0.04044161,-0.02169462,-0.00191565,-0.05464288],[0.00532142,0.00891843,-0.03679899,-0.02325057,0.05228512],[-0.00841393,-0.31124022,-0.00453141,0.00490538,0.0093747],[0.00605911,-0.01422534,-0.02190432,-0.01092198,-0.17006181],[-0.00036395,-0.01905378,-0.04133912,-0.02730321,-0.07895893],[-0.00386347,-0.01325683,-0.02968859,-0.00458373,-0.09655175],[0.01980806,-0.01257244,-0.03131294,0.05258562,-0.02234642],[-0.00404563,0.0,0.00290517,-0.00873679,-0.06650353],[0.0,0.0,-0.03742717,-0.04770914,-0.07070365],[0.0,0.0,-0.00469314,-0.01226672,-0.73407054],[0.01034127,-0.18672971,0.01036057,-0.0406507,-0.09713694],[0.01239715,-0.04082706,-0.04085653,-0.1071439,-0.03762176],[-0.00447336,-0.1398287,-0.04749345,-0.01013251,-0.34654798],[-0.01637684,-0.04922435,-0.02508782,-0.01725737,-0.18054948],[0.00418362,-0.02366548,-0.03195417,-0.00052349,-0.01660373],[0.01668083,-0.02455965,-0.02219986,-0.05911978,0.00105572],[0.03481818,-0.00483609,-0.00077546,0.00180503,-0.07266663],[0.01151148,-0.02092861,-0.025051,-0.00719733,0.03300624],[0.01105651,-0.05757374,-0.01504249,-0.02016622,-0.01382198],[0.0,0.0,0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 72)
    return ret


@cache('step')
def inertial_rate_energy_intensity_h_top_down():
    """
    inertial rate energy intensity h top down



    component


    """
    
    ret = functions.if_then_else(
        time() > 2008,
        functions.if_then_else(
            (historical_mean_rate_energy_intensity_h() + efficiency_energy_aceleration_h()) < 0,
            evol_final_energy_intensity_h() *
            (historical_mean_rate_energy_intensity_h() + efficiency_energy_aceleration_h()) *
            available_improvement_efficiency_h(),
            initial_energy_intensity_x1995x_h() *
            (historical_mean_rate_energy_intensity_h() + efficiency_energy_aceleration_h())),
        historic_rate_final_energy_intensity_h())
    #test_nans(ret, 73)
    return ret


@cache('run')
def p_rr_ti_rest():
    """
    p rr ti rest



    constant


    """
    
    ret = 0.91
    #test_nans(ret, 74)
    return ret


@cache('run')
def p_rr_li_rest():
    """
    p rr li rest



    constant


    """
    
    ret = 0.005
    #test_nans(ret, 75)
    return ret


@cache('run')
def p_rr_v_rest():
    """
    p rr v rest



    constant


    """
    
    ret = 0.005
    #test_nans(ret, 76)
    return ret


@cache('run')
def p_rr_mg_rest():
    """
    p rr mg rest



    constant


    """
    
    ret = 0.39
    #test_nans(ret, 77)
    return ret


@cache('run')
def p_rr_zn_rest():
    """
    p rr zn rest



    constant


    """
    
    ret = 0.395
    #test_nans(ret, 78)
    return ret


@cache('run')
def p_rr_minerals_rest():
    """
    p rr minerals rest



    constant


    """
    
    ret = xr.DataArray(data=[0.0,
        p_rr_al_rest(),0.0,
        p_rr_cd_rest(),0.0,0.0,
        p_rr_cr_rest(),
        p_rr_cu_rest(),0.0,0.0,0.0,0.0,0.0,0.0,
        p_rr_ga_rest(),0.0,0.0,0.0,
        p_rr_in_rest(),
        p_rr_fe_rest(),0.0,0.0,0.0,0.0,
        p_rr_li_rest(),0.0,
        p_rr_mg_rest(),
        p_rr_mn_rest(),0.0,0.0,
        p_rr_mo_rest(),0.0,0.0,0.0,
        p_rr_ni_rest(),0.0,0.0,0.0,
        p_rr_pb_rest(),0.0,0.0,0.0,0.0,0.0,0.0,0.0,
        p_rr_ag_rest(),0.0,
        p_rr_sn_rest(),0.0,0.0,0.0,
        p_rr_te_rest(),
        p_rr_ti_rest(),0.0,
        p_rr_v_rest(),0.0,
        p_rr_zn_rest()],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 79)
    return ret


@cache('run')
def p_rr_cd_rest():
    """
    p rr cd rest



    constant


    """
    
    ret = 0.15
    #test_nans(ret, 80)
    return ret


@cache('step')
def improvement_recycling_rates_minerals_rest():
    """
    improvement recycling rates minerals rest



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        historic_improvement_recycling_rates_minerals(),
        functions.if_then_else(
            abs(choose_targets_mineral_recycling_rates() - 2) < equality_threshold,
            common_rr_minerals_variation_rest() * recycling_rates_minerals_rest(),
            by_mineral_rr_variation_rest())) * constrain_rr_improv_for_rest_per_mineral()
    #test_nans(ret, 81)
    return ret


@cache('run')
def b_lineal_regr_rr_alt_techn():
    """
    b lineal regr rr alt techn



    component


    """
    
    ret = p_rr_minerals_alt_techn() - a_lineal_regr_rr_alt_techn(
    ) * target_year_p_rr_minerals_alt_techn()
    #test_nans(ret, 82)
    return ret


@cache('run')
def b_lineal_regr_rr_rest():
    """
    b lineal regr rr rest



    component


    """
    
    ret = p_rr_minerals_rest() - a_lineal_regr_rr_rest() * target_year_p_rr_minerals_rest()
    #test_nans(ret, 83)
    return ret


@cache('step')
def by_mineral_rr_rest_1yr():
    """
    by mineral rr rest 1yr



    component


    """
    
    ret = xr.DataArray(data=[x() for x in delays_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step],
        dims=by_mineral_rr_rest().dims,
        coords=by_mineral_rr_rest().coords)
    #test_nans(ret, 84)
    return ret


@cache('run')
def p_rr_cu_rest():
    """
    p rr cu rest



    constant


    """
    
    ret = 0.48
    #test_nans(ret, 85)
    return ret


@cache('run')
def a_lineal_regr_rr_alt_techn():
    """
    a lineal regr rr alt techn



    component


    """
    
    ret = (p_rr_minerals_alt_techn() - current_recycling_rates_minerals_alt_techn()) / (
        target_year_p_rr_minerals_alt_techn() - start_year_p_rr_minerals_alt_techn())
    #test_nans(ret, 86)
    return ret


@cache('run')
def a_lineal_regr_rr_rest():
    """
    a lineal regr rr rest



    component


    """
    
    ret = (p_rr_minerals_rest() - current_recycling_rates_minerals()) / (
        target_year_p_rr_minerals_rest() - start_year_p_rr_minerals_rest())
    #test_nans(ret, 87)
    return ret


@cache('run')
def start_year_p_rr_minerals_rest():
    """
    start year p rr minerals rest



    constant


    """
    
    ret = 2020
    #test_nans(ret, 88)
    return ret


@cache('step')
def common_rr_minerals_variation_rest():
    """
    common rr minerals variation rest



    component


    """
    
    ret = functions.if_then_else(time() < start_year_p_common_rr_minerals_rest(),
                                  historic_improvement_recycling_rates_minerals(),
                                  p_common_rr_minerals_variation_rest())
    #test_nans(ret, 89)
    return ret


@cache('run')
def p_rr_in_rest():
    """
    p rr in rest



    constant


    """
    
    ret = 0.005
    #test_nans(ret, 90)
    return ret


@cache('run')
def p_rr_te_rest():
    """
    p rr te rest



    constant


    """
    
    ret = 0.005
    #test_nans(ret, 91)
    return ret


@cache('run')
def p_rr_mo_rest():
    """
    p rr mo rest



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 92)
    return ret


@cache('step')
def by_mineral_rr_rest():
    """
    by mineral rr rest



    component


    """
    
    ret = a_lineal_regr_rr_rest() * time() + b_lineal_regr_rr_rest()
    #test_nans(ret, 93)
    return ret


@cache('run')
def p_rr_ag_rest():
    """
    p rr ag rest



    constant


    """
    
    ret = 0.635
    #test_nans(ret, 94)
    return ret


@cache('run')
def p_rr_mn_rest():
    """
    p rr mn rest



    constant


    """
    
    ret = 0.53
    #test_nans(ret, 95)
    return ret


@cache('run')
def p_rr_al_rest():
    """
    p rr al rest



    constant


    """
    
    ret = 0.5599999999999999
    #test_nans(ret, 96)
    return ret


@cache('run')
def target_year_p_rr_minerals_rest():
    """
    target year p rr minerals rest



    constant


    """
    
    ret = 2050
    #test_nans(ret, 97)
    return ret


@cache('run')
def p_rr_ga_rest():
    """
    p rr ga rest



    constant


    """
    
    ret = 0.005
    #test_nans(ret, 98)
    return ret


@cache('run')
def p_rr_sn_rest():
    """
    p rr sn rest



    constant


    """
    
    ret = 0.75
    #test_nans(ret, 99)
    return ret


@cache('run')
def p_rr_pb_rest():
    """
    p rr pb rest



    constant


    """
    
    ret = 0.735
    #test_nans(ret, 100)
    return ret


@cache('run')
def p_rr_cr_rest():
    """
    p rr cr rest



    constant


    """
    
    ret = 0.9
    #test_nans(ret, 101)
    return ret


@cache('run')
def p_rr_ni_rest():
    """
    p rr ni rest



    constant


    """
    
    ret = 0.6
    #test_nans(ret, 102)
    return ret


@cache('run')
def p_rr_fe_rest():
    """
    p rr fe rest



    constant


    """
    
    ret = 0.71
    #test_nans(ret, 103)
    return ret


@cache('step')
def by_mineral_rr_variation_rest():
    """
    by mineral rr variation rest



    component


    """
    
    ret = functions.if_then_else(time() < 2015,
                                  historic_improvement_recycling_rates_minerals(),
                                  functions.if_then_else(
                                      time() < start_year_p_rr_minerals_rest(),
                                      historic_improvement_recycling_rates_minerals(),
                                      by_mineral_rr_rest() - by_mineral_rr_rest_1yr()))
    #test_nans(ret, 104)
    return ret


@cache('run')
def current_recycling_rates_minerals_alt_techn():
    """
    current recycling rates minerals alt techn



    component


    """
    
    ret = current_recycling_rates_minerals(
    ) * eolxdashxrr_minerals_alt_techn_res_vsx_total_economy()
    #test_nans(ret, 105)
    return ret


@cache('run')
def p_rr_mg_alt_techn():
    """
    p rr mg alt techn



    constant


    """
    
    ret = 0.13
    #test_nans(ret, 106)
    return ret


@cache('run')
def p_rr_minerals_alt_techn():
    """
    p rr minerals alt techn



    constant


    """
    
    ret = xr.DataArray(data=[0.0,
        p_rr_al_alt_techn(),0.0,
        p_rr_cd_alt_techn(),0.0,0.0,
        p_rr_cr_alt_techn(),
        p_rr_cu_alt_techn(),0.0,0.0,0.0,0.0,0.0,0.0,
        p_rr_ga_alt_techn(),0.0,0.0,0.0,
        p_rr_in_alt_techn(),
        p_rr_fe_alt_techn(),0.0,0.0,0.0,0.0,
        p_rr_li_alt_techn(),0.0,
        p_rr_mg_alt_techn(),
        p_rr_mn_alt_techn(),0.0,0.0,
        p_rr_mo_alt_techn(),0.0,0.0,0.0,
        p_rr_ni_alt_techn(),0.0,0.0,0.0,
        p_rr_pb_alt_techn(),0.0,0.0,0.0,0.0,0.0,0.0,0.0,
        p_rr_ag_alt_techn(),0.0,
        p_rr_sn_alt_techn(),0.0,0.0,0.0,
        p_rr_te_alt_techn(),
        p_rr_ti_alt_techn(),0.0,
        p_rr_v_alt_techn(),0.0,
        p_rr_zn_alt_techn()],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 107)
    return ret


@cache('run')
def p_rr_mn_alt_techn():
    """
    p rr mn alt techn



    constant


    """
    
    ret = 0.17666666666666667
    #test_nans(ret, 108)
    return ret


@cache('run')
def p_rr_mo_alt_techn():
    """
    p rr mo alt techn



    constant


    """
    
    ret = 0.09999999999999999
    #test_nans(ret, 109)
    return ret


@cache('run')
def p_rr_ni_alt_techn():
    """
    p rr ni alt techn



    constant


    """
    
    ret = 0.19999999999999998
    #test_nans(ret, 110)
    return ret


@cache('run')
def p_rr_pb_alt_techn():
    """
    p rr pb alt techn



    constant


    """
    
    ret = 0.245
    #test_nans(ret, 111)
    return ret


@cache('step')
def by_mineral_rr_alt_techn():
    """
    by mineral rr alt techn



    component


    """
    
    ret = a_lineal_regr_rr_alt_techn() * time() + b_lineal_regr_rr_alt_techn()
    #test_nans(ret, 112)
    return ret


@cache('step')
def by_mineral_rr_alt_techn_1yr():
    """
    by mineral rr alt techn 1yr



    component


    """
    
    ret = xr.DataArray(data=[x() for x in delays_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step],
        dims=by_mineral_rr_alt_techn().dims,
        coords=by_mineral_rr_alt_techn().coords)
    #test_nans(ret, 113)
    return ret


@cache('run')
def p_rr_al_alt_techn():
    """
    p rr al alt techn



    constant


    """
    
    ret = 0.18666666666666665
    #test_nans(ret, 114)
    return ret


@cache('run')
def p_rr_cd_alt_techn():
    """
    p rr cd alt techn



    constant


    """
    
    ret = 0.049999999999999996
    #test_nans(ret, 115)
    return ret


@cache('run')
def p_rr_cu_alt_techn():
    """
    p rr cu alt techn



    constant


    """
    
    ret = 0.15999999999999998
    #test_nans(ret, 116)
    return ret


@cache('run')
def p_rr_fe_alt_techn():
    """
    p rr fe alt techn



    constant


    """
    
    ret = 0.23666666666666664
    #test_nans(ret, 117)
    return ret


@cache('run')
def p_rr_ga_alt_techn():
    """
    p rr ga alt techn



    constant


    """
    
    ret = 0.0016666666666666666
    #test_nans(ret, 118)
    return ret


@cache('run')
def p_rr_in_alt_techn():
    """
    p rr in alt techn



    constant


    """
    
    ret = 0.0016666666666666666
    #test_nans(ret, 119)
    return ret


@cache('run')
def p_rr_li_alt_techn():
    """
    p rr li alt techn



    constant


    """
    
    ret = 0.0016666666666666666
    #test_nans(ret, 120)
    return ret


@cache('step')
def recycling_rates_minerals_alt_techn():
    """
    recycling rates minerals alt techn



    component


    """
    
    ret = integ_recycling_rates_minerals_alt_techn()
    #test_nans(ret, 121)
    return ret


@cache('run')
def p_rr_v_alt_techn():
    """
    p rr v alt techn



    constant


    """
    
    ret = 0.0016666666666666666
    #test_nans(ret, 122)
    return ret


@cache('run')
def p_rr_zn_alt_techn():
    """
    p rr zn alt techn



    constant


    """
    
    ret = 0.13166666666666665
    #test_nans(ret, 123)
    return ret


@cache('run')
def p_rr_ti_alt_techn():
    """
    p rr ti alt techn



    constant


    """
    
    ret = 0.30333333333333334
    #test_nans(ret, 124)
    return ret


@cache('run')
def p_rr_cr_alt_techn():
    """
    p rr cr alt techn



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 125)
    return ret


@cache('run')
def p_rr_ag_alt_techn():
    """
    p rr ag alt techn



    constant


    """
    
    ret = 0.21166666666666667
    #test_nans(ret, 126)
    return ret


@cache('run')
def p_rr_sn_alt_techn():
    """
    p rr sn alt techn



    constant


    """
    
    ret = 0.25
    #test_nans(ret, 127)
    return ret


@cache('run')
def p_rr_te_alt_techn():
    """
    p rr te alt techn



    constant


    """
    
    ret = 0.0016666666666666666
    #test_nans(ret, 128)
    return ret


@cache('step')
def improvement_recycling_rates_minerals_alt_techn():
    """
    improvement recycling rates minerals alt techn



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        historic_improvement_recycling_rates_minerals(),
        functions.if_then_else(
            abs(choose_targets_mineral_recycling_rates() - 2) < equality_threshold,
            common_rr_minerals_variation_alt_techn() * recycling_rates_minerals_alt_techn(),
            by_mineral_rr_variation_alt_techn())) * constrain_rr_improv_for_alt_techn_per_mineral(
            )
    #test_nans(ret, 129)
    return ret


@cache('step')
def by_mineral_rr_variation_alt_techn():
    """
    by mineral rr variation alt techn



    component


    """
    
    ret = functions.if_then_else(time() < 2015,
                                  historic_improvement_recycling_rates_minerals(),
                                  functions.if_then_else(
                                      time() < start_year_p_rr_minerals_alt_techn(),
                                      historic_improvement_recycling_rates_minerals(),
                                      by_mineral_rr_alt_techn() - by_mineral_rr_alt_techn_1yr()))
    #test_nans(ret, 130)
    return ret


@cache('run')
def start_year_p_rr_minerals_alt_techn():
    """
    start year p rr minerals alt techn



    constant


    """
    
    ret = 2020
    #test_nans(ret, 131)
    return ret


@cache('run')
def target_year_p_rr_minerals_alt_techn():
    """
    target year p rr minerals alt techn



    constant


    """
    
    ret = 2050
    #test_nans(ret, 132)
    return ret


@cache('run')
def temp_change_x1x5c():
    """
    temp change x1x5c



    constant


    """
    
    ret = 1.5
    #test_nans(ret, 133)
    return ret


@cache('run')
def density():
    """
    density



    constant


    """
    
    ret = 1000
    #test_nans(ret, 134)
    return ret


@cache('run')
def sec_per_day():
    """
    sec per day



    constant


    """
    
    ret = 86400.0
    #test_nans(ret, 135)
    return ret


@cache('step')
def sec_per_yr():
    """
    sec per yr



    component


    """
    
    ret = initial_days_per_yearsec_per_day()
    #test_nans(ret, 136)
    return ret


@cache('run')
def init_atmos_uocean_temp():
    """
    init atmos uocean temp



    constant


    """
    
    ret = 0.585
    #test_nans(ret, 137)
    return ret


@cache('run')
def watt_per_j_s():
    """
    watt per j s



    constant


    """
    
    ret = 1.0
    #test_nans(ret, 138)
    return ret


@cache('step')
def lower_layer_volume_vu():
    """
    lower layer volume vu



    component


    """
    
    ret = initial_area1land_area_fractionlayer_depth()
    #test_nans(ret, 139)
    return ret


@cache('step')
def volumetric_heat_capacity():
    """
    volumetric heat capacity



    component


    """
    
    ret = initial_mass_heat_capwatt_per_j_ssec_per_yrdensity()
    #test_nans(ret, 140)
    return ret


@cache('run')
def mass_heat_cap():
    """
    mass heat cap



    constant


    """
    
    ret = 4186
    #test_nans(ret, 141)
    return ret


@cache('run')
def heat_diffusion_covar():
    """
    heat diffusion covar



    constant


    """
    
    ret = 1
    #test_nans(ret, 142)
    return ret


@cache('step')
def x2x_co2x_forcing():
    """
    x2x co2x forcing



    component


    """
    
    ret = initial_co2_rad_forcenplog2()
    #test_nans(ret, 143)
    return ret


@cache('run')
def climate_sensitivity_to_2x_co2():
    """
    climate sensitivity to 2x co2



    constant


    """
    
    ret = 2.9
    #test_nans(ret, 144)
    return ret


@cache('step')
def temperature_change():
    """
    temperature change



    component


    """
    
    ret = heat_in_atmosphere_and_upper_ocean() / atm_and_upper_ocean_heat_cap()
    #test_nans(ret, 145)
    return ret


@cache('run')
def land_thickness():
    """
    land thickness



    constant


    """
    
    ret = 8.4
    #test_nans(ret, 146)
    return ret


@cache('step')
def heat_transfer():
    """
    heat transfer



    component


    """
    ret = xr.DataArray(data=[
        (temperature_change() - relative_deep_ocean_temp().loc['layer1'].values) * heat_transfer_coeff() /
        mean_depth_of_adjacent_layers().loc['layer1'].values,
        (relative_deep_ocean_temp().loc['layer1'].values - relative_deep_ocean_temp().loc['layer2'].values) * heat_transfer_coeff() /
        mean_depth_of_adjacent_layers().loc['layer2'].values,
        (relative_deep_ocean_temp().loc['layer2'].values - relative_deep_ocean_temp().loc['layer3'].values) * heat_transfer_coeff() /
        mean_depth_of_adjacent_layers().loc['layer3'].values,
        (relative_deep_ocean_temp().loc['layer3'].values - relative_deep_ocean_temp().loc['layer4'].values) * heat_transfer_coeff() /
        mean_depth_of_adjacent_layers().loc['layer4'].values],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])

    #test_nans(ret, 147)
    return ret


@cache('step')
def climate_feedback_param():
    """
    climate feedback param



    component


    """
    
    ret = initial_x2x_co2x_forcingclimate_sensitivity_to_2x_co2()
    #test_nans(ret, 148)
    return ret


@cache('run')
def area():
    """
    area



    constant


    """
    
    ret = 510000000000000
    #test_nans(ret, 149)
    return ret


@cache('step')
def heat_transfer_coeff():
    """
    heat transfer coeff



    component


    """
    
    ret = initial_heat_transfer_ratemean_depth_of_adjacent_layersheat_diffusion_covareddy_diff_coeffeddy_diff_mean1heat_diffusion_covar(
    )
    #test_nans(ret, 150)
    return ret


@cache('step')
def eddy_diff_coeff():
    """
    eddy diff coeff



    component


    """
    
    ret = initial_eddy_diff_coeff_indexeddy_diff_mean()
    #test_nans(ret, 151)
    return ret


@cache('step')
def upper_layer_volume_vu():
    """
    upper layer volume vu



    component


    """
    
    ret = initial_arealand_area_fractionland_thickness1land_area_fractionmixed_depth()
    #test_nans(ret, 152)
    return ret


@cache('step')
def heat_in_deep_ocean():
    """
    heat in deep ocean



    component


    """
    
    ret = integ_heat_in_deep_ocean()
    #test_nans(ret, 153)
    return ret


@cache('run')
def land_area_fraction():
    """
    land area fraction



    constant


    """
    
    ret = 0.292
    #test_nans(ret, 154)
    return ret


@cache('step')
def relative_deep_ocean_temp():
    """
    relative deep ocean temp



    component


    """
    
    ret = heat_in_deep_ocean() / deep_ocean_heat_cap()
    #test_nans(ret, 155)
    return ret


@cache('run')
def co2_rad_force_coeffcroads():
    """
    co2 rad force coeffcroads



    constant


    """
    
    ret = 1
    #test_nans(ret, 156)
    return ret


@cache('run')
def temp_change_x2x():
    """
    temp change x2x



    constant


    """
    
    ret = 2
    #test_nans(ret, 157)
    return ret


@cache('step')
def feedback_cooling():
    """
    feedback cooling



    component


    """
    
    ret = temperature_change() * climate_feedback_param()
    #test_nans(ret, 158)
    return ret


@cache('run')
def heat_transfer_rate():
    """
    heat transfer rate



    constant


    """
    
    ret = 1.23
    #test_nans(ret, 159)
    return ret


@cache('step')
def atm_and_upper_ocean_heat_cap():
    """
    atm and upper ocean heat cap



    component


    """
    
    ret = initial_upper_layer_volume_vuvolumetric_heat_capacityarea()
    #test_nans(ret, 160)
    return ret


@cache('step')
def deep_ocean_heat_cap():
    """
    deep ocean heat cap



    component


    """
    
    ret = initial_lower_layer_volume_vuvolumetric_heat_capacityarea()
    #test_nans(ret, 161)
    return ret


@cache('run')
def init_deep_ocean_temp():
    """
    init deep ocean temp



    constant


    """
    
    ret = xr.DataArray(data=[0.3506,0.1814,0.1492,0.00038],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 162)
    return ret


@cache('step')
def heat_in_atmosphere_and_upper_ocean():
    """
    heat in atmosphere and upper ocean



    component


    """
    print('time = {:.2f}'.format(time()))
    ret = integ_heat_in_atmosphere_and_upper_ocean()
    #test_nans(ret, 163)
    return ret


@cache('run')
def ch4_n2o_unit_adj():
    """
    ch4 n2o unit adj



    constant


    """
    
    ret = 1
    #test_nans(ret, 164)
    return ret


@cache('run')
def adjustment_for_ch4ref_and_n2oref():
    """
    adjustment for ch4ref and n2oref



    component


    """
    
    ret = ch4_n2o_interaction_coeffient() * np.log(1 + ch4_n2o_inter_coef_x2x() * (
        ch4_reference_conc() * n2o_reference_conc() * ch4_n2o_unit_adj() * ch4_n2o_unit_adj()
    )**ch4_n20_inter_exp() + ch4_n2o_inter_coef_x3x() * ch4_reference_conc() * ch4_n2o_unit_adj(
    ) * (ch4_reference_conc() * n2o_reference_conc() * ch4_n2o_unit_adj() * ch4_n2o_unit_adj()
         )**ch4_n20_inter_exp_x2x())
    #test_nans(ret, 165)
    return ret


@cache('run')
def ch4_radiative_efficiency_coefficient():
    """
    ch4 radiative efficiency coefficient



    constant


    """
    
    ret = 0.036
    #test_nans(ret, 166)
    return ret


@cache('step')
def total_radiative_forcing():
    """
    total radiative forcing



    component


    """
    
    ret = wellxdashxmixed_ghg_forcing() + adjusted_other_forcings()
    #test_nans(ret, 167)
    return ret


@cache('run')
def last_historical_rf_year():
    """
    last historical rf year



    constant


    """
    
    ret = 2010
    #test_nans(ret, 168)
    return ret


@cache('run')
def ch4_reference_conc():
    """
    ch4 reference conc



    constant


    """
    
    ret = 722
    #test_nans(ret, 169)
    return ret


@cache('step')
def rf_from_f_gases():
    """
    rf from f gases



    component


    """
    
    ret = pfc_rf() + sf6_rf() + hfc_rf_total()
    #test_nans(ret, 170)
    return ret


@cache('step')
def wellxdashxmixed_ghg_forcing():
    """
    wellxdashxmixed ghg forcing



    component


    """
    
    ret = co2_radiative_forcing() + ch4_and_n2o_radiative_forcing() + halocarbon_rf()
    #test_nans(ret, 171)
    return ret


@cache('run')
def mineral_aerosols_and_land_rf():
    """
    mineral aerosols and land rf



    constant


    """
    
    ret = -0.3
    #test_nans(ret, 172)
    return ret


@cache('run')
def time_to_commit_rf():
    """
    time to commit rf



    constant


    """
    
    ret = 2200
    #test_nans(ret, 173)
    return ret


@cache('run')
def ch4_n20_inter_exp():
    """
    ch4 n20 inter exp



    constant


    """
    
    ret = 0.75
    #test_nans(ret, 174)
    return ret


@cache('run')
def ch4_n20_inter_exp_x2x():
    """
    ch4 n20 inter exp x2x



    constant


    """
    
    ret = 1.52
    #test_nans(ret, 175)
    return ret


@cache('run')
def ch4_n2o_inter_coef_x2x():
    """
    ch4 n2o inter coef x2x



    constant


    """
    
    ret = 2.01e-05
    #test_nans(ret, 176)
    return ret


@cache('step')
def adjusted_other_forcings():
    """
    adjusted other forcings



    component


    """
    
    ret = other_forcings() + functions.if_then_else(time() > last_historical_rf_year(),
                                                     mineral_aerosols_and_land_rf(), 0)
    #test_nans(ret, 177)
    return ret


@cache('step')
def adjustment_for_ch4_and_n2oref():
    """
    adjustment for ch4 and n2oref



    component


    """
    
    ret = ch4_n2o_interaction_coeffient() * np.log(1 + ch4_n2o_inter_coef_x2x() * (
        ch4_atm_conc() * n2o_reference_conc() * ch4_n2o_unit_adj() * ch4_n2o_unit_adj()
    )**ch4_n20_inter_exp() + ch4_n2o_inter_coef_x3x() * ch4_atm_conc() * ch4_n2o_unit_adj() * (
        ch4_atm_conc() * n2o_reference_conc() * ch4_n2o_unit_adj() * ch4_n2o_unit_adj()
    )**ch4_n20_inter_exp_x2x())
    #test_nans(ret, 178)
    return ret


@cache('step')
def adjustment_for_ch4ref_and_n2o():
    """
    adjustment for ch4ref and n2o



    component


    """
    
    ret = ch4_n2o_interaction_coeffient() * np.log(1 + ch4_n2o_inter_coef_x2x() * (
        ch4_reference_conc() * n2o_atm_conc() * ch4_n2o_unit_adj() * ch4_n2o_unit_adj()
    )**ch4_n20_inter_exp() + ch4_n2o_inter_coef_x3x() * ch4_reference_conc() * ch4_n2o_unit_adj(
    ) * (ch4_reference_conc() * n2o_atm_conc() * ch4_n2o_unit_adj() * ch4_n2o_unit_adj()
         )**ch4_n20_inter_exp_x2x())
    #test_nans(ret, 179)
    return ret


@cache('run')
def n2o_reference_conc():
    """
    n2o reference conc



    constant


    """
    
    ret = 270
    #test_nans(ret, 180)
    return ret


@cache('step')
def delay_effective_radiative_forcing():
    """
    delay effective radiative forcing



    component


    """
    
    ret = delay_total_radiative_forcing_roundtime_step__time_step___time_step_total_radiative_forcing_time_step__time_step(
    )
    #test_nans(ret, 181)
    return ret


@cache('step')
def effective_radiative_forcing():
    """
    effective radiative forcing



    component


    """
    
    ret = functions.if_then_else(time() <= time_to_commit_rf(),
                                  total_radiative_forcing(), delay_effective_radiative_forcing())
    #test_nans(ret, 182)
    return ret


@cache('run')
def co2_rad_force():
    """
    co2 rad force



    constant


    """
    
    ret = 5.35
    #test_nans(ret, 183)
    return ret


@cache('step')
def co2_radiative_forcing():
    """
    co2 radiative forcing



    component


    """
    
    ret = co2_rad_force() * np.log(c_in_atmosphere() / preindustrial_c())
    #test_nans(ret, 184)
    return ret


@cache('run')
def ch4_n2o_interaction_coeffient():
    """
    ch4 n2o interaction coeffient



    constant


    """
    
    ret = 0.47
    #test_nans(ret, 185)
    return ret


@cache('run')
def n2o_radiative_efficiency_coefficient():
    """
    n2o radiative efficiency coefficient



    constant


    """
    
    ret = 0.12
    #test_nans(ret, 186)
    return ret


@cache('run')
def ch4_n2o_inter_coef_x3x():
    """
    ch4 n2o inter coef x3x



    constant


    """
    
    ret = 5.31e-15
    #test_nans(ret, 187)
    return ret


@cache('step')
def n2o_radiative_forcing():
    """
    n2o radiative forcing



    component


    """
    
    ret = n2o_radiative_efficiency_coefficient() * (np.sqrt(n2o_atm_conc() * ch4_n2o_unit_adj(
    )) - np.sqrt(n2o_reference_conc() * ch4_n2o_unit_adj())) - (
        adjustment_for_ch4ref_and_n2o() - adjustment_for_ch4ref_and_n2oref())
    #test_nans(ret, 188)
    return ret


@cache('step')
def ch4_radiative_forcing():
    """
    ch4 radiative forcing



    component


    """
    
    ret = ch4_radiative_efficiency_coefficient() * (np.sqrt(ch4_atm_conc() * ch4_n2o_unit_adj(
    )) - np.sqrt(ch4_reference_conc() * ch4_n2o_unit_adj())) - (
        adjustment_for_ch4_and_n2oref() - adjustment_for_ch4ref_and_n2oref())
    #test_nans(ret, 189)
    return ret


@cache('step')
def other_ghg_rad_forcing_xnon_co2x():
    """
    other ghg rad forcing xnon co2x



    component


    """
    
    ret = total_radiative_forcing() - co2_radiative_forcing()
    #test_nans(ret, 190)
    return ret


@cache('step')
def ch4_and_n2o_radiative_forcing():
    """
    ch4 and n2o radiative forcing



    component


    """
    
    ret = ch4_radiative_forcing() + n2o_radiative_forcing()
    #test_nans(ret, 191)
    return ret


@cache('step')
def hfc_rf_total():
    """
    hfc rf total



    component


    """
    
    ret = functions.sum(hfc_rf())
    #test_nans(ret, 192)
    return ret


@cache('run')
def time_const_for_hfc():
    """
    time const for hfc



    constant


    """
    
    ret = xr.DataArray(data=[13.4,222.0,5.2,28.2,47.1,1.5,38.9,6.5,16.1],
        coords={'hfc type': ['hfc134a', 'hfc23', 'hfc32', 'hfc125', 'hfc143a', 'hfc152a', 'hfc227ea', 'hfc245ca', 'hfc4310mee']},
        dims=['hfc type'])
    #test_nans(ret, 193)
    return ret


@cache('run')
def time_const_for_n2o():
    """
    time const for n2o



    constant


    """
    
    ret = 117
    #test_nans(ret, 194)
    return ret


@cache('run')
def time_const_for_pfc():
    """
    time const for pfc



    constant


    """
    
    ret = 50000
    #test_nans(ret, 195)
    return ret


@cache('run')
def sensitivity_of_methane_emissions_to_permafrost_and_clathrate():
    """
    sensitivity of methane emissions to permafrost and clathrate



    constant


    """
    
    ret = 0
    #test_nans(ret, 196)
    return ret


@cache('run')
def reference_ch4_time_constant():
    """
    reference ch4 time constant



    constant


    """
    
    ret = 8.5
    #test_nans(ret, 197)
    return ret


@cache('run')
def stratospheric_ch4_path_share():
    """
    stratospheric ch4 path share



    constant


    """
    
    ret = 0.08
    #test_nans(ret, 198)
    return ret


@cache('run')
def g_per_ton():
    """
    g per ton



    constant


    """
    
    ret = 1000000.0
    #test_nans(ret, 199)
    return ret


@cache('run')
def init_pfc_in_atm_con():
    """
    init pfc in atm con



    constant


    """
    
    ret = 76
    #test_nans(ret, 200)
    return ret


@cache('run')
def inital_hfc_con():
    """
    inital hfc con



    constant


    """
    
    ret = xr.DataArray(data=[1.67,11.5,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'hfc type': ['hfc134a', 'hfc23', 'hfc32', 'hfc125', 'hfc143a', 'hfc152a', 'hfc227ea', 'hfc245ca', 'hfc4310mee']},
        dims=['hfc type'])
    #test_nans(ret, 201)
    return ret


@cache('step')
def sf6():
    """
    sf6



    component


    """
    
    ret = integ_sf6()
    #test_nans(ret, 202)
    return ret


@cache('step')
def sf6_atm_conc():
    """
    sf6 atm conc



    component


    """
    
    ret = sf6() * ppt_sf6_per_tons_sf6()
    #test_nans(ret, 203)
    return ret


@cache('run')
def sf6_molar_mass():
    """
    sf6 molar mass



    constant


    """
    
    ret = 146
    #test_nans(ret, 204)
    return ret


@cache('run')
def sf6_radiative_efficiency():
    """
    sf6 radiative efficiency



    constant


    """
    
    ret = 0.57
    #test_nans(ret, 205)
    return ret


@cache('step')
def sf6_rf():
    """
    sf6 rf



    component


    """
    
    ret = (sf6_atm_conc() - preindustrial_sf6_conc()) * sf6_radiative_efficiency() / ppt_per_ppb()
    #test_nans(ret, 206)
    return ret


@cache('step')
def sf6_uptake():
    """
    sf6 uptake



    component


    """
    
    ret = sf6() / time_const_for_sf6()
    #test_nans(ret, 207)
    return ret


@cache('step')
def initial_ch4():
    """
    initial ch4



    component


    """
    
    ret = initial_initial_ch4_concppb_ch4_per_mton_ch4()
    #test_nans(ret, 208)
    return ret


@cache('run')
def initial_ch4_conc():
    """
    initial ch4 conc



    constant


    """
    
    ret = 1726
    #test_nans(ret, 209)
    return ret


@cache('run')
def pfc_radiative_efficiency():
    """
    pfc radiative efficiency



    constant


    """
    
    ret = 0.09
    #test_nans(ret, 210)
    return ret


@cache('step')
def ch4_emissions_from_permafrost_and_clathrate():
    """
    ch4 emissions from permafrost and clathrate



    component


    """
    
    ret = sensitivity_of_methane_emissions_to_permafrost_and_clathrate(
    ) * reference_sensitivity_of_ch4_from_permafrost_and_clathrate_to_temperature() * np.maximum(
        0,
        temperature_change() -
        temperature_threshold_for_methane_emissions_from_permafrost_and_clathrate())
    #test_nans(ret, 211)
    return ret


@cache('step')
def hfc_in_atm():
    """
    hfc in atm



    component


    """
    
    ret = integ_hfc_in_atm()
    #test_nans(ret, 212)
    return ret


@cache('run')
def hfc_molar_mass():
    """
    hfc molar mass



    constant


    """
    
    ret = xr.DataArray(data=[102.0,70.0,52.0,120.0,84.0,66.0,170.0,134.0,252.0],
        coords={'hfc type': ['hfc134a', 'hfc23', 'hfc32', 'hfc125', 'hfc143a', 'hfc152a', 'hfc227ea', 'hfc245ca', 'hfc4310mee']},
        dims=['hfc type'])
    #test_nans(ret, 213)
    return ret


@cache('run')
def hfc_radiative_efficiency():
    """
    hfc radiative efficiency



    constant


    """
    
    ret = xr.DataArray(data=[0.19,0.18,0.11,0.23,0.16,0.1,0.26,0.24,0.42],
        coords={'hfc type': ['hfc134a', 'hfc23', 'hfc32', 'hfc125', 'hfc143a', 'hfc152a', 'hfc227ea', 'hfc245ca', 'hfc4310mee']},
        dims=['hfc type'])
    #test_nans(ret, 214)
    return ret


@cache('step')
def ch4_fractional_uptake():
    """
    ch4 fractional uptake



    component


    """
    
    ret = 1 / reference_ch4_time_constant() * (
        tropospheric_ch4_path_share() /
        (stratospheric_ch4_path_share() *
         (ch4_in_atm() / preindustrial_ch4()) + 1 - stratospheric_ch4_path_share()) +
        (1 - tropospheric_ch4_path_share()))
    #test_nans(ret, 215)
    return ret


@cache('step')
def hfc_uptake():
    """
    hfc uptake



    component


    """
    
    ret = hfc_in_atm() / time_const_for_hfc()
    #test_nans(ret, 216)
    return ret


@cache('step')
def total_c_from_permafrost():
    """
    total c from permafrost



    component


    """
    
    ret = integ_total_c_from_permafrost()
    #test_nans(ret, 217)
    return ret


@cache('run')
def ch4_molar_mass():
    """
    ch4 molar mass



    constant


    """
    
    ret = 16
    #test_nans(ret, 218)
    return ret


@cache('run')
def n2oxdashxn_molar_mass():
    """
    n2oxdashxn molar mass



    constant


    """
    
    ret = 28
    #test_nans(ret, 219)
    return ret


@cache('step')
def ch4_uptake():
    """
    ch4 uptake



    component


    """
    
    ret = ch4_in_atm() * ch4_fractional_uptake()
    #test_nans(ret, 220)
    return ret


@cache('step')
def total_ch4_released():
    """
    total ch4 released



    component


    """
    
    ret = integ_total_ch4_released()
    #test_nans(ret, 221)
    return ret


@cache('step')
def natural_pfc_emissions():
    """
    natural pfc emissions



    component


    """
    
    ret = preindustrial_pfc() / time_const_for_pfc()
    #test_nans(ret, 222)
    return ret


@cache('run')
def choose_rcp():
    """
    choose rcp



    constant


    """
    
    ret = 4
    #test_nans(ret, 223)
    return ret


@cache('step')
def pfc_atm_conc():
    """
    pfc atm conc



    component


    """
    
    ret = pfc_in_atm() * ppt_pfc_per_tons_pfc()
    #test_nans(ret, 224)
    return ret


@cache('step')
def ppb_ch4_per_mton_ch4():
    """
    ppb ch4 per mton ch4



    component


    """
    
    ret = initial_ppt_per_molch4_molar_massg_per_tonton_per_mtonppt_per_ppb()
    #test_nans(ret, 225)
    return ret


@cache('step')
def ppb_n2o_per_mtonn():
    """
    ppb n2o per mtonn



    component


    """
    
    ret = initial_ppt_per_moln2oxdashxn_molar_massg_per_tonton_per_mtonppt_per_ppb()
    #test_nans(ret, 226)
    return ret


@cache('step')
def ppt_hfc_per_tons_hfc():
    """
    ppt hfc per tons hfc



    component


    """
    
    ret = initial_ppt_per_molhfc_molar_massg_per_ton()
    #test_nans(ret, 227)
    return ret


@cache('run')
def ppt_per_mol():
    """
    ppt per mol



    constant


    """
    
    ret = 5.68e-09
    #test_nans(ret, 228)
    return ret


@cache('run')
def ppt_per_ppb():
    """
    ppt per ppb



    constant


    """
    
    ret = 1000.0
    #test_nans(ret, 229)
    return ret


@cache('step')
def ppt_pfc_per_tons_pfc():
    """
    ppt pfc per tons pfc



    component


    """
    
    ret = initial_ppt_per_molcf4_molar_massg_per_ton()
    #test_nans(ret, 230)
    return ret


@cache('step')
def c_from_ch4_oxidation():
    """
    c from ch4 oxidation



    component


    """
    
    ret = ch4_uptake() / ch4_per_c() / mtons_per_gtons()
    #test_nans(ret, 231)
    return ret


@cache('run')
def reference_sensitivity_of_ch4_from_permafrost_and_clathrate_to_temperature():
    """
    reference sensitivity of ch4 from permafrost and clathrate to temperature



    constant


    """
    
    ret = 50
    #test_nans(ret, 232)
    return ret


@cache('step')
def global_ch4_emissions():
    """
    global ch4 emissions



    component


    """
    
    ret = global_ch4_anthro_emissions() + natural_ch4_emissions()
    #test_nans(ret, 233)
    return ret


@cache('run')
def preindustrial_ch4():
    """
    preindustrial ch4



    constant


    """
    
    ret = 2213
    #test_nans(ret, 234)
    return ret


@cache('run')
def preindustrial_hfc_conc():
    """
    preindustrial hfc conc



    constant


    """
    
    ret = 0
    #test_nans(ret, 235)
    return ret


@cache('step')
def pfc_in_atm():
    """
    pfc in atm



    component


    """
    
    ret = integ_pfc_in_atm()
    #test_nans(ret, 236)
    return ret


@cache('run')
def initial_sf6_con():
    """
    initial sf6 con



    constant


    """
    
    ret = 3.47
    #test_nans(ret, 237)
    return ret


@cache('step')
def pfc_rf():
    """
    pfc rf



    component


    """
    
    ret = (pfc_atm_conc() - preindustrial_pfc_conc()) * pfc_radiative_efficiency() / ppt_per_ppb()
    #test_nans(ret, 238)
    return ret


@cache('step')
def pfc_uptake():
    """
    pfc uptake



    component


    """
    
    ret = pfc_in_atm() / time_const_for_pfc()
    #test_nans(ret, 239)
    return ret


@cache('step')
def init_pfc_in_atm():
    """
    init pfc in atm



    component


    """
    
    ret = init_pfc_in_atm_con() / ppt_pfc_per_tons_pfc()
    #test_nans(ret, 240)
    return ret


@cache('step')
def n2o_atm_conc():
    """
    n2o atm conc



    component


    """
    
    ret = n2o_in_atm() * ppb_n2o_per_mtonn()
    #test_nans(ret, 241)
    return ret


@cache('step')
def n2o_in_atm():
    """
    n2o in atm



    component


    """
    
    ret = integ_n2o_in_atm()
    #test_nans(ret, 242)
    return ret


@cache('step')
def n2o_uptake():
    """
    n2o uptake



    component


    """
    
    ret = n2o_in_atm() / time_const_for_n2o()
    #test_nans(ret, 243)
    return ret


@cache('step')
def hfc_atm_conc():
    """
    hfc atm conc



    component


    """
    
    ret = hfc_in_atm() * ppt_hfc_per_tons_hfc()
    #test_nans(ret, 244)
    return ret


@cache('run')
def ton_per_mton():
    """
    ton per mton



    constant


    """
    
    ret = 1000000.0
    #test_nans(ret, 245)
    return ret


@cache('run')
def natural_n2o_emissions():
    """
    natural n2o emissions



    constant


    """
    
    ret = 11
    #test_nans(ret, 246)
    return ret


@cache('run')
def cf4_molar_mass():
    """
    cf4 molar mass



    constant


    """
    
    ret = 88
    #test_nans(ret, 247)
    return ret


@cache('step')
def ch4_atm_conc():
    """
    ch4 atm conc



    component


    """
    
    ret = ch4_in_atm() * ppb_ch4_per_mton_ch4()
    #test_nans(ret, 248)
    return ret


@cache('run')
def preindustrial_sf6_conc():
    """
    preindustrial sf6 conc



    constant


    """
    
    ret = 0
    #test_nans(ret, 249)
    return ret


@cache('step')
def time_const_for_ch4():
    """
    time const for ch4



    component


    """
    
    ret = 1 / ch4_fractional_uptake()
    #test_nans(ret, 250)
    return ret


@cache('step')
def initial_hfc():
    """
    initial hfc



    component


    """
    
    ret = initial_inital_hfc_conppt_hfc_per_tons_hfc()
    #test_nans(ret, 251)
    return ret


@cache('step')
def flux_c_from_permafrost_release():
    """
    flux c from permafrost release



    component


    """
    
    ret = sensitivity_of_methane_emissions_to_permafrost_and_clathrate(
    ) * reference_sensitivity_of_c_from_permafrost_and_clathrate_to_temperature() * np.maximum(
        0,
        temperature_change() -
        temperature_threshold_for_methane_emissions_from_permafrost_and_clathrate())
    #test_nans(ret, 252)
    return ret


@cache('step')
def global_n2o_emissions():
    """
    global n2o emissions



    component


    """
    
    ret = global_n2o_anthro_emissions() + natural_n2o_emissions()
    #test_nans(ret, 253)
    return ret


@cache('run')
def tropospheric_ch4_path_share():
    """
    tropospheric ch4 path share



    constant


    """
    
    ret = 0.88
    #test_nans(ret, 254)
    return ret


@cache('step')
def preindustrial_pfc():
    """
    preindustrial pfc



    component


    """
    
    ret = initial_preindustrial_pfc_concppt_pfc_per_tons_pfc()
    #test_nans(ret, 255)
    return ret


@cache('step')
def ppt_sf6_per_tons_sf6():
    """
    ppt sf6 per tons sf6



    component


    """
    
    ret = initial_ppt_per_molsf6_molar_massg_per_ton()
    #test_nans(ret, 256)
    return ret


@cache('step')
def hfc_rf():
    """
    hfc rf



    component


    """
    
    ret = (hfc_atm_conc() - preindustrial_hfc_conc()) * hfc_radiative_efficiency() / ppt_per_ppb()
    #test_nans(ret, 257)
    return ret


@cache('step')
def initial_n2o():
    """
    initial n2o



    component


    """
    
    ret = initial_initial_n2o_concppb_n2o_per_mtonn()
    #test_nans(ret, 258)
    return ret


@cache('run')
def initial_n2o_conc():
    """
    initial n2o conc



    constant


    """
    
    ret = 312.3
    #test_nans(ret, 259)
    return ret


@cache('run')
def temperature_threshold_for_methane_emissions_from_permafrost_and_clathrate():
    """
    temperature threshold for methane emissions from permafrost and clathrate



    constant


    """
    
    ret = 1
    #test_nans(ret, 260)
    return ret


@cache('run')
def time_const_for_sf6():
    """
    time const for sf6



    constant


    """
    
    ret = 3200
    #test_nans(ret, 261)
    return ret


@cache('run')
def preindustrial_pfc_conc():
    """
    preindustrial pfc conc



    constant


    """
    
    ret = 40
    #test_nans(ret, 262)
    return ret


@cache('step')
def initial_sf6():
    """
    initial sf6



    component


    """
    
    ret = initial_sf6_con() / ppt_sf6_per_tons_sf6()
    #test_nans(ret, 263)
    return ret


@cache('run')
def reference_sensitivity_of_c_from_permafrost_and_clathrate_to_temperature():
    """
    reference sensitivity of c from permafrost and clathrate to temperature



    constant


    """
    
    ret = 1
    #test_nans(ret, 264)
    return ret


@cache('step')
def global_total_pfc_emissions():
    """
    global total pfc emissions



    component


    """
    
    ret = global_pfc_emissions() + natural_pfc_emissions()
    #test_nans(ret, 265)
    return ret


@cache('step')
def ch4_in_atm():
    """
    ch4 in atm



    component


    """
    
    ret = integ_ch4_in_atm()
    #test_nans(ret, 266)
    return ret


@cache('step')
def global_anthropogenic_ch4_emissions():
    """
    global anthropogenic ch4 emissions



    component


    """
    
    ret = global_ch4_anthro_emissions()
    #test_nans(ret, 267)
    return ret


@cache('step')
def sensitivity_of_pco2_dic_to_temperature():
    """
    sensitivity of pco2 dic to temperature



    component


    """
    
    ret = initial_sensitivity_of_c_uptake_to_temperaturesensitivity_of_pco2_dic_to_temperature_mean(
    )
    #test_nans(ret, 268)
    return ret


@cache('run')
def sensitivity_of_pco2_dic_to_temperature_mean():
    """
    sensitivity of pco2 dic to temperature mean



    constant


    """
    
    ret = 0.003
    #test_nans(ret, 269)
    return ret


@cache('step')
def mean_depth_of_adjacent_layers():
    """
    mean depth of adjacent layers



    component


    """
    
    ret = xr.DataArray(data=[
        initial_mixed_depthlayer_depth2(),
        (layer_depth().loc['layer1'].values + layer_depth().loc['layer2'].values) / 2,
        (layer_depth().loc['layer2'].values + layer_depth().loc['layer3'].values) / 2,
        (layer_depth().loc['layer3'].values + layer_depth().loc['layer4'].values) / 2],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 270)
    return ret


@cache('step')
def diffusion_flux():
    """
    diffusion flux



    component


    """
    
    ret = xr.DataArray(data=[
        (c_in_mixed_layer_per_meter() - c_in_deep_ocean_per_meter().loc['layer1'].values) * eddy_diff_coeff() /
        mean_depth_of_adjacent_layers().loc['layer1'].values,
        (c_in_deep_ocean_per_meter().loc['layer1'].values - c_in_deep_ocean_per_meter().loc['layer2'].values) * eddy_diff_coeff() /
        mean_depth_of_adjacent_layers().loc['layer2'].values,
        (c_in_deep_ocean_per_meter().loc['layer2'].values - c_in_deep_ocean_per_meter().loc['layer3'].values) * eddy_diff_coeff() /
        mean_depth_of_adjacent_layers().loc['layer3'].values,
        (c_in_deep_ocean_per_meter().loc['layer3'].values - c_in_deep_ocean_per_meter().loc['layer4'].values) * eddy_diff_coeff() /
        mean_depth_of_adjacent_layers().loc['layer4'].values],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 271)
    return ret


@cache('run')
def init_c_in_biomass():
    """
    init c in biomass



    constant


    """
    
    ret = 988.158
    #test_nans(ret, 272)
    return ret


@cache('run')
def init_c_in_deep_ocean_per_meter():
    """
    init c in deep ocean per meter



    constant


    """
    
    ret = xr.DataArray(data=[10.4027,10.345,10.2768,10.265],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 273)
    return ret


@cache('run')
def init_c_in_humus():
    """
    init c in humus



    constant


    """
    
    ret = 1076.17
    #test_nans(ret, 274)
    return ret


@cache('step')
def flux_atm_to_biomass():
    """
    flux atm to biomass



    component


    """
    
    ret = init_npp() * (1 + biostim_coeff() * np.log(c_in_atmosphere() / preindustrial_c())
                         ) * effect_of_warming_on_c_flux_to_biomass()
    #test_nans(ret, 275)
    return ret


@cache('step')
def flux_atm_to_ocean():
    """
    flux atm to ocean



    component


    """
    
    ret = ((equil_c_in_mixed_layer() - c_in_mixed_layer()) / mixing_time())
    #test_nans(ret, 276)
    return ret


@cache('step')
def flux_biomass_to_atmosphere():
    """
    flux biomass to atmosphere



    component


    """
    
    ret = c_in_biomass() / biomass_res_time() * (1 - humification_fraction())
    #test_nans(ret, 277)
    return ret


@cache('step')
def flux_biomass_to_ch4():
    """
    flux biomass to ch4



    component


    """
    
    ret = c_in_biomass() * ch4_generation_rate_from_biomass(
    ) * effect_of_warming_on_ch4_release_from_biological_activity()
    #test_nans(ret, 278)
    return ret


@cache('step')
def flux_biomass_to_humus():
    """
    flux biomass to humus



    component


    """
    
    ret = c_in_biomass() / biomass_res_time() * humification_fraction()
    #test_nans(ret, 279)
    return ret


@cache('step')
def flux_biosphere_to_ch4():
    """
    flux biosphere to ch4



    component


    """
    
    ret = flux_biomass_to_ch4() + flux_humus_to_ch4()
    #test_nans(ret, 280)
    return ret


@cache('step')
def flux_humus_to_atmosphere():
    """
    flux humus to atmosphere



    component


    """
    
    ret = c_in_humus() / humus_res_time()
    #test_nans(ret, 281)
    return ret


@cache('step')
def co2_ppm_concentrations():
    """
    co2 ppm concentrations



    component


    """
    
    ret = c_in_atmosphere() / 2.13
    #test_nans(ret, 282)
    return ret


@cache('step')
def total_c_anthro_emissions():
    """
    total c anthro emissions



    component


    """
    
    ret = total_co2_emissions_gtco2() * c_per_co2()
    #test_nans(ret, 283)
    return ret


@cache('step')
def effect_of_temp_on_dic_pco2():
    """
    effect of temp on dic pco2



    component


    """
    
    ret = 1 - sensitivity_of_pco2_dic_to_temperature() * temperature_change()
    #test_nans(ret, 284)
    return ret


@cache('step')
def effect_of_warming_on_c_flux_to_biomass():
    """
    effect of warming on c flux to biomass



    component


    """
    
    ret = 1 + strength_of_temp_effect_on_c_flux_to_land() * temperature_change()
    #test_nans(ret, 285)
    return ret


@cache('step')
def effect_of_warming_on_ch4_release_from_biological_activity():
    """
    effect of warming on ch4 release from biological activity



    component


    """
    
    ret = 1 + sensitivity_of_methane_emissions_to_temperature() * (temperature_change()) / (
        reference_temperature_change_for_effect_of_warming_on_ch4_from_respiration())
    #test_nans(ret, 286)
    return ret


@cache('step')
def c_in_mixed_layer_per_meter():
    """
    c in mixed layer per meter



    component


    """
    
    ret = c_in_mixed_layer() / mixed_depth()
    #test_nans(ret, 287)
    return ret


@cache('run')
def biomass_res_time():
    """
    biomass res time



    constant


    """
    
    ret = 10.6
    #test_nans(ret, 288)
    return ret


@cache('step')
def biostim_coeff():
    """
    biostim coeff



    component


    """
    
    ret = initial_biostim_coeff_indexbiostim_coeff_mean()
    #test_nans(ret, 289)
    return ret


@cache('run')
def mtons_per_gtons():
    """
    mtons per gtons



    constant


    """
    
    ret = 1000.0
    #test_nans(ret, 290)
    return ret


@cache('run')
def biostim_coeff_mean():
    """
    biostim coeff mean



    constant


    """
    
    ret = 0.42
    #test_nans(ret, 291)
    return ret


@cache('run')
def buff_c_coeff():
    """
    buff c coeff



    constant


    """
    
    ret = 3.92
    #test_nans(ret, 292)
    return ret


@cache('step')
def natural_ch4_emissions():
    """
    natural ch4 emissions



    component


    """
    
    ret = flux_biosphere_to_ch4() * ch4_per_c() * mtons_per_gtons()
    #test_nans(ret, 293)
    return ret


@cache('run')
def humification_fraction():
    """
    humification fraction



    constant


    """
    
    ret = 0.428
    #test_nans(ret, 294)
    return ret


@cache('run')
def humus_res_time():
    """
    humus res time



    constant


    """
    
    ret = 27.8
    #test_nans(ret, 295)
    return ret


@cache('run')
def layer_depth():
    """
    layer depth



    constant


    """
    
    ret = xr.DataArray(data=[300.0,300.0,1300.0,1800.0],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 296)
    return ret


@cache('run')
def ref_buffer_factor():
    """
    ref buffer factor



    constant


    """
    
    ret = 9.7
    #test_nans(ret, 297)
    return ret


@cache('run')
def reference_temperature_change_for_effect_of_warming_on_ch4_from_respiration():
    """
    reference temperature change for effect of warming on ch4 from respiration



    constant


    """
    
    ret = 5
    #test_nans(ret, 298)
    return ret


@cache('step')
def c_in_humus():
    """
    c in humus



    component


    """
    
    ret = integ_c_in_humus()
    #test_nans(ret, 299)
    return ret


@cache('step')
def c_in_mixed_layer():
    """
    c in mixed layer



    component


    """
    
    ret = integ_c_in_mixed_layer()
    #test_nans(ret, 300)
    return ret


@cache('run')
def init_c_in_atmos():
    """
    init c in atmos



    component


    """
    
    ret = init_co2_in_atmos_ppm() * gtc_per_ppm()
    #test_nans(ret, 301)
    return ret


@cache('run')
def strength_of_temp_effect_on_land_c_flux_mean():
    """
    strength of temp effect on land c flux mean



    constant


    """
    
    ret = -0.01
    #test_nans(ret, 302)
    return ret


@cache('run')
def pre_industrial_value_ppm():
    """
    pre industrial value ppm



    constant


    """
    
    ret = 275
    #test_nans(ret, 303)
    return ret


@cache('run')
def biostim_coeff_index():
    """
    biostim coeff index



    constant


    """
    
    ret = 1
    #test_nans(ret, 304)
    return ret


@cache('run')
def init_c_in_mixed_ocean_per_meter():
    """
    init c in mixed ocean per meter



    constant


    """
    
    ret = 10.4652
    #test_nans(ret, 305)
    return ret


@cache('run')
def init_co2_in_atmos_ppm():
    """
    init co2 in atmos ppm



    constant


    """
    
    ret = 360.3
    #test_nans(ret, 306)
    return ret


@cache('run')
def init_npp():
    """
    init npp



    constant


    """
    
    ret = 85.1771
    #test_nans(ret, 307)
    return ret


@cache('step')
def buffer_factor():
    """
    buffer factor



    component


    """
    
    ret = functions.active_initial(
        ref_buffer_factor() * (c_in_mixed_layer() / preind_c_in_mixed_layer())**buff_c_coeff(),
        ref_buffer_factor())
    #test_nans(ret, 308)
    return ret


@cache('step')
def c_in_atmosphere():
    """
    c in atmosphere



    component


    """
    
    ret = integ_c_in_atmosphere()
    #test_nans(ret, 309)
    return ret


@cache('run')
def mixed_depth():
    """
    mixed depth



    constant


    """
    
    ret = 100
    #test_nans(ret, 310)
    return ret


@cache('run')
def mixing_time():
    """
    mixing time



    constant


    """
    
    ret = 1
    #test_nans(ret, 311)
    return ret


@cache('step')
def c_in_deep_ocean_per_meter():
    """
    c in deep ocean per meter



    component


    """
    
    ret = c_in_deep_ocean() / layer_depth()
    #test_nans(ret, 312)
    return ret


@cache('step')
def elf_concentrations_logistic():
    """
    elf concentrations logistic



    component


    """
    
    ret = 1 - 1 / (1 + np.exp((co2_ppm_concentrations() - a_logistic()) / b_logistic()))
    #test_nans(ret, 313)
    return ret


@cache('step')
def strength_of_temp_effect_on_c_flux_to_land():
    """
    strength of temp effect on c flux to land



    component


    """
    
    ret = initial_sensitivity_of_c_uptake_to_temperaturestrength_of_temp_effect_on_land_c_flux_mean(
    )
    #test_nans(ret, 314)
    return ret


@cache('run')
def ch4_generation_rate_from_humus():
    """
    ch4 generation rate from humus



    constant


    """
    
    ret = 8e-05
    #test_nans(ret, 315)
    return ret


@cache('step')
def preind_c_in_mixed_layer():
    """
    preind c in mixed layer



    component


    """
    
    ret = initial_preind_ocean_c_per_metermixed_depth()
    #test_nans(ret, 316)
    return ret


@cache('run')
def preind_ocean_c_per_meter():
    """
    preind ocean c per meter



    constant


    """
    
    ret = 10.2373
    #test_nans(ret, 317)
    return ret


@cache('run')
def preindustrial_c():
    """
    preindustrial c



    constant


    """
    
    ret = 590
    #test_nans(ret, 318)
    return ret


@cache('step')
def equilibrium_c_per_meter_in_mixed_layer():
    """
    equilibrium c per meter in mixed layer



    component


    """
    
    ret = equil_c_in_mixed_layer() / mixed_depth()
    #test_nans(ret, 319)
    return ret


@cache('run')
def sensitivity_of_c_uptake_to_temperature():
    """
    sensitivity of c uptake to temperature



    constant


    """
    
    ret = 1
    #test_nans(ret, 320)
    return ret


@cache('run')
def ch4_generation_rate_from_biomass():
    """
    ch4 generation rate from biomass



    constant


    """
    
    ret = 8e-05
    #test_nans(ret, 321)
    return ret


@cache('run')
def ch4_per_c():
    """
    ch4 per c



    constant


    """
    
    ret = 1.33
    #test_nans(ret, 322)
    return ret


@cache('step')
def flux_humus_to_ch4():
    """
    flux humus to ch4



    component


    """
    
    ret = c_in_humus() * ch4_generation_rate_from_humus(
    ) * effect_of_warming_on_ch4_release_from_biological_activity()
    #test_nans(ret, 323)
    return ret


@cache('run')
def sensitivity_of_methane_emissions_to_temperature():
    """
    sensitivity of methane emissions to temperature



    constant


    """
    
    ret = 0
    #test_nans(ret, 324)
    return ret


@cache('run')
def eddy_diff_coeff_index():
    """
    eddy diff coeff index



    constant


    """
    
    ret = 1
    #test_nans(ret, 325)
    return ret


@cache('step')
def c_in_biomass():
    """
    c in biomass



    component


    """
    
    ret = integ_c_in_biomass()
    #test_nans(ret, 326)
    return ret


@cache('run')
def gtc_per_ppm():
    """
    gtc per ppm



    constant


    """
    
    ret = 2.13
    #test_nans(ret, 327)
    return ret


@cache('step')
def c_in_deep_ocean():
    """
    c in deep ocean



    component


    """
    
    ret = integ_c_in_deep_ocean()
    #test_nans(ret, 328)
    return ret


@cache('step')
def equil_c_in_mixed_layer():
    """
    equil c in mixed layer



    component


    """
    
    ret = preind_c_in_mixed_layer() * effect_of_temp_on_dic_pco2() * (
        c_in_atmosphere() / preindustrial_c())**(1 / buffer_factor())
    #test_nans(ret, 329)
    return ret


@cache('step')
def layer_time_constant():
    """
    layer time constant



    component


    """
    
    ret = xr.DataArray(data=[
        initial_layer_deptheddy_diff_coeffmean_depth_of_adjacent_layers(),
        initial_layer_deptheddy_diff_coeffmean_depth_of_adjacent_layers(),
        initial_layer_deptheddy_diff_coeffmean_depth_of_adjacent_layers(),
        initial_layer_deptheddy_diff_coeffmean_depth_of_adjacent_layers()],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 330)
    return ret


@cache('run')
def eddy_diff_mean():
    """
    eddy diff mean



    constant


    """
    
    ret = 4400
    #test_nans(ret, 331)
    return ret


@cache('step')
def common_rr_minerals_variation_alt_techn():
    """
    common rr minerals variation alt techn



    component


    """
    
    ret = functions.if_then_else(time() < start_year_p_common_rr_minerals_alt_techn(),
                                  historic_improvement_recycling_rates_minerals(),
                                  p_common_rr_minerals_variation_alt_techn())
    #test_nans(ret, 332)
    return ret


@cache('run')
def choose_targets_mineral_recycling_rates():
    """
    choose targets mineral recycling rates



    constant


    """
    
    ret = 2
    #test_nans(ret, 333)
    return ret


@cache('step')
def total_water_use_per_capita():
    """
    total water use per capita



    component


    """
    
    ret = total_water_use() / population()
    #test_nans(ret, 334)
    return ret


@cache('step')
def total_water_use():
    """
    total water use



    component


    """
    
    ret = functions.sum(total_water_use_by_type())
    #test_nans(ret, 335)
    return ret


@cache('step')
def water_use_per_type_per_capita():
    """
    water use per type per capita



    component


    """
    
    ret = total_water_use_by_type() / population()
    #test_nans(ret, 336)
    return ret


@cache('step')
def variation_water_intensity_by_sector():
    """
    variation water intensity by sector



    component


    """
    
    ret = functions.if_then_else(time() < 2008,
                                  historic_water_by_type_intensities_by_sector() -
                                  historic_water_intensities_by_sector_delayed_1yr(), 0)
    ret = utils.align_by_dims(ret, ('sectors', 'water color'))
    #test_nans(ret, 337)
    return ret


@cache('step')
def historic_water_intensities_by_sector_delayed_1yr():
    """
    historic water intensities by sector delayed 1yr



    component


    """
    a = [x() for x in delays_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step]

    ret = xr.DataArray(data=np.reshape(a, historic_water_by_type_intensities_by_sector().T.shape).T,
        dims=historic_water_by_type_intensities_by_sector().dims,
        coords=historic_water_by_type_intensities_by_sector().coords)
    #ret = utils.align_by_dims(ret, ('sectors', 'water color'))
    #test_nans(ret, 338)
    return ret


@cache('step')
def historic_water_intensities_for_households_delayed_1yr():
    """
    historic water intensities for households delayed 1yr



    component


    """
    
    ret = xr.DataArray(data=[x() for x in delays_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step],
        dims=historic_water_by_type_intensities_for_households().dims,
        coords=historic_water_by_type_intensities_for_households().coords)
    #test_nans(ret, 339)
    return ret


@cache('step')
def variation_water_intensity_households():
    """
    variation water intensity households



    component


    """
    
    ret = functions.if_then_else(time() < 2008,
                                  historic_water_by_type_intensities_for_households() -
                                  historic_water_intensities_for_households_delayed_1yr(), 0)
    #test_nans(ret, 340)
    return ret


@cache('step')
def water_intensity_for_households():
    """
    water intensity for households



    component


    """
    
    ret = integ_water_intensity_for_households()
    #test_nans(ret, 341)
    return ret


@cache('step')
def total_water_use_by_type():
    """
    total water use by type



    component


    """
    
    ret = functions.sum(dim='sectors', data=water_use_by_sector()) + water_use_by_households(
    ) + total_water_for_oxandxm_required_by_res_elec_dam3()
    #test_nans(ret, 342)
    return ret


@cache('step')
def total_water_for_oxandxm_required_by_res_elec_dam3():
    """
    total water for oxandxm required by res elec dam3



    component


    """
    
    ret = total_water_for_oxandxm_required_by_res_elec() * mt_to_dam3()
    #test_nans(ret, 343)
    return ret


@cache('step')
def initial_water_intensity_for_households():
    """
    initial water intensity for households



    component


    """
    
    ret = functions.if_then_else(time() < 1996,
                                  initial_water_use_by_households() / household_demand_total(), 0)
    #test_nans(ret, 344)
    return ret


@cache('step')
def water_intensity_by_sector():
    """
    water intensity by sector



    component


    """
    
    ret = integ_water_intensity_by_sector()
    ret = utils.align_by_dims(ret, ('sectors', 'water color'))
    #test_nans(ret, 345)
    return ret


@cache('step')
def water_use_by_sector():
    """
    water use by sector



    component


    """
    
    ret = water_intensity_by_sector() * real_total_output_by_sector()
    ret = utils.align_by_dims(ret, ('sectors', 'water color'))
    #test_nans(ret, 346)
    return ret


@cache('step')
def water_use_by_households():
    """
    water use by households



    component


    """
    
    ret = water_intensity_for_households() * household_demand_total()
    #test_nans(ret, 347)
    return ret


@cache('step')
def modern_bioe_in_households():
    """
    modern bioe in households



    component


    """
    
    ret = households_final_energy_demand().loc['solids'].values - pe_traditional_biomass_consum_ej()
    #test_nans(ret, 348)
    return ret


@cache('step')
def tfec_res_ej():
    """
    tfec res ej



    component


    """
    
    ret = fe_tot_generation_all_res_elec_ej() + fes_res_for_heat_ej(
    ) + pe_traditional_biomass_consum_ej() + fes_total_biofuels_production_ej(
    ) + pes_biogas_for_tfc()
    #test_nans(ret, 349)
    return ret


@cache('step')
def pe_traditional_biomass_ej_delayed_1yr():
    """
    pe traditional biomass ej delayed 1yr



    component


    """
    
    ret = delay_pe_traditional_biomass_consum_ej_round1__time_step___time_step_30_1__time_step()
    #test_nans(ret, 350)
    return ret


@cache('step')
def pe_traditional_biomass_demand_ej():
    """
    pe traditional biomass demand ej



    component


    """
    
    ret = households_final_energy_demand().loc['solids'].values * share_trad_biomass_vs_solids_in_households()
    #test_nans(ret, 351)
    return ret


@cache('step')
def population_dependent_on_trad_biomass():
    """
    population dependent on trad biomass



    component


    """
    
    ret = pe_traditional_biomass_consum_ej() / pepc_consumption_people_depending_on_trad_biomass()
    #test_nans(ret, 352)
    return ret


@cache('run')
def mt_to_dam3():
    """
    mt to dam3



    constant


    """
    
    ret = 1000.0
    #test_nans(ret, 353)
    return ret


@cache('step')
def total_water_for_oxandxm_required_by_res_elec():
    """
    total water for oxandxm required by res elec



    component


    """
    
    ret = xr.DataArray(data=[
        functions.sum(total_water_for_oxandxm_required_by_res_elec_per_techn()),0.0,0.0],
        coords={'water color': ['blue water', 'green water', 'gray water']},
        dims=['water color'])
    #test_nans(ret, 354)
    return ret


@cache('step')
def initial_water_intensity_by_sector():
    """
    initial water intensity by sector



    component


    """
    
    ret = functions.if_then_else(time() < 1996,
                                  initial_water_use_by_sector() / real_total_output_by_sector(), 0)
    ret = utils.align_by_dims(ret, ('sectors', 'water color'))
    #test_nans(ret, 355)
    return ret


@cache('run')
def initial_water_use_by_households():
    """
    initial water use by households



    constant


    """
    
    ret = xr.DataArray(data=[39538356.48517995,0.0,264130630.0],
        coords={'water color': ['blue water', 'green water', 'gray water']},
        dims=['water color'])
    #test_nans(ret, 356)
    return ret


@cache('run')
def initial_water_use_by_sector():
    """
    initial water use by sector



    constant


    """
    
    ret = xr.DataArray(data=[[875813247.0,5977864530.0,651138685.0],[0.0,0.0,0.0],[6043346.01034091,0.0,51655857.26464701],[2856810.35086267,0.0,25563592.7501675],[0.0,0.0,0.0],[0.0,0.0,0.0],[5511624.24222653,0.0,48897115.23912518],[0.0,0.0,0.0],[7734732.5382935,0.0,71068432.36246447],[0.0,0.0,0.0],[1654566.63492239,0.0,17130547.55065427],[7855774.10285661,0.0,78961833.92970943],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[607162225.0,0.0,710.30657137],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[0.0,0.0,0.0],[76.61435359,0.0,496.96858614],[76.30793914,0.0,498.00510337],[0.0,0.0,0.0],[0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'water color': ['blue water', 'green water', 'gray water']},
        dims=['sectors', 'water color'])
    ret = utils.align_by_dims(ret, ('sectors', 'water color'))
    #test_nans(ret, 357)
    return ret


@cache('run')
def max_potential_npp_bioe_conventional_for_heatxplusxelec():
    """
    max potential npp bioe conventional for heatxplusxelec



    constant


    """
    
    ret = 30
    #test_nans(ret, 358)
    return ret


@cache('step')
def total_transport_fed_by_fuel():
    """
    total transport fed by fuel



    component


    """
    
    ret = required_final_energy_air_transport() + required_final_energy_inland_transport(
    ) + required_final_energy_other_transport() + required_final_energy_water_transport(
    ) + transport_households_final_energy_demand()
    #test_nans(ret, 359)
    return ret


@cache('step')
def diff_demand():
    """
    diff demand



    component


    """
    
    ret = functions.if_then_else(time() < 2009, 1,
                                  (gdp_delayed_1yr() *
                                   (1 + desired_annual_gdp_growth_rate())) / total_demand())
    #test_nans(ret, 360)
    return ret


@cache('step')
def variation_lc():
    """
    variation lc



    component


    """
    
    ret = real_demand() * labour_share() * (
        desired_annual_gdp_growth_rate() + growth_labour_share() +
        desired_annual_gdp_growth_rate() * growth_labour_share())
    #test_nans(ret, 361)
    return ret


@cache('step')
def variation_cc():
    """
    variation cc



    component


    """
    
    ret = capital_share() * (
        desired_annual_gdp_growth_rate() + growth_capital_share() +
        desired_annual_gdp_growth_rate() * growth_capital_share()) * real_demand()
    #test_nans(ret, 362)
    return ret


@cache('step')
def demand_by_sector_fd_adjusted():
    """
    demand by sector fd adjusted



    component


    """
    
    ret = demand_by_sector_fd() * diff_demand()
    #test_nans(ret, 363)
    return ret


@cache('step')
def total_demand_adjusted():
    """
    total demand adjusted



    component


    """
    
    ret = functions.sum(demand_by_sector_fd_adjusted()) / 1e+006
    #test_nans(ret, 364)
    return ret


@cache('step')
def desired_gdp():
    """
    desired gdp



    component


    """
    
    ret = functions.if_then_else(time() < 2009,
                                  total_demand(),
                                  desired_gdppc() * population() / dollars_to_tdollars() - 0.56)
    #test_nans(ret, 365)
    return ret


@cache('step')
def demand_by_sector():
    """
    demand by sector



    component


    """
    
    ret = demand_by_sector_fd_adjusted()
    #test_nans(ret, 366)
    return ret


@cache('step')
def aux4():
    """
    aux4



    component


    """
    
    ret = delay_eroist_system_until_x2015x_roundtime_step__time_step___time_step_eroistx1995x_time_step__time_step(
    )
    #test_nans(ret, 367)
    return ret


@cache('run')
def x1x_year():
    """
    x1x year



    constant


    """
    
    ret = 1
    #test_nans(ret, 368)
    return ret


@cache('step')
def eptb_dynamic():
    """
    eptb dynamic



    component


    """
    
    ret = functions.zidz(lifetime_res_elec(), xdynamicx_eroi_res_elec_var())
    #test_nans(ret, 369)
    return ret


@cache('step')
def eroist_system_delayed():
    """
    eroist system delayed



    component


    """
    
    ret = delay_eroist_system_roundtime_step__time_step___time_step_eroistx1995x_time_step__time_step(
    )
    #test_nans(ret, 370)
    return ret


@cache('step')
def eroist_system_delayed_1yr():
    """
    eroist system delayed 1yr



    component


    """
    
    ret = delay_eroist_system_round1__time_step___time_step_eroistx1995x_1__time_step()
    #test_nans(ret, 371)
    return ret


@cache('run')
def pipeline_transport_constant_x2x6x_ej_in_x2014():
    """
    pipeline transport constant x2x6x ej in x2014



    constant


    """
    
    ret = 2.6
    #test_nans(ret, 372)
    return ret


@cache('step')
def total_distribution_losses():
    """
    total distribution losses



    component


    """
    
    ret = electrical_distribution_losses_ej() + heatxdashxcom_distribution_losses(
    ) + heatxdashxnc_distribution_losses() + pipeline_transport_constant_x2x6x_ej_in_x2014(
    ) + functions.sum(energy_distr_losses_ff_ej())
    #test_nans(ret, 373)
    return ret


@cache('run')
def ia_matrix_x2002x():
    """
    ia matrix x2002x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.68684171e-01, -9.33669000e-04, -2.57643610e-01, -3.94381980e-02, -3.66334540e-02,
            -1.29627905e-01, -1.21232260e-02, -1.56225600e-03, -4.95520600e-03, -1.40601000e-02,
            -6.23449000e-04, -2.71843000e-04, -2.76016000e-04, -1.88636000e-04, -1.49733000e-04,
            -9.52747100e-03, -5.10374000e-04, -1.02566090e-02, -4.55947000e-04, -3.13305400e-03,
            -2.64502400e-03, -3.86784270e-02, -1.33808600e-03, -1.27999600e-03, -3.07578000e-04,
            -3.93945100e-03, -2.52928000e-04, -1.41304000e-04, -3.71861000e-04, -6.27869000e-04,
            -1.90611800e-03, -2.01888000e-03, -2.15198400e-03, -2.33462200e-03, -1.53724580e-02
        ], [
            -3.52914000e-03, 9.26812613e-01, -2.61520200e-03, -2.01922000e-03, -8.51768000e-04,
            -3.65366900e-03, -4.38863700e-03, -4.42566481e-01, -3.18202300e-02, -2.93755700e-03,
            -6.42380160e-02, -4.39890980e-02, -2.94532000e-03, -1.27805600e-03, -1.23764600e-03,
            -6.00550100e-03, -1.06270138e-01, -1.75276250e-02, -4.49917000e-04, -6.74029000e-04,
            -1.07797000e-03, -1.57100300e-03, -3.84548100e-03, -2.40335400e-03, -2.42027900e-03,
            -1.56751100e-03, -4.95052000e-04, -2.69923000e-04, -1.65231700e-03, -8.23667000e-04,
            -2.74932900e-03, -2.99711500e-03, -8.69258000e-04, -2.45149800e-03, -6.17356550e-02
        ], [
            -5.41804650e-02, -6.43059000e-04, 8.52960376e-01, -2.67716400e-03, -8.23854610e-02,
            -1.29955300e-03, -2.74504000e-03, -7.47645000e-04, -9.35253900e-03, -2.54585000e-03,
            -1.45185700e-03, -9.18340000e-04, -1.06471600e-03, -9.07012000e-04, -8.08241000e-04,
            -2.02632800e-03, -7.06891000e-04, -8.96595000e-04, -1.58689300e-03, -4.66331200e-03,
            -4.47896200e-03, -1.46896488e-01, -1.22975200e-03, -6.26254800e-03, -3.06450600e-03,
            -1.61752400e-03, -1.46103600e-03, -6.89845000e-04, -2.32331000e-04, -1.57290900e-03,
            -6.31706000e-03, -7.56527900e-03, -1.28740170e-02, -6.38742200e-03, -3.28818100e-03
        ], [
            -2.00684300e-03, -1.07570700e-03, -1.53823000e-03, 6.98491953e-01, -4.86333770e-02,
            -3.53967400e-03, -5.30404300e-03, -7.26834000e-04, -3.19763100e-03, -1.11495600e-02,
            -4.15175800e-03, -1.60237600e-03, -2.76374000e-03, -1.94525600e-03, -5.59024800e-03,
            -2.08541770e-02, -5.14172000e-04, -2.18717600e-03, -1.39390800e-03, -2.43663700e-03,
            -3.10760800e-03, -2.55751600e-03, -1.55273200e-03, -1.84772400e-03, -1.13333900e-03,
            -1.49635500e-03, -8.26729000e-04, -4.91484000e-04, -2.52858000e-04, -1.06109200e-03,
            -3.32436900e-03, -9.56161000e-04, -2.77338300e-03, -6.15834200e-03, -1.42113000e-04
        ], [
            -1.33399000e-04, -9.47579000e-05, -1.46871000e-04, -5.91240000e-03, 7.87478676e-01,
            -4.24063000e-04, -5.71795000e-04, -4.85136000e-05, -2.18821000e-04, -1.41752200e-03,
            -2.86707000e-04, -1.92086000e-04, -2.45768000e-04, -2.36819000e-04, -1.23947500e-03,
            -3.99612500e-03, -1.23783000e-04, -1.26771000e-04, -4.00562000e-04, -4.17897000e-04,
            -3.22356000e-04, -1.27830000e-04, -1.86509000e-04, -1.42570000e-04, -1.20078000e-04,
            -1.73915000e-04, -2.54114000e-04, -3.41829000e-05, -1.26275000e-05, -1.09905000e-04,
            -4.91493000e-04, -1.07976000e-04, -1.07733000e-04, -3.87677000e-04, -4.19055000e-06
        ], [
            -2.13308600e-03, -1.49575500e-03, -1.64664900e-03, -1.30495800e-03, -1.72427700e-03,
            7.87339416e-01, -1.01615140e-02, -2.72584000e-04, -1.47928200e-03, -3.36993100e-03,
            -6.13136100e-03, -2.93257000e-03, -2.83651000e-03, -1.54527800e-03, -2.84319000e-03,
            -6.79460240e-02, -8.33441000e-04, -3.24406520e-02, -6.83174000e-04, -1.50209000e-03,
            -1.30039500e-03, -2.35710600e-03, -1.18023700e-03, -4.88472000e-04, -3.38699000e-04,
            -2.99664100e-03, -7.06730000e-04, -3.74270000e-04, -1.99487700e-03, -8.82198000e-04,
            -1.48979300e-03, -2.02427800e-03, -9.62912000e-04, -2.69524600e-03, -1.01196000e-04
        ], [
            -2.58947200e-03, -2.64731600e-03, -2.48109460e-02, -1.05795620e-02, -1.33046330e-02,
            -1.24135780e-02, 7.71667173e-01, -1.57506300e-03, -1.67181170e-02, -1.67998530e-02,
            -1.96015110e-02, -5.40854200e-03, -8.68868200e-03, -1.12740080e-02, -4.79594600e-03,
            -2.00922130e-02, -3.48756900e-03, -4.27354300e-03, -7.70017100e-03, -1.35081980e-02,
            -1.25454550e-02, -8.72608800e-03, -5.94804900e-03, -2.64805000e-03, -5.33639400e-03,
            -1.34575860e-02, -1.15211060e-02, -1.32532540e-02, -2.21609700e-03, -2.13379050e-02,
            -1.80574390e-02, -1.79708070e-02, -9.09541200e-03, -2.52621040e-02, -2.07308000e-04
        ], [
            -1.31888050e-02, -1.03376180e-02, -3.38034900e-03, -5.31808600e-03, -3.69576500e-03,
            -5.13561800e-03, -4.46991500e-03, 9.51354947e-01, -3.18189040e-02, -8.33868900e-03,
            -1.79956370e-02, -9.82711400e-03, -3.79829100e-03, -2.27818700e-03, -1.94240700e-03,
            -3.97617000e-03, -1.95133340e-02, -1.31155230e-02, -4.12743800e-03, -5.72151300e-03,
            -4.13923600e-03, -3.60959700e-03, -5.17070050e-02, -7.98596240e-02, -6.24325250e-02,
            -1.40976380e-02, -3.21290700e-03, -1.17154200e-03, -1.02821300e-03, -4.43403800e-03,
            -8.78331600e-03, -2.70972600e-03, -2.99007200e-03, -5.02824200e-03, -4.72208000e-05
        ], [
            -3.80387870e-02, -1.04943260e-02, -1.05423730e-02, -6.92010630e-02, -3.75121990e-02,
            -3.24417150e-02, -3.51548030e-02, -1.50841210e-02, 7.69136332e-01, -2.01669822e-01,
            -3.30134690e-02, -1.56961040e-02, -1.18752810e-02, -1.72896550e-02, -1.20146370e-02,
            -2.57830890e-02, -6.30401400e-03, -1.05390870e-02, -6.32097300e-03, -3.15034600e-03,
            -2.32794400e-03, -4.52537900e-03, -2.96728800e-03, -3.55207700e-03, -2.45691100e-03,
            -3.42771100e-03, -1.41466700e-03, -8.53278000e-04, -1.29568600e-03, -4.74344200e-03,
            -9.94468400e-03, -3.79983100e-03, -6.45609530e-02, -1.00089930e-02, -1.55806000e-04
        ], [
            -5.15539200e-03, -6.69762300e-03, -1.72067390e-02, -1.00848830e-02, -3.48189060e-02,
            -9.34337900e-03, -1.56141520e-02, -2.88649200e-03, -1.84117570e-02, 8.72355980e-01,
            -1.07227360e-02, -7.22221000e-03, -2.82825500e-02, -2.54121260e-02, -3.53983380e-02,
            -3.10845680e-02, -2.49315600e-03, -1.81209070e-02, -1.65263160e-02, -4.77971900e-03,
            -5.15900300e-03, -4.19222200e-03, -9.94734100e-03, -3.04540500e-03, -2.97722200e-03,
            -5.93219200e-03, -3.37561000e-03, -6.02723000e-04, -1.00611900e-03, -2.76454200e-03,
            -3.28199500e-03, -9.49348000e-04, -5.07268000e-03, -3.76252200e-03, -3.29240000e-05
        ], [
            -2.39653600e-03, -4.36869100e-03, -6.03080500e-03, -2.43471000e-03, -1.79705000e-03,
            -8.05323300e-03, -1.27663400e-03, -2.18359100e-03, -5.98963500e-03, -6.68710300e-03,
            8.84982649e-01, -1.07765890e-02, -5.54010900e-03, -1.19471620e-02, -7.08569900e-03,
            -6.54272800e-03, -1.93877600e-03, -7.66061340e-02, -2.91835600e-03, -1.29270500e-03,
            -1.07574400e-03, -3.52905700e-03, -1.27691700e-03, -6.37478000e-04, -5.08203000e-04,
            -1.41430300e-03, -1.25792300e-03, -1.84129000e-04, -1.88767000e-03, -1.08005800e-03,
            -1.34079100e-03, -1.69940000e-03, -2.03615000e-03, -2.00125900e-03, -2.79889000e-05
        ], [
            -4.67420000e-03, -2.17643860e-02, -1.39626970e-02, -5.84558500e-03, -1.00566680e-02,
            -2.50679020e-02, -9.13616100e-03, -6.78449800e-03, -1.44905760e-02, -2.52338140e-02,
            -3.18555070e-02, 6.84288956e-01, -1.63871829e-01, -6.28872240e-02, -9.45659470e-02,
            -7.96385090e-02, -7.35455400e-03, -8.82024280e-02, -1.74216140e-02, -4.14447000e-03,
            -3.69242800e-03, -4.54866900e-03, -7.02709600e-03, -7.30681900e-03, -4.11025800e-03,
            -6.48778900e-03, -4.05684900e-03, -6.66126000e-04, -2.24325200e-03, -3.68047200e-03,
            -4.66225600e-03, -1.76517100e-03, -2.20114100e-03, -4.45241400e-03, -2.00665000e-04
        ], [
            -6.91762800e-03, -1.46907900e-02, -3.87125900e-03, -8.24742800e-03, -5.13591800e-03,
            -8.08065300e-03, -6.32550800e-03, -5.47105200e-03, -6.62573700e-03, -1.08116180e-02,
            -1.67048780e-02, -1.60715540e-02, 8.75736679e-01, -1.44414670e-02, -3.03362650e-02,
            -9.41384700e-03, -9.71477700e-03, -1.72154570e-02, -2.06114960e-02, -3.88603900e-03,
            -2.51223700e-03, -2.01678100e-03, -5.64571900e-03, -9.28211200e-03, -4.57942000e-03,
            -4.49821600e-03, -2.86731700e-03, -1.01141100e-03, -1.39054500e-03, -3.02711400e-03,
            -5.86823100e-03, -2.13707600e-03, -2.23717500e-03, -3.35756200e-03, -7.71250000e-05
        ], [
            -1.87717800e-03, -6.29625800e-03, -3.31354200e-03, -5.92671100e-03, -4.82406000e-03,
            -6.73438900e-03, -9.78098700e-03, -5.12829800e-03, -9.52733600e-03, -1.37832660e-02,
            -1.08409970e-02, -1.61998550e-02, -8.77665980e-02, 6.97609799e-01, -5.38059940e-02,
            -2.08465920e-02, -1.64223710e-02, -3.03057740e-02, -2.65653450e-02, -9.05250000e-03,
            -7.42347500e-03, -4.11744700e-03, -6.59685200e-03, -4.45616100e-03, -4.47243400e-03,
            -7.69636600e-03, -3.85184240e-02, -3.86178000e-03, -1.90029800e-03, -1.50093420e-02,
            -1.36512720e-02, -6.24981300e-03, -2.11732820e-02, -9.78084200e-03, -5.84822000e-04
        ], [
            -4.50828600e-03, -5.98442400e-03, -2.25596300e-03, -2.81451000e-03, -2.51982800e-03,
            -3.54235600e-03, -2.42439100e-03, -1.29875500e-03, -2.24070300e-03, -4.42194600e-03,
            -3.69189300e-03, -5.36205000e-03, -1.41371010e-02, -5.03342100e-03, 7.00211819e-01,
            -4.76648900e-03, -2.03079400e-03, -4.49449700e-03, -8.91118820e-02, -6.04989700e-03,
            -4.35266100e-03, -1.37874000e-03, -3.55158170e-02, -4.16849780e-02, -5.24586570e-02,
            -1.16713990e-02, -4.00554700e-03, -1.19791600e-03, -9.91480000e-04, -4.30604000e-03,
            -1.62303070e-02, -1.60560000e-03, -1.33165300e-03, -5.40542300e-03, -1.87691000e-04
        ], [
            -7.04125000e-04, -1.02498900e-03, -1.09515900e-03, -4.84813400e-03, -2.87872900e-03,
            -4.72945700e-03, -3.71975700e-03, -5.59699000e-04, -1.70148300e-03, -2.56233300e-03,
            -2.84147200e-03, -9.19394800e-03, -3.86811500e-03, -2.76839200e-03, -4.98124800e-03,
            9.58832067e-01, -1.28001500e-03, -5.06950100e-03, -2.25094100e-03, -1.31292900e-03,
            -1.50863400e-03, -2.48774400e-03, -1.87055800e-03, -7.49417000e-04, -1.40904900e-03,
            -1.66130000e-03, -1.02363500e-03, -9.20886000e-04, -1.64223700e-03, -1.89540200e-03,
            -3.91170300e-03, -3.09807500e-03, -2.82035000e-03, -5.51060800e-03, -2.03763400e-03
        ], [
            -1.08321120e-02, -2.70292950e-02, -1.30782520e-02, -1.80762810e-02, -8.34192500e-03,
            -1.71037300e-02, -2.30474590e-02, -1.40042610e-02, -2.64047440e-02, -2.16239490e-02,
            -3.82753680e-02, -2.82614520e-02, -1.16122540e-02, -8.31688000e-03, -8.67889700e-03,
            -9.08069100e-03, 9.03801502e-01, -5.81099500e-03, -1.01777620e-02, -6.38161700e-03,
            -1.36269300e-02, -2.33830140e-02, -1.48693820e-02, -3.42003600e-03, -4.38625400e-03,
            -1.36120500e-02, -9.08157200e-03, -4.15866100e-03, -9.80194600e-03, -5.80349600e-03,
            -1.54193720e-02, -1.99294680e-02, -1.26962520e-02, -1.56321670e-02, -1.94963000e-04
        ], [
            -3.75561400e-03, -8.67010400e-03, -2.47763600e-03, -2.84738200e-03, -1.72106300e-03,
            -3.35726200e-03, -3.55450300e-03, -2.73272900e-03, -3.48839100e-03, -3.03705200e-03,
            -7.22275400e-03, -4.31059300e-03, -3.36590200e-03, -2.41033400e-03, -2.06496600e-03,
            -2.47373900e-03, -2.26265140e-02, 9.52351146e-01, -4.29858000e-03, -3.99404100e-03,
            -4.94498600e-03, -6.06517400e-03, -9.94798600e-03, -2.77825400e-03, -2.78239900e-03,
            -1.26733610e-02, -1.11576270e-02, -5.06383200e-03, -4.10321530e-02, -4.94718300e-03,
            -1.81078350e-02, -1.53458340e-02, -5.98612200e-03, -1.03147830e-02, -7.09895000e-04
        ], [
            -5.03835600e-03, -4.03665800e-03, -6.80559100e-03, -5.79496200e-03, -7.67988100e-03,
            -5.91391200e-03, -5.00113400e-03, -2.60957600e-03, -5.99328600e-03, -6.36060800e-03,
            -7.42143900e-03, -6.05872200e-03, -5.26768100e-03, -4.09416200e-03, -4.84823300e-03,
            -6.16458900e-03, -8.24844700e-03, -7.08269300e-03, 9.89504485e-01, -4.17164300e-03,
            -4.20918300e-03, -4.97288300e-03, -1.67374130e-02, -2.13140100e-03, -3.74898400e-03,
            -7.96413700e-03, -3.82425100e-03, -2.53316400e-03, -1.85997600e-03, -5.79809200e-03,
            -4.41169600e-03, -2.90428500e-03, -3.65486700e-03, -6.24327200e-03, -2.77020000e-05
        ], [
            -3.36581310e-02, -2.48604850e-02, -6.43193250e-02, -5.25079640e-02, -6.68130140e-02,
            -5.53354020e-02, -5.09153850e-02, -4.22571270e-02, -4.94220470e-02, -4.99629940e-02,
            -4.37239600e-02, -4.64426740e-02, -4.83645420e-02, -4.29194120e-02, -4.44818620e-02,
            -4.67215750e-02, -2.31783180e-02, -4.10977340e-02, -3.52501230e-02, 9.74329958e-01,
            -1.29287650e-02, -4.46645640e-02, -2.20341110e-02, -1.93623540e-02, -2.00785160e-02,
            -1.97883580e-02, -1.05040790e-02, -4.21021300e-03, -3.56180600e-03, -1.29719920e-02,
            -1.82813680e-02, -1.04105660e-02, -2.64021140e-02, -1.62351670e-02, -1.90568000e-04
        ], [
            -1.70781170e-02, -9.52873500e-03, -3.09976190e-02, -2.78676380e-02, -4.06711490e-02,
            -2.09868930e-02, -1.89151500e-02, -1.09720350e-02, -2.18399690e-02, -2.05350670e-02,
            -1.94683960e-02, -1.67214490e-02, -1.75515060e-02, -1.41050770e-02, -1.90239270e-02,
            -2.46050820e-02, -8.44660000e-03, -2.94881140e-02, -1.59464280e-02, -6.85054000e-03,
            9.92287939e-01, -2.54442940e-02, -1.35331090e-02, -7.82644700e-03, -9.28510600e-03,
            -8.76515500e-03, -8.02241800e-03, -4.26602900e-03, -2.96737500e-03, -9.12057400e-03,
            -8.65724200e-03, -5.73249000e-03, -1.15738780e-02, -1.07318210e-02, -3.26523000e-04
        ], [
            -1.31714600e-03, -3.07132500e-03, -3.78440900e-03, -4.37212200e-03, -4.14808600e-03,
            -4.82770000e-03, -6.34109200e-03, -1.32256800e-03, -4.39779200e-03, -5.76705000e-03,
            -5.51955200e-03, -4.58670800e-03, -6.16365300e-03, -5.00524200e-03, -2.75119700e-03,
            -3.78620600e-03, -5.56554600e-03, -4.43126400e-03, -5.74858600e-03, -8.43644500e-03,
            -5.76776900e-03, 9.89590774e-01, -7.55370700e-03, -4.01990600e-03, -2.11377340e-02,
            -1.67889230e-02, -9.43789500e-03, -9.47501900e-03, -2.50547500e-03, -1.41470870e-02,
            -1.07588740e-02, -6.14565700e-03, -8.49628200e-03, -1.17991520e-02, -6.66582000e-05
        ], [
            -1.27171700e-02, -2.12356300e-02, -2.32016990e-02, -1.96546460e-02, -2.06674140e-02,
            -2.32791710e-02, -1.92150180e-02, -3.36845080e-02, -1.79871410e-02, -1.75936490e-02,
            -3.36912940e-02, -1.80046230e-02, -1.27669150e-02, -8.64276200e-03, -1.14524310e-02,
            -1.70827200e-02, -2.20073120e-02, -1.55737290e-02, -1.41565920e-02, -1.85786580e-02,
            -1.17090290e-02, -9.31574700e-03, 9.54727388e-01, -4.36975760e-02, -1.38620120e-02,
            -5.68737370e-02, -8.65676000e-03, -5.25938000e-03, -1.76671500e-03, -5.76086000e-03,
            -1.07979880e-02, -1.00365280e-02, -6.88791500e-03, -9.18936600e-03, -2.97054000e-04
        ], [
            -3.78423500e-03, -5.94394200e-03, -4.81580200e-03, -4.71346700e-03, -5.91733900e-03,
            -8.27253500e-03, -4.11368600e-03, -8.05974600e-03, -3.80786100e-03, -4.08709000e-03,
            -1.08747110e-02, -7.03389800e-03, -3.70505400e-03, -2.58253400e-03, -2.23616600e-03,
            -3.11404000e-03, -3.55268000e-03, -4.73824800e-03, -1.13362000e-03, -3.16662800e-03,
            -1.64455300e-03, -1.74389100e-03, -3.19557200e-03, 8.95683618e-01, -2.70829900e-03,
            -5.66155500e-03, -8.21042000e-04, -5.11302000e-04, -5.94771000e-04, -1.05671700e-03,
            -1.44572600e-03, -1.10658300e-03, -5.32451000e-04, -1.30467400e-03, -2.10223000e-05
        ], [
            -6.63915000e-04, -1.98498400e-03, -1.35336000e-03, -1.80996300e-03, -2.64622600e-03,
            -1.80261100e-03, -2.29401100e-03, -6.16803000e-04, -1.69019200e-03, -1.93033100e-03,
            -1.67832200e-03, -1.35300800e-03, -2.19909000e-03, -1.94664800e-03, -1.44786800e-03,
            -1.29390500e-03, -8.61576000e-04, -1.00912800e-03, -1.56004300e-03, -3.07825500e-03,
            -1.39626300e-03, -1.08880700e-03, -2.30582900e-03, -6.68273500e-03, 9.66231120e-01,
            -1.74398280e-02, -3.39137300e-03, -2.72733100e-03, -4.75161000e-04, -3.90291600e-03,
            -3.75977600e-03, -1.91559900e-03, -1.14729400e-03, -3.92819800e-03, -1.25732000e-05
        ], [
            -3.54887800e-03, -5.38514900e-03, -6.34120400e-03, -3.89294000e-03, -4.73597600e-03,
            -6.79209400e-03, -9.35665200e-03, -7.25441100e-03, -5.16352700e-03, -5.35286400e-03,
            -8.32183100e-03, -5.27485000e-03, -4.74164700e-03, -3.39722100e-03, -4.41522300e-03,
            -4.43716700e-03, -4.18651000e-03, -4.06452000e-03, -1.03923780e-02, -2.04018530e-02,
            -1.05888110e-02, -7.61447100e-03, -4.02878300e-02, -1.12198366e-01, -9.95615190e-02,
            8.94596273e-01, -4.50571400e-03, -3.21723700e-03, -9.21526000e-04, -4.89377500e-03,
            -4.82298600e-03, -2.37007700e-03, -3.02740600e-03, -5.82907900e-03, -2.53379000e-05
        ], [
            -2.67326000e-03, -6.04683100e-03, -4.88149700e-03, -6.06192600e-03, -7.12224100e-03,
            -6.19128700e-03, -1.35511760e-02, -3.00459600e-03, -8.57162300e-03, -6.87327900e-03,
            -7.47177100e-03, -6.50515600e-03, -8.56673400e-03, -7.85575300e-03, -4.58603000e-03,
            -7.51297600e-03, -6.60929800e-03, -1.25398450e-02, -1.54375790e-02, -2.12307250e-02,
            -2.19903420e-02, -1.39212730e-02, -1.64186960e-02, -1.71028000e-02, -2.47445160e-02,
            -2.52875590e-02, 8.73460895e-01, -2.82661060e-02, -5.00375600e-03, -2.62892800e-02,
            -2.23435070e-02, -9.93552300e-03, -1.27233620e-02, -2.27930850e-02, -4.36304000e-04
        ], [
            -2.10141350e-02, -2.91125780e-02, -1.53488670e-02, -2.19309020e-02, -1.54466040e-02,
            -1.75892850e-02, -2.34599340e-02, -1.17067350e-02, -1.77447910e-02, -1.79959410e-02,
            -2.49177260e-02, -1.84244480e-02, -1.86040380e-02, -1.43176840e-02, -1.32763640e-02,
            -2.19250420e-02, -2.35850370e-02, -2.11743500e-02, -2.19825140e-02, -3.75290530e-02,
            -3.54050100e-02, -2.60025470e-02, -4.95341200e-02, -4.35103810e-02, -3.86330290e-02,
            -2.68911340e-02, -2.21849700e-02, 8.01075656e-01, -7.36152270e-02, -3.74223600e-02,
            -2.49282950e-02, -1.22847240e-02, -2.37091710e-02, -4.31607760e-02, -7.38841500e-03
        ], [
            -5.10560900e-03, -5.50722400e-03, -4.82133800e-03, -5.10235900e-03, -4.70979700e-03,
            -6.40151300e-03, -1.00223010e-02, -2.26737400e-03, -4.77634200e-03, -6.37849000e-03,
            -5.12941700e-03, -5.48774200e-03, -6.27561100e-03, -4.78982000e-03, -4.64670900e-03,
            -6.55279100e-03, -5.14756900e-03, -1.02093850e-02, -2.35151900e-02, -2.26312300e-02,
            -4.09056830e-02, -2.95787220e-02, -9.26782700e-03, -1.84085200e-02, -1.13927950e-02,
            -2.22910940e-02, -1.67216030e-02, -2.24875850e-02, 9.67236156e-01, -2.41105130e-02,
            -1.46001580e-02, -1.99729180e-02, -3.40486470e-02, -2.60112500e-02, -5.12455000e-05
        ], [
            -1.12261750e-02, -2.87928250e-02, -3.88442320e-02, -2.69587290e-02, -2.19879350e-02,
            -2.68199140e-02, -6.38834440e-02, -1.51799000e-02, -6.78153230e-02, -3.92017640e-02,
            -3.49662050e-02, -3.11778290e-02, -4.54631560e-02, -5.09075870e-02, -4.00249180e-02,
            -3.03309280e-02, -3.82241300e-02, -5.31974200e-02, -5.10051710e-02, -5.47224340e-02,
            -6.03395800e-02, -4.21755100e-02, -4.39072660e-02, -3.40908740e-02, -8.56393220e-02,
            -6.63770940e-02, -7.03776260e-02, -9.35943400e-02, -3.27132610e-02, 8.57256968e-01,
            -7.75693690e-02, -3.20312870e-02, -5.73596800e-02, -7.54432840e-02, -2.39329400e-03
        ], [
            -1.42390800e-03, -3.19993200e-03, -1.82888600e-03, -1.11061100e-03, -7.44897000e-04,
            -3.18836900e-03, -3.22297200e-03, -1.03781400e-03, -2.75221000e-03, -2.03998000e-03,
            -2.16453900e-03, -2.44600400e-03, -1.11767500e-03, -1.09976000e-03, -1.14443700e-03,
            -1.84990800e-03, -4.71496500e-03, -1.47852200e-03, -2.42773500e-03, -1.46077100e-03,
            -2.66898300e-03, -3.62999800e-03, -3.05062200e-03, -5.96128800e-03, -9.44209400e-03,
            -3.34172000e-03, -2.94054500e-03, -1.46970100e-03, -3.06816000e-03, -2.69003700e-03,
            9.92723398e-01, -4.43986700e-03, -4.45018000e-03, -8.16574300e-03, -6.27059000e-06
        ], [
            -5.07793000e-04, -4.50761000e-04, -4.64532000e-04, -4.42401000e-04, -4.79210000e-04,
            -3.97113000e-04, -6.77882000e-04, -3.98894000e-04, -9.96413000e-04, -6.69651000e-04,
            -6.91382000e-04, -6.32005000e-04, -8.95125000e-04, -8.28707000e-04, -7.15183000e-04,
            -4.45345000e-04, -8.06414000e-04, -4.89027000e-04, -8.11212000e-04, -7.41528000e-04,
            -1.03393500e-03, -5.85263000e-04, -1.14229700e-03, -3.75702000e-04, -1.88586000e-03,
            -1.18229400e-03, -1.52282800e-03, -1.19573100e-03, -2.98592000e-04, -2.32797200e-03,
            -4.12804200e-03, 9.88067408e-01, -1.08734200e-03, -1.58537400e-03, -5.05288000e-06
        ], [
            -1.49048400e-03, -3.81636000e-04, -5.15390000e-04, -4.12807000e-04, -1.08684300e-03,
            -4.03250000e-04, -4.31896000e-04, -1.94739000e-04, -5.26594000e-04, -4.05704000e-04,
            -4.27699000e-04, -3.73029000e-04, -5.35757000e-04, -3.72755000e-04, -3.40403000e-04,
            -4.99627000e-04, -3.43530000e-04, -4.34422000e-04, -5.67715000e-04, -6.11904000e-04,
            -4.64216000e-04, -1.08357800e-03, -6.62960000e-04, -3.40282000e-04, -4.56104000e-04,
            -6.26340000e-04, -6.87475000e-04, -5.32380000e-04, -9.48140000e-05, -7.26930000e-04,
            -4.75022700e-03, -1.50571600e-03, 9.75124459e-01, -1.14675300e-03, -1.45770000e-07
        ], [
            -2.56815400e-03, -6.47688800e-03, -5.63597300e-03, -5.23361200e-03, -6.52633500e-03,
            -5.50954600e-03, -1.34506170e-02, -2.86294300e-03, -7.96398400e-03, -5.81574000e-03,
            -5.99028300e-03, -6.36244900e-03, -5.84032100e-03, -4.48211900e-03, -4.26095200e-03,
            -5.26540000e-03, -5.96178500e-03, -6.03579000e-03, -8.27721200e-03, -8.21787100e-03,
            -8.63744900e-03, -1.31567380e-02, -9.22678600e-03, -5.18577800e-03, -8.00377700e-03,
            -1.02179080e-02, -2.06491870e-02, -9.87035100e-03, -7.25621700e-03, -2.62278870e-02,
            -1.59218830e-02, -1.05316380e-02, -1.36945030e-02, 9.20896753e-01, -1.87235100e-03
        ], [
            -1.72340000e-05, -2.03920000e-05, -2.61728000e-05, -5.94183000e-05, -2.38557000e-05,
            -3.22037000e-05, -3.81192000e-05, -1.34552000e-05, -3.20485000e-05, -3.92075000e-05,
            -2.20377000e-05, -2.06752000e-05, -2.66707000e-05, -1.55686000e-05, -4.20794000e-05,
            -1.41105000e-05, -9.84367000e-06, -6.57959000e-05, -5.45386000e-05, -1.32667000e-05,
            -3.83636000e-05, -5.16637000e-05, -7.15444000e-05, -1.04754000e-05, -2.32562000e-05,
            -1.88441000e-05, -3.94316000e-05, -4.90602000e-05, -2.42861000e-04, -6.11539000e-05,
            -7.05908000e-05, -3.89425000e-05, -6.60497000e-05, -1.10022000e-04, 9.99904553e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 374)
    return ret


@cache('run')
def ia_matrix_x2003x():
    """
    ia matrix x2003x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.71748522e-01, -8.02303000e-04, -2.56680270e-01, -4.26759000e-02, -4.24833530e-02,
            -1.24470047e-01, -1.28684910e-02, -1.94582100e-03, -5.33110400e-03, -1.32032440e-02,
            -5.64509000e-04, -2.52567000e-04, -2.65907000e-04, -1.72078000e-04, -1.50330000e-04,
            -9.71301700e-03, -4.72600000e-04, -8.13610600e-03, -4.95083000e-04, -2.97151100e-03,
            -2.56921500e-03, -3.87911500e-02, -1.61640900e-03, -1.17875000e-03, -2.89130000e-04,
            -4.03185100e-03, -2.62692000e-04, -1.39846000e-04, -3.60426000e-04, -6.95940000e-04,
            -2.17600800e-03, -2.26295600e-03, -2.16439500e-03, -2.43113000e-03, -1.75574470e-02
        ], [
            -3.32194000e-03, 9.17765794e-01, -2.67074300e-03, -2.43971100e-03, -1.26203900e-03,
            -4.13203900e-03, -4.69810900e-03, -4.66175612e-01, -3.34491200e-02, -3.32369800e-03,
            -6.69594540e-02, -4.75339290e-02, -3.01725900e-03, -1.22849300e-03, -1.24480800e-03,
            -5.44923200e-03, -1.13529653e-01, -1.76229880e-02, -4.90317000e-04, -6.58781000e-04,
            -1.18091200e-03, -1.76269300e-03, -4.44899700e-03, -2.72816100e-03, -2.36512700e-03,
            -1.74729900e-03, -5.60834000e-04, -2.64762000e-04, -1.28854300e-03, -8.37147000e-04,
            -3.08853900e-03, -2.89221900e-03, -1.09291300e-03, -2.73859800e-03, -6.02880180e-02
        ], [
            -5.65114770e-02, -7.62079000e-04, 8.49354659e-01, -3.34235200e-03, -8.72385390e-02,
            -1.55732300e-03, -2.96378400e-03, -1.08170200e-03, -1.01178540e-02, -2.95153500e-03,
            -1.95463100e-03, -1.20464500e-03, -1.23883900e-03, -1.06121300e-03, -9.44560000e-04,
            -2.33036800e-03, -8.13332000e-04, -1.03660100e-03, -1.61463500e-03, -4.78902300e-03,
            -4.53473500e-03, -1.50432337e-01, -1.35537900e-03, -6.08859000e-03, -3.40722200e-03,
            -1.62817300e-03, -1.41967500e-03, -6.82969000e-04, -2.36472000e-04, -1.67090100e-03,
            -7.21670200e-03, -8.04687600e-03, -1.32670670e-02, -6.67873200e-03, -2.99125300e-03
        ], [
            -1.98340400e-03, -1.02539400e-03, -1.58340000e-03, 6.90839270e-01, -5.21977980e-02,
            -3.44990300e-03, -5.13589600e-03, -5.17824000e-04, -3.29715500e-03, -1.18119250e-02,
            -3.77385700e-03, -1.65243700e-03, -2.45568500e-03, -1.87360400e-03, -5.40423100e-03,
            -2.08679020e-02, -5.80324000e-04, -2.24290800e-03, -1.37783200e-03, -2.42818400e-03,
            -2.94633500e-03, -2.67015400e-03, -1.66837800e-03, -2.21542700e-03, -1.33451500e-03,
            -1.49340700e-03, -7.61255000e-04, -4.54390000e-04, -2.48974000e-04, -1.17287500e-03,
            -3.00941400e-03, -9.38677000e-04, -2.87904100e-03, -5.47380500e-03, -1.04462000e-04
        ], [
            -1.37004000e-04, -8.05120000e-05, -1.26554000e-04, -7.74751800e-03, 7.87508376e-01,
            -4.58159000e-04, -5.10193000e-04, -4.43231000e-05, -2.16748000e-04, -1.17198100e-03,
            -3.16792000e-04, -1.80410000e-04, -2.17237000e-04, -2.49284000e-04, -1.44764000e-03,
            -4.22544100e-03, -1.10859000e-04, -1.20735000e-04, -3.96394000e-04, -4.05119000e-04,
            -3.02006000e-04, -1.22890000e-04, -1.90731000e-04, -1.31819000e-04, -1.26042000e-04,
            -1.50889000e-04, -2.44135000e-04, -3.16055000e-05, -1.06295000e-05, -1.08215000e-04,
            -4.17681000e-04, -1.00702000e-04, -1.04411000e-04, -3.85726000e-04, -1.74238000e-06
        ], [
            -1.94547100e-03, -1.53071000e-03, -1.66213100e-03, -1.23041100e-03, -1.71112400e-03,
            7.77953834e-01, -1.04227650e-02, -2.55988000e-04, -1.48333000e-03, -3.34074400e-03,
            -6.49399100e-03, -3.12936600e-03, -2.93076300e-03, -1.56011200e-03, -2.78347800e-03,
            -7.07923560e-02, -8.67246000e-04, -3.24768460e-02, -7.78512000e-04, -1.44826800e-03,
            -1.23942500e-03, -2.31339400e-03, -1.17710500e-03, -5.54803000e-04, -4.07349000e-04,
            -2.84945900e-03, -6.93898000e-04, -3.46182000e-04, -1.87363000e-03, -9.13146000e-04,
            -1.59978000e-03, -2.19571400e-03, -1.07239400e-03, -2.72011800e-03, -9.64006000e-05
        ], [
            -2.52337600e-03, -2.73276100e-03, -2.40862590e-02, -1.10697230e-02, -1.38687430e-02,
            -1.29966260e-02, 7.67410711e-01, -1.26319600e-03, -1.61222870e-02, -1.65689350e-02,
            -1.95180140e-02, -5.28792200e-03, -8.13990500e-03, -1.04797490e-02, -4.70477200e-03,
            -1.99337120e-02, -3.12849600e-03, -4.33303200e-03, -7.54508400e-03, -1.29497450e-02,
            -1.19816130e-02, -8.41454900e-03, -6.04787400e-03, -2.95376100e-03, -5.30732700e-03,
            -1.31828380e-02, -1.11385330e-02, -1.30264470e-02, -2.11870100e-03, -2.07015330e-02,
            -1.85981700e-02, -1.78997930e-02, -8.68849200e-03, -2.46048340e-02, -1.73880000e-04
        ], [
            -1.21573040e-02, -9.10531400e-03, -3.04697800e-03, -5.07561300e-03, -3.74425000e-03,
            -4.76922700e-03, -4.35797200e-03, 9.61798473e-01, -3.39840570e-02, -8.17941400e-03,
            -1.71384030e-02, -1.02829340e-02, -3.71982000e-03, -2.07222200e-03, -1.81411000e-03,
            -3.55665100e-03, -1.85488950e-02, -1.20683890e-02, -3.96800100e-03, -4.99577400e-03,
            -3.78056900e-03, -3.35549100e-03, -4.92104870e-02, -7.24765890e-02, -6.39565230e-02,
            -1.57864180e-02, -3.17990800e-03, -1.11410500e-03, -9.21764000e-04, -4.41494700e-03,
            -8.35064300e-03, -2.94373800e-03, -2.81665200e-03, -5.07439900e-03, -3.48911000e-05
        ], [
            -3.83364240e-02, -1.05225340e-02, -9.99355000e-03, -6.98951300e-02, -3.75624900e-02,
            -3.33928600e-02, -3.54118890e-02, -1.33670750e-02, 7.62568528e-01, -2.05155668e-01,
            -3.48999270e-02, -1.59331900e-02, -1.14403060e-02, -1.72954430e-02, -1.18895920e-02,
            -2.56873810e-02, -6.46313300e-03, -1.08241100e-02, -6.28254700e-03, -3.16639100e-03,
            -2.22327000e-03, -4.50257600e-03, -3.23309000e-03, -4.40826200e-03, -3.28092700e-03,
            -3.83292600e-03, -1.40199500e-03, -8.07658000e-04, -1.30358800e-03, -4.75064000e-03,
            -1.00089100e-02, -3.80054200e-03, -6.32926920e-02, -9.94818300e-03, -1.43276000e-04
        ], [
            -5.04272100e-03, -6.34016900e-03, -1.76829380e-02, -1.03675410e-02, -3.60742100e-02,
            -9.09816900e-03, -1.58804390e-02, -2.57975800e-03, -1.81464690e-02, 8.67111406e-01,
            -1.05183570e-02, -7.11050200e-03, -2.69913260e-02, -2.45443220e-02, -3.62190780e-02,
            -3.28039210e-02, -2.41965200e-03, -1.84336740e-02, -1.65728210e-02, -4.54399900e-03,
            -5.11787500e-03, -4.00521800e-03, -1.02388110e-02, -3.05991600e-03, -3.14877000e-03,
            -6.06173800e-03, -3.40252400e-03, -5.98153000e-04, -9.94060000e-04, -2.78973300e-03,
            -3.29233300e-03, -1.05112300e-03, -5.05447600e-03, -3.81921700e-03, -2.64243000e-05
        ], [
            -1.95863600e-03, -4.55900200e-03, -5.93824400e-03, -2.27687700e-03, -1.65278600e-03,
            -7.61714200e-03, -1.19644100e-03, -1.79981700e-03, -5.72934200e-03, -6.10429200e-03,
            8.76847359e-01, -1.04859170e-02, -5.22872700e-03, -1.06343660e-02, -6.93464300e-03,
            -6.61289000e-03, -1.80079000e-03, -8.01263430e-02, -3.03667400e-03, -1.20353500e-03,
            -1.01262400e-03, -3.31612000e-03, -1.28933400e-03, -6.08874000e-04, -5.66356000e-04,
            -1.31297100e-03, -1.18723600e-03, -1.51955000e-04, -1.53507400e-03, -9.70465000e-04,
            -1.33774900e-03, -1.66046100e-03, -1.93602200e-03, -1.87401000e-03, -2.18944000e-05
        ], [
            -4.45348300e-03, -2.11623880e-02, -1.28311610e-02, -5.68115800e-03, -1.01335150e-02,
            -2.44299490e-02, -8.70371700e-03, -5.61301800e-03, -1.36680060e-02, -2.43789880e-02,
            -3.20334650e-02, 6.83268302e-01, -1.61803666e-01, -6.28867850e-02, -9.29603370e-02,
            -7.59513640e-02, -7.03531500e-03, -8.87433030e-02, -1.74194720e-02, -3.93979200e-03,
            -3.53993900e-03, -4.26510700e-03, -6.70576000e-03, -6.47188600e-03, -4.73943500e-03,
            -6.93764200e-03, -3.88620600e-03, -6.36467000e-04, -2.02614800e-03, -3.63643400e-03,
            -4.62910200e-03, -1.73618300e-03, -2.17874800e-03, -4.32337700e-03, -1.83784000e-04
        ], [
            -6.80399900e-03, -1.56523790e-02, -3.87610800e-03, -8.34418000e-03, -5.87487800e-03,
            -8.44999800e-03, -6.49771500e-03, -5.65669000e-03, -6.96159900e-03, -1.15141510e-02,
            -1.73442180e-02, -1.83043550e-02, 8.68813017e-01, -1.48749720e-02, -3.27397680e-02,
            -9.43820400e-03, -8.69624400e-03, -1.74760870e-02, -2.22262200e-02, -3.69117100e-03,
            -2.53544100e-03, -2.07649600e-03, -5.55158100e-03, -9.47585700e-03, -5.30898000e-03,
            -5.36304700e-03, -2.88183800e-03, -9.36743000e-04, -1.36902900e-03, -2.92712300e-03,
            -5.93672200e-03, -2.06732000e-03, -2.59423000e-03, -3.49969500e-03, -9.85335000e-05
        ], [
            -1.78492900e-03, -6.39153100e-03, -3.42484600e-03, -6.25244600e-03, -4.82102700e-03,
            -6.90412600e-03, -1.00111540e-02, -3.89356200e-03, -9.55181400e-03, -1.36217440e-02,
            -1.06208300e-02, -1.67401340e-02, -9.24645740e-02, 6.84819637e-01, -5.79010200e-02,
            -2.31955840e-02, -1.80001150e-02, -3.22094730e-02, -2.96276410e-02, -8.85242300e-03,
            -7.69887900e-03, -4.31067200e-03, -6.80328500e-03, -4.21233000e-03, -4.79170100e-03,
            -8.63094300e-03, -3.91045100e-02, -3.66642300e-03, -1.97076300e-03, -1.54137720e-02,
            -1.48755000e-02, -7.39000900e-03, -2.26058780e-02, -1.07449060e-02, -6.51832000e-04
        ], [
            -4.41238200e-03, -5.72494700e-03, -2.06147600e-03, -2.79327300e-03, -2.51920500e-03,
            -3.48837000e-03, -2.56182000e-03, -1.16098900e-03, -2.33490200e-03, -4.62144800e-03,
            -4.33671000e-03, -5.81990100e-03, -1.49530030e-02, -4.88174300e-03, 6.94968018e-01,
            -5.23822700e-03, -2.34375500e-03, -4.79683400e-03, -8.75580610e-02, -5.57886800e-03,
            -3.98116100e-03, -1.46966900e-03, -3.63919170e-02, -4.07638230e-02, -5.05060060e-02,
            -1.20386510e-02, -4.06360100e-03, -1.09553600e-03, -8.87212000e-04, -4.24850900e-03,
            -1.71196080e-02, -1.82615900e-03, -1.31894500e-03, -6.47342400e-03, -1.92978000e-04
        ], [
            -8.21098000e-04, -8.54983000e-04, -9.76825000e-04, -4.35056600e-03, -2.70638800e-03,
            -4.47907900e-03, -3.66212400e-03, -5.15020000e-04, -1.65610100e-03, -2.57464000e-03,
            -2.90577600e-03, -9.18124300e-03, -3.74735400e-03, -2.57256100e-03, -4.90683500e-03,
            9.55632787e-01, -1.16701600e-03, -5.00799600e-03, -1.94378500e-03, -1.21626000e-03,
            -1.42844300e-03, -2.32282600e-03, -1.93328300e-03, -6.94995000e-04, -1.53492800e-03,
            -1.47491700e-03, -9.69247000e-04, -8.58368000e-04, -1.18818100e-03, -1.77861700e-03,
            -3.75793000e-03, -2.91918600e-03, -2.87640600e-03, -4.99227400e-03, -2.11064600e-03
        ], [
            -1.08226790e-02, -2.99651750e-02, -1.31546470e-02, -1.80926510e-02, -8.33211400e-03,
            -1.71324460e-02, -2.31612270e-02, -1.38867800e-02, -2.58379720e-02, -2.08274460e-02,
            -3.90163360e-02, -2.81010050e-02, -1.13982810e-02, -8.03979100e-03, -8.69658800e-03,
            -9.00837000e-03, 8.96458287e-01, -6.21072000e-03, -1.01182420e-02, -6.44308300e-03,
            -1.36312860e-02, -2.31124130e-02, -1.49167330e-02, -3.59501200e-03, -5.21531200e-03,
            -1.42997480e-02, -9.31008900e-03, -4.22939500e-03, -8.48696800e-03, -5.72282300e-03,
            -1.53207510e-02, -2.03415050e-02, -1.21602380e-02, -1.55778360e-02, -2.01872000e-04
        ], [
            -3.63488300e-03, -7.73865700e-03, -2.42325100e-03, -2.78575100e-03, -1.65299600e-03,
            -3.27332100e-03, -3.50614200e-03, -2.27949300e-03, -3.35730000e-03, -2.90060700e-03,
            -7.04022700e-03, -4.17303200e-03, -3.14640500e-03, -2.22779200e-03, -2.01334800e-03,
            -2.51354200e-03, -1.97842610e-02, 9.51294172e-01, -4.41140300e-03, -3.94767500e-03,
            -4.88128700e-03, -5.88780100e-03, -9.56650500e-03, -2.59456700e-03, -3.29991600e-03,
            -1.25886560e-02, -1.08319680e-02, -4.98545200e-03, -3.96755550e-02, -4.82872800e-03,
            -1.84855390e-02, -1.39542890e-02, -6.26095800e-03, -9.78832300e-03, -4.58219000e-04
        ], [
            -4.91710100e-03, -3.63949900e-03, -6.51661800e-03, -5.48061300e-03, -6.80912000e-03,
            -6.11808300e-03, -4.94002500e-03, -2.35465900e-03, -5.87054200e-03, -6.16409000e-03,
            -7.20242100e-03, -5.78330300e-03, -4.97191600e-03, -3.74302800e-03, -4.82082200e-03,
            -6.13993600e-03, -8.11789800e-03, -7.06839200e-03, 9.89378492e-01, -4.05462100e-03,
            -4.06038300e-03, -4.85399100e-03, -1.62035010e-02, -2.14253800e-03, -4.08317900e-03,
            -7.27792600e-03, -3.67328400e-03, -2.51570300e-03, -1.81555400e-03, -5.63835900e-03,
            -4.44887700e-03, -2.83336100e-03, -3.56052200e-03, -6.05758300e-03, -2.19605000e-05
        ], [
            -3.31954970e-02, -2.24457430e-02, -6.46998280e-02, -5.22232380e-02, -6.29310430e-02,
            -5.50456180e-02, -5.03992430e-02, -4.16481710e-02, -4.72265520e-02, -4.84229700e-02,
            -4.14908560e-02, -4.51046930e-02, -4.61255180e-02, -3.87535640e-02, -4.39951290e-02,
            -4.40947280e-02, -2.32186150e-02, -4.11012000e-02, -3.42170090e-02, 9.75390168e-01,
            -1.27887300e-02, -4.64740860e-02, -2.13042860e-02, -2.00534820e-02, -2.02959090e-02,
            -1.94146480e-02, -1.05319590e-02, -3.87646400e-03, -3.33753700e-03, -1.27999820e-02,
            -1.89011370e-02, -1.07751930e-02, -2.59458130e-02, -1.63532180e-02, -1.53094000e-04
        ], [
            -1.70791690e-02, -8.89843400e-03, -3.06894870e-02, -2.69343840e-02, -3.71371090e-02,
            -2.09384840e-02, -1.88116600e-02, -1.09098760e-02, -2.16748130e-02, -2.00665080e-02,
            -1.91602640e-02, -1.63882400e-02, -1.66133440e-02, -1.27348240e-02, -1.85196270e-02,
            -2.47860060e-02, -8.60239000e-03, -3.12572980e-02, -1.55580380e-02, -6.50250700e-03,
            9.92618511e-01, -2.52443450e-02, -1.39709910e-02, -7.86867500e-03, -9.42383900e-03,
            -8.69117800e-03, -8.10811300e-03, -4.35226900e-03, -2.89556400e-03, -8.94767200e-03,
            -8.65356200e-03, -5.90895600e-03, -1.16283190e-02, -1.06826860e-02, -2.51602000e-04
        ], [
            -1.35948200e-03, -3.20475200e-03, -3.76675200e-03, -4.23592100e-03, -4.26205800e-03,
            -4.76913900e-03, -6.03520000e-03, -1.03703800e-03, -4.20023300e-03, -5.45748300e-03,
            -5.27497600e-03, -4.46489900e-03, -5.68673000e-03, -4.59696300e-03, -2.73689000e-03,
            -3.76276500e-03, -4.02270600e-03, -4.60230900e-03, -5.69305600e-03, -8.25776900e-03,
            -5.70977300e-03, 9.89536881e-01, -7.90712600e-03, -4.31708400e-03, -2.24712560e-02,
            -1.74598070e-02, -9.68346500e-03, -9.76856700e-03, -2.46058100e-03, -1.41412070e-02,
            -1.13450950e-02, -6.79655700e-03, -8.66855100e-03, -1.16623870e-02, -6.96350000e-05
        ], [
            -1.36149350e-02, -2.05547840e-02, -2.32978410e-02, -1.91030390e-02, -2.01975000e-02,
            -2.38061470e-02, -1.94923010e-02, -3.54943870e-02, -1.78761040e-02, -1.71643660e-02,
            -3.34782430e-02, -1.74966300e-02, -1.23143300e-02, -8.24426400e-03, -1.12267750e-02,
            -1.77376570e-02, -1.95629090e-02, -1.65995920e-02, -1.38976300e-02, -1.93970030e-02,
            -1.20212640e-02, -9.26114700e-03, 9.54790615e-01, -3.40293690e-02, -1.25907180e-02,
            -5.53300650e-02, -8.79374700e-03, -5.22550300e-03, -1.59457800e-03, -5.68492600e-03,
            -1.11142050e-02, -1.03182640e-02, -6.94049800e-03, -9.03265100e-03, -2.18592000e-04
        ], [
            -3.48930400e-03, -5.94693700e-03, -4.85479800e-03, -4.80660500e-03, -5.79989300e-03,
            -7.80699000e-03, -3.89159100e-03, -5.51181000e-03, -3.50766200e-03, -3.83438900e-03,
            -9.49082800e-03, -6.30640300e-03, -3.50738900e-03, -2.38596800e-03, -2.22157100e-03,
            -3.18962300e-03, -3.22197100e-03, -4.43164900e-03, -1.20768800e-03, -3.94483500e-03,
            -1.98189600e-03, -1.69936000e-03, -3.29176000e-03, 8.80785424e-01, -2.86400300e-03,
            -5.30057900e-03, -8.36337000e-04, -4.95629000e-04, -9.07016000e-04, -1.08764300e-03,
            -1.35315900e-03, -1.06164800e-03, -5.14627000e-04, -1.32215100e-03, -1.86795000e-05
        ], [
            -6.99556000e-04, -1.88220300e-03, -1.38822400e-03, -1.75069100e-03, -2.62776700e-03,
            -1.88402500e-03, -2.49053800e-03, -5.76455000e-04, -1.67305300e-03, -1.89862000e-03,
            -1.63891000e-03, -1.35824500e-03, -2.05649400e-03, -1.79009700e-03, -1.39107800e-03,
            -1.33581100e-03, -7.99612000e-04, -1.09733800e-03, -1.72025300e-03, -2.98952100e-03,
            -1.48766700e-03, -1.25030800e-03, -2.79451900e-03, -7.42701100e-03, 9.68621591e-01,
            -1.71313870e-02, -3.66352500e-03, -2.96186100e-03, -4.56989000e-04, -4.20066600e-03,
            -4.59582800e-03, -2.16929200e-03, -1.25995000e-03, -4.01324900e-03, -1.58223000e-05
        ], [
            -3.30856600e-03, -5.35343700e-03, -6.64638300e-03, -3.93591000e-03, -4.63773700e-03,
            -6.90959300e-03, -9.41968000e-03, -6.47927400e-03, -4.94318500e-03, -5.25752000e-03,
            -8.28373100e-03, -5.15628700e-03, -4.50325200e-03, -3.18760600e-03, -4.13520300e-03,
            -4.35577000e-03, -4.01559200e-03, -5.50103700e-03, -1.06738510e-02, -2.01225400e-02,
            -1.12352150e-02, -7.59859400e-03, -4.03021020e-02, -1.13733278e-01, -9.51037680e-02,
            8.96913511e-01, -4.42478400e-03, -3.30643100e-03, -9.02015000e-04, -5.01641400e-03,
            -4.95666500e-03, -2.42877600e-03, -3.13073800e-03, -5.79038800e-03, -1.69145000e-05
        ], [
            -2.91993300e-03, -5.60112000e-03, -4.77936900e-03, -6.19343300e-03, -7.02229000e-03,
            -6.16571500e-03, -1.31452670e-02, -2.63791700e-03, -8.19369700e-03, -6.38348700e-03,
            -7.09439300e-03, -6.50994700e-03, -7.64009400e-03, -7.52646800e-03, -4.53772000e-03,
            -8.27922700e-03, -6.18993700e-03, -1.24466610e-02, -1.57148960e-02, -2.07644460e-02,
            -2.18391140e-02, -1.44347680e-02, -1.67021330e-02, -1.73526010e-02, -2.56235960e-02,
            -2.51807350e-02, 8.72396950e-01, -2.83027110e-02, -4.98196700e-03, -2.60129770e-02,
            -2.29523960e-02, -1.03936030e-02, -1.28827200e-02, -2.23437140e-02, -5.25984000e-04
        ], [
            -1.95158890e-02, -2.69575900e-02, -1.51380350e-02, -2.10811350e-02, -1.51525560e-02,
            -1.70841910e-02, -2.30849000e-02, -1.11280460e-02, -1.75449700e-02, -1.68560460e-02,
            -2.30945780e-02, -1.79813110e-02, -1.76861670e-02, -1.37642570e-02, -1.32428010e-02,
            -2.50466650e-02, -2.22873920e-02, -2.16586740e-02, -2.15664210e-02, -3.51889020e-02,
            -3.60044000e-02, -2.55757500e-02, -4.79366670e-02, -3.72616570e-02, -3.75762350e-02,
            -2.88828210e-02, -2.23056370e-02, 7.97899626e-01, -7.68052550e-02, -3.80343580e-02,
            -2.17113230e-02, -1.38259940e-02, -2.42107640e-02, -4.28219410e-02, -4.68646200e-03
        ], [
            -4.58697100e-03, -6.31436000e-03, -4.62150900e-03, -5.19848600e-03, -5.14994800e-03,
            -6.11770500e-03, -9.70760800e-03, -1.94145800e-03, -4.60344300e-03, -6.14459100e-03,
            -5.10752100e-03, -5.33350000e-03, -5.89028400e-03, -4.45189800e-03, -4.71305200e-03,
            -6.67042000e-03, -4.69492400e-03, -1.03164050e-02, -2.33946550e-02, -2.21248170e-02,
            -4.11597570e-02, -2.89267750e-02, -9.30485000e-03, -1.75321550e-02, -1.32574460e-02,
            -2.27650480e-02, -1.63154580e-02, -2.30255090e-02, 9.70599511e-01, -2.46864650e-02,
            -1.47040420e-02, -1.98227600e-02, -3.42477030e-02, -2.64201050e-02, -1.11853000e-06
        ], [
            -1.14800060e-02, -2.92061280e-02, -3.78559390e-02, -2.69012910e-02, -2.19754130e-02,
            -2.66305250e-02, -6.28159690e-02, -1.20450420e-02, -6.62188090e-02, -3.80612370e-02,
            -3.44347910e-02, -3.03237800e-02, -4.39365830e-02, -4.82893380e-02, -3.91720570e-02,
            -2.97846210e-02, -3.31710700e-02, -5.32554090e-02, -5.13070370e-02, -5.48596900e-02,
            -6.02001360e-02, -4.23401100e-02, -4.44158700e-02, -3.23667550e-02, -8.56976520e-02,
            -6.64814240e-02, -6.88716540e-02, -9.73475200e-02, -3.30967680e-02, 8.58610603e-01,
            -8.05510630e-02, -3.28567710e-02, -5.81590640e-02, -7.57519390e-02, -2.61784600e-03
        ], [
            -1.49802100e-03, -3.32790000e-03, -1.78846300e-03, -1.12916400e-03, -7.75220000e-04,
            -3.12528400e-03, -3.21711600e-03, -7.25439000e-04, -2.72031000e-03, -1.98567400e-03,
            -2.36379300e-03, -2.48675600e-03, -1.12960400e-03, -1.05827000e-03, -1.18068600e-03,
            -1.85020300e-03, -4.95820600e-03, -1.54509800e-03, -2.39868900e-03, -1.47813800e-03,
            -2.66704200e-03, -3.65436800e-03, -3.08008200e-03, -5.95330100e-03, -9.61778500e-03,
            -3.46850900e-03, -2.96870300e-03, -1.52010700e-03, -2.99225000e-03, -2.75202400e-03,
            9.92549204e-01, -4.42299600e-03, -4.53982300e-03, -8.38793500e-03, -4.47473000e-06
        ], [
            -5.13110000e-04, -4.25779000e-04, -4.73136000e-04, -4.24244000e-04, -4.88117000e-04,
            -3.95908000e-04, -6.84568000e-04, -3.71784000e-04, -9.74420000e-04, -6.78854000e-04,
            -7.02750000e-04, -6.34745000e-04, -8.31845000e-04, -7.63514000e-04, -6.70291000e-04,
            -4.55483000e-04, -7.71864000e-04, -5.06516000e-04, -8.13492000e-04, -7.51655000e-04,
            -9.60005000e-04, -5.96017000e-04, -1.11360000e-03, -3.90144000e-04, -2.00404500e-03,
            -1.22312300e-03, -1.63123100e-03, -1.21071600e-03, -3.08708000e-04, -2.28539900e-03,
            -4.28326900e-03, 9.88389884e-01, -1.14450600e-03, -1.57445500e-03, -3.27316000e-06
        ], [
            -1.34367900e-03, -3.74161000e-04, -4.75534000e-04, -4.02072000e-04, -9.92738000e-04,
            -4.31161000e-04, -4.30067000e-04, -1.67401000e-04, -5.23693000e-04, -3.98995000e-04,
            -4.83237000e-04, -3.86851000e-04, -5.83040000e-04, -3.70009000e-04, -3.61021000e-04,
            -5.09022000e-04, -3.45452000e-04, -4.19761000e-04, -5.61167000e-04, -6.24270000e-04,
            -4.53612000e-04, -1.05566600e-03, -6.55319000e-04, -3.43884000e-04, -4.11240000e-04,
            -6.03165000e-04, -7.07389000e-04, -4.89352000e-04, -9.62737000e-05, -7.12889000e-04,
            -4.80347200e-03, -1.46999000e-03, 9.74389171e-01, -1.13717500e-03, -1.22023000e-07
        ], [
            -2.62254800e-03, -6.09626400e-03, -5.30562300e-03, -4.89638000e-03, -6.31519800e-03,
            -5.41292900e-03, -1.28469910e-02, -2.25259400e-03, -7.51884800e-03, -5.45490500e-03,
            -5.77032800e-03, -5.85929600e-03, -5.27365200e-03, -4.05446100e-03, -4.26233600e-03,
            -5.02349700e-03, -5.47450500e-03, -6.02626200e-03, -7.66298700e-03, -7.88405900e-03,
            -8.51262500e-03, -1.34085050e-02, -1.02947310e-02, -5.20771100e-03, -8.01052200e-03,
            -1.11312290e-02, -2.01232750e-02, -9.83748000e-03, -7.36914600e-03, -2.57598730e-02,
            -1.67622290e-02, -1.08870180e-02, -1.36843470e-02, 9.21643845e-01, -2.18087300e-03
        ], [
            -1.40602000e-05, -2.24296000e-05, -2.47640000e-05, -5.75753000e-05, -1.70001000e-05,
            -3.04476000e-05, -3.74455000e-05, -8.13903000e-06, -1.92898000e-05, -2.43695000e-05,
            -2.01898000e-05, -1.81431000e-05, -2.73016000e-05, -1.53573000e-05, -7.15171000e-05,
            -1.18537000e-05, -5.84632000e-06, -7.03975000e-05, -5.89372000e-05, -1.16449000e-05,
            -3.91367000e-05, -5.29344000e-05, -7.64027000e-05, -2.63495000e-05, -2.58013000e-05,
            -1.91726000e-05, -4.07749000e-05, -5.50902000e-05, -2.32072000e-04, -6.80072000e-05,
            -7.22623000e-05, -3.80176000e-05, -5.41430000e-05, -1.17980000e-04, 9.99886489e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 375)
    return ret


@cache('run')
def ia_matrix_x2004x():
    """
    ia matrix x2004x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.69899915e-01, -7.74275000e-04, -2.61175907e-01, -4.43782240e-02, -4.87998030e-02,
            -1.22809367e-01, -1.36818370e-02, -1.49563600e-03, -5.50217500e-03, -1.30916380e-02,
            -5.76532000e-04, -2.36829000e-04, -2.30561000e-04, -1.22512000e-04, -1.36909000e-04,
            -8.78868000e-03, -4.96432000e-04, -6.82507100e-03, -7.64070000e-04, -2.78480400e-03,
            -2.59458000e-03, -3.94939950e-02, -1.64377500e-03, -1.06356500e-03, -3.26902000e-04,
            -5.20693700e-03, -2.72803000e-04, -1.42262000e-04, -3.76732000e-04, -7.92102000e-04,
            -2.07108600e-03, -2.49696600e-03, -2.19447400e-03, -2.58058700e-03, -1.97274660e-02
        ], [
            -2.92742800e-03, 9.15250896e-01, -2.70467300e-03, -2.51431900e-03, -1.71031400e-03,
            -4.21851500e-03, -4.70128400e-03, -4.58014421e-01, -3.38342440e-02, -3.72342000e-03,
            -7.08707080e-02, -5.09911150e-02, -3.06740000e-03, -1.26768900e-03, -1.47983500e-03,
            -5.68405300e-03, -1.11050254e-01, -1.73704950e-02, -5.56769000e-04, -6.27479000e-04,
            -1.08086900e-03, -1.96787300e-03, -4.88574900e-03, -3.12517400e-03, -2.76601200e-03,
            -2.13013500e-03, -5.73123000e-04, -2.84015000e-04, -1.68068800e-03, -9.09418000e-04,
            -3.42853000e-03, -2.52310700e-03, -1.31156300e-03, -2.97379300e-03, -6.13276040e-02
        ], [
            -5.73441770e-02, -7.91808000e-04, 8.47234911e-01, -3.72887100e-03, -8.80748290e-02,
            -1.79564700e-03, -2.84369500e-03, -1.32218100e-03, -1.00918890e-02, -3.04923300e-03,
            -1.91290900e-03, -1.31039000e-03, -1.32970400e-03, -1.12625900e-03, -9.86803000e-04,
            -2.35137800e-03, -8.68037000e-04, -1.11816700e-03, -1.76236600e-03, -4.59091700e-03,
            -4.59084800e-03, -1.50576988e-01, -1.49648300e-03, -5.23177200e-03, -3.41117300e-03,
            -1.74716900e-03, -1.49606900e-03, -6.54823000e-04, -2.53407000e-04, -1.73284300e-03,
            -7.04671300e-03, -8.45133000e-03, -1.32167730e-02, -6.84673500e-03, -2.76167500e-03
        ], [
            -1.89731300e-03, -1.04573100e-03, -1.39734000e-03, 6.88542011e-01, -5.37366680e-02,
            -3.21995800e-03, -5.29225000e-03, -4.48678000e-04, -3.21208900e-03, -1.21132000e-02,
            -3.49149700e-03, -1.57590400e-03, -2.28697300e-03, -1.84824300e-03, -5.24790600e-03,
            -2.05916710e-02, -6.22831000e-04, -2.36300900e-03, -1.50351200e-03, -2.34570000e-03,
            -3.06205300e-03, -2.77321800e-03, -1.74956900e-03, -2.37808300e-03, -1.50064400e-03,
            -1.54020400e-03, -7.10122000e-04, -4.07962000e-04, -3.34025000e-04, -1.27402700e-03,
            -2.69417100e-03, -8.74733000e-04, -3.08136200e-03, -5.16281200e-03, -9.82311000e-05
        ], [
            -1.36653000e-04, -6.72078000e-05, -1.26619000e-04, -8.06747700e-03, 8.00801276e-01,
            -3.93610000e-04, -5.21965000e-04, -4.45521000e-05, -2.05855000e-04, -8.93470000e-04,
            -2.37038000e-04, -1.66558000e-04, -2.02861000e-04, -2.49043000e-04, -1.50929600e-03,
            -4.00420400e-03, -9.48250000e-05, -1.12269000e-04, -3.82759000e-04, -3.56891000e-04,
            -2.86177000e-04, -1.19914000e-04, -2.04100000e-04, -1.14343000e-04, -1.21599000e-04,
            -1.39781000e-04, -2.45406000e-04, -2.94447000e-05, -9.67592000e-06, -9.63656000e-05,
            -3.33309000e-04, -9.02760000e-05, -1.04453000e-04, -3.79359000e-04, -2.89658000e-06
        ], [
            -1.88173400e-03, -1.66202200e-03, -1.63154400e-03, -1.29212200e-03, -1.83380200e-03,
            7.66562426e-01, -1.06145140e-02, -2.80412000e-04, -1.47249300e-03, -3.07529300e-03,
            -6.67882900e-03, -3.08011300e-03, -2.83516600e-03, -1.58760300e-03, -2.79181400e-03,
            -7.00314850e-02, -8.37597000e-04, -3.20621010e-02, -1.10202500e-03, -1.44501000e-03,
            -1.27293300e-03, -2.35086800e-03, -1.15739800e-03, -5.56006000e-04, -3.83990000e-04,
            -2.72961100e-03, -6.70007000e-04, -3.36158000e-04, -2.46698100e-03, -9.28017000e-04,
            -1.73362200e-03, -2.44731100e-03, -1.22732100e-03, -2.71138700e-03, -5.12038000e-05
        ], [
            -2.54116200e-03, -2.71543900e-03, -2.44572680e-02, -1.14413470e-02, -1.50201560e-02,
            -1.33262280e-02, 7.60326480e-01, -1.35901000e-03, -1.55347430e-02, -1.61416780e-02,
            -1.99575180e-02, -4.96466500e-03, -7.73937300e-03, -9.72318400e-03, -4.50702000e-03,
            -1.92733640e-02, -3.02021000e-03, -4.26374700e-03, -8.24010000e-03, -1.25484860e-02,
            -1.19433240e-02, -8.44065100e-03, -6.15293300e-03, -3.08726900e-03, -5.13811500e-03,
            -1.35479490e-02, -1.07914290e-02, -1.29751030e-02, -2.23717700e-03, -2.09162580e-02,
            -1.85996000e-02, -1.81896110e-02, -8.60403300e-03, -2.48032210e-02, -1.85744000e-04
        ], [
            -1.25060410e-02, -8.36822500e-03, -3.11362900e-03, -5.60864300e-03, -4.37486500e-03,
            -4.94344600e-03, -4.34252400e-03, 9.57676389e-01, -3.74541360e-02, -8.95223500e-03,
            -1.78669280e-02, -1.16825760e-02, -4.10164700e-03, -2.18167500e-03, -1.94482500e-03,
            -3.57470600e-03, -1.96875490e-02, -1.18249540e-02, -4.53408900e-03, -4.89939600e-03,
            -3.78211100e-03, -3.50508700e-03, -5.08409060e-02, -7.24934360e-02, -7.26199330e-02,
            -1.84623880e-02, -3.14673000e-03, -1.15803700e-03, -1.10134000e-03, -4.61319300e-03,
            -8.52161200e-03, -3.14821200e-03, -2.88016100e-03, -5.17497900e-03, -2.90503000e-05
        ], [
            -3.86814450e-02, -1.01517980e-02, -9.60994100e-03, -6.99295390e-02, -3.95372670e-02,
            -3.28385180e-02, -3.49428140e-02, -1.35189790e-02, 7.61344036e-01, -2.06079210e-01,
            -3.48357800e-02, -1.54707340e-02, -1.11080440e-02, -1.75834710e-02, -1.15955410e-02,
            -2.47430380e-02, -6.51564700e-03, -1.06742830e-02, -6.35800500e-03, -3.14610200e-03,
            -2.31491500e-03, -4.43074000e-03, -3.34210800e-03, -4.39989600e-03, -3.63344500e-03,
            -4.10193700e-03, -1.39985300e-03, -7.56893000e-04, -1.57367000e-03, -4.83808300e-03,
            -9.62127900e-03, -3.82603300e-03, -6.19188830e-02, -9.95181600e-03, -1.59609000e-04
        ], [
            -4.91848500e-03, -6.30012600e-03, -1.76853050e-02, -1.03939880e-02, -3.80435640e-02,
            -9.11333500e-03, -1.58774250e-02, -2.39104500e-03, -1.71495730e-02, 8.66067244e-01,
            -1.01524700e-02, -6.71890700e-03, -2.53951400e-02, -2.36846110e-02, -3.56123790e-02,
            -3.17162140e-02, -2.30425900e-03, -1.84065440e-02, -1.67156400e-02, -4.50675400e-03,
            -5.33826800e-03, -4.05170500e-03, -1.02617910e-02, -2.76749700e-03, -3.29625100e-03,
            -6.37087800e-03, -3.35501100e-03, -6.14463000e-04, -1.21697400e-03, -2.90960500e-03,
            -3.38874700e-03, -1.12373300e-03, -5.39072500e-03, -4.32946800e-03, -3.94199000e-05
        ], [
            -1.83492700e-03, -4.73406400e-03, -5.58548400e-03, -2.19006400e-03, -1.64232700e-03,
            -7.75258600e-03, -1.18673800e-03, -1.66415600e-03, -5.47911300e-03, -5.77139300e-03,
            8.74537849e-01, -1.00129380e-02, -5.12704800e-03, -1.03246700e-02, -6.96756200e-03,
            -6.42340500e-03, -1.69316600e-03, -8.33431630e-02, -3.40294700e-03, -1.21487400e-03,
            -1.05728400e-03, -3.27957500e-03, -1.35804500e-03, -5.23046000e-04, -6.02087000e-04,
            -1.35966400e-03, -1.16962200e-03, -1.34833000e-04, -1.71737400e-03, -9.99411000e-04,
            -1.35990800e-03, -1.63521000e-03, -1.99869800e-03, -1.90270400e-03, -2.61914000e-05
        ], [
            -4.26142100e-03, -2.03620280e-02, -1.23863260e-02, -5.27216900e-03, -1.01998280e-02,
            -2.32808420e-02, -8.57055700e-03, -5.67189400e-03, -1.23977100e-02, -2.35553560e-02,
            -3.13337240e-02, 6.78481754e-01, -1.59678259e-01, -6.37515550e-02, -9.38530370e-02,
            -7.49378040e-02, -6.61351300e-03, -8.65322240e-02, -1.58889730e-02, -3.62584000e-03,
            -3.43342000e-03, -4.04700300e-03, -6.37722900e-03, -5.28279500e-03, -4.32403500e-03,
            -7.17126800e-03, -3.62112400e-03, -5.78018000e-04, -2.44146400e-03, -3.65553100e-03,
            -4.36118800e-03, -1.75067000e-03, -2.11396600e-03, -4.33073100e-03, -1.54949000e-04
        ], [
            -7.16682900e-03, -1.63475880e-02, -3.80971700e-03, -8.63931900e-03, -6.72844500e-03,
            -8.89499600e-03, -6.71229000e-03, -6.24966500e-03, -7.10484500e-03, -1.19523290e-02,
            -1.81516250e-02, -1.97196310e-02, 8.58187549e-01, -1.59561020e-02, -3.55062580e-02,
            -1.10541150e-02, -8.45380800e-03, -1.81149960e-02, -2.42086710e-02, -3.40759500e-03,
            -2.60478000e-03, -2.20296300e-03, -5.71824800e-03, -9.26745000e-03, -6.33776600e-03,
            -6.28032900e-03, -3.00798900e-03, -8.95135000e-04, -1.63282300e-03, -2.96721300e-03,
            -6.23944100e-03, -2.18566300e-03, -3.05097000e-03, -3.90165000e-03, -1.77651000e-04
        ], [
            -1.92209500e-03, -7.05703800e-03, -3.62699900e-03, -6.11952700e-03, -5.09153400e-03,
            -7.17913100e-03, -1.03087810e-02, -3.63592300e-03, -9.85502200e-03, -1.36832450e-02,
            -1.07845880e-02, -1.68243090e-02, -9.78528840e-02, 6.70568868e-01, -6.03165670e-02,
            -2.40051900e-02, -2.06115970e-02, -3.40080270e-02, -3.18982340e-02, -8.77337800e-03,
            -8.31992700e-03, -4.75184900e-03, -7.06986400e-03, -4.02719300e-03, -5.16827400e-03,
            -9.67168000e-03, -4.06759020e-02, -3.91680700e-03, -2.33424100e-03, -1.62705030e-02,
            -1.63360870e-02, -8.47123800e-03, -2.38879020e-02, -1.17781010e-02, -7.35315000e-04
        ], [
            -4.32081200e-03, -5.75334400e-03, -2.00998400e-03, -2.73914100e-03, -2.56601500e-03,
            -3.44556700e-03, -2.71425700e-03, -1.10646500e-03, -2.29699700e-03, -4.54680900e-03,
            -4.57504800e-03, -6.00268000e-03, -1.52880360e-02, -4.58553600e-03, 6.94224702e-01,
            -5.90220700e-03, -2.61306200e-03, -4.98851400e-03, -8.80457040e-02, -4.90788100e-03,
            -4.02506200e-03, -1.65617100e-03, -3.64458140e-02, -3.51951150e-02, -4.86405290e-02,
            -1.31487800e-02, -3.80990200e-03, -1.06482800e-03, -9.78667000e-04, -4.21362700e-03,
            -1.69242610e-02, -1.92079600e-03, -1.38348800e-03, -7.22164700e-03, -2.11328000e-04
        ], [
            -5.79394000e-04, -7.78085000e-04, -9.24919000e-04, -4.51234200e-03, -2.86760800e-03,
            -4.23044800e-03, -3.55995600e-03, -5.44110000e-04, -1.66305800e-03, -2.54539200e-03,
            -2.79940400e-03, -8.73186600e-03, -3.73033400e-03, -2.38868400e-03, -5.03155200e-03,
            9.54567277e-01, -1.26663900e-03, -5.13510100e-03, -1.95875400e-03, -1.22738600e-03,
            -1.50478800e-03, -2.33212800e-03, -2.31371700e-03, -7.33862000e-04, -1.84630700e-03,
            -1.53559600e-03, -9.74208000e-04, -7.52306000e-04, -1.37894100e-03, -1.73341400e-03,
            -3.67817100e-03, -2.87813700e-03, -2.85204300e-03, -4.94621000e-03, -2.09599700e-03
        ], [
            -1.08950780e-02, -3.18011060e-02, -1.34750230e-02, -1.84037150e-02, -9.01468200e-03,
            -1.71999580e-02, -2.27255370e-02, -1.65378610e-02, -2.55147410e-02, -2.04693410e-02,
            -4.08645610e-02, -2.87075070e-02, -1.20247760e-02, -7.94338100e-03, -8.67453700e-03,
            -9.46170300e-03, 8.88403618e-01, -6.34555300e-03, -1.10901140e-02, -6.44474900e-03,
            -1.41201930e-02, -2.30139650e-02, -1.56641850e-02, -3.58496000e-03, -5.40354000e-03,
            -1.46211150e-02, -9.37905500e-03, -4.05899300e-03, -9.64319500e-03, -5.65569400e-03,
            -1.49486370e-02, -2.06081400e-02, -1.20665500e-02, -1.58761360e-02, -2.01539000e-04
        ], [
            -3.47034200e-03, -7.12033200e-03, -2.37889100e-03, -2.65884100e-03, -1.65362200e-03,
            -3.14126900e-03, -3.20737000e-03, -2.54911000e-03, -3.15237300e-03, -2.70067000e-03,
            -6.62492600e-03, -3.86061400e-03, -2.95116000e-03, -2.07737600e-03, -1.89885300e-03,
            -2.55400100e-03, -1.79182720e-02, 9.51817653e-01, -4.83784000e-03, -3.78297400e-03,
            -4.80761700e-03, -5.72269700e-03, -9.61139800e-03, -2.17123000e-03, -3.09811100e-03,
            -1.25651080e-02, -1.02372910e-02, -4.91485000e-03, -3.31207210e-02, -4.72712900e-03,
            -1.83539230e-02, -1.26027670e-02, -6.40561500e-03, -9.39697700e-03, -7.04410000e-04
        ], [
            -5.28291200e-03, -3.59824700e-03, -6.85647600e-03, -5.42104900e-03, -6.92924900e-03,
            -6.35665100e-03, -4.94823100e-03, -2.50685900e-03, -5.91674700e-03, -6.26503800e-03,
            -7.20621000e-03, -5.75847700e-03, -5.02592600e-03, -3.59584200e-03, -4.94526700e-03,
            -6.53669500e-03, -8.21100200e-03, -7.04338300e-03, 9.89033527e-01, -4.26974800e-03,
            -4.18707200e-03, -4.96463100e-03, -1.63459930e-02, -2.09637200e-03, -4.92444200e-03,
            -7.61504000e-03, -3.73143800e-03, -2.73989700e-03, -1.97804400e-03, -5.65881800e-03,
            -4.44415300e-03, -2.86377000e-03, -3.69497800e-03, -6.25185600e-03, -2.80576000e-05
        ], [
            -3.27775210e-02, -2.11055500e-02, -6.36830100e-02, -4.92490780e-02, -6.14694690e-02,
            -5.54478740e-02, -5.06948720e-02, -4.44648360e-02, -4.64903330e-02, -4.82829260e-02,
            -4.04892020e-02, -4.51494440e-02, -4.64784730e-02, -3.69236670e-02, -4.48964310e-02,
            -4.32366260e-02, -2.20016040e-02, -4.11811600e-02, -3.50629240e-02, 9.74133592e-01,
            -1.31860390e-02, -4.70304140e-02, -2.27472380e-02, -2.02841250e-02, -2.17776470e-02,
            -1.92424520e-02, -1.10229510e-02, -3.98018900e-03, -4.10401100e-03, -1.31871600e-02,
            -1.95261110e-02, -1.11428190e-02, -2.66232800e-02, -1.70406820e-02, -1.83597000e-04
        ], [
            -1.67548450e-02, -7.88811500e-03, -3.05777930e-02, -2.60950400e-02, -3.59238660e-02,
            -2.06619300e-02, -1.83592880e-02, -1.16462030e-02, -2.18034690e-02, -2.00766380e-02,
            -1.83112360e-02, -1.65604160e-02, -1.66213810e-02, -1.21657890e-02, -1.88427110e-02,
            -2.58647970e-02, -7.75756200e-03, -3.17293720e-02, -1.59921100e-02, -6.31801400e-03,
            9.92515284e-01, -2.41508240e-02, -1.40641970e-02, -7.34233000e-03, -9.42702600e-03,
            -8.64514000e-03, -7.99204800e-03, -4.41516500e-03, -3.28514800e-03, -8.62798000e-03,
            -8.33814200e-03, -5.85678200e-03, -1.18744970e-02, -1.07476630e-02, -2.83483000e-04
        ], [
            -1.40388600e-03, -3.42211900e-03, -3.81387300e-03, -4.13898200e-03, -4.45410800e-03,
            -4.49465000e-03, -5.70412300e-03, -1.03552500e-03, -3.97257700e-03, -5.08278000e-03,
            -5.06533100e-03, -4.09355500e-03, -5.43052300e-03, -4.21937900e-03, -2.66690900e-03,
            -3.59280300e-03, -3.20274900e-03, -4.70831300e-03, -5.59666400e-03, -8.02711000e-03,
            -6.40856000e-03, 9.89444392e-01, -7.87309600e-03, -4.08579000e-03, -1.88424700e-02,
            -1.94853690e-02, -1.03236010e-02, -1.04699290e-02, -2.96638800e-03, -1.41742880e-02,
            -1.18148440e-02, -8.00883000e-03, -9.02257000e-03, -1.18070140e-02, -7.71145000e-05
        ], [
            -1.28680660e-02, -2.09082680e-02, -2.39986750e-02, -1.98419460e-02, -2.09371680e-02,
            -2.39983490e-02, -1.92816980e-02, -3.70862840e-02, -1.79377400e-02, -1.70151590e-02,
            -3.32454210e-02, -1.72338140e-02, -1.24261170e-02, -8.17037700e-03, -1.11318940e-02,
            -1.82597740e-02, -1.85308420e-02, -1.79586030e-02, -1.55891940e-02, -2.01398600e-02,
            -1.21552450e-02, -9.45805000e-03, 9.54690728e-01, -2.51701080e-02, -1.14199870e-02,
            -5.48867530e-02, -8.70592200e-03, -4.91893100e-03, -1.90371100e-03, -5.64568300e-03,
            -1.09826640e-02, -1.00528440e-02, -7.35656700e-03, -9.25709300e-03, -3.45466000e-04
        ], [
            -3.27364600e-03, -6.68783400e-03, -4.79557500e-03, -4.83077400e-03, -5.37470000e-03,
            -7.23859500e-03, -3.74233200e-03, -4.47819600e-03, -3.14604900e-03, -3.67099100e-03,
            -8.91332000e-03, -5.74590400e-03, -3.47010500e-03, -2.28116800e-03, -2.21654400e-03,
            -2.90981800e-03, -3.21779100e-03, -3.89156600e-03, -1.70262200e-03, -4.75212500e-03,
            -2.42280800e-03, -1.65016000e-03, -3.29911200e-03, 8.69779911e-01, -2.98873900e-03,
            -5.14618700e-03, -9.30586000e-04, -5.76917000e-04, -1.46454600e-03, -1.30416200e-03,
            -1.31489000e-03, -1.05909300e-03, -5.58266000e-04, -1.29071600e-03, -2.86248000e-05
        ], [
            -7.43863000e-04, -1.74894700e-03, -1.42468000e-03, -1.70301800e-03, -2.41070900e-03,
            -1.82829600e-03, -2.35236000e-03, -5.79938000e-04, -1.65475300e-03, -1.73519300e-03,
            -1.58711500e-03, -1.21661600e-03, -1.99510500e-03, -1.72380700e-03, -1.40979600e-03,
            -1.31886200e-03, -7.16554000e-04, -1.05099300e-03, -2.01702900e-03, -2.92126800e-03,
            -1.52603600e-03, -1.31019800e-03, -2.66080300e-03, -6.74706300e-03, 9.67616397e-01,
            -1.56661110e-02, -3.79746900e-03, -3.03238900e-03, -5.41470000e-04, -4.06841400e-03,
            -4.43700100e-03, -2.51598800e-03, -1.30216600e-03, -4.04459500e-03, -1.86978000e-05
        ], [
            -3.27324300e-03, -4.37042400e-03, -6.58271000e-03, -3.98432800e-03, -4.73592100e-03,
            -6.91487400e-03, -9.12793000e-03, -6.72988600e-03, -4.83850700e-03, -5.23318700e-03,
            -8.26775200e-03, -4.91378200e-03, -4.38512800e-03, -3.09134900e-03, -4.11535100e-03,
            -4.46465600e-03, -3.88728400e-03, -6.75518400e-03, -1.12498190e-02, -2.05741490e-02,
            -1.19000010e-02, -7.56136100e-03, -4.00586470e-02, -1.14880954e-01, -8.90478130e-02,
            8.96920670e-01, -4.46332100e-03, -3.42245100e-03, -9.87415000e-04, -5.03540500e-03,
            -4.83435500e-03, -2.66042000e-03, -3.21474300e-03, -5.94276400e-03, -2.15542000e-05
        ], [
            -3.26090100e-03, -5.38072100e-03, -5.03943600e-03, -6.07903500e-03, -6.95857400e-03,
            -6.14659700e-03, -1.26153710e-02, -2.85334000e-03, -7.76185500e-03, -6.03018300e-03,
            -6.84695500e-03, -6.29302200e-03, -7.15513200e-03, -7.34178600e-03, -4.46178800e-03,
            -8.79578400e-03, -6.12461900e-03, -1.18794350e-02, -1.68298600e-02, -2.01091240e-02,
            -2.20208810e-02, -1.47548380e-02, -1.68211310e-02, -1.81339090e-02, -2.32168890e-02,
            -2.49641940e-02, 8.79178202e-01, -2.82480060e-02, -5.59294300e-03, -2.64718250e-02,
            -2.27310150e-02, -1.07914480e-02, -1.35783320e-02, -2.20509360e-02, -6.16615000e-04
        ], [
            -1.85422100e-02, -2.35228610e-02, -1.48695320e-02, -2.09248750e-02, -1.51554470e-02,
            -1.59880040e-02, -2.04029870e-02, -1.06939930e-02, -1.63664960e-02, -1.46514170e-02,
            -2.07937000e-02, -1.61677760e-02, -1.62810460e-02, -1.26402410e-02, -1.24462090e-02,
            -2.69633730e-02, -2.04353950e-02, -2.06243690e-02, -1.90764630e-02, -3.19634360e-02,
            -3.45663290e-02, -2.30973020e-02, -4.44176570e-02, -3.03579650e-02, -3.19604730e-02,
            -2.99466420e-02, -1.97651890e-02, 7.93360473e-01, -7.95881410e-02, -3.58365850e-02,
            -2.33638680e-02, -1.26131150e-02, -2.20908940e-02, -4.10019450e-02, -4.49045300e-03
        ], [
            -4.04621400e-03, -4.97516000e-03, -4.77929900e-03, -4.97562500e-03, -5.56376900e-03,
            -6.02942400e-03, -9.82295800e-03, -1.75156000e-03, -4.32843600e-03, -5.94821200e-03,
            -4.88501400e-03, -4.75414200e-03, -5.48476500e-03, -4.13049300e-03, -4.52306600e-03,
            -6.69106400e-03, -4.75688800e-03, -1.02340630e-02, -2.28393310e-02, -2.15548050e-02,
            -4.39882290e-02, -2.94661940e-02, -9.15289600e-03, -1.54794640e-02, -1.21738200e-02,
            -2.25251540e-02, -1.61851750e-02, -2.40661650e-02, 9.68583263e-01, -2.52612650e-02,
            -1.41365950e-02, -1.94337900e-02, -3.54475070e-02, -2.72451680e-02, -9.70666000e-07
        ], [
            -1.16844190e-02, -2.81276270e-02, -3.78859430e-02, -2.44815470e-02, -2.18416850e-02,
            -2.58918980e-02, -6.00645810e-02, -1.11489730e-02, -6.25612360e-02, -3.63508260e-02,
            -3.33984370e-02, -2.77272480e-02, -4.09906950e-02, -4.39324440e-02, -3.81107940e-02,
            -2.79152900e-02, -3.03308500e-02, -5.22923450e-02, -5.09250380e-02, -5.58059200e-02,
            -6.13278170e-02, -4.30135960e-02, -4.32819370e-02, -3.01744220e-02, -7.77569380e-02,
            -6.56572160e-02, -6.59504010e-02, -9.92202760e-02, -3.77846070e-02, 8.57201047e-01,
            -8.16976210e-02, -3.36622210e-02, -5.99778760e-02, -7.65176090e-02, -2.81316800e-03
        ], [
            -1.48843700e-03, -2.91252500e-03, -1.85040900e-03, -9.94792000e-04, -7.73519000e-04,
            -3.27295300e-03, -3.07774600e-03, -6.36403000e-04, -2.53232800e-03, -1.81190100e-03,
            -2.35175900e-03, -2.44771000e-03, -1.07363400e-03, -9.77694000e-04, -1.12327500e-03,
            -1.97013300e-03, -4.37816500e-03, -1.50649500e-03, -2.68250900e-03, -1.54196600e-03,
            -2.75010900e-03, -3.70132500e-03, -3.47957800e-03, -5.61457600e-03, -9.02984700e-03,
            -3.27008000e-03, -2.96100700e-03, -1.58472600e-03, -3.81479100e-03, -3.01024000e-03,
            9.92639381e-01, -4.43589800e-03, -4.58191400e-03, -8.65115900e-03, -3.20839000e-06
        ], [
            -5.56463000e-04, -4.47731000e-04, -4.72050000e-04, -4.00693000e-04, -5.01891000e-04,
            -3.79925000e-04, -6.42913000e-04, -3.41180000e-04, -9.22470000e-04, -6.59945000e-04,
            -6.07519000e-04, -5.99086000e-04, -8.18485000e-04, -7.10904000e-04, -7.16141000e-04,
            -4.45602000e-04, -7.77906000e-04, -5.19213000e-04, -8.01209000e-04, -8.53002000e-04,
            -9.55051000e-04, -6.17786000e-04, -1.13914000e-03, -3.51224000e-04, -1.95100400e-03,
            -1.26037200e-03, -1.68989800e-03, -1.20888500e-03, -3.17854000e-04, -2.43153800e-03,
            -4.52612700e-03, 9.86506013e-01, -1.21640500e-03, -1.65583800e-03, -1.01413000e-05
        ], [
            -1.31027800e-03, -4.19133000e-04, -4.43122000e-04, -3.94217000e-04, -9.33898000e-04,
            -4.35614000e-04, -4.29299000e-04, -1.51702000e-04, -5.10766000e-04, -3.83228000e-04,
            -5.10215000e-04, -3.77838000e-04, -6.48206000e-04, -3.64316000e-04, -3.77947000e-04,
            -4.98168000e-04, -3.62422000e-04, -3.94267000e-04, -5.43517000e-04, -5.66984000e-04,
            -4.38814000e-04, -9.33967000e-04, -6.35268000e-04, -2.97199000e-04, -4.08273000e-04,
            -5.63743000e-04, -6.85173000e-04, -4.87641000e-04, -8.93916000e-05, -6.77312000e-04,
            -4.85638200e-03, -1.51250000e-03, 9.73546975e-01, -1.22818400e-03, -1.97896000e-07
        ], [
            -2.74204100e-03, -6.22361600e-03, -5.31939300e-03, -4.66787000e-03, -6.79101600e-03,
            -5.36808100e-03, -1.25907390e-02, -2.02027600e-03, -7.13448800e-03, -5.22632100e-03,
            -5.80637600e-03, -5.36287300e-03, -4.99537900e-03, -3.74488100e-03, -4.30369500e-03,
            -4.93669900e-03, -5.40312500e-03, -6.18648400e-03, -7.84787600e-03, -7.80679300e-03,
            -8.86076700e-03, -1.34148610e-02, -1.11418210e-02, -5.43260300e-03, -7.86259700e-03,
            -1.25011830e-02, -1.97383310e-02, -1.00350200e-02, -8.13337100e-03, -2.60672780e-02,
            -1.71875880e-02, -1.18511850e-02, -1.41278850e-02, 9.21440057e-01, -2.35424000e-03
        ], [
            -1.40514000e-05, -2.34077000e-05, -2.46892000e-05, -6.78197000e-05, -2.22322000e-05,
            -2.95985000e-05, -3.51020000e-05, -8.62592000e-06, -1.84598000e-05, -2.19341000e-05,
            -1.99980000e-05, -1.61258000e-05, -2.82623000e-05, -1.47014000e-05, -7.49342000e-05,
            -1.17246000e-05, -5.32583000e-06, -6.75919000e-05, -5.31389000e-05, -1.13143000e-05,
            -4.09037000e-05, -5.39261000e-05, -9.46746000e-05, -1.75429000e-05, -2.16006000e-05,
            -1.80988000e-05, -3.93390000e-05, -6.18009000e-05, -2.34087000e-04, -6.63067000e-05,
            -6.98526000e-05, -3.79116000e-05, -5.46024000e-05, -1.18257000e-04, 9.99870855e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 376)
    return ret


@cache('run')
def ia_matrix_x2005x():
    """
    ia matrix x2005x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.70592734e-01, -6.50647000e-04, -2.60644047e-01, -4.82116640e-02, -5.34406310e-02,
            -1.30996855e-01, -1.54319550e-02, -1.46540400e-03, -6.00279500e-03, -1.32278500e-02,
            -5.84270000e-04, -2.34330000e-04, -2.23745000e-04, -1.15862000e-04, -1.38397000e-04,
            -9.04673700e-03, -5.18845000e-04, -5.75928600e-03, -8.39906000e-04, -2.67512200e-03,
            -2.48980600e-03, -3.96429490e-02, -1.80484700e-03, -9.90375000e-04, -3.38074000e-04,
            -6.38440700e-03, -2.54595000e-04, -1.41305000e-04, -3.66732000e-04, -8.31057000e-04,
            -2.22532000e-03, -2.59700300e-03, -2.09542600e-03, -2.58776800e-03, -2.11229470e-02
        ], [
            -2.90581900e-03, 9.12955335e-01, -2.95081000e-03, -2.77411800e-03, -2.09778800e-03,
            -4.65929200e-03, -4.99714500e-03, -4.52335484e-01, -3.12404690e-02, -4.07666600e-03,
            -7.14362550e-02, -5.25436550e-02, -3.23321600e-03, -1.37566900e-03, -1.83426700e-03,
            -5.41606900e-03, -1.13475558e-01, -1.90153290e-02, -5.56346000e-04, -5.49434000e-04,
            -1.06800800e-03, -2.15148100e-03, -5.34343600e-03, -3.53346200e-03, -2.97795100e-03,
            -2.66198500e-03, -5.53977000e-04, -2.85576000e-04, -1.58184300e-03, -8.68381000e-04,
            -3.29631900e-03, -2.38784000e-03, -1.52008500e-03, -3.00242000e-03, -6.20859740e-02
        ], [
            -5.93533220e-02, -7.89966000e-04, 8.45582028e-01, -4.33457000e-03, -9.59870140e-02,
            -2.09252400e-03, -2.96625600e-03, -1.48326100e-03, -1.07752610e-02, -3.42603100e-03,
            -2.24522800e-03, -1.53884700e-03, -1.52992500e-03, -1.29479200e-03, -1.03479700e-03,
            -2.47167200e-03, -9.60980000e-04, -1.28225500e-03, -1.83122300e-03, -4.56515300e-03,
            -4.77448800e-03, -1.51746417e-01, -1.68998800e-03, -5.13683500e-03, -3.55751400e-03,
            -1.81543400e-03, -1.55329800e-03, -6.66602000e-04, -2.77455000e-04, -1.79361000e-03,
            -7.54766900e-03, -8.37846700e-03, -1.30379270e-02, -6.92779300e-03, -2.98013700e-03
        ], [
            -1.91152900e-03, -9.49837000e-04, -1.48260000e-03, 6.70378120e-01, -5.53173290e-02,
            -3.34860400e-03, -5.64421300e-03, -4.47171000e-04, -3.39575300e-03, -1.28944430e-02,
            -3.63370700e-03, -1.68094200e-03, -2.28019600e-03, -1.89392600e-03, -4.96849300e-03,
            -2.13555380e-02, -7.14140000e-04, -2.45479100e-03, -1.59257200e-03, -2.29040900e-03,
            -3.28074200e-03, -2.88068000e-03, -1.87744300e-03, -2.63730500e-03, -1.66522000e-03,
            -1.70100600e-03, -7.24024000e-04, -4.17828000e-04, -3.53352000e-04, -1.35372700e-03,
            -2.78864700e-03, -9.88379000e-04, -3.34928300e-03, -5.01898300e-03, -9.82303000e-05
        ], [
            -1.34632000e-04, -5.66370000e-05, -1.20508000e-04, -9.52629800e-03, 7.95572027e-01,
            -3.99627000e-04, -6.03808000e-04, -4.06085000e-05, -2.07013000e-04, -8.62661000e-04,
            -2.34157000e-04, -1.55928000e-04, -1.89402000e-04, -2.55821000e-04, -1.62171400e-03,
            -4.16672800e-03, -8.59049000e-05, -1.06069000e-04, -4.13266000e-04, -3.33671000e-04,
            -2.60546000e-04, -1.20863000e-04, -2.17261000e-04, -1.14866000e-04, -1.26583000e-04,
            -1.34258000e-04, -2.42413000e-04, -2.97885000e-05, -8.53178000e-06, -9.18227000e-05,
            -4.51424000e-04, -9.75274000e-05, -1.15549000e-04, -4.00131000e-04, -3.81655000e-06
        ], [
            -1.75327300e-03, -1.53749800e-03, -1.57046000e-03, -1.30713300e-03, -1.81278700e-03,
            7.61210698e-01, -1.08821510e-02, -3.11857000e-04, -1.45211100e-03, -3.04493300e-03,
            -7.06020200e-03, -3.28445200e-03, -2.92554000e-03, -1.60589000e-03, -2.72332100e-03,
            -6.85203980e-02, -8.29455000e-04, -3.17535110e-02, -1.22778800e-03, -1.40497800e-03,
            -1.25994000e-03, -2.38220800e-03, -1.21233600e-03, -5.74753000e-04, -4.00727000e-04,
            -2.65719200e-03, -6.21036000e-04, -3.24807000e-04, -2.58561400e-03, -8.98688000e-04,
            -1.81941100e-03, -2.58663400e-03, -1.28389300e-03, -2.67975100e-03, -2.84157000e-05
        ], [
            -2.48513400e-03, -2.77719900e-03, -2.46400690e-02, -1.17339460e-02, -1.52084910e-02,
            -1.36807840e-02, 7.58846508e-01, -1.62271700e-03, -1.54984480e-02, -1.64171420e-02,
            -2.09860160e-02, -5.08901200e-03, -7.47380000e-03, -9.25057500e-03, -4.32310300e-03,
            -1.90032300e-02, -3.15642300e-03, -4.26417700e-03, -8.21267400e-03, -1.16149580e-02,
            -1.21378810e-02, -8.44208800e-03, -6.31131400e-03, -3.36299000e-03, -4.93521300e-03,
            -1.33085840e-02, -1.04247700e-02, -1.30946800e-02, -2.23727900e-03, -2.04456820e-02,
            -1.88234580e-02, -1.82178110e-02, -9.01597400e-03, -2.45514740e-02, -1.58570000e-04
        ], [
            -1.19587100e-02, -6.85894500e-03, -3.05210200e-03, -5.25028300e-03, -4.22580800e-03,
            -4.73276700e-03, -4.32947500e-03, 9.33158629e-01, -3.99686080e-02, -8.73692200e-03,
            -1.80582300e-02, -1.16808310e-02, -3.88183600e-03, -1.99121000e-03, -1.88717900e-03,
            -3.49931900e-03, -2.11263940e-02, -1.11181510e-02, -4.59370500e-03, -4.83278900e-03,
            -3.64767400e-03, -3.48167700e-03, -5.02206880e-02, -6.75729620e-02, -8.35631970e-02,
            -1.95609370e-02, -3.14750800e-03, -1.12848800e-03, -1.07463900e-03, -4.50510000e-03,
            -8.79931300e-03, -2.95259500e-03, -2.82224400e-03, -5.00833300e-03, -2.64063000e-05
        ], [
            -3.67139870e-02, -9.56834300e-03, -9.23830000e-03, -6.72894770e-02, -3.74788800e-02,
            -3.21211290e-02, -3.39972790e-02, -1.58569320e-02, 7.61785304e-01, -2.08101049e-01,
            -3.49918160e-02, -1.52188580e-02, -1.05709190e-02, -1.70431930e-02, -1.08401960e-02,
            -2.40959290e-02, -6.30241800e-03, -1.04436170e-02, -5.78148800e-03, -2.73401200e-03,
            -2.13785800e-03, -4.38381200e-03, -3.40062300e-03, -4.45714800e-03, -4.09432100e-03,
            -4.21371200e-03, -1.30113100e-03, -6.95778000e-04, -1.69194600e-03, -4.67863800e-03,
            -9.80899700e-03, -3.68212600e-03, -6.13150200e-02, -9.55409700e-03, -1.65512000e-04
        ], [
            -4.68061700e-03, -5.65750600e-03, -1.82395210e-02, -1.07109110e-02, -3.74844870e-02,
            -8.70858700e-03, -1.59148370e-02, -2.53713900e-03, -1.74600220e-02, 8.60400957e-01,
            -1.06880060e-02, -6.82001600e-03, -2.46354280e-02, -2.34167670e-02, -3.47885630e-02,
            -3.20439210e-02, -2.32139300e-03, -1.85830040e-02, -1.63663310e-02, -4.29452800e-03,
            -5.21975200e-03, -4.12488100e-03, -1.06827570e-02, -2.75437200e-03, -3.42456700e-03,
            -6.28606900e-03, -3.29556500e-03, -6.12400000e-04, -1.32393400e-03, -2.84299000e-03,
            -3.31631700e-03, -1.17729500e-03, -5.23657000e-03, -4.39089000e-03, -6.19045000e-05
        ], [
            -1.65030000e-03, -4.56207700e-03, -5.48705500e-03, -2.18711500e-03, -1.61026100e-03,
            -7.21820900e-03, -1.19498400e-03, -1.87315700e-03, -5.59603100e-03, -5.63121800e-03,
            8.67189694e-01, -1.02300440e-02, -5.09578100e-03, -1.00794010e-02, -6.81332100e-03,
            -6.52423500e-03, -1.80005100e-03, -8.65633170e-02, -3.57048500e-03, -1.01478700e-03,
            -9.93044000e-04, -3.22593700e-03, -1.39243000e-03, -5.05353000e-04, -6.33514000e-04,
            -1.43352500e-03, -1.14918400e-03, -1.21686000e-04, -1.74206900e-03, -9.57210000e-04,
            -1.38266400e-03, -1.64552600e-03, -2.02484500e-03, -1.87696000e-03, -3.25150000e-05
        ], [
            -4.21289100e-03, -1.92697360e-02, -1.15961650e-02, -5.27505000e-03, -9.93610400e-03,
            -2.22018360e-02, -8.34938100e-03, -5.63551900e-03, -1.24252350e-02, -2.31378560e-02,
            -3.15833910e-02, 6.78946697e-01, -1.58598335e-01, -6.43216450e-02, -8.99552160e-02,
            -7.32794790e-02, -6.61667300e-03, -8.67899330e-02, -1.55043230e-02, -3.36144400e-03,
            -3.02494800e-03, -3.93069400e-03, -6.19080000e-03, -4.26811400e-03, -3.71436500e-03,
            -7.87293200e-03, -3.44129900e-03, -5.53337000e-04, -2.48705900e-03, -3.56077300e-03,
            -4.23089500e-03, -1.80412100e-03, -2.17170300e-03, -4.38636700e-03, -1.48229000e-04
        ], [
            -7.09731400e-03, -1.44488570e-02, -4.01987300e-03, -8.92137300e-03, -6.94582400e-03,
            -9.08603400e-03, -6.94780100e-03, -6.99541100e-03, -7.53800400e-03, -1.23596280e-02,
            -1.92908120e-02, -2.07620910e-02, 8.52743007e-01, -1.64887680e-02, -3.65284150e-02,
            -1.23196170e-02, -8.39813100e-03, -1.82615350e-02, -2.41911780e-02, -2.98565900e-03,
            -2.74044800e-03, -2.23312500e-03, -5.72427500e-03, -8.88778800e-03, -6.75399200e-03,
            -7.16273300e-03, -2.90687700e-03, -8.69142000e-04, -1.65643100e-03, -2.85176200e-03,
            -6.30091900e-03, -2.11250200e-03, -3.31100900e-03, -3.94700900e-03, -2.88290000e-04
        ], [
            -1.93112100e-03, -6.57557100e-03, -3.81891000e-03, -6.62701200e-03, -5.07429500e-03,
            -7.58110500e-03, -1.05867950e-02, -3.86199600e-03, -1.03793760e-02, -1.40536590e-02,
            -1.14257920e-02, -1.79305960e-02, -1.00950522e-01, 6.51286268e-01, -6.23567420e-02,
            -2.64964090e-02, -2.25675430e-02, -3.61001440e-02, -3.27286100e-02, -8.69377400e-03,
            -8.82298800e-03, -4.95385700e-03, -7.71230600e-03, -3.79703700e-03, -5.28243700e-03,
            -1.02537640e-02, -4.20606950e-02, -4.00810400e-03, -2.47212200e-03, -1.69193410e-02,
            -1.65526040e-02, -9.33983200e-03, -2.44490150e-02, -1.26243210e-02, -8.71221000e-04
        ], [
            -4.42621800e-03, -5.34238400e-03, -2.00826000e-03, -2.84951800e-03, -2.54109700e-03,
            -3.75074300e-03, -2.98671600e-03, -1.18084000e-03, -2.40432200e-03, -4.58664900e-03,
            -5.18780200e-03, -6.23808300e-03, -1.59569180e-02, -4.72805400e-03, 6.89927280e-01,
            -6.42771100e-03, -2.91642300e-03, -5.24473300e-03, -8.54633620e-02, -4.27999800e-03,
            -4.52730600e-03, -1.78110100e-03, -3.67579000e-02, -3.56240650e-02, -5.04491920e-02,
            -1.43203420e-02, -3.71704400e-03, -1.05187800e-03, -9.83324000e-04, -4.20547900e-03,
            -1.72873410e-02, -2.11203100e-03, -1.56138400e-03, -7.75663100e-03, -2.24268000e-04
        ], [
            -5.58685000e-04, -7.33802000e-04, -8.76293000e-04, -4.78963300e-03, -2.82577000e-03,
            -4.11319400e-03, -3.35138600e-03, -5.91640000e-04, -1.72861300e-03, -2.60413100e-03,
            -2.84245800e-03, -8.37870500e-03, -3.73008600e-03, -2.41991800e-03, -5.03085900e-03,
            9.53801849e-01, -1.45909800e-03, -5.14644600e-03, -1.86121700e-03, -1.14455800e-03,
            -1.47937900e-03, -2.25576000e-03, -2.78017600e-03, -7.11091000e-04, -1.96050100e-03,
            -1.45360000e-03, -1.05837100e-03, -7.24389000e-04, -1.35911200e-03, -1.67887600e-03,
            -3.57882800e-03, -3.01056400e-03, -2.91215600e-03, -5.08520800e-03, -2.13757500e-03
        ], [
            -1.11821540e-02, -3.09358540e-02, -1.40824160e-02, -1.85703700e-02, -8.86742100e-03,
            -1.76808450e-02, -2.33265200e-02, -2.34749050e-02, -2.67821590e-02, -2.11106390e-02,
            -4.34010520e-02, -3.04971090e-02, -1.22708020e-02, -7.89399900e-03, -8.52912200e-03,
            -9.46265000e-03, 8.78496262e-01, -6.79274600e-03, -1.12279070e-02, -6.41096300e-03,
            -1.39830720e-02, -2.41646190e-02, -1.59811970e-02, -3.63746300e-03, -5.55629100e-03,
            -1.54235080e-02, -9.78838900e-03, -4.12463600e-03, -1.03975610e-02, -5.79541300e-03,
            -1.47499370e-02, -2.13192080e-02, -1.24752320e-02, -1.61022950e-02, -1.86442000e-04
        ], [
            -3.26486100e-03, -6.70227100e-03, -2.16569400e-03, -2.39665400e-03, -1.57174500e-03,
            -2.92159600e-03, -2.95846600e-03, -3.62124700e-03, -3.05062800e-03, -2.53819900e-03,
            -6.04943000e-03, -3.83520700e-03, -2.81516100e-03, -1.78502500e-03, -1.67281300e-03,
            -2.47476200e-03, -1.67365220e-02, 9.50846412e-01, -4.95749200e-03, -3.42729800e-03,
            -4.32516900e-03, -5.40782000e-03, -9.05683100e-03, -1.99466500e-03, -2.64926100e-03,
            -1.20233830e-02, -1.02779600e-02, -4.59414000e-03, -3.29100010e-02, -4.45610800e-03,
            -1.78066670e-02, -1.18490470e-02, -6.24018500e-03, -8.92202600e-03, -6.37504000e-04
        ], [
            -5.07719400e-03, -3.24488900e-03, -6.81239400e-03, -5.10396800e-03, -6.53888000e-03,
            -6.19164300e-03, -4.85571200e-03, -2.63331700e-03, -5.84696600e-03, -6.20891700e-03,
            -6.84021700e-03, -5.77684800e-03, -4.84549500e-03, -3.38049600e-03, -4.86111200e-03,
            -6.45140800e-03, -7.58197400e-03, -7.12209500e-03, 9.88437725e-01, -4.43869400e-03,
            -4.14537600e-03, -5.21259800e-03, -1.61719800e-02, -2.09189300e-03, -5.09827300e-03,
            -7.60210200e-03, -3.76376300e-03, -2.83170200e-03, -2.00517800e-03, -5.60188400e-03,
            -4.41962500e-03, -2.87996600e-03, -3.94092500e-03, -6.28486900e-03, -2.38989000e-05
        ], [
            -3.08519120e-02, -1.79751240e-02, -6.22201430e-02, -4.55181500e-02, -5.48668150e-02,
            -5.20009570e-02, -4.73366160e-02, -4.59509680e-02, -4.45511900e-02, -4.56071380e-02,
            -3.71322180e-02, -4.18906230e-02, -4.31869820e-02, -3.33951720e-02, -4.26049760e-02,
            -4.11796350e-02, -2.02195900e-02, -3.96438320e-02, -3.44516590e-02, 9.75640730e-01,
            -1.28313190e-02, -4.64877390e-02, -2.21280220e-02, -1.81022980e-02, -2.05559980e-02,
            -1.88795080e-02, -1.07788540e-02, -3.90607300e-03, -4.13166000e-03, -1.25005730e-02,
            -2.03458290e-02, -1.10285270e-02, -2.70635080e-02, -1.66801460e-02, -1.74201000e-04
        ], [
            -1.78302260e-02, -7.94090500e-03, -3.10001430e-02, -2.57500750e-02, -3.61797930e-02,
            -2.10474310e-02, -1.89301280e-02, -1.34137070e-02, -2.20717610e-02, -2.12515280e-02,
            -1.94475140e-02, -1.72970040e-02, -1.77349830e-02, -1.33018580e-02, -1.90007990e-02,
            -2.59033410e-02, -7.70022600e-03, -3.28329030e-02, -1.54857170e-02, -6.32032900e-03,
            9.92759865e-01, -2.50493260e-02, -1.46496770e-02, -7.96667100e-03, -9.99188100e-03,
            -8.75079200e-03, -8.75057900e-03, -4.56220200e-03, -3.33457800e-03, -8.53000400e-03,
            -8.31350800e-03, -6.21778600e-03, -1.30762110e-02, -1.13219330e-02, -2.49546000e-04
        ], [
            -1.44417800e-03, -3.15269100e-03, -3.86328600e-03, -4.17612200e-03, -4.64299400e-03,
            -4.19620500e-03, -5.43371800e-03, -1.09390600e-03, -3.99887300e-03, -4.97360700e-03,
            -5.17857600e-03, -4.19573900e-03, -5.26005800e-03, -3.93627700e-03, -2.62639800e-03,
            -3.46277200e-03, -3.52623100e-03, -4.85914000e-03, -5.39746300e-03, -7.59400200e-03,
            -6.79631000e-03, 9.89630379e-01, -7.95348600e-03, -4.45894800e-03, -1.66479720e-02,
            -2.11417410e-02, -9.97474700e-03, -1.05362260e-02, -3.03208000e-03, -1.34803710e-02,
            -1.15871320e-02, -8.65088800e-03, -8.82891600e-03, -1.20543380e-02, -7.08649000e-05
        ], [
            -1.30939330e-02, -1.91585680e-02, -2.40824480e-02, -1.99747620e-02, -2.04309240e-02,
            -2.38266500e-02, -1.88997100e-02, -3.89884710e-02, -1.81417910e-02, -1.69559310e-02,
            -3.36482460e-02, -1.76162780e-02, -1.26329470e-02, -8.03909700e-03, -1.08335380e-02,
            -1.76024900e-02, -1.77922340e-02, -1.87420370e-02, -1.64359060e-02, -2.07393180e-02,
            -1.29385060e-02, -9.38793600e-03, 9.53643203e-01, -1.83667720e-02, -1.03057540e-02,
            -5.55209550e-02, -8.69498100e-03, -4.85622400e-03, -1.82682800e-03, -5.62076200e-03,
            -1.05917610e-02, -9.86981300e-03, -7.38844900e-03, -9.36656500e-03, -3.75522000e-04
        ], [
            -2.83193600e-03, -6.16164800e-03, -4.63601500e-03, -4.52293300e-03, -4.72234500e-03,
            -6.70143100e-03, -3.55520000e-03, -4.02531900e-03, -3.11251800e-03, -3.37456300e-03,
            -8.86635200e-03, -5.72516500e-03, -3.28693500e-03, -2.05487600e-03, -2.30521500e-03,
            -2.47997800e-03, -2.87397800e-03, -3.24491000e-03, -1.84905600e-03, -4.69775000e-03,
            -3.52184000e-03, -1.59787900e-03, -3.41030600e-03, 8.52031266e-01, -2.66940400e-03,
            -5.14349200e-03, -1.00465500e-03, -6.18611000e-04, -1.96923900e-03, -1.31391900e-03,
            -1.21323900e-03, -9.40972000e-04, -5.87853000e-04, -1.27814600e-03, -3.04805000e-05
        ], [
            -7.30591000e-04, -1.46519500e-03, -1.34859600e-03, -1.64284400e-03, -2.36644500e-03,
            -1.72205800e-03, -2.16497300e-03, -6.02314000e-04, -1.66042700e-03, -1.64675400e-03,
            -1.55072800e-03, -1.17937900e-03, -2.01117900e-03, -1.66486600e-03, -1.32683300e-03,
            -1.23025900e-03, -7.16295000e-04, -1.05931200e-03, -1.98424000e-03, -2.77807600e-03,
            -1.59816300e-03, -1.24783200e-03, -2.45236500e-03, -5.74873100e-03, 9.66516804e-01,
            -1.44920360e-02, -3.81425000e-03, -2.86010600e-03, -5.57478000e-04, -3.70005700e-03,
            -4.08998100e-03, -2.58964300e-03, -1.19966300e-03, -4.03853700e-03, -1.60473000e-05
        ], [
            -3.37478800e-03, -4.54322900e-03, -6.71862100e-03, -4.02974700e-03, -4.76440100e-03,
            -7.08258700e-03, -8.99497900e-03, -6.68994000e-03, -4.94354000e-03, -5.29458300e-03,
            -8.36515400e-03, -5.07826200e-03, -4.33340900e-03, -2.95953200e-03, -3.96170800e-03,
            -4.41526400e-03, -3.73740900e-03, -8.72927800e-03, -1.20539680e-02, -2.12867850e-02,
            -1.22453010e-02, -7.71205800e-03, -3.98555240e-02, -1.08508255e-01, -9.19568260e-02,
            8.99821487e-01, -4.47932500e-03, -3.46017800e-03, -1.02346100e-03, -5.19085800e-03,
            -4.88452900e-03, -2.78031100e-03, -3.28720100e-03, -6.21550200e-03, -1.84633000e-05
        ], [
            -3.43404000e-03, -5.24085100e-03, -5.07271400e-03, -6.12626300e-03, -6.61096500e-03,
            -6.00848100e-03, -1.24047760e-02, -3.01514500e-03, -7.63468700e-03, -5.83526300e-03,
            -6.56458300e-03, -6.50021100e-03, -6.79714100e-03, -7.22554000e-03, -4.22200000e-03,
            -9.20250500e-03, -6.56017500e-03, -1.18539210e-02, -1.68463700e-02, -2.00899330e-02,
            -2.18342290e-02, -1.50094060e-02, -1.72298660e-02, -1.76476140e-02, -2.18089440e-02,
            -2.31582570e-02, 8.75814328e-01, -2.87635780e-02, -5.90028600e-03, -2.52048050e-02,
            -2.29283710e-02, -1.10172390e-02, -1.39029880e-02, -2.21182510e-02, -6.27489000e-04
        ], [
            -1.73471310e-02, -2.26303240e-02, -1.45083190e-02, -2.04780150e-02, -1.49888240e-02,
            -1.51629960e-02, -1.98871030e-02, -1.10176240e-02, -1.67338020e-02, -1.45795190e-02,
            -2.05717940e-02, -1.67979950e-02, -1.62465810e-02, -1.22721650e-02, -1.21038280e-02,
            -2.61881870e-02, -2.13324810e-02, -2.07591790e-02, -1.95534360e-02, -3.13400610e-02,
            -3.46593600e-02, -2.34150870e-02, -4.34195790e-02, -2.67248350e-02, -2.98989730e-02,
            -3.13590540e-02, -1.97418060e-02, 7.83889766e-01, -8.35569380e-02, -3.49657260e-02,
            -2.34070480e-02, -1.27676850e-02, -2.20390240e-02, -4.21469290e-02, -5.90765100e-03
        ], [
            -3.49904700e-03, -5.35296600e-03, -4.66014300e-03, -5.05475400e-03, -5.83540200e-03,
            -5.86563300e-03, -1.00706760e-02, -1.73364200e-03, -4.45020600e-03, -5.97112700e-03,
            -4.98846200e-03, -4.82579300e-03, -5.38487200e-03, -4.00943700e-03, -4.63398100e-03,
            -6.64741100e-03, -4.43335600e-03, -1.01945260e-02, -2.26428750e-02, -2.12050730e-02,
            -4.46184450e-02, -3.07552980e-02, -9.31686200e-03, -1.38448260e-02, -1.03793090e-02,
            -2.11765070e-02, -1.67890210e-02, -2.59458860e-02, 9.70527461e-01, -2.55448850e-02,
            -1.37029360e-02, -2.02620070e-02, -3.65810150e-02, -2.77583780e-02, -1.08714000e-06
        ], [
            -1.17657590e-02, -2.98272300e-02, -3.83874580e-02, -2.36314220e-02, -2.08202190e-02,
            -2.54850480e-02, -5.95374400e-02, -1.20549020e-02, -6.39721240e-02, -3.60318480e-02,
            -3.24874340e-02, -2.81076630e-02, -4.04378500e-02, -4.11924040e-02, -3.87422980e-02,
            -2.76554760e-02, -3.16215940e-02, -5.39928430e-02, -5.07884750e-02, -5.67647430e-02,
            -6.12994590e-02, -4.42548100e-02, -4.27485950e-02, -2.69316870e-02, -7.65274580e-02,
            -6.25291080e-02, -6.49183490e-02, -1.03094220e-01, -4.02085300e-02, 8.56240651e-01,
            -8.35122640e-02, -3.53434180e-02, -6.23403110e-02, -7.83900970e-02, -2.78826500e-03
        ], [
            -1.50284300e-03, -2.74461100e-03, -1.86603200e-03, -9.64354000e-04, -7.55469000e-04,
            -3.27083400e-03, -3.29028300e-03, -8.75671000e-04, -2.63327700e-03, -1.87441900e-03,
            -2.49649800e-03, -2.79226500e-03, -1.15807700e-03, -9.29848000e-04, -1.13044500e-03,
            -1.94333600e-03, -4.40282000e-03, -1.54376100e-03, -2.73122300e-03, -1.56776200e-03,
            -2.66917300e-03, -3.69099100e-03, -3.73481400e-03, -4.99296300e-03, -8.94015600e-03,
            -3.21215300e-03, -2.86957700e-03, -1.60389100e-03, -4.07286900e-03, -3.01024000e-03,
            9.92468497e-01, -4.51978300e-03, -4.29902700e-03, -8.69747200e-03, -4.05720000e-06
        ], [
            -7.35356000e-04, -4.10946000e-04, -4.40117000e-04, -3.69504000e-04, -4.45293000e-04,
            -3.53449000e-04, -5.54251000e-04, -3.11603000e-04, -9.13317000e-04, -5.97087000e-04,
            -5.45281000e-04, -5.50824000e-04, -7.59887000e-04, -6.48529000e-04, -7.03221000e-04,
            -4.21326000e-04, -7.12778000e-04, -4.81851000e-04, -7.84288000e-04, -8.44198000e-04,
            -1.04081100e-03, -6.15682000e-04, -1.08246100e-03, -3.17147000e-04, -1.90674800e-03,
            -1.17688500e-03, -1.45969500e-03, -1.23601200e-03, -2.96414000e-04, -2.50537600e-03,
            -4.62074300e-03, 9.86218279e-01, -1.44035400e-03, -1.59984300e-03, -1.20743000e-05
        ], [
            -1.27832700e-03, -4.12609000e-04, -4.79785000e-04, -4.00545000e-04, -9.72120000e-04,
            -4.70736000e-04, -4.60941000e-04, -1.64704000e-04, -5.23372000e-04, -4.03503000e-04,
            -6.09777000e-04, -4.26443000e-04, -7.48771000e-04, -3.82807000e-04, -4.21159000e-04,
            -5.00744000e-04, -3.83126000e-04, -4.02224000e-04, -5.53848000e-04, -5.79074000e-04,
            -4.41520000e-04, -9.86869000e-04, -6.36852000e-04, -3.07647000e-04, -4.14647000e-04,
            -5.22371000e-04, -6.71283000e-04, -5.56354000e-04, -9.28391000e-05, -6.37537000e-04,
            -5.10291300e-03, -1.63061500e-03, 9.65080541e-01, -1.29086600e-03, -3.40349000e-07
        ], [
            -2.86094700e-03, -6.29624800e-03, -5.40670300e-03, -4.56465200e-03, -6.56139600e-03,
            -5.09183000e-03, -1.22856270e-02, -1.97139800e-03, -7.10155700e-03, -5.07267000e-03,
            -5.72406500e-03, -5.40487900e-03, -4.73855100e-03, -3.41151100e-03, -4.14815100e-03,
            -4.61282000e-03, -5.59376300e-03, -6.21852200e-03, -7.52703000e-03, -7.57647200e-03,
            -9.05654400e-03, -1.37627580e-02, -1.19289350e-02, -5.01657100e-03, -7.52232600e-03,
            -1.27619270e-02, -1.93011460e-02, -1.01776140e-02, -8.35531900e-03, -2.52706320e-02,
            -1.73660000e-02, -1.23705270e-02, -1.40767320e-02, 9.21761694e-01, -1.97260300e-03
        ], [
            -1.43478000e-05, -2.44395000e-05, -2.37410000e-05, -7.77550000e-05, -2.57450000e-05,
            -2.88407000e-05, -3.09734000e-05, -1.10871000e-05, -1.95301000e-05, -2.14994000e-05,
            -2.08148000e-05, -1.65598000e-05, -3.07705000e-05, -1.46546000e-05, -7.64555000e-05,
            -1.25023000e-05, -6.27270000e-06, -6.72929000e-05, -4.57973000e-05, -1.13875000e-05,
            -3.80354000e-05, -5.28132000e-05, -1.09889000e-04, -2.57128000e-05, -1.60338000e-05,
            -1.49606000e-05, -3.75788000e-05, -6.07456000e-05, -2.24378000e-04, -6.42811000e-05,
            -6.58038000e-05, -4.05495000e-05, -5.21875000e-05, -1.10644000e-04, 9.99883852e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 377)
    return ret


@cache('run')
def ia_matrix_x2006x():
    """
    ia matrix x2006x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.73646392e-01, -5.78661000e-04, -2.59175245e-01, -4.95743970e-02, -5.55316380e-02,
            -1.27181871e-01, -1.53532420e-02, -1.61342100e-03, -6.44217500e-03, -1.33140010e-02,
            -5.79479000e-04, -2.12050000e-04, -1.94730000e-04, -1.01236000e-04, -1.26664000e-04,
            -1.06274600e-02, -5.40087000e-04, -4.59393200e-03, -8.69679000e-04, -2.49936500e-03,
            -2.25854200e-03, -3.94663770e-02, -1.76078800e-03, -9.77682000e-04, -3.75788000e-04,
            -6.13866800e-03, -2.28343000e-04, -1.33382000e-04, -3.85660000e-04, -8.19952000e-04,
            -2.16222300e-03, -2.70548600e-03, -2.15996100e-03, -2.48901700e-03, -2.50289000e-02
        ], [
            -2.35429900e-03, 9.22973090e-01, -2.80849200e-03, -2.78350600e-03, -2.32096900e-03,
            -4.24980500e-03, -4.82723700e-03, -4.71836678e-01, -3.03588790e-02, -3.71974200e-03,
            -7.00274670e-02, -5.20029840e-02, -2.91153800e-03, -1.12861800e-03, -1.58406400e-03,
            -5.09945100e-03, -1.09191662e-01, -1.79073170e-02, -5.77412000e-04, -5.20574000e-04,
            -1.06199000e-03, -2.06298700e-03, -4.99784700e-03, -3.39516400e-03, -3.14828600e-03,
            -2.85229100e-03, -4.65793000e-04, -2.66682000e-04, -1.61016100e-03, -8.07234000e-04,
            -3.06520700e-03, -1.79645300e-03, -1.51586700e-03, -2.70259300e-03, -6.30789720e-02
        ], [
            -6.33833790e-02, -8.59104000e-04, 8.40702521e-01, -4.96443400e-03, -1.01407016e-01,
            -2.52871600e-03, -3.19557900e-03, -1.42513900e-03, -1.15205030e-02, -3.85197900e-03,
            -2.60618400e-03, -1.84842700e-03, -1.69456900e-03, -1.38088500e-03, -1.11943100e-03,
            -2.78536300e-03, -1.10127500e-03, -1.48681900e-03, -1.67385000e-03, -4.62608500e-03,
            -4.74507100e-03, -1.53086001e-01, -1.80139200e-03, -5.77481400e-03, -4.20067000e-03,
            -1.79016400e-03, -1.43380000e-03, -6.79566000e-04, -2.94701000e-04, -1.86180200e-03,
            -7.42721900e-03, -9.02346200e-03, -1.26853960e-02, -7.43701900e-03, -3.18999100e-03
        ], [
            -1.85618100e-03, -9.88801000e-04, -1.58420700e-03, 6.51323876e-01, -5.84062000e-02,
            -3.40054200e-03, -5.92668200e-03, -4.24688000e-04, -3.53313500e-03, -1.38809350e-02,
            -3.90667100e-03, -1.92345600e-03, -2.31611400e-03, -1.85155900e-03, -4.64148900e-03,
            -2.30087880e-02, -8.24310000e-04, -2.53101600e-03, -1.64429400e-03, -2.42033500e-03,
            -3.30973300e-03, -2.91681200e-03, -1.97563400e-03, -3.07261000e-03, -1.94817000e-03,
            -1.84277000e-03, -7.07605000e-04, -4.25935000e-04, -3.92042000e-04, -1.44346100e-03,
            -2.83894400e-03, -1.03965600e-03, -3.52486600e-03, -4.86862400e-03, -1.00120000e-04
        ], [
            -1.55707000e-04, -5.65087000e-05, -1.31264000e-04, -1.12714190e-02, 7.85526327e-01,
            -4.75366000e-04, -7.32601000e-04, -3.65431000e-05, -2.31047000e-04, -9.32479000e-04,
            -2.45296000e-04, -1.50184000e-04, -1.90717000e-04, -2.80806000e-04, -1.76134400e-03,
            -4.97463800e-03, -8.66439000e-05, -1.09017000e-04, -3.88298000e-04, -3.38559000e-04,
            -2.64887000e-04, -1.30199000e-04, -2.15471000e-04, -1.07699000e-04, -1.28098000e-04,
            -1.52806000e-04, -2.34722000e-04, -3.02590000e-05, -9.10679000e-06, -9.09710000e-05,
            -5.76995000e-04, -9.75224000e-05, -1.24782000e-04, -4.26651000e-04, -1.59646000e-06
        ], [
            -1.73271900e-03, -1.53971700e-03, -1.68589400e-03, -1.42460000e-03, -1.92907400e-03,
            7.47173565e-01, -1.13769950e-02, -3.27804000e-04, -1.47708300e-03, -3.14878800e-03,
            -7.66342400e-03, -3.59363900e-03, -3.01057900e-03, -1.59714700e-03, -2.68053300e-03,
            -7.21306280e-02, -8.67512000e-04, -3.10570580e-02, -1.20833700e-03, -1.35510000e-03,
            -1.23268500e-03, -2.26983800e-03, -1.23807400e-03, -6.14261000e-04, -3.92538000e-04,
            -2.46283000e-03, -5.98313000e-04, -3.03450000e-04, -2.66669800e-03, -8.95028000e-04,
            -1.90757800e-03, -2.70894900e-03, -1.35978100e-03, -2.67528700e-03, -2.93231000e-05
        ], [
            -2.44537800e-03, -2.77182700e-03, -2.43901710e-02, -1.16725740e-02, -1.48943880e-02,
            -1.39479680e-02, 7.54337124e-01, -1.53601400e-03, -1.50101310e-02, -1.63603680e-02,
            -2.10762780e-02, -4.96863600e-03, -6.99696900e-03, -8.44742000e-03, -4.15189500e-03,
            -1.88352280e-02, -3.15051100e-03, -4.19833100e-03, -8.13985500e-03, -1.12920640e-02,
            -1.14875430e-02, -8.29640500e-03, -6.37667600e-03, -3.33291900e-03, -5.09992600e-03,
            -1.33578560e-02, -9.69479000e-03, -1.23002740e-02, -2.14753400e-03, -1.99028360e-02,
            -1.83656820e-02, -1.82563000e-02, -8.98407500e-03, -2.34503490e-02, -1.71420000e-04
        ], [
            -1.22966890e-02, -6.19751400e-03, -3.18973600e-03, -5.24662400e-03, -4.10644400e-03,
            -4.72870400e-03, -4.42581600e-03, 8.89537770e-01, -4.63487200e-02, -8.88800200e-03,
            -1.90292490e-02, -1.24731510e-02, -3.61508700e-03, -1.89258200e-03, -1.84719000e-03,
            -3.78790100e-03, -2.44489840e-02, -1.10625650e-02, -4.65650300e-03, -5.25118800e-03,
            -3.51007200e-03, -3.51833200e-03, -5.30297940e-02, -7.07011640e-02, -9.19706870e-02,
            -2.01308340e-02, -3.16980700e-03, -1.12625400e-03, -1.09738700e-03, -4.45548200e-03,
            -8.23397800e-03, -3.07902500e-03, -2.78392000e-03, -5.07863200e-03, -4.14333000e-05
        ], [
            -3.60253300e-02, -9.27348700e-03, -9.18874100e-03, -6.71882620e-02, -3.73160620e-02,
            -3.35657200e-02, -3.43238520e-02, -1.66777580e-02, 7.54820444e-01, -2.15229144e-01,
            -3.65473730e-02, -1.50059200e-02, -1.02181150e-02, -1.71763450e-02, -1.06072780e-02,
            -2.56625630e-02, -6.44150400e-03, -1.06374800e-02, -5.50137400e-03, -2.67382600e-03,
            -2.11434700e-03, -4.45096900e-03, -3.47196700e-03, -4.76737900e-03, -4.10387700e-03,
            -4.35696900e-03, -1.23356900e-03, -6.46906000e-04, -1.87237600e-03, -4.67927000e-03,
            -9.91215100e-03, -3.74077300e-03, -6.21861740e-02, -9.77193700e-03, -1.97471000e-04
        ], [
            -4.64423500e-03, -5.48007800e-03, -1.85361920e-02, -1.08648140e-02, -3.87099750e-02,
            -8.56205000e-03, -1.62979880e-02, -3.06799900e-03, -1.75425980e-02, 8.52569750e-01,
            -1.14481230e-02, -6.91939000e-03, -2.36212310e-02, -2.30094030e-02, -3.40191380e-02,
            -3.29576200e-02, -2.38333900e-03, -1.86568200e-02, -1.57057080e-02, -4.33561100e-03,
            -5.00357100e-03, -4.14310100e-03, -1.07808790e-02, -2.97790500e-03, -3.68196500e-03,
            -6.33803900e-03, -3.17575300e-03, -5.88139000e-04, -1.34618700e-03, -2.83659000e-03,
            -3.33880000e-03, -1.31373000e-03, -5.31280100e-03, -4.58211700e-03, -6.24787000e-05
        ], [
            -1.57033000e-03, -4.40443200e-03, -5.25770200e-03, -2.11421100e-03, -1.60913400e-03,
            -7.33382000e-03, -1.22566700e-03, -1.66287100e-03, -5.54292700e-03, -5.52242200e-03,
            8.63103446e-01, -1.01264910e-02, -4.96587800e-03, -9.64662100e-03, -6.52125900e-03,
            -6.84673200e-03, -1.76150900e-03, -8.96454080e-02, -3.46350000e-03, -9.89584000e-04,
            -9.93221000e-04, -3.20720400e-03, -1.43017100e-03, -4.67887000e-04, -6.29122000e-04,
            -1.44819600e-03, -1.10785300e-03, -1.16744000e-04, -1.74404600e-03, -9.73874000e-04,
            -1.49653000e-03, -1.63641000e-03, -1.93872600e-03, -1.85037200e-03, -2.93181000e-05
        ], [
            -4.29880500e-03, -1.85326250e-02, -1.10378260e-02, -5.19694100e-03, -1.00017690e-02,
            -2.21054400e-02, -8.68889400e-03, -5.44955300e-03, -1.26055920e-02, -2.33703940e-02,
            -3.29890080e-02, 6.70935499e-01, -1.56950122e-01, -6.46402280e-02, -8.83797280e-02,
            -7.57037190e-02, -6.67943700e-03, -8.81291260e-02, -1.48087760e-02, -3.27352200e-03,
            -2.92148800e-03, -3.97887400e-03, -6.16625900e-03, -4.62109900e-03, -3.77231900e-03,
            -7.94078200e-03, -3.20752900e-03, -5.26760000e-04, -2.49509100e-03, -3.56781300e-03,
            -4.28648300e-03, -1.83800600e-03, -2.17525600e-03, -4.45667700e-03, -1.54051000e-04
        ], [
            -7.34264600e-03, -1.51933460e-02, -4.10638900e-03, -9.87837400e-03, -7.65059900e-03,
            -1.01561540e-02, -7.53240300e-03, -6.94062200e-03, -8.22495800e-03, -1.34893680e-02,
            -2.16697710e-02, -2.38729460e-02, 8.39923845e-01, -1.75441150e-02, -3.88519030e-02,
            -1.37138390e-02, -8.69580100e-03, -1.91724330e-02, -2.36748460e-02, -2.89761800e-03,
            -2.68014400e-03, -2.39438400e-03, -5.92374500e-03, -1.01558500e-02, -8.02596300e-03,
            -8.00966600e-03, -2.86766300e-03, -8.81810000e-04, -1.67340100e-03, -2.86899400e-03,
            -6.43931600e-03, -2.22901300e-03, -3.95957100e-03, -4.05124000e-03, -3.48229000e-04
        ], [
            -2.05310200e-03, -7.36156300e-03, -3.97327700e-03, -7.02759600e-03, -5.41742600e-03,
            -8.14768300e-03, -1.07994070e-02, -3.69302500e-03, -1.07907540e-02, -1.47533590e-02,
            -1.23935130e-02, -1.86877950e-02, -1.06936267e-01, 6.27283316e-01, -6.44719400e-02,
            -3.09045040e-02, -2.50800970e-02, -3.97678090e-02, -3.26290100e-02, -8.86479700e-03,
            -8.92372200e-03, -5.50974400e-03, -8.35208800e-03, -4.02328200e-03, -5.73490300e-03,
            -1.08797780e-02, -4.33723020e-02, -4.02329300e-03, -2.66761900e-03, -1.83651800e-02,
            -1.78942740e-02, -1.11887720e-02, -2.56985060e-02, -1.37958600e-02, -1.11331700e-03
        ], [
            -4.81975200e-03, -5.13424300e-03, -2.01833400e-03, -2.89466000e-03, -2.65222500e-03,
            -4.04398600e-03, -3.36527600e-03, -1.14951200e-03, -2.50715300e-03, -4.81600600e-03,
            -5.93484400e-03, -7.05070600e-03, -1.62724460e-02, -4.62688700e-03, 6.86491500e-01,
            -7.18769100e-03, -3.18613100e-03, -5.57383400e-03, -8.08245930e-02, -4.35952800e-03,
            -4.48219900e-03, -1.94638500e-03, -3.83890810e-02, -3.88115540e-02, -5.31724400e-02,
            -1.64184110e-02, -3.46922700e-03, -1.05176400e-03, -1.03197100e-03, -4.22553900e-03,
            -1.71531640e-02, -2.37326300e-03, -1.57340700e-03, -7.94159000e-03, -2.57602000e-04
        ], [
            -6.31183000e-04, -8.59341000e-04, -9.52911000e-04, -4.40474100e-03, -2.65629000e-03,
            -3.91535100e-03, -4.14845700e-03, -5.63763000e-04, -1.67816500e-03, -2.85084700e-03,
            -3.17582000e-03, -9.57033400e-03, -3.64435400e-03, -2.25014800e-03, -4.77563000e-03,
            9.53683887e-01, -1.37846100e-03, -4.94419100e-03, -1.80850000e-03, -1.12912800e-03,
            -1.41367400e-03, -2.25629000e-03, -2.51077800e-03, -7.26874000e-04, -1.80856000e-03,
            -1.53827500e-03, -9.96998000e-04, -7.13683000e-04, -1.41411600e-03, -1.62598800e-03,
            -3.65949800e-03, -3.44268300e-03, -2.96201300e-03, -5.10636500e-03, -1.79764400e-03
        ], [
            -1.09963930e-02, -2.89206770e-02, -1.34300820e-02, -1.72941830e-02, -8.39498800e-03,
            -1.67959260e-02, -2.29692610e-02, -2.93571970e-02, -2.53822190e-02, -2.01155000e-02,
            -4.22977900e-02, -2.88196960e-02, -1.11757070e-02, -7.07891500e-03, -7.95619800e-03,
            -9.26646400e-03, 8.68931936e-01, -6.61278200e-03, -1.11939870e-02, -6.18844000e-03,
            -1.30820270e-02, -2.31793740e-02, -1.56063560e-02, -3.44779500e-03, -5.74421300e-03,
            -1.63784670e-02, -9.39093200e-03, -3.75251300e-03, -9.67181300e-03, -5.52646400e-03,
            -1.41533570e-02, -2.06614750e-02, -1.21669000e-02, -1.57323680e-02, -1.97298000e-04
        ], [
            -3.22401700e-03, -6.58430000e-03, -2.11131000e-03, -2.25504600e-03, -1.43721000e-03,
            -2.90886700e-03, -2.99230600e-03, -5.05980700e-03, -3.02203900e-03, -2.48871800e-03,
            -5.96204500e-03, -3.68772000e-03, -2.63285000e-03, -1.60386900e-03, -1.65841300e-03,
            -2.49462100e-03, -1.60965340e-02, 9.49535989e-01, -5.28668000e-03, -3.48044000e-03,
            -4.31968900e-03, -5.45652400e-03, -8.81614400e-03, -2.37136700e-03, -2.74317700e-03,
            -1.22634420e-02, -1.00914410e-02, -4.36378600e-03, -3.24284370e-02, -4.39533900e-03,
            -1.77935490e-02, -1.15943430e-02, -6.63999000e-03, -8.73411300e-03, -6.40140000e-04
        ], [
            -5.10972300e-03, -3.06561900e-03, -6.65234200e-03, -4.66277600e-03, -6.05769200e-03,
            -5.92349000e-03, -4.89146500e-03, -2.49103800e-03, -5.57752700e-03, -5.99639000e-03,
            -6.59903900e-03, -5.52532500e-03, -4.54793700e-03, -3.04162900e-03, -4.73404400e-03,
            -6.27005800e-03, -7.48764800e-03, -7.13900500e-03, 9.88340268e-01, -4.41706700e-03,
            -4.13687700e-03, -5.22445000e-03, -1.56178940e-02, -2.07731900e-03, -5.11958600e-03,
            -7.92507300e-03, -3.71880900e-03, -2.78302100e-03, -2.05219200e-03, -5.48519500e-03,
            -4.53493700e-03, -3.08693100e-03, -4.03623400e-03, -6.33189100e-03, -4.30297000e-05
        ], [
            -3.13614350e-02, -1.71113920e-02, -6.06459710e-02, -4.10440260e-02, -4.95452990e-02,
            -5.06661320e-02, -4.67958720e-02, -5.62869800e-02, -4.43950240e-02, -4.46186120e-02,
            -3.72182480e-02, -4.11935810e-02, -4.13911330e-02, -3.11237260e-02, -4.13708890e-02,
            -4.14886180e-02, -2.14146740e-02, -3.96023970e-02, -3.33618230e-02, 9.75168005e-01,
            -1.25859010e-02, -4.55410270e-02, -2.24999150e-02, -1.79674850e-02, -2.07400130e-02,
            -1.98027080e-02, -1.07734320e-02, -3.74018400e-03, -4.39955200e-03, -1.24747670e-02,
            -1.99856270e-02, -1.13230970e-02, -2.64975660e-02, -1.65111910e-02, -3.08406000e-04
        ], [
            -1.74896450e-02, -7.31440400e-03, -3.01579950e-02, -2.35098100e-02, -3.25145270e-02,
            -2.02346770e-02, -1.87849860e-02, -1.32111130e-02, -2.15437710e-02, -2.05277950e-02,
            -1.89375720e-02, -1.65318580e-02, -1.70603490e-02, -1.24192540e-02, -1.85413200e-02,
            -2.61877820e-02, -7.59731800e-03, -3.24887090e-02, -1.46983240e-02, -6.18192200e-03,
            9.92926206e-01, -2.47923850e-02, -1.43426930e-02, -7.81500900e-03, -9.71827600e-03,
            -8.88796500e-03, -8.51243400e-03, -4.39304800e-03, -3.43017100e-03, -8.19964300e-03,
            -8.27376400e-03, -6.30985300e-03, -1.32903860e-02, -1.13605620e-02, -4.34081000e-04
        ], [
            -1.48156200e-03, -3.25654900e-03, -3.95929000e-03, -3.98825700e-03, -4.60560600e-03,
            -4.16476200e-03, -5.45450900e-03, -1.03559700e-03, -4.01924600e-03, -4.92157100e-03,
            -5.30914000e-03, -4.26803400e-03, -5.16925200e-03, -3.71863000e-03, -2.58534000e-03,
            -3.49540100e-03, -3.25502400e-03, -5.04312500e-03, -5.23087700e-03, -7.58799000e-03,
            -6.66465700e-03, 9.89578257e-01, -7.72527600e-03, -4.26455000e-03, -1.61712680e-02,
            -2.09413490e-02, -9.59444800e-03, -1.01821180e-02, -2.88440900e-03, -1.33225080e-02,
            -1.14015960e-02, -9.15896300e-03, -8.97434100e-03, -1.22716160e-02, -6.50288000e-05
        ], [
            -1.32579140e-02, -1.84951140e-02, -2.36358360e-02, -1.97076400e-02, -1.92947010e-02,
            -2.36026990e-02, -1.87571350e-02, -4.36833020e-02, -1.83022480e-02, -1.68773160e-02,
            -3.42043620e-02, -1.72853080e-02, -1.24217170e-02, -7.59709600e-03, -1.07512470e-02,
            -1.90643810e-02, -1.93372810e-02, -1.93487530e-02, -1.71228330e-02, -2.10260120e-02,
            -1.28826470e-02, -9.43315200e-03, 9.53946883e-01, -1.27433270e-02, -9.63078000e-03,
            -5.61040460e-02, -8.64046300e-03, -4.76633400e-03, -1.90288800e-03, -5.47094000e-03,
            -1.02696020e-02, -9.67911500e-03, -7.34148300e-03, -9.30827800e-03, -6.12149000e-04
        ], [
            -2.58898900e-03, -5.99110100e-03, -4.83744500e-03, -4.33091600e-03, -4.27422500e-03,
            -6.56285300e-03, -3.39136300e-03, -3.02330700e-03, -3.05992500e-03, -3.25294300e-03,
            -8.26994700e-03, -5.50013300e-03, -3.00664800e-03, -1.83173100e-03, -2.24579400e-03,
            -2.67301900e-03, -2.66323700e-03, -3.00983100e-03, -1.96252400e-03, -4.79800500e-03,
            -3.56761200e-03, -1.57604100e-03, -3.55971100e-03, 8.46927701e-01, -2.65915500e-03,
            -5.01413600e-03, -1.00867400e-03, -5.79947000e-04, -2.16247000e-03, -1.29822400e-03,
            -1.12634300e-03, -8.87266000e-04, -5.75120000e-04, -1.26097900e-03, -4.36263000e-05
        ], [
            -7.80924000e-04, -1.37348300e-03, -1.38005600e-03, -1.58168700e-03, -2.32051600e-03,
            -1.76346300e-03, -2.21801800e-03, -5.26607000e-04, -1.61800800e-03, -1.62599000e-03,
            -1.60279200e-03, -1.13123200e-03, -2.00815600e-03, -1.60973000e-03, -1.32581900e-03,
            -1.32450200e-03, -7.14399000e-04, -1.09264300e-03, -2.05571800e-03, -2.87421800e-03,
            -1.63687700e-03, -1.29486100e-03, -2.56129100e-03, -5.59651500e-03, 9.68826709e-01,
            -1.38790020e-02, -3.77859500e-03, -2.78904000e-03, -5.34050000e-04, -3.73771300e-03,
            -4.07898700e-03, -2.86115600e-03, -1.21917400e-03, -4.18205100e-03, -2.10266000e-05
        ], [
            -3.45855500e-03, -4.44868000e-03, -6.74955700e-03, -3.83754300e-03, -4.56436700e-03,
            -7.12157300e-03, -9.48839200e-03, -6.86092700e-03, -4.85866600e-03, -5.20839700e-03,
            -8.30031700e-03, -4.97147900e-03, -4.21041500e-03, -2.79868600e-03, -3.94777300e-03,
            -4.30750200e-03, -3.86753800e-03, -9.46554700e-03, -1.31540110e-02, -2.16033110e-02,
            -1.22486060e-02, -7.61269600e-03, -3.95457300e-02, -1.11721654e-01, -9.17322560e-02,
            8.99194731e-01, -4.32683800e-03, -3.26032700e-03, -1.06901500e-03, -5.16407700e-03,
            -4.70121200e-03, -2.79481800e-03, -3.27294600e-03, -6.31929700e-03, -3.45637000e-05
        ], [
            -3.72836600e-03, -4.92957400e-03, -5.01569400e-03, -6.02011000e-03, -6.17690900e-03,
            -5.99860800e-03, -1.27219320e-02, -3.15401500e-03, -7.50296100e-03, -5.57661900e-03,
            -6.47240000e-03, -6.67213200e-03, -6.37032700e-03, -6.92366700e-03, -4.08226400e-03,
            -1.04434410e-02, -6.81377400e-03, -1.16793250e-02, -1.70541470e-02, -2.02201270e-02,
            -2.16898350e-02, -1.51474900e-02, -1.75669900e-02, -1.86687090e-02, -2.15214400e-02,
            -2.30119050e-02, 8.79054202e-01, -2.80460040e-02, -5.69252300e-03, -2.49688570e-02,
            -2.29964100e-02, -1.13841710e-02, -1.39393440e-02, -2.20765090e-02, -7.34218000e-04
        ], [
            -1.74514720e-02, -2.37936600e-02, -1.50450450e-02, -2.12857400e-02, -1.56438920e-02,
            -1.58458780e-02, -2.12799980e-02, -1.17382920e-02, -1.70255330e-02, -1.47898250e-02,
            -2.19754250e-02, -1.72279320e-02, -1.61417320e-02, -1.21004090e-02, -1.25960420e-02,
            -2.68520160e-02, -2.24331580e-02, -2.13100010e-02, -1.96871060e-02, -3.15047010e-02,
            -3.50920520e-02, -2.40748960e-02, -4.54894050e-02, -2.50209960e-02, -3.12988950e-02,
            -3.20151680e-02, -1.99252310e-02, 7.81798313e-01, -8.28821660e-02, -3.61923090e-02,
            -2.32107180e-02, -1.39027090e-02, -2.33309540e-02, -4.44536460e-02, -6.56911500e-03
        ], [
            -2.83314500e-03, -5.25314700e-03, -4.55405100e-03, -4.89390600e-03, -6.10389900e-03,
            -5.73899900e-03, -1.00243560e-02, -1.75278200e-03, -4.22565900e-03, -5.79981100e-03,
            -4.92091800e-03, -4.49736400e-03, -5.14329000e-03, -3.65578700e-03, -4.60502600e-03,
            -6.72269300e-03, -4.43092200e-03, -1.03880280e-02, -2.23096520e-02, -2.19519530e-02,
            -4.23191480e-02, -3.00677540e-02, -9.10397900e-03, -1.37462920e-02, -9.89001200e-03,
            -2.10676410e-02, -1.52831390e-02, -2.38499480e-02, 9.69003338e-01, -2.45867980e-02,
            -1.38359000e-02, -1.98924380e-02, -3.61136630e-02, -2.71952190e-02, -1.15790000e-06
        ], [
            -1.21931350e-02, -2.90706640e-02, -3.76370610e-02, -2.20835200e-02, -1.97452690e-02,
            -2.54268650e-02, -6.09193310e-02, -1.29689500e-02, -6.00784860e-02, -3.50610710e-02,
            -3.18549430e-02, -2.68590280e-02, -3.86718820e-02, -3.82416120e-02, -3.72311410e-02,
            -2.72312230e-02, -3.07469830e-02, -5.35214720e-02, -5.07106860e-02, -5.85233450e-02,
            -6.18631480e-02, -4.51613920e-02, -4.27714480e-02, -2.52294050e-02, -7.74088780e-02,
            -6.18873680e-02, -6.35951670e-02, -1.00316901e-01, -3.93959310e-02, 8.52462341e-01,
            -8.59339560e-02, -3.62495660e-02, -6.38461830e-02, -7.98915630e-02, -2.85261500e-03
        ], [
            -1.38965900e-03, -2.46882400e-03, -1.76863300e-03, -8.34706000e-04, -7.17963000e-04,
            -3.10096400e-03, -3.30241500e-03, -1.02571900e-03, -2.48914300e-03, -1.84768200e-03,
            -2.55243000e-03, -2.65718100e-03, -1.14982700e-03, -8.40598000e-04, -1.08788700e-03,
            -2.02233000e-03, -4.43847400e-03, -1.52981900e-03, -2.67413000e-03, -1.62686200e-03,
            -2.75140700e-03, -3.64830700e-03, -3.64102700e-03, -5.18259200e-03, -9.22393900e-03,
            -3.18728400e-03, -2.66618600e-03, -1.50929700e-03, -3.94605600e-03, -2.95558900e-03,
            9.92659688e-01, -4.54507500e-03, -4.25010500e-03, -8.59860900e-03, -1.16376000e-05
        ], [
            -5.23124000e-04, -4.05677000e-04, -4.23461000e-04, -3.27745000e-04, -3.97983000e-04,
            -3.45694000e-04, -5.56251000e-04, -3.49300000e-04, -8.49717000e-04, -5.64303000e-04,
            -5.26547000e-04, -5.07992000e-04, -7.43153000e-04, -6.00637000e-04, -7.28553000e-04,
            -4.04849000e-04, -6.58764000e-04, -4.83712000e-04, -8.03955000e-04, -7.58210000e-04,
            -1.29003500e-03, -6.01973000e-04, -1.01923800e-03, -3.12864000e-04, -1.98647500e-03,
            -1.14902400e-03, -1.45771100e-03, -1.20092200e-03, -2.95953000e-04, -2.25999900e-03,
            -4.25945200e-03, 9.85893207e-01, -1.44036200e-03, -1.60218400e-03, -8.02746000e-06
        ], [
            -1.26252000e-03, -4.44055000e-04, -4.64720000e-04, -3.85805000e-04, -9.66525000e-04,
            -5.08451000e-04, -4.83276000e-04, -1.60569000e-04, -5.20592000e-04, -3.90897000e-04,
            -7.03365000e-04, -4.78640000e-04, -8.36386000e-04, -3.95394000e-04, -4.48690000e-04,
            -4.80971000e-04, -3.91574000e-04, -3.91191000e-04, -5.25232000e-04, -5.76669000e-04,
            -4.13770000e-04, -9.69334000e-04, -6.00947000e-04, -3.19742000e-04, -4.14631000e-04,
            -4.95014000e-04, -6.19392000e-04, -5.56063000e-04, -9.32945000e-05, -5.94428000e-04,
            -5.32303500e-03, -1.64284600e-03, 9.65771780e-01, -1.32090400e-03, -6.75229000e-07
        ], [
            -2.93223200e-03, -6.21750700e-03, -5.22962800e-03, -4.33966700e-03, -6.20424600e-03,
            -4.99114000e-03, -1.24472150e-02, -1.88962200e-03, -6.87835200e-03, -4.85995700e-03,
            -5.63729300e-03, -5.20632600e-03, -4.46728700e-03, -3.04982500e-03, -4.05129400e-03,
            -4.43593300e-03, -5.53501900e-03, -6.09961700e-03, -7.26622600e-03, -7.52213700e-03,
            -9.01960800e-03, -1.38604930e-02, -1.24717960e-02, -4.71810800e-03, -7.74069900e-03,
            -1.28955440e-02, -1.74379730e-02, -9.89173800e-03, -8.36419800e-03, -2.45666860e-02,
            -1.74038660e-02, -1.24895310e-02, -1.40263140e-02, 9.21077485e-01, -1.86571000e-03
        ], [
            -1.39759000e-05, -2.52242000e-05, -2.32951000e-05, -8.84825000e-05, -2.75316000e-05,
            -3.06553000e-05, -3.17017000e-05, -1.02875000e-05, -1.83767000e-05, -2.11395000e-05,
            -2.14217000e-05, -1.54637000e-05, -3.06786000e-05, -1.34160000e-05, -4.76845000e-05,
            -1.32694000e-05, -6.86315000e-06, -6.39855000e-05, -3.79468000e-05, -1.21205000e-05,
            -3.77271000e-05, -5.20396000e-05, -1.12481000e-04, -5.17696000e-06, -1.29305000e-05,
            -1.21912000e-05, -3.39461000e-05, -5.82010000e-05, -2.22058000e-04, -6.16742000e-05,
            -6.59491000e-05, -4.01160000e-05, -5.41520000e-05, -1.15509000e-04, 9.99878026e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 378)
    return ret


@cache('run')
def ia_matrix_x2007x():
    """
    ia matrix x2007x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.80942664e-01, -5.58434000e-04, -2.55589334e-01, -4.92304180e-02, -5.49396990e-02,
            -1.20320652e-01, -1.59753830e-02, -1.88997400e-03, -6.55901000e-03, -1.23076220e-02,
            -5.53514000e-04, -1.93687000e-04, -1.72585000e-04, -8.74380000e-05, -1.14068000e-04,
            -1.04623170e-02, -4.99416000e-04, -3.30531700e-03, -8.96418000e-04, -2.29702900e-03,
            -2.28344500e-03, -3.84872530e-02, -1.71526600e-03, -8.64287000e-04, -3.61627000e-04,
            -6.50908500e-03, -2.06132000e-04, -1.47712000e-04, -3.77055000e-04, -8.30846000e-04,
            -2.10689600e-03, -2.63460000e-03, -2.10945700e-03, -2.47604200e-03, -2.86601580e-02
        ], [
            -2.19520400e-03, 9.15781723e-01, -2.78563000e-03, -2.94615500e-03, -2.38647800e-03,
            -4.16586600e-03, -4.70893700e-03, -4.43794268e-01, -2.96214790e-02, -3.67807100e-03,
            -6.88066160e-02, -5.02072430e-02, -2.53300400e-03, -1.01534000e-03, -1.48454800e-03,
            -5.08881500e-03, -1.06609079e-01, -1.87225030e-02, -5.52766000e-04, -5.02553000e-04,
            -1.02061200e-03, -2.05272500e-03, -4.92319600e-03, -3.15648500e-03, -3.60361400e-03,
            -2.91077900e-03, -4.40509000e-04, -2.58582000e-04, -1.44268000e-03, -7.58565000e-04,
            -3.16882100e-03, -1.54411400e-03, -1.58781100e-03, -2.68271400e-03, -6.34688520e-02
        ], [
            -6.70259970e-02, -9.34072000e-04, 8.35976229e-01, -5.46756300e-03, -1.04246785e-01,
            -2.84369800e-03, -3.47172700e-03, -1.79039400e-03, -1.23537210e-02, -4.05743700e-03,
            -2.82306800e-03, -1.99329000e-03, -1.78618200e-03, -1.43820100e-03, -1.18680500e-03,
            -2.92117600e-03, -1.17666000e-03, -1.60715900e-03, -1.67420500e-03, -4.70256900e-03,
            -4.74087000e-03, -1.54833857e-01, -1.84923900e-03, -5.84260800e-03, -4.43814300e-03,
            -1.81413600e-03, -1.40711000e-03, -7.15459000e-04, -3.08740000e-04, -1.91057000e-03,
            -7.40253700e-03, -9.01568000e-03, -1.26527090e-02, -7.31649900e-03, -2.84902900e-03
        ], [
            -1.76900800e-03, -1.09515300e-03, -1.64905500e-03, 6.38283714e-01, -6.02458060e-02,
            -3.16077900e-03, -6.38849600e-03, -5.18073000e-04, -3.41593900e-03, -1.45862590e-02,
            -4.04188300e-03, -1.94488200e-03, -2.30086600e-03, -1.79035600e-03, -4.23098800e-03,
            -2.36801580e-02, -9.53238000e-04, -2.43463300e-03, -1.61574900e-03, -2.54947200e-03,
            -3.01560800e-03, -2.84316300e-03, -2.03927800e-03, -3.10340800e-03, -2.12225500e-03,
            -1.95407500e-03, -6.67888000e-04, -4.50210000e-04, -4.01018000e-04, -1.50177500e-03,
            -2.89190100e-03, -1.10812900e-03, -3.66578500e-03, -4.50057600e-03, -7.05294000e-05
        ], [
            -1.54658000e-04, -5.82186000e-05, -1.30786000e-04, -1.23235810e-02, 7.79528919e-01,
            -4.79752000e-04, -6.64946000e-04, -4.10119000e-05, -2.41362000e-04, -9.27583000e-04,
            -2.48218000e-04, -1.44156000e-04, -1.84082000e-04, -2.84860000e-04, -1.89201400e-03,
            -5.36751800e-03, -8.63831000e-05, -1.05979000e-04, -3.71955000e-04, -3.22270000e-04,
            -2.69953000e-04, -1.23719000e-04, -2.44854000e-04, -9.54716000e-05, -1.37984000e-04,
            -1.64010000e-04, -2.32944000e-04, -3.35435000e-05, -9.72326000e-06, -9.41628000e-05,
            -7.32372000e-04, -1.07599000e-04, -2.39632000e-04, -4.39559000e-04, -1.28632000e-06
        ], [
            -1.67238400e-03, -1.66849200e-03, -1.75261800e-03, -1.54247800e-03, -2.01875100e-03,
            7.37317107e-01, -1.23406900e-02, -3.66759000e-04, -1.53356900e-03, -3.13758500e-03,
            -8.27143000e-03, -3.78922300e-03, -3.07982500e-03, -1.61355300e-03, -2.67203000e-03,
            -7.53429430e-02, -8.12325000e-04, -3.02243730e-02, -1.30934400e-03, -1.30004600e-03,
            -1.23457100e-03, -2.30204400e-03, -1.25869500e-03, -6.14343000e-04, -4.00009000e-04,
            -2.29382200e-03, -5.53077000e-04, -3.02388000e-04, -2.25207800e-03, -8.90219000e-04,
            -2.07184700e-03, -2.91571600e-03, -1.45285600e-03, -2.67712100e-03, -2.28600000e-05
        ], [
            -2.40506200e-03, -2.94455100e-03, -2.46480430e-02, -1.20444000e-02, -1.51248240e-02,
            -1.44412150e-02, 7.49784127e-01, -1.80932600e-03, -1.48529350e-02, -1.59970560e-02,
            -2.16654260e-02, -5.06836100e-03, -6.66009300e-03, -8.10565800e-03, -4.09149300e-03,
            -1.93964600e-02, -3.11992500e-03, -4.20378500e-03, -8.31538600e-03, -1.12109540e-02,
            -1.09297310e-02, -8.09895900e-03, -6.54625100e-03, -3.32695900e-03, -4.65787300e-03,
            -1.33159760e-02, -9.41505200e-03, -1.26944260e-02, -2.04741900e-03, -1.99830000e-02,
            -1.81174610e-02, -1.87612660e-02, -9.30171000e-03, -2.35381080e-02, -1.49384000e-04
        ], [
            -1.19962860e-02, -6.27247600e-03, -3.09441000e-03, -5.01580400e-03, -4.07362300e-03,
            -4.43757600e-03, -4.27055000e-03, 9.01909306e-01, -4.27561200e-02, -8.52248000e-03,
            -1.85180990e-02, -1.22491030e-02, -3.47378600e-03, -1.77519200e-03, -1.73458400e-03,
            -3.75568000e-03, -2.66105780e-02, -1.02496720e-02, -5.04938600e-03, -4.52589400e-03,
            -3.48951500e-03, -3.36880100e-03, -5.39605190e-02, -5.83610640e-02, -9.18475580e-02,
            -2.09697070e-02, -3.22088600e-03, -1.16503400e-03, -1.06131600e-03, -4.42669700e-03,
            -8.20465900e-03, -2.93332100e-03, -2.67865100e-03, -4.82234000e-03, -2.80191000e-05
        ], [
            -3.75777040e-02, -1.04665140e-02, -9.46161900e-03, -6.73171090e-02, -3.83275490e-02,
            -3.55316110e-02, -3.60570250e-02, -1.85436120e-02, 7.48776996e-01, -2.21470999e-01,
            -3.85453790e-02, -1.55135450e-02, -1.02674340e-02, -1.76978610e-02, -1.06062900e-02,
            -2.67041110e-02, -6.82996900e-03, -1.09001910e-02, -5.76006200e-03, -2.78443000e-03,
            -2.19031600e-03, -4.67709000e-03, -3.39193000e-03, -4.54346900e-03, -3.83040300e-03,
            -4.50502400e-03, -1.22271000e-03, -6.92513000e-04, -2.08560800e-03, -4.85103800e-03,
            -9.66906500e-03, -3.97742900e-03, -6.40789310e-02, -1.04778620e-02, -1.60148000e-04
        ], [
            -4.67057200e-03, -5.86223200e-03, -1.92160500e-02, -1.11090470e-02, -3.93827040e-02,
            -8.35460800e-03, -1.69486070e-02, -3.23050200e-03, -1.76162360e-02, 8.47841302e-01,
            -1.21130350e-02, -6.91298500e-03, -2.29783180e-02, -2.29514020e-02, -3.33112500e-02,
            -3.34190470e-02, -2.44171600e-03, -1.86194300e-02, -1.56148110e-02, -4.34144800e-03,
            -4.91381500e-03, -4.20332300e-03, -1.13735990e-02, -2.77207700e-03, -3.86955200e-03,
            -6.49045600e-03, -3.07742700e-03, -6.24394000e-04, -1.30480000e-03, -2.86059700e-03,
            -3.31687000e-03, -1.46432700e-03, -5.19201800e-03, -4.58696200e-03, -5.35151000e-05
        ], [
            -1.51471800e-03, -4.50340100e-03, -5.34490700e-03, -2.12310600e-03, -1.65629800e-03,
            -7.12593400e-03, -1.25580200e-03, -2.06748300e-03, -5.63304300e-03, -5.42794500e-03,
            8.56049479e-01, -1.05972010e-02, -4.96602100e-03, -9.79072300e-03, -6.35551300e-03,
            -7.30405700e-03, -1.79479500e-03, -9.41882020e-02, -3.60186800e-03, -9.85001000e-04,
            -1.01819100e-03, -3.29145700e-03, -1.47393600e-03, -4.35769000e-04, -6.65638000e-04,
            -1.47678000e-03, -1.08739900e-03, -1.18165000e-04, -1.71914000e-03, -9.82791000e-04,
            -1.62516200e-03, -1.70423100e-03, -1.93927500e-03, -1.88708800e-03, -2.58891000e-05
        ], [
            -4.37392400e-03, -1.92756320e-02, -1.03382810e-02, -5.09987400e-03, -9.71703500e-03,
            -2.18241640e-02, -9.08484200e-03, -5.92592400e-03, -1.26982040e-02, -2.31846130e-02,
            -3.45272100e-02, 6.65902468e-01, -1.57428779e-01, -6.40438170e-02, -8.52319840e-02,
            -7.63341560e-02, -6.97673800e-03, -8.92423260e-02, -1.47789950e-02, -3.20134100e-03,
            -2.86417400e-03, -4.07979600e-03, -6.36652500e-03, -4.93161400e-03, -4.26023100e-03,
            -8.31728900e-03, -3.19539600e-03, -5.40176000e-04, -2.44478000e-03, -3.62619600e-03,
            -4.29447000e-03, -1.85369100e-03, -2.19274300e-03, -4.60348000e-03, -1.21105000e-04
        ], [
            -7.60787900e-03, -1.66201700e-02, -4.27188700e-03, -1.09440360e-02, -8.44839100e-03,
            -1.11476910e-02, -8.08782400e-03, -8.38208100e-03, -9.11788300e-03, -1.45040330e-02,
            -2.41840460e-02, -2.63273460e-02, 8.26632175e-01, -1.85268780e-02, -4.08778590e-02,
            -1.53419000e-02, -9.69712500e-03, -2.01435180e-02, -2.45546560e-02, -2.84828000e-03,
            -2.59508300e-03, -2.47986600e-03, -6.12152500e-03, -9.68641100e-03, -8.62814200e-03,
            -9.17006800e-03, -2.93151300e-03, -9.97167000e-04, -1.66487000e-03, -3.09773600e-03,
            -7.01918300e-03, -2.41279000e-03, -4.41447800e-03, -4.20750000e-03, -3.36107000e-04
        ], [
            -2.05075800e-03, -8.56902400e-03, -4.01303000e-03, -6.74760500e-03, -5.40078800e-03,
            -7.84585800e-03, -1.07053790e-02, -4.22626400e-03, -1.11481420e-02, -1.36045200e-02,
            -1.23111460e-02, -1.86419950e-02, -1.08693092e-01, 6.10535522e-01, -6.63021810e-02,
            -3.27207070e-02, -2.99642760e-02, -4.27539650e-02, -3.32942190e-02, -9.07393100e-03,
            -8.39274700e-03, -5.84553600e-03, -8.93904400e-03, -3.91523700e-03, -6.18839300e-03,
            -1.14506890e-02, -4.54074330e-02, -4.66246500e-03, -2.75003500e-03, -2.11079810e-02,
            -2.03136610e-02, -1.29745290e-02, -2.78012610e-02, -1.62404900e-02, -8.77418000e-04
        ], [
            -5.32712500e-03, -5.55798400e-03, -2.05586600e-03, -3.14811100e-03, -2.88233000e-03,
            -4.73199900e-03, -4.13320000e-03, -1.48718600e-03, -2.67414400e-03, -5.26585500e-03,
            -7.32316700e-03, -8.02768000e-03, -1.80881240e-02, -4.70059700e-03, 6.77166600e-01,
            -8.06015700e-03, -4.02394500e-03, -6.18006300e-03, -8.04733830e-02, -5.24460100e-03,
            -4.11412500e-03, -2.09946000e-03, -4.10718480e-02, -4.00117980e-02, -5.89447020e-02,
            -1.86306500e-02, -3.55084600e-03, -1.12317200e-03, -1.20802400e-03, -4.47422400e-03,
            -1.96629890e-02, -2.76611500e-03, -1.61753800e-03, -8.49344200e-03, -1.82814000e-04
        ], [
            -6.39972000e-04, -9.11154000e-04, -9.80984000e-04, -4.39435800e-03, -2.84352300e-03,
            -3.88043200e-03, -4.42101400e-03, -6.70311000e-04, -1.68933000e-03, -2.94332300e-03,
            -3.52636900e-03, -9.89722100e-03, -3.73394900e-03, -2.20202900e-03, -4.73495400e-03,
            9.49567402e-01, -1.49660400e-03, -4.81984400e-03, -1.69654400e-03, -1.11988900e-03,
            -1.40950400e-03, -2.33275400e-03, -2.62585700e-03, -7.11492000e-04, -1.92433700e-03,
            -1.52109300e-03, -1.07080400e-03, -8.29159000e-04, -1.37069700e-03, -1.68598400e-03,
            -3.61796800e-03, -3.78414200e-03, -3.05804700e-03, -5.24985500e-03, -1.46207000e-03
        ], [
            -1.07320260e-02, -3.08286120e-02, -1.35370930e-02, -1.73745860e-02, -8.01036400e-03,
            -1.72076330e-02, -2.33489930e-02, -3.13870550e-02, -2.61791630e-02, -2.04640940e-02,
            -4.26095320e-02, -2.91689170e-02, -1.10642290e-02, -7.13990300e-03, -7.64785900e-03,
            -9.35816500e-03, 8.57528691e-01, -6.98299100e-03, -1.10477400e-02, -6.15161600e-03,
            -1.29180340e-02, -2.30960680e-02, -1.54608610e-02, -3.32092300e-03, -5.82629300e-03,
            -1.57511790e-02, -9.43971400e-03, -3.95564900e-03, -7.95091200e-03, -5.47773100e-03,
            -1.47269370e-02, -2.11195190e-02, -1.20403870e-02, -1.55315020e-02, -1.60901000e-04
        ], [
            -3.00986300e-03, -6.35594000e-03, -2.02962300e-03, -2.11266300e-03, -1.34647600e-03,
            -2.88668300e-03, -2.97415600e-03, -4.90014200e-03, -3.03780900e-03, -2.34328200e-03,
            -5.83542400e-03, -3.65273400e-03, -2.47636100e-03, -1.51322000e-03, -1.59204100e-03,
            -2.40522900e-03, -1.56484400e-02, 9.50543246e-01, -5.37816100e-03, -3.28554300e-03,
            -4.33036400e-03, -5.35452700e-03, -8.47520600e-03, -1.62424000e-03, -2.67512000e-03,
            -1.19654910e-02, -1.02423610e-02, -4.37926900e-03, -3.16991460e-02, -4.26973100e-03,
            -1.80689340e-02, -1.07674510e-02, -6.93619700e-03, -8.73033100e-03, -5.07318000e-04
        ], [
            -5.15941300e-03, -3.14114800e-03, -6.59688900e-03, -4.39727400e-03, -5.74073600e-03,
            -5.87175200e-03, -4.74900900e-03, -2.86776200e-03, -5.35770500e-03, -5.73129000e-03,
            -6.41421100e-03, -5.37310800e-03, -4.24821300e-03, -2.91905500e-03, -4.40116900e-03,
            -6.26269000e-03, -7.52317500e-03, -6.94595800e-03, 9.88423083e-01, -4.15788700e-03,
            -4.14792700e-03, -5.17894600e-03, -1.52310420e-02, -2.00847600e-03, -5.01022000e-03,
            -7.95432300e-03, -3.61247900e-03, -2.79493500e-03, -2.13307100e-03, -5.27552100e-03,
            -4.36118100e-03, -2.81085500e-03, -3.95969300e-03, -6.13193500e-03, -3.86579000e-05
        ], [
            -3.42399550e-02, -1.86201570e-02, -6.29222280e-02, -4.07263970e-02, -4.90271920e-02,
            -5.06686320e-02, -4.61183850e-02, -5.89030880e-02, -4.37384580e-02, -4.45387340e-02,
            -3.90692220e-02, -4.11629380e-02, -4.11761010e-02, -3.05761560e-02, -4.00663710e-02,
            -4.16482400e-02, -2.19854500e-02, -4.12995120e-02, -3.24905770e-02, 9.75830316e-01,
            -1.28253420e-02, -4.71476870e-02, -2.26870280e-02, -1.84921850e-02, -2.25906650e-02,
            -2.03944920e-02, -1.14323040e-02, -3.94755600e-03, -4.57304100e-03, -1.32295810e-02,
            -1.93261110e-02, -1.20999750e-02, -2.79003760e-02, -1.73416630e-02, -2.57006000e-04
        ], [
            -1.61331250e-02, -6.71324800e-03, -2.79970050e-02, -2.08831890e-02, -2.77548600e-02,
            -1.80106950e-02, -1.71060660e-02, -1.47618090e-02, -2.01271300e-02, -1.85684530e-02,
            -1.65151680e-02, -1.53077040e-02, -1.48324940e-02, -1.06257810e-02, -1.67044680e-02,
            -2.52823680e-02, -7.20475200e-03, -2.80189000e-02, -1.46205050e-02, -5.78647400e-03,
            9.93120621e-01, -2.35600840e-02, -1.38212760e-02, -6.41180400e-03, -9.51187300e-03,
            -8.58615300e-03, -7.89408600e-03, -4.37532800e-03, -3.28597100e-03, -7.89545300e-03,
            -8.13160400e-03, -5.79618500e-03, -1.23928520e-02, -1.07245200e-02, -3.37962000e-04
        ], [
            -1.48623200e-03, -3.28119700e-03, -3.92242300e-03, -3.81740900e-03, -4.41023500e-03,
            -4.00463200e-03, -5.19442500e-03, -1.09503500e-03, -3.91690200e-03, -4.47162100e-03,
            -5.32213200e-03, -4.20172900e-03, -4.87219200e-03, -3.44466600e-03, -2.51040900e-03,
            -3.42581000e-03, -3.04264300e-03, -5.03877300e-03, -5.06695400e-03, -7.41681900e-03,
            -5.84424900e-03, 9.90096959e-01, -7.54504500e-03, -4.04892800e-03, -1.79153330e-02,
            -2.17279260e-02, -8.93308100e-03, -9.99237400e-03, -2.56215600e-03, -1.25515160e-02,
            -1.12967500e-02, -9.31317700e-03, -8.45978400e-03, -1.19946940e-02, -5.24579000e-05
        ], [
            -1.30668820e-02, -1.86342820e-02, -2.32066490e-02, -1.92265990e-02, -1.87173100e-02,
            -2.26229250e-02, -1.85465120e-02, -4.48529780e-02, -1.74657820e-02, -1.61944400e-02,
            -3.36341770e-02, -1.68247360e-02, -1.18396320e-02, -7.12182600e-03, -1.02997190e-02,
            -1.90818640e-02, -1.89333010e-02, -1.95672610e-02, -1.71078080e-02, -2.14605360e-02,
            -1.21898620e-02, -9.39204300e-03, 9.53847435e-01, -8.03714700e-03, -9.26669600e-03,
            -5.80735090e-02, -8.70642500e-03, -4.93827000e-03, -1.76597900e-03, -5.46932400e-03,
            -1.08520940e-02, -9.61731200e-03, -7.26876700e-03, -9.08750000e-03, -5.04099000e-04
        ], [
            -2.38457400e-03, -6.27222200e-03, -5.11156800e-03, -4.48599500e-03, -4.05050400e-03,
            -6.56319900e-03, -3.09790400e-03, -3.24440000e-03, -3.04182800e-03, -3.12873300e-03,
            -7.90811700e-03, -5.21367500e-03, -2.83751300e-03, -1.68345300e-03, -2.41140900e-03,
            -2.68520400e-03, -2.66410600e-03, -2.87742400e-03, -2.06568500e-03, -5.80558000e-03,
            -3.11580700e-03, -1.66384500e-03, -3.82373300e-03, 8.49383588e-01, -2.99255700e-03,
            -5.45203000e-03, -1.12973800e-03, -6.57072000e-04, -2.58329000e-03, -1.40968700e-03,
            -1.20994200e-03, -9.11299000e-04, -5.86839000e-04, -1.36374400e-03, -4.58498000e-05
        ], [
            -8.23373000e-04, -1.52479800e-03, -1.42343000e-03, -1.62739300e-03, -2.36185300e-03,
            -1.80560400e-03, -2.18007300e-03, -5.80558000e-04, -1.62633100e-03, -1.51990800e-03,
            -1.63025100e-03, -1.10929400e-03, -1.87258500e-03, -1.45494300e-03, -1.24832800e-03,
            -1.34889700e-03, -7.39640000e-04, -1.14979600e-03, -2.26021300e-03, -2.97204200e-03,
            -1.55994100e-03, -1.33034900e-03, -2.59677500e-03, -5.32932500e-03, 9.69729780e-01,
            -1.39801200e-02, -3.76517500e-03, -2.88781000e-03, -5.35426000e-04, -3.71741100e-03,
            -4.20727100e-03, -3.28402600e-03, -1.22614100e-03, -4.32591100e-03, -1.64330000e-05
        ], [
            -3.61375100e-03, -4.55449200e-03, -6.62440000e-03, -4.01275000e-03, -4.35730800e-03,
            -6.96646400e-03, -9.20120400e-03, -7.19875000e-03, -4.80213500e-03, -5.13572000e-03,
            -8.23944200e-03, -4.96144200e-03, -4.14351500e-03, -2.69301700e-03, -3.84432900e-03,
            -4.33255600e-03, -3.81352300e-03, -1.18911100e-02, -1.36436010e-02, -2.07200140e-02,
            -1.22600690e-02, -7.43457800e-03, -3.92836380e-02, -1.14736797e-01, -9.39921660e-02,
            8.97716326e-01, -4.26700900e-03, -3.39698600e-03, -1.07432400e-03, -5.24063800e-03,
            -4.87606100e-03, -2.88564200e-03, -3.25659300e-03, -6.30647200e-03, -2.65925000e-05
        ], [
            -3.97924200e-03, -4.93037500e-03, -5.01386700e-03, -6.00765800e-03, -5.70409400e-03,
            -5.91529100e-03, -1.22512420e-02, -3.67758200e-03, -7.42437800e-03, -5.09162000e-03,
            -6.43631800e-03, -6.79453000e-03, -6.00970200e-03, -6.69226100e-03, -4.12123600e-03,
            -1.16438540e-02, -7.06818400e-03, -1.14225900e-02, -1.75538870e-02, -1.91937880e-02,
            -2.17523410e-02, -1.49751000e-02, -1.79552790e-02, -1.76382520e-02, -2.17700930e-02,
            -2.28448920e-02, 8.78616314e-01, -2.92912930e-02, -5.08240400e-03, -2.47728050e-02,
            -2.30523240e-02, -1.16063630e-02, -1.39019550e-02, -2.21250820e-02, -6.64909000e-04
        ], [
            -1.79013450e-02, -2.49896230e-02, -1.56346740e-02, -2.23204590e-02, -1.66481480e-02,
            -1.66290700e-02, -2.18889160e-02, -1.32502170e-02, -1.79351210e-02, -1.44485030e-02,
            -2.32527250e-02, -1.78329030e-02, -1.59755430e-02, -1.19590440e-02, -1.30970470e-02,
            -2.82494030e-02, -2.39940540e-02, -2.19639720e-02, -2.04874440e-02, -3.13255870e-02,
            -3.47239530e-02, -2.46732360e-02, -4.80110860e-02, -2.32181390e-02, -3.33592430e-02,
            -3.56647250e-02, -1.98558990e-02, 7.87121095e-01, -7.80848120e-02, -3.59927900e-02,
            -2.34907420e-02, -1.54072820e-02, -2.41520330e-02, -4.54093220e-02, -7.72869800e-03
        ], [
            -2.09656800e-03, -5.04798900e-03, -4.46366400e-03, -4.80982700e-03, -6.10433800e-03,
            -5.85532600e-03, -9.73608200e-03, -1.76739200e-03, -4.08362200e-03, -5.42263800e-03,
            -4.82709000e-03, -4.46720200e-03, -4.95482400e-03, -3.33440300e-03, -4.59112700e-03,
            -6.77709700e-03, -4.35740500e-03, -1.01913680e-02, -2.24308460e-02, -1.99958100e-02,
            -4.30542670e-02, -2.78616590e-02, -8.87109300e-03, -1.28166720e-02, -1.01392740e-02,
            -2.03697960e-02, -1.48917140e-02, -2.39316640e-02, 9.65037881e-01, -2.33695440e-02,
            -1.37262360e-02, -2.04024090e-02, -3.54317630e-02, -2.66274610e-02, -1.02562000e-06
        ], [
            -1.28606130e-02, -3.00443240e-02, -3.67309040e-02, -2.07133460e-02, -1.86422080e-02,
            -2.54953060e-02, -6.17137300e-02, -1.40473440e-02, -6.12350600e-02, -3.30930850e-02,
            -3.22106980e-02, -2.72210170e-02, -3.77960630e-02, -3.71856520e-02, -3.84446010e-02,
            -2.74697180e-02, -3.12613590e-02, -5.41806900e-02, -5.18427980e-02, -5.82740480e-02,
            -6.12401730e-02, -4.46129090e-02, -4.29795360e-02, -2.47650070e-02, -6.55085500e-02,
            -6.33626320e-02, -6.41472470e-02, -1.05368509e-01, -3.43869680e-02, 8.50410496e-01,
            -8.53214560e-02, -3.73896140e-02, -6.50878950e-02, -8.21746310e-02, -2.70358200e-03
        ], [
            -1.35942700e-03, -2.52266000e-03, -1.73717200e-03, -7.20481000e-04, -6.16526000e-04,
            -2.75302700e-03, -3.40470300e-03, -1.10584900e-03, -2.56292600e-03, -1.82767900e-03,
            -2.44078900e-03, -2.52646500e-03, -1.15506100e-03, -8.39458000e-04, -1.03601800e-03,
            -2.03364900e-03, -4.35217100e-03, -1.48232600e-03, -2.61523700e-03, -1.52770300e-03,
            -2.57715100e-03, -3.31727600e-03, -3.66680700e-03, -5.09669600e-03, -8.71305400e-03,
            -3.09914000e-03, -2.72510600e-03, -1.57649700e-03, -3.67147200e-03, -2.91651200e-03,
            9.92102569e-01, -4.64113600e-03, -4.19614100e-03, -8.36387600e-03, -9.75590000e-06
        ], [
            -5.20827000e-04, -3.88092000e-04, -4.06248000e-04, -3.06738000e-04, -3.56518000e-04,
            -3.37344000e-04, -5.17489000e-04, -3.68250000e-04, -8.55582000e-04, -5.33440000e-04,
            -4.88106000e-04, -4.78327000e-04, -6.90522000e-04, -5.43115000e-04, -6.72776000e-04,
            -3.84285000e-04, -6.52400000e-04, -4.62830000e-04, -7.85045000e-04, -7.28236000e-04,
            -1.02346200e-03, -5.76307000e-04, -9.65735000e-04, -2.71470000e-04, -1.96090200e-03,
            -1.18648100e-03, -1.39825800e-03, -1.21082100e-03, -3.10617000e-04, -2.31908200e-03,
            -4.03417900e-03, 9.84799190e-01, -1.40582500e-03, -1.58822500e-03, -1.18414000e-05
        ], [
            -1.25826700e-03, -4.44989000e-04, -4.46316000e-04, -3.80262000e-04, -8.32332000e-04,
            -5.07128000e-04, -4.81664000e-04, -1.76250000e-04, -5.09277000e-04, -3.80639000e-04,
            -7.29546000e-04, -4.71071000e-04, -8.08864000e-04, -3.85930000e-04, -4.36742000e-04,
            -4.65957000e-04, -4.12578000e-04, -4.06625000e-04, -5.32685000e-04, -5.05027000e-04,
            -4.40676000e-04, -9.57922000e-04, -5.77831000e-04, -3.08100000e-04, -4.14969000e-04,
            -4.80771000e-04, -6.12237000e-04, -5.77970000e-04, -9.34326000e-05, -5.81793000e-04,
            -5.58802100e-03, -1.65070800e-03, 9.66535455e-01, -1.33646200e-03, -4.97349000e-07
        ], [
            -2.98270100e-03, -6.23305400e-03, -4.96523400e-03, -4.02459100e-03, -5.86675200e-03,
            -4.81733300e-03, -1.21689470e-02, -1.93591300e-03, -6.61386600e-03, -4.40857100e-03,
            -5.58388500e-03, -5.05239200e-03, -4.14389900e-03, -2.73726200e-03, -3.93180400e-03,
            -4.37086300e-03, -5.58076000e-03, -5.85304700e-03, -7.02651200e-03, -7.50496800e-03,
            -8.49951900e-03, -1.34017880e-02, -1.24913700e-02, -4.77913100e-03, -6.90419300e-03,
            -1.39132670e-02, -1.77251380e-02, -1.00219800e-02, -7.87246600e-03, -2.37718910e-02,
            -1.76605260e-02, -1.21793270e-02, -1.36317190e-02, 9.22099747e-01, -1.42256900e-03
        ], [
            -1.38338000e-05, -2.65096000e-05, -2.33817000e-05, -8.20845000e-05, -2.93447000e-05,
            -3.27502000e-05, -3.13764000e-05, -1.22082000e-05, -1.95113000e-05, -1.89003000e-05,
            -2.25061000e-05, -1.66082000e-05, -3.06067000e-05, -1.16841000e-05, -4.44892000e-05,
            -1.30882000e-05, -6.68618000e-06, -5.79481000e-05, -3.69515000e-05, -1.23283000e-05,
            -3.80263000e-05, -5.14698000e-05, -1.13660000e-04, -4.91111000e-06, -1.25213000e-05,
            -1.19771000e-05, -3.48460000e-05, -6.31349000e-05, -2.13945000e-04, -6.39272000e-05,
            -7.22280000e-05, -4.11406000e-05, -5.56648000e-05, -1.10319000e-04, 9.99901777e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 379)
    return ret


@cache('run')
def ia_matrix_x2008x():
    """
    ia matrix x2008x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.83066833e-01, -4.61251000e-04, -2.59949765e-01, -5.11155780e-02, -5.50065500e-02,
            -1.19948473e-01, -1.66625480e-02, -1.51567900e-03, -6.84340100e-03, -1.25391490e-02,
            -5.52577000e-04, -1.99458000e-04, -1.75244000e-04, -8.86027000e-05, -1.22722000e-04,
            -1.11327140e-02, -5.06196000e-04, -3.28738900e-03, -1.01025300e-03, -2.32764500e-03,
            -2.31336800e-03, -3.91386070e-02, -1.60973300e-03, -8.38660000e-04, -3.97172000e-04,
            -6.87484600e-03, -2.15030000e-04, -1.77389000e-04, -3.76736000e-04, -8.37432000e-04,
            -1.95095800e-03, -2.64956800e-03, -2.12887400e-03, -2.57343400e-03, -3.00640790e-02
        ], [
            -2.39323400e-03, 9.23721242e-01, -3.14111800e-03, -3.89802300e-03, -2.87269000e-03,
            -4.74788600e-03, -5.27074700e-03, -4.74645435e-01, -2.89510970e-02, -3.88681600e-03,
            -7.00685080e-02, -5.23580490e-02, -2.92306500e-03, -1.10746800e-03, -1.50198600e-03,
            -5.59058000e-03, -1.14341392e-01, -2.04384710e-02, -5.39608000e-04, -5.23278000e-04,
            -8.64394000e-04, -2.32468800e-03, -5.79818600e-03, -3.82566500e-03, -4.14381100e-03,
            -3.55080500e-03, -4.68452000e-04, -2.70790000e-04, -1.30673700e-03, -8.47985000e-04,
            -3.11656600e-03, -1.76428700e-03, -1.87697400e-03, -2.83658800e-03, -6.57581210e-02
        ], [
            -6.65828930e-02, -7.45260000e-04, 8.34572535e-01, -5.60458700e-03, -1.05215897e-01,
            -3.08971800e-03, -3.53709000e-03, -1.42654000e-03, -1.24157870e-02, -4.11795700e-03,
            -2.92486600e-03, -2.06675000e-03, -1.87996900e-03, -1.52601200e-03, -1.27950900e-03,
            -3.11614200e-03, -1.13320800e-03, -1.65563900e-03, -1.67748800e-03, -4.65116800e-03,
            -4.69498900e-03, -1.51651187e-01, -1.84416000e-03, -5.72074800e-03, -4.56561700e-03,
            -1.83329200e-03, -1.34432800e-03, -7.61500000e-04, -2.92801000e-04, -1.91656100e-03,
            -7.21473500e-03, -8.54507000e-03, -1.19857270e-02, -7.32096400e-03, -3.25964500e-03
        ], [
            -1.70738400e-03, -7.69116000e-04, -1.64026500e-03, 6.17995742e-01, -6.26948730e-02,
            -3.38224200e-03, -6.70550200e-03, -5.27862000e-04, -3.71053800e-03, -1.57467450e-02,
            -4.43434000e-03, -2.25187200e-03, -2.49248400e-03, -1.92373700e-03, -4.20426700e-03,
            -2.52190700e-02, -1.03534600e-03, -2.40319300e-03, -1.68994100e-03, -2.57875800e-03,
            -2.80930400e-03, -2.85318200e-03, -2.04881900e-03, -3.08533900e-03, -2.45651900e-03,
            -1.96958700e-03, -6.31169000e-04, -4.70528000e-04, -4.06803000e-04, -1.48879800e-03,
            -2.77796100e-03, -1.12787000e-03, -3.56173400e-03, -4.21884200e-03, -6.15950000e-05
        ], [
            -1.57735000e-04, -4.92665000e-05, -1.24929000e-04, -1.34268170e-02, 7.68583886e-01,
            -5.25368000e-04, -6.45853000e-04, -3.64237000e-05, -2.46830000e-04, -1.05724200e-03,
            -2.46083000e-04, -1.42569000e-04, -1.81057000e-04, -3.01046000e-04, -2.09882500e-03,
            -5.82707000e-03, -7.62748000e-05, -1.06714000e-04, -3.80078000e-04, -2.90440000e-04,
            -2.32897000e-04, -1.20488000e-04, -2.24672000e-04, -9.09419000e-05, -1.36775000e-04,
            -1.85752000e-04, -1.91092000e-04, -3.57580000e-05, -9.94814000e-06, -9.28969000e-05,
            -7.36368000e-04, -1.15953000e-04, -2.77723000e-04, -4.22013000e-04, -1.62332000e-06
        ], [
            -1.61311400e-03, -1.29438500e-03, -1.67528900e-03, -1.53032900e-03, -1.95405900e-03,
            7.29938015e-01, -1.25838220e-02, -2.45325000e-04, -1.49331000e-03, -3.07976300e-03,
            -8.36271900e-03, -3.84926500e-03, -3.05461300e-03, -1.61243100e-03, -2.49878000e-03,
            -7.72599360e-02, -7.19147000e-04, -2.90517030e-02, -1.34569900e-03, -1.27547400e-03,
            -1.22172600e-03, -2.22687200e-03, -1.20543800e-03, -5.95997000e-04, -4.12795000e-04,
            -2.13028800e-03, -5.25836000e-04, -2.87069000e-04, -1.49169900e-03, -9.32256000e-04,
            -2.03282400e-03, -2.96074000e-03, -1.44680400e-03, -2.60477400e-03, -2.13907000e-05
        ], [
            -2.34789700e-03, -2.40298300e-03, -2.44608700e-02, -1.20502430e-02, -1.49338600e-02,
            -1.46551720e-02, 7.47318057e-01, -1.55861400e-03, -1.46032310e-02, -1.62367920e-02,
            -2.16916900e-02, -4.96217600e-03, -6.35508100e-03, -7.99014700e-03, -3.91571700e-03,
            -2.03392820e-02, -2.82531700e-03, -4.07844900e-03, -8.29299700e-03, -1.12262900e-02,
            -1.05750630e-02, -7.90498700e-03, -6.56264300e-03, -3.26023800e-03, -4.75778100e-03,
            -1.28483320e-02, -9.04715800e-03, -1.27510390e-02, -1.78315100e-03, -1.96565900e-02,
            -1.76567250e-02, -1.89516240e-02, -8.97149800e-03, -2.31809850e-02, -1.05017000e-04
        ], [
            -1.29030250e-02, -5.16866900e-03, -3.38048000e-03, -5.07960500e-03, -4.40303700e-03,
            -4.70328600e-03, -4.70579700e-03, 8.25244230e-01, -5.38585600e-02, -9.52122800e-03,
            -2.15960170e-02, -1.48529870e-02, -4.04663700e-03, -2.06037300e-03, -2.00909900e-03,
            -4.16294100e-03, -3.56759950e-02, -1.04141300e-02, -5.56907300e-03, -4.69033100e-03,
            -3.70743300e-03, -3.55838800e-03, -5.52321120e-02, -6.36275070e-02, -1.11863795e-01,
            -2.32041910e-02, -3.46552000e-03, -1.21976500e-03, -1.10952300e-03, -4.66339000e-03,
            -8.40685300e-03, -3.13153900e-03, -2.72382700e-03, -5.25308200e-03, -1.99640000e-05
        ], [
            -3.80930780e-02, -8.80287900e-03, -9.42089900e-03, -6.14067880e-02, -3.67501240e-02,
            -3.60410730e-02, -3.60781060e-02, -1.63464800e-02, 7.47935176e-01, -2.19871974e-01,
            -3.82294310e-02, -1.51683730e-02, -9.85014200e-03, -1.75484550e-02, -1.05290520e-02,
            -2.67330450e-02, -6.64523500e-03, -1.07680780e-02, -5.69940600e-03, -2.71083900e-03,
            -2.05752200e-03, -4.63111000e-03, -3.32883300e-03, -4.28179200e-03, -3.93169200e-03,
            -4.51447600e-03, -1.16798400e-03, -6.96455000e-04, -1.71856100e-03, -4.73850300e-03,
            -9.04668400e-03, -3.92578500e-03, -6.15326460e-02, -1.02790860e-02, -1.45392000e-04
        ], [
            -4.58381700e-03, -4.73405000e-03, -1.90022380e-02, -1.10591500e-02, -3.97834620e-02,
            -7.94605200e-03, -1.69134540e-02, -3.12428800e-03, -1.77164090e-02, 8.44309793e-01,
            -1.21469580e-02, -6.77871100e-03, -2.19404160e-02, -2.29816660e-02, -3.31631410e-02,
            -3.42656090e-02, -2.29274400e-03, -1.81629870e-02, -1.48063010e-02, -4.39947400e-03,
            -4.79063600e-03, -4.21936400e-03, -1.11968070e-02, -2.71570400e-03, -3.86015200e-03,
            -6.58671400e-03, -3.09253900e-03, -6.27683000e-04, -1.17056700e-03, -2.80490700e-03,
            -3.07630700e-03, -1.46913000e-03, -5.07980100e-03, -4.53534700e-03, -6.23224000e-05
        ], [
            -1.50066200e-03, -3.51570500e-03, -5.27051300e-03, -2.08445200e-03, -1.61810300e-03,
            -6.65033800e-03, -1.24841700e-03, -1.64803600e-03, -5.60730800e-03, -5.41184900e-03,
            8.54354550e-01, -1.07959070e-02, -4.97177400e-03, -9.91945600e-03, -6.44032700e-03,
            -7.52448900e-03, -1.63415100e-03, -9.37402080e-02, -3.51445700e-03, -9.50720000e-04,
            -9.88566000e-04, -3.28257100e-03, -1.45453300e-03, -4.14784000e-04, -6.94666000e-04,
            -1.49090700e-03, -1.06214700e-03, -1.20308000e-04, -1.60955100e-03, -9.83769000e-04,
            -1.49302200e-03, -1.71571100e-03, -1.92174900e-03, -1.87824000e-03, -2.80840000e-05
        ], [
            -4.33652100e-03, -1.64852250e-02, -1.01942690e-02, -4.90873700e-03, -9.17386600e-03,
            -2.10849420e-02, -9.33112400e-03, -5.24970200e-03, -1.26605400e-02, -2.30746260e-02,
            -3.44227300e-02, 6.72902104e-01, -1.52374823e-01, -6.36843820e-02, -8.53073370e-02,
            -7.59412680e-02, -6.48376600e-03, -9.02654480e-02, -1.44265770e-02, -3.19267700e-03,
            -2.86531700e-03, -4.31362500e-03, -6.40257200e-03, -4.12054500e-03, -4.37210000e-03,
            -8.47443600e-03, -3.29351800e-03, -5.75075000e-04, -2.19950100e-03, -3.77187700e-03,
            -4.17679000e-03, -1.80635200e-03, -2.21598500e-03, -4.70739200e-03, -1.42877000e-04
        ], [
            -7.96367600e-03, -1.38994180e-02, -4.35988900e-03, -1.09703940e-02, -8.49577100e-03,
            -1.16361560e-02, -8.23305200e-03, -7.88405200e-03, -9.12263100e-03, -1.52030000e-02,
            -2.48505480e-02, -2.69177930e-02, 8.24012356e-01, -1.88902870e-02, -4.37516460e-02,
            -1.61269660e-02, -9.33007900e-03, -2.08465950e-02, -2.51001520e-02, -2.90652700e-03,
            -2.61182600e-03, -2.61043200e-03, -6.29304500e-03, -9.62909900e-03, -9.11138700e-03,
            -9.66297500e-03, -2.95052600e-03, -1.11278700e-03, -1.61758200e-03, -3.16796700e-03,
            -7.24013200e-03, -2.47790700e-03, -4.78504600e-03, -4.29740600e-03, -3.58567000e-04
        ], [
            -2.14387000e-03, -7.45467000e-03, -4.22606500e-03, -7.02667000e-03, -5.49298400e-03,
            -7.88424800e-03, -1.09607250e-02, -3.64782700e-03, -1.13201830e-02, -1.38748650e-02,
            -1.28825510e-02, -1.85027150e-02, -1.09169596e-01, 5.99170482e-01, -6.98599490e-02,
            -3.36365880e-02, -3.03222640e-02, -4.47319890e-02, -3.36406750e-02, -9.55185200e-03,
            -8.08248200e-03, -6.21385000e-03, -9.65195400e-03, -3.93244600e-03, -6.58879300e-03,
            -1.21618280e-02, -4.45819730e-02, -4.92069300e-03, -2.59832600e-03, -2.20788240e-02,
            -2.17911460e-02, -1.40943870e-02, -2.91227450e-02, -1.76941740e-02, -9.53278000e-04
        ], [
            -5.36925700e-03, -4.52061700e-03, -2.03126600e-03, -3.22428900e-03, -2.81520000e-03,
            -4.92150200e-03, -4.37098200e-03, -1.22564400e-03, -2.67520400e-03, -5.46501200e-03,
            -7.50656000e-03, -8.28892700e-03, -1.79078360e-02, -4.60213400e-03, 6.73285039e-01,
            -8.49811100e-03, -3.97536800e-03, -6.45908700e-03, -7.50617240e-02, -5.54682900e-03,
            -4.12909300e-03, -2.16600000e-03, -4.08582530e-02, -3.91673550e-02, -6.28726250e-02,
            -2.15402100e-02, -3.63245700e-03, -1.18308200e-03, -1.38794200e-03, -4.44238200e-03,
            -1.91739560e-02, -2.80094400e-03, -1.57817100e-03, -8.68328200e-03, -1.75499000e-04
        ], [
            -7.17495000e-04, -7.26470000e-04, -9.71834000e-04, -4.86763700e-03, -2.93476900e-03,
            -3.73530800e-03, -4.20081100e-03, -6.13319000e-04, -1.71629900e-03, -2.88804000e-03,
            -3.69927100e-03, -9.63344900e-03, -3.67818300e-03, -2.18324600e-03, -5.00408300e-03,
            9.45162601e-01, -1.65845000e-03, -4.54956500e-03, -1.55616400e-03, -1.11345700e-03,
            -1.39611100e-03, -2.38537700e-03, -3.38802100e-03, -6.79840000e-04, -2.01787700e-03,
            -1.47350700e-03, -1.33395700e-03, -8.25184000e-04, -1.22751500e-03, -1.69054900e-03,
            -3.40950300e-03, -3.68771400e-03, -3.12020300e-03, -5.17037900e-03, -1.69708700e-03
        ], [
            -1.11678190e-02, -2.79441260e-02, -1.46199310e-02, -1.82518000e-02, -8.00340000e-03,
            -1.83898210e-02, -2.48454600e-02, -4.66233950e-02, -2.78646260e-02, -2.18405890e-02,
            -4.49849480e-02, -3.05407850e-02, -1.17605230e-02, -7.68949300e-03, -8.31893900e-03,
            -1.05388930e-02, 8.52588953e-01, -7.66439300e-03, -1.12818220e-02, -6.65961700e-03,
            -1.36361100e-02, -2.50211930e-02, -1.64107010e-02, -3.35993000e-03, -6.99522000e-03,
            -1.72844920e-02, -1.00787040e-02, -4.41241600e-03, -7.69580500e-03, -5.84505700e-03,
            -1.45189570e-02, -2.22834320e-02, -1.25456130e-02, -1.63566930e-02, -1.20179000e-04
        ], [
            -3.03439200e-03, -5.81646400e-03, -2.11622100e-03, -2.11641100e-03, -1.26722500e-03,
            -2.76664400e-03, -2.93728800e-03, -5.89846400e-03, -2.88870800e-03, -2.19154800e-03,
            -5.67131100e-03, -3.52105700e-03, -2.37906500e-03, -1.45447100e-03, -1.57629800e-03,
            -2.46934000e-03, -1.39599860e-02, 9.48979888e-01, -5.63453300e-03, -3.31905900e-03,
            -4.45579300e-03, -5.72439100e-03, -8.73039300e-03, -1.49854600e-03, -2.86778200e-03,
            -1.21279410e-02, -1.01169180e-02, -4.91983400e-03, -3.09621640e-02, -4.33466800e-03,
            -1.76490770e-02, -1.08368170e-02, -7.28966600e-03, -8.98154000e-03, -4.15690000e-04
        ], [
            -5.12167900e-03, -2.61061800e-03, -6.45378300e-03, -3.84848700e-03, -5.11449900e-03,
            -5.56630500e-03, -4.64567900e-03, -2.48847700e-03, -5.22242600e-03, -5.46150300e-03,
            -6.00902400e-03, -5.15807300e-03, -4.02892300e-03, -2.78414700e-03, -4.42781600e-03,
            -6.18362500e-03, -6.55620800e-03, -6.64217600e-03, 9.88496564e-01, -4.01372600e-03,
            -4.05479500e-03, -5.02311700e-03, -1.43192230e-02, -1.80821100e-03, -5.02860400e-03,
            -8.15535100e-03, -3.35508000e-03, -2.87937700e-03, -2.00336300e-03, -5.21906800e-03,
            -4.01044700e-03, -2.57932300e-03, -3.77922000e-03, -5.82286300e-03, -1.81871000e-05
        ], [
            -3.26992390e-02, -1.61132600e-02, -6.22475750e-02, -3.72590350e-02, -4.76788780e-02,
            -4.93098070e-02, -4.61360450e-02, -6.15445150e-02, -4.44989950e-02, -4.38862890e-02,
            -3.82793780e-02, -4.19608790e-02, -4.02576280e-02, -3.09263990e-02, -4.13698470e-02,
            -4.13387200e-02, -1.98046850e-02, -4.12984370e-02, -3.15051870e-02, 9.76630971e-01,
            -1.22978760e-02, -4.75732610e-02, -2.17110060e-02, -1.75114240e-02, -2.19964260e-02,
            -2.05392670e-02, -1.16144280e-02, -4.16488100e-03, -4.23666100e-03, -1.32336170e-02,
            -2.01342990e-02, -1.22546520e-02, -2.74860970e-02, -1.73198520e-02, -1.20548000e-04
        ], [
            -1.61712490e-02, -5.93766900e-03, -2.80743650e-02, -1.96882260e-02, -2.60274140e-02,
            -1.78602310e-02, -1.70994090e-02, -1.30224770e-02, -2.04364840e-02, -1.86936950e-02,
            -1.64347360e-02, -1.56147090e-02, -1.49203420e-02, -1.07215170e-02, -1.72032760e-02,
            -2.46380870e-02, -6.94577700e-03, -2.56027040e-02, -1.47593190e-02, -5.84330100e-03,
            9.93122735e-01, -2.36834660e-02, -1.37729660e-02, -5.89180100e-03, -9.36010400e-03,
            -9.03739600e-03, -7.78213300e-03, -4.73533500e-03, -2.86355800e-03, -7.84464800e-03,
            -7.83722900e-03, -5.73126400e-03, -1.24034890e-02, -1.06685910e-02, -1.63198000e-04
        ], [
            -1.48798400e-03, -2.65706800e-03, -3.91283000e-03, -3.78971200e-03, -4.44957000e-03,
            -3.76272000e-03, -4.94488700e-03, -8.61543000e-04, -3.81221900e-03, -4.25874100e-03,
            -5.19616700e-03, -4.01349300e-03, -4.71161000e-03, -3.45631100e-03, -2.53824700e-03,
            -3.46565500e-03, -2.65172600e-03, -4.95334100e-03, -5.07346800e-03, -7.48111300e-03,
            -5.67022700e-03, 9.90136690e-01, -7.28399900e-03, -3.82000300e-03, -1.66836300e-02,
            -2.22774900e-02, -8.03210000e-03, -1.06691670e-02, -2.01374300e-03, -1.23808980e-02,
            -1.09829290e-02, -9.41216100e-03, -8.10390400e-03, -1.18796830e-02, -3.41955000e-05
        ], [
            -1.28149570e-02, -1.55665250e-02, -2.37971970e-02, -1.84635540e-02, -1.85863070e-02,
            -2.25259770e-02, -1.85842360e-02, -5.12643560e-02, -1.82505250e-02, -1.65054820e-02,
            -3.36884210e-02, -1.71682890e-02, -1.20167200e-02, -7.33587200e-03, -1.06631240e-02,
            -1.84066600e-02, -1.81071150e-02, -1.98652110e-02, -1.67914740e-02, -2.11272600e-02,
            -1.21454390e-02, -9.54596200e-03, 9.54043297e-01, -7.91150400e-03, -9.45078600e-03,
            -5.74330110e-02, -8.63522700e-03, -5.22687900e-03, -1.58382500e-03, -5.66842700e-03,
            -1.04231370e-02, -9.62531400e-03, -7.18187700e-03, -9.23023200e-03, -2.97154000e-04
        ], [
            -2.28333400e-03, -5.02839400e-03, -5.12589200e-03, -4.70022400e-03, -4.22108400e-03,
            -7.28244400e-03, -3.23186400e-03, -2.26543100e-03, -3.12609200e-03, -3.38265500e-03,
            -8.38274200e-03, -5.33234900e-03, -3.12133000e-03, -1.83557300e-03, -2.56110600e-03,
            -2.72722000e-03, -2.44077100e-03, -2.97859500e-03, -1.65320700e-03, -6.05376300e-03,
            -3.10663200e-03, -1.68292200e-03, -3.45360900e-03, 8.51180069e-01, -3.07337300e-03,
            -5.73803000e-03, -1.05265600e-03, -6.37242000e-04, -2.76218800e-03, -1.36566500e-03,
            -1.14149600e-03, -8.48277000e-04, -5.12857000e-04, -1.24005300e-03, -1.43427000e-05
        ], [
            -8.05283000e-04, -1.29036600e-03, -1.42091100e-03, -1.54071600e-03, -2.21153400e-03,
            -1.81482000e-03, -2.10969700e-03, -4.57170000e-04, -1.59103800e-03, -1.53532300e-03,
            -1.56464200e-03, -1.06999900e-03, -1.87084400e-03, -1.46732800e-03, -1.21915200e-03,
            -1.29575900e-03, -6.92783000e-04, -1.17045400e-03, -2.35537200e-03, -3.05556000e-03,
            -1.62427400e-03, -1.35107900e-03, -2.65652900e-03, -4.91640100e-03, 9.69574012e-01,
            -1.35342590e-02, -3.98354100e-03, -3.15614700e-03, -4.47514000e-04, -3.73975900e-03,
            -4.03914900e-03, -3.38759100e-03, -1.23089500e-03, -4.42636700e-03, -7.86761000e-06
        ], [
            -3.70568300e-03, -4.02834100e-03, -6.70204100e-03, -3.90312900e-03, -4.07397700e-03,
            -6.67041000e-03, -8.95392000e-03, -5.87531400e-03, -4.84296800e-03, -5.14801400e-03,
            -8.05393700e-03, -4.95676400e-03, -4.07154000e-03, -2.67086100e-03, -3.86036500e-03,
            -4.37522700e-03, -3.48278500e-03, -1.25625800e-02, -1.44159720e-02, -2.08827140e-02,
            -1.23068050e-02, -7.57199800e-03, -3.79974320e-02, -1.14025202e-01, -1.00829449e-01,
            8.99226378e-01, -4.14388000e-03, -3.39062700e-03, -1.12003200e-03, -5.15001200e-03,
            -4.68962100e-03, -2.90583500e-03, -3.29363700e-03, -6.36851400e-03, -1.22365000e-05
        ], [
            -4.25702500e-03, -4.49507100e-03, -5.44013200e-03, -6.31940700e-03, -6.22270800e-03,
            -5.97268700e-03, -1.21628650e-02, -3.46436100e-03, -7.63639300e-03, -5.20897000e-03,
            -6.55277100e-03, -7.10726600e-03, -5.99548600e-03, -7.11954300e-03, -4.15656500e-03,
            -1.19941230e-02, -6.84019100e-03, -1.19833060e-02, -1.82734520e-02, -1.96004030e-02,
            -2.20156340e-02, -1.56323670e-02, -1.90773480e-02, -1.87632360e-02, -2.20013340e-02,
            -2.27658690e-02, 8.79523226e-01, -3.02278530e-02, -4.64413500e-03, -2.51914720e-02,
            -2.34510910e-02, -1.22405200e-02, -1.43976990e-02, -2.25063690e-02, -5.91323000e-04
        ], [
            -1.70846690e-02, -1.98701630e-02, -1.60202520e-02, -2.21489760e-02, -1.66902780e-02,
            -1.61557010e-02, -2.14098870e-02, -1.25569690e-02, -1.75413340e-02, -1.47967260e-02,
            -2.29921930e-02, -1.67633600e-02, -1.54341160e-02, -1.21342990e-02, -1.30628410e-02,
            -2.74137060e-02, -2.21553390e-02, -2.11528640e-02, -2.06555510e-02, -3.08938850e-02,
            -3.34357690e-02, -2.47880050e-02, -4.62625570e-02, -2.13099990e-02, -3.21771060e-02,
            -3.37595190e-02, -1.91077270e-02, 7.97001282e-01, -8.11384030e-02, -3.45548200e-02,
            -2.21544980e-02, -1.57736840e-02, -2.30089850e-02, -4.12615930e-02, -8.57966900e-03
        ], [
            -2.54756600e-03, -4.80333100e-03, -4.47814800e-03, -4.56945900e-03, -5.77010400e-03,
            -5.60339300e-03, -9.14329200e-03, -1.78253700e-03, -3.94481100e-03, -5.23050200e-03,
            -4.53071500e-03, -4.37255900e-03, -4.76398600e-03, -3.18195800e-03, -4.45805300e-03,
            -6.79896700e-03, -4.01941900e-03, -1.02823450e-02, -2.18936530e-02, -1.94666880e-02,
            -4.23133210e-02, -2.73593550e-02, -8.50186900e-03, -1.16380820e-02, -9.27850900e-03,
            -2.00869870e-02, -1.34956620e-02, -2.30664310e-02, 9.65962298e-01, -2.16794670e-02,
            -1.44947730e-02, -2.04008030e-02, -3.39659950e-02, -2.62321580e-02, -9.48375000e-07
        ], [
            -1.31301360e-02, -2.68780610e-02, -3.72804700e-02, -2.00583280e-02, -1.76258270e-02,
            -2.36567210e-02, -6.18092470e-02, -1.26334400e-02, -6.09999890e-02, -3.35392890e-02,
            -3.08258760e-02, -2.66989880e-02, -3.65318530e-02, -3.73555430e-02, -3.72935260e-02,
            -2.81222090e-02, -2.90696410e-02, -5.45095790e-02, -5.33770560e-02, -6.14194610e-02,
            -6.20803340e-02, -4.78557860e-02, -4.28032590e-02, -2.33163300e-02, -6.60444000e-02,
            -6.27447090e-02, -6.20367010e-02, -1.10465483e-01, -2.91936310e-02, 8.45877933e-01,
            -8.81274420e-02, -3.81976000e-02, -6.86258130e-02, -8.52181750e-02, -1.87925800e-03
        ], [
            -1.36357900e-03, -2.25503800e-03, -1.88062700e-03, -6.87351000e-04, -5.89046000e-04,
            -2.65727600e-03, -3.79643900e-03, -1.09176000e-03, -2.68163700e-03, -1.96012700e-03,
            -2.57216600e-03, -3.26010800e-03, -1.18499100e-03, -8.38597000e-04, -1.02503000e-03,
            -2.27841700e-03, -3.96363700e-03, -1.47736600e-03, -2.85503900e-03, -1.60049000e-03,
            -2.71251600e-03, -3.67916300e-03, -3.51178900e-03, -4.66151100e-03, -8.55809200e-03,
            -3.17415600e-03, -2.67353400e-03, -1.68624200e-03, -3.12173800e-03, -2.89561900e-03,
            9.92543038e-01, -4.52693600e-03, -4.31655400e-03, -8.14812100e-03, -3.79376000e-06
        ], [
            -5.41350000e-04, -3.35151000e-04, -4.01494000e-04, -3.00798000e-04, -3.44699000e-04,
            -3.28907000e-04, -5.08096000e-04, -3.37824000e-04, -8.44288000e-04, -5.36318000e-04,
            -4.82540000e-04, -4.72144000e-04, -6.84372000e-04, -5.09286000e-04, -6.76231000e-04,
            -3.80958000e-04, -5.88282000e-04, -4.57967000e-04, -7.65797000e-04, -7.33248000e-04,
            -1.09529300e-03, -5.61342000e-04, -9.28639000e-04, -2.63037000e-04, -2.05645400e-03,
            -1.13996400e-03, -1.25768700e-03, -1.21391800e-03, -2.86844000e-04, -2.17208500e-03,
            -4.05620900e-03, 9.84936723e-01, -1.37111500e-03, -1.65862200e-03, -6.70971000e-06
        ], [
            -1.26177700e-03, -3.79093000e-04, -4.50631000e-04, -3.85540000e-04, -9.15347000e-04,
            -5.46368000e-04, -5.04663000e-04, -1.40682000e-04, -5.06592000e-04, -3.64872000e-04,
            -8.00419000e-04, -4.95891000e-04, -8.92746000e-04, -4.06520000e-04, -4.86336000e-04,
            -4.58898000e-04, -3.89192000e-04, -3.85055000e-04, -5.22866000e-04, -4.66226000e-04,
            -4.27195000e-04, -8.55483000e-04, -5.51576000e-04, -2.97208000e-04, -4.27820000e-04,
            -4.55112000e-04, -5.38112000e-04, -5.59514000e-04, -8.87142000e-05, -5.67770000e-04,
            -5.70998500e-03, -1.64409700e-03, 9.67511122e-01, -1.34320900e-03, -2.60735000e-07
        ], [
            -2.98399800e-03, -5.33203600e-03, -5.01730200e-03, -4.01899500e-03, -5.77162200e-03,
            -4.57285400e-03, -1.20877390e-02, -1.51778700e-03, -6.55214200e-03, -4.35735100e-03,
            -5.45836800e-03, -5.09792700e-03, -3.94622800e-03, -2.66669800e-03, -3.86167900e-03,
            -4.39067600e-03, -5.14994400e-03, -5.60670400e-03, -6.99220600e-03, -7.70054500e-03,
            -8.56441600e-03, -1.42067690e-02, -1.27443580e-02, -4.51043700e-03, -7.08629200e-03,
            -1.44817490e-02, -1.67333240e-02, -1.05685990e-02, -6.69412200e-03, -2.38859320e-02,
            -1.72972800e-02, -1.20668630e-02, -1.35922850e-02, 9.23436988e-01, -1.04760600e-03
        ], [
            -1.31904000e-05, -2.25995000e-05, -2.36995000e-05, -8.45047000e-05, -2.70827000e-05,
            -2.88484000e-05, -2.97422000e-05, -1.11246000e-05, -1.73989000e-05, -1.79229000e-05,
            -1.98974000e-05, -1.43748000e-05, -2.90354000e-05, -1.13338000e-05, -4.37457000e-05,
            -1.18856000e-05, -6.18323000e-06, -5.23174000e-05, -2.98652000e-05, -1.34951000e-05,
            -3.75016000e-05, -5.31313000e-05, -1.19306000e-04, -4.89536000e-06, -1.31267000e-05,
            -1.07090000e-05, -3.37309000e-05, -6.97744000e-05, -2.06878000e-04, -6.25151000e-05,
            -6.82776000e-05, -4.17579000e-05, -5.77957000e-05, -1.03009000e-04, 9.99926417e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 380)
    return ret


@cache('step')
def ia_matrix_for_python():
    """
    ia matrix for python



    component


    """
    
    ret = functions.if_then_else(
        time() < 1996,
        ia_matrix_x1995x(),
        functions.if_then_else(
            time() < 1997,
            ia_matrix_x1996x(),
            functions.if_then_else(
                time() < 1998,
                ia_matrix_x1997x(),
                functions.if_then_else(
                    time() < 1999,
                    ia_matrix_x1998x(),
                    functions.if_then_else(
                        time() < 2000,
                        ia_matrix_x1999x(),
                        functions.if_then_else(
                            time() < 2001,
                            ia_matrix_x2000x(),
                            functions.if_then_else(
                                time() < 2002,
                                ia_matrix_x2001x(),
                                functions.if_then_else(
                                    time() < 2003,
                                    ia_matrix_x2002x(),
                                    functions.if_then_else(
                                        time() < 2004,
                                        ia_matrix_x2003x(),
                                        functions.if_then_else(
                                            time() < 2005,
                                            ia_matrix_x2004x(),
                                            functions.if_then_else(
                                                time() < 2006,
                                                ia_matrix_x2005x(),
                                                functions.if_then_else(
                                                    time() < 2007,
                                                    ia_matrix_x2006x(),
                                                    functions.if_then_else(
                                                        time() < 2008,
                                                        ia_matrix_x2007x(),
                                                        functions.if_then_else(
                                                            time() < 2009,
                                                            ia_matrix_x2008x(),
                                                            ia_matrix_x2009x()))))))))))))))
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 381)
    return ret


@cache('run')
def ia_matrix_x1995x():
    """
    ia matrix x1995x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.66652148e-01, -2.99406100e-03, -2.69202856e-01, -4.06458790e-02, -2.95500070e-02,
            -1.40634738e-01, -1.25128850e-02, -3.10544200e-03, -6.63895500e-03, -1.38106240e-02,
            -1.21552300e-03, -3.33429000e-04, -3.35808000e-04, -2.65041000e-04, -1.98707000e-04,
            -1.30784040e-02, -5.00613000e-04, -3.33077300e-03, -5.28276000e-04, -2.33610700e-03,
            -2.52906400e-03, -3.67591510e-02, -7.12228000e-04, -1.99406000e-03, -4.48338000e-04,
            -1.10414200e-03, -1.88924000e-04, -2.41248000e-04, -5.34767000e-04, -1.13528300e-03,
            -1.74651800e-03, -1.85645800e-03, -2.28500600e-03, -2.24151700e-03, -1.55967450e-02
        ], [
            -3.35117100e-03, 9.17930103e-01, -3.16290700e-03, -2.16177100e-03, -1.49126100e-03,
            -3.43566000e-03, -5.06743500e-03, -4.26548063e-01, -3.30852510e-02, -3.34696700e-03,
            -8.03877830e-02, -4.47561160e-02, -3.36983600e-03, -2.32725100e-03, -1.54824500e-03,
            -6.60731100e-03, -9.81133800e-02, -1.97980520e-02, -7.12524000e-04, -1.69235600e-03,
            -7.99184000e-04, -2.07736800e-03, -6.64434700e-03, -2.75341900e-03, -2.78959800e-03,
            -3.95077700e-03, -5.58722000e-04, -6.71999000e-04, -2.22122400e-03, -1.01972400e-03,
            -2.56993000e-03, -2.84237900e-03, -1.35566900e-03, -4.44448300e-03, -6.55890540e-02
        ], [
            -5.30134200e-02, -8.22757000e-04, 8.62749493e-01, -2.38990500e-03, -8.41760650e-02,
            -1.16540000e-03, -2.81594500e-03, -1.11828600e-03, -9.81928900e-03, -2.48638600e-03,
            -1.72415400e-03, -9.81881000e-04, -1.16261700e-03, -1.17883700e-03, -8.19823000e-04,
            -1.68515000e-03, -8.46380000e-04, -1.06426600e-03, -1.73146300e-03, -4.98086600e-03,
            -4.54224100e-03, -1.65189944e-01, -1.62123400e-03, -5.53345000e-03, -4.48595700e-03,
            -2.30117300e-03, -1.32517700e-03, -9.99116000e-04, -2.44173000e-04, -2.64925000e-03,
            -8.17163300e-03, -8.96135000e-03, -1.46091060e-02, -5.02763100e-03, -1.20130900e-03
        ], [
            -2.52086400e-03, -1.51701500e-03, -1.45943800e-03, 7.17239641e-01, -5.21930650e-02,
            -5.47208000e-03, -7.12738800e-03, -5.43699000e-04, -3.97701200e-03, -1.85617530e-02,
            -4.74143800e-03, -1.61893700e-03, -2.32742000e-03, -2.24731600e-03, -6.02747400e-03,
            -2.51873070e-02, -7.02411000e-04, -2.57563100e-03, -1.72704000e-03, -3.18714900e-03,
            -3.72918300e-03, -3.32570500e-03, -1.70665400e-03, -2.93672300e-03, -1.27145000e-03,
            -1.73536500e-03, -2.94326600e-03, -7.59909000e-04, -2.67241000e-04, -1.40002500e-03,
            -3.46328100e-03, -7.48320000e-04, -3.63663300e-03, -4.95268700e-03, -5.23151000e-04
        ], [
            -2.33707000e-04, -2.99946000e-04, -1.96779000e-04, -6.39377600e-03, 8.13537579e-01,
            -9.06580000e-04, -1.39761200e-03, -1.04402000e-04, -2.62644000e-04, -9.31782000e-04,
            -5.64356000e-04, -3.75085000e-04, -4.10098000e-04, -3.97849000e-04, -1.20364800e-03,
            -3.73267300e-03, -1.89194000e-04, -1.82299000e-04, -4.18842000e-04, -5.42536000e-04,
            -5.56168000e-04, -2.21703000e-04, -2.05248000e-04, -2.16585000e-04, -1.46464000e-04,
            -1.61638000e-04, -6.40394000e-04, -7.67576000e-05, -1.99478000e-05, -1.59219000e-04,
            -9.10199000e-04, -1.33144000e-04, -1.62040000e-04, -6.89726000e-04, -1.52209000e-05
        ], [
            -1.58541200e-03, -1.93726700e-03, -1.51096900e-03, -1.47041300e-03, -2.26386000e-03,
            8.12337536e-01, -1.12859350e-02, -4.02701000e-04, -1.52607800e-03, -2.69409900e-03,
            -6.47300000e-03, -2.43993700e-03, -2.86987700e-03, -1.95204000e-03, -3.23730800e-03,
            -7.31049670e-02, -7.90780000e-04, -3.47932190e-02, -8.31092000e-04, -1.89151800e-03,
            -1.67770900e-03, -2.44670000e-03, -1.43669500e-03, -6.11308000e-04, -4.25251000e-04,
            -4.94516400e-03, -7.70322000e-04, -6.62780000e-04, -2.15946100e-03, -8.81898000e-04,
            -1.42205700e-03, -1.08434800e-03, -1.08098800e-03, -2.90503300e-03, -3.05355000e-04
        ], [
            -2.91121600e-03, -3.41131300e-03, -2.39790950e-02, -1.07469010e-02, -1.28439570e-02,
            -1.25985470e-02, 7.59016386e-01, -1.89934200e-03, -2.01729770e-02, -1.85839950e-02,
            -2.23451740e-02, -5.95230000e-03, -1.02255100e-02, -1.58890580e-02, -6.02285000e-03,
            -2.24738810e-02, -3.53113000e-03, -5.27049600e-03, -7.87651000e-03, -1.63126390e-02,
            -1.55690310e-02, -1.15575810e-02, -6.26440200e-03, -4.72594300e-03, -8.40044700e-03,
            -1.71345270e-02, -1.22696840e-02, -1.66200490e-02, -2.64343300e-03, -2.27607600e-02,
            -1.66955350e-02, -1.54090390e-02, -1.00800230e-02, -2.53401690e-02, -7.47692000e-04
        ], [
            -1.28889790e-02, -1.37629980e-02, -3.72694300e-03, -5.11792700e-03, -3.52507500e-03,
            -5.07661200e-03, -4.58926700e-03, 9.53904425e-01, -2.54159500e-02, -8.66373500e-03,
            -1.56805250e-02, -8.95218200e-03, -3.75339600e-03, -3.26041300e-03, -2.06575900e-03,
            -4.45412100e-03, -2.09774640e-02, -1.24626800e-02, -4.40259000e-03, -6.47064500e-03,
            -5.54614600e-03, -4.76564600e-03, -5.13086510e-02, -4.97308180e-02, -6.09664380e-02,
            -1.47074580e-02, -3.72522800e-03, -1.38846600e-03, -1.19005100e-03, -4.65098000e-03,
            -7.33593100e-03, -2.96344800e-03, -3.47094200e-03, -6.68675000e-03, -8.60444000e-05
        ], [
            -4.14411930e-02, -1.37416210e-02, -1.04493000e-02, -6.99531410e-02, -3.54891230e-02,
            -2.52683630e-02, -3.42056640e-02, -1.50738790e-02, 7.69832158e-01, -2.05193359e-01,
            -2.89116920e-02, -1.50032740e-02, -1.25109290e-02, -1.91328990e-02, -1.25837600e-02,
            -3.26817150e-02, -5.77638800e-03, -9.70174400e-03, -7.14978900e-03, -3.38840800e-03,
            -2.42983800e-03, -4.61114700e-03, -3.16290000e-03, -2.79450500e-03, -2.16983300e-03,
            -3.39442100e-03, -1.39087100e-03, -9.14331000e-04, -1.49026000e-03, -4.48936400e-03,
            -7.90469200e-03, -3.33409000e-03, -6.21144700e-02, -9.03895100e-03, -1.40874000e-04
        ], [
            -4.41217700e-03, -8.10249900e-03, -1.43219970e-02, -8.22424000e-03, -3.20648390e-02,
            -9.51537000e-03, -1.36008650e-02, -1.84594600e-03, -1.78498020e-02, 8.95866017e-01,
            -9.84279600e-03, -7.19746300e-03, -2.59603970e-02, -2.56898080e-02, -3.50115530e-02,
            -4.01150870e-02, -2.35931500e-03, -1.76632830e-02, -1.98714720e-02, -4.90668400e-03,
            -5.46881600e-03, -4.34727500e-03, -1.08399340e-02, -2.28575900e-03, -2.52148400e-03,
            -6.25739100e-03, -3.75993800e-03, -7.88450000e-04, -9.39062000e-04, -2.50352700e-03,
            -3.01066800e-03, -7.98776000e-04, -5.38946200e-03, -3.79121500e-03, -9.23900000e-05
        ], [
            -1.99931200e-03, -5.26811200e-03, -5.60743700e-03, -1.86907800e-03, -1.48181700e-03,
            -8.74495700e-03, -1.33050900e-03, -2.16503500e-03, -6.33751800e-03, -5.97473000e-03,
            8.93761872e-01, -8.92049200e-03, -5.51560500e-03, -1.18616620e-02, -6.78703000e-03,
            -6.93804500e-03, -1.93475300e-03, -7.93920060e-02, -2.88965800e-03, -1.65925000e-03,
            -1.09082800e-03, -3.54270300e-03, -1.66686700e-03, -6.68844000e-04, -5.58392000e-04,
            -1.59768600e-03, -1.07123800e-03, -3.21869000e-04, -2.08714000e-03, -9.31003000e-04,
            -1.46712600e-03, -1.51421200e-03, -2.06888900e-03, -2.14784900e-03, -6.80936000e-05
        ], [
            -4.97894400e-03, -2.46960710e-02, -1.51311540e-02, -5.47369400e-03, -1.07071260e-02,
            -2.65188030e-02, -9.09681300e-03, -6.67839900e-03, -1.78100270e-02, -2.36683380e-02,
            -3.13604230e-02, 6.81665290e-01, -1.63482217e-01, -7.87595410e-02, -1.00378673e-01,
            -1.11146266e-01, -7.50772700e-03, -8.39325350e-02, -1.99705240e-02, -5.32627700e-03,
            -4.26973300e-03, -4.75214200e-03, -7.86430500e-03, -1.10181460e-02, -8.30151100e-03,
            -6.85303000e-03, -3.86015000e-03, -8.82027000e-04, -2.56250500e-03, -3.19706700e-03,
            -5.16680900e-03, -1.77709900e-03, -2.54126600e-03, -4.93998800e-03, -3.41272000e-04
        ], [
            -7.15906200e-03, -1.97893720e-02, -3.93078400e-03, -6.36462300e-03, -5.04760000e-03,
            -8.81041300e-03, -7.26190600e-03, -5.04678100e-03, -7.72564400e-03, -1.06140270e-02,
            -1.21670680e-02, -1.60491860e-02, 8.82344234e-01, -1.61338220e-02, -2.91472910e-02,
            -9.52158600e-03, -7.46765000e-03, -1.61480450e-02, -2.18554960e-02, -3.89182900e-03,
            -3.38663000e-03, -2.33306500e-03, -6.36071200e-03, -5.00622800e-03, -3.94669600e-03,
            -4.34659100e-03, -2.93233000e-03, -1.23717800e-03, -1.44046200e-03, -3.05587300e-03,
            -6.25106600e-03, -2.30526400e-03, -2.07539000e-03, -3.54867800e-03, -1.83255000e-04
        ], [
            -1.52114200e-03, -7.11838600e-03, -2.95859300e-03, -4.29943900e-03, -3.45913200e-03,
            -5.69636500e-03, -8.59306400e-03, -3.03411600e-03, -8.24239300e-03, -1.13560890e-02,
            -7.72721200e-03, -1.41780450e-02, -6.82509340e-02, 7.49832343e-01, -4.76611840e-02,
            -1.88866320e-02, -1.19260890e-02, -2.47550170e-02, -2.33438490e-02, -8.51558600e-03,
            -7.50712600e-03, -3.82019600e-03, -5.66432300e-03, -3.02784900e-03, -3.37589600e-03,
            -5.73091500e-03, -2.60546580e-02, -3.05624500e-03, -1.34764000e-03, -1.15179590e-02,
            -1.10388820e-02, -5.12537400e-03, -1.69069830e-02, -6.99991400e-03, -7.04334000e-04
        ], [
            -3.63293000e-03, -5.70745700e-03, -1.68249800e-03, -2.47907500e-03, -2.18207600e-03,
            -2.85622200e-03, -1.82427400e-03, -1.20983000e-03, -1.95159200e-03, -3.15704800e-03,
            -3.47676700e-03, -4.85075400e-03, -1.34387910e-02, -4.76323000e-03, 7.11231886e-01,
            -4.47906200e-03, -1.45511000e-03, -3.85170200e-03, -8.91331900e-02, -5.50514400e-03,
            -4.43568400e-03, -1.23027000e-03, -2.71012240e-02, -3.53235870e-02, -4.66739940e-02,
            -9.47266600e-03, -3.03054200e-03, -9.61046000e-04, -4.79330000e-04, -3.12807800e-03,
            -1.81180910e-02, -9.52987000e-04, -1.32352200e-03, -3.38928500e-03, -1.49565000e-04
        ], [
            -9.14527000e-04, -1.25422500e-03, -1.03769500e-03, -5.65642200e-03, -3.55540500e-03,
            -6.02976200e-03, -3.03828100e-03, -6.48539000e-04, -1.83178700e-03, -2.52796700e-03,
            -3.21451100e-03, -6.87942000e-03, -3.81762100e-03, -3.04123300e-03, -4.94095300e-03,
            9.65669615e-01, -8.99634000e-04, -5.14240800e-03, -2.19417200e-03, -1.75077300e-03,
            -1.79809200e-03, -3.14042900e-03, -1.29791400e-03, -1.12260100e-03, -1.16832800e-03,
            -1.13777900e-03, -8.79809000e-04, -1.22326500e-03, -9.86836000e-04, -2.48722400e-03,
            -3.88473700e-03, -3.35224400e-03, -2.92510700e-03, -5.99562000e-03, -4.11218300e-03
        ], [
            -9.21759900e-03, -2.33302870e-02, -1.17312170e-02, -1.68461130e-02, -7.87993000e-03,
            -1.56563380e-02, -2.06995710e-02, -1.51302160e-02, -2.84424080e-02, -2.15940530e-02,
            -3.68625850e-02, -2.66437440e-02, -1.10373020e-02, -1.00974130e-02, -9.06555900e-03,
            -1.05497610e-02, 9.07625091e-01, -5.09712500e-03, -1.01026730e-02, -6.86054100e-03,
            -1.38977560e-02, -2.36364000e-02, -1.57796590e-02, -4.49871300e-03, -4.02465500e-03,
            -1.29140920e-02, -8.28563900e-03, -4.71604800e-03, -8.34719700e-03, -6.38247300e-03,
            -1.46550650e-02, -1.83223790e-02, -1.29158160e-02, -1.64427520e-02, -2.44617000e-04
        ], [
            -3.67014000e-03, -8.29995900e-03, -2.69237000e-03, -2.64559300e-03, -2.16941000e-03,
            -4.60780800e-03, -4.06959000e-03, -3.75031600e-03, -4.56510500e-03, -3.86711600e-03,
            -7.57054500e-03, -4.95340000e-03, -3.54669500e-03, -3.57798900e-03, -2.46230000e-03,
            -3.54691700e-03, -2.66812510e-02, 9.63577767e-01, -4.67155000e-03, -4.46491500e-03,
            -5.03772600e-03, -5.87033900e-03, -9.75350600e-03, -3.66930400e-03, -2.53694900e-03,
            -1.40843400e-02, -1.26370680e-02, -5.74089300e-03, -4.38081700e-02, -5.77388800e-03,
            -2.12684230e-02, -1.35055250e-02, -6.84881900e-03, -1.34749520e-02, -3.14975300e-03
        ], [
            -5.16213000e-03, -4.44235800e-03, -6.28932300e-03, -5.12886100e-03, -7.23768800e-03,
            -5.94424600e-03, -4.41072700e-03, -3.79948400e-03, -5.99058200e-03, -6.02615500e-03,
            -7.37434400e-03, -5.88838500e-03, -5.04706600e-03, -4.89809800e-03, -4.64226300e-03,
            -6.09328300e-03, -9.56168100e-03, -7.82762200e-03, 9.92072161e-01, -5.56519400e-03,
            -5.14537600e-03, -5.33397100e-03, -1.85036910e-02, -3.57452700e-03, -2.84185400e-03,
            -7.15521900e-03, -4.68879300e-03, -2.43792900e-03, -1.84106900e-03, -5.91403500e-03,
            -3.87152200e-03, -3.04032900e-03, -3.37476500e-03, -6.42295200e-03, -4.56524000e-05
        ], [
            -3.19741750e-02, -2.52176040e-02, -5.82652280e-02, -5.25649370e-02, -6.94703250e-02,
            -5.38555180e-02, -4.79394780e-02, -4.51243690e-02, -4.77820970e-02, -4.84886530e-02,
            -4.35538310e-02, -4.23636700e-02, -4.65339600e-02, -5.21889750e-02, -4.11185110e-02,
            -4.91696100e-02, -2.11951020e-02, -4.27889510e-02, -3.72979500e-02, 9.66832361e-01,
            -1.39913310e-02, -4.42719560e-02, -2.37192710e-02, -1.78465670e-02, -1.81252980e-02,
            -2.15581840e-02, -8.94624100e-03, -5.71808200e-03, -3.36068000e-03, -1.21227670e-02,
            -1.90271980e-02, -9.38630200e-03, -2.69905220e-02, -1.50017910e-02, -3.33733000e-04
        ], [
            -1.71680740e-02, -1.04417080e-02, -2.94509780e-02, -2.59343350e-02, -3.98626330e-02,
            -2.10886520e-02, -1.81901050e-02, -1.57868860e-02, -2.23353720e-02, -2.21774670e-02,
            -1.95260020e-02, -1.61886260e-02, -1.71674470e-02, -1.72959030e-02, -1.80256610e-02,
            -2.63150770e-02, -8.63216000e-03, -2.74079050e-02, -1.74994400e-02, -9.72761400e-03,
            9.90968385e-01, -2.65271900e-02, -1.58320430e-02, -9.39996100e-03, -8.60570300e-03,
            -9.53778500e-03, -9.80460500e-03, -4.20331900e-03, -3.24572500e-03, -9.58832500e-03,
            -7.86226600e-03, -5.78623700e-03, -1.17326190e-02, -1.25572070e-02, -6.37070000e-04
        ], [
            -1.35472900e-03, -3.52130900e-03, -4.01453700e-03, -4.29582900e-03, -3.99717700e-03,
            -4.93017900e-03, -5.68826900e-03, -2.34784200e-03, -5.37577800e-03, -5.72997400e-03,
            -6.01260300e-03, -5.09961900e-03, -6.16424000e-03, -6.70754500e-03, -3.22502600e-03,
            -4.62262900e-03, -4.97444600e-03, -5.37042200e-03, -6.80361100e-03, -9.85261200e-03,
            -6.38963300e-03, 9.90341747e-01, -7.76612400e-03, -6.65935000e-03, -2.37448760e-02,
            -1.93413490e-02, -4.90903400e-03, -1.01062250e-02, -2.35328100e-03, -1.32333580e-02,
            -8.86526600e-03, -6.07893700e-03, -8.26891400e-03, -1.08686430e-02, -1.10140000e-04
        ], [
            -1.50133460e-02, -2.86198640e-02, -2.18792320e-02, -1.78147170e-02, -2.13550130e-02,
            -2.47635820e-02, -1.85477120e-02, -4.42607260e-02, -2.00835200e-02, -1.72027030e-02,
            -3.52063940e-02, -1.82428400e-02, -1.25829790e-02, -1.04937930e-02, -1.19473190e-02,
            -1.87870340e-02, -2.66848660e-02, -1.85049060e-02, -1.33875910e-02, -2.17586780e-02,
            -1.15943970e-02, -1.06053400e-02, 9.58384304e-01, -1.76314520e-02, -1.21138190e-02,
            -3.12444610e-02, -1.13440050e-02, -7.01823100e-03, -1.91411900e-03, -7.32737600e-03,
            -1.18573200e-02, -9.75037000e-03, -7.98969300e-03, -1.10320500e-02, -7.24509000e-04
        ], [
            -2.71456500e-03, -4.43304000e-03, -3.23554900e-03, -2.56741500e-03, -3.36083700e-03,
            -6.19356900e-03, -1.51003900e-03, -5.81041300e-03, -2.98980300e-03, -2.04761500e-03,
            -5.37452700e-03, -3.93068900e-03, -1.62025000e-03, -1.32947400e-03, -1.62424800e-03,
            -1.81507400e-03, -2.43363100e-03, -2.42854000e-03, -1.09155500e-03, -3.03647400e-03,
            -1.45985800e-03, -1.44253800e-03, -3.44821000e-03, 9.01563350e-01, -2.88114200e-03,
            -4.99692100e-03, -1.54933200e-03, -6.96928000e-04, -1.51062000e-04, -9.54653000e-04,
            -1.25011500e-03, -1.14584100e-03, -5.99910000e-04, -1.02445400e-03, -6.41201000e-05
        ], [
            -8.01535000e-04, -2.17519300e-03, -1.49861000e-03, -1.71061000e-03, -2.86603900e-03,
            -1.38556500e-03, -2.16040200e-03, -1.11977900e-03, -1.97696800e-03, -1.87729700e-03,
            -1.50048500e-03, -1.45717200e-03, -1.67500400e-03, -2.13272400e-03, -1.67788300e-03,
            -1.62907900e-03, -8.37357000e-04, -1.11559000e-03, -1.95197900e-03, -4.51236500e-03,
            -1.67838700e-03, -1.20316800e-03, -2.03926600e-03, -4.50521500e-03, 9.69588808e-01,
            -1.66595740e-02, -5.26749300e-03, -3.15606700e-03, -4.54596000e-04, -4.30284700e-03,
            -4.37377000e-03, -2.16948800e-03, -1.30730600e-03, -3.95090400e-03, -2.49721000e-05
        ], [
            -3.67938600e-03, -8.34657400e-03, -6.34081300e-03, -4.00549600e-03, -5.02074100e-03,
            -6.35297100e-03, -7.00011600e-03, -6.99518000e-03, -5.59595100e-03, -5.24501500e-03,
            -9.66293600e-03, -5.59323100e-03, -4.87959400e-03, -4.34588800e-03, -4.20417500e-03,
            -4.75323000e-03, -5.03774600e-03, -3.49527800e-03, -7.39326500e-03, -2.11123830e-02,
            -9.87220000e-03, -7.80684700e-03, -3.59188110e-02, -1.37260249e-01, -9.76386040e-02,
            8.93686791e-01, -3.46519500e-03, -3.84622600e-03, -6.83581000e-04, -4.75957800e-03,
            -3.26517500e-03, -2.36620900e-03, -3.01467100e-03, -5.66267600e-03, -6.42988000e-05
        ], [
            -1.84616800e-03, -4.81831200e-03, -3.31159700e-03, -3.53185000e-03, -3.56549100e-03,
            -3.55659200e-03, -9.25444700e-03, -2.29463300e-03, -5.43030600e-03, -4.64141400e-03,
            -4.18085500e-03, -4.09659300e-03, -5.52646900e-03, -5.69405500e-03, -3.08034400e-03,
            -5.77821100e-03, -4.31198200e-03, -6.13063100e-03, -9.98501500e-03, -1.77440710e-02,
            -1.71538450e-02, -1.06200770e-02, -1.27562580e-02, -1.84521930e-02, -2.85945190e-02,
            -1.87154590e-02, 9.29335704e-01, -2.04155580e-02, -3.16512300e-03, -1.86076710e-02,
            -1.55439390e-02, -6.70432400e-03, -9.40140300e-03, -1.82956510e-02, -1.61843000e-04
        ], [
            -2.01340250e-02, -3.33924060e-02, -1.61274390e-02, -2.24144640e-02, -1.96392470e-02,
            -2.09740240e-02, -2.12169770e-02, -1.50618770e-02, -2.16007990e-02, -1.92603800e-02,
            -2.26134600e-02, -2.10022610e-02, -2.01029740e-02, -1.83925780e-02, -1.42568860e-02,
            -2.23494220e-02, -2.29203910e-02, -1.94387750e-02, -2.60587740e-02, -4.59878370e-02,
            -3.56776550e-02, -2.38824950e-02, -4.10661990e-02, -4.18779080e-02, -3.83408580e-02,
            -2.80737320e-02, -2.42869150e-02, 8.36023118e-01, -4.71131850e-02, -3.51450630e-02,
            -2.37205060e-02, -1.30639270e-02, -2.12673110e-02, -3.26302540e-02, -4.23854500e-03
        ], [
            -7.18451900e-03, -5.64261800e-03, -4.21249100e-03, -5.77872700e-03, -6.71549600e-03,
            -5.29245600e-03, -8.48111200e-03, -2.82983300e-03, -4.99543300e-03, -6.52846500e-03,
            -5.87642400e-03, -5.45742300e-03, -6.48794100e-03, -5.83362400e-03, -4.03879200e-03,
            -9.16450100e-03, -6.45789300e-03, -1.25353120e-02, -2.51101100e-02, -3.36076110e-02,
            -4.46200690e-02, -3.30929500e-02, -1.16107240e-02, -2.85855100e-02, -1.75660970e-02,
            -2.20193620e-02, -1.30453710e-02, -2.64286420e-02, 9.70291095e-01, -2.35685420e-02,
            -1.11246270e-02, -1.81384490e-02, -3.02038170e-02, -2.77705530e-02, -7.47191000e-04
        ], [
            -1.16000350e-02, -2.96608740e-02, -3.42420570e-02, -2.69846890e-02, -2.51165220e-02,
            -2.11225140e-02, -5.38524740e-02, -1.65654080e-02, -5.81262450e-02, -3.36324470e-02,
            -3.15190760e-02, -2.66542470e-02, -4.03290160e-02, -5.62342670e-02, -3.54710210e-02,
            -3.69198720e-02, -3.58138260e-02, -5.11799760e-02, -4.28181610e-02, -6.16761550e-02,
            -6.31015710e-02, -3.26514390e-02, -3.80296270e-02, -5.42762180e-02, -9.28609570e-02,
            -6.16610350e-02, -6.17600970e-02, -8.12557500e-02, -3.15959840e-02, 8.77539555e-01,
            -5.34062260e-02, -2.55613530e-02, -4.76823090e-02, -6.22734730e-02, -2.18640000e-04
        ], [
            -2.49437800e-03, -3.78825100e-03, -2.29677000e-03, -1.77186800e-03, -1.23063800e-03,
            -5.39286300e-03, -4.08355600e-03, -1.83085200e-03, -3.93396400e-03, -2.71140800e-03,
            -3.00056400e-03, -2.99902100e-03, -2.15653300e-03, -2.08951900e-03, -1.87643200e-03,
            -3.25939100e-03, -7.59944000e-03, -2.00836500e-03, -3.25030800e-03, -3.19689600e-03,
            -4.83499300e-03, -4.07674700e-03, -3.80506200e-03, -1.00658010e-02, -1.15963790e-02,
            -4.68575100e-03, -2.81922500e-03, -2.36088300e-03, -4.46757700e-03, -4.75873500e-03,
            9.91073301e-01, -3.91276100e-03, -5.67358100e-03, -1.06930850e-02, -8.64271000e-06
        ], [
            -8.26024000e-04, -4.54732000e-04, -2.78524000e-04, -2.98782000e-04, -3.20412000e-04,
            -2.10336000e-04, -5.40029000e-04, -3.57722000e-04, -6.63323000e-04, -3.70730000e-04,
            -4.52345000e-04, -4.11344000e-04, -6.95833000e-04, -9.00181000e-04, -3.59644000e-04,
            -2.74731000e-04, -6.98912000e-04, -3.96204000e-04, -3.52169000e-04, -9.66145000e-04,
            -1.92897000e-03, -3.70008000e-04, -8.77586000e-04, -2.89613000e-04, -1.54280000e-03,
            -8.58598000e-04, -1.98494800e-03, -1.04605700e-03, -1.33984000e-04, -1.88756000e-03,
            -4.35399800e-03, 9.94572451e-01, -8.97500000e-04, -1.21341500e-03, -7.16322000e-07
        ], [
            -1.85591100e-03, -4.16151000e-04, -3.44550000e-04, -3.96891000e-04, -3.74634000e-04,
            -3.35750000e-04, -3.72334000e-04, -1.83688000e-04, -4.06592000e-04, -3.57481000e-04,
            -4.16871000e-04, -3.29254000e-04, -4.59991000e-04, -4.36183000e-04, -2.49237000e-04,
            -4.37500000e-04, -3.26708000e-04, -4.67006000e-04, -4.58031000e-04, -6.46540000e-04,
            -4.49927000e-04, -9.71287000e-04, -6.58392000e-04, -8.30341000e-04, -5.02966000e-04,
            -5.60183000e-04, -4.88003000e-04, -5.35000000e-04, -8.40707000e-05, -6.14651000e-04,
            -6.16584400e-03, -1.37985700e-03, 9.79604544e-01, -1.39360300e-03, -6.53207000e-08
        ], [
            -2.85682300e-03, -6.54476900e-03, -6.78245200e-03, -5.32237000e-03, -6.46488500e-03,
            -6.18954300e-03, -1.29099140e-02, -3.77392800e-03, -8.79523500e-03, -7.14829000e-03,
            -6.07422200e-03, -7.35663400e-03, -6.52029000e-03, -7.19023800e-03, -4.45224800e-03,
            -7.13745800e-03, -5.26693200e-03, -5.88957900e-03, -9.39745500e-03, -9.99130300e-03,
            -1.14559760e-02, -1.34061090e-02, -1.02126820e-02, -7.98831200e-03, -9.69681300e-03,
            -1.13298320e-02, -1.78657940e-02, -1.09787160e-02, -7.41925400e-03, -2.93237450e-02,
            -1.70132380e-02, -1.32139010e-02, -1.57177230e-02, 9.28542903e-01, -6.03860000e-04
        ], [
            -1.09748000e-05, -1.62163000e-05, -3.36440000e-05, -3.99474000e-05, -2.78826000e-05,
            -3.61057000e-05, -4.10564000e-05, -1.53774000e-05, -2.99816000e-05, -3.54221000e-05,
            -2.40688000e-05, -3.54234000e-05, -3.18596000e-05, -2.17691000e-05, -4.58056000e-05,
            -4.05146000e-05, -1.61634000e-05, -8.94356000e-05, -1.55791000e-04, -5.69875000e-05,
            -7.37625000e-05, -7.50137000e-05, -1.73589000e-04, -2.76345000e-05, -4.22151000e-05,
            -3.28437000e-05, -9.18283000e-05, -6.75954000e-05, -2.89781000e-04, -7.64379000e-05,
            -1.00741000e-04, -5.50673000e-05, -1.26119000e-04, -1.27429000e-04, 9.99972073e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 382)
    return ret


@cache('run')
def ia_matrix_x1996x():
    """
    ia matrix x1996x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.70340047e-01, -2.89655800e-03, -2.72639635e-01, -4.03552090e-02, -2.94822670e-02,
            -1.37980540e-01, -1.17443400e-02, -2.88679700e-03, -7.03027300e-03, -1.29181510e-02,
            -1.11729700e-03, -3.17862000e-04, -3.17932000e-04, -2.41614000e-04, -1.94006000e-04,
            -1.17108050e-02, -5.07344000e-04, -3.25173700e-03, -5.54363000e-04, -2.36507800e-03,
            -2.58115600e-03, -3.77338880e-02, -7.12811000e-04, -2.05385200e-03, -4.38430000e-04,
            -1.17113700e-03, -2.10216000e-04, -1.72263000e-04, -5.25149000e-04, -1.11142400e-03,
            -1.83796600e-03, -1.97965100e-03, -2.42926800e-03, -2.35734500e-03, -1.52277700e-02
        ], [
            -3.71855600e-03, 9.02686674e-01, -3.58988700e-03, -2.28638300e-03, -1.37105100e-03,
            -3.84541700e-03, -5.39214600e-03, -4.36414845e-01, -3.30476050e-02, -3.80495600e-03,
            -7.83925890e-02, -4.52001890e-02, -3.65140100e-03, -2.42643500e-03, -1.58742700e-03,
            -7.57228100e-03, -9.89919930e-02, -2.01192980e-02, -7.20529000e-04, -1.78655700e-03,
            -8.08648000e-04, -2.65920600e-03, -7.09566700e-03, -2.99664500e-03, -2.96443500e-03,
            -4.14224800e-03, -5.41520000e-04, -4.89003000e-04, -2.06372000e-03, -1.09119500e-03,
            -2.94056300e-03, -3.78399300e-03, -1.55811100e-03, -3.52308600e-03, -6.41444240e-02
        ], [
            -5.33737750e-02, -8.04982000e-04, 8.60990071e-01, -2.31154100e-03, -8.34375030e-02,
            -1.16862400e-03, -2.77742800e-03, -9.53989000e-04, -9.71160700e-03, -2.36833100e-03,
            -1.85279000e-03, -9.30163000e-04, -1.12338700e-03, -1.10394300e-03, -7.94550000e-04,
            -1.58651900e-03, -7.36104000e-04, -1.00046800e-03, -1.62166700e-03, -4.61825600e-03,
            -4.40440500e-03, -1.61299227e-01, -1.60649300e-03, -6.79571600e-03, -4.32281600e-03,
            -2.45388400e-03, -1.51664100e-03, -8.03836000e-04, -2.19753000e-04, -2.29153600e-03,
            -8.04227200e-03, -9.03260700e-03, -1.43242200e-02, -5.31845600e-03, -1.26068200e-03
        ], [
            -2.41846200e-03, -1.54642600e-03, -1.48578200e-03, 7.14668847e-01, -5.42134320e-02,
            -5.88599300e-03, -7.26785000e-03, -5.34526000e-04, -4.17253000e-03, -1.98405870e-02,
            -5.14490900e-03, -1.65699700e-03, -2.46489700e-03, -2.27589100e-03, -6.17053600e-03,
            -2.49272390e-02, -5.59012000e-04, -2.73246400e-03, -1.80495800e-03, -3.01044900e-03,
            -3.67547600e-03, -3.25178100e-03, -1.74469300e-03, -3.11809200e-03, -1.28741000e-03,
            -1.79326300e-03, -2.86994200e-03, -6.65858000e-04, -2.63621000e-04, -1.36993700e-03,
            -3.29581100e-03, -8.10695000e-04, -3.56445100e-03, -4.93945000e-03, -5.08203000e-04
        ], [
            -2.46407000e-04, -3.28695000e-04, -2.07026000e-04, -6.66782200e-03, 8.11140039e-01,
            -1.12923600e-03, -1.33403000e-03, -1.09040000e-04, -2.91481000e-04, -1.04097800e-03,
            -6.52049000e-04, -4.35987000e-04, -4.47200000e-04, -4.10281000e-04, -1.22940100e-03,
            -3.68610800e-03, -2.13053000e-04, -1.81860000e-04, -4.22446000e-04, -5.06424000e-04,
            -5.35474000e-04, -2.34643000e-04, -2.17696000e-04, -2.11960000e-04, -1.48200000e-04,
            -1.73049000e-04, -6.77218000e-04, -7.91158000e-05, -2.14835000e-05, -1.62299000e-04,
            -8.89472000e-04, -1.50069000e-04, -1.72596000e-04, -7.34594000e-04, -1.40127000e-05
        ], [
            -1.62649500e-03, -1.98762500e-03, -1.51215400e-03, -1.53434800e-03, -2.22086700e-03,
            8.12358622e-01, -1.11154580e-02, -4.21848000e-04, -1.57342400e-03, -2.68443000e-03,
            -6.66533700e-03, -2.52589800e-03, -2.89714200e-03, -1.87319400e-03, -3.11117500e-03,
            -7.29521910e-02, -8.31569000e-04, -3.55833190e-02, -8.16670000e-04, -1.71679400e-03,
            -1.53205700e-03, -2.30670200e-03, -1.37679000e-03, -5.95805000e-04, -4.30067000e-04,
            -4.34358900e-03, -7.30557000e-04, -6.25581000e-04, -2.23838200e-03, -9.36208000e-04,
            -1.38655900e-03, -1.14453400e-03, -9.40626000e-04, -2.83003000e-03, -3.02171000e-04
        ], [
            -2.83359600e-03, -3.38435100e-03, -2.40694790e-02, -1.01893540e-02, -1.27073450e-02,
            -1.26769820e-02, 7.64873577e-01, -1.67600600e-03, -1.97520920e-02, -1.84329810e-02,
            -2.32279180e-02, -6.05178100e-03, -1.00457100e-02, -1.55448100e-02, -5.94684600e-03,
            -2.22895900e-02, -3.47940700e-03, -5.32657900e-03, -7.97565300e-03, -1.51469090e-02,
            -1.50114430e-02, -1.10580550e-02, -6.20552100e-03, -4.39382500e-03, -7.65488400e-03,
            -1.63011400e-02, -1.14814050e-02, -1.62277200e-02, -2.67781900e-03, -2.27375930e-02,
            -1.63930560e-02, -1.62997890e-02, -9.91971700e-03, -2.47799700e-02, -6.94923000e-04
        ], [
            -1.21708180e-02, -1.35575680e-02, -3.62104800e-03, -4.81641100e-03, -3.23979400e-03,
            -4.76086600e-03, -4.57062700e-03, 9.54789826e-01, -2.62594450e-02, -8.58899000e-03,
            -1.60021310e-02, -8.92975900e-03, -3.67193200e-03, -3.06361900e-03, -2.04829700e-03,
            -4.44544400e-03, -1.94984040e-02, -1.27730200e-02, -4.42553000e-03, -6.39371800e-03,
            -5.40968100e-03, -3.93684300e-03, -5.10776110e-02, -5.12532680e-02, -5.62984410e-02,
            -1.47204020e-02, -3.45236000e-03, -1.26998400e-03, -1.17175000e-03, -4.48890900e-03,
            -7.47928400e-03, -3.00309600e-03, -3.41003600e-03, -6.29934500e-03, -7.94193000e-05
        ], [
            -4.02089790e-02, -1.38239370e-02, -1.06544720e-02, -7.08722450e-02, -3.41498380e-02,
            -2.63943820e-02, -3.39563040e-02, -1.41711100e-02, 7.69512650e-01, -1.99881221e-01,
            -3.07828700e-02, -1.54116300e-02, -1.26464840e-02, -1.92200200e-02, -1.28513260e-02,
            -3.30055120e-02, -5.83645300e-03, -9.84446400e-03, -7.05564300e-03, -3.45727400e-03,
            -2.53766300e-03, -4.53409200e-03, -3.35039000e-03, -3.06005100e-03, -2.18490800e-03,
            -3.45521800e-03, -1.42089100e-03, -8.85157000e-04, -1.53371200e-03, -4.41479000e-03,
            -8.23775300e-03, -3.67052500e-03, -6.14578570e-02, -9.12438600e-03, -1.40615000e-04
        ], [
            -4.38620300e-03, -8.64206900e-03, -1.46696300e-02, -8.21640000e-03, -3.35166190e-02,
            -9.57960400e-03, -1.42285490e-02, -1.76510500e-03, -1.80040240e-02, 8.90695569e-01,
            -1.07133060e-02, -7.49338800e-03, -2.60821330e-02, -2.62431180e-02, -3.50303570e-02,
            -3.82087230e-02, -2.27886800e-03, -1.77702970e-02, -1.85726370e-02, -4.72059900e-03,
            -5.30347300e-03, -4.22589400e-03, -1.15018800e-02, -2.28463800e-03, -2.43012700e-03,
            -6.21790400e-03, -3.77119700e-03, -7.93289000e-04, -9.77558000e-04, -2.56074700e-03,
            -3.00129800e-03, -8.63652000e-04, -5.38612500e-03, -3.38766600e-03, -9.77402000e-05
        ], [
            -1.87914700e-03, -5.11069800e-03, -5.41338000e-03, -1.93528700e-03, -1.60648400e-03,
            -9.36805300e-03, -1.39705700e-03, -1.98384600e-03, -6.52722500e-03, -6.06408400e-03,
            8.94245647e-01, -9.44097000e-03, -5.61935600e-03, -1.20207980e-02, -6.91765300e-03,
            -6.89996200e-03, -1.80679900e-03, -7.98095760e-02, -2.86898800e-03, -1.55484300e-03,
            -1.05188000e-03, -3.58898100e-03, -1.44171900e-03, -5.42678000e-04, -5.27728000e-04,
            -1.36458800e-03, -1.07568200e-03, -2.53746000e-04, -2.12181600e-03, -9.67686000e-04,
            -1.48335000e-03, -1.64010400e-03, -2.00488900e-03, -1.87183800e-03, -7.98460000e-05
        ], [
            -4.94188800e-03, -2.58255780e-02, -1.49448720e-02, -5.58665800e-03, -1.04255940e-02,
            -2.71576160e-02, -9.12746900e-03, -6.03700300e-03, -1.76530910e-02, -2.35607740e-02,
            -3.20702180e-02, 6.85389778e-01, -1.60905187e-01, -7.66866150e-02, -1.00230074e-01,
            -1.08767456e-01, -7.47818200e-03, -8.50537010e-02, -1.98382840e-02, -5.09037800e-03,
            -4.13713300e-03, -4.67120400e-03, -8.00910500e-03, -1.08288300e-02, -8.00819600e-03,
            -6.76396200e-03, -3.87445100e-03, -9.10141000e-04, -2.58602200e-03, -3.38361900e-03,
            -5.02712400e-03, -1.88656800e-03, -2.68755000e-03, -4.50100000e-03, -3.24620000e-04
        ], [
            -7.02948100e-03, -1.92389100e-02, -3.85439000e-03, -6.36539400e-03, -4.90491600e-03,
            -8.67016500e-03, -7.05803100e-03, -4.44504400e-03, -7.64638400e-03, -1.05583490e-02,
            -1.24298300e-02, -1.60898310e-02, 8.81238648e-01, -1.52148060e-02, -2.87739030e-02,
            -9.39545900e-03, -7.47245900e-03, -1.60472270e-02, -2.37566510e-02, -3.92731400e-03,
            -3.20322000e-03, -2.27660200e-03, -6.62327300e-03, -5.48836800e-03, -4.20780200e-03,
            -4.65298800e-03, -2.96222900e-03, -1.22410500e-03, -1.42538400e-03, -3.07896800e-03,
            -6.41473500e-03, -2.14206800e-03, -2.19586600e-03, -3.58241600e-03, -1.79988000e-04
        ], [
            -1.52322100e-03, -6.88872600e-03, -3.17147500e-03, -4.63535600e-03, -3.48884200e-03,
            -6.30229300e-03, -9.16404600e-03, -2.88619100e-03, -8.76574500e-03, -1.19213070e-02,
            -8.51808300e-03, -1.55510740e-02, -7.40598220e-02, 7.46152961e-01, -4.91450000e-02,
            -2.06658800e-02, -1.22820660e-02, -2.57822280e-02, -2.44691910e-02, -8.97157700e-03,
            -7.81768700e-03, -4.08942400e-03, -6.20242000e-03, -3.19893800e-03, -3.61549200e-03,
            -6.55234600e-03, -2.77168680e-02, -3.55538600e-03, -1.49335900e-03, -1.24755030e-02,
            -1.16854670e-02, -5.54419700e-03, -1.80214590e-02, -7.66001600e-03, -6.58983000e-04
        ], [
            -3.73853100e-03, -5.69781500e-03, -1.73061000e-03, -2.53599300e-03, -2.26139500e-03,
            -2.88074300e-03, -1.86032000e-03, -1.01817500e-03, -2.04571600e-03, -3.30515800e-03,
            -3.52510900e-03, -4.73007300e-03, -1.35421580e-02, -4.75098100e-03, 7.08610081e-01,
            -4.34900300e-03, -1.48744200e-03, -3.84243500e-03, -8.92999660e-02, -5.34047400e-03,
            -4.42361200e-03, -1.26206500e-03, -2.90366970e-02, -3.97421670e-02, -4.49333380e-02,
            -1.01927400e-02, -2.94439300e-03, -8.66372000e-04, -4.63256000e-04, -3.05530100e-03,
            -1.79957270e-02, -9.89197000e-04, -1.36483600e-03, -3.26006400e-03, -1.26873000e-04
        ], [
            -9.05640000e-04, -1.43316200e-03, -1.13957000e-03, -5.55736300e-03, -3.54924300e-03,
            -6.30201700e-03, -3.14445500e-03, -5.58260000e-04, -1.82999800e-03, -2.47004000e-03,
            -3.45253500e-03, -7.02299400e-03, -3.88486400e-03, -3.18288200e-03, -5.11665800e-03,
            9.64310556e-01, -1.03776200e-03, -4.97372500e-03, -2.23600500e-03, -1.60551900e-03,
            -1.67746900e-03, -2.87615400e-03, -1.43033500e-03, -1.03703800e-03, -1.17315700e-03,
            -1.40445000e-03, -9.35553000e-04, -1.21316700e-03, -1.06574200e-03, -2.29099500e-03,
            -3.92990700e-03, -3.63015300e-03, -2.90406600e-03, -5.95657500e-03, -3.62851600e-03
        ], [
            -9.35334000e-03, -2.49532870e-02, -1.16245470e-02, -1.65080770e-02, -7.63212800e-03,
            -1.57395750e-02, -2.16684000e-02, -1.42530460e-02, -2.91090130e-02, -2.17319540e-02,
            -3.68986800e-02, -2.74405740e-02, -1.12961180e-02, -1.03996440e-02, -9.00922300e-03,
            -1.12389500e-02, 9.06563328e-01, -5.13480200e-03, -1.05533140e-02, -6.87308200e-03,
            -1.41285670e-02, -2.34804760e-02, -1.57312080e-02, -4.17514500e-03, -4.30110600e-03,
            -1.36183740e-02, -8.28930500e-03, -4.64171400e-03, -8.35671300e-03, -6.56370900e-03,
            -1.51751700e-02, -1.94922220e-02, -1.33911490e-02, -1.60758510e-02, -2.36234000e-04
        ], [
            -3.63017500e-03, -8.06058900e-03, -2.53073800e-03, -2.58394800e-03, -1.98190300e-03,
            -4.49470800e-03, -4.02008900e-03, -3.18725200e-03, -4.48788200e-03, -3.78754800e-03,
            -7.81841000e-03, -5.08144900e-03, -3.44583300e-03, -3.53371000e-03, -2.37183800e-03,
            -3.32068600e-03, -2.65042010e-02, 9.64834982e-01, -4.34022100e-03, -4.39403300e-03,
            -5.00887800e-03, -5.56273400e-03, -9.90016400e-03, -3.53927700e-03, -2.33723300e-03,
            -1.53230270e-02, -1.17908930e-02, -5.62398600e-03, -4.49934100e-02, -5.62191700e-03,
            -2.08006320e-02, -1.44869710e-02, -7.04423800e-03, -1.30822050e-02, -2.65612300e-03
        ], [
            -5.01522400e-03, -4.21434700e-03, -6.31625700e-03, -5.15444900e-03, -7.23077400e-03,
            -6.15921500e-03, -4.48428000e-03, -3.14137500e-03, -6.20455200e-03, -6.24412700e-03,
            -7.70004200e-03, -6.28700100e-03, -5.28084600e-03, -5.28703100e-03, -4.66005100e-03,
            -6.19191300e-03, -9.15027500e-03, -7.71991000e-03, 9.91635823e-01, -4.52428200e-03,
            -4.62699200e-03, -5.39558000e-03, -1.83307240e-02, -3.61861600e-03, -2.82256500e-03,
            -7.49521800e-03, -4.68928000e-03, -2.49557900e-03, -1.86534800e-03, -6.16004900e-03,
            -3.99953700e-03, -3.27276600e-03, -3.32832400e-03, -6.59682700e-03, -4.79614000e-05
        ], [
            -3.28078450e-02, -2.64584460e-02, -6.01108500e-02, -5.15955120e-02, -6.99411010e-02,
            -5.31737170e-02, -4.80845460e-02, -3.80120490e-02, -4.81843470e-02, -4.99857690e-02,
            -4.40081800e-02, -4.23373000e-02, -4.80403010e-02, -5.22829620e-02, -4.21455730e-02,
            -5.15928440e-02, -2.27154860e-02, -4.33796640e-02, -3.88607740e-02, 9.66101321e-01,
            -1.38534010e-02, -4.41972400e-02, -2.27088190e-02, -1.78505770e-02, -1.78637630e-02,
            -1.95392500e-02, -9.20532900e-03, -4.96299800e-03, -3.23827000e-03, -1.19693100e-02,
            -2.00590660e-02, -9.90876800e-03, -2.74347450e-02, -1.41334380e-02, -3.24585000e-04
        ], [
            -1.67657280e-02, -1.07865130e-02, -3.00146550e-02, -2.59580160e-02, -3.97472640e-02,
            -2.11607730e-02, -1.80543190e-02, -1.36718970e-02, -2.24634560e-02, -2.20352530e-02,
            -1.95254140e-02, -1.60265790e-02, -1.73205410e-02, -1.68111050e-02, -1.84780440e-02,
            -2.62786590e-02, -8.20198800e-03, -2.80088780e-02, -1.76729230e-02, -9.21018100e-03,
            9.91301399e-01, -2.78569950e-02, -1.53421320e-02, -9.09147400e-03, -7.96663100e-03,
            -9.62069800e-03, -1.00777050e-02, -4.10414200e-03, -3.33225800e-03, -9.55550700e-03,
            -7.92394600e-03, -5.91700200e-03, -1.17291190e-02, -1.26605720e-02, -6.34183000e-04
        ], [
            -1.32276100e-03, -3.60413700e-03, -3.74206400e-03, -4.21066800e-03, -3.96724700e-03,
            -5.19313500e-03, -5.53594900e-03, -1.69424900e-03, -5.16136800e-03, -5.77181000e-03,
            -5.93887200e-03, -5.12862500e-03, -6.27245800e-03, -6.65444500e-03, -3.16871100e-03,
            -4.61691700e-03, -4.60066000e-03, -5.08912700e-03, -6.54282800e-03, -9.53343300e-03,
            -6.23720700e-03, 9.90132968e-01, -7.96721800e-03, -6.22425500e-03, -2.26437320e-02,
            -2.02804920e-02, -7.92611800e-03, -1.02051380e-02, -2.34562000e-03, -1.36189020e-02,
            -9.22085500e-03, -6.85441300e-03, -8.19961500e-03, -1.10875290e-02, -1.01520000e-04
        ], [
            -1.40852380e-02, -2.83482680e-02, -2.17519760e-02, -1.80129950e-02, -2.07858000e-02,
            -2.44926940e-02, -1.82017430e-02, -3.45114890e-02, -1.96703820e-02, -1.73026140e-02,
            -3.55085170e-02, -1.82004290e-02, -1.24428950e-02, -1.00599330e-02, -1.17965010e-02,
            -1.84679470e-02, -2.53184410e-02, -1.77919950e-02, -1.38663450e-02, -1.92038140e-02,
            -1.07434220e-02, -1.03006680e-02, 9.58088681e-01, -1.87712510e-02, -1.14536670e-02,
            -3.09313050e-02, -9.78883100e-03, -6.44585200e-03, -1.74783300e-03, -6.96929600e-03,
            -1.16834310e-02, -1.00807190e-02, -7.73308500e-03, -9.96273400e-03, -7.34395000e-04
        ], [
            -2.67132900e-03, -4.15413800e-03, -3.63067200e-03, -2.87063700e-03, -3.64353400e-03,
            -6.98938500e-03, -1.61189600e-03, -4.46816900e-03, -2.94171600e-03, -2.18220400e-03,
            -5.13270900e-03, -3.98125400e-03, -1.56830400e-03, -1.42926500e-03, -1.61918900e-03,
            -2.12194400e-03, -2.02299000e-03, -2.46330400e-03, -1.11354100e-03, -2.55951000e-03,
            -1.36867500e-03, -1.59058800e-03, -3.32748900e-03, 8.99123637e-01, -2.90532100e-03,
            -5.73824900e-03, -1.16967300e-03, -6.55583000e-04, -1.28525000e-04, -1.00392800e-03,
            -1.42711000e-03, -1.52324800e-03, -5.69985000e-04, -9.18254000e-04, -6.69162000e-05
        ], [
            -8.26701000e-04, -2.49519700e-03, -1.58259200e-03, -1.76971500e-03, -2.83779900e-03,
            -1.43694400e-03, -2.21276000e-03, -8.96282000e-04, -2.06443000e-03, -1.91882400e-03,
            -1.58350100e-03, -1.53223600e-03, -1.88122700e-03, -2.25674400e-03, -1.84202600e-03,
            -1.69724100e-03, -8.43277000e-04, -1.23061400e-03, -2.15401200e-03, -4.37273100e-03,
            -1.74811300e-03, -1.24944500e-03, -2.15968700e-03, -4.60322000e-03, 9.67727476e-01,
            -1.92855520e-02, -5.25849000e-03, -3.45257300e-03, -5.05297000e-04, -4.48061200e-03,
            -4.66076300e-03, -2.21281300e-03, -1.35262300e-03, -4.04379800e-03, -2.17695000e-05
        ], [
            -3.35493500e-03, -7.00174400e-03, -6.02676400e-03, -3.89947800e-03, -4.75147600e-03,
            -6.48427000e-03, -6.95810300e-03, -6.68578200e-03, -5.43635200e-03, -5.32363500e-03,
            -9.48070900e-03, -5.73258500e-03, -4.78550600e-03, -4.18373900e-03, -4.16943500e-03,
            -4.91782300e-03, -5.02445100e-03, -3.42870700e-03, -7.52364300e-03, -1.96353190e-02,
            -9.59546400e-03, -8.14973500e-03, -3.69935560e-02, -1.40839805e-01, -9.64912240e-02,
            8.87023623e-01, -3.48517600e-03, -3.81434000e-03, -6.53910000e-04, -4.89648200e-03,
            -4.47318300e-03, -2.42898200e-03, -2.99607500e-03, -5.20542100e-03, -6.89827000e-05
        ], [
            -1.91589800e-03, -5.29348300e-03, -3.27810800e-03, -3.61386500e-03, -3.71690500e-03,
            -3.80288400e-03, -9.60482700e-03, -2.08967200e-03, -5.72946800e-03, -4.72524400e-03,
            -4.60478700e-03, -4.49804200e-03, -5.88269300e-03, -5.89171500e-03, -3.17996300e-03,
            -6.03127400e-03, -4.46341100e-03, -7.51547500e-03, -1.06998170e-02, -1.77518450e-02,
            -1.76130170e-02, -1.07294290e-02, -1.33286430e-02, -1.94581000e-02, -2.81623410e-02,
            -2.07877320e-02, 9.19776804e-01, -2.13136190e-02, -3.49919000e-03, -1.93264750e-02,
            -1.61875180e-02, -7.35737600e-03, -9.70270900e-03, -1.95811970e-02, -1.43597000e-04
        ], [
            -1.88279240e-02, -3.20386340e-02, -1.56497310e-02, -2.14250270e-02, -1.84480760e-02,
            -2.16388540e-02, -2.11343030e-02, -1.24329750e-02, -2.12986810e-02, -1.90133750e-02,
            -2.36214090e-02, -2.13744340e-02, -2.00745170e-02, -1.81882440e-02, -1.40494330e-02,
            -2.14107560e-02, -2.26998860e-02, -1.98338260e-02, -2.60597690e-02, -4.30392730e-02,
            -3.57279430e-02, -2.33550660e-02, -4.12109000e-02, -4.03561060e-02, -3.71513680e-02,
            -2.73690630e-02, -2.29817510e-02, 8.29972362e-01, -4.89432800e-02, -3.52202220e-02,
            -2.28484930e-02, -1.30216100e-02, -2.14768780e-02, -3.16771410e-02, -3.65031000e-03
        ], [
            -6.68476500e-03, -5.49092700e-03, -4.10209900e-03, -5.42833600e-03, -5.77926500e-03,
            -5.30694000e-03, -8.28307300e-03, -2.47934900e-03, -4.78333100e-03, -6.18481300e-03,
            -5.42795800e-03, -5.27276600e-03, -6.28131800e-03, -5.44065100e-03, -3.94782500e-03,
            -8.74453900e-03, -5.70100800e-03, -1.26676000e-02, -2.46429280e-02, -2.91749900e-02,
            -4.29421120e-02, -2.87488690e-02, -1.06983910e-02, -2.76885210e-02, -1.63908280e-02,
            -2.12820130e-02, -1.20241020e-02, -2.50733020e-02, 9.69908969e-01, -2.36992820e-02,
            -1.09451950e-02, -1.75051790e-02, -2.93671140e-02, -2.78209130e-02, -6.25306000e-04
        ], [
            -1.17851020e-02, -2.98761390e-02, -3.51173010e-02, -2.65848900e-02, -2.49166270e-02,
            -2.25501230e-02, -5.50703420e-02, -1.51137380e-02, -5.81793420e-02, -3.36193180e-02,
            -3.24563890e-02, -2.72559630e-02, -4.18765730e-02, -5.82297100e-02, -3.55283590e-02,
            -3.65255020e-02, -3.46433970e-02, -5.10828260e-02, -4.29725110e-02, -5.86967630e-02,
            -6.31592650e-02, -3.26790280e-02, -3.87653150e-02, -5.34899010e-02, -9.15498790e-02,
            -6.37052870e-02, -6.29431370e-02, -8.49579750e-02, -3.33754720e-02, 8.73411255e-01,
            -5.43430210e-02, -2.56198290e-02, -4.94820310e-02, -6.33675690e-02, -2.14075000e-04
        ], [
            -1.71345200e-03, -3.66117300e-03, -1.85293700e-03, -1.22896400e-03, -9.33599000e-04,
            -3.18161700e-03, -3.20031200e-03, -1.15410900e-03, -3.14655400e-03, -2.20728600e-03,
            -2.44883500e-03, -2.46777000e-03, -1.37636400e-03, -1.63094000e-03, -1.21022300e-03,
            -2.48023600e-03, -5.46099900e-03, -1.49161000e-03, -2.34958400e-03, -1.99306100e-03,
            -3.73726400e-03, -3.66800300e-03, -3.24996300e-03, -8.84485400e-03, -9.02373800e-03,
            -3.89364700e-03, -2.65554700e-03, -1.68398700e-03, -3.62626900e-03, -2.82865100e-03,
            9.92225223e-01, -3.90622600e-03, -5.28722700e-03, -7.79486900e-03, -2.87498000e-06
        ], [
            -8.07044000e-04, -4.75993000e-04, -3.40113000e-04, -3.63190000e-04, -3.66291000e-04,
            -2.59773000e-04, -6.21323000e-04, -4.25021000e-04, -8.17771000e-04, -4.66198000e-04,
            -5.03941000e-04, -4.76328000e-04, -7.83265000e-04, -9.51313000e-04, -4.69793000e-04,
            -3.41829000e-04, -1.01819300e-03, -4.81609000e-04, -4.27017000e-04, -9.60798000e-04,
            -1.93564000e-03, -4.16410000e-04, -1.07320000e-03, -3.46020000e-04, -1.74799300e-03,
            -1.10361600e-03, -1.61626600e-03, -1.25978400e-03, -1.72764000e-04, -2.14162200e-03,
            -4.43864600e-03, 9.93594615e-01, -1.05306700e-03, -1.40268200e-03, -8.32451000e-07
        ], [
            -2.02869400e-03, -3.93451000e-04, -3.87978000e-04, -4.27931000e-04, -3.93002000e-04,
            -3.70815000e-04, -4.21959000e-04, -1.93740000e-04, -4.33264000e-04, -3.85758000e-04,
            -4.35331000e-04, -3.62063000e-04, -5.15120000e-04, -4.51149000e-04, -2.95970000e-04,
            -5.08409000e-04, -4.11754000e-04, -5.20905000e-04, -5.21361000e-04, -6.64020000e-04,
            -4.71231000e-04, -1.01670000e-03, -6.88203000e-04, -9.10047000e-04, -5.62589000e-04,
            -6.85200000e-04, -7.75819000e-04, -6.04241000e-04, -1.00222000e-04, -7.85110000e-04,
            -6.17086900e-03, -1.42533400e-03, 9.77709530e-01, -1.64383400e-03, -1.02927000e-07
        ], [
            -2.41573300e-03, -6.08118200e-03, -6.28652600e-03, -5.38850100e-03, -6.30551600e-03,
            -5.57368200e-03, -1.30087880e-02, -3.52743600e-03, -8.28376900e-03, -6.00050100e-03,
            -6.05061400e-03, -7.11990700e-03, -6.64209300e-03, -6.25037600e-03, -4.25327600e-03,
            -6.80361500e-03, -6.24532200e-03, -6.45160000e-03, -1.00998070e-02, -9.90124300e-03,
            -1.11468240e-02, -1.39515060e-02, -9.81524900e-03, -7.06116000e-03, -9.62364500e-03,
            -1.14737280e-02, -1.79020880e-02, -1.10897160e-02, -7.68608900e-03, -2.94085260e-02,
            -1.55232310e-02, -1.03900650e-02, -1.55511250e-02, 9.21347411e-01, -4.93677000e-04
        ], [
            -1.06019000e-05, -1.51742000e-05, -3.21763000e-05, -3.93540000e-05, -2.83863000e-05,
            -3.55654000e-05, -3.71680000e-05, -1.39332000e-05, -3.03032000e-05, -3.47757000e-05,
            -2.49402000e-05, -3.42023000e-05, -3.14170000e-05, -2.10951000e-05, -4.03277000e-05,
            -3.57644000e-05, -1.65735000e-05, -8.50735000e-05, -1.38434000e-04, -4.47814000e-05,
            -6.21993000e-05, -6.92429000e-05, -1.65706000e-04, -2.85625000e-05, -3.80083000e-05,
            -3.43312000e-05, -8.41192000e-05, -6.59996000e-05, -2.83537000e-04, -7.43580000e-05,
            -9.01757000e-05, -5.44688000e-05, -1.28192000e-04, -1.23181000e-04, 9.99974904e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 383)
    return ret


@cache('run')
def ia_matrix_x1997x():
    """
    ia matrix x1997x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.71602101e-01, -2.88089000e-03, -2.68910601e-01, -3.86902020e-02, -2.98296020e-02,
            -1.32857977e-01, -1.16068700e-02, -3.12973100e-03, -7.08906700e-03, -1.28918350e-02,
            -1.01811400e-03, -3.03137000e-04, -3.03133000e-04, -2.28253000e-04, -1.81488000e-04,
            -1.19703300e-02, -4.78186000e-04, -3.04915000e-03, -4.93111000e-04, -2.19627100e-03,
            -2.38149400e-03, -3.70651430e-02, -6.82396000e-04, -1.97686400e-03, -4.42483000e-04,
            -1.15739800e-03, -1.99273000e-04, -1.76779000e-04, -5.18675000e-04, -1.10539100e-03,
            -1.78707200e-03, -1.89981900e-03, -2.34746000e-03, -2.34285200e-03, -1.30284690e-02
        ], [
            -3.44592600e-03, 9.13546916e-01, -3.43435200e-03, -2.06580500e-03, -1.31667900e-03,
            -3.56408300e-03, -5.24390700e-03, -4.26213558e-01, -3.23644680e-02, -3.54443700e-03,
            -7.75344270e-02, -4.16812880e-02, -3.37092700e-03, -2.03672800e-03, -1.48267700e-03,
            -6.82363300e-03, -9.51042840e-02, -1.80982650e-02, -6.35543000e-04, -1.48654300e-03,
            -8.57415000e-04, -2.24075000e-03, -6.12124600e-03, -2.91212000e-03, -2.73783000e-03,
            -3.56269000e-03, -5.09548000e-04, -4.10769000e-04, -1.96359200e-03, -1.03709800e-03,
            -2.72039900e-03, -3.80213900e-03, -1.50178300e-03, -3.09685600e-03, -5.48802350e-02
        ], [
            -5.43333350e-02, -8.02140000e-04, 8.61354218e-01, -2.30603300e-03, -8.50710370e-02,
            -1.26228800e-03, -2.79000400e-03, -1.01949600e-03, -9.66361200e-03, -2.45951200e-03,
            -1.90852300e-03, -9.67204000e-04, -1.17188800e-03, -1.15891800e-03, -8.17001000e-04,
            -1.55390500e-03, -7.82009000e-04, -1.05108300e-03, -1.69371800e-03, -4.55339000e-03,
            -4.33724500e-03, -1.59100006e-01, -1.61340300e-03, -7.33783800e-03, -4.58550800e-03,
            -2.44286300e-03, -1.39984800e-03, -8.30922000e-04, -2.38143000e-04, -2.14520400e-03,
            -7.87059900e-03, -8.91169400e-03, -1.41551020e-02, -5.67321600e-03, -1.07860600e-03
        ], [
            -2.47360500e-03, -1.42091400e-03, -1.49985100e-03, 7.19721144e-01, -5.56681820e-02,
            -5.81654500e-03, -7.12646700e-03, -5.33107000e-04, -4.20430100e-03, -1.98742350e-02,
            -4.93941200e-03, -1.58239300e-03, -2.34583400e-03, -2.13014200e-03, -6.27864300e-03,
            -2.41874830e-02, -6.11740000e-04, -2.57056000e-03, -1.77015200e-03, -2.84509600e-03,
            -3.47016000e-03, -3.06128300e-03, -1.70316100e-03, -3.11178900e-03, -1.13452900e-03,
            -1.70351700e-03, -3.04825000e-03, -6.66469000e-04, -2.63628000e-04, -1.27928400e-03,
            -3.35470600e-03, -8.61130000e-04, -3.57036700e-03, -4.94858000e-03, -4.34805000e-04
        ], [
            -2.37830000e-04, -2.96835000e-04, -2.23715000e-04, -7.25957000e-03, 8.07483255e-01,
            -1.14425500e-03, -1.36584300e-03, -1.20687000e-04, -2.88560000e-04, -1.09537100e-03,
            -6.48439000e-04, -4.36689000e-04, -4.39169000e-04, -3.98534000e-04, -1.28799500e-03,
            -3.43177200e-03, -1.91620000e-04, -1.77943000e-04, -4.51412000e-04, -5.61101000e-04,
            -5.51029000e-04, -2.28630000e-04, -2.18574000e-04, -2.15384000e-04, -1.47444000e-04,
            -1.66611000e-04, -7.15269000e-04, -7.04103000e-05, -2.16715000e-05, -1.60175000e-04,
            -9.22190000e-04, -1.51426000e-04, -1.69642000e-04, -7.27576000e-04, -1.19889000e-05
        ], [
            -1.62733300e-03, -2.05764000e-03, -1.54080600e-03, -1.57158600e-03, -2.22415500e-03,
            8.07390959e-01, -1.09197040e-02, -4.44378000e-04, -1.58412000e-03, -2.70098800e-03,
            -6.70570500e-03, -2.61859300e-03, -2.86223800e-03, -1.85520800e-03, -3.04649900e-03,
            -7.13011050e-02, -7.95723000e-04, -3.44173520e-02, -7.83804000e-04, -1.61832000e-03,
            -1.48716200e-03, -2.25525500e-03, -1.31139700e-03, -5.28664000e-04, -3.85784000e-04,
            -4.20015400e-03, -7.61861000e-04, -6.03439000e-04, -2.17055700e-03, -9.15428000e-04,
            -1.40747700e-03, -1.16979800e-03, -1.01607300e-03, -2.79315700e-03, -2.58529000e-04
        ], [
            -2.83808700e-03, -3.39603200e-03, -2.45744650e-02, -1.05778290e-02, -1.28079400e-02,
            -1.24075440e-02, 7.63693789e-01, -1.70674400e-03, -1.94191920e-02, -1.85863570e-02,
            -2.28527500e-02, -5.95629200e-03, -1.00964350e-02, -1.51771240e-02, -6.09351600e-03,
            -2.15158790e-02, -3.57708700e-03, -5.35528600e-03, -7.87312900e-03, -1.47503220e-02,
            -1.43770400e-02, -1.06967790e-02, -6.05459000e-03, -4.15106600e-03, -7.16235500e-03,
            -1.61190690e-02, -1.20077720e-02, -1.57563540e-02, -2.71130000e-03, -2.26191830e-02,
            -1.65478940e-02, -1.67164180e-02, -1.00637160e-02, -2.54481840e-02, -5.94557000e-04
        ], [
            -1.24424100e-02, -1.31492220e-02, -3.56014200e-03, -4.63773300e-03, -3.13684700e-03,
            -4.61971900e-03, -4.45787200e-03, 9.55350204e-01, -2.70298600e-02, -8.80552900e-03,
            -1.62146260e-02, -9.01217600e-03, -3.70490700e-03, -2.94038400e-03, -1.98777900e-03,
            -4.28700600e-03, -1.98519680e-02, -1.27651960e-02, -4.37867500e-03, -5.93602700e-03,
            -5.07397600e-03, -3.72919700e-03, -5.06329720e-02, -5.07350360e-02, -5.53679060e-02,
            -1.38512370e-02, -3.54921000e-03, -1.25278200e-03, -1.21337500e-03, -4.46406700e-03,
            -7.57968400e-03, -2.97212200e-03, -3.53755200e-03, -6.62985700e-03, -6.79490000e-05
        ], [
            -4.07645500e-02, -1.40358240e-02, -1.09413920e-02, -7.09275620e-02, -3.33142520e-02,
            -2.59778000e-02, -3.40814780e-02, -1.43772920e-02, 7.67144994e-01, -2.01560220e-01,
            -3.02286700e-02, -1.55752750e-02, -1.27701100e-02, -1.91341010e-02, -1.28375340e-02,
            -3.20997370e-02, -5.87645500e-03, -1.00939580e-02, -6.92965700e-03, -3.42980600e-03,
            -2.52187000e-03, -4.38653400e-03, -3.25119600e-03, -2.62075100e-03, -2.15496300e-03,
            -3.47806800e-03, -1.47210600e-03, -9.06522000e-04, -1.54416100e-03, -4.30980300e-03,
            -8.23443100e-03, -3.75991800e-03, -6.39578500e-02, -9.26720900e-03, -1.20306000e-04
        ], [
            -4.53588500e-03, -9.08180400e-03, -1.49746280e-02, -8.37167500e-03, -3.48259620e-02,
            -9.53368400e-03, -1.40092740e-02, -1.89949500e-03, -1.81862690e-02, 8.89249681e-01,
            -1.08403700e-02, -7.53967100e-03, -2.63607250e-02, -2.59931180e-02, -3.61096170e-02,
            -3.83373180e-02, -2.28796200e-03, -1.81231840e-02, -1.89767290e-02, -4.60447400e-03,
            -5.15765200e-03, -4.17514900e-03, -1.12065290e-02, -2.24513900e-03, -2.44333500e-03,
            -6.13387300e-03, -3.77512900e-03, -7.41477000e-04, -9.87898000e-04, -2.51471100e-03,
            -3.04566500e-03, -9.14915000e-04, -5.55556500e-03, -3.58720400e-03, -8.36239000e-05
        ], [
            -1.89473300e-03, -5.02504400e-03, -5.54977200e-03, -2.00575900e-03, -1.72043900e-03,
            -9.15505700e-03, -1.41341700e-03, -2.16118500e-03, -6.47018400e-03, -6.10299200e-03,
            8.90835862e-01, -9.64538000e-03, -5.60835400e-03, -1.17736150e-02, -7.12507500e-03,
            -6.69937400e-03, -1.89966700e-03, -8.00919040e-02, -2.92303100e-03, -1.47965300e-03,
            -1.01181900e-03, -3.62702400e-03, -1.32361900e-03, -5.05460000e-04, -4.81673000e-04,
            -1.33852600e-03, -1.12491900e-03, -1.73390000e-04, -2.23267500e-03, -1.00370800e-03,
            -1.51277400e-03, -1.69499700e-03, -2.04259200e-03, -1.85607500e-03, -6.83141000e-05
        ], [
            -4.98721800e-03, -2.65465750e-02, -1.50638250e-02, -5.73767800e-03, -1.02026290e-02,
            -2.69213530e-02, -9.16149900e-03, -6.24960300e-03, -1.74866990e-02, -2.36213020e-02,
            -3.18615460e-02, 6.82249547e-01, -1.61494400e-01, -7.52186290e-02, -1.00445259e-01,
            -1.11083954e-01, -7.56722700e-03, -8.47903180e-02, -1.97290370e-02, -4.78527800e-03,
            -3.94403300e-03, -4.44734400e-03, -7.77031100e-03, -1.04342690e-02, -7.75112000e-03,
            -6.54462800e-03, -4.04609700e-03, -8.01088000e-04, -2.53224700e-03, -3.34270200e-03,
            -5.23866800e-03, -2.02258400e-03, -2.75295600e-03, -4.60606000e-03, -2.77736000e-04
        ], [
            -7.24522100e-03, -1.95244010e-02, -3.83868500e-03, -6.38060700e-03, -4.84337500e-03,
            -8.61906400e-03, -7.04493500e-03, -4.76841600e-03, -7.59385300e-03, -1.03534500e-02,
            -1.29850060e-02, -1.61089250e-02, 8.81968817e-01, -1.49661910e-02, -2.86921860e-02,
            -8.89509400e-03, -7.68213900e-03, -1.58997870e-02, -2.29301460e-02, -3.95428000e-03,
            -3.04365400e-03, -2.16301000e-03, -6.46984000e-03, -5.91439200e-03, -4.14464800e-03,
            -4.74442900e-03, -3.22886800e-03, -1.18821700e-03, -1.43341600e-03, -3.02002800e-03,
            -6.52847000e-03, -2.46252200e-03, -2.43437800e-03, -3.67357700e-03, -1.53993000e-04
        ], [
            -1.62452100e-03, -7.54180300e-03, -3.27537800e-03, -4.79568800e-03, -3.53470300e-03,
            -6.50949200e-03, -9.41114500e-03, -3.07649800e-03, -8.89752300e-03, -1.21974870e-02,
            -8.34007600e-03, -1.54988000e-02, -7.33739350e-02, 7.44680266e-01, -5.06410620e-02,
            -1.96809360e-02, -1.26595620e-02, -2.63802520e-02, -2.48242810e-02, -9.01647200e-03,
            -7.72095000e-03, -4.03407700e-03, -6.13855800e-03, -3.15799200e-03, -3.61590400e-03,
            -6.43376400e-03, -2.92258920e-02, -3.46916400e-03, -1.60510600e-03, -1.30308490e-02,
            -1.20948280e-02, -5.99779900e-03, -1.86377490e-02, -7.83202700e-03, -5.63808000e-04
        ], [
            -4.10204000e-03, -5.68596900e-03, -1.78108500e-03, -2.56425800e-03, -2.24640500e-03,
            -2.93255400e-03, -1.90856200e-03, -1.00133400e-03, -2.07909300e-03, -3.30905000e-03,
            -3.46982600e-03, -4.90743400e-03, -1.37480130e-02, -4.93416500e-03, 7.07913617e-01,
            -4.51990600e-03, -1.56738700e-03, -4.12496900e-03, -8.55829230e-02, -5.06541000e-03,
            -4.26539800e-03, -1.24056400e-03, -3.02579550e-02, -4.09783220e-02, -4.58618660e-02,
            -1.03089900e-02, -3.03800900e-03, -8.34248000e-04, -5.06621000e-04, -2.97545600e-03,
            -1.83820850e-02, -1.00261700e-03, -1.43178100e-03, -3.27488600e-03, -1.08549000e-04
        ], [
            -8.71320000e-04, -1.37443700e-03, -1.14673600e-03, -5.60069400e-03, -3.31818500e-03,
            -6.30625400e-03, -3.23526900e-03, -6.00324000e-04, -1.91119600e-03, -2.53637300e-03,
            -3.46487400e-03, -7.08407000e-03, -3.85385900e-03, -2.97678600e-03, -5.14012100e-03,
            9.64869694e-01, -1.06212500e-03, -5.21192900e-03, -2.25406400e-03, -1.54639400e-03,
            -1.74708900e-03, -2.78449500e-03, -1.48272400e-03, -1.04417000e-03, -1.04691100e-03,
            -1.34719400e-03, -9.34863000e-04, -1.20857400e-03, -1.14570700e-03, -2.45357200e-03,
            -4.02148400e-03, -3.52910100e-03, -3.03348900e-03, -5.77598900e-03, -3.10446000e-03
        ], [
            -9.08191100e-03, -2.26703920e-02, -1.14782420e-02, -1.70897860e-02, -7.33425200e-03,
            -1.53423500e-02, -2.05436390e-02, -1.33181010e-02, -2.78128750e-02, -2.09931180e-02,
            -3.49788980e-02, -2.64284470e-02, -1.06668210e-02, -9.61521000e-03, -8.91545700e-03,
            -1.04787830e-02, 9.04717144e-01, -5.05803100e-03, -1.01057200e-02, -6.15583300e-03,
            -1.30977010e-02, -2.27552830e-02, -1.48584190e-02, -3.90355200e-03, -3.90292500e-03,
            -1.21604050e-02, -8.32669100e-03, -4.49095500e-03, -8.13759600e-03, -6.16374500e-03,
            -1.46102270e-02, -1.86642480e-02, -1.27851580e-02, -1.55246240e-02, -2.02115000e-04
        ], [
            -3.52243500e-03, -8.07659100e-03, -2.49356100e-03, -2.58058600e-03, -2.00458800e-03,
            -4.26375400e-03, -3.90482000e-03, -3.02673700e-03, -4.21389300e-03, -3.55217400e-03,
            -7.11151000e-03, -4.78625500e-03, -3.32809900e-03, -3.21559500e-03, -2.37794100e-03,
            -3.20773900e-03, -2.58192330e-02, 9.63686693e-01, -4.28870500e-03, -3.94889900e-03,
            -4.70854100e-03, -5.16858200e-03, -9.54588200e-03, -3.22333500e-03, -2.23004000e-03,
            -1.46538950e-02, -1.18192480e-02, -5.24021000e-03, -4.27070100e-02, -5.44115100e-03,
            -1.93199970e-02, -1.31728370e-02, -6.64315000e-03, -1.21911790e-02, -2.27250700e-03
        ], [
            -4.99921400e-03, -4.31466500e-03, -6.29048200e-03, -5.36115100e-03, -7.52687500e-03,
            -6.00992400e-03, -4.44256700e-03, -3.20836400e-03, -6.06436400e-03, -6.05005500e-03,
            -7.32891200e-03, -6.00027900e-03, -5.24626300e-03, -4.88611200e-03, -4.68428600e-03,
            -6.07245600e-03, -9.05039300e-03, -7.75168700e-03, 9.91295215e-01, -5.04134900e-03,
            -4.75228000e-03, -5.28293800e-03, -1.83095400e-02, -3.36599500e-03, -3.09534800e-03,
            -7.27324900e-03, -4.70215300e-03, -2.44548300e-03, -1.93889800e-03, -6.04978200e-03,
            -3.95775900e-03, -3.16813300e-03, -3.34480600e-03, -6.57204100e-03, -4.10344000e-05
        ], [
            -3.32741090e-02, -2.54377310e-02, -6.08313800e-02, -5.24124220e-02, -7.06367200e-02,
            -5.41825470e-02, -4.80070810e-02, -3.78574090e-02, -4.84403580e-02, -4.86595180e-02,
            -4.20848660e-02, -4.19210860e-02, -4.69350420e-02, -5.09232970e-02, -4.28090420e-02,
            -5.02118650e-02, -2.23359180e-02, -4.27455320e-02, -3.84967200e-02, 9.68259650e-01,
            -1.37700850e-02, -4.43477960e-02, -2.24782960e-02, -1.77953620e-02, -1.78973620e-02,
            -1.92377050e-02, -9.49573000e-03, -4.90132100e-03, -3.38654000e-03, -1.22102270e-02,
            -2.04139770e-02, -1.01376030e-02, -2.79010130e-02, -1.44082550e-02, -2.77706000e-04
        ], [
            -1.68664560e-02, -1.03636150e-02, -2.97305640e-02, -2.64180140e-02, -3.98121360e-02,
            -2.09832390e-02, -1.76836340e-02, -1.29416990e-02, -2.25723190e-02, -2.14598590e-02,
            -1.89360850e-02, -1.58855510e-02, -1.68053120e-02, -1.59687880e-02, -1.81830190e-02,
            -2.55951690e-02, -8.17545300e-03, -2.77460500e-02, -1.73078770e-02, -8.62625600e-03,
            9.91593793e-01, -2.65508520e-02, -1.52872080e-02, -8.96383900e-03, -8.17203200e-03,
            -9.18234800e-03, -1.01896480e-02, -3.98721200e-03, -3.30940200e-03, -9.46500800e-03,
            -7.83332800e-03, -5.79732000e-03, -1.16361460e-02, -1.24992610e-02, -5.42590000e-04
        ], [
            -1.39331300e-03, -3.66884500e-03, -4.10760900e-03, -4.44376600e-03, -4.05510100e-03,
            -5.29508700e-03, -5.66747900e-03, -1.81860900e-03, -5.20244700e-03, -5.77872400e-03,
            -6.17264900e-03, -5.14633500e-03, -6.48089100e-03, -6.73434100e-03, -3.39243100e-03,
            -4.54912600e-03, -4.80281400e-03, -5.32428000e-03, -6.68032700e-03, -9.49970100e-03,
            -6.31243500e-03, 9.89827310e-01, -7.91699800e-03, -6.56655700e-03, -2.28887830e-02,
            -2.17064190e-02, -6.75389700e-03, -1.00560310e-02, -2.44553000e-03, -1.38119020e-02,
            -9.62384900e-03, -7.19243300e-03, -8.10553500e-03, -1.13817460e-02, -8.68578000e-05
        ], [
            -1.42785620e-02, -2.68677950e-02, -2.19466540e-02, -1.88226040e-02, -2.06704280e-02,
            -2.46457210e-02, -1.82757560e-02, -3.30573280e-02, -1.97566610e-02, -1.72261600e-02,
            -3.50242810e-02, -1.81046430e-02, -1.21718560e-02, -9.74973300e-03, -1.18066810e-02,
            -1.82515970e-02, -2.58503110e-02, -1.72638570e-02, -1.38719640e-02, -1.82821870e-02,
            -1.05096610e-02, -9.95362500e-03, 9.58640654e-01, -1.88095570e-02, -1.04955430e-02,
            -3.06316010e-02, -1.03564970e-02, -6.30804800e-03, -1.77625500e-03, -6.80200300e-03,
            -1.14747100e-02, -1.03678140e-02, -7.68463700e-03, -9.79484100e-03, -6.28328000e-04
        ], [
            -2.88333300e-03, -4.18935800e-03, -3.97983400e-03, -3.05790500e-03, -3.93454200e-03,
            -7.41992700e-03, -1.71347400e-03, -4.44411400e-03, -3.13682400e-03, -2.24653000e-03,
            -5.90973900e-03, -4.02256500e-03, -1.59906100e-03, -1.50756800e-03, -1.65203800e-03,
            -2.29319900e-03, -2.18412100e-03, -2.64916900e-03, -1.19792800e-03, -2.14561600e-03,
            -1.27166300e-03, -1.73074300e-03, -3.99450000e-03, 9.00864963e-01, -3.08909800e-03,
            -6.08180900e-03, -1.13859400e-03, -6.84809000e-04, -1.37419000e-04, -1.07106200e-03,
            -1.48133200e-03, -1.90485800e-03, -6.18081000e-04, -1.03992500e-03, -5.72517000e-05
        ], [
            -8.27348000e-04, -2.37798300e-03, -1.60048700e-03, -1.79021100e-03, -2.84203600e-03,
            -1.38686300e-03, -2.20556000e-03, -8.76691000e-04, -2.05856700e-03, -1.91166500e-03,
            -1.49946800e-03, -1.48509300e-03, -1.94987400e-03, -2.26482300e-03, -1.77818000e-03,
            -1.65147900e-03, -8.51503000e-04, -1.25423400e-03, -2.17529300e-03, -4.13955800e-03,
            -1.73028400e-03, -1.25120100e-03, -2.12890500e-03, -4.23108200e-03, 9.66735768e-01,
            -2.02891230e-02, -5.64289900e-03, -3.48093200e-03, -5.24360000e-04, -4.60918500e-03,
            -4.77694500e-03, -2.20361200e-03, -1.35024500e-03, -4.16600000e-03, -1.86254000e-05
        ], [
            -3.32641600e-03, -6.45424700e-03, -6.22103900e-03, -3.88779300e-03, -4.67819500e-03,
            -6.76123600e-03, -7.00322500e-03, -6.38065000e-03, -5.35910500e-03, -5.07723600e-03,
            -9.15084000e-03, -5.40310000e-03, -4.54172300e-03, -3.98822400e-03, -4.12748000e-03,
            -4.69156300e-03, -4.86228700e-03, -3.30346600e-03, -7.69254000e-03, -1.90231960e-02,
            -9.81682600e-03, -8.19934700e-03, -3.88025660e-02, -1.45834168e-01, -9.68617290e-02,
            8.85141795e-01, -3.53439200e-03, -3.82491300e-03, -7.00313000e-04, -4.92165400e-03,
            -3.81449700e-03, -2.49002500e-03, -3.03964900e-03, -5.54892600e-03, -5.90197000e-05
        ], [
            -1.98728500e-03, -5.53039700e-03, -3.54545700e-03, -3.99317200e-03, -4.07198400e-03,
            -4.01878600e-03, -9.96192000e-03, -2.29031900e-03, -5.87807000e-03, -5.06108300e-03,
            -4.81364700e-03, -4.85620300e-03, -6.21917700e-03, -6.10218500e-03, -3.41353400e-03,
            -6.23336200e-03, -5.03300200e-03, -7.60662000e-03, -1.13281170e-02, -1.78656380e-02,
            -1.77754980e-02, -1.10423910e-02, -1.40586380e-02, -2.00990960e-02, -2.86734020e-02,
            -2.16707550e-02, 9.11312061e-01, -2.26805630e-02, -3.69353400e-03, -2.05396130e-02,
            -1.69967080e-02, -8.11023000e-03, -1.01528950e-02, -2.10075720e-02, -1.22858000e-04
        ], [
            -1.84069540e-02, -3.25990270e-02, -1.56807150e-02, -2.19064940e-02, -1.79015360e-02,
            -2.20880790e-02, -2.10893240e-02, -1.34490480e-02, -2.11835960e-02, -1.85247100e-02,
            -2.30069700e-02, -2.11754640e-02, -1.97235100e-02, -1.75217460e-02, -1.42419360e-02,
            -2.15255240e-02, -2.30974850e-02, -2.03601500e-02, -2.63178890e-02, -4.15695720e-02,
            -3.43940570e-02, -2.32514010e-02, -4.16860420e-02, -3.96516550e-02, -3.82626830e-02,
            -2.58375240e-02, -2.29594200e-02, 8.26583954e-01, -5.02083550e-02, -3.62311890e-02,
            -2.32167830e-02, -1.36665330e-02, -2.17628760e-02, -3.26398480e-02, -3.12310700e-03
        ], [
            -6.68624100e-03, -5.45715900e-03, -4.03094200e-03, -5.62668100e-03, -5.62081900e-03,
            -5.13275000e-03, -8.23386400e-03, -2.36442500e-03, -4.72225200e-03, -6.05750300e-03,
            -5.57207900e-03, -5.24793800e-03, -6.22211500e-03, -5.33887600e-03, -3.93170900e-03,
            -8.50786000e-03, -5.91299800e-03, -1.21619810e-02, -2.43045260e-02, -2.86509610e-02,
            -4.19871860e-02, -2.77770520e-02, -1.05711720e-02, -2.65195950e-02, -1.60253240e-02,
            -2.12366610e-02, -1.27117790e-02, -2.38550120e-02, 9.70519393e-01, -2.36045750e-02,
            -1.09102720e-02, -1.75074720e-02, -2.95370780e-02, -2.83248160e-02, -5.34995000e-04
        ], [
            -1.24456770e-02, -3.16038510e-02, -3.63384960e-02, -2.79748010e-02, -2.55549670e-02,
            -2.23907120e-02, -5.69883830e-02, -1.51423990e-02, -5.96644510e-02, -3.40564440e-02,
            -3.16328380e-02, -2.78807910e-02, -4.29749980e-02, -5.94936910e-02, -3.70814340e-02,
            -3.65289350e-02, -3.69388060e-02, -5.48021510e-02, -4.44947480e-02, -5.62077010e-02,
            -6.26907730e-02, -3.37204480e-02, -4.00311050e-02, -5.18904660e-02, -9.37651580e-02,
            -6.60873730e-02, -6.86543910e-02, -8.65516130e-02, -3.32739180e-02, 8.67931639e-01,
            -5.59218870e-02, -2.66202880e-02, -5.05906390e-02, -6.62277060e-02, -1.83157000e-04
        ], [
            -1.68611500e-03, -3.37449400e-03, -1.79804500e-03, -1.25650200e-03, -8.86649000e-04,
            -3.11974800e-03, -3.03310800e-03, -1.08615200e-03, -3.08320700e-03, -2.13667300e-03,
            -2.24836900e-03, -2.18462200e-03, -1.28990000e-03, -1.57061200e-03, -1.20639800e-03,
            -2.38018700e-03, -5.70881700e-03, -1.41947000e-03, -2.18477200e-03, -1.72566300e-03,
            -3.47224300e-03, -3.54475000e-03, -3.14137300e-03, -9.41197900e-03, -1.03204880e-02,
            -3.51101800e-03, -2.77585000e-03, -1.60083800e-03, -3.37450800e-03, -2.77878500e-03,
            9.92443596e-01, -3.85140000e-03, -5.13284700e-03, -7.58782600e-03, -2.45976000e-06
        ], [
            -8.37405000e-04, -5.21545000e-04, -3.34798000e-04, -3.59483000e-04, -3.78462000e-04,
            -2.61623000e-04, -5.96198000e-04, -4.39618000e-04, -7.91672000e-04, -4.72913000e-04,
            -5.12544000e-04, -4.75034000e-04, -7.87729000e-04, -9.56195000e-04, -4.60961000e-04,
            -3.04941000e-04, -1.04343200e-03, -4.55238000e-04, -4.53062000e-04, -8.77146000e-04,
            -1.81212200e-03, -3.99979000e-04, -1.08601500e-03, -3.50085000e-04, -1.63816200e-03,
            -1.07164600e-03, -1.72134100e-03, -1.24613900e-03, -1.73368000e-04, -2.06010000e-03,
            -4.45103100e-03, 9.93410539e-01, -1.02974600e-03, -1.39157800e-03, -7.12222000e-07
        ], [
            -2.06107600e-03, -4.13464000e-04, -4.02877000e-04, -4.41949000e-04, -4.12586000e-04,
            -3.64425000e-04, -4.42435000e-04, -2.11542000e-04, -4.44978000e-04, -3.89162000e-04,
            -4.38210000e-04, -3.69547000e-04, -5.59140000e-04, -4.53355000e-04, -2.99884000e-04,
            -4.88012000e-04, -4.34559000e-04, -5.44137000e-04, -5.87822000e-04, -6.97499000e-04,
            -4.91380000e-04, -1.03803500e-03, -7.78636000e-04, -5.93938000e-04, -5.67302000e-04,
            -7.52563000e-04, -7.57130000e-04, -6.21843000e-04, -1.10096000e-04, -7.94063000e-04,
            -6.32777000e-03, -1.49927700e-03, 9.77619242e-01, -1.67600600e-03, -8.80619000e-08
        ], [
            -2.58193300e-03, -5.95196100e-03, -6.30608400e-03, -5.28491000e-03, -6.15893100e-03,
            -5.98419400e-03, -1.33683380e-02, -3.50626100e-03, -8.38557900e-03, -6.42754100e-03,
            -6.05452500e-03, -7.21608000e-03, -6.57553900e-03, -6.54436000e-03, -4.25458400e-03,
            -6.90224800e-03, -6.35951400e-03, -6.21870000e-03, -9.62292500e-03, -8.92464000e-03,
            -1.05309820e-02, -1.33291190e-02, -9.39921400e-03, -7.33454900e-03, -8.97097600e-03,
            -1.15760190e-02, -1.84766240e-02, -1.05309840e-02, -7.73744700e-03, -2.87769040e-02,
            -1.58294370e-02, -1.15529780e-02, -1.49604210e-02, 9.20789655e-01, -4.22377000e-04
        ], [
            -1.12306000e-05, -1.35700000e-05, -3.13485000e-05, -4.09308000e-05, -2.69900000e-05,
            -3.45605000e-05, -3.58591000e-05, -1.16021000e-05, -3.16394000e-05, -3.45246000e-05,
            -2.30040000e-05, -3.24389000e-05, -2.75784000e-05, -2.02194000e-05, -3.70828000e-05,
            -3.63046000e-05, -1.58638000e-05, -8.57338000e-05, -1.26296000e-04, -3.62699000e-05,
            -5.40855000e-05, -6.78024000e-05, -1.57967000e-04, -2.62135000e-05, -3.57885000e-05,
            -3.18437000e-05, -8.56240000e-05, -6.33893000e-05, -2.68572000e-04, -7.47766000e-05,
            -8.93421000e-05, -5.27179000e-05, -1.32325000e-04, -1.19849000e-04, 9.99978528e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 384)
    return ret


@cache('run')
def ia_matrix_x1998x():
    """
    ia matrix x1998x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.70035800e-01, -2.80485500e-03, -2.67820482e-01, -3.97993970e-02, -3.08431530e-02,
            -1.35661811e-01, -1.17846770e-02, -3.48140200e-03, -7.09167200e-03, -1.41068510e-02,
            -1.04388600e-03, -3.03774000e-04, -3.13830000e-04, -2.21246000e-04, -1.81211000e-04,
            -1.11678440e-02, -5.32670000e-04, -4.80767000e-03, -4.92935000e-04, -2.36285000e-03,
            -2.49507600e-03, -3.85933680e-02, -7.25217000e-04, -1.92753700e-03, -4.66472000e-04,
            -2.13981600e-03, -2.15559000e-04, -1.80183000e-04, -5.36462000e-04, -9.78037000e-04,
            -1.87687300e-03, -2.03063100e-03, -2.40036300e-03, -2.39695300e-03, -1.59688630e-02
        ], [
            -3.20195500e-03, 9.17243041e-01, -3.02383500e-03, -1.92603700e-03, -1.15983100e-03,
            -3.50995400e-03, -4.94939500e-03, -4.08844307e-01, -3.74203440e-02, -3.58559400e-03,
            -7.60215320e-02, -4.12525050e-02, -3.11976000e-03, -1.79511100e-03, -1.43600400e-03,
            -6.30593200e-03, -9.48360260e-02, -1.92181670e-02, -5.68977000e-04, -1.28875000e-03,
            -8.82285000e-04, -1.99254800e-03, -5.29100700e-03, -3.44664100e-03, -2.49122000e-03,
            -2.96129500e-03, -4.82434000e-04, -3.47380000e-04, -2.25567000e-03, -8.99979000e-04,
            -2.40540900e-03, -3.28240500e-03, -1.27706600e-03, -2.45479300e-03, -6.72661470e-02
        ], [
            -5.39455590e-02, -7.80719000e-04, 8.59733397e-01, -2.28397500e-03, -8.38677970e-02,
            -1.24009700e-03, -2.73081600e-03, -1.16458000e-03, -9.70814500e-03, -2.45485500e-03,
            -1.88054000e-03, -9.62687000e-04, -1.18519400e-03, -1.10898400e-03, -8.28070000e-04,
            -1.57996400e-03, -7.86752000e-04, -1.05101800e-03, -1.68332200e-03, -4.61115200e-03,
            -4.25945100e-03, -1.57726913e-01, -1.64034500e-03, -7.86147800e-03, -4.67406200e-03,
            -2.41612600e-03, -1.42503600e-03, -8.41402000e-04, -2.40109000e-04, -1.94032900e-03,
            -7.98159900e-03, -9.07325200e-03, -1.42692260e-02, -5.84065900e-03, -1.32203600e-03
        ], [
            -2.36477800e-03, -1.62646100e-03, -1.48441600e-03, 7.24373891e-01, -5.48973490e-02,
            -5.47034700e-03, -7.04793100e-03, -6.35456000e-04, -4.16311900e-03, -1.92223420e-02,
            -5.10942000e-03, -1.60547800e-03, -2.47594000e-03, -2.05117600e-03, -6.17707500e-03,
            -2.61378790e-02, -5.98463000e-04, -2.60369900e-03, -1.75720300e-03, -2.84516000e-03,
            -3.47448600e-03, -2.91003500e-03, -1.69084500e-03, -2.77250500e-03, -1.18066300e-03,
            -1.76731200e-03, -3.12346200e-03, -6.52776000e-04, -2.86592000e-04, -1.25289600e-03,
            -3.59315800e-03, -9.12808000e-04, -3.60043600e-03, -5.03728200e-03, -5.32936000e-04
        ], [
            -2.26326000e-04, -2.69802000e-04, -2.12412000e-04, -6.60909500e-03, 8.09717565e-01,
            -9.68683000e-04, -1.31335100e-03, -1.27019000e-04, -2.85108000e-04, -1.17088400e-03,
            -6.55225000e-04, -4.23792000e-04, -4.48146000e-04, -3.72801000e-04, -1.30347900e-03,
            -3.36753900e-03, -1.81817000e-04, -1.81278000e-04, -4.42559000e-04, -5.07054000e-04,
            -5.29929000e-04, -2.17160000e-04, -2.02074000e-04, -1.95974000e-04, -1.41969000e-04,
            -1.69460000e-04, -6.81859000e-04, -6.66719000e-05, -1.79120000e-05, -1.46136000e-04,
            -9.38500000e-04, -1.50543000e-04, -1.75482000e-04, -7.05979000e-04, -1.46947000e-05
        ], [
            -1.69127800e-03, -2.14928200e-03, -1.58924900e-03, -1.48638200e-03, -2.01472400e-03,
            8.03664081e-01, -1.07551230e-02, -4.31923000e-04, -1.51664600e-03, -2.75233600e-03,
            -6.41603800e-03, -2.70364800e-03, -2.95834300e-03, -1.80006000e-03, -2.87960300e-03,
            -7.36166920e-02, -8.10010000e-04, -3.31174600e-02, -7.90533000e-04, -1.63832200e-03,
            -1.55692700e-03, -2.14960200e-03, -1.31285100e-03, -5.36127000e-04, -4.01538000e-04,
            -3.94424100e-03, -7.34393000e-04, -5.52113000e-04, -2.34286200e-03, -8.93368000e-04,
            -1.42601400e-03, -1.53421000e-03, -1.02674900e-03, -2.67300000e-03, -3.16876000e-04
        ], [
            -2.85211400e-03, -3.31980100e-03, -2.43401690e-02, -1.04391750e-02, -1.29009880e-02,
            -1.25381510e-02, 7.66939573e-01, -2.08591900e-03, -1.91165890e-02, -1.86591440e-02,
            -2.29742730e-02, -6.10140200e-03, -9.84115700e-03, -1.46023220e-02, -6.11638500e-03,
            -2.27969600e-02, -3.61825200e-03, -5.32229100e-03, -7.89146600e-03, -1.45711690e-02,
            -1.41591000e-02, -1.02118650e-02, -6.25072900e-03, -3.89482800e-03, -7.12726100e-03,
            -1.58006650e-02, -1.21761930e-02, -1.53580730e-02, -2.85213100e-03, -2.22727200e-02,
            -1.70359940e-02, -1.72910880e-02, -1.01538750e-02, -2.58194840e-02, -7.28743000e-04
        ], [
            -1.18616620e-02, -1.35586240e-02, -3.59882500e-03, -4.23716700e-03, -2.99286000e-03,
            -4.56839500e-03, -4.36429200e-03, 9.53570708e-01, -2.61031480e-02, -7.96941800e-03,
            -1.56683800e-02, -9.15013000e-03, -3.60518300e-03, -2.74481900e-03, -1.93213600e-03,
            -4.28769400e-03, -1.86668430e-02, -1.29739670e-02, -4.33812200e-03, -5.73204300e-03,
            -4.85030600e-03, -3.49190000e-03, -5.07455100e-02, -5.39768460e-02, -5.41475070e-02,
            -1.45390260e-02, -3.56208900e-03, -1.27185400e-03, -1.43446700e-03, -4.39701700e-03,
            -7.54516100e-03, -2.91709400e-03, -3.33419800e-03, -6.37188800e-03, -8.32844000e-05
        ], [
            -4.04800860e-02, -1.39888960e-02, -1.11097490e-02, -6.98917200e-02, -3.40406120e-02,
            -2.68853180e-02, -3.45022500e-02, -1.57561090e-02, 7.71435562e-01, -1.99015311e-01,
            -3.05255240e-02, -1.57575500e-02, -1.28164350e-02, -1.86725790e-02, -1.30896150e-02,
            -3.37067100e-02, -6.25675300e-03, -1.05823590e-02, -6.98718300e-03, -3.56394200e-03,
            -2.60207700e-03, -4.40667300e-03, -3.41560900e-03, -2.80186000e-03, -2.31138400e-03,
            -3.77935700e-03, -1.49508400e-03, -9.54481000e-04, -1.80551400e-03, -4.46121800e-03,
            -8.57815300e-03, -3.68278000e-03, -6.43073530e-02, -9.42796700e-03, -1.47458000e-04
        ], [
            -4.67935900e-03, -8.99119900e-03, -1.52237450e-02, -8.56878100e-03, -3.38450410e-02,
            -9.73035800e-03, -1.44573380e-02, -2.13414600e-03, -1.84979460e-02, 8.87675834e-01,
            -1.11856100e-02, -7.70515000e-03, -2.66421860e-02, -2.61844210e-02, -3.58781840e-02,
            -4.12673970e-02, -2.37811900e-03, -1.82877150e-02, -1.81493610e-02, -4.62843500e-03,
            -5.23626600e-03, -4.17432900e-03, -1.12299330e-02, -2.70138400e-03, -2.59933400e-03,
            -6.35034100e-03, -3.77566800e-03, -7.28513000e-04, -1.13250900e-03, -2.54256200e-03,
            -3.13820400e-03, -9.24553000e-04, -5.83152500e-03, -3.40814000e-03, -1.02497000e-04
        ], [
            -1.92553000e-03, -5.13284400e-03, -5.40330500e-03, -1.97258100e-03, -1.67815500e-03,
            -8.97737800e-03, -1.37448700e-03, -2.09216100e-03, -6.37819000e-03, -6.06941400e-03,
            8.90309276e-01, -9.80534000e-03, -5.62436200e-03, -1.13632280e-02, -6.86614500e-03,
            -6.95233400e-03, -1.98419900e-03, -7.80741190e-02, -2.87831100e-03, -1.44385200e-03,
            -1.01321600e-03, -3.55537500e-03, -1.28340000e-03, -5.74259000e-04, -5.16091000e-04,
            -1.38545400e-03, -1.08727200e-03, -1.76631000e-04, -2.29905200e-03, -9.58448000e-04,
            -1.46618400e-03, -1.63548800e-03, -2.03483200e-03, -1.80379400e-03, -8.37319000e-05
        ], [
            -4.81464100e-03, -2.51565140e-02, -1.48379820e-02, -5.61796900e-03, -9.70087200e-03,
            -2.68667340e-02, -9.26371600e-03, -6.67297500e-03, -1.75800030e-02, -2.39005990e-02,
            -3.22330520e-02, 6.85574010e-01, -1.59608119e-01, -7.28366170e-02, -1.00655544e-01,
            -1.18647079e-01, -7.85064000e-03, -8.51219080e-02, -1.93922390e-02, -4.65453700e-03,
            -3.97316200e-03, -4.41205700e-03, -7.71254800e-03, -9.14489300e-03, -7.96859300e-03,
            -6.64322600e-03, -4.12565700e-03, -7.88282000e-04, -2.93355200e-03, -3.30092200e-03,
            -5.25234900e-03, -1.90813700e-03, -2.72588700e-03, -4.47459100e-03, -3.40419000e-04
        ], [
            -7.01977800e-03, -1.88000520e-02, -3.74420500e-03, -6.26629200e-03, -4.77577500e-03,
            -8.48158300e-03, -6.86036700e-03, -4.67609900e-03, -7.47768800e-03, -1.00713570e-02,
            -1.28438560e-02, -1.60069020e-02, 8.87482423e-01, -1.43779970e-02, -2.79510350e-02,
            -9.43184500e-03, -8.04720500e-03, -1.58315580e-02, -2.19629330e-02, -4.01672800e-03,
            -3.09405600e-03, -2.08871300e-03, -6.63028600e-03, -5.70243000e-03, -4.07111400e-03,
            -4.91508200e-03, -3.08113100e-03, -1.26395800e-03, -1.62385400e-03, -2.86681300e-03,
            -6.41702300e-03, -2.34961200e-03, -2.47619200e-03, -3.45546100e-03, -1.88747000e-04
        ], [
            -1.64561200e-03, -7.75108600e-03, -3.39178100e-03, -4.88126000e-03, -3.64182300e-03,
            -7.12488000e-03, -9.49439000e-03, -3.41471900e-03, -9.39533200e-03, -1.28057820e-02,
            -8.77685900e-03, -1.60642790e-02, -7.42763670e-02, 7.44046596e-01, -5.08979170e-02,
            -2.15634670e-02, -1.28732670e-02, -2.74482860e-02, -2.41280290e-02, -9.01336800e-03,
            -8.10441000e-03, -4.11688500e-03, -6.31987600e-03, -3.41500300e-03, -3.79021900e-03,
            -6.67178000e-03, -3.15041440e-02, -3.55497900e-03, -1.81934800e-03, -1.33659090e-02,
            -1.26544950e-02, -6.04821300e-03, -1.96216820e-02, -8.06230800e-03, -6.91054000e-04
        ], [
            -4.10650500e-03, -6.34197100e-03, -1.88437500e-03, -2.68482800e-03, -2.34662600e-03,
            -3.27458700e-03, -2.09353100e-03, -1.03961500e-03, -2.09592800e-03, -3.37629600e-03,
            -3.49585400e-03, -5.23678600e-03, -1.36699090e-02, -5.09316400e-03, 7.17537383e-01,
            -4.90893200e-03, -1.77624100e-03, -4.47780400e-03, -8.66669170e-02, -5.23015100e-03,
            -4.18110500e-03, -1.39687100e-03, -3.19561890e-02, -3.71127350e-02, -4.64723790e-02,
            -1.26978130e-02, -3.43743600e-03, -9.36111000e-04, -9.21986000e-04, -3.29098900e-03,
            -1.79271720e-02, -1.13340800e-03, -1.54040800e-03, -3.50151200e-03, -1.33047000e-04
        ], [
            -7.17457000e-04, -1.34936600e-03, -1.12402300e-03, -5.40124700e-03, -3.07569600e-03,
            -6.40169200e-03, -3.37536700e-03, -5.75076000e-04, -1.89657300e-03, -2.52875400e-03,
            -3.46018100e-03, -7.54278900e-03, -3.88261400e-03, -2.87811800e-03, -5.16925400e-03,
            9.62063117e-01, -1.06144900e-03, -5.21109800e-03, -2.35464300e-03, -1.44555700e-03,
            -1.73189100e-03, -2.62899300e-03, -1.56728100e-03, -9.17782000e-04, -1.00407200e-03,
            -1.42008800e-03, -9.12740000e-04, -1.19815300e-03, -1.43663400e-03, -2.43682600e-03,
            -3.99618600e-03, -3.48091600e-03, -3.19156600e-03, -5.63469900e-03, -3.80510500e-03
        ], [
            -8.98600200e-03, -2.16084630e-02, -1.14290730e-02, -1.70709290e-02, -7.32641300e-03,
            -1.56711040e-02, -2.04239000e-02, -1.40073920e-02, -2.80311470e-02, -2.09800060e-02,
            -3.52270040e-02, -2.69288190e-02, -1.09935600e-02, -9.35087900e-03, -8.59365200e-03,
            -1.09882030e-02, 9.01575594e-01, -5.05351200e-03, -1.00138890e-02, -6.07608700e-03,
            -1.32324520e-02, -2.23868100e-02, -1.48163100e-02, -3.60686800e-03, -3.90210900e-03,
            -1.20775170e-02, -8.10663700e-03, -4.53300400e-03, -9.33474000e-03, -6.12770800e-03,
            -1.50121960e-02, -1.85174440e-02, -1.27430840e-02, -1.52138330e-02, -2.47731000e-04
        ], [
            -3.43829000e-03, -7.14115100e-03, -2.47898500e-03, -2.49837000e-03, -1.97351600e-03,
            -4.16619800e-03, -3.75226100e-03, -2.90872700e-03, -4.12366700e-03, -3.47359300e-03,
            -6.97165900e-03, -4.77334900e-03, -3.27945300e-03, -3.08236900e-03, -2.30513400e-03,
            -3.27641200e-03, -2.59366630e-02, 9.63657156e-01, -4.23831000e-03, -3.82265700e-03,
            -4.63530400e-03, -5.18777300e-03, -9.88724900e-03, -2.98844500e-03, -2.30695800e-03,
            -1.49765570e-02, -1.18321720e-02, -5.34300100e-03, -4.37832650e-02, -5.32087700e-03,
            -1.94432100e-02, -1.39081460e-02, -6.70216000e-03, -1.17192730e-02, -2.78538900e-03
        ], [
            -4.90099800e-03, -4.30747100e-03, -6.39925100e-03, -5.60810900e-03, -7.76566000e-03,
            -5.92149100e-03, -4.61606000e-03, -3.72301700e-03, -6.14992400e-03, -6.00563600e-03,
            -7.43585200e-03, -6.01754500e-03, -5.34064800e-03, -4.72038800e-03, -4.70618200e-03,
            -6.35530200e-03, -8.90106100e-03, -7.49241300e-03, 9.91116250e-01, -4.80587700e-03,
            -4.58091200e-03, -5.21136600e-03, -1.82851270e-02, -3.25522100e-03, -3.37589800e-03,
            -7.38306400e-03, -4.68557400e-03, -2.46354400e-03, -2.14289100e-03, -5.95341100e-03,
            -4.06474300e-03, -3.19323800e-03, -3.39449800e-03, -6.41844700e-03, -5.02955000e-05
        ], [
            -3.37668080e-02, -2.66688910e-02, -6.13148180e-02, -5.48445620e-02, -7.12548260e-02,
            -5.48299510e-02, -5.00222590e-02, -3.91272330e-02, -4.96748040e-02, -4.90487590e-02,
            -4.34740940e-02, -4.37975770e-02, -4.71052150e-02, -4.99904480e-02, -4.35236760e-02,
            -5.41605490e-02, -2.25674890e-02, -4.27058130e-02, -3.76929370e-02, 9.69187236e-01,
            -1.39607540e-02, -4.43629810e-02, -2.31852560e-02, -1.90992990e-02, -2.00160420e-02,
            -2.05816730e-02, -1.02549260e-02, -4.91492300e-03, -4.09228400e-03, -1.26863190e-02,
            -2.12297620e-02, -1.03262160e-02, -2.89158140e-02, -1.50108340e-02, -3.40382000e-04
        ], [
            -1.66254750e-02, -1.06006150e-02, -2.96098800e-02, -2.72907980e-02, -4.01547230e-02,
            -2.13115750e-02, -1.81662490e-02, -1.45271060e-02, -2.23014450e-02, -2.13224660e-02,
            -1.94554180e-02, -1.63756540e-02, -1.71116050e-02, -1.57931180e-02, -1.87440230e-02,
            -2.70640190e-02, -8.37706900e-03, -2.87196400e-02, -1.72766230e-02, -8.46540600e-03,
            9.91574203e-01, -2.57968070e-02, -1.56223770e-02, -9.09549100e-03, -9.11155500e-03,
            -9.78927500e-03, -1.04135080e-02, -4.00088900e-03, -3.65569000e-03, -9.35434700e-03,
            -7.98990100e-03, -5.84961800e-03, -1.19209460e-02, -1.24829700e-02, -6.65047000e-04
        ], [
            -1.36628000e-03, -3.89012900e-03, -4.01894000e-03, -4.43396100e-03, -4.07832600e-03,
            -4.83669400e-03, -5.73058900e-03, -1.84529800e-03, -5.13900800e-03, -5.64668400e-03,
            -5.98441300e-03, -5.07981400e-03, -6.46876700e-03, -6.38967500e-03, -3.16241600e-03,
            -4.73121600e-03, -4.96000100e-03, -5.09760500e-03, -6.42855200e-03, -9.42145200e-03,
            -6.35821900e-03, 9.89723024e-01, -8.10068100e-03, -5.89255400e-03, -2.31367610e-02,
            -2.24376410e-02, -7.21605400e-03, -1.02756180e-02, -2.77820100e-03, -1.38080840e-02,
            -1.00823820e-02, -7.54241300e-03, -8.52632400e-03, -1.15314450e-02, -1.06461000e-04
        ], [
            -1.41143000e-02, -2.65849430e-02, -2.20920620e-02, -1.96098810e-02, -2.10150490e-02,
            -2.48450870e-02, -1.83941170e-02, -3.09213200e-02, -1.95501250e-02, -1.77047360e-02,
            -3.39086960e-02, -1.83092140e-02, -1.24723250e-02, -9.41944700e-03, -1.17907800e-02,
            -2.00860750e-02, -2.70628160e-02, -1.73393280e-02, -1.44138400e-02, -1.80172600e-02,
            -1.07340700e-02, -9.87921300e-03, 9.56062763e-01, -3.13193000e-02, -1.21332580e-02,
            -3.22612900e-02, -1.03791950e-02, -6.27879300e-03, -2.15430000e-03, -6.64914500e-03,
            -1.17667320e-02, -1.06480330e-02, -7.95893600e-03, -9.72868000e-03, -7.70135000e-04
        ], [
            -3.03212100e-03, -4.90503300e-03, -4.05524300e-03, -3.91083900e-03, -5.08746000e-03,
            -7.74139800e-03, -2.45684000e-03, -5.98200900e-03, -3.18371400e-03, -2.70120200e-03,
            -5.84661400e-03, -5.26171400e-03, -1.99413300e-03, -1.71764000e-03, -1.85029700e-03,
            -2.22655100e-03, -2.62478100e-03, -3.13362000e-03, -1.24136200e-03, -2.36542000e-03,
            -1.38091600e-03, -1.67811800e-03, -3.93750200e-03, 8.82894138e-01, -3.26273700e-03,
            -5.06248500e-03, -1.09034700e-03, -5.89197000e-04, -1.99720000e-04, -9.52526000e-04,
            -1.29461700e-03, -1.47246200e-03, -6.43155000e-04, -1.12921700e-03, -7.01728000e-05
        ], [
            -7.91110000e-04, -2.19254500e-03, -1.58701700e-03, -1.80712900e-03, -2.74739600e-03,
            -1.45569300e-03, -2.14995500e-03, -8.64679000e-04, -2.00976500e-03, -1.95523700e-03,
            -1.49930100e-03, -1.49036600e-03, -1.89055300e-03, -2.12625600e-03, -1.84214300e-03,
            -1.79650200e-03, -8.65702000e-04, -1.21517600e-03, -2.21190100e-03, -3.84042200e-03,
            -1.69082800e-03, -1.21882200e-03, -2.16584300e-03, -3.75716100e-03, 9.65248457e-01,
            -2.02610220e-02, -5.30433800e-03, -3.42225800e-03, -5.95722000e-04, -4.44960700e-03,
            -4.65229100e-03, -2.23972000e-03, -1.36592900e-03, -4.12996700e-03, -2.28290000e-05
        ], [
            -3.49483000e-03, -6.59047200e-03, -6.29611000e-03, -4.22859300e-03, -4.93325300e-03,
            -6.78776400e-03, -7.07649000e-03, -7.15555000e-03, -5.28098500e-03, -5.03506600e-03,
            -8.50369100e-03, -5.23844300e-03, -4.53043000e-03, -3.81369500e-03, -4.37578100e-03,
            -4.93457600e-03, -4.82162400e-03, -3.62652900e-03, -8.07605100e-03, -1.84691640e-02,
            -1.04244020e-02, -8.25371400e-03, -3.82438590e-02, -1.30828902e-01, -9.87785790e-02,
            8.81539292e-01, -3.59037500e-03, -4.01077100e-03, -8.25356000e-04, -5.02245800e-03,
            -4.29447500e-03, -2.59193800e-03, -3.24034200e-03, -5.65339300e-03, -7.23399000e-05
        ], [
            -2.06509500e-03, -5.65511200e-03, -3.67765600e-03, -4.18987700e-03, -4.42366500e-03,
            -4.38272500e-03, -1.04114820e-02, -2.86913000e-03, -6.16717100e-03, -5.24027300e-03,
            -5.02463300e-03, -5.13887400e-03, -6.53294500e-03, -6.27577500e-03, -3.54483300e-03,
            -7.03607700e-03, -5.38801000e-03, -8.62358700e-03, -1.15755570e-02, -1.81219750e-02,
            -1.85344980e-02, -1.12265040e-02, -1.46253760e-02, -1.79852890e-02, -2.97838040e-02,
            -2.27901110e-02, 9.01407634e-01, -2.40172280e-02, -4.07219400e-03, -2.09119100e-02,
            -1.77738960e-02, -8.53083300e-03, -1.05707850e-02, -2.15704490e-02, -1.50585000e-04
        ], [
            -1.86649740e-02, -3.12642590e-02, -1.59588820e-02, -2.21190300e-02, -1.73491080e-02,
            -2.07864490e-02, -2.13667160e-02, -1.40318020e-02, -2.05581020e-02, -1.85717570e-02,
            -2.29555440e-02, -2.05614850e-02, -2.00263770e-02, -1.68490730e-02, -1.41355290e-02,
            -2.20714440e-02, -2.36076660e-02, -2.03421200e-02, -2.66752360e-02, -4.01871710e-02,
            -3.53531720e-02, -2.37704880e-02, -4.37507630e-02, -3.93063900e-02, -4.00911460e-02,
            -2.65374790e-02, -2.32958760e-02, 8.18408126e-01, -5.47590010e-02, -3.65589030e-02,
            -2.44458100e-02, -1.40645380e-02, -2.26354160e-02, -3.30797750e-02, -3.82796100e-03
        ], [
            -6.65114600e-03, -5.35935200e-03, -4.02750400e-03, -5.54072800e-03, -5.55714500e-03,
            -5.32039300e-03, -8.19160200e-03, -3.37330400e-03, -4.67772600e-03, -5.92346300e-03,
            -5.36306300e-03, -5.28283500e-03, -6.22975900e-03, -5.14970100e-03, -4.08505900e-03,
            -8.70941700e-03, -6.02328800e-03, -1.19944900e-02, -2.44229230e-02, -2.71841100e-02,
            -4.23014360e-02, -2.66805480e-02, -1.06827190e-02, -2.21380400e-02, -1.59801580e-02,
            -2.13164810e-02, -1.28476770e-02, -2.37652280e-02, 9.69009295e-01, -2.31893840e-02,
            -1.21129400e-02, -1.78366630e-02, -3.05670880e-02, -2.84037240e-02, -6.55738000e-04
        ], [
            -1.19484670e-02, -3.14343850e-02, -3.73908300e-02, -2.78016740e-02, -2.47261260e-02,
            -2.40201620e-02, -5.84524630e-02, -1.79844340e-02, -6.22046180e-02, -3.50624710e-02,
            -3.27345560e-02, -2.91624070e-02, -4.34758570e-02, -5.94115000e-02, -3.80823990e-02,
            -3.97607780e-02, -3.90680010e-02, -5.57109490e-02, -4.64095510e-02, -5.54694920e-02,
            -6.48915420e-02, -3.49183420e-02, -4.16525660e-02, -4.62058850e-02, -9.96899500e-02,
            -6.84903230e-02, -7.23439320e-02, -9.09369800e-02, -3.65356890e-02, 8.64371828e-01,
            -6.00903540e-02, -2.79637990e-02, -5.42936310e-02, -6.79790110e-02, -2.24493000e-04
        ], [
            -1.66147700e-03, -3.22547400e-03, -1.76060700e-03, -1.14486200e-03, -8.22183000e-04,
            -3.24757800e-03, -3.02669900e-03, -1.13441800e-03, -3.07989100e-03, -2.11209600e-03,
            -2.24723800e-03, -2.45645200e-03, -1.23678900e-03, -1.46297700e-03, -1.18826000e-03,
            -2.28143100e-03, -6.08338000e-03, -1.38394400e-03, -2.17985700e-03, -1.60862700e-03,
            -3.43952500e-03, -3.45304200e-03, -3.03806900e-03, -7.85467400e-03, -9.95249200e-03,
            -3.46351000e-03, -2.82867200e-03, -1.60006800e-03, -3.30811500e-03, -2.67286600e-03,
            9.92286019e-01, -3.84375200e-03, -5.13695300e-03, -7.49547500e-03, -3.01490000e-06
        ], [
            -8.20751000e-04, -5.32727000e-04, -3.39980000e-04, -3.58169000e-04, -3.87841000e-04,
            -2.52363000e-04, -5.96689000e-04, -4.74332000e-04, -7.88635000e-04, -4.83537000e-04,
            -4.93862000e-04, -4.67633000e-04, -7.49497000e-04, -9.40670000e-04, -4.71700000e-04,
            -3.16210000e-04, -1.09764900e-03, -4.39320000e-04, -4.88801000e-04, -8.39621000e-04,
            -1.86835700e-03, -4.14147000e-04, -1.08908700e-03, -3.26702000e-04, -1.68003100e-03,
            -1.04600900e-03, -1.56575300e-03, -1.27117800e-03, -1.78569000e-04, -2.07190200e-03,
            -4.41167800e-03, 9.93577970e-01, -1.01418700e-03, -1.36618600e-03, -8.72964000e-07
        ], [
            -1.94205100e-03, -4.29249000e-04, -4.15162000e-04, -4.37552000e-04, -6.89506000e-04,
            -3.79451000e-04, -4.36995000e-04, -2.45603000e-04, -4.49492000e-04, -3.72917000e-04,
            -4.26141000e-04, -3.79665000e-04, -5.49105000e-04, -4.35666000e-04, -3.08307000e-04,
            -4.81052000e-04, -4.24076000e-04, -5.21606000e-04, -5.72193000e-04, -6.56224000e-04,
            -4.72679000e-04, -1.02480900e-03, -7.77496000e-04, -4.68097000e-04, -5.48396000e-04,
            -7.06767000e-04, -6.79086000e-04, -6.16829000e-04, -1.05267000e-04, -7.84117000e-04,
            -6.28859000e-03, -1.46596100e-03, 9.77467038e-01, -1.62432000e-03, -1.07937000e-07
        ], [
            -2.54810000e-03, -6.36026400e-03, -6.19131600e-03, -5.24926200e-03, -6.39461700e-03,
            -5.88426300e-03, -1.37525370e-02, -4.19932300e-03, -8.49751400e-03, -6.13336800e-03,
            -6.08891900e-03, -7.34355900e-03, -6.65061400e-03, -6.07097700e-03, -4.16120700e-03,
            -6.95932000e-03, -6.58794400e-03, -6.45767000e-03, -9.68912500e-03, -8.86167000e-03,
            -1.05072270e-02, -1.31254440e-02, -9.30243600e-03, -6.55781600e-03, -9.32257400e-03,
            -1.15470500e-02, -1.94424630e-02, -1.07228380e-02, -8.16392900e-03, -2.82981340e-02,
            -1.59418430e-02, -1.13237990e-02, -1.53496520e-02, 9.21316829e-01, -5.17703000e-04
        ], [
            -1.27881000e-05, -1.30555000e-05, -3.35149000e-05, -4.02804000e-05, -2.82574000e-05,
            -4.02956000e-05, -3.99982000e-05, -1.18184000e-05, -3.64898000e-05, -4.02854000e-05,
            -2.55138000e-05, -3.37570000e-05, -3.18059000e-05, -2.05234000e-05, -3.75568000e-05,
            -4.47019000e-05, -1.65868000e-05, -1.02034000e-04, -1.33615000e-04, -3.50090000e-05,
            -5.74162000e-05, -7.38739000e-05, -1.69120000e-04, -2.45854000e-05, -3.62652000e-05,
            -3.53916000e-05, -9.24426000e-05, -7.23952000e-05, -2.99940000e-04, -8.41283000e-05,
            -1.02276000e-04, -5.30281000e-05, -1.56425000e-04, -1.31215000e-04, 9.99973682e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 385)
    return ret


@cache('run')
def ia_matrix_x1999x():
    """
    ia matrix x1999x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.68684240e-01, -2.42297800e-03, -2.66148851e-01, -3.97066390e-02, -3.25684930e-02,
            -1.45684008e-01, -1.18797750e-02, -3.17723500e-03, -6.46879800e-03, -1.36284080e-02,
            -9.20189000e-04, -2.94182000e-04, -3.15697000e-04, -2.07127000e-04, -1.76158000e-04,
            -1.04250390e-02, -4.49511000e-04, -6.33649300e-03, -4.85853000e-04, -2.30830100e-03,
            -2.39718200e-03, -3.97368530e-02, -1.01405500e-03, -1.72647600e-03, -4.11803000e-04,
            -3.03949400e-03, -2.43280000e-04, -1.72042000e-04, -5.24739000e-04, -8.53588000e-04,
            -2.01588200e-03, -2.11440900e-03, -2.47074100e-03, -2.49920800e-03, -1.68528260e-02
        ], [
            -3.15114100e-03, 9.17525593e-01, -2.85437100e-03, -1.99251200e-03, -1.05692200e-03,
            -3.64776900e-03, -4.83207000e-03, -4.31581970e-01, -3.87822920e-02, -3.64351800e-03,
            -7.31281550e-02, -4.26904990e-02, -3.24818600e-03, -1.53071800e-03, -1.35767700e-03,
            -5.11901800e-03, -1.02404978e-01, -1.93907010e-02, -4.97901000e-04, -1.40905600e-03,
            -9.46596000e-04, -1.62957400e-03, -4.57922200e-03, -3.55535100e-03, -2.38260600e-03,
            -2.23703100e-03, -4.61939000e-04, -2.75993000e-04, -1.79287700e-03, -9.31225000e-04,
            -2.08380200e-03, -3.82777100e-03, -1.28076300e-03, -2.13524700e-03, -7.09896990e-02
        ], [
            -5.24961420e-02, -7.65765000e-04, 8.59592227e-01, -2.35052100e-03, -8.23441810e-02,
            -1.17613900e-03, -2.57464700e-03, -9.39005000e-04, -9.63344000e-03, -2.40444500e-03,
            -1.67837600e-03, -9.17534000e-04, -1.15147600e-03, -9.88400000e-04, -7.90248000e-04,
            -1.74223300e-03, -7.41705000e-04, -1.00416000e-03, -1.60215500e-03, -4.31748600e-03,
            -4.17517500e-03, -1.52825653e-01, -1.37552400e-03, -8.52634800e-03, -4.19638900e-03,
            -2.26206600e-03, -1.42707400e-03, -7.78865000e-04, -2.44373000e-04, -1.78268400e-03,
            -7.46395200e-03, -8.67896300e-03, -1.37634150e-02, -6.17173300e-03, -1.39521800e-03
        ], [
            -2.24057300e-03, -1.51168500e-03, -1.36223000e-03, 7.17894293e-01, -5.56375780e-02,
            -4.42254600e-03, -6.86887900e-03, -5.83430000e-04, -3.86853100e-03, -1.68787230e-02,
            -4.74330200e-03, -1.50079600e-03, -2.47809500e-03, -1.88954600e-03, -6.25481700e-03,
            -2.20193480e-02, -5.44414000e-04, -2.34994600e-03, -1.65198700e-03, -2.65254900e-03,
            -3.31237200e-03, -2.78305000e-03, -1.59874200e-03, -2.38058600e-03, -1.12430900e-03,
            -1.68034200e-03, -2.73202300e-03, -5.62164000e-04, -2.73708000e-04, -1.18335500e-03,
            -3.80090700e-03, -9.39315000e-04, -3.39695700e-03, -5.47877600e-03, -5.62437000e-04
        ], [
            -1.93652000e-04, -2.01338000e-04, -1.75272000e-04, -6.93297600e-03, 8.10879529e-01,
            -7.21434000e-04, -1.25028300e-03, -8.83567000e-05, -2.48687000e-04, -1.07902300e-03,
            -5.50777000e-04, -3.38904000e-04, -3.74978000e-04, -2.93151000e-04, -1.29367100e-03,
            -3.29292800e-03, -1.59530000e-04, -1.56532000e-04, -4.20527000e-04, -4.35189000e-04,
            -4.50351000e-04, -1.87716000e-04, -1.92788000e-04, -1.85642000e-04, -1.29950000e-04,
            -1.60444000e-04, -5.73804000e-04, -5.53705000e-05, -1.63002000e-05, -1.30733000e-04,
            -8.90536000e-04, -1.41477000e-04, -1.56025000e-04, -6.42795000e-04, -1.55081000e-05
        ], [
            -1.80143700e-03, -1.97403100e-03, -1.54459300e-03, -1.33035700e-03, -1.82209400e-03,
            8.03053341e-01, -1.07646760e-02, -3.06143000e-04, -1.45938200e-03, -2.91751500e-03,
            -5.94313100e-03, -2.70411100e-03, -2.90196600e-03, -1.63557200e-03, -2.88660700e-03,
            -7.05374130e-02, -8.15221000e-04, -3.28038970e-02, -7.20681000e-04, -1.55556400e-03,
            -1.45795600e-03, -2.12887100e-03, -1.29149600e-03, -5.10364000e-04, -3.70179000e-04,
            -3.73786400e-03, -7.86722000e-04, -4.79013000e-04, -2.25186900e-03, -8.75686000e-04,
            -1.43104000e-03, -1.80973900e-03, -9.87688000e-04, -2.72973600e-03, -3.34417000e-04
        ], [
            -2.72956500e-03, -3.02382000e-03, -2.41186180e-02, -1.08087950e-02, -1.32831470e-02,
            -1.17084030e-02, 7.66105199e-01, -2.00894900e-03, -1.85962760e-02, -1.79485860e-02,
            -2.22003720e-02, -5.85038000e-03, -9.73189400e-03, -1.37917090e-02, -5.82622300e-03,
            -2.01442770e-02, -3.91173700e-03, -4.91548000e-03, -7.77845000e-03, -1.45637660e-02,
            -1.39892590e-02, -9.61093400e-03, -6.05800000e-03, -3.50966200e-03, -6.65243500e-03,
            -1.51014330e-02, -1.21892200e-02, -1.50431670e-02, -2.74810100e-03, -2.23721120e-02,
            -1.71666380e-02, -1.85042040e-02, -9.92166700e-03, -2.51148670e-02, -7.69083000e-04
        ], [
            -1.23714840e-02, -1.28072080e-02, -3.57658500e-03, -4.98959200e-03, -3.54902500e-03,
            -4.50204700e-03, -4.32534000e-03, 9.50765947e-01, -2.64090050e-02, -7.61029800e-03,
            -1.61057390e-02, -8.94310100e-03, -3.55104200e-03, -2.37130500e-03, -1.87010600e-03,
            -3.93268300e-03, -1.90890470e-02, -1.26451910e-02, -4.36599700e-03, -5.80489300e-03,
            -4.77476600e-03, -3.56728300e-03, -5.18106530e-02, -6.13231420e-02, -6.16495050e-02,
            -1.48076480e-02, -3.68961100e-03, -1.27865700e-03, -1.36188400e-03, -4.48922800e-03,
            -7.81720200e-03, -2.89258300e-03, -3.19538700e-03, -6.00390500e-03, -8.78946000e-05
        ], [
            -3.93452050e-02, -1.26005730e-02, -1.06746370e-02, -7.05929430e-02, -3.54299340e-02,
            -2.63772760e-02, -3.41728070e-02, -1.44286900e-02, 7.72314425e-01, -1.99714635e-01,
            -3.04908880e-02, -1.53886570e-02, -1.29748450e-02, -1.78719880e-02, -1.32127680e-02,
            -2.85782640e-02, -6.46823100e-03, -1.02419880e-02, -6.81707200e-03, -3.50750600e-03,
            -2.59685700e-03, -4.35849100e-03, -3.29047300e-03, -3.21382600e-03, -2.61198200e-03,
            -3.76628100e-03, -1.49436300e-03, -9.65085000e-04, -1.69305400e-03, -4.63751300e-03,
            -8.49644600e-03, -3.68394000e-03, -6.44824020e-02, -9.75129200e-03, -1.55621000e-04
        ], [
            -4.69651100e-03, -8.56835300e-03, -1.59020690e-02, -9.18029700e-03, -3.65539700e-02,
            -9.01571300e-03, -1.46114940e-02, -2.07869900e-03, -1.88897540e-02, 8.84935040e-01,
            -1.07449190e-02, -7.43371700e-03, -2.72706110e-02, -2.48317330e-02, -3.64761980e-02,
            -3.55838070e-02, -2.55447700e-03, -1.80512930e-02, -1.81621920e-02, -4.77281500e-03,
            -5.35205200e-03, -4.15833700e-03, -1.05958330e-02, -2.76462600e-03, -2.80154500e-03,
            -6.16451500e-03, -3.49670600e-03, -7.03611000e-04, -1.19218100e-03, -2.61796000e-03,
            -3.26103700e-03, -9.43673000e-04, -5.81242600e-03, -3.44085600e-03, -1.08171000e-04
        ], [
            -1.99371600e-03, -4.85901700e-03, -5.66139700e-03, -2.10845900e-03, -1.80625200e-03,
            -8.55270100e-03, -1.36103100e-03, -2.20101800e-03, -6.46577200e-03, -6.21399300e-03,
            8.87758557e-01, -9.89337400e-03, -5.94229300e-03, -1.07956970e-02, -7.02276400e-03,
            -6.40301000e-03, -2.08249300e-03, -7.73374050e-02, -2.92823800e-03, -1.37582300e-03,
            -1.04692500e-03, -3.50947200e-03, -1.30590800e-03, -6.04800000e-04, -5.34368000e-04,
            -1.40911900e-03, -1.14768600e-03, -1.83189000e-04, -2.36864600e-03, -1.01916500e-03,
            -1.43148000e-03, -1.78445500e-03, -2.07579100e-03, -1.88376300e-03, -8.83670000e-05
        ], [
            -4.61559600e-03, -2.47191090e-02, -1.39611720e-02, -5.63337400e-03, -9.96818700e-03,
            -2.49029190e-02, -8.87044600e-03, -6.52896400e-03, -1.71643290e-02, -2.38102640e-02,
            -3.12370360e-02, 6.91470299e-01, -1.61601634e-01, -6.74627080e-02, -9.98360650e-02,
            -9.39841010e-02, -7.69357100e-03, -8.54013070e-02, -1.92153870e-02, -4.41327900e-03,
            -3.85461100e-03, -4.35544700e-03, -7.40912100e-03, -8.13629900e-03, -6.87543900e-03,
            -6.40008300e-03, -4.13487800e-03, -7.67182000e-04, -2.82348700e-03, -3.48407800e-03,
            -5.15464600e-03, -1.90686800e-03, -2.61666200e-03, -4.53511900e-03, -3.59263000e-04
        ], [
            -6.56870600e-03, -1.67066030e-02, -3.78108600e-03, -6.70422600e-03, -5.06130600e-03,
            -7.75874500e-03, -6.43213200e-03, -4.77820400e-03, -7.22507700e-03, -1.04125430e-02,
            -1.34767820e-02, -1.58374360e-02, 8.86598365e-01, -1.36482910e-02, -2.89016970e-02,
            -8.59200600e-03, -7.54295300e-03, -1.59289590e-02, -2.09485110e-02, -3.91382900e-03,
            -2.90687100e-03, -2.01052400e-03, -5.94331300e-03, -6.39296700e-03, -3.87006200e-03,
            -4.31236200e-03, -2.94807300e-03, -1.03830800e-03, -1.44352800e-03, -2.89074000e-03,
            -6.04026000e-03, -2.20797400e-03, -2.49734200e-03, -3.42304900e-03, -1.99195000e-04
        ], [
            -1.71273800e-03, -7.15782500e-03, -3.41614600e-03, -5.41659200e-03, -4.05631700e-03,
            -6.97113600e-03, -9.85686900e-03, -3.66359400e-03, -9.79327600e-03, -1.37451590e-02,
            -9.31977700e-03, -1.66289370e-02, -8.12207950e-02, 7.28344852e-01, -5.32789200e-02,
            -2.05548660e-02, -1.38419790e-02, -2.80170870e-02, -2.53508440e-02, -9.24349900e-03,
            -8.56778200e-03, -4.26019800e-03, -6.40964500e-03, -3.59568800e-03, -3.88391900e-03,
            -6.76566600e-03, -3.65432250e-02, -3.76154100e-03, -1.87046800e-03, -1.42930180e-02,
            -1.27604940e-02, -6.13888100e-03, -1.99084180e-02, -8.89766000e-03, -7.29307000e-04
        ], [
            -3.92866400e-03, -6.42433000e-03, -1.91173500e-03, -2.51816800e-03, -2.55864200e-03,
            -3.07315300e-03, -2.09243000e-03, -9.80640000e-04, -2.05782200e-03, -3.51648300e-03,
            -3.35418200e-03, -4.95705200e-03, -1.33767850e-02, -4.80579500e-03, 7.13787961e-01,
            -4.51275900e-03, -1.68609700e-03, -4.13397200e-03, -9.01516920e-02, -5.37012700e-03,
            -4.30123600e-03, -1.30630100e-03, -3.20565860e-02, -3.84135260e-02, -4.68181530e-02,
            -1.08986970e-02, -3.54497000e-03, -9.86397000e-04, -9.85704000e-04, -3.64595400e-03,
            -1.69826160e-02, -1.26972500e-03, -1.47307800e-03, -3.42693000e-03, -1.40412000e-04
        ], [
            -7.26135000e-04, -1.36425900e-03, -1.16101300e-03, -5.16708700e-03, -3.07867400e-03,
            -5.73252300e-03, -3.45714300e-03, -5.38794000e-04, -1.81635900e-03, -2.56575500e-03,
            -3.23978200e-03, -7.87694700e-03, -3.91540600e-03, -2.79741000e-03, -5.13205800e-03,
            9.62293456e-01, -1.14509400e-03, -5.11443200e-03, -2.36742100e-03, -1.37976900e-03,
            -1.64685600e-03, -2.62508300e-03, -1.63760100e-03, -8.33065000e-04, -1.05381200e-03,
            -1.47585100e-03, -9.54760000e-04, -1.12056200e-03, -1.68174300e-03, -2.10130000e-03,
            -3.91038500e-03, -3.44766400e-03, -2.93154300e-03, -5.34212400e-03, -4.01573800e-03
        ], [
            -9.49134800e-03, -2.35684880e-02, -1.14786010e-02, -1.73598660e-02, -7.51317000e-03,
            -1.55853940e-02, -2.11569270e-02, -1.41628700e-02, -2.78701690e-02, -2.16773680e-02,
            -3.47626000e-02, -2.68749140e-02, -1.12306290e-02, -9.27720300e-03, -8.39954800e-03,
            -9.51484700e-03, 9.13976040e-01, -5.34639100e-03, -9.98506100e-03, -6.30914100e-03,
            -1.42531670e-02, -2.37056610e-02, -1.45319240e-02, -3.68350700e-03, -3.89952400e-03,
            -1.22416880e-02, -8.53740700e-03, -4.73124800e-03, -1.06422600e-02, -6.67372000e-03,
            -1.58150900e-02, -1.87395830e-02, -1.29734330e-02, -1.50331220e-02, -2.61444000e-04
        ], [
            -3.66097900e-03, -7.43530500e-03, -2.35550100e-03, -2.61207000e-03, -2.01887100e-03,
            -3.69462100e-03, -3.59257100e-03, -2.54047400e-03, -3.92684000e-03, -3.33955700e-03,
            -7.29490600e-03, -4.65140500e-03, -3.57197300e-03, -3.06693000e-03, -2.33485400e-03,
            -2.84961900e-03, -2.59300490e-02, 9.62284823e-01, -4.17847600e-03, -3.90072400e-03,
            -4.62964400e-03, -5.29065800e-03, -9.55615200e-03, -2.79139200e-03, -2.07889700e-03,
            -1.46131540e-02, -1.16008990e-02, -5.31980200e-03, -4.29276750e-02, -5.17372200e-03,
            -1.93290460e-02, -1.51321020e-02, -6.61542200e-03, -1.11338400e-02, -2.93957600e-03
        ], [
            -4.81606800e-03, -4.35068800e-03, -6.40680900e-03, -5.59027200e-03, -7.61905000e-03,
            -5.77412300e-03, -4.70495700e-03, -3.30090900e-03, -6.07586800e-03, -6.03770100e-03,
            -7.49954300e-03, -5.93026400e-03, -5.40650800e-03, -4.43041000e-03, -4.75785900e-03,
            -6.23452400e-03, -9.13742700e-03, -7.34891900e-03, 9.90878223e-01, -4.68704700e-03,
            -4.48896200e-03, -5.06206900e-03, -1.83651110e-02, -3.02575900e-03, -3.19679200e-03,
            -7.45674200e-03, -4.78761000e-03, -2.60204600e-03, -2.18551000e-03, -6.01821100e-03,
            -4.39019200e-03, -3.22295400e-03, -3.47045000e-03, -6.53026300e-03, -5.30796000e-05
        ], [
            -3.30083840e-02, -2.81193420e-02, -6.28274030e-02, -5.74064000e-02, -7.27468540e-02,
            -5.36602280e-02, -5.00075410e-02, -3.74516660e-02, -4.92744070e-02, -4.91914570e-02,
            -4.37586610e-02, -4.51628170e-02, -4.86442670e-02, -4.75788980e-02, -4.46418800e-02,
            -4.84512470e-02, -2.23323220e-02, -4.25336330e-02, -3.77127060e-02, 9.70112405e-01,
            -1.36183650e-02, -4.48122700e-02, -2.27284880e-02, -1.96170320e-02, -2.09158750e-02,
            -2.03168920e-02, -1.07198870e-02, -4.71916800e-03, -4.12515000e-03, -1.28070380e-02,
            -1.97129630e-02, -1.05931180e-02, -2.83557430e-02, -1.56864460e-02, -3.59224000e-04
        ], [
            -1.64545830e-02, -1.09040760e-02, -3.02236590e-02, -2.75131850e-02, -3.95878290e-02,
            -2.12410080e-02, -1.83447260e-02, -1.20982510e-02, -2.21636680e-02, -2.09214670e-02,
            -1.95151120e-02, -1.64699280e-02, -1.73934440e-02, -1.50684860e-02, -1.92407710e-02,
            -2.57428590e-02, -8.21751600e-03, -2.83223190e-02, -1.71034430e-02, -8.01086200e-03,
            9.91687115e-01, -2.54912200e-02, -1.53935490e-02, -8.84689800e-03, -9.08803500e-03,
            -9.51672200e-03, -9.75149000e-03, -3.97293100e-03, -3.45884000e-03, -9.51849700e-03,
            -8.07378800e-03, -5.91968800e-03, -1.18183990e-02, -1.21429700e-02, -7.01861000e-04
        ], [
            -1.35914500e-03, -3.93028000e-03, -3.89588400e-03, -4.36852500e-03, -4.12403100e-03,
            -4.51956000e-03, -5.66436300e-03, -1.80222200e-03, -4.91647200e-03, -5.59018800e-03,
            -5.58355100e-03, -4.77236200e-03, -6.43444400e-03, -5.67624200e-03, -2.94055000e-03,
            -4.14480200e-03, -6.30957400e-03, -4.84666100e-03, -6.04759800e-03, -9.44905100e-03,
            -6.37154500e-03, 9.89648484e-01, -8.19925900e-03, -5.33594300e-03, -2.10962620e-02,
            -2.01303730e-02, -8.47863700e-03, -1.00311600e-02, -2.65370400e-03, -1.41959480e-02,
            -1.08412550e-02, -7.29298300e-03, -8.33687700e-03, -1.17503730e-02, -1.12354000e-04
        ], [
            -1.43204000e-02, -2.45072230e-02, -2.29215470e-02, -2.02878250e-02, -2.15459980e-02,
            -2.41480040e-02, -1.85916440e-02, -2.86176560e-02, -1.91521040e-02, -1.75878130e-02,
            -3.32496240e-02, -1.78652670e-02, -1.28739240e-02, -9.15099800e-03, -1.20790750e-02,
            -1.81085360e-02, -2.70667530e-02, -1.73239510e-02, -1.37553100e-02, -1.74047510e-02,
            -1.08643940e-02, -1.00184310e-02, 9.54270511e-01, -4.02014350e-02, -1.35450240e-02,
            -3.37641340e-02, -1.00623300e-02, -5.91071600e-03, -2.06015700e-03, -6.54532100e-03,
            -1.15936160e-02, -1.13106360e-02, -7.94580800e-03, -9.57702700e-03, -8.12767000e-04
        ], [
            -3.19180000e-03, -5.60705900e-03, -4.54510800e-03, -3.83653700e-03, -5.09319300e-03,
            -7.39843500e-03, -3.02390600e-03, -6.32283700e-03, -3.39195600e-03, -3.12307600e-03,
            -6.65738500e-03, -5.79158500e-03, -2.55218800e-03, -1.98549800e-03, -1.99097500e-03,
            -2.04531500e-03, -2.97546000e-03, -3.73783100e-03, -1.21579000e-03, -2.60028900e-03,
            -1.54862800e-03, -1.87103400e-03, -3.19500400e-03, 8.92025812e-01, -3.32083700e-03,
            -5.20148100e-03, -9.77011000e-04, -5.20044000e-04, -2.39504000e-04, -9.76758000e-04,
            -1.38102100e-03, -1.47526300e-03, -6.03755000e-04, -1.20735200e-03, -7.40572000e-05
        ], [
            -7.02038000e-04, -1.99096900e-03, -1.50811500e-03, -1.87596000e-03, -2.84746600e-03,
            -1.46425000e-03, -2.14473600e-03, -8.20355000e-04, -1.97523300e-03, -1.99142900e-03,
            -1.54629100e-03, -1.53038800e-03, -2.11757900e-03, -2.11783600e-03, -1.82589100e-03,
            -1.45397200e-03, -1.09555500e-03, -1.16228800e-03, -2.04505100e-03, -3.84489100e-03,
            -1.71996600e-03, -1.16928900e-03, -1.99319800e-03, -3.68821800e-03, 9.65462191e-01,
            -1.93619910e-02, -4.82251000e-03, -3.39566200e-03, -5.96694000e-04, -4.60029200e-03,
            -4.76572600e-03, -2.32253400e-03, -1.28795700e-03, -4.35471400e-03, -2.40927000e-05
        ], [
            -3.58665100e-03, -6.10972300e-03, -6.32777300e-03, -4.12904200e-03, -4.92750200e-03,
            -6.43842400e-03, -7.27445900e-03, -6.47336000e-03, -5.17715000e-03, -4.99220600e-03,
            -8.32296500e-03, -5.08489700e-03, -4.63885200e-03, -3.55346100e-03, -4.51622100e-03,
            -4.53456900e-03, -4.36441400e-03, -3.95668400e-03, -8.20158500e-03, -1.81812970e-02,
            -1.06898470e-02, -7.93021400e-03, -3.81430280e-02, -1.27819027e-01, -1.01403199e-01,
            8.77032968e-01, -3.38380400e-03, -4.06170500e-03, -9.43906000e-04, -5.07766700e-03,
            -4.77306000e-03, -2.63911800e-03, -3.16092600e-03, -5.77943000e-03, -7.63443000e-05
        ], [
            -2.16059800e-03, -6.01833900e-03, -3.91768700e-03, -4.68021700e-03, -5.14034000e-03,
            -4.56875400e-03, -1.13454530e-02, -3.00065100e-03, -6.81275100e-03, -5.70612200e-03,
            -5.57117300e-03, -5.58670400e-03, -7.37166900e-03, -6.93209000e-03, -3.82473100e-03,
            -6.47751500e-03, -6.00484900e-03, -9.70325400e-03, -1.20878880e-02, -1.94935400e-02,
            -1.98245580e-02, -1.17748040e-02, -1.48015360e-02, -1.75825030e-02, -2.86517310e-02,
            -2.38870640e-02, 8.87701448e-01, -2.60247270e-02, -4.35807900e-03, -2.25807710e-02,
            -1.93920520e-02, -8.83488800e-03, -1.10876680e-02, -2.26711910e-02, -1.58921000e-04
        ], [
            -1.99597830e-02, -3.21250310e-02, -1.64440950e-02, -2.23027060e-02, -1.79075110e-02,
            -1.95415810e-02, -2.21157250e-02, -1.26541130e-02, -2.04173810e-02, -1.89309880e-02,
            -2.34835770e-02, -2.00652000e-02, -2.06594170e-02, -1.63999910e-02, -1.43484620e-02,
            -2.17852280e-02, -2.63697000e-02, -2.11779240e-02, -2.61189880e-02, -4.10349420e-02,
            -3.71256370e-02, -2.47282720e-02, -4.62099610e-02, -4.18816850e-02, -3.92490890e-02,
            -2.74402570e-02, -2.44440170e-02, 8.08920548e-01, -5.68996400e-02, -3.89684840e-02,
            -2.57023400e-02, -1.48225480e-02, -2.31991230e-02, -3.60887370e-02, -4.03985900e-03
        ], [
            -5.34825700e-03, -5.01831300e-03, -4.03606000e-03, -5.16490300e-03, -5.06700200e-03,
            -5.23200400e-03, -7.99148400e-03, -2.87121700e-03, -4.56021000e-03, -5.90766700e-03,
            -5.08880100e-03, -5.18489200e-03, -6.14282800e-03, -4.73146000e-03, -3.98596800e-03,
            -7.77913000e-03, -6.12233300e-03, -1.12207280e-02, -2.33924090e-02, -2.63342850e-02,
            -4.12577240e-02, -2.58568150e-02, -1.03580010e-02, -2.03455210e-02, -1.39689080e-02,
            -2.09628790e-02, -1.33576770e-02, -2.37458040e-02, 9.68498771e-01, -2.29435180e-02,
            -1.35293710e-02, -1.80500980e-02, -3.19405050e-02, -2.65005440e-02, -6.92036000e-04
        ], [
            -1.17941100e-02, -3.04522050e-02, -3.75878640e-02, -2.86571730e-02, -2.56717140e-02,
            -2.44346010e-02, -5.78985850e-02, -1.73903440e-02, -6.35002370e-02, -3.56170360e-02,
            -3.32389950e-02, -2.95265610e-02, -4.43984030e-02, -5.76222630e-02, -3.77882660e-02,
            -3.42032610e-02, -4.08162420e-02, -5.31760010e-02, -4.68744420e-02, -5.54132930e-02,
            -6.52624350e-02, -3.57869410e-02, -4.15888920e-02, -4.23440570e-02, -9.43531290e-02,
            -6.65179010e-02, -7.26535390e-02, -9.38795120e-02, -3.55413180e-02, 8.62156236e-01,
            -6.34282900e-02, -2.81875560e-02, -5.43686750e-02, -7.02600360e-02, -2.36920000e-04
        ], [
            -1.60226300e-03, -3.18271800e-03, -1.71920100e-03, -1.15649600e-03, -8.60151000e-04,
            -3.15485600e-03, -2.96528800e-03, -1.25008700e-03, -2.94549200e-03, -2.11800800e-03,
            -2.17367500e-03, -2.51797600e-03, -1.25847000e-03, -1.40994800e-03, -1.16229100e-03,
            -1.91715700e-03, -6.33030300e-03, -1.38340900e-03, -2.26649400e-03, -1.54416900e-03,
            -3.37966000e-03, -3.45991000e-03, -2.95307400e-03, -6.99187500e-03, -9.38253400e-03,
            -3.42501900e-03, -2.92772800e-03, -1.65548500e-03, -3.41601200e-03, -2.80839200e-03,
            9.92645962e-01, -4.03522100e-03, -5.16738500e-03, -7.62393200e-03, -3.18179000e-06
        ], [
            -7.51384000e-04, -5.62102000e-04, -3.79733000e-04, -3.90997000e-04, -4.53497000e-04,
            -2.69972000e-04, -6.49448000e-04, -4.80915000e-04, -8.70227000e-04, -5.40525000e-04,
            -5.41719000e-04, -4.90996000e-04, -8.08118000e-04, -9.55307000e-04, -5.08190000e-04,
            -3.58058000e-04, -9.43264000e-04, -4.25663000e-04, -5.07651000e-04, -8.18727000e-04,
            -1.76640400e-03, -4.61574000e-04, -1.12579100e-03, -3.23779000e-04, -1.69790800e-03,
            -1.07028900e-03, -1.59022100e-03, -1.24091700e-03, -1.94457000e-04, -2.18891000e-03,
            -4.40323900e-03, 9.93369454e-01, -1.08333200e-03, -1.46610600e-03, -9.21287000e-07
        ], [
            -2.27573900e-03, -5.06621000e-04, -4.64903000e-04, -4.43249000e-04, -9.05399000e-04,
            -3.79965000e-04, -4.44153000e-04, -2.17847000e-04, -4.72835000e-04, -3.72320000e-04,
            -4.31527000e-04, -3.86659000e-04, -5.52906000e-04, -4.11579000e-04, -3.07064000e-04,
            -4.85693000e-04, -3.77709000e-04, -5.34072000e-04, -5.32213000e-04, -6.60599000e-04,
            -4.67571000e-04, -1.07050100e-03, -7.85900000e-04, -4.21653000e-04, -5.22167000e-04,
            -6.77184000e-04, -6.80221000e-04, -5.81831000e-04, -1.01037000e-04, -7.93382000e-04,
            -5.80853300e-03, -1.49257400e-03, 9.77119573e-01, -1.52768500e-03, -1.13911000e-07
        ], [
            -2.55109600e-03, -6.43162700e-03, -6.03766000e-03, -5.46462900e-03, -6.73441500e-03,
            -5.41708300e-03, -1.34703400e-02, -3.78531700e-03, -8.31188500e-03, -5.88010100e-03,
            -5.96734700e-03, -6.97288300e-03, -6.56417300e-03, -5.51613200e-03, -4.08994800e-03,
            -6.18577000e-03, -6.51725000e-03, -6.16277700e-03, -9.38798900e-03, -8.94116500e-03,
            -1.03143800e-02, -1.30644800e-02, -8.83369500e-03, -6.06707000e-03, -8.73893100e-03,
            -1.12909360e-02, -1.94819980e-02, -1.08515420e-02, -7.70490200e-03, -2.78749520e-02,
            -1.60650240e-02, -1.09200290e-02, -1.51871650e-02, 9.19985749e-01, -5.46361000e-04
        ], [
            -1.41814000e-05, -1.80947000e-05, -2.88465000e-05, -4.46990000e-05, -2.71306000e-05,
            -3.28999000e-05, -3.63865000e-05, -1.28306000e-05, -3.22773000e-05, -3.68142000e-05,
            -2.23390000e-05, -2.66350000e-05, -2.92626000e-05, -2.18194000e-05, -4.24215000e-05,
            -2.49480000e-05, -1.76084000e-05, -7.91595000e-05, -1.04801000e-04, -2.43351000e-05,
            -5.63432000e-05, -6.02808000e-05, -1.31893000e-04, -1.94472000e-05, -3.33877000e-05,
            -2.95610000e-05, -7.32409000e-05, -6.91712000e-05, -2.89134000e-04, -7.74291000e-05,
            -8.90481000e-05, -4.60337000e-05, -1.40025000e-04, -1.28978000e-04, 9.99972225e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 386)
    return ret


@cache('run')
def ia_matrix_x2000x():
    """
    ia matrix x2000x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.69645917e-01, -1.71056600e-03, -2.59683952e-01, -3.79350740e-02, -3.39944000e-02,
            -1.28116338e-01, -1.15111030e-02, -2.60641200e-03, -5.22461700e-03, -1.28483840e-02,
            -8.09544000e-04, -2.73754000e-04, -2.69196000e-04, -1.75765000e-04, -1.62405000e-04,
            -9.67625000e-03, -4.33682000e-04, -7.13126300e-03, -4.90607000e-04, -2.61283000e-03,
            -2.20816600e-03, -3.76146870e-02, -1.11581300e-03, -1.69555500e-03, -3.34468000e-04,
            -3.54809600e-03, -2.22814000e-04, -1.25807000e-04, -4.26838000e-04, -6.80882000e-04,
            -1.93279800e-03, -2.02328800e-03, -2.36400500e-03, -2.47844200e-03, -1.49646690e-02
        ], [
            -3.73729000e-03, 9.10538311e-01, -3.09540100e-03, -2.18250000e-03, -1.07769100e-03,
            -4.33027200e-03, -4.76932200e-03, -4.53979031e-01, -3.50820820e-02, -3.82798900e-03,
            -7.40709190e-02, -4.65997970e-02, -3.57026900e-03, -1.37169200e-03, -1.37701000e-03,
            -5.64807900e-03, -1.17370365e-01, -2.04457830e-02, -4.26999000e-04, -8.04239000e-04,
            -9.35464000e-04, -1.67733700e-03, -4.85951000e-03, -3.70075900e-03, -2.02398200e-03,
            -1.74956400e-03, -4.82607000e-04, -2.61793000e-04, -1.66030700e-03, -9.66788000e-04,
            -2.13242100e-03, -4.47964300e-03, -1.32263900e-03, -2.48421500e-03, -6.39877740e-02
        ], [
            -5.38528890e-02, -7.55421000e-04, 8.57434718e-01, -2.66785800e-03, -8.40657230e-02,
            -1.31553400e-03, -2.72047900e-03, -8.66592000e-04, -8.97714200e-03, -2.68644200e-03,
            -1.58525000e-03, -9.24906000e-04, -1.09513900e-03, -9.55136000e-04, -9.01956000e-04,
            -2.04919700e-03, -6.97721000e-04, -9.55748000e-04, -1.63276800e-03, -4.32934700e-03,
            -4.05741300e-03, -1.48954922e-01, -1.26389800e-03, -9.68050100e-03, -3.17243400e-03,
            -2.03981900e-03, -1.45206900e-03, -6.83158000e-04, -2.50011000e-04, -1.55908200e-03,
            -7.97277200e-03, -8.41335400e-03, -1.38466870e-02, -6.51051500e-03, -2.70562700e-03
        ], [
            -2.16147300e-03, -1.26019200e-03, -1.33714300e-03, 7.11380499e-01, -5.38375940e-02,
            -4.14352900e-03, -5.68283700e-03, -5.08027000e-04, -3.51695800e-03, -1.45523550e-02,
            -4.32300500e-03, -1.41732000e-03, -2.41647600e-03, -1.75694500e-03, -6.03575600e-03,
            -2.38393230e-02, -4.56864000e-04, -2.33210600e-03, -1.47550700e-03, -2.44229200e-03,
            -3.01892600e-03, -2.78332300e-03, -1.55652600e-03, -2.11337900e-03, -1.07809100e-03,
            -1.56080300e-03, -1.05324800e-03, -4.88914000e-04, -2.66701000e-04, -1.08564100e-03,
            -3.50357200e-03, -9.69164000e-04, -3.37419900e-03, -5.42066000e-03, -2.85036000e-04
        ], [
            -1.70070000e-04, -1.38642000e-04, -1.53691000e-04, -6.19560100e-03, 8.07849138e-01,
            -6.23692000e-04, -8.07469000e-04, -7.29396000e-05, -2.17703000e-04, -1.11318100e-03,
            -4.61377000e-04, -2.79405000e-04, -3.21979000e-04, -2.46887000e-04, -1.39356100e-03,
            -3.52996700e-03, -1.37971000e-04, -1.50238000e-04, -4.10145000e-04, -4.37500000e-04,
            -3.65036000e-04, -1.55886000e-04, -1.91843000e-04, -1.70271000e-04, -1.24055000e-04,
            -1.69529000e-04, -3.32256000e-04, -4.48662000e-05, -1.44184000e-05, -1.13028000e-04,
            -7.08735000e-04, -1.33359000e-04, -1.60926000e-04, -4.84902000e-04, -1.40049000e-05
        ], [
            -1.93152400e-03, -1.76725300e-03, -1.66889600e-03, -1.35003500e-03, -1.89610900e-03,
            7.93425863e-01, -1.02075950e-02, -2.92846000e-04, -1.43694700e-03, -2.96109800e-03,
            -5.88781900e-03, -2.68642900e-03, -2.74030100e-03, -1.45499200e-03, -2.70322100e-03,
            -7.41007540e-02, -7.64796000e-04, -3.21442070e-02, -6.80010000e-04, -1.56182200e-03,
            -1.39384900e-03, -2.20281300e-03, -1.22546000e-03, -5.14211000e-04, -3.46707000e-04,
            -3.27945600e-03, -7.39832000e-04, -4.04184000e-04, -2.16931000e-03, -8.52307000e-04,
            -1.41265800e-03, -1.92304300e-03, -9.76874000e-04, -2.79566000e-03, -1.68483000e-04
        ], [
            -2.65252500e-03, -2.99006400e-03, -2.48424810e-02, -1.07669740e-02, -1.30413700e-02,
            -1.26766520e-02, 7.68076982e-01, -1.40026100e-03, -1.77223090e-02, -1.76825640e-02,
            -2.14365040e-02, -5.59654900e-03, -9.09888100e-03, -1.23126210e-02, -5.20278000e-03,
            -2.22377140e-02, -3.95886600e-03, -4.82237000e-03, -7.78953900e-03, -1.39792860e-02,
            -1.36187280e-02, -9.60244100e-03, -5.97178600e-03, -3.12726800e-03, -6.18702200e-03,
            -1.39719990e-02, -1.18672500e-02, -1.42414950e-02, -2.56564000e-03, -2.25251170e-02,
            -1.67295410e-02, -1.82036630e-02, -9.52189700e-03, -2.50804440e-02, -3.07766000e-04
        ], [
            -1.25116110e-02, -1.12479010e-02, -3.34331000e-03, -5.17563600e-03, -3.64738400e-03,
            -4.84613000e-03, -4.17000300e-03, 9.56887892e-01, -2.85416400e-02, -7.58674100e-03,
            -1.56842700e-02, -8.29698500e-03, -3.39697100e-03, -2.06229400e-03, -1.78102000e-03,
            -3.84102800e-03, -1.85767130e-02, -1.20010530e-02, -4.15698800e-03, -5.69818000e-03,
            -4.43605200e-03, -3.57915700e-03, -5.10715780e-02, -6.75315260e-02, -6.46276840e-02,
            -1.40173440e-02, -3.28289500e-03, -1.22427000e-03, -1.09285600e-03, -4.40828200e-03,
            -7.56518600e-03, -2.80459700e-03, -3.06170500e-03, -5.37734100e-03, -5.83827000e-05
        ], [
            -3.85596910e-02, -1.12918360e-02, -1.05909410e-02, -6.99464460e-02, -3.55162510e-02,
            -3.04247060e-02, -3.36621530e-02, -1.33802150e-02, 7.68574499e-01, -2.00300407e-01,
            -3.08316000e-02, -1.49899680e-02, -1.14767860e-02, -1.66907570e-02, -1.19011700e-02,
            -2.94925290e-02, -6.39235500e-03, -1.02365650e-02, -6.06942800e-03, -3.27362200e-03,
            -2.42368000e-03, -4.67513900e-03, -3.41811600e-03, -3.70595900e-03, -3.78838800e-03,
            -3.72846700e-03, -1.49648100e-03, -9.22535000e-04, -1.70507300e-03, -4.74142400e-03,
            -8.98217900e-03, -3.70313400e-03, -6.22293140e-02, -9.77183800e-03, -1.48825000e-04
        ], [
            -4.89155600e-03, -7.68157500e-03, -1.65467000e-02, -9.47500100e-03, -3.63229140e-02,
            -9.51104300e-03, -1.46339320e-02, -2.25430000e-03, -1.84799490e-02, 8.82905812e-01,
            -1.05931900e-02, -7.31185300e-03, -2.66801660e-02, -2.31879330e-02, -3.49126940e-02,
            -3.73329950e-02, -2.58391000e-03, -1.83357660e-02, -1.73117770e-02, -4.72656400e-03,
            -5.22646600e-03, -4.16454900e-03, -1.01689080e-02, -2.64784200e-03, -2.75909200e-03,
            -6.07938900e-03, -3.42130700e-03, -6.48649000e-04, -1.14712300e-03, -2.60375800e-03,
            -3.18503800e-03, -8.91421000e-04, -5.56982300e-03, -3.42442300e-03, -6.04898000e-05
        ], [
            -2.17563300e-03, -4.54220100e-03, -5.98207700e-03, -2.21567000e-03, -1.96110400e-03,
            -8.59931500e-03, -1.32738900e-03, -1.80754000e-03, -6.16592000e-03, -6.38109200e-03,
            8.86568736e-01, -9.96217600e-03, -5.84121500e-03, -1.04548640e-02, -6.94308900e-03,
            -6.66834700e-03, -1.93393100e-03, -7.66706240e-02, -2.90377300e-03, -1.55068300e-03,
            -1.09126900e-03, -3.70067100e-03, -1.25695900e-03, -6.43650000e-04, -5.60002000e-04,
            -1.48104700e-03, -1.22642900e-03, -1.89855000e-04, -2.12564900e-03, -1.04087300e-03,
            -1.42487600e-03, -1.80008600e-03, -2.15560900e-03, -2.02800400e-03, -5.40018000e-05
        ], [
            -4.53042200e-03, -2.29814850e-02, -1.39307550e-02, -5.60724600e-03, -1.01059730e-02,
            -2.58476230e-02, -8.92488800e-03, -6.14515900e-03, -1.51909600e-02, -2.46314670e-02,
            -3.12263990e-02, 6.89684971e-01, -1.58319387e-01, -6.18672740e-02, -9.71371980e-02,
            -9.66942420e-02, -7.08485200e-03, -8.56236330e-02, -1.75791050e-02, -4.41101100e-03,
            -3.80714000e-03, -4.44183100e-03, -6.78596100e-03, -6.61763700e-03, -6.20091600e-03,
            -6.53812000e-03, -4.05644700e-03, -6.93859000e-04, -2.62668300e-03, -3.44432400e-03,
            -4.80061900e-03, -1.77116100e-03, -2.49896400e-03, -4.57657300e-03, -2.26787000e-04
        ], [
            -6.77013900e-03, -1.52288680e-02, -4.10177600e-03, -7.17865100e-03, -5.07266100e-03,
            -7.93243000e-03, -6.55049600e-03, -5.37455900e-03, -6.61466700e-03, -1.06757130e-02,
            -1.42435060e-02, -1.55475650e-02, 8.82442825e-01, -1.32877730e-02, -2.97708860e-02,
            -9.34786700e-03, -7.70563700e-03, -1.64650530e-02, -2.18972030e-02, -4.16853000e-03,
            -2.81071500e-03, -2.13714400e-03, -5.94795300e-03, -7.67324300e-03, -4.03111200e-03,
            -4.27414100e-03, -2.97185600e-03, -9.87612000e-04, -1.41380100e-03, -2.96004700e-03,
            -6.38418700e-03, -2.09992500e-03, -2.54669500e-03, -3.51346000e-03, -1.24370000e-04
        ], [
            -1.87725200e-03, -6.07642400e-03, -3.70675400e-03, -6.35859200e-03, -4.58663600e-03,
            -7.35362300e-03, -1.07496100e-02, -3.73439400e-03, -1.00835690e-02, -1.48277040e-02,
            -1.04717200e-02, -1.78886310e-02, -8.52147630e-02, 7.07341610e-01, -5.50966870e-02,
            -2.55043890e-02, -1.49438520e-02, -3.02577370e-02, -2.72807430e-02, -1.01887170e-02,
            -8.84022000e-03, -4.79463300e-03, -6.54907900e-03, -4.02557300e-03, -4.23901300e-03,
            -7.42299100e-03, -4.26386300e-02, -3.89594600e-03, -1.93014700e-03, -1.50429480e-02,
            -1.38924760e-02, -6.00073000e-03, -2.02336990e-02, -9.75602800e-03, -6.21100000e-04
        ], [
            -4.06700500e-03, -6.14419900e-03, -2.11634700e-03, -2.70840800e-03, -2.40684500e-03,
            -3.27494500e-03, -2.22328200e-03, -8.63733000e-04, -2.35758500e-03, -3.91326900e-03,
            -3.47921500e-03, -4.93638700e-03, -1.29611130e-02, -4.51631200e-03, 7.07387894e-01,
            -4.51966100e-03, -1.76543500e-03, -4.22848700e-03, -9.10374100e-02, -5.83836000e-03,
            -4.69429500e-03, -1.33435200e-03, -3.26256080e-02, -3.90383550e-02, -4.87601520e-02,
            -1.19576650e-02, -3.52842000e-03, -1.02705300e-03, -9.26581000e-04, -3.93443100e-03,
            -1.71769310e-02, -1.38564500e-03, -1.42901000e-03, -3.52646200e-03, -2.02191000e-04
        ], [
            -7.19088000e-04, -1.27213100e-03, -1.16412100e-03, -5.35612500e-03, -3.07892600e-03,
            -5.05237700e-03, -3.73307300e-03, -6.72229000e-04, -1.80120800e-03, -2.61919500e-03,
            -3.04577300e-03, -9.55645300e-03, -4.00614400e-03, -2.67003200e-03, -4.66101600e-03,
            9.57886563e-01, -1.21821900e-03, -5.05807800e-03, -2.41531200e-03, -1.35264100e-03,
            -1.57253700e-03, -2.80074900e-03, -1.74379000e-03, -8.25923000e-04, -1.24156600e-03,
            -1.65414500e-03, -1.04242200e-03, -1.01384900e-03, -1.47004800e-03, -1.99550800e-03,
            -3.87646500e-03, -3.41624900e-03, -2.71972600e-03, -5.56783000e-03, -2.31479000e-03
        ], [
            -1.01825410e-02, -2.56055610e-02, -1.25543440e-02, -1.80649110e-02, -7.98733800e-03,
            -1.79415010e-02, -2.26144510e-02, -1.22164010e-02, -2.79994780e-02, -2.23801940e-02,
            -3.68992690e-02, -2.72567110e-02, -1.11950400e-02, -8.69274600e-03, -8.80807200e-03,
            -1.00877980e-02, 9.11034092e-01, -5.64666000e-03, -1.01412970e-02, -6.44773600e-03,
            -1.45871110e-02, -2.44061250e-02, -1.48703070e-02, -3.63354800e-03, -3.95471200e-03,
            -1.26840530e-02, -9.00732200e-03, -4.69853000e-03, -1.17708730e-02, -6.78963500e-03,
            -1.59993790e-02, -1.98020420e-02, -1.37410950e-02, -1.57765190e-02, -2.01186000e-04
        ], [
            -3.74601100e-03, -6.60087200e-03, -2.33047600e-03, -2.61277300e-03, -1.89557800e-03,
            -3.67359100e-03, -3.35522800e-03, -2.11629000e-03, -3.45575400e-03, -3.03748900e-03,
            -6.90462600e-03, -4.27768100e-03, -3.28062800e-03, -2.61204900e-03, -2.10480400e-03,
            -2.74367300e-03, -2.52460620e-02, 9.58006506e-01, -4.32858200e-03, -3.85537600e-03,
            -4.82571800e-03, -5.53301000e-03, -9.36990900e-03, -2.73477200e-03, -2.13263500e-03,
            -1.26412950e-02, -1.12885270e-02, -4.90312400e-03, -4.18146230e-02, -4.86353500e-03,
            -1.90984850e-02, -1.52133990e-02, -6.61265800e-03, -1.04375070e-02, -1.47113400e-03
        ], [
            -4.98027100e-03, -4.06067500e-03, -6.69583500e-03, -5.87962400e-03, -7.94011500e-03,
            -6.07621400e-03, -4.84610000e-03, -2.98244500e-03, -6.19113900e-03, -6.24466800e-03,
            -7.37183600e-03, -6.00897500e-03, -5.15548400e-03, -4.18869100e-03, -4.93525900e-03,
            -6.34728800e-03, -8.40825000e-03, -7.23847300e-03, 9.89417105e-01, -4.12053900e-03,
            -4.20226800e-03, -5.03116100e-03, -1.69061790e-02, -2.40355900e-03, -3.15522400e-03,
            -7.08727300e-03, -4.19568600e-03, -2.45751300e-03, -2.01472700e-03, -5.81421500e-03,
            -4.42359100e-03, -3.05025100e-03, -3.54311100e-03, -6.61314000e-03, -3.38226000e-05
        ], [
            -3.34778390e-02, -2.77956830e-02, -6.39761810e-02, -5.51818720e-02, -7.16281590e-02,
            -5.54628270e-02, -4.94917520e-02, -3.77532320e-02, -4.91723920e-02, -4.88071070e-02,
            -4.36400440e-02, -4.54036170e-02, -4.75412820e-02, -4.44911780e-02, -4.44561420e-02,
            -5.36412160e-02, -2.28205160e-02, -4.13878530e-02, -3.71042920e-02, 9.71214496e-01,
            -1.39117960e-02, -4.38485800e-02, -2.24273520e-02, -1.96993680e-02, -2.08253150e-02,
            -2.02363050e-02, -1.06846110e-02, -4.28807800e-03, -3.64239900e-03, -1.31262250e-02,
            -2.02157560e-02, -1.05275250e-02, -2.71381910e-02, -1.55784750e-02, -2.26252000e-04
        ], [
            -1.66908940e-02, -1.00383100e-02, -3.03520230e-02, -2.71754150e-02, -3.88077410e-02,
            -2.09544130e-02, -1.86821670e-02, -1.07957500e-02, -2.22622550e-02, -2.06630660e-02,
            -1.86227120e-02, -1.58494500e-02, -1.66297680e-02, -1.39390020e-02, -1.88885210e-02,
            -2.56573430e-02, -7.95233500e-03, -2.83050940e-02, -1.63160140e-02, -7.48229300e-03,
            9.92026384e-01, -2.55542270e-02, -1.42224150e-02, -8.40891300e-03, -8.86719900e-03,
            -9.13481200e-03, -9.02514700e-03, -4.01341200e-03, -3.13807800e-03, -9.15433700e-03,
            -8.17889000e-03, -5.75575300e-03, -1.16288610e-02, -1.18023390e-02, -4.21444000e-04
        ], [
            -1.30509100e-03, -3.25647800e-03, -3.74148800e-03, -4.12200600e-03, -4.01006700e-03,
            -4.71560000e-03, -5.68394900e-03, -1.16944900e-03, -4.36240300e-03, -5.42834800e-03,
            -5.23331100e-03, -4.45145100e-03, -5.97161800e-03, -5.03909600e-03, -2.84432200e-03,
            -4.19011100e-03, -6.69840200e-03, -4.46712500e-03, -5.89057600e-03, -8.70001500e-03,
            -6.00786800e-03, 9.89898438e-01, -7.90803600e-03, -4.19774700e-03, -2.13699640e-02,
            -1.87325120e-02, -8.64539300e-03, -9.37384200e-03, -2.60345700e-03, -1.40478960e-02,
            -1.05325500e-02, -6.71361600e-03, -8.26005000e-03, -1.18262960e-02, -7.63822000e-05
        ], [
            -1.33858620e-02, -2.13350170e-02, -2.33316040e-02, -2.00324940e-02, -2.17707720e-02,
            -2.41246360e-02, -1.90230470e-02, -2.57642940e-02, -1.83425600e-02, -1.72162600e-02,
            -3.35404680e-02, -1.72889610e-02, -1.25076720e-02, -8.40373000e-03, -1.21207680e-02,
            -1.89214620e-02, -2.37191630e-02, -1.60846190e-02, -1.38850950e-02, -1.92238400e-02,
            -1.14877020e-02, -9.66068000e-03, 9.54184625e-01, -4.28540730e-02, -1.31121920e-02,
            -5.98202400e-02, -9.46170600e-03, -5.47305200e-03, -1.80141700e-03, -6.18510500e-03,
            -1.12100050e-02, -1.02842560e-02, -7.41848700e-03, -9.45637300e-03, -4.61871000e-04
        ], [
            -3.57521600e-03, -5.60032200e-03, -4.92494100e-03, -4.38117600e-03, -6.06715700e-03,
            -8.20122800e-03, -3.55771500e-03, -5.39717800e-03, -3.35709500e-03, -3.40287300e-03,
            -7.77902500e-03, -6.12183300e-03, -2.82989700e-03, -2.13863800e-03, -2.12335200e-03,
            -2.57685500e-03, -2.84682700e-03, -4.02548500e-03, -1.17593800e-03, -2.74778700e-03,
            -1.53175800e-03, -2.08488500e-03, -3.21210100e-03, 9.01025364e-01, -3.16431000e-03,
            -6.18113200e-03, -8.79505000e-04, -4.72539000e-04, -2.40324000e-04, -9.27105000e-04,
            -1.24293200e-03, -1.40573700e-03, -5.97107000e-04, -1.22364200e-03, -1.89214000e-05
        ], [
            -7.02209000e-04, -1.89233800e-03, -1.45277600e-03, -1.86593500e-03, -2.91427700e-03,
            -1.59412400e-03, -2.39070900e-03, -6.28888000e-04, -1.89982800e-03, -2.02734900e-03,
            -1.60123600e-03, -1.47725500e-03, -2.26489300e-03, -2.13934200e-03, -1.85541800e-03,
            -1.60814000e-03, -1.15464900e-03, -1.17516700e-03, -1.85000300e-03, -3.57236300e-03,
            -1.61762900e-03, -1.20296200e-03, -1.86300300e-03, -3.80468200e-03, 9.64949174e-01,
            -1.97969790e-02, -4.32248100e-03, -3.31382200e-03, -6.00806000e-04, -4.58653200e-03,
            -4.36126900e-03, -2.21333700e-03, -1.26459000e-03, -4.51273300e-03, -1.24950000e-05
        ], [
            -3.56529300e-03, -5.46337200e-03, -6.19539400e-03, -4.10723900e-03, -5.24867000e-03,
            -7.22589300e-03, -8.63769200e-03, -5.64294100e-03, -5.08059700e-03, -5.10167500e-03,
            -8.38912700e-03, -5.06949600e-03, -4.69872600e-03, -3.52248200e-03, -4.52169000e-03,
            -4.73062100e-03, -4.30348800e-03, -3.95238600e-03, -9.40433600e-03, -2.17944130e-02,
            -1.07052850e-02, -7.89669300e-03, -4.11907550e-02, -1.32571599e-01, -9.57297390e-02,
            8.93633613e-01, -4.65420500e-03, -3.83287300e-03, -8.52613000e-04, -5.09527000e-03,
            -4.81002800e-03, -2.58407500e-03, -2.99760300e-03, -5.94350500e-03, -4.06301000e-05
        ], [
            -2.29236400e-03, -5.81364000e-03, -4.26725700e-03, -4.98984000e-03, -5.68613800e-03,
            -5.23143400e-03, -1.22924660e-02, -2.39728000e-03, -7.36324100e-03, -6.03584200e-03,
            -6.14534800e-03, -5.65699600e-03, -7.56553800e-03, -6.89440700e-03, -4.37892700e-03,
            -7.15007600e-03, -6.30834600e-03, -1.07179430e-02, -1.33019900e-02, -2.03915890e-02,
            -2.09112120e-02, -1.27100470e-02, -1.50209460e-02, -1.72602080e-02, -2.63459010e-02,
            -2.35819680e-02, 8.78383098e-01, -2.78007640e-02, -4.80808900e-03, -2.43341570e-02,
            -2.04518090e-02, -9.41139100e-03, -1.18500260e-02, -2.36618750e-02, -3.15848000e-04
        ], [
            -2.03821940e-02, -3.05197300e-02, -1.56910930e-02, -2.16861110e-02, -1.69302130e-02,
            -1.99645250e-02, -2.35754040e-02, -1.09006050e-02, -1.86944100e-02, -1.80410570e-02,
            -2.28449610e-02, -1.85611360e-02, -1.87423330e-02, -1.47374890e-02, -1.41854450e-02,
            -2.24496770e-02, -2.57493840e-02, -2.10912210e-02, -2.42248900e-02, -3.86478080e-02,
            -3.57053980e-02, -2.53203680e-02, -4.71560830e-02, -4.09628990e-02, -3.81523710e-02,
            -2.57814450e-02, -2.30614770e-02, 8.02261867e-01, -5.97102530e-02, -3.82918450e-02,
            -2.49818710e-02, -1.26932540e-02, -2.26412840e-02, -3.73341060e-02, -5.60951200e-03
        ], [
            -5.29093600e-03, -5.08894300e-03, -4.37820700e-03, -5.04262200e-03, -4.70968800e-03,
            -5.67021400e-03, -8.96387700e-03, -2.05461800e-03, -4.52316300e-03, -6.08976300e-03,
            -4.88509700e-03, -5.14596100e-03, -5.97795400e-03, -4.42992800e-03, -4.46442200e-03,
            -7.45980400e-03, -5.73221400e-03, -9.33572900e-03, -2.33819510e-02, -2.44356170e-02,
            -4.23012320e-02, -2.79797340e-02, -9.58868600e-03, -1.98556210e-02, -1.36472650e-02,
            -2.07618010e-02, -1.63217550e-02, -2.40402980e-02, 9.65290311e-01, -2.40001060e-02,
            -1.39644430e-02, -1.84549580e-02, -3.29025790e-02, -2.61221850e-02, -2.17713000e-04
        ], [
            -1.17201700e-02, -3.02333010e-02, -3.87362560e-02, -2.81632600e-02, -2.45472870e-02,
            -2.54141710e-02, -6.34851290e-02, -1.28772570e-02, -6.45078810e-02, -3.68928350e-02,
            -3.29984530e-02, -2.94769820e-02, -4.38745470e-02, -5.24078530e-02, -3.93445210e-02,
            -3.63057610e-02, -4.23805470e-02, -5.52348600e-02, -4.81608130e-02, -5.61393330e-02,
            -6.41435930e-02, -3.89119020e-02, -4.25388810e-02, -3.77242260e-02, -9.45751230e-02,
            -6.60920460e-02, -7.40166940e-02, -9.67860240e-02, -3.31306650e-02, 8.60112142e-01,
            -6.47975920e-02, -2.93428060e-02, -5.49369330e-02, -7.49792450e-02, -1.73238300e-03
        ], [
            -1.53329600e-03, -3.52352600e-03, -1.80898200e-03, -1.16181200e-03, -8.59841000e-04,
            -3.38919700e-03, -2.91006400e-03, -7.80683000e-04, -2.69158600e-03, -2.08190900e-03,
            -2.16623600e-03, -2.17323600e-03, -1.17977800e-03, -1.24226900e-03, -1.19861800e-03,
            -2.03152300e-03, -4.79311500e-03, -1.61283700e-03, -2.19655600e-03, -1.58207500e-03,
            -3.20119100e-03, -3.58110300e-03, -2.82842900e-03, -6.52675500e-03, -8.13696800e-03,
            -3.13430600e-03, -2.96432400e-03, -1.56750800e-03, -3.06347800e-03, -2.65468900e-03,
            9.92754556e-01, -4.05003400e-03, -4.47301400e-03, -7.72696800e-03, -7.21756000e-06
        ], [
            -6.49592000e-04, -4.73713000e-04, -4.53634000e-04, -4.20937000e-04, -4.30526000e-04,
            -3.48009000e-04, -7.17030000e-04, -3.52765000e-04, -9.42392000e-04, -6.21962000e-04,
            -5.99509000e-04, -6.16173000e-04, -8.07938000e-04, -8.47838000e-04, -7.61920000e-04,
            -4.42849000e-04, -9.82237000e-04, -4.88133000e-04, -7.27229000e-04, -8.36370000e-04,
            -1.75627700e-03, -5.33268000e-04, -1.18519700e-03, -3.71192000e-04, -1.65905800e-03,
            -1.14824200e-03, -1.53155700e-03, -1.24811400e-03, -2.89270000e-04, -2.34722500e-03,
            -4.46265600e-03, 9.90436000e-01, -1.09285600e-03, -1.59562900e-03, -3.04567000e-06
        ], [
            -1.54317500e-03, -4.08586000e-04, -4.74207000e-04, -4.22906000e-04, -8.98024000e-04,
            -3.97119000e-04, -4.23144000e-04, -1.62654000e-04, -4.94802000e-04, -3.97268000e-04,
            -4.06935000e-04, -3.60302000e-04, -5.25006000e-04, -3.77479000e-04, -3.43459000e-04,
            -4.67281000e-04, -3.49448000e-04, -4.79681000e-04, -5.90122000e-04, -6.27712000e-04,
            -4.84990000e-04, -1.13234500e-03, -7.44808000e-04, -3.84915000e-04, -4.87678000e-04,
            -6.13436000e-04, -7.19737000e-04, -5.64745000e-04, -9.99390000e-05, -7.92345000e-04,
            -5.59470800e-03, -1.47453000e-03, 9.74990608e-01, -1.31079300e-03, -1.56567000e-07
        ], [
            -2.49435000e-03, -6.25173400e-03, -5.87297700e-03, -5.27695300e-03, -6.26516100e-03,
            -5.46552300e-03, -1.33355680e-02, -2.72073400e-03, -7.80179500e-03, -5.57794400e-03,
            -5.56736700e-03, -6.40291200e-03, -6.03313700e-03, -4.83425600e-03, -4.36000800e-03,
            -6.43106300e-03, -6.31769600e-03, -6.18628200e-03, -8.97510500e-03, -8.45113600e-03,
            -9.33925900e-03, -1.36459160e-02, -8.04686900e-03, -5.48695400e-03, -8.59932900e-03,
            -1.12235750e-02, -1.98538250e-02, -1.06303220e-02, -7.66493700e-03, -2.78107530e-02,
            -1.55394180e-02, -1.05695880e-02, -1.51531150e-02, 9.20320471e-01, -1.33139600e-03
        ], [
            -1.63360000e-05, -1.97956000e-05, -2.85695000e-05, -5.32798000e-05, -2.79486000e-05,
            -3.83786000e-05, -4.07698000e-05, -1.24051000e-05, -3.46268000e-05, -4.08863000e-05,
            -2.28410000e-05, -2.36591000e-05, -2.75941000e-05, -2.03107000e-05, -3.96587000e-05,
            -2.71574000e-05, -1.66392000e-05, -7.72898000e-05, -1.05669000e-04, -2.16105000e-05,
            -5.31545000e-05, -6.02959000e-05, -1.04877000e-04, -1.55566000e-05, -2.71625000e-05,
            -2.45078000e-05, -6.14635000e-05, -6.45951000e-05, -2.85292000e-04, -7.31980000e-05,
            -8.23840000e-05, -4.19294000e-05, -1.12235000e-04, -1.35981000e-04, 9.99924150e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 387)
    return ret


@cache('run')
def ia_matrix_x2001x():
    """
    ia matrix x2001x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.71009342e-01, -1.30797000e-03, -2.56410650e-01, -3.84148180e-02, -3.55696110e-02,
            -1.25822581e-01, -1.16950000e-02, -1.74143900e-03, -5.02429400e-03, -1.35028660e-02,
            -7.05558000e-04, -2.57418000e-04, -2.64066000e-04, -1.88959000e-04, -1.53783000e-04,
            -9.66078500e-03, -4.37962000e-04, -8.64806300e-03, -4.72555000e-04, -2.89747200e-03,
            -2.37260700e-03, -3.78170460e-02, -1.31487300e-03, -1.50552600e-03, -3.14297000e-04,
            -3.89048900e-03, -2.45394000e-04, -1.39340000e-04, -3.85154000e-04, -6.50331000e-04,
            -2.01858900e-03, -2.14067600e-03, -2.23650800e-03, -2.39171600e-03, -1.58910510e-02
        ], [
            -3.23134900e-03, 9.24562415e-01, -2.58087500e-03, -1.94322500e-03, -8.49562000e-04,
            -3.70330300e-03, -4.12063400e-03, -4.34928026e-01, -3.32943680e-02, -3.05412300e-03,
            -6.65674540e-02, -4.37872240e-02, -2.98195300e-03, -1.25025400e-03, -1.19456700e-03,
            -5.64433600e-03, -1.09148246e-01, -1.77243040e-02, -4.43248000e-04, -7.17295000e-04,
            -9.85758000e-04, -1.53025700e-03, -3.94611100e-03, -2.65834200e-03, -1.99205600e-03,
            -1.56470700e-03, -4.84474000e-04, -2.61638000e-04, -1.80393500e-03, -8.28771000e-04,
            -1.85995600e-03, -3.39083100e-03, -1.00830100e-03, -2.45463500e-03, -6.24078360e-02
        ], [
            -5.44539810e-02, -7.18921000e-04, 8.53886462e-01, -2.78272900e-03, -8.29329580e-02,
            -1.30140900e-03, -2.67399800e-03, -8.20696000e-04, -9.60506900e-03, -2.85487900e-03,
            -1.51381600e-03, -9.43530000e-04, -1.11195100e-03, -9.90502000e-04, -8.99276000e-04,
            -2.08344500e-03, -7.01955000e-04, -9.58400000e-04, -1.70075300e-03, -4.59164700e-03,
            -4.39094700e-03, -1.48280380e-01, -1.26299700e-03, -8.07671800e-03, -3.22853600e-03,
            -1.85518200e-03, -1.61244700e-03, -7.16764000e-04, -2.41357000e-04, -1.55686000e-03,
            -8.26102100e-03, -7.72254900e-03, -1.33918480e-02, -6.25097200e-03, -3.16299400e-03
        ], [
            -2.04124900e-03, -1.13511700e-03, -1.32070500e-03, 7.07021092e-01, -5.02215900e-02,
            -3.53014600e-03, -5.29110300e-03, -5.69802000e-04, -3.26053200e-03, -1.22626370e-02,
            -4.19833800e-03, -1.43649800e-03, -2.52076100e-03, -1.84211100e-03, -5.65082800e-03,
            -2.33258410e-02, -4.63625000e-04, -2.18027000e-03, -1.44248100e-03, -2.37588200e-03,
            -2.80258900e-03, -2.58730600e-03, -1.48750200e-03, -1.98229800e-03, -1.09099300e-03,
            -1.49390900e-03, -9.38267000e-04, -4.79772000e-04, -2.52415000e-04, -1.03098500e-03,
            -3.42047200e-03, -9.20227000e-04, -2.92298300e-03, -5.64666000e-03, -2.04700000e-04
        ], [
            -1.54005000e-04, -1.06551000e-04, -1.48194000e-04, -6.25746200e-03, 7.96680666e-01,
            -5.51543000e-04, -6.68556000e-04, -7.02552000e-05, -2.30984000e-04, -1.28576200e-03,
            -4.00703000e-04, -2.46287000e-04, -2.96748000e-04, -2.56744000e-04, -1.35630200e-03,
            -3.83513200e-03, -1.33012000e-04, -1.41174000e-04, -4.26617000e-04, -4.26954000e-04,
            -3.36744000e-04, -1.44461000e-04, -1.95533000e-04, -1.52498000e-04, -1.28700000e-04,
            -1.76916000e-04, -3.18897000e-04, -4.09079000e-05, -1.21218000e-05, -1.09059000e-04,
            -6.17432000e-04, -1.21628000e-04, -1.36797000e-04, -4.37800000e-04, -7.87636000e-06
        ], [
            -2.01546800e-03, -1.64408500e-03, -1.61421200e-03, -1.31502700e-03, -1.83257400e-03,
            7.88789098e-01, -1.00445160e-02, -3.25046000e-04, -1.53489700e-03, -3.18442900e-03,
            -6.00982400e-03, -2.81546300e-03, -2.76125200e-03, -1.51130900e-03, -2.70877000e-03,
            -7.31575910e-02, -8.83134000e-04, -3.18520380e-02, -6.91880000e-04, -1.49484700e-03,
            -1.31732100e-03, -2.15807900e-03, -1.17419000e-03, -4.96645000e-04, -3.54870000e-04,
            -3.10675600e-03, -7.03157000e-04, -3.85638000e-04, -1.97522100e-03, -8.55742000e-04,
            -1.39089300e-03, -2.00403300e-03, -9.45602000e-04, -2.75261700e-03, -1.43899000e-04
        ], [
            -2.61751500e-03, -2.92066700e-03, -2.44581900e-02, -1.06941580e-02, -1.28317590e-02,
            -1.24863360e-02, 7.73680179e-01, -1.40137000e-03, -1.71774090e-02, -1.73233860e-02,
            -2.03783450e-02, -5.49369500e-03, -8.79714100e-03, -1.21127470e-02, -5.01477300e-03,
            -2.15236770e-02, -4.55286600e-03, -4.58042000e-03, -7.73480700e-03, -1.31960770e-02,
            -1.26730720e-02, -8.69461100e-03, -5.86427900e-03, -2.82630200e-03, -5.92366100e-03,
            -1.37673520e-02, -1.18846160e-02, -1.34144500e-02, -2.35596200e-03, -2.16598400e-02,
            -1.72528520e-02, -1.81515130e-02, -9.01752000e-03, -2.52117220e-02, -2.37953000e-04
        ], [
            -1.26693510e-02, -1.09872130e-02, -3.33911800e-03, -5.38871500e-03, -3.69662200e-03,
            -5.02482800e-03, -4.32724100e-03, 9.54259563e-01, -2.99469090e-02, -8.01612000e-03,
            -1.68267000e-02, -8.95807400e-03, -3.61831800e-03, -2.25294300e-03, -1.90652500e-03,
            -4.03291300e-03, -2.03581860e-02, -1.24884600e-02, -4.22654600e-03, -5.77608700e-03,
            -4.30413400e-03, -3.54138500e-03, -5.12058190e-02, -6.94792160e-02, -6.39622340e-02,
            -1.39503000e-02, -3.29768200e-03, -1.21098100e-03, -1.12205600e-03, -4.38167900e-03,
            -7.88705900e-03, -2.78795600e-03, -2.99129200e-03, -5.26366800e-03, -5.23389000e-05
        ], [
            -3.84596140e-02, -1.12035030e-02, -1.04116660e-02, -6.89154710e-02, -3.56247300e-02,
            -3.12958550e-02, -3.41006370e-02, -1.41785960e-02, 7.72548756e-01, -1.98434015e-01,
            -3.15653180e-02, -1.52389370e-02, -1.15371400e-02, -1.68711700e-02, -1.19634070e-02,
            -2.86167630e-02, -6.20398500e-03, -1.03020710e-02, -6.25810600e-03, -3.19812000e-03,
            -2.41604400e-03, -4.73601700e-03, -3.15597400e-03, -3.48366300e-03, -3.21878300e-03,
            -3.61362400e-03, -1.47994400e-03, -8.95550000e-04, -1.93523000e-03, -4.71859600e-03,
            -9.23861100e-03, -3.76522700e-03, -6.29819820e-02, -1.00246680e-02, -1.44636000e-04
        ], [
            -4.95674900e-03, -7.20987500e-03, -1.65147870e-02, -9.67522000e-03, -3.49416310e-02,
            -9.25078200e-03, -1.49068780e-02, -2.83508500e-03, -1.82750620e-02, 8.76839599e-01,
            -1.06189570e-02, -7.27505900e-03, -2.69600950e-02, -2.42407400e-02, -3.47315510e-02,
            -3.55307980e-02, -2.75674200e-03, -1.77799250e-02, -1.65138290e-02, -4.56269800e-03,
            -4.98763200e-03, -4.04638500e-03, -9.86792600e-03, -2.87594900e-03, -2.93588200e-03,
            -5.86808100e-03, -3.41271800e-03, -6.10190000e-04, -1.06058000e-03, -2.64711900e-03,
            -3.11271000e-03, -9.05112000e-04, -5.22599600e-03, -3.58833900e-03, -4.32137000e-05
        ], [
            -2.26899100e-03, -4.55456600e-03, -5.88184600e-03, -2.31883600e-03, -1.92232700e-03,
            -8.12975800e-03, -1.31594500e-03, -1.96863600e-03, -6.14599600e-03, -6.58013100e-03,
            8.84510384e-01, -1.02900570e-02, -5.77486900e-03, -1.11229900e-02, -6.99008300e-03,
            -6.56866300e-03, -2.09659500e-03, -7.58552090e-02, -2.90372900e-03, -1.38150700e-03,
            -1.05654400e-03, -3.53555100e-03, -1.27658300e-03, -6.39461000e-04, -5.55258000e-04,
            -1.38077600e-03, -1.18281200e-03, -1.87299000e-04, -2.02306600e-03, -1.01768000e-03,
            -1.34216900e-03, -1.72674700e-03, -2.07946900e-03, -2.01769200e-03, -4.08388000e-05
        ], [
            -4.56034800e-03, -2.29940760e-02, -1.37970710e-02, -5.74736400e-03, -1.03779350e-02,
            -2.52438310e-02, -8.70812300e-03, -6.96594600e-03, -1.46391470e-02, -2.50150510e-02,
            -3.17973430e-02, 6.87799304e-01, -1.59792443e-01, -6.27917490e-02, -9.45996530e-02,
            -9.29040610e-02, -7.78114400e-03, -8.70544280e-02, -1.79260840e-02, -4.20336200e-03,
            -3.71424600e-03, -4.30333500e-03, -6.63318400e-03, -6.49656600e-03, -5.37317900e-03,
            -6.38515200e-03, -3.95582800e-03, -6.68918000e-04, -2.52366400e-03, -3.53604300e-03,
            -4.59119800e-03, -1.77137800e-03, -2.35126100e-03, -4.52100700e-03, -2.05571000e-04
        ], [
            -6.85873800e-03, -1.55368500e-02, -3.94727000e-03, -7.89263200e-03, -5.24052800e-03,
            -8.05141700e-03, -6.45090600e-03, -5.83449000e-03, -6.56255300e-03, -1.08538240e-02,
            -1.61402860e-02, -1.60489950e-02, 8.78015708e-01, -1.41587860e-02, -3.01177020e-02,
            -9.68315900e-03, -8.08741300e-03, -1.69350040e-02, -2.16237690e-02, -3.95874300e-03,
            -2.67228400e-03, -2.10676700e-03, -5.58036500e-03, -8.44815500e-03, -4.31436000e-03,
            -4.67368400e-03, -2.89111800e-03, -1.00468600e-03, -1.41235200e-03, -2.98392000e-03,
            -5.89720500e-03, -2.15687800e-03, -2.39133700e-03, -3.43293700e-03, -8.52735000e-05
        ], [
            -1.86764500e-03, -6.57285000e-03, -3.50797800e-03, -6.14547000e-03, -5.03050700e-03,
            -6.56277100e-03, -9.74451000e-03, -4.49926200e-03, -9.36923800e-03, -1.38492620e-02,
            -1.02801520e-02, -1.65548660e-02, -8.40049250e-02, 7.05977916e-01, -5.35069970e-02,
            -2.32792330e-02, -1.60172060e-02, -3.09172380e-02, -2.57191130e-02, -9.31387900e-03,
            -7.64853400e-03, -4.39461300e-03, -6.60265900e-03, -4.25367100e-03, -4.38879500e-03,
            -7.72821900e-03, -3.97481790e-02, -3.81215500e-03, -1.86657000e-03, -1.52399650e-02,
            -1.36234500e-02, -6.28485600e-03, -2.09166500e-02, -9.78478400e-03, -5.38650000e-04
        ], [
            -4.27862500e-03, -6.14432200e-03, -2.19818300e-03, -2.77306800e-03, -2.40931000e-03,
            -3.31451600e-03, -2.29747600e-03, -1.05934500e-03, -2.18323200e-03, -4.07145800e-03,
            -3.57481700e-03, -5.08267600e-03, -1.31970380e-02, -4.91951300e-03, 7.05141366e-01,
            -4.71445000e-03, -1.94823600e-03, -4.23553900e-03, -8.56690220e-02, -5.76623300e-03,
            -4.05171600e-03, -1.32820100e-03, -3.34116810e-02, -4.31366620e-02, -5.10926030e-02,
            -1.16043360e-02, -3.55657200e-03, -1.12814400e-03, -1.00831600e-03, -4.00270900e-03,
            -1.58143350e-02, -1.51877500e-03, -1.32706800e-03, -4.74632000e-03, -1.83717000e-04
        ], [
            -7.17957000e-04, -1.25374600e-03, -1.14812200e-03, -5.27360000e-03, -2.98931800e-03,
            -4.74327100e-03, -3.74336400e-03, -6.24477000e-04, -1.78747000e-03, -2.62821000e-03,
            -2.99202300e-03, -9.12002000e-03, -3.89716900e-03, -2.77325500e-03, -4.83461500e-03,
            9.56186959e-01, -1.26427000e-03, -5.00971600e-03, -2.18431300e-03, -1.31782000e-03,
            -1.49433300e-03, -2.57313500e-03, -1.84878400e-03, -7.82346000e-04, -1.33153000e-03,
            -1.64254900e-03, -1.01866600e-03, -9.48025000e-04, -1.49656200e-03, -1.93558900e-03,
            -3.91536200e-03, -3.13644900e-03, -2.65624600e-03, -5.75861000e-03, -2.32936100e-03
        ], [
            -1.10001350e-02, -2.75328690e-02, -1.37019640e-02, -1.86617150e-02, -8.25285700e-03,
            -1.85412180e-02, -2.44185890e-02, -1.38835480e-02, -2.85957630e-02, -2.40926670e-02,
            -3.94953600e-02, -2.90213460e-02, -1.17755320e-02, -9.26941500e-03, -9.29544800e-03,
            -1.04182210e-02, 9.11737396e-01, -6.18143900e-03, -1.07320330e-02, -6.41279600e-03,
            -1.48734920e-02, -2.61210910e-02, -1.49801330e-02, -3.67285100e-03, -4.45950900e-03,
            -1.37610680e-02, -9.51172200e-03, -4.62705700e-03, -1.27984280e-02, -6.96837000e-03,
            -1.65305150e-02, -2.05894040e-02, -1.42915730e-02, -1.67838580e-02, -1.80232000e-04
        ], [
            -3.81848900e-03, -7.54710600e-03, -2.37735800e-03, -2.79864900e-03, -1.85221100e-03,
            -3.32717400e-03, -3.39003000e-03, -2.61082400e-03, -3.45513200e-03, -3.02749300e-03,
            -7.09915600e-03, -4.23668400e-03, -3.40652500e-03, -2.88747400e-03, -2.19851100e-03,
            -2.68343100e-03, -2.67057330e-02, 9.55885280e-01, -4.39413500e-03, -3.91960600e-03,
            -4.77619200e-03, -5.76295500e-03, -9.71777900e-03, -2.73701400e-03, -2.63035600e-03,
            -1.27720980e-02, -1.14197820e-02, -4.95003500e-03, -4.08412010e-02, -4.88876900e-03,
            -1.81918840e-02, -1.56651380e-02, -6.21945400e-03, -1.03344470e-02, -1.00998000e-03
        ], [
            -4.88606000e-03, -4.03578900e-03, -6.77100600e-03, -5.77750800e-03, -7.81704000e-03,
            -5.92220900e-03, -4.86675600e-03, -2.75554700e-03, -6.09152200e-03, -6.28469400e-03,
            -7.31421800e-03, -5.94667500e-03, -5.21299000e-03, -4.31344700e-03, -4.95322900e-03,
            -6.27677400e-03, -8.01667300e-03, -6.97247000e-03, 9.89932764e-01, -3.94470300e-03,
            -4.01524500e-03, -4.94030400e-03, -1.63466180e-02, -2.21696200e-03, -3.51830300e-03,
            -7.15110100e-03, -3.92361900e-03, -2.42646700e-03, -1.87230100e-03, -5.69824300e-03,
            -4.12950900e-03, -2.87227900e-03, -3.49264700e-03, -6.27955100e-03, -2.75203000e-05
        ], [
            -3.32177120e-02, -2.69209570e-02, -6.39654870e-02, -5.59415540e-02, -6.92499690e-02,
            -5.60246760e-02, -4.96310630e-02, -4.04723080e-02, -5.07287470e-02, -5.02207600e-02,
            -4.37715280e-02, -4.55129240e-02, -4.82482410e-02, -4.54487770e-02, -4.43369580e-02,
            -5.23717730e-02, -2.27319230e-02, -4.17049000e-02, -3.62740840e-02, 9.73368856e-01,
            -1.31368430e-02, -4.39126740e-02, -2.16276870e-02, -1.89039080e-02, -2.10031490e-02,
            -1.96186380e-02, -1.03694690e-02, -4.20001400e-03, -3.94358600e-03, -1.30351980e-02,
            -2.06731580e-02, -1.03648420e-02, -2.67946610e-02, -1.61540550e-02, -1.91677000e-04
        ], [
            -1.66711940e-02, -1.01881350e-02, -3.09114610e-02, -2.76630710e-02, -4.12455610e-02,
            -2.12294300e-02, -1.90032390e-02, -1.16532810e-02, -2.23478130e-02, -2.08325840e-02,
            -1.95248400e-02, -1.64501410e-02, -1.73071270e-02, -1.47537830e-02, -1.92041300e-02,
            -2.59156650e-02, -8.48860500e-03, -2.93120390e-02, -1.61648450e-02, -7.13047400e-03,
            9.92255281e-01, -2.57889040e-02, -1.38988240e-02, -8.05979100e-03, -9.71079400e-03,
            -9.09278400e-03, -8.64205000e-03, -4.17408100e-03, -2.90074900e-03, -9.22778700e-03,
            -8.38263400e-03, -5.81636800e-03, -1.16246050e-02, -1.14863890e-02, -3.40523000e-04
        ], [
            -1.29763400e-03, -3.27194200e-03, -3.67418600e-03, -4.23769700e-03, -4.01787200e-03,
            -4.70197900e-03, -5.96394300e-03, -1.22298900e-03, -4.42882500e-03, -5.63608900e-03,
            -5.38384500e-03, -4.55400000e-03, -6.07562100e-03, -5.19339800e-03, -2.85717900e-03,
            -4.17474800e-03, -9.26507100e-03, -4.43204800e-03, -5.87014800e-03, -8.46823600e-03,
            -5.78702300e-03, 9.90039037e-01, -7.93414900e-03, -4.00751100e-03, -2.35740980e-02,
            -1.78354990e-02, -8.94146800e-03, -9.27104300e-03, -2.48159200e-03, -1.35413870e-02,
            -1.04732480e-02, -6.31002600e-03, -8.00218300e-03, -1.17767890e-02, -7.13389000e-05
        ], [
            -1.28722620e-02, -2.17307780e-02, -2.29240550e-02, -1.95997510e-02, -2.06386180e-02,
            -2.39601350e-02, -1.89054820e-02, -3.13139190e-02, -1.81821960e-02, -1.74774600e-02,
            -3.33868710e-02, -1.74270550e-02, -1.25028470e-02, -8.77581000e-03, -1.18957790e-02,
            -1.86995110e-02, -2.35247570e-02, -1.59716310e-02, -1.41101450e-02, -1.90517550e-02,
            -1.15063970e-02, -9.41690700e-03, 9.54110944e-01, -4.41531990e-02, -1.43059550e-02,
            -5.92931360e-02, -9.04370100e-03, -5.31541000e-03, -1.89793000e-03, -5.90049800e-03,
            -1.09118210e-02, -1.02196890e-02, -7.14471800e-03, -9.39150800e-03, -3.33653000e-04
        ], [
            -3.59933700e-03, -5.84325200e-03, -4.82961600e-03, -4.72748500e-03, -6.01985300e-03,
            -8.64534700e-03, -3.96846600e-03, -6.69878600e-03, -3.63587700e-03, -3.89031900e-03,
            -9.20776300e-03, -6.67693900e-03, -3.21569100e-03, -2.42839600e-03, -2.17375500e-03,
            -2.85489400e-03, -3.07433400e-03, -4.44984300e-03, -1.14089500e-03, -2.96904100e-03,
            -1.52115900e-03, -1.85321000e-03, -3.19029800e-03, 8.94894468e-01, -2.97359600e-03,
            -6.34147800e-03, -8.08859000e-04, -4.86273000e-04, -4.20691000e-04, -1.01930700e-03,
            -1.44319500e-03, -1.27214000e-03, -5.77043000e-04, -1.33225600e-03, -2.24734000e-05
        ], [
            -6.49311000e-04, -1.90504300e-03, -1.32807300e-03, -1.78834400e-03, -2.73087900e-03,
            -1.67894300e-03, -2.32686000e-03, -6.08131000e-04, -1.75526700e-03, -1.94199200e-03,
            -1.61253500e-03, -1.40986600e-03, -2.19524700e-03, -2.07541500e-03, -1.68639300e-03,
            -1.48991900e-03, -1.40289700e-03, -1.07687600e-03, -1.68752700e-03, -3.14294400e-03,
            -1.41235500e-03, -1.08818100e-03, -2.09098100e-03, -5.27987600e-03, 9.64872796e-01,
            -1.90605530e-02, -3.65274100e-03, -2.93696000e-03, -5.36496000e-04, -4.14185100e-03,
            -3.88886100e-03, -1.94475500e-03, -1.17590400e-03, -4.04695000e-03, -1.02730000e-05
        ], [
            -3.58025000e-03, -5.51436900e-03, -6.15506900e-03, -4.03627000e-03, -4.96367500e-03,
            -6.74013200e-03, -9.24131400e-03, -6.74967000e-03, -5.09791800e-03, -5.24428100e-03,
            -8.31291900e-03, -5.16576500e-03, -4.67059000e-03, -3.57001800e-03, -4.53543700e-03,
            -4.72341200e-03, -4.45977500e-03, -4.13622000e-03, -9.93430700e-03, -2.08262100e-02,
            -1.01955080e-02, -7.71745700e-03, -4.04625940e-02, -1.21261142e-01, -9.94583210e-02,
            8.92817943e-01, -4.62218500e-03, -3.44468800e-03, -9.37338000e-04, -5.02281400e-03,
            -4.75573900e-03, -2.48385600e-03, -3.03393600e-03, -6.05519100e-03, -2.93923000e-05
        ], [
            -2.50119200e-03, -6.23880100e-03, -4.63544100e-03, -5.61954400e-03, -6.46338900e-03,
            -5.67001500e-03, -1.33390700e-02, -2.72545800e-03, -8.02430200e-03, -6.60875300e-03,
            -6.83647500e-03, -6.31959600e-03, -8.09418000e-03, -7.79598100e-03, -4.47118200e-03,
            -7.92283600e-03, -7.60521600e-03, -1.20443210e-02, -1.45837510e-02, -2.03241310e-02,
            -2.09896350e-02, -1.32118300e-02, -1.59666070e-02, -1.78111160e-02, -2.73335830e-02,
            -2.43262340e-02, 8.72970880e-01, -2.76716260e-02, -4.86639900e-03, -2.58135820e-02,
            -2.16489230e-02, -9.74362200e-03, -1.25118010e-02, -2.35658250e-02, -3.64199000e-04
        ], [
            -2.05071850e-02, -3.07397450e-02, -1.53483590e-02, -2.18814500e-02, -1.59174410e-02,
            -1.91202170e-02, -2.40896330e-02, -1.16541490e-02, -1.83099430e-02, -1.80806490e-02,
            -2.35854420e-02, -1.90864120e-02, -1.83991990e-02, -1.49696090e-02, -1.38433430e-02,
            -2.34712270e-02, -3.01791190e-02, -2.10326470e-02, -2.31242050e-02, -3.70681320e-02,
            -3.46131940e-02, -2.55376100e-02, -4.68516170e-02, -4.03074290e-02, -3.92817500e-02,
            -2.58651980e-02, -2.22635010e-02, 7.95524057e-01, -6.56959350e-02, -3.77589610e-02,
            -2.40384830e-02, -1.18374140e-02, -2.27499810e-02, -4.09235320e-02, -7.76897000e-03
        ], [
            -5.45538100e-03, -5.79498200e-03, -4.64506800e-03, -5.09572500e-03, -4.58488300e-03,
            -6.13884500e-03, -9.71471600e-03, -2.31251500e-03, -4.72505300e-03, -6.33386700e-03,
            -5.08407900e-03, -5.49077000e-03, -6.17418100e-03, -4.86759000e-03, -4.65797300e-03,
            -7.55412300e-03, -7.23313200e-03, -1.00531680e-02, -2.25848900e-02, -2.35862800e-02,
            -3.95055930e-02, -2.88926370e-02, -9.59541700e-03, -1.92397370e-02, -1.34689720e-02,
            -2.15728380e-02, -1.70401410e-02, -2.27065290e-02, 9.64696522e-01, -2.40730150e-02,
            -1.53831050e-02, -1.94870100e-02, -3.33008870e-02, -2.65695530e-02, -1.22766000e-04
        ], [
            -1.13855670e-02, -3.10265750e-02, -3.90024040e-02, -2.81160500e-02, -2.28872430e-02,
            -2.54362260e-02, -6.50601160e-02, -1.40347720e-02, -6.67617930e-02, -3.86819270e-02,
            -3.38942530e-02, -3.10111710e-02, -4.51315090e-02, -5.48746940e-02, -4.09814600e-02,
            -3.66000570e-02, -5.41757540e-02, -5.52422580e-02, -5.04444850e-02, -5.35487750e-02,
            -5.99308560e-02, -3.99313470e-02, -4.32000870e-02, -3.41894650e-02, -9.32848070e-02,
            -6.65297800e-02, -7.28774450e-02, -9.35951030e-02, -3.25157040e-02, 8.57281314e-01,
            -7.04048480e-02, -3.09039320e-02, -5.61496090e-02, -7.58942010e-02, -2.21978400e-03
        ], [
            -1.50294700e-03, -3.85129800e-03, -1.86344600e-03, -1.14776000e-03, -7.84077000e-04,
            -3.25702300e-03, -3.09404900e-03, -9.10514000e-04, -2.75079100e-03, -2.18927200e-03,
            -2.18574400e-03, -2.35493300e-03, -1.19229300e-03, -1.24847900e-03, -1.23506700e-03,
            -2.03901700e-03, -5.07916300e-03, -1.67287600e-03, -2.29986500e-03, -1.47912900e-03,
            -2.90918900e-03, -3.64717500e-03, -2.97314000e-03, -6.19151000e-03, -9.03999200e-03,
            -3.29247400e-03, -2.91448600e-03, -1.49054300e-03, -3.20762900e-03, -2.73398900e-03,
            9.92681121e-01, -4.22128700e-03, -4.51013600e-03, -8.06136700e-03, -5.96045000e-06
        ], [
            -5.87064000e-04, -4.70894000e-04, -4.73913000e-04, -4.56108000e-04, -4.80337000e-04,
            -3.86598000e-04, -7.72204000e-04, -4.00876000e-04, -9.10050000e-04, -6.70137000e-04,
            -6.20913000e-04, -6.42119000e-04, -8.54990000e-04, -8.86816000e-04, -7.15297000e-04,
            -4.77143000e-04, -1.09079900e-03, -5.09598000e-04, -8.00971000e-04, -7.78768000e-04,
            -1.46519300e-03, -5.66704000e-04, -1.14323800e-03, -4.39780000e-04, -1.88285800e-03,
            -1.17141200e-03, -1.61697800e-03, -1.27749600e-03, -2.96894000e-04, -2.28851800e-03,
            -4.38345400e-03, 9.89324308e-01, -1.13598500e-03, -1.58405000e-03, -4.00135000e-06
        ], [
            -1.49181400e-03, -4.15041000e-04, -5.08749000e-04, -4.22707000e-04, -1.02841900e-03,
            -4.10861000e-04, -4.36340000e-04, -1.89798000e-04, -5.18675000e-04, -4.12513000e-04,
            -4.21351000e-04, -3.81125000e-04, -5.47040000e-04, -4.05702000e-04, -3.61286000e-04,
            -4.93901000e-04, -3.42344000e-04, -4.61085000e-04, -5.78658000e-04, -6.16261000e-04,
            -4.67646000e-04, -1.11432500e-03, -7.12470000e-04, -3.60557000e-04, -4.62823000e-04,
            -6.43993000e-04, -6.98550000e-04, -5.39821000e-04, -9.42046000e-05, -7.70266000e-04,
            -5.25415700e-03, -1.52714500e-03, 9.75007213e-01, -1.22662200e-03, -9.29981000e-08
        ], [
            -2.54826700e-03, -6.74309000e-03, -5.72993300e-03, -5.37901300e-03, -6.54448700e-03,
            -5.39289600e-03, -1.37775460e-02, -2.79429600e-03, -7.76551900e-03, -5.74677200e-03,
            -5.73831500e-03, -6.32099500e-03, -5.83165700e-03, -4.86239100e-03, -4.34581900e-03,
            -6.13716000e-03, -7.11828400e-03, -6.16188300e-03, -8.62004400e-03, -8.13994300e-03,
            -8.67277200e-03, -1.31311250e-02, -9.18784300e-03, -5.28844800e-03, -8.49249200e-03,
            -1.06589290e-02, -1.97885410e-02, -9.88881800e-03, -7.26416100e-03, -2.70754690e-02,
            -1.55662050e-02, -1.03781090e-02, -1.42741350e-02, 9.20004935e-01, -1.68832400e-03
        ], [
            -1.73470000e-05, -2.28642000e-05, -2.65530000e-05, -5.61090000e-05, -2.62342000e-05,
            -3.41391000e-05, -4.13070000e-05, -1.33692000e-05, -3.36486000e-05, -4.08335000e-05,
            -2.24359000e-05, -2.30104000e-05, -2.61862000e-05, -1.94134000e-05, -4.09892000e-05,
            -2.43773000e-05, -1.94098000e-05, -7.32694000e-05, -6.68004000e-05, -1.56339000e-05,
            -4.19840000e-05, -5.36295000e-05, -8.86582000e-05, -1.30926000e-05, -2.73819000e-05,
            -2.13976000e-05, -5.13832000e-05, -5.63822000e-05, -2.61832000e-04, -6.71376000e-05,
            -7.32823000e-05, -4.11591000e-05, -8.69702000e-05, -1.24841000e-04, 9.99910943e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 388)
    return ret


@cache('run')
def ia_matrix_x2009x():
    """
    ia matrix x2009x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            8.79111299e-01, -5.81938000e-04, -2.66267579e-01, -5.17407410e-02, -5.75346530e-02,
            -1.36416322e-01, -1.81384890e-02, -2.40840900e-03, -7.66694700e-03, -1.28760320e-02,
            -5.57553000e-04, -1.85118000e-04, -1.82636000e-04, -8.44346000e-05, -1.27455000e-04,
            -1.16996830e-02, -5.94999000e-04, -3.50352200e-03, -8.75825000e-04, -2.44212800e-03,
            -2.27743600e-03, -4.15169330e-02, -1.67914900e-03, -8.42528000e-04, -4.34503000e-04,
            -7.51397300e-03, -1.93071000e-04, -1.74063000e-04, -3.90308000e-04, -8.68088000e-04,
            -1.74489500e-03, -2.79459100e-03, -2.24282700e-03, -2.67193200e-03, -2.66329330e-02
        ], [
            -2.51613400e-03, 9.04573142e-01, -3.34469600e-03, -4.22470900e-03, -3.16699300e-03,
            -5.46354600e-03, -5.89082300e-03, -3.88906730e-01, -3.21714390e-02, -4.58706500e-03,
            -7.60148210e-02, -6.01296650e-02, -3.95835600e-03, -1.21388300e-03, -1.64634300e-03,
            -6.24829900e-03, -1.06990704e-01, -2.32421370e-02, -5.21410000e-04, -5.70472000e-04,
            -1.22255300e-03, -2.77759700e-03, -6.37054800e-03, -4.65355500e-03, -4.64494500e-03,
            -4.57728500e-03, -4.93351000e-04, -2.85769000e-04, -1.88029600e-03, -8.86143000e-04,
            -3.31124000e-03, -1.95744900e-03, -1.96533100e-03, -3.13366700e-03, -6.01013240e-02
        ], [
            -6.55427480e-02, -8.30643000e-04, 8.35170015e-01, -5.41761400e-03, -1.03532923e-01,
            -3.22150100e-03, -3.57361700e-03, -2.03033300e-03, -1.27560780e-02, -4.10547400e-03,
            -3.05267300e-03, -2.32178400e-03, -2.10338000e-03, -1.58704900e-03, -1.39888000e-03,
            -3.11846600e-03, -1.16346000e-03, -1.76365200e-03, -1.63691500e-03, -4.90823900e-03,
            -4.76734100e-03, -1.50665483e-01, -1.94506700e-03, -5.95881600e-03, -4.86859700e-03,
            -1.82422100e-03, -1.24571100e-03, -7.24307000e-04, -2.91239000e-04, -1.94344400e-03,
            -6.21467300e-03, -7.73548900e-03, -1.08549320e-02, -7.22962000e-03, -3.45645100e-03
        ], [
            -1.64338700e-03, -9.62437000e-04, -1.61071700e-03, 6.07609988e-01, -6.63069200e-02,
            -3.39591100e-03, -6.73060500e-03, -6.95214000e-04, -3.89681100e-03, -1.62459940e-02,
            -4.85849900e-03, -2.63497900e-03, -2.79978300e-03, -2.04149700e-03, -4.15230100e-03,
            -2.57936470e-02, -1.15114600e-03, -2.45253100e-03, -1.50094500e-03, -2.71238900e-03,
            -2.71354200e-03, -2.74835600e-03, -2.19388900e-03, -3.38950400e-03, -2.73769900e-03,
            -2.11810500e-03, -5.97480000e-04, -4.65139000e-04, -4.24981000e-04, -1.60805300e-03,
            -2.49895300e-03, -1.10569800e-03, -3.18243400e-03, -4.23507000e-03, -7.63043000e-05
        ], [
            -1.55303000e-04, -5.53333000e-05, -1.20728000e-04, -1.40659000e-02, 7.62807900e-01,
            -5.58476000e-04, -7.05806000e-04, -4.68754000e-05, -2.76589000e-04, -1.17235700e-03,
            -2.57708000e-04, -1.29412000e-04, -1.89738000e-04, -3.20787000e-04, -2.48705700e-03,
            -5.96676400e-03, -7.19530000e-05, -1.05511000e-04, -3.46567000e-04, -2.65196000e-04,
            -2.14776000e-04, -1.08468000e-04, -2.47034000e-04, -9.28678000e-05, -1.42616000e-04,
            -1.92127000e-04, -1.56027000e-04, -3.18536000e-05, -9.49538000e-06, -9.33202000e-05,
            -6.22743000e-04, -1.10526000e-04, -2.53237000e-04, -3.87710000e-04, -1.91532000e-06
        ], [
            -1.67302000e-03, -1.47684100e-03, -1.59097400e-03, -1.50656600e-03, -1.85510200e-03,
            7.27318525e-01, -1.30107510e-02, -3.13161000e-04, -1.50716500e-03, -3.04616500e-03,
            -8.59104800e-03, -4.18761800e-03, -3.26176300e-03, -1.65819100e-03, -2.66532000e-03,
            -7.34634630e-02, -7.59106000e-04, -2.83240870e-02, -1.00891500e-03, -1.09799000e-03,
            -1.14533100e-03, -2.24042200e-03, -1.22612300e-03, -6.07341000e-04, -3.96911000e-04,
            -2.16409900e-03, -4.63643000e-04, -2.57006000e-04, -1.35252500e-03, -9.75165000e-04,
            -1.99209600e-03, -3.01801900e-03, -1.42238200e-03, -2.54386000e-03, -2.41936000e-05
        ], [
            -2.27117300e-03, -2.03386100e-03, -2.39558090e-02, -1.15218870e-02, -1.48468480e-02,
            -1.43623710e-02, 7.49837484e-01, -2.11067800e-03, -1.42837080e-02, -1.58855310e-02,
            -2.22364870e-02, -4.89785200e-03, -6.16219300e-03, -7.82757600e-03, -3.90022700e-03,
            -1.88658420e-02, -2.84549600e-03, -3.83846000e-03, -7.43120400e-03, -1.06930510e-02,
            -9.96806300e-03, -7.52369800e-03, -6.55735800e-03, -3.51225700e-03, -4.77294100e-03,
            -1.26461870e-02, -8.46496700e-03, -1.12922420e-02, -1.68125500e-03, -1.91970020e-02,
            -1.68560230e-02, -1.81970350e-02, -7.97777500e-03, -2.20971230e-02, -1.08888000e-04
        ], [
            -1.28168240e-02, -6.03793500e-03, -3.43034900e-03, -4.83813300e-03, -4.15504300e-03,
            -4.61938100e-03, -4.56976500e-03, 8.94893150e-01, -4.35223100e-02, -9.23279300e-03,
            -1.97420040e-02, -1.39986090e-02, -3.97258500e-03, -1.95027900e-03, -2.08992200e-03,
            -4.13326900e-03, -2.55139180e-02, -1.07665490e-02, -5.50504500e-03, -4.91267900e-03,
            -3.70126200e-03, -3.53221800e-03, -5.64917630e-02, -6.58363710e-02, -9.05608270e-02,
            -2.54683830e-02, -3.30425500e-03, -1.23368700e-03, -1.16140500e-03, -4.77358300e-03,
            -8.85163500e-03, -3.16672800e-03, -2.66022700e-03, -5.29016100e-03, -2.18733000e-05
        ], [
            -3.69032240e-02, -8.49416100e-03, -9.32029500e-03, -5.96670270e-02, -3.60238760e-02,
            -3.68877460e-02, -3.65390660e-02, -1.95196590e-02, 7.56217871e-01, -2.21976294e-01,
            -3.95741740e-02, -1.53138690e-02, -1.00667120e-02, -1.82628330e-02, -1.07602630e-02,
            -2.67799040e-02, -6.39452900e-03, -1.12575860e-02, -5.57864900e-03, -2.62786600e-03,
            -1.89042300e-03, -4.36733800e-03, -3.44550800e-03, -4.73847400e-03, -4.03426100e-03,
            -5.02617500e-03, -1.08699400e-03, -6.59480000e-04, -1.60431300e-03, -4.71666100e-03,
            -7.59195000e-03, -3.84470800e-03, -5.93854150e-02, -1.01398100e-02, -1.50674000e-04
        ], [
            -4.58678100e-03, -4.80101800e-03, -1.91091140e-02, -1.07984070e-02, -3.95390440e-02,
            -7.53076500e-03, -1.71823900e-02, -6.09965300e-03, -1.82419330e-02, 8.33874804e-01,
            -1.26951800e-02, -6.99966700e-03, -2.28035740e-02, -2.43349420e-02, -3.40330080e-02,
            -3.38042630e-02, -2.43792000e-03, -1.76019110e-02, -1.46064040e-02, -4.10964500e-03,
            -4.50364500e-03, -4.16521300e-03, -1.15121290e-02, -3.04469000e-03, -3.96933100e-03,
            -6.86859400e-03, -2.92761900e-03, -5.75032000e-04, -1.12737500e-03, -2.81794800e-03,
            -2.82514500e-03, -1.42175500e-03, -4.67849500e-03, -4.66770700e-03, -7.38282000e-05
        ], [
            -1.36718100e-03, -3.53291700e-03, -4.71949900e-03, -1.84396300e-03, -1.49634000e-03,
            -5.58662500e-03, -1.16712900e-03, -2.11925500e-03, -5.29309700e-03, -4.89149100e-03,
            8.60238172e-01, -1.05899330e-02, -4.86050400e-03, -9.47593100e-03, -5.91678000e-03,
            -7.39348300e-03, -1.52762100e-03, -9.01568190e-02, -3.08396100e-03, -8.45736000e-04,
            -8.58787000e-04, -2.93078700e-03, -1.36239100e-03, -3.89106000e-04, -6.22785000e-04,
            -1.34636500e-03, -8.65694000e-04, -1.01145000e-04, -1.40943800e-03, -8.81753000e-04,
            -1.28762800e-03, -1.58889000e-03, -1.65339300e-03, -1.66773100e-03, -2.97486000e-05
        ], [
            -4.19571000e-03, -1.65444800e-02, -1.02771290e-02, -4.85225300e-03, -9.18284400e-03,
            -2.01638650e-02, -9.46375200e-03, -7.05358500e-03, -1.27410750e-02, -2.30017310e-02,
            -3.65312680e-02, 6.60984060e-01, -1.62989368e-01, -6.75013770e-02, -8.94427770e-02,
            -7.38888410e-02, -6.63879600e-03, -9.68969950e-02, -1.36740360e-02, -2.85087600e-03,
            -2.63420800e-03, -4.17170100e-03, -6.38984700e-03, -4.34042400e-03, -3.87695400e-03,
            -9.42291300e-03, -2.83683000e-03, -4.88432000e-04, -2.18980600e-03, -3.72264000e-03,
            -4.09711300e-03, -1.88014000e-03, -2.08447000e-03, -4.50780700e-03, -1.51494000e-04
        ], [
            -6.98588700e-03, -1.44215770e-02, -3.86799200e-03, -9.94327700e-03, -7.72405700e-03,
            -1.03149990e-02, -7.57177500e-03, -7.93316000e-03, -8.59242700e-03, -1.37906000e-02,
            -2.33665940e-02, -2.66059830e-02, 8.32732887e-01, -1.72418300e-02, -4.28024940e-02,
            -1.66017880e-02, -7.85232600e-03, -1.88579250e-02, -2.11318060e-02, -2.43457700e-03,
            -2.19475600e-03, -2.26322600e-03, -5.42435000e-03, -9.46586400e-03, -7.46083100e-03,
            -8.26833000e-03, -2.45722600e-03, -7.77152000e-04, -1.24309200e-03, -2.85083800e-03,
            -6.22205900e-03, -2.19348800e-03, -4.39002900e-03, -3.65488100e-03, -3.66523000e-04
        ], [
            -2.07625400e-03, -8.60742300e-03, -4.15737300e-03, -6.98653400e-03, -5.35038600e-03,
            -7.64183300e-03, -1.07979260e-02, -5.64637700e-03, -1.14936370e-02, -1.34642560e-02,
            -1.28485530e-02, -1.89631350e-02, -1.12463223e-01, 5.89879915e-01, -7.35877620e-02,
            -3.99616270e-02, -3.12711090e-02, -4.49165050e-02, -3.26770750e-02, -8.90544400e-03,
            -8.03835300e-03, -6.08661400e-03, -9.66794300e-03, -4.07744500e-03, -6.68530500e-03,
            -1.18613670e-02, -4.35228740e-02, -4.23793200e-03, -2.51306000e-03, -2.28956110e-02,
            -2.09595580e-02, -1.38827050e-02, -2.76660560e-02, -1.78744360e-02, -1.08315000e-03
        ], [
            -4.05340600e-03, -4.22914300e-03, -1.84729300e-03, -3.06977300e-03, -2.56486600e-03,
            -4.30936600e-03, -4.54972600e-03, -1.65411600e-03, -2.60284900e-03, -5.19395700e-03,
            -7.00147400e-03, -8.37381500e-03, -1.71357230e-02, -4.23715900e-03, 6.84685520e-01,
            -9.83742600e-03, -3.80416200e-03, -5.23189800e-03, -6.89745270e-02, -4.89482700e-03,
            -3.59654000e-03, -2.11665700e-03, -3.93375860e-02, -4.17703770e-02, -5.82611500e-02,
            -1.46744020e-02, -3.34850400e-03, -1.02963100e-03, -8.96676000e-04, -4.36490100e-03,
            -1.85235810e-02, -2.73314300e-03, -1.41829800e-03, -8.44161200e-03, -2.73222000e-04
        ], [
            -6.20695000e-04, -7.19550000e-04, -9.67854000e-04, -5.28552700e-03, -3.25775400e-03,
            -3.53831400e-03, -4.42735400e-03, -9.41862000e-04, -1.78464600e-03, -2.83243000e-03,
            -3.68133500e-03, -8.37629900e-03, -3.94553200e-03, -2.14065100e-03, -5.56976700e-03,
            9.33388188e-01, -1.96587200e-03, -4.18932500e-03, -1.37765800e-03, -9.98719000e-04,
            -1.32995000e-03, -2.31005400e-03, -4.08182600e-03, -6.86082000e-04, -2.30538800e-03,
            -1.45762500e-03, -1.55272000e-03, -8.30639000e-04, -1.15312000e-03, -1.69773400e-03,
            -3.08481500e-03, -3.53920700e-03, -2.78759600e-03, -4.82409200e-03, -1.90047900e-03
        ], [
            -1.05365460e-02, -2.77201160e-02, -1.38997600e-02, -1.75131450e-02, -7.43927700e-03,
            -1.74728570e-02, -2.32116990e-02, -2.70168770e-02, -2.64568950e-02, -2.10335850e-02,
            -4.39813870e-02, -3.05186980e-02, -1.21476840e-02, -7.19245300e-03, -8.42048700e-03,
            -9.53943700e-03, 8.54311151e-01, -7.45152200e-03, -1.05220300e-02, -6.34622000e-03,
            -1.26056620e-02, -2.33916630e-02, -1.65708290e-02, -3.30745800e-03, -6.89995600e-03,
            -1.80855140e-02, -9.57258200e-03, -4.02866300e-03, -8.22084600e-03, -5.76708000e-03,
            -1.44541380e-02, -2.06776280e-02, -1.17129850e-02, -1.54647770e-02, -1.34556000e-04
        ], [
            -3.28440200e-03, -6.24311700e-03, -2.23811500e-03, -2.24517600e-03, -1.22265000e-03,
            -2.79440500e-03, -2.94276000e-03, -5.22038200e-03, -2.87940400e-03, -2.26669100e-03,
            -5.88610300e-03, -3.39951800e-03, -2.49367700e-03, -1.43435000e-03, -1.73084200e-03,
            -2.73064900e-03, -1.58872610e-02, 9.47280650e-01, -5.24660500e-03, -3.34156500e-03,
            -4.64702700e-03, -6.12686600e-03, -9.64309400e-03, -1.55895000e-03, -3.34374000e-03,
            -1.27154520e-02, -1.02980210e-02, -5.04762200e-03, -3.18315820e-02, -4.70972200e-03,
            -1.87728990e-02, -1.17074540e-02, -7.52941900e-03, -9.26242400e-03, -5.29321000e-04
        ], [
            -4.66129900e-03, -2.61181100e-03, -5.85396900e-03, -3.46753000e-03, -4.65669900e-03,
            -4.77856400e-03, -4.29751900e-03, -3.53580000e-03, -4.81048300e-03, -5.15613400e-03,
            -5.52004800e-03, -4.56950800e-03, -3.80416800e-03, -2.44749500e-03, -4.32916400e-03,
            -5.75845400e-03, -6.36979000e-03, -5.94068400e-03, 9.88925242e-01, -3.59116700e-03,
            -3.74598100e-03, -4.78142000e-03, -1.39667820e-02, -1.78956300e-03, -4.88959200e-03,
            -7.56228800e-03, -3.04177200e-03, -2.50854300e-03, -1.84116500e-03, -5.29122800e-03,
            -4.00368800e-03, -2.46301300e-03, -3.54485100e-03, -5.42187400e-03, -1.63446000e-05
        ], [
            -3.23857050e-02, -1.55600770e-02, -6.23766550e-02, -3.66053040e-02, -4.49165020e-02,
            -4.76161010e-02, -4.50508860e-02, -5.85177820e-02, -4.40436660e-02, -4.32831130e-02,
            -3.75317260e-02, -3.82553330e-02, -3.88128100e-02, -3.11484020e-02, -4.11798040e-02,
            -3.94432140e-02, -1.86337160e-02, -3.99017100e-02, -3.06192600e-02, 9.78205901e-01,
            -1.27384130e-02, -4.54220080e-02, -2.20832640e-02, -1.75961520e-02, -2.31688660e-02,
            -2.24591830e-02, -1.19242350e-02, -3.92363700e-03, -3.97955900e-03, -1.34446060e-02,
            -1.86434500e-02, -1.12099580e-02, -2.59575990e-02, -1.66074250e-02, -1.02972000e-04
        ], [
            -1.60275390e-02, -6.13409000e-03, -2.71781620e-02, -1.87237910e-02, -2.42385370e-02,
            -1.75640290e-02, -1.74393560e-02, -1.90871520e-02, -1.97342990e-02, -1.85570400e-02,
            -1.65973900e-02, -1.58016280e-02, -1.55706510e-02, -1.11883190e-02, -1.83844110e-02,
            -2.41301120e-02, -7.52811500e-03, -2.40358670e-02, -1.50754970e-02, -5.57427300e-03,
            9.93243009e-01, -2.32480400e-02, -1.42569530e-02, -6.64954400e-03, -1.01643470e-02,
            -9.78923700e-03, -7.86404000e-03, -4.41230600e-03, -2.72739500e-03, -8.07547700e-03,
            -8.01769100e-03, -5.72768900e-03, -1.23151840e-02, -1.06054910e-02, -1.27396000e-04
        ], [
            -1.46905200e-03, -3.10196300e-03, -3.89418600e-03, -3.73939200e-03, -4.31473800e-03,
            -3.65367500e-03, -4.64037300e-03, -1.10132300e-03, -3.75309300e-03, -4.22069500e-03,
            -5.35075300e-03, -3.99477500e-03, -4.71250800e-03, -3.29788300e-03, -2.49841200e-03,
            -3.32137000e-03, -2.88861000e-03, -5.05800000e-03, -4.58188000e-03, -7.21863800e-03,
            -5.39440600e-03, 9.90187525e-01, -7.59103200e-03, -3.92734900e-03, -1.55711680e-02,
            -2.26994210e-02, -6.87130900e-03, -1.00440710e-02, -1.92382000e-03, -1.23627540e-02,
            -1.07017790e-02, -9.31950000e-03, -7.78185800e-03, -1.11921810e-02, -3.80526000e-05
        ], [
            -1.22425530e-02, -1.63742290e-02, -2.25842900e-02, -1.77555320e-02, -1.74345120e-02,
            -2.10347050e-02, -1.78669670e-02, -4.36518220e-02, -1.75652330e-02, -1.60174480e-02,
            -3.24188900e-02, -1.67094480e-02, -1.23227480e-02, -7.27922200e-03, -1.08192750e-02,
            -1.85736510e-02, -1.83121040e-02, -1.97509160e-02, -1.59982320e-02, -2.14622930e-02,
            -1.23390420e-02, -9.12723700e-03, 9.53779788e-01, -7.57586900e-03, -9.47721600e-03,
            -5.64065970e-02, -8.59781900e-03, -5.13812900e-03, -1.61223900e-03, -5.79363800e-03,
            -9.88756000e-03, -9.27582500e-03, -6.72513500e-03, -9.09408800e-03, -2.79940000e-04
        ], [
            -2.19343800e-03, -6.06963300e-03, -4.98145200e-03, -4.50629700e-03, -4.04194700e-03,
            -6.98648400e-03, -3.43438200e-03, -3.41747200e-03, -3.41006900e-03, -3.50793400e-03,
            -9.12631100e-03, -6.18793300e-03, -3.53299300e-03, -1.98522800e-03, -2.72418700e-03,
            -2.69637700e-03, -2.70748400e-03, -3.01606600e-03, -1.64985600e-03, -6.28428500e-03,
            -2.92250400e-03, -1.50713500e-03, -2.93385000e-03, 8.43272123e-01, -2.91911500e-03,
            -5.41202800e-03, -9.09763000e-04, -5.24271000e-04, -1.73818100e-03, -1.17433600e-03,
            -1.09620400e-03, -7.66948000e-04, -5.05851000e-04, -1.13478000e-03, -1.26804000e-05
        ], [
            -7.44320000e-04, -1.30123200e-03, -1.33656800e-03, -1.44557900e-03, -2.07666700e-03,
            -1.58179200e-03, -1.90543100e-03, -7.20588000e-04, -1.50540000e-03, -1.41512500e-03,
            -1.49759300e-03, -9.86284000e-04, -1.82330600e-03, -1.40744800e-03, -1.22694900e-03,
            -1.23660900e-03, -7.21091000e-04, -1.11240500e-03, -1.95580500e-03, -2.89346400e-03,
            -1.58430900e-03, -1.40325800e-03, -2.41006300e-03, -4.29143400e-03, 9.68672815e-01,
            -1.25060330e-02, -3.63311100e-03, -2.91591200e-03, -4.21928000e-04, -3.64805500e-03,
            -3.63135100e-03, -3.55403400e-03, -1.12208900e-03, -4.23538700e-03, -7.30692000e-06
        ], [
            -3.77251900e-03, -4.42941100e-03, -6.62126300e-03, -3.65330100e-03, -3.82203000e-03,
            -6.58817900e-03, -8.68325400e-03, -9.48871200e-03, -4.67652600e-03, -5.13582800e-03,
            -7.81232900e-03, -4.77892900e-03, -3.89350500e-03, -2.60333800e-03, -3.80070700e-03,
            -4.04554100e-03, -3.82904000e-03, -1.35012680e-02, -1.38782330e-02, -1.96337010e-02,
            -1.28967490e-02, -7.59355300e-03, -3.71616570e-02, -1.13707308e-01, -1.02474447e-01,
            9.02400846e-01, -4.00896200e-03, -3.46726700e-03, -1.13774600e-03, -5.17047400e-03,
            -4.89850500e-03, -2.83128300e-03, -3.37543600e-03, -6.29510000e-03, -9.73793000e-06
        ], [
            -4.46396700e-03, -4.24023900e-03, -5.74444200e-03, -6.86245500e-03, -6.10121500e-03,
            -6.53625600e-03, -1.20705030e-02, -4.44449900e-03, -7.85652900e-03, -5.40303600e-03,
            -6.89462900e-03, -7.78954000e-03, -6.09869700e-03, -7.86238500e-03, -4.76072600e-03,
            -1.63942320e-02, -7.78767200e-03, -1.26373450e-02, -1.72457950e-02, -1.83154660e-02,
            -2.21913580e-02, -1.64403010e-02, -2.13281350e-02, -2.00813450e-02, -2.39053760e-02,
            -2.44674500e-02, 8.80024485e-01, -2.97912550e-02, -4.70960100e-03, -2.57996570e-02,
            -2.39631410e-02, -1.24106480e-02, -1.44290800e-02, -2.24202020e-02, -7.36682000e-04
        ], [
            -1.88611970e-02, -1.98655060e-02, -1.70984380e-02, -2.24535660e-02, -1.72398110e-02,
            -1.65729400e-02, -2.28175050e-02, -1.50299700e-02, -1.87291970e-02, -1.55032240e-02,
            -2.48542710e-02, -1.80330140e-02, -1.62574680e-02, -1.32142170e-02, -1.48606920e-02,
            -3.06049330e-02, -2.46319390e-02, -2.14462180e-02, -2.14588490e-02, -2.97265790e-02,
            -3.45836790e-02, -2.81645970e-02, -5.02575490e-02, -2.28621060e-02, -3.31565310e-02,
            -3.63854420e-02, -1.93604010e-02, 7.89311681e-01, -8.17110460e-02, -3.88063420e-02,
            -2.46211580e-02, -1.70929740e-02, -2.51509960e-02, -4.45760530e-02, -1.02504300e-02
        ], [
            -2.64326900e-03, -5.28204700e-03, -4.39485300e-03, -4.48442000e-03, -5.66431700e-03,
            -5.35403900e-03, -8.88047900e-03, -2.05756100e-03, -3.74304500e-03, -5.05364200e-03,
            -4.42512600e-03, -4.06656700e-03, -4.57257200e-03, -3.20101600e-03, -4.36697300e-03,
            -6.46255300e-03, -4.46859800e-03, -1.03596870e-02, -2.16859750e-02, -1.88119960e-02,
            -4.15489490e-02, -2.86018150e-02, -8.65713700e-03, -1.17268850e-02, -8.68494900e-03,
            -2.15092660e-02, -1.29188900e-02, -2.18554380e-02, 9.67253836e-01, -2.18768480e-02,
            -1.41371560e-02, -2.11780320e-02, -3.34770060e-02, -2.59005200e-02, -1.06513000e-06
        ], [
            -1.35131200e-02, -2.22303470e-02, -3.90755840e-02, -1.95476250e-02, -1.73815640e-02,
            -2.39682570e-02, -6.18890450e-02, -2.00300730e-02, -6.33456250e-02, -3.46157640e-02,
            -3.08848690e-02, -2.67847900e-02, -3.64468520e-02, -3.79899920e-02, -3.93383190e-02,
            -2.76025310e-02, -3.36054610e-02, -5.45094170e-02, -5.47710510e-02, -5.79910000e-02,
            -6.26181740e-02, -5.08647570e-02, -4.46320290e-02, -2.47786810e-02, -7.03045990e-02,
            -6.60507420e-02, -5.85068040e-02, -1.05897677e-01, -2.90868240e-02, 8.37983117e-01,
            -9.08507990e-02, -3.86063160e-02, -6.96876670e-02, -8.62371340e-02, -2.25064200e-03
        ], [
            -1.35696800e-03, -1.83068700e-03, -1.95939600e-03, -6.39838000e-04, -5.41984000e-04,
            -2.74102200e-03, -3.81121600e-03, -1.13210700e-03, -2.62397700e-03, -1.94037600e-03,
            -2.47491200e-03, -3.16871900e-03, -1.18331000e-03, -8.22726000e-04, -1.07783200e-03,
            -1.98748100e-03, -4.34659300e-03, -1.43807700e-03, -2.61513000e-03, -1.44068300e-03,
            -2.83853400e-03, -3.86929900e-03, -3.66046600e-03, -5.01583900e-03, -1.04688460e-02,
            -3.54183300e-03, -2.54622700e-03, -1.62676700e-03, -3.28213100e-03, -2.94667100e-03,
            9.92273126e-01, -4.64874500e-03, -4.56606700e-03, -8.72816000e-03, -4.61683000e-06
        ], [
            -5.47815000e-04, -4.05660000e-04, -3.87450000e-04, -2.81477000e-04, -3.21371000e-04,
            -3.10157000e-04, -5.16227000e-04, -4.36186000e-04, -7.87919000e-04, -4.98952000e-04,
            -4.69940000e-04, -4.46195000e-04, -7.09430000e-04, -4.95287000e-04, -6.22777000e-04,
            -3.53625000e-04, -5.98958000e-04, -4.73916000e-04, -7.78217000e-04, -7.37070000e-04,
            -1.24499100e-03, -5.70168000e-04, -9.18027000e-04, -2.81241000e-04, -2.10271800e-03,
            -1.07889700e-03, -1.18085700e-03, -1.19354100e-03, -2.71294000e-04, -2.19322100e-03,
            -4.29547100e-03, 9.83911679e-01, -1.29849600e-03, -1.59338600e-03, -9.54745000e-06
        ], [
            -1.26649700e-03, -4.92357000e-04, -4.68988000e-04, -3.87123000e-04, -9.96002000e-04,
            -5.90477000e-04, -5.43451000e-04, -2.20118000e-04, -5.36451000e-04, -3.80023000e-04,
            -9.16234000e-04, -5.86740000e-04, -1.09116600e-03, -4.46769000e-04, -5.64677000e-04,
            -4.81039000e-04, -4.27029000e-04, -3.98325000e-04, -5.67782000e-04, -4.69579000e-04,
            -4.38077000e-04, -8.92320000e-04, -6.01365000e-04, -3.28890000e-04, -4.50356000e-04,
            -4.55659000e-04, -5.17206000e-04, -5.49961000e-04, -8.36447000e-05, -6.17285000e-04,
            -5.88392900e-03, -1.70381300e-03, 9.66582568e-01, -1.47311000e-03, -4.13318000e-07
        ], [
            -3.03690300e-03, -5.45380100e-03, -5.18260800e-03, -4.21411300e-03, -5.71260300e-03,
            -4.63599500e-03, -1.20957070e-02, -2.29243400e-03, -6.62170800e-03, -4.55486400e-03,
            -5.70093000e-03, -5.02214900e-03, -4.04490200e-03, -2.74930300e-03, -4.23297200e-03,
            -4.32305100e-03, -5.61415000e-03, -5.70500400e-03, -6.91681400e-03, -7.65658100e-03,
            -8.82503200e-03, -1.44884510e-02, -1.40305920e-02, -4.77944700e-03, -7.67473200e-03,
            -1.54716820e-02, -1.55241870e-02, -1.03568370e-02, -6.91027100e-03, -2.60610110e-02,
            -1.74308040e-02, -1.17896670e-02, -1.37198490e-02, 9.23194110e-01, -1.26307400e-03
        ], [
            -1.42671000e-05, -2.40286000e-05, -2.47281000e-05, -1.00001000e-04, -3.23713000e-05,
            -3.04051000e-05, -2.71023000e-05, -1.27161000e-05, -1.66471000e-05, -1.86450000e-05,
            -1.94115000e-05, -1.32096000e-05, -3.37777000e-05, -1.22646000e-05, -6.18991000e-05,
            -1.52228000e-05, -7.79118000e-06, -4.49630000e-05, -2.54850000e-05, -8.75894000e-06,
            -3.42579000e-05, -5.08565000e-05, -1.34128000e-04, -5.77412000e-06, -1.45253000e-05,
            -1.28275000e-05, -2.95250000e-05, -5.80156000e-05, -1.61812000e-04, -5.93361000e-05,
            -6.40354000e-05, -4.01316000e-05, -5.07878000e-05, -9.44490000e-05, 9.99909811e-01
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 389)
    return ret


@cache('step')
def leontief_matrix_for_python():
    """
    leontief matrix for python



    component


    """
    
    ret = functions.if_then_else(
        time() < 1996,
        leontief_matrix_x1995x(),
        functions.if_then_else(
            time() < 1997,
            leontief_matrix_x1996x(),
            functions.if_then_else(
                time() < 1998,
                leontief_matrix_x1997x(),
                functions.if_then_else(
                    time() < 1999,
                    leontief_matrix_x1998x(),
                    functions.if_then_else(
                        time() < 2000,
                        leontief_matrix_x1999x(),
                        functions.if_then_else(
                            time() < 2001,
                            leontief_matrix_x2000x(),
                            functions.if_then_else(
                                time() < 2002,
                                leontief_matrix_x2001x(),
                                functions.if_then_else(
                                    time() < 2003,
                                    leontief_matrix_x2002x(),
                                    functions.if_then_else(
                                        time() < 2004,
                                        leontief_matrix_x2003x(),
                                        functions.if_then_else(
                                            time() < 2005,
                                            leontief_matrix_x2004x(),
                                            functions.if_then_else(
                                                time() < 2006,
                                                leontief_matrix_x2005x(),
                                                functions.if_then_else(
                                                    time() < 2007,
                                                    leontief_matrix_x2006x(),
                                                    functions.if_then_else(
                                                        time() < 2008,
                                                        leontief_matrix_x2007x(),
                                                        functions.if_then_else(
                                                            time() < 2009,
                                                            leontief_matrix_x2008x(),
                                                            leontief_matrix_x2009x()))))))))))))))
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 390)
    return ret


@cache('run')
def leontief_matrix_x1996x():
    """
    leontief matrix x1996x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17575305e+00, 8.32659202e-03, 3.77221238e-01, 7.52160716e-02, 9.28578382e-02,
            2.06052968e-01, 2.85525958e-02, 9.94415558e-03, 2.21734806e-02, 2.92395654e-02,
            1.04966512e-02, 7.78537906e-03, 8.36514893e-03, 8.44549620e-03, 9.24354804e-03,
            3.89121816e-02, 5.30670320e-03, 1.77448005e-02, 6.40871057e-03, 9.07401780e-03,
            8.59456056e-03, 1.09667171e-01, 6.56618636e-03, 1.14147746e-02, 9.27584662e-03,
            9.95894183e-03, 4.84832243e-03, 4.51526682e-03, 3.37078235e-03, 7.28950337e-03,
            1.00822916e-02, 8.90046873e-03, 1.36371101e-02, 1.10712001e-02, 1.92964894e-02
        ], [
            2.36521725e-02, 1.13063555e+00, 2.61051069e-02, 2.77620299e-02, 2.32798820e-02,
            2.96108526e-02, 2.80583313e-02, 5.25149073e-01, 8.35126525e-02, 4.20722582e-02,
            1.28658091e-01, 9.86101641e-02, 3.71188365e-02, 2.97818406e-02, 3.28015029e-02,
            3.82664186e-02, 1.42457997e-01, 5.88467486e-02, 1.68244647e-02, 1.34081842e-02,
            1.15912435e-02, 1.94389090e-02, 4.65253373e-02, 4.63119380e-02, 4.45391050e-02,
            2.61817527e-02, 1.01747769e-02, 6.99250284e-03, 9.35709613e-03, 1.13304048e-02,
            1.75049791e-02, 1.35243252e-02, 1.64673177e-02, 1.82778877e-02, 7.35197697e-02
        ], [
            7.61004883e-02, 5.16755677e-03, 1.18984480e+00, 1.57263805e-02, 1.31615973e-01,
            2.01041379e-02, 1.18570724e-02, 6.07340918e-03, 2.14208310e-02, 1.37153385e-02,
            8.75045786e-03, 7.54115905e-03, 8.21245721e-03, 8.61502385e-03, 8.01807904e-03,
            1.09492980e-02, 4.95333459e-03, 7.74952879e-03, 6.99787695e-03, 1.04290314e-02,
            9.16160298e-03, 1.99304453e-01, 7.06144157e-03, 1.53629727e-02, 1.42904222e-02,
            1.14328528e-02, 6.19036350e-03, 5.76495921e-03, 2.30948326e-03, 8.68766190e-03,
            1.46174288e-02, 1.40197480e-02, 2.28849543e-02, 1.25472888e-02, 3.16076006e-03
        ], [
            6.48889338e-03, 5.14168534e-03, 7.70022571e-03, 1.40473261e+00, 9.94155158e-02,
            1.50563888e-02, 1.74722472e-02, 4.61735333e-03, 1.19629941e-02, 3.67305962e-02,
            1.24272641e-02, 7.58274965e-03, 9.53959703e-03, 9.46119075e-03, 1.85501817e-02,
            4.28521321e-02, 3.43704738e-03, 9.29826010e-03, 7.17011922e-03, 6.65185465e-03,
            7.31987016e-03, 8.23200872e-03, 5.80551649e-03, 8.73618769e-03, 5.60471726e-03,
            6.02371500e-03, 6.53023428e-03, 2.93801147e-03, 1.67790720e-03, 4.55977326e-03,
            7.48352708e-03, 2.81733373e-03, 8.01186040e-03, 1.03741925e-02, 1.40293576e-03
        ], [
            6.94736498e-04, 8.21390334e-04, 9.40973359e-04, 1.20558078e-02, 1.23425639e+00,
            2.38758119e-03, 2.74593756e-03, 7.13339394e-04, 1.08632409e-03, 2.33622305e-03,
            1.54227127e-03, 1.35772122e-03, 1.40866722e-03, 1.35145805e-03, 3.00140272e-03,
            5.81959430e-03, 6.47916232e-04, 9.73964566e-04, 1.17271712e-03, 9.77999235e-04,
            9.86566318e-04, 7.96940512e-04, 7.32499773e-04, 8.62805237e-04, 7.24331139e-04,
            6.86940437e-04, 1.21886908e-03, 3.76052122e-04, 1.94022907e-04, 5.72028830e-04,
            1.51618198e-03, 4.23087077e-04, 6.10627631e-04, 1.42532892e-03, 1.21130076e-04
        ], [
            4.36684012e-03, 5.26038452e-03, 6.10351413e-03, 6.44536438e-03, 7.55708364e-03,
            1.23525967e+00, 2.09893385e-02, 4.24654743e-03, 6.20586228e-03, 7.91826524e-03,
            1.32468443e-02, 8.67674671e-03, 8.72228659e-03, 7.25405572e-03, 1.02833140e-02,
            9.72816793e-02, 4.58714989e-03, 4.98055556e-02, 4.39541103e-03, 4.46318632e-03,
            4.09305084e-03, 6.21865591e-03, 4.75141640e-03, 4.77650314e-03, 4.00492206e-03,
            9.34749179e-03, 3.30780119e-03, 2.95585277e-03, 5.92554577e-03, 3.79128229e-03,
            5.04455486e-03, 3.75988948e-03, 3.84273015e-03, 7.22612785e-03, 1.33154546e-03
        ], [
            1.58106752e-02, 1.56386981e-02, 5.25627664e-02, 3.56904447e-02, 4.28486957e-02,
            3.62645506e-02, 1.32355451e+00, 1.57245350e-02, 4.94643252e-02, 4.93343376e-02,
            4.90305679e-02, 2.66605805e-02, 3.43410674e-02, 4.48000908e-02, 3.14405949e-02,
            5.00881674e-02, 1.52979233e-02, 2.63167433e-02, 2.44602092e-02, 3.05714148e-02,
            2.91645704e-02, 3.24582836e-02, 2.06767636e-02, 2.38297764e-02, 2.68900370e-02,
            3.66869667e-02, 2.59195130e-02, 3.41080835e-02, 9.85195647e-03, 4.25783449e-02,
            3.22451156e-02, 2.77227363e-02, 2.58276287e-02, 4.62473152e-02, 2.79947967e-03
        ], [
            2.20112315e-02, 2.37864261e-02, 1.87000655e-02, 1.97987169e-02, 1.74799691e-02,
            1.95259393e-02, 1.64164451e-02, 1.06390305e+00, 4.62912440e-02, 2.77426619e-02,
            3.17420729e-02, 2.45029561e-02, 1.62999064e-02, 1.50298078e-02, 1.53682653e-02,
            1.79416909e-02, 3.08681266e-02, 2.57439285e-02, 1.20883869e-02, 1.26236969e-02,
            1.04394737e-02, 1.30133064e-02, 6.25847506e-02, 7.04675053e-02, 6.97117249e-02,
            2.66093798e-02, 8.71281888e-03, 5.48774224e-03, 4.34004191e-03, 9.98800863e-03,
            1.39125151e-02, 7.26636670e-03, 1.10109037e-02, 1.32823770e-02, 2.27134904e-03
        ], [
            7.12781541e-02, 3.23406089e-02, 5.29518282e-02, 1.47961331e-01, 9.41342329e-02,
            6.99498946e-02, 7.63147022e-02, 3.92468787e-02, 1.32227052e+00, 3.10962492e-01,
            6.47097093e-02, 4.68069720e-02, 4.89483695e-02, 5.97174739e-02, 5.96307481e-02,
            8.14768013e-02, 2.01570011e-02, 4.00833985e-02, 2.92029646e-02, 1.42062516e-02,
            1.25081769e-02, 2.54264940e-02, 1.93312974e-02, 1.95381991e-02, 1.70697430e-02,
            1.81522545e-02, 1.13638138e-02, 7.91269977e-03, 7.09862413e-03, 1.55582564e-02,
            2.21792162e-02, 1.14451920e-02, 9.26443836e-02, 2.44391873e-02, 4.16390548e-03
        ], [
            1.24370112e-02, 1.72885491e-02, 2.88376537e-02, 2.32712215e-02, 5.85116954e-02,
            2.32369162e-02, 2.90517226e-02, 1.35038289e-02, 3.48988070e-02, 1.13733355e+00,
            2.36709474e-02, 2.20570335e-02, 4.69008346e-02, 4.91354667e-02, 6.92555293e-02,
            5.68846850e-02, 9.96793365e-03, 3.19787271e-02, 3.34088460e-02, 1.04169761e-02,
            1.03907042e-02, 1.40437829e-02, 2.11777072e-02, 1.29798210e-02, 1.21663586e-02,
            1.48988717e-02, 9.70643574e-03, 4.59649664e-03, 4.22187736e-03, 8.12650745e-03,
            1.00829610e-02, 4.68868171e-03, 1.29949896e-02, 1.01527154e-02, 1.90242475e-03
        ], [
            5.66489001e-03, 1.01944711e-02, 1.16504460e-02, 7.61343708e-03, 7.66038977e-03,
            1.78742761e-02, 6.42428628e-03, 8.76287956e-03, 1.42131462e-02, 1.42028848e-02,
            1.12397875e+00, 2.03391057e-02, 1.55640227e-02, 2.36629051e-02, 1.88925922e-02,
            1.54778577e-02, 8.14224143e-03, 9.83782688e-02, 8.40658917e-03, 4.47303787e-03,
            3.80525159e-03, 8.63204937e-03, 5.93372845e-03, 5.35217871e-03, 4.78226773e-03,
            6.35536158e-03, 4.75568123e-03, 2.60273163e-03, 7.81196334e-03, 4.00500062e-03,
            6.24141358e-03, 4.76697244e-03, 6.06659398e-03, 6.17852108e-03, 1.23875647e-03
        ], [
            2.42209236e-02, 6.24446239e-02, 4.69192583e-02, 3.49102103e-02, 4.48204250e-02,
            7.33819694e-02, 3.90320856e-02, 4.65737462e-02, 5.77711731e-02, 6.97492831e-02,
            8.07715425e-02, 1.49101445e+00, 3.02119657e-01, 1.75703119e-01, 2.49247407e-01,
            1.95602451e-01, 3.57780813e-02, 1.61908747e-01, 7.23544773e-02, 2.08935660e-02,
            1.80553163e-02, 2.62120682e-02, 3.63232670e-02, 4.64822551e-02, 3.91697989e-02,
            3.12808013e-02, 2.15065616e-02, 1.05574991e-02, 1.56769196e-02, 1.86418877e-02,
            2.77530588e-02, 1.32859073e-02, 2.05248552e-02, 2.34278778e-02, 6.48466094e-03
        ], [
            1.41212739e-02, 3.00014601e-02, 1.47403431e-02, 1.78879937e-02, 1.62327147e-02,
            2.13132188e-02, 1.74493173e-02, 2.19236372e-02, 2.00637127e-02, 2.39520001e-02,
            2.60502100e-02, 3.53941274e-02, 1.14899249e+00, 3.28270605e-02, 5.88493491e-02,
            2.33335432e-02, 1.72270903e-02, 3.03448137e-02, 3.74234317e-02, 8.80392497e-03,
            7.51583475e-03, 9.26351098e-03, 1.53869223e-02, 1.61728769e-02, 1.35260530e-02,
            1.22936044e-02, 7.89019675e-03, 4.64623928e-03, 4.61592126e-03, 7.92242170e-03,
            1.28628085e-02, 5.58634185e-03, 7.65238769e-03, 9.44516541e-03, 2.66297287e-03
        ], [
            1.11752146e-02, 2.22783784e-02, 1.82636799e-02, 2.24849204e-02, 2.12028429e-02,
            2.47702519e-02, 2.93456408e-02, 1.99193282e-02, 3.01209812e-02, 3.57654753e-02,
            2.88331004e-02, 4.59420621e-02, 1.32173316e-01, 1.35874304e+00, 1.15151430e-01,
            4.75502215e-02, 2.91951991e-02, 5.36580034e-02, 5.43995761e-02, 2.10649668e-02,
            1.86689123e-02, 1.64398460e-02, 2.25252672e-02, 2.24880198e-02, 2.21854770e-02,
            2.24664342e-02, 4.78720575e-02, 1.30968261e-02, 8.00896404e-03, 2.67090711e-02,
            2.70254382e-02, 1.33677419e-02, 3.37985480e-02, 2.13810943e-02, 3.06327350e-03
        ], [
            1.16781719e-02, 1.58578697e-02, 1.32095137e-02, 1.36658592e-02, 1.44100638e-02,
            1.51760344e-02, 1.13526237e-02, 1.35196308e-02, 1.29941073e-02, 1.49634855e-02,
            1.59612433e-02, 1.91466014e-02, 3.19632747e-02, 1.80526936e-02, 1.42240037e+00,
            1.67772695e-02, 1.01733620e-02, 1.56384578e-02, 1.33381693e-01, 1.31675101e-02,
            1.07225191e-02, 8.98112878e-03, 5.07009596e-02, 7.24831266e-02, 7.36943201e-02,
            2.50959630e-02, 9.27803369e-03, 5.19425816e-03, 3.18086439e-03, 9.60918107e-03,
            3.07844255e-02, 4.89134273e-03, 6.93402074e-03, 1.07322628e-02, 1.63966011e-03
        ], [
            2.49487283e-03, 3.49741895e-03, 3.84971662e-03, 1.03106098e-02, 7.62020548e-03,
            1.06092419e-02, 6.55441783e-03, 3.09923388e-03, 4.96818679e-03, 6.00274424e-03,
            6.69680342e-03, 1.29054124e-02, 9.07613762e-03, 7.76562911e-03, 1.17732848e-02,
            1.04136703e+00, 2.98630109e-03, 9.01295699e-03, 5.15948675e-03, 3.16673017e-03,
            3.11402638e-03, 4.99781452e-03, 3.53803508e-03, 3.78085685e-03, 3.71954763e-03,
            3.66290448e-03, 2.53002130e-03, 2.70497738e-03, 2.17417007e-03, 4.13447177e-03,
            5.86470143e-03, 4.79757401e-03, 4.72995350e-03, 8.36560746e-03, 4.12173269e-03
        ], [
            2.17211904e-02, 4.05202320e-02, 3.16730674e-02, 4.11586572e-02, 2.94931527e-02,
            3.77164742e-02, 4.43816069e-02, 4.03332449e-02, 5.68357241e-02, 5.01096678e-02,
            6.27474563e-02, 5.84721434e-02, 3.63296186e-02, 3.34308359e-02, 3.59345855e-02,
            3.51389106e-02, 1.11491704e+00, 2.69811063e-02, 2.42998815e-02, 1.56923037e-02,
            2.27464139e-02, 3.89042014e-02, 2.92040935e-02, 2.01675725e-02, 1.87341798e-02,
            2.76827000e-02, 1.69243981e-02, 1.21166394e-02, 1.37290338e-02, 1.58524035e-02,
            2.55629891e-02, 2.70253777e-02, 2.55635071e-02, 2.83475531e-02, 3.69174424e-03
        ], [
            8.97191308e-03, 1.49177163e-02, 1.10733195e-02, 1.11836836e-02, 1.06640501e-02,
            1.35483281e-02, 1.29121334e-02, 1.35709106e-02, 1.42931727e-02, 1.33564460e-02,
            1.77164648e-02, 1.56706787e-02, 1.27381094e-02, 1.26842336e-02, 1.20756272e-02,
            1.23910937e-02, 3.58750410e-02, 1.04499252e+00, 1.14007141e-02, 1.06063947e-02,
            1.14655107e-02, 1.36113372e-02, 1.72513519e-02, 1.47033020e-02, 1.14245509e-02,
            2.48820248e-02, 1.77384091e-02, 1.19705594e-02, 5.06262470e-02, 1.20103969e-02,
            2.69035838e-02, 1.90993894e-02, 1.37254219e-02, 2.11215051e-02, 4.09787074e-03
        ], [
            9.58115487e-03, 8.91650730e-03, 1.43701277e-02, 1.31764268e-02, 1.63199048e-02,
            1.41106634e-02, 1.13222265e-02, 1.00016903e-02, 1.40692465e-02, 1.44618089e-02,
            1.50160680e-02, 1.48383569e-02, 1.33697877e-02, 1.33304796e-02, 1.40510920e-02,
            1.39737745e-02, 1.43541371e-02, 1.48555008e-02, 1.01341363e+00, 8.09532671e-03,
            7.75353980e-03, 1.11356805e-02, 2.34538737e-02, 1.03220529e-02, 8.58302024e-03,
            1.32039302e-02, 8.17501257e-03, 5.70236050e-03, 3.95485412e-03, 9.92716152e-03,
            7.74312558e-03, 5.67648203e-03, 7.29620624e-03, 1.09574153e-02, 9.80087544e-04
        ], [
            5.93665927e-02, 4.92552199e-02, 1.07920694e-01, 1.03870207e-01, 1.30377154e-01,
            1.00161144e-01, 8.94843156e-02, 7.29871126e-02, 9.23759748e-02, 9.89408562e-02,
            8.10581337e-02, 9.00216049e-02, 9.73270881e-02, 1.03405122e-01, 1.02984212e-01,
            9.78036124e-02, 4.57566717e-02, 8.24062111e-02, 6.73699235e-02, 1.04975481e+00,
            2.76014748e-02, 7.76585957e-02, 4.61148614e-02, 4.80313060e-02, 4.48292131e-02,
            4.35437059e-02, 2.46139114e-02, 1.69850169e-02, 1.24931215e-02, 2.81219049e-02,
            3.87837708e-02, 2.13363699e-02, 4.84536151e-02, 3.36787329e-02, 5.60071950e-03
        ], [
            2.93677308e-02, 2.10886312e-02, 5.28982101e-02, 5.10653383e-02, 6.97797826e-02,
            4.21765239e-02, 3.61318822e-02, 2.89471277e-02, 4.27313284e-02, 4.43092348e-02,
            3.61270254e-02, 3.60689123e-02, 3.76703618e-02, 3.73979056e-02, 4.45165223e-02,
            4.66722118e-02, 1.86949037e-02, 4.54483133e-02, 3.04105412e-02, 1.70082222e-02,
            1.01561955e+00, 4.41255896e-02, 2.63833370e-02, 2.35920298e-02, 2.10551748e-02,
            2.15261439e-02, 1.80292713e-02, 1.08763938e-02, 8.33153575e-03, 1.79798262e-02,
            1.69842870e-02, 1.15075256e-02, 2.17465552e-02, 2.27478407e-02, 3.03134335e-03
        ], [
            5.87006517e-03, 9.19411572e-03, 1.15097346e-02, 1.29939256e-02, 1.33069235e-02,
            1.34736432e-02, 1.43567473e-02, 9.07894807e-03, 1.43221543e-02, 1.51166713e-02,
            1.40861242e-02, 1.45259430e-02, 1.60356481e-02, 1.69895741e-02, 1.35687782e-02,
            1.35036114e-02, 1.02491296e-02, 1.33483862e-02, 1.29559309e-02, 1.49624261e-02,
            1.08773227e-02, 1.01625786e+00, 1.45466846e-02, 1.71610118e-02, 3.24576452e-02,
            2.94540140e-02, 1.31563734e-02, 1.65452478e-02, 5.38874120e-03, 1.97430930e-02,
            1.43144130e-02, 9.90885442e-03, 1.35327041e-02, 1.74049782e-02, 1.03588425e-03
        ], [
            2.78268225e-02, 4.34258592e-02, 4.54417359e-02, 4.24062103e-02, 4.75187895e-02,
            4.91020083e-02, 3.88318894e-02, 6.32268352e-02, 4.38296983e-02, 4.16177918e-02,
            5.97644035e-02, 4.38437736e-02, 3.46303856e-02, 3.10895027e-02, 3.74084987e-02,
            4.15377451e-02, 4.18240526e-02, 3.95989230e-02, 2.72617969e-02, 2.87117321e-02,
            1.86834073e-02, 2.70184351e-02, 1.05744017e+00, 4.10321582e-02, 2.92495194e-02,
            4.75930098e-02, 1.81675634e-02, 1.38569437e-02, 6.94028073e-03, 1.57208273e-02,
            2.16448161e-02, 1.65959884e-02, 1.84204753e-02, 2.08862412e-02, 4.54195926e-03
        ], [
            5.19117046e-03, 6.87217592e-03, 7.95585015e-03, 7.04263692e-03, 8.40996891e-03,
            1.25323523e-02, 4.62867423e-03, 9.32617953e-03, 6.91930874e-03, 6.12384053e-03,
            9.34965886e-03, 8.95354634e-03, 5.46218479e-03, 4.95663496e-03, 5.97319126e-03,
            6.40827466e-03, 4.55431570e-03, 6.38534651e-03, 3.27379223e-03, 4.23890483e-03,
            2.66550069e-03, 4.43509555e-03, 5.95282574e-03, 1.11547883e+00, 6.10082166e-03,
            9.02001998e-03, 2.51072933e-03, 1.75203564e-03, 9.16921640e-04, 2.37880925e-03,
            3.04866479e-03, 2.61582574e-03, 2.21540209e-03, 2.57663027e-03, 6.85436039e-04
        ], [
            2.76958665e-03, 4.92062577e-03, 4.92796835e-03, 5.44052896e-03, 7.05445857e-03,
            4.77998039e-03, 5.84165003e-03, 4.51882750e-03, 5.84010454e-03, 5.70176512e-03,
            4.94198534e-03, 5.20572367e-03, 5.62219774e-03, 6.24114421e-03, 6.16008691e-03,
            5.20165682e-03, 3.15486242e-03, 4.40302315e-03, 4.82659631e-03, 6.83654639e-03,
            3.67619469e-03, 3.93383090e-03, 5.29495431e-03, 1.13856542e-02, 1.03824757e+00,
            2.48198931e-02, 7.57271366e-03, 5.92177649e-03, 1.61170443e-03, 6.93737047e-03,
            6.79264226e-03, 3.45130826e-03, 3.43401851e-03, 6.61508540e-03, 4.76071350e-04
        ], [
            1.12374476e-02, 1.65980031e-02, 1.96402992e-02, 1.68723852e-02, 1.98449138e-02,
            2.09842651e-02, 1.96339640e-02, 2.12470284e-02, 1.89038415e-02, 1.88174260e-02,
            2.37601155e-02, 2.01095673e-02, 1.77677975e-02, 1.69133650e-02, 1.86720178e-02,
            1.80247599e-02, 1.39389951e-02, 1.54924838e-02, 1.65299077e-02, 2.86787053e-02,
            1.57020348e-02, 1.82283037e-02, 5.06836583e-02, 1.85656547e-01, 1.20776566e-01,
            1.13829769e+00, 9.47668797e-03, 9.39179831e-03, 3.46910651e-03, 1.13862953e-02,
            1.12520285e-02, 6.71859342e-03, 9.44540856e-03, 1.25304611e-02, 1.62978050e-03
        ], [
            9.33691098e-03, 1.45261453e-02, 1.59161067e-02, 1.70140210e-02, 1.83062521e-02,
            1.64764606e-02, 2.48080431e-02, 1.41535808e-02, 2.02553559e-02, 1.89790948e-02,
            1.73306796e-02, 1.83150311e-02, 2.01480812e-02, 2.09092505e-02, 1.81008905e-02,
            1.96530464e-02, 1.36331113e-02, 2.03515439e-02, 2.12940626e-02, 2.78377899e-02,
            2.64174598e-02, 2.16618073e-02, 2.46344808e-02, 3.79330110e-02, 4.46292775e-02,
            3.56904370e-02, 1.09442061e+00, 3.42223123e-02, 8.91279934e-03, 3.06601618e-02,
            2.54264826e-02, 1.27779641e-02, 1.87970407e-02, 3.14440680e-02, 1.64952217e-03
        ], [
            4.53859959e-02, 6.35674434e-02, 5.86577378e-02, 6.75338041e-02, 6.59375285e-02,
            6.56467993e-02, 6.26532342e-02, 5.79038628e-02, 6.61416561e-02, 6.37873380e-02,
            6.54957466e-02, 6.82078619e-02, 6.48297434e-02, 6.28211232e-02, 6.19955060e-02,
            6.52369877e-02, 5.36732742e-02, 6.02577730e-02, 5.80607429e-02, 7.21142437e-02,
            6.14151318e-02, 5.76165418e-02, 7.49913540e-02, 8.72187058e-02, 7.67371534e-02,
            6.30327414e-02, 4.68914149e-02, 1.22167500e+00, 7.02905284e-02, 6.45728991e-02,
            4.73952143e-02, 2.85347743e-02, 4.81668241e-02, 6.27031493e-02, 1.01523717e-02
        ], [
            1.70105852e-02, 1.64002017e-02, 2.20121918e-02, 2.30832392e-02, 2.55902838e-02,
            2.23298794e-02, 2.56888142e-02, 1.67592389e-02, 2.21148882e-02, 2.35853589e-02,
            2.09245473e-02, 2.22428894e-02, 2.36498151e-02, 2.34028497e-02, 2.25521124e-02,
            2.59809365e-02, 1.67568776e-02, 2.88932198e-02, 3.77717897e-02, 4.07483141e-02,
            5.33149858e-02, 4.33757173e-02, 2.37105814e-02, 4.85924943e-02, 3.34442324e-02,
            3.74772424e-02, 2.23922623e-02, 3.91759974e-02, 1.03730688e+00, 3.65893621e-02,
            2.13513373e-02, 2.41506545e-02, 4.10244349e-02, 4.17115125e-02, 2.48926377e-03
        ], [
            4.73707499e-02, 7.11564215e-02, 9.51780515e-02, 9.26142264e-02, 9.44545709e-02,
            8.04353967e-02, 1.25739173e-01, 7.04320405e-02, 1.32969188e-01, 1.07544261e-01,
            9.15001520e-02, 9.22898400e-02, 1.12502451e-01, 1.37673371e-01, 1.16295767e-01,
            1.00663368e-01, 7.72469760e-02, 1.11162226e-01, 9.22675163e-02, 9.87935520e-02,
            9.93659743e-02, 8.20798741e-02, 8.49152077e-02, 1.22537660e-01, 1.55205870e-01,
            1.20516687e-01, 1.03562556e-01, 1.36299977e-01, 5.82954726e-02, 1.17526955e+00,
            9.33261872e-02, 4.89112911e-02, 9.19520348e-02, 1.10984361e-01, 7.48811386e-03
        ], [
            3.75609017e-03, 6.05567550e-03, 5.33074870e-03, 4.61625365e-03, 4.48070658e-03,
            6.93185857e-03, 6.90503099e-03, 5.15321449e-03, 7.11869963e-03, 6.14415430e-03,
            5.94662001e-03, 6.47456165e-03, 4.94559296e-03, 5.19653632e-03, 5.09264707e-03,
            6.12581271e-03, 8.16458391e-03, 4.73828678e-03, 4.69353327e-03, 3.86990263e-03,
            5.46841001e-03, 6.38274370e-03, 5.64729305e-03, 1.30600089e-02, 1.22563668e-02,
            6.91786445e-03, 4.51796485e-03, 3.55539884e-03, 4.72706472e-03, 4.97248703e-03,
            1.00972721e+00, 5.20412233e-03, 7.48232305e-03, 1.04794961e-02, 5.45051936e-04
        ], [
            1.53096123e-03, 1.22847263e-03, 1.59467640e-03, 1.53042602e-03, 1.60375554e-03,
            1.38343993e-03, 1.80034932e-03, 1.42187577e-03, 2.11935021e-03, 1.73657004e-03,
            1.56042409e-03, 1.64750928e-03, 2.03549444e-03, 2.33187207e-03, 1.83637930e-03,
            1.47123713e-03, 1.82752779e-03, 1.53930950e-03, 1.28367760e-03, 1.66696739e-03,
            2.58279399e-03, 1.32584575e-03, 1.91612182e-03, 1.52998291e-03, 2.91354915e-03,
            2.12428979e-03, 2.38497998e-03, 2.12466181e-03, 5.86339779e-04, 3.00767356e-03,
            5.17186705e-03, 1.00686285e+00, 1.80950013e-03, 2.28675933e-03, 1.35016301e-04
        ], [
            2.79177799e-03, 8.81247867e-04, 1.76697414e-03, 1.32133761e-03, 1.35250233e-03,
            1.44467599e-03, 1.18988785e-03, 8.65198022e-04, 1.23032466e-03, 1.19687224e-03,
            1.12182063e-03, 1.12924160e-03, 1.29597065e-03, 1.25501765e-03, 1.14076503e-03,
            1.30028950e-03, 9.23851404e-04, 1.21332060e-03, 1.04989223e-03, 1.09990068e-03,
            8.65302268e-04, 1.79191993e-03, 1.22025549e-03, 1.75071850e-03, 1.27692620e-03,
            1.33009264e-03, 1.23196667e-03, 1.08321698e-03, 3.60844578e-04, 1.29816155e-03,
            6.77230039e-03, 1.74588266e-03, 1.02326055e+00, 2.29201583e-03, 1.21566362e-04
        ], [
            9.74263658e-03, 1.50785046e-02, 1.92441004e-02, 1.92417218e-02, 2.13349738e-02,
            1.82186492e-02, 2.94912150e-02, 1.54075623e-02, 2.38128194e-02, 2.10069309e-02,
            1.88098926e-02, 2.18577734e-02, 2.15708897e-02, 2.18557871e-02, 2.00835171e-02,
            2.08659665e-02, 1.55580608e-02, 1.95458276e-02, 2.07285716e-02, 1.91676062e-02,
            1.98120387e-02, 2.52881145e-02, 2.01620866e-02, 2.16205071e-02, 2.38277603e-02,
            2.43873652e-02, 2.82599078e-02, 2.22857085e-02, 1.32456747e-02, 4.23728685e-02,
            2.49740428e-02, 1.61663126e-02, 2.55940348e-02, 1.09457767e+00, 2.02732839e-03
        ], [
            4.80985248e-05, 5.83062239e-05, 9.41535010e-05, 1.10374700e-04, 1.02333785e-04,
            9.98930715e-05, 1.04199249e-04, 6.73697321e-05, 9.72789540e-05, 1.03461985e-04,
            8.57559870e-05, 1.03574045e-04, 9.99745458e-05, 8.81312500e-05, 1.21661532e-04,
            1.03802489e-04, 6.09055197e-05, 1.49703133e-04, 1.93925924e-04, 9.18345830e-05,
            1.08285004e-04, 1.26712162e-04, 2.19469169e-04, 9.98065386e-05, 1.00857829e-04,
            9.60586634e-05, 1.28294613e-04, 1.19029245e-04, 3.16445850e-04, 1.25758585e-04,
            1.32734920e-04, 8.34228095e-05, 1.77193186e-04, 1.82126670e-04, 1.00003186e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 391)
    return ret


@cache('run')
def leontief_matrix_x1997x():
    """
    leontief matrix x1997x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17407328e+00, 8.10385233e-03, 3.71432733e-01, 7.19222826e-02, 9.36765402e-02,
            1.99469376e-01, 2.80716229e-02, 1.00181436e-02, 2.20700237e-02, 2.90689181e-02,
            1.01448740e-02, 7.60187355e-03, 8.16722562e-03, 8.18065116e-03, 9.15418334e-03,
            3.79584574e-02, 5.19253818e-03, 1.69371708e-02, 6.21794016e-03, 8.55791050e-03,
            8.13237937e-03, 1.06900106e-01, 6.39411535e-03, 1.14033693e-02, 9.23629672e-03,
            9.85914401e-03, 4.71719680e-03, 4.40465908e-03, 3.25152658e-03, 7.14765507e-03,
            9.85025930e-03, 8.69969438e-03, 1.33607056e-02, 1.10859202e-02, 1.64594222e-02
        ], [
            2.25628180e-02, 1.11545863e+00, 2.46494568e-02, 2.61150722e-02, 2.20460426e-02,
            2.76086130e-02, 2.64361394e-02, 5.05352591e-01, 8.06563636e-02, 4.01773810e-02,
            1.24812373e-01, 9.09206676e-02, 3.43415011e-02, 2.69965339e-02, 3.07134910e-02,
            3.51405621e-02, 1.35594559e-01, 5.44221007e-02, 1.55120028e-02, 1.17736978e-02,
            1.05878293e-02, 1.76229757e-02, 4.31933673e-02, 4.36920434e-02, 4.17992295e-02,
            2.36056441e-02, 9.90890810e-03, 6.41798463e-03, 8.65537074e-03, 1.06537965e-02,
            1.63945197e-02, 1.29049458e-02, 1.58606247e-02, 1.71236341e-02, 6.20163012e-02
        ], [
            7.73270143e-02, 5.10096830e-03, 1.18951224e+00, 1.57447633e-02, 1.34702144e-01,
            2.00997226e-02, 1.19262283e-02, 6.10251897e-03, 2.14243724e-02, 1.38864298e-02,
            8.80628412e-03, 7.58252979e-03, 8.28730404e-03, 8.63621607e-03, 8.20332498e-03,
            1.07858252e-02, 5.06184634e-03, 7.84274002e-03, 7.08080199e-03, 1.01917782e-02,
            9.01134343e-03, 1.96619455e-01, 7.07142657e-03, 1.61689318e-02, 1.46616765e-02,
            1.16970217e-02, 5.94724309e-03, 5.74533846e-03, 2.33753170e-03, 8.56246771e-03,
            1.44933777e-02, 1.39682306e-02, 2.27036368e-02, 1.30886604e-02, 2.71593185e-03
        ], [
            6.52589025e-03, 4.80705078e-03, 7.64597296e-03, 1.39492399e+00, 1.01673007e-01,
            1.48177474e-02, 1.70651919e-02, 4.36365302e-03, 1.18872756e-02, 3.65414816e-02,
            1.19171921e-02, 7.28489317e-03, 9.15019218e-03, 8.96462333e-03, 1.86282211e-02,
            4.12817286e-02, 3.42437309e-03, 8.89631591e-03, 6.97357414e-03, 6.24583592e-03,
            6.90409414e-03, 7.80406060e-03, 5.66224182e-03, 8.62166285e-03, 5.31080588e-03,
            5.78799219e-03, 6.84054978e-03, 2.88315002e-03, 1.62509797e-03, 4.38291685e-03,
            7.49716141e-03, 2.86755588e-03, 7.97349488e-03, 1.03117244e-02, 1.17082399e-03
        ], [
            7.00393012e-04, 7.76053170e-04, 9.84397924e-04, 1.30608667e-02, 1.23996191e+00,
            2.44439981e-03, 2.82983224e-03, 7.05068554e-04, 1.10354013e-03, 2.45768177e-03,
            1.54773489e-03, 1.36934896e-03, 1.41229048e-03, 1.34203140e-03, 3.15013037e-03,
            5.53675075e-03, 6.23831391e-04, 9.82314721e-04, 1.22180218e-03, 1.04458950e-03,
            1.00916838e-03, 7.96995157e-04, 7.48514298e-04, 8.89011023e-04, 7.40717692e-04,
            6.89352645e-04, 1.30616331e-03, 3.70138676e-04, 1.95138723e-04, 5.81802264e-04,
            1.57768156e-03, 4.34831218e-04, 6.24002586e-04, 1.43894871e-03, 1.01045614e-04
        ], [
            4.36165609e-03, 5.25531524e-03, 6.12127741e-03, 6.46665885e-03, 7.57404121e-03,
            1.24275301e+00, 2.07667381e-02, 4.18297051e-03, 6.18472133e-03, 7.92438348e-03,
            1.32674297e-02, 8.79066883e-03, 8.63962602e-03, 7.10998442e-03, 1.02203668e-02,
            9.55884730e-02, 4.46563771e-03, 4.86339578e-02, 4.26939428e-03, 4.20923615e-03,
            3.95415602e-03, 6.05212233e-03, 4.62020953e-03, 4.62782469e-03, 3.88215681e-03,
            9.09284655e-03, 3.40713487e-03, 2.86273635e-03, 5.68570044e-03, 3.75485137e-03,
            4.97754376e-03, 3.71789421e-03, 3.92158756e-03, 7.12469319e-03, 1.13196975e-03
        ], [
            1.59504566e-02, 1.54514057e-02, 5.34099459e-02, 3.63042562e-02, 4.34652934e-02,
            3.59116521e-02, 1.32562155e+00, 1.54668709e-02, 4.90908959e-02, 4.95716127e-02,
            4.83592059e-02, 2.63981774e-02, 3.42063652e-02, 4.39676374e-02, 3.19079143e-02,
            4.86715872e-02, 1.55305285e-02, 2.63502959e-02, 2.42527400e-02, 2.96306272e-02,
            2.81760603e-02, 3.18744151e-02, 2.05283927e-02, 2.35613729e-02, 2.64319125e-02,
            3.65484902e-02, 2.73382998e-02, 3.36120619e-02, 9.88253050e-03, 4.27718370e-02,
            3.25973568e-02, 2.84755303e-02, 2.62143319e-02, 4.75179985e-02, 2.38237628e-03
        ], [
            2.23616804e-02, 2.28106693e-02, 1.86382108e-02, 1.95450494e-02, 1.74224005e-02,
            1.92801508e-02, 1.62114553e-02, 1.06242938e+00, 4.72745701e-02, 2.81887353e-02,
            3.17512381e-02, 2.43996355e-02, 1.61718533e-02, 1.46124897e-02, 1.53050602e-02,
            1.75152605e-02, 3.11321669e-02, 2.56044861e-02, 1.19046479e-02, 1.17797037e-02,
            9.91727053e-03, 1.25758108e-02, 6.19272776e-02, 6.94977843e-02, 6.85122800e-02,
            2.55207719e-02, 9.03773956e-03, 5.42963508e-03, 4.31673156e-03, 9.96751235e-03,
            1.39474786e-02, 7.25456385e-03, 1.12843114e-02, 1.36675635e-02, 1.89101430e-03
        ], [
            7.24890007e-02, 3.25142858e-02, 5.37937824e-02, 1.47714766e-01, 9.45470630e-02,
            6.95663938e-02, 7.68255278e-02, 3.93734741e-02, 1.32654219e+00, 3.14955323e-01,
            6.41860724e-02, 4.72932934e-02, 4.93924613e-02, 5.95734359e-02, 6.07072527e-02,
            8.01984924e-02, 2.03194764e-02, 4.06152480e-02, 2.91342694e-02, 1.38748310e-02,
            1.22857129e-02, 2.51120844e-02, 1.92157849e-02, 1.90329585e-02, 1.71406298e-02,
            1.81799538e-02, 1.19117866e-02, 7.91549705e-03, 7.08791650e-03, 1.55535019e-02,
            2.23787473e-02, 1.17249504e-02, 9.64691338e-02, 2.49409974e-02, 3.58710370e-03
        ], [
            1.28015166e-02, 1.76768964e-02, 2.93912779e-02, 2.35994794e-02, 6.08724150e-02,
            2.33205016e-02, 2.88689759e-02, 1.36462519e-02, 3.53384467e-02, 1.13934384e+00,
            2.38619664e-02, 2.21976271e-02, 4.72718739e-02, 4.87914452e-02, 7.13261958e-02,
            5.69336230e-02, 1.00620462e-02, 3.25221083e-02, 3.38174657e-02, 1.01383514e-02,
            1.01553172e-02, 1.39375817e-02, 2.09918856e-02, 1.31481765e-02, 1.23601034e-02,
            1.48404602e-02, 1.00222099e-02, 4.52163326e-03, 4.21088432e-03, 8.15724368e-03,
            1.02368866e-02, 4.81417574e-03, 1.33832870e-02, 1.04980366e-02, 1.65582364e-03
        ], [
            5.72049347e-03, 1.00053646e-02, 1.18571192e-02, 7.70756388e-03, 7.91363446e-03,
            1.76618033e-02, 6.43468235e-03, 8.75707281e-03, 1.41575422e-02, 1.42778262e-02,
            1.12812590e+00, 2.06896152e-02, 1.55696798e-02, 2.33003324e-02, 1.93828896e-02,
            1.51970747e-02, 8.19573419e-03, 9.91520379e-02, 8.43217502e-03, 4.26240507e-03,
            3.68190631e-03, 8.60450475e-03, 5.77659381e-03, 5.29765569e-03, 4.74900017e-03,
            6.25780417e-03, 4.95220225e-03, 2.45043347e-03, 7.74594821e-03, 4.06018383e-03,
            6.18194047e-03, 4.74884541e-03, 6.13059584e-03, 6.12395025e-03, 1.05079900e-03
        ], [
            2.46021107e-02, 6.31347754e-02, 4.73327839e-02, 3.52597876e-02, 4.48984931e-02,
            7.34252598e-02, 3.92142604e-02, 4.65809266e-02, 5.77105253e-02, 7.00784607e-02,
            8.07309571e-02, 1.49751515e+00, 3.03706764e-01, 1.73363273e-01, 2.51272469e-01,
            1.99050736e-01, 3.59882000e-02, 1.62266432e-01, 7.13591499e-02, 1.99806092e-02,
            1.74727199e-02, 2.55560450e-02, 3.61548655e-02, 4.63013574e-02, 3.90949348e-02,
            3.08578479e-02, 2.25539484e-02, 1.02771985e-02, 1.53000951e-02, 1.87349351e-02,
            2.81472531e-02, 1.35564994e-02, 2.09239560e-02, 2.36877765e-02, 5.60254057e-03
        ], [
            1.44206885e-02, 2.99378415e-02, 1.47640322e-02, 1.78277290e-02, 1.62505467e-02,
            2.12478425e-02, 1.73964275e-02, 2.19096074e-02, 1.99671701e-02, 2.36713531e-02,
            2.66284591e-02, 3.53273897e-02, 1.14792875e+00, 3.22622445e-02, 5.87874397e-02,
            2.25584963e-02, 1.73952096e-02, 3.01492761e-02, 3.62038145e-02, 8.63858391e-03,
            7.21451220e-03, 8.99567364e-03, 1.51721427e-02, 1.67302559e-02, 1.34869724e-02,
            1.23230647e-02, 8.41012931e-03, 4.56967205e-03, 4.54688477e-03, 7.87014160e-03,
            1.29748103e-02, 5.95537253e-03, 7.97318326e-03, 9.56958895e-03, 2.27484607e-03
        ], [
            1.16482258e-02, 2.31896737e-02, 1.88404019e-02, 2.31343111e-02, 2.18363769e-02,
            2.54504984e-02, 3.01864682e-02, 2.04548014e-02, 3.07901170e-02, 3.65946430e-02,
            2.88216823e-02, 4.62315709e-02, 1.31549865e-01, 1.36143565e+00, 1.18695264e-01,
            4.63516339e-02, 3.01711683e-02, 5.49120277e-02, 5.49388109e-02, 2.10576808e-02,
            1.86132804e-02, 1.64871003e-02, 2.29094657e-02, 2.30637705e-02, 2.28826699e-02,
            2.26955776e-02, 5.11401210e-02, 1.33275929e-02, 8.22352365e-03, 2.79931113e-02,
            2.79922032e-02, 1.42453813e-02, 3.50863974e-02, 2.21294764e-02, 2.68209337e-03
        ], [
            1.23487045e-02, 1.56324590e-02, 1.35285207e-02, 1.38437481e-02, 1.46330589e-02,
            1.54414174e-02, 1.14944461e-02, 1.32006009e-02, 1.30901669e-02, 1.49937424e-02,
            1.58380284e-02, 1.94314522e-02, 3.22979088e-02, 1.82404801e-02, 1.42394952e+00,
            1.69596113e-02, 1.03206014e-02, 1.60605808e-02, 1.28255338e-01, 1.26370337e-02,
            1.04441398e-02, 8.90167713e-03, 5.24911117e-02, 7.45036804e-02, 7.52892506e-02,
            2.54489504e-02, 9.71787696e-03, 5.16219137e-03, 3.23978274e-03, 9.55299839e-03,
            3.13914522e-02, 5.00148325e-03, 7.10928430e-03, 1.08459162e-02, 1.40253012e-03
        ], [
            2.48690781e-03, 3.41192960e-03, 3.88246701e-03, 1.03541497e-02, 7.41219904e-03,
            1.06698303e-02, 6.71195399e-03, 3.07549970e-03, 5.11037072e-03, 6.13495842e-03,
            6.71100415e-03, 1.30309827e-02, 9.04538335e-03, 7.47710731e-03, 1.18759553e-02,
            1.04076428e+00, 3.03312989e-03, 9.29599950e-03, 5.14882903e-03, 3.05832967e-03,
            3.16941439e-03, 4.88138656e-03, 3.61175517e-03, 3.81498679e-03, 3.62426942e-03,
            3.61972188e-03, 2.62393124e-03, 2.72526274e-03, 2.26214658e-03, 4.35188231e-03,
            5.98798869e-03, 4.72290618e-03, 4.90671921e-03, 8.21584030e-03, 3.52061547e-03
        ], [
            2.10338925e-02, 3.68408665e-02, 3.07955670e-02, 4.12029806e-02, 2.86535557e-02,
            3.64967734e-02, 4.22571257e-02, 3.68980746e-02, 5.43601520e-02, 4.83303223e-02,
            5.94766435e-02, 5.61138245e-02, 3.44894461e-02, 3.12246214e-02, 3.50381597e-02,
            3.31204630e-02, 1.11635308e+00, 2.58411830e-02, 2.31440466e-02, 1.42147785e-02,
            2.11434176e-02, 3.74036316e-02, 2.75851698e-02, 1.89997103e-02, 1.75405766e-02,
            2.53904891e-02, 1.70287938e-02, 1.16569683e-02, 1.32694779e-02, 1.51179366e-02,
            2.46188291e-02, 2.60022014e-02, 2.46140415e-02, 2.74598724e-02, 2.92851546e-03
        ], [
            8.65066663e-03, 1.44096470e-02, 1.06756408e-02, 1.08659405e-02, 1.03893032e-02,
            1.28962223e-02, 1.24083553e-02, 1.27993531e-02, 1.35527878e-02, 1.26418026e-02,
            1.64307071e-02, 1.47909767e-02, 1.20813387e-02, 1.17360587e-02, 1.17173595e-02,
            1.16999684e-02, 3.49624292e-02, 1.04574130e+00, 1.09685474e-02, 9.65322590e-03,
            1.07289343e-02, 1.27171616e-02, 1.65815825e-02, 1.38942370e-02, 1.09550498e-02,
            2.38072863e-02, 1.79012413e-02, 1.12797235e-02, 4.80793566e-02, 1.16215152e-02,
            2.51644949e-02, 1.76212307e-02, 1.30536597e-02, 1.99665304e-02, 3.46570687e-03
        ], [
            9.59518497e-03, 8.84410603e-03, 1.43496236e-02, 1.34576262e-02, 1.68472533e-02,
            1.39286359e-02, 1.12736480e-02, 9.90440844e-03, 1.39013397e-02, 1.42222444e-02,
            1.45103895e-02, 1.43846917e-02, 1.31613340e-02, 1.26567852e-02, 1.40655011e-02,
            1.36744834e-02, 1.42659668e-02, 1.48011333e-02, 1.01368070e+00, 8.45873093e-03,
            7.79314597e-03, 1.09317256e-02, 2.34097589e-02, 1.00210665e-02, 8.84008384e-03,
            1.29444865e-02, 8.35785310e-03, 5.64439255e-03, 3.99289363e-03, 9.85031462e-03,
            7.69483216e-03, 5.59490146e-03, 7.34118400e-03, 1.09763908e-02, 8.33000625e-04
        ], [
            5.99879630e-02, 4.72438668e-02, 1.08591154e-01, 1.04347239e-01, 1.32057945e-01,
            1.01291119e-01, 8.92071730e-02, 7.11857977e-02, 9.24821131e-02, 9.74054016e-02,
            7.82250655e-02, 8.89210605e-02, 9.52475675e-02, 1.00714472e-01, 1.03828013e-01,
            9.53726949e-02, 4.49800377e-02, 8.11804567e-02, 6.63740135e-02, 1.04683629e+00,
            2.70785102e-02, 7.72009263e-02, 4.55853210e-02, 4.78466227e-02, 4.47779116e-02,
            4.29979893e-02, 2.54969667e-02, 1.68437550e-02, 1.24123969e-02, 2.84572877e-02,
            3.90741797e-02, 2.16070508e-02, 4.91249289e-02, 3.41100054e-02, 4.67593970e-03
        ], [
            2.94342208e-02, 2.01820073e-02, 5.23723785e-02, 5.13206474e-02, 7.01555897e-02,
            4.17859926e-02, 3.55031588e-02, 2.74316460e-02, 4.26858360e-02, 4.35728678e-02,
            3.50833636e-02, 3.55675228e-02, 3.66449312e-02, 3.58207465e-02, 4.39775193e-02,
            4.53951835e-02, 1.84576722e-02, 4.48719308e-02, 2.96457259e-02, 1.59826913e-02,
            1.01504691e+00, 4.23394966e-02, 2.60744766e-02, 2.31876136e-02, 2.10282595e-02,
            2.08571879e-02, 1.84018659e-02, 1.06515085e-02, 8.15366036e-03, 1.78507410e-02,
            1.67692150e-02, 1.13580845e-02, 2.16548551e-02, 2.25629365e-02, 2.53500708e-03
        ], [
            6.08516208e-03, 9.21543889e-03, 1.21210276e-02, 1.34524373e-02, 1.37145827e-02,
            1.37786654e-02, 1.46888614e-02, 9.14526464e-03, 1.45475153e-02, 1.52707384e-02,
            1.43739241e-02, 1.46332859e-02, 1.63327761e-02, 1.71185666e-02, 1.41285668e-02,
            1.34344967e-02, 1.06043714e-02, 1.37561683e-02, 1.31776745e-02, 1.47920350e-02,
            1.09293794e-02, 1.01665348e+00, 1.46898159e-02, 1.79812174e-02, 3.30735777e-02,
            3.12490982e-02, 1.22802094e-02, 1.65046409e-02, 5.52892595e-03, 2.01638895e-02,
            1.48381056e-02, 1.03797784e-02, 1.35907742e-02, 1.79356906e-02, 8.92863249e-04
        ], [
            2.79843641e-02, 4.09035286e-02, 4.54936352e-02, 4.32050780e-02, 4.75979270e-02,
            4.91552516e-02, 3.87143272e-02, 5.98662566e-02, 4.36140402e-02, 4.13407335e-02,
            5.86523453e-02, 4.31685972e-02, 3.38395439e-02, 3.00787502e-02, 3.72922647e-02,
            4.07180922e-02, 4.19469525e-02, 3.86007717e-02, 2.69192877e-02, 2.72271103e-02,
            1.81418018e-02, 2.62575524e-02, 1.05651491e+00, 4.07917164e-02, 2.78975474e-02,
            4.69759254e-02, 1.90767072e-02, 1.36271689e-02, 6.81089925e-03, 1.55091032e-02,
            2.12799407e-02, 1.68641500e-02, 1.83775238e-02, 2.07014371e-02, 3.74166599e-03
        ], [
            5.53848001e-03, 6.86703253e-03, 8.53217967e-03, 7.38383906e-03, 8.99822674e-03,
            1.32575568e-02, 4.84483529e-03, 9.23611369e-03, 7.27258398e-03, 6.33695051e-03,
            1.03709420e-02, 9.06646009e-03, 5.55030349e-03, 5.08478239e-03, 6.13143008e-03,
            6.67483962e-03, 4.79024255e-03, 6.73028171e-03, 3.39243530e-03, 3.76251814e-03,
            2.57326679e-03, 4.67368930e-03, 6.76388633e-03, 1.11348796e+00, 6.40196205e-03,
            9.51584622e-03, 2.58119329e-03, 1.83166345e-03, 9.51541619e-04, 2.51981235e-03,
            3.16124638e-03, 3.08987965e-03, 2.35428957e-03, 2.80126265e-03, 5.94362649e-04
        ], [
            2.80167786e-03, 4.74140843e-03, 4.99533279e-03, 5.50010151e-03, 7.14523590e-03,
            4.76176012e-03, 5.89523314e-03, 4.36662666e-03, 5.87788747e-03, 5.72183760e-03,
            4.82113800e-03, 5.12833018e-03, 5.68948960e-03, 6.23825219e-03, 6.13104324e-03,
            5.11382544e-03, 3.18437103e-03, 4.44446776e-03, 4.86475395e-03, 6.56185351e-03,
            3.67561753e-03, 3.94549827e-03, 5.38405439e-03, 1.13052705e-02, 1.03951443e+00,
            2.61143127e-02, 8.19223069e-03, 6.04862981e-03, 1.65406124e-03, 7.18900545e-03,
            6.96034165e-03, 3.49878147e-03, 3.49425526e-03, 6.86182092e-03, 3.98543773e-04
        ], [
            1.13563870e-02, 1.56681211e-02, 2.00764721e-02, 1.69883699e-02, 2.00789631e-02,
            2.16072197e-02, 1.97988344e-02, 2.02957593e-02, 1.88799572e-02, 1.85276742e-02,
            2.33907280e-02, 1.95161768e-02, 1.72666174e-02, 1.64139346e-02, 1.86282143e-02,
            1.76073410e-02, 1.37567833e-02, 1.52999520e-02, 1.66654443e-02, 2.77427293e-02,
            1.58929847e-02, 1.82789628e-02, 5.29624246e-02, 1.91961885e-01, 1.21616732e-01,
            1.14098359e+00, 9.87538329e-03, 9.49892868e-03, 3.51566180e-03, 1.15766547e-02,
            1.05905436e-02, 6.95518784e-03, 9.59916227e-03, 1.31172857e-02, 1.34578512e-03
        ], [
            9.91157940e-03, 1.51555546e-02, 1.70175818e-02, 1.83568537e-02, 1.97509070e-02,
            1.75260268e-02, 2.61995440e-02, 1.47748731e-02, 2.13447336e-02, 2.01756930e-02,
            1.81457310e-02, 1.95230042e-02, 2.13064852e-02, 2.18632329e-02, 1.94574264e-02,
            2.05033834e-02, 1.49378626e-02, 2.12991127e-02, 2.26324216e-02, 2.82868127e-02,
            2.70861966e-02, 2.25803610e-02, 2.62696538e-02, 3.98573794e-02, 4.64955935e-02,
            3.77427003e-02, 1.10538585e+00, 3.68641212e-02, 9.50576863e-03, 3.30598773e-02,
            2.70360844e-02, 1.41120462e-02, 1.99962331e-02, 3.40992341e-02, 1.47618168e-03
        ], [
            4.51281865e-02, 6.36409322e-02, 5.87479659e-02, 6.84679195e-02, 6.58400017e-02,
            6.64772390e-02, 6.30025158e-02, 5.84275786e-02, 6.64408072e-02, 6.33769858e-02,
            6.45633661e-02, 6.78903746e-02, 6.41725336e-02, 6.14751938e-02, 6.27880973e-02,
            6.49608915e-02, 5.45856341e-02, 6.10300642e-02, 5.83897347e-02, 6.98696842e-02,
            5.98246480e-02, 5.72241650e-02, 7.59369489e-02, 8.65090957e-02, 7.86794974e-02,
            6.15565880e-02, 4.83685680e-02, 1.22693591e+00, 7.21205337e-02, 6.67343715e-02,
            4.82813499e-02, 2.97556287e-02, 4.90915527e-02, 6.48069408e-02, 8.70573057e-03
        ], [
            1.69514514e-02, 1.59778282e-02, 2.17924475e-02, 2.32363563e-02, 2.53977535e-02,
            2.19719672e-02, 2.55417530e-02, 1.61259088e-02, 2.19254423e-02, 2.32088646e-02,
            2.06715427e-02, 2.19259282e-02, 2.32409577e-02, 2.28416553e-02, 2.24996826e-02,
            2.52542968e-02, 1.69286134e-02, 2.82256918e-02, 3.72035380e-02, 3.96191267e-02,
            5.20273312e-02, 4.20139097e-02, 2.34836248e-02, 4.70975263e-02, 3.30708163e-02,
            3.73693327e-02, 2.36012001e-02, 3.77967623e-02, 1.03654448e+00, 3.66370605e-02,
            2.13003740e-02, 2.42346008e-02, 4.11912054e-02, 4.23870134e-02, 2.09544126e-03
        ], [
            4.94170352e-02, 7.34141674e-02, 9.85392623e-02, 9.63478279e-02, 9.78372441e-02,
            8.18779526e-02, 1.30435538e-01, 7.11765820e-02, 1.37195707e-01, 1.10268645e-01,
            9.15219210e-02, 9.45260259e-02, 1.15216962e-01, 1.40544994e-01, 1.21261800e-01,
            1.01392962e-01, 8.16185896e-02, 1.17011432e-01, 9.51043763e-02, 9.60421801e-02,
            9.94820741e-02, 8.40460616e-02, 8.80600418e-02, 1.23090651e-01, 1.60608811e-01,
            1.25412424e-01, 1.13824047e-01, 1.40467772e-01, 5.90838558e-02, 1.18396570e+00,
            9.67558399e-02, 5.13382655e-02, 9.51353556e-02, 1.16784365e-01, 6.60679080e-03
        ], [
            3.67253890e-03, 5.57840577e-03, 5.16198610e-03, 4.56032880e-03, 4.34256388e-03,
            6.76169017e-03, 6.60666103e-03, 4.74706935e-03, 6.92966385e-03, 5.94966941e-03,
            5.55771970e-03, 5.92778255e-03, 4.64787217e-03, 4.93544491e-03, 4.94975644e-03,
            5.81298868e-03, 8.34401198e-03, 4.49953018e-03, 4.41675221e-03, 3.45844137e-03,
            5.10562456e-03, 6.12154351e-03, 5.44513500e-03, 1.35326963e-02, 1.34991318e-02,
            6.45739574e-03, 4.70597257e-03, 3.42546981e-03, 4.42692587e-03, 4.89248843e-03,
            1.00946330e+00, 5.14001895e-03, 7.27747066e-03, 1.02340431e-02, 4.35576245e-04
        ], [
            1.56006926e-03, 1.25203508e-03, 1.58095241e-03, 1.51016457e-03, 1.60977517e-03,
            1.37239289e-03, 1.75636253e-03, 1.42293204e-03, 2.07673787e-03, 1.72253856e-03,
            1.54425939e-03, 1.62802143e-03, 2.01611147e-03, 2.30737676e-03, 1.81976615e-03,
            1.39837460e-03, 1.86131833e-03, 1.50282277e-03, 1.29174928e-03, 1.54383912e-03,
            2.43723462e-03, 1.28445766e-03, 1.92404904e-03, 1.52429237e-03, 2.80276474e-03,
            2.08135935e-03, 2.54102931e-03, 2.11225546e-03, 5.80025922e-04, 2.93199945e-03,
            5.18182826e-03, 1.00705390e+00, 1.78317576e-03, 2.28255418e-03, 1.16647262e-04
        ], [
            2.84386212e-03, 9.03636180e-04, 1.80732821e-03, 1.35733038e-03, 1.41531417e-03,
            1.45159086e-03, 1.23841650e-03, 8.92528891e-04, 1.27011380e-03, 1.22609651e-03,
            1.13799525e-03, 1.15423513e-03, 1.36108706e-03, 1.26957635e-03, 1.17661604e-03,
            1.28744542e-03, 9.70673068e-04, 1.25622284e-03, 1.12942612e-03, 1.12839746e-03,
            8.87205285e-04, 1.82215209e-03, 1.33352404e-03, 1.42867399e-03, 1.31841458e-03,
            1.42460130e-03, 1.24784330e-03, 1.11782744e-03, 3.76248386e-04, 1.32865192e-03,
            6.94865901e-03, 1.83686192e-03, 1.02337217e+00, 2.35063440e-03, 1.06346641e-04
        ], [
            9.93694492e-03, 1.46711935e-02, 1.92680423e-02, 1.90215866e-02, 2.11787796e-02,
            1.87019046e-02, 2.99936711e-02, 1.49278038e-02, 2.39026536e-02, 2.14571036e-02,
            1.85368283e-02, 2.18452105e-02, 2.13729406e-02, 2.20351257e-02, 2.01954229e-02,
            2.07366029e-02, 1.56888188e-02, 1.92910662e-02, 2.01127181e-02, 1.77035340e-02,
            1.89492747e-02, 2.44218583e-02, 1.96718043e-02, 2.18716004e-02, 2.31677938e-02,
            2.45194442e-02, 2.94984350e-02, 2.16663696e-02, 1.32271609e-02, 4.18944399e-02,
            2.53415995e-02, 1.75182513e-02, 2.50198567e-02, 1.09540360e+00, 1.71137070e-03
        ], [
            4.74615746e-05, 5.37539349e-05, 9.09681683e-05, 1.10333315e-04, 9.84966309e-05,
            9.63668086e-05, 1.00559311e-04, 6.05835635e-05, 9.65572748e-05, 1.00966204e-04,
            8.00894129e-05, 9.81431229e-05, 9.22401543e-05, 8.34860139e-05, 1.14712388e-04,
            1.00535148e-04, 5.86191133e-05, 1.47459673e-04, 1.78676789e-04, 7.97945275e-05,
            9.74895078e-05, 1.21827895e-04, 2.09251559e-04, 9.38929724e-05, 9.65045852e-05,
            9.12672177e-05, 1.31305289e-04, 1.14934948e-04, 3.00071265e-04, 1.25449383e-04,
            1.30693957e-04, 8.11277864e-05, 1.80203452e-04, 1.77801090e-04, 1.00002693e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 392)
    return ret


@cache('run')
def leontief_matrix_x1998x():
    """
    leontief matrix x1998x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17610288e+00, 8.15580396e-03, 3.71439005e-01, 7.34348506e-02, 9.46030156e-02,
            2.04951637e-01, 2.85184471e-02, 1.06335145e-02, 2.22395797e-02, 3.08540870e-02,
            1.03476559e-02, 7.89188645e-03, 8.39276860e-03, 8.17743569e-03, 9.22773982e-03,
            3.87795393e-02, 5.51588011e-03, 1.91936747e-02, 6.32414150e-03, 8.85957779e-03,
            8.40883775e-03, 1.08345701e-01, 6.77118074e-03, 1.18834931e-02, 9.77285521e-03,
            1.15349869e-02, 5.03350774e-03, 4.59739931e-03, 3.63654695e-03, 7.03211476e-03,
            1.03023863e-02, 9.16830208e-03, 1.37347173e-02, 1.13859420e-02, 2.02250296e-02
        ], [
            2.16113439e-02, 1.11040090e+00, 2.35899462e-02, 2.55476001e-02, 2.14299018e-02,
            2.74811163e-02, 2.58051465e-02, 4.84111069e-01, 8.56942499e-02, 4.03213470e-02,
            1.21823897e-01, 8.94856413e-02, 3.31349534e-02, 2.56043331e-02, 2.96537905e-02,
            3.57762146e-02, 1.34219812e-01, 5.48593740e-02, 1.50206497e-02, 1.10857860e-02,
            1.03875275e-02, 1.67263973e-02, 4.12172195e-02, 4.52222033e-02, 3.98178835e-02,
            2.30090939e-02, 9.84389233e-03, 6.35321111e-03, 9.50619699e-03, 1.02106143e-02,
            1.59179080e-02, 1.21705716e-02, 1.57894179e-02, 1.59016675e-02, 7.56532803e-02
        ], [
            7.70565013e-02, 5.11248552e-03, 1.19158081e+00, 1.56342493e-02, 1.32896439e-01,
            2.03304446e-02, 1.19126478e-02, 6.40607691e-03, 2.14617749e-02, 1.39551107e-02,
            8.78414542e-03, 7.65763806e-03, 8.27088130e-03, 8.37208136e-03, 8.07547538e-03,
            1.12088501e-02, 5.20771030e-03, 7.97476749e-03, 7.01956088e-03, 1.02246555e-02,
            8.99546633e-03, 1.95432315e-01, 7.25081062e-03, 1.69024442e-02, 1.50255539e-02,
            1.20465808e-02, 6.21209640e-03, 5.91144815e-03, 2.55046392e-03, 8.31619002e-03,
            1.48297226e-02, 1.42913024e-02, 2.30562681e-02, 1.33640351e-02, 3.33094456e-03
        ], [
            6.30386033e-03, 5.07275782e-03, 7.52838563e-03, 1.38586020e+00, 9.93475762e-02,
            1.42429026e-02, 1.68144002e-02, 4.63732325e-03, 1.17455489e-02, 3.53485358e-02,
            1.21657229e-02, 7.41077720e-03, 9.25205953e-03, 8.68236655e-03, 1.80580380e-02,
            4.42211827e-02, 3.49282772e-03, 8.94543262e-03, 6.86192894e-03, 6.18446620e-03,
            6.90326380e-03, 7.52251912e-03, 5.70115222e-03, 8.10664131e-03, 5.48598294e-03,
            5.99916547e-03, 7.05123207e-03, 2.91828503e-03, 1.77281572e-03, 4.34084470e-03,
            7.85153982e-03, 2.96807240e-03, 8.05129957e-03, 1.04028242e-02, 1.45530549e-03
        ], [
            6.63817616e-04, 7.24022744e-04, 9.38415987e-04, 1.18136744e-02, 1.23641801e+00,
            2.15224956e-03, 2.70291984e-03, 6.82291956e-04, 1.06863188e-03, 2.50073787e-03,
            1.52842312e-03, 1.32838674e-03, 1.38660211e-03, 1.25992202e-03, 3.09189592e-03,
            5.45283117e-03, 6.01963153e-04, 9.56435023e-04, 1.18999537e-03, 9.56288319e-04,
            9.69986000e-04, 7.52507428e-04, 7.23193718e-04, 8.32276564e-04, 7.32286723e-04,
            6.94149063e-04, 1.26644492e-03, 3.61030447e-04, 1.99095676e-04, 5.48364625e-04,
            1.58682406e-03, 4.27919161e-04, 6.22339422e-04, 1.38790339e-03, 1.18162181e-04
        ], [
            4.44112703e-03, 5.31146671e-03, 6.22787485e-03, 6.29660996e-03, 7.22896348e-03,
            1.24859670e+00, 2.05504246e-02, 4.17179903e-03, 6.09439964e-03, 8.00044157e-03,
            1.29205399e-02, 9.06714442e-03, 8.77641308e-03, 6.95692948e-03, 9.87411990e-03,
            9.95469164e-02, 4.51656609e-03, 4.71972335e-02, 4.27024444e-03, 4.19405361e-03,
            4.07210357e-03, 5.92463410e-03, 4.67726400e-03, 4.46526366e-03, 3.95330307e-03,
            8.86956431e-03, 3.44144946e-03, 2.84093449e-03, 6.01716170e-03, 3.72225025e-03,
            5.04799325e-03, 4.22738496e-03, 4.01291243e-03, 6.96058666e-03, 1.40536738e-03
        ], [
            1.58303223e-02, 1.51977209e-02, 5.29752805e-02, 3.57327401e-02, 4.31519931e-02,
            3.63577635e-02, 1.32023862e+00, 1.60573919e-02, 4.83883714e-02, 4.93831944e-02,
            4.84607991e-02, 2.67778336e-02, 3.35472064e-02, 4.24942120e-02, 3.14227425e-02,
            5.15467355e-02, 1.58354825e-02, 2.63671423e-02, 2.41663390e-02, 2.91297442e-02,
            2.80142766e-02, 3.10495258e-02, 2.10533358e-02, 2.27548848e-02, 2.69730026e-02,
            3.65554799e-02, 2.81330162e-02, 3.35022208e-02, 1.06004832e-02, 4.22913151e-02,
            3.34988284e-02, 2.92687021e-02, 2.65999554e-02, 4.79722380e-02, 2.90663072e-03
        ], [
            2.16024949e-02, 2.31455969e-02, 1.84529153e-02, 1.88058266e-02, 1.70970287e-02,
            1.93743603e-02, 1.60826727e-02, 1.06435518e+00, 4.59385048e-02, 2.68857920e-02,
            3.10696599e-02, 2.47469228e-02, 1.59434283e-02, 1.40355660e-02, 1.49486645e-02,
            1.82162496e-02, 3.01411371e-02, 2.58953224e-02, 1.18184528e-02, 1.15096198e-02,
            9.77546276e-03, 1.22179266e-02, 6.23986199e-02, 7.52731736e-02, 6.78087262e-02,
            2.66689213e-02, 9.21182100e-03, 5.57573838e-03, 4.82171604e-03, 9.87993159e-03,
            1.40206453e-02, 7.22213612e-03, 1.11027909e-02, 1.33595648e-02, 2.33148487e-03
        ], [
            7.18350621e-02, 3.22029341e-02, 5.38654555e-02, 1.44392152e-01, 9.43628966e-02,
            7.14620505e-02, 7.72264126e-02, 4.09287050e-02, 1.31954947e+00, 3.10213429e-01,
            6.46094056e-02, 4.77036110e-02, 4.90646260e-02, 5.82624001e-02, 5.99081954e-02,
            8.45736213e-02, 2.11181753e-02, 4.14660846e-02, 2.88229643e-02, 1.40198046e-02,
            1.25589964e-02, 2.51232953e-02, 1.97873040e-02, 1.96086035e-02, 1.79308012e-02,
            1.92775437e-02, 1.23875898e-02, 8.25663898e-03, 7.94186715e-03, 1.58388875e-02,
            2.31055399e-02, 1.18193910e-02, 9.68588753e-02, 2.51767750e-02, 4.38437513e-03
        ], [
            1.30643488e-02, 1.75901879e-02, 2.99993388e-02, 2.39459936e-02, 5.96375301e-02,
            2.40897627e-02, 2.97040880e-02, 1.38463909e-02, 3.59405791e-02, 1.14156456e+00,
            2.44969963e-02, 2.27578225e-02, 4.76353753e-02, 4.91349660e-02, 7.03389839e-02,
            6.15711502e-02, 1.04131519e-02, 3.30794552e-02, 3.29590687e-02, 1.02175803e-02,
            1.04055900e-02, 1.40879072e-02, 2.13636354e-02, 1.38104061e-02, 1.29308141e-02,
            1.56581558e-02, 1.04338915e-02, 4.71400765e-03, 4.70207975e-03, 8.32913263e-03,
            1.05625207e-02, 4.98444326e-03, 1.40201733e-02, 1.04415280e-02, 2.05146348e-03
        ], [
            5.69235663e-03, 9.94182718e-03, 1.16557875e-02, 7.55993270e-03, 7.74091090e-03,
            1.75273295e-02, 6.33728150e-03, 8.55480589e-03, 1.39816693e-02, 1.41663156e-02,
            1.12873672e+00, 2.08878377e-02, 1.54264412e-02, 2.25471746e-02, 1.86616299e-02,
            1.59198932e-02, 8.29469087e-03, 9.68763310e-02, 8.26605207e-03, 4.15447859e-03,
            3.68864547e-03, 8.44903789e-03, 5.77123244e-03, 5.24594285e-03, 4.84281806e-03,
            6.42544436e-03, 4.98965136e-03, 2.49998491e-03, 7.93728953e-03, 3.97236722e-03,
            6.12731530e-03, 4.73238839e-03, 6.15820266e-03, 5.97139610e-03, 1.27880387e-03
        ], [
            2.39433128e-02, 6.00461667e-02, 4.66366045e-02, 3.46579924e-02, 4.35001449e-02,
            7.36254307e-02, 3.92506014e-02, 4.51458888e-02, 5.75733838e-02, 7.01497816e-02,
            8.06576396e-02, 1.49046477e+00, 2.97069533e-01, 1.67388582e-01, 2.46484798e-01,
            2.11790184e-01, 3.63795506e-02, 1.62081357e-01, 6.99404608e-02, 1.95876186e-02,
            1.76270011e-02, 2.52372356e-02, 3.64649728e-02, 4.35519070e-02, 3.97197233e-02,
            3.20238474e-02, 2.32841089e-02, 1.05921468e-02, 1.66813243e-02, 1.86640475e-02,
            2.81724318e-02, 1.35398855e-02, 2.12317423e-02, 2.33640082e-02, 6.68909997e-03
        ], [
            1.39519958e-02, 2.86504738e-02, 1.44030082e-02, 1.74019431e-02, 1.58776916e-02,
            2.10419409e-02, 1.69772425e-02, 2.08558853e-02, 1.96460848e-02, 2.30827387e-02,
            2.61287692e-02, 3.48110445e-02, 1.14047873e+00, 3.08761621e-02, 5.63104061e-02,
            2.36685051e-02, 1.77031654e-02, 2.98159796e-02, 3.46722088e-02, 8.56606171e-03,
            7.24654126e-03, 8.76007932e-03, 1.52923734e-02, 1.62715561e-02, 1.33800021e-02,
            1.27021895e-02, 8.34435120e-03, 4.72389362e-03, 4.95616052e-03, 7.61023186e-03,
            1.27744532e-02, 5.81460951e-03, 8.04030732e-03, 9.19165684e-03, 2.69394459e-03
        ], [
            1.19038461e-02, 2.36495959e-02, 1.95141485e-02, 2.37565175e-02, 2.24888593e-02,
            2.73006855e-02, 3.09647243e-02, 2.13104095e-02, 3.22596757e-02, 3.82171161e-02,
            3.00596305e-02, 4.79448970e-02, 1.32830835e-01, 1.36278129e+00, 1.18288560e-01,
            5.10665376e-02, 3.12294641e-02, 5.72430534e-02, 5.43966337e-02, 2.13267740e-02,
            1.96371440e-02, 1.70002487e-02, 2.40063927e-02, 2.35467106e-02, 2.42652115e-02,
            2.42901984e-02, 5.56595650e-02, 1.42490695e-02, 9.30143831e-03, 2.89906810e-02,
            2.93816207e-02, 1.46973808e-02, 3.71488407e-02, 2.29646148e-02, 3.35919268e-03
        ], [
            1.24089900e-02, 1.66522522e-02, 1.39487498e-02, 1.44046162e-02, 1.51438763e-02,
            1.64298757e-02, 1.22208208e-02, 1.37998642e-02, 1.34910456e-02, 1.54191269e-02,
            1.61664572e-02, 2.04427265e-02, 3.21332581e-02, 1.85306430e-02, 1.40524702e+00,
            1.85336129e-02, 1.11141532e-02, 1.69089541e-02, 1.28414085e-01, 1.29349688e-02,
            1.05379839e-02, 9.32248854e-03, 5.48950995e-02, 7.00037860e-02, 7.63450237e-02,
            2.95714771e-02, 1.06588577e-02, 5.64436356e-03, 4.18151756e-03, 1.01749092e-02,
            3.07468206e-02, 5.34642107e-03, 7.58633881e-03, 1.13599734e-02, 1.80218031e-03
        ], [
            2.30747709e-03, 3.38884157e-03, 3.83772539e-03, 1.00430193e-02, 7.06683623e-03,
            1.09016016e-02, 6.93834043e-03, 3.05794455e-03, 5.13663186e-03, 6.16701874e-03,
            6.77434803e-03, 1.37602783e-02, 9.15945818e-03, 7.36049718e-03, 1.18900795e-02,
            1.04416696e+00, 3.10292126e-03, 9.41181983e-03, 5.29447270e-03, 2.96239434e-03,
            3.20971994e-03, 4.72525421e-03, 3.78702426e-03, 3.66887902e-03, 3.69914428e-03,
            3.82838821e-03, 2.70048694e-03, 2.79729256e-03, 2.67338457e-03, 4.37452478e-03,
            6.03744557e-03, 4.72236073e-03, 5.15231829e-03, 8.11034605e-03, 4.32491740e-03
        ], [
            2.09605048e-02, 3.55220666e-02, 3.08669058e-02, 4.11353576e-02, 2.86305753e-02,
            3.74683239e-02, 4.22815086e-02, 3.69461723e-02, 5.47970729e-02, 4.85257917e-02,
            5.99537909e-02, 5.70411396e-02, 3.48604942e-02, 3.06459678e-02, 3.43057698e-02,
            3.53097015e-02, 1.12031737e+00, 2.61179225e-02, 2.30879055e-02, 1.41261522e-02,
            2.15497579e-02, 3.71016184e-02, 2.78660526e-02, 1.87735404e-02, 1.79821656e-02,
            2.58659953e-02, 1.72447889e-02, 1.20456261e-02, 1.50801254e-02, 1.52011662e-02,
            2.53887931e-02, 2.60468997e-02, 2.49507960e-02, 2.72693320e-02, 3.51314815e-03
        ], [
            8.53118905e-03, 1.32563694e-02, 1.06449546e-02, 1.07319540e-02, 1.03075797e-02,
            1.29199071e-02, 1.22527927e-02, 1.21840032e-02, 1.34234428e-02, 1.25234638e-02,
            1.61761324e-02, 1.47853925e-02, 1.19450467e-02, 1.13756460e-02, 1.14475228e-02,
            1.22522576e-02, 3.52101604e-02, 1.04579173e+00, 1.09469847e-02, 9.44378949e-03,
            1.08200202e-02, 1.26966114e-02, 1.70771689e-02, 1.33980637e-02, 1.13382256e-02,
            2.44912299e-02, 1.82404526e-02, 1.16619391e-02, 4.95499647e-02, 1.15360083e-02,
            2.55046707e-02, 1.84664663e-02, 1.33193358e-02, 1.95077165e-02, 4.17269857e-03
        ], [
            9.48822556e-03, 8.78421761e-03, 1.44792230e-02, 1.37752608e-02, 1.71194194e-02,
            1.39691031e-02, 1.15416306e-02, 1.03878988e-02, 1.40408739e-02, 1.42308823e-02,
            1.46510529e-02, 1.44656603e-02, 1.31928672e-02, 1.23175479e-02, 1.39144174e-02,
            1.44688259e-02, 1.42433162e-02, 1.45991951e-02, 1.01386781e+00, 8.19582156e-03,
            7.70334143e-03, 1.08644159e-02, 2.35607169e-02, 1.01425874e-02, 9.40325670e-03,
            1.33220434e-02, 8.53316402e-03, 5.81239275e-03, 4.37767175e-03, 9.79317373e-03,
            7.92238248e-03, 5.67872691e-03, 7.51885879e-03, 1.08534695e-02, 1.01899263e-03
        ], [
            6.08727074e-02, 4.87303218e-02, 1.10035032e-01, 1.07761584e-01, 1.33158302e-01,
            1.03759133e-01, 9.24050583e-02, 7.33045943e-02, 9.46691183e-02, 9.88351572e-02,
            8.06632293e-02, 9.24138302e-02, 9.57674406e-02, 9.95778906e-02, 1.04278957e-01,
            1.03365858e-01, 4.61745733e-02, 8.22793608e-02, 6.58927639e-02, 1.04612214e+00,
            2.78522567e-02, 7.76399098e-02, 4.73709269e-02, 5.04910674e-02, 4.84974134e-02,
            4.60739568e-02, 2.74600706e-02, 1.76372465e-02, 1.41131689e-02, 2.94671147e-02,
            4.07607788e-02, 2.23258698e-02, 5.11210500e-02, 3.53397533e-02, 5.89492030e-03
        ], [
            2.92137385e-02, 2.04700774e-02, 5.23851531e-02, 5.23433846e-02, 7.04823630e-02,
            4.27386843e-02, 3.63083432e-02, 2.92816551e-02, 4.25224956e-02, 4.35557504e-02,
            3.59453596e-02, 3.65781853e-02, 3.70442384e-02, 3.54979396e-02, 4.44170213e-02,
            4.84457458e-02, 1.90969239e-02, 4.62895729e-02, 2.97694536e-02, 1.58697029e-02,
            1.01530464e+00, 4.16275643e-02, 2.69436412e-02, 2.38865089e-02, 2.27251138e-02,
            2.22302764e-02, 1.91968630e-02, 1.10247129e-02, 9.00684247e-03, 1.79028328e-02,
            1.73091219e-02, 1.16370337e-02, 2.22844614e-02, 2.27471820e-02, 3.14376707e-03
        ], [
            6.08700069e-03, 9.46313779e-03, 1.21364442e-02, 1.35194292e-02, 1.37718426e-02,
            1.34400567e-02, 1.49402785e-02, 9.39323342e-03, 1.46219030e-02, 1.52308362e-02,
            1.42957504e-02, 1.47181933e-02, 1.62753742e-02, 1.65723792e-02, 1.36937405e-02,
            1.42323805e-02, 1.10224901e-02, 1.36380436e-02, 1.29865119e-02, 1.47249468e-02,
            1.11861706e-02, 1.01683625e+00, 1.51495677e-02, 1.72639756e-02, 3.39294063e-02,
            3.25199145e-02, 1.31502969e-02, 1.71671678e-02, 6.19860297e-03, 2.03343704e-02,
            1.55600584e-02, 1.08564779e-02, 1.42806935e-02, 1.82566677e-02, 1.11750403e-03
        ], [
            2.79186244e-02, 4.06421273e-02, 4.59835304e-02, 4.44503830e-02, 4.83128951e-02,
            5.02433166e-02, 3.92077880e-02, 5.74635226e-02, 4.36245858e-02, 4.21231042e-02,
            5.77261710e-02, 4.39238917e-02, 3.42582698e-02, 2.95058684e-02, 3.70970836e-02,
            4.43809659e-02, 4.37341005e-02, 3.89652511e-02, 2.76356407e-02, 2.70487985e-02,
            1.86772982e-02, 2.63548307e-02, 1.05966925e+00, 5.60511013e-02, 3.03651345e-02,
            4.97562369e-02, 1.96535944e-02, 1.39897032e-02, 7.71931336e-03, 1.55242853e-02,
            2.19811920e-02, 1.73979901e-02, 1.90333269e-02, 2.08405990e-02, 4.59089900e-03
        ], [
            5.99162709e-03, 8.05018736e-03, 9.09929342e-03, 9.05912930e-03, 1.10995122e-02,
            1.43743231e-02, 6.24849113e-03, 1.17120553e-02, 7.84100570e-03, 7.37661837e-03,
            1.08765782e-02, 1.15612873e-02, 6.72257189e-03, 5.82774205e-03, 7.03831322e-03,
            7.55174089e-03, 5.73775977e-03, 7.86344868e-03, 3.74327875e-03, 4.15634468e-03,
            2.86419607e-03, 4.87476753e-03, 7.09065894e-03, 1.13623159e+00, 6.94511141e-03,
            8.69470293e-03, 2.75116824e-03, 1.84106102e-03, 1.19422894e-03, 2.54002827e-03,
            3.20451433e-03, 2.77784628e-03, 2.60349517e-03, 3.11471499e-03, 8.29809135e-04
        ], [
            2.72960073e-03, 4.49097136e-03, 4.93996786e-03, 5.48256878e-03, 6.96506335e-03,
            4.86004878e-03, 5.81291744e-03, 4.24805264e-03, 5.77864569e-03, 5.72651470e-03,
            4.76267305e-03, 5.12219291e-03, 5.54580205e-03, 5.94362194e-03, 6.10837675e-03,
            5.44876863e-03, 3.20701467e-03, 4.39991985e-03, 4.88976745e-03, 6.19350600e-03,
            3.66831033e-03, 3.87572903e-03, 5.44845035e-03, 1.05079589e-02, 1.04123699e+00,
            2.62642957e-02, 7.93980673e-03, 6.07286352e-03, 1.82053897e-03, 7.02122864e-03,
            6.88463973e-03, 3.54871030e-03, 3.55004213e-03, 6.82289111e-03, 4.71286614e-04
        ], [
            1.16148619e-02, 1.59059703e-02, 2.03476023e-02, 1.78124594e-02, 2.07266766e-02,
            2.19574180e-02, 2.02338195e-02, 2.14357026e-02, 1.89187111e-02, 1.86780140e-02,
            2.26227962e-02, 1.96939531e-02, 1.72984731e-02, 1.60360416e-02, 1.89120734e-02,
            1.87734657e-02, 1.40685192e-02, 1.58295105e-02, 1.72307106e-02, 2.71744848e-02,
            1.67941805e-02, 1.84275436e-02, 5.28421849e-02, 1.77904886e-01, 1.24993687e-01,
            1.14583343e+00, 1.02416604e-02, 1.00100893e-02, 4.00959558e-03, 1.18154212e-02,
            1.13513412e-02, 7.12622590e-03, 1.00810784e-02, 1.34259957e-02, 1.67912093e-03
        ], [
            1.04433129e-02, 1.57849095e-02, 1.79856295e-02, 1.94708792e-02, 2.10214810e-02,
            1.89786391e-02, 2.78125067e-02, 1.61772753e-02, 2.26528097e-02, 2.13188465e-02,
            1.92151478e-02, 2.09069065e-02, 2.24699463e-02, 2.26592058e-02, 2.03188420e-02,
            2.32169986e-02, 1.62209044e-02, 2.34195695e-02, 2.36632950e-02, 2.91268719e-02,
            2.87844162e-02, 2.35062151e-02, 2.80257139e-02, 3.86136169e-02, 4.96174920e-02,
            4.05557294e-02, 1.11827254e+00, 3.99580398e-02, 1.08756339e-02, 3.44034718e-02,
            2.89115136e-02, 1.51037503e-02, 2.14114357e-02, 3.56934804e-02, 1.90104848e-03
        ], [
            4.60770610e-02, 6.24824667e-02, 6.02950076e-02, 6.97239532e-02, 6.59223040e-02,
            6.63638187e-02, 6.47016892e-02, 5.90001546e-02, 6.67170074e-02, 6.43284555e-02,
            6.54033614e-02, 6.81253046e-02, 6.49432883e-02, 6.05393702e-02, 6.26295729e-02,
            6.88977261e-02, 5.65499789e-02, 6.21242660e-02, 5.97471334e-02, 6.89853285e-02,
            6.24712398e-02, 5.88336809e-02, 8.02571443e-02, 8.83882167e-02, 8.36053663e-02,
            6.48205461e-02, 5.07840597e-02, 1.24032611e+00, 7.97124724e-02, 6.84878350e-02,
            5.12908714e-02, 3.10904532e-02, 5.17386954e-02, 6.66916136e-02, 1.06935368e-02
        ], [
            1.68883635e-02, 1.58125228e-02, 2.17904384e-02, 2.31640871e-02, 2.52033587e-02,
            2.24179887e-02, 2.56737900e-02, 1.73254801e-02, 2.19587702e-02, 2.30871420e-02,
            2.05046410e-02, 2.21777594e-02, 2.31382974e-02, 2.22286387e-02, 2.24590389e-02,
            2.65133335e-02, 1.73539754e-02, 2.82154620e-02, 3.73889862e-02, 3.79781082e-02,
            5.26408534e-02, 4.08878963e-02, 2.40078880e-02, 4.24297869e-02, 3.38467889e-02,
            3.80576495e-02, 2.43574536e-02, 3.83739628e-02, 1.03872715e+00, 3.63708384e-02,
            2.29280928e-02, 2.47412863e-02, 4.26731931e-02, 4.26557350e-02, 2.56474556e-03
        ], [
            5.00791767e-02, 7.43168269e-02, 1.01941945e-01, 9.81710930e-02, 9.87253565e-02,
            8.70228926e-02, 1.35124520e-01, 7.60300809e-02, 1.43108976e-01, 1.14415988e-01,
            9.52235824e-02, 9.92031096e-02, 1.17765896e-01, 1.41694601e-01, 1.23804359e-01,
            1.11649507e-01, 8.67619386e-02, 1.21049243e-01, 9.91554938e-02, 9.63950787e-02,
            1.04291277e-01, 8.71986335e-02, 9.31667345e-02, 1.19044008e-01, 1.72435237e-01,
            1.32620588e-01, 1.22163191e-01, 1.50082107e-01, 6.59545995e-02, 1.19039056e+00,
            1.04224115e-01, 5.43646634e-02, 1.02312789e-01, 1.21222527e-01, 8.29113744e-03
        ], [
            3.62310438e-03, 5.38543234e-03, 5.09872284e-03, 4.38458370e-03, 4.21757767e-03,
            6.97324147e-03, 6.60507999e-03, 4.69786257e-03, 6.92068084e-03, 5.91817130e-03,
            5.54620496e-03, 6.33832070e-03, 4.61193221e-03, 4.74750527e-03, 4.89259565e-03,
            5.95198738e-03, 8.79570110e-03, 4.50532768e-03, 4.40530991e-03, 3.29780414e-03,
            5.09494106e-03, 5.99849663e-03, 5.36991446e-03, 1.18923182e-02, 1.32018548e-02,
            6.45664266e-03, 4.84512558e-03, 3.48099203e-03, 4.45035492e-03, 4.77357322e-03,
            1.00966595e+00, 5.14479613e-03, 7.32552624e-03, 1.01331927e-02, 5.20558433e-04
        ], [
            1.54416028e-03, 1.26066400e-03, 1.59337024e-03, 1.51554215e-03, 1.62396428e-03,
            1.38754024e-03, 1.77422378e-03, 1.47482057e-03, 2.08954217e-03, 1.74615887e-03,
            1.53816118e-03, 1.63906835e-03, 1.96982163e-03, 2.27385259e-03, 1.82001572e-03,
            1.48615798e-03, 1.95140598e-03, 1.50542686e-03, 1.33606363e-03, 1.50040223e-03,
            2.51346359e-03, 1.30531460e-03, 1.95754727e-03, 1.48742493e-03, 2.89781646e-03,
            2.09751418e-03, 2.42486521e-03, 2.18770744e-03, 6.27016725e-04, 2.96009067e-03,
            5.17126161e-03, 1.00689732e+00, 1.79671247e-03, 2.26834669e-03, 1.44049227e-04
        ], [
            2.70048183e-03, 9.09966724e-04, 1.77150555e-03, 1.33778456e-03, 1.74236543e-03,
            1.46133346e-03, 1.22668075e-03, 9.29163792e-04, 1.27067701e-03, 1.20295683e-03,
            1.12015174e-03, 1.17064642e-03, 1.33605666e-03, 1.22405756e-03, 1.16604203e-03,
            1.31718991e-03, 9.70416127e-04, 1.23433289e-03, 1.10729159e-03, 1.07335989e-03,
            8.70030042e-04, 1.79202995e-03, 1.33835876e-03, 1.26692975e-03, 1.31118048e-03,
            1.38779288e-03, 1.18122134e-03, 1.12715282e-03, 3.88979572e-04, 1.31366476e-03,
            6.91749839e-03, 1.80368707e-03, 1.02353856e+00, 2.28969718e-03, 1.28399836e-04
        ], [
            9.93781123e-03, 1.51402823e-02, 1.92848093e-02, 1.90544488e-02, 2.14918251e-02,
            1.89281524e-02, 3.07148868e-02, 1.60643917e-02, 2.42980927e-02, 2.13152828e-02,
            1.88129847e-02, 2.23168068e-02, 2.14535299e-02, 2.12364876e-02, 1.99264741e-02,
            2.18008556e-02, 1.63487757e-02, 1.98095348e-02, 2.03001487e-02, 1.76102584e-02,
            1.92055164e-02, 2.42875543e-02, 1.99686616e-02, 2.09425554e-02, 2.42823725e-02,
            2.50578590e-02, 3.12836673e-02, 2.24870486e-02, 1.42108696e-02, 4.15888204e-02,
            2.58811299e-02, 1.74647328e-02, 2.58475161e-02, 1.09502944e+00, 2.13939336e-03
        ], [
            5.24861716e-05, 5.66493563e-05, 9.92561096e-05, 1.14745717e-04, 1.05629029e-04,
            1.09931762e-04, 1.12072400e-04, 6.50933172e-05, 1.08469806e-04, 1.14017737e-04,
            8.81173032e-05, 1.05686449e-04, 1.02103658e-04, 8.81892415e-05, 1.19873006e-04,
            1.19868502e-04, 6.48732117e-05, 1.70632249e-04, 1.91259816e-04, 8.22296070e-05,
            1.06593076e-04, 1.32987374e-04, 2.27116244e-04, 9.88017422e-05, 1.05932713e-04,
            1.03438127e-04, 1.45725660e-04, 1.32576924e-04, 3.38133028e-04, 1.40874956e-04,
            1.49868442e-04, 8.52774376e-05, 2.11510328e-04, 1.95882925e-04, 1.00003360e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 393)
    return ret


@cache('run')
def leontief_matrix_x1999x():
    """
    leontief matrix x1999x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17714229e+00, 7.53359026e-03, 3.69508268e-01, 7.40367861e-02, 9.63099284e-02,
            2.19506266e-01, 2.87281512e-02, 9.94342628e-03, 2.10753079e-02, 2.99165162e-02,
            9.86152204e-03, 7.67112252e-03, 8.42826106e-03, 7.74064612e-03, 9.20129837e-03,
            3.70771672e-02, 5.56107316e-03, 2.12676459e-02, 6.20104617e-03, 8.65934694e-03,
            8.25433361e-03, 1.07622113e-01, 6.98278287e-03, 1.18025533e-02, 9.34458262e-03,
            1.23639110e-02, 5.31136908e-03, 4.57926352e-03, 3.69530583e-03, 6.89782932e-03,
            1.04057681e-02, 9.26027079e-03, 1.34749399e-02, 1.17555181e-02, 2.13192958e-02
        ], [
            2.26136455e-02, 1.11049499e+00, 2.42037458e-02, 2.77917955e-02, 2.30367433e-02,
            2.81675761e-02, 2.65812238e-02, 5.12177607e-01, 8.92431379e-02, 4.20117373e-02,
            1.19930683e-01, 9.18189101e-02, 3.47912693e-02, 2.51113559e-02, 3.05792933e-02,
            3.11419524e-02, 1.42760850e-01, 5.56445276e-02, 1.55084603e-02, 1.16497153e-02,
            1.10006416e-02, 1.70250566e-02, 4.26981139e-02, 5.14033227e-02, 4.54445295e-02,
            2.32244761e-02, 1.05916919e-02, 6.64893251e-03, 9.30872687e-03, 1.08926108e-02,
            1.64075736e-02, 1.33267723e-02, 1.63146331e-02, 1.59707448e-02, 7.98615534e-02
        ], [
            7.50135849e-02, 4.86269404e-03, 1.19083923e+00, 1.56872233e-02, 1.30466786e-01,
            2.03877004e-02, 1.14563202e-02, 5.90473038e-03, 2.09715605e-02, 1.35638645e-02,
            8.18711076e-03, 7.22450653e-03, 8.08284395e-03, 7.69725352e-03, 7.79453171e-03,
            1.02228676e-02, 5.28583844e-03, 7.71286781e-03, 6.68478724e-03, 9.69829833e-03,
            8.77757335e-03, 1.89368142e-01, 6.74413622e-03, 1.72150095e-02, 1.36857874e-02,
            1.11283440e-02, 6.45680910e-03, 5.72677996e-03, 2.46383163e-03, 8.07859090e-03,
            1.42186990e-02, 1.37218815e-02, 2.22352034e-02, 1.37218428e-02, 3.45398644e-03
        ], [
            5.96820245e-03, 4.69864747e-03, 7.08543090e-03, 1.39823135e+00, 1.01128494e-01,
            1.21067999e-02, 1.63686874e-02, 4.36474143e-03, 1.10042004e-02, 3.17305640e-02,
            1.13703534e-02, 6.88668997e-03, 9.05625222e-03, 8.02634069e-03, 1.80298204e-02,
            3.73900122e-02, 3.31294890e-03, 8.20135968e-03, 6.60630612e-03, 5.83050030e-03,
            6.62226332e-03, 7.15844765e-03, 5.39982038e-03, 7.35268627e-03, 5.26511705e-03,
            5.67019541e-03, 6.52965675e-03, 2.74412226e-03, 1.67700054e-03, 4.19309906e-03,
            8.06014821e-03, 2.97255984e-03, 7.62598551e-03, 1.09980098e-02, 1.47671174e-03
        ], [
            5.79504111e-04, 5.87049905e-04, 8.19583698e-04, 1.24145731e-02, 1.23464927e+00,
            1.68422739e-03, 2.55158422e-03, 5.57760989e-04, 9.45451773e-04, 2.29142170e-03,
            1.31393253e-03, 1.10777049e-03, 1.21220299e-03, 1.05338168e-03, 3.00863150e-03,
            5.12280899e-03, 5.30942902e-04, 8.27513252e-04, 1.12504900e-03, 8.37182111e-04,
            8.46533327e-04, 6.61954933e-04, 6.67961403e-04, 7.68195196e-04, 6.69684334e-04,
            6.31832122e-04, 1.10840251e-03, 3.26456358e-04, 1.78856786e-04, 5.01953832e-04,
            1.49632400e-03, 4.01756143e-04, 5.57787410e-04, 1.27897496e-03, 1.10910720e-04
        ], [
            4.53845398e-03, 4.98075418e-03, 6.13927150e-03, 6.02691198e-03, 6.91933585e-03,
            1.24930112e+00, 2.05189261e-02, 3.88187001e-03, 5.87105658e-03, 8.12503059e-03,
            1.21385541e-02, 8.88868957e-03, 8.70521997e-03, 6.54379961e-03, 9.84187282e-03,
            9.49594470e-02, 4.45639199e-03, 4.66956654e-02, 4.13787587e-03, 4.03039811e-03,
            3.90245101e-03, 5.80799203e-03, 4.55238243e-03, 4.27636817e-03, 3.80120322e-03,
            8.48746434e-03, 3.55391413e-03, 2.72945945e-03, 5.84110077e-03, 3.65010998e-03,
            5.00199757e-03, 4.62069180e-03, 3.86552915e-03, 6.92632285e-03, 1.43990901e-03
        ], [
            1.54107765e-02, 1.44514822e-02, 5.25396084e-02, 3.69082906e-02, 4.40175328e-02,
            3.44882612e-02, 1.32153253e+00, 1.55726836e-02, 4.73640716e-02, 4.82360024e-02,
            4.71904349e-02, 2.58849796e-02, 3.36590949e-02, 4.09742337e-02, 3.09667252e-02,
            4.52651363e-02, 1.62759115e-02, 2.53506107e-02, 2.39431853e-02, 2.90504867e-02,
            2.78706379e-02, 2.99600060e-02, 2.06316739e-02, 2.18030157e-02, 2.59556770e-02,
            3.53544188e-02, 2.87978133e-02, 3.36025790e-02, 1.04215345e-02, 4.26990614e-02,
            3.37849572e-02, 3.09384674e-02, 2.61515411e-02, 4.73045413e-02, 2.97724243e-03
        ], [
            2.23907373e-02, 2.22968526e-02, 1.89124386e-02, 2.06274636e-02, 1.85502749e-02,
            1.95830543e-02, 1.63491174e-02, 1.06751933e+00, 4.65512658e-02, 2.69242922e-02,
            3.17731230e-02, 2.44542202e-02, 1.62941405e-02, 1.34671936e-02, 1.51517713e-02,
            1.62648590e-02, 3.06198491e-02, 2.57634374e-02, 1.19507426e-02, 1.17226899e-02,
            9.88761842e-03, 1.24887079e-02, 6.38614896e-02, 8.42486487e-02, 7.65091746e-02,
            2.74830396e-02, 9.65866130e-03, 5.78667694e-03, 4.78870224e-03, 1.02289750e-02,
            1.45267115e-02, 7.42618566e-03, 1.10707024e-02, 1.32000046e-02, 2.41067800e-03
        ], [
            6.99233073e-02, 2.96586548e-02, 5.26117079e-02, 1.47121324e-01, 9.77831650e-02,
            7.00132456e-02, 7.65807668e-02, 3.85089219e-02, 1.31776782e+00, 3.11307834e-01,
            6.38653845e-02, 4.61178617e-02, 4.97044523e-02, 5.64438591e-02, 6.05306745e-02,
            7.24640446e-02, 2.11919939e-02, 4.04924068e-02, 2.87127599e-02, 1.38529163e-02,
            1.25301485e-02, 2.45056794e-02, 1.92002187e-02, 2.01181200e-02, 1.82847412e-02,
            1.88854032e-02, 1.27020997e-02, 8.35899048e-03, 7.71119253e-03, 1.61872226e-02,
            2.29294007e-02, 1.19273469e-02, 9.68201366e-02, 2.56288038e-02, 4.35952167e-03
        ], [
            1.30971660e-02, 1.68568478e-02, 3.09767756e-02, 2.54541191e-02, 6.39461668e-02,
            2.28863214e-02, 3.00331221e-02, 1.37105877e-02, 3.65071615e-02, 1.14526125e+00,
            2.38850550e-02, 2.20749867e-02, 4.88692235e-02, 4.76995145e-02, 7.19165162e-02,
            5.34085200e-02, 1.06424094e-02, 3.26805396e-02, 3.33866858e-02, 1.03818541e-02,
            1.05969538e-02, 1.40972513e-02, 2.06651598e-02, 1.40968700e-02, 1.32664575e-02,
            1.52942005e-02, 1.05054113e-02, 4.78344653e-03, 4.73875673e-03, 8.56254340e-03,
            1.07001226e-02, 5.11377808e-03, 1.40256239e-02, 1.05585090e-02, 2.08144031e-03
        ], [
            5.81081154e-03, 9.58347734e-03, 1.20448235e-02, 7.98305649e-03, 8.13991512e-03,
            1.68684658e-02, 6.33056182e-03, 8.69003382e-03, 1.41152817e-02, 1.44727854e-02,
            1.13193927e+00, 2.08830664e-02, 1.61222811e-02, 2.19443857e-02, 1.91339702e-02,
            1.42634471e-02, 8.40525198e-03, 9.63830218e-02, 8.43869915e-03, 4.09655806e-03,
            3.76189078e-03, 8.44291055e-03, 5.78296587e-03, 5.34859126e-03, 4.89851264e-03,
            6.38928372e-03, 5.25794942e-03, 2.57637505e-03, 7.93634184e-03, 4.11331749e-03,
            6.11447043e-03, 5.05369825e-03, 6.22341579e-03, 6.06661152e-03, 1.31924287e-03
        ], [
            2.28681020e-02, 5.76990158e-02, 4.42848146e-02, 3.45912319e-02, 4.36420219e-02,
            6.79925995e-02, 3.76237840e-02, 4.47348215e-02, 5.57431740e-02, 6.91997131e-02,
            7.79043666e-02, 1.47666759e+00, 2.98334067e-01, 1.57260497e-01, 2.43914643e-01,
            1.68802437e-01, 3.55683513e-02, 1.60270739e-01, 6.93730825e-02, 1.88186626e-02,
            1.71660517e-02, 2.41496120e-02, 3.51702476e-02, 4.17382177e-02, 3.76367032e-02,
            3.03213568e-02, 2.38972871e-02, 1.04936285e-02, 1.61329934e-02, 1.88225562e-02,
            2.72110492e-02, 1.34697439e-02, 2.03662032e-02, 2.29017963e-02, 6.67388425e-03
        ], [
            1.32170392e-02, 2.58500480e-02, 1.40784677e-02, 1.81597695e-02, 1.63352341e-02,
            1.96098971e-02, 1.61858200e-02, 2.02136764e-02, 1.89580845e-02, 2.33043476e-02,
            2.64270803e-02, 3.39494372e-02, 1.14168077e+00, 2.97388689e-02, 5.79733663e-02,
            2.07051478e-02, 1.67584232e-02, 2.96834774e-02, 3.37652232e-02, 8.33722122e-03,
            6.97804211e-03, 8.43695740e-03, 1.43377950e-02, 1.70262405e-02, 1.30896070e-02,
            1.16799658e-02, 8.37606298e-03, 4.44223075e-03, 4.67469264e-03, 7.65020085e-03,
            1.22631449e-02, 5.66356742e-03, 7.90709114e-03, 9.06859283e-03, 2.61490946e-03
        ], [
            1.26996420e-02, 2.36866178e-02, 2.08549270e-02, 2.67987145e-02, 2.52195817e-02,
            2.80416310e-02, 3.32718564e-02, 2.26344032e-02, 3.45542634e-02, 4.18177089e-02,
            3.24378954e-02, 5.06172609e-02, 1.48072540e-01, 1.39275828e+00, 1.27738880e-01,
            4.89242588e-02, 3.38118589e-02, 6.03536178e-02, 5.89985023e-02, 2.28263558e-02,
            2.14889117e-02, 1.82470570e-02, 2.55217526e-02, 2.55953942e-02, 2.60907837e-02,
            2.56445975e-02, 6.62318633e-02, 1.60447484e-02, 9.96589912e-03, 3.20918452e-02,
            3.10724247e-02, 1.57583712e-02, 3.90340815e-02, 2.57042571e-02, 3.61169591e-03
        ], [
            1.22151617e-02, 1.67001201e-02, 1.41549901e-02, 1.45683388e-02, 1.58907371e-02,
            1.60072872e-02, 1.24455362e-02, 1.39557176e-02, 1.35577282e-02, 1.58540898e-02,
            1.60868923e-02, 1.99163917e-02, 3.21637344e-02, 1.80924557e-02, 1.41281265e+00,
            1.67023065e-02, 1.12204121e-02, 1.65222653e-02, 1.34047487e-01, 1.31968748e-02,
            1.08624841e-02, 9.25987011e-03, 5.53777387e-02, 7.18711579e-02, 7.71732571e-02,
            2.70668553e-02, 1.11802103e-02, 5.97016303e-03, 4.31845247e-03, 1.09683697e-02,
            2.97254010e-02, 5.70947189e-03, 7.57783338e-03, 1.14991980e-02, 1.89738693e-03
        ], [
            2.27824022e-03, 3.33767710e-03, 3.83641962e-03, 9.79075198e-03, 7.07514140e-03,
            9.92323809e-03, 6.97872194e-03, 3.01701451e-03, 4.96231905e-03, 6.14765125e-03,
            6.46848681e-03, 1.40856148e-02, 9.32208018e-03, 7.18436902e-03, 1.19178267e-02,
            1.04320761e+00, 3.17661042e-03, 9.22928941e-03, 5.31346005e-03, 2.85689977e-03,
            3.09823182e-03, 4.67715779e-03, 3.81655632e-03, 3.53036888e-03, 3.67334106e-03,
            3.80686958e-03, 2.78520136e-03, 2.70050902e-03, 2.89311106e-03, 3.99347321e-03,
            5.91724398e-03, 4.69855535e-03, 4.82823167e-03, 7.77144530e-03, 4.55475157e-03
        ], [
            2.14266385e-02, 3.72775643e-02, 3.10131074e-02, 4.20385651e-02, 2.94072247e-02,
            3.69495265e-02, 4.31168979e-02, 3.83886321e-02, 5.42788053e-02, 4.91599487e-02,
            5.89817473e-02, 5.61068256e-02, 3.54496103e-02, 3.01278007e-02, 3.41620324e-02,
            3.04719832e-02, 1.10590845e+00, 2.63078671e-02, 2.30730913e-02, 1.44207037e-02,
            2.26930170e-02, 3.82674151e-02, 2.74911722e-02, 1.92486205e-02, 1.82966853e-02,
            2.59664180e-02, 1.82456770e-02, 1.26298746e-02, 1.64755914e-02, 1.61147916e-02,
            2.62779445e-02, 2.62744608e-02, 2.51279235e-02, 2.71458830e-02, 3.82249768e-03
        ], [
            8.69707969e-03, 1.35027230e-02, 1.05305197e-02, 1.10557487e-02, 1.04572175e-02,
            1.21892708e-02, 1.20202009e-02, 1.20361522e-02, 1.30957284e-02, 1.23258462e-02,
            1.64595353e-02, 1.44485874e-02, 1.24032772e-02, 1.11867150e-02, 1.15269662e-02,
            1.06213474e-02, 3.49720588e-02, 1.04716761e+00, 1.08164282e-02, 9.44238344e-03,
            1.07872590e-02, 1.27228307e-02, 1.66655310e-02, 1.29572289e-02, 1.09305329e-02,
            2.40540813e-02, 1.83375620e-02, 1.17979565e-02, 4.87535234e-02, 1.14420453e-02,
            2.55273149e-02, 1.98052797e-02, 1.32319139e-02, 1.88294865e-02, 4.41997387e-03
        ], [
            9.37158570e-03, 8.74047759e-03, 1.44880060e-02, 1.40075505e-02, 1.70928478e-02,
            1.36944532e-02, 1.16992217e-02, 1.00175507e-02, 1.39295664e-02, 1.42918463e-02,
            1.47001908e-02, 1.42197521e-02, 1.34213643e-02, 1.18828137e-02, 1.40935707e-02,
            1.33531298e-02, 1.44573291e-02, 1.44191276e-02, 1.01415468e+00, 8.08647281e-03,
            7.67680217e-03, 1.06921058e-02, 2.36726765e-02, 1.00493106e-02, 9.27437659e-03,
            1.34346893e-02, 8.86958844e-03, 6.12429657e-03, 4.44310654e-03, 9.98001678e-03,
            8.32101510e-03, 5.79077765e-03, 7.60132192e-03, 1.10729474e-02, 1.06672636e-03
        ], [
            5.98175822e-02, 4.97607046e-02, 1.11417047e-01, 1.12832497e-01, 1.36082637e-01,
            1.01686085e-01, 9.24112855e-02, 7.27604296e-02, 9.40017589e-02, 9.90922588e-02,
            8.08870446e-02, 9.35412380e-02, 9.88408686e-02, 9.67274957e-02, 1.06913756e-01,
            9.16571608e-02, 4.61574291e-02, 8.20532299e-02, 6.64072777e-02, 1.04513021e+00,
            2.76261783e-02, 7.77808123e-02, 4.68302167e-02, 5.14402772e-02, 4.97185531e-02,
            4.54677631e-02, 2.89800499e-02, 1.77595697e-02, 1.41059274e-02, 3.00061030e-02,
            3.93561968e-02, 2.29064259e-02, 5.03908692e-02, 3.63481335e-02, 6.23281125e-03
        ], [
            2.88584606e-02, 2.04482718e-02, 5.28925104e-02, 5.32577504e-02, 7.00506415e-02,
            4.23165177e-02, 3.64456948e-02, 2.69360756e-02, 4.20941341e-02, 4.29874438e-02,
            3.57967438e-02, 3.62306048e-02, 3.76743664e-02, 3.43875267e-02, 4.53282367e-02,
            4.44472315e-02, 1.89132930e-02, 4.56934875e-02, 2.96813600e-02, 1.53149947e-02,
            1.01518353e+00, 4.11013051e-02, 2.64943830e-02, 2.34862524e-02, 2.25137047e-02,
            2.16490639e-02, 1.89034847e-02, 1.11575303e-02, 8.74294846e-03, 1.81955320e-02,
            1.74045097e-02, 1.18075525e-02, 2.20490588e-02, 2.24542338e-02, 3.29197924e-03
        ], [
            6.05144049e-03, 9.42776773e-03, 1.20445243e-02, 1.37400713e-02, 1.40125410e-02,
            1.28739741e-02, 1.49487301e-02, 9.40084033e-03, 1.43951828e-02, 1.52037353e-02,
            1.38603400e-02, 1.41885257e-02, 1.63821565e-02, 1.55705075e-02, 1.34408234e-02,
            1.24349226e-02, 1.25649635e-02, 1.32645138e-02, 1.26067937e-02, 1.47527370e-02,
            1.13135413e-02, 1.01692579e+00, 1.51969290e-02, 1.61955720e-02, 3.16115652e-02,
            2.99596166e-02, 1.48964380e-02, 1.72636234e-02, 6.11346721e-03, 2.09740442e-02,
            1.64808802e-02, 1.07232498e-02, 1.41184310e-02, 1.87214697e-02, 1.16796179e-03
        ], [
            2.81332218e-02, 3.81070074e-02, 4.71018433e-02, 4.61549333e-02, 4.95996666e-02,
            4.91944798e-02, 3.95273305e-02, 5.47106992e-02, 4.28551974e-02, 4.19177057e-02,
            5.67015996e-02, 4.27774345e-02, 3.50118941e-02, 2.87510739e-02, 3.76865718e-02,
            3.92885672e-02, 4.33696671e-02, 3.86650373e-02, 2.70458843e-02, 2.64273983e-02,
            1.89070874e-02, 2.65074899e-02, 1.06146212e+00, 6.64418899e-02, 3.23016306e-02,
            5.16385403e-02, 1.97652396e-02, 1.37694284e-02, 7.58780810e-03, 1.55987136e-02,
            2.18845928e-02, 1.82660667e-02, 1.89100830e-02, 2.07884996e-02, 4.64985132e-03
        ], [
            6.27625196e-03, 8.89996575e-03, 9.88434894e-03, 9.19539009e-03, 1.13376616e-02,
            1.39443167e-02, 7.17347528e-03, 1.26300957e-02, 8.31043400e-03, 8.12335196e-03,
            1.20273405e-02, 1.24350322e-02, 7.82063118e-03, 6.37780276e-03, 7.60887747e-03,
            6.89311466e-03, 6.33291413e-03, 8.79772005e-03, 3.88049334e-03, 4.45364695e-03,
            3.11869811e-03, 5.24688958e-03, 6.32976090e-03, 1.12487772e+00, 7.21440482e-03,
            8.91080383e-03, 2.76556915e-03, 1.82702828e-03, 1.30482153e-03, 2.67888309e-03,
            3.41076236e-03, 2.87445045e-03, 2.65996631e-03, 3.32314564e-03, 9.43287379e-04
        ], [
            2.60845608e-03, 4.23160867e-03, 4.83118683e-03, 5.69341352e-03, 7.15962015e-03,
            4.78667436e-03, 5.83922727e-03, 4.11835579e-03, 5.73484558e-03, 5.79235720e-03,
            4.79600463e-03, 5.14118742e-03, 5.93323809e-03, 5.94173585e-03, 6.18571736e-03,
            4.67580369e-03, 3.47384520e-03, 4.35068963e-03, 4.75061186e-03, 6.18508066e-03,
            3.73211106e-03, 3.79525780e-03, 5.22637970e-03, 1.01561814e-02, 1.04095254e+00,
            2.53000923e-02, 7.56188593e-03, 6.16192001e-03, 1.84123907e-03, 7.26296792e-03,
            7.05640508e-03, 3.66929593e-03, 3.47623480e-03, 7.14231478e-03, 4.74077975e-04
        ], [
            1.17388577e-02, 1.52601358e-02, 2.06432301e-02, 1.81013697e-02, 2.10495742e-02,
            2.12685946e-02, 2.07365761e-02, 2.05420770e-02, 1.87516811e-02, 1.87434974e-02,
            2.24594380e-02, 1.93859078e-02, 1.78422546e-02, 1.55771424e-02, 1.93897561e-02,
            1.67505046e-02, 1.36085727e-02, 1.62329319e-02, 1.74707864e-02, 2.69720634e-02,
            1.72597263e-02, 1.81370679e-02, 5.28538745e-02, 1.73723438e-01, 1.28782080e-01,
            1.15164715e+00, 1.02302112e-02, 1.02933752e-02, 4.19750110e-03, 1.21097541e-02,
            1.20230873e-02, 7.33781409e-03, 9.98630736e-03, 1.38030078e-02, 1.72362900e-03
        ], [
            1.13290274e-02, 1.70504846e-02, 1.96847421e-02, 2.20883235e-02, 2.38617878e-02,
            2.02147301e-02, 3.07045003e-02, 1.75247956e-02, 2.49993250e-02, 2.35541008e-02,
            2.11899645e-02, 2.27691195e-02, 2.55059873e-02, 2.49240391e-02, 2.25705861e-02,
            2.22103416e-02, 1.81621586e-02, 2.60445080e-02, 2.55230228e-02, 3.18057997e-02,
            3.14674078e-02, 2.53439258e-02, 2.95695852e-02, 3.96143997e-02, 5.01303752e-02,
            4.34260603e-02, 1.13659951e+00, 4.45788851e-02, 1.19772988e-02, 3.80100602e-02,
            3.21087070e-02, 1.62832807e-02, 2.31192790e-02, 3.87017953e-02, 2.15546895e-03
        ], [
            4.89352041e-02, 6.47386763e-02, 6.35097565e-02, 7.34904911e-02, 6.98201201e-02,
            6.62101230e-02, 6.80428609e-02, 6.00379756e-02, 6.85890007e-02, 6.69515544e-02,
            6.79586008e-02, 6.87830391e-02, 6.85759542e-02, 6.12091599e-02, 6.54577866e-02,
            6.51766340e-02, 6.19942135e-02, 6.47548183e-02, 6.08299087e-02, 7.15853649e-02,
            6.64028233e-02, 6.18275998e-02, 8.51603189e-02, 9.39892835e-02, 8.47483291e-02,
            6.78526079e-02, 5.50652155e-02, 1.25639870e+00, 8.38720633e-02, 7.39604314e-02,
            5.47783111e-02, 3.33819771e-02, 5.40330891e-02, 7.28242389e-02, 1.15818727e-02
        ], [
            1.51597354e-02, 1.51977014e-02, 2.11710363e-02, 2.28101103e-02, 2.44683782e-02,
            2.15345942e-02, 2.52303685e-02, 1.63653814e-02, 2.14741644e-02, 2.28115136e-02,
            1.99047054e-02, 2.16029050e-02, 2.31640461e-02, 2.11589290e-02, 2.22778299e-02,
            2.32814390e-02, 1.74220602e-02, 2.70172582e-02, 3.61805032e-02, 3.69677492e-02,
            5.16000227e-02, 3.97255778e-02, 2.34715082e-02, 3.98176034e-02, 3.13466380e-02,
            3.73733508e-02, 2.53402279e-02, 3.88822550e-02, 1.03922110e+00, 3.62576730e-02,
            2.44344632e-02, 2.50086012e-02, 4.39632481e-02, 4.07366987e-02, 2.61394904e-03
        ], [
            5.02684929e-02, 7.32381448e-02, 1.03194927e-01, 1.02590416e-01, 1.02514175e-01,
            8.71712266e-02, 1.35769746e-01, 7.58255872e-02, 1.45705940e-01, 1.16840333e-01,
            9.65042458e-02, 9.97026943e-02, 1.21838767e-01, 1.40445661e-01, 1.25719042e-01,
            9.84522956e-02, 8.97602832e-02, 1.18935884e-01, 1.00741474e-01, 9.70675580e-02,
            1.05923562e-01, 8.87285628e-02, 9.38971758e-02, 1.14611734e-01, 1.67015587e-01,
            1.30829744e-01, 1.26108417e-01, 1.57083920e-01, 6.56319381e-02, 1.19505013e+00,
            1.09272786e-01, 5.56463244e-02, 1.03289202e-01, 1.25805528e-01, 8.65820327e-03
        ], [
            3.54106070e-03, 5.32658101e-03, 5.03766581e-03, 4.48452210e-03, 4.32078111e-03,
            6.79944597e-03, 6.55794132e-03, 4.87079211e-03, 6.76284442e-03, 5.92653093e-03,
            5.45331531e-03, 6.38090350e-03, 4.74136327e-03, 4.65741109e-03, 4.91759848e-03,
            5.13709399e-03, 9.04054270e-03, 4.49918215e-03, 4.51410695e-03, 3.25082193e-03,
            5.08071773e-03, 6.00014164e-03, 5.29539650e-03, 1.08620047e-02, 1.26321729e-02,
            6.41038409e-03, 5.08568628e-03, 3.64783360e-03, 4.58687975e-03, 4.99392501e-03,
            1.00935999e+00, 5.37715801e-03, 7.36996560e-03, 1.03339841e-02, 5.41221146e-04
        ], [
            1.47944698e-03, 1.29864566e-03, 1.64319710e-03, 1.62385475e-03, 1.76294382e-03,
            1.41164937e-03, 1.87865536e-03, 1.51740733e-03, 2.22351962e-03, 1.86588116e-03,
            1.61708034e-03, 1.68478933e-03, 2.11009799e-03, 2.33313418e-03, 1.93398606e-03,
            1.42630164e-03, 1.80952003e-03, 1.52220366e-03, 1.39216621e-03, 1.50262826e-03,
            2.44233228e-03, 1.37038038e-03, 2.02239823e-03, 1.51677256e-03, 2.94804644e-03,
            2.15399329e-03, 2.53228588e-03, 2.22229265e-03, 6.57233374e-04, 3.13917956e-03,
            5.20411092e-03, 1.00713151e+00, 1.89774036e-03, 2.42569793e-03, 1.54098078e-04
        ], [
            3.10625031e-03, 9.87354554e-04, 1.95565472e-03, 1.40111562e-03, 2.07342262e-03,
            1.54988846e-03, 1.24727891e-03, 9.44950759e-04, 1.30641456e-03, 1.22055700e-03,
            1.13151735e-03, 1.17531841e-03, 1.36260622e-03, 1.18944460e-03, 1.17985137e-03,
            1.24432959e-03, 9.32050353e-04, 1.25787726e-03, 1.06923459e-03, 1.07846924e-03,
            8.68516645e-04, 1.87853362e-03, 1.34621313e-03, 1.20289979e-03, 1.27265734e-03,
            1.35441479e-03, 1.20916228e-03, 1.10477101e-03, 3.83813109e-04, 1.33427805e-03,
            6.43414917e-03, 1.83963283e-03, 1.02390493e+00, 2.19779972e-03, 1.47813639e-04
        ], [
            9.84007538e-03, 1.50178298e-02, 1.90532006e-02, 1.97247784e-02, 2.21537642e-02,
            1.80061348e-02, 3.03182138e-02, 1.56432374e-02, 2.39555276e-02, 2.09598554e-02,
            1.85317195e-02, 2.14878018e-02, 2.14750244e-02, 2.01708544e-02, 1.97973789e-02,
            1.90396890e-02, 1.63290266e-02, 1.92082906e-02, 1.99341112e-02, 1.76626277e-02,
            1.90527495e-02, 2.41336364e-02, 1.93867769e-02, 2.00715542e-02, 2.33405921e-02,
            2.45895508e-02, 3.19266986e-02, 2.30824880e-02, 1.37054325e-02, 4.13665153e-02,
            2.61831224e-02, 1.71246868e-02, 2.56287428e-02, 1.09674250e+00, 2.23515158e-03
        ], [
            4.96034370e-05, 5.67035561e-05, 8.82714140e-05, 1.16518615e-04, 9.84335417e-05,
            9.32906428e-05, 1.01435484e-04, 6.25010945e-05, 9.68207649e-05, 1.03233533e-04,
            7.80715174e-05, 8.86568090e-05, 9.33352776e-05, 8.23259364e-05, 1.20141787e-04,
            8.35753750e-05, 6.19002465e-05, 1.39427913e-04, 1.57776609e-04, 6.69176356e-05,
            1.01670296e-04, 1.13410260e-04, 1.83976653e-04, 8.62207502e-05, 9.56493183e-05,
            8.99906946e-05, 1.23440560e-04, 1.26852812e-04, 3.24179480e-04, 1.30448814e-04,
            1.33176976e-04, 7.58603296e-05, 1.90681825e-04, 1.89017683e-04, 1.00003504e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 394)
    return ret


@cache('run')
def leontief_matrix_x2000x():
    """
    leontief matrix x2000x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17557246e+00, 6.02766542e-03, 3.60851853e-01, 7.11276783e-02, 9.77218124e-02,
            1.95749199e-01, 2.71552215e-02, 8.21416733e-03, 1.81265028e-02, 2.78203334e-02,
            8.88535993e-03, 7.09810986e-03, 7.59843642e-03, 6.84687835e-03, 8.48959009e-03,
            3.53615683e-02, 5.18248592e-03, 2.08856845e-02, 5.81357865e-03, 8.62341007e-03,
            7.58130127e-03, 1.02120942e-01, 6.61982658e-03, 1.15551032e-02, 8.32112824e-03,
            1.20438054e-02, 4.94042578e-03, 4.10081408e-03, 3.32518556e-03, 6.23844967e-03,
            1.00165229e-02, 8.72673770e-03, 1.27506795e-02, 1.14279885e-02, 1.92144286e-02
        ], [
            2.44486476e-02, 1.11904765e+00, 2.60637113e-02, 2.96537470e-02, 2.45236916e-02,
            3.19472187e-02, 2.79688034e-02, 5.38574025e-01, 8.79340087e-02, 4.36147870e-02,
            1.24100089e-01, 1.00017030e-01, 3.68916980e-02, 2.52624745e-02, 3.22660922e-02,
            3.47663253e-02, 1.62904819e-01, 5.88962193e-02, 1.61257418e-02, 1.16720816e-02,
            1.15300848e-02, 1.84119587e-02, 4.47906963e-02, 5.73584987e-02, 4.88731484e-02,
            2.42933981e-02, 1.12836470e-02, 6.97947723e-03, 9.60291156e-03, 1.15402608e-02,
            1.72862850e-02, 1.48804780e-02, 1.67386638e-02, 1.71568773e-02, 7.24939234e-02
        ], [
            7.68158023e-02, 4.41120574e-03, 1.19372836e+00, 1.58400576e-02, 1.33772807e-01,
            1.95806777e-02, 1.14618897e-02, 5.26252949e-03, 1.95842204e-02, 1.34885970e-02,
            7.79096027e-03, 6.97778224e-03, 7.63078404e-03, 7.17829174e-03, 7.78335322e-03,
            1.07902732e-02, 5.16906365e-03, 7.48999462e-03, 6.56951748e-03, 9.49333204e-03,
            8.42071986e-03, 1.84998061e-01, 6.38984014e-03, 1.81442472e-02, 1.21239227e-02,
            1.03190525e-02, 6.44437351e-03, 5.34821520e-03, 2.36087904e-03, 7.63064707e-03,
            1.46339614e-02, 1.32655911e-02, 2.21430229e-02, 1.40721472e-02, 4.81377800e-03
        ], [
            5.69178389e-03, 4.01296624e-03, 6.72574980e-03, 1.41056731e+00, 9.87694376e-02,
            1.15041591e-02, 1.39589237e-02, 3.79491843e-03, 9.98516376e-03, 2.79201949e-02,
            1.04118339e-02, 6.54648159e-03, 8.54356230e-03, 7.33786114e-03, 1.73101437e-02,
            4.03566395e-02, 2.99538299e-03, 7.87988778e-03, 6.07164337e-03, 5.36074302e-03,
            6.01438762e-03, 6.95837494e-03, 5.07504924e-03, 6.62802252e-03, 4.87897529e-03,
            5.23640930e-03, 3.79260301e-03, 2.40873123e-03, 1.54460737e-03, 3.83220989e-03,
            7.44926939e-03, 2.87996728e-03, 7.35896025e-03, 1.07475626e-02, 9.33365873e-04
        ], [
            5.13770009e-04, 4.52609289e-04, 7.21760505e-04, 1.12311473e-02, 1.23911337e+00,
            1.48915399e-03, 1.77184317e-03, 4.51795259e-04, 8.11767148e-04, 2.22464470e-03,
            1.11312791e-03, 9.62495980e-04, 1.05960675e-03, 9.02295779e-04, 3.12367906e-03,
            5.41583948e-03, 4.59653306e-04, 7.49819860e-04, 1.08408057e-03, 7.98307294e-04,
            6.94302063e-04, 5.69021480e-04, 6.27372678e-04, 6.98612761e-04, 6.13945634e-04,
            5.93762538e-04, 7.21981933e-04, 2.62521137e-04, 1.50964170e-04, 4.22517039e-04,
            1.22252879e-03, 3.56313537e-04, 5.10512680e-04, 1.01232479e-03, 8.02873079e-05
        ], [
            4.76573133e-03, 4.59040082e-03, 6.42248540e-03, 6.16809201e-03, 7.12331952e-03,
            1.26445104e+00, 1.98437812e-02, 3.64598179e-03, 5.71851200e-03, 8.21106792e-03,
            1.20837365e-02, 9.16099925e-03, 8.50880053e-03, 6.17273194e-03, 9.57370456e-03,
            1.01390690e-01, 4.37090311e-03, 4.65680987e-02, 4.07874637e-03, 4.02969111e-03,
            3.80283832e-03, 6.00894043e-03, 4.42734918e-03, 4.13683758e-03, 3.63231150e-03,
            7.73334593e-03, 3.54532469e-03, 2.57166063e-03, 5.68758903e-03, 3.60593877e-03,
            4.97863016e-03, 4.78993817e-03, 3.81456520e-03, 7.07699316e-03, 9.56802660e-04
        ], [
            1.51568484e-02, 1.36520700e-02, 5.34359329e-02, 3.65393198e-02, 4.33823018e-02,
            3.63312264e-02, 1.31826908e+00, 1.37783015e-02, 4.53323843e-02, 4.72271448e-02,
            4.56585454e-02, 2.50718402e-02, 3.19152180e-02, 3.78284019e-02, 2.93535377e-02,
            4.89996819e-02, 1.61868152e-02, 2.49983547e-02, 2.35469113e-02, 2.80623899e-02,
            2.70317934e-02, 2.99068496e-02, 2.01092911e-02, 2.04161122e-02, 2.45000442e-02,
            3.31548359e-02, 2.85587070e-02, 3.25324701e-02, 9.99263933e-03, 4.27739876e-02,
            3.30398200e-02, 3.04102152e-02, 2.51889212e-02, 4.73170030e-02, 2.23667799e-03
        ], [
            2.24627373e-02, 1.99897892e-02, 1.86174260e-02, 2.11300393e-02, 1.89080298e-02,
            2.03232182e-02, 1.62711715e-02, 1.05951780e+00, 4.88139097e-02, 2.73194910e-02,
            3.11263139e-02, 2.32984923e-02, 1.56292837e-02, 1.25609767e-02, 1.47214134e-02,
            1.67720060e-02, 2.96506608e-02, 2.48396015e-02, 1.15338471e-02, 1.16486311e-02,
            9.38803358e-03, 1.23338014e-02, 6.25732774e-02, 9.03118927e-02, 7.90125436e-02,
            2.76902071e-02, 9.11889477e-03, 5.60014540e-03, 4.34714834e-03, 1.00110224e-02,
            1.40695868e-02, 7.19064479e-03, 1.08047609e-02, 1.24566005e-02, 1.96222197e-03
        ], [
            6.89262173e-02, 2.70314264e-02, 5.22514148e-02, 1.47677197e-01, 9.83216553e-02,
            7.65030128e-02, 7.57523073e-02, 3.58856549e-02, 1.32317961e+00, 3.13711646e-01,
            6.42334212e-02, 4.55095424e-02, 4.71162177e-02, 5.42128925e-02, 5.77248004e-02,
            7.65141550e-02, 2.10980289e-02, 4.09391008e-02, 2.72846736e-02, 1.35237907e-02,
            1.21089376e-02, 2.46519640e-02, 1.89340103e-02, 2.05136644e-02, 1.94980685e-02,
            1.87630000e-02, 1.27634179e-02, 8.22311662e-03, 7.61166708e-03, 1.63377138e-02,
            2.34452832e-02, 1.19060306e-02, 9.41412665e-02, 2.58020046e-02, 3.64283375e-03
        ], [
            1.33980403e-02, 1.53675151e-02, 3.20457396e-02, 2.60922694e-02, 6.41150224e-02,
            2.39771937e-02, 3.01014690e-02, 1.31899044e-02, 3.57588861e-02, 1.14775923e+00,
            2.36263705e-02, 2.19577632e-02, 4.81041335e-02, 4.58508220e-02, 6.99586269e-02,
            5.65522685e-02, 1.06552160e-02, 3.32030442e-02, 3.23915736e-02, 1.03887504e-02,
            1.04178084e-02, 1.42320714e-02, 2.00605722e-02, 1.39296342e-02, 1.31361431e-02,
            1.53814227e-02, 1.06667645e-02, 4.68956057e-03, 4.62088315e-03, 8.56964849e-03,
            1.06317995e-02, 5.02860910e-03, 1.36036941e-02, 1.06376103e-02, 1.68360411e-03
        ], [
            6.08559917e-03, 8.98408090e-03, 1.26184245e-02, 8.22571799e-03, 8.49182192e-03,
            1.72593775e-02, 6.31830557e-03, 7.94387946e-03, 1.35403663e-02, 1.46378243e-02,
            1.13339882e+00, 2.10527994e-02, 1.60059054e-02, 2.16704347e-02, 1.91240563e-02,
            1.50607977e-02, 8.21826517e-03, 9.62082124e-02, 8.45827403e-03, 4.31154541e-03,
            3.81658623e-03, 8.78981207e-03, 5.65876084e-03, 5.35591708e-03, 4.88883822e-03,
            6.30457912e-03, 5.52654369e-03, 2.55981522e-03, 7.54896043e-03, 4.15104878e-03,
            6.12039199e-03, 5.08624320e-03, 6.26635417e-03, 6.23482029e-03, 1.01406139e-03
        ], [
            2.26599736e-02, 5.37841802e-02, 4.44865669e-02, 3.47111102e-02, 4.39601945e-02,
            7.06096222e-02, 3.77939806e-02, 4.27144690e-02, 5.12664976e-02, 6.99185938e-02,
            7.78640897e-02, 1.48045305e+00, 2.94299344e-01, 1.49154050e-01, 2.40571412e-01,
            1.75822791e-01, 3.48258050e-02, 1.61745839e-01, 6.72045921e-02, 1.89850406e-02,
            1.70016584e-02, 2.43695898e-02, 3.39493247e-02, 3.94971526e-02, 3.67252690e-02,
            3.06274229e-02, 2.45584871e-02, 1.02857321e-02, 1.55998653e-02, 1.87831149e-02,
            2.67637725e-02, 1.31961795e-02, 1.96199525e-02, 2.31149188e-02, 5.28569735e-03
        ], [
            1.36497671e-02, 2.40635496e-02, 1.48452844e-02, 1.92386011e-02, 1.67595820e-02,
            2.03878045e-02, 1.66111689e-02, 2.03577266e-02, 1.80386846e-02, 2.37638183e-02,
            2.75754929e-02, 3.38943015e-02, 1.14702701e+00, 2.96171218e-02, 6.00950473e-02,
            2.25021085e-02, 1.72474543e-02, 3.07912381e-02, 3.53232786e-02, 8.79909059e-03,
            6.96812238e-03, 8.83312288e-03, 1.45343910e-02, 1.88934767e-02, 1.35791985e-02,
            1.19836876e-02, 8.74833518e-03, 4.47505457e-03, 4.67263172e-03, 7.89793568e-03,
            1.28581190e-02, 5.64540228e-03, 7.97351397e-03, 9.41192871e-03, 2.15690767e-03
        ], [
            1.41639463e-02, 2.30231788e-02, 2.35416870e-02, 3.10666774e-02, 2.85843972e-02,
            3.16828832e-02, 3.76129388e-02, 2.30612302e-02, 3.70515117e-02, 4.64532497e-02,
            3.66096467e-02, 5.60913468e-02, 1.60834464e-01, 1.43480613e+00, 1.38206853e-01,
            6.06637417e-02, 3.74834646e-02, 6.75561310e-02, 6.54425817e-02, 2.59079855e-02,
            2.33888910e-02, 2.08935005e-02, 2.76641755e-02, 2.84982474e-02, 2.88649912e-02,
            2.88626930e-02, 7.95497137e-02, 1.79908215e-02, 1.08730501e-02, 3.54448328e-02,
            3.48773106e-02, 1.67342828e-02, 4.15446293e-02, 2.92562086e-02, 3.24954977e-03
        ], [
            1.27888283e-02, 1.62214191e-02, 1.51671285e-02, 1.56063842e-02, 1.64840620e-02,
            1.71913056e-02, 1.33572695e-02, 1.35559231e-02, 1.42914987e-02, 1.70891086e-02,
            1.67996772e-02, 2.03833724e-02, 3.20546233e-02, 1.78978906e-02, 1.42598516e+00,
            1.78974564e-02, 1.14677422e-02, 1.71758263e-02, 1.36960042e-01, 1.42864158e-02,
            1.16295065e-02, 9.71563537e-03, 5.68301355e-02, 7.36180773e-02, 8.09791799e-02,
            3.03644810e-02, 1.14897599e-02, 6.24354558e-03, 4.27751444e-03, 1.16785571e-02,
            3.04352195e-02, 5.96682165e-03, 7.70140468e-03, 1.20847231e-02, 1.78365967e-03
        ], [
            2.31646753e-03, 3.24205541e-03, 3.96258899e-03, 1.02458484e-02, 7.21484200e-03,
            9.36063772e-03, 7.45314529e-03, 3.11796429e-03, 4.97919185e-03, 6.36049232e-03,
            6.40410619e-03, 1.67585474e-02, 9.92580904e-03, 7.24465990e-03, 1.17434520e-02,
            1.04849235e+00, 3.33250961e-03, 9.49223885e-03, 5.44515837e-03, 2.87443194e-03,
            3.03775062e-03, 4.94190516e-03, 3.97662327e-03, 3.57865451e-03, 3.92291897e-03,
            4.08876855e-03, 2.99095532e-03, 2.60440687e-03, 2.67699860e-03, 3.94067638e-03,
            5.94520359e-03, 4.71502351e-03, 4.63210188e-03, 8.12482737e-03, 2.75362778e-03
        ], [
            2.26960004e-02, 3.96953222e-02, 3.33055782e-02, 4.40041410e-02, 3.09873475e-02,
            4.15207669e-02, 4.58244266e-02, 3.76067559e-02, 5.48874858e-02, 5.07850375e-02,
            6.24771453e-02, 5.76419520e-02, 3.57648844e-02, 2.94412996e-02, 3.53706298e-02,
            3.32269089e-02, 1.11041089e+00, 2.76724181e-02, 2.36446150e-02, 1.49088594e-02,
            2.33292330e-02, 3.98192952e-02, 2.81699033e-02, 1.97170640e-02, 1.86711494e-02,
            2.70659160e-02, 1.94313099e-02, 1.29417678e-02, 1.79205228e-02, 1.66826806e-02,
            2.68994987e-02, 2.77999705e-02, 2.62436386e-02, 2.86237636e-02, 3.58659677e-03
        ], [
            8.68738957e-03, 1.23122179e-02, 1.04377268e-02, 1.08863718e-02, 1.01191657e-02,
            1.22633873e-02, 1.17170010e-02, 1.07820149e-02, 1.20667355e-02, 1.16514952e-02,
            1.57858390e-02, 1.36696811e-02, 1.16082243e-02, 1.01059036e-02, 1.09360131e-02,
            1.06702073e-02, 3.42217361e-02, 1.05148520e+00, 1.07791550e-02, 9.20744943e-03,
            1.08547716e-02, 1.29864539e-02, 1.62024115e-02, 1.22823416e-02, 1.03206586e-02,
            2.15826315e-02, 1.82625561e-02, 1.12875111e-02, 4.78246239e-02, 1.10466160e-02,
            2.52426430e-02, 1.99248000e-02, 1.30767204e-02, 1.80258872e-02, 2.69028616e-03
        ], [
            9.48925821e-03, 8.10031075e-03, 1.47774529e-02, 1.43425121e-02, 1.74750372e-02,
            1.41236925e-02, 1.18177039e-02, 9.15353911e-03, 1.37700132e-02, 1.44058465e-02,
            1.43739200e-02, 1.41653116e-02, 1.29245596e-02, 1.13283088e-02, 1.42460437e-02,
            1.38209882e-02, 1.35408174e-02, 1.42271466e-02, 1.01555223e+00, 7.46226114e-03,
            7.27092285e-03, 1.05991659e-02, 2.20344370e-02, 9.15646299e-03, 8.97511895e-03,
            1.31845897e-02, 8.24158211e-03, 5.89893144e-03, 4.18591277e-03, 9.69902042e-03,
            8.26565092e-03, 5.53773617e-03, 7.54200025e-03, 1.11381269e-02, 9.04562509e-04
        ], [
            6.05024279e-02, 4.83706316e-02, 1.13131187e-01, 1.10479493e-01, 1.35194495e-01,
            1.05138315e-01, 9.16962886e-02, 7.21003189e-02, 9.32622064e-02, 9.86330692e-02,
            8.07611231e-02, 9.42033516e-02, 9.72042527e-02, 9.28196939e-02, 1.06819192e-01,
            9.99567311e-02, 4.70584924e-02, 8.13703716e-02, 6.57616229e-02, 1.04402420e+00,
            2.76882422e-02, 7.67830188e-02, 4.62904303e-02, 5.15770227e-02, 4.94798683e-02,
            4.54104078e-02, 2.93957810e-02, 1.71795099e-02, 1.33557412e-02, 3.03639364e-02,
            3.98553967e-02, 2.27663513e-02, 4.87782281e-02, 3.64743392e-02, 5.37889676e-03
        ], [
            2.89958172e-02, 1.87989918e-02, 5.29431141e-02, 5.28675841e-02, 6.90770311e-02,
            4.20828957e-02, 3.66274644e-02, 2.44636374e-02, 4.15667743e-02, 4.23939426e-02,
            3.43841988e-02, 3.50636048e-02, 3.61218528e-02, 3.24214929e-02, 4.44580792e-02,
            4.51370890e-02, 1.84159235e-02, 4.54166197e-02, 2.86088746e-02, 1.46269052e-02,
            1.01458466e+00, 4.08984780e-02, 2.48820954e-02, 2.25048031e-02, 2.17752976e-02,
            2.10422552e-02, 1.81405344e-02, 1.10805492e-02, 8.23621521e-03, 1.76637545e-02,
            1.73186093e-02, 1.14946815e-02, 2.15464342e-02, 2.20284397e-02, 2.62107750e-03
        ], [
            5.79772211e-03, 8.28526593e-03, 1.16644390e-02, 1.31243151e-02, 1.35238169e-02,
            1.31306333e-02, 1.49430938e-02, 7.87655815e-03, 1.32472005e-02, 1.46120928e-02,
            1.31442814e-02, 1.34161868e-02, 1.53624842e-02, 1.42567123e-02, 1.29686147e-02,
            1.27048741e-02, 1.27791784e-02, 1.26248961e-02, 1.22462375e-02, 1.38866110e-02,
            1.07595607e-02, 1.01653970e+00, 1.46362301e-02, 1.43179750e-02, 3.12545159e-02,
            2.79865768e-02, 1.51708104e-02, 1.65470088e-02, 5.93920866e-03, 2.07427883e-02,
            1.60316179e-02, 1.00482273e-02, 1.38042105e-02, 1.88061800e-02, 9.84622411e-04
        ], [
            2.72756321e-02, 3.42961139e-02, 4.79210786e-02, 4.63451524e-02, 5.06690375e-02,
            5.01550064e-02, 4.06089804e-02, 5.00342784e-02, 4.14853621e-02, 4.15277752e-02,
            5.71822776e-02, 4.22119576e-02, 3.44836509e-02, 2.75731807e-02, 3.80094867e-02,
            4.16821748e-02, 3.97451748e-02, 3.76503614e-02, 2.75379291e-02, 2.91314771e-02,
            1.97391279e-02, 2.64244141e-02, 1.06262404e+00, 7.35433360e-02, 3.43281596e-02,
            8.15265702e-02, 1.94546149e-02, 1.33572566e-02, 7.15064769e-03, 1.53842978e-02,
            2.15817749e-02, 1.72375473e-02, 1.82302211e-02, 2.09479028e-02, 3.63191043e-03
        ], [
            6.80651960e-03, 8.80120908e-03, 1.05841799e-02, 1.01561422e-02, 1.29023593e-02,
            1.53124755e-02, 8.00532861e-03, 1.14939993e-02, 8.21361824e-03, 8.54491073e-03,
            1.34560885e-02, 1.30260090e-02, 8.30229888e-03, 6.66648533e-03, 8.00248442e-03,
            8.03028854e-03, 6.28397171e-03, 9.38920596e-03, 3.93297088e-03, 4.68102863e-03,
            3.11683110e-03, 5.61638611e-03, 6.35717616e-03, 1.11391169e+00, 7.06519392e-03,
            1.00846313e-02, 2.73671937e-03, 1.78463526e-03, 1.31934371e-03, 2.66702055e-03,
            3.30870394e-03, 2.83130726e-03, 2.66948816e-03, 3.42453141e-03, 7.96212110e-04
        ], [
            2.59476687e-03, 4.02406364e-03, 4.77982572e-03, 5.67861933e-03, 7.23970813e-03,
            5.06717529e-03, 6.25484788e-03, 3.73266021e-03, 5.56341586e-03, 5.81010746e-03,
            4.85184195e-03, 5.05400499e-03, 6.05577343e-03, 5.91449505e-03, 6.25671461e-03,
            5.05233662e-03, 3.54918495e-03, 4.39240466e-03, 4.57976162e-03, 5.97360528e-03,
            3.60042075e-03, 3.83329668e-03, 5.12927373e-03, 1.02882259e-02, 1.04131446e+00,
            2.54462322e-02, 7.11637480e-03, 6.11415334e-03, 1.83369971e-03, 7.27420068e-03,
            6.64290148e-03, 3.55556363e-03, 3.41379425e-03, 7.37284258e-03, 4.20498934e-04
        ], [
            1.19278867e-02, 1.41975970e-02, 2.10360002e-02, 1.85933145e-02, 2.22294073e-02,
            2.32167505e-02, 2.31761935e-02, 1.88303215e-02, 1.86272010e-02, 1.91923257e-02,
            2.29350722e-02, 1.96789772e-02, 1.81384577e-02, 1.56181603e-02, 1.98130472e-02,
            1.82574151e-02, 1.35516320e-02, 1.66028339e-02, 1.89291553e-02, 3.09343138e-02,
            1.71838004e-02, 1.82643764e-02, 5.56853391e-02, 1.75524115e-01, 1.20231734e-01,
            1.13211339e+00, 1.19190106e-02, 1.00226947e-02, 4.06078070e-03, 1.21934337e-02,
            1.20890761e-02, 7.28095338e-03, 9.82166756e-03, 1.41831268e-02, 1.45056665e-03
        ], [
            1.21442431e-02, 1.70657208e-02, 2.13491907e-02, 2.37048111e-02, 2.58366253e-02,
            2.26888141e-02, 3.36861819e-02, 1.67941302e-02, 2.66046927e-02, 2.51887837e-02,
            2.28430572e-02, 2.37960485e-02, 2.65847877e-02, 2.55129445e-02, 2.46617533e-02,
            2.51230733e-02, 1.93751923e-02, 2.85145858e-02, 2.78974336e-02, 3.37388702e-02,
            3.34598966e-02, 2.75796169e-02, 3.07901608e-02, 3.99047105e-02, 4.83684144e-02,
            4.40256258e-02, 1.14948827e+00, 4.85305460e-02, 1.31183253e-02, 4.13147451e-02,
            3.42964068e-02, 1.74413038e-02, 2.47898521e-02, 4.13476502e-02, 2.30144681e-03
        ], [
            4.94499751e-02, 6.18653155e-02, 6.28215125e-02, 7.25189462e-02, 6.81004308e-02,
            6.77987130e-02, 7.10005494e-02, 5.59872905e-02, 6.50152751e-02, 6.50416973e-02,
            6.68985485e-02, 6.61170817e-02, 6.47376218e-02, 5.72098945e-02, 6.46724952e-02,
            6.80800974e-02, 6.16739350e-02, 6.44925395e-02, 5.85121540e-02, 6.90372199e-02,
            6.49833298e-02, 6.29108741e-02, 8.63514412e-02, 9.21870339e-02, 8.28810811e-02,
            6.66802883e-02, 5.43452256e-02, 1.26699918e+00, 8.80348849e-02, 7.38303999e-02,
            5.41334319e-02, 3.09558388e-02, 5.32193511e-02, 7.53582677e-02, 1.27489555e-02
        ], [
            1.51953277e-02, 1.49483910e-02, 2.18266701e-02, 2.27585903e-02, 2.41448000e-02,
            2.26095832e-02, 2.71463631e-02, 1.49282383e-02, 2.13549623e-02, 2.31561835e-02,
            1.97390137e-02, 2.15834762e-02, 2.28276538e-02, 2.03974984e-02, 2.31895271e-02,
            2.40817362e-02, 1.71893661e-02, 2.53476090e-02, 3.64773096e-02, 3.52913814e-02,
            5.29324789e-02, 4.23487112e-02, 2.28197363e-02, 3.90659285e-02, 3.10699442e-02,
            3.72338098e-02, 2.93899891e-02, 4.01173826e-02, 1.04269959e+00, 3.79452073e-02,
            2.51911079e-02, 2.55859472e-02, 4.52306312e-02, 4.10144925e-02, 2.02008568e-03
        ], [
            5.10373249e-02, 7.21048532e-02, 1.06502535e-01, 1.03580074e-01, 1.02613051e-01,
            9.18674693e-02, 1.46178689e-01, 6.93477202e-02, 1.47447519e-01, 1.20022068e-01,
            9.74302929e-02, 1.00598290e-01, 1.21183237e-01, 1.33627341e-01, 1.29548949e-01,
            1.06158544e-01, 9.29983673e-02, 1.22982504e-01, 1.03280716e-01, 9.87633615e-02,
            1.05205819e-01, 9.38760148e-02, 9.58000914e-02, 1.09045102e-01, 1.67233340e-01,
            1.31024373e-01, 1.30563167e-01, 1.63465336e-01, 6.39350931e-02, 1.19909946e+00,
            1.12015739e-01, 5.74623120e-02, 1.04487954e-01, 1.33601150e-01, 9.74823181e-03
        ], [
            3.38486729e-03, 5.53066856e-03, 5.02481717e-03, 4.35463508e-03, 4.20837770e-03,
            7.06454439e-03, 6.40207745e-03, 4.37174841e-03, 6.23588305e-03, 5.67436989e-03,
            5.31541895e-03, 5.77109549e-03, 4.38839968e-03, 4.20124169e-03, 4.77552570e-03,
            5.30261358e-03, 7.39426008e-03, 4.62941163e-03, 4.33488779e-03, 3.21115577e-03,
            4.79472086e-03, 6.04404941e-03, 5.03640603e-03, 1.00936853e-02, 1.11510117e-02,
            5.95849489e-03, 5.12001904e-03, 3.50182089e-03, 4.18528018e-03, 4.76800179e-03,
            1.00915828e+00, 5.33361049e-03, 6.54061520e-03, 1.03830081e-02, 5.08269627e-04
        ], [
            1.39949023e-03, 1.21538075e-03, 1.76150751e-03, 1.72711399e-03, 1.80585969e-03,
            1.57618533e-03, 2.06206668e-03, 1.33579790e-03, 2.35966817e-03, 2.04119613e-03,
            1.73686833e-03, 1.91303906e-03, 2.18309650e-03, 2.24679792e-03, 2.38996924e-03,
            1.66351857e-03, 1.89097489e-03, 1.67575407e-03, 1.70165359e-03, 1.56850037e-03,
            2.47098430e-03, 1.50510385e-03, 2.13979362e-03, 1.62349134e-03, 2.96402449e-03,
            2.30854320e-03, 2.55223780e-03, 2.30904113e-03, 7.80134914e-04, 3.37565852e-03,
            5.33120329e-03, 1.01013945e+00, 1.95106838e-03, 2.64696850e-03, 1.43968621e-04
        ], [
            2.21893507e-03, 8.44021432e-04, 1.67353553e-03, 1.30729511e-03, 1.98914799e-03,
            1.38421448e-03, 1.19395171e-03, 7.93176235e-04, 1.28128143e-03, 1.21342630e-03,
            1.07028401e-03, 1.10701437e-03, 1.29312452e-03, 1.10789773e-03, 1.21021178e-03,
            1.22353183e-03, 8.69958694e-04, 1.17327567e-03, 1.11891241e-03, 1.03209487e-03,
            8.66191798e-04, 1.85088778e-03, 1.28142688e-03, 1.11448531e-03, 1.19086679e-03,
            1.26277993e-03, 1.25642077e-03, 1.08427480e-03, 3.67305029e-04, 1.32012943e-03,
            6.20813904e-03, 1.81008525e-03, 1.02611005e+00, 1.95319767e-03, 1.12310889e-04
        ], [
            9.59666534e-03, 1.43480583e-02, 1.87216369e-02, 1.92143684e-02, 2.12048434e-02,
            1.82308216e-02, 3.02738275e-02, 1.37114949e-02, 2.28267004e-02, 2.02390863e-02,
            1.77600622e-02, 2.03644597e-02, 2.02152497e-02, 1.85127198e-02, 1.98732847e-02,
            1.97611383e-02, 1.60664833e-02, 1.90520969e-02, 1.93835862e-02, 1.70688357e-02,
            1.78500777e-02, 2.47387604e-02, 1.83345228e-02, 1.88758653e-02, 2.27732987e-02,
            2.42714666e-02, 3.27132355e-02, 2.30719661e-02, 1.35829092e-02, 4.13591257e-02,
            2.55738114e-02, 1.67063049e-02, 2.54006445e-02, 1.09654793e+00, 2.93535090e-03
        ], [
            5.06102790e-05, 5.48363849e-05, 8.69482458e-05, 1.27030203e-04, 9.79591082e-05,
            1.00017290e-04, 1.06515454e-04, 5.77389070e-05, 9.65771423e-05, 1.06302007e-04,
            7.59709688e-05, 8.18325335e-05, 8.74762934e-05, 7.63573474e-05, 1.14372246e-04,
            8.76402563e-05, 5.86599044e-05, 1.35360430e-04, 1.56570950e-04, 6.18247403e-05,
            9.67392517e-05, 1.12884641e-04, 1.53412802e-04, 7.69541331e-05, 8.51381589e-05,
            8.33448999e-05, 1.11021881e-04, 1.20971841e-04, 3.20042011e-04, 1.25109237e-04,
            1.24867624e-04, 7.06738158e-05, 1.61133600e-04, 1.95592991e-04, 1.00008239e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 395)
    return ret


@cache('run')
def leontief_matrix_x2001x():
    """
    leontief matrix x2001x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17376324e+00, 5.42692320e-03, 3.57302102e-01, 7.23802767e-02, 1.00172097e-01,
            1.93117710e-01, 2.72004328e-02, 6.97740870e-03, 1.79681832e-02, 2.88294478e-02,
            8.75474334e-03, 7.14435505e-03, 7.75673501e-03, 7.14127789e-03, 8.53506619e-03,
            3.49588758e-02, 5.61866050e-03, 2.26215053e-02, 5.77760178e-03, 8.89303282e-03,
            7.70944611e-03, 1.01477301e-01, 6.81076529e-03, 1.04984726e-02, 8.56473815e-03,
            1.22629266e-02, 5.05326739e-03, 4.05856738e-03, 3.28621564e-03, 6.11431748e-03,
            1.02110598e-02, 8.59071056e-03, 1.22960926e-02, 1.12621339e-02, 2.03713870e-02
        ], [
            2.27107367e-02, 1.10103012e+00, 2.39487917e-02, 2.80103369e-02, 2.28312283e-02,
            2.94758750e-02, 2.59220461e-02, 5.09889282e-01, 8.28524776e-02, 4.09382639e-02,
            1.13007496e-01, 9.39857982e-02, 3.47025140e-02, 2.43625708e-02, 2.99968432e-02,
            3.24294883e-02, 1.50921292e-01, 5.35653491e-02, 1.50624474e-02, 1.06908577e-02,
            1.06243747e-02, 1.71884065e-02, 4.14822093e-02, 5.43547499e-02, 4.60897178e-02,
            2.26970539e-02, 1.05833018e-02, 6.41495695e-03, 9.29579172e-03, 1.06938025e-02,
            1.61598379e-02, 1.30363692e-02, 1.54012781e-02, 1.63300377e-02, 6.95446830e-02
        ], [
            7.78999990e-02, 4.35897632e-03, 1.19876344e+00, 1.64883093e-02, 1.34718669e-01,
            1.96987748e-02, 1.15794859e-02, 5.27263066e-03, 2.06201348e-02, 1.42512470e-02,
            7.90218161e-03, 7.19702363e-03, 7.89934270e-03, 7.54974595e-03, 7.93797933e-03,
            1.09454957e-02, 5.89882869e-03, 7.74764063e-03, 6.67906219e-03, 9.73278721e-03,
            8.72650722e-03, 1.85006323e-01, 6.44936256e-03, 1.60054067e-02, 1.27100117e-02,
            9.97361307e-03, 6.74851265e-03, 5.36505750e-03, 2.37783796e-03, 7.56613612e-03,
            1.50810080e-02, 1.24630015e-02, 2.16929862e-02, 1.38695519e-02, 5.46433930e-03
        ], [
            5.39785760e-03, 3.69288608e-03, 6.44316714e-03, 1.41902705e+00, 9.39018449e-02,
            1.02453040e-02, 1.30335091e-02, 3.73964202e-03, 9.30048519e-03, 2.43366272e-02,
            1.01065508e-02, 6.45235315e-03, 8.59030498e-03, 7.44376416e-03, 1.63606477e-02,
            3.94310592e-02, 3.01654150e-03, 7.47618667e-03, 5.70741384e-03, 5.12820467e-03,
            5.56258859e-03, 6.53912294e-03, 4.85489583e-03, 6.32986381e-03, 4.85062490e-03,
            4.99424587e-03, 3.52975382e-03, 2.29866623e-03, 1.47802218e-03, 3.66411502e-03,
            7.24111640e-03, 2.74882513e-03, 6.58600379e-03, 1.10724048e-02, 7.96172939e-04
        ], [
            4.89937894e-04, 4.02188375e-04, 7.02755468e-04, 1.15620148e-02, 1.25644823e+00,
            1.37513930e-03, 1.55202250e-03, 4.32529827e-04, 8.23461311e-04, 2.48075040e-03,
            1.03059929e-03, 9.10350900e-04, 1.03724794e-03, 9.40900414e-04, 3.09838835e-03,
            5.86613488e-03, 4.58770114e-04, 7.28768537e-04, 1.08838105e-03, 7.78917757e-04,
            6.47826556e-04, 5.43724326e-04, 6.33026739e-04, 6.79715701e-04, 6.29048374e-04,
            5.98224058e-04, 7.02756382e-04, 2.46550663e-04, 1.44941942e-04, 4.09371506e-04,
            1.11176500e-03, 3.36967314e-04, 4.72408325e-04, 9.61816282e-04, 6.94891963e-05
        ], [
            4.90243192e-03, 4.44144788e-03, 6.38092521e-03, 6.18189695e-03, 7.06914392e-03,
            1.27180543e+00, 1.95697291e-02, 3.65672269e-03, 5.84900382e-03, 8.64963430e-03,
            1.23409717e-02, 9.40058902e-03, 8.68584160e-03, 6.44660547e-03, 9.69302468e-03,
            1.00831919e-01, 4.69597744e-03, 4.65785339e-02, 4.02793954e-03, 3.86734582e-03,
            3.60297016e-03, 5.91928253e-03, 4.38303974e-03, 4.02323224e-03, 3.71755861e-03,
            7.50484787e-03, 3.49429736e-03, 2.48491486e-03, 5.41613382e-03, 3.58163383e-03,
            4.93542656e-03, 4.89763953e-03, 3.73613214e-03, 7.07456188e-03, 9.02443832e-04
        ], [
            1.46909271e-02, 1.31531553e-02, 5.21316193e-02, 3.59242093e-02, 4.23254728e-02,
            3.53582992e-02, 1.30838415e+00, 1.36317730e-02, 4.35965344e-02, 4.61206927e-02,
            4.37377553e-02, 2.45380451e-02, 3.11082850e-02, 3.73542113e-02, 2.83393569e-02,
            4.69765239e-02, 1.75240887e-02, 2.40964835e-02, 2.27634819e-02, 2.61885873e-02,
            2.49373672e-02, 2.80764023e-02, 1.95263271e-02, 1.90649986e-02, 2.38759535e-02,
            3.22529760e-02, 2.80465854e-02, 3.06852653e-02, 9.58580386e-03, 4.10385151e-02,
            3.33757021e-02, 3.00178211e-02, 2.39824179e-02, 4.70743308e-02, 2.16920890e-03
        ], [
            2.28651976e-02, 1.98025841e-02, 1.88946509e-02, 2.20396259e-02, 1.93825453e-02,
            2.10323023e-02, 1.68674647e-02, 1.06282561e+00, 5.08659705e-02, 2.87861179e-02,
            3.31552624e-02, 2.48718386e-02, 1.66522713e-02, 1.35818862e-02, 1.53988662e-02,
            1.72594846e-02, 3.20682175e-02, 2.60119233e-02, 1.17878686e-02, 1.16877758e-02,
            9.21605150e-03, 1.24741862e-02, 6.30445635e-02, 9.34455288e-02, 7.88908594e-02,
            2.78046494e-02, 9.20300813e-03, 5.55895278e-03, 4.54376105e-03, 1.00740384e-02,
            1.46098754e-02, 7.29692372e-03, 1.09636875e-02, 1.25939256e-02, 1.94753930e-03
        ], [
            6.83114649e-02, 2.63129650e-02, 5.14611931e-02, 1.46054420e-01, 9.79558148e-02,
            7.73238864e-02, 7.56013783e-02, 3.66297986e-02, 1.31608454e+00, 3.11206531e-01,
            6.50821325e-02, 4.58123344e-02, 4.75155231e-02, 5.52437612e-02, 5.74535357e-02,
            7.40969498e-02, 2.11939990e-02, 4.09901938e-02, 2.67786708e-02, 1.30199739e-02,
            1.15929598e-02, 2.43238042e-02, 1.84270506e-02, 2.02084871e-02, 1.89941934e-02,
            1.83763707e-02, 1.25607195e-02, 7.97526443e-03, 7.83938111e-03, 1.61989747e-02,
            2.36029632e-02, 1.19300079e-02, 9.43884575e-02, 2.62297361e-02, 3.59471813e-03
        ], [
            1.35275635e-02, 1.47458071e-02, 3.22214330e-02, 2.66846816e-02, 6.32251536e-02,
            2.37047311e-02, 3.04608987e-02, 1.37083309e-02, 3.54462440e-02, 1.15562085e+00,
            2.38734033e-02, 2.20308746e-02, 4.91917134e-02, 4.82137446e-02, 7.03276249e-02,
            5.45826806e-02, 1.11181487e-02, 3.28938606e-02, 3.12519041e-02, 1.00760505e-02,
            9.96895635e-03, 1.40835464e-02, 1.98290186e-02, 1.44600567e-02, 1.37209060e-02,
            1.51739135e-02, 1.06737851e-02, 4.59789132e-03, 4.51723929e-03, 8.68454388e-03,
            1.05248246e-02, 5.06413870e-03, 1.32401678e-02, 1.10208978e-02, 1.61349217e-03
        ], [
            6.22975544e-03, 9.05001083e-03, 1.25820042e-02, 8.51493780e-03, 8.52133269e-03,
            1.66739635e-02, 6.31411379e-03, 8.22324800e-03, 1.35000882e-02, 1.50481643e-02,
            1.13615070e+00, 2.17275078e-02, 1.63254050e-02, 2.30714908e-02, 1.94068294e-02,
            1.48692920e-02, 8.71419091e-03, 9.58148461e-02, 8.40613728e-03, 4.05758057e-03,
            3.67480281e-03, 8.62076109e-03, 5.74827110e-03, 5.40810046e-03, 5.05325188e-03,
            6.21731554e-03, 5.50234122e-03, 2.52945417e-03, 7.34923130e-03, 4.15355237e-03,
            5.96472537e-03, 5.06439871e-03, 6.16443677e-03, 6.28335070e-03, 9.61236422e-04
        ], [
            2.27926547e-02, 5.36462678e-02, 4.43609713e-02, 3.55162839e-02, 4.48167857e-02,
            6.97054063e-02, 3.72262227e-02, 4.39443782e-02, 5.00418604e-02, 7.09845625e-02,
            7.97569039e-02, 1.48463394e+00, 2.99271662e-01, 1.52841887e-01, 2.36940329e-01,
            1.70256987e-01, 3.68437223e-02, 1.65053640e-01, 6.60300500e-02, 1.82660980e-02,
            1.62690742e-02, 2.40915705e-02, 3.38252399e-02, 4.04461573e-02, 3.64180875e-02,
            3.05230778e-02, 2.42095675e-02, 1.01645436e-02, 1.55413655e-02, 1.90354537e-02,
            2.60439133e-02, 1.33564576e-02, 1.93281927e-02, 2.34894314e-02, 5.11765676e-03
        ], [
            1.38630763e-02, 2.43884399e-02, 1.47970881e-02, 2.07082292e-02, 1.72845820e-02,
            2.07736038e-02, 1.65724039e-02, 2.10143049e-02, 1.80499824e-02, 2.43979727e-02,
            3.03397104e-02, 3.51407878e-02, 1.15338840e+00, 3.16857925e-02, 6.13566232e-02,
            2.30180512e-02, 1.80631615e-02, 3.20063094e-02, 3.50037800e-02, 8.52095267e-03,
            6.71194086e-03, 8.85034151e-03, 1.43179975e-02, 2.03909997e-02, 1.43961518e-02,
            1.26198128e-02, 8.74045480e-03, 4.51017467e-03, 4.77237761e-03, 8.03671430e-03,
            1.23917014e-02, 5.79983406e-03, 7.86583844e-03, 9.55738192e-03, 2.11389750e-03
        ], [
            1.39361207e-02, 2.35374653e-02, 2.28205291e-02, 3.06370851e-02, 2.88661332e-02,
            2.99050474e-02, 3.54900588e-02, 2.46256747e-02, 3.54413845e-02, 4.47822222e-02,
            3.66437715e-02, 5.36549387e-02, 1.59485735e-01, 1.43784764e+00, 1.34885804e-01,
            5.64851042e-02, 3.99961908e-02, 6.84275782e-02, 6.20142074e-02, 2.40887010e-02,
            2.10668021e-02, 2.00430805e-02, 2.75879030e-02, 2.91239748e-02, 2.94855047e-02,
            2.91710361e-02, 7.52254632e-02, 1.75438596e-02, 1.08168605e-02, 3.56761899e-02,
            3.42535831e-02, 1.71574846e-02, 4.22854322e-02, 2.93593894e-02, 3.14374564e-03
        ], [
            1.31367902e-02, 1.62369296e-02, 1.54329074e-02, 1.59579514e-02, 1.65880131e-02,
            1.74551613e-02, 1.36307297e-02, 1.42610541e-02, 1.40408929e-02, 1.76069473e-02,
            1.73021404e-02, 2.09998720e-02, 3.30311333e-02, 1.92002727e-02, 1.43068146e+00,
            1.81999159e-02, 1.20324969e-02, 1.74589811e-02, 1.29681510e-01, 1.40821351e-02,
            1.05918835e-02, 9.73487874e-03, 5.80651711e-02, 8.07213764e-02, 8.50310430e-02,
            3.00674524e-02, 1.15399495e-02, 6.39399934e-03, 4.46560792e-03, 1.18619796e-02,
            2.86379811e-02, 6.18367349e-03, 7.56743638e-03, 1.40264959e-02, 1.76509887e-03
        ], [
            2.29544480e-03, 3.17952480e-03, 3.90850095e-03, 1.01867069e-02, 7.09663489e-03,
            8.93421838e-03, 7.40944635e-03, 3.06993047e-03, 4.89625969e-03, 6.36772636e-03,
            6.35191924e-03, 1.61627808e-02, 9.78273224e-03, 7.43828944e-03, 1.18884171e-02,
            1.05013737e+00, 3.47977980e-03, 9.40833665e-03, 5.12401911e-03, 2.76897632e-03,
            2.87303144e-03, 4.67247009e-03, 4.07871801e-03, 3.52594935e-03, 4.06001362e-03,
            4.05602588e-03, 2.93997204e-03, 2.48685046e-03, 2.69514314e-03, 3.86206345e-03,
            5.97195977e-03, 4.42956063e-03, 4.53726108e-03, 8.36436505e-03, 2.76778759e-03
        ], [
            2.41368982e-02, 4.18794246e-02, 3.56431963e-02, 4.58694971e-02, 3.24008373e-02,
            4.31721821e-02, 4.87311972e-02, 4.05538981e-02, 5.61645206e-02, 5.40214691e-02,
            6.65845061e-02, 6.13043518e-02, 3.80684144e-02, 3.17270794e-02, 3.71872004e-02,
            3.42126595e-02, 1.11063838e+00, 2.95489592e-02, 2.46839930e-02, 1.49589674e-02,
            2.36473025e-02, 4.23362320e-02, 2.88648787e-02, 2.04960393e-02, 2.03607621e-02,
            2.88839734e-02, 2.04604477e-02, 1.30668082e-02, 1.93362130e-02, 1.73091978e-02,
            2.79886094e-02, 2.89727832e-02, 2.72417799e-02, 3.04551641e-02, 3.72970520e-03
        ], [
            8.90978695e-03, 1.35083763e-02, 1.07208438e-02, 1.14743997e-02, 1.02839488e-02,
            1.20469890e-02, 1.20611991e-02, 1.20544999e-02, 1.23429815e-02, 1.20548823e-02,
            1.64980127e-02, 1.40835356e-02, 1.21531114e-02, 1.09391423e-02, 1.13327392e-02,
            1.06894345e-02, 3.65089621e-02, 1.05411429e+00, 1.09209151e-02, 9.17828604e-03,
            1.06080718e-02, 1.34467714e-02, 1.67925293e-02, 1.23234953e-02, 1.12805420e-02,
            2.20053765e-02, 1.86379639e-02, 1.13301929e-02, 4.70516365e-02, 1.11905120e-02,
            2.45793150e-02, 2.06046091e-02, 1.27767264e-02, 1.81793830e-02, 2.30701901e-03
        ], [
            9.30106304e-03, 7.94036937e-03, 1.47265898e-02, 1.41754240e-02, 1.72897098e-02,
            1.38000081e-02, 1.17091505e-02, 8.92124989e-03, 1.34724706e-02, 1.44255222e-02,
            1.42725497e-02, 1.40585348e-02, 1.30368983e-02, 1.16192224e-02, 1.41885213e-02,
            1.35179437e-02, 1.32319673e-02, 1.39107710e-02, 1.01488623e+00, 7.10559904e-03,
            6.89012293e-03, 1.04052658e-02, 2.13663882e-02, 8.77740841e-03, 9.40259572e-03,
            1.31345375e-02, 7.85827319e-03, 5.75683213e-03, 4.01699294e-03, 9.50641487e-03,
            7.92000271e-03, 5.31435025e-03, 7.39036386e-03, 1.07608974e-02, 8.89834126e-04
        ], [
            6.01643546e-02, 4.68935814e-02, 1.12948810e-01, 1.12304600e-01, 1.33190682e-01,
            1.05813730e-01, 9.13279640e-02, 7.44413348e-02, 9.47090067e-02, 1.01050876e-01,
            8.12092915e-02, 9.45740815e-02, 9.89146107e-02, 9.52813795e-02, 1.06723311e-01,
            9.79200352e-02, 4.77401287e-02, 8.20995289e-02, 6.41877387e-02, 1.04125190e+00,
            2.62508947e-02, 7.65021679e-02, 4.54457976e-02, 5.07821127e-02, 5.03222020e-02,
            4.45612524e-02, 2.88794266e-02, 1.68227836e-02, 1.36954729e-02, 3.02105692e-02,
            4.02720759e-02, 2.25667474e-02, 4.83309947e-02, 3.73678155e-02, 5.25227829e-03
        ], [
            2.90750544e-02, 1.89262768e-02, 5.37491970e-02, 5.40872192e-02, 7.29263174e-02,
            4.25980080e-02, 3.70111539e-02, 2.56138828e-02, 4.16752024e-02, 4.30223120e-02,
            3.57922984e-02, 3.62911456e-02, 3.76393545e-02, 3.42798982e-02, 4.53142986e-02,
            4.54189743e-02, 1.96852458e-02, 4.69922759e-02, 2.83763951e-02, 1.41045391e-02,
            1.01413347e+00, 4.12374268e-02, 2.47229983e-02, 2.22903592e-02, 2.31616143e-02,
            2.10812310e-02, 1.78125894e-02, 1.12416419e-02, 8.12015253e-03, 1.78204057e-02,
            1.76701340e-02, 1.16402762e-02, 2.15651925e-02, 2.19559989e-02, 2.57792736e-03
        ], [
            5.81020914e-03, 8.35151072e-03, 1.16222065e-02, 1.34802797e-02, 1.35806419e-02,
            1.31864302e-02, 1.53883249e-02, 8.15271571e-03, 1.34178793e-02, 1.51307781e-02,
            1.35898219e-02, 1.38227142e-02, 1.57662877e-02, 1.48039055e-02, 1.31149286e-02,
            1.26693009e-02, 1.60485437e-02, 1.27396748e-02, 1.22035523e-02, 1.34341091e-02,
            1.03347361e-02, 1.01643755e+00, 1.46827252e-02, 1.37670162e-02, 3.36607328e-02,
            2.70576747e-02, 1.55073299e-02, 1.63749264e-02, 5.92425221e-03, 2.02318738e-02,
            1.60983178e-02, 9.72473874e-03, 1.35695114e-02, 1.89125352e-02, 1.01808317e-03
        ], [
            2.67080884e-02, 3.44482176e-02, 4.73164915e-02, 4.61024369e-02, 4.94135372e-02,
            4.99834204e-02, 4.03845236e-02, 5.62860442e-02, 4.13676418e-02, 4.22198252e-02,
            5.74410990e-02, 4.28003435e-02, 3.50204376e-02, 2.87273781e-02, 3.78093002e-02,
            4.11790449e-02, 4.00629061e-02, 3.78446792e-02, 2.75818694e-02, 2.86297295e-02,
            1.94331367e-02, 2.59844742e-02, 1.06292394e+00, 7.50545937e-02, 3.64184226e-02,
            8.10604714e-02, 1.89592472e-02, 1.30146127e-02, 7.31690052e-03, 1.50440200e-02,
            2.13248384e-02, 1.71559003e-02, 1.78555914e-02, 2.10409365e-02, 3.49136617e-03
        ], [
            7.01663877e-03, 9.20841271e-03, 1.07526448e-02, 1.10845218e-02, 1.32876845e-02,
            1.63218559e-02, 8.82192991e-03, 1.33129848e-02, 8.90679077e-03, 9.62346018e-03,
            1.56565529e-02, 1.43115209e-02, 9.36003955e-03, 7.58724333e-03, 8.53967074e-03,
            8.70425087e-03, 6.86559765e-03, 1.04440262e-02, 4.07535487e-03, 5.01005758e-03,
            3.16971737e-03, 5.50984771e-03, 6.58514467e-03, 1.12176506e+00, 7.22464845e-03,
            1.05084630e-02, 2.78517840e-03, 1.87399994e-03, 1.62904300e-03, 2.90723312e-03,
            3.69738538e-03, 2.78969230e-03, 2.79659571e-03, 3.75406014e-03, 8.30515550e-04
        ], [
            2.41832699e-03, 3.90802118e-03, 4.43559921e-03, 5.42673284e-03, 6.80367775e-03,
            5.00276018e-03, 5.98512676e-03, 3.66283896e-03, 5.18599843e-03, 5.55829127e-03,
            4.76944139e-03, 4.85339027e-03, 5.84143499e-03, 5.74198262e-03, 5.81563333e-03,
            4.70676714e-03, 3.84988404e-03, 4.15012471e-03, 4.21196183e-03, 5.29630016e-03,
            3.16937373e-03, 3.52566274e-03, 5.18333982e-03, 1.14713758e-02, 1.04124628e+00,
            2.44654786e-02, 6.19071822e-03, 5.46621345e-03, 1.70435761e-03, 6.62250025e-03,
            6.03344487e-03, 3.20360061e-03, 3.16786872e-03, 6.73219982e-03, 4.09490155e-04
        ], [
            1.17807790e-02, 1.40351108e-02, 2.07243998e-02, 1.84783061e-02, 2.15570641e-02,
            2.24092175e-02, 2.38595635e-02, 2.04494704e-02, 1.85481355e-02, 1.95102158e-02,
            2.30103762e-02, 1.98673857e-02, 1.82751332e-02, 1.59635849e-02, 1.97056104e-02,
            1.79248881e-02, 1.40070539e-02, 1.68183967e-02, 1.93004249e-02, 2.95578106e-02,
            1.63509836e-02, 1.78293201e-02, 5.48583762e-02, 1.62827032e-01, 1.24818446e-01,
            1.13294060e+00, 1.17208113e-02, 9.37699072e-03, 4.18889102e-03, 1.20260382e-02,
            1.20056867e-02, 7.09024571e-03, 9.74065499e-03, 1.43386138e-02, 1.42452232e-03
        ], [
            1.28726430e-02, 1.80868793e-02, 2.26683294e-02, 2.58220762e-02, 2.79228849e-02,
            2.41619175e-02, 3.61692903e-02, 1.83316447e-02, 2.85111452e-02, 2.73584886e-02,
            2.49295092e-02, 2.60361730e-02, 2.87737564e-02, 2.84288567e-02, 2.60353172e-02,
            2.69035927e-02, 2.27071044e-02, 3.13159819e-02, 2.99828847e-02, 3.38532765e-02,
            3.37078066e-02, 2.88996232e-02, 3.26798400e-02, 4.12733286e-02, 5.10173951e-02,
            4.59128941e-02, 1.15700904e+00, 4.92093606e-02, 1.38567745e-02, 4.39106434e-02,
            3.65989521e-02, 1.83196259e-02, 2.62355092e-02, 4.23998033e-02, 2.56571316e-03
        ], [
            5.00235058e-02, 6.24281309e-02, 6.29856225e-02, 7.42471838e-02, 6.73172772e-02,
            6.73669809e-02, 7.27294165e-02, 5.78898469e-02, 6.51852115e-02, 6.65457091e-02,
            6.93175812e-02, 6.85176542e-02, 6.60528674e-02, 5.95927084e-02, 6.51073297e-02,
            6.96696681e-02, 6.98521198e-02, 6.57186567e-02, 5.75294322e-02, 6.71019176e-02,
            6.36117667e-02, 6.40647845e-02, 8.69717676e-02, 9.22123987e-02, 8.60280119e-02,
            6.77472620e-02, 5.40553366e-02, 1.27751418e+00, 9.68361843e-02, 7.42511278e-02,
            5.41608055e-02, 3.06408199e-02, 5.41434525e-02, 8.16266183e-02, 1.55917359e-02
        ], [
            1.53692007e-02, 1.57376885e-02, 2.21770621e-02, 2.30931303e-02, 2.40382427e-02,
            2.32903252e-02, 2.82938662e-02, 1.58779684e-02, 2.17878970e-02, 2.39173013e-02,
            2.04333057e-02, 2.25120189e-02, 2.36140625e-02, 2.16721987e-02, 2.37249987e-02,
            2.41913455e-02, 1.99780654e-02, 2.64609070e-02, 3.56291488e-02, 3.40711356e-02,
            4.97135798e-02, 4.33704247e-02, 2.29314728e-02, 3.83131060e-02, 3.15362264e-02,
            3.82843262e-02, 3.03403768e-02, 3.85899449e-02, 1.04354587e+00, 3.81677744e-02,
            2.69703754e-02, 2.68201022e-02, 4.57373343e-02, 4.18829638e-02, 2.05908742e-03
        ], [
            5.10934386e-02, 7.35069080e-02, 1.07488442e-01, 1.05392899e-01, 1.01245086e-01,
            9.27459906e-02, 1.49381458e-01, 7.26075642e-02, 1.51459308e-01, 1.25075440e-01,
            1.00943121e-01, 1.05216916e-01, 1.25810515e-01, 1.40680365e-01, 1.33936104e-01,
            1.06884278e-01, 1.10868912e-01, 1.24991267e-01, 1.06034453e-01, 9.49357691e-02,
            9.96418631e-02, 9.56477638e-02, 9.74785287e-02, 1.04972834e-01, 1.68394182e-01,
            1.32491467e-01, 1.30049913e-01, 1.60214146e-01, 6.44847949e-02, 1.20320848e+00,
            1.19472598e-01, 5.99651760e-02, 1.06718504e-01, 1.36659290e-01, 1.06796098e-02
        ], [
            3.38074155e-03, 5.88822378e-03, 5.12835839e-03, 4.42625447e-03, 4.17692762e-03,
            6.96859832e-03, 6.70338567e-03, 4.70098277e-03, 6.37754124e-03, 5.93931160e-03,
            5.47543595e-03, 6.15874212e-03, 4.56991915e-03, 4.37953490e-03, 4.93021594e-03,
            5.33835353e-03, 7.89823567e-03, 4.79852520e-03, 4.46960918e-03, 3.08816651e-03,
            4.47149097e-03, 6.15724506e-03, 5.25842216e-03, 9.82398671e-03, 1.22397422e-02,
            6.21440802e-03, 5.11857309e-03, 3.42536155e-03, 4.37692655e-03, 4.90471711e-03,
            1.00931026e+00, 5.55421512e-03, 6.61911745e-03, 1.08342055e-02, 5.35465875e-04
        ], [
            1.32204381e-03, 1.21438054e-03, 1.75625348e-03, 1.78171678e-03, 1.86197449e-03,
            1.61056387e-03, 2.13699013e-03, 1.40608839e-03, 2.31763305e-03, 2.11252318e-03,
            1.78816941e-03, 1.97792180e-03, 2.27694981e-03, 2.34840616e-03, 2.34230880e-03,
            1.69236426e-03, 2.08390673e-03, 1.71818313e-03, 1.76620837e-03, 1.48650593e-03,
            2.15029430e-03, 1.53547025e-03, 2.10761213e-03, 1.69095166e-03, 3.23284017e-03,
            2.34569983e-03, 2.65531255e-03, 2.34567502e-03, 8.06321188e-04, 3.32709774e-03,
            5.27641571e-03, 1.01128365e+00, 1.99717805e-03, 2.66097384e-03, 1.49670104e-04
        ], [
            2.15443549e-03, 8.42269407e-04, 1.68785007e-03, 1.31471365e-03, 2.16823038e-03,
            1.38488461e-03, 1.20562317e-03, 8.26596616e-04, 1.30521877e-03, 1.24662235e-03,
            1.09313411e-03, 1.14506363e-03, 1.33895993e-03, 1.16948824e-03, 1.24369731e-03,
            1.24463562e-03, 8.89530951e-04, 1.16365143e-03, 1.10135380e-03, 1.00148912e-03,
            8.25908669e-04, 1.82482752e-03, 1.24702806e-03, 1.07494820e-03, 1.18127143e-03,
            1.28883769e-03, 1.22690081e-03, 1.04318445e-03, 3.60997944e-04, 1.28925810e-03,
            5.85823710e-03, 1.86306022e-03, 1.02608642e+00, 1.86233953e-03, 1.14558507e-04
        ], [
            9.58544228e-03, 1.48069787e-02, 1.84905619e-02, 1.94635821e-02, 2.15182386e-02,
            1.80909547e-02, 3.07799548e-02, 1.41581620e-02, 2.27369139e-02, 2.06224556e-02,
            1.81660216e-02, 2.04456836e-02, 2.01556799e-02, 1.88916819e-02, 1.98330961e-02,
            1.92301869e-02, 1.77955222e-02, 1.90858066e-02, 1.88552872e-02, 1.63237129e-02,
            1.66672109e-02, 2.40822634e-02, 1.95282962e-02, 1.82646946e-02, 2.27131497e-02,
            2.35727657e-02, 3.25979481e-02, 2.18774673e-02, 1.31612569e-02, 4.04860565e-02,
            2.57430895e-02, 1.65476667e-02, 2.43578332e-02, 1.09698303e+00, 3.38644611e-03
        ], [
            4.87081991e-05, 5.51288856e-05, 8.01461508e-05, 1.27091389e-04, 9.04820185e-05,
            9.03063014e-05, 1.02452658e-04, 5.78432215e-05, 9.07233764e-05, 1.02409679e-04,
            7.21083238e-05, 7.76195984e-05, 8.23543889e-05, 7.27279048e-05, 1.11684991e-04,
            7.91787551e-05, 6.15820626e-05, 1.27170575e-04, 1.12694728e-04, 5.06765822e-05,
            7.95970486e-05, 1.01608096e-04, 1.32607515e-04, 6.82763419e-05, 8.16108091e-05,
            7.45916628e-05, 9.61986155e-05, 1.06657104e-04, 2.94175618e-04, 1.14430936e-04,
            1.12792260e-04, 6.78900950e-05, 1.31197609e-04, 1.80070916e-04, 1.00009558e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 396)
    return ret


@cache('run')
def leontief_matrix_x2002x():
    """
    leontief matrix x2002x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17709342e+00, 4.80351142e-03, 3.60603400e-01, 7.51606409e-02, 1.03332048e-01,
            1.99793526e-01, 2.83219925e-02, 6.64419487e-03, 1.79185656e-02, 2.98087705e-02,
            8.78426692e-03, 7.34206896e-03, 8.11876687e-03, 7.25519862e-03, 8.74946518e-03,
            3.36189737e-02, 5.04378544e-03, 2.51359621e-02, 5.83194536e-03, 9.30840923e-03,
            8.18164730e-03, 1.02788382e-01, 6.91441094e-03, 9.47717184e-03, 8.17388216e-03,
            1.21635401e-02, 5.09308048e-03, 4.11306191e-03, 3.39724424e-03, 6.24396722e-03,
            9.62452291e-03, 8.49373137e-03, 1.21866168e-02, 1.13937382e-02, 1.97991282e-02
        ], [
            2.35432358e-02, 1.09763368e+00, 2.43588107e-02, 2.83125767e-02, 2.32578670e-02,
            2.95876673e-02, 2.63884462e-02, 5.19069381e-01, 8.20305377e-02, 4.09808157e-02,
            1.10753054e-01, 9.52880418e-02, 3.57654922e-02, 2.46446525e-02, 3.03403920e-02,
            3.02686338e-02, 1.47415295e-01, 5.40867643e-02, 1.50329732e-02, 1.06366108e-02,
            1.04821968e-02, 1.69103242e-02, 4.21761053e-02, 6.05284260e-02, 4.59009114e-02,
            2.26009873e-02, 1.03636784e-02, 6.21406070e-03, 8.51384470e-03, 1.05204672e-02,
            1.75479219e-02, 1.23951301e-02, 1.49863841e-02, 1.59420139e-02, 6.85648350e-02
        ], [
            7.77654407e-02, 3.96040646e-03, 1.20012660e+00, 1.64205646e-02, 1.35596576e-01,
            1.99896356e-02, 1.17689521e-02, 5.08340477e-03, 2.01810486e-02, 1.38726059e-02,
            7.73837799e-03, 7.09042643e-03, 7.87931987e-03, 7.28227609e-03, 7.67309806e-03,
            1.01442286e-02, 4.82718082e-03, 7.75735236e-03, 6.44866830e-03, 9.77903546e-03,
            8.80884749e-03, 1.83635427e-01, 6.27105277e-03, 1.33638919e-02, 1.17206931e-02,
            9.25864083e-03, 6.53422955e-03, 5.29025543e-03, 2.33836282e-03, 7.62630099e-03,
            1.27673391e-02, 1.22048136e-02, 2.10932817e-02, 1.39390361e-02, 5.53921466e-03
        ], [
            5.47123676e-03, 3.55602374e-03, 6.94279727e-03, 1.43631231e+00, 9.32568012e-02,
            1.04275949e-02, 1.32787272e-02, 4.04501893e-03, 9.32334346e-03, 2.29884861e-02,
            1.01792833e-02, 6.90994142e-03, 9.28842478e-03, 7.80763034e-03, 1.65619785e-02,
            3.56960463e-02, 2.97064577e-03, 7.66124582e-03, 5.76365947e-03, 5.29852767e-03,
            6.08556987e-03, 6.65728827e-03, 5.07591350e-03, 6.19937427e-03, 4.92030981e-03,
            5.03312616e-03, 3.39999329e-03, 2.34198898e-03, 1.50116351e-03, 3.77384106e-03,
            7.23149895e-03, 2.83978136e-03, 6.46095747e-03, 1.19709607e-02, 6.81865664e-04
        ], [
            4.54503060e-04, 3.60151543e-04, 6.87112880e-04, 1.11843839e-02, 1.27107124e+00,
            1.16133137e-03, 1.39731996e-03, 3.90817389e-04, 7.88519582e-04, 2.66363451e-03,
            8.51229939e-04, 8.08164336e-04, 9.59224308e-04, 8.99459778e-04, 2.91338908e-03,
            6.00047646e-03, 4.16058183e-04, 6.77759084e-04, 1.04356570e-03, 7.62951411e-04,
            6.27483452e-04, 5.09273312e-04, 6.13804752e-04, 6.34228458e-04, 5.86268032e-04,
            5.73337312e-04, 5.97479171e-04, 2.23030289e-04, 1.36780398e-04, 3.95686915e-04,
            9.45180052e-04, 3.10674062e-04, 4.22290029e-04, 8.85035506e-04, 5.83178125e-05
        ], [
            5.10203631e-03, 4.18137845e-03, 6.53993781e-03, 6.13245975e-03, 6.96331368e-03,
            1.27417091e+00, 1.98445731e-02, 3.55662668e-03, 5.73864115e-03, 8.93919760e-03,
            1.24980025e-02, 9.61198909e-03, 8.95638462e-03, 6.53178767e-03, 1.00266183e-02,
            9.34974816e-02, 4.28642352e-03, 4.76472416e-02, 4.08117336e-03, 3.88880652e-03,
            3.60525998e-03, 6.20410535e-03, 4.44208326e-03, 3.95153953e-03, 3.61399339e-03,
            7.32842272e-03, 3.47644085e-03, 2.45557545e-03, 5.47857120e-03, 3.60962693e-03,
            5.09559400e-03, 4.90946277e-03, 3.75578240e-03, 6.94516391e-03, 7.73024951e-04
        ], [
            1.46575658e-02, 1.21093124e-02, 5.27747455e-02, 3.57723099e-02, 4.33625863e-02,
            3.52444680e-02, 1.31161079e+00, 1.37053790e-02, 4.27548699e-02, 4.53866427e-02,
            4.25529708e-02, 2.42950471e-02, 3.11500822e-02, 3.57703098e-02, 2.77638254e-02,
            4.30460541e-02, 1.45812082e-02, 2.34991508e-02, 2.25890947e-02, 2.65401520e-02,
            2.47285179e-02, 2.81271980e-02, 1.96458886e-02, 1.84060222e-02, 2.22427955e-02,
            3.14712327e-02, 2.71642608e-02, 3.01174020e-02, 9.39902511e-03, 4.04456731e-02,
            3.45209953e-02, 2.97770788e-02, 2.39715419e-02, 4.69720323e-02, 2.02090642e-03
        ], [
            2.39050914e-02, 1.90283979e-02, 1.96237987e-02, 2.27191983e-02, 2.01614583e-02,
            2.18520680e-02, 1.75622476e-02, 1.06648515e+00, 5.38379363e-02, 3.03650183e-02,
            3.51994437e-02, 2.67940444e-02, 1.78545308e-02, 1.41351185e-02, 1.60550670e-02,
            1.64456080e-02, 3.09333525e-02, 2.74823263e-02, 1.19268969e-02, 1.17499068e-02,
            9.15252386e-03, 1.27063485e-02, 6.39615375e-02, 1.05899743e-01, 7.72023620e-02,
            2.77676641e-02, 9.07862883e-03, 5.47331494e-03, 4.41092801e-03, 1.01642668e-02,
            1.57674481e-02, 7.24471003e-03, 1.12229711e-02, 1.23885683e-02, 1.87548992e-03
        ], [
            6.84975995e-02, 2.50157918e-02, 5.26664065e-02, 1.49402220e-01, 1.03031439e-01,
            8.02946261e-02, 7.85726949e-02, 3.80134082e-02, 1.32230736e+00, 3.19251050e-01,
            6.78099623e-02, 4.74253044e-02, 5.01929933e-02, 5.77915210e-02, 5.93177997e-02,
            6.76102055e-02, 2.07187859e-02, 4.26380325e-02, 2.75614707e-02, 1.32573497e-02,
            1.17520232e-02, 2.44287459e-02, 1.87546341e-02, 2.07769893e-02, 1.79297018e-02,
            1.82184130e-02, 1.25444291e-02, 7.94048082e-03, 7.02631137e-03, 1.64494284e-02,
            2.49519217e-02, 1.21048971e-02, 9.69747229e-02, 2.65485640e-02, 3.43680437e-03
        ], [
            1.41103674e-02, 1.40403538e-02, 3.37346249e-02, 2.79586347e-02, 6.44754475e-02,
            2.43414327e-02, 3.20190724e-02, 1.39026251e-02, 3.60935448e-02, 1.16203756e+00,
            2.43899087e-02, 2.23947655e-02, 5.21676829e-02, 5.10728894e-02, 7.26979680e-02,
            4.86150363e-02, 1.05571461e-02, 3.39248865e-02, 3.20297181e-02, 1.05207340e-02,
            1.03525955e-02, 1.45877834e-02, 2.04029390e-02, 1.48691718e-02, 1.39141152e-02,
            1.53858154e-02, 1.07796731e-02, 4.67543591e-03, 4.49758132e-03, 9.01902379e-03,
            1.10183033e-02, 5.20275056e-03, 1.33212228e-02, 1.14243963e-02, 1.52730438e-03
        ], [
            6.46045186e-03, 8.83135455e-03, 1.29410724e-02, 8.83023424e-03, 8.51348518e-03,
            1.67253555e-02, 6.39132346e-03, 8.56961007e-03, 1.33671172e-02, 1.53375743e-02,
            1.13566279e+00, 2.27122163e-02, 1.66937431e-02, 2.47067536e-02, 1.99763829e-02,
            1.42290235e-02, 8.04774282e-03, 9.72369503e-02, 8.61057306e-03, 4.00823489e-03,
            3.75877031e-03, 8.70691793e-03, 5.91681593e-03, 5.52007729e-03, 4.98857401e-03,
            6.28118620e-03, 5.63082377e-03, 2.55445623e-03, 7.23338408e-03, 4.28630301e-03,
            6.05969608e-03, 5.03635301e-03, 6.14729496e-03, 6.29753150e-03, 8.98601612e-04
        ], [
            2.34422758e-02, 5.14576945e-02, 4.53451437e-02, 3.63836699e-02, 4.50729483e-02,
            7.02274553e-02, 3.85973388e-02, 4.37711472e-02, 5.01982139e-02, 7.23779780e-02,
            8.06770406e-02, 1.49257997e+00, 3.09931781e-01, 1.55865311e-01, 2.40437160e-01,
            1.48359531e-01, 3.55519866e-02, 1.68741515e-01, 6.67394991e-02, 1.84288876e-02,
            1.64581104e-02, 2.46525886e-02, 3.54081910e-02, 4.22608318e-02, 3.48101245e-02,
            3.07616143e-02, 2.44085465e-02, 1.02351532e-02, 1.52077469e-02, 1.94701908e-02,
            2.66483419e-02, 1.33737158e-02, 1.92373762e-02, 2.35214127e-02, 4.81334273e-03
        ], [
            1.41068607e-02, 2.32509463e-02, 1.49357079e-02, 2.16480033e-02, 1.74423958e-02,
            2.10875944e-02, 1.66561255e-02, 2.05420266e-02, 1.82381270e-02, 2.46988199e-02,
            3.12448622e-02, 3.56057170e-02, 1.15707008e+00, 3.26447713e-02, 6.24797197e-02,
            2.16966648e-02, 1.97200624e-02, 3.27819759e-02, 3.42944820e-02, 8.51672458e-03,
            6.62176936e-03, 8.83111620e-03, 1.46580218e-02, 2.15338211e-02, 1.46138475e-02,
            1.24256972e-02, 8.72995708e-03, 4.52299512e-03, 4.74942188e-03, 8.13833941e-03,
            1.25571722e-02, 5.82468627e-03, 7.75490439e-03, 9.53814535e-03, 1.99988084e-03
        ], [
            1.43309274e-02, 2.28789405e-02, 2.29923442e-02, 3.09412450e-02, 2.90896114e-02,
            3.08186170e-02, 3.61630459e-02, 2.59986133e-02, 3.62400482e-02, 4.56665054e-02,
            3.83252916e-02, 5.40984788e-02, 1.68581229e-01, 1.45515893e+00, 1.38267637e-01,
            5.13601714e-02, 4.02121241e-02, 6.89110655e-02, 6.45651887e-02, 2.40850245e-02,
            2.10578230e-02, 1.98781163e-02, 2.84811070e-02, 2.98615914e-02, 2.94789429e-02,
            2.93296100e-02, 7.39184661e-02, 1.76347714e-02, 1.09063807e-02, 3.56866010e-02,
            3.50054058e-02, 1.72994511e-02, 4.31285513e-02, 2.95509331e-02, 3.12440030e-03
        ], [
            1.39863522e-02, 1.61355302e-02, 1.63431467e-02, 1.67757988e-02, 1.75742534e-02,
            1.85468180e-02, 1.45540474e-02, 1.53611807e-02, 1.46952259e-02, 1.89478363e-02,
            1.83011096e-02, 2.24138496e-02, 3.57996715e-02, 2.01102890e-02, 1.44142028e+00,
            1.78979622e-02, 1.21290993e-02, 1.86522126e-02, 1.35992743e-01, 1.48699493e-02,
            1.13925259e-02, 1.02596802e-02, 6.20921338e-02, 7.93736846e-02, 8.76932255e-02,
            3.06029137e-02, 1.24922780e-02, 6.73637555e-03, 4.60100633e-03, 1.26693341e-02,
            2.98424935e-02, 6.52190979e-03, 7.89302463e-03, 1.53876680e-02, 1.76599652e-03
        ], [
            2.27273750e-03, 2.82399161e-03, 3.84207050e-03, 9.60565921e-03, 6.92499869e-03,
            8.90052931e-03, 7.36989680e-03, 2.89720870e-03, 4.73501837e-03, 6.26357499e-03,
            6.14304327e-03, 1.62938035e-02, 9.90880242e-03, 7.45930256e-03, 1.21571904e-02,
            1.04676592e+00, 3.28792314e-03, 9.49291807e-03, 5.23042837e-03, 2.75332858e-03,
            2.88282438e-03, 4.55860062e-03, 4.12765009e-03, 3.46310245e-03, 4.04185927e-03,
            4.02797324e-03, 2.90517560e-03, 2.41647107e-03, 2.82982654e-03, 3.78523735e-03,
            5.97347589e-03, 4.37274715e-03, 4.67812332e-03, 8.03959880e-03, 2.42500827e-03
        ], [
            2.36871254e-02, 4.06077875e-02, 3.44910409e-02, 4.48331395e-02, 3.19805032e-02,
            4.08782619e-02, 4.65605511e-02, 4.06824002e-02, 5.28037492e-02, 5.03154296e-02,
            6.48245641e-02, 6.02490005e-02, 3.77449526e-02, 2.98406511e-02, 3.56290145e-02,
            2.96246815e-02, 1.11879922e+00, 2.85789766e-02, 2.35637705e-02, 1.45004763e-02,
            2.19306189e-02, 3.88862233e-02, 2.84910257e-02, 1.97524356e-02, 1.90836617e-02,
            2.79602489e-02, 1.93110806e-02, 1.18284187e-02, 1.56681594e-02, 1.53423842e-02,
            2.64939893e-02, 2.80494392e-02, 2.48576723e-02, 2.84902359e-02, 3.56665242e-03
        ], [
            8.83860583e-03, 1.44306590e-02, 1.08243816e-02, 1.15077291e-02, 1.01628506e-02,
            1.20112986e-02, 1.21913084e-02, 1.28344872e-02, 1.22531568e-02, 1.19600168e-02,
            1.65809575e-02, 1.41819329e-02, 1.22060726e-02, 1.01531272e-02, 1.10549626e-02,
            9.63683810e-03, 3.17006553e-02, 1.05803473e+00, 1.08034729e-02, 9.20432502e-03,
            1.07900412e-02, 1.36672582e-02, 1.70817983e-02, 1.22619446e-02, 1.10994330e-02,
            2.18044635e-02, 1.81986962e-02, 1.13575772e-02, 4.72103136e-02, 1.11774164e-02,
            2.45029126e-02, 2.02611812e-02, 1.24655306e-02, 1.80166936e-02, 2.02757481e-03
        ], [
            9.57870761e-03, 7.83519752e-03, 1.50041203e-02, 1.44292571e-02, 1.74222196e-02,
            1.39764736e-02, 1.20414669e-02, 8.98416979e-03, 1.34484686e-02, 1.46809848e-02,
            1.45336068e-02, 1.44234464e-02, 1.34268355e-02, 1.14582452e-02, 1.42298692e-02,
            1.28106761e-02, 1.32371971e-02, 1.42221760e-02, 1.01541941e+00, 7.40031111e-03,
            7.15108170e-03, 1.05343366e-02, 2.19210178e-02, 8.79963639e-03, 9.59379283e-03,
            1.40189066e-02, 7.71926619e-03, 5.87636892e-03, 4.01680647e-03, 9.63821827e-03,
            8.31346363e-03, 5.37795197e-03, 7.59143705e-03, 1.07514294e-02, 8.71922909e-04
        ], [
            6.08662527e-02, 4.37198251e-02, 1.13920853e-01, 1.08290088e-01, 1.31103859e-01,
            1.05387469e-01, 9.33043216e-02, 7.58601438e-02, 9.30653923e-02, 1.00963867e-01,
            8.11346198e-02, 9.62635998e-02, 1.00442376e-01, 9.25355201e-02, 1.07496977e-01,
            8.75564664e-02, 4.65265582e-02, 8.19706292e-02, 6.33706647e-02, 1.04027004e+00,
            2.60347486e-02, 7.72220323e-02, 4.62320778e-02, 5.15449247e-02, 4.85333884e-02,
            4.42791297e-02, 2.85716362e-02, 1.66430643e-02, 1.30695110e-02, 3.00218291e-02,
            3.79289337e-02, 2.24767601e-02, 4.76849236e-02, 3.72346691e-02, 4.93467194e-03
        ], [
            2.95750324e-02, 1.77455882e-02, 5.40824161e-02, 5.47851877e-02, 7.28805332e-02,
            4.24571928e-02, 3.69472023e-02, 2.47236873e-02, 4.09056230e-02, 4.27157861e-02,
            3.56204427e-02, 3.67460167e-02, 3.83930465e-02, 3.35224685e-02, 4.52358656e-02,
            4.21375334e-02, 1.86355600e-02, 4.73541177e-02, 2.81705872e-02, 1.37546608e-02,
            1.01406657e+00, 4.08428381e-02, 2.43437586e-02, 2.17483661e-02, 2.20537959e-02,
            2.03466935e-02, 1.68519490e-02, 1.11714072e-02, 8.12085567e-03, 1.75645940e-02,
            1.79244000e-02, 1.14605560e-02, 2.13827583e-02, 2.09873792e-02, 2.43730885e-03
        ], [
            5.81029900e-03, 7.75589622e-03, 1.17474414e-02, 1.36079024e-02, 1.37779320e-02,
            1.32634234e-02, 1.58091482e-02, 8.12958614e-03, 1.32381409e-02, 1.52207256e-02,
            1.35813147e-02, 1.37530114e-02, 1.60195488e-02, 1.44750837e-02, 1.29240707e-02,
            1.13706034e-02, 1.13079954e-02, 1.27530437e-02, 1.20579933e-02, 1.33969709e-02,
            1.03220045e-02, 1.01684121e+00, 1.42812343e-02, 1.34426716e-02, 3.06764623e-02,
            2.56414087e-02, 1.59470623e-02, 1.65600236e-02, 5.98506160e-03, 2.08704240e-02,
            1.64925197e-02, 9.50226618e-03, 1.40551839e-02, 1.88747546e-02, 9.60763648e-04
        ], [
            2.66275702e-02, 3.32896974e-02, 4.76952860e-02, 4.63884127e-02, 4.97750119e-02,
            4.90684088e-02, 4.08121063e-02, 5.90035878e-02, 4.10507334e-02, 4.24406169e-02,
            5.77525690e-02, 4.38312964e-02, 3.59821788e-02, 2.86393933e-02, 3.73843841e-02,
            3.73315471e-02, 3.76042804e-02, 3.77035877e-02, 2.75758962e-02, 2.79770718e-02,
            1.95533502e-02, 2.57031582e-02, 1.06230885e+00, 7.39512226e-02, 3.51531251e-02,
            7.77352589e-02, 1.82385683e-02, 1.26974144e-02, 6.99986013e-03, 1.47130728e-02,
            2.11624758e-02, 1.68106535e-02, 1.74130853e-02, 2.05721671e-02, 3.30936753e-03
        ], [
            7.41273584e-03, 9.34099565e-03, 1.10219149e-02, 1.13665717e-02, 1.34654485e-02,
            1.60752790e-02, 9.23166549e-03, 1.52001686e-02, 9.39580089e-03, 1.02015425e-02,
            1.79726290e-02, 1.52096933e-02, 1.04634454e-02, 8.13033197e-03, 9.01361281e-03,
            8.66298208e-03, 7.44994910e-03, 1.12521047e-02, 4.23958919e-03, 5.28743733e-03,
            3.38144190e-03, 5.51529876e-03, 6.78678640e-03, 1.12097389e+00, 6.93565887e-03,
            9.71940211e-03, 2.85694594e-03, 1.93721433e-03, 1.86944847e-03, 3.02036849e-03,
            3.82871547e-03, 2.66166844e-03, 2.85047466e-03, 3.80613705e-03, 8.31136542e-04
        ], [
            2.38946276e-03, 3.81904656e-03, 4.39139121e-03, 5.36237943e-03, 6.63166720e-03,
            5.07178413e-03, 5.82152246e-03, 3.69262576e-03, 4.97553671e-03, 5.43658737e-03,
            4.76266847e-03, 4.68980814e-03, 5.78820966e-03, 5.43209409e-03, 5.34371859e-03,
            4.09517777e-03, 2.95343648e-03, 3.99353130e-03, 3.95617757e-03, 5.11030910e-03,
            3.06862688e-03, 3.42929921e-03, 5.26040419e-03, 1.25292938e-02, 1.03935935e+00,
            2.23853715e-02, 5.73850129e-03, 5.05322862e-03, 1.58812833e-03, 6.22787355e-03,
            5.83125853e-03, 3.10832892e-03, 3.04100874e-03, 6.45578245e-03, 3.92815930e-04
        ], [
            1.17408317e-02, 1.34281678e-02, 2.08925211e-02, 1.81134491e-02, 2.11381141e-02,
            2.22087622e-02, 2.39751710e-02, 2.12230384e-02, 1.84612133e-02, 1.96225820e-02,
            2.30776460e-02, 2.00493870e-02, 1.85962623e-02, 1.55508940e-02, 1.94287634e-02,
            1.63195301e-02, 1.30059103e-02, 1.67229177e-02, 1.96827380e-02, 2.89045775e-02,
            1.67059373e-02, 1.75954133e-02, 5.45436549e-02, 1.51199483e-01, 1.24033657e-01,
            1.12999436e+00, 1.13216522e-02, 8.86716535e-03, 4.08926220e-03, 1.16959333e-02,
            1.19865304e-02, 6.83837769e-03, 9.60322772e-03, 1.38626880e-02, 1.34619468e-03
        ], [
            1.34401867e-02, 1.74446712e-02, 2.35600011e-02, 2.70046881e-02, 2.95199430e-02,
            2.53180344e-02, 3.68948763e-02, 1.90283765e-02, 2.95651367e-02, 2.83503472e-02,
            2.61698695e-02, 2.67749341e-02, 3.02432779e-02, 2.87167459e-02, 2.66976140e-02,
            2.50680707e-02, 2.01474302e-02, 3.24344528e-02, 3.12505500e-02, 3.50849799e-02,
            3.50622115e-02, 3.00880658e-02, 3.37004115e-02, 4.06220046e-02, 4.76100121e-02,
            4.70161074e-02, 1.15629629e+00, 4.97936587e-02, 1.43864260e-02, 4.45525242e-02,
            3.78637333e-02, 1.86590021e-02, 2.67538686e-02, 4.16077939e-02, 2.58182721e-03
        ], [
            5.06455689e-02, 5.85884120e-02, 6.32266481e-02, 7.42422051e-02, 6.68009428e-02,
            6.49405212e-02, 7.15267080e-02, 5.73987131e-02, 6.35322781e-02, 6.60074212e-02,
            7.05031573e-02, 6.71154629e-02, 6.66854483e-02, 5.78637138e-02, 6.37388510e-02,
            6.30767978e-02, 5.78688893e-02, 6.58326920e-02, 5.60073495e-02, 6.74278622e-02,
            6.46594071e-02, 6.45767604e-02, 9.00874293e-02, 9.62337427e-02, 8.32904786e-02,
            6.84144936e-02, 5.32389796e-02, 1.26868287e+00, 1.05912144e-01, 7.33868757e-02,
            5.53046196e-02, 3.10422206e-02, 5.51964353e-02, 8.38495979e-02, 1.47115289e-02
        ], [
            1.49364663e-02, 1.47142949e-02, 2.22109221e-02, 2.29043484e-02, 2.40890473e-02,
            2.33820564e-02, 2.85152940e-02, 1.56460854e-02, 2.15279533e-02, 2.38387329e-02,
            2.03228735e-02, 2.23535592e-02, 2.38922681e-02, 2.11533720e-02, 2.35280873e-02,
            2.13707335e-02, 1.60853024e-02, 2.65439630e-02, 3.64166006e-02, 3.29372716e-02,
            5.10011697e-02, 4.39083663e-02, 2.26025721e-02, 3.70077580e-02, 2.84839446e-02,
            3.85985797e-02, 2.95967080e-02, 3.78814246e-02, 1.04091696e+00, 3.79569161e-02,
            2.62215191e-02, 2.71946316e-02, 4.63273219e-02, 4.09466270e-02, 1.86658932e-03
        ], [
            5.08646954e-02, 6.81591405e-02, 1.07208279e-01, 1.03488018e-01, 1.00193057e-01,
            9.41579396e-02, 1.47366419e-01, 7.28955233e-02, 1.51858077e-01, 1.25811565e-01,
            1.01387062e-01, 1.04739420e-01, 1.27066336e-01, 1.34328794e-01, 1.32052401e-01,
            9.27692358e-02, 8.66557670e-02, 1.22663147e-01, 1.06209865e-01, 9.58652470e-02,
            9.97676823e-02, 9.74275131e-02, 9.84136102e-02, 1.03958021e-01, 1.56842334e-01,
            1.30650148e-01, 1.25437985e-01, 1.58563286e-01, 6.47885127e-02, 1.20235023e+00,
            1.27360311e-01, 6.06032435e-02, 1.07676166e-01, 1.35143491e-01, 1.03180195e-02
        ], [
            3.26640182e-03, 5.05455133e-03, 5.03711470e-03, 4.33749758e-03, 4.09227595e-03,
            6.83441761e-03, 6.83081323e-03, 4.53777067e-03, 6.28398636e-03, 5.73511749e-03,
            5.35875532e-03, 6.22214022e-03, 4.51716494e-03, 4.12167458e-03, 4.75510822e-03,
            4.76880867e-03, 7.21016070e-03, 4.55400231e-03, 4.55501398e-03, 3.04551716e-03,
            4.20314798e-03, 6.07531631e-03, 5.31377154e-03, 9.50467728e-03, 1.24825400e-02,
            6.17266160e-03, 5.08118015e-03, 3.34420911e-03, 4.18515954e-03, 4.80906878e-03,
            1.00925970e+00, 5.74599189e-03, 6.50917043e-03, 1.08753396e-02, 4.72341683e-04
        ], [
            1.21235466e-03, 1.12682853e-03, 1.68326374e-03, 1.72823683e-03, 1.81740277e-03,
            1.57939523e-03, 1.98302361e-03, 1.37935165e-03, 2.38741376e-03, 2.10387717e-03,
            1.83292443e-03, 1.93246532e-03, 2.31545154e-03, 2.23459960e-03, 2.30959762e-03,
            1.49998241e-03, 1.64980316e-03, 1.66856463e-03, 1.75605906e-03, 1.43217965e-03,
            1.69689468e-03, 1.51031204e-03, 2.09173851e-03, 1.58433951e-03, 3.15949024e-03,
            2.31344319e-03, 2.50887858e-03, 2.21329378e-03, 7.95351055e-04, 3.34264227e-03,
            5.02044860e-03, 1.01254990e+00, 1.93056846e-03, 2.62565014e-03, 1.39283202e-04
        ], [
            2.15112604e-03, 7.68736326e-04, 1.68851702e-03, 1.29608863e-03, 2.25058920e-03,
            1.36968403e-03, 1.18597950e-03, 8.09482752e-04, 1.29439833e-03, 1.23004051e-03,
            1.08347349e-03, 1.12081899e-03, 1.32169644e-03, 1.10361107e-03, 1.19552419e-03,
            1.16459302e-03, 8.20829828e-04, 1.12313769e-03, 1.07450072e-03, 9.85205295e-04,
            8.10351000e-04, 1.78219890e-03, 1.18358134e-03, 1.01984739e-03, 1.12301004e-03,
            1.23736733e-03, 1.18945042e-03, 1.01113007e-03, 3.53231220e-04, 1.22174819e-03,
            5.32904076e-03, 1.83218628e-03, 1.02594835e+00, 1.75139355e-03, 1.06927287e-04
        ], [
            9.57326921e-03, 1.39606686e-02, 1.83039931e-02, 1.91687742e-02, 2.14891937e-02,
            1.81797514e-02, 3.01802071e-02, 1.41275699e-02, 2.27719736e-02, 2.06770704e-02,
            1.83185576e-02, 2.03984549e-02, 2.02713470e-02, 1.80115777e-02, 1.95276133e-02,
            1.68324441e-02, 1.51071418e-02, 1.88423004e-02, 1.83746955e-02, 1.63163627e-02,
            1.65414592e-02, 2.40009002e-02, 1.95235051e-02, 1.78415633e-02, 2.12990123e-02,
            2.26990942e-02, 3.32349927e-02, 2.15205963e-02, 1.30961893e-02, 3.92801408e-02,
            2.62196246e-02, 1.66205461e-02, 2.36097590e-02, 1.09567758e+00, 3.49391590e-03
        ], [
            4.58467061e-05, 4.75028257e-05, 7.53905398e-05, 1.27820513e-04, 8.32554601e-05,
            8.33302557e-05, 9.33268660e-05, 5.33530714e-05, 8.34624202e-05, 9.56290362e-05,
            6.65491873e-05, 6.95182511e-05, 7.84071469e-05, 6.23296142e-05, 1.08004044e-04,
            5.83851201e-05, 4.21672796e-05, 1.14663652e-04, 9.66790345e-05, 4.44163857e-05,
            7.25774561e-05, 9.53917837e-05, 1.11039068e-04, 5.93617191e-05, 6.97524535e-05,
            6.54592746e-05, 7.79636427e-05, 9.29698689e-05, 2.71633778e-04, 1.03284840e-04,
            1.06473216e-04, 6.29505123e-05, 1.05777359e-04, 1.58961571e-04, 1.00010105e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 397)
    return ret


@cache('run')
def leontief_matrix_x2003x():
    """
    leontief matrix x2003x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17385679e+00, 4.68124904e-03, 3.59831224e-01, 8.21291166e-02, 1.14865729e-01,
            1.94114745e-01, 2.96927432e-02, 7.04004375e-03, 1.89509653e-02, 2.93029282e-02,
            9.07055229e-03, 7.53843659e-03, 8.10745511e-03, 7.22186382e-03, 9.01630099e-03,
            3.45332104e-02, 4.75513695e-03, 2.26015499e-02, 5.94722435e-03, 9.11522601e-03,
            8.09402509e-03, 1.03972322e-01, 7.40725627e-03, 9.49584439e-03, 8.55467288e-03,
            1.23774237e-02, 5.10992095e-03, 4.21255093e-03, 3.20070297e-03, 6.41410477e-03,
            1.04350089e-02, 9.04998811e-03, 1.24771761e-02, 1.16023940e-02, 2.21779895e-02
        ], [
            2.36881404e-02, 1.10923628e+00, 2.50156082e-02, 3.09489553e-02, 2.55971311e-02,
            3.16080478e-02, 2.82653143e-02, 5.45896216e-01, 8.88604857e-02, 4.43078242e-02,
            1.17726775e-01, 1.03706192e-01, 3.81032618e-02, 2.59315960e-02, 3.26021189e-02,
            3.11320078e-02, 1.58888890e-01, 5.68197907e-02, 1.60070242e-02, 1.07259577e-02,
            1.09234466e-02, 1.77435037e-02, 4.37028983e-02, 6.01970053e-02, 4.92352211e-02,
            2.51055207e-02, 1.09994837e-02, 6.55633927e-03, 8.07025019e-03, 1.10300162e-02,
            1.87943803e-02, 1.30853967e-02, 1.59674841e-02, 1.70545052e-02, 6.77092457e-02
        ], [
            8.12607679e-02, 4.28664302e-03, 1.20635460e+00, 1.91477915e-02, 1.44946293e-01,
            2.09803814e-02, 1.25611954e-02, 5.63320649e-03, 2.18548189e-02, 1.50610781e-02,
            8.78254446e-03, 7.86328336e-03, 8.38887400e-03, 7.72996490e-03, 8.46158473e-03,
            1.11759729e-02, 4.81696869e-03, 8.33105142e-03, 6.75381420e-03, 1.00546676e-02,
            9.03495988e-03, 1.89158293e-01, 6.79996694e-03, 1.36398602e-02, 1.27736779e-02,
            9.76673069e-03, 6.73126766e-03, 5.57476437e-03, 2.41953417e-03, 7.98415285e-03,
            1.43377130e-02, 1.31550580e-02, 2.19899461e-02, 1.45894434e-02, 5.44373377e-03
        ], [
            5.47863943e-03, 3.50094046e-03, 7.07728954e-03, 1.45262453e+00, 1.00997703e-01,
            1.04662599e-02, 1.31938779e-02, 3.68035165e-03, 9.64822462e-03, 2.44977461e-02,
            9.77885117e-03, 7.10399192e-03, 8.91999032e-03, 7.76767265e-03, 1.66301740e-02,
            3.63859494e-02, 3.04555587e-03, 7.89072493e-03, 5.78478940e-03, 5.29519405e-03,
            5.89538574e-03, 6.90303954e-03, 5.31437923e-03, 6.83147500e-03, 5.23451560e-03,
            5.10737740e-03, 3.29668787e-03, 2.31081965e-03, 1.46728928e-03, 3.93473386e-03,
            6.90380902e-03, 2.85779422e-03, 6.70310115e-03, 1.10149103e-02, 6.28867306e-04
        ], [
            4.63767298e-04, 3.41057705e-04, 6.57551238e-04, 1.46837395e-02, 1.27129412e+00,
            1.24071927e-03, 1.32100949e-03, 3.69594419e-04, 7.93723254e-04, 2.37646320e-03,
            9.09886382e-04, 8.02773518e-04, 9.24311467e-04, 9.20704449e-04, 3.33386234e-03,
            6.40675020e-03, 3.96401691e-04, 6.92462831e-04, 1.07116327e-03, 7.49239888e-04,
            6.06674178e-04, 5.09740780e-04, 6.41470521e-04, 6.43500332e-04, 6.14230192e-04,
            5.52052742e-04, 5.85561326e-04, 2.20734158e-04, 1.30309601e-04, 3.96160789e-04,
            8.71997263e-04, 3.06904671e-04, 4.29881600e-04, 8.94601402e-04, 5.52201887e-05
        ], [
            4.86735529e-03, 4.26422173e-03, 6.53902039e-03, 6.13255676e-03, 7.09228096e-03,
            1.28952822e+00, 2.06247360e-02, 3.55173143e-03, 5.83545115e-03, 9.06138881e-03,
            1.33422210e-02, 1.01966960e-02, 9.30575815e-03, 6.66949952e-03, 1.02462544e-02,
            9.88264776e-02, 4.24534605e-03, 4.85280849e-02, 4.26537109e-03, 3.81585013e-03,
            3.54646203e-03, 6.19517641e-03, 4.50292706e-03, 4.03522794e-03, 3.76754084e-03,
            7.21530496e-03, 3.47931908e-03, 2.45759554e-03, 5.26249602e-03, 3.67490152e-03,
            5.39148436e-03, 5.15819883e-03, 3.98016599e-03, 7.01258631e-03, 7.74578126e-04
        ], [
            1.43277909e-02, 1.20126625e-02, 5.16104123e-02, 3.69831226e-02, 4.44557994e-02,
            3.63267140e-02, 1.31847067e+00, 1.28114441e-02, 4.17275639e-02, 4.49113299e-02,
            4.26399502e-02, 2.38533504e-02, 2.99167185e-02, 3.40865029e-02, 2.75765658e-02,
            4.31962157e-02, 1.36435987e-02, 2.37534714e-02, 2.22365028e-02, 2.55441306e-02,
            2.38968304e-02, 2.76167758e-02, 1.96525927e-02, 1.83941201e-02, 2.19189520e-02,
            3.10651597e-02, 2.64020995e-02, 2.99503052e-02, 9.18291613e-03, 3.94195906e-02,
            3.53982491e-02, 2.98733566e-02, 2.33204502e-02, 4.59914739e-02, 1.89115960e-03
        ], [
            2.22438808e-02, 1.73611181e-02, 1.83505523e-02, 2.24454487e-02, 2.00211128e-02,
            2.08508139e-02, 1.71892225e-02, 1.05388741e+00, 5.61885106e-02, 3.04806740e-02,
            3.37449196e-02, 2.68196483e-02, 1.73256443e-02, 1.34638021e-02, 1.57048013e-02,
            1.58082623e-02, 2.91102581e-02, 2.60522319e-02, 1.15372414e-02, 1.07641864e-02,
            8.59931931e-03, 1.20049592e-02, 6.05019452e-02, 9.70281123e-02, 7.77309568e-02,
            2.89987064e-02, 8.82979273e-03, 5.33225710e-03, 4.05864046e-03, 9.93229929e-03,
            1.52045862e-02, 7.36205205e-03, 1.09509862e-02, 1.21529306e-02, 1.71956283e-03
        ], [
            6.93017038e-02, 2.54427651e-02, 5.27367629e-02, 1.54687454e-01, 1.06617336e-01,
            8.33190264e-02, 8.05595102e-02, 3.58976480e-02, 1.33401586e+00, 3.29401606e-01,
            7.20895685e-02, 4.88432336e-02, 5.04433655e-02, 5.89580208e-02, 6.14272355e-02,
            6.98271401e-02, 2.11956726e-02, 4.45808986e-02, 2.82448076e-02, 1.32566506e-02,
            1.17188065e-02, 2.47792126e-02, 1.95542265e-02, 2.21220280e-02, 1.94286134e-02,
            1.93584825e-02, 1.27305523e-02, 8.04855499e-03, 7.01255573e-03, 1.66568637e-02,
            2.57101461e-02, 1.24680950e-02, 9.64136509e-02, 2.67856919e-02, 3.54543608e-03
        ], [
            1.40593691e-02, 1.37343299e-02, 3.45612029e-02, 2.91727226e-02, 6.72061980e-02,
            2.43703050e-02, 3.28058558e-02, 1.33459161e-02, 3.61696455e-02, 1.16919059e+00,
            2.45786113e-02, 2.25875876e-02, 5.12440464e-02, 5.05839672e-02, 7.53603407e-02,
            5.13523139e-02, 1.04242157e-02, 3.48043778e-02, 3.25375567e-02, 1.02207864e-02,
            1.03452006e-02, 1.46679853e-02, 2.10109828e-02, 1.49381290e-02, 1.42676869e-02,
            1.57954337e-02, 1.08767783e-02, 4.73870460e-03, 4.42586300e-03, 9.08991363e-03,
            1.14003391e-02, 5.46597354e-03, 1.34543627e-02, 1.16571288e-02, 1.49835864e-03
        ], [
            5.82263503e-03, 9.05684166e-03, 1.26190572e-02, 8.61514089e-03, 8.29488948e-03,
            1.61621269e-02, 6.20160928e-03, 8.16233157e-03, 1.30405705e-02, 1.45638030e-02,
            1.14615755e+00, 2.24251875e-02, 1.61683660e-02, 2.29146342e-02, 1.98365290e-02,
            1.43126026e-02, 7.72656536e-03, 1.02340247e-01, 8.72999567e-03, 3.81010950e-03,
            3.63047251e-03, 8.41332986e-03, 5.88411379e-03, 5.30145179e-03, 5.05027502e-03,
            6.20351271e-03, 5.45574640e-03, 2.48942214e-03, 6.83627170e-03, 4.08119620e-03,
            6.19782816e-03, 4.95103408e-03, 6.02673421e-03, 6.09780273e-03, 8.62835011e-04
        ], [
            2.26223035e-02, 5.09843431e-02, 4.30328249e-02, 3.63839714e-02, 4.54832130e-02,
            6.96351208e-02, 3.79549419e-02, 4.18337247e-02, 4.89744626e-02, 7.12732520e-02,
            8.20415734e-02, 1.49598031e+00, 3.10182041e-01, 1.58316420e-01, 2.41224242e-01,
            1.44074771e-01, 3.47698021e-02, 1.70952335e-01, 6.74112288e-02, 1.77385817e-02,
            1.60800061e-02, 2.39244649e-02, 3.50190979e-02, 4.06000031e-02, 3.56777071e-02,
            3.19174617e-02, 2.42213590e-02, 1.01329537e-02, 1.44836115e-02, 1.92832427e-02,
            2.72340567e-02, 1.33704013e-02, 1.94312607e-02, 2.34748321e-02, 4.65573841e-03
        ], [
            1.41667910e-02, 2.50298464e-02, 1.52122109e-02, 2.26343725e-02, 1.92598662e-02,
            2.22932108e-02, 1.74135981e-02, 2.18801043e-02, 1.93815207e-02, 2.64498962e-02,
            3.32023661e-02, 4.02295511e-02, 1.16758802e+00, 3.45729273e-02, 6.85339140e-02,
            2.27291127e-02, 1.91251274e-02, 3.44713732e-02, 3.71936372e-02, 8.43008429e-03,
            6.79309923e-03, 9.13577868e-03, 1.51751945e-02, 2.25779421e-02, 1.61584773e-02,
            1.40000695e-02, 9.01742932e-03, 4.56648677e-03, 4.74028615e-03, 8.22380766e-03,
            1.31839653e-02, 5.95062461e-03, 8.46969725e-03, 1.00324582e-02, 2.12016947e-03
        ], [
            1.47214817e-02, 2.42244370e-02, 2.39631917e-02, 3.33209205e-02, 3.06392171e-02,
            3.27033556e-02, 3.80910852e-02, 2.51072446e-02, 3.77329828e-02, 4.72992790e-02,
            4.00797923e-02, 5.78266366e-02, 1.81688452e-01, 1.48298840e+00, 1.52279777e-01,
            5.70310769e-02, 4.38030573e-02, 7.45564161e-02, 7.19537223e-02, 2.43921265e-02,
            2.21595910e-02, 2.11063045e-02, 3.03398518e-02, 3.09115882e-02, 3.16037812e-02,
            3.23478445e-02, 7.67416006e-02, 1.81740753e-02, 1.13550248e-02, 3.73290648e-02,
            3.85911988e-02, 1.97837499e-02, 4.66221130e-02, 3.22465094e-02, 3.31510095e-03
        ], [
            1.38616626e-02, 1.59277059e-02, 1.60360231e-02, 1.70624663e-02, 1.76832004e-02,
            1.87748236e-02, 1.50283680e-02, 1.50357372e-02, 1.50013275e-02, 1.94855628e-02,
            1.96008800e-02, 2.35912289e-02, 3.76456314e-02, 2.00273238e-02, 1.45271059e+00,
            1.89912902e-02, 1.25190301e-02, 1.96899917e-02, 1.34957693e-01, 1.43077094e-02,
            1.09881551e-02, 1.04628389e-02, 6.38545757e-02, 7.91480415e-02, 8.52718474e-02,
            3.12570182e-02, 1.27164781e-02, 6.71053943e-03, 4.46664674e-03, 1.26827063e-02,
            3.16219139e-02, 7.01985902e-03, 8.00583543e-03, 1.71530932e-02, 1.74960014e-03
        ], [
            2.33687881e-03, 2.58138337e-03, 3.62789689e-03, 8.91960171e-03, 6.64254129e-03,
            8.60487592e-03, 7.23250421e-03, 2.65727417e-03, 4.58166349e-03, 6.19610825e-03,
            6.19326211e-03, 1.62733440e-02, 9.71430145e-03, 7.15495245e-03, 1.20896178e-02,
            1.05017044e+00, 3.04145980e-03, 9.43931056e-03, 4.84861236e-03, 2.55784931e-03,
            2.71926244e-03, 4.30101983e-03, 4.12719670e-03, 3.25264677e-03, 4.07264395e-03,
            3.76428770e-03, 2.75410876e-03, 2.28888298e-03, 2.28119664e-03, 3.56150484e-03,
            5.79851167e-03, 4.15831940e-03, 4.67839982e-03, 7.38529052e-03, 2.48451153e-03
        ], [
            2.38020873e-02, 4.48104800e-02, 3.48289827e-02, 4.59775606e-02, 3.28168943e-02,
            4.16697309e-02, 4.73878917e-02, 4.29207345e-02, 5.31241744e-02, 5.00660277e-02,
            6.74176091e-02, 6.11703510e-02, 3.78211529e-02, 2.96898671e-02, 3.63624647e-02,
            3.00614365e-02, 1.12850970e+00, 3.00421708e-02, 2.37737439e-02, 1.45330547e-02,
            2.20427696e-02, 3.90060194e-02, 2.89838279e-02, 2.01020525e-02, 2.05229570e-02,
            2.92577890e-02, 1.97257119e-02, 1.20978144e-02, 1.42059456e-02, 1.53107144e-02,
            2.68986069e-02, 2.88900911e-02, 2.44524081e-02, 2.86458821e-02, 3.78784376e-03
        ], [
            8.44025876e-03, 1.33134886e-02, 1.04068984e-02, 1.12704963e-02, 9.89644976e-03,
            1.16477978e-02, 1.18328972e-02, 1.16969731e-02, 1.17253935e-02, 1.14074134e-02,
            1.60934482e-02, 1.36219155e-02, 1.15302590e-02, 9.52775347e-03, 1.07539018e-02,
            9.50572764e-03, 2.82722506e-02, 1.05906519e+00, 1.06996941e-02, 8.89532877e-03,
            1.05248915e-02, 1.31969094e-02, 1.64419375e-02, 1.15887346e-02, 1.14464970e-02,
            2.15102187e-02, 1.76183508e-02, 1.12197941e-02, 4.55047603e-02, 1.08502027e-02,
            2.48079870e-02, 1.86932870e-02, 1.25614019e-02, 1.71990081e-02, 1.64975232e-03
        ], [
            9.30403991e-03, 7.32990653e-03, 1.44618017e-02, 1.40263320e-02, 1.62493515e-02,
            1.41637129e-02, 1.18602901e-02, 8.36382728e-03, 1.31612442e-02, 1.43038762e-02,
            1.42201740e-02, 1.38853728e-02, 1.28235336e-02, 1.07768379e-02, 1.40880251e-02,
            1.27301384e-02, 1.28964241e-02, 1.41603402e-02, 1.01543030e+00, 7.13450378e-03,
            6.90842424e-03, 1.02882994e-02, 2.12294231e-02, 8.36102538e-03, 9.69053181e-03,
            1.31182474e-02, 7.43186625e-03, 5.83647581e-03, 3.88186984e-03, 9.34763010e-03,
            8.34466599e-03, 5.29359418e-03, 7.40972072e-03, 1.04323974e-02, 8.15481780e-04
        ], [
            5.97719516e-02, 4.08562551e-02, 1.13664446e-01, 1.08900254e-01, 1.27020258e-01,
            1.05115925e-01, 9.25099914e-02, 7.29951925e-02, 9.01529549e-02, 9.86091822e-02,
            7.85941972e-02, 9.39988002e-02, 9.68825272e-02, 8.64201019e-02, 1.06790098e-01,
            8.46970912e-02, 4.57018330e-02, 8.16493835e-02, 6.19220004e-02, 1.03858239e+00,
            2.55056842e-02, 7.90492713e-02, 4.50221712e-02, 5.13814490e-02, 4.84098812e-02,
            4.37398095e-02, 2.81078302e-02, 1.61753119e-02, 1.23906056e-02, 2.94184281e-02,
            3.87522812e-02, 2.28963843e-02, 4.68315189e-02, 3.69787830e-02, 4.67299302e-03
        ], [
            2.94317361e-02, 1.70342413e-02, 5.36253005e-02, 5.42100814e-02, 6.89052921e-02,
            4.25834978e-02, 3.68703343e-02, 2.40793487e-02, 4.07920084e-02, 4.22559807e-02,
            3.54751697e-02, 3.62607535e-02, 3.70337576e-02, 3.15154047e-02, 4.46717777e-02,
            4.24711994e-02, 1.85019617e-02, 4.93194418e-02, 2.76455225e-02, 1.31979273e-02,
            1.01362816e+00, 4.06788531e-02, 2.47292740e-02, 2.15034418e-02, 2.21114944e-02,
            2.03142243e-02, 1.68014490e-02, 1.13266526e-02, 7.96864820e-03, 1.72450814e-02,
            1.81081197e-02, 1.17225317e-02, 2.14072904e-02, 2.08238336e-02, 2.30430626e-03
        ], [
            5.76098506e-03, 7.82221425e-03, 1.16220441e-02, 1.34816363e-02, 1.38962121e-02,
            1.32149235e-02, 1.53628595e-02, 7.70873598e-03, 1.28618406e-02, 1.46979853e-02,
            1.32521241e-02, 1.34682267e-02, 1.52552817e-02, 1.37169067e-02, 1.28591466e-02,
            1.13628625e-02, 9.45136372e-03, 1.30047385e-02, 1.19647433e-02, 1.31258623e-02,
            1.02527699e-02, 1.01688311e+00, 1.46597143e-02, 1.37878522e-02, 3.19872183e-02,
            2.64085116e-02, 1.61644887e-02, 1.70807327e-02, 5.95278841e-03, 2.08181959e-02,
            1.71838474e-02, 1.02147139e-02, 1.42189455e-02, 1.86828752e-02, 9.27013162e-04
        ], [
            2.74942101e-02, 3.24979102e-02, 4.80099224e-02, 4.62481284e-02, 4.96489046e-02,
            5.01012686e-02, 4.12392381e-02, 6.00829285e-02, 4.10661103e-02, 4.19679999e-02,
            5.76736152e-02, 4.29875363e-02, 3.50746529e-02, 2.77568941e-02, 3.71401228e-02,
            3.81991728e-02, 3.47585115e-02, 3.91199455e-02, 2.72255744e-02, 2.85538443e-02,
            1.97699757e-02, 2.58022126e-02, 1.06205974e+00, 6.28964562e-02, 3.34226140e-02,
            7.57986421e-02, 1.82822732e-02, 1.26932825e-02, 6.67524721e-03, 1.45166153e-02,
            2.16616441e-02, 1.71720534e-02, 1.74457428e-02, 2.02836428e-02, 3.15377228e-03
        ], [
            7.04319025e-03, 9.45940961e-03, 1.10902443e-02, 1.18122222e-02, 1.36265440e-02,
            1.56524883e-02, 9.04878825e-03, 1.23966020e-02, 9.02129540e-03, 9.88758439e-03,
            1.65201009e-02, 1.42502229e-02, 1.01019709e-02, 7.79531697e-03, 9.00089863e-03,
            8.75890567e-03, 7.08297727e-03, 1.08614425e-02, 4.34400374e-03, 6.18105081e-03,
            3.77917670e-03, 5.55127582e-03, 6.79106821e-03, 1.13947091e+00, 6.96894202e-03,
            9.36149203e-03, 2.86517648e-03, 1.94607014e-03, 2.17365261e-03, 3.06002578e-03,
            3.77727818e-03, 2.64103760e-03, 2.83737103e-03, 3.83152249e-03, 8.21670045e-04
        ], [
            2.45028474e-03, 3.75348628e-03, 4.51106859e-03, 5.42425655e-03, 6.71311180e-03,
            5.29342076e-03, 6.17663109e-03, 3.59919146e-03, 5.02479065e-03, 5.46426218e-03,
            4.79937526e-03, 4.75298123e-03, 5.64870411e-03, 5.24052207e-03, 5.35613617e-03,
            4.25082584e-03, 2.88685698e-03, 4.23425286e-03, 4.17915095e-03, 5.04376629e-03,
            3.23526358e-03, 3.68698731e-03, 5.83482913e-03, 1.35288155e-02, 1.03677154e+00,
            2.20623490e-02, 6.10216101e-03, 5.47545498e-03, 1.63064957e-03, 6.61944990e-03,
            6.80972506e-03, 3.44628698e-03, 3.23288962e-03, 6.62749740e-03, 3.84417310e-04
        ], [
            1.13407461e-02, 1.32964229e-02, 2.11436673e-02, 1.83392492e-02, 2.10194276e-02,
            2.23869982e-02, 2.40731991e-02, 1.98158677e-02, 1.80349978e-02, 1.93044847e-02,
            2.28713671e-02, 1.96512491e-02, 1.79840642e-02, 1.48896143e-02, 1.89794534e-02,
            1.62768042e-02, 1.25583870e-02, 1.85081863e-02, 1.98988378e-02, 2.85315676e-02,
            1.74287364e-02, 1.76820539e-02, 5.43984473e-02, 1.54398156e-01, 1.18210880e-01,
            1.12688195e+00, 1.12017116e-02, 9.07355201e-03, 4.11961604e-03, 1.17988119e-02,
            1.23320251e-02, 7.01303872e-03, 9.70684304e-03, 1.37731199e-02, 1.29637888e-03
        ], [
            1.34933698e-02, 1.66927197e-02, 2.32071957e-02, 2.73200293e-02, 2.92099973e-02,
            2.53063651e-02, 3.61963278e-02, 1.78621599e-02, 2.87336825e-02, 2.73288133e-02,
            2.54768081e-02, 2.64998989e-02, 2.86171873e-02, 2.77243112e-02, 2.65582837e-02,
            2.60573583e-02, 1.91007591e-02, 3.24806899e-02, 3.14439195e-02, 3.42741812e-02,
            3.48527561e-02, 3.06422379e-02, 3.39225144e-02, 4.06744621e-02, 4.83263419e-02,
            4.68956690e-02, 1.15750081e+00, 5.02051629e-02, 1.43702519e-02, 4.40880795e-02,
            3.86984497e-02, 1.93305809e-02, 2.69064875e-02, 4.09452133e-02, 2.51091801e-03
        ], [
            4.80220217e-02, 5.58255598e-02, 6.17085177e-02, 7.33658154e-02, 6.61972242e-02,
            6.37543667e-02, 7.08839283e-02, 5.49353805e-02, 6.28694902e-02, 6.39562343e-02,
            6.78309752e-02, 6.59255096e-02, 6.45933251e-02, 5.60973424e-02, 6.37425914e-02,
            6.70777156e-02, 5.53525396e-02, 6.66633472e-02, 5.54921143e-02, 6.43405728e-02,
            6.56404125e-02, 6.39917301e-02, 8.82580982e-02, 8.76719835e-02, 8.22186550e-02,
            7.12118768e-02, 5.33704935e-02, 1.27426550e+00, 1.09877563e-01, 7.43263786e-02,
            5.20091148e-02, 3.32929236e-02, 5.60691196e-02, 8.35631671e-02, 1.11182317e-02
        ], [
            1.40991815e-02, 1.54551671e-02, 2.15531363e-02, 2.31206024e-02, 2.43820631e-02,
            2.29000615e-02, 2.80043598e-02, 1.53339852e-02, 2.11566156e-02, 2.32853977e-02,
            2.02212240e-02, 2.19802503e-02, 2.30062855e-02, 2.01905253e-02, 2.35876832e-02,
            2.15363633e-02, 1.53088156e-02, 2.68502643e-02, 3.61495249e-02, 3.21323039e-02,
            5.11162889e-02, 4.30820159e-02, 2.26475322e-02, 3.60042030e-02, 3.03769391e-02,
            3.91225683e-02, 2.90110260e-02, 3.88204130e-02, 1.03743246e+00, 3.84367688e-02,
            2.64899181e-02, 2.71282031e-02, 4.64723357e-02, 4.12521433e-02, 1.76049532e-03
        ], [
            5.04154740e-02, 6.82102266e-02, 1.05257142e-01, 1.04221963e-01, 1.00253842e-01,
            9.41480589e-02, 1.45801799e-01, 6.83300755e-02, 1.49409202e-01, 1.23694697e-01,
            1.00668170e-01, 1.03018099e-01, 1.24003154e-01, 1.29694478e-01, 1.31286870e-01,
            9.27736111e-02, 7.96738193e-02, 1.23517669e-01, 1.06274350e-01, 9.50236567e-02,
            9.94369040e-02, 9.74900967e-02, 9.87575650e-02, 1.00623526e-01, 1.56122938e-01,
            1.30892435e-01, 1.23148012e-01, 1.64413517e-01, 6.54058977e-02, 1.20039560e+00,
            1.31064264e-01, 6.19765615e-02, 1.08492328e-01, 1.35166878e-01, 1.01479433e-02
        ], [
            3.34826122e-03, 5.25862046e-03, 5.02929300e-03, 4.46748598e-03, 4.21158128e-03,
            6.85917793e-03, 6.88692858e-03, 4.31402995e-03, 6.30441155e-03, 5.71850371e-03,
            5.68471138e-03, 6.34780537e-03, 4.54601615e-03, 4.06940821e-03, 4.89137609e-03,
            4.84249309e-03, 7.53099456e-03, 4.74228610e-03, 4.56100278e-03, 3.05829212e-03,
            4.22023213e-03, 6.14423561e-03, 5.38866115e-03, 9.58416889e-03, 1.26826291e-02,
            6.36327746e-03, 5.13271019e-03, 3.46618981e-03, 4.10811421e-03, 4.89191539e-03,
            1.00952113e+00, 5.77922439e-03, 6.63395681e-03, 1.11369103e-02, 4.78007443e-04
        ], [
            1.20642734e-03, 1.09409386e-03, 1.67928131e-03, 1.71590636e-03, 1.83349192e-03,
            1.58000723e-03, 1.98687821e-03, 1.31057892e-03, 2.35069025e-03, 2.10042307e-03,
            1.84545305e-03, 1.92708860e-03, 2.22240598e-03, 2.12750982e-03, 2.25181419e-03,
            1.51925565e-03, 1.58507065e-03, 1.70078383e-03, 1.75027024e-03, 1.42812867e-03,
            1.61959411e-03, 1.52441769e-03, 2.05955166e-03, 1.57661358e-03, 3.27056037e-03,
            2.36239776e-03, 2.62221449e-03, 2.25301544e-03, 8.05750448e-04, 3.28802013e-03,
            5.19246523e-03, 1.01223137e+00, 1.99004094e-03, 2.61046640e-03, 1.30957818e-04
        ], [
            1.96402946e-03, 7.60135250e-04, 1.58041523e-03, 1.29207012e-03, 2.12830306e-03,
            1.37692737e-03, 1.18037295e-03, 7.67657207e-04, 1.28814132e-03, 1.21682297e-03,
            1.15378975e-03, 1.14331546e-03, 1.37716259e-03, 1.09646201e-03, 1.24221687e-03,
            1.18284904e-03, 8.12230611e-04, 1.12110715e-03, 1.07138861e-03, 9.86033433e-04,
            7.93841259e-04, 1.73468393e-03, 1.17233083e-03, 1.00741132e-03, 1.06794839e-03,
            1.20933469e-03, 1.20767851e-03, 9.62253504e-04, 3.47436455e-04, 1.19895682e-03,
            5.39483230e-03, 1.79972823e-03, 1.02671997e+00, 1.73506873e-03, 1.03300044e-04
        ], [
            9.41069123e-03, 1.33929912e-02, 1.75924935e-02, 1.86387044e-02, 2.09952069e-02,
            1.79525127e-02, 2.91536667e-02, 1.29552217e-02, 2.18519877e-02, 1.98120082e-02,
            1.78606309e-02, 1.93542951e-02, 1.89946991e-02, 1.68587553e-02, 1.92462184e-02,
            1.64111060e-02, 1.41027811e-02, 1.87134215e-02, 1.75255294e-02, 1.57122668e-02,
            1.62664932e-02, 2.40955553e-02, 2.05679993e-02, 1.76299260e-02, 2.11598280e-02,
            2.37032152e-02, 3.23697518e-02, 2.15720963e-02, 1.31131813e-02, 3.84763338e-02,
            2.71425223e-02, 1.70239287e-02, 2.34541931e-02, 1.09459326e+00, 3.72551716e-03
        ], [
            4.14836825e-05, 5.04402773e-05, 7.24732522e-05, 1.25555858e-04, 7.40513874e-05,
            8.10343775e-05, 9.27068266e-05, 4.89527299e-05, 6.75650743e-05, 7.56403707e-05,
            6.52045624e-05, 6.66500844e-05, 7.87462294e-05, 6.12163049e-05, 1.50439658e-04,
            5.61195168e-05, 3.77631614e-05, 1.20517918e-04, 1.04933170e-04, 4.31650698e-05,
            7.40060377e-05, 9.65913864e-05, 1.18625981e-04, 7.85697339e-05, 7.61095210e-05,
            6.81317256e-05, 8.03661094e-05, 1.02595340e-04, 2.60948310e-04, 1.11813636e-04,
            1.10415299e-04, 6.27998295e-05, 9.33406825e-05, 1.68534011e-04, 1.00011908e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 398)
    return ret


@cache('run')
def leontief_matrix_x2004x():
    """
    leontief matrix x2004x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17725946e+00, 4.70979270e-03, 3.68109222e-01, 8.60168424e-02, 1.24470513e-01,
            1.95156060e-01, 3.13576462e-02, 6.69016059e-03, 1.93278549e-02, 2.93976764e-02,
            9.20834979e-03, 7.55312145e-03, 8.15624037e-03, 7.23932161e-03, 9.18402919e-03,
            3.35242687e-02, 4.69029300e-03, 2.11568587e-02, 6.62282583e-03, 8.96028719e-03,
            8.45022854e-03, 1.06356276e-01, 7.71100607e-03, 9.25456380e-03, 8.44727149e-03,
            1.44263803e-02, 5.25037132e-03, 4.46782876e-03, 3.40237684e-03, 6.77913302e-03,
            1.04930680e-02, 9.82753149e-03, 1.28617904e-02, 1.21306042e-02, 2.47451793e-02
        ], [
            2.37460738e-02, 1.11243893e+00, 2.57055433e-02, 3.23143775e-02, 2.77943773e-02,
            3.27820766e-02, 2.88452239e-02, 5.41087803e-01, 9.19562368e-02, 4.63584813e-02,
            1.24481620e-01, 1.11717152e-01, 4.07062624e-02, 2.77401707e-02, 3.51558440e-02,
            3.24770914e-02, 1.58296214e-01, 5.83311133e-02, 1.75218633e-02, 1.09109743e-02,
            1.13014807e-02, 1.84041746e-02, 4.55323381e-02, 6.08595369e-02, 5.45017966e-02,
            2.78940523e-02, 1.11766064e-02, 6.87491324e-03, 8.97052173e-03, 1.16098336e-02,
            1.95487849e-02, 1.31032069e-02, 1.67495093e-02, 1.79352720e-02, 6.91498577e-02
        ], [
            8.28748340e-02, 4.35727906e-03, 1.21033259e+00, 2.02292587e-02, 1.45282887e-01,
            2.16801677e-02, 1.25370759e-02, 5.98751073e-03, 2.18486990e-02, 1.51372252e-02,
            8.73834774e-03, 7.98949688e-03, 8.58456424e-03, 7.83346987e-03, 8.66645169e-03,
            1.11750330e-02, 4.73716991e-03, 8.46964101e-03, 7.12453987e-03, 9.83390209e-03,
            9.37644365e-03, 1.90123245e-01, 7.12770128e-03, 1.25725976e-02, 1.21621259e-02,
            1.06205850e-02, 6.95193752e-03, 5.83084430e-03, 2.69134651e-03, 8.20126763e-03,
            1.43295143e-02, 1.40376771e-02, 2.21671268e-02, 1.50059206e-02, 5.40807514e-03
        ], [
            5.32625848e-03, 3.52287627e-03, 6.79283451e-03, 1.45752158e+00, 1.02608580e-01,
            1.01481690e-02, 1.36217987e-02, 3.60806586e-03, 9.47121682e-03, 2.50046296e-02,
            9.35564240e-03, 6.94894577e-03, 8.71192485e-03, 7.79290611e-03, 1.64267812e-02,
            3.60562082e-02, 3.10748005e-03, 8.07936702e-03, 6.06703186e-03, 5.18862732e-03,
            6.14529853e-03, 7.04348502e-03, 5.50298316e-03, 7.00323228e-03, 5.43341671e-03,
            5.32218184e-03, 3.19590650e-03, 2.29462013e-03, 1.65649769e-03, 4.14415025e-03,
            6.48781627e-03, 2.82276078e-03, 7.06765184e-03, 1.06382580e-02, 6.37806732e-04
        ], [
            4.42528670e-04, 3.04714560e-04, 6.24163322e-04, 1.50436750e-02, 1.25021604e+00,
            1.10477211e-03, 1.30889071e-03, 3.43716991e-04, 7.33179894e-04, 1.92490149e-03,
            7.58971575e-04, 7.38769795e-04, 8.62897970e-04, 8.81650041e-04, 3.35643646e-03,
            5.99076584e-03, 3.56014982e-04, 6.38552779e-04, 1.04029459e-03, 6.66800930e-04,
            5.77360484e-04, 4.87849536e-04, 6.45413093e-04, 5.80012439e-04, 5.84691409e-04,
            5.30094273e-04, 5.63095427e-04, 2.10375163e-04, 1.29219343e-04, 3.70882924e-04,
            7.41023319e-04, 2.86146753e-04, 4.19412056e-04, 8.65465885e-04, 5.44663426e-05
        ], [
            4.81650492e-03, 4.45658678e-03, 6.59895441e-03, 6.36447591e-03, 7.43623569e-03,
            1.30867855e+00, 2.13885824e-02, 3.72145638e-03, 5.85325384e-03, 8.75513864e-03,
            1.38416297e-02, 1.02301406e-02, 9.35789592e-03, 6.84437731e-03, 1.04671178e-02,
            9.93496486e-02, 4.19440523e-03, 4.87314535e-02, 4.86631774e-03, 3.85839652e-03,
            3.72083227e-03, 6.35059320e-03, 4.61825473e-03, 3.97420782e-03, 3.73558953e-03,
            7.25755397e-03, 3.44019941e-03, 2.52681646e-03, 5.90707626e-03, 3.79850545e-03,
            5.66145311e-03, 5.55867397e-03, 4.31066376e-03, 7.15397928e-03, 7.59333385e-04
        ], [
            1.44686883e-02, 1.18218684e-02, 5.27459990e-02, 3.77728812e-02, 4.64618072e-02,
            3.74765318e-02, 1.33045218e+00, 1.29261588e-02, 4.07064423e-02, 4.41326788e-02,
            4.34975769e-02, 2.31748451e-02, 2.93915237e-02, 3.29706681e-02, 2.72838204e-02,
            4.25378301e-02, 1.33440738e-02, 2.38838197e-02, 2.34917880e-02, 2.51719363e-02,
            2.42556571e-02, 2.79916740e-02, 1.99916204e-02, 1.82265768e-02, 2.12261948e-02,
            3.21442911e-02, 2.57158598e-02, 3.04852484e-02, 9.86228579e-03, 4.01775329e-02,
            3.58222862e-02, 3.07367065e-02, 2.34931124e-02, 4.68209153e-02, 1.96576992e-03
        ], [
            2.32432235e-02, 1.70656546e-02, 1.94017945e-02, 2.44764643e-02, 2.21015347e-02,
            2.21363523e-02, 1.79711976e-02, 1.05859477e+00, 6.15673118e-02, 3.31420366e-02,
            3.55510077e-02, 2.96843032e-02, 1.89897024e-02, 1.45805049e-02, 1.70886546e-02,
            1.66375930e-02, 3.08429953e-02, 2.68059228e-02, 1.30605036e-02, 1.11332066e-02,
            9.08320247e-03, 1.27106169e-02, 6.31031013e-02, 9.86805581e-02, 8.78854182e-02,
            3.29069877e-02, 9.04601051e-03, 5.71399079e-03, 4.57355814e-03, 1.05825403e-02,
            1.58242523e-02, 7.93012889e-03, 1.17100319e-02, 1.28300439e-02, 1.81526715e-03
        ], [
            7.00559239e-02, 2.49858685e-02, 5.29666424e-02, 1.55831285e-01, 1.10506152e-01,
            8.36546196e-02, 8.07104525e-02, 3.59308314e-02, 1.33572413e+00, 3.31546925e-01,
            7.22619752e-02, 4.82072604e-02, 5.02268832e-02, 6.02243833e-02, 6.12997389e-02,
            6.83113454e-02, 2.13265666e-02, 4.46590336e-02, 2.90202847e-02, 1.32012053e-02,
            1.21987527e-02, 2.49620876e-02, 2.00243724e-02, 2.16648502e-02, 2.00377555e-02,
            2.05262580e-02, 1.27102099e-02, 8.18768351e-03, 7.63777275e-03, 1.70943667e-02,
            2.54018669e-02, 1.27879277e-02, 9.51995398e-02, 2.72795561e-02, 3.75190870e-03
        ], [
            1.38811015e-02, 1.36460746e-02, 3.46833212e-02, 2.92198216e-02, 6.93963390e-02,
            2.45849655e-02, 3.30039753e-02, 1.31042821e-02, 3.46344556e-02, 1.17015730e+00,
            2.41226577e-02, 2.20106403e-02, 4.97535639e-02, 4.99712389e-02, 7.46602124e-02,
            5.01107505e-02, 1.02802632e-02, 3.48195379e-02, 3.30138185e-02, 1.01285072e-02,
            1.07551767e-02, 1.48195057e-02, 2.11561357e-02, 1.39366904e-02, 1.42908222e-02,
            1.65374882e-02, 1.07550500e-02, 4.88495331e-03, 4.73401795e-03, 9.36533750e-03,
            1.15718375e-02, 5.66993929e-03, 1.39183142e-02, 1.24865467e-02, 1.56595825e-03
        ], [
            5.63142980e-03, 9.23748737e-03, 1.21544212e-02, 8.43975767e-03, 8.24316615e-03,
            1.65216367e-02, 6.16804775e-03, 8.12417392e-03, 1.26287864e-02, 1.40103794e-02,
            1.14917594e+00, 2.18368362e-02, 1.61469684e-02, 2.27921200e-02, 1.99746641e-02,
            1.40810391e-02, 7.55016512e-03, 1.06368846e-01, 9.33086097e-03, 3.79342249e-03,
            3.75380173e-03, 8.32941799e-03, 6.06519369e-03, 4.97015800e-03, 5.04966386e-03,
            6.45996790e-03, 5.38417192e-03, 2.52896080e-03, 6.63874867e-03, 4.17084073e-03,
            6.31565063e-03, 4.90158211e-03, 6.17356466e-03, 6.20754134e-03, 9.26472582e-04
        ], [
            2.23627373e-02, 5.02977026e-02, 4.25370906e-02, 3.56405205e-02, 4.59170501e-02,
            6.87783203e-02, 3.80865479e-02, 4.17465259e-02, 4.67313728e-02, 6.98741547e-02,
            8.19101500e-02, 1.50725458e+00, 3.13943867e-01, 1.64593009e-01, 2.47253368e-01,
            1.44209140e-01, 3.43298540e-02, 1.69447449e-01, 6.75935874e-02, 1.71583693e-02,
            1.63822402e-02, 2.37714001e-02, 3.52389580e-02, 3.73808881e-02, 3.54532933e-02,
            3.36108888e-02, 2.40447199e-02, 1.02790772e-02, 1.46479912e-02, 1.97641969e-02,
            2.73642780e-02, 1.36188501e-02, 1.99064161e-02, 2.42386168e-02, 4.76287449e-03
        ], [
            1.50920573e-02, 2.65565823e-02, 1.59333480e-02, 2.39151789e-02, 2.13245541e-02,
            2.40220437e-02, 1.84606752e-02, 2.35661438e-02, 2.02498287e-02, 2.78667636e-02,
            3.54486140e-02, 4.40344429e-02, 1.18355937e+00, 3.80897065e-02, 7.53498282e-02,
            2.57252822e-02, 1.95769512e-02, 3.66425003e-02, 4.11299268e-02, 8.41203686e-03,
            7.30878678e-03, 9.75940046e-03, 1.62900175e-02, 2.29111952e-02, 1.82677078e-02,
            1.61220792e-02, 9.56152458e-03, 4.83233438e-03, 5.29915421e-03, 8.75520695e-03,
            1.41469342e-02, 6.43601878e-03, 9.53955528e-03, 1.11667681e-02, 2.41952730e-03
        ], [
            1.60082208e-02, 2.66643039e-02, 2.59816552e-02, 3.48311052e-02, 3.31063826e-02,
            3.54377708e-02, 4.06337398e-02, 2.68437150e-02, 3.99166513e-02, 4.95107361e-02,
            4.28405208e-02, 6.12165335e-02, 1.98180870e-01, 1.51587110e+00, 1.63335465e-01,
            6.09582617e-02, 4.99371744e-02, 8.03913090e-02, 7.92822035e-02, 2.52475796e-02,
            2.44703911e-02, 2.32165022e-02, 3.27628348e-02, 3.13892690e-02, 3.36197423e-02,
            3.64536416e-02, 8.09273547e-02, 1.99247695e-02, 1.28830206e-02, 4.04224793e-02,
            4.26642716e-02, 2.25750043e-02, 5.06554856e-02, 3.58380952e-02, 3.80178641e-03
        ], [
            1.37137909e-02, 1.59827767e-02, 1.60514085e-02, 1.69393528e-02, 1.77353835e-02,
            1.88345924e-02, 1.52969080e-02, 1.50127363e-02, 1.48077573e-02, 1.92528287e-02,
            2.00116455e-02, 2.40647112e-02, 3.86755443e-02, 1.96644018e-02, 1.45455151e+00,
            2.00915181e-02, 1.28648011e-02, 2.01508647e-02, 1.36272747e-01, 1.34527336e-02,
            1.12482555e-02, 1.07999381e-02, 6.41752283e-02, 7.03172004e-02, 8.26468358e-02,
            3.31117014e-02, 1.22303478e-02, 6.80489653e-03, 4.84498961e-03, 1.27522535e-02,
            3.14323122e-02, 7.29234695e-03, 8.27274038e-03, 1.84974371e-02, 1.85069963e-03
        ], [
            2.03901830e-03, 2.46830526e-03, 3.48715492e-03, 9.16701750e-03, 6.82674429e-03,
            8.31167644e-03, 7.12634944e-03, 2.66053005e-03, 4.52551062e-03, 6.11057538e-03,
            6.06052236e-03, 1.56918541e-02, 9.65992285e-03, 6.93006345e-03, 1.22713772e-02,
            1.05126403e+00, 3.12571889e-03, 9.50429984e-03, 4.92700573e-03, 2.55554022e-03,
            2.83507123e-03, 4.29145609e-03, 4.56348097e-03, 3.19286126e-03, 4.35303001e-03,
            3.92633319e-03, 2.71216641e-03, 2.18161837e-03, 2.49653892e-03, 3.52594442e-03,
            5.72908467e-03, 4.14668053e-03, 4.68597415e-03, 7.36827342e-03, 2.47039998e-03
        ], [
            2.44009696e-02, 4.76733237e-02, 3.60904428e-02, 4.73582046e-02, 3.47121216e-02,
            4.29896912e-02, 4.77398012e-02, 4.76658836e-02, 5.36169920e-02, 5.03295324e-02,
            7.12808601e-02, 6.37722805e-02, 3.98948333e-02, 3.05891280e-02, 3.75565084e-02,
            3.12322341e-02, 1.13913815e+00, 3.14045648e-02, 2.57567149e-02, 1.47779719e-02,
            2.31795051e-02, 3.96468900e-02, 3.07400152e-02, 2.04316538e-02, 2.13625924e-02,
            3.07659095e-02, 1.99659312e-02, 1.22760643e-02, 1.60225071e-02, 1.56782359e-02,
            2.70277010e-02, 2.98030284e-02, 2.49274683e-02, 2.96778246e-02, 4.10658992e-03
        ], [
            7.99748340e-03, 1.22519695e-02, 1.00209392e-02, 1.06517388e-02, 9.54908651e-03,
            1.11536443e-02, 1.09736933e-02, 1.13835037e-02, 1.09307933e-02, 1.05932447e-02,
            1.51865341e-02, 1.27258912e-02, 1.08770430e-02, 8.94117073e-03, 1.01766696e-02,
            9.17442598e-03, 2.58626298e-02, 1.05806273e+00, 1.08554918e-02, 8.35229006e-03,
            1.01614530e-02, 1.25634363e-02, 1.62184077e-02, 1.03902857e-02, 1.05625446e-02,
            2.11574220e-02, 1.63966920e-02, 1.09011006e-02, 3.85310030e-02, 1.03968309e-02,
            2.43299423e-02, 1.70275342e-02, 1.23556659e-02, 1.64001613e-02, 1.86547611e-03
        ], [
            9.84349284e-03, 7.32844176e-03, 1.52093134e-02, 1.41312375e-02, 1.66009551e-02,
            1.48215774e-02, 1.20263723e-02, 8.63591969e-03, 1.32680058e-02, 1.44976879e-02,
            1.43739319e-02, 1.40154027e-02, 1.31053669e-02, 1.07593483e-02, 1.44784993e-02,
            1.32871187e-02, 1.30653748e-02, 1.43155180e-02, 1.01603656e+00, 7.41875517e-03,
            7.18258630e-03, 1.06268224e-02, 2.15427126e-02, 8.18188768e-03, 1.05457518e-02,
            1.37346827e-02, 7.46863670e-03, 6.24437947e-03, 4.19369838e-03, 9.48951725e-03,
            8.45267617e-03, 5.43700230e-03, 7.69908973e-03, 1.08036756e-02, 8.75280859e-04
        ], [
            5.96261456e-02, 3.95514778e-02, 1.13297652e-01, 1.05253589e-01, 1.24752910e-01,
            1.07189478e-01, 9.38232950e-02, 7.57757929e-02, 8.93926587e-02, 9.84467984e-02,
            7.80030104e-02, 9.49421223e-02, 9.87460186e-02, 8.54047859e-02, 1.09222219e-01,
            8.40354938e-02, 4.45024260e-02, 8.22579408e-02, 6.40656866e-02, 1.03995342e+00,
            2.64857401e-02, 7.99275934e-02, 4.73212885e-02, 5.13409858e-02, 5.03551808e-02,
            4.49673092e-02, 2.86066231e-02, 1.68254209e-02, 1.35809755e-02, 3.03642311e-02,
            3.98114235e-02, 2.37837368e-02, 4.80265787e-02, 3.83979782e-02, 4.82811099e-03
        ], [
            2.91065771e-02, 1.58198505e-02, 5.35936972e-02, 5.30984191e-02, 6.68532733e-02,
            4.25724362e-02, 3.63678511e-02, 2.43207916e-02, 4.06842647e-02, 4.20917054e-02,
            3.44210889e-02, 3.65132651e-02, 3.73504207e-02, 3.10264204e-02, 4.53491025e-02,
            4.34746620e-02, 1.73269473e-02, 4.97506812e-02, 2.84342537e-02, 1.28920135e-02,
            1.01385430e+00, 3.95629856e-02, 2.49331288e-02, 2.04060909e-02, 2.18247107e-02,
            2.05893648e-02, 1.64167154e-02, 1.14911348e-02, 8.42117681e-03, 1.69402194e-02,
            1.77859185e-02, 1.17612016e-02, 2.17530677e-02, 2.09748075e-02, 2.35378367e-03
        ], [
            5.81870183e-03, 7.92611433e-03, 1.17529736e-02, 1.32585382e-02, 1.40823333e-02,
            1.29660383e-02, 1.48945903e-02, 7.79480176e-03, 1.23909455e-02, 1.40356494e-02,
            1.29480674e-02, 1.28601351e-02, 1.48685220e-02, 1.31161996e-02, 1.26888270e-02,
            1.11114264e-02, 8.46827386e-03, 1.30946011e-02, 1.20172716e-02, 1.29485143e-02,
            1.11403015e-02, 1.01705391e+00, 1.47536162e-02, 1.35780111e-02, 2.80832319e-02,
            2.87615374e-02, 1.66844728e-02, 1.81963926e-02, 6.78120106e-03, 2.09667149e-02,
            1.77509834e-02, 1.15482570e-02, 1.46989403e-02, 1.89650890e-02, 9.79258188e-04
        ], [
            2.67785340e-02, 3.28080032e-02, 4.89235500e-02, 4.75545758e-02, 5.07060211e-02,
            5.08677451e-02, 4.12248623e-02, 6.21384984e-02, 4.12979751e-02, 4.18622115e-02,
            5.76857739e-02, 4.30014878e-02, 3.56264323e-02, 2.80008049e-02, 3.74195757e-02,
            3.88437766e-02, 3.37749184e-02, 4.08755100e-02, 2.95250119e-02, 2.93417646e-02,
            2.01554511e-02, 2.62300805e-02, 1.06251455e+00, 5.26633565e-02, 3.21637200e-02,
            7.57883761e-02, 1.80557414e-02, 1.24954467e-02, 7.24431779e-03, 1.46450912e-02,
            2.16718257e-02, 1.71102079e-02, 1.81427081e-02, 2.07829161e-02, 3.40338431e-03
        ], [
            6.89115196e-03, 1.05220859e-02, 1.12293641e-02, 1.20877789e-02, 1.32173266e-02,
            1.52273463e-02, 9.08115383e-03, 1.18410343e-02, 8.68920703e-03, 9.74443414e-03,
            1.61493252e-02, 1.37571817e-02, 1.02285203e-02, 7.83397855e-03, 9.18141378e-03,
            8.47683685e-03, 7.30571172e-03, 1.03472904e-02, 5.13232335e-03, 7.23953144e-03,
            4.44744053e-03, 5.66715983e-03, 6.96229284e-03, 1.15375556e+00, 7.22601676e-03,
            9.44570960e-03, 3.04195005e-03, 2.19123712e-03, 2.90305089e-03, 3.46257937e-03,
            3.84586610e-03, 2.73751497e-03, 3.01618937e-03, 3.95632152e-03, 9.26533662e-04
        ], [
            2.46539955e-03, 3.51302671e-03, 4.51757115e-03, 5.26944043e-03, 6.32465241e-03,
            5.19354054e-03, 5.90829415e-03, 3.45017558e-03, 4.86113505e-03, 5.14239731e-03,
            4.62625880e-03, 4.43961940e-03, 5.49322911e-03, 5.07916632e-03, 5.30072230e-03,
            4.14490503e-03, 2.68889209e-03, 4.11255591e-03, 4.49570941e-03, 4.91899522e-03,
            3.28248393e-03, 3.71895057e-03, 5.59644575e-03, 1.25021450e-02, 1.03743835e+00,
            2.03897703e-02, 6.14822465e-03, 5.59818437e-03, 1.78939328e-03, 6.46654869e-03,
            6.62871705e-03, 3.81827930e-03, 3.27596448e-03, 6.65102808e-03, 3.85568630e-04
        ], [
            1.12875169e-02, 1.21620724e-02, 2.12291399e-02, 1.84622309e-02, 2.11046521e-02,
            2.26092529e-02, 2.37584317e-02, 1.96329476e-02, 1.77347707e-02, 1.91373113e-02,
            2.27353197e-02, 1.92044959e-02, 1.79464073e-02, 1.47779472e-02, 1.90981161e-02,
            1.63802332e-02, 1.22032339e-02, 2.00012920e-02, 2.09569795e-02, 2.91893193e-02,
            1.83744107e-02, 1.77473599e-02, 5.42401878e-02, 1.56859581e-01, 1.11255574e-01,
            1.12680269e+00, 1.11694617e-02, 9.36913319e-03, 4.53527977e-03, 1.19192798e-02,
            1.22502715e-02, 7.40610205e-03, 9.96143829e-03, 1.40862605e-02, 1.28937666e-03
        ], [
            1.37408644e-02, 1.59281646e-02, 2.34927571e-02, 2.66025670e-02, 2.86818977e-02,
            2.51949444e-02, 3.49507191e-02, 1.77058837e-02, 2.74161701e-02, 2.60773687e-02,
            2.46629045e-02, 2.56478552e-02, 2.76569431e-02, 2.70333072e-02, 2.61427285e-02,
            2.63367337e-02, 1.84878065e-02, 3.14576537e-02, 3.26601876e-02, 3.31851391e-02,
            3.50186264e-02, 3.07943498e-02, 3.37428712e-02, 4.07314746e-02, 4.42814273e-02,
            4.64801931e-02, 1.14823205e+00, 5.02593189e-02, 1.54460330e-02, 4.43746041e-02,
            3.82918914e-02, 1.97730699e-02, 2.76241917e-02, 4.03951394e-02, 2.63349165e-03
        ], [
            4.56855315e-02, 4.99160559e-02, 5.96353539e-02, 7.11758054e-02, 6.41693737e-02,
            6.06424520e-02, 6.50332203e-02, 5.09400059e-02, 5.84982040e-02, 5.81905133e-02,
            6.25140343e-02, 6.08076783e-02, 6.05944987e-02, 5.24133532e-02, 6.03592480e-02,
            6.76846158e-02, 5.10299323e-02, 6.32912125e-02, 5.18189436e-02, 5.96171428e-02,
            6.40824864e-02, 5.99439930e-02, 8.29984636e-02, 7.58432132e-02, 7.28083163e-02,
            7.18237023e-02, 4.83069305e-02, 1.28107605e+00, 1.14366798e-01, 7.09572898e-02,
            5.31281881e-02, 3.14839540e-02, 5.30354847e-02, 8.07350880e-02, 1.06755269e-02
        ], [
            1.34717020e-02, 1.36928398e-02, 2.17444583e-02, 2.26096159e-02, 2.48269249e-02,
            2.28504489e-02, 2.81196818e-02, 1.43658480e-02, 2.04426149e-02, 2.26912159e-02,
            1.96689097e-02, 2.08763724e-02, 2.23855757e-02, 1.95315852e-02, 2.32988077e-02,
            2.15572202e-02, 1.49110031e-02, 2.67275026e-02, 3.58770075e-02, 3.16004499e-02,
            5.44137190e-02, 4.37753314e-02, 2.25473278e-02, 3.32075754e-02, 2.84880242e-02,
            3.92415554e-02, 2.85819411e-02, 4.07564740e-02, 1.04016739e+00, 3.93493574e-02,
            2.61739472e-02, 2.70045768e-02, 4.80770794e-02, 4.24776685e-02, 1.72151699e-03
        ], [
            5.01497283e-02, 6.54816733e-02, 1.05237963e-01, 9.90369818e-02, 9.90272609e-02,
            9.30275509e-02, 1.41215116e-01, 6.59605003e-02, 1.42452895e-01, 1.18634428e-01,
            9.79325196e-02, 9.77102375e-02, 1.19166001e-01, 1.22626567e-01, 1.28469265e-01,
            8.95634596e-02, 7.49584747e-02, 1.21374975e-01, 1.06198747e-01, 9.54186389e-02,
            1.01363671e-01, 9.80709380e-02, 9.70088380e-02, 9.47917391e-02, 1.44231594e-01,
            1.30459927e-01, 1.17622335e-01, 1.68433299e-01, 7.22129519e-02, 1.20215446e+00,
            1.32434339e-01, 6.31866909e-02, 1.10603048e-01, 1.36306274e-01, 1.04433229e-02
        ], [
            3.34460390e-03, 4.79184614e-03, 5.15658690e-03, 4.26334626e-03, 4.24280832e-03,
            7.13005873e-03, 6.74255433e-03, 4.01173141e-03, 6.00635688e-03, 5.43257497e-03,
            5.64362639e-03, 6.27665277e-03, 4.49761112e-03, 3.97408687e-03, 4.83930093e-03,
            4.97970475e-03, 6.85079266e-03, 4.73257119e-03, 4.93933121e-03, 3.16002833e-03,
            4.41005488e-03, 6.25846235e-03, 5.83145243e-03, 9.16196019e-03, 1.20007008e-02,
            6.24822364e-03, 5.10714328e-03, 3.66682256e-03, 5.05919246e-03, 5.25308914e-03,
            1.00948210e+00, 5.85864468e-03, 6.75536908e-03, 1.15026693e-02, 4.68372931e-04
        ], [
            1.27175422e-03, 1.11756966e-03, 1.73139687e-03, 1.69576564e-03, 1.87919614e-03,
            1.60166262e-03, 1.95709786e-03, 1.30672415e-03, 2.29126203e-03, 2.07149339e-03,
            1.75399870e-03, 1.89412597e-03, 2.23189372e-03, 2.07796381e-03, 2.34615335e-03,
            1.52304344e-03, 1.59956807e-03, 1.72969647e-03, 1.78835888e-03, 1.54945666e-03,
            1.65271057e-03, 1.58152642e-03, 2.11609021e-03, 1.51664028e-03, 3.20524488e-03,
            2.45269733e-03, 2.68047507e-03, 2.31060622e-03, 8.66645168e-04, 3.48746156e-03,
            5.48485926e-03, 1.01418931e+00, 2.10077547e-03, 2.74243737e-03, 1.47434699e-04
        ], [
            1.92121394e-03, 8.00101189e-04, 1.53209371e-03, 1.26618493e-03, 2.02732230e-03,
            1.37885927e-03, 1.16780673e-03, 7.66263262e-04, 1.25160803e-03, 1.17475014e-03,
            1.18116576e-03, 1.12690629e-03, 1.45355985e-03, 1.08463226e-03, 1.26605816e-03,
            1.16047073e-03, 8.24048199e-04, 1.08872295e-03, 1.06348017e-03, 9.17372251e-04,
            7.80653601e-04, 1.59694235e-03, 1.14957200e-03, 9.20151420e-04, 1.02778616e-03,
            1.16891251e-03, 1.16328633e-03, 9.63538211e-04, 3.54450518e-04, 1.15975393e-03,
            5.45086098e-03, 1.85160648e-03, 1.02760624e+00, 1.83566623e-03, 1.10673495e-04
        ], [
            9.58457660e-03, 1.34147103e-02, 1.78166190e-02, 1.81987733e-02, 2.16090414e-02,
            1.80937764e-02, 2.88416718e-02, 1.27833185e-02, 2.10971297e-02, 1.92245612e-02,
            1.78939930e-02, 1.85371912e-02, 1.85197638e-02, 1.62515148e-02, 1.92553447e-02,
            1.62441082e-02, 1.38965036e-02, 1.88884105e-02, 1.79630422e-02, 1.56835592e-02,
            1.69054182e-02, 2.42438668e-02, 2.16311917e-02, 1.77724098e-02, 2.05980656e-02,
            2.56291327e-02, 3.15528260e-02, 2.21892295e-02, 1.43818632e-02, 3.90034899e-02,
            2.77591575e-02, 1.82576627e-02, 2.41388104e-02, 1.09502154e+00, 3.97294274e-03
        ], [
            4.18050290e-05, 5.11311920e-05, 7.34884595e-05, 1.41068179e-04, 8.24330625e-05,
            8.12994538e-05, 9.01467836e-05, 5.06044363e-05, 6.60644332e-05, 7.24717068e-05,
            6.55169027e-05, 6.38918461e-05, 7.98806002e-05, 6.00109393e-05, 1.55716111e-04,
            5.67188124e-05, 3.69342234e-05, 1.18009057e-04, 1.00623762e-04, 4.30226451e-05,
            7.74985710e-05, 9.83081175e-05, 1.38314965e-04, 6.67200986e-05, 7.01950550e-05,
            6.91866975e-05, 7.77059523e-05, 1.12341898e-04, 2.65073014e-04, 1.10736972e-04,
            1.08588850e-04, 6.33057301e-05, 9.49631205e-05, 1.69949584e-04, 1.00013505e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 399)
    return ret


@cache('run')
def leontief_matrix_x2005x():
    """
    leontief matrix x2005x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17727159e+00, 4.50981961e-03, 3.68368415e-01, 9.61223075e-02, 1.36885580e-01,
            2.09523354e-01, 3.46974862e-02, 7.07646538e-03, 2.07705540e-02, 3.07556175e-02,
            1.00495108e-02, 8.07923837e-03, 8.60996687e-03, 7.70957320e-03, 9.59164862e-03,
            3.52297143e-02, 4.96273144e-03, 2.07761690e-02, 6.96527950e-03, 8.89496146e-03,
            8.65829406e-03, 1.07208064e-01, 8.32200388e-03, 9.50081219e-03, 8.75286422e-03,
            1.63893295e-02, 5.36740709e-03, 4.71628353e-03, 3.57282938e-03, 6.93394497e-03,
            1.11323439e-02, 1.02343656e-02, 1.31137171e-02, 1.25184522e-02, 2.64683703e-02
        ], [
            2.35435901e-02, 1.11416277e+00, 2.63473979e-02, 3.36459447e-02, 2.88900290e-02,
            3.39347223e-02, 2.98132523e-02, 5.51364699e-01, 9.14553010e-02, 4.77262105e-02,
            1.28070114e-01, 1.15642590e-01, 4.21312950e-02, 2.91226622e-02, 3.61336281e-02,
            3.26958232e-02, 1.64680639e-01, 6.16852415e-02, 1.80041340e-02, 1.09019025e-02,
            1.15828766e-02, 1.92784285e-02, 4.68344140e-02, 6.00093622e-02, 6.22328632e-02,
            3.01584781e-02, 1.16142685e-02, 7.21909226e-03, 9.35278239e-03, 1.17212204e-02,
            2.00032389e-02, 1.33589200e-02, 1.74962415e-02, 1.83831759e-02, 7.01531577e-02
        ], [
            8.59104074e-02, 4.30245808e-03, 1.21375282e+00, 2.32711551e-02, 1.59734173e-01,
            2.37974674e-02, 1.31849758e-02, 6.59604937e-03, 2.32998624e-02, 1.64098166e-02,
            9.63576783e-03, 8.69636087e-03, 9.20728469e-03, 8.45797077e-03, 9.12537932e-03,
            1.18642678e-02, 5.13286213e-03, 9.11185249e-03, 7.37426792e-03, 9.80780254e-03,
            9.83098433e-03, 1.92322078e-01, 7.65308386e-03, 1.27733928e-02, 1.22739385e-02,
            1.13735037e-02, 7.11709092e-03, 6.08837803e-03, 2.89177112e-03, 8.25605176e-03,
            1.51642088e-02, 1.42745834e-02, 2.24122113e-02, 1.54137382e-02, 5.87188301e-03
        ], [
            5.56000206e-03, 3.43853228e-03, 7.30645229e-03, 1.49762605e+00, 1.09331251e-01,
            1.09094725e-02, 1.48540542e-02, 3.87179496e-03, 1.02752759e-02, 2.74681560e-02,
            1.01570342e-02, 7.53316268e-03, 9.21419134e-03, 8.46992913e-03, 1.65987466e-02,
            3.85107555e-02, 3.46370485e-03, 8.75279164e-03, 6.41524263e-03, 5.27082797e-03,
            6.73796551e-03, 7.55357368e-03, 6.00760103e-03, 7.74582636e-03, 5.99845157e-03,
            5.87544608e-03, 3.40489347e-03, 2.50354742e-03, 1.82212099e-03, 4.43874905e-03,
            6.93923081e-03, 3.18047162e-03, 7.89062726e-03, 1.08478587e-02, 6.66986402e-04
        ], [
            4.54009228e-04, 2.89836506e-04, 6.40658585e-04, 1.83107967e-02, 1.25870582e+00,
            1.15991558e-03, 1.49461652e-03, 3.57490265e-04, 7.72640723e-04, 1.98672564e-03,
            8.02791172e-04, 7.50361554e-04, 8.82299535e-04, 9.42815786e-04, 3.63068052e-03,
            6.34924088e-03, 3.63659438e-04, 6.68367386e-04, 1.11401997e-03, 6.51026812e-04,
            5.74382512e-04, 5.14885703e-04, 7.00947824e-04, 6.14718863e-04, 6.35210871e-04,
            5.55070209e-04, 5.79022963e-04, 2.27667163e-04, 1.39157888e-04, 3.82302672e-04,
            9.23714781e-04, 3.15940283e-04, 4.70562223e-04, 9.34630856e-04, 5.79749013e-05
        ], [
            4.61778422e-03, 4.22281590e-03, 6.48839764e-03, 6.63565828e-03, 7.52571839e-03,
            1.31786825e+00, 2.19837491e-02, 3.89438484e-03, 5.89860242e-03, 8.86633367e-03,
            1.46804340e-02, 1.06786410e-02, 9.68931449e-03, 7.10811367e-03, 1.04674527e-02,
            9.80871128e-02, 4.22624501e-03, 4.88904072e-02, 5.05381128e-03, 3.76561993e-03,
            3.73497592e-03, 6.41697156e-03, 4.77325893e-03, 3.92942213e-03, 3.80754751e-03,
            7.19074218e-03, 3.42120391e-03, 2.57480088e-03, 6.11312493e-03, 3.74879480e-03,
            5.80130205e-03, 5.79851087e-03, 4.49783409e-03, 7.17753366e-03, 7.23740277e-04
        ], [
            1.43449720e-02, 1.16196931e-02, 5.30993052e-02, 3.93036960e-02, 4.74797667e-02,
            3.82432414e-02, 1.33285340e+00, 1.39195841e-02, 4.08650611e-02, 4.50025652e-02,
            4.56012693e-02, 2.35892270e-02, 2.91318488e-02, 3.26690342e-02, 2.69141282e-02,
            4.23100320e-02, 1.38253145e-02, 2.44977252e-02, 2.33593407e-02, 2.38183882e-02,
            2.46000375e-02, 2.82200418e-02, 2.03414645e-02, 1.81226451e-02, 2.10034065e-02,
            3.17629665e-02, 2.52255865e-02, 3.12380576e-02, 1.02023228e-02, 3.94158733e-02,
            3.62678292e-02, 3.10046927e-02, 2.44316298e-02, 4.66764214e-02, 1.99065971e-03
        ], [
            2.31373144e-02, 1.54013235e-02, 1.96863517e-02, 2.54649233e-02, 2.27681907e-02,
            2.26478513e-02, 1.85685410e-02, 1.08635781e+00, 6.68211814e-02, 3.50146633e-02,
            3.72775037e-02, 3.06411007e-02, 1.93870676e-02, 1.50468319e-02, 1.74127296e-02,
            1.69954782e-02, 3.34827180e-02, 2.70952069e-02, 1.35573683e-02, 1.13313518e-02,
            9.39297933e-03, 1.31176550e-02, 6.42872168e-02, 9.63684219e-02, 1.02693599e-01,
            3.52064105e-02, 9.40076870e-03, 5.99368742e-03, 4.82180164e-03, 1.07296205e-02,
            1.66263783e-02, 8.05263125e-03, 1.23745682e-02, 1.31367347e-02, 1.77236121e-03
        ], [
            6.68543742e-02, 2.35175837e-02, 5.16757294e-02, 1.55349166e-01, 1.08364265e-01,
            8.28957117e-02, 7.93829188e-02, 3.98089497e-02, 1.33550058e+00, 3.36867365e-01,
            7.35931498e-02, 4.80297034e-02, 4.96252441e-02, 6.06712064e-02, 5.98264288e-02,
            6.76683393e-02, 2.13873726e-02, 4.49853867e-02, 2.80008910e-02, 1.23447275e-02,
            1.20259122e-02, 2.47350805e-02, 2.05437469e-02, 2.17613667e-02, 2.14091767e-02,
            2.09359899e-02, 1.25976334e-02, 8.23836214e-03, 7.95236934e-03, 1.67074584e-02,
            2.57109520e-02, 1.27338399e-02, 9.53085074e-02, 2.67695018e-02, 3.74062342e-03
        ], [
            1.36872880e-02, 1.26216850e-02, 3.56704060e-02, 3.09653782e-02, 6.99875875e-02,
            2.43966756e-02, 3.33901653e-02, 1.35124249e-02, 3.54873757e-02, 1.17830582e+00,
            2.54294951e-02, 2.25401726e-02, 4.96948039e-02, 5.12720012e-02, 7.43699121e-02,
            5.10915824e-02, 1.05266568e-02, 3.57458047e-02, 3.26801869e-02, 9.83691541e-03,
            1.08193865e-02, 1.52331571e-02, 2.19349339e-02, 1.38479647e-02, 1.48299235e-02,
            1.67546788e-02, 1.09043517e-02, 5.05852718e-03, 5.00594832e-03, 9.34083975e-03,
            1.16891620e-02, 5.90459071e-03, 1.41551974e-02, 1.28043642e-02, 1.58514568e-03
        ], [
            5.39940698e-03, 8.97608617e-03, 1.20346725e-02, 8.73634081e-03, 8.37431308e-03,
            1.59402871e-02, 6.22764077e-03, 8.73835690e-03, 1.29866086e-02, 1.41456943e-02,
            1.15904440e+00, 2.25080140e-02, 1.64852272e-02, 2.32107218e-02, 1.99898157e-02,
            1.43424974e-02, 7.84537284e-03, 1.11434957e-01, 9.60749970e-03, 3.53043870e-03,
            3.72279647e-03, 8.33759620e-03, 6.22101176e-03, 4.91933534e-03, 5.25185173e-03,
            6.68765265e-03, 5.51501140e-03, 2.59135812e-03, 6.87739181e-03, 4.14090611e-03,
            6.44909621e-03, 4.98887490e-03, 6.39060918e-03, 6.27742772e-03, 9.34394107e-04
        ], [
            2.19888176e-02, 4.73591028e-02, 4.11388435e-02, 3.68385676e-02, 4.58369026e-02,
            6.72663314e-02, 3.78364017e-02, 4.21465424e-02, 4.71292897e-02, 7.00665570e-02,
            8.36650527e-02, 1.50715080e+00, 3.15172990e-01, 1.70632375e-01, 2.41486355e-01,
            1.42498459e-01, 3.48844275e-02, 1.71154790e-01, 6.62963641e-02, 1.63377244e-02,
            1.60519299e-02, 2.35732035e-02, 3.51078105e-02, 3.53001849e-02, 3.53982072e-02,
            3.53039878e-02, 2.42896302e-02, 1.04740218e-02, 1.49348418e-02, 1.96029298e-02,
            2.72999955e-02, 1.39217831e-02, 2.06418411e-02, 2.46553557e-02, 4.69922430e-03
        ], [
            1.51457814e-02, 2.41617790e-02, 1.64443094e-02, 2.54877918e-02, 2.23396296e-02,
            2.48884971e-02, 1.91723579e-02, 2.43086946e-02, 2.12509482e-02, 2.92460269e-02,
            3.77836349e-02, 4.63830224e-02, 1.19208474e+00, 4.06858847e-02, 7.84735673e-02,
            2.79094601e-02, 1.98261069e-02, 3.79592797e-02, 4.16703008e-02, 7.96604257e-03,
            7.69380653e-03, 1.00865185e-02, 1.67639471e-02, 2.29059367e-02, 1.96571917e-02,
            1.77202628e-02, 9.75415029e-03, 5.00626737e-03, 5.51997204e-03, 8.76008870e-03,
            1.45427574e-02, 6.56565513e-03, 1.02838376e-02, 1.15554022e-02, 2.46971785e-03
        ], [
            1.69832372e-02, 2.65280156e-02, 2.80200652e-02, 3.90481733e-02, 3.54464343e-02,
            3.85367897e-02, 4.34481030e-02, 2.96582766e-02, 4.34666491e-02, 5.35559124e-02,
            4.73239715e-02, 6.72540993e-02, 2.12502649e-01, 1.56254710e+00, 1.75300256e-01,
            6.82656415e-02, 5.60346249e-02, 8.82424723e-02, 8.39911250e-02, 2.61407960e-02,
            2.67117519e-02, 2.52687117e-02, 3.60115640e-02, 3.25809399e-02, 3.65703219e-02,
            3.97341252e-02, 8.66516239e-02, 2.18118162e-02, 1.43106612e-02, 4.31523615e-02,
            4.53413563e-02, 2.53290025e-02, 5.45786297e-02, 3.93899704e-02, 4.21165690e-03
        ], [
            1.39504162e-02, 1.51264675e-02, 1.62860285e-02, 1.78430760e-02, 1.80446756e-02,
            1.97753140e-02, 1.59843519e-02, 1.55246100e-02, 1.53020675e-02, 1.97971521e-02,
            2.15585059e-02, 2.50163104e-02, 4.05595691e-02, 2.06750163e-02, 1.46387179e+00,
            2.11927700e-02, 1.34860648e-02, 2.12683457e-02, 1.33650198e-01, 1.27063659e-02,
            1.22906004e-02, 1.12515190e-02, 6.52803856e-02, 7.22065444e-02, 8.63974222e-02,
            3.53109806e-02, 1.23494414e-02, 7.06212086e-03, 5.09226445e-03, 1.28447710e-02,
            3.22877206e-02, 7.77759656e-03, 8.89451317e-03, 1.96482008e-02, 1.87774441e-03
        ], [
            2.00815802e-03, 2.34885481e-03, 3.41847823e-03, 9.88049246e-03, 6.88351029e-03,
            8.19824909e-03, 6.87010056e-03, 2.81980185e-03, 4.65741930e-03, 6.27691772e-03,
            6.22250624e-03, 1.52215361e-02, 9.66289361e-03, 7.12432474e-03, 1.22397492e-02,
            1.05208297e+00, 3.40722488e-03, 9.57787350e-03, 4.80506467e-03, 2.44530132e-03,
            2.83103965e-03, 4.23036462e-03, 5.10348305e-03, 3.11101047e-03, 4.51680471e-03,
            3.89069286e-03, 2.83274944e-03, 2.19386544e-03, 2.50460309e-03, 3.45100973e-03,
            5.64990311e-03, 4.32242973e-03, 4.83985302e-03, 7.55393505e-03, 2.51516952e-03
        ], [
            2.54395688e-02, 4.71677167e-02, 3.80254973e-02, 5.02713131e-02, 3.63549802e-02,
            4.52814154e-02, 4.98493339e-02, 5.79098639e-02, 5.72024139e-02, 5.34963669e-02,
            7.70767851e-02, 6.83810782e-02, 4.21083059e-02, 3.24334213e-02, 3.87083294e-02,
            3.24621065e-02, 1.15277416e+00, 3.41328860e-02, 2.66686301e-02, 1.50758535e-02,
            2.37097562e-02, 4.21343451e-02, 3.24881273e-02, 2.14261892e-02, 2.35939816e-02,
            3.29836820e-02, 2.11473600e-02, 1.30782349e-02, 1.74582983e-02, 1.62933086e-02,
            2.76727878e-02, 3.13836010e-02, 2.65861336e-02, 3.08547340e-02, 4.19934417e-03
        ], [
            7.56980382e-03, 1.15070438e-02, 9.45899531e-03, 1.02654795e-02, 9.22620741e-03,
            1.06540245e-02, 1.03994750e-02, 1.26579514e-02, 1.06862138e-02, 1.02685858e-02,
            1.44736455e-02, 1.25720977e-02, 1.05110125e-02, 8.42384004e-03, 9.54416792e-03,
            8.83224084e-03, 2.46439945e-02, 1.05904294e+00, 1.07743621e-02, 7.79916872e-03,
            9.57844509e-03, 1.21105624e-02, 1.55538529e-02, 9.67950191e-03, 9.98971914e-03,
            2.02889107e-02, 1.63896698e-02, 1.06419019e-02, 3.82922798e-02, 9.90541562e-03,
            2.36302995e-02, 1.62256718e-02, 1.22384539e-02, 1.57727659e-02, 1.76567113e-03
        ], [
            9.56807445e-03, 6.75821379e-03, 1.50911446e-02, 1.40190336e-02, 1.62680961e-02,
            1.46734855e-02, 1.18790271e-02, 8.91873852e-03, 1.32212733e-02, 1.45376362e-02,
            1.40908469e-02, 1.40732761e-02, 1.29480370e-02, 1.06353509e-02, 1.43226029e-02,
            1.31574117e-02, 1.24652380e-02, 1.45144974e-02, 1.01661032e+00, 7.54785247e-03,
            7.17585783e-03, 1.09096609e-02, 2.14261487e-02, 7.92431938e-03, 1.08296517e-02,
            1.37361305e-02, 7.52727437e-03, 6.49369483e-03, 4.30697746e-03, 9.39747440e-03,
            8.45256781e-03, 5.49442303e-03, 8.07510998e-03, 1.08825316e-02, 8.60043056e-04
        ], [
            5.64340856e-02, 3.44647303e-02, 1.09812399e-01, 1.01205744e-01, 1.16212919e-01,
            1.01932109e-01, 8.83264770e-02, 7.72157731e-02, 8.60977866e-02, 9.44515839e-02,
            7.36639761e-02, 8.91010605e-02, 9.30417553e-02, 8.04969841e-02, 1.03648202e-01,
            8.00002682e-02, 4.20671971e-02, 7.93045253e-02, 6.18777934e-02, 1.03750362e+00,
            2.57580178e-02, 7.84492664e-02, 4.61194901e-02, 4.75656102e-02, 4.92119997e-02,
            4.40308087e-02, 2.77249022e-02, 1.66211765e-02, 1.34970542e-02, 2.87592471e-02,
            3.99801409e-02, 2.33762553e-02, 4.82872044e-02, 3.73487239e-02, 4.55874589e-03
        ], [
            3.05972458e-02, 1.57895589e-02, 5.49286493e-02, 5.47496459e-02, 6.88802922e-02,
            4.41734771e-02, 3.76157269e-02, 2.74154049e-02, 4.17672318e-02, 4.45182048e-02,
            3.67729366e-02, 3.82980869e-02, 3.98077121e-02, 3.40158402e-02, 4.65382721e-02,
            4.42268953e-02, 1.79007368e-02, 5.20620233e-02, 2.83381065e-02, 1.29817408e-02,
            1.01393084e+00, 4.10803642e-02, 2.61225911e-02, 2.14400797e-02, 2.32527359e-02,
            2.13128329e-02, 1.76773282e-02, 1.21196751e-02, 8.81056392e-03, 1.70353108e-02,
            1.82132597e-02, 1.24636451e-02, 2.36507861e-02, 2.20276486e-02, 2.44731958e-03
        ], [
            5.80566175e-03, 7.46371110e-03, 1.17709517e-02, 1.35739493e-02, 1.43672035e-02,
            1.25664280e-02, 1.44175071e-02, 8.02610308e-03, 1.24187325e-02, 1.39815226e-02,
            1.31879754e-02, 1.30404287e-02, 1.46624737e-02, 1.28142149e-02, 1.25286115e-02,
            1.08875775e-02, 8.90909129e-03, 1.34151587e-02, 1.17553088e-02, 1.24395839e-02,
            1.15459829e-02, 1.01692507e+00, 1.48969727e-02, 1.38333511e-02, 2.59624540e-02,
            3.04076583e-02, 1.62646213e-02, 1.85031508e-02, 6.99736574e-03, 2.00775388e-02,
            1.74972363e-02, 1.22602536e-02, 1.46697538e-02, 1.92436973e-02, 9.80507247e-04
        ], [
            2.70620989e-02, 3.04060427e-02, 4.92921218e-02, 4.92443289e-02, 5.09919078e-02,
            5.11963505e-02, 4.08544221e-02, 6.52433097e-02, 4.20103227e-02, 4.24835062e-02,
            5.88834341e-02, 4.37954351e-02, 3.62801112e-02, 2.85150406e-02, 3.71521711e-02,
            3.82977724e-02, 3.33065288e-02, 4.25103258e-02, 3.05113468e-02, 2.98897246e-02,
            2.11281795e-02, 2.64763105e-02, 1.06396457e+00, 4.45107647e-02, 3.21920758e-02,
            7.66735642e-02, 1.82073756e-02, 1.26985246e-02, 7.39365607e-03, 1.45800689e-02,
            2.14672909e-02, 1.70789811e-02, 1.86079690e-02, 2.10813256e-02, 3.38310080e-03
        ], [
            6.37972134e-03, 9.92960110e-03, 1.10580808e-02, 1.20833336e-02, 1.25946711e-02,
            1.47124539e-02, 8.94754141e-03, 1.15242117e-02, 8.77995453e-03, 9.58411129e-03,
            1.65127191e-02, 1.39719842e-02, 1.01807663e-02, 7.73855778e-03, 9.42597653e-03,
            8.01648226e-03, 6.99697025e-03, 9.84634948e-03, 5.39682886e-03, 7.28332142e-03,
            5.85552922e-03, 5.69274455e-03, 7.23253327e-03, 1.17757258e+00, 7.09334186e-03,
            9.63913699e-03, 3.20447856e-03, 2.36170535e-03, 3.57750893e-03, 3.52660255e-03,
            3.79611659e-03, 2.65843983e-03, 3.17486000e-03, 4.03068514e-03, 9.04808833e-04
        ], [
            2.35764851e-03, 3.06067103e-03, 4.29339345e-03, 5.18016854e-03, 6.16191404e-03,
            4.93618512e-03, 5.50600333e-03, 3.33914636e-03, 4.75191502e-03, 4.95679024e-03,
            4.48490618e-03, 4.27769432e-03, 5.40177476e-03, 4.95473477e-03, 5.04238229e-03,
            3.91929978e-03, 2.61382140e-03, 4.06893303e-03, 4.35896193e-03, 4.65808962e-03,
            3.29857876e-03, 3.57197673e-03, 5.24159547e-03, 1.10005118e-02, 1.03843044e+00,
            1.88850160e-02, 6.10928653e-03, 5.39060648e-03, 1.80293984e-03, 5.95182997e-03,
            6.19645964e-03, 3.87046180e-03, 3.14735945e-03, 6.57130689e-03, 3.61253467e-04
        ], [
            1.13355310e-02, 1.19284208e-02, 2.14413891e-02, 1.89506249e-02, 2.12435122e-02,
            2.28620636e-02, 2.34870410e-02, 2.02614812e-02, 1.80443524e-02, 1.94144552e-02,
            2.31120888e-02, 1.95540863e-02, 1.79946604e-02, 1.47940682e-02, 1.88801756e-02,
            1.62670926e-02, 1.21450174e-02, 2.25332937e-02, 2.18912033e-02, 2.98446322e-02,
            1.89936944e-02, 1.80769829e-02, 5.40446743e-02, 1.50509462e-01, 1.14557070e-01,
            1.12326771e+00, 1.13126636e-02, 9.65349379e-03, 4.86025661e-03, 1.20568002e-02,
            1.24046489e-02, 7.65102162e-03, 1.03218674e-02, 1.45471668e-02, 1.31827712e-03
        ], [
            1.38601642e-02, 1.54313724e-02, 2.35670876e-02, 2.72508523e-02, 2.83308037e-02,
            2.50555956e-02, 3.45210313e-02, 1.86056510e-02, 2.73835318e-02, 2.60369324e-02,
            2.46176373e-02, 2.61976199e-02, 2.73662724e-02, 2.72058631e-02, 2.57125340e-02,
            2.67809506e-02, 1.93501660e-02, 3.18451965e-02, 3.26701895e-02, 3.30661974e-02,
            3.49926079e-02, 3.13487599e-02, 3.43143264e-02, 3.95056567e-02, 4.26042007e-02,
            4.42134939e-02, 1.15256284e+00, 5.18607685e-02, 1.62845526e-02, 4.27770045e-02,
            3.86698350e-02, 2.02680785e-02, 2.85118143e-02, 4.07487850e-02, 2.72295701e-03
        ], [
            4.42502295e-02, 4.86653385e-02, 5.92428073e-02, 7.29577476e-02, 6.50755004e-02,
            6.01092066e-02, 6.49047026e-02, 5.33307469e-02, 6.01293058e-02, 5.95382410e-02,
            6.37941699e-02, 6.30782551e-02, 6.17753599e-02, 5.32984515e-02, 6.05833332e-02,
            6.73933551e-02, 5.35425120e-02, 6.51371029e-02, 5.30684265e-02, 5.93622713e-02,
            6.54665022e-02, 6.14859247e-02, 8.31633872e-02, 7.09335371e-02, 7.15713059e-02,
            7.44968820e-02, 4.91811038e-02, 1.29754106e+00, 1.21249998e-01, 7.05823205e-02,
            5.40614996e-02, 3.25896342e-02, 5.48323627e-02, 8.37374941e-02, 1.25806713e-02
        ], [
            1.28413406e-02, 1.39333018e-02, 2.15498885e-02, 2.33218477e-02, 2.53766326e-02,
            2.26878875e-02, 2.84970461e-02, 1.53039670e-02, 2.09390043e-02, 2.31321332e-02,
            2.02257092e-02, 2.13214479e-02, 2.25439895e-02, 1.97341148e-02, 2.36021256e-02,
            2.16006899e-02, 1.49701685e-02, 2.72477889e-02, 3.57696970e-02, 3.12336686e-02,
            5.52519913e-02, 4.53342477e-02, 2.29911143e-02, 3.09402931e-02, 2.67339797e-02,
            3.78202404e-02, 2.95089682e-02, 4.40491726e-02, 1.03875310e+00, 3.97132789e-02,
            2.60339493e-02, 2.81371842e-02, 5.00665698e-02, 4.34444563e-02, 1.82349953e-03
        ], [
            5.04032469e-02, 6.67758624e-02, 1.06665178e-01, 1.00842646e-01, 9.92844217e-02,
            9.35033765e-02, 1.41141787e-01, 7.13927188e-02, 1.46082208e-01, 1.20809287e-01,
            9.92298031e-02, 1.00105179e-01, 1.19899213e-01, 1.21285407e-01, 1.30304649e-01,
            8.99574698e-02, 7.86765771e-02, 1.25611399e-01, 1.06621469e-01, 9.64595027e-02,
            1.02412086e-01, 1.01033492e-01, 9.76659277e-02, 8.93230748e-02, 1.44118035e-01,
            1.27458337e-01, 1.17377737e-01, 1.77164961e-01, 7.70962949e-02, 1.20354829e+00,
            1.35616298e-01, 6.61990661e-02, 1.15853042e-01, 1.40107869e-01, 1.09231534e-02
        ], [
            3.38304868e-03, 4.59135053e-03, 5.24333426e-03, 4.39087849e-03, 4.33709008e-03,
            7.24149680e-03, 7.07355169e-03, 4.39184878e-03, 6.23164369e-03, 5.66311280e-03,
            5.96680623e-03, 6.87641074e-03, 4.76426555e-03, 4.06495006e-03, 4.96721491e-03,
            5.03085697e-03, 6.99469731e-03, 4.95669518e-03, 5.04208665e-03, 3.19340104e-03,
            4.38961956e-03, 6.33457612e-03, 6.17518204e-03, 8.49049671e-03, 1.19999803e-02,
            6.23484099e-03, 5.05412942e-03, 3.80407042e-03, 5.39035763e-03, 5.26432060e-03,
            1.00971013e+00, 6.01220961e-03, 6.60600815e-03, 1.16255855e-02, 4.70471583e-04
        ], [
            1.47034277e-03, 1.04710265e-03, 1.74951543e-03, 1.69157554e-03, 1.82423184e-03,
            1.59732115e-03, 1.83086693e-03, 1.30082028e-03, 2.28492301e-03, 2.00843485e-03,
            1.68534287e-03, 1.82336709e-03, 2.14555966e-03, 2.00028358e-03, 2.30347325e-03,
            1.48071508e-03, 1.54053502e-03, 1.69719105e-03, 1.75396122e-03, 1.52704033e-03,
            1.73987236e-03, 1.59910694e-03, 2.05232149e-03, 1.40484477e-03, 3.15320031e-03,
            2.33817701e-03, 2.41977858e-03, 2.39040431e-03, 8.71499741e-04, 3.55384287e-03,
            5.58643501e-03, 1.01449358e+00, 2.36535433e-03, 2.69093714e-03, 1.54976796e-04
        ], [
            1.91127620e-03, 8.00153098e-04, 1.59766062e-03, 1.34816594e-03, 2.14826922e-03,
            1.47573491e-03, 1.24026229e-03, 8.29689234e-04, 1.30581698e-03, 1.24792659e-03,
            1.35142092e-03, 1.24499500e-03, 1.63588060e-03, 1.17002416e-03, 1.38299296e-03,
            1.20482436e-03, 8.83400805e-04, 1.16048553e-03, 1.10740698e-03, 9.41794871e-04,
            8.04029386e-04, 1.69082790e-03, 1.18650982e-03, 9.36250702e-04, 1.06603659e-03,
            1.15249004e-03, 1.17274978e-03, 1.08484712e-03, 3.86558927e-04, 1.13384649e-03,
            5.77544066e-03, 2.00958747e-03, 1.03664513e+00, 1.94416119e-03, 1.16594626e-04
        ], [
            9.66776380e-03, 1.33031724e-02, 1.79380616e-02, 1.84167668e-02, 2.14240112e-02,
            1.77524284e-02, 2.82927184e-02, 1.33159916e-02, 2.11170400e-02, 1.91626825e-02,
            1.80118751e-02, 1.87173888e-02, 1.81709573e-02, 1.58108920e-02, 1.88902753e-02,
            1.57791876e-02, 1.43644341e-02, 1.91399723e-02, 1.75049088e-02, 1.53220128e-02,
            1.71381492e-02, 2.47493980e-02, 2.25425444e-02, 1.68463003e-02, 2.02095537e-02,
            2.57908784e-02, 3.10208686e-02, 2.27557377e-02, 1.48453923e-02, 3.79091216e-02,
            2.79637185e-02, 1.89433502e-02, 2.44079488e-02, 1.09471687e+00, 3.60946758e-03
        ], [
            4.20195217e-05, 5.17537231e-05, 7.25789601e-05, 1.60044070e-04, 8.91216616e-05,
            8.11456494e-05, 8.47801249e-05, 5.66028325e-05, 6.82793313e-05, 7.35491413e-05,
            6.82386433e-05, 6.58147240e-05, 8.36941618e-05, 6.09342495e-05, 1.58950870e-04,
            5.80088732e-05, 3.90206393e-05, 1.19189352e-04, 9.30829306e-05, 4.24544077e-05,
            7.45454608e-05, 9.73732987e-05, 1.54469031e-04, 7.42679567e-05, 6.41988043e-05,
            6.58564541e-05, 7.55172317e-05, 1.12621055e-04, 2.55330081e-04, 1.07381403e-04,
            1.04375434e-04, 6.63176702e-05, 9.36354609e-05, 1.61794229e-04, 1.00012229e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 400)
    return ret


@cache('run')
def leontief_matrix_x2006x():
    """
    leontief matrix x2006x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.17488806e+00, 4.40977486e-03, 3.67799437e-01, 1.02134026e-01, 1.44956970e-01,
            2.07452363e-01, 3.52882800e-02, 7.90503803e-03, 2.21322191e-02, 3.20947309e-02,
            1.07100321e-02, 8.63220712e-03, 8.90836498e-03, 7.99040041e-03, 9.82198665e-03,
            3.86932708e-02, 5.16174804e-03, 1.95243678e-02, 6.92797053e-03, 8.75656775e-03,
            8.34807987e-03, 1.07327108e-01, 8.41499188e-03, 1.00131459e-02, 9.21375712e-03,
            1.61393571e-02, 5.15066436e-03, 4.58213820e-03, 3.56290356e-03, 6.97438619e-03,
            1.10830511e-02, 1.07611485e-02, 1.32737574e-02, 1.26868714e-02, 3.10875447e-02
        ], [
            2.41138494e-02, 1.10172077e+00, 2.68566691e-02, 3.57238048e-02, 3.08218051e-02,
            3.49109214e-02, 3.09271579e-02, 5.98205733e-01, 9.81126255e-02, 5.08950101e-02,
            1.28616545e-01, 1.16956261e-01, 4.25788159e-02, 2.99984137e-02, 3.63213635e-02,
            3.44587157e-02, 1.63318821e-01, 6.20679673e-02, 1.83400810e-02, 1.16619714e-02,
            1.16519081e-02, 1.95339448e-02, 5.09085652e-02, 6.67476096e-02, 7.21276568e-02,
            3.27179250e-02, 1.16225725e-02, 7.17469972e-03, 9.40300890e-03, 1.20568189e-02,
            2.00309275e-02, 1.31053221e-02, 1.83116966e-02, 1.86322590e-02, 7.06364243e-02
        ], [
            9.20543911e-02, 4.49649618e-03, 1.22276539e+00, 2.67151341e-02, 1.72531316e-01,
            2.59675262e-02, 1.43062008e-02, 7.43919690e-03, 2.52697878e-02, 1.82834822e-02,
            1.07928865e-02, 9.77695845e-03, 1.00290897e-02, 9.16330642e-03, 9.84507491e-03,
            1.35956437e-02, 5.59816908e-03, 9.94285725e-03, 7.36994684e-03, 1.01094563e-02,
            9.93919333e-03, 1.95740947e-01, 8.08218275e-03, 1.41244664e-02, 1.34443034e-02,
            1.17210512e-02, 6.97891182e-03, 6.11198635e-03, 2.99262386e-03, 8.56287839e-03,
            1.53401577e-02, 1.54904708e-02, 2.24851407e-02, 1.64976244e-02, 6.67917881e-03
        ], [
            5.78551814e-03, 3.65587530e-03, 7.93622071e-03, 1.54223794e+00, 1.20429016e-01,
            1.17885485e-02, 1.63235664e-02, 4.53839928e-03, 1.12179322e-02, 3.07597456e-02,
            1.13913011e-02, 8.76028211e-03, 1.00601912e-02, 9.16330961e-03, 1.68752708e-02,
            4.30004220e-02, 3.96858213e-03, 9.59508557e-03, 6.71414922e-03, 5.74714861e-03,
            7.05402388e-03, 8.01680594e-03, 6.54165856e-03, 9.07414603e-03, 6.89951059e-03,
            6.54145640e-03, 3.51024803e-03, 2.62257146e-03, 2.00387330e-03, 4.83281306e-03,
            7.38541057e-03, 3.51696069e-03, 8.59429884e-03, 1.11087588e-02, 7.31706164e-04
        ], [
            5.26981074e-04, 3.13075269e-04, 7.30676472e-04, 2.25557992e-02, 1.27525499e+00,
            1.39822736e-03, 1.83728133e-03, 4.25435061e-04, 8.95236595e-04, 2.29180414e-03,
            9.20910730e-04, 8.49550703e-04, 9.87741749e-04, 1.08872322e-03, 4.03743656e-03,
            7.72562268e-03, 4.10248433e-04, 7.62783817e-04, 1.14188853e-03, 7.00799789e-04,
            6.19597602e-04, 5.83929871e-04, 7.67507425e-04, 7.02564222e-04, 7.25565598e-04,
            6.50702452e-04, 5.95983471e-04, 2.46886111e-04, 1.58473872e-04, 4.22208569e-04,
            1.14613085e-03, 3.57860987e-04, 5.42819915e-04, 1.04267261e-03, 6.38120226e-05
        ], [
            4.75724550e-03, 4.30029055e-03, 6.90621167e-03, 7.25405978e-03, 8.15815704e-03,
            1.34282804e+00, 2.36629084e-02, 4.49221298e-03, 6.24072191e-03, 9.54129136e-03,
            1.62339389e-02, 1.19933917e-02, 1.04274874e-02, 7.58795885e-03, 1.08799865e-02,
            1.05371734e-01, 4.45538623e-03, 4.92896330e-02, 5.15893258e-03, 3.82295537e-03,
            3.77097763e-03, 6.48807795e-03, 4.97565588e-03, 4.25173105e-03, 4.02893188e-03,
            7.19760246e-03, 3.41450345e-03, 2.54362745e-03, 6.32721417e-03, 3.86519415e-03,
            6.11193416e-03, 6.20129216e-03, 4.80883220e-03, 7.39837867e-03, 7.48893736e-04
        ], [
            1.44225860e-02, 1.13089318e-02, 5.30216271e-02, 4.01379183e-02, 4.78950694e-02,
            3.94811331e-02, 1.34099260e+00, 1.51049510e-02, 4.04035376e-02, 4.56357698e-02,
            4.63819732e-02, 2.36750433e-02, 2.85810693e-02, 3.17657583e-02, 2.64615599e-02,
            4.29856825e-02, 1.40343277e-02, 2.47392568e-02, 2.29285899e-02, 2.34590446e-02,
            2.36381445e-02, 2.80484083e-02, 2.05378503e-02, 1.83468529e-02, 2.15191958e-02,
            3.20019474e-02, 2.37437337e-02, 2.96540307e-02, 9.88828223e-03, 3.87558912e-02,
            3.57303343e-02, 3.13605447e-02, 2.44379512e-02, 4.53500413e-02, 2.08666425e-03
        ], [
            2.55369389e-02, 1.55671858e-02, 2.19314298e-02, 2.92496866e-02, 2.56758091e-02,
            2.55673235e-02, 2.11324358e-02, 1.14192703e+00, 8.11254249e-02, 4.12372047e-02,
            4.21698217e-02, 3.47983288e-02, 2.13419934e-02, 1.70163538e-02, 1.93982416e-02,
            2.01312689e-02, 4.03395474e-02, 2.98338364e-02, 1.48570885e-02, 1.28474645e-02,
            1.00082863e-02, 1.44947542e-02, 7.14606842e-02, 1.06559654e-01, 1.18189832e-01,
            3.86921307e-02, 1.01152761e-02, 6.38981683e-03, 5.31221128e-03, 1.16060101e-02,
            1.73431117e-02, 9.08392180e-03, 1.41055838e-02, 1.45170045e-02, 2.03779031e-03
        ], [
            6.68404885e-02, 2.32708862e-02, 5.27567157e-02, 1.62313127e-01, 1.13516719e-01,
            8.81091560e-02, 8.25563925e-02, 4.45730233e-02, 1.34937661e+00, 3.55521927e-01,
            7.86359328e-02, 4.98943152e-02, 5.13896813e-02, 6.43493804e-02, 6.14756410e-02,
            7.37300633e-02, 2.27285590e-02, 4.75959018e-02, 2.80097215e-02, 1.27754221e-02,
            1.22451245e-02, 2.55277577e-02, 2.18351896e-02, 2.39727857e-02, 2.32002155e-02,
            2.22842385e-02, 1.27600154e-02, 8.21804330e-03, 8.46931403e-03, 1.72934051e-02,
            2.66284623e-02, 1.35328061e-02, 9.80262170e-02, 2.79311576e-02, 4.10927155e-03
        ], [
            1.40837888e-02, 1.24539707e-02, 3.67804538e-02, 3.28898638e-02, 7.40734439e-02,
            2.52271943e-02, 3.49762750e-02, 1.56083474e-02, 3.66916605e-02, 1.19013638e+00,
            2.74940002e-02, 2.38109205e-02, 5.01683288e-02, 5.30530096e-02, 7.46844044e-02,
            5.38525453e-02, 1.11624420e-02, 3.70255558e-02, 3.19268311e-02, 1.01298438e-02,
            1.07101606e-02, 1.57093687e-02, 2.26400857e-02, 1.50550770e-02, 1.59930840e-02,
            1.75124064e-02, 1.08822909e-02, 5.02883977e-03, 5.15085143e-03, 9.61830006e-03,
            1.20496177e-02, 6.44065998e-03, 1.47176294e-02, 1.34289303e-02, 1.69032799e-03
        ], [
            5.37939297e-03, 8.73374110e-03, 1.18381812e-02, 8.94882480e-03, 8.64020596e-03,
            1.65727221e-02, 6.51676931e-03, 9.34908212e-03, 1.32269657e-02, 1.44393035e-02,
            1.16479786e+00, 2.28659627e-02, 1.67625073e-02, 2.33940017e-02, 1.98041889e-02,
            1.52702603e-02, 7.99988082e-03, 1.16121345e-01, 9.46892843e-03, 3.58646679e-03,
            3.74845916e-03, 8.39318468e-03, 6.42552158e-03, 5.20545773e-03, 5.54219475e-03,
            6.97685157e-03, 5.48357424e-03, 2.55520605e-03, 7.01632994e-03, 4.25228654e-03,
            6.74040418e-03, 5.12420728e-03, 6.48074092e-03, 6.37725817e-03, 9.61046022e-04
        ], [
            2.30409757e-02, 4.68009843e-02, 4.15286644e-02, 3.93846902e-02, 4.85190230e-02,
            7.05555442e-02, 4.07334386e-02, 4.61253692e-02, 4.98426506e-02, 7.41550176e-02,
            8.98808898e-02, 1.52849723e+00, 3.23774653e-01, 1.80905328e-01, 2.45309056e-01,
            1.51464755e-01, 3.68770253e-02, 1.78438360e-01, 6.55255088e-02, 1.68857722e-02,
            1.63142916e-02, 2.44973306e-02, 3.67736983e-02, 3.90306101e-02, 3.83106523e-02,
            3.77090722e-02, 2.47054256e-02, 1.05852124e-02, 1.54324476e-02, 2.05877174e-02,
            2.85573616e-02, 1.50753940e-02, 2.21129664e-02, 2.59541955e-02, 4.93895004e-03
        ], [
            1.62301994e-02, 2.56372842e-02, 1.76254753e-02, 2.92358809e-02, 2.53153207e-02,
            2.84175357e-02, 2.14914161e-02, 2.76816414e-02, 2.38998055e-02, 3.31002317e-02,
            4.32566780e-02, 5.42920779e-02, 1.21305715e+00, 4.60034397e-02, 8.60436404e-02,
            3.23417012e-02, 2.15861164e-02, 4.19449052e-02, 4.24183223e-02, 8.41391051e-03,
            8.04125005e-03, 1.09941779e-02, 1.83416645e-02, 2.67127310e-02, 2.31117131e-02,
            2.01906420e-02, 1.02623599e-02, 5.24976107e-03, 5.90004840e-03, 9.40622352e-03,
            1.55791344e-02, 7.30579124e-03, 1.19176757e-02, 1.25084069e-02, 2.80471578e-03
        ], [
            1.90921841e-02, 2.96267965e-02, 3.09445336e-02, 4.44395689e-02, 4.00637577e-02,
            4.40769817e-02, 4.82580744e-02, 3.57792807e-02, 4.84476978e-02, 6.03171263e-02,
            5.44994645e-02, 7.58579446e-02, 2.37530123e-01, 1.62521369e+00, 1.91184957e-01,
            8.18064921e-02, 6.50445046e-02, 1.01077131e-01, 8.81931819e-02, 2.84759990e-02,
            2.86920547e-02, 2.85111083e-02, 4.06440514e-02, 3.76789253e-02, 4.20499909e-02,
            4.49617498e-02, 9.27023783e-02, 2.32577734e-02, 1.59147991e-02, 4.84829293e-02,
            5.08046118e-02, 3.04768274e-02, 6.02672845e-02, 4.44909842e-02, 5.15304352e-03
        ], [
            1.49609843e-02, 1.48598391e-02, 1.69720123e-02, 1.88710214e-02, 1.90290478e-02,
            2.13512271e-02, 1.74772821e-02, 1.73340472e-02, 1.61958417e-02, 2.11620004e-02,
            2.37869774e-02, 2.77137875e-02, 4.26090319e-02, 2.15958738e-02, 1.47201379e+00,
            2.36402366e-02, 1.45145679e-02, 2.29012382e-02, 1.27857649e-01, 1.32402831e-02,
            1.24560159e-02, 1.18947428e-02, 6.84798909e-02, 7.92319927e-02, 9.16712043e-02,
            3.95389548e-02, 1.20523153e-02, 7.08517014e-03, 5.32142943e-03, 1.31455740e-02,
            3.25278729e-02, 8.52111205e-03, 9.28535302e-03, 2.03665883e-02, 2.05581930e-03
        ], [
            2.15892526e-03, 2.50920091e-03, 3.64203283e-03, 9.63041549e-03, 6.89708022e-03,
            8.23135181e-03, 8.13772277e-03, 3.18036579e-03, 4.76323589e-03, 6.83224748e-03,
            6.91326100e-03, 1.73988831e-02, 1.01583055e-02, 7.35592214e-03, 1.22722663e-02,
            1.05263326e+00, 3.45134606e-03, 9.76049662e-03, 4.75631270e-03, 2.47721857e-03,
            2.78315470e-03, 4.31555266e-03, 4.91590836e-03, 3.29905392e-03, 4.51321408e-03,
            4.10674781e-03, 2.75674707e-03, 2.17199051e-03, 2.58121279e-03, 3.45552258e-03,
            5.80418926e-03, 4.85887731e-03, 4.96948386e-03, 7.66445943e-03, 2.19167852e-03
        ], [
            2.54348163e-02, 4.43567127e-02, 3.73373341e-02, 4.96949891e-02, 3.63718321e-02,
            4.49901764e-02, 5.02039427e-02, 6.87467525e-02, 5.68514850e-02, 5.33938909e-02,
            7.69713385e-02, 6.69800478e-02, 4.08260399e-02, 3.17210099e-02, 3.75413578e-02,
            3.31253183e-02, 1.16549989e+00, 3.42948482e-02, 2.63027233e-02, 1.49744234e-02,
            2.26663196e-02, 4.11821766e-02, 3.29942529e-02, 2.27354453e-02, 2.55749144e-02,
            3.48215024e-02, 2.03943940e-02, 1.22851883e-02, 1.66526786e-02, 1.59713479e-02,
            2.71198991e-02, 3.10312481e-02, 2.63945748e-02, 3.06110056e-02, 4.20994762e-03
        ], [
            7.53463647e-03, 1.11337875e-02, 9.35380727e-03, 1.02108929e-02, 9.14227315e-03,
            1.07769879e-02, 1.05826260e-02, 1.53919208e-02, 1.08565712e-02, 1.03963540e-02,
            1.45245129e-02, 1.24474732e-02, 1.02829560e-02, 8.21160551e-03, 9.45232600e-03,
            9.09561523e-03, 2.42359114e-02, 1.06059479e+00, 1.10268397e-02, 7.90127250e-03,
            9.41505194e-03, 1.20924501e-02, 1.54768416e-02, 1.05066325e-02, 1.04839418e-02,
            2.06966435e-02, 1.59381527e-02, 1.01197379e-02, 3.77967481e-02, 9.78047827e-03,
            2.36156774e-02, 1.59904524e-02, 1.26686146e-02, 1.55562976e-02, 1.79932187e-03
        ], [
            9.60895486e-03, 6.38372287e-03, 1.48987261e-02, 1.36260039e-02, 1.58290210e-02,
            1.45136660e-02, 1.20448375e-02, 9.42926581e-03, 1.29773691e-02, 1.44383151e-02,
            1.39308729e-02, 1.38319868e-02, 1.26049700e-02, 1.02725093e-02, 1.40425583e-02,
            1.31903608e-02, 1.25032724e-02, 1.45733412e-02, 1.01656167e+00, 7.53796879e-03,
            7.10861498e-03, 1.08939663e-02, 2.09055636e-02, 8.02929770e-03, 1.10239151e-02,
            1.41476144e-02, 7.33978892e-03, 6.32876361e-03, 4.32329659e-03, 9.27102268e-03,
            8.54517359e-03, 5.75304097e-03, 8.17739447e-03, 1.09452177e-02, 9.13760212e-04
        ], [
            5.77453637e-02, 3.32136457e-02, 1.09044458e-01, 9.80057570e-02, 1.12057687e-01,
            1.02755535e-01, 8.92147226e-02, 9.41685567e-02, 8.85433385e-02, 9.60859922e-02,
            7.58040147e-02, 9.03188069e-02, 9.26234422e-02, 7.97758480e-02, 1.02636612e-01,
            8.26634729e-02, 4.48371123e-02, 8.07100512e-02, 6.02664796e-02, 1.03834196e+00,
            2.55106666e-02, 7.78263901e-02, 4.80346573e-02, 5.02747675e-02, 5.23708149e-02,
            4.62875331e-02, 2.74593909e-02, 1.61688609e-02, 1.38455599e-02, 2.90102946e-02,
            3.99508329e-02, 2.42903757e-02, 4.82742826e-02, 3.76136352e-02, 4.97056068e-03
        ], [
            3.02402838e-02, 1.47495690e-02, 5.39157374e-02, 5.27889244e-02, 6.52052505e-02,
            4.37187178e-02, 3.78005736e-02, 2.87952301e-02, 4.15523314e-02, 4.43615834e-02,
            3.66114432e-02, 3.77226863e-02, 3.92772789e-02, 3.34400106e-02, 4.57855958e-02,
            4.52051295e-02, 1.79855656e-02, 5.18753532e-02, 2.70877761e-02, 1.28304656e-02,
            1.01359506e+00, 4.06707812e-02, 2.59332867e-02, 2.17461771e-02, 2.34355911e-02,
            2.16160425e-02, 1.70831998e-02, 1.16337963e-02, 8.81051538e-03, 1.66392645e-02,
            1.80996470e-02, 1.27010572e-02, 2.39024404e-02, 2.20596989e-02, 2.71625803e-03
        ], [
            5.89711495e-03, 7.41043945e-03, 1.18915409e-02, 1.35390902e-02, 1.44778272e-02,
            1.27540057e-02, 1.46542775e-02, 8.80268243e-03, 1.25261782e-02, 1.41541528e-02,
            1.35320787e-02, 1.33101464e-02, 1.47061174e-02, 1.27218555e-02, 1.24849174e-02,
            1.12441750e-02, 8.76693424e-03, 1.37908964e-02, 1.14848385e-02, 1.24554044e-02,
            1.13372399e-02, 1.01699450e+00, 1.47311360e-02, 1.37642023e-02, 2.55921950e-02,
            3.02444914e-02, 1.56336365e-02, 1.78938028e-02, 6.77606811e-03, 1.99369529e-02,
            1.73187734e-02, 1.28658058e-02, 1.48718623e-02, 1.95274261e-02, 1.02605118e-03
        ], [
            2.76959980e-02, 2.93260236e-02, 4.92702203e-02, 5.05804253e-02, 5.08429837e-02,
            5.22048704e-02, 4.15768583e-02, 7.50120542e-02, 4.38529894e-02, 4.39366307e-02,
            6.06965393e-02, 4.43975538e-02, 3.67126553e-02, 2.87619418e-02, 3.74234552e-02,
            4.12061218e-02, 3.60347433e-02, 4.41049061e-02, 3.12485077e-02, 3.04266377e-02,
            2.10977189e-02, 2.67691132e-02, 1.06461183e+00, 3.95881334e-02, 3.33189802e-02,
            7.80687457e-02, 1.80427778e-02, 1.24628623e-02, 7.55004102e-03, 1.45835639e-02,
            2.13449290e-02, 1.72215920e-02, 1.89354746e-02, 2.13267087e-02, 3.75685438e-03
        ], [
            6.11510414e-03, 9.58147418e-03, 1.13142573e-02, 1.20780496e-02, 1.22041752e-02,
            1.47603101e-02, 8.83767939e-03, 1.10211384e-02, 8.82885750e-03, 9.61030846e-03,
            1.58957081e-02, 1.37990833e-02, 9.86404672e-03, 7.51252279e-03, 9.30008289e-03,
            8.48815192e-03, 6.78652088e-03, 9.59952146e-03, 5.45315026e-03, 7.45202088e-03,
            5.90459788e-03, 5.71517148e-03, 7.44690315e-03, 1.18470770e+00, 7.17652547e-03,
            9.56896839e-03, 3.14866484e-03, 2.26127014e-03, 3.80524321e-03, 3.51575835e-03,
            3.68495244e-03, 2.63192502e-03, 3.19377685e-03, 4.02541139e-03, 9.36124964e-04
        ], [
            2.44402078e-03, 2.90374929e-03, 4.37369977e-03, 5.20972661e-03, 6.19716894e-03,
            5.10557018e-03, 5.67610217e-03, 3.51934283e-03, 4.75047349e-03, 5.02587232e-03,
            4.61611138e-03, 4.27313094e-03, 5.46081078e-03, 4.97908799e-03, 5.06539722e-03,
            4.16072322e-03, 2.66259041e-03, 4.18432844e-03, 4.41826188e-03, 4.76930307e-03,
            3.32461521e-03, 3.65029279e-03, 5.37160899e-03, 1.08656553e-02, 1.03598236e+00,
            1.82115781e-02, 6.00935177e-03, 5.25948544e-03, 1.77107562e-03, 6.01496010e-03,
            6.19862344e-03, 4.19368336e-03, 3.20327072e-03, 6.76049599e-03, 3.79974271e-04
        ], [
            1.16166284e-02, 1.16164707e-02, 2.17376393e-02, 1.90858085e-02, 2.12471911e-02,
            2.34829389e-02, 2.46086477e-02, 2.22620486e-02, 1.84167962e-02, 1.98270227e-02,
            2.34695415e-02, 1.97652708e-02, 1.80667745e-02, 1.48292385e-02, 1.89845691e-02,
            1.68484259e-02, 1.26988692e-02, 2.37469229e-02, 2.31188636e-02, 3.03772255e-02,
            1.90302299e-02, 1.81273458e-02, 5.40491796e-02, 1.55807816e-01, 1.14632024e-01,
            1.12427897e+00, 1.10405853e-02, 9.29049249e-03, 4.97720003e-03, 1.21245378e-02,
            1.22861878e-02, 7.86405150e-03, 1.04610362e-02, 1.48350279e-02, 1.41766646e-03
        ], [
            1.43205334e-02, 1.47358003e-02, 2.36057536e-02, 2.75159649e-02, 2.79362616e-02,
            2.55584692e-02, 3.54628028e-02, 2.04920145e-02, 2.73992725e-02, 2.61441245e-02,
            2.49219298e-02, 2.67931807e-02, 2.70676549e-02, 2.71330782e-02, 2.55030584e-02,
            2.89269905e-02, 1.99970131e-02, 3.19576551e-02, 3.26304347e-02, 3.32378601e-02,
            3.46295209e-02, 3.15047842e-02, 3.48749851e-02, 4.11322714e-02, 4.25790897e-02,
            4.41815241e-02, 1.14804094e+00, 5.04344125e-02, 1.58579900e-02, 4.24932578e-02,
            3.87021590e-02, 2.08809463e-02, 2.86428795e-02, 4.07874855e-02, 2.93508516e-03
        ], [
            4.52599117e-02, 5.00480658e-02, 6.11927885e-02, 7.73256884e-02, 6.84794470e-02,
            6.34148367e-02, 6.93780434e-02, 6.10959515e-02, 6.27215562e-02, 6.24436630e-02,
            6.81891694e-02, 6.59997361e-02, 6.36200195e-02, 5.50840484e-02, 6.29265521e-02,
            7.11872574e-02, 5.70380429e-02, 6.81359571e-02, 5.37804335e-02, 6.05933347e-02,
            6.63133285e-02, 6.33567082e-02, 8.75010058e-02, 7.06155309e-02, 7.57806524e-02,
            7.71909364e-02, 4.93836723e-02, 1.30077538e+00, 1.21129940e-01, 7.32034915e-02,
            5.48962907e-02, 3.49900133e-02, 5.76202786e-02, 8.81045329e-02, 1.38921157e-02
        ], [
            1.19226554e-02, 1.33443456e-02, 2.08538843e-02, 2.29268491e-02, 2.53197719e-02,
            2.24516240e-02, 2.85237154e-02, 1.66292903e-02, 2.04360538e-02, 2.28800792e-02,
            2.01536578e-02, 2.07664563e-02, 2.19561832e-02, 1.90392579e-02, 2.31270870e-02,
            2.18698221e-02, 1.50157435e-02, 2.72994595e-02, 3.49657986e-02, 3.18700462e-02,
            5.25929180e-02, 4.43208830e-02, 2.26451571e-02, 3.08960705e-02, 2.62404942e-02,
            3.75603431e-02, 2.71529127e-02, 4.08298457e-02, 1.03995455e+00, 3.84403534e-02,
            2.59258832e-02, 2.78151222e-02, 4.94501728e-02, 4.26661703e-02, 1.85256645e-03
        ], [
            5.13951945e-02, 6.49059830e-02, 1.06651734e-01, 1.00478741e-01, 9.93424930e-02,
            9.57235634e-02, 1.45665361e-01, 7.93629745e-02, 1.42803110e-01, 1.21535908e-01,
            1.00506262e-01, 9.99457696e-02, 1.19048956e-01, 1.19454974e-01, 1.28503054e-01,
            9.24282895e-02, 7.93338100e-02, 1.26819483e-01, 1.05842381e-01, 9.91359684e-02,
            1.02994922e-01, 1.02669107e-01, 9.90928575e-02, 8.89972900e-02, 1.47495653e-01,
            1.28248625e-01, 1.14904625e-01, 1.73342750e-01, 7.60513300e-02, 1.20890180e+00,
            1.39025862e-01, 6.83615012e-02, 1.18400171e-01, 1.43113588e-01, 1.13876701e-02
        ], [
            3.23900285e-03, 4.18895651e-03, 5.07074343e-03, 4.23033341e-03, 4.28280181e-03,
            7.08918849e-03, 7.14259109e-03, 4.78210752e-03, 6.07704237e-03, 5.66997506e-03,
            6.05726072e-03, 6.71528468e-03, 4.72489551e-03, 3.95545474e-03, 4.85231490e-03,
            5.19162691e-03, 7.07240420e-03, 4.95296235e-03, 4.91613254e-03, 3.24736915e-03,
            4.41882777e-03, 6.24424603e-03, 6.09544862e-03, 8.78458820e-03, 1.23359676e-02,
            6.21583830e-03, 4.72280465e-03, 3.59863548e-03, 5.23171703e-03, 5.17646439e-03,
            1.00949194e+00, 6.05140205e-03, 6.53631312e-03, 1.15071600e-02, 4.70837667e-04
        ], [
            1.20553193e-03, 9.95919317e-04, 1.61999534e-03, 1.60148617e-03, 1.71940193e-03,
            1.53590363e-03, 1.81404600e-03, 1.43058070e-03, 2.17457437e-03, 1.94671956e-03,
            1.65526238e-03, 1.74866847e-03, 2.09740618e-03, 1.92229836e-03, 2.29478388e-03,
            1.46906950e-03, 1.47945791e-03, 1.67730503e-03, 1.72833843e-03, 1.41879044e-03,
            1.95778586e-03, 1.53593110e-03, 1.97654825e-03, 1.39615978e-03, 3.22798786e-03,
            2.28308441e-03, 2.35956898e-03, 2.28328289e-03, 8.41001991e-04, 3.26316056e-03,
            5.18717465e-03, 1.01482050e+00, 2.33739129e-03, 2.66451172e-03, 1.49196073e-04
        ], [
            1.89452117e-03, 8.26491140e-04, 1.57690610e-03, 1.36532120e-03, 2.17760654e-03,
            1.55331221e-03, 1.29284621e-03, 9.26398286e-04, 1.32420578e-03, 1.26816940e-03,
            1.49942524e-03, 1.35972324e-03, 1.79315732e-03, 1.23492800e-03, 1.45768602e-03,
            1.23722926e-03, 9.19208082e-04, 1.19048388e-03, 1.07925256e-03, 9.40715852e-04,
            7.71284106e-04, 1.67099151e-03, 1.16218824e-03, 9.73875660e-04, 1.09194313e-03,
            1.13274951e-03, 1.09965805e-03, 1.06908139e-03, 3.85415536e-04, 1.08776617e-03,
            6.00292314e-03, 2.03266429e-03, 1.03591156e+00, 1.98451302e-03, 1.28198121e-04
        ], [
            9.75154428e-03, 1.28864118e-02, 1.76587672e-02, 1.82741006e-02, 2.10460758e-02,
            1.78497340e-02, 2.87628852e-02, 1.44004545e-02, 2.08074100e-02, 1.90588197e-02,
            1.80703656e-02, 1.85014857e-02, 1.77378932e-02, 1.52546057e-02, 1.85177430e-02,
            1.59019027e-02, 1.44265063e-02, 1.90415922e-02, 1.69457532e-02, 1.52501188e-02,
            1.69085754e-02, 2.47642878e-02, 2.31790139e-02, 1.65980032e-02, 2.06067809e-02,
            2.59752312e-02, 2.83570049e-02, 2.19412853e-02, 1.47007959e-02, 3.70607410e-02,
            2.79166381e-02, 1.91537340e-02, 2.43231208e-02, 1.09545784e+00, 3.54746127e-03
        ], [
            4.09813241e-05, 5.08327020e-05, 7.10203250e-05, 1.79548024e-04, 9.32441450e-05,
            8.38762906e-05, 8.61808660e-05, 6.06552435e-05, 6.65736994e-05, 7.34613056e-05,
            6.91100235e-05, 6.39661425e-05, 8.23825513e-05, 5.87214984e-05, 1.16549081e-04,
            6.04265675e-05, 3.98016510e-05, 1.15575184e-04, 8.00872989e-05, 4.27037700e-05,
            7.25828728e-05, 9.57662633e-05, 1.55263387e-04, 4.77758566e-05, 5.89560056e-05,
            6.18651463e-05, 6.91487208e-05, 1.07268549e-04, 2.52275887e-04, 1.03659153e-04,
            1.03244033e-04, 6.58487158e-05, 9.52198862e-05, 1.66200355e-04, 1.00012835e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 401)
    return ret


@cache('run')
def leontief_matrix_x2007x():
    """
    leontief matrix x2007x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.16627557e+00, 4.57397835e-03, 3.62216629e-01, 1.03410891e-01, 1.45429358e-01,
            1.98016500e-01, 3.65721220e-02, 8.41306471e-03, 2.26597856e-02, 3.10510025e-02,
            1.10934062e-02, 8.77802331e-03, 8.92311671e-03, 8.03232300e-03, 9.76085720e-03,
            3.88149366e-02, 5.24380161e-03, 1.75692568e-02, 6.92400843e-03, 8.42703075e-03,
            8.12406837e-03, 1.05489493e-01, 8.40933335e-03, 9.70093577e-03, 9.52692147e-03,
            1.66861673e-02, 4.97876695e-03, 4.62211585e-03, 3.26160900e-03, 6.91180531e-03,
            1.09909619e-02, 1.06971784e-02, 1.31596217e-02, 1.24976567e-02, 3.49457700e-02
        ], [
            2.28566761e-02, 1.11067029e+00, 2.59519508e-02, 3.53945706e-02, 3.02532510e-02,
            3.40244627e-02, 3.05754203e-02, 5.60844689e-01, 9.30796851e-02, 4.95335820e-02,
            1.27741309e-01, 1.14630218e-01, 4.17493508e-02, 2.96443735e-02, 3.51836775e-02,
            3.44511732e-02, 1.63740328e-01, 6.25670784e-02, 1.81163246e-02, 1.08312172e-02,
            1.10974624e-02, 1.89700123e-02, 4.92755450e-02, 5.44720163e-02, 6.90216387e-02,
            3.24859407e-02, 1.14168720e-02, 7.19474162e-03, 8.68991919e-03, 1.17332361e-02,
            1.99592227e-02, 1.27545165e-02, 1.80235618e-02, 1.81937937e-02, 7.16010182e-02
        ], [
            9.72134324e-02, 4.93053655e-03, 1.23105299e+00, 2.93623123e-02, 1.80165908e-01,
            2.71226146e-02, 1.53955091e-02, 8.31333438e-03, 2.72167203e-02, 1.94220510e-02,
            1.17410797e-02, 1.04658322e-02, 1.05982723e-02, 9.69495465e-03, 1.04335767e-02,
            1.46397913e-02, 6.05071831e-03, 1.04596235e-02, 7.60214869e-03, 1.03080821e-02,
            9.87986343e-03, 1.99287857e-01, 8.43441443e-03, 1.42859038e-02, 1.44551297e-02,
            1.24390238e-02, 6.97759789e-03, 6.30402764e-03, 2.92833289e-03, 8.69012456e-03,
            1.56463842e-02, 1.58118145e-02, 2.28094160e-02, 1.66180809e-02, 6.78420701e-03
        ], [
            5.83673238e-03, 4.12261822e-03, 8.36702210e-03, 1.57429565e+00, 1.27787834e-01,
            1.17566656e-02, 1.79223215e-02, 5.02816152e-03, 1.14625812e-02, 3.30055991e-02,
            1.22936369e-02, 9.29469843e-03, 1.05751257e-02, 9.57051476e-03, 1.67064431e-02,
            4.55475915e-02, 4.54959914e-03, 9.90184607e-03, 6.87761927e-03, 6.12939584e-03,
            6.74876572e-03, 8.16189508e-03, 6.95960946e-03, 9.28293960e-03, 7.51309669e-03,
            7.10804302e-03, 3.59156880e-03, 2.83639846e-03, 2.03671544e-03, 5.13145130e-03,
            7.79576723e-03, 3.83371663e-03, 9.13206112e-03, 1.08675560e-02, 7.22102362e-04
        ], [
            5.50928284e-04, 3.52815685e-04, 7.66391156e-04, 2.53445961e-02, 1.28536725e+00,
            1.46348676e-03, 1.82109334e-03, 4.70840339e-04, 9.57212361e-04, 2.41188591e-03,
            1.00385935e-03, 9.07109547e-04, 1.05210379e-03, 1.16060959e-03, 4.40491501e-03,
            8.47713115e-03, 4.58586792e-04, 8.12742705e-04, 1.17630473e-03, 7.08684566e-04,
            6.37963988e-04, 6.03855044e-04, 8.67247490e-04, 7.27203777e-04, 8.19848547e-04,
            7.26575525e-04, 6.16376100e-04, 2.72424198e-04, 1.65345203e-04, 4.53869826e-04,
            1.40090791e-03, 4.00022652e-04, 7.35737631e-04, 1.09992922e-03, 6.59120489e-05
        ], [
            4.79194163e-03, 4.72867908e-03, 7.23112099e-03, 7.91704629e-03, 8.72091407e-03,
            1.36095453e+00, 2.60476963e-02, 4.75880371e-03, 6.61505696e-03, 9.91408044e-03,
            1.78868361e-02, 1.29564069e-02, 1.11295453e-02, 8.08017906e-03, 1.13700207e-02,
            1.12140044e-01, 4.65504792e-03, 4.91448353e-02, 5.48482816e-03, 3.81376198e-03,
            3.81326754e-03, 6.68314828e-03, 5.20564657e-03, 4.21666514e-03, 4.24188215e-03,
            7.21602924e-03, 3.47503031e-03, 2.67636331e-03, 5.77211543e-03, 4.01100576e-03,
            6.57430244e-03, 6.69384625e-03, 5.13576524e-03, 7.65787698e-03, 7.50807001e-04
        ], [
            1.48494108e-02, 1.22054847e-02, 5.41982158e-02, 4.21789456e-02, 4.94538795e-02,
            4.12550614e-02, 1.34964734e+00, 1.60895135e-02, 4.09628760e-02, 4.59018376e-02,
            4.86005030e-02, 2.46266432e-02, 2.88426209e-02, 3.20831435e-02, 2.70031085e-02,
            4.50505536e-02, 1.48320043e-02, 2.56714095e-02, 2.35435639e-02, 2.35089269e-02,
            2.29637812e-02, 2.82070065e-02, 2.13160263e-02, 1.82060742e-02, 2.11914038e-02,
            3.28229890e-02, 2.36537343e-02, 3.07357202e-02, 9.53299313e-03, 3.93476234e-02,
            3.59536787e-02, 3.25778485e-02, 2.53426018e-02, 4.60774997e-02, 2.15319749e-03
        ], [
            2.45720007e-02, 1.59174679e-02, 2.12212193e-02, 2.84525773e-02, 2.50626134e-02,
            2.45610321e-02, 2.07407975e-02, 1.12639741e+00, 7.52744832e-02, 3.95743581e-02,
            4.14188756e-02, 3.41940932e-02, 2.09137686e-02, 1.66519077e-02, 1.87711992e-02,
            2.00079886e-02, 4.32533367e-02, 2.86821556e-02, 1.51251970e-02, 1.18689649e-02,
            9.65647934e-03, 1.40500407e-02, 7.16982982e-02, 8.82049893e-02, 1.16534781e-01,
            3.96193630e-02, 1.00832202e-02, 6.50077670e-03, 5.01119888e-03, 1.14521478e-02,
            1.72130522e-02, 8.98587413e-03, 1.36821583e-02, 1.40989616e-02, 2.06668784e-03
        ], [
            7.01389375e-02, 2.65908498e-02, 5.56862036e-02, 1.68948859e-01, 1.20016465e-01,
            9.43208546e-02, 8.87359886e-02, 4.89967387e-02, 1.36184895e+00, 3.71519131e-01,
            8.53163520e-02, 5.33620184e-02, 5.45300933e-02, 6.90321141e-02, 6.44313613e-02,
            7.91767072e-02, 2.54882715e-02, 5.08728322e-02, 2.97330715e-02, 1.35169928e-02,
            1.27061334e-02, 2.70621799e-02, 2.34517644e-02, 2.38809658e-02, 2.45644621e-02,
            2.42386782e-02, 1.36149420e-02, 9.07362630e-03, 8.94386985e-03, 1.85752288e-02,
            2.77506326e-02, 1.48444354e-02, 1.02363418e-01, 3.01808623e-02, 4.56598129e-03
        ], [
            1.46185575e-02, 1.36528605e-02, 3.84886858e-02, 3.47882200e-02, 7.69072124e-02,
            2.57804133e-02, 3.69839634e-02, 1.64922026e-02, 3.77247088e-02, 1.19754915e+00,
            2.96926394e-02, 2.48497676e-02, 5.10751578e-02, 5.49927237e-02, 7.54983325e-02,
            5.59099857e-02, 1.22767615e-02, 3.81198753e-02, 3.23594913e-02, 1.03950926e-02,
            1.06927920e-02, 1.63335465e-02, 2.40525872e-02, 1.48161433e-02, 1.71139507e-02,
            1.85936224e-02, 1.11708845e-02, 5.42301430e-03, 5.13809741e-03, 1.01103279e-02,
            1.27327105e-02, 7.04131276e-03, 1.51597465e-02, 1.39979140e-02, 1.77942715e-03
        ], [
            5.49191317e-03, 9.28468778e-03, 1.22342606e-02, 9.42989080e-03, 9.11236164e-03,
            1.67937622e-02, 6.95234589e-03, 1.00579719e-02, 1.38076891e-02, 1.48290813e-02,
            1.17490257e+00, 2.43920571e-02, 1.77079040e-02, 2.47283868e-02, 2.03340513e-02,
            1.65062262e-02, 8.64513711e-03, 1.23019943e-01, 9.96641814e-03, 3.67901594e-03,
            3.87206947e-03, 8.72045408e-03, 6.80395238e-03, 5.11428531e-03, 5.97834852e-03,
            7.43189005e-03, 5.76958350e-03, 2.75029681e-03, 7.16796791e-03, 4.49649366e-03,
            7.34477248e-03, 5.39371114e-03, 6.83428738e-03, 6.74782140e-03, 9.93750526e-04
        ], [
            2.40472076e-02, 5.06045076e-02, 4.17203055e-02, 4.18404445e-02, 5.01625029e-02,
            7.28762317e-02, 4.37932214e-02, 4.86821595e-02, 5.22072802e-02, 7.65287235e-02,
            9.70645931e-02, 1.54336627e+00, 3.34716608e-01, 1.87261420e-01, 2.46552715e-01,
            1.57172768e-01, 4.09327672e-02, 1.84734819e-01, 6.71835903e-02, 1.74701801e-02,
            1.64663483e-02, 2.54350749e-02, 3.90610342e-02, 3.97468506e-02, 4.19312059e-02,
            4.09371573e-02, 2.60528481e-02, 1.15676869e-02, 1.55960151e-02, 2.21078507e-02,
            3.08832481e-02, 1.62545061e-02, 2.37451336e-02, 2.78390648e-02, 5.13072124e-03
        ], [
            1.74669156e-02, 2.91467038e-02, 1.91301055e-02, 3.34807880e-02, 2.85657117e-02,
            3.19368760e-02, 2.41082455e-02, 3.12994326e-02, 2.69817960e-02, 3.68706099e-02,
            4.97140568e-02, 6.15067564e-02, 1.23551943e+00, 5.12621860e-02, 9.40610994e-02,
            3.71999683e-02, 2.51037402e-02, 4.63492731e-02, 4.55050063e-02, 8.99297844e-03,
            8.39153520e-03, 1.19860898e-02, 2.03021964e-02, 2.72196563e-02, 2.61457041e-02,
            2.34831921e-02, 1.12706103e-02, 6.04116443e-03, 6.22569880e-03, 1.05915860e-02,
            1.77199254e-02, 8.29441043e-03, 1.35804331e-02, 1.38469999e-02, 3.11021357e-03
        ], [
            2.11020493e-02, 3.53849674e-02, 3.38213941e-02, 4.83851135e-02, 4.35813303e-02,
            4.76140717e-02, 5.25745984e-02, 4.13077455e-02, 5.36111974e-02, 6.30745433e-02,
            6.05708811e-02, 8.23985445e-02, 2.54139811e-01, 1.67276579e+00, 2.06417616e-01,
            9.08073262e-02, 7.97582061e-02, 1.12278495e-01, 9.43881429e-02, 3.08076876e-02,
            2.96464237e-02, 3.15357309e-02, 4.57480376e-02, 3.98731248e-02, 4.76726296e-02,
            5.08162988e-02, 1.00653643e-01, 2.72984614e-02, 1.70517210e-02, 5.64995173e-02,
            5.90292877e-02, 3.59067706e-02, 6.74812063e-02, 5.23112151e-02, 5.33441119e-03
        ], [
            1.65995101e-02, 1.68524414e-02, 1.85045015e-02, 2.11895536e-02, 2.10391374e-02,
            2.43503254e-02, 2.04062674e-02, 1.95146484e-02, 1.78316814e-02, 2.34746849e-02,
            2.82413244e-02, 3.17578215e-02, 4.84864192e-02, 2.36534085e-02, 1.49393357e+00,
            2.70551819e-02, 1.74002933e-02, 2.60199366e-02, 1.30099014e-01, 1.53286636e-02,
            1.24248789e-02, 1.30554188e-02, 7.45280357e-02, 8.28954791e-02, 1.02971911e-01,
            4.53308038e-02, 1.29725581e-02, 7.90850262e-03, 5.87909946e-03, 1.43485625e-02,
            3.75882906e-02, 9.83966302e-03, 1.02084409e-02, 2.23152871e-02, 2.17726513e-03
        ], [
            2.25814753e-03, 2.75874978e-03, 3.80914654e-03, 9.98793111e-03, 7.39531949e-03,
            8.44753594e-03, 8.76500675e-03, 3.47086648e-03, 4.97051986e-03, 7.17396594e-03,
            7.70002572e-03, 1.83114677e-02, 1.07523670e-02, 7.65247243e-03, 1.26032072e-02,
            1.05750787e+00, 3.83175646e-03, 9.98251661e-03, 4.78590580e-03, 2.53214369e-03,
            2.81448570e-03, 4.49418577e-03, 5.21789952e-03, 3.31501158e-03, 4.83761520e-03,
            4.31774528e-03, 2.94300470e-03, 2.42434049e-03, 2.55244907e-03, 3.63934463e-03,
            5.93981323e-03, 5.34136473e-03, 5.20417031e-03, 7.98019043e-03, 1.86823459e-03
        ], [
            2.57040685e-02, 4.83423962e-02, 3.82530301e-02, 5.19591418e-02, 3.71941844e-02,
            4.69711604e-02, 5.23760240e-02, 7.23503471e-02, 5.95185586e-02, 5.58272119e-02,
            8.01964353e-02, 6.97276926e-02, 4.24879413e-02, 3.33944668e-02, 3.83182885e-02,
            3.49979001e-02, 1.18222675e+00, 3.64110006e-02, 2.68142180e-02, 1.50974278e-02,
            2.26337244e-02, 4.17407568e-02, 3.38322645e-02, 2.18510240e-02, 2.66389476e-02,
            3.52653089e-02, 2.09231048e-02, 1.29428166e-02, 1.47515922e-02, 1.63275706e-02,
            2.86034576e-02, 3.23126956e-02, 2.69965564e-02, 3.10700360e-02, 4.49963490e-03
        ], [
            7.17638804e-03, 1.10351065e-02, 9.05138684e-03, 1.00105277e-02, 8.87788117e-03,
            1.06453870e-02, 1.05302040e-02, 1.47591961e-02, 1.07579741e-02, 1.01248937e-02,
            1.44408535e-02, 1.23852916e-02, 1.00770013e-02, 8.07645340e-03, 9.27645217e-03,
            9.08877188e-03, 2.40011428e-02, 1.05949216e+00, 1.10657751e-02, 7.49444896e-03,
            9.28969541e-03, 1.17397530e-02, 1.50523901e-02, 9.05804031e-03, 1.02849357e-02,
            2.03806599e-02, 1.60466156e-02, 1.01209648e-02, 3.69180307e-02, 9.53616853e-03,
            2.38895375e-02, 1.51440630e-02, 1.29074093e-02, 1.54637069e-02, 1.65786878e-03
        ], [
            9.64715532e-03, 6.62762659e-03, 1.48189026e-02, 1.34041896e-02, 1.54704645e-02,
            1.44895319e-02, 1.19350499e-02, 9.81345470e-03, 1.27274663e-02, 1.41085735e-02,
            1.39182088e-02, 1.37288035e-02, 1.23293604e-02, 1.01951369e-02, 1.35721365e-02,
            1.33396065e-02, 1.28154929e-02, 1.44281039e-02, 1.01642855e+00, 7.21263143e-03,
            7.02119713e-03, 1.07990323e-02, 2.05525310e-02, 7.66381948e-03, 1.09371958e-02,
            1.43127716e-02, 7.20444899e-03, 6.36062160e-03, 4.27635940e-03, 9.03523650e-03,
            8.40548429e-03, 5.51831443e-03, 8.10257401e-03, 1.07271518e-02, 9.41226419e-04
        ], [
            6.19168154e-02, 3.64132393e-02, 1.13920523e-01, 1.00712492e-01, 1.14406451e-01,
            1.05083024e-01, 9.02805165e-02, 9.77612386e-02, 8.91717623e-02, 9.76670498e-02,
            8.05944528e-02, 9.24506301e-02, 9.49351494e-02, 8.15693722e-02, 1.02845875e-01,
            8.53539393e-02, 4.78832270e-02, 8.44857550e-02, 6.01442585e-02, 1.03802444e+00,
            2.58424364e-02, 8.08425615e-02, 4.95152151e-02, 4.99982713e-02, 5.59289081e-02,
            4.87350177e-02, 2.88680855e-02, 1.72310353e-02, 1.40129642e-02, 3.06216886e-02,
            4.05181209e-02, 2.59357108e-02, 5.07065503e-02, 3.94709016e-02, 5.37064458e-03
        ], [
            2.81930283e-02, 1.41608607e-02, 5.04143141e-02, 4.88459927e-02, 5.83392049e-02,
            4.01130368e-02, 3.53495821e-02, 2.95284364e-02, 3.93528616e-02, 4.13191903e-02,
            3.37040469e-02, 3.56587317e-02, 3.60010384e-02, 3.04783679e-02, 4.23455168e-02,
            4.37833552e-02, 1.76086144e-02, 4.63223228e-02, 2.63078403e-02, 1.20467951e-02,
            1.01289434e+00, 3.85046990e-02, 2.51238728e-02, 1.89760309e-02, 2.29682290e-02,
            2.11153578e-02, 1.59857161e-02, 1.14317622e-02, 8.11516292e-03, 1.59838684e-02,
            1.76222993e-02, 1.19801198e-02, 2.25682770e-02, 2.09611572e-02, 2.54712708e-03
        ], [
            5.87173347e-03, 7.57473424e-03, 1.17140335e-02, 1.33354213e-02, 1.41048105e-02,
            1.24960715e-02, 1.42451741e-02, 8.78200853e-03, 1.22859550e-02, 1.34659940e-02,
            1.36543374e-02, 1.32196362e-02, 1.43129176e-02, 1.23134901e-02, 1.22579208e-02,
            1.12265814e-02, 8.69566768e-03, 1.37848901e-02, 1.12213694e-02, 1.21041908e-02,
            1.02845011e-02, 1.01628350e+00, 1.45378692e-02, 1.32854579e-02, 2.72803812e-02,
            3.12351095e-02, 1.47745954e-02, 1.75245361e-02, 6.13644560e-03, 1.89742665e-02,
            1.71284216e-02, 1.30419534e-02, 1.42357977e-02, 1.91153852e-02, 1.02708330e-03
        ], [
            2.75248843e-02, 3.01366097e-02, 4.88834862e-02, 5.07707097e-02, 5.04977856e-02,
            5.12368977e-02, 4.17556682e-02, 7.54854716e-02, 4.28175492e-02, 4.32327116e-02,
            6.09183542e-02, 4.42231289e-02, 3.64249671e-02, 2.85013306e-02, 3.68937750e-02,
            4.18431050e-02, 3.65522560e-02, 4.48355488e-02, 3.13472005e-02, 3.08008127e-02,
            2.02518730e-02, 2.67263638e-02, 1.06509395e+00, 3.30421080e-02, 3.36010249e-02,
            8.09379839e-02, 1.82093172e-02, 1.28756480e-02, 7.20924627e-03, 1.47237508e-02,
            2.22251737e-02, 1.73772024e-02, 1.90110588e-02, 2.12427410e-02, 3.74490589e-03
        ], [
            5.98117401e-03, 1.01677245e-02, 1.18126519e-02, 1.27380859e-02, 1.22027635e-02,
            1.49983934e-02, 8.64836338e-03, 1.14137482e-02, 8.99230932e-03, 9.65885353e-03,
            1.57742989e-02, 1.35990866e-02, 9.83116667e-03, 7.46635485e-03, 9.72078219e-03,
            8.78715740e-03, 7.09870847e-03, 9.63581565e-03, 5.72522984e-03, 8.70611068e-03,
            5.42416891e-03, 6.00708936e-03, 7.96544886e-03, 1.18131541e+00, 7.86983948e-03,
            1.03887496e-02, 3.39406824e-03, 2.49345200e-03, 4.31215225e-03, 3.77171889e-03,
            3.94435332e-03, 2.77221860e-03, 3.36902854e-03, 4.29553299e-03, 9.89002789e-04
        ], [
            2.54688634e-03, 3.17939341e-03, 4.49710836e-03, 5.43461628e-03, 6.35373642e-03,
            5.25681588e-03, 5.71341454e-03, 3.70068344e-03, 4.83816272e-03, 4.96509219e-03,
            4.79423029e-03, 4.34104619e-03, 5.37197710e-03, 4.83611025e-03, 5.01225269e-03,
            4.32045451e-03, 2.83693945e-03, 4.37345261e-03, 4.66619120e-03, 4.86810044e-03,
            3.23157553e-03, 3.72822082e-03, 5.48797964e-03, 1.05143848e-02, 1.03510180e+00,
            1.84634648e-02, 6.01963621e-03, 5.42868126e-03, 1.73355677e-03, 6.03351319e-03,
            6.39028747e-03, 4.69413646e-03, 3.26603871e-03, 6.97356408e-03, 4.02536451e-04
        ], [
            1.19282611e-02, 1.22247550e-02, 2.18472557e-02, 1.98503298e-02, 2.11752804e-02,
            2.35435231e-02, 2.44502980e-02, 2.27993077e-02, 1.84595588e-02, 1.98647260e-02,
            2.38864518e-02, 2.00237141e-02, 1.82301517e-02, 1.48854761e-02, 1.90113843e-02,
            1.73115327e-02, 1.31602983e-02, 2.68750737e-02, 2.38353802e-02, 2.96254638e-02,
            1.89621180e-02, 1.80559594e-02, 5.41388928e-02, 1.59025134e-01, 1.17605154e-01,
            1.12671426e+00, 1.11327689e-02, 9.69569272e-03, 5.05365366e-03, 1.23902295e-02,
            1.27661680e-02, 8.20513453e-03, 1.06301610e-02, 1.50498597e-02, 1.47703634e-03
        ], [
            1.48296906e-02, 1.53431496e-02, 2.38125234e-02, 2.80735056e-02, 2.74651300e-02,
            2.57622363e-02, 3.51743691e-02, 2.13787546e-02, 2.75780508e-02, 2.56399305e-02,
            2.55581327e-02, 2.74572245e-02, 2.69963705e-02, 2.72122629e-02, 2.58638055e-02,
            3.10116363e-02, 2.11003494e-02, 3.20967251e-02, 3.34028704e-02, 3.20418330e-02,
            3.45671662e-02, 3.13534774e-02, 3.57828778e-02, 3.92650418e-02, 4.30040978e-02,
            4.47837758e-02, 1.14872993e+00, 5.22875364e-02, 1.48033591e-02, 4.24516231e-02,
            3.90410135e-02, 2.14573084e-02, 2.88289052e-02, 4.11498895e-02, 2.97437818e-03
        ], [
            4.61729569e-02, 5.29744226e-02, 6.25864684e-02, 8.11275635e-02, 7.11830544e-02,
            6.56046087e-02, 7.13205791e-02, 6.37931169e-02, 6.49295695e-02, 6.29912503e-02,
            7.19574272e-02, 6.83931267e-02, 6.47269919e-02, 5.60531697e-02, 6.48519834e-02,
            7.47630050e-02, 6.11012401e-02, 7.02954939e-02, 5.52158753e-02, 5.99193488e-02,
            6.52129621e-02, 6.39254464e-02, 9.14328839e-02, 6.71098639e-02, 7.91419507e-02,
            8.34999086e-02, 4.93641596e-02, 1.29265218e+00, 1.14015221e-01, 7.27792730e-02,
            5.57656536e-02, 3.74204010e-02, 5.88671209e-02, 8.93937818e-02, 1.55906806e-02
        ], [
            1.09522164e-02, 1.33816824e-02, 2.01605351e-02, 2.26956789e-02, 2.48175800e-02,
            2.22757552e-02, 2.79776112e-02, 1.63803158e-02, 1.99746864e-02, 2.20081641e-02,
            2.01249308e-02, 2.06390473e-02, 2.15094256e-02, 1.83737712e-02, 2.28614525e-02,
            2.20823415e-02, 1.52046107e-02, 2.69185185e-02, 3.50104768e-02, 2.95854820e-02,
            5.31392338e-02, 4.17169970e-02, 2.23868327e-02, 2.90035609e-02, 2.61520279e-02,
            3.70719265e-02, 2.65614123e-02, 4.08467994e-02, 1.04361962e+00, 3.69834676e-02,
            2.57180046e-02, 2.84735665e-02, 4.86814884e-02, 4.19891677e-02, 1.84881967e-03
        ], [
            5.35142221e-02, 6.88838205e-02, 1.07333088e-01, 1.01575220e-01, 9.97015670e-02,
            9.82772197e-02, 1.49623905e-01, 8.24540798e-02, 1.46893629e-01, 1.21663673e-01,
            1.04835474e-01, 1.03360082e-01, 1.20986750e-01, 1.21150227e-01, 1.33228142e-01,
            9.62266910e-02, 8.37332088e-02, 1.30334898e-01, 1.08679328e-01, 9.91636240e-02,
            1.02281836e-01, 1.02743251e-01, 1.01793939e-01, 8.74832713e-02, 1.36101864e-01,
            1.33307526e-01, 1.16635121e-01, 1.81076612e-01, 6.98221571e-02, 1.21260099e+00,
            1.40244881e-01, 7.11575111e-02, 1.21414631e-01, 1.47548927e-01, 1.17821236e-02
        ], [
            3.20307149e-03, 4.33946848e-03, 5.01273444e-03, 4.11382296e-03, 4.13993627e-03,
            6.66149768e-03, 7.32186951e-03, 4.84277538e-03, 6.19592983e-03, 5.68342388e-03,
            6.02564556e-03, 6.58055471e-03, 4.74150123e-03, 3.98536806e-03, 4.77247912e-03,
            5.24998845e-03, 7.12199197e-03, 4.90600313e-03, 4.85220516e-03, 3.11362801e-03,
            4.18998558e-03, 5.86016113e-03, 6.14266802e-03, 8.52502071e-03, 1.17843931e-02,
            6.17881568e-03, 4.78119504e-03, 3.69611816e-03, 4.89554879e-03, 5.11928631e-03,
            1.01007236e+00, 6.17807505e-03, 6.48353316e-03, 1.12532899e-02, 4.80836896e-04
        ], [
            1.19954173e-03, 1.00536472e-03, 1.58496034e-03, 1.57950466e-03, 1.65042411e-03,
            1.52003962e-03, 1.77671134e-03, 1.43674514e-03, 2.19056767e-03, 1.90972806e-03,
            1.63302550e-03, 1.71401453e-03, 2.03547454e-03, 1.85115136e-03, 2.21531791e-03,
            1.46271581e-03, 1.50917937e-03, 1.65388475e-03, 1.70455258e-03, 1.38042740e-03,
            1.67517791e-03, 1.49282863e-03, 1.93208875e-03, 1.30514108e-03, 3.18647624e-03,
            2.35568384e-03, 2.29551994e-03, 2.31614905e-03, 8.24706472e-04, 3.33723771e-03,
            4.97388391e-03, 1.01595943e+00, 2.31034533e-03, 2.65945246e-03, 1.56337728e-04
        ], [
            1.88120804e-03, 8.51317051e-04, 1.54717965e-03, 1.37738436e-03, 2.00991582e-03,
            1.54968139e-03, 1.30551700e-03, 9.38771483e-04, 1.31844441e-03, 1.25928713e-03,
            1.56010075e-03, 1.36655132e-03, 1.78389290e-03, 1.24274587e-03, 1.45233142e-03,
            1.24504413e-03, 9.73583693e-04, 1.22371489e-03, 1.08971031e-03, 8.61417547e-04,
            7.90911893e-04, 1.65016374e-03, 1.14911396e-03, 9.35150449e-04, 1.09648060e-03,
            1.13878179e-03, 1.09568440e-03, 1.10230840e-03, 3.74359227e-04, 1.07808245e-03,
            6.28681996e-03, 2.05226294e-03, 1.03510033e+00, 2.00798663e-03, 1.34670997e-04
        ], [
            9.80446417e-03, 1.31881245e-02, 1.72522254e-02, 1.79102596e-02, 2.04751095e-02,
            1.76508244e-02, 2.84721657e-02, 1.43617477e-02, 2.04308361e-02, 1.83347895e-02,
            1.82645552e-02, 1.83830326e-02, 1.73013602e-02, 1.47674402e-02, 1.82926062e-02,
            1.60284192e-02, 1.48143322e-02, 1.88153177e-02, 1.66493686e-02, 1.50582723e-02,
            1.61156231e-02, 2.40529066e-02, 2.32965235e-02, 1.63229224e-02, 1.95016262e-02,
            2.74066868e-02, 2.85970487e-02, 2.21242443e-02, 1.37826030e-02, 3.60554490e-02,
            2.81473843e-02, 1.89000789e-02, 2.38207990e-02, 1.09424874e+00, 3.10768800e-03
        ], [
            4.12296599e-05, 5.38749624e-05, 7.15885581e-05, 1.73707091e-04, 9.63313297e-05,
            8.79420542e-05, 8.71913214e-05, 6.37106527e-05, 6.90883477e-05, 7.18726737e-05,
            7.26015393e-05, 6.72082108e-05, 8.38505018e-05, 5.73492495e-05, 1.13702603e-04,
            6.25068447e-05, 4.15424501e-05, 1.10745979e-04, 7.93810687e-05, 4.25622990e-05,
            7.25164545e-05, 9.47337370e-05, 1.57661303e-04, 4.55323247e-05, 5.91659899e-05,
            6.36502078e-05, 7.03044570e-05, 1.13935863e-04, 2.43965365e-04, 1.06175772e-04,
            1.10414759e-04, 6.76185800e-05, 9.71599326e-05, 1.61167013e-04, 1.00010492e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 402)
    return ret


@cache('run')
def leontief_matrix_x2008x():
    """
    leontief matrix x2008x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.16382123e+00, 3.81765699e-03, 3.68272895e-01, 1.10566088e-01, 1.49689545e-01,
            1.99304999e-01, 3.80603006e-02, 8.26174063e-03, 2.34987198e-02, 3.20772025e-02,
            1.14001215e-02, 8.93466443e-03, 9.08286363e-03, 8.47023620e-03, 1.02475815e-02,
            4.10925771e-02, 5.15057172e-03, 1.75290884e-02, 7.15821808e-03, 8.58418330e-03,
            8.16980659e-03, 1.06041737e-01, 8.37081251e-03, 9.70470156e-03, 1.00543458e-02,
            1.73284214e-02, 4.87131728e-03, 4.87341073e-03, 2.97916148e-03, 7.04272395e-03,
            1.07978171e-02, 1.07134868e-02, 1.30494513e-02, 1.27311970e-02, 3.66370802e-02
        ], [
            2.70992488e-02, 1.10073744e+00, 3.07087131e-02, 4.25957177e-02, 3.60689657e-02,
            3.98730238e-02, 3.60501870e-02, 6.54241984e-01, 1.09802266e-01, 5.83271841e-02,
            1.37490437e-01, 1.23817824e-01, 4.64850347e-02, 3.43061806e-02, 4.03080496e-02,
            3.99110920e-02, 1.84022238e-01, 6.95217172e-02, 2.08340210e-02, 1.28738080e-02,
            1.27089783e-02, 2.24611460e-02, 5.83623038e-02, 6.74599190e-02, 9.47681188e-02,
            4.00952068e-02, 1.33219654e-02, 8.68288451e-03, 9.17647207e-03, 1.39370091e-02,
            2.23849779e-02, 1.50611077e-02, 2.08550541e-02, 2.12178691e-02, 7.36947008e-02
        ], [
            9.65554339e-02, 4.05313401e-03, 1.23342933e+00, 3.09480967e-02, 1.84696585e-01,
            2.75887418e-02, 1.56226958e-02, 8.12154782e-03, 2.74995016e-02, 1.97178723e-02,
            1.18750384e-02, 1.04679671e-02, 1.06787058e-02, 1.01082128e-02, 1.09197512e-02,
            1.54131868e-02, 5.79688772e-03, 1.05205760e-02, 7.60818060e-03, 1.02804143e-02,
            9.75387135e-03, 1.95794691e-01, 8.34421883e-03, 1.39918120e-02, 1.46734240e-02,
            1.25883258e-02, 6.60064447e-03, 6.51342830e-03, 2.67904121e-03, 8.66334320e-03,
            1.52972817e-02, 1.52653596e-02, 2.18578062e-02, 1.65740871e-02, 7.35493725e-03
        ], [
            6.01387468e-03, 3.38505087e-03, 8.82537848e-03, 1.62676352e+00, 1.39355509e-01,
            1.29231274e-02, 1.94897354e-02, 5.46470844e-03, 1.27535103e-02, 3.68730669e-02,
            1.36883968e-02, 1.04371910e-02, 1.16344907e-02, 1.07711068e-02, 1.79401409e-02,
            5.04966673e-02, 4.85272894e-03, 1.05291662e-02, 7.31573165e-03, 6.48124221e-03,
            6.68407269e-03, 8.57080710e-03, 7.36947260e-03, 9.61750899e-03, 8.72766017e-03,
            7.59395393e-03, 3.68982339e-03, 3.06393863e-03, 2.06180213e-03, 5.39111130e-03,
            7.95009534e-03, 4.09259905e-03, 9.38858760e-03, 1.08642219e-02, 7.04179830e-04
        ], [
            5.79327831e-04, 3.08548755e-04, 7.99884876e-04, 2.89069848e-02, 1.30408052e+00,
            1.62329692e-03, 1.88775749e-03, 5.12094169e-04, 1.03608462e-03, 2.78401254e-03,
            1.07342568e-03, 9.62574012e-04, 1.11813763e-03, 1.28239817e-03, 4.97507768e-03,
            9.45331849e-03, 4.63851101e-04, 8.68539735e-04, 1.23891632e-03, 7.00760221e-04,
            6.10473599e-04, 6.33104205e-04, 9.01472608e-04, 7.72819281e-04, 9.33862434e-04,
            8.17406138e-04, 5.75431518e-04, 2.92593347e-04, 1.66548131e-04, 4.77649875e-04,
            1.44843453e-03, 4.33377077e-04, 8.21019612e-04, 1.11725004e-03, 6.93599452e-05
        ], [
            4.71845868e-03, 3.82252890e-03, 7.17140610e-03, 8.22703406e-03, 8.78888178e-03,
            1.37466705e+00, 2.67963191e-02, 4.71661072e-03, 6.62798476e-03, 9.95321447e-03,
            1.81938501e-02, 1.30110334e-02, 1.11055648e-02, 8.29702793e-03, 1.13374076e-02,
            1.16649524e-01, 4.35404501e-03, 4.79991486e-02, 5.47546866e-03, 3.80412855e-03,
            3.75625836e-03, 6.61078095e-03, 5.22374539e-03, 4.09414213e-03, 4.46346979e-03,
            7.02769538e-03, 3.40492237e-03, 2.68925331e-03, 4.57859430e-03, 4.09528145e-03,
            6.48214760e-03, 6.80197056e-03, 5.12867664e-03, 7.59077586e-03, 7.29698018e-04
        ], [
            1.46801625e-02, 1.01025481e-02, 5.41640240e-02, 4.31479349e-02, 4.99164264e-02,
            4.18724996e-02, 1.35410089e+00, 1.63857315e-02, 4.08354578e-02, 4.65622692e-02,
            4.86656501e-02, 2.41337160e-02, 2.81160441e-02, 3.24884341e-02, 2.71100916e-02,
            4.70271891e-02, 1.40007675e-02, 2.54517883e-02, 2.34089712e-02, 2.36575598e-02,
            2.24114794e-02, 2.79932210e-02, 2.12556341e-02, 1.77506066e-02, 2.19671803e-02,
            3.22151129e-02, 2.28304744e-02, 3.08263661e-02, 8.79572831e-03, 3.90663891e-02,
            3.53203420e-02, 3.29464527e-02, 2.48280302e-02, 4.55671111e-02, 1.99460577e-03
        ], [
            2.98811544e-02, 1.57406367e-02, 2.63435149e-02, 3.53244250e-02, 3.14784875e-02,
            3.06711729e-02, 2.63195296e-02, 1.23529645e+00, 1.02804410e-01, 5.17208268e-02,
            5.29063400e-02, 4.41445547e-02, 2.65280249e-02, 2.18422680e-02, 2.44415264e-02,
            2.55857128e-02, 6.11923579e-02, 3.45363423e-02, 1.86170541e-02, 1.41916360e-02,
            1.16890315e-02, 1.73955838e-02, 8.15469922e-02, 1.05412619e-01, 1.55277974e-01,
            4.83541611e-02, 1.22735773e-02, 8.09164827e-03, 5.85694871e-03, 1.39275348e-02,
            2.02477517e-02, 1.11796773e-02, 1.70252186e-02, 1.74694270e-02, 2.32891952e-03
        ], [
            7.08281680e-02, 2.21074636e-02, 5.62307918e-02, 1.62059085e-01, 1.19219343e-01,
            9.60769450e-02, 8.93436907e-02, 4.85603048e-02, 1.36401097e+00, 3.71327972e-01,
            8.50328752e-02, 5.21388631e-02, 5.31494696e-02, 6.99821477e-02, 6.50843308e-02,
            8.06804959e-02, 2.50303874e-02, 5.05222250e-02, 2.91029179e-02, 1.35388759e-02,
            1.23467380e-02, 2.71243434e-02, 2.33643161e-02, 2.34036715e-02, 2.62830456e-02,
            2.46578164e-02, 1.33255424e-02, 9.23024630e-03, 8.00823883e-03, 1.85155781e-02,
            2.67288493e-02, 1.48977191e-02, 9.88234214e-02, 2.98987745e-02, 4.44506681e-03
        ], [
            1.45583155e-02, 1.12040623e-02, 3.84961952e-02, 3.58290875e-02, 7.90076258e-02,
            2.54721628e-02, 3.73257552e-02, 1.70910288e-02, 3.83280684e-02, 1.20285288e+00,
            2.98805902e-02, 2.44623947e-02, 4.98468323e-02, 5.63186729e-02, 7.65740327e-02,
            5.78136622e-02, 1.19170196e-02, 3.78574024e-02, 3.12278797e-02, 1.06072652e-02,
            1.05497740e-02, 1.64056112e-02, 2.40347343e-02, 1.47776355e-02, 1.81875931e-02,
            1.91524313e-02, 1.11523654e-02, 5.57884530e-03, 4.79595479e-03, 1.02029869e-02,
            1.24802072e-02, 7.16504693e-03, 1.51040609e-02, 1.40921601e-02, 1.69270605e-03
        ], [
            5.49009266e-03, 7.50087108e-03, 1.22323474e-02, 9.58083019e-03, 9.18772403e-03,
            1.61968037e-02, 7.01514000e-03, 9.92430102e-03, 1.38876943e-02, 1.49311277e-02,
            1.17716168e+00, 2.44781350e-02, 1.77373903e-02, 2.55098771e-02, 2.09458747e-02,
            1.70104940e-02, 8.13417121e-03, 1.22974084e-01, 9.87668470e-03, 3.69589846e-03,
            3.84063171e-03, 8.78128789e-03, 6.86697680e-03, 5.07352701e-03, 6.45884244e-03,
            7.62494569e-03, 5.70438711e-03, 2.87973780e-03, 6.87053103e-03, 4.57409959e-03,
            7.17280801e-03, 5.46381176e-03, 6.87473881e-03, 6.82837017e-03, 9.02660718e-04
        ], [
            2.39526811e-02, 4.23186705e-02, 4.15498055e-02, 4.21503843e-02, 4.94792593e-02,
            7.15363801e-02, 4.44147517e-02, 4.92151171e-02, 5.23591231e-02, 7.64977697e-02,
            9.61292211e-02, 1.52693391e+00, 3.23318410e-01, 1.88074342e-01, 2.47960259e-01,
            1.56722227e-01, 3.91780247e-02, 1.85267996e-01, 6.52114953e-02, 1.77171846e-02,
            1.63553655e-02, 2.59500030e-02, 3.93157471e-02, 3.80888754e-02, 4.48922009e-02,
            4.22986179e-02, 2.59126281e-02, 1.20721139e-02, 1.47076990e-02, 2.26424200e-02,
            3.06428098e-02, 1.64498035e-02, 2.40216171e-02, 2.83203278e-02, 4.77956915e-03
        ], [
            1.82554546e-02, 2.45234069e-02, 1.99689464e-02, 3.49786171e-02, 2.96980813e-02,
            3.35561573e-02, 2.50291080e-02, 3.24221630e-02, 2.78350788e-02, 3.87537039e-02,
            5.11841799e-02, 6.24749244e-02, 1.23985790e+00, 5.35518531e-02, 1.01132948e-01,
            3.94255310e-02, 2.47391005e-02, 4.81539646e-02, 4.66021905e-02, 9.42378251e-03,
            8.59992089e-03, 1.26165377e-02, 2.11921008e-02, 2.77997004e-02, 2.89881154e-02,
            2.51562319e-02, 1.14960211e-02, 6.53310268e-03, 6.12618411e-03, 1.10776716e-02,
            1.83459694e-02, 8.68216720e-03, 1.43685942e-02, 1.44233719e-02, 2.96301352e-03
        ], [
            2.24671120e-02, 3.13142763e-02, 3.63040146e-02, 5.22393796e-02, 4.64116891e-02,
            5.01064567e-02, 5.57067215e-02, 4.51892295e-02, 5.68461091e-02, 6.68261462e-02,
            6.41696022e-02, 8.42011060e-02, 2.61128011e-01, 1.70632610e+00, 2.22855292e-01,
            9.64035133e-02, 8.22835701e-02, 1.19459089e-01, 9.74855298e-02, 3.31603274e-02,
            3.02660136e-02, 3.41313982e-02, 4.93184903e-02, 4.18454895e-02, 5.36797389e-02,
            5.53282300e-02, 1.01324313e-01, 2.94282574e-02, 1.69535293e-02, 6.03846326e-02,
            6.35273427e-02, 3.92703023e-02, 7.19327474e-02, 5.69775353e-02, 5.38757293e-03
        ], [
            1.67941282e-02, 1.39864065e-02, 1.88933573e-02, 2.20069741e-02, 2.14185696e-02,
            2.51963049e-02, 2.13127077e-02, 2.04590154e-02, 1.84078279e-02, 2.43867037e-02,
            2.88069649e-02, 3.23124153e-02, 4.85843822e-02, 2.42366331e-02, 1.50327655e+00,
            2.84238153e-02, 1.70485698e-02, 2.69046900e-02, 1.22879090e-01, 1.60905524e-02,
            1.26014916e-02, 1.34711451e-02, 7.47132290e-02, 8.23980160e-02, 1.11146644e-01,
            5.06140998e-02, 1.31497828e-02, 8.31661862e-03, 6.01159756e-03, 1.45823458e-02,
            3.70911236e-02, 1.00755401e-02, 1.03090876e-02, 2.28732541e-02, 2.02865299e-03
        ], [
            2.39412958e-03, 2.30011968e-03, 3.91093383e-03, 1.11880520e-02, 7.79979539e-03,
            8.39294828e-03, 8.60316707e-03, 3.67979379e-03, 5.13339134e-03, 7.27014839e-03,
            7.99851642e-03, 1.78536225e-02, 1.05863426e-02, 7.77939211e-03, 1.32171090e-02,
            1.06254760e+00, 4.01440354e-03, 9.77367953e-03, 4.65414493e-03, 2.58978518e-03,
            2.82429167e-03, 4.63699693e-03, 6.13121215e-03, 3.30666943e-03, 5.21670099e-03,
            4.42466426e-03, 3.24706576e-03, 2.47250923e-03, 2.34560317e-03, 3.69808172e-03,
            5.76199573e-03, 5.29465454e-03, 5.31882504e-03, 7.95546368e-03, 2.10473636e-03
        ], [
            2.78655002e-02, 4.39569074e-02, 4.20071061e-02, 5.72222337e-02, 4.03960371e-02,
            5.13928618e-02, 5.70030989e-02, 1.01266833e-01, 6.67190067e-02, 6.15046734e-02,
            8.63425626e-02, 7.38705814e-02, 4.54587940e-02, 3.70151879e-02, 4.24436116e-02,
            3.93355583e-02, 1.19119001e+00, 3.98558661e-02, 2.84219847e-02, 1.67203124e-02,
            2.42699395e-02, 4.57087187e-02, 3.78883898e-02, 2.54881331e-02, 3.47475000e-02,
            3.97976568e-02, 2.25230904e-02, 1.44540841e-02, 1.46698120e-02, 1.79062359e-02,
            2.95954725e-02, 3.46989857e-02, 2.88378993e-02, 3.34467546e-02, 4.41707835e-03
        ], [
            7.30668109e-03, 9.82652450e-03, 9.34986142e-03, 1.02936206e-02, 8.97698203e-03,
            1.05997784e-02, 1.05908272e-02, 1.74954396e-02, 1.09910180e-02, 1.01348937e-02,
            1.43257098e-02, 1.21671726e-02, 9.89455187e-03, 8.17973113e-03, 9.43783616e-03,
            9.35301613e-03, 2.21531716e-02, 1.06128802e+00, 1.13512329e-02, 7.61415644e-03,
            9.40775312e-03, 1.22635410e-02, 1.55336625e-02, 9.10277860e-03, 1.13769149e-02,
            2.07779279e-02, 1.58137858e-02, 1.08277994e-02, 3.60809154e-02, 9.65196780e-03,
            2.35145141e-02, 1.52822233e-02, 1.32692599e-02, 1.57784430e-02, 1.52115962e-03
        ], [
            9.49209795e-03, 5.42993596e-03, 1.45833300e-02, 1.26352609e-02, 1.46213602e-02,
            1.39907147e-02, 1.17093546e-02, 9.90050635e-03, 1.25563098e-02, 1.37097611e-02,
            1.32325271e-02, 1.30986535e-02, 1.17566475e-02, 1.00106955e-02, 1.35884105e-02,
            1.32079611e-02, 1.15162657e-02, 1.38916146e-02, 1.01618891e+00, 7.02828724e-03,
            6.83446196e-03, 1.05446884e-02, 1.95089615e-02, 7.32567797e-03, 1.12863589e-02,
            1.44357853e-02, 6.76129303e-03, 6.43533414e-03, 3.98437570e-03, 8.93809400e-03,
            7.91711867e-03, 5.22802925e-03, 7.79732696e-03, 1.02947251e-02, 8.57038066e-04
        ], [
            6.03622332e-02, 3.11094966e-02, 1.13356235e-01, 9.80710713e-02, 1.14244876e-01,
            1.04204741e-01, 9.11489683e-02, 1.08829191e-01, 9.25542513e-02, 9.85890617e-02,
            8.04686190e-02, 9.33304490e-02, 9.40863150e-02, 8.41296571e-02, 1.06815735e-01,
            8.65519582e-02, 4.61210192e-02, 8.50565802e-02, 5.91631721e-02, 1.03757254e+00,
            2.53351206e-02, 8.12153009e-02, 4.94908136e-02, 5.00388697e-02, 5.99664425e-02,
            5.00958982e-02, 2.89862455e-02, 1.79423902e-02, 1.32051582e-02, 3.10643540e-02,
            4.14722910e-02, 2.63726935e-02, 5.04265155e-02, 3.98040512e-02, 5.02726214e-03
        ], [
            2.82988708e-02, 1.22146262e-02, 5.08920221e-02, 4.84008360e-02, 5.71356424e-02,
            4.03760361e-02, 3.56834332e-02, 2.97901830e-02, 4.03420331e-02, 4.20658879e-02,
            3.38277741e-02, 3.59196101e-02, 3.61725557e-02, 3.14049597e-02, 4.39201179e-02,
            4.38418690e-02, 1.72785099e-02, 4.40275209e-02, 2.64371963e-02, 1.22378132e-02,
            1.01286858e+00, 3.87392013e-02, 2.52320182e-02, 1.84929721e-02, 2.40356445e-02,
            2.19395584e-02, 1.57398172e-02, 1.19950263e-02, 7.39896556e-03, 1.60681841e-02,
            1.72879329e-02, 1.19912560e-02, 2.25858462e-02, 2.09763683e-02, 2.32964880e-03
        ], [
            5.82663825e-03, 6.20792997e-03, 1.17228994e-02, 1.34774413e-02, 1.42504277e-02,
            1.21211665e-02, 1.38994731e-02, 8.82423425e-03, 1.21985230e-02, 1.32400728e-02,
            1.33582491e-02, 1.27119256e-02, 1.39101350e-02, 1.24927193e-02, 1.24015770e-02,
            1.13233786e-02, 8.00566033e-03, 1.35981140e-02, 1.11606116e-02, 1.22002231e-02,
            1.00295887e-02, 1.01628003e+00, 1.41975563e-02, 1.28958838e-02, 2.64885990e-02,
            3.17478484e-02, 1.35853116e-02, 1.82569795e-02, 5.44406701e-03, 1.88024340e-02,
            1.67293794e-02, 1.31350966e-02, 1.38096398e-02, 1.89029887e-02, 9.37152407e-04
        ], [
            2.77610238e-02, 2.53651232e-02, 5.02841221e-02, 5.14753015e-02, 5.17769723e-02,
            5.20531802e-02, 4.26250442e-02, 8.95883164e-02, 4.63121591e-02, 4.52472090e-02,
            6.19145767e-02, 4.50977568e-02, 3.70943422e-02, 3.00071402e-02, 3.87348507e-02,
            4.23159873e-02, 3.65333271e-02, 4.58771447e-02, 3.13501085e-02, 3.07910374e-02,
            2.03771290e-02, 2.73796001e-02, 1.06594335e+00, 3.42668587e-02, 3.80755357e-02,
            8.11965361e-02, 1.82669710e-02, 1.35455366e-02, 6.89576007e-03, 1.53350242e-02,
            2.20710220e-02, 1.76928461e-02, 1.92749297e-02, 2.17973489e-02, 3.34503799e-03
        ], [
            5.83842745e-03, 8.17896957e-03, 1.18681619e-02, 1.34859283e-02, 1.27296367e-02,
            1.62675443e-02, 8.96131421e-03, 1.04051990e-02, 9.20622226e-03, 1.01636580e-02,
            1.63653488e-02, 1.36450494e-02, 1.02703911e-02, 7.99018242e-03, 1.02797741e-02,
            9.16056354e-03, 6.65544518e-03, 9.89789212e-03, 5.27130635e-03, 9.01599866e-03,
            5.40624739e-03, 6.05765530e-03, 7.49190954e-03, 1.17883434e+00, 8.29362539e-03,
            1.07940360e-02, 3.28379316e-03, 2.49756643e-03, 4.45920696e-03, 3.75344871e-03,
            3.87045871e-03, 2.73065266e-03, 3.30231232e-03, 4.17466478e-03, 8.48841151e-04
        ], [
            2.53584526e-03, 2.68503228e-03, 4.54500231e-03, 5.42678484e-03, 6.25479471e-03,
            5.30940488e-03, 5.67335561e-03, 3.74680037e-03, 4.87334387e-03, 5.04858435e-03,
            4.71663606e-03, 4.25824069e-03, 5.36268556e-03, 4.98087682e-03, 5.08259505e-03,
            4.35574583e-03, 2.73455787e-03, 4.42479558e-03, 4.78212339e-03, 4.99402496e-03,
            3.30689632e-03, 3.80913470e-03, 5.54105806e-03, 9.91642733e-03, 1.03548669e+00,
            1.79888253e-02, 6.25129297e-03, 5.79676545e-03, 1.62314209e-03, 6.11578799e-03,
            6.23906554e-03, 4.83425735e-03, 3.29922401e-03, 7.10979995e-03, 3.71604117e-04
        ], [
            1.19802816e-02, 1.04141172e-02, 2.20762889e-02, 1.99850852e-02, 2.10515043e-02,
            2.33515835e-02, 2.42309595e-02, 2.27654447e-02, 1.88912840e-02, 2.01591967e-02,
            2.36525499e-02, 1.98681068e-02, 1.80793161e-02, 1.52303897e-02, 1.94460245e-02,
            1.75767897e-02, 1.25324083e-02, 2.76429632e-02, 2.45587780e-02, 2.98345020e-02,
            1.89568664e-02, 1.83027526e-02, 5.25406407e-02, 1.57489640e-01, 1.25980627e-01,
            1.12498705e+00, 1.09184639e-02, 9.80516592e-03, 4.96566268e-03, 1.23704219e-02,
            1.25156061e-02, 8.29023256e-03, 1.06614267e-02, 1.51475331e-02, 1.36675989e-03
        ], [
            1.54156231e-02, 1.34947847e-02, 2.50744094e-02, 2.95925854e-02, 2.91162337e-02,
            2.63694308e-02, 3.56229351e-02, 2.30822392e-02, 2.87174659e-02, 2.66182092e-02,
            2.61057756e-02, 2.81091585e-02, 2.73428138e-02, 2.89847232e-02, 2.69720321e-02,
            3.22702698e-02, 2.07745790e-02, 3.32485774e-02, 3.45021994e-02, 3.28583429e-02,
            3.49642835e-02, 3.27193084e-02, 3.73372962e-02, 4.07301661e-02, 4.46327299e-02,
            4.51446528e-02, 1.14754955e+00, 5.35708578e-02, 1.41966833e-02, 4.33493084e-02,
            3.97420162e-02, 2.24676896e-02, 2.97011602e-02, 4.17990942e-02, 2.85102785e-03
        ], [
            4.40983665e-02, 4.19752538e-02, 6.18791331e-02, 8.07927644e-02, 7.06165160e-02,
            6.34005614e-02, 6.92477998e-02, 6.42111176e-02, 6.36904863e-02, 6.25639880e-02,
            6.95251379e-02, 6.40687010e-02, 6.17549886e-02, 5.61037374e-02, 6.41769273e-02,
            7.29243414e-02, 5.64790859e-02, 6.74644273e-02, 5.40537947e-02, 5.84687249e-02,
            6.23755135e-02, 6.31488829e-02, 8.74677441e-02, 6.24142648e-02, 7.82681930e-02,
            7.95045155e-02, 4.66853070e-02, 1.27661469e+00, 1.15387838e-01, 6.96601631e-02,
            5.28588317e-02, 3.72882584e-02, 5.61648444e-02, 8.22968119e-02, 1.58206878e-02
        ], [
            1.11831407e-02, 1.16067303e-02, 1.99908177e-02, 2.21481882e-02, 2.40958908e-02,
            2.15533267e-02, 2.67309210e-02, 1.71497485e-02, 1.95828018e-02, 2.14783480e-02,
            1.92284856e-02, 1.98426272e-02, 2.05538862e-02, 1.80314508e-02, 2.24696542e-02,
            2.18595425e-02, 1.41840411e-02, 2.63660020e-02, 3.39626534e-02, 2.87600266e-02,
            5.18972303e-02, 4.09691845e-02, 2.14725304e-02, 2.69307730e-02, 2.54886215e-02,
            3.62637022e-02, 2.42761293e-02, 3.91020893e-02, 1.04204730e+00, 3.47203228e-02,
            2.59963591e-02, 2.82573390e-02, 4.66427802e-02, 4.09090803e-02, 1.73617575e-03
        ], [
            5.45108831e-02, 6.01402490e-02, 1.10182019e-01, 1.03163494e-01, 1.00779372e-01,
            9.70940936e-02, 1.51873920e-01, 8.71678919e-02, 1.49434137e-01, 1.24605121e-01,
            1.03948752e-01, 1.02525555e-01, 1.19765629e-01, 1.25044948e-01, 1.34795995e-01,
            9.92984088e-02, 8.07390535e-02, 1.31803237e-01, 1.10851845e-01, 1.03867741e-01,
            1.03544932e-01, 1.08148782e-01, 1.02522929e-01, 8.55605140e-02, 1.41571177e-01,
            1.34178445e-01, 1.13982392e-01, 1.88211305e-01, 6.38775333e-02, 1.21945468e+00,
            1.44253119e-01, 7.30635015e-02, 1.26198144e-01, 1.51913446e-01, 1.06493441e-02
        ], [
            3.26273294e-03, 3.81359312e-03, 5.30090590e-03, 4.21058892e-03, 4.26479001e-03,
            6.66698911e-03, 7.93724400e-03, 5.24040066e-03, 6.51224357e-03, 6.02988649e-03,
            6.27503455e-03, 7.66500040e-03, 5.00331372e-03, 4.24392027e-03, 5.05663832e-03,
            5.73734104e-03, 6.69525071e-03, 5.07937224e-03, 5.13905560e-03, 3.22624424e-03,
            4.31762513e-03, 6.31096217e-03, 6.04335386e-03, 8.03952162e-03, 1.18957319e-02,
            6.33096593e-03, 4.69863756e-03, 3.85585291e-03, 4.27845143e-03, 5.14002123e-03,
            1.00965182e+00, 6.09247695e-03, 6.62603643e-03, 1.10461364e-02, 4.53953058e-04
        ], [
            1.21339964e-03, 8.43665046e-04, 1.58483431e-03, 1.57726673e-03, 1.63885916e-03,
            1.49682628e-03, 1.75056524e-03, 1.47714805e-03, 2.17699877e-03, 1.91004358e-03,
            1.60133329e-03, 1.67042732e-03, 1.98854858e-03, 1.81220538e-03, 2.22607022e-03,
            1.46381228e-03, 1.39892106e-03, 1.62559173e-03, 1.66124009e-03, 1.37747767e-03,
            1.72487297e-03, 1.47850839e-03, 1.87169956e-03, 1.25858656e-03, 3.32139605e-03,
            2.28515517e-03, 2.09618875e-03, 2.29027782e-03, 7.62579221e-04, 3.16587564e-03,
            4.97021113e-03, 1.01581101e+00, 2.25492780e-03, 2.70841800e-03, 1.42026503e-04
        ], [
            1.87865618e-03, 7.18183196e-04, 1.56278184e-03, 1.42005817e-03, 2.14848559e-03,
            1.61180212e-03, 1.34379203e-03, 9.40226323e-04, 1.32524061e-03, 1.25294638e-03,
            1.64440516e-03, 1.39651452e-03, 1.89258080e-03, 1.30759486e-03, 1.56561692e-03,
            1.26745670e-03, 9.30223751e-04, 1.21391420e-03, 1.08280514e-03, 8.20729529e-04,
            7.70232774e-04, 1.54716184e-03, 1.11694357e-03, 9.09569749e-04, 1.14321497e-03,
            1.11206404e-03, 9.97411693e-04, 1.07216194e-03, 3.50261538e-04, 1.06188072e-03,
            6.40060673e-03, 2.04444695e-03, 1.03405401e+00, 2.00725565e-03, 1.29438844e-04
        ], [
            9.82260511e-03, 1.11820311e-02, 1.75029446e-02, 1.82427079e-02, 2.06326524e-02,
            1.73533760e-02, 2.84912776e-02, 1.46734451e-02, 2.05842504e-02, 1.84723780e-02,
            1.80314861e-02, 1.82445933e-02, 1.69084066e-02, 1.49857327e-02, 1.84607516e-02,
            1.62981812e-02, 1.40307061e-02, 1.85469284e-02, 1.65811931e-02, 1.53954699e-02,
            1.61082878e-02, 2.50761302e-02, 2.35687384e-02, 1.59034942e-02, 2.03959238e-02,
            2.81213476e-02, 2.71601903e-02, 2.28669812e-02, 1.22277319e-02, 3.62882074e-02,
            2.77384304e-02, 1.88193883e-02, 2.37640694e-02, 1.09270489e+00, 2.59937601e-03
        ], [
            4.04163728e-05, 4.53579620e-05, 7.21476339e-05, 1.81893494e-04, 9.51072002e-05,
            8.24970967e-05, 8.48368538e-05, 6.65472371e-05, 6.70178312e-05, 7.07291008e-05,
            6.88593507e-05, 6.28055814e-05, 8.04250093e-05, 5.72138987e-05, 1.12985835e-04,
            6.15178390e-05, 3.95943034e-05, 1.03712344e-04, 7.12278202e-05, 4.37928622e-05,
            7.12318026e-05, 9.64611358e-05, 1.63071438e-04, 4.44092752e-05, 6.18283338e-05,
            6.23793650e-05, 6.74247531e-05, 1.21016152e-04, 2.35664874e-04, 1.04120345e-04,
            1.05818892e-04, 6.80542736e-05, 9.83366199e-05, 1.52442597e-04, 1.00007983e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 403)
    return ret


@cache('run')
def leontief_matrix_x2009x():
    """
    leontief matrix x2009x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.16963337e+00, 4.46228395e-03, 3.79008126e-01, 1.14867590e-01, 1.56998544e-01,
            2.27894740e-01, 4.14547398e-02, 9.57824267e-03, 2.53612530e-02, 3.41504227e-02,
            1.25157962e-02, 1.00098975e-02, 1.01417261e-02, 9.40939862e-03, 1.12997795e-02,
            4.45508194e-02, 5.69120001e-03, 1.91333144e-02, 7.04078033e-03, 8.95905282e-03,
            8.37311766e-03, 1.10574755e-01, 9.11846284e-03, 1.06843084e-02, 1.06594976e-02,
            1.87913047e-02, 4.80596043e-03, 4.90944297e-03, 3.11206255e-03, 7.59117410e-03,
            1.05138959e-02, 1.09815029e-02, 1.30661855e-02, 1.32167933e-02, 3.29691110e-02
        ], [
            2.33323085e-02, 1.12290405e+00, 2.79416797e-02, 3.87241659e-02, 3.28951250e-02,
            3.76569067e-02, 3.35291866e-02, 5.03034646e-01, 9.49260436e-02, 5.33560577e-02,
            1.39509033e-01, 1.35126045e-01, 4.98773520e-02, 3.51675332e-02, 4.11731397e-02,
            3.96545621e-02, 1.64604507e-01, 7.25139135e-02, 1.84140978e-02, 1.11994472e-02,
            1.15555207e-02, 2.06327991e-02, 5.03590771e-02, 5.85598810e-02, 6.60981360e-02,
            3.70425171e-02, 1.16119497e-02, 7.56324060e-03, 9.40983308e-03, 1.28002277e-02,
            2.04280525e-02, 1.36885271e-02, 1.83735476e-02, 1.94256867e-02, 6.86362233e-02
        ], [
            9.54704553e-02, 4.43080396e-03, 1.23296788e+00, 3.11901994e-02, 1.83631263e-01,
            2.98669982e-02, 1.58604724e-02, 8.57648438e-03, 2.78673612e-02, 2.02135237e-02,
            1.23050846e-02, 1.12873953e-02, 1.14192374e-02, 1.06651248e-02, 1.15496790e-02,
            1.59248838e-02, 5.98342902e-03, 1.09592600e-02, 7.34670651e-03, 1.04476688e-02,
            9.77526028e-03, 1.94698504e-01, 8.70848800e-03, 1.47329003e-02, 1.48487073e-02,
            1.27905946e-02, 6.11696901e-03, 6.23467326e-03, 2.63854686e-03, 8.86392432e-03,
            1.39403840e-02, 1.42521770e-02, 2.02902964e-02, 1.63185866e-02, 7.26209850e-03
        ], [
            6.03027012e-03, 4.00667670e-03, 9.04268739e-03, 1.65517472e+00, 1.50809867e-01,
            1.34113043e-02, 2.00652967e-02, 5.72045014e-03, 1.34217874e-02, 3.92542068e-02,
            1.50148732e-02, 1.19762270e-02, 1.30860236e-02, 1.19547506e-02, 1.87185596e-02,
            5.34267141e-02, 5.33892518e-03, 1.11782339e-02, 7.06534071e-03, 6.78937044e-03,
            6.68157444e-03, 8.62198940e-03, 7.96756239e-03, 1.07517436e-02, 9.39491430e-03,
            8.08298194e-03, 3.70055983e-03, 3.09595897e-03, 2.14950874e-03, 5.89614633e-03,
            7.65735732e-03, 4.16703886e-03, 8.91339712e-03, 1.11540102e-02, 7.66576485e-04
        ], [
            5.81988712e-04, 3.46949317e-04, 8.17427954e-04, 3.10345555e-02, 1.31430340e+00,
            1.72743067e-03, 2.04652840e-03, 5.17744759e-04, 1.11736731e-03, 3.10697396e-03,
            1.15212110e-03, 1.00356224e-03, 1.21716032e-03, 1.40834108e-03, 5.75728622e-03,
            9.95929560e-03, 4.85066620e-04, 9.03958627e-04, 1.22095290e-03, 6.79012236e-04,
            5.97127273e-04, 6.30364687e-04, 9.92262204e-04, 8.71654878e-04, 9.89390126e-04,
            8.34886674e-04, 5.31636127e-04, 2.86385863e-04, 1.67344885e-04, 5.09943132e-04,
            1.31895450e-03, 4.36383486e-04, 7.89533403e-04, 1.09775967e-03, 7.43118023e-05
        ], [
            4.76856977e-03, 4.23927436e-03, 7.06960308e-03, 8.30952936e-03, 8.69530213e-03,
            1.37961253e+00, 2.75923769e-02, 4.39110456e-03, 6.56798420e-03, 1.00272710e-02,
            1.86355039e-02, 1.38032813e-02, 1.18000667e-02, 8.71230292e-03, 1.19118981e-02,
            1.13044914e-01, 4.55653608e-03, 4.72606725e-02, 4.85256643e-03, 3.47756399e-03,
            3.62590586e-03, 6.61225437e-03, 5.39718526e-03, 4.26475973e-03, 4.39173676e-03,
            7.08523064e-03, 3.26616991e-03, 2.57001658e-03, 4.36931836e-03, 4.25768768e-03,
            6.40311566e-03, 6.88883109e-03, 4.98164070e-03, 7.44577954e-03, 7.55936409e-04
        ], [
            1.40488245e-02, 9.48398008e-03, 5.27230104e-02, 4.17284445e-02, 4.90884474e-02,
            4.09009207e-02, 1.34905261e+00, 1.51921536e-02, 3.91931657e-02, 4.57832518e-02,
            4.87552527e-02, 2.39254390e-02, 2.76856911e-02, 3.24746577e-02, 2.68621738e-02,
            4.50142305e-02, 1.37747425e-02, 2.45969450e-02, 2.14235777e-02, 2.22203366e-02,
            2.12497656e-02, 2.69582118e-02, 2.10694440e-02, 1.82407207e-02, 2.13793058e-02,
            3.15259480e-02, 2.12543709e-02, 2.79299458e-02, 8.26596242e-03, 3.85367550e-02,
            3.37346706e-02, 3.15626233e-02, 2.27604299e-02, 4.34972618e-02, 1.90257036e-03
        ], [
            2.60444932e-02, 1.52729421e-02, 2.30514435e-02, 2.94198712e-02, 2.65795080e-02,
            2.65576290e-02, 2.21460551e-02, 1.13486963e+00, 7.70261485e-02, 4.25080622e-02,
            4.43372116e-02, 3.87148260e-02, 2.35860513e-02, 1.91568928e-02, 2.15215953e-02,
            2.22562860e-02, 4.25253533e-02, 3.10692960e-02, 1.58609368e-02, 1.27509410e-02,
            1.02501817e-02, 1.50306045e-02, 7.58948548e-02, 1.00597658e-01, 1.17703440e-01,
            4.60556617e-02, 1.03088856e-02, 6.91077598e-03, 5.08017144e-03, 1.26102985e-02,
            1.81538694e-02, 9.58333172e-03, 1.36684774e-02, 1.51542677e-02, 1.99668439e-03
        ], [
            6.83434374e-02, 2.22653463e-02, 5.55455729e-02, 1.59033689e-01, 1.18277898e-01,
            9.77366080e-02, 8.94349368e-02, 4.87433089e-02, 1.34890775e+00, 3.75269974e-01,
            8.65163196e-02, 5.34943171e-02, 5.50429148e-02, 7.36023548e-02, 6.64018903e-02,
            8.16775933e-02, 2.42893246e-02, 5.13113614e-02, 2.80356473e-02, 1.29809917e-02,
            1.18514698e-02, 2.64888489e-02, 2.39036521e-02, 2.51108771e-02, 2.54254588e-02,
            2.52837834e-02, 1.28433168e-02, 8.75874521e-03, 7.73934421e-03, 1.87993696e-02,
            2.43292131e-02, 1.45906865e-02, 9.44452513e-02, 2.94311634e-02, 4.11235800e-03
        ], [
            1.45912395e-02, 1.19182539e-02, 3.91094368e-02, 3.63191729e-02, 8.02176943e-02,
            2.53301915e-02, 3.82449732e-02, 1.95261293e-02, 3.92999477e-02, 1.21838656e+00,
            3.12190849e-02, 2.59784139e-02, 5.25904714e-02, 6.09964742e-02, 7.91299631e-02,
            5.93634724e-02, 1.24469226e-02, 3.80259554e-02, 3.07828355e-02, 1.02184607e-02,
            1.03087635e-02, 1.65454739e-02, 2.50219690e-02, 1.62838781e-02, 1.82966185e-02,
            1.94531979e-02, 1.10243953e-02, 5.40767885e-03, 4.74472547e-03, 1.06910961e-02,
            1.22519457e-02, 7.20174898e-03, 1.46217516e-02, 1.44535275e-02, 1.70773548e-03
        ], [
            5.02100703e-03, 7.61262606e-03, 1.11035866e-02, 8.82813479e-03, 8.50432905e-03,
            1.41613913e-02, 6.55192980e-03, 8.96730004e-03, 1.28288749e-02, 1.38680851e-02,
            1.16893556e+00, 2.42140167e-02, 1.74835768e-02, 2.48113902e-02, 1.96902301e-02,
            1.66862845e-02, 7.88764383e-03, 1.17803725e-01, 8.73437694e-03, 3.31945985e-03,
            3.51812047e-03, 8.03430225e-03, 6.58175211e-03, 4.93314732e-03, 5.80059289e-03,
            7.11261743e-03, 5.14873479e-03, 2.64122820e-03, 6.46773013e-03, 4.39123597e-03,
            6.69667984e-03, 5.20040208e-03, 6.19467766e-03, 6.31175336e-03, 8.58909772e-04
        ], [
            2.34896201e-02, 4.51662933e-02, 4.21226636e-02, 4.30213158e-02, 5.04080486e-02,
            7.07920172e-02, 4.55492744e-02, 4.69610459e-02, 5.26374266e-02, 7.85162975e-02,
            1.01705305e-01, 1.55680092e+00, 3.48761600e-01, 2.04617443e-01, 2.61647277e-01,
            1.60726361e-01, 4.05247978e-02, 2.00049817e-01, 6.33336354e-02, 1.69976702e-02,
            1.62230125e-02, 2.62233293e-02, 4.04198035e-02, 4.11723502e-02, 4.31734470e-02,
            4.31771165e-02, 2.56963623e-02, 1.17843577e-02, 1.52149622e-02, 2.39095847e-02,
            3.12327896e-02, 1.71281957e-02, 2.39956290e-02, 2.86855290e-02, 4.83613537e-03
        ], [
            1.58543465e-02, 2.51178207e-02, 1.76640273e-02, 3.17300113e-02, 2.68576510e-02,
            2.99085125e-02, 2.26479634e-02, 2.73391696e-02, 2.50210630e-02, 3.53561696e-02,
            4.75146349e-02, 6.16704862e-02, 1.22661586e+00, 5.01016181e-02, 9.65486270e-02,
            3.91640175e-02, 2.14665140e-02, 4.41860607e-02, 3.94631089e-02, 7.95135443e-03,
            7.39244445e-03, 1.10906347e-02, 1.88008922e-02, 2.67252283e-02, 2.41966911e-02,
            2.14434550e-02, 9.84865714e-03, 5.31788596e-03, 5.17592984e-03, 1.01406381e-02,
            1.60404918e-02, 7.70163515e-03, 1.27411095e-02, 1.26080028e-02, 2.75698681e-03
        ], [
            2.18976141e-02, 3.48392049e-02, 3.62675437e-02, 5.29320301e-02, 4.63789133e-02,
            4.98234989e-02, 5.58994495e-02, 4.43119027e-02, 5.70651844e-02, 6.74859385e-02,
            6.51556297e-02, 8.89554918e-02, 2.71425788e-01, 1.73438282e+00, 2.33820532e-01,
            1.11459106e-01, 8.54080221e-02, 1.21932722e-01, 9.50568608e-02, 3.16408625e-02,
            3.04169385e-02, 3.43552203e-02, 5.05018106e-02, 4.44627573e-02, 5.32524343e-02,
            5.44550938e-02, 1.00182165e-01, 2.80129108e-02, 1.68796428e-02, 6.37730214e-02,
            6.30803065e-02, 3.94713330e-02, 7.01039086e-02, 5.80609010e-02, 5.77830879e-03
        ], [
            1.35267293e-02, 1.33475893e-02, 1.65038913e-02, 2.01357961e-02, 1.91312669e-02,
            2.21325430e-02, 2.01625483e-02, 1.71163297e-02, 1.65415886e-02, 2.25949577e-02,
            2.63739376e-02, 3.16501446e-02, 4.58365433e-02, 2.27657754e-02, 1.47756597e+00,
            2.94174628e-02, 1.56543769e-02, 2.35379298e-02, 1.10562773e-01, 1.39654252e-02,
            1.10224478e-02, 1.21553743e-02, 7.00023676e-02, 8.41495695e-02, 1.00124241e-01,
            3.73851376e-02, 1.16876758e-02, 7.26312404e-03, 4.76839005e-03, 1.39181029e-02,
            3.47901988e-02, 9.32678649e-03, 9.09482932e-03, 2.13489501e-02, 1.91821571e-03
        ], [
            2.26330816e-03, 2.37720520e-03, 3.89604843e-03, 1.22104793e-02, 8.45104168e-03,
            8.15251727e-03, 8.99617595e-03, 3.71697893e-03, 5.19475480e-03, 7.35546125e-03,
            8.05601959e-03, 1.64216687e-02, 1.09200422e-02, 7.85976809e-03, 1.40355452e-02,
            1.07590234e+00, 4.43436135e-03, 9.39349569e-03, 4.36832925e-03, 2.44915517e-03,
            2.77716254e-03, 4.59335619e-03, 7.02267954e-03, 3.50386072e-03, 5.52713890e-03,
            4.44736085e-03, 3.46931172e-03, 2.47394271e-03, 2.26388881e-03, 3.81976898e-03,
            5.46246535e-03, 5.19610161e-03, 4.95983030e-03, 7.64637200e-03, 2.33893311e-03
        ], [
            2.54675861e-02, 4.40223143e-02, 3.93177135e-02, 5.42804827e-02, 3.76679628e-02,
            4.85150021e-02, 5.28821840e-02, 6.35081554e-02, 5.95792996e-02, 5.81505764e-02,
            8.26850941e-02, 7.36933204e-02, 4.58083344e-02, 3.61309904e-02, 4.18183974e-02,
            3.71594126e-02, 1.18599004e+00, 3.83304941e-02, 2.59445599e-02, 1.53648833e-02,
            2.23900300e-02, 4.25970740e-02, 3.55678091e-02, 2.28672308e-02, 2.83724773e-02,
            3.89993085e-02, 2.08281863e-02, 1.31557960e-02, 1.48846134e-02, 1.73719907e-02,
            2.83276494e-02, 3.20880950e-02, 2.63912341e-02, 3.12499155e-02, 4.06415621e-03
        ], [
            7.68585660e-03, 1.07334735e-02, 9.82089286e-03, 1.08055137e-02, 9.17400215e-03,
            1.09828244e-02, 1.08085854e-02, 1.48454268e-02, 1.08137291e-02, 1.04924950e-02,
            1.48868561e-02, 1.25824233e-02, 1.04622916e-02, 8.60891760e-03, 1.00556201e-02,
            1.00523707e-02, 2.45629876e-02, 1.06351099e+00, 1.09685230e-02, 7.64584656e-03,
            9.78443152e-03, 1.30493818e-02, 1.67873695e-02, 9.52878729e-03, 1.17120987e-02,
            2.18499800e-02, 1.60535736e-02, 1.11264853e-02, 3.71520662e-02, 1.04889025e-02,
            2.49395574e-02, 1.64105492e-02, 1.36527509e-02, 1.63451071e-02, 1.67078151e-03
        ], [
            8.67979367e-03, 5.42145712e-03, 1.34013552e-02, 1.16762128e-02, 1.34956573e-02,
            1.25881351e-02, 1.08748740e-02, 9.57211823e-03, 1.14949938e-02, 1.29903971e-02,
            1.22845342e-02, 1.21683465e-02, 1.11678671e-02, 9.36263881e-03, 1.30188961e-02,
            1.24576864e-02, 1.10641892e-02, 1.27408577e-02, 1.01529927e+00, 6.33713773e-03,
            6.36272146e-03, 9.93263673e-03, 1.89744772e-02, 7.20588979e-03, 1.06753090e-02,
            1.34891303e-02, 6.13521932e-03, 5.79679393e-03, 3.67817593e-03, 8.98732143e-03,
            7.67096797e-03, 4.93567506e-03, 7.22901624e-03, 9.62517608e-03, 7.81072769e-04
        ], [
            5.89052269e-02, 3.08696396e-02, 1.12508312e-01, 9.66351411e-02, 1.10050024e-01,
            1.01573342e-01, 8.84820922e-02, 9.40516304e-02, 8.81253854e-02, 9.70596637e-02,
            7.79369142e-02, 8.83757938e-02, 9.17196303e-02, 8.51705802e-02, 1.04955436e-01,
            8.44020984e-02, 4.28084791e-02, 8.23774609e-02, 5.60175566e-02, 1.03505736e+00,
            2.52223535e-02, 7.83014194e-02, 4.88805489e-02, 5.01469636e-02, 5.69043972e-02,
            5.08759102e-02, 2.83120607e-02, 1.68853857e-02, 1.25274973e-02, 3.13777102e-02,
            3.89010726e-02, 2.46936709e-02, 4.74497851e-02, 3.81808557e-02, 4.67777968e-03
        ], [
            2.79925001e-02, 1.29125961e-02, 4.99018337e-02, 4.73293807e-02, 5.49613604e-02,
            4.04895076e-02, 3.61421206e-02, 3.38238830e-02, 3.90521736e-02, 4.22607928e-02,
            3.42766051e-02, 3.72294912e-02, 3.78374027e-02, 3.31892438e-02, 4.60179788e-02,
            4.41304200e-02, 1.80450252e-02, 4.28339768e-02, 2.64017832e-02, 1.18145309e-02,
            1.01275000e+00, 3.82057501e-02, 2.63254422e-02, 2.05319850e-02, 2.48520568e-02,
            2.29390776e-02, 1.56660130e-02, 1.14940308e-02, 7.18770132e-03, 1.67184403e-02,
            1.74414468e-02, 1.19844595e-02, 2.21954055e-02, 2.09284902e-02, 2.22445568e-03
        ], [
            5.70650109e-03, 6.77956833e-03, 1.16072781e-02, 1.33490376e-02, 1.39443161e-02,
            1.18964785e-02, 1.33354985e-02, 8.33194307e-03, 1.18584859e-02, 1.32142063e-02,
            1.35034570e-02, 1.29067394e-02, 1.39892555e-02, 1.24941519e-02, 1.23970119e-02,
            1.12746528e-02, 8.33772357e-03, 1.37045483e-02, 1.03619106e-02, 1.16529529e-02,
            9.68779956e-03, 1.01619168e+00, 1.45486649e-02, 1.32391030e-02, 2.52344709e-02,
            3.21019443e-02, 1.20198250e-02, 1.73563105e-02, 5.25363927e-03, 1.89665474e-02,
            1.63646579e-02, 1.29788556e-02, 1.33155170e-02, 1.80715869e-02, 9.58083346e-04
        ], [
            2.62282936e-02, 2.66870585e-02, 4.79472081e-02, 4.97749568e-02, 4.92862380e-02,
            4.94368192e-02, 4.08273723e-02, 7.18963844e-02, 4.24256819e-02, 4.36694411e-02,
            5.92815646e-02, 4.45914061e-02, 3.74743108e-02, 3.02055194e-02, 3.86014999e-02,
            4.22170044e-02, 3.51955408e-02, 4.50852553e-02, 2.94783827e-02, 3.04816039e-02,
            2.02739042e-02, 2.62013604e-02, 1.06514311e+00, 3.28958597e-02, 3.42761632e-02,
            7.91072947e-02, 1.76748790e-02, 1.30775429e-02, 6.74810532e-03, 1.54703263e-02,
            2.09456019e-02, 1.69506921e-02, 1.79594368e-02, 2.11295921e-02, 3.17921664e-03
        ], [
            5.75895785e-03, 9.89023088e-03, 1.17742788e-02, 1.34488776e-02, 1.26423085e-02,
            1.61010410e-02, 9.37178036e-03, 1.12031521e-02, 9.67756828e-03, 1.07465072e-02,
            1.78034018e-02, 1.58516688e-02, 1.16154845e-02, 8.88773415e-03, 1.11103198e-02,
            9.50265266e-03, 7.19455593e-03, 1.04554831e-02, 5.22037902e-03, 9.25118982e-03,
            5.15643132e-03, 5.82398167e-03, 7.02810794e-03, 1.19005824e+00, 8.01345625e-03,
            1.04452910e-02, 3.05984347e-03, 2.22535669e-03, 3.23220756e-03, 3.57794647e-03,
            3.81248011e-03, 2.62389363e-03, 3.22576006e-03, 4.02667491e-03, 8.95327306e-04
        ], [
            2.36626683e-03, 2.69605067e-03, 4.28903003e-03, 5.14690250e-03, 5.89923394e-03,
            4.84614235e-03, 5.23361256e-03, 3.55504652e-03, 4.55879943e-03, 4.78872435e-03,
            4.50079248e-03, 4.10186901e-03, 5.21466850e-03, 4.88494179e-03, 4.98531580e-03,
            4.20987150e-03, 2.69968556e-03, 4.22725005e-03, 4.17417100e-03, 4.64983248e-03,
            3.19016968e-03, 3.75700588e-03, 5.17741899e-03, 9.06269271e-03, 1.03620074e+00,
            1.66806414e-02, 5.71554194e-03, 5.40446586e-03, 1.52547813e-03, 6.02679900e-03,
            5.73094525e-03, 4.94531621e-03, 3.06271513e-03, 6.80079906e-03, 3.54278232e-04
        ], [
            1.18244335e-02, 1.12998861e-02, 2.16472987e-02, 1.93123600e-02, 2.02436692e-02,
            2.29319052e-02, 2.34971565e-02, 2.47225377e-02, 1.81616639e-02, 2.00301860e-02,
            2.32588164e-02, 1.99602221e-02, 1.79887468e-02, 1.53791159e-02, 1.92818690e-02,
            1.71743668e-02, 1.29137787e-02, 2.85627540e-02, 2.33938582e-02, 2.81225607e-02,
            1.94626802e-02, 1.80749817e-02, 5.15231610e-02, 1.58223170e-01, 1.27208473e-01,
            1.12087013e+00, 1.04587366e-02, 9.69141137e-03, 4.82120888e-03, 1.25013626e-02,
            1.25358841e-02, 8.10159466e-03, 1.04280245e-02, 1.48735278e-02, 1.34894760e-03
        ], [
            1.58479723e-02, 1.37145736e-02, 2.59548708e-02, 3.11883413e-02, 2.94544708e-02,
            2.77979690e-02, 3.58633267e-02, 2.22311352e-02, 2.89688460e-02, 2.76739029e-02,
            2.71585279e-02, 3.03377191e-02, 2.88377929e-02, 3.17440868e-02, 2.92178799e-02,
            3.88089979e-02, 2.24786368e-02, 3.48357110e-02, 3.33818104e-02, 3.12533430e-02,
            3.54668699e-02, 3.41875016e-02, 4.07901848e-02, 4.38772867e-02, 4.73304090e-02,
            4.80134491e-02, 1.14686439e+00, 5.33078531e-02, 1.43817519e-02, 4.51967769e-02,
            4.07842951e-02, 2.29329132e-02, 2.99333024e-02, 4.21987818e-02, 3.07293048e-03
        ], [
            4.78231186e-02, 4.43270368e-02, 6.65282418e-02, 8.48291285e-02, 7.46091353e-02,
            6.72922498e-02, 7.39424529e-02, 6.16039512e-02, 6.66422952e-02, 6.70172633e-02,
            7.48109112e-02, 7.02413124e-02, 6.71461886e-02, 6.24836963e-02, 7.12211677e-02,
            8.11380100e-02, 6.15456791e-02, 7.12083219e-02, 5.62464861e-02, 5.79554016e-02,
            6.54846665e-02, 7.00707673e-02, 9.55785921e-02, 6.83357813e-02, 8.12679131e-02,
            8.62160333e-02, 4.80000721e-02, 1.28986318e+00, 1.17705305e-01, 7.88349679e-02,
            5.80156975e-02, 4.03274363e-02, 6.04487229e-02, 8.91736963e-02, 1.81216218e-02
        ], [
            1.11412040e-02, 1.23183436e-02, 1.98000109e-02, 2.18981855e-02, 2.36511406e-02,
            2.11289040e-02, 2.61854873e-02, 1.60076982e-02, 1.88722020e-02, 2.12768701e-02,
            1.90968285e-02, 1.97138496e-02, 2.04768365e-02, 1.84947984e-02, 2.24744366e-02,
            2.16783627e-02, 1.48507465e-02, 2.63405501e-02, 3.33090157e-02, 2.76519331e-02,
            5.10395373e-02, 4.22463572e-02, 2.18926176e-02, 2.77166124e-02, 2.48083844e-02,
            3.79777434e-02, 2.32284525e-02, 3.75010638e-02, 1.04051962e+00, 3.54796729e-02,
            2.56394427e-02, 2.90268776e-02, 4.59201815e-02, 4.04940085e-02, 1.76245333e-03
        ], [
            5.54176205e-02, 5.64759885e-02, 1.13962352e-01, 1.03905657e-01, 1.01723667e-01,
            9.91796353e-02, 1.53269080e-01, 8.49449679e-02, 1.51940061e-01, 1.29346974e-01,
            1.05332997e-01, 1.05861665e-01, 1.22930926e-01, 1.31003940e-01, 1.40841869e-01,
            1.02271282e-01, 8.70155275e-02, 1.33647262e-01, 1.11845782e-01, 9.94897809e-02,
            1.05139685e-01, 1.13506570e-01, 1.07237251e-01, 9.13627058e-02, 1.47351098e-01,
            1.40286067e-01, 1.09482166e-01, 1.83913015e-01, 6.39076130e-02, 1.23275783e+00,
            1.48857937e-01, 7.43825370e-02, 1.28115763e-01, 1.54925392e-01, 1.08489795e-02
        ], [
            3.23523654e-03, 3.45426334e-03, 5.40833640e-03, 4.13897846e-03, 4.20288681e-03,
            6.82183484e-03, 7.95460466e-03, 4.40106937e-03, 6.28859511e-03, 6.03780910e-03,
            6.15318824e-03, 7.67880505e-03, 5.11934618e-03, 4.36175865e-03, 5.20854581e-03,
            5.51193546e-03, 7.05764764e-03, 5.06826070e-03, 4.83499329e-03, 3.03361287e-03,
            4.46714878e-03, 6.55838283e-03, 6.25142506e-03, 8.62766096e-03, 1.38092709e-02,
            6.80914244e-03, 4.51575596e-03, 3.78174398e-03, 4.44221862e-03, 5.33459992e-03,
            1.00994872e+00, 6.24105502e-03, 6.86350125e-03, 1.16981417e-02, 4.15663021e-04
        ], [
            1.21743074e-03, 9.34742566e-04, 1.58361158e-03, 1.54843739e-03, 1.60326517e-03,
            1.48855462e-03, 1.75969712e-03, 1.47441187e-03, 2.08197246e-03, 1.87312196e-03,
            1.59684025e-03, 1.67988219e-03, 2.04028357e-03, 1.83966383e-03, 2.16581973e-03,
            1.46627884e-03, 1.43378052e-03, 1.65218452e-03, 1.64085990e-03, 1.35395802e-03,
            1.88057622e-03, 1.50564142e-03, 1.88428922e-03, 1.31310538e-03, 3.36419346e-03,
            2.23112624e-03, 1.98615832e-03, 2.26181748e-03, 7.46788744e-04, 3.24220858e-03,
            5.22520350e-03, 1.01687110e+00, 2.17215085e-03, 2.65060564e-03, 1.48492086e-04
        ], [
            1.91052355e-03, 8.89385435e-04, 1.63635352e-03, 1.48391258e-03, 2.32075697e-03,
            1.75681394e-03, 1.44083142e-03, 1.00449903e-03, 1.39554056e-03, 1.34395662e-03,
            1.84950593e-03, 1.63444235e-03, 2.24048280e-03, 1.47397018e-03, 1.78133332e-03,
            1.38277643e-03, 1.02380698e-03, 1.31545211e-03, 1.15058399e-03, 8.31160685e-04,
            8.02097169e-04, 1.62756417e-03, 1.22300586e-03, 1.01600444e-03, 1.21062445e-03,
            1.16089525e-03, 9.85578236e-04, 1.07903480e-03, 3.58364795e-04, 1.17034440e-03,
            6.62092261e-03, 2.13085659e-03, 1.03506984e+00, 2.18969307e-03, 1.35631287e-04
        ], [
            1.00686668e-02, 1.17083099e-02, 1.82070873e-02, 1.90789700e-02, 2.10315496e-02,
            1.79461276e-02, 2.89295045e-02, 1.43021953e-02, 2.08949911e-02, 1.94175718e-02,
            1.88293586e-02, 1.89835791e-02, 1.78045294e-02, 1.60649932e-02, 1.98177794e-02,
            1.69705296e-02, 1.50259447e-02, 1.92787977e-02, 1.66344933e-02, 1.53583907e-02,
            1.67149153e-02, 2.59255856e-02, 2.56407103e-02, 1.71407847e-02, 2.15587971e-02,
            2.99532200e-02, 2.57547874e-02, 2.29169789e-02, 1.26107092e-02, 3.97369938e-02,
            2.83759363e-02, 1.87698052e-02, 2.42013302e-02, 1.09347280e+00, 2.85986433e-03
        ], [
            4.04895638e-05, 4.71993336e-05, 7.20084273e-05, 2.08043750e-04, 1.03358389e-04,
            8.35898297e-05, 7.91585223e-05, 6.00524519e-05, 6.30610739e-05, 7.08493614e-05,
            6.74483163e-05, 6.16791325e-05, 8.58808464e-05, 5.89878193e-05, 1.38261001e-04,
            6.57925250e-05, 4.07162921e-05, 9.45149603e-05, 6.49793764e-05, 3.61462622e-05,
            6.47226923e-05, 9.21098548e-05, 1.78163731e-04, 4.64990597e-05, 6.14599494e-05,
            6.33246009e-05, 5.96884892e-05, 1.03204885e-04, 1.86698998e-04, 9.96143461e-05,
            9.97043849e-05, 6.46479979e-05, 8.74744069e-05, 1.40406788e-04, 1.00009630e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 404)
    return ret


@cache('run')
def leontief_matrix_x1995x():
    """
    leontief matrix x1995x



    constant


    """
    
    ret = xr.DataArray(
        data=[[
            1.18047535e+00, 8.35773041e-03, 3.73351437e-01, 7.57840203e-02, 9.28131090e-02,
            2.10652131e-01, 3.02072845e-02, 1.06100688e-02, 2.17755101e-02, 3.03903773e-02,
            1.05098394e-02, 7.87583112e-03, 8.43834730e-03, 8.66736477e-03, 9.39775549e-03,
            4.12123614e-02, 5.47831896e-03, 1.79706275e-02, 6.52210075e-03, 9.48589583e-03,
            8.75995967e-03, 1.09648210e-01, 6.59886817e-03, 1.08023703e-02, 9.67370345e-03,
            9.79247304e-03, 4.44970881e-03, 4.72262254e-03, 3.37763614e-03, 7.45688147e-03,
            1.00516862e-02, 8.57633240e-03, 1.35758412e-02, 1.08985522e-02, 1.98424787e-02
        ], [
            2.32033833e-02, 1.11101577e+00, 2.48720641e-02, 2.69139312e-02, 2.30183022e-02,
            2.82449786e-02, 2.69429477e-02, 5.06060749e-01, 8.09541836e-02, 4.05753956e-02,
            1.27978039e-01, 9.60459711e-02, 3.58116965e-02, 2.91127225e-02, 3.17185000e-02,
            3.63251549e-02, 1.39403729e-01, 5.69431742e-02, 1.61973227e-02, 1.33764672e-02,
            1.14427609e-02, 1.88286435e-02, 4.45460968e-02, 4.32047749e-02, 4.53996269e-02,
            2.46728331e-02, 1.00334744e-02, 7.10996504e-03, 9.21370822e-03, 1.09771950e-02,
            1.64553143e-02, 1.17785460e-02, 1.57490392e-02, 1.92213429e-02, 7.39074825e-02
        ], [
            7.59279909e-02, 5.17925041e-03, 1.18721723e+00, 1.59690530e-02, 1.32222369e-01,
            2.03036923e-02, 1.22640201e-02, 6.75013650e-03, 2.17360933e-02, 1.41384365e-02,
            8.63369745e-03, 7.73498807e-03, 8.33575902e-03, 8.91981203e-03, 8.19588943e-03,
            1.13632058e-02, 5.30536893e-03, 7.97099629e-03, 7.26404524e-03, 1.11672969e-02,
            9.47703403e-03, 2.03503804e-01, 7.11989254e-03, 1.37394763e-02, 1.49823673e-02,
            1.10335521e-02, 5.33528661e-03, 6.05800699e-03, 2.36238083e-03, 9.13621706e-03,
            1.47701047e-02, 1.37615076e-02, 2.32589480e-02, 1.22054279e-02, 3.14376112e-03
        ], [
            6.65890399e-03, 4.91987921e-03, 7.57248636e-03, 1.39958000e+00, 9.51712374e-02,
            1.42176333e-02, 1.71936313e-02, 4.69153485e-03, 1.14910482e-02, 3.43465419e-02,
            1.15628574e-02, 7.37726747e-03, 9.07702632e-03, 9.22311364e-03, 1.79263281e-02,
            4.28891046e-02, 3.61234135e-03, 8.83776612e-03, 6.91965532e-03, 6.97426579e-03,
            7.39731609e-03, 8.33457098e-03, 5.60561818e-03, 8.23482377e-03, 5.63581937e-03,
            5.76574765e-03, 6.50612123e-03, 3.05171151e-03, 1.62721955e-03, 4.55633822e-03,
            7.65334885e-03, 2.65051337e-03, 8.02117270e-03, 1.03072999e-02, 1.44681899e-03
        ], [
            6.71835288e-04, 7.47348868e-04, 9.01035830e-04, 1.14965709e-02, 1.23053574e+00,
            2.01564608e-03, 2.83203186e-03, 6.90238230e-04, 1.01444268e-03, 2.12113283e-03,
            1.37506967e-03, 1.22208018e-03, 1.29966123e-03, 1.29534693e-03, 2.88583488e-03,
            5.79207288e-03, 6.01634082e-04, 9.13729108e-04, 1.13535540e-03, 1.02783679e-03,
            1.00683039e-03, 7.71192618e-04, 6.89618068e-04, 8.24953271e-04, 7.19701316e-04,
            6.42070905e-04, 1.14535296e-03, 3.65053934e-04, 1.81426706e-04, 5.57517973e-04,
            1.52699540e-03, 3.87319554e-04, 5.81145210e-04, 1.34918300e-03, 1.21945173e-04
        ], [
            4.38264258e-03, 5.11262425e-03, 6.11220909e-03, 6.39489170e-03, 7.67764656e-03,
            1.23528213e+00, 2.14372507e-02, 4.33529849e-03, 6.17162127e-03, 7.95177676e-03,
            1.29115169e-02, 8.51718329e-03, 8.64667165e-03, 7.39617803e-03, 1.04492888e-02,
            9.74005388e-02, 4.52308929e-03, 4.88282410e-02, 4.42833512e-03, 4.84718213e-03,
            4.33948765e-03, 6.50606892e-03, 4.81652027e-03, 4.84014622e-03, 4.20416568e-03,
            1.00055623e-02, 3.37112626e-03, 3.01313318e-03, 5.70384009e-03, 3.74769212e-03,
            5.10400086e-03, 3.57641382e-03, 4.01027201e-03, 7.35033290e-03, 1.41004466e-03
        ], [
            1.65127169e-02, 1.57189890e-02, 5.30117402e-02, 3.73223823e-02, 4.38256755e-02,
            3.66334681e-02, 1.33412027e+00, 1.71332451e-02, 5.08281713e-02, 5.03737650e-02,
            4.80499098e-02, 2.68219660e-02, 3.47137739e-02, 4.58527001e-02, 3.18779672e-02,
            5.10637515e-02, 1.57298970e-02, 2.63761919e-02, 2.45740608e-02, 3.30634901e-02,
            3.03857309e-02, 3.38122611e-02, 2.10001227e-02, 2.45088099e-02, 2.88485097e-02,
            3.78446973e-02, 2.69607542e-02, 3.46877184e-02, 9.73673031e-03, 4.28915124e-02,
            3.29881339e-02, 2.67176202e-02, 2.63104929e-02, 4.73338758e-02, 2.99935309e-03
        ], [
            2.32256460e-02, 2.37360355e-02, 1.91493534e-02, 2.02848437e-02, 1.81132063e-02,
            2.01548954e-02, 1.67228855e-02, 1.06579011e+00, 4.53839581e-02, 2.78879958e-02,
            3.13842327e-02, 2.46784969e-02, 1.64609056e-02, 1.54612689e-02, 1.54646409e-02,
            1.81892546e-02, 3.27919098e-02, 2.54944245e-02, 1.21048410e-02, 1.32239102e-02,
            1.08162368e-02, 1.42238026e-02, 6.29302071e-02, 6.84205256e-02, 7.51086488e-02,
            2.63255171e-02, 9.15307296e-03, 5.72698292e-03, 4.35361112e-03, 1.02747756e-02,
            1.38264676e-02, 7.13843261e-03, 1.11198926e-02, 1.39530247e-02, 2.36903669e-03
        ], [
            7.35229860e-02, 3.14672398e-02, 5.28462251e-02, 1.45956070e-01, 9.53981415e-02,
            6.84319314e-02, 7.70950313e-02, 4.04670994e-02, 1.32164841e+00, 3.16636982e-01,
            6.13053822e-02, 4.57709351e-02, 4.81955274e-02, 5.92686305e-02, 5.87968297e-02,
            8.16230086e-02, 2.00775598e-02, 3.93106250e-02, 2.94764125e-02, 1.45822545e-02,
            1.25788549e-02, 2.58585172e-02, 1.87394501e-02, 1.85787492e-02, 1.75460061e-02,
            1.77226283e-02, 1.11184825e-02, 7.91777846e-03, 6.87913702e-03, 1.55580068e-02,
            2.17087374e-02, 1.07158084e-02, 9.32437345e-02, 2.44046729e-02, 4.28530450e-03
        ], [
            1.24586416e-02, 1.61585763e-02, 2.80355448e-02, 2.29526299e-02, 5.59543441e-02,
            2.28171757e-02, 2.80744266e-02, 1.33978427e-02, 3.42836103e-02, 1.13064836e+00,
            2.20207094e-02, 2.12115492e-02, 4.58433132e-02, 4.77458134e-02, 6.82295162e-02,
            5.84810651e-02, 9.87418787e-03, 3.13259709e-02, 3.43562879e-02, 1.07396353e-02,
            1.05447073e-02, 1.41085781e-02, 2.00204887e-02, 1.22617809e-02, 1.23619707e-02,
            1.44419000e-02, 9.41957159e-03, 4.48059889e-03, 4.04233206e-03, 7.90172892e-03,
            9.91136703e-03, 4.38431651e-03, 1.27770877e-02, 1.05065759e-02, 1.90332651e-03
        ], [
            5.89987586e-03, 1.01967328e-02, 1.19271779e-02, 7.48164942e-03, 7.53392815e-03,
            1.70051653e-02, 6.33597903e-03, 9.15387800e-03, 1.39277578e-02, 1.40068011e-02,
            1.12443736e+00, 1.94891876e-02, 1.51147088e-02, 2.32917893e-02, 1.84244680e-02,
            1.53824031e-02, 8.32225539e-03, 9.78689929e-02, 8.33557271e-03, 4.71012002e-03,
            3.88225801e-03, 8.74485654e-03, 6.11050478e-03, 5.36935478e-03, 4.96363075e-03,
            6.38905433e-03, 4.72809003e-03, 2.67544568e-03, 7.61271600e-03, 3.93493990e-03,
            6.22218851e-03, 4.47449815e-03, 6.06838446e-03, 6.52293824e-03, 1.30522811e-03
        ], [
            2.47480060e-02, 6.01033488e-02, 4.73096959e-02, 3.47772329e-02, 4.55979356e-02,
            7.24269356e-02, 3.93438165e-02, 4.73407141e-02, 5.81049464e-02, 7.01145337e-02,
            7.92725066e-02, 1.49859679e+00, 3.06590639e-01, 1.80189631e-01, 2.49982843e-01,
            1.99845226e-01, 3.57854948e-02, 1.60800482e-01, 7.22086765e-02, 2.18490371e-02,
            1.85367891e-02, 2.68104121e-02, 3.54555861e-02, 4.51250546e-02, 4.04694656e-02,
            3.03967575e-02, 2.12903741e-02, 1.04198389e-02, 1.53332280e-02, 1.82906551e-02,
            2.79383639e-02, 1.27001818e-02, 2.01712312e-02, 2.42584211e-02, 6.68350124e-03
        ], [
            1.44186714e-02, 3.01064456e-02, 1.47976589e-02, 1.78362871e-02, 1.64070077e-02,
            2.14510891e-02, 1.78228612e-02, 2.28234072e-02, 2.01313788e-02, 2.39595468e-02,
            2.55756522e-02, 3.53217517e-02, 1.14746210e+00, 3.40957090e-02, 5.91580274e-02,
            2.35292235e-02, 1.72693533e-02, 3.03364345e-02, 3.51915513e-02, 8.96686273e-03,
            7.79131306e-03, 9.41973195e-03, 1.48911606e-02, 1.50785060e-02, 1.35025676e-02,
            1.16072162e-02, 7.78064750e-03, 4.62275757e-03, 4.56563061e-03, 7.84059733e-03,
            1.26463212e-02, 5.61892561e-03, 7.46959562e-03, 9.44980107e-03, 2.76257009e-03
        ], [
            1.07363330e-02, 2.13214324e-02, 1.69728159e-02, 2.08729912e-02, 2.00846785e-02,
            2.26780093e-02, 2.74865879e-02, 1.98861755e-02, 2.81200568e-02, 3.35394824e-02,
            2.62642223e-02, 4.21084076e-02, 1.21143536e-01, 1.35091737e+00, 1.09810355e-01,
            4.36486506e-02, 2.79737125e-02, 5.06095835e-02, 5.11029437e-02, 2.01228599e-02,
            1.77349347e-02, 1.54238737e-02, 2.05445015e-02, 2.01946345e-02, 2.11265104e-02,
            1.97742798e-02, 4.42828685e-02, 1.15661737e-02, 7.24477585e-03, 2.44848690e-02,
            2.52746398e-02, 1.21732228e-02, 3.13788235e-02, 1.96334053e-02, 3.10494245e-03
        ], [
            1.15767859e-02, 1.54796478e-02, 1.28616637e-02, 1.33595598e-02, 1.40907462e-02,
            1.48811583e-02, 1.12499476e-02, 1.42920322e-02, 1.26809895e-02, 1.44941424e-02,
            1.55220593e-02, 1.91327968e-02, 3.14711606e-02, 1.79213531e-02, 1.41701749e+00,
            1.68166479e-02, 1.01857706e-02, 1.54761538e-02, 1.32429653e-01, 1.37335943e-02,
            1.08367410e-02, 8.88461870e-03, 4.75807029e-02, 6.46565059e-02, 7.59149603e-02,
            2.32539022e-02, 9.32190849e-03, 5.28031705e-03, 3.16279128e-03, 9.63198628e-03,
            3.07491068e-02, 4.64324608e-03, 6.79292377e-03, 1.09417587e-02, 1.68992774e-03
        ], [
            2.53174709e-03, 3.21243658e-03, 3.71884036e-03, 1.04179804e-02, 7.60398886e-03,
            1.02291750e-02, 6.44641061e-03, 3.16912543e-03, 4.95535956e-03, 6.04310846e-03,
            6.32211921e-03, 1.26766407e-02, 8.89611229e-03, 7.56101180e-03, 1.14131493e-02,
            1.03988970e+00, 2.81309430e-03, 9.09088082e-03, 5.04572537e-03, 3.38687599e-03,
            3.26024134e-03, 5.27376844e-03, 3.33854066e-03, 3.73725406e-03, 3.75416573e-03,
            3.27905143e-03, 2.43354397e-03, 2.69938356e-03, 2.06842144e-03, 4.32837062e-03,
            5.81087140e-03, 4.47455627e-03, 4.72673265e-03, 8.36820400e-03, 4.61519640e-03
        ], [
            2.16696159e-02, 3.76591070e-02, 3.14120212e-02, 4.12443542e-02, 2.95642728e-02,
            3.71984550e-02, 4.30204001e-02, 4.04112135e-02, 5.54606779e-02, 4.94281160e-02,
            6.18794875e-02, 5.68365584e-02, 3.53106068e-02, 3.26421596e-02, 3.53314867e-02,
            3.40556221e-02, 1.11332954e+00, 2.63269770e-02, 2.34727853e-02, 1.59609193e-02,
            2.25054244e-02, 3.90755293e-02, 2.88835466e-02, 1.98099439e-02, 1.86230525e-02,
            2.62402835e-02, 1.65574573e-02, 1.20169457e-02, 1.35358693e-02, 1.54145492e-02,
            2.47690251e-02, 2.54065765e-02, 2.47597101e-02, 2.85696674e-02, 3.61488031e-03
        ], [
            9.25820241e-03, 1.49834963e-02, 1.13694883e-02, 1.14349454e-02, 1.11341019e-02,
            1.38177504e-02, 1.31535963e-02, 1.46236039e-02, 1.45264143e-02, 1.36042652e-02,
            1.74682902e-02, 1.55596425e-02, 1.28308423e-02, 1.28555760e-02, 1.22339888e-02,
            1.27705344e-02, 3.62646263e-02, 1.04631173e+00, 1.17485932e-02, 1.11605056e-02,
            1.16614439e-02, 1.42853435e-02, 1.71486403e-02, 1.45861449e-02, 1.19256437e-02,
            2.32783499e-02, 1.85983184e-02, 1.21199160e-02, 4.93882078e-02, 1.21917886e-02,
            2.74621869e-02, 1.80514298e-02, 1.35539775e-02, 2.15987512e-02, 4.67811194e-03
        ], [
            9.91077198e-03, 9.01685545e-03, 1.44229760e-02, 1.31957221e-02, 1.64185424e-02,
            1.39168995e-02, 1.13311239e-02, 1.11760189e-02, 1.38605607e-02, 1.42065992e-02,
            1.46075674e-02, 1.43036186e-02, 1.29039705e-02, 1.27818102e-02, 1.38631330e-02,
            1.38594895e-02, 1.48895611e-02, 1.48725087e-02, 1.01289409e+00, 9.34051784e-03,
            8.31926309e-03, 1.12186620e-02, 2.36040630e-02, 1.00565687e-02, 8.75132380e-03,
            1.26403875e-02, 8.06497345e-03, 5.55172451e-03, 3.86848838e-03, 9.59395958e-03,
            7.58661324e-03, 5.36903031e-03, 7.30341016e-03, 1.07547199e-02, 1.02529661e-03
        ], [
            5.87852180e-02, 4.69900709e-02, 1.04976547e-01, 1.04740222e-01, 1.29071853e-01,
            1.00587825e-01, 8.97782188e-02, 8.06317122e-02, 9.16765519e-02, 9.68825796e-02,
            7.97043932e-02, 8.97572744e-02, 9.46941579e-02, 1.02906934e-01, 1.00633769e-01,
            9.52971690e-02, 4.41520129e-02, 8.13220045e-02, 6.52001127e-02, 1.04980900e+00,
            2.80195703e-02, 7.77285851e-02, 4.70645693e-02, 4.75444914e-02, 4.64827141e-02,
            4.50642412e-02, 2.38819288e-02, 1.77776023e-02, 1.23896368e-02, 2.81444800e-02,
            3.75358695e-02, 2.02667437e-02, 4.77078367e-02, 3.46368743e-02, 5.64516734e-03
        ], [
            3.01410017e-02, 2.04180345e-02, 5.23180891e-02, 5.10115137e-02, 6.98214932e-02,
            4.22108401e-02, 3.66363652e-02, 3.15411516e-02, 4.26606740e-02, 4.44927024e-02,
            3.59087598e-02, 3.63353630e-02, 3.73684501e-02, 3.80820524e-02, 4.37807893e-02,
            4.69202256e-02, 1.92785546e-02, 4.48021209e-02, 3.01121292e-02, 1.79542547e-02,
            1.01610384e+00, 4.30294637e-02, 2.68262267e-02, 2.35381214e-02, 2.21901017e-02,
            2.10575417e-02, 1.74924365e-02, 1.08890922e-02, 8.10180842e-03, 1.79305769e-02,
            1.68905513e-02, 1.11775748e-02, 2.16953076e-02, 2.26194353e-02, 3.10200555e-03
        ], [
            6.03207389e-03, 8.91043511e-03, 1.17638998e-02, 1.31104346e-02, 1.33737519e-02,
            1.30809519e-02, 1.45356877e-02, 1.00305739e-02, 1.45422860e-02, 1.50620115e-02,
            1.39847857e-02, 1.43873671e-02, 1.56723628e-02, 1.69321666e-02, 1.34770132e-02,
            1.34428248e-02, 1.06410030e-02, 1.34761124e-02, 1.30406786e-02, 1.54218800e-02,
            1.09698808e-02, 1.01609399e+00, 1.41600276e-02, 1.71114279e-02, 3.34802636e-02,
            2.78870022e-02, 9.68508791e-03, 1.60856829e-02, 5.25946314e-03, 1.90923909e-02,
            1.38110197e-02, 9.01210255e-03, 1.34717409e-02, 1.69959224e-02, 1.06877245e-03
        ], [
            2.98023675e-02, 4.35309050e-02, 4.64968821e-02, 4.29120806e-02, 4.90574045e-02,
            5.02898681e-02, 4.02632946e-02, 7.47113610e-02, 4.52835687e-02, 4.23901402e-02,
            5.99643291e-02, 4.46108183e-02, 3.52846429e-02, 3.23895151e-02, 3.81265662e-02,
            4.26906895e-02, 4.40532136e-02, 4.08798896e-02, 2.71553695e-02, 3.22691664e-02,
            2.01053628e-02, 2.82838270e-02, 1.05794868e+00, 4.01903249e-02, 3.16865337e-02,
            4.80134698e-02, 2.00469983e-02, 1.47837213e-02, 7.23707258e-03, 1.64763640e-02,
            2.21960007e-02, 1.62701773e-02, 1.91028116e-02, 2.26444666e-02, 4.72039546e-03
        ], [
            5.32189104e-03, 7.13662645e-03, 7.48851966e-03, 6.63054130e-03, 7.99909661e-03,
            1.14925474e-02, 4.55729774e-03, 1.11298441e-02, 7.08161543e-03, 6.03458009e-03,
            9.69447440e-03, 8.96043715e-03, 5.52877980e-03, 4.86889749e-03, 5.97532290e-03,
            6.02735762e-03, 5.15229318e-03, 6.35770385e-03, 3.25921614e-03, 4.85075855e-03,
            2.80587042e-03, 4.27854813e-03, 6.14169370e-03, 1.11234450e+00, 6.18380693e-03,
            8.03542012e-03, 2.94397894e-03, 1.80599436e-03, 9.36031928e-04, 2.32863333e-03,
            2.85920219e-03, 2.16866160e-03, 2.27027872e-03, 2.74444337e-03, 7.19744709e-04
        ], [
            2.69400994e-03, 4.37646958e-03, 4.66593917e-03, 5.21980684e-03, 6.92716352e-03,
            4.54787283e-03, 5.65122156e-03, 4.58982211e-03, 5.56150738e-03, 5.47096849e-03,
            4.60909813e-03, 4.90257427e-03, 5.14420075e-03, 5.89963472e-03, 5.69932112e-03,
            4.95640867e-03, 3.01527545e-03, 4.08251487e-03, 4.42605019e-03, 6.93136380e-03,
            3.51346637e-03, 3.76488781e-03, 4.89408551e-03, 1.04821572e-02, 1.03591307e+00,
            2.13947947e-02, 7.39629621e-03, 5.40276746e-03, 1.46098903e-03, 6.59086251e-03,
            6.34923681e-03, 3.30903269e-03, 3.26160327e-03, 6.37584872e-03, 4.53901847e-04
        ], [
            1.18733730e-02, 1.78727789e-02, 2.00247429e-02, 1.70534682e-02, 2.02929665e-02,
            2.06843066e-02, 1.98171149e-02, 2.31821546e-02, 1.92203979e-02, 1.87454496e-02,
            2.39338332e-02, 1.99276439e-02, 1.77257926e-02, 1.71559595e-02, 1.85851512e-02,
            1.77826229e-02, 1.42182986e-02, 1.55308739e-02, 1.62157407e-02, 3.05650360e-02,
            1.60389652e-02, 1.80024706e-02, 4.91490311e-02, 1.79173802e-01, 1.21340557e-01,
            1.12926546e+00, 9.41718800e-03, 9.30383120e-03, 3.44213045e-03, 1.11235961e-02,
            9.78021144e-03, 6.44053028e-03, 9.46130638e-03, 1.30983082e-02, 1.77025343e-03
        ], [
            9.08424894e-03, 1.32642411e-02, 1.53016450e-02, 1.63454993e-02, 1.75292314e-02,
            1.55021533e-02, 2.37660273e-02, 1.42560571e-02, 1.91787183e-02, 1.81562769e-02,
            1.59542403e-02, 1.69840316e-02, 1.87381014e-02, 1.98740054e-02, 1.70843935e-02,
            1.86517799e-02, 1.29885961e-02, 1.79779080e-02, 1.97799384e-02, 2.76092077e-02,
            2.54318404e-02, 2.10411320e-02, 2.32241655e-02, 3.52498032e-02, 4.42472446e-02,
            3.19622983e-02, 1.08276508e+00, 3.21236552e-02, 8.03197038e-03, 2.90293907e-02,
            2.40900204e-02, 1.16362248e-02, 1.78433967e-02, 2.91925255e-02, 1.62287304e-03
        ], [
            4.77891210e-02, 6.40274131e-02, 5.96816698e-02, 6.93529064e-02, 6.81434279e-02,
            6.49707903e-02, 6.32158026e-02, 6.31083213e-02, 6.67297219e-02, 6.42759639e-02,
            6.38381936e-02, 6.75641942e-02, 6.42224089e-02, 6.30699620e-02, 6.19210769e-02,
            6.64413270e-02, 5.41998658e-02, 5.93106625e-02, 5.75982461e-02, 7.65063829e-02,
            6.13969088e-02, 5.89105749e-02, 7.45764773e-02, 8.82854526e-02, 7.90786822e-02,
            6.28562484e-02, 4.78447015e-02, 1.21273666e+00, 6.73137902e-02, 6.38855531e-02,
            4.81749791e-02, 2.81561231e-02, 4.76344260e-02, 6.35438185e-02, 1.10987308e-02
        ], [
            1.83835290e-02, 1.68339135e-02, 2.30608946e-02, 2.45854394e-02, 2.79328126e-02,
            2.32251868e-02, 2.68857188e-02, 1.88029296e-02, 2.32505067e-02, 2.48832982e-02,
            2.20066772e-02, 2.32720836e-02, 2.44910495e-02, 2.47753789e-02, 2.34199953e-02,
            2.73405439e-02, 1.81728770e-02, 2.94003685e-02, 3.86789853e-02, 4.63646630e-02,
            5.55233998e-02, 4.86551035e-02, 2.51884956e-02, 4.99710338e-02, 3.57847020e-02,
            3.83742328e-02, 2.35095134e-02, 4.06827142e-02, 1.03690442e+00, 3.66421734e-02,
            2.18972364e-02, 2.49134196e-02, 4.22047375e-02, 4.19248404e-02, 2.77798766e-03
        ], [
            4.75183071e-02, 6.90212706e-02, 9.30110997e-02, 9.25801194e-02, 9.41652839e-02,
            7.75485523e-02, 1.23828898e-01, 7.35173775e-02, 1.31899514e-01, 1.06762412e-01,
            8.85258885e-02, 9.01614805e-02, 1.08156353e-01, 1.33446341e-01, 1.14168169e-01,
            1.00047438e-01, 7.81111854e-02, 1.09478705e-01, 9.05049693e-02, 1.02795777e-01,
            9.87580230e-02, 8.17276730e-02, 8.27431000e-02, 1.20504253e-01, 1.56412069e-01,
            1.15119096e-01, 1.00169893e-01, 1.29374552e-01, 5.50203625e-02, 1.16894586e+00,
            9.12638241e-02, 4.77923722e-02, 8.87311652e-02, 1.08394399e-01, 7.64094059e-03
        ], [
            5.30590828e-03, 6.74168603e-03, 6.98114403e-03, 6.36907058e-03, 6.00281728e-03,
            1.06815827e-02, 9.09250124e-03, 6.75586161e-03, 9.15910377e-03, 7.91086306e-03,
            7.47037651e-03, 8.15026785e-03, 6.85342848e-03, 6.84527294e-03, 7.10706148e-03,
            8.20575953e-03, 1.11344715e-02, 6.29164682e-03, 6.39119388e-03, 5.86721423e-03,
            7.21627383e-03, 7.75319723e-03, 6.95067719e-03, 1.53078580e-02, 1.59691794e-02,
            8.50009373e-03, 5.23233582e-03, 4.89021760e-03, 5.92755198e-03, 7.68617115e-03,
            1.01153733e+00, 5.60065679e-03, 8.55343430e-03, 1.42622098e-02, 6.75848654e-04
        ], [
            1.50596807e-03, 1.10248973e-03, 1.43201987e-03, 1.33891049e-03, 1.43887322e-03,
            1.23683407e-03, 1.60115552e-03, 1.29788211e-03, 1.80725872e-03, 1.48769111e-03,
            1.37372510e-03, 1.44458904e-03, 1.79599344e-03, 2.14422335e-03, 1.54722096e-03,
            1.28751492e-03, 1.41428453e-03, 1.33466080e-03, 1.10442574e-03, 1.63235709e-03,
            2.51743916e-03, 1.20007167e-03, 1.61469698e-03, 1.31166388e-03, 2.60735534e-03,
            1.72512634e-03, 2.68498683e-03, 1.78916544e-03, 4.88421699e-04, 2.65555016e-03,
            5.00923819e-03, 1.00581957e+00, 1.56166356e-03, 2.00500030e-03, 1.27525204e-04
        ], [
            2.56972884e-03, 8.49572533e-04, 1.59119151e-03, 1.21511897e-03, 1.25248322e-03,
            1.33452373e-03, 1.07233427e-03, 8.49078357e-04, 1.13463892e-03, 1.10088267e-03,
            1.03919618e-03, 1.02767141e-03, 1.15912511e-03, 1.17002667e-03, 1.00466506e-03,
            1.16422106e-03, 8.05848958e-04, 1.09156134e-03, 9.26733545e-04, 1.05435381e-03,
            8.06307070e-04, 1.67561769e-03, 1.13176743e-03, 1.56569496e-03, 1.15892864e-03,
            1.10941582e-03, 8.58015257e-04, 9.41899049e-04, 3.13803806e-04, 1.06114288e-03,
            6.71021406e-03, 1.66094218e-03, 1.02123300e+00, 1.96737608e-03, 1.17971112e-04
        ], [
            1.05020459e-02, 1.52878883e-02, 1.99030214e-02, 1.91889248e-02, 2.16702420e-02,
            1.90219963e-02, 2.93820193e-02, 1.63419984e-02, 2.44370881e-02, 2.24434302e-02,
            1.85976007e-02, 2.20951836e-02, 2.12859640e-02, 2.30647661e-02, 2.03577627e-02,
            2.13835052e-02, 1.44944551e-02, 1.88319954e-02, 1.98105920e-02, 1.95727156e-02,
            2.00823614e-02, 2.48016522e-02, 2.03785544e-02, 2.23218479e-02, 2.40853641e-02,
            2.36258151e-02, 2.76844157e-02, 2.16382274e-02, 1.26585224e-02, 4.17495846e-02,
            2.63453713e-02, 1.89270060e-02, 2.55225284e-02, 1.08612722e+00, 2.23051979e-03
        ], [
            5.22319300e-05, 6.10758103e-05, 9.99656982e-05, 1.15348340e-04, 1.07210683e-04,
            1.04563337e-04, 1.13696035e-04, 7.67287665e-05, 1.01159198e-04, 1.08246490e-04,
            8.77381585e-05, 1.09008406e-04, 1.03844219e-04, 9.32394370e-05, 1.32965273e-04,
            1.13429182e-04, 6.40546165e-05, 1.57512103e-04, 2.14461103e-04, 1.10296425e-04,
            1.23227418e-04, 1.37986484e-04, 2.30563938e-04, 1.01137752e-04, 1.10740065e-04,
            9.61642076e-05, 1.37519465e-04, 1.22094045e-04, 3.23264127e-04, 1.29800422e-04,
            1.45869651e-04, 8.52453077e-05, 1.77231453e-04, 1.88710579e-04, 1.00003545e+00
        ]],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ],
            'sectors1': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors', 'sectors1'])
    ret = utils.align_by_dims(ret, ('sectors', 'sectors1'))
    #test_nans(ret, 405)
    return ret


@cache('step')
def replacement_batteries():
    """
    replacement batteries



    component


    """
    
    ret = discarded_batteries()
    #test_nans(ret, 406)
    return ret


@cache('step')
def share_of_electric_light_vehicles():
    """
    share of electric light vehicles



    component


    """
    
    ret = total_number_elec_light_vehicles() / total_number_light_vehicles()
    #test_nans(ret, 407)
    return ret


@cache('step')
def share_elecxplusxhyb_light_vehicles():
    """
    share elecxplusxhyb light vehicles



    component


    """
    
    ret = (total_number_hybrid_light_vehicles() + total_number_elec_light_vehicles()
            ) / total_number_light_vehicles()
    #test_nans(ret, 408)
    return ret


@cache('step')
def aux_p_inland_transp():
    """
    aux p inland transp



    component


    """
    
    ret = functions.if_then_else(
        time() < t_fin_inlandt(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_inlandt() - 1) < equality_threshold,time() > t_ini_inlandt()),
                               (p_inlandt() - initial_percent_t_vehicles()) /
                               (t_fin_inlandt() - t_ini_inlandt()), hist_var_inlandt()), 0)
    #test_nans(ret, 409)
    return ret


@cache('run')
def t_fin_inlandt():
    """
    t fin inlandt



    constant


    """
    
    ret = 2050
    #test_nans(ret, 410)
    return ret


@cache('step')
def h_elec_initial_growth():
    """
    h elec initial growth



    component


    """
    
    ret = functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold,time() > t_ini_h_veh()),
                               (p_h_vehicle().loc['elec 4wheels'].values - percent_h_vehicles_initial().loc['elec 4wheels'].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc['elec 4wheels'].values), 0)
    #test_nans(ret, 411)
    return ret


@cache('run')
def t_fin_h_veh():
    """
    t fin h veh



    constant


    """
    
    ret = 2050
    #test_nans(ret, 412)
    return ret


@cache('step')
def h_gas_initial_growth():
    """
    h gas initial growth



    component


    """
    
    ret = functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold,time() > t_ini_h_veh()),
                               (p_h_vehicle().loc['gas 4wheels'].values - percent_h_vehicles_initial().loc['gas 4wheels'].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc['gas 4wheels'].values), 0)
    #test_nans(ret, 413)
    return ret


@cache('run')
def t_ini_h_veh():
    """
    t ini h veh



    constant


    """
    
    ret = 2020
    #test_nans(ret, 414)
    return ret


@cache('step')
def h_hyb_initial_growth():
    """
    h hyb initial growth



    component


    """
    
    ret = functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold,time() > t_ini_h_veh()),
                               (p_h_vehicle().loc['hib 4wheels'].values - percent_h_vehicles_initial().loc['hib 4wheels'].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc['hib 4wheels'].values), 0)
    #test_nans(ret, 415)
    return ret


@cache('step')
def policy_2wheels():
    """
    policy 2wheels



    component


    """
    
    ret = functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold,time() > t_ini_h_veh()),
                               (p_share_x2x_wheelers() - initial_2w_percent()) /
                               (t_fin_h_veh() - t_ini_h_veh()), 0), 0)
    #test_nans(ret, 416)
    return ret


@cache('step')
def h_2w_initial_growth():
    """
    h 2w initial growth



    component


    """
    
    ret = functions.if_then_else(
        time() < t_fin_h_veh(),
        functions.if_then_else(functions.logical_and(abs(activate_policy_h_transp() - 1) < equality_threshold,time() > t_ini_h_veh()),
                               (p_h_vehicle().loc['elec 2wheels'].values - percent_h_vehicles_initial().loc['elec 2wheels'].values) /
                               (t_fin_h_veh() - t_ini_h_veh()), aux_hist_h().loc['elec 2wheels'].values), 0)
    #test_nans(ret, 417)
    return ret


@cache('step')
def var_percents_h_vehicles():
    """
    var percents h vehicles



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(
            time() < t_ini_h_veh(),
            aux_hist_h().loc['liq 4wheels'].values,
            -h_ev_adapt_growth() - h_hyb_adapt_growth() - h_gas_adapt_growth() - rate_4w_to_2w()),
        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc['hib 4wheels'].values, h_hyb_adapt_growth()),
        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc['elec 4wheels'].values, h_ev_adapt_growth()),
        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc['gas 4wheels'].values, h_gas_adapt_growth()),
        functions.if_then_else(time() < t_ini_h_veh(),
                               aux_hist_h().loc['liq 2wheels'].values, -h_2we_adapt_growth() + rate_4w_to_2w()),
        functions.if_then_else(time() < t_ini_h_veh(), aux_hist_h().loc['elec 2wheels'].values, h_2we_adapt_growth())],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 418)
    return ret


@cache('step')
def increase_households_energy_final_demand_for_transp():
    """
    increase households energy final demand for transp



    component


    """
    
    ret = (energy_intensity_of_households_transport() -
            initial_energy_intensity_of_households_transport_x2009x()
            ) * household_demand_total() / 1e+006
    #test_nans(ret, 419)
    return ret


@cache('run')
def start_year_p_common_rr_minerals_alt_techn():
    """
    start year p common rr minerals alt techn



    constant


    """
    
    ret = 2020
    #test_nans(ret, 420)
    return ret


@cache('run')
def start_year_p_common_rr_minerals_rest():
    """
    start year p common rr minerals rest



    constant


    """
    
    ret = 2020
    #test_nans(ret, 421)
    return ret


@cache('run')
def select_pop_ssps():
    """
    select pop ssps



    constant


    """
    
    ret = 2
    #test_nans(ret, 422)
    return ret


@cache('step')
def dynamic_low_range_fec_good_standard_of_living():
    """
    dynamic low range fec good standard of living



    component


    """
    
    ret = low_range_fec_good_standard_of_living() * (
        1 + cumulative_tfec_intensity_change_from_x2009x())
    #test_nans(ret, 423)
    return ret


@cache('step')
def cp_ev_batteries_for_elec_storage():
    """
    cp ev batteries for elec storage



    component


    """
    
    ret = np.minimum(cp_ev_batteries_required(), max_cp_ev_batteries_for_elec_storage())
    #test_nans(ret, 424)
    return ret


@cache('step')
def dynamic_threshold_xhigh_developmentx():
    """
    dynamic threshold xhigh developmentx



    component


    """
    
    ret = threshold_fec_xhigh_developmentx() * (
        1 + cumulative_tfec_intensity_change_from_x2009x())
    #test_nans(ret, 425)
    return ret


@cache('run')
def max_cp_ev_batteries_for_elec_storage():
    """
    max cp ev batteries for elec storage



    component


    """
    
    ret = cp_ev_batteries_for_transp()
    #test_nans(ret, 426)
    return ret


@cache('step')
def tfec_intensity_until_x2009x():
    """
    tfec intensity until x2009x



    component


    """
    
    ret = functions.if_then_else(time() < 2009, tfes_intensity_ej_tdollar(), aux15())
    #test_nans(ret, 427)
    return ret


@cache('step')
def desired_annual_gdp_growth_rate():
    """
    desired annual gdp growth rate



    component


    """
    
    ret = -1 + (desired_gdp() / desired_gdp_delayed_1yr())
    #test_nans(ret, 428)
    return ret


@cache('step')
def cumulative_tfec_intensity_change_from_x2009x():
    """
    cumulative tfec intensity change from x2009x



    component


    """
    
    ret = functions.if_then_else(time() < 2009, 0, -1 +
                                  (tfes_intensity_ej_tdollar() / tfec_intensity_until_x2009x()))
    #test_nans(ret, 429)
    return ret


@cache('step')
def lifetime_ev_batteries():
    """
    lifetime ev batteries



    component


    """
    
    ret = functions.zidz(net_stored_energy_ev_battery_over_lifetime(),
                          ((cp_ev_batteries_for_elec_storage() + cp_ev_batteries_for_transp()) *
                           mw_in_x1x_year_to_mj() * (kw_per_battery_ev() / kw_per_mw())))
    #test_nans(ret, 430)
    return ret


@cache('step')
def dynamic_high_range_fec_good_standard_of_living():
    """
    dynamic high range fec good standard of living



    component


    """
    
    ret = high_range_fec_good_standard_of_living() * (
        1 + cumulative_tfec_intensity_change_from_x2009x())
    #test_nans(ret, 431)
    return ret


@cache('step')
def tfec_per_capita():
    """
    tfec per capita



    component


    """
    
    ret = functions.zidz(real_tfec() * gj_per_ej(), population())
    #test_nans(ret, 432)
    return ret


@cache('step')
def aux15():
    """
    aux15



    component


    """
    
    ret = delay_tfec_intensity_until_x2009x_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 433)
    return ret


@cache('run')
def cp_ev_batteries_for_transp():
    """
    cp ev batteries for transp



    constant


    """
    
    ret = 0.0055
    #test_nans(ret, 434)
    return ret


@cache('run')
def p_customized_cte_gdppc_variation():
    """
    p customized cte gdppc variation



    constant


    """
    
    ret = 0.02
    #test_nans(ret, 435)
    return ret


@cache('step')
def annual_growth_rate_electricity_generation_res_elec_tot():
    """
    annual growth rate electricity generation res elec tot



    component


    """
    
    ret = -1 + fe_tot_generation_all_res_elec_twh(
    ) / fe_tot_generation_all_res_elec_twh_delayed_1yr()
    #test_nans(ret, 436)
    return ret


@cache('step')
def annual_growth_rate_res_for_heat():
    """
    annual growth rate res for heat



    component


    """
    
    ret = -1 + fes_res_for_heat_ej() / fes_res_for_heat_delayed_1yr()
    #test_nans(ret, 437)
    return ret


@cache('run')
def p_customized_year_pop_evolution():
    """
    p customized year pop evolution



    constant


    """
    
    ret = 2020
    #test_nans(ret, 438)
    return ret


@cache('step')
def implementatio_policy_to_change_final_energy():
    """
    implementatio policy to change final energy



    component


    """
    
    ret = functions.if_then_else(
        functions.logical_or(year_policy_change_energy() < 2015
       ,year_policy_change_energy() > year_to_finish_policy_change_energy()), 0,
        functions.if_then_else(
            time() < year_policy_change_energy(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_change_energy(), 1,
                functions.if_then_else(
                    abs(policy_change_energy_speed() - 1) < equality_threshold,
                    ((time() - year_policy_change_energy()) /
                     (year_to_finish_policy_change_energy() - year_policy_change_energy()))
                    **(exp_rapid_evol_change_energy()),
                    functions.if_then_else(
                        abs(policy_change_energy_speed() - 2) < equality_threshold,
                        ((time() - year_policy_change_energy()) /
                         (year_to_finish_policy_change_energy() - year_policy_change_energy())),
                        functions.if_then_else(
                            abs(policy_change_energy_speed() - 3) < equality_threshold,
                            ((time() - year_policy_change_energy()) /
                             (year_to_finish_policy_change_energy() - year_policy_change_energy()))
                            **exp_slow_evol_change_energy(), 0))))))
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 439)
    return ret


@cache('step')
def implementatio_policy_to_change_final_energy_h():
    """
    implementatio policy to change final energy h



    component


    """
    
    ret = functions.if_then_else(
        functions.logical_or(year_policy_change_energy_h() < 2015
       ,year_policy_change_energy_h() > year_to_finish_policy_change_energy_h()), 0,
        functions.if_then_else(
            time() < year_policy_change_energy_h(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_change_energy_h(), 1,
                functions.if_then_else(
                    abs(policy_change_energy_speed_h() - 1) < equality_threshold,
                    ((time() - year_policy_change_energy_h()) /
                     (year_to_finish_policy_change_energy_h() - year_policy_change_energy_h()))
                    **(exp_rapid_evolution_change_energy_h()),
                    functions.if_then_else(abs(policy_change_energy_speed_h() - 2) < equality_threshold, (
                        (time() - year_policy_change_energy_h()) /
                        (year_to_finish_policy_change_energy_h() - year_policy_change_energy_h())),
                                           functions.if_then_else(
                                               abs(policy_change_energy_speed_h() - 3) < equality_threshold,
                                               ((time() - year_policy_change_energy_h()) /
                                                (year_to_finish_policy_change_energy_h() -
                                                 year_policy_change_energy_h()))
                                               **exp_slow_evolution_change_energy_h(), 0))))))
    #test_nans(ret, 440)
    return ret


@cache('step')
def implementation_policy_to_improve_energy_intensity_effciency():
    """
    implementation policy to improve energy intensity effciency



    component


    """
    
    ret = functions.if_then_else(
        functions.logical_or(year_policy_to_improve_efficiency() < 2015
       ,year_policy_to_improve_efficiency() > year_to_finish_policy_improve_efficiency()), 0,
        functions.if_then_else(
            time() < year_policy_to_improve_efficiency(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_improve_efficiency(), 1,
                functions.if_then_else(abs(policy_to_improve_efficiency_speed() - 1) < equality_threshold,
                                       ((time() - year_policy_to_improve_efficiency()) /
                                        (year_to_finish_policy_improve_efficiency() -
                                         year_policy_to_improve_efficiency()))
                                       **(exp_rapid_evolution_improve_efficiency()),
                                       functions.if_then_else(
                                           abs(policy_to_improve_efficiency_speed() - 2) < equality_threshold,
                                           ((time() - year_policy_to_improve_efficiency()) /
                                            (year_to_finish_policy_improve_efficiency() -
                                             year_policy_to_improve_efficiency())),
                                           functions.if_then_else(
                                               abs(policy_to_improve_efficiency_speed() - 3) < equality_threshold,
                                               ((time() - year_policy_to_improve_efficiency()) /
                                                (year_to_finish_policy_improve_efficiency() -
                                                 year_policy_to_improve_efficiency()))
                                               **exp_slow_evolution_improve_efficiency(), 0))))))
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 441)
    return ret


@cache('run')
def select_gdppc_evolution_input():
    """
    select gdppc evolution input



    constant


    """
    
    ret = 0
    #test_nans(ret, 442)
    return ret


@cache('run')
def select_population_evolution_input():
    """
    select population evolution input



    constant


    """
    
    ret = 0
    #test_nans(ret, 443)
    return ret


@cache('run')
def p_customized_year_gdppc_evolution():
    """
    p customized year gdppc evolution



    constant


    """
    
    ret = 2020
    #test_nans(ret, 444)
    return ret


@cache('step')
def implementation_policy_to_improve_energy_intensity_effciency_h():
    """
    implementation policy to improve energy intensity effciency h



    component


    """
    
    ret = functions.if_then_else(
        functions.logical_or(year_policy_to_improve_efficiency_h() < 2015
       ,year_policy_to_improve_efficiency_h() > year_to_finish_policy_improve_efficiency_h()), 0,
        functions.if_then_else(
            time() < year_policy_to_improve_efficiency_h(), 0,
            functions.if_then_else(
                time() > year_to_finish_policy_improve_efficiency_h(), 1,
                functions.if_then_else(abs(policy_to_improve_efficiency_speed_h() - 1) < equality_threshold,
                                       ((time() - year_policy_to_improve_efficiency_h()) /
                                        (year_to_finish_policy_improve_efficiency_h() -
                                         year_policy_to_improve_efficiency_h()))
                                       **(exp_rapid_evolution_improve_efficiency_h()),
                                       functions.if_then_else(
                                           abs(policy_to_improve_efficiency_speed_h() - 2) < equality_threshold,
                                           ((time() - year_policy_to_improve_efficiency_h()) /
                                            (year_to_finish_policy_improve_efficiency_h() -
                                             year_policy_to_improve_efficiency_h())),
                                           functions.if_then_else(
                                               abs(policy_to_improve_efficiency_speed_h() - 3) < equality_threshold,
                                               ((time() - year_policy_to_improve_efficiency_h()) /
                                                (year_to_finish_policy_improve_efficiency_h() -
                                                 year_policy_to_improve_efficiency_h()))
                                               **exp_slow_evolution_improve_efficiency_h(), 0))))))
    #test_nans(ret, 445)
    return ret


@cache('step')
def fe_tot_generation_all_res_elec_twh_delayed_1yr():
    """
    fe tot generation all res elec twh delayed 1yr



    component


    """
    
    ret = delay_fe_tot_generation_all_res_elec_twh_round1__time_step___time_step_2463_1__time_step(
    )
    #test_nans(ret, 446)
    return ret


@cache('run')
def p_customized_cte_pop_variation():
    """
    p customized cte pop variation



    constant


    """
    
    ret = 0.02
    #test_nans(ret, 447)
    return ret


@cache('step')
def fes_res_for_heat_delayed_1yr():
    """
    fes res for heat delayed 1yr



    component


    """
    
    ret = delay_fes_res_for_heat_ej_round1__time_step___time_step_3488_1__time_step()
    #test_nans(ret, 448)
    return ret


@cache('run')
def exp_slow_evol_change_energy():
    """
    exp slow evol change energy



    constant


    """
    
    ret = 2
    #test_nans(ret, 449)
    return ret


@cache('run')
def year_to_finish_policy_improve_efficiency():
    """
    year to finish policy improve efficiency



    constant


    """
    
    ret = 2050
    #test_nans(ret, 450)
    return ret


@cache('run')
def exp_slow_evolution_improve_efficiency():
    """
    exp slow evolution improve efficiency



    constant


    """
    
    ret = 2
    #test_nans(ret, 451)
    return ret


@cache('run')
def exp_rapid_evolution_improve_efficiency():
    """
    exp rapid evolution improve efficiency



    constant


    """
    
    ret = 1 / 2
    #test_nans(ret, 452)
    return ret


@cache('run')
def exp_rapid_evol_change_energy():
    """
    exp rapid evol change energy



    constant


    """
    
    ret = 1 / 2
    #test_nans(ret, 453)
    return ret


@cache('run')
def year_to_finish_policy_change_energy():
    """
    year to finish policy change energy



    constant


    """
    
    ret = 2050
    #test_nans(ret, 454)
    return ret


@cache('run')
def year_to_finish_policy_improve_efficiency_h():
    """
    year to finish policy improve efficiency h



    constant


    """
    
    ret = 2050
    #test_nans(ret, 455)
    return ret


@cache('run')
def year_to_finish_policy_change_energy_h():
    """
    year to finish policy change energy h



    constant


    """
    
    ret = 2050
    #test_nans(ret, 456)
    return ret


@cache('run')
def exp_rapid_evolution_change_energy_h():
    """
    exp rapid evolution change energy h



    constant


    """
    
    ret = 1 / 2
    #test_nans(ret, 457)
    return ret


@cache('step')
def pressure_to_improve_energy_intensity_efficiency_h():
    """
    pressure to improve energy intensity efficiency h



    component


    """
    
    ret = energy_cost_pressure_h(
    ) + implementation_policy_to_improve_energy_intensity_effciency_h()
    #test_nans(ret, 458)
    return ret


@cache('run')
def exp_slow_evolution_change_energy_h():
    """
    exp slow evolution change energy h



    constant


    """
    
    ret = 2
    #test_nans(ret, 459)
    return ret


@cache('run')
def exp_rapid_evolution_improve_efficiency_h():
    """
    exp rapid evolution improve efficiency h



    constant


    """
    
    ret = 1 / 2
    #test_nans(ret, 460)
    return ret


@cache('run')
def exp_slow_evolution_improve_efficiency_h():
    """
    exp slow evolution improve efficiency h



    constant


    """
    
    ret = 2
    #test_nans(ret, 461)
    return ret


@cache('step')
def desired_gdp_delayed_1yr():
    """
    desired gdp delayed 1yr



    component


    """
    
    ret = delay_desired_gdp_round1__time_step___time_step_2916_1__time_step()
    #test_nans(ret, 462)
    return ret


@cache('step')
def diff_annual_gdp_growth_rate():
    """
    diff annual gdp growth rate



    component


    """
    
    ret = functions.zidz((annual_gdp_growth_rate() - desired_annual_gdp_growth_rate()),
                          desired_annual_gdp_growth_rate())
    #test_nans(ret, 463)
    return ret


@cache('run')
def gdppc_initial_year():
    """
    gdppc initial year



    constant


    """
    
    ret = 5106.7947854704435
    #test_nans(ret, 464)
    return ret


@cache('step')
def desired_gdppc():
    """
    desired gdppc



    component


    """
    
    ret = integ_desired_gdppc()
    #test_nans(ret, 465)
    return ret


@cache('step')
def tpes_intensity_until_x2009x():
    """
    tpes intensity until x2009x



    component


    """
    
    ret = functions.if_then_else(time() < 2009, tpes_intensity_ej_tdollar(), aux14())
    #test_nans(ret, 466)
    return ret


@cache('step')
def sharexdashxe_losses_cc_until_x2015():
    """
    sharexdashxe losses cc until x2015



    component


    """
    
    ret = functions.if_then_else(time() < 2015, elf(), aux1())
    #test_nans(ret, 467)
    return ret


@cache('step')
def aux13():
    """
    aux13



    component


    """
    
    ret = delay_tfec_intensity_until_x2009x_without_eroi_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 468)
    return ret


@cache('step')
def aux14():
    """
    aux14



    component


    """
    
    ret = delay_tpes_intensity_until_x2009x_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 469)
    return ret


@cache('step')
def share_exdashxlosses_cc_from_x2015():
    """
    share exdashxlosses cc from x2015



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  np.maximum(0,
                                             np.minimum(
                                                 1, elf() - sharexdashxe_losses_cc_until_x2015())))
    #test_nans(ret, 470)
    return ret


@cache('step')
def elf():
    """
    elf



    component


    """
    
    ret = elf_concentrations_logistic()
    #test_nans(ret, 471)
    return ret


@cache('step')
def cumulative_tpes_intensity_change_from_x2009x():
    """
    cumulative tpes intensity change from x2009x



    component


    """
    
    ret = functions.if_then_else(time() < 2009, 0, -1 +
                                  (tpes_intensity_ej_tdollar() / tpes_intensity_until_x2009x()))
    #test_nans(ret, 472)
    return ret


@cache('step')
def cumulative_tfec_intensity_change_from_x2009x_without_eroi():
    """
    cumulative tfec intensity change from x2009x without eroi



    component


    """
    
    ret = functions.if_then_else(
        time() < 2009, 0, -1 +
        (tfes_intensity_ej_tdollar_without_eroi() / tfec_intensity_until_x2009x_without_eroi()))
    #test_nans(ret, 473)
    return ret


@cache('step')
def tfec_intensity_until_x2009x_without_eroi():
    """
    tfec intensity until x2009x without eroi



    component


    """
    
    ret = functions.if_then_else(time() < 2009, tfes_intensity_ej_tdollar_without_eroi(), aux13())
    #test_nans(ret, 474)
    return ret


@cache('step')
def tfei_sectors():
    """
    tfei sectors



    component


    """
    
    ret = functions.sum(final_energy_intensity_by_sector_and_fuel())
    #test_nans(ret, 475)
    return ret


@cache('step')
def annual_tfes_intensity_growth_rate_without_eroi():
    """
    annual tfes intensity growth rate without eroi



    component


    """
    
    ret = -1 + functions.zidz(tfes_intensity_ej_tdollar_without_eroi(),
                               tfes_intensity_without_eroi_delayed_1yr())
    #test_nans(ret, 476)
    return ret


@cache('step')
def final_energy_intensity_by_sector_and_fuel():
    """
    final energy intensity by sector and fuel



    component


    """
    
    ret = evol_final_energy_intensity_by_sector_and_fuel_x2x()
    ret = utils.align_by_dims(ret, ('final sources', 'sectors'))
    #test_nans(ret, 477)
    return ret


@cache('step')
def energy_intensity_of_households():
    """
    energy intensity of households



    component


    """
    
    ret = functions.if_then_else(
        time() < 2009,
        energy_intensity_of_households_rest(),
        functions.if_then_else(
            abs(activate_transport_h_bottom_up_method() - 0) < equality_threshold,
            energy_intensity_of_households_rest(),
            energy_intensity_of_households_transport() + energy_intensity_of_households_rest()))
    #test_nans(ret, 478)
    return ret


@cache('step')
def tfes_intensity_without_eroi_delayed_1yr():
    """
    tfes intensity without eroi delayed 1yr



    component


    """
    
    ret = delay_tfes_intensity_ej_tdollar_without_eroi_round1__time_step___time_step_8827_1__time_step(
    )
    #test_nans(ret, 479)
    return ret


@cache('step')
def households_total_final_energy_demand():
    """
    households total final energy demand



    component


    """
    
    ret = functions.sum(households_final_energy_demand())
    #test_nans(ret, 480)
    return ret


@cache('step')
def required_tfed_sectors():
    """
    required tfed sectors



    component


    """
    
    ret = functions.sum(required_fed_sectors_by_fuel())
    #test_nans(ret, 481)
    return ret


@cache('step')
def ratio_fed_households_vs_sectors():
    """
    ratio fed households vs sectors



    component


    """
    
    ret = functions.zidz(households_total_final_energy_demand(), required_tfed_sectors())
    #test_nans(ret, 482)
    return ret


@cache('step')
def tfes_intensity_ej_tdollar_without_eroi():
    """
    tfes intensity ej tdollar without eroi



    component


    """
    
    ret = functions.zidz(real_tfec() / eroi_fc_system_from_x2015x(), gdp())
    #test_nans(ret, 483)
    return ret


@cache('run')
def ghg_emissions_x2050x_mlt2030():
    """
    ghg emissions x2050x mlt2030



    constant


    """
    
    ret = 7.958767677327897
    #test_nans(ret, 484)
    return ret


@cache('run')
def ghg_emissions_x2050x_mlt2020():
    """
    ghg emissions x2050x mlt2020



    constant


    """
    
    ret = 17.232523345825705
    #test_nans(ret, 485)
    return ret


@cache('run')
def start_year_p_growth_res_elec():
    """
    start year p growth res elec



    constant


    """
    
    ret = 2020
    #test_nans(ret, 486)
    return ret


@cache('run')
def start_year_p_growth_res_heat():
    """
    start year p growth res heat



    constant


    """
    
    ret = 2020
    #test_nans(ret, 487)
    return ret


@cache('run')
def labour_share_growth():
    """
    labour share growth



    component


    """
    
    ret = ((p_labour_share_x2050x() / initial_labour_share())**
            (1 / (year_final_labour_share() - year_initial_labour_share()))) - 1
    #test_nans(ret, 488)
    return ret


@cache('run')
def year_initial_labour_share():
    """
    year initial labour share



    constant


    """
    
    ret = 2014
    #test_nans(ret, 489)
    return ret


@cache('run')
def year_final_labour_share():
    """
    year final labour share



    constant


    """
    
    ret = 2050
    #test_nans(ret, 490)
    return ret


@cache('run')
def p_labour_share_x2050x():
    """
    p labour share x2050x



    constant


    """
    
    ret = 0.52
    #test_nans(ret, 491)
    return ret


@cache('run')
def initial_labour_share():
    """
    initial labour share



    constant


    """
    
    ret = 0.5621
    #test_nans(ret, 492)
    return ret


@cache('run')
def low_range_fec_good_standard_of_living():
    """
    low range fec good standard of living



    constant


    """
    
    ret = 30.0
    #test_nans(ret, 493)
    return ret


@cache('run')
def threshold_fec_xhigh_developmentx():
    """
    threshold fec xhigh developmentx



    constant


    """
    
    ret = 75.0
    #test_nans(ret, 494)
    return ret


@cache('run')
def high_range_fec_good_standard_of_living():
    """
    high range fec good standard of living



    constant


    """
    
    ret = 40.0
    #test_nans(ret, 495)
    return ret


@cache('step')
def decrease_of_intensity_due_to_energy_a_technology_change_top_down():
    """
    decrease of intensity due to energy a technology change top down



    component


    """
    
    ret = functions.if_then_else(
        abs(activate_bottom_up_method() - 0) < equality_threshold,
        functions.if_then_else(
            (functions.zidz(evol_final_energy_intensity_by_sector_and_fuel(),
                            global_energy_intensity_by_sector())) >= minimum_fraction(),
            max_yearly_change() * evol_final_energy_intensity_by_sector_and_fuel() *
            pressure_to_change_energy_technology(), 0), 0)
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 496)
    return ret


@cache('run')
def maximun_yearly_aceleration_of_intensity_improvement_pct():
    """
    maximun yearly aceleration of intensity improvement pct



    constant


    """
    
    ret = xr.DataArray(data=[[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01],[0.01,0.01,0.01,0.01,0.01]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 497)
    return ret


@cache('run')
def policy_change_energy_speed():
    """
    policy change energy speed



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 498)
    return ret


@cache('run')
def policy_to_improve_efficiency_speed():
    """
    policy to improve efficiency speed



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 499)
    return ret


@cache('run')
def minimum_fraction():
    """
    minimum fraction



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 500)
    return ret


@cache('step')
def increase_of_intensity_due_to_energy_a_technology_change_top_down():
    """
    increase of intensity due to energy a technology change top down



    component


    """
    
    ret = xr.DataArray(data=[
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'solids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'electricity','solids'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'gases'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'electricity','gases'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'heat'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'electricity','heat'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'liquids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'electricity','liquids'].drop('final sources').drop('final sources1'),
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'solids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'heat','solids'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'gases'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'heat','gases'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'electricity'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'heat','electricity'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'liquids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'heat','liquids'].drop('final sources').drop('final sources1'),
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'solids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'liquids','solids'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'gases'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'liquids','gases'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'electricity'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'liquids','electricity'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'heat'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'liquids','heat'].drop('final sources').drop('final sources1'),
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'solids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'gases','solids'].drop('final sources').drop('final sources1') +
        efficiency_rate_of_substitution().loc[:,'gases','electricity'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'heat'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'gases','heat'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'liquids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'gases','liquids'].drop('final sources').drop('final sources1'),
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'gases'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'solids','gases'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'electricity'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'solids','electricity'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'heat'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'solids','heat'].drop('final sources').drop('final sources1') +
        decrease_of_intensity_due_to_energy_a_technology_change_top_down().loc[:,'liquids'].drop('final sources') *
        efficiency_rate_of_substitution().loc[:,'solids','liquids'].drop('final sources').drop('final sources1')],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources', 'sectors'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 501)
    return ret


@cache('run')
def year_policy_change_energy():
    """
    year policy change energy



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 502)
    return ret


@cache('run')
def max_yearly_change():
    """
    max yearly change



    constant


    """
    
    ret = xr.DataArray(data=[[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05],[0.05,0.05,0.05,0.05,0.05]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 503)
    return ret


@cache('run')
def efficiency_rate_of_substitution():
    """
    efficiency rate of substitution



    constant


    """
    
    ret = xr.DataArray(data=[[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]],[[0.0, 0.0, 0.33, 0.33, 0.33],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0],[0.0, 0.0, 0.0, 0.0, 0.0]]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'], 'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources', 'final sources1'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources', 'final sources1'))
    #test_nans(ret, 504)
    return ret


@cache('run')
def activate_bottom_up_method():
    """
    activate bottom up method



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 505)
    return ret


@cache('run')
def year_policy_to_improve_efficiency():
    """
    year policy to improve efficiency



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 506)
    return ret


@cache('step')
def decrease_of_intensity_due_to_energy_a_technology_change_h_top_down():
    """
    decrease of intensity due to energy a technology change h top down



    component


    """
    
    ret = functions.if_then_else((functions.zidz(
        evol_final_energy_intensity_h(), global_energy_intensity_h())) >= minimum_fraction_h(),
                                  max_yearly_change_h() * evol_final_energy_intensity_h() *
                                  pressure_to_change_energy_technology_h(), 0)
    #test_nans(ret, 507)
    return ret


@cache('step')
def increase_of_intensity_due_to_energy_a_technology_change_h_top_down():
    """
    increase of intensity due to energy a technology change h top down



    component


    """
    
    ret = xr.DataArray(data=[
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['solids'].values *
        efficiency_rate_of_substitution_h().loc['electricity','solids'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['gases'].values *
        efficiency_rate_of_substitution_h().loc['electricity','gases'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['heat'].values *
        efficiency_rate_of_substitution_h().loc['electricity','heat'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['liquids'].values *
        efficiency_rate_of_substitution_h().loc['electricity','liquids'].values,
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['solids'].values *
        efficiency_rate_of_substitution_h().loc['heat','solids'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['gases'].values *
        efficiency_rate_of_substitution_h().loc['heat','gases'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['electricity'].values *
        efficiency_rate_of_substitution_h().loc['heat','electricity'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['liquids'].values *
        efficiency_rate_of_substitution_h().loc['heat','liquids'].values,
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['solids'].values *
        efficiency_rate_of_substitution_h().loc['liquids','solids'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['gases'].values *
        efficiency_rate_of_substitution_h().loc['liquids','gases'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['electricity'].values *
        efficiency_rate_of_substitution_h().loc['liquids','electricity'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['heat'].values *
        efficiency_rate_of_substitution_h().loc['liquids','heat'].values,
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['solids'].values *
        efficiency_rate_of_substitution_h().loc['gases','solids'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['electricity'].values *
        efficiency_rate_of_substitution_h().loc['gases','electricity'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['heat'].values *
        efficiency_rate_of_substitution_h().loc['gases','heat'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['liquids'].values *
        efficiency_rate_of_substitution_h().loc['gases','liquids'].values,
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['gases'].values *
        efficiency_rate_of_substitution_h().loc['solids','gases'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['electricity'].values *
        efficiency_rate_of_substitution_h().loc['solids','electricity'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['heat'].values *
        efficiency_rate_of_substitution_h().loc['solids','heat'].values +
        decrease_of_intensity_due_to_energy_a_technology_change_h_top_down().loc['liquids'].values *
        efficiency_rate_of_substitution_h().loc['solids','liquids'].values],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 508)
    return ret


@cache('step')
def total_co2e_xgwpxequalsx100x_yearsx():
    """
    total co2e xgwpxequalsx100x yearsx



    component


    """
    
    ret = total_ch4_emissions_fossil_fuels() * gwp_x100x_years_ch4(
    ) / 1000 + total_co2_emissions_gtco2()
    #test_nans(ret, 509)
    return ret


@cache('step')
def net_tfec_per_capita():
    """
    net tfec per capita



    component


    """
    
    ret = functions.zidz(real_tfec() * gj_per_ej() / eroi_fc_system_from_x2015x(), population())
    #test_nans(ret, 510)
    return ret


@cache('run')
def gwp_x100x_years_ch4():
    """
    gwp x100x years ch4



    constant


    """
    
    ret = 34
    #test_nans(ret, 511)
    return ret


@cache('run')
def activate_energy_scarcity_feedbackxquestx():
    """
    activate energy scarcity feedbackxquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 512)
    return ret


@cache('run')
def max_potential_phs_twh():
    """
    max potential phs twh



    component


    """
    
    ret = max_capacity_potential_phs() * cp_phs() / twe_per_twh()
    #test_nans(ret, 513)
    return ret


@cache('step')
def replacement_rate_phs():
    """
    replacement rate phs



    component


    """
    
    ret = functions.if_then_else(real_fe_elec_stored_phs_twh() < max_potential_phs_twh(), 1, 0.8)
    #test_nans(ret, 514)
    return ret


@cache('step')
def energy_scarcity_feedback_shortage_coeff():
    """
    energy scarcity feedback shortage coeff



    component


    """
    
    ret = functions.if_then_else(activate_energy_scarcity_feedbackxquestx() == 1,
        np.minimum(1,
                   functions.zidz(real_fe_consumption_by_fuel_before_heat_correction(),
                                  required_fed_by_fuel_before_heat_correction())), 1)
    #test_nans(ret, 515)
    return ret


@cache('step')
def real_fe_elec_stored_ev_batteries_twh():
    """
    real fe elec stored ev batteries twh



    component


    """
    
    ret = used_ev_batteries_for_elec_storage() / twe_per_twh()
    #test_nans(ret, 516)
    return ret


@cache('step')
def share_tot_fei_res_elec_var():
    """
    share tot fei res elec var



    component


    """
    
    ret = total_final_energy_invested_res_elec_var() / total_dyn_fei_res()
    #test_nans(ret, 517)
    return ret


@cache('step')
def cp_ev_batteries_required():
    """
    cp ev batteries required



    component


    """
    
    ret = np.maximum(0, demand_ev_batteries_for_elec_storage() / ev_batteries_tw())
    #test_nans(ret, 518)
    return ret


@cache('step')
def output_ev_bateries_for_storage_over_lifetime():
    """
    output ev bateries for storage over lifetime



    component


    """
    
    ret = cp_ev_batteries_for_elec_storage() * newxplusxreplaced_batteries_tw() * (
        1 / twe_per_twh()) * lifetime_ev_batteries() * ej_per_twh()
    #test_nans(ret, 519)
    return ret


@cache('step')
def discarded_batteries():
    """
    discarded batteries



    component


    """
    
    ret = np.maximum(0, functions.zidz(batteries_evxplusxhibxplusx2we(), lifetime_ev_batteries()))
    #test_nans(ret, 520)
    return ret


@cache('step')
def esoi_elec_storage():
    """
    esoi elec storage



    component


    """
    
    ret = (esoi_phs() * installed_capacity_phs_tw() + esoi_ev_batteries(
    ) * used_ev_batteries_for_elec_storage()) / total_capacity_elec_storage_tw()
    #test_nans(ret, 521)
    return ret


@cache('step')
def esoi_ev_batteries():
    """
    esoi ev batteries



    component


    """
    
    ret = lifetime_ev_batteries() * cp_ev_batteries_for_elec_storage() * mw_in_x1x_year_to_mj(
    ) / (quality_of_electricity() * energy_intensity_construction_ev_batteries_mjxdivxmw() *
         (1 + share_energy_requirements_for_decom_ev_batteries(
         ) + grid_correction_factor_ev_batteries()))
    #test_nans(ret, 522)
    return ret


@cache('step')
def rt_elec_storage_efficiency():
    """
    rt elec storage efficiency



    component


    """
    
    ret = (rt_storage_efficiency_phs() * installed_capacity_phs_tw() +
            rt_storage_efficiency_ev_batteries() * used_ev_batteries_for_elec_storage()
            ) / total_capacity_elec_storage_tw()
    #test_nans(ret, 523)
    return ret


@cache('step')
def xxabundancexx_storage():
    """
    xxabundancexx storage



    component


    """
    
    ret = 1 - functions.if_then_else(
        demand_storage_capacity() <= total_capacity_elec_storage_tw(), 1,
        np.maximum(0, 1 - (demand_storage_capacity() - total_capacity_elec_storage_tw()) /
                   total_capacity_elec_storage_tw()))
    #test_nans(ret, 524)
    return ret


@cache('step')
def used_ev_batteries_for_elec_storage():
    """
    used ev batteries for elec storage



    component


    """
    
    ret = ev_batteries_tw() * cp_ev_batteries_for_elec_storage()
    #test_nans(ret, 525)
    return ret


@cache('step')
def max_cp_ev_batteries():
    """
    max cp ev batteries



    component


    """
    
    ret = net_stored_energy_ev_battery_over_lifetime() / (
        min_lifetime_ev_batteries() * mw_in_x1x_year_to_mj() * (kw_per_battery_ev() / kw_per_mw()))
    #test_nans(ret, 526)
    return ret


@cache('step')
def total_capacity_elec_storage_tw():
    """
    total capacity elec storage tw



    component


    """
    
    ret = installed_capacity_phs_tw() + used_ev_batteries_for_elec_storage()
    #test_nans(ret, 527)
    return ret


@cache('step')
def demand_ev_batteries_for_elec_storage():
    """
    demand ev batteries for elec storage



    component


    """
    
    ret = np.maximum(0, demand_storage_capacity() - installed_capacity_phs_tw())
    #test_nans(ret, 528)
    return ret


@cache('step')
def ced_per_tw_over_lifetime_phs():
    """
    ced per tw over lifetime phs



    component


    """
    
    ret = functions.zidz((cp_phs() * lifetime_res_elec().loc['hydro'].values * ej_per_twh() / twe_per_twh()),
                          esoi_static_phs() * quality_of_electricity_x2015x())
    #test_nans(ret, 529)
    return ret


@cache('step')
def remaining_potential_constraint_on_new_phs_capacity():
    """
    remaining potential constraint on new phs capacity



    component


    """
    
    ret = functions.if_then_else(
        remaining_potential_phs() > threshold_remaining_potential_new_capacity(), 1,
        remaining_potential_phs() * (1 / threshold_remaining_potential_new_capacity()))
    #test_nans(ret, 530)
    return ret


@cache('step')
def new_required_phs_capacity():
    """
    new required phs capacity



    component


    """
    
    ret = functions.if_then_else(time() < (2014 - total_time_planxplusxconstr_res_elec().loc['hydro'].values),
                                  historic_new_required_capacity_phs(),
                                  installed_capacity_phs_tw() * adapt_growth_phs() *
                                  remaining_potential_constraint_on_new_phs_capacity())
    #test_nans(ret, 531)
    return ret


@cache('step')
def share_dyn_fei_for_res_vs_tfec():
    """
    share dyn fei for res vs tfec



    component


    """
    
    ret = total_dyn_fei_res() / real_tfec()
    #test_nans(ret, 532)
    return ret


@cache('step')
def fei_ev_batteries():
    """
    fei ev batteries



    component


    """
    
    ret = functions.zidz(output_ev_bateries_for_storage_over_lifetime(), esoi_ev_batteries())
    #test_nans(ret, 533)
    return ret


@cache('step')
def output_phs_over_lifetime():
    """
    output phs over lifetime



    component


    """
    
    ret = cp_phs() * phs_capacity_under_construction() * (
        1 / twe_per_twh()) * lifetime_res_elec().loc['hydro'].values * ej_per_twh()
    #test_nans(ret, 534)
    return ret


@cache('step')
def esoi_static_phs():
    """
    esoi static phs



    component


    """
    
    ret = a_lineal_regr() * installed_capacity_phs_tw() + b_lineal_regr()
    #test_nans(ret, 535)
    return ret


@cache('step')
def total_dyn_fei_res():
    """
    total dyn fei res



    component


    """
    
    ret = total_final_energy_invested_res_elec_var() + fei_ev_batteries(
    ) + total_fei_over_lifetime_res_elec_dispatch() + final_energy_invested_phs()
    #test_nans(ret, 536)
    return ret


@cache('step')
def adapt_growth_res_elec_after_allocation():
    """
    adapt growth res elec after allocation



    component


    """
    
    ret = functions.if_then_else(activate_eroi_allocation_rule(
    ) == 0, adapt_growth_res_elec(), adapt_growth_res_elec() * eroi_allocation_rule_per_res_elec()
                                  ) * constraint_elec_storage_availability()
    #test_nans(ret, 537)
    return ret


@cache('step')
def esoi_phs():
    """
    esoi phs



    component


    """
    
    ret = functions.zidz(output_phs_over_lifetime(),
                          cedtot_over_lifetime_phs() * quality_of_electricity())
    #test_nans(ret, 538)
    return ret


@cache('run')
def p_phs_growth():
    """
    p phs growth



    constant


    """
    
    ret = 0.023
    #test_nans(ret, 539)
    return ret


@cache('step')
def constraint_elec_storage_availability():
    """
    constraint elec storage availability



    component


    """
    
    ret = functions.if_then_else(
        abs(res_elec_variablesxquestx()) < equality_threshold, 1,
        functions.if_then_else(
            demand_storage_capacity() <= total_capacity_elec_storage_tw(), 1,
            np.maximum(0, 1 - (demand_storage_capacity() - total_capacity_elec_storage_tw()) /
                       total_capacity_elec_storage_tw())))
    #test_nans(ret, 540)
    return ret


@cache('step')
def eroist_system():
    """
    eroist system



    component


    """
    
    ret = np.maximum(
        0, (real_tfec()) /
        (((energy_ownxdashxuse_constant_x830x_mtoe_in_x2013() / mtoe_per_ej()) / 375.5) *
         (real_tfec()) + total_dyn_fei_res()))
    #test_nans(ret, 541)
    return ret


@cache('run')
def initial_instal_cap_phs():
    """
    initial instal cap phs



    constant


    """
    
    ret = 0.0916300043623243
    #test_nans(ret, 542)
    return ret


@cache('step')
def cedtot_over_lifetime_phs():
    """
    cedtot over lifetime phs



    component


    """
    
    ret = phs_capacity_under_construction() * ced_per_tw_over_lifetime_phs()
    #test_nans(ret, 543)
    return ret


@cache('step')
def installed_capacity_phs_tw():
    """
    installed capacity phs tw



    component


    """
    
    ret = integ_installed_capacity_phs_tw()
    #test_nans(ret, 544)
    return ret


@cache('step')
def phs_capacity_under_construction():
    """
    phs capacity under construction



    component


    """
    
    ret = phs_planned_capacity() / time_construction_res_elec().loc['hydro'].values
    #test_nans(ret, 545)
    return ret


@cache('run')
def past_phs_capacity_growth():
    """
    past phs capacity growth



    constant


    """
    
    ret = 0.023
    #test_nans(ret, 546)
    return ret


@cache('step')
def new_phs_capacity_under_planning():
    """
    new phs capacity under planning



    component


    """
    
    ret = required_capacity_phs() / time_planification_res_elec().loc['hydro'].values
    #test_nans(ret, 547)
    return ret


@cache('step')
def wear_phs():
    """
    wear phs



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  installed_capacity_phs_tw() / lifetime_res_elec().loc['hydro'].values)
    #test_nans(ret, 548)
    return ret


@cache('step')
def replacement_capacity_phs():
    """
    replacement capacity phs



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0, replacement_rate_phs() * wear_phs())
    #test_nans(ret, 549)
    return ret


@cache('step')
def real_fe_elec_stored_phs_twh():
    """
    real fe elec stored phs twh



    component


    """
    
    ret = installed_capacity_phs_tw() * cp_phs() / twe_per_twh()
    #test_nans(ret, 550)
    return ret


@cache('step')
def phs_planned_capacity():
    """
    phs planned capacity



    component


    """
    
    ret = integ_phs_planned_capacity()
    #test_nans(ret, 551)
    return ret


@cache('step')
def final_energy_invested_phs():
    """
    final energy invested phs



    component


    """
    
    ret = real_fe_elec_stored_phs_twh() * ej_per_twh() / esoi_phs()
    #test_nans(ret, 552)
    return ret


@cache('step')
def remaining_potential_phs():
    """
    remaining potential phs



    component


    """
    
    ret = functions.if_then_else(max_capacity_potential_phs() > installed_capacity_phs_tw(), (
        max_capacity_potential_phs() - installed_capacity_phs_tw()) / max_capacity_potential_phs(),
                                  0)
    #test_nans(ret, 553)
    return ret


@cache('step')
def initial_capacity_in_construction_phs():
    """
    initial capacity in construction phs



    component


    """
    
    ret = initial_required_capacity_phs()
    #test_nans(ret, 554)
    return ret


@cache('step')
def required_capacity_phs():
    """
    required capacity phs



    component


    """
    
    ret = integ_required_capacity_phs()
    #test_nans(ret, 555)
    return ret


@cache('step')
def total_fei_over_lifetime_res_elec_dispatch():
    """
    total fei over lifetime res elec dispatch



    component


    """
    
    ret = functions.sum(fei_over_lifetime_res_elec_dispatch())
    #test_nans(ret, 556)
    return ret


@cache('run')
def min_lifetime_ev_batteries():
    """
    min lifetime ev batteries



    constant


    """
    
    ret = 5
    #test_nans(ret, 557)
    return ret


@cache('step')
def total_final_energy_invested_res_elec_var():
    """
    total final energy invested res elec var



    component


    """
    
    ret = functions.sum(fei_res_elec_var())
    #test_nans(ret, 558)
    return ret


@cache('step')
def materials_required_for_ev_batteries_mt():
    """
    materials required for ev batteries mt



    component


    """
    
    ret = newxplusxreplaced_batteries_tw(
    ) * materials_per_new_capacity_installed_xdashx_ev_batteries() * m_per_t() / kg_per_mt()
    #test_nans(ret, 559)
    return ret


@cache('step')
def newxplusxreplaced_batteries_tw():
    """
    newxplusxreplaced batteries tw



    component


    """
    
    ret = (new_batteries() + replacement_batteries()) * kw_per_battery_ev() / kwh_per_twh()
    #test_nans(ret, 560)
    return ret


@cache('step')
def energy_intensity_construction_ev_batteries_mjxdivxmw():
    """
    energy intensity construction ev batteries mjxdivxmw



    component


    """
    
    ret = functions.zidz(
        total_energy_required_for_total_material_consumption_for_ev_batteries() * mj_per_ej(),
        newxplusxreplaced_batteries_tw() * m_per_t())
    #test_nans(ret, 561)
    return ret


@cache('step')
def new_batteries():
    """
    new batteries



    component


    """
    
    ret = required_number_standard_batteries() - batteries_evxplusxhibxplusx2we()
    #test_nans(ret, 562)
    return ret


@cache('step')
def batteries_evxplusxhibxplusx2we():
    """
    batteries evxplusxhibxplusx2we



    component


    """
    
    ret = integ_batteries_evxplusxhibxplusx2we()
    #test_nans(ret, 563)
    return ret


@cache('run')
def kw_per_mw():
    """
    kw per mw



    constant


    """
    
    ret = 1000.0
    #test_nans(ret, 564)
    return ret


@cache('run')
def net_stored_energy_ev_battery_over_lifetime():
    """
    net stored energy ev battery over lifetime



    constant


    """
    
    ret = 138024
    #test_nans(ret, 565)
    return ret


@cache('run')
def grid_correction_factor_ev_batteries():
    """
    grid correction factor ev batteries



    constant


    """
    
    ret = 0.1
    #test_nans(ret, 566)
    return ret


@cache('run')
def share_energy_requirements_for_decom_ev_batteries():
    """
    share energy requirements for decom ev batteries



    constant


    """
    
    ret = 0.1
    #test_nans(ret, 567)
    return ret


@cache('run')
def working_hours_per_year():
    """
    working hours per year



    component


    """
    
    ret = working_hours_per_day() * days_per_year()
    #test_nans(ret, 568)
    return ret


@cache('run')
def days_per_year():
    """
    days per year



    constant


    """
    
    ret = 365.0
    #test_nans(ret, 569)
    return ret


@cache('step')
def total_energy_required_for_material_consumption_for_res_elec():
    """
    total energy required for material consumption for res elec



    component


    """
    
    ret = functions.sum(energy_required_for_material_consumption_per_res_elec())
    #test_nans(ret, 570)
    return ret


@cache('step')
def total_energy_required_for_material_consumption_per_res_elec():
    """
    total energy required for material consumption per res elec



    component


    """
    
    ret = functions.sum(dim='materials', data=energy_required_for_material_consumption_per_res_elec())
    #test_nans(ret, 571)
    return ret


@cache('run')
def mw_in_x1x_year_to_mj():
    """
    mw in x1x year to mj



    constant


    """
    
    ret = 31536000.0
    #test_nans(ret, 572)
    return ret


@cache('run')
def working_hours_per_day():
    """
    working hours per day



    constant


    """
    
    ret = 8
    #test_nans(ret, 573)
    return ret


@cache('step')
def annual_work_hours_for_res():
    """
    annual work hours for res



    component


    """
    
    ret = total_jobs_res() * working_hours_per_year()
    #test_nans(ret, 574)
    return ret


@cache('step')
def hours_work_per_gj_res_delivered():
    """
    hours work per gj res delivered



    component


    """
    
    ret = functions.zidz(annual_work_hours_for_res(), (tfec_res_ej() * gj_per_ej()))
    #test_nans(ret, 575)
    return ret


@cache('step')
def total_energy_required_for_total_material_consumption_for_ev_batteries():
    """
    total energy required for total material consumption for ev batteries



    component


    """
    
    ret = functions.sum(energy_required_for_material_consumption_for_ev_batteries())
    #test_nans(ret, 576)
    return ret


@cache('step')
def tfe_required_for_total_material_consumption_for_alt_techn():
    """
    tfe required for total material consumption for alt techn



    component


    """
    
    ret = total_energy_required_for_material_consumption_for_res_elec(
    ) + total_energy_required_for_total_material_consumption_for_ev_batteries()
    #test_nans(ret, 577)
    return ret


@cache('step')
def energy_required_for_material_consumption_per_res_elec():
    """
    energy required for material consumption per res elec



    component


    """
    
    ret = energy_required_for_material_consumption_for_oxandxm_res_elec(
    ) + energy_required_for_material_consumption_for_new_res_elec()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 578)
    return ret


@cache('step')
def total_energy_required_per_material_for_alt_techn():
    """
    total energy required per material for alt techn



    component


    """
    
    ret = functions.sum(dim='res elec', data=energy_required_for_material_consumption_per_res_elec()
                          ) + energy_required_for_material_consumption_for_ev_batteries()
    #test_nans(ret, 579)
    return ret


@cache('step')
def total_jobs_res_elec():
    """
    total jobs res elec



    component


    """
    
    ret = functions.sum(total_dxplusxi_jobs_res_elec_per_techn())
    #test_nans(ret, 580)
    return ret


@cache('step')
def total_jobs_biofuels():
    """
    total jobs biofuels



    component


    """
    
    ret = employment_factor_biofuels() * fes_total_biofuels_production_ej()
    #test_nans(ret, 581)
    return ret


@cache('step')
def total_jobs_res():
    """
    total jobs res



    component


    """
    
    ret = total_jobs_res_elec() + total_jobs_res_heat() + d_jobs_fuel_supply_solids_bioe(
    ) + total_jobs_biofuels()
    #test_nans(ret, 582)
    return ret


@cache('step')
def total_dxplusxi_jobs_res_heat_per_techn():
    """
    total dxplusxi jobs res heat per techn



    component


    """
    
    ret = total_d_jobs_res_heat_per_techn() * ratio_total_vs_d_jobs_res_heat()
    #test_nans(ret, 583)
    return ret


@cache('step')
def total_jobs_res_heat():
    """
    total jobs res heat



    component


    """
    
    ret = functions.sum(total_dxplusxi_jobs_res_heat_per_techn())
    #test_nans(ret, 584)
    return ret


@cache('step')
def total_dxplusxi_jobs_res_elec_per_techn():
    """
    total dxplusxi jobs res elec per techn



    component


    """
    
    ret = total_d_jobs_res_elec_per_techn() * ratio_total_vs_d_jobs_res_elec()
    #test_nans(ret, 585)
    return ret


@cache('run')
def ratio_total_vs_d_jobs_res_heat():
    """
    ratio total vs d jobs res heat



    constant


    """
    
    ret = xr.DataArray(data=[1.0,1.9,1.0],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 586)
    return ret


@cache('run')
def ratio_total_vs_d_jobs_res_elec():
    """
    ratio total vs d jobs res elec



    constant


    """
    
    ret = xr.DataArray(data=[2.6,1.9,1.0,1.0,1.25,1.25,1.5,1.7],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 587)
    return ret


@cache('run')
def employment_factor_biofuels():
    """
    employment factor biofuels



    constant


    """
    
    ret = 740231.8591670244
    #test_nans(ret, 588)
    return ret


@cache('step')
def installed_capacity_res_elec_delayed_1yr():
    """
    installed capacity res elec delayed 1yr



    component


    """
    
    ret = xr.DataArray(data=[x() for x in delays_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step],
        dims=installed_capacity_res_elec_tw().dims,
        coords=installed_capacity_res_elec_tw().coords)
    #test_nans(ret, 589)
    return ret


@cache('step')
def jobs_oxandxm_res_heat_per_techn():
    """
    jobs oxandxm res heat per techn



    component


    """
    
    ret = (installed_capacity_res_heatxdashxcom_tw() + installed_capacity_res_heatxdashxnc_tw()
            ) * employment_factors_oxandxm_res_heat() * m_per_t()
    #test_nans(ret, 590)
    return ret


@cache('step')
def d_jobs_new_installed_res_heat_per_techn():
    """
    d jobs new installed res heat per techn



    component


    """
    
    ret = (new_res_capacity_for_heatxdashxcom_tw() + replacement_res_for_heatxdashxcom_tw() +
            new_res_capacity_for_heatxdashxnc_tw() + replacement_res_for_heatxdashxnc_tw()
            ) * employment_factors_new_res_heat() * m_per_t()
    #test_nans(ret, 591)
    return ret


@cache('step')
def d_jobs_fuel_supply_solids_bioe():
    """
    d jobs fuel supply solids bioe



    component


    """
    
    ret = pes_solids_bioe_ej() * employment_factor_fuel_supply_solids_bioe() * 1000
    #test_nans(ret, 592)
    return ret


@cache('step')
def new_capacity_installed_growth_rate_res_elec():
    """
    new capacity installed growth rate res elec



    component


    """
    
    ret = -1 + functions.zidz(installed_capacity_res_elec_tw(),
                               installed_capacity_res_elec_delayed_1yr())
    #test_nans(ret, 593)
    return ret


@cache('run')
def employment_factor_fuel_supply_solids_bioe():
    """
    employment factor fuel supply solids bioe



    constant


    """
    
    ret = 0.029
    #test_nans(ret, 594)
    return ret


@cache('step')
def exogenous_growth_gtl():
    """
    exogenous growth gtl



    component


    """
    
    ret = functions.if_then_else(time() < 2015,
                                  hist_growth_gtl(),
                                  functions.if_then_else(abs(crash_programme_gtlxquestx() - 0) < equality_threshold,
                                                         p_gtl(),
                                                         functions.if_then_else(
                                                             functions.logical_and(abs(crash_programme_gtlxquestx() - 1) < equality_threshold,abundance_liquids() >= 1),
                                                             hist_growth_gtl(), p_gtl())))
    #test_nans(ret, 595)
    return ret


@cache('step')
def replacement_gtl():
    """
    replacement gtl



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  functions.if_then_else(
                                      abs(crash_programme_gtlxquestx() - 0) < equality_threshold, 0,
                                      functions.if_then_else(
                                          check_liquids() < - equality_threshold,
                                          constrain_liquids_exogenous_growthxquestx() * wear_gtl(),
                                          wear_gtl()))) * scarcity_conv_oil()
    #test_nans(ret, 596)
    return ret


@cache('run')
def crash_programme_gtlxquestx():
    """
    crash programme gtlxquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 597)
    return ret


@cache('step')
def exogenous_growth_ctl():
    """
    exogenous growth ctl



    component


    """
    
    ret = functions.if_then_else(time() < 2015,
                                  hist_growth_ctl(),
                                  functions.if_then_else(abs(crash_programme_ctlxquestx() - 0) < equality_threshold,
                                                         p_ctl(),
                                                         functions.if_then_else(
                                                             functions.logical_and(abs(crash_programme_ctlxquestx() - 1) < equality_threshold,abundance_liquids() >= 1),
                                                             hist_growth_ctl(), p_ctl())))
    #test_nans(ret, 598)
    return ret


@cache('step')
def replacement_ctl():
    """
    replacement ctl



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  functions.if_then_else(
                                      abs(crash_programme_ctlxquestx() - 0) < equality_threshold, 0,
                                      functions.if_then_else(
                                          check_liquids() < - equality_threshold,
                                          constrain_liquids_exogenous_growthxquestx() * wear_ctl(),
                                          wear_ctl()))) * scarcity_conv_oil()
    #test_nans(ret, 599)
    return ret


@cache('step')
def aux12():
    """
    aux12



    component


    """
    
    ret = delay_rurr_coal_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 600)
    return ret


@cache('step')
def coal_to_leave_underground():
    """
    coal to leave underground



    component


    """
    
    ret = functions.if_then_else(
        time() < start_policy_leave_in_ground_coal(), 0,
        share_rurr_coal_to_leave_underground() * rurr_coal_start_year_plg())
    #test_nans(ret, 601)
    return ret


@cache('step')
def rurr_coal_start_year_plg():
    """
    rurr coal start year plg



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_coal(),
                                  rurr_coal(), aux12())
    #test_nans(ret, 602)
    return ret


@cache('step')
def rurr_conv_oil_until_start_year_plg():
    """
    rurr conv oil until start year plg



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_conv_oil(),
                                  rurr_conv_oil(), aux6())
    #test_nans(ret, 603)
    return ret


@cache('step')
def conv_gas_to_leave_underground():
    """
    conv gas to leave underground



    component


    """
    
    ret = functions.if_then_else(
        time() < start_policy_leave_in_ground_conv_gas(), 0,
        rurr_conv_gas_until_start_year_plg() * share_rurr_conv_gas_to_leave_underground())
    #test_nans(ret, 604)
    return ret


@cache('step')
def conv_oil_to_leave_underground():
    """
    conv oil to leave underground



    component


    """
    
    ret = functions.if_then_else(
        time() < start_policy_leave_in_ground_conv_oil(), 0,
        rurr_conv_oil_until_start_year_plg() * share_rurr_conv_oil_to_leave_underground())
    #test_nans(ret, 605)
    return ret


@cache('step')
def rurr_unconv_oil_until_start_year_plg():
    """
    rurr unconv oil until start year plg



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_unconv_oil(),
                                  rurr_unconv_oil_ej(), aux7())
    #test_nans(ret, 606)
    return ret


@cache('step')
def rurr_unconv_gas_until_start_year_plg():
    """
    rurr unconv gas until start year plg



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_unconv_gas(),
                                  rurr_unconv_gas(), aux10())
    #test_nans(ret, 607)
    return ret


@cache('step')
def unconv_gas_to_leave_underground():
    """
    unconv gas to leave underground



    component


    """
    
    ret = functions.if_then_else(
        time() < start_policy_leave_in_ground_unconv_gas(), 0,
        rurr_unconv_gas_until_start_year_plg() * share_rurr_unconv_gas_to_leave_underground())
    #test_nans(ret, 608)
    return ret


@cache('step')
def tot_agg_gas_to_leave_underground():
    """
    tot agg gas to leave underground



    component


    """
    
    ret = functions.if_then_else(
        time() < start_policy_leave_in_ground_tot_agg_gas(), 0,
        rurr_tot_gas_until_start_year_plg() * share_rurr_tot_agg_gas_to_leave_underground())
    #test_nans(ret, 609)
    return ret


@cache('step')
def unconv_oil_to_leave_underground():
    """
    unconv oil to leave underground



    component


    """
    
    ret = functions.if_then_else(
        time() < start_policy_leave_in_ground_unconv_oil(), 0,
        rurr_unconv_oil_until_start_year_plg() * share_rurr_unconv_oil_to_leave_underground())
    #test_nans(ret, 610)
    return ret


@cache('step')
def aux11():
    """
    aux11



    component


    """
    
    ret = delay_rurr_tot_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 611)
    return ret


@cache('step')
def tot_agg_oil_to_leave_underground():
    """
    tot agg oil to leave underground



    component


    """
    
    ret = functions.if_then_else(
        time() < start_policy_leave_in_ground_tot_agg_oil(), 0,
        rurr_tot_oil_until_start_year_plg() * share_rurr_tot_agg_oil_to_leave_underground())
    #test_nans(ret, 612)
    return ret


@cache('step')
def aux6():
    """
    aux6



    component


    """
    
    ret = delay_rurr_conv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 613)
    return ret


@cache('step')
def aux7():
    """
    aux7



    component


    """
    
    ret = delay_rurr_unconv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 614)
    return ret


@cache('step')
def aux8():
    """
    aux8



    component


    """
    
    ret = delay_rurr_tot_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 615)
    return ret


@cache('step')
def aux9():
    """
    aux9



    component


    """
    
    ret = delay_rurr_conv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 616)
    return ret


@cache('step')
def rurr_conv_gas_until_start_year_plg():
    """
    rurr conv gas until start year plg



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_conv_gas(),
                                  rurr_conv_gas(), aux9())
    #test_nans(ret, 617)
    return ret


@cache('step')
def aux10():
    """
    aux10



    component


    """
    
    ret = delay_rurr_unconv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 618)
    return ret


@cache('step')
def rurr_tot_gas_until_start_year_plg():
    """
    rurr tot gas until start year plg



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_gas(),
                                  rurr_tot_agg_gas(), aux11())
    #test_nans(ret, 619)
    return ret


@cache('step')
def rurr_tot_oil_until_start_year_plg():
    """
    rurr tot oil until start year plg



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_oil(),
                                  rurr_tot_agg_oil(), aux8())
    #test_nans(ret, 620)
    return ret


@cache('step')
def required_fed_by_fuel_before_heat_correction():
    """
    required fed by fuel before heat correction



    component


    """
    
    ret = (required_fed_sectors_by_fuel() + households_final_energy_demand()
            ) * eroi_fc_system_from_x2015x()
    #test_nans(ret, 621)
    return ret


@cache('step')
def share_elec_demand_covered_by_res():
    """
    share elec demand covered by res



    component


    """
    
    ret = functions.if_then_else(
        total_fe_elec_demand_twh() > 0,
        np.minimum(1, fe_tot_generation_all_res_elec_twh() / total_fe_elec_demand_twh()), 0.5)
    #test_nans(ret, 622)
    return ret


@cache('step')
def rurr_conv_gas():
    """
    rurr conv gas



    component


    """
    
    ret = integ_rurr_conv_gas()
    #test_nans(ret, 623)
    return ret


@cache('step')
def rurr_unconv_gas():
    """
    rurr unconv gas



    component


    """
    
    ret = integ_rurr_unconv_gas()
    #test_nans(ret, 624)
    return ret


@cache('step')
def rurr_conv_oil():
    """
    rurr conv oil



    component


    """
    
    ret = integ_rurr_conv_oil()
    #test_nans(ret, 625)
    return ret


@cache('step')
def variation_share_transmxandxdistr_losses_elec():
    """
    variation share transmxandxdistr losses elec



    component


    """
    
    ret = share_transmxandxdistr_elec_losses_initial() * (
        0.0115 * np.exp(4.2297 * share_res_electricity_generation()) - 0.00251)
    #test_nans(ret, 626)
    return ret


@cache('step')
def share_demand_solids_in_transport():
    """
    share demand solids in transport



    component


    """
    
    ret = share_demand_by_fuel_in_transport().loc['solids'].values
    #test_nans(ret, 627)
    return ret


@cache('step')
def share_demand_gas_in_transport():
    """
    share demand gas in transport



    component


    """
    
    ret = share_demand_by_fuel_in_transport().loc['gases'].values
    #test_nans(ret, 628)
    return ret


@cache('step')
def share_demand_by_fuel_in_transport():
    """
    share demand by fuel in transport



    component


    """
    
    ret = total_transport_fed_by_fuel() / transport_tfed()
    #test_nans(ret, 629)
    return ret


@cache('step')
def share_demand_electricity_in_transport():
    """
    share demand electricity in transport



    component


    """
    
    ret = share_demand_by_fuel_in_transport().loc['electricity'].values
    #test_nans(ret, 630)
    return ret


@cache('step')
def share_demand_heat_in_transport():
    """
    share demand heat in transport



    component


    """
    
    ret = share_demand_by_fuel_in_transport().loc['heat'].values
    #test_nans(ret, 631)
    return ret


@cache('step')
def share_demand_liquids_in_transport():
    """
    share demand liquids in transport



    component


    """
    
    ret = share_demand_by_fuel_in_transport().loc['liquids'].values
    #test_nans(ret, 632)
    return ret


@cache('step')
def co2_emissions_per_value_added():
    """
    co2 emissions per value added



    component


    """
    
    ret = functions.zidz(total_co2_emissions_gtco2(), gdp())
    #test_nans(ret, 633)
    return ret


@cache('run')
def liquids_per_x_bus():
    """
    liquids per x bus



    component


    """
    
    ret = energy_initial_inland_transport().loc['bus liq'].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    #test_nans(ret, 634)
    return ret


@cache('run')
def liquids_per_x_hv():
    """
    liquids per x hv



    component


    """
    
    ret = energy_initial_inland_transport().loc['hv liq'].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    #test_nans(ret, 635)
    return ret


@cache('run')
def liquids_per_x_lv():
    """
    liquids per x lv



    component


    """
    
    ret = energy_initial_inland_transport().loc['lv liq'].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    #test_nans(ret, 636)
    return ret


@cache('run')
def energy_per_x_train():
    """
    energy per x train



    component


    """
    
    ret = energy_initial_inland_transport().loc['train liq'].values * adjust_energy_for_transport_to_inland_transport(
    ) / initial_xt_inland()
    #test_nans(ret, 637)
    return ret


@cache('run')
def adjust_energy_for_transport_to_inland_transport():
    """
    adjust energy for transport to inland transport



    constant


    """
    
    ret = 0.3767
    #test_nans(ret, 638)
    return ret


@cache('step')
def effects_shortage_elec_on_evxdivxhib():
    """
    effects shortage elec on evxdivxhib



    component


    """
    
    ret = functions.if_then_else(abundance_electricity() > 0.8,
                                  ((abundance_electricity() - 0.8) * 5)**2, 0)
    #test_nans(ret, 639)
    return ret


@cache('step')
def hist_var_percent_h():
    """
    hist var percent h



    constant


    """
    
    ret = xr.DataArray(data=[0.0,
        functions.if_then_else(time() > 2005,
                               (percent_h_vehicles_initial().loc['hib 4wheels'].values - 0) / (t_hist_h_transp() - 2005), 0),
        functions.if_then_else(time() < 2005, 0,
                               (percent_h_vehicles_initial().loc['elec 4wheels'].values - 0) / (t_hist_h_transp() - 2005)),
        functions.if_then_else(time() < 2005, 0,
                               (percent_h_vehicles_initial().loc['gas 4wheels'].values - 0) / (t_hist_h_transp() - 2005)),0.0,
        functions.if_then_else(time() < 2005, 0,
                               (percent_h_vehicles_initial().loc['elec 2wheels'].values - 0) / (t_hist_h_transp() - 2005))],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 640)
    return ret


@cache('step')
def required_number_standard_batteries():
    """
    required number standard batteries



    component


    """
    
    ret = bat_number_2w() + bat_number_ev() + bat_number_hib() + 1
    #test_nans(ret, 641)
    return ret


@cache('step')
def transport_tfed_energy_intensity():
    """
    transport tfed energy intensity



    component


    """
    
    ret = functions.zidz(transport_tfed(), gdp())
    #test_nans(ret, 642)
    return ret


@cache('step')
def total_number_light_vehicles():
    """
    total number light vehicles



    component


    """
    
    ret = number_vehicles_h().loc['liq 4wheels'].values + number_vehicles_h().loc['hib 4wheels'].values + number_vehicles_h().loc['elec 4wheels'].values + number_vehicles_h().loc['gas 4wheels'].values + vehicles_inlandt().loc['lv liq'].values + vehicles_inlandt().loc['lv elec'].values + vehicles_inlandt().loc['lv hib'].values + vehicles_inlandt().loc['lv gas'].values
    #test_nans(ret, 643)
    return ret


@cache('step')
def transport_tfed():
    """
    transport tfed



    component


    """
    
    ret = functions.sum(total_transport_fed_by_fuel())
    #test_nans(ret, 644)
    return ret


@cache('step')
def transport_households_final_energy_demand():
    """
    transport households final energy demand



    component


    """
    
    ret = energy_intensity_of_households_transport() * household_demand_total() / 1e+006
    #test_nans(ret, 645)
    return ret


@cache('step')
def real_fe_consumption_liquids_ej():
    """
    real fe consumption liquids ej



    component


    """
    
    ret = (pes_liquids_ej() - other_liquids_required_ej()) * share_liquids_for_final_energy()
    #test_nans(ret, 646)
    return ret


@cache('step')
def co2_emissions_unconv_gas():
    """
    co2 emissions unconv gas



    component


    """
    
    ret = real_extraction_unconv_gas_emissions_relevant_ej() * gco2_per_mj_unconv_gas(
    ) * mj_per_ej() / g_per_gt()
    #test_nans(ret, 647)
    return ret


@cache('run')
def phasexdashxout_oil_for_electricityxquestx():
    """
    phasexdashxout oil for electricityxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 648)
    return ret


@cache('run')
def max_solar_on_land_mha():
    """
    max solar on land mha



    constant


    """
    
    ret = 100
    #test_nans(ret, 649)
    return ret


@cache('step')
def max_solar_pv_on_land_mha():
    """
    max solar pv on land mha



    component


    """
    
    ret = max_solar_on_land_mha() - surface_csp_mha()
    #test_nans(ret, 650)
    return ret


@cache('step')
def max_solar_pv_on_land_twe():
    """
    max solar pv on land twe



    component


    """
    
    ret = max_solar_pv_on_land_mha() * power_density_solar_pv()
    #test_nans(ret, 651)
    return ret


@cache('step')
def total_ch4_emissions_fossil_fuels():
    """
    total ch4 emissions fossil fuels



    component


    """
    
    ret = ch4_emissions_conv_gas_without_gtl() + ch4_emissions_unconv_gas(
    ) + ch4_emissions_coal_without_ctl() + ch4_emissions_oil() + ch4_emissions_ctl(
    ) + ch4_emissions_gtl()
    #test_nans(ret, 652)
    return ret


@cache('step')
def max_csp_on_land_mha():
    """
    max csp on land mha



    component


    """
    
    ret = max_solar_on_land_mha() - surface_solar_pv_mha()
    #test_nans(ret, 653)
    return ret


@cache('step')
def max_csp_twe():
    """
    max csp twe



    component


    """
    
    ret = max_csp_on_land_mha() * power_density_csp()
    #test_nans(ret, 654)
    return ret


@cache('run')
def gch4_per_mj_gtl():
    """
    gch4 per mj gtl



    constant


    """
    
    ret = 0.094
    #test_nans(ret, 655)
    return ret


@cache('step')
def ch4_emissions_gtl():
    """
    ch4 emissions gtl



    component


    """
    
    ret = ped_natx_gas_for_gtl_ej() * gch4_per_mj_gtl() * mj_per_ej() / g_per_mt()
    #test_nans(ret, 656)
    return ret


@cache('step')
def ch4_emissions_ctl():
    """
    ch4 emissions ctl



    component


    """
    
    ret = extraction_coal_for_ctl_ej() * gch4_per_mj_ctl() * mj_per_ej() / g_per_mt()
    #test_nans(ret, 657)
    return ret


@cache('run')
def phasexdashxout_oil_for_heatxdashxcomxquestx():
    """
    phasexdashxout oil for heatxdashxcomxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 658)
    return ret


@cache('run')
def gch4_per_mj_ctl():
    """
    gch4 per mj ctl



    constant


    """
    
    ret = 0.094
    #test_nans(ret, 659)
    return ret


@cache('run')
def g_per_gt():
    """
    g per gt



    constant


    """
    
    ret = 1000000000000000.0
    #test_nans(ret, 660)
    return ret


@cache('step')
def bioe_co2_emissions():
    """
    bioe co2 emissions



    component


    """
    
    ret = gco2_per_mj_conv_gas() * (
        oil_liquids_saved_by_biofuels_ej() + solid_biofuels_emissions_relevant_ej() +
        pes_tot_biogas_for_heatxdashxcom()) * mj_per_ej() / g_per_gt()
    #test_nans(ret, 661)
    return ret


@cache('run')
def maximum_2w():
    """
    maximum 2w



    component


    """
    
    ret = p_share_x2x_wheelers()
    #test_nans(ret, 662)
    return ret


@cache('run')
def p_h_vehicle():
    """
    p h vehicle



    component


    """
    
    ret = xr.DataArray(data=[
        (1 - p_share_x2x_wheelers()) * (-p_elec() - p_gas() - p_hyb()),
        p_hyb() * (1 - p_share_x2x_wheelers()),
        p_elec() * (1 - p_share_x2x_wheelers()),
        p_gas() * (1 - p_share_x2x_wheelers()),
        p_share_x2x_wheelers() * (1 - p_2we()),
        p_share_x2x_wheelers() * p_2we()],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 663)
    return ret


@cache('step')
def solid_biofuels_emissions_relevant_ej():
    """
    solid biofuels emissions relevant ej



    component


    """
    
    ret = pe_bioe_for_elec_generation_ej() + pes_res_for_heatxdashxcom_by_techn().loc['solid bioexdashxheat'].values + pes_res_for_heatxdashxnc_by_techn().loc['solid bioexdashxheat'].values
    #test_nans(ret, 664)
    return ret


@cache('run')
def eroistx1995x():
    """
    eroistx1995x



    constant


    """
    
    ret = 10.57
    #test_nans(ret, 665)
    return ret


@cache('step')
def eroist_system_until_x2015x():
    """
    eroist system until x2015x



    component


    """
    
    ret = functions.if_then_else(time() < 2015, eroist_system_delayed(), aux4())
    #test_nans(ret, 666)
    return ret


@cache('step')
def remaining_potential_elec_storage_by_res_techn():
    """
    remaining potential elec storage by res techn



    component


    """
    
    ret = functions.if_then_else(
        max_capacity_elec_storage() >= demand_storage_capacity(),
        (max_capacity_elec_storage() - demand_storage_capacity()) / max_capacity_elec_storage(), 0)
    #test_nans(ret, 667)
    return ret


@cache('run')
def cp_hydro_x2015x():
    """
    cp hydro x2015x



    constant


    """
    
    ret = 0.42302360615086493
    #test_nans(ret, 668)
    return ret


@cache('run')
def esoi_phs_full_potential():
    """
    esoi phs full potential



    component


    """
    
    ret = eroixdashxini_res_elec_dispatch().loc['hydro'].values * (cp_phs() / cp_hydro_x2015x())
    #test_nans(ret, 669)
    return ret


@cache('step')
def max_capacity_elec_storage():
    """
    max capacity elec storage



    component


    """
    
    ret = max_capacity_potential_phs() + used_ev_batteries_for_elec_storage()
    #test_nans(ret, 670)
    return ret


@cache('step')
def cc_impacts_feedback_shortage_coeff():
    """
    cc impacts feedback shortage coeff



    component


    """
    
    ret = (1 - share_exdashxlosses_cc())
    #test_nans(ret, 671)
    return ret


@cache('run')
def eolxdashxrr_minerals_alt_techn_res_vsx_total_economy():
    """
    eolxdashxrr minerals alt techn res vsx total economy



    constant


    """
    
    ret = 0.3333333333333333
    #test_nans(ret, 672)
    return ret


@cache('run')
def crash_programme_ctlxquestx():
    """
    crash programme ctlxquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 673)
    return ret


@cache('step')
def recycling_rates_minerals_rest():
    """
    recycling rates minerals rest



    component


    """
    
    ret = integ_recycling_rates_minerals_rest()
    #test_nans(ret, 674)
    return ret


@cache('run')
def p_elec():
    """
    p elec



    constant


    """
    
    ret = 0.0064
    #test_nans(ret, 675)
    return ret


@cache('step')
def percents_h_vehicles():
    """
    percents h vehicles



    component


    """
    
    ret = integ_percents_h_vehicles()
    #test_nans(ret, 676)
    return ret


@cache('run')
def t_hist_h_transp():
    """
    t hist h transp



    constant


    """
    
    ret = 2015
    #test_nans(ret, 677)
    return ret


@cache('step')
def bat_number_2w():
    """
    bat number 2w



    component


    """
    
    ret = number_vehicles_h().loc['elec 2wheels'].values * bateries_ratio_2w_e()
    #test_nans(ret, 678)
    return ret


@cache('run')
def bateries_ratio_2w_e():
    """
    bateries ratio 2w e



    constant


    """
    
    ret = 0.29780564263322884
    #test_nans(ret, 679)
    return ret


@cache('step')
def bat_number_ev():
    """
    bat number ev



    component


    """
    
    ret = number_vehicles_h().loc['elec 4wheels'].values + vehicles_inlandt().loc['lv elec'].values + vehicles_inlandt().loc['bus elec'].values * bateries_ratio_bus_e()
    #test_nans(ret, 680)
    return ret


@cache('step')
def bat_number_hib():
    """
    bat number hib



    component


    """
    
    ret = vehicles_inlandt().loc['lv hib'].values * bateries_ratio_hib_lv(
    ) + vehicles_inlandt().loc['hv hib'].values * bateries_ratio_hib_hv() + vehicles_inlandt().loc['bus hib'].values * bateries_ratio_hib_bus(
    ) + number_vehicles_h().loc['hib 4wheels'].values * bateries_ratio_hib_lv()
    #test_nans(ret, 681)
    return ret


@cache('run')
def bateries_ratio_bus_e():
    """
    bateries ratio bus e



    constant


    """
    
    ret = 9.801724137931034
    #test_nans(ret, 682)
    return ret


@cache('run')
def bateries_ratio_hib_bus():
    """
    bateries ratio hib bus



    constant


    """
    
    ret = 0.6573926316519808
    #test_nans(ret, 683)
    return ret


@cache('run')
def bateries_ratio_hib_hv():
    """
    bateries ratio hib hv



    constant


    """
    
    ret = 0.8382058284923753
    #test_nans(ret, 684)
    return ret


@cache('run')
def bateries_ratio_hib_lv():
    """
    bateries ratio hib lv



    constant


    """
    
    ret = 0.10223304298097888
    #test_nans(ret, 685)
    return ret


@cache('step')
def total_number_hybrid_light_vehicles():
    """
    total number hybrid light vehicles



    component


    """
    
    ret = number_vehicles_h().loc['hib 4wheels'].values + vehicles_inlandt().loc['lv hib'].values
    #test_nans(ret, 686)
    return ret


@cache('step')
def total_number_elec_light_vehicles():
    """
    total number elec light vehicles



    component


    """
    
    ret = number_vehicles_h().loc['elec 4wheels'].values + vehicles_inlandt().loc['lv elec'].values
    #test_nans(ret, 687)
    return ret


@cache('step')
def total_number_gas_light_vehicles():
    """
    total number gas light vehicles



    component


    """
    
    ret = number_vehicles_h().loc['gas 4wheels'].values + vehicles_inlandt().loc['lv gas'].values
    #test_nans(ret, 688)
    return ret


@cache('step')
def ev_batteries_tw():
    """
    ev batteries tw



    component


    """
    
    ret = batteries_evxplusxhibxplusx2we() * kw_per_battery_ev() / kwh_per_twh()
    #test_nans(ret, 689)
    return ret


@cache('step')
def var_percent_t_vehicles():
    """
    var percent t vehicles



    component


    """
    
    ret = xr.DataArray(data=[
        -adapt_var_inlandt().loc['hv hib'].values - adapt_var_inlandt().loc['hv gas'].values,
        adapt_var_inlandt().loc['hv hib'].values,
        adapt_var_inlandt().loc['hv gas'].values,
        -adapt_var_inlandt().loc['lv hib'].values - adapt_var_inlandt().loc['lv elec'].values - adapt_var_inlandt().loc['lv gas'].values,
        adapt_var_inlandt().loc['lv elec'].values,
        adapt_var_inlandt().loc['lv hib'].values,
        adapt_var_inlandt().loc['lv gas'].values,
        -adapt_var_inlandt().loc['bus elec'].values - adapt_var_inlandt().loc['bus hib'].values - adapt_var_inlandt().loc['bus gas'].values,
        adapt_var_inlandt().loc['bus elec'].values,
        adapt_var_inlandt().loc['bus hib'].values,
        adapt_var_inlandt().loc['bus gas'].values,
        -adapt_var_inlandt().loc['train elec'].values,
        adapt_var_inlandt().loc['train elec'].values],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 690)
    return ret


@cache('run')
def p_train_elec():
    """
    p train elec



    constant


    """
    
    ret = 0.2
    #test_nans(ret, 691)
    return ret


@cache('run')
def p_hv_gas():
    """
    p hv gas



    constant


    """
    
    ret = 0.00045
    #test_nans(ret, 692)
    return ret


@cache('run')
def t_ini_inlandt():
    """
    t ini inlandt



    constant


    """
    
    ret = 2020
    #test_nans(ret, 693)
    return ret


@cache('run')
def p_inlandt():
    """
    p inlandt



    component


    """
    
    ret = xr.DataArray(data=[
        -p_hv_gas() - p_hv_hyb(),
        p_hv_hyb(),
        p_hv_gas(),
        -p_lv_elec() - p_lv_hyb() - p_lv_gas(),
        p_lv_elec(),
        p_lv_hyb(),
        p_lv_gas(),
        -p_bus_hyb() - p_bus_gas() - p_bus_elec(),
        p_bus_elec(),
        p_bus_hyb(),
        p_bus_gas(),
        -p_train_elec(),
        p_train_elec()],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 694)
    return ret


@cache('run')
def activate_policy_inlandt():
    """
    activate policy inlandt



    constant


    """
    
    ret = 1
    #test_nans(ret, 695)
    return ret


@cache('run')
def p_lv_elec():
    """
    p lv elec



    constant


    """
    
    ret = 0.00074
    #test_nans(ret, 696)
    return ret


@cache('run')
def p_lv_hyb():
    """
    p lv hyb



    constant


    """
    
    ret = 0.00036
    #test_nans(ret, 697)
    return ret


@cache('run')
def p_bus_elec():
    """
    p bus elec



    constant


    """
    
    ret = 0
    #test_nans(ret, 698)
    return ret


@cache('run')
def p_bus_gas():
    """
    p bus gas



    constant


    """
    
    ret = 0
    #test_nans(ret, 699)
    return ret


@cache('run')
def p_bus_hyb():
    """
    p bus hyb



    constant


    """
    
    ret = 0
    #test_nans(ret, 700)
    return ret


@cache('step')
def percents_bus():
    """
    percents bus



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.0,
        percent_t_vehicles().loc['lv hib'].values,0.0,
        percent_t_vehicles().loc['bus liq'].values,
        percent_t_vehicles().loc['bus elec'].values,
        percent_t_vehicles().loc['bus hib'].values,
        percent_t_vehicles().loc['bus gas'].values,0.0,0.0],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 701)
    return ret


@cache('step')
def percents_train():
    """
    percents train



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.0,
        percent_t_vehicles().loc['lv hib'].values,0.0,0.0,0.0,0.0,0.0,
        percent_t_vehicles().loc['train liq'].values,
        percent_t_vehicles().loc['train elec'].values],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 702)
    return ret


@cache('run')
def p_hv_hyb():
    """
    p hv hyb



    constant


    """
    
    ret = 0.00045
    #test_nans(ret, 703)
    return ret


@cache('step')
def adapt_var_inlandt():
    """
    adapt var inlandt



    component


    """
    
    ret = aux_p_inland_transp() * efects_shortage_inlandt()
    #test_nans(ret, 704)
    return ret


@cache('step')
def percents_lv():
    """
    percents lv



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,
        percent_t_vehicles().loc['lv liq'].values,
        percent_t_vehicles().loc['lv elec'].values,
        percent_t_vehicles().loc['lv hib'].values,
        percent_t_vehicles().loc['lv gas'].values,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 705)
    return ret


@cache('step')
def var_i_inland_elec():
    """
    var i inland elec



    component


    """
    
    ret = energy_per_x_t().loc['lv elec'].values * var_percent_t_vehicles().loc['lv elec'].values + energy_per_x_t().loc['train elec'].values * var_percent_t_vehicles().loc['train elec'].values + energy_per_x_t().loc['bus elec'].values * var_percent_t_vehicles().loc['bus elec'].values
    #test_nans(ret, 706)
    return ret


@cache('step')
def var_i_inlandt_gas():
    """
    var i inlandt gas



    component


    """
    
    ret = energy_per_x_t().loc['hv gas'].values * var_percent_t_vehicles().loc['hv gas'].values + energy_per_x_t().loc['bus gas'].values * var_percent_t_vehicles().loc['bus gas'].values + energy_per_x_t().loc['lv gas'].values * var_percent_t_vehicles().loc['lv gas'].values
    #test_nans(ret, 707)
    return ret


@cache('step')
def var_i_inlandt_liq():
    """
    var i inlandt liq



    component


    """
    
    ret = energy_per_x_t().loc['hv liq'].values * var_percent_t_vehicles().loc['hv liq'].values + energy_per_x_t().loc['lv liq'].values * var_percent_t_vehicles().loc['lv liq'].values + energy_per_x_t().loc['bus liq'].values * var_percent_t_vehicles().loc['bus liq'].values + energy_per_x_t().loc['hv liq'].values * var_percent_t_vehicles().loc['hv hib'].values + energy_per_x_t().loc['lv liq'].values * var_percent_t_vehicles().loc['lv hib'].values + energy_per_x_t().loc['bus liq'].values * var_percent_t_vehicles().loc['bus hib'].values + energy_per_x_t().loc['train liq'].values * var_percent_t_vehicles().loc['train liq'].values
    #test_nans(ret, 708)
    return ret


@cache('step')
def shares_available_t():
    """
    shares available t



    component


    """
    
    ret = xr.DataArray(data=[
        1 - (percent_t_vehicles().loc['hv hib'].values + percent_t_vehicles().loc['hv gas'].values),
        1 - (percent_t_vehicles().loc['hv hib'].values + percent_t_vehicles().loc['hv gas'].values),
        1 - (percent_t_vehicles().loc['hv hib'].values + percent_t_vehicles().loc['hv gas'].values),
        1 - (percent_t_vehicles().loc['lv elec'].values + percent_t_vehicles().loc['lv hib'].values + percent_t_vehicles().loc['lv gas'].values),
        1 - (percent_t_vehicles().loc['lv elec'].values + percent_t_vehicles().loc['lv hib'].values + percent_t_vehicles().loc['lv gas'].values),
        1 - (percent_t_vehicles().loc['lv elec'].values + percent_t_vehicles().loc['lv hib'].values + percent_t_vehicles().loc['lv gas'].values),
        1 - (percent_t_vehicles().loc['lv elec'].values + percent_t_vehicles().loc['lv hib'].values + percent_t_vehicles().loc['lv gas'].values),
        1 - (percent_t_vehicles().loc['bus elec'].values + percent_t_vehicles().loc['bus hib'].values + percent_t_vehicles().loc['bus gas'].values),
        1 - (percent_t_vehicles().loc['bus elec'].values + percent_t_vehicles().loc['bus hib'].values + percent_t_vehicles().loc['bus gas'].values),
        1 - (percent_t_vehicles().loc['bus elec'].values + percent_t_vehicles().loc['bus hib'].values + percent_t_vehicles().loc['bus gas'].values),
        1 - (percent_t_vehicles().loc['bus elec'].values + percent_t_vehicles().loc['bus hib'].values + percent_t_vehicles().loc['bus gas'].values),
        1 - percent_t_vehicles().loc['train elec'].values,
        1 - percent_t_vehicles().loc['train elec'].values],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 709)
    return ret


@cache('step')
def vehicles_inlandt():
    """
    vehicles inlandt



    component


    """
    
    ret = percent_t_vehicles() * real_total_output_inland_transport(
    ) * nx0_vehicles_per_xinland_t()
    #test_nans(ret, 710)
    return ret


@cache('step')
def percent_t_vehicles():
    """
    percent t vehicles



    component


    """
    
    ret = integ_percent_t_vehicles()
    #test_nans(ret, 711)
    return ret


@cache('step')
def efects_shortage_inlandt():
    """
    efects shortage inlandt



    constant


    """
    
    ret = xr.DataArray(data=[1.0,1.0,
        effects_shortage_gas(),1.0,
        effects_shortage_elec_on_ev(),1.0,
        effects_shortage_gas(),1.0,
        effects_shortage_elec_on_ev(),1.0,
        effects_shortage_gas(),1.0,
        effects_shortage_elec_on_ev()],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 712)
    return ret


@cache('run')
def p_lv_gas():
    """
    p lv gas



    constant


    """
    
    ret = 0.01597
    #test_nans(ret, 713)
    return ret


@cache('step')
def percents_hv():
    """
    percents hv



    component


    """
    
    ret = xr.DataArray(data=[
        percent_t_vehicles().loc['hv liq'].values,
        percent_t_vehicles().loc['hv hib'].values,
        percent_t_vehicles().loc['hv gas'].values,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 714)
    return ret


@cache('run')
def initial_xt_inland():
    """
    initial xt inland



    constant


    """
    
    ret = 2.221228
    #test_nans(ret, 715)
    return ret


@cache('run')
def nx_bus_inlandt():
    """
    nx bus inlandt



    component


    """
    
    ret = (n_vehicles_inland_t_x0x().loc['bus liq'].values + n_vehicles_inland_t_x0x().loc['bus hib'].values + n_vehicles_inland_t_x0x().loc['bus gas'].values +
            n_vehicles_inland_t_x0x().loc['bus elec'].values) / initial_xt_inland()
    #test_nans(ret, 716)
    return ret


@cache('run')
def nx_hv_inland_t():
    """
    nx hv inland t



    component


    """
    
    ret = (n_vehicles_inland_t_x0x().loc['hv liq'].values + n_vehicles_inland_t_x0x().loc['hv hib'].values + n_vehicles_inland_t_x0x().loc['hv gas'].values
            ) / initial_xt_inland()
    #test_nans(ret, 717)
    return ret


@cache('run')
def nx_lv_inland_t():
    """
    nx lv inland t



    component


    """
    
    ret = (n_vehicles_inland_t_x0x().loc['lv liq'].values + n_vehicles_inland_t_x0x().loc['lv elec'].values + n_vehicles_inland_t_x0x().loc['lv hib'].values +
            n_vehicles_inland_t_x0x().loc['lv gas'].values) / initial_xt_inland()
    #test_nans(ret, 718)
    return ret


@cache('run')
def n_vehicles_inland_t_x0x():
    """
    n vehicles inland t x0x



    constant


    """
    
    ret = xr.DataArray(data=[58453889.6747974,26420.04832386,26420.04832386,209702761.0,159229.97493238,76996.06181609,3406478.98166824,15600000.0,0.0,0.0,0.0,1.0,1.0],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 719)
    return ret


@cache('run')
def energy_initial_inland_transport():
    """
    energy initial inland transport



    constant


    """
    
    ret = xr.DataArray(data=[22.28147355,0.0,0.0,8.10305879,0.0,0.0,0.0,2.66129086,0.0,0.0,0.0,1.30366657,0.32591664],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 720)
    return ret


@cache('run')
def energy_per_x_t():
    """
    energy per x t



    component


    """
    
    ret = xr.DataArray(data=[
        liquids_per_x_hv() * saving_ratios_v().loc['hv liq'].values,
        liquids_per_x_hv() * saving_ratios_v().loc['hv hib'].values,
        liquids_per_x_hv() * saving_ratios_v().loc['hv gas'].values,
        liquids_per_x_lv() * saving_ratios_v().loc['lv liq'].values,
        liquids_per_x_lv() * saving_ratios_v().loc['lv elec'].values,
        liquids_per_x_lv() * saving_ratios_v().loc['lv hib'].values,
        liquids_per_x_lv() * saving_ratios_v().loc['lv gas'].values,
        liquids_per_x_bus() * saving_ratios_v().loc['bus liq'].values,
        liquids_per_x_bus() * saving_ratios_v().loc['bus elec'].values,
        liquids_per_x_bus() * saving_ratios_v().loc['bus hib'].values,
        liquids_per_x_bus() * saving_ratios_v().loc['bus gas'].values,
        energy_per_x_train() * 0.8,
        energy_per_x_train() * 0.2],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 721)
    return ret


@cache('run')
def nx_train_inland_t():
    """
    nx train inland t



    component


    """
    
    ret = 1 / initial_xt_inland()
    #test_nans(ret, 722)
    return ret


@cache('run')
def a2_coef_th():
    """
    a2 coef th



    component


    """
    
    ret = electricity_2we() / (demand_h() * percent_h_vehicles_initial().loc['elec 2wheels'].values * saving_ratio_2we())
    #test_nans(ret, 723)
    return ret


@cache('step')
def h_gas_adapt_growth():
    """
    h gas adapt growth



    component


    """
    
    ret = h_gas_initial_growth() * effects_shortage_gas_h_veh()
    #test_nans(ret, 724)
    return ret


@cache('step')
def h_hyb_adapt_growth():
    """
    h hyb adapt growth



    component


    """
    
    ret = h_hyb_initial_growth()
    #test_nans(ret, 725)
    return ret


@cache('run')
def activate_policy_h_transp():
    """
    activate policy h transp



    constant


    """
    
    ret = 1
    #test_nans(ret, 726)
    return ret


@cache('run')
def initial_energy_intensity_of_households_transport_x2009x():
    """
    initial energy intensity of households transport x2009x



    constant


    """
    
    ret = xr.DataArray(data=[0.00472,0.0,1.245,0.01711,0.0],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 727)
    return ret


@cache('step')
def h_ev_adapt_growth():
    """
    h ev adapt growth



    component


    """
    
    ret = h_elec_initial_growth() * effects_shortage_elec_on_ev()
    #test_nans(ret, 728)
    return ret


@cache('step')
def var_ih_e2():
    """
    var ih e2



    component


    """
    
    ret = a1_coef_th() * var_percents_h_vehicles().loc['elec 4wheels'].values * saving_ratios_v().loc['lv elec'].values + a2_coef_th(
    ) * var_percents_h_vehicles().loc['elec 2wheels'].values * saving_ratio_2we()
    #test_nans(ret, 729)
    return ret


@cache('step')
def var_ih_gas2():
    """
    var ih gas2



    component


    """
    
    ret = a1_coef_th() * var_percents_h_vehicles().loc['gas 4wheels'].values * saving_ratios_v().loc['lv gas'].values
    #test_nans(ret, 730)
    return ret


@cache('run')
def a1_coef_th():
    """
    a1 coef th



    component


    """
    
    ret = ((liq_4w() / demand_h()) - (a2_coef_th() * percent_2w_liq())) / percent_4w_liq()
    #test_nans(ret, 731)
    return ret


@cache('step')
def var_ih_liq2():
    """
    var ih liq2



    component


    """
    
    ret = a1_coef_th() * var_percents_h_vehicles().loc['liq 4wheels'].values + a1_coef_th() * var_percents_h_vehicles().loc['hib 4wheels'].values + a2_coef_th() * var_percents_h_vehicles().loc['liq 2wheels'].values
    #test_nans(ret, 732)
    return ret


@cache('run')
def percent_h_vehicles_initial():
    """
    percent h vehicles initial



    constant


    """
    
    ret = xr.DataArray(data=[0.64285241,0.00161276,0.00095159,0.02209406,0.26410155,0.06838764],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 733)
    return ret


@cache('step')
def energy_intensity_of_households_transport():
    """
    energy intensity of households transport



    component


    """
    
    ret = integ_energy_intensity_of_households_transport()
    #test_nans(ret, 734)
    return ret


@cache('step')
def h_2we_adapt_growth():
    """
    h 2we adapt growth



    component


    """
    
    ret = h_2w_initial_growth()
    #test_nans(ret, 735)
    return ret


@cache('run')
def liq_4w():
    """
    liq 4w



    constant


    """
    
    ret = 41.97111631434505
    #test_nans(ret, 736)
    return ret


@cache('run')
def percent_2w_liq():
    """
    percent 2w liq



    constant


    """
    
    ret = 0.2641015512843279
    #test_nans(ret, 737)
    return ret


@cache('run')
def percent_4w_liq():
    """
    percent 4w liq



    constant


    """
    
    ret = 0.642852406005333
    #test_nans(ret, 738)
    return ret


@cache('run')
def electricity_2we():
    """
    electricity 2we



    constant


    """
    
    ret = 0.3420237795619384
    #test_nans(ret, 739)
    return ret


@cache('run')
def n_vehicles_h():
    """
    n vehicles h



    constant


    """
    
    ret = 2534633971.758305
    #test_nans(ret, 740)
    return ret


@cache('run')
def demand_h():
    """
    demand h



    constant


    """
    
    ret = 30.3
    #test_nans(ret, 741)
    return ret


@cache('run')
def ratio_n_veh_demand_h():
    """
    ratio n veh demand h



    component


    """
    
    ret = n_vehicles_h() / demand_h()
    #test_nans(ret, 742)
    return ret


@cache('step')
def percents_2w_h_vehicles():
    """
    percents 2w h vehicles



    component


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,
        percents_h_vehicles().loc['liq 2wheels'].values / (percents_h_vehicles().loc['elec 2wheels'].values + percents_h_vehicles().loc['liq 2wheels'].values),
        percents_h_vehicles().loc['elec 2wheels'].values / (percents_h_vehicles().loc['elec 2wheels'].values + percents_h_vehicles().loc['liq 2wheels'].values)],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 743)
    return ret


@cache('step')
def percents_4w_h_vehicles():
    """
    percents 4w h vehicles



    component


    """
    
    ret = xr.DataArray(data=[
        percents_h_vehicles().loc['liq 4wheels'].values / (percents_h_vehicles().loc['elec 4wheels'].values + percents_h_vehicles().loc['hib 4wheels'].values +
                                 percents_h_vehicles().loc['gas 4wheels'].values + percents_h_vehicles().loc['liq 4wheels'].values),
        percents_h_vehicles().loc['hib 4wheels'].values / (percents_h_vehicles().loc['elec 4wheels'].values + percents_h_vehicles().loc['hib 4wheels'].values +
                                 percents_h_vehicles().loc['gas 4wheels'].values + percents_h_vehicles().loc['liq 4wheels'].values),
        percents_h_vehicles().loc['elec 4wheels'].values / (percents_h_vehicles().loc['elec 4wheels'].values + percents_h_vehicles().loc['hib 4wheels'].values +
                                 percents_h_vehicles().loc['gas 4wheels'].values + percents_h_vehicles().loc['liq 4wheels'].values),
        percents_h_vehicles().loc['gas 4wheels'].values / (percents_h_vehicles().loc['elec 4wheels'].values + percents_h_vehicles().loc['hib 4wheels'].values +
                                 percents_h_vehicles().loc['gas 4wheels'].values + percents_h_vehicles().loc['liq 4wheels'].values),0.0,0.0],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 744)
    return ret


@cache('run')
def p_2we():
    """
    p 2we



    constant


    """
    
    ret = 0.9254
    #test_nans(ret, 745)
    return ret


@cache('step')
def max_percent_x2x_wheels():
    """
    max percent x2x wheels



    component


    """
    
    ret = integ_max_percent_x2x_wheels()
    #test_nans(ret, 746)
    return ret


@cache('step')
def max_percent_x4x_wheels():
    """
    max percent x4x wheels



    component


    """
    
    ret = integ_max_percent_x4x_wheels()
    #test_nans(ret, 747)
    return ret


@cache('step')
def aux_hist_h():
    """
    aux hist h



    component


    """
    
    ret = xr.DataArray(data=[
        -hist_var_percent_h().loc['hib 4wheels'].values - hist_var_percent_h().loc['elec 4wheels'].values - hist_var_percent_h().loc['gas 4wheels'].values,
        hist_var_percent_h().loc['hib 4wheels'].values,
        hist_var_percent_h().loc['elec 4wheels'].values,
        hist_var_percent_h().loc['gas 4wheels'].values,
        -hist_var_percent_h().loc['elec 2wheels'].values,
        hist_var_percent_h().loc['elec 2wheels'].values],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 748)
    return ret


@cache('run')
def p_gas():
    """
    p gas



    constant


    """
    
    ret = 0.1489
    #test_nans(ret, 749)
    return ret


@cache('run')
def p_share_x2x_wheelers():
    """
    p share x2x wheelers



    constant


    """
    
    ret = 0.3325
    #test_nans(ret, 750)
    return ret


@cache('run')
def p_hyb():
    """
    p hyb



    constant


    """
    
    ret = 0.0108
    #test_nans(ret, 751)
    return ret


@cache('run')
def initial_2w_percent():
    """
    initial 2w percent



    constant


    """
    
    ret = 0.33248919227982615
    #test_nans(ret, 752)
    return ret


@cache('step')
def share_available_2w():
    """
    share available 2w



    component


    """
    
    ret = (max_percent_x2x_wheels() - percents_h_vehicles().loc['elec 2wheels'].values) / max_percent_x2x_wheels()
    #test_nans(ret, 753)
    return ret


@cache('step')
def rate_4w_to_2w():
    """
    rate 4w to 2w



    component


    """
    
    ret = policy_2wheels()
    #test_nans(ret, 754)
    return ret


@cache('run')
def share_feh_over_fed_by_final_fuel():
    """
    share feh over fed by final fuel



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,
        share_feh_over_fed_oil(),
        share_feh_over_fed_natx_gas(),
        (share_feh_over_fed_coal() + share_feh_over_fed_solid_bioe())],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 755)
    return ret


@cache('run')
def share_feh_over_fed_oil():
    """
    share feh over fed oil



    constant


    """
    
    ret = 0.24447829398324447
    #test_nans(ret, 756)
    return ret


@cache('run')
def share_feh_over_fed_coal():
    """
    share feh over fed coal



    constant


    """
    
    ret = 0.39271002371738867
    #test_nans(ret, 757)
    return ret


@cache('run')
def share_feh_over_fed_natx_gas():
    """
    share feh over fed natx gas



    constant


    """
    
    ret = 0.768348623853211
    #test_nans(ret, 758)
    return ret


@cache('step')
def pes_oil_ej_delayed():
    """
    pes oil ej delayed



    component


    """
    
    ret = delay_pes_oil_ej_roundtime_step__time_step___time_step_1395_time_step__time_step()
    #test_nans(ret, 759)
    return ret


@cache('step')
def energy_required_for_material_consumption_for_ev_batteries():
    """
    energy required for material consumption for ev batteries



    component


    """
    
    ret = materials_required_for_ev_batteries_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    #test_nans(ret, 760)
    return ret


@cache('step')
def households_final_energy_demand():
    """
    households final energy demand



    component


    """
    
    ret = household_demand_total() * energy_intensity_of_households() / 1e+006
    #test_nans(ret, 761)
    return ret


@cache('step')
def change_total_intensity_to_rest():
    """
    change total intensity to rest



    component


    """
    
    ret = xr.DataArray(data=[
        1 - functions.step(0.007, 2009),0.0,
        1 - functions.step(0.78, 2009),
        1 - functions.step(0.025, 2009),0.0],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 762)
    return ret


@cache('run')
def activate_transport_h_bottom_up_method():
    """
    activate transport h bottom up method



    constant


    """
    
    ret = 1
    #test_nans(ret, 763)
    return ret


@cache('step')
def variation_energy_intensity_of_households_transport():
    """
    variation energy intensity of households transport



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(time() > 2009, var_ih_e2(), 0),0.0,
        functions.if_then_else(time() < 2009, 0, var_ih_liq2()),
        functions.if_then_else(time() > 2009, var_ih_gas2(), 0),0.0],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 764)
    return ret


@cache('step')
def number_vehicles_h():
    """
    number vehicles h



    component


    """
    
    ret = ratio_n_veh_demand_h() * household_demand_total() * 1e-006 * percents_h_vehicles()
    #test_nans(ret, 765)
    return ret


@cache('step')
def effects_shortage_elec_on_ev():
    """
    effects shortage elec on ev



    component


    """
    
    ret = functions.if_then_else(abundance_electricity() > 0.8,
                                  ((abundance_electricity() - 0.8) * 5)**2, 0)
    #test_nans(ret, 766)
    return ret


@cache('step')
def effects_shortage_gas_h_veh():
    """
    effects shortage gas h veh



    component


    """
    
    ret = functions.if_then_else(abundance_gases() > 0.8, ((abundance_gases() - 0.8) * 5)**2, 0)
    #test_nans(ret, 767)
    return ret


@cache('run')
def xax_extraction_projection_minerals():
    """
    xax extraction projection minerals



    constant


    """
    
    ret = xr.DataArray(data=[0.0,1006851.44431989,0.0,119.69287194,0.0,0.0,561172.97577378,230741.48718745,0.0,0.0,0.0,0.0,0.0,0.0,10.11098981,0.0,0.0,0.0,12.05290211,80787194.16497986,0.0,0.0,0.0,0.0,804.07748379,0.0,302664.46064236,343350.03530151,0.0,0.0,5023.5108697,0.0,0.0,0.0,40510.95804217,0.0,0.0,0.0,73765.76663225,0.0,0.0,0.0,0.0,0.0,0.0,0.0,257.68585968,0.0,2274.62093105,0.0,0.0,0.0,0.0,102968.10545833,0.0,1274.7444251,0.0,207576.31076445],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 768)
    return ret


@cache('run')
def xbx_extraction_projection_minerals():
    """
    xbx extraction projection minerals



    constant


    """
    
    ret = xr.DataArray(data=[0.0,-24747508.84046872,0.0,12862.99276901,0.0,0.0,-13481662.07152069,1147112.83580538,0.0,0.0,0.0,0.0,0.0,0.0,-434.34489379,0.0,0.0,0.0,-286.28398042,-2777690170.0,0.0,0.0,0.0,0.0,-25209.40506256,0.0,-11844018.97356096,-8586601.90070709,0.0,0.0,-107292.91153227,0.0,0.0,0.0,-772331.36137761,0.0,0.0,0.0,-554097.26168084,0.0,0.0,0.0,0.0,0.0,0.0,0.0,4731.27412551,0.0,111474.4915326,0.0,0.0,0.0,0.0,-448351.45343147,0.0,-18947.84206194,0.0,-1819702.6572439],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 769)
    return ret


@cache('step')
def variation_minerals_extraction_rest():
    """
    variation minerals extraction rest



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        historical_variation_minerals_extraction_rest(),
        functions.if_then_else(minerals_extraction_projection_rest_cte_rr() > 0.01,
                               (xax_extraction_projection_minerals() *
                                (gdp() - gdp_delayed_1yr())), 0)) * mt_per_tonne()
    #test_nans(ret, 770)
    return ret


@cache('step')
def total_recycled_materials_for_other_mt():
    """
    total recycled materials for other mt



    component


    """
    
    ret = minerals_consumption_estimation_rest_cte_rr(
    ) - minerals_extraction_projection_rest_with_rr()
    #test_nans(ret, 771)
    return ret


@cache('step')
def share_minerals_consumption_alt_techn_vs_total_economy():
    """
    share minerals consumption alt techn vs total economy



    component


    """
    
    ret = functions.zidz(total_materials_required_for_res_elec_xplusx_ev_batteries_mt(),
                          (minerals_consumption_estimation_rest_cte_rr() +
                           total_materials_required_for_res_elec_xplusx_ev_batteries_mt()))
    #test_nans(ret, 772)
    return ret


@cache('run')
def labor_share_ctexquestx():
    """
    labor share ctexquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 773)
    return ret


@cache('run')
def initial_minerals_extraction_rest():
    """
    initial minerals extraction rest



    constant


    """
    
    ret = xr.DataArray(data=[0.0,19393200.0,0.0,18500.0,0.0,0.0,11999960.0,9992170.0,0.0,0.0,0.0,0.0,0.0,0.0,35.0,0.0,0.0,0.0,239.0,999928230.0,0.0,0.0,0.0,0.0,6300.0,0.0,2640000.0,7579970.0,0.0,0.0,126000.0,0.0,0.0,0.0,1039680.0,0.0,0.0,0.0,2710000.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,14600.0,0.0,188970.0,0.0,0.0,0.0,0.0,3814000.0,0.0,33700.0,0.0,7119990.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 774)
    return ret


@cache('step')
def minerals_extraction_projection_rest_cte_rr():
    """
    minerals extraction projection rest cte rr



    component


    """
    
    ret = integ_minerals_extraction_projection_rest_cte_rr()
    #test_nans(ret, 775)
    return ret


@cache('step')
def materials_to_extract_rest_mt():
    """
    materials to extract rest mt



    component


    """
    
    ret = minerals_extraction_projection_rest_with_rr()
    #test_nans(ret, 776)
    return ret


@cache('step')
def minerals_extraction_projection_rest_with_rr():
    """
    minerals extraction projection rest with rr



    component


    """
    
    ret = minerals_consumption_estimation_rest_cte_rr() * (1 - recycling_rates_minerals_rest())
    #test_nans(ret, 777)
    return ret


@cache('step')
def minerals_consumption_estimation_rest_cte_rr():
    """
    minerals consumption estimation rest cte rr



    component


    """
    
    ret = minerals_extraction_projection_rest_cte_rr() / (1 - current_recycling_rates_minerals())
    #test_nans(ret, 778)
    return ret


@cache('step')
def total_materials_required_for_res_elec_xplusx_ev_batteries_mt():
    """
    total materials required for res elec xplusx ev batteries mt



    component


    """
    
    ret = total_materials_required_for_ev_batteries() + total_materials_required_for_res_elec_mt()
    #test_nans(ret, 779)
    return ret


@cache('step')
def variation_nonxdashxenergy_use():
    """
    variation nonxdashxenergy use



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,
        functions.if_then_else(nonxdashxenergy_use_demand_by_final_fuel_ej().loc['liquids'].values > 0.01,
                               0.461414 * (gdp() - gdp_delayed_1yr()), 0),
        functions.if_then_else(nonxdashxenergy_use_demand_by_final_fuel_ej().loc['gases'].values > 0.01,
                               0.123925 * (gdp() - gdp_delayed_1yr()), 0),
        functions.if_then_else(nonxdashxenergy_use_demand_by_final_fuel_ej().loc['solids'].values > 0.01,
                               0.0797511 * (gdp() - gdp_delayed_1yr()), 0)],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 780)
    return ret


@cache('step')
def number_2w():
    """
    number 2w



    component


    """
    
    ret = number_vehicles_h().loc['liq 2wheels'].values + number_vehicles_h().loc['elec 2wheels'].values
    #test_nans(ret, 781)
    return ret


@cache('step')
def number_4w():
    """
    number 4w



    component


    """
    
    ret = number_vehicles_h().loc['liq 4wheels'].values + number_vehicles_h().loc['hib 4wheels'].values + number_vehicles_h().loc['elec 4wheels'].values + number_vehicles_h().loc['gas 4wheels'].values
    #test_nans(ret, 782)
    return ret


@cache('step')
def number_all():
    """
    number all



    component


    """
    
    ret = number_2w() + number_4w()
    #test_nans(ret, 783)
    return ret


@cache('step')
def sum_4w_shares():
    """
    sum 4w shares



    component


    """
    
    ret = percents_h_vehicles().loc['hib 4wheels'].values + percents_h_vehicles().loc['elec 4wheels'].values + percents_h_vehicles().loc['gas 4wheels'].values
    #test_nans(ret, 784)
    return ret


@cache('step')
def share_available_4w():
    """
    share available 4w



    component


    """
    
    ret = (max_percent_x4x_wheels() - sum_4w_shares()) / max_percent_x4x_wheels()
    #test_nans(ret, 785)
    return ret


@cache('step')
def percent_4w():
    """
    percent 4w



    component


    """
    
    ret = percents_h_vehicles().loc['liq 4wheels'].values + percents_h_vehicles().loc['hib 4wheels'].values + percents_h_vehicles().loc['elec 4wheels'].values + percents_h_vehicles().loc['gas 4wheels'].values
    #test_nans(ret, 786)
    return ret


@cache('run')
def saving_ratio_2we():
    """
    saving ratio 2we



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 787)
    return ret


@cache('step')
def percent_all():
    """
    percent all



    component


    """
    
    ret = percent_2w() + percent_4w()
    #test_nans(ret, 788)
    return ret


@cache('step')
def percent_2w():
    """
    percent 2w



    component


    """
    
    ret = percents_h_vehicles().loc['liq 2wheels'].values + percents_h_vehicles().loc['elec 2wheels'].values
    #test_nans(ret, 789)
    return ret


@cache('step')
def share_fed_coal_vs_nre_heatxdashxnc():
    """
    share fed coal vs nre heatxdashxnc



    component


    """
    
    ret = functions.zidz(fed_coal_for_heatxdashxnc(), fed_nre_for_heatxdashxnc())
    #test_nans(ret, 790)
    return ret


@cache('step')
def share_fed_gas_vs_nre_heatxdashxnc():
    """
    share fed gas vs nre heatxdashxnc



    component


    """
    
    ret = functions.zidz(fed_natx_gas_for_heatxdashxnc(), fed_nre_for_heatxdashxnc())
    #test_nans(ret, 791)
    return ret


@cache('step')
def fed_nre_for_heatxdashxnc():
    """
    fed nre for heatxdashxnc



    component


    """
    
    ret = fed_coal_for_heatxdashxnc() + fed_natx_gas_for_heatxdashxnc(
    ) + fed_oil_for_heatxdashxnc()
    #test_nans(ret, 792)
    return ret


@cache('step')
def share_fed_liquids_vs_nre_heatxdashxnc():
    """
    share fed liquids vs nre heatxdashxnc



    component


    """
    
    ret = functions.zidz(fed_oil_for_heatxdashxnc(), fed_nre_for_heatxdashxnc())
    #test_nans(ret, 793)
    return ret


@cache('step')
def required_fed_by_fuel():
    """
    required fed by fuel



    component


    """
    
    ret = xr.DataArray(data=[
        required_fed_by_fuel_before_heat_correction().loc['electricity'].values,
        required_fed_by_fuel_before_heat_correction().loc['heat'].values *
        (1 + ratio_fed_for_heatxdashxnc_vs_fed_for_heatxdashxcom()),
        required_fed_by_fuel_before_heat_correction().loc['liquids'].values * (1 - share_feh_over_fed_by_final_fuel().loc['liquids'].values),
        required_fed_by_fuel_before_heat_correction().loc['gases'].values * (1 - share_feh_over_fed_by_final_fuel().loc['gases'].values),
        required_fed_by_fuel_before_heat_correction().loc['solids'].values * (1 - share_feh_over_fed_by_final_fuel().loc['solids'].values)],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 794)
    return ret


@cache('step')
def bioe_gen_land_marg_available():
    """
    bioe gen land marg available



    component


    """
    
    ret = (max_peavail_potential_biofuels_marginal_lands() -
            potential_peavail_biofuels_land_marg_ej()
            ) / max_peavail_potential_biofuels_marginal_lands()
    #test_nans(ret, 795)
    return ret


@cache('step')
def potential_peavail_cellulosic_biofuel_ej():
    """
    potential peavail cellulosic biofuel ej



    component


    """
    
    ret = potential_pe_cellulosic_biofuel_ej() * conv_efficiency_from_npp_to_biofuels()
    #test_nans(ret, 796)
    return ret


@cache('step')
def potential_peavail_total_biofuels():
    """
    potential peavail total biofuels



    component


    """
    
    ret = potential_peavail_biofuels_2gen_land_compet_ej(
    ) + potential_peavail_biofuels_prod_3gen_ej() + potential_peavail_biofuels_land_marg_ej(
    ) + potential_peavail_cellulosic_biofuel_ej()
    #test_nans(ret, 797)
    return ret


@cache('step')
def biofuels_3gen_land_compet_available():
    """
    biofuels 3gen land compet available



    component


    """
    
    ret = (max_land_compet_biofuels_2gen() - land_compet_biofuels_3gen_mha()
            ) / max_land_compet_biofuels_2gen()
    #test_nans(ret, 798)
    return ret


@cache('step')
def fes_total_biofuels_production_ej():
    """
    fes total biofuels production ej



    component


    """
    
    ret = peavail_biofuels_2gen_land_compet_ej() + peavail_biofuels_3gen_land_compet_ej(
    ) + peavail_biofuels_land_marg_ej() + peavail_cellulosic_biofuel_ej()
    #test_nans(ret, 799)
    return ret


@cache('step')
def fes_total_biofuels_production_ej_x2x():
    """
    fes total biofuels production ej x2x



    component


    """
    
    ret = np.minimum(ped_liquids_ej(), potential_peavail_total_biofuels())
    #test_nans(ret, 800)
    return ret


@cache('step')
def fes_total_biofuels_production_mbxdivxd():
    """
    fes total biofuels production mbxdivxd



    component


    """
    
    ret = fes_total_biofuels_production_ej() * mbxdivxd_per_ejxdivxyear()
    #test_nans(ret, 801)
    return ret


@cache('step')
def peavail_biofuels_2gen_land_compet_ej():
    """
    peavail biofuels 2gen land compet ej



    component


    """
    
    ret = potential_peavail_biofuels_2gen_land_compet_ej() * (1 - share_biofuels_overcapacity())
    #test_nans(ret, 802)
    return ret


@cache('step')
def peavail_biofuels_3gen_land_compet_ej():
    """
    peavail biofuels 3gen land compet ej



    component


    """
    
    ret = potential_peavail_biofuels_prod_3gen_ej() * (1 - share_biofuels_overcapacity())
    #test_nans(ret, 803)
    return ret


@cache('step')
def peavail_biofuels_land_marg_ej():
    """
    peavail biofuels land marg ej



    component


    """
    
    ret = potential_peavail_biofuels_land_marg_ej() * (1 - share_biofuels_overcapacity())
    #test_nans(ret, 804)
    return ret


@cache('step')
def pe_biofuels_land_marg_ej():
    """
    pe biofuels land marg ej



    component


    """
    
    ret = peavail_biofuels_land_marg_ej() / conv_efficiency_from_npp_to_biofuels()
    #test_nans(ret, 805)
    return ret


@cache('step')
def pe_biofuels_prod_x2genxplusx3gen_ej():
    """
    pe biofuels prod x2genxplusx3gen ej



    component


    """
    
    ret = (peavail_biofuels_2gen_land_compet_ej() + peavail_biofuels_3gen_land_compet_ej()
            ) / conv_efficiency_from_npp_to_biofuels()
    #test_nans(ret, 806)
    return ret


@cache('step')
def pe_biomass_for_biofuels_production_ej():
    """
    pe biomass for biofuels production ej



    component


    """
    
    ret = pe_biofuels_land_marg_ej() + pe_cellulosic_biofuel_ej(
    ) + pe_biofuels_prod_x2genxplusx3gen_ej()
    #test_nans(ret, 807)
    return ret


@cache('step')
def pe_cellulosic_biofuel_ej():
    """
    pe cellulosic biofuel ej



    component


    """
    
    ret = potential_pe_cellulosic_biofuel_ej() * (1 - share_biofuels_overcapacity())
    #test_nans(ret, 808)
    return ret


@cache('step')
def max_peavail_biofuels_potential():
    """
    max peavail biofuels potential



    component


    """
    
    ret = max_peavail_potential_bioe_residues_for_cellulosic_biofuels(
    ) + max_peavail_potential_biofuels_x2xdashx3gen(
    ) + max_peavail_potential_biofuels_marginal_lands()
    #test_nans(ret, 809)
    return ret


@cache('run')
def start_year_biofuels_land_marg():
    """
    start year biofuels land marg



    constant


    """
    
    ret = 2020
    #test_nans(ret, 810)
    return ret


@cache('step')
def land_required_biofuels_land_marg():
    """
    land required biofuels land marg



    component


    """
    
    ret = potential_peavail_biofuels_land_marg_ej() * land_occupation_ratio_biofuels_marg_land(
    ) / conv_efficiency_from_npp_to_biofuels()
    #test_nans(ret, 811)
    return ret


@cache('run')
def max_peavail_potential_biofuels_marginal_lands():
    """
    max peavail potential biofuels marginal lands



    component


    """
    
    ret = bioe_potential_npp_marginal_lands() * conv_efficiency_from_npp_to_biofuels()
    #test_nans(ret, 812)
    return ret


@cache('step')
def share_biofuels_overcapacity():
    """
    share biofuels overcapacity



    component


    """
    
    ret = functions.zidz(
        (potential_peavail_total_biofuels() - fes_total_biofuels_production_ej_x2x()),
        potential_peavail_total_biofuels())
    #test_nans(ret, 813)
    return ret


@cache('step')
def total_land_requirements_renew_mha():
    """
    total land requirements renew mha



    component


    """
    
    ret = surface_solar_pv_mha() + surface_csp_mha() + surface_hydro_mha(
    ) + land_compet_required_dedicated_crops_for_biofuels() + land_required_biofuels_land_marg(
    ) + surface_onshore_wind_mha() * 0
    #test_nans(ret, 814)
    return ret


@cache('run')
def land_occupation_ratio_biofuels_marg_land():
    """
    land occupation ratio biofuels marg land



    constant


    """
    
    ret = 14.3
    #test_nans(ret, 815)
    return ret


@cache('step')
def peavail_cellulosic_biofuel_ej():
    """
    peavail cellulosic biofuel ej



    component


    """
    
    ret = pe_cellulosic_biofuel_ej() * efficiency_bioe_residues_to_cellulosic_liquids()
    #test_nans(ret, 816)
    return ret


@cache('step')
def potential_peavail_biofuels_land_marg_ej():
    """
    potential peavail biofuels land marg ej



    component


    """
    
    ret = integ_potential_peavail_biofuels_land_marg_ej()
    #test_nans(ret, 817)
    return ret


@cache('run')
def bioe_potential_npp_marginal_lands():
    """
    bioe potential npp marginal lands



    constant


    """
    
    ret = 27
    #test_nans(ret, 818)
    return ret


@cache('step')
def additional_pe_production_of_bioenergy_for_biofuels():
    """
    additional pe production of bioenergy for biofuels



    component


    """
    
    ret = pe_biomass_for_biofuels_production_ej() - oil_liquids_saved_by_biofuels_ej()
    #test_nans(ret, 819)
    return ret


@cache('step')
def oil_liquids_saved_by_biofuels_ej():
    """
    oil liquids saved by biofuels ej



    component


    """
    
    ret = fes_total_biofuels_production_ej()
    #test_nans(ret, 820)
    return ret


@cache('run')
def efficiency_bioe_residues_to_cellulosic_liquids():
    """
    efficiency bioe residues to cellulosic liquids



    component


    """
    
    ret = conv_efficiency_from_npp_to_biofuels()
    #test_nans(ret, 821)
    return ret


@cache('run')
def conv_efficiency_from_npp_to_biofuels():
    """
    conv efficiency from npp to biofuels



    constant


    """
    
    ret = 0.15
    #test_nans(ret, 822)
    return ret


@cache('step')
def pes_heat_res():
    """
    pes heat res



    component


    """
    
    ret = pes_tot_biogas_for_heatxdashxcom() + functions.sum(
        pes_res_for_heatxdashxcom_by_techn()) + functions.sum(pes_res_for_heatxdashxnc_by_techn())
    #test_nans(ret, 823)
    return ret


@cache('step')
def ped_nre_liquids():
    """
    ped nre liquids



    component


    """
    
    ret = np.maximum(0, ped_liquids_ej() - fes_total_biofuels_production_ej())
    #test_nans(ret, 824)
    return ret


@cache('step')
def potential_fes_ctlxplusxgtl_ej():
    """
    potential fes ctlxplusxgtl ej



    component


    """
    
    ret = ctl_potential_production() + gtl_potential_production()
    #test_nans(ret, 825)
    return ret


@cache('step')
def tfec_res_delayed_1yr():
    """
    tfec res delayed 1yr



    component


    """
    
    ret = delay_tfec_res_ej_round1__time_step___time_step_43_1__time_step()
    #test_nans(ret, 826)
    return ret


@cache('step')
def pe_supply_from_res_nonxdashxelec_without_trad_bioe_ej():
    """
    pe supply from res nonxdashxelec without trad bioe ej



    component


    """
    
    ret = pes_tot_res_for_heat() + fes_total_biofuels_production_ej() + pes_biogas_for_tfc()
    #test_nans(ret, 827)
    return ret


@cache('step')
def pes_gases():
    """
    pes gases



    component


    """
    
    ret = pes_natx_gas() + pes_biogas_for_tfc()
    #test_nans(ret, 828)
    return ret


@cache('step')
def fes_ctlxplusxgtl_ej():
    """
    fes ctlxplusxgtl ej



    component


    """
    
    ret = np.minimum(ped_nre_liquids(), potential_fes_ctlxplusxgtl_ej())
    #test_nans(ret, 829)
    return ret


@cache('run')
def activate_elf_all_scenxquestx():
    """
    activate elf all scenxquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 830)
    return ret


@cache('step')
def tpes_res_delayed_1yr():
    """
    tpes res delayed 1yr



    component


    """
    
    ret = delay_tpe_from_res_ej_round1__time_step___time_step_4803_1__time_step()
    #test_nans(ret, 831)
    return ret


@cache('step')
def annual_tpes_res_growth_rate():
    """
    annual tpes res growth rate



    component


    """
    
    ret = -1 + tpe_from_res_ej() / tpes_res_delayed_1yr()
    #test_nans(ret, 832)
    return ret


@cache('step')
def ped_natx_gas_ej():
    """
    ped natx gas ej



    component


    """
    
    ret = np.maximum(0, ped_gases() - pes_biogas_for_tfc())
    #test_nans(ret, 833)
    return ret


@cache('step')
def annual_share_res_vs_tfec_growth_rate():
    """
    annual share res vs tfec growth rate



    component


    """
    
    ret = -1 + share_res_vs_tfec() / share_res_vs_tfec_delayed_1yr()
    #test_nans(ret, 834)
    return ret


@cache('step')
def annual_share_res_vs_tpes_growth_rate():
    """
    annual share res vs tpes growth rate



    component


    """
    
    ret = -1 + share_res_vs_tpes() / share_res_vs_tpes_delayed_1yr()
    #test_nans(ret, 835)
    return ret


@cache('step')
def share_res_vs_tpes_delayed_1yr():
    """
    share res vs tpes delayed 1yr



    component


    """
    
    ret = delay_share_res_vs_tpes_round1__time_step___time_step_0123_1__time_step()
    #test_nans(ret, 836)
    return ret


@cache('step')
def gtl_production():
    """
    gtl production



    component


    """
    
    ret = gtl_potential_production() * (1 - share_ctlxplusxgtl_overcapacity())
    #test_nans(ret, 837)
    return ret


@cache('step')
def pes_biogas_for_tfc():
    """
    pes biogas for tfc



    component


    """
    
    ret = np.minimum(ped_gases(), potential_pes_biogas_for_tfc())
    #test_nans(ret, 838)
    return ret


@cache('step')
def share_ctlxplusxgtl_overcapacity():
    """
    share ctlxplusxgtl overcapacity



    component


    """
    
    ret = functions.zidz((potential_fes_ctlxplusxgtl_ej() - fes_ctlxplusxgtl_ej()),
                          potential_fes_ctlxplusxgtl_ej())
    #test_nans(ret, 839)
    return ret


@cache('step')
def share_res_vs_tfec_delayed_1yr():
    """
    share res vs tfec delayed 1yr



    component


    """
    
    ret = delay_share_res_vs_tfec_round1__time_step___time_step_01614_1__time_step()
    #test_nans(ret, 840)
    return ret


@cache('step')
def share_exdashxlosses_cc():
    """
    share exdashxlosses cc



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  functions.if_then_else(abs(activate_elf_all_scenxquestx() - 0) < equality_threshold, 0,
                                                         functions.if_then_else(
                                                             abs(activate_elf_by_scenxquestx() - 0) < equality_threshold, 0,
                                                             share_exdashxlosses_cc_from_x2015())))
    #test_nans(ret, 841)
    return ret


@cache('step')
def ped_natx_gas_for_gtl_ej():
    """
    ped natx gas for gtl ej



    component


    """
    
    ret = gtl_production() / gtl_efficiency()
    #test_nans(ret, 842)
    return ret


@cache('step')
def ped_coal_for_ctl_ej():
    """
    ped coal for ctl ej



    component


    """
    
    ret = ctl_production() / ctl_efficiency()
    #test_nans(ret, 843)
    return ret


@cache('step')
def annual_tfec_res_growth_rate():
    """
    annual tfec res growth rate



    component


    """
    
    ret = -1 + tfec_res_ej() / tfec_res_delayed_1yr()
    #test_nans(ret, 844)
    return ret


@cache('step')
def ped_total_oil_ej():
    """
    ped total oil ej



    component


    """
    
    ret = np.maximum(0, ped_nre_liquids() - fes_ctlxplusxgtl_ej() - oil_refinery_gains_ej())
    #test_nans(ret, 845)
    return ret


@cache('step')
def ctl_production():
    """
    ctl production



    component


    """
    
    ret = ctl_potential_production() * (1 - share_ctlxplusxgtl_overcapacity())
    #test_nans(ret, 846)
    return ret


@cache('step')
def ch4_emissions_coal_without_ctl():
    """
    ch4 emissions coal without ctl



    component


    """
    
    ret = extraction_coal_emissions_relevant_ej() * gch4_per_mj_coal() * mj_per_ej() / g_per_mt()
    #test_nans(ret, 847)
    return ret


@cache('step')
def pes_tot_res_for_heat():
    """
    pes tot res for heat



    component


    """
    
    ret = functions.sum(pes_res_for_heatxdashxcom_by_techn()) + functions.sum(
        pes_res_for_heatxdashxnc_by_techn()) + pes_tot_biogas_for_heatxdashxcom()
    #test_nans(ret, 848)
    return ret


@cache('step')
def total_fe_real_supply_res_for_heatxdashxnc_ej():
    """
    total fe real supply res for heatxdashxnc ej



    component


    """
    
    ret = functions.sum(fe_real_generation_res_heatxdashxnc_ej())
    #test_nans(ret, 849)
    return ret


@cache('step')
def real_extraction_conv_oil_emissions_relevant_ej():
    """
    real extraction conv oil emissions relevant ej



    component


    """
    
    ret = np.maximum(
        0,
        real_extraction_conv_oil_ej() -
        (nonxdashxenergy_use_demand_by_final_fuel_ej().loc['liquids'].values) * share_conv_vs_total_oil_extraction())
    #test_nans(ret, 850)
    return ret


@cache('step')
def co2_emissions_coal_without_ctl():
    """
    co2 emissions coal without ctl



    component


    """
    
    ret = extraction_coal_emissions_relevant_ej() * gco2_per_mj_coal() * mj_per_ej() / g_per_gt()
    #test_nans(ret, 851)
    return ret


@cache('step')
def real_extraction_conv_gas_emissions_relevant_ej():
    """
    real extraction conv gas emissions relevant ej



    component


    """
    
    ret = np.maximum(0,
                      real_extraction_conv_gas_ej() -
                      (ped_natx_gas_for_gtl_ej() + nonxdashxenergy_use_demand_by_final_fuel_ej().loc['gases'].values) *
                      share_conv_vs_total_gas_extraction())
    #test_nans(ret, 852)
    return ret


@cache('step')
def pes_res_for_heatxdashxnc_by_techn():
    """
    pes res for heatxdashxnc by techn



    component


    """
    
    ret = xr.DataArray(data=[
        fe_real_generation_res_heatxdashxnc_ej().loc['solarxdashxheat'].values * efficiency_solar_panels_for_heat() /
        efficiency_res_heat().loc['solarxdashxheat'].values,
        fe_real_generation_res_heatxdashxnc_ej().loc['geotxdashxheat'].values / efficiency_res_heat().loc['geotxdashxheat'].values,
        fe_real_generation_res_heatxdashxnc_ej().loc['solid bioexdashxheat'].values / efficiency_res_heat().loc['solid bioexdashxheat'].values],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 853)
    return ret


@cache('step')
def fes_nre_for_heat():
    """
    fes nre for heat



    component


    """
    
    ret = fes_heatxdashxcom_fossil_fuels_chp_plants_ej() + fes_heat_from_coal(
    ) + fes_heat_from_natx_gas() + fes_heat_from_oil() + fes_heatxdashxcom_nuclear_chp_plants_ej()
    #test_nans(ret, 854)
    return ret


@cache('step')
def installed_capacity_res_heatxdashxnc_tw():
    """
    installed capacity res heatxdashxnc tw



    component


    """
    
    ret = integ_installed_capacity_res_heatxdashxnc_tw()
    #test_nans(ret, 855)
    return ret


@cache('step')
def fed_heatxdashxcom_plants_fossil_fuels_ej():
    """
    fed heatxdashxcom plants fossil fuels ej



    component


    """
    
    ret = np.maximum(fed_heatxdashxcom_nre_ej() - fes_heatxdashxcom_fossil_fuels_chp_plants_ej() -
                      fes_heatxdashxcom_nuclear_chp_plants_ej(), 0)
    #test_nans(ret, 856)
    return ret


@cache('step')
def real_extraction_unconv_gas_emissions_relevant_ej():
    """
    real extraction unconv gas emissions relevant ej



    component


    """
    
    ret = np.maximum(0,
                      real_extraction_unconv_gas_ej() -
                      (ped_natx_gas_for_gtl_ej() + nonxdashxenergy_use_demand_by_final_fuel_ej().loc['gases'].values) *
                      (1 - share_conv_vs_total_gas_extraction()))
    #test_nans(ret, 857)
    return ret


@cache('step')
def real_extraction_unconv_oil_emissions_relevant_ej():
    """
    real extraction unconv oil emissions relevant ej



    component


    """
    
    ret = np.maximum(0,
                      real_extraction_unconv_oil_ej() -
                      (nonxdashxenergy_use_demand_by_final_fuel_ej().loc['liquids'].values) *
                      (1 - share_conv_vs_total_oil_extraction()))
    #test_nans(ret, 858)
    return ret


@cache('step')
def available_pe_potential_solid_bioe_for_elec_ej():
    """
    available pe potential solid bioe for elec ej



    component


    """
    
    ret = np.maximum(0,
                      total_pe_solid_bioe_potential_heatxplusxelec_ej() -
                      pes_res_for_heatxdashxcom_by_techn().loc['solid bioexdashxheat'].values - pes_res_for_heatxdashxnc_by_techn().loc['solid bioexdashxheat'].values)
    #test_nans(ret, 859)
    return ret


@cache('step')
def remaining_potential_res_for_heat():
    """
    remaining potential res for heat



    component


    """
    
    ret = functions.zidz(
        (np.maximum(0,
                    max_fe_potential_res_for_heat() - potential_fes_res_for_heatxdashxcom_ej() -
                    potential_fes_res_for_heatxdashxnc_ej())), max_fe_potential_res_for_heat())
    #test_nans(ret, 860)
    return ret


@cache('step')
def extraction_coal_emissions_relevant_ej():
    """
    extraction coal emissions relevant ej



    component


    """
    
    ret = np.maximum(
        0, extraction_coal_without_ctl_ej() - nonxdashxenergy_use_demand_by_final_fuel_ej().loc['solids'].values)
    #test_nans(ret, 861)
    return ret


@cache('step')
def wear_res_capacity_for_heatxdashxnc_tw():
    """
    wear res capacity for heatxdashxnc tw



    component


    """
    
    ret = installed_capacity_res_heatxdashxnc_tw() / life_time_res_for_heat()
    #test_nans(ret, 862)
    return ret


@cache('step')
def fe_real_generation_res_heatxdashxnc_ej():
    """
    fe real generation res heatxdashxnc ej



    component


    """
    
    ret = potential_fes_res_for_heatxdashxnc_ej() * (1 - res_heatxdashxnc_tot_overcapacity())
    #test_nans(ret, 863)
    return ret


@cache('step')
def total_fe_real_supply_res_for_heatxdashxcom_ej():
    """
    total fe real supply res for heatxdashxcom ej



    component


    """
    
    ret = functions.sum(fe_real_generation_res_heatxdashxcom_ej())
    #test_nans(ret, 864)
    return ret


@cache('run')
def past_res_growth_for_heatxdashxcom_x0():
    """
    past res growth for heatxdashxcom x0



    constant


    """
    
    ret = xr.DataArray(data=[0.2771226,0.03571556,0.06278667],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 865)
    return ret


@cache('run')
def efficiency_conversion_bioe_plants_to_heat_x0x():
    """
    efficiency conversion bioe plants to heat x0x



    constant


    """
    
    ret = 0.55
    #test_nans(ret, 866)
    return ret


@cache('step')
def fed_heatxdashxcom_after_priorities_ej():
    """
    fed heatxdashxcom after priorities ej



    component


    """
    
    ret = np.maximum(0,
                      total_fed_heatxdashxcom_ej() - fes_heatxdashxcom_from_waste_ej() -
                      fes_heatxdashxcom_from_biogas_ej())
    #test_nans(ret, 867)
    return ret


@cache('step')
def fes_heat_from_biow():
    """
    fes heat from biow



    component


    """
    
    ret = fe_real_supply_res_for_heatxdashxcom_tot_ej(
    ) + fe_real_supply_res_for_heatxdashxnc_tot_ej() + fes_heatxdashxcom_from_biogas_ej(
    ) + fes_heatxdashxcom_from_waste_ej()
    #test_nans(ret, 868)
    return ret


@cache('step')
def abundance_res_heatxdashxnc():
    """
    abundance res heatxdashxnc



    component


    """
    
    ret = functions.if_then_else(
        abs(total_fed_heatxdashxnc_ej()) < equality_threshold, 0,
        functions.zidz(
            (total_fed_heatxdashxnc_ej() - fe_real_supply_res_for_heatxdashxnc_tot_ej()),
            total_fed_heatxdashxnc_ej()))
    #test_nans(ret, 869)
    return ret


@cache('run')
def efficiency_geothermal_for_heat_x0x():
    """
    efficiency geothermal for heat x0x



    constant


    """
    
    ret = 0.6960561017716173
    #test_nans(ret, 870)
    return ret


@cache('step')
def fes_heatxdashxcom_nuclear_chp_plants_ej():
    """
    fes heatxdashxcom nuclear chp plants ej



    component


    """
    
    ret = np.minimum(potential_fes_heatxdashxcom_nuclear_chp_plants_ej(),
                      fed_heatxdashxcom_nre_ej())
    #test_nans(ret, 871)
    return ret


@cache('step')
def fe_real_supply_res_for_heatxdashxnc_tot_ej():
    """
    fe real supply res for heatxdashxnc tot ej



    component


    """
    
    ret = np.minimum(
        np.maximum(total_fed_heatxdashxnc_ej(), 0), potential_fes_tot_res_for_heatxdashxnc_ej())
    #test_nans(ret, 872)
    return ret


@cache('run')
def initial_value_res_for_heatxdashxnc():
    """
    initial value res for heatxdashxnc



    constant


    """
    
    ret = xr.DataArray(data=[0.03252055,0.01718051,0.37315399],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 873)
    return ret


@cache('run')
def past_res_growth_for_heatxdashxnc():
    """
    past res growth for heatxdashxnc



    constant


    """
    
    ret = xr.DataArray(data=[0.14412798,0.077,0.03534992],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 874)
    return ret


@cache('run')
def p_solid_bioe_for_heat_x0x():
    """
    p solid bioe for heat x0x



    constant


    """
    
    ret = 0.115
    #test_nans(ret, 875)
    return ret


@cache('step')
def fed_heat_fossil_fuels_chp_plants_ej():
    """
    fed heat fossil fuels chp plants ej



    component


    """
    
    ret = np.maximum(
        fed_heatxdashxcom_by_nre_chp_plants_ej() - fes_heatxdashxcom_nuclear_chp_plants_ej(), 0)
    #test_nans(ret, 876)
    return ret


@cache('run')
def replacement_res_for_heatxdashxnc():
    """
    replacement res for heatxdashxnc



    constant


    """
    
    ret = xr.DataArray(data=[1.0,1.0,1.0],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 877)
    return ret


@cache('run')
def efficiency_solar_panels_for_heat_x0x():
    """
    efficiency solar panels for heat x0x



    constant


    """
    
    ret = 0.095
    #test_nans(ret, 878)
    return ret


@cache('run')
def p_res_for_heat_x0x():
    """
    p res for heat x0x



    component


    """
    
    ret = xr.DataArray(data=[
        p_solar_for_heat_x0x(),
        p_geothermal_for_heat_x0x(),
        p_solid_bioe_for_heat_x0x()],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 879)
    return ret


@cache('run')
def p_solar_for_heat_x0x():
    """
    p solar for heat x0x



    constant


    """
    
    ret = 0.127
    #test_nans(ret, 880)
    return ret


@cache('step')
def abundance_res_heatxdashxcom():
    """
    abundance res heatxdashxcom



    component


    """
    
    ret = functions.if_then_else(abs(fed_heatxdashxcom_after_priorities_ej() - 0) < equality_threshold, 0,
                                  functions.zidz((fed_heatxdashxcom_after_priorities_ej() -
                                                  fe_real_supply_res_for_heatxdashxcom_tot_ej()),
                                                 fed_heatxdashxcom_after_priorities_ej()))
    #test_nans(ret, 881)
    return ret


@cache('step')
def abundance_res_heatxdashxnc2():
    """
    abundance res heatxdashxnc2



    component


    """
    
    ret = np.sqrt(abundance_res_heatxdashxnc())
    #test_nans(ret, 882)
    return ret


@cache('step')
def potential_fes_tot_res_for_heatxdashxnc_ej():
    """
    potential fes tot res for heatxdashxnc ej



    component


    """
    
    ret = functions.sum(potential_fes_res_for_heatxdashxnc_ej())
    #test_nans(ret, 883)
    return ret


@cache('run')
def life_time_res_for_heat_x0x():
    """
    life time res for heat x0x



    constant


    """
    
    ret = xr.DataArray(data=[25.0,25.0,30.0],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 884)
    return ret


@cache('run')
def efficiency_res_heat_x0x():
    """
    efficiency res heat x0x



    component


    """
    
    ret = xr.DataArray(data=[
        efficiency_solar_panels_for_heat_x0x() * losses_solar_for_heat_x0x(),
        efficiency_geothermal_for_heat_x0x(),
        efficiency_conversion_bioe_plants_to_heat_x0x()],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 885)
    return ret


@cache('run')
def replacement_res_for_heat_x0x():
    """
    replacement res for heat x0x



    constant


    """
    
    ret = xr.DataArray(data=[1.0,1.0,1.0],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 886)
    return ret


@cache('run')
def p_geothermal_for_heat_x0x():
    """
    p geothermal for heat x0x



    constant


    """
    
    ret = 0.076
    #test_nans(ret, 887)
    return ret


@cache('step')
def fes_res_for_heat_ej():
    """
    fes res for heat ej



    component


    """
    
    ret = fe_real_supply_res_for_heatxdashxcom_tot_ej(
    ) + fe_real_supply_res_for_heatxdashxnc_tot_ej() + fes_heatxdashxcom_from_biogas_ej()
    #test_nans(ret, 888)
    return ret


@cache('step')
def fe_real_supply_res_for_heatxdashxcom_tot_ej():
    """
    fe real supply res for heatxdashxcom tot ej



    component


    """
    
    ret = np.minimum(
        np.maximum(fed_heatxdashxcom_after_priorities_ej(), 0),
        potential_fes_tot_res_for_heatxdashxcom_ej())
    #test_nans(ret, 889)
    return ret


@cache('run')
def losses_solar_for_heat_x0x():
    """
    losses solar for heat x0x



    constant


    """
    
    ret = 0.37
    #test_nans(ret, 890)
    return ret


@cache('step')
def res_heatxdashxnc_tot_overcapacity():
    """
    res heatxdashxnc tot overcapacity



    component


    """
    
    ret = functions.if_then_else(abs(potential_fes_tot_res_for_heatxdashxnc_ej() - 0) < equality_threshold, 0, (
        potential_fes_tot_res_for_heatxdashxnc_ej() - fe_real_supply_res_for_heatxdashxnc_tot_ej())
                                  / potential_fes_tot_res_for_heatxdashxnc_ej())
    #test_nans(ret, 891)
    return ret


@cache('step')
def potential_fes_res_for_heatxdashxnc_ej():
    """
    potential fes res for heatxdashxnc ej



    component


    """
    
    ret = potential_fes_res_for_heatxdashxnc_twh() * ej_per_twh()
    #test_nans(ret, 892)
    return ret


@cache('step')
def other_solids_required():
    """
    other solids required



    component


    """
    
    ret = transformation_ff_losses_ej().loc['solids'].values + energy_distr_losses_ff_ej().loc['solids'].values + nonxdashxenergy_use_demand_by_final_fuel_ej().loc['solids'].values
    #test_nans(ret, 893)
    return ret


@cache('step')
def other_gases_required():
    """
    other gases required



    component


    """
    
    ret = transformation_ff_losses_ej().loc['gases'].values + energy_distr_losses_ff_ej().loc['gases'].values + nonxdashxenergy_use_demand_by_final_fuel_ej().loc['gases'].values
    #test_nans(ret, 894)
    return ret


@cache('step')
def transformation_ff_losses_ej():
    """
    transformation ff losses ej



    constant


    """
    
    tff_solids = pes_fossil_fuel_extraction_delayed().loc['solids'].values * historic_share_of_transformation_losses_vs_extraction_subsolids(time()) 
    ret = xr.DataArray(data=[0.0,0.0,
        pes_fossil_fuel_extraction_delayed().loc['liquids'].values *
        historic_share_of_transformation_losses_vs_extraction_subliquids(time()),
        tff_solids * ratio_gain_gas_vs_lose_solids_in_tranf_processes(time()),
        tff_solids],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 895)
    return ret


@cache('step')
def fes_elec_fossil_fuel_chp_plants_ej():
    """
    fes elec fossil fuel chp plants ej



    component


    """
    
    ret = np.minimum(potential_fe_gen_elec_fossil_fuel_chp_plants_ej(),
                      demand_elec_nre_twh() * ej_per_twh())
    #test_nans(ret, 896)
    return ret


@cache('step')
def total_fe_elec_demand_after_priorities_twh():
    """
    total fe elec demand after priorities twh



    component


    """
    
    ret = np.maximum(total_fe_elec_demand_twh() - fes_elec_from_res_with_priority_twh() -
                      fes_elec_from_waste_twh(), 0)
    #test_nans(ret, 897)
    return ret


@cache('step')
def demand_elec_nre_twh():
    """
    demand elec nre twh



    component


    """
    
    ret = np.maximum(0,
                      total_fe_elec_demand_twh() - fe_tot_generation_all_res_elec_twh() -
                      fes_elec_from_waste_twh())
    #test_nans(ret, 898)
    return ret


@cache('step')
def nonxdashxenergy_use_demand_by_final_fuel_ej():
    """
    nonxdashxenergy use demand by final fuel ej



    component


    """
    
    ret = integ_nonxdashxenergy_use_demand_by_final_fuel_ej()
    #test_nans(ret, 899)
    return ret


@cache('step')
def share_solids_for_final_energy():
    """
    share solids for final energy



    component


    """
    
    ret = functions.zidz(required_fed_solids(),
                          (ped_solids() - ped_coal_for_ctl_ej() - other_solids_required()))
    #test_nans(ret, 900)
    return ret


@cache('step')
def real_fe_consumption_gases_ej():
    """
    real fe consumption gases ej



    component


    """
    
    ret = (pes_gases() - ped_natx_gas_for_gtl_ej() - other_gases_required()
            ) * share_gases_for_final_energy()
    #test_nans(ret, 901)
    return ret


@cache('step')
def total_real_nonxdashxenergy_use_consumption_ej():
    """
    total real nonxdashxenergy use consumption ej



    component


    """
    
    ret = functions.sum(nonxdashxenergy_use_demand_by_final_fuel_ej())
    #test_nans(ret, 902)
    return ret


@cache('step')
def share_gases_for_final_energy():
    """
    share gases for final energy



    component


    """
    
    ret = functions.zidz(required_fed_by_gas(),
                          (ped_gases() - ped_natx_gas_for_gtl_ej() - other_gases_required()))
    #test_nans(ret, 903)
    return ret


@cache('step')
def ratio_fed_for_heatxdashxnc_vs_fed_for_heatxdashxcom():
    """
    ratio fed for heatxdashxnc vs fed for heatxdashxcom



    component


    """
    
    ret = functions.sum(fed_by_fuel_for_heatxdashxnc()) * functions.zidz(
        1, required_fed_by_fuel_before_heat_correction().loc['heat'].values)
    #test_nans(ret, 904)
    return ret


@cache('step')
def real_fe_consumption_by_fuel_before_heat_correction():
    """
    real fe consumption by fuel before heat correction



    component


    """
    
    ret = xr.DataArray(data=[
        real_fe_consumption_by_fuel().loc['electricity'].values,
        real_fe_consumption_by_fuel().loc['heat'].values /
        (1 + ratio_fed_for_heatxdashxnc_vs_fed_for_heatxdashxcom()),
        real_fe_consumption_by_fuel().loc['liquids'].values / (1 - share_feh_over_fed_by_final_fuel().loc['liquids'].values),
        real_fe_consumption_by_fuel().loc['gases'].values / (1 - share_feh_over_fed_by_final_fuel().loc['gases'].values),
        real_fe_consumption_by_fuel().loc['solids'].values / (1 - share_feh_over_fed_by_final_fuel().loc['solids'].values)],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 905)
    return ret


@cache('step')
def required_tfed_before_heat_dem_corr():
    """
    required tfed before heat dem corr



    component


    """
    
    ret = functions.sum(required_fed_by_fuel_before_heat_correction())
    #test_nans(ret, 906)
    return ret


@cache('step')
def cum_materials_to_extract_for_ev_batteries():
    """
    cum materials to extract for ev batteries



    component


    """
    
    ret = integ_cum_materials_to_extract_for_ev_batteries()
    #test_nans(ret, 907)
    return ret


@cache('step')
def share_cum_dem_materials_to_extract_alt_techn_vs_total():
    """
    share cum dem materials to extract alt techn vs total



    component


    """
    
    ret = functions.if_then_else(total_cumulative_demand_materials_to_extract_from_x2015x() <= 0,
                                  0, functions.zidz(cum_materials_to_extract_for_alt_techn_from_x2015x(),
                                  total_cumulative_demand_materials_to_extract_from_x2015x()))
    #test_nans(ret, 908)
    return ret


@cache('step')
def total_recycled_materials_for_ev_batteries_mt():
    """
    total recycled materials for ev batteries mt



    component


    """
    
    ret = total_materials_required_for_ev_batteries(
    ) - total_materials_to_extract_for_ev_batteries_mt()
    #test_nans(ret, 909)
    return ret


@cache('step')
def share_res_vs_tfec():
    """
    share res vs tfec



    component


    """
    
    ret = functions.zidz(tfec_res_ej(), real_tfec())
    #test_nans(ret, 910)
    return ret


@cache('step')
def cum_materials_to_extract_for_alt_techn_from_x2015x():
    """
    cum materials to extract for alt techn from x2015x



    component


    """
    
    ret = cum_materials_to_extract_for_ev_batteries_from_x2015x(
    ) + cum_materials_to_extract_for_res_elec_from_x2015x()
    #test_nans(ret, 911)
    return ret


@cache('step')
def total_materials_to_extract_for_ev_batteries_from_x2015x_mt():
    """
    total materials to extract for ev batteries from x2015x mt



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  total_materials_to_extract_for_ev_batteries_mt())
    #test_nans(ret, 912)
    return ret


@cache('run')
def materials_per_new_capacity_installed_xdashx_ev_batteries():
    """
    materials per new capacity installed xdashx ev batteries



    constant


    """
    
    ret = xr.DataArray(data=[0.0,500.0,0.0,0.0,0.0,0.0,0.0,289.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,34.4,0.0,0.0,1631.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,125.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 913)
    return ret


@cache('step')
def total_materials_to_extract_for_ev_batteries_mt():
    """
    total materials to extract for ev batteries mt



    component


    """
    
    ret = total_materials_required_for_ev_batteries() * (1 - recycling_rates_minerals_alt_techn())
    #test_nans(ret, 914)
    return ret


@cache('step')
def total_materials_to_extract_mt():
    """
    total materials to extract mt



    component


    """
    
    ret = materials_to_extract_rest_mt() + total_materials_to_extract_for_res_elec_mt(
    ) + total_materials_to_extract_for_ev_batteries_mt()
    #test_nans(ret, 915)
    return ret


@cache('step')
def cum_materials_requirements_for_ev_batteries():
    """
    cum materials requirements for ev batteries



    component


    """
    
    ret = integ_cum_materials_requirements_for_ev_batteries()
    #test_nans(ret, 916)
    return ret


@cache('step')
def xstaticx_eroigrid_res_elec():
    """
    xstaticx eroigrid res elec



    component


    """
    
    ret = functions.if_then_else(
        xstaticx_eroi_res_elec() <= 0, 0,
        (1 - share_res_elec_generation_curtailedxandxstored() +
         share_res_elec_generation_curtailedxandxstored() * rt_elec_storage_efficiency()) /
        (1 / xstaticx_eroi_res_elec() + share_res_elec_generation_curtailedxandxstored(
        ) * rt_elec_storage_efficiency() / esoi_elec_storage()))
    #test_nans(ret, 917)
    return ret


@cache('run')
def t_per_x():
    """
    t per x



    constant


    """
    
    ret = 1000000000000.0
    #test_nans(ret, 918)
    return ret


@cache('step')
def total_materials_required_for_ev_batteries():
    """
    total materials required for ev batteries



    component


    """
    
    ret = materials_required_for_ev_batteries_mt()
    #test_nans(ret, 919)
    return ret


@cache('step')
def tfec_from_res_per_capita():
    """
    tfec from res per capita



    component


    """
    
    ret = functions.zidz(tfec_res_ej() * gj_per_ej(), population())
    #test_nans(ret, 920)
    return ret


@cache('step')
def share_materials_cum_demand_to_extract_vs_reserves_for_res_elec():
    """
    share materials cum demand to extract vs reserves for res elec



    component


    """
    
    ret = functions.if_then_else(
        abs(current_mineral_reserves_mt() - 0) < equality_threshold, 0,
        cum_materials_to_extract_for_alt_techn_from_x2015x() / current_mineral_reserves_mt())
    #test_nans(ret, 921)
    return ret


@cache('run')
def rt_storage_efficiency_ev_batteries():
    """
    rt storage efficiency ev batteries



    constant


    """
    
    ret = 0.9
    #test_nans(ret, 922)
    return ret


@cache('step')
def cum_materials_to_extract_for_ev_batteries_from_x2015x():
    """
    cum materials to extract for ev batteries from x2015x



    component


    """
    
    ret = integ_cum_materials_to_extract_for_ev_batteries_from_x2015x()
    #test_nans(ret, 923)
    return ret


@cache('step')
def total_cumulative_demand_materials_to_extract_from_x2015x():
    """
    total cumulative demand materials to extract from x2015x



    component


    """
    
    ret = cum_materials_to_extract_for_alt_techn_from_x2015x(
    ) + cum_materials_to_extract_rest_from_x2015x()
    #test_nans(ret, 924)
    return ret


@cache('run')
def initial_cumulated_material_requirements_for_ev_batteries_x1995x():
    """
    initial cumulated material requirements for ev batteries x1995x



    constant


    """
    
    ret = 0
    #test_nans(ret, 925)
    return ret


@cache('step')
def share_materials_cum_demand_to_extract_vs_resources_for_res_elec():
    """
    share materials cum demand to extract vs resources for res elec



    component


    """
    
    ret = functions.if_then_else(
        abs(current_mineral_resources_mt() - 0) < equality_threshold, 0,
        cum_materials_to_extract_for_alt_techn_from_x2015x() / current_mineral_resources_mt())
    #test_nans(ret, 926)
    return ret


@cache('step')
def fe_tot_generation_all_res_elec_ej():
    """
    fe tot generation all res elec ej



    component


    """
    
    ret = fe_tot_generation_all_res_elec_twh() * ej_per_twh()
    #test_nans(ret, 927)
    return ret


@cache('step')
def carbon_footprint_tonnescxdivxperson():
    """
    carbon footprint tonnescxdivxperson



    component


    """
    
    ret = carbon_footprint_tco2xdivxperson() * c_per_co2()
    #test_nans(ret, 928)
    return ret


@cache('step')
def potential_max_hdi():
    """
    potential max hdi



    component


    """
    
    ret = functions.if_then_else(net_tfec_per_capita() <= 0, 0,
                                  np.minimum(1, 0.1395 * np.log(net_tfec_per_capita()) + 0.1508))
    #test_nans(ret, 929)
    return ret


@cache('run')
def kw_per_battery_ev():
    """
    kw per battery ev



    constant


    """
    
    ret = 80
    #test_nans(ret, 930)
    return ret


@cache('step')
def pes_fossil_fuel_extraction_delayed():
    """
    pes fossil fuel extraction delayed



    component


    """
    
    ret = xr.DataArray(data=[0.0,0.0,
        delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_139_time_step__time_step(
        ),
        delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_79_time_step__time_step(
        ),
        delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_101_time_step__time_step(
        )],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 931)
    return ret


@cache('step')
def energy_distr_losses_ff_ej():
    """
    energy distr losses ff ej



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,
        pes_fossil_fuel_extraction_delayed().loc['liquids'].values *
        historic_share_of_losses_vs_extraction_subliquids(time()),
        pes_fossil_fuel_extraction_delayed().loc['gases'].values *
        historic_share_of_losses_vs_extraction_subgases(time()),
        pes_fossil_fuel_extraction_delayed().loc['solids'].values *
        historic_share_of_losses_vs_extraction_subsolids(time())],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 932)
    return ret


@cache('step')
def pes_fossil_fuel_extraction():
    """
    pes fossil fuel extraction



    component


    """
    
    ret = xr.DataArray(data=[0.0,0.0,
        pes_oil_ej(),
        pes_natx_gas(),
        extraction_coal_ej()],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 933)
    return ret


@cache('run')
def initial_energy_intensity_x1995x_h():
    """
    initial energy intensity x1995x h



    constant


    """
    
    ret = xr.DataArray(
        data=[0.63194525, 0.28551574, 1.568424, 0.80691299, 1.95104725],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 934)
    return ret


@cache('run')
def min_energy_intensity_vs_intial_h():
    """
    min energy intensity vs intial h



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 935)
    return ret


@cache('run')
def minimum_fraction_h():
    """
    minimum fraction h



    constant


    """
    
    ret = xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 936)
    return ret


@cache('run')
def policy_to_improve_efficiency_speed_h():
    """
    policy to improve efficiency speed h



    constant


    """
    
    ret = xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 937)
    return ret


@cache('run')
def max_yearly_change_h():
    """
    max yearly change h



    constant


    """
    
    ret = xr.DataArray(
        data=[0.05, 0.05, 0.05, 0.05, 0.05],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 938)
    return ret


@cache('step')
def available_improvement_efficiency_h():
    """
    available improvement efficiency h



    component


    """
    
    ret = np.minimum(
        1,
        functions.if_then_else(
            time() > 2009,
            functions.zidz((
                global_energy_intensity_h() -
                (min_energy_intensity_vs_intial_h() * initial_global_energy_intensity_x2009x_h())),
                           0.7 * initial_global_energy_intensity_x2009x_h()), 1))
    #test_nans(ret, 939)
    return ret


@cache('run')
def year_policy_change_energy_h():
    """
    year policy change energy h



    constant


    """
    
    ret = xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 940)
    return ret


@cache('run')
def efficiency_rate_of_substitution_h():
    """
    efficiency rate of substitution h



    constant


    """
    
    ret = xr.DataArray(
        data=[[0., 0., 0.33, 0.33, 0.33], [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.],
              [0., 0., 0., 0., 0.], [0., 0., 0., 0., 0.]],
        coords={
            'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'],
            'final sources1': ['electricity', 'heat', 'liquids', 'gases', 'solids']
        },
        dims=['final sources', 'final sources1'])
    ret = utils.align_by_dims(ret, ('final sources', 'final sources1'))
    #test_nans(ret, 941)
    return ret


@cache('step')
def energy_cost_pressure_h():
    """
    energy cost pressure h



    component


    """
    
    ret = 1 - abundance_of_energy_h()
    #test_nans(ret, 942)
    return ret


@cache('step')
def abundance_of_energy_h():
    """
    abundance of energy h



    component


    """
    
    ret = xr.DataArray(data=[1.0,1.0,
        abundance_liquids(),
        abundance_gases(),
        abundance_solids()],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 943)
    return ret


@cache('step')
def global_energy_intensity_h():
    """
    global energy intensity h



    component


    """
    
    ret = functions.sum(evol_final_energy_intensity_h())
    #test_nans(ret, 944)
    return ret


@cache('run')
def maximun_yearly_aceleration_of_intensity_improvement_pct_h():
    """
    maximun yearly aceleration of intensity improvement pct h



    constant


    """
    
    ret = xr.DataArray(
        data=[0.01, 0.01, 0.01, 0.01, 0.01],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 945)
    return ret


@cache('run')
def historical_mean_rate_energy_intensity_h():
    """
    historical mean rate energy intensity h



    constant


    """
    
    ret = xr.DataArray(
        data=[0.00481188, -0.04466554, 0.00105756, -0.0108818, -0.02195653],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 946)
    return ret


@cache('run')
def policy_change_energy_speed_h():
    """
    policy change energy speed h



    constant


    """
    
    ret = xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 947)
    return ret


@cache('run')
def initial_global_energy_intensity_x2009x_h():
    """
    initial global energy intensity x2009x h



    constant


    """
    
    ret = 4.597962348639566
    #test_nans(ret, 948)
    return ret


@cache('run')
def year_policy_to_improve_efficiency_h():
    """
    year policy to improve efficiency h



    constant


    """
    
    ret = xr.DataArray(
        data=[0., 0., 0., 0., 0.],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 949)
    return ret


@cache('step')
def share_conv_vs_total_oil_extraction():
    """
    share conv vs total oil extraction



    component


    """
    
    ret = functions.zidz(real_extraction_conv_oil_ej(),
                          (real_extraction_conv_oil_ej() + real_extraction_unconv_oil_ej()))
    #test_nans(ret, 950)
    return ret


@cache('step')
def pes_natx_gas_without_gtl():
    """
    pes natx gas without gtl



    component


    """
    
    ret = pes_natx_gas() - ped_natx_gas_for_gtl_ej()
    #test_nans(ret, 951)
    return ret


@cache('step')
def co2_emissions_conv_gas_without_gtl():
    """
    co2 emissions conv gas without gtl



    component


    """
    
    ret = functions.if_then_else(
        abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
        real_extraction_conv_gas_emissions_relevant_ej() * gco2_per_mj_conv_gas(),
        (1 - adapt_co2_emissions_unconv_gas()
         ) * real_extraction_conv_gas_emissions_relevant_ej() * gco2_per_mj_conv_gas() +
        adapt_co2_emissions_unconv_gas() * real_extraction_conv_gas_emissions_relevant_ej(
        ) * gco2_per_mj_unconv_gas()) * mj_per_ej() / g_per_gt()
    #test_nans(ret, 952)
    return ret


@cache('step')
def co2_emissions_conv_oil():
    """
    co2 emissions conv oil



    component


    """
    
    ret = real_extraction_conv_oil_emissions_relevant_ej() * gco2_per_mj_conv_oil() * mj_per_ej(
    ) / g_per_gt()
    #test_nans(ret, 953)
    return ret


@cache('step')
def co2_emissions_gtl():
    """
    co2 emissions gtl



    component


    """
    
    ret = ped_natx_gas_for_gtl_ej() * gco2_per_mj_gtl() * mj_per_ej() / g_per_gt()
    #test_nans(ret, 954)
    return ret


@cache('step')
def ch4_emissions_conv_gas_without_gtl():
    """
    ch4 emissions conv gas without gtl



    component


    """
    
    ret = functions.if_then_else(
        abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
        real_extraction_conv_gas_emissions_relevant_ej() * gch4_per_mj_conv_gas(),
        (1 - adapt_co2_emissions_unconv_gas()
         ) * real_extraction_conv_gas_emissions_relevant_ej() * gch4_per_mj_conv_gas() +
        adapt_co2_emissions_unconv_gas() * real_extraction_conv_gas_emissions_relevant_ej(
        ) * gch4_per_mj_unconv_gas()) * mj_per_ej() / g_per_mt()
    #test_nans(ret, 955)
    return ret


@cache('step')
def co2_emissions_unconv_oil():
    """
    co2 emissions unconv oil



    component


    """
    
    ret = (real_extraction_unconv_oil_emissions_relevant_ej() *
            (gco2_per_mj_unconv_oil() +
             (gco2_per_mj_shale_oil() - gco2_per_mj_unconv_oil()) * adapt_emissions_shale_oil())
            ) * mj_per_ej() / g_per_gt()
    #test_nans(ret, 956)
    return ret


@cache('step')
def share_conv_vs_total_gas_extraction():
    """
    share conv vs total gas extraction



    component


    """
    
    ret = functions.zidz(real_extraction_conv_gas_ej(),
                          (real_extraction_conv_gas_ej() + real_extraction_unconv_gas_ej()))
    #test_nans(ret, 957)
    return ret


@cache('step')
def ch4_emissions_unconv_gas():
    """
    ch4 emissions unconv gas



    component


    """
    
    ret = real_extraction_unconv_gas_emissions_relevant_ej() * gch4_per_mj_unconv_gas(
    ) * mj_per_ej() / g_per_mt()
    #test_nans(ret, 958)
    return ret


@cache('step')
def fe_heat_demand_consum():
    """
    fe heat demand consum



    component


    """
    
    ret = required_fed_by_fuel().loc['heat'].values
    #test_nans(ret, 959)
    return ret


@cache('step')
def share_electricity_vs_tfes():
    """
    share electricity vs tfes



    component


    """
    
    ret = real_fe_consumption_by_fuel().loc['electricity'].values / functions.sum(real_fe_consumption_by_fuel())
    #test_nans(ret, 960)
    return ret


@cache('step')
def co2_emissions_peat():
    """
    co2 emissions peat



    component


    """
    
    ret = pes_peat_ej() * gco2_per_mj_shale_oil() * mj_per_ej() / g_per_gt()
    #test_nans(ret, 961)
    return ret


@cache('step')
def required_fed_by_gas():
    """
    required fed by gas



    component


    """
    
    ret = required_fed_by_fuel().loc['gases'].values
    #test_nans(ret, 962)
    return ret


@cache('step')
def required_tfed():
    """
    required tfed



    component


    """
    
    ret = functions.sum(required_fed_by_fuel())
    #test_nans(ret, 963)
    return ret


@cache('step')
def total_fe_elec_consumption_ej():
    """
    total fe elec consumption ej



    component


    """
    
    ret = total_fe_elec_consumption_twh() * ej_per_twh()
    #test_nans(ret, 964)
    return ret


@cache('step')
def total_fe_elec_consumption_twh():
    """
    total fe elec consumption twh



    component


    """
    
    ret = total_fe_elec_generation_twh() / (1 + share_transmxandxdistr_elec_losses())
    #test_nans(ret, 965)
    return ret


@cache('step')
def ped_gases():
    """
    ped gases



    component


    """
    
    ret = np.maximum(
        0,
        required_fed_by_gas() + ped_natx_gas_for_gtl_ej() + pe_demand_gas_elec_plants_ej() +
        ped_gases_for_heat_plants_ej() + ped_gas_for_chp_plants_ej() + ped_gas_heatxdashxnc() +
        other_gases_required())
    #test_nans(ret, 966)
    return ret


@cache('step')
def total_fe_heat_consumption_ej():
    """
    total fe heat consumption ej



    component


    """
    
    ret = total_fe_heat_generation_ej() / (1 + share_heat_distribution_losses())
    #test_nans(ret, 967)
    return ret


@cache('step')
def share_solids_vs_tfes():
    """
    share solids vs tfes



    component


    """
    
    ret = real_fe_consumption_by_fuel().loc['solids'].values / functions.sum(real_fe_consumption_by_fuel())
    #test_nans(ret, 968)
    return ret


@cache('step')
def share_gases_vs_tfes():
    """
    share gases vs tfes



    component


    """
    
    ret = real_fe_consumption_by_fuel().loc['gases'].values / functions.sum(real_fe_consumption_by_fuel())
    #test_nans(ret, 969)
    return ret


@cache('step')
def required_fed_solids():
    """
    required fed solids



    component


    """
    
    ret = required_fed_by_fuel().loc['solids'].values
    #test_nans(ret, 970)
    return ret


@cache('step')
def share_heat_vs_tfes():
    """
    share heat vs tfes



    component


    """
    
    ret = real_fe_consumption_by_fuel().loc['heat'].values / real_tfec()
    #test_nans(ret, 971)
    return ret


@cache('step')
def ped_solids():
    """
    ped solids



    component


    """
    
    ret = np.maximum(
        0,
        required_fed_solids() + ped_coal_for_ctl_ej() + pe_demand_coal_elec_plants_ej() +
        ped_coal_for_heat_plants_ej() + ped_coal_for_chp_plants_ej() + ped_coal_heatxdashxnc() +
        other_solids_required())
    #test_nans(ret, 972)
    return ret


@cache('step')
def scarcity_conv_oil():
    """
    scarcity conv oil



    component


    """
    
    ret = functions.if_then_else(
        abs(max_extraction_conv_oil_ej() - 0) < equality_threshold, 0,
        functions.if_then_else(max_extraction_conv_oil_ej() >= extraction_conv_oil_ej(), 1 -
                               ((max_extraction_conv_oil_ej() - extraction_conv_oil_ej()) /
                                max_extraction_conv_oil_ej())**exponent_availability_conv_oil(),
                               0))
    #test_nans(ret, 973)
    return ret


@cache('step')
def share_liquids_vs_tfes():
    """
    share liquids vs tfes



    component


    """
    
    ret = real_fe_consumption_by_fuel().loc['liquids'].values / real_tfec()
    #test_nans(ret, 974)
    return ret


@cache('step')
def gfcf_not_covered():
    """
    gfcf not covered



    component


    """
    
    ret = functions.if_then_else(time() < 2009, 0, gross_fixed_capital_formation() - real_gfcf())
    #test_nans(ret, 975)
    return ret


@cache('step')
def lc_not_covered():
    """
    lc not covered



    component


    """
    
    ret = demand_not_covered_total_fd() * labour_share()
    #test_nans(ret, 976)
    return ret


@cache('step')
def cc_total_not_covered():
    """
    cc total not covered



    component


    """
    
    ret = demand_not_covered_total_fd() * capital_share()
    #test_nans(ret, 977)
    return ret


@cache('step')
def household_demand_not_covered():
    """
    household demand not covered



    component


    """
    
    ret = functions.if_then_else(time() < 2009, 0, household_demand() - real_household_demand())
    #test_nans(ret, 978)
    return ret


@cache('step')
def real_total_output_inland_transport():
    """
    real total output inland transport



    component


    """
    
    ret = real_total_output_by_sector().loc['inland transport'].values / 1e+006
    #test_nans(ret, 979)
    return ret


@cache('step')
def tfes_intensity_ej_tdollar():
    """
    tfes intensity ej tdollar



    component


    """
    
    ret = functions.zidz(real_tfec(), gdp())
    #test_nans(ret, 980)
    return ret


@cache('step')
def tfes_intensity_ej_tdollar_delayed_1yr():
    """
    tfes intensity ej tdollar delayed 1yr



    component


    """
    
    ret = delay_tfes_intensity_ej_tdollar_round1__time_step___time_step_8827_1__time_step()
    #test_nans(ret, 981)
    return ret


@cache('step')
def tpes_intensity_ej_tdollar_delayed_1yr():
    """
    tpes intensity ej tdollar delayed 1yr



    component


    """
    
    ret = delay_tpes_intensity_ej_tdollar_round1__time_step___time_step_1314_1__time_step()
    #test_nans(ret, 982)
    return ret


@cache('step')
def annual_tpes_intensity_growth_rate():
    """
    annual tpes intensity growth rate



    component


    """
    
    ret = -1 + functions.zidz(tpes_intensity_ej_tdollar(),
                               tpes_intensity_ej_tdollar_delayed_1yr())
    #test_nans(ret, 983)
    return ret


@cache('step')
def annual_tfes_intensity_growth_rate():
    """
    annual tfes intensity growth rate



    component


    """
    
    ret = -1 + functions.zidz(tfes_intensity_ej_tdollar(),
                               tfes_intensity_ej_tdollar_delayed_1yr())
    #test_nans(ret, 984)
    return ret


@cache('step')
def total_d_jobs_res_elec_per_techn():
    """
    total d jobs res elec per techn



    component


    """
    
    ret = d_jobs_new_installed_res_elec_per_techn() + jobs_oxandxm_res_elec_per_techn()
    #test_nans(ret, 985)
    return ret


@cache('step')
def total_d_jobs_res_heat_per_techn():
    """
    total d jobs res heat per techn



    component


    """
    
    ret = d_jobs_new_installed_res_heat_per_techn() + jobs_oxandxm_res_heat_per_techn()
    #test_nans(ret, 986)
    return ret


@cache('run')
def employment_factors_new_res_elec():
    """
    employment factors new res elec



    constant


    """
    
    ret = xr.DataArray(data=[10.9,16.6,16.9,20.4,7.9,23.6,19.7,9.3],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 987)
    return ret


@cache('run')
def employment_factors_new_res_heat():
    """
    employment factors new res heat



    constant


    """
    
    ret = xr.DataArray(data=[8.4,6.9,16.9],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 988)
    return ret


@cache('run')
def employment_factors_oxandxm_res_elec():
    """
    employment factors oxandxm res elec



    constant


    """
    
    ret = xr.DataArray(data=[0.2,0.4,1.5,0.6,0.3,0.2,0.7,0.6],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 989)
    return ret


@cache('run')
def employment_factors_oxandxm_res_heat():
    """
    employment factors oxandxm res heat



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,1.5],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 990)
    return ret


@cache('step')
def d_jobs_new_installed_res_elec_per_techn():
    """
    d jobs new installed res elec per techn



    component


    """
    
    ret = res_elec_capacity_under_construction_tw() * employment_factors_new_res_elec() * m_per_t(
    )
    #test_nans(ret, 991)
    return ret


@cache('step')
def jobs_oxandxm_res_elec_per_techn():
    """
    jobs oxandxm res elec per techn



    component


    """
    
    ret = installed_capacity_res_elec_tw() * employment_factors_oxandxm_res_elec() * m_per_t()
    #test_nans(ret, 992)
    return ret


@cache('run')
def nx0_vehicles_per_xinland_t():
    """
    nx0 vehicles per xinland t



    component


    """
    
    ret = xr.DataArray(data=[
        nx_hv_inland_t(),
        nx_hv_inland_t(),
        nx_hv_inland_t(),
        nx_lv_inland_t(),
        nx_lv_inland_t(),
        nx_lv_inland_t(),
        nx_lv_inland_t(),
        nx_bus_inlandt(),
        nx_bus_inlandt(),
        nx_bus_inlandt(),
        nx_bus_inlandt(),
        nx_train_inland_t(),
        nx_train_inland_t()],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 993)
    return ret


@cache('run')
def initial_percent_t_vehicles():
    """
    initial percent t vehicles



    constant


    """
    
    ret = xr.DataArray(data=[0.99909685,0.00045157,0.00045157,0.98292579,0.00074635,0.0003609,0.01596696,1.0,0.0,0.0,0.0,0.8,0.2],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 994)
    return ret


@cache('run')
def saving_ratios_v():
    """
    saving ratios v



    constant


    """
    
    ret = xr.DataArray(data=[1.0,0.66,1.0,1.0,0.33,0.66,1.0,1.0,0.5,1.0,1.0,1.0,0.6],
        coords={'vehiclet': ['hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq', 'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec']},
        dims=['vehiclet'])
    #test_nans(ret, 995)
    return ret


@cache('step')
def share_gases_dem_for_heatxdashxnc():
    """
    share gases dem for heatxdashxnc



    component


    """
    
    ret = functions.zidz(ped_gas_heatxdashxnc(), (pes_gases() - ped_natx_gas_for_gtl_ej()))
    #test_nans(ret, 996)
    return ret


@cache('step')
def increase_scarcity_conv_gas():
    """
    increase scarcity conv gas



    component


    """
    
    ret = scarcity_conv_gas_stock() * functions.zidz(
        (scarcity_conv_gas() - scarcity_conv_gas_delayed_1yr()), scarcity_conv_gas())
    #test_nans(ret, 997)
    return ret


@cache('step')
def scarcity_conv_gas_stock():
    """
    scarcity conv gas stock



    component


    """
    
    ret = integ_scarcity_conv_gas_stock()
    #test_nans(ret, 998)
    return ret


@cache('step')
def pes_natx_gas_for_heatxdashxnc_plants():
    """
    pes natx gas for heatxdashxnc plants



    component


    """
    
    ret = (pes_gases() - ped_natx_gas_for_gtl_ej()) * share_gases_dem_for_heatxdashxnc()
    #test_nans(ret, 999)
    return ret


@cache('step')
def real_growth_ctl():
    """
    real growth ctl



    component


    """
    
    ret = functions.if_then_else(abundance_coal() >= abundance_liquids(),
                                  functions.if_then_else(abs(abundance_coal() - 1) < equality_threshold,
                                                         exogenous_growth_ctl(), 0),
                                  0) * abundance_liquids_ctl() * scarcity_conv_oil()
    #test_nans(ret, 1000)
    return ret


@cache('step')
def ctl_potential_production():
    """
    ctl potential production



    component


    """
    
    ret = integ_ctl_potential_production()
    #test_nans(ret, 1001)
    return ret


@cache('step')
def abundance_liquids_gtl():
    """
    abundance liquids gtl



    component


    """
    
    ret = np.sqrt(abs((ped_liquids_ej() - gtl_potential_production()) / ped_liquids_ej()))
    #test_nans(ret, 1002)
    return ret


@cache('step')
def gtl_potential_production():
    """
    gtl potential production



    component


    """
    
    ret = integ_gtl_potential_production()
    #test_nans(ret, 1003)
    return ret


@cache('step')
def abundance_liquids_ctl():
    """
    abundance liquids ctl



    component


    """
    
    ret = np.sqrt(abs((ped_liquids_ej() - ctl_potential_production()) / ped_liquids_ej()))
    #test_nans(ret, 1004)
    return ret


@cache('step')
def wear_gtl():
    """
    wear gtl



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0, gtl_potential_production() / lifetime_gtl())
    #test_nans(ret, 1005)
    return ret


@cache('run')
def lifetime_gtl():
    """
    lifetime gtl



    constant


    """
    
    ret = 40.0
    #test_nans(ret, 1006)
    return ret


@cache('run')
def lifetime_ctl():
    """
    lifetime ctl



    constant


    """
    
    ret = 40.0
    #test_nans(ret, 1007)
    return ret


@cache('step')
def wear_ctl():
    """
    wear ctl



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0, ctl_potential_production() / lifetime_ctl())
    #test_nans(ret, 1008)
    return ret


@cache('step')
def abundance_unconv_oil_stock():
    """
    abundance unconv oil stock



    component


    """
    
    ret = integ_abundance_unconv_oil_stock()
    #test_nans(ret, 1009)
    return ret


@cache('step')
def increase_abundance_unconv_oil():
    """
    increase abundance unconv oil



    component


    """
    
    ret = abundance_unconv_oil() - abundance_unconv_oil_delayed_1yr()
    #test_nans(ret, 1010)
    return ret


@cache('step')
def abundance_unconv_oil():
    """
    abundance unconv oil



    component


    """
    
    ret = functions.if_then_else(
        abs(ped_total_oil_ej() - 0) < equality_threshold, 0,
        functions.if_then_else(
            ped_total_oil_ej() > real_extraction_unconv_oil_ej(),
            (ped_total_oil_ej() - real_extraction_unconv_oil_ej()) / ped_total_oil_ej(), 0))
    #test_nans(ret, 1011)
    return ret


@cache('step')
def abundance_unconv_oil_delayed_1yr():
    """
    abundance unconv oil delayed 1yr



    component


    """
    
    ret = delay_abundance_unconv_oil_round1__time_step___time_step_1_1__time_step()
    #test_nans(ret, 1012)
    return ret


@cache('step')
def abundance_unconv_oil2():
    """
    abundance unconv oil2



    component


    """
    
    ret = abundance_unconv_oil_stock()
    #test_nans(ret, 1013)
    return ret


@cache('step')
def real_extraction_unconv_oil_ej():
    """
    real extraction unconv oil ej



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_oilxquestx() - 1) < equality_threshold,
                                  extraction_unconv_oil_ej(),
                                  extraction_unconv_oil_xdashx_tot_agg())
    #test_nans(ret, 1014)
    return ret


@cache('step')
def scarcity_conv_oil_stock():
    """
    scarcity conv oil stock



    component


    """
    
    ret = integ_scarcity_conv_oil_stock()
    #test_nans(ret, 1015)
    return ret


@cache('step')
def share_variable_res_elec_generation_vs_total_gen():
    """
    share variable res elec generation vs total gen



    component


    """
    
    ret = integ_share_variable_res_elec_generation_vs_total_gen()
    #test_nans(ret, 1016)
    return ret


@cache('step')
def extra_monet_invest_to_cope_with_variable_elec_res():
    """
    extra monet invest to cope with variable elec res



    component


    """
    
    ret = (fe_elec_generation_from_onshore_wind_twh() + fe_elec_generation_from_offshore_wind_twh(
    )) * balancing_costs() + grid_reinforcement_costs_tdollar()
    #test_nans(ret, 1017)
    return ret


@cache('step')
def increase_scarcity_conv_oil():
    """
    increase scarcity conv oil



    component


    """
    
    ret = scarcity_conv_oil() - scarcity_conv_oil_delayed_1yr()
    #test_nans(ret, 1018)
    return ret


@cache('step')
def share_extra_monet_invest_to_cope_with_variable_elec_res():
    """
    share extra monet invest to cope with variable elec res



    component


    """
    
    ret = extra_monet_invest_to_cope_with_variable_elec_res(
    ) / total_monet_invest_res_for_elec_tdolar()
    #test_nans(ret, 1019)
    return ret


@cache('run')
def initial_share_variable_res_elec_gen_vs_total():
    """
    initial share variable res elec gen vs total



    constant


    """
    
    ret = 0.0071
    #test_nans(ret, 1020)
    return ret


@cache('step')
def pes_oil_for_heatxdashxnc_plants():
    """
    pes oil for heatxdashxnc plants



    component


    """
    
    ret = pes_liquids_ej() * share_liquids_dem_for_heatxdashxnc()
    #test_nans(ret, 1021)
    return ret


@cache('step')
def cumulated_invest_e_grid():
    """
    cumulated invest e grid



    component


    """
    
    ret = integ_cumulated_invest_e_grid()
    #test_nans(ret, 1022)
    return ret


@cache('step')
def total_monet_invest_res_for_elec_tdolar():
    """
    total monet invest res for elec tdolar



    component


    """
    
    ret = invest_res_for_elec() + extra_monet_invest_to_cope_with_variable_elec_res()
    #test_nans(ret, 1023)
    return ret


@cache('step')
def cp_exogenous_res_elec_var_reduction():
    """
    cp exogenous res elec var reduction



    component


    """
    
    ret = 1 / (1 + 0.0001 * np.exp(9.85 * share_variable_res_elec_generation_vs_total_gen()))
    #test_nans(ret, 1024)
    return ret


@cache('step')
def cp_exogenous_res_elec_dispatch_reduction():
    """
    cp exogenous res elec dispatch reduction



    component


    """
    
    ret = np.minimum(1,
                      -0.6209 * (share_variable_res_elec_generation_vs_total_gen())**2 - 0.3998 *
                      (share_variable_res_elec_generation_vs_total_gen()) + 1.0222)
    #test_nans(ret, 1025)
    return ret


@cache('step')
def increase_variable_res_share_elec_vs_total_generation():
    """
    increase variable res share elec vs total generation



    component


    """
    
    ret = share_variable_res_elec_generation_vs_total(
    ) - share_variable_res_elec_vs_total_generation_delayed_1yr()
    #test_nans(ret, 1026)
    return ret


@cache('step')
def available_improvement_efficiency():
    """
    available improvement efficiency



    component


    """
    
    ret = np.minimum(1,
                      functions.if_then_else(
                          time() > 2009,
                          functions.zidz((global_energy_intensity_by_sector() -
                                          (min_energy_intensity_vs_intial() *
                                           initial_global_energy_intensity_by_sector_x2009x())),
                                         0.7 * initial_global_energy_intensity_by_sector_x2009x()),
                          1))
    #test_nans(ret, 1027)
    return ret


@cache('step')
def pes_nre_heatxdashxnc():
    """
    pes nre heatxdashxnc



    component


    """
    
    ret = pes_coal_for_heatxdashxnc_plants() + pes_natx_gas_for_heatxdashxnc_plants(
    ) + pes_oil_for_heatxdashxnc_plants()
    #test_nans(ret, 1028)
    return ret


@cache('step')
def tpes_heat():
    """
    tpes heat



    component


    """
    
    ret = pes_nre_heat() + pes_heat_res() + pes_tot_waste_for_heatxdashxcom()
    #test_nans(ret, 1029)
    return ret


@cache('step')
def pes_nre_heat():
    """
    pes nre heat



    component


    """
    
    ret = pes_nre_heatxdashxcom() + pes_nre_heatxdashxnc()
    #test_nans(ret, 1030)
    return ret


@cache('step')
def share_liquids_dem_for_heatxdashxnc():
    """
    share liquids dem for heatxdashxnc



    component


    """
    
    ret = functions.zidz(ped_liquids_heatxdashxnc(), pes_liquids_ej())
    #test_nans(ret, 1031)
    return ret


@cache('step')
def pes_nre_heatxdashxcom():
    """
    pes nre heatxdashxcom



    component


    """
    
    ret = pes_coal_for_heatxdashxcom_plants() + pes_natx_gas_for_heatxdashxcom_plants(
    ) + pes_oil_for_heatxdashxcom_plants()
    #test_nans(ret, 1032)
    return ret


@cache('step')
def check_tpe():
    """
    check tpe



    component


    """
    
    ret = (tped_by_fuel() - tpes_ej()) * 100 / tpes_ej()
    #test_nans(ret, 1033)
    return ret


@cache('step')
def fed_by_fuel_for_heatxdashxnc():
    """
    fed by fuel for heatxdashxnc



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,
        fed_oil_for_heatxdashxnc(),
        fed_natx_gas_for_heatxdashxnc(),
        fed_coal_for_heatxdashxnc() + fed_solid_bioe_for_heatxdashxnc()],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1034)
    return ret


@cache('step')
def share_fed_heatxdashxcom_vs_total_heat():
    """
    share fed heatxdashxcom vs total heat



    component


    """
    
    ret = total_fed_heatxdashxcom_ej() / (total_fed_heat_ej() + total_fed_heatxdashxcom_ej())
    #test_nans(ret, 1035)
    return ret


@cache('step')
def average_elec_consumption_per_capita():
    """
    average elec consumption per capita



    component


    """
    
    ret = total_fe_elec_consumption_twh() * kwh_per_twh() / population()
    #test_nans(ret, 1036)
    return ret


@cache('step')
def fed_solid_bioe_for_heatxdashxnc():
    """
    fed solid bioe for heatxdashxnc



    component


    """
    
    ret = required_fed_by_fuel_before_heat_correction().loc['solids'].values * share_feh_over_fed_solid_bioe(
    ) * efficiency_conversion_bioe_plants_to_heat() / (1 + share_heat_distribution_losses())
    #test_nans(ret, 1037)
    return ret


@cache('step')
def total_fed_heat_ej():
    """
    total fed heat ej



    component


    """
    
    ret = total_fed_heatxdashxcom_ej() + total_fed_heatxdashxnc_ej()
    #test_nans(ret, 1038)
    return ret


@cache('step')
def heatxdashxnc_distribution_losses():
    """
    heatxdashxnc distribution losses



    component


    """
    
    ret = total_fed_heatxdashxnc_ej() - fed_heatxdashxnc_ej()
    #test_nans(ret, 1039)
    return ret


@cache('step')
def total_fed_heatxdashxnc_ej():
    """
    total fed heatxdashxnc ej



    component


    """
    
    ret = fed_heatxdashxnc_ej() * (1 + share_heat_distribution_losses())
    #test_nans(ret, 1040)
    return ret


@cache('step')
def total_fed_nre_heatxdashxnc():
    """
    total fed nre heatxdashxnc



    component


    """
    
    ret = np.maximum(
        0, (total_fed_heatxdashxnc_ej() - total_fe_real_supply_res_for_heatxdashxnc_ej()))
    #test_nans(ret, 1041)
    return ret


@cache('run')
def deactivate_heat_dem_correctionxquestx():
    """
    deactivate heat dem correctionxquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 1042)
    return ret


@cache('run')
def share_feh_over_fed_solid_bioe():
    """
    share feh over fed solid bioe



    constant


    """
    
    ret = 0.13480214704780927
    #test_nans(ret, 1043)
    return ret


@cache('step')
def abundance_heat():
    """
    abundance heat



    component


    """
    
    ret = functions.if_then_else(
        total_fe_heat_generation_ej() > total_fed_heat_ej(), 1, 1 - functions.zidz(
            total_fed_heat_ej() - total_fe_heat_generation_ej(), total_fed_heat_ej()))
    #test_nans(ret, 1044)
    return ret


@cache('step')
def pes_coal_for_heatxdashxnc_plants():
    """
    pes coal for heatxdashxnc plants



    component


    """
    
    ret = extraction_coal_ej() * share_coal_dem_for_heatxdashxnc()
    #test_nans(ret, 1045)
    return ret


@cache('step')
def required_fed_sectors_by_fuel():
    """
    required fed sectors by fuel



    component


    """
    
    ret = functions.sum(dim='sectors', data=required_final_energy_by_sector_and_fuel())
    #test_nans(ret, 1046)
    return ret


@cache('run')
def share_trad_biomass_vs_solids_in_households():
    """
    share trad biomass vs solids in households



    constant


    """
    
    ret = 0.9
    #test_nans(ret, 1047)
    return ret


@cache('step')
def share_global_pop_dependent_on_trad_biomass():
    """
    share global pop dependent on trad biomass



    component


    """
    
    ret = population_dependent_on_trad_biomass() / population()
    #test_nans(ret, 1048)
    return ret


@cache('step')
def pes_oil_for_heatxdashxcom_plants():
    """
    pes oil for heatxdashxcom plants



    component


    """
    
    ret = pes_oil_ej() * share_oil_dem_for_heatxdashxcom()
    #test_nans(ret, 1049)
    return ret


@cache('step')
def share_coal_dem_for_heatxdashxnc():
    """
    share coal dem for heatxdashxnc



    component


    """
    
    ret = functions.zidz(ped_coal_heatxdashxnc(), ped_coal_ej())
    #test_nans(ret, 1050)
    return ret


@cache('step')
def res_heatxdashxcom_tot_overcapacity():
    """
    res heatxdashxcom tot overcapacity



    component


    """
    
    ret = functions.if_then_else(abs(potential_fes_tot_res_for_heatxdashxcom_ej() - 0) < equality_threshold, 0,
                                  (potential_fes_tot_res_for_heatxdashxcom_ej() -
                                   fe_real_supply_res_for_heatxdashxcom_tot_ej()) /
                                  potential_fes_tot_res_for_heatxdashxcom_ej())
    #test_nans(ret, 1051)
    return ret


@cache('step')
def pes_natx_gas_for_heatxdashxcom_plants():
    """
    pes natx gas for heatxdashxcom plants



    component


    """
    
    ret = pes_natx_gas() * share_natx_gas_dem_for_heatxdashxcom()
    #test_nans(ret, 1052)
    return ret


@cache('step')
def pes_coal_for_heatxdashxcom_plants():
    """
    pes coal for heatxdashxcom plants



    component


    """
    
    ret = extraction_coal_ej() * share_coal_dem_for_heatxdashxcom()
    #test_nans(ret, 1053)
    return ret


@cache('step')
def required_heatxdashxcom():
    """
    required heatxdashxcom



    component


    """
    
    ret = required_fed_by_fuel_before_heat_correction().loc['heat'].values
    #test_nans(ret, 1054)
    return ret


@cache('step')
def variation_labour_share():
    """
    variation labour share



    component


    """
    
    ret = growth_labour_share() * labour_share()
    #test_nans(ret, 1055)
    return ret


@cache('step')
def variation_household_demand():
    """
    variation household demand



    component


    """
    
    ret = functions.if_then_else(time() < 2009,
                                  variation_historic_demand(),
                                  np.exp(bet_x0x_lab()) * np.exp(beta_x1x_lab()) *
                                  ((lc_next_step())**beta_x2x_lab() - (lc())**beta_x2x_lab()))
    #test_nans(ret, 1056)
    return ret


@cache('step')
def labour_share():
    """
    labour share



    component


    """
    
    ret = integ_labour_share()
    #test_nans(ret, 1057)
    return ret


@cache('step')
def capital_share():
    """
    capital share



    component


    """
    
    ret = 1 - labour_share()
    #test_nans(ret, 1058)
    return ret


@cache('step')
def growth_capital_share():
    """
    growth capital share



    component


    """
    
    ret = -growth_labour_share() * labour_share() / (1 - labour_share())
    #test_nans(ret, 1059)
    return ret


@cache('run')
def include_materials_for_overgridsxquestx():
    """
    include materials for overgridsxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1060)
    return ret


@cache('run')
def all_minerals_virginxquestx():
    """
    all minerals virginxquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 1061)
    return ret


@cache('step')
def materials_for_new_res_elec_per_capacity_installed():
    """
    materials for new res elec per capacity installed



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        materials_per_new_capacity_installed_xdashx_wind_onshore() +
        (materials_per_new_res_elec_capacity_installed_xdashx_hvdcs() +
         materials_per_new_res_elec_capacity_installed_xdashx_material_overgrid_high_power()) *
        include_materials_for_overgridsxquestx(),
        materials_per_new_capacity_installed_xdashx_wind_offshore() +
        (materials_per_new_res_elec_capacity_installed_xdashx_hvdcs() +
         materials_per_new_res_elec_capacity_installed_xdashx_material_overgrid_high_power()) *
        include_materials_for_overgridsxquestx(),
        materials_per_new_capacity_installed_xdashx_pv() +
        (materials_per_new_res_elec_capacity_installed_xdashx_hvdcs() +
         materials_per_new_res_elec_capacity_installed_xdashx_material_overgrid_high_power()) *
        include_materials_for_overgridsxquestx(),
        materials_per_new_capacity_installed_xdashx_csp() +
        (materials_per_new_res_elec_capacity_installed_xdashx_hvdcs() +
         materials_per_new_res_elec_capacity_installed_xdashx_material_overgrid_high_power()) *
        include_materials_for_overgridsxquestx()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp'], 'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['res elec', 'materials'])
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1062)
    return ret


@cache('step')
def cedtot_per_material_res_elec_var():
    """
    cedtot per material res elec var



    component


    """
    
    ret = ced_new_cap_per_material_res_elec_var(
    ) + ced_oxandxm_over_lifetime_per_material_res_elec_var()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1063)
    return ret


@cache('step')
def cedtot_per_tw_res_elec_var():
    """
    cedtot per tw res elec var



    component


    """
    
    ret = functions.sum(dim='materials', data=cedtot_per_tw_per_material_res_elec_var()) * 1e+006
    #test_nans(ret, 1064)
    return ret


@cache('step')
def cedtot_per_tw_per_material_res_elec_var():
    """
    cedtot per tw per material res elec var



    component


    """
    
    ret = functions.zidz(cedtot_per_material_res_elec_var(),
                          res_elec_capacity_under_construction_tw())
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1065)
    return ret


@cache('step')
def demand_by_sector_fd():
    """
    demand by sector fd



    component


    """
    
    ret = integ_demand_by_sector_fd()
    #test_nans(ret, 1066)
    return ret


@cache('step')
def sum_variation():
    """
    sum variation



    component


    """
    
    ret = functions.sum(variation_demand_flow_fd())
    #test_nans(ret, 1067)
    return ret


@cache('step')
def cc_sectoral_next_step():
    """
    cc sectoral next step



    component


    """
    
    ret = cc_sectoral() + variation_cc_sectoral()
    #test_nans(ret, 1068)
    return ret


@cache('step')
def gross_fixed_capital_formation():
    """
    gross fixed capital formation



    component


    """
    
    ret = integ_gross_fixed_capital_formation()
    #test_nans(ret, 1069)
    return ret


@cache('step')
def pct_gfcf_vs_gfcfxplusxhd():
    """
    pct gfcf vs gfcfxplusxhd



    component


    """
    
    ret = gross_fixed_capital_formation() / (gross_fixed_capital_formation() + household_demand())
    #test_nans(ret, 1070)
    return ret


@cache('step')
def household_demand():
    """
    household demand



    component


    """
    
    ret = integ_household_demand()
    #test_nans(ret, 1071)
    return ret


@cache('step')
def lc_next_step():
    """
    lc next step



    component


    """
    
    ret = lc() + variation_lc()
    #test_nans(ret, 1072)
    return ret


@cache('step')
def household_demand_total():
    """
    household demand total



    component


    """
    
    ret = functions.sum(household_demand())
    #test_nans(ret, 1073)
    return ret


@cache('step')
def quality_of_electricity_x2015x():
    """
    quality of electricity x2015x



    component


    """
    
    ret = functions.if_then_else(time() < 2015, dynamic_quality_of_electricity(), aux3())
    #test_nans(ret, 1074)
    return ret


@cache('step')
def dynamic_quality_of_electricity():
    """
    dynamic quality of electricity



    component


    """
    
    ret = real_tfec() / (tpes_ej() - total_real_nonxdashxenergy_use_consumption_ej())
    #test_nans(ret, 1075)
    return ret


@cache('step')
def quality_of_electricity():
    """
    quality of electricity



    component


    """
    
    ret = functions.if_then_else(abs(staticxdivxdynamic_quality_of_electricityxquestx() - 1) < equality_threshold,
                                  quality_of_electricity_x2015x(),
                                  dynamic_quality_of_electricity())
    #test_nans(ret, 1076)
    return ret


@cache('step')
def cp_exogenous_res_elec_reduction():
    """
    cp exogenous res elec reduction



    component


    """
    
    ret = xr.DataArray(data=[
        cp_exogenous_res_elec_dispatch_reduction(),
        cp_exogenous_res_elec_dispatch_reduction(),
        cp_exogenous_res_elec_dispatch_reduction(),
        cp_exogenous_res_elec_dispatch_reduction(),
        cp_exogenous_res_elec_var_reduction(),
        cp_exogenous_res_elec_var_reduction(),
        cp_exogenous_res_elec_var_reduction(),
        cp_exogenous_res_elec_var_reduction()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1077)
    return ret


@cache('run')
def res_elec_variablesxquestx():
    """
    res elec variablesxquestx



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,1.0,1.0,1.0,1.0],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1078)
    return ret


@cache('step')
def aux3():
    """
    aux3



    component


    """
    
    ret = delay_quality_of_electricity_x2015x_round003125__time_step___time_step_075_003125__time_step(
    )
    #test_nans(ret, 1079)
    return ret


@cache('step')
def fei_res_elec_var():
    """
    fei res elec var



    component


    """
    
    ret = (cedtot_new_cap_res_elec_var() *
            (1 + grid_correction_factor_res_elec()) + ced_decom_res_elec_capacity() +
            cedtot_oxandxm_res_elec_var()) * quality_of_electricity(
            ) + real_generation_res_elec_ej() * selfxdashxelectricity_consumption_res_elec()
    #test_nans(ret, 1080)
    return ret


@cache('step')
def xstaticx_eroi_res_elec():
    """
    xstaticx eroi res elec



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(fei_over_lifetime_res_elec_dispatch().loc['hydro'].values == 0, 0,
                               output_elec_over_lifetime_res_elec().loc['hydro'].values /
                               (fei_over_lifetime_res_elec_dispatch().loc['hydro'].values * quality_of_electricity())),
        functions.if_then_else(fei_over_lifetime_res_elec_dispatch().loc['geotxdashxelec'].values == 0, 0,
                               output_elec_over_lifetime_res_elec().loc['geotxdashxelec'].values /
                               (fei_over_lifetime_res_elec_dispatch().loc['geotxdashxelec'].values * quality_of_electricity())),
        functions.if_then_else(fei_over_lifetime_res_elec_dispatch().loc['solid bioexdashxelec'].values == 0, 0,
                               output_elec_over_lifetime_res_elec().loc['solid bioexdashxelec'].values /
                               (fei_over_lifetime_res_elec_dispatch().loc['solid bioexdashxelec'].values * quality_of_electricity())),
        functions.if_then_else(fei_over_lifetime_res_elec_dispatch().loc['oceanic'].values == 0, 0,
                               output_elec_over_lifetime_res_elec().loc['oceanic'].values /
                               (fei_over_lifetime_res_elec_dispatch().loc['oceanic'].values * quality_of_electricity())),
        functions.if_then_else(
            fei_over_lifetime_res_elec_var().loc['wind onshore'].values == 0, 0,
            output_elec_over_lifetime_res_elec().loc['wind onshore'].values / fei_over_lifetime_res_elec_var().loc['wind onshore'].values),
        functions.if_then_else(
            fei_over_lifetime_res_elec_var().loc['wind offshore'].values == 0, 0,
            output_elec_over_lifetime_res_elec().loc['wind offshore'].values / fei_over_lifetime_res_elec_var().loc['wind offshore'].values),
        functions.if_then_else(
            fei_over_lifetime_res_elec_var().loc['solar pv'].values == 0, 0,
            output_elec_over_lifetime_res_elec().loc['solar pv'].values / fei_over_lifetime_res_elec_var().loc['solar pv'].values),
        functions.if_then_else(
            fei_over_lifetime_res_elec_var().loc['csp'].values == 0, 0,
            functions.zidz(output_elec_over_lifetime_res_elec().loc['csp'].values , fei_over_lifetime_res_elec_var().loc['csp'].values))],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1081)
    return ret


@cache('step')
def materials_required_for_new_res_elec_mt():
    """
    materials required for new res elec mt



    component


    """
    
    ret = res_elec_capacity_under_construction_tw(
    ) * materials_for_new_res_elec_per_capacity_installed() * m_per_t() / kg_per_mt()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1082)
    return ret


@cache('step')
def fei_over_lifetime_res_elec_var():
    """
    fei over lifetime res elec var



    component


    """
    
    ret = (
        cedtot_new_cap_res_elec_var() *
        (1 + share_energy_requirements_for_decom_res_elec() + grid_correction_factor_res_elec()) +
        ced_oxandxm_over_lifetime_res_elec_var()) * quality_of_electricity(
        ) + output_elec_over_lifetime_res_elec() * selfxdashxelectricity_consumption_res_elec()
    #test_nans(ret, 1083)
    return ret


@cache('step')
def ced_decom_res_elec_capacity():
    """
    ced decom res elec capacity



    component


    """
    
    ret = functions.zidz(share_energy_requirements_for_decom_res_elec() *
                          cedtot_new_cap_res_elec_var() * wear_res_elec(),
                          res_elec_capacity_under_construction_tw())
    #test_nans(ret, 1084)
    return ret


@cache('step')
def ced_oxandxm_per_material_res_elec_var():
    """
    ced oxandxm per material res elec var



    component


    """
    
    ret = materials_required_for_oxandxm_res_elec_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1085)
    return ret


@cache('step')
def cedtot_new_cap_res_elec_var():
    """
    cedtot new cap res elec var



    component


    """
    
    ret = functions.sum(dim='materials', data=ced_new_cap_per_material_res_elec_var())
    #test_nans(ret, 1086)
    return ret


@cache('step')
def cedtot_oxandxm_res_elec_var():
    """
    cedtot oxandxm res elec var



    component


    """
    
    ret = functions.sum(dim='materials', data=ced_oxandxm_per_material_res_elec_var()
                          ) + total_energy_requirements_oxandxm_for_water_consumption_res_elec()
    #test_nans(ret, 1087)
    return ret


@cache('step')
def aux1():
    """
    aux1



    component


    """
    
    ret = delay_sharexdashxe_losses_cc_until_x2015_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 1088)
    return ret


@cache('run')
def b_lineal_regr():
    """
    b lineal regr



    component


    """
    
    ret = esoi_phs_depleted_potential() - a_lineal_regr() * max_capacity_potential_phs()
    #test_nans(ret, 1089)
    return ret


@cache('run')
def a_lineal_regr():
    """
    a lineal regr



    component


    """
    
    ret = (esoi_phs_full_potential() - esoi_phs_depleted_potential()) / (
        0 - max_capacity_potential_phs())
    #test_nans(ret, 1090)
    return ret


@cache('run')
def esoi_phs_depleted_potential():
    """
    esoi phs depleted potential



    constant


    """
    
    ret = 5
    #test_nans(ret, 1091)
    return ret


@cache('step')
def cc_total():
    """
    cc total



    component


    """
    
    ret = integ_cc_total()
    #test_nans(ret, 1092)
    return ret


@cache('run')
def initial_household_demand():
    """
    initial household demand



    constant


    """
    
    ret = xr.DataArray(
        data=[
            748525.184544, 41789.6873, 1723750.213929, 470469.2814293, 97413.51078294,
            27311.3812115, 218308.785156, 217859.5983007, 329088.21596, 91494.368471,
            50849.6902961, 98211.1859259, 169551.78439, 269085.07894, 559770.13765, 222979.9293546,
            425936.77411, 43757.5723889, 329630.413413, 953189.751819, 1442744.514577,
            1113431.49591, 512283.1868, 44308.95214, 137453.87786, 138435.92747, 324538.4833,
            992982.57458, 2441020.9794, 314984.6207, 376202.178, 340722.53, 1143478.152,
            959930.083, 47919.81617848
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    #test_nans(ret, 1093)
    return ret


@cache('step')
def total_demand():
    """
    total demand



    component


    """
    
    ret = functions.sum(demand_by_sector_fd()) / 1e+006
    #test_nans(ret, 1094)
    return ret


@cache('step')
def demand_not_covered_by_sector_fd():
    """
    demand not covered by sector fd



    component


    """
    
    ret = functions.if_then_else(time() > 2009, demand_by_sector_fd() - real_demand_by_sector(),
                                  0)
    #test_nans(ret, 1095)
    return ret


@cache('run')
def initial_demand_by_sectot():
    """
    initial demand by sectot



    constant


    """
    
    ret = xr.DataArray(
        data=[
            821277.351644, 106956.558, 1761460.089629, 500387.9312293, 102249.58580294,
            58713.0731415, 261905.835856, 224714.8110007, 401281.55396, 113840.587681,
            64807.2040561, 259983.9362259, 777405.42339, 895155.41074, 1109416.05655,
            323158.7554546, 440415.90923, 3373315.0363289, 352813.949813, 1263743.815819,
            1598464.786877, 1135913.81656, 579016.9834, 49225.02703, 143095.69405, 165486.95717,
            336812.13263, 1035860.06854, 2560435.958453, 715097.68812, 3163684.928, 1176056.855178,
            2116985.945408, 1121312.875, 48106.64623338
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    #test_nans(ret, 1096)
    return ret


@cache('run')
def beta_x1x_lab():
    """
    beta x1x lab



    constant


    """
    
    ret = xr.DataArray(
        data=[
            0., -3.041303, 0.8496438, -0.4561882, -2.064467, -3.356857, -1.226881, -1.186757,
            -0.712791, -2.035976, -2.704609, -2.061479, -1.488448, -0.675253, -0.1846172, -1.18592,
            -0.5334909, -2.756193, -0.7490777, 0.3986385, 0.7057005, 0.4404486, -0.3547073,
            -2.553138, -1.624648, -1.630129, -0.3472749, 0.4875999, 1.229081, -0.7056298,
            -0.7456611, -0.7185996, 0.5389905, 0.3212347, -2.770384
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    #test_nans(ret, 1097)
    return ret


@cache('run')
def initial_lc_total():
    """
    initial lc total



    constant


    """
    
    ret = 1.85847e+007
    #test_nans(ret, 1098)
    return ret


@cache('run')
def initial_gfcf():
    """
    initial gfcf



    constant


    """
    
    ret = xr.DataArray(
        data=[
            3.5732832e+04, 4.9168223e+04, 3.1533279e+03, 9.8054414e+03, 6.5427881e+02,
            2.6489537e+04, 3.6798738e+04, 1.3280853e+03, 1.0869645e+04, 1.2706039e+04,
            5.8332832e+03, 1.5145072e+05, 5.9107069e+05, 5.9219844e+05, 5.2574288e+05,
            8.5966852e+04, 9.1633457e+03, 3.3157355e+06, 2.0114217e+04, 2.7278438e+05,
            1.3478013e+05, 6.3380215e+03, 4.9677738e+04, 2.4625908e+03, 4.3897974e+03,
            7.1630146e+03, 9.4000742e+03, 3.7534692e+03, 1.0564882e+05, 3.2219016e+05,
            1.6466713e+04, 8.7741577e+02, 2.0205743e+03, 1.9066770e+04, 1.3384763e+02
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    #test_nans(ret, 1099)
    return ret


@cache('run')
def beta_x0x_cap():
    """
    beta x0x cap



    constant


    """
    
    ret = 5.09048
    #test_nans(ret, 1100)
    return ret


@cache('run')
def initial_cc_total():
    """
    initial cc total



    constant


    """
    
    ret = 1.05739e+007
    #test_nans(ret, 1101)
    return ret


@cache('run')
def beta_x2x_cap():
    """
    beta x2x cap



    constant


    """
    
    ret = 0.434675
    #test_nans(ret, 1102)
    return ret


@cache('step')
def demand_not_covered_total_fd():
    """
    demand not covered total fd



    component


    """
    
    ret = functions.sum(demand_not_covered_by_sector_fd())
    #test_nans(ret, 1103)
    return ret


@cache('run')
def beta_x1x_cap():
    """
    beta x1x cap



    constant


    """
    
    ret = xr.DataArray(
        data=[
            0., 0.6549913, -2.409549, -0.7288079, -2.802861, 0.2653398, 0.3796384, -3.044304,
            -1.282831, -0.5025426, -1.428307, 1.431519, 3.098772, 2.975706, 2.985555, 1.702436,
            -1.410645, 4.458449, 0.0193238, 1.628343, 1.206609, -2.001936, 0.4501797, -1.538287,
            -1.335444, -1.169046, -0.8956122, -2.335751, 0.3825687, 2.065982, -1.44898, -2.976849,
            -2.671927, -0.4900947, -3.099085
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    #test_nans(ret, 1104)
    return ret


@cache('step')
def lc():
    """
    lc



    component


    """
    
    ret = integ_lc()
    #test_nans(ret, 1105)
    return ret


@cache('run')
def bet_x0x_lab():
    """
    bet x0x lab



    constant


    """
    
    ret = -5.52042
    #test_nans(ret, 1106)
    return ret


@cache('step')
def tped_by_fuel():
    """
    tped by fuel



    component


    """
    
    ret = extraction_uranium_ej() + pe_supply_res_nonxdashxelec_ej(
    ) + pe_elec_generation_from_res_ej() + ped_total_oil_ej() + ped_coal_ej() + ped_natx_gas_ej(
    ) + pes_waste_ej()
    #test_nans(ret, 1107)
    return ret


@cache('step')
def abundance_tpe():
    """
    abundance tpe



    component


    """
    
    ret = functions.if_then_else(tpes_ej() > tped_by_fuel(), 1,
                                  1 - ((tped_by_fuel() - tpes_ej()) / tped_by_fuel()))
    #test_nans(ret, 1108)
    return ret


@cache('run')
def beta_x2x_lab():
    """
    beta x2x lab



    constant


    """
    
    ret = 1.13545
    #test_nans(ret, 1109)
    return ret


@cache('run')
def initial_global_energy_intensity_by_sector_x2009x():
    """
    initial global energy intensity by sector x2009x



    constant


    """
    
    ret = xr.DataArray(
        data=[
            3.86995232, 8.59048794, 2.2730697, 2.0108685, 0.93840275, 3.07986057, 3.88216863,
            11.23922135, 5.78914497, 6.90556672, 13.82397954, 6.66828099, 0.8634489, 0.4932426,
            0.66711798, 4.29142682, 8.77651863, 0.93924068, 0.95827484, 0.73265328, 1.63878385,
            2.61806399, 8.12139811, 23.63826497, 25.08570885, 3.1151168, 0.9165701, 0.378893,
            0.51315756, 0.85824151, 2.28456015, 1.98597538, 1.41782258, 1.73390014, 0.
        ],
        coords={
            'sectors': [
                'agriculture hunting forestry and fishing', 'mining and quarrying',
                'food beverages and tobacco', 'textiles and textile products',
                'leather leather and footwear', 'wood and products of woood and cork',
                'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel',
                'chemicals and chemical products', 'rubber and plastics',
                'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec',
                'electrical and optical equipment', 'transport equipment',
                'manufacturing nec recycling', 'electricity gas and water supply', 'construction',
                'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel',
                'wholesale trade and commissions trade except of motor vehicles and motorcycles',
                'retail trade except of motor vehicles and motorcycles repair of household goods',
                'hotels and restaurants', 'inland transport', 'water transport', 'air transport',
                'other supporting and auxiliary transport activities activities of travel agencies',
                'post and telecommunications', 'financial intermedation', 'real estate activities',
                'renting od meq and other business activities',
                'public admin and defence compulsory social security', 'education',
                'health and social work', 'other community social and persona services',
                'private households with employed persons'
            ]
        },
        dims=['sectors'])
    #test_nans(ret, 1110)
    return ret


@cache('step')
def available_pe_potential_solid_bioe_for_heat_ej():
    """
    available pe potential solid bioe for heat ej



    component


    """
    
    ret = np.maximum(
        0, total_pe_solid_bioe_potential_heatxplusxelec_ej() - pe_bioe_for_elec_generation_ej())
    #test_nans(ret, 1111)
    return ret


@cache('step')
def max_pe_potential_res_for_heat():
    """
    max pe potential res for heat



    component


    """
    
    ret = xr.DataArray(data=[
        fe_solar_potential_for_heat(),
        geot_pe_potential_for_heat_ej(),
        available_pe_potential_solid_bioe_for_heat_ej()],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1112)
    return ret


@cache('step')
def pes_res_for_heatxdashxcom_by_techn():
    """
    pes res for heatxdashxcom by techn



    component


    """
    
    ret = xr.DataArray(data=[
        fe_real_generation_res_heatxdashxcom_ej().loc['solarxdashxheat'].values * efficiency_solar_panels_for_heat() /
        efficiency_res_heat().loc['solarxdashxheat'].values,
        fe_real_generation_res_heatxdashxcom_ej().loc['geotxdashxheat'].values / efficiency_res_heat().loc['geotxdashxheat'].values,
        fe_real_generation_res_heatxdashxcom_ej().loc['solid bioexdashxheat'].values / efficiency_res_heat().loc['solid bioexdashxheat'].values],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1113)
    return ret


@cache('run')
def p_res_for_heat():
    """
    p res for heat



    component


    """
    
    ret = xr.DataArray(data=[
        p_solar_for_heat(),
        p_geothermal_for_heat(),
        p_solid_bioe_for_heat()],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1114)
    return ret


@cache('step')
def fe_real_generation_res_heatxdashxcom_ej():
    """
    fe real generation res heatxdashxcom ej



    component


    """
    
    ret = potential_fes_res_for_heatxdashxcom_ej() * (1 - res_heatxdashxcom_tot_overcapacity())
    #test_nans(ret, 1115)
    return ret


@cache('step')
def potential_fes_tot_res_for_heatxdashxcom_ej():
    """
    potential fes tot res for heatxdashxcom ej



    component


    """
    
    ret = functions.sum(potential_fes_res_for_heatxdashxcom_ej())
    #test_nans(ret, 1116)
    return ret


@cache('step')
def available_potential_fe_solid_bioe_for_elec_ej():
    """
    available potential fe solid bioe for elec ej



    component


    """
    
    ret = available_pe_potential_solid_bioe_for_elec_ej() * efficiency_conversion_bioe_to_elec()
    #test_nans(ret, 1117)
    return ret


@cache('step')
def max_bioe_twe():
    """
    max bioe twe



    component


    """
    
    ret = available_potential_fe_solid_bioe_for_elec_ej() * twe_per_twh() / ej_per_twh()
    #test_nans(ret, 1118)
    return ret


@cache('step')
def remaining_potential_tot_res_heat():
    """
    remaining potential tot res heat



    component


    """
    
    ret = functions.if_then_else(max_pe_potential_tot_res_heat_ej() > pes_tot_res_for_heat(),
                                  functions.zidz(
                                      max_pe_potential_tot_res_heat_ej() - pes_tot_res_for_heat(),
                                      max_pe_potential_tot_res_heat_ej()), 0)
    #test_nans(ret, 1119)
    return ret


@cache('step')
def potential_fes_heatxdashxcom_nuclear_chp_plants_ej():
    """
    potential fes heatxdashxcom nuclear chp plants ej



    component


    """
    
    ret = fe_nuclear_elec_generation_twh(
    ) * share_of_heat_production_in_chp_plants_vs_total_nucelar_elec_generation()
    #test_nans(ret, 1120)
    return ret


@cache('run')
def share_of_heat_production_in_chp_plants_vs_total_nucelar_elec_generation():
    """
    share of heat production in chp plants vs total nucelar elec generation



    constant


    """
    
    ret = 9.150844475380217e-06
    #test_nans(ret, 1121)
    return ret


@cache('step')
def pes_waste_for_elec_plants():
    """
    pes waste for elec plants



    component


    """
    
    ret = pes_waste_ej() * share_pes_waste_for_elec_plants()
    #test_nans(ret, 1122)
    return ret


@cache('step')
def losses_chp_biogas():
    """
    losses chp biogas



    component


    """
    
    ret = pes_biogas_for_chp() - fes_heatxdashxcom_from_biogas_in_chp_plants(
    ) - fes_elec_from_biogas_in_chp_plants()
    #test_nans(ret, 1123)
    return ret


@cache('step')
def losses_chp_waste():
    """
    losses chp waste



    component


    """
    
    ret = pes_waste_for_chp_plants() - fes_elec_from_waste_in_chp_plants(
    ) - fes_heatxdashxcom_from_waste_in_chp_plants()
    #test_nans(ret, 1124)
    return ret


@cache('step')
def share_pes_biogas_for_heat():
    """
    share pes biogas for heat



    component


    """
    
    ret = pes_tot_biogas_for_heatxdashxcom() / pes_biogas_ej()
    #test_nans(ret, 1125)
    return ret


@cache('step')
def pe_losses_res_for_elec():
    """
    pe losses res for elec



    component


    """
    
    ret = pe_losses_bioe_for_elec_ej() + pe_losses_biogas_for_elec() + pe_losses_waste_for_elec()
    #test_nans(ret, 1126)
    return ret


@cache('step')
def pes_tot_waste_for_elec():
    """
    pes tot waste for elec



    component


    """
    
    ret = pes_waste_for_elec_plants() + fes_elec_from_waste_in_chp_plants() + losses_chp_waste(
    ) * share_efficiency_waste_for_elec_in_chp_plants()
    #test_nans(ret, 1127)
    return ret


@cache('step')
def pes_tot_waste_for_heatxdashxcom():
    """
    pes tot waste for heatxdashxcom



    component


    """
    
    ret = pes_waste_for_heatxdashxcom_plants() + fes_heatxdashxcom_from_waste_in_chp_plants(
    ) + losses_chp_waste() * (1 - share_efficiency_waste_for_elec_in_chp_plants())
    #test_nans(ret, 1128)
    return ret


@cache('step')
def abundance_res_heatxdashxcom2():
    """
    abundance res heatxdashxcom2



    component


    """
    
    ret = np.sqrt(abundance_res_heatxdashxcom())
    #test_nans(ret, 1129)
    return ret


@cache('step')
def max_pe_potential_biogas_for_heat():
    """
    max pe potential biogas for heat



    component


    """
    
    ret = max_biogas_ej() * share_pes_biogas_for_heat()
    #test_nans(ret, 1130)
    return ret


@cache('run')
def share_efficiency_biogas_for_elec_in_chp_plants():
    """
    share efficiency biogas for elec in chp plants



    component


    """
    
    ret = efficiency_biogas_for_elec_chp_plants() / (
        efficiency_biogas_for_elec_chp_plants() + efficiency_biogas_for_heat_chp_plants())
    #test_nans(ret, 1131)
    return ret


@cache('run')
def share_efficiency_waste_for_elec_in_chp_plants():
    """
    share efficiency waste for elec in chp plants



    component


    """
    
    ret = efficiency_waste_for_elec_chp_plants() / (
        efficiency_waste_for_elec_chp_plants() + efficiency_waste_for_heat_chp_plants())
    #test_nans(ret, 1132)
    return ret


@cache('step')
def max_potential_pe_nonxdashxelectric_res():
    """
    max potential pe nonxdashxelectric res



    component


    """
    
    ret = max_pe_potential_tot_res_heat_ej() + max_peavail_biofuels_potential()
    #test_nans(ret, 1133)
    return ret


@cache('step')
def pes_tot_biogas_for_elec():
    """
    pes tot biogas for elec



    component


    """
    
    ret = pes_biogas_for_elec_plants() + fes_elec_from_biogas_in_chp_plants() + losses_chp_biogas(
    ) * share_efficiency_biogas_for_elec_in_chp_plants()
    #test_nans(ret, 1134)
    return ret


@cache('step')
def pes_tot_biogas_for_heatxdashxcom():
    """
    pes tot biogas for heatxdashxcom



    component


    """
    
    ret = pes_biogas_for_heatxdashxcom_plants() + fes_heatxdashxcom_from_biogas_in_chp_plants(
    ) + losses_chp_biogas() * (1 - share_efficiency_biogas_for_elec_in_chp_plants())
    #test_nans(ret, 1135)
    return ret


@cache('step')
def pe_losses_nre_elec_generation():
    """
    pe losses nre elec generation



    component


    """
    
    ret = pe_losses_coal_for_elec_ej() + pe_losses_conv_gas_for_elec_ej(
    ) + pe_losses_oil_for_elec_ej() + pe_losses_uncon_gas_for_elec_ej(
    ) + pe_losses_uranium_for_elec_ej()
    #test_nans(ret, 1136)
    return ret


@cache('step')
def elec_gen_related_losses_ej():
    """
    elec gen related losses ej



    component


    """
    
    ret = pe_losses_nre_elec_generation() + pe_losses_res_for_elec()
    #test_nans(ret, 1137)
    return ret


@cache('step')
def pe_losses_biogas_for_elec():
    """
    pe losses biogas for elec



    component


    """
    
    ret = pes_tot_biogas_for_elec() - fes_elec_from_biogas_ej()
    #test_nans(ret, 1138)
    return ret


@cache('step')
def pe_losses_waste_for_elec():
    """
    pe losses waste for elec



    component


    """
    
    ret = pes_tot_waste_for_elec() - fes_elec_from_waste_ej()
    #test_nans(ret, 1139)
    return ret


@cache('step')
def max_pe_potential_tot_res_heat_ej():
    """
    max pe potential tot res heat ej



    component


    """
    
    ret = max_pe_potential_biogas_for_heat() + functions.sum(max_pe_potential_res_for_heat())
    #test_nans(ret, 1140)
    return ret


@cache('step')
def share_pes_biogas_for_elec():
    """
    share pes biogas for elec



    component


    """
    
    ret = pes_tot_biogas_for_elec() / pes_biogas_ej()
    #test_nans(ret, 1141)
    return ret


@cache('run')
def p_hydro_growth():
    """
    p hydro growth



    constant


    """
    
    ret = 0.028
    #test_nans(ret, 1142)
    return ret


@cache('step')
def fe_real_tot_generation_res_elec_twh():
    """
    fe real tot generation res elec twh



    component


    """
    
    ret = np.minimum(
        np.maximum(total_fe_elec_demand_after_priorities_twh(), 0),
        potential_tot_generation_res_elec_twh())
    #test_nans(ret, 1143)
    return ret


@cache('run')
def p_solar_pv_growth():
    """
    p solar pv growth



    constant


    """
    
    ret = 0.35
    #test_nans(ret, 1144)
    return ret


@cache('step')
def remaining_potential_constraint_on_new_res_heat_capacity():
    """
    remaining potential constraint on new res heat capacity



    component


    """
    
    ret = functions.if_then_else(
        remaining_potential_res_for_heat() > threshold_remaining_potential_new_capacity(), 1,
        remaining_potential_res_for_heat() * (1 / threshold_remaining_potential_new_capacity()))
    #test_nans(ret, 1145)
    return ret


@cache('step')
def total_fe_elec_generation_twh():
    """
    total fe elec generation twh



    component


    """
    
    ret = fe_elec_generation_from_nre_twh() + fe_tot_generation_all_res_elec_twh(
    ) + fes_elec_from_waste_twh()
    #test_nans(ret, 1146)
    return ret


@cache('step')
def fe_tot_generation_all_res_elec_twh():
    """
    fe tot generation all res elec twh



    component


    """
    
    ret = fe_real_tot_generation_res_elec_twh() + fes_elec_from_res_with_priority_twh()
    #test_nans(ret, 1147)
    return ret


@cache('run')
def p_oceanic_growth():
    """
    p oceanic growth



    constant


    """
    
    ret = 0.048
    #test_nans(ret, 1148)
    return ret


@cache('step')
def pe_elec_generation_from_res_ej():
    """
    pe elec generation from res ej



    component


    """
    
    ret = pe_bioe_for_elec_generation_ej() + pe_geotxdashxelec_for_elec_generation_ej(
    ) + pe_hydro_for_elec_generation_ej() + pe_oceanic_for_elec_generation_ej(
    ) + pe_solar_pv_for_elec_generation_ej() + pe_csp_for_elec_generation_ej(
    ) + pe_onshore_wind_for_elec_generation_ej() + pe_offshore_wind_for_elec_generation_ej(
    ) + pes_tot_biogas_for_elec()
    #test_nans(ret, 1149)
    return ret


@cache('run')
def p_res_elec_growth():
    """
    p res elec growth



    component


    """
    
    ret = xr.DataArray(data=[
        p_hydro_growth(),
        p_geot_growth(),
        p_solid_bioexdashxelec_growth(),
        p_oceanic_growth(),
        p_wind_onshore_growth(),
        p_wind_offshore_growth(),
        p_solar_pv_growth(),
        p_csp_growth()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1150)
    return ret


@cache('step')
def share_res_electricity_generation():
    """
    share res electricity generation



    component


    """
    
    ret = fe_tot_generation_all_res_elec_twh() / total_fe_elec_generation_twh()
    #test_nans(ret, 1151)
    return ret


@cache('run')
def p_solid_bioexdashxelec_growth():
    """
    p solid bioexdashxelec growth



    constant


    """
    
    ret = 0.072
    #test_nans(ret, 1152)
    return ret


@cache('run')
def p_csp_growth():
    """
    p csp growth



    constant


    """
    
    ret = 0.295
    #test_nans(ret, 1153)
    return ret


@cache('step')
def remaining_potential_tot_res_elec():
    """
    remaining potential tot res elec



    component


    """
    
    ret = functions.if_then_else(
        max_potential_tot_res_elec_twh() > fe_tot_generation_all_res_elec_twh(),
        (max_potential_tot_res_elec_twh() - fe_tot_generation_all_res_elec_twh()) /
        max_potential_tot_res_elec_twh(), 0)
    #test_nans(ret, 1154)
    return ret


@cache('run')
def p_wind_offshore_growth():
    """
    p wind offshore growth



    constant


    """
    
    ret = 0.41
    #test_nans(ret, 1155)
    return ret


@cache('run')
def p_wind_onshore_growth():
    """
    p wind onshore growth



    constant


    """
    
    ret = 0.251
    #test_nans(ret, 1156)
    return ret


@cache('step')
def elec_generation_dispatch_from_res_twh():
    """
    elec generation dispatch from res twh



    component


    """
    
    ret = fe_elec_generation_from_bioe_twh() + fe_elec_generation_from_geotxdashxelec_twh(
    ) + fe_elec_generation_from_hydro_twh() + fe_elec_generation_from_oceanic_twh(
    ) + fes_elec_from_biogas_twh()
    #test_nans(ret, 1157)
    return ret


@cache('step')
def fes_elec_from_res_with_priority_twh():
    """
    fes elec from res with priority twh



    component


    """
    
    ret = fes_elec_from_biogas_twh()
    #test_nans(ret, 1158)
    return ret


@cache('step')
def abundance_res_elec():
    """
    abundance res elec



    component


    """
    
    ret = functions.if_then_else(
        abs(total_fe_elec_demand_after_priorities_twh() - 0) < equality_threshold, 0,
        functions.if_then_else(
            total_fe_elec_demand_after_priorities_twh() > fe_real_tot_generation_res_elec_twh(),
            (total_fe_elec_demand_after_priorities_twh() - fe_real_tot_generation_res_elec_twh()) /
            total_fe_elec_demand_after_priorities_twh(), 0))
    #test_nans(ret, 1159)
    return ret


@cache('step')
def fes_elec_from_biow():
    """
    fes elec from biow



    component


    """
    
    ret = real_generation_res_elec_twh().loc['solid bioexdashxelec'].values + fes_elec_from_biogas_twh() + fes_elec_from_waste_twh()
    #test_nans(ret, 1160)
    return ret


@cache('run')
def p_geot_growth():
    """
    p geot growth



    constant


    """
    
    ret = 0.024
    #test_nans(ret, 1161)
    return ret


@cache('step')
def max_potential_tot_res_elec_twh():
    """
    max potential tot res elec twh



    component


    """
    
    ret = functions.sum(max_potential_res_elec_twh()) + max_potential_phs_twe() / twe_per_twh(
    ) + max_biogas_ej() * share_pes_biogas_for_elec() / ej_per_twh()
    #test_nans(ret, 1162)
    return ret


@cache('run')
def p_solid_bioe_for_heat():
    """
    p solid bioe for heat



    constant


    """
    
    ret = 0.115
    #test_nans(ret, 1163)
    return ret


@cache('step')
def pe_real_generation_res_elec():
    """
    pe real generation res elec



    component


    """
    
    ret = xr.DataArray(data=[
        real_generation_res_elec_twh().loc['hydro'].values * ej_per_twh() * res_to_fossil_accounting(),
        real_generation_res_elec_twh().loc['geotxdashxelec'].values * ej_per_twh() * res_to_fossil_accounting(),
        (real_generation_res_elec_twh().loc['solid bioexdashxelec'].values / efficiency_conversion_bioe_to_elec()) * ej_per_twh(),
        real_generation_res_elec_twh().loc['oceanic'].values * ej_per_twh() * res_to_fossil_accounting(),
        real_generation_res_elec_twh().loc['wind onshore'].values * ej_per_twh() * res_to_fossil_accounting(),
        real_generation_res_elec_twh().loc['wind offshore'].values * ej_per_twh() * res_to_fossil_accounting(),
        real_generation_res_elec_twh().loc['solar pv'].values * ej_per_twh() * res_to_fossil_accounting(),
        real_generation_res_elec_twh().loc['csp'].values * ej_per_twh() * res_to_fossil_accounting()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1164)
    return ret


@cache('run')
def efficiency_res_heat():
    """
    efficiency res heat



    component


    """
    
    ret = xr.DataArray(data=[
        efficiency_solar_panels_for_heat() * losses_solar_for_heat(),
        efficiency_geothermal_for_heat(),
        efficiency_conversion_bioe_plants_to_heat()],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1165)
    return ret


@cache('run')
def constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx():
    """
    constant final energy intensity sectorsxandxhouseholds tdxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1166)
    return ret


@cache('step')
def share_oil_dem_for_heatxdashxcom():
    """
    share oil dem for heatxdashxcom



    component


    """
    
    ret = functions.if_then_else(ped_total_oil_ej() > 0,
                                  ped_oil_for_heat_plants_ej() / ped_total_oil_ej(), 0)
    #test_nans(ret, 1167)
    return ret


@cache('step')
def fe_elec_generation_from_total_oil_twh():
    """
    fe elec generation from total oil twh



    component


    """
    
    ret = pes_oil_ej() * share_oil_dem_for_elec() * efficiency_liquids_for_electricity(
    ) / ej_per_twh()
    #test_nans(ret, 1168)
    return ret


@cache('step')
def total_fe_heat_generation_ej():
    """
    total fe heat generation ej



    component


    """
    
    ret = fes_res_for_heat_ej() + fes_heatxdashxcom_from_waste_ej() + fes_nre_for_heat()
    #test_nans(ret, 1169)
    return ret


@cache('step')
def share_coal_dem_for_elec():
    """
    share coal dem for elec



    component


    """
    
    ret = functions.if_then_else(ped_coal_ej() > 0,
                                  pe_demand_coal_elec_plants_ej() / ped_coal_ej(), 0)
    #test_nans(ret, 1170)
    return ret


@cache('step')
def pe_losses_oil_for_elec_ej():
    """
    pe losses oil for elec ej



    component


    """
    
    ret = pes_oil_ej() * share_oil_dem_for_elec() * (1 - efficiency_liquids_for_electricity())
    #test_nans(ret, 1171)
    return ret


@cache('step')
def share_oil_dem_for_elec():
    """
    share oil dem for elec



    component


    """
    
    ret = functions.if_then_else(ped_total_oil_ej() > 0,
                                  pe_demand_oil_elec_plants_ej() / ped_total_oil_ej(), 0)
    #test_nans(ret, 1172)
    return ret


@cache('step')
def share_coal_dem_for_heatxdashxcom():
    """
    share coal dem for heatxdashxcom



    component


    """
    
    ret = functions.if_then_else(ped_coal_ej() > 0,
                                  ped_coal_for_heat_plants_ej() / ped_coal_ej(), 0)
    #test_nans(ret, 1173)
    return ret


@cache('step')
def fe_elec_generation_from_conv_gas_twh():
    """
    fe elec generation from conv gas twh



    component


    """
    
    ret = real_extraction_conv_gas_ej() * share_natx_gas_dem_for_elec(
    ) * efficiency_gas_for_electricity() / ej_per_twh()
    #test_nans(ret, 1174)
    return ret


@cache('step')
def share_res_heat_generation():
    """
    share res heat generation



    component


    """
    
    ret = fes_res_for_heat_ej() / total_fe_heat_generation_ej()
    #test_nans(ret, 1175)
    return ret


@cache('step')
def share_natx_gas_dem_for_elec():
    """
    share natx gas dem for elec



    component


    """
    
    ret = functions.if_then_else(ped_natx_gas_ej() > 0,
                                  pe_demand_gas_elec_plants_ej() / ped_natx_gas_ej(), 0)
    #test_nans(ret, 1176)
    return ret


@cache('step')
def pe_losses_conv_gas_for_elec_ej():
    """
    pe losses conv gas for elec ej



    component


    """
    
    ret = real_extraction_conv_gas_ej() * share_natx_gas_dem_for_elec() * (
        1 - efficiency_gas_for_electricity())
    #test_nans(ret, 1177)
    return ret


@cache('step')
def pe_losses_coal_for_elec_ej():
    """
    pe losses coal for elec ej



    component


    """
    
    ret = extraction_coal_ej() * share_coal_dem_for_elec() * (
        1 - efficiency_coal_for_electricity())
    #test_nans(ret, 1178)
    return ret


@cache('step')
def fe_elec_generation_from_coal_twh():
    """
    fe elec generation from coal twh



    component


    """
    
    ret = extraction_coal_ej() * efficiency_coal_for_electricity() * share_coal_dem_for_elec(
    ) / ej_per_twh()
    #test_nans(ret, 1179)
    return ret


@cache('step')
def share_natx_gas_dem_for_heatxdashxcom():
    """
    share natx gas dem for heatxdashxcom



    component


    """
    
    ret = functions.if_then_else(ped_natx_gas_ej() > 0,
                                  ped_gases_for_heat_plants_ej() / ped_natx_gas_ej(), 0)
    #test_nans(ret, 1180)
    return ret


@cache('step')
def year_scarcity_heat():
    """
    year scarcity heat



    component


    """
    
    ret = functions.if_then_else(abundance_heat() > 0.95, 0, time())
    #test_nans(ret, 1181)
    return ret


@cache('step')
def fes_heatxdashxcom_from_biogas_in_chp_plants():
    """
    fes heatxdashxcom from biogas in chp plants



    component


    """
    
    ret = pes_biogas_for_chp() * efficiency_biogas_for_heat_chp_plants()
    #test_nans(ret, 1182)
    return ret


@cache('run')
def efficiency_biogas_for_heat_plants():
    """
    efficiency biogas for heat plants



    constant


    """
    
    ret = 0.7357016523145766
    #test_nans(ret, 1183)
    return ret


@cache('step')
def fes_elec_from_biogas_in_chp_plants():
    """
    fes elec from biogas in chp plants



    component


    """
    
    ret = pes_biogas_for_chp() * efficiency_biogas_for_elec_chp_plants()
    #test_nans(ret, 1184)
    return ret


@cache('step')
def fes_elec_from_biogas_ej():
    """
    fes elec from biogas ej



    component


    """
    
    ret = fes_elec_from_biogas_in_chp_plants() + fes_elec_from_biogas_in_elec_plants()
    #test_nans(ret, 1185)
    return ret


@cache('step')
def pes_biogas_for_heatxdashxcom_plants():
    """
    pes biogas for heatxdashxcom plants



    component


    """
    
    ret = pes_biogas_ej() * share_pes_biogas_for_heatxdashxcom_plants()
    #test_nans(ret, 1186)
    return ret


@cache('step')
def fes_elec_from_biogas_in_elec_plants():
    """
    fes elec from biogas in elec plants



    component


    """
    
    ret = pes_biogas_for_elec_plants() * efficiency_biogas_for_elec_plants()
    #test_nans(ret, 1187)
    return ret


@cache('run')
def efficiency_biogas_for_elec_plants():
    """
    efficiency biogas for elec plants



    constant


    """
    
    ret = 0.29585066538416954
    #test_nans(ret, 1188)
    return ret


@cache('step')
def fes_biogas_for_heatxdashxcom_plants():
    """
    fes biogas for heatxdashxcom plants



    component


    """
    
    ret = pes_biogas_for_heatxdashxcom_plants() * efficiency_biogas_for_heat_plants()
    #test_nans(ret, 1189)
    return ret


@cache('step')
def pes_biogas_for_elec_plants():
    """
    pes biogas for elec plants



    component


    """
    
    ret = pes_biogas_ej() * share_pes_biogas_for_elec_plants()
    #test_nans(ret, 1190)
    return ret


@cache('run')
def share_pes_biogas_tfc():
    """
    share pes biogas tfc



    constant


    """
    
    ret = 0.4098230262379001
    #test_nans(ret, 1191)
    return ret


@cache('step')
def potential_pes_biogas_for_tfc():
    """
    potential pes biogas for tfc



    component


    """
    
    ret = pes_biogas_ej() * share_pes_biogas_tfc()
    #test_nans(ret, 1192)
    return ret


@cache('step')
def fes_elec_from_biogas_twh():
    """
    fes elec from biogas twh



    component


    """
    
    ret = fes_elec_from_biogas_ej() / ej_per_twh()
    #test_nans(ret, 1193)
    return ret


@cache('step')
def pes_biogas_for_chp():
    """
    pes biogas for chp



    component


    """
    
    ret = pes_biogas_ej() * share_pes_biogas_for_chp()
    #test_nans(ret, 1194)
    return ret


@cache('run')
def share_pes_biogas_for_heatxdashxcom_plants():
    """
    share pes biogas for heatxdashxcom plants



    constant


    """
    
    ret = 0.007384519814561412
    #test_nans(ret, 1195)
    return ret


@cache('run')
def efficiency_biogas_for_elec_chp_plants():
    """
    efficiency biogas for elec chp plants



    constant


    """
    
    ret = 0.3901372793044364
    #test_nans(ret, 1196)
    return ret


@cache('step')
def fes_heatxdashxcom_from_biogas_ej():
    """
    fes heatxdashxcom from biogas ej



    component


    """
    
    ret = fes_biogas_for_heatxdashxcom_plants() + fes_heatxdashxcom_from_biogas_in_chp_plants()
    #test_nans(ret, 1197)
    return ret


@cache('run')
def efficiency_biogas_for_heat_chp_plants():
    """
    efficiency biogas for heat chp plants



    constant


    """
    
    ret = 0.07955076327527948
    #test_nans(ret, 1198)
    return ret


@cache('run')
def share_pes_biogas_for_elec_plants():
    """
    share pes biogas for elec plants



    constant


    """
    
    ret = 0.34932337340295677
    #test_nans(ret, 1199)
    return ret


@cache('run')
def share_pes_biogas_for_chp():
    """
    share pes biogas for chp



    constant


    """
    
    ret = 0.23346908054458176
    #test_nans(ret, 1200)
    return ret


@cache('step')
def demand_elec_plants_fossil_fuels_twh():
    """
    demand elec plants fossil fuels twh



    component


    """
    
    ret = np.maximum(demand_elec_nre_twh() - fe_nuclear_elec_generation_twh() -
                      fes_elec_fossil_fuel_chp_plants_twh(), 0)
    #test_nans(ret, 1201)
    return ret


@cache('step')
def total_extraction_nre_ej():
    """
    total extraction nre ej



    component


    """
    
    ret = extraction_coal_ej() + real_extraction_conv_gas_ej() + real_extraction_conv_oil_ej(
    ) + real_extraction_unconv_gas_ej() + real_extraction_unconv_oil_ej() + extraction_uranium_ej(
    )
    #test_nans(ret, 1202)
    return ret


@cache('step')
def energy_cost_pressure():
    """
    energy cost pressure



    component


    """
    
    ret = 1 - abundance_of_energy()
    #test_nans(ret, 1203)
    return ret


@cache('step')
def pressure_to_improve_energy_intensity_efficiency():
    """
    pressure to improve energy intensity efficiency



    component


    """
    
    ret = energy_cost_pressure() + implementation_policy_to_improve_energy_intensity_effciency()
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 1204)
    return ret


@cache('step')
def abundance_of_energy():
    """
    abundance of energy



    component


    """
    
    ret = xr.DataArray(data=[1.0,1.0,
        abundance_liquids(),
        abundance_gases(),
        abundance_solids()],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1205)
    return ret


@cache('step')
def constrain_rr_improv_for_alt_techn_per_mineral():
    """
    constrain rr improv for alt techn per mineral



    component


    """
    
    ret = functions.if_then_else(
        recycling_rates_minerals_alt_techn() < max_recycling_rates_minerals(), 1, 0)
    #test_nans(ret, 1206)
    return ret


@cache('step')
def constrain_rr_improv_for_rest_per_mineral():
    """
    constrain rr improv for rest per mineral



    component


    """
    
    ret = functions.if_then_else(recycling_rates_minerals_rest() < max_recycling_rates_minerals(),
                                  1, 0)
    #test_nans(ret, 1207)
    return ret


@cache('step')
def ced_oxandxm_over_lifetime_per_material_res_elec_var():
    """
    ced oxandxm over lifetime per material res elec var



    component


    """
    
    ret = res_elec_capacity_under_construction_tw(
    ) * materials_for_oxandxm_per_capacity_installed_res_elec(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * lifetime_res_elec() * (
        m_per_t() / kg_per_mt()) * (kg_per_mt() / mj_per_ej())
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1208)
    return ret


@cache('step')
def energy_cons_per_unit_of_material_cons_for_res_elec():
    """
    energy cons per unit of material cons for res elec



    component


    """
    
    ret = recycling_rates_minerals_alt_techn(
    ) * initial_energy_cons_per_unit_of_material_cons_xrecycledx() + (
        1 - recycling_rates_minerals_alt_techn()
    ) * initial_energy_cons_per_unit_of_material_cons_xvirginx()
    #test_nans(ret, 1209)
    return ret


@cache('step')
def ced_new_cap_per_material_res_elec_var():
    """
    ced new cap per material res elec var



    component


    """
    
    ret = materials_required_for_new_res_elec_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1210)
    return ret


@cache('step')
def energy_required_for_material_consumption_for_new_res_elec():
    """
    energy required for material consumption for new res elec



    component


    """
    
    ret = materials_required_for_new_res_elec_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1211)
    return ret


@cache('step')
def energy_required_for_material_consumption_for_oxandxm_res_elec():
    """
    energy required for material consumption for oxandxm res elec



    component


    """
    
    ret = materials_required_for_oxandxm_res_elec_mt(
    ) * energy_cons_per_unit_of_material_cons_for_res_elec() * kg_per_mt() / mj_per_ej()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1212)
    return ret


@cache('step')
def share_other_cumulative_demand_to_extract_vs_resources_materials():
    """
    share other cumulative demand to extract vs resources materials



    component


    """
    
    ret = functions.if_then_else(
        current_mineral_resources_mt() <= 0, 0,
        cum_materials_to_extract_rest_from_x2015x() / current_mineral_resources_mt())
    #test_nans(ret, 1213)
    return ret


@cache('step')
def materials_to_extract_rest_from_x2015x_mt():
    """
    materials to extract rest from x2015x mt



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0, materials_to_extract_rest_mt())
    #test_nans(ret, 1214)
    return ret


@cache('step')
def total_recycled_materials_for_res_elec_mt():
    """
    total recycled materials for res elec mt



    component


    """
    
    ret = total_materials_required_for_res_elec_mt() - total_materials_to_extract_for_res_elec_mt(
    )
    #test_nans(ret, 1215)
    return ret


@cache('step')
def cum_materials_to_extract_rest():
    """
    cum materials to extract rest



    component


    """
    
    ret = integ_cum_materials_to_extract_rest()
    #test_nans(ret, 1216)
    return ret


@cache('step')
def cum_materials_to_extract_rest_from_x2015x():
    """
    cum materials to extract rest from x2015x



    component


    """
    
    ret = integ_cum_materials_to_extract_rest_from_x2015x()
    #test_nans(ret, 1217)
    return ret


@cache('step')
def share_other_cumulative_demand_to_extract_vs_reserves_materials():
    """
    share other cumulative demand to extract vs reserves materials



    component


    """
    
    ret = functions.if_then_else(
        current_mineral_reserves_mt() <= 0, 0,
        cum_materials_to_extract_rest_from_x2015x() / current_mineral_reserves_mt())
    #test_nans(ret, 1218)
    return ret


@cache('step')
def materials_availability_xannual_extractionx():
    """
    materials availability xannual extractionx



    component


    """
    
    ret = functions.if_then_else(
        total_materials_to_extract_mt() < maximum_annual_extraction_materials(), 1, 0)
    #test_nans(ret, 1219)
    return ret


@cache('run')
def mt_per_tonne():
    """
    mt per tonne



    constant


    """
    
    ret = 1e-06
    #test_nans(ret, 1220)
    return ret


@cache('run')
def initial_cumulated_material_requirements_for_rest_x1995x():
    """
    initial cumulated material requirements for rest x1995x



    constant


    """
    
    ret = 0
    #test_nans(ret, 1221)
    return ret


@cache('run')
def p_common_rr_minerals_variation_rest():
    """
    p common rr minerals variation rest



    constant


    """
    
    ret = 0
    #test_nans(ret, 1222)
    return ret


@cache('step')
def cum_materials_to_extract_for_res_elec_from_x2015x():
    """
    cum materials to extract for res elec from x2015x



    component


    """
    
    ret = integ_cum_materials_to_extract_for_res_elec_from_x2015x()
    #test_nans(ret, 1223)
    return ret


@cache('step')
def total_materials_to_extract_for_res_elec_mt():
    """
    total materials to extract for res elec mt



    component


    """
    
    ret = total_materials_required_for_res_elec_mt() * (1 - recycling_rates_minerals_alt_techn())
    #test_nans(ret, 1224)
    return ret


@cache('step')
def cum_materials_to_extract_for_res_elec():
    """
    cum materials to extract for res elec



    component


    """
    
    ret = integ_cum_materials_to_extract_for_res_elec()
    #test_nans(ret, 1225)
    return ret


@cache('step')
def total_materials_to_extract_for_res_elec_from_x2015x_mt():
    """
    total materials to extract for res elec from x2015x mt



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0, total_materials_to_extract_for_res_elec_mt())
    #test_nans(ret, 1226)
    return ret


@cache('run')
def max_recycling_rates_minerals():
    """
    max recycling rates minerals



    constant


    """
    
    ret = 0.95
    #test_nans(ret, 1227)
    return ret


@cache('run')
def p_common_rr_minerals_variation_alt_techn():
    """
    p common rr minerals variation alt techn



    constant


    """
    
    ret = 0
    #test_nans(ret, 1228)
    return ret


@cache('run')
def current_recycling_rates_minerals():
    """
    current recycling rates minerals



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.56,0.0,0.15,0.0,0.0,0.9,0.48,0.0,0.0,0.0,0.0,0.0,0.0,0.005,0.0,0.0,0.0,0.005,0.71,0.0,0.0,0.0,0.0,0.005,0.0,0.39,0.53,0.0,0.0,0.3,0.0,0.0,0.0,0.6,0.0,0.0,0.0,0.735,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.635,0.0,0.75,0.0,0.0,0.0,0.005,0.91,0.0,0.005,0.0,0.395],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1229)
    return ret


@cache('run')
def historic_improvement_recycling_rates_minerals():
    """
    historic improvement recycling rates minerals



    constant


    """
    
    ret = xr.DataArray(
        data=[
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.,
            0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.
        ],
        coords={
            'materials': [
                'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
                'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components',
                'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
                'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt',
                'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese',
                'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic',
                'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint',
                'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand',
                'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel',
                'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires',
                'zinc'
            ]
        },
        dims=['materials'])
    #test_nans(ret, 1230)
    return ret


@cache('run')
def min_energy_intensity_vs_intial():
    """
    min energy intensity vs intial



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 1231)
    return ret


@cache('step')
def global_energy_intensity_by_sector():
    """
    global energy intensity by sector



    component


    """
    
    ret = functions.sum(dim='final sources', data=evol_final_energy_intensity_by_sector_and_fuel())
    #test_nans(ret, 1232)
    return ret


@cache('run')
def initial_energy_cons_per_unit_of_material_cons_xrecycledx():
    """
    initial energy cons per unit of material cons xrecycledx



    component


    """
    
    ret = functions.if_then_else(
        abs(initial_energy_cons_per_unit_of_material_cons_xrecycledx_xdashx_data() - 0) < equality_threshold,
        initial_energy_cons_per_unit_of_material_cons_xvirginx(),
        initial_energy_cons_per_unit_of_material_cons_xrecycledx_xdashx_data())
    #test_nans(ret, 1233)
    return ret


@cache('step')
def materials_required_for_oxandxm_res_elec_mt():
    """
    materials required for oxandxm res elec mt



    component


    """
    
    ret = installed_capacity_res_elec_tw(
    ) * materials_for_oxandxm_per_capacity_installed_res_elec() * m_per_t() / kg_per_mt()
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1234)
    return ret


@cache('step')
def share_tot_cum_dem_vs_reserves_materials():
    """
    share tot cum dem vs reserves materials



    component


    """
    
    ret = functions.if_then_else(
        current_mineral_reserves_mt() <= 0, 0,
        total_cumulative_demand_materials_to_extract_from_x2015x() / current_mineral_reserves_mt())
    #test_nans(ret, 1235)
    return ret


@cache('step')
def share_tot_cum_dem_vs_resources_materials():
    """
    share tot cum dem vs resources materials



    component


    """
    
    ret = functions.if_then_else(current_mineral_resources_mt() <= 0, 0,
                                  total_cumulative_demand_materials_to_extract_from_x2015x() /
                                  current_mineral_resources_mt())
    #test_nans(ret, 1236)
    return ret


@cache('run')
def maximum_annual_extraction_materials():
    """
    maximum annual extraction materials



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.0,0.0,36.0,32.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.26,0.0,1.95,25.0,0.0,0.0,0.28,0.0,0.0,0.0,6.2,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,43.0,0.0,0.52,0.0,2100.0,0.0,0.0,0.0,0.0,0.0,0.0,17.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1237)
    return ret


@cache('step')
def materials_availability_xresourcesx():
    """
    materials availability xresourcesx



    component


    """
    
    ret = functions.if_then_else(share_tot_cum_dem_vs_resources_materials() < 1, 1, 0)
    #test_nans(ret, 1238)
    return ret


@cache('step')
def materials_availability_xreservesx():
    """
    materials availability xreservesx



    component


    """
    
    ret = functions.if_then_else(share_tot_cum_dem_vs_reserves_materials() < 1, 1, 0)
    #test_nans(ret, 1239)
    return ret


@cache('step')
def tpes_ej():
    """
    tpes ej



    component


    """
    
    ret = total_extraction_nre_ej() + tpe_from_res_ej() + pes_waste_ej()
    #test_nans(ret, 1240)
    return ret


@cache('step')
def fes_heatxdashxcom_from_waste_ej():
    """
    fes heatxdashxcom from waste ej



    component


    """
    
    ret = fes_waste_for_heatxdashxcom_plants() + fes_heatxdashxcom_from_waste_in_chp_plants()
    #test_nans(ret, 1241)
    return ret


@cache('step')
def fes_waste_for_heatxdashxcom_plants():
    """
    fes waste for heatxdashxcom plants



    component


    """
    
    ret = pes_waste_for_heatxdashxcom_plants() * efficiency_waste_for_heat_plants()
    #test_nans(ret, 1242)
    return ret


@cache('step')
def fes_elec_from_waste_in_chp_plants():
    """
    fes elec from waste in chp plants



    component


    """
    
    ret = pes_waste_for_chp_plants() * efficiency_waste_for_elec_chp_plants()
    #test_nans(ret, 1243)
    return ret


@cache('step')
def fes_elec_from_waste_in_elec_plants():
    """
    fes elec from waste in elec plants



    component


    """
    
    ret = pes_waste_for_elec_plants() * efficiency_waste_for_elec_plants()
    #test_nans(ret, 1244)
    return ret


@cache('step')
def fes_heatxdashxcom_from_waste_in_chp_plants():
    """
    fes heatxdashxcom from waste in chp plants



    component


    """
    
    ret = pes_waste_for_chp_plants() * efficiency_waste_for_heat_chp_plants()
    #test_nans(ret, 1245)
    return ret


@cache('step')
def pes_waste_for_heatxdashxcom_plants():
    """
    pes waste for heatxdashxcom plants



    component


    """
    
    ret = pes_waste_ej() * share_pes_waste_for_heatxdashxcom_plants()
    #test_nans(ret, 1246)
    return ret


@cache('run')
def share_pes_waste_for_heatxdashxcom_plants():
    """
    share pes waste for heatxdashxcom plants



    constant


    """
    
    ret = 0.09050587321058197
    #test_nans(ret, 1247)
    return ret


@cache('run')
def share_pes_waste_tfc():
    """
    share pes waste tfc



    constant


    """
    
    ret = 0.19153003642106164
    #test_nans(ret, 1248)
    return ret


@cache('run')
def share_pes_waste_for_chp():
    """
    share pes waste for chp



    constant


    """
    
    ret = 0.31858724855960674
    #test_nans(ret, 1249)
    return ret


@cache('run')
def share_pes_waste_for_elec_plants():
    """
    share pes waste for elec plants



    constant


    """
    
    ret = 0.39937684180874966
    #test_nans(ret, 1250)
    return ret


@cache('step')
def waste_change():
    """
    waste change



    component


    """
    
    ret = functions.if_then_else(annual_gdp_growth_rate() < 0,
                                  annual_gdp_growth_rate(), p_waste_change())
    #test_nans(ret, 1251)
    return ret


@cache('step')
def fes_elec_from_waste_ej():
    """
    fes elec from waste ej



    component


    """
    
    ret = fes_elec_from_waste_in_chp_plants() + fes_elec_from_waste_in_elec_plants()
    #test_nans(ret, 1252)
    return ret


@cache('step')
def fes_elec_from_waste_twh():
    """
    fes elec from waste twh



    component


    """
    
    ret = fes_elec_from_waste_ej() / ej_per_twh()
    #test_nans(ret, 1253)
    return ret


@cache('run')
def efficiency_waste_for_elec_plants():
    """
    efficiency waste for elec plants



    constant


    """
    
    ret = 0.24469672999410882
    #test_nans(ret, 1254)
    return ret


@cache('step')
def pes_waste_ej():
    """
    pes waste ej



    component


    """
    
    ret = integ_pes_waste_ej()
    #test_nans(ret, 1255)
    return ret


@cache('step')
def pes_waste_for_chp_plants():
    """
    pes waste for chp plants



    component


    """
    
    ret = pes_waste_ej() * share_pes_waste_for_chp()
    #test_nans(ret, 1256)
    return ret


@cache('run')
def efficiency_waste_for_elec_chp_plants():
    """
    efficiency waste for elec chp plants



    constant


    """
    
    ret = 0.17149829235093653
    #test_nans(ret, 1257)
    return ret


@cache('run')
def initial_pes_waste():
    """
    initial pes waste



    constant


    """
    
    ret = 0.94403359
    #test_nans(ret, 1258)
    return ret


@cache('step')
def pes_waste_for_tfc():
    """
    pes waste for tfc



    component


    """
    
    ret = pes_waste_ej() * share_pes_waste_tfc()
    #test_nans(ret, 1259)
    return ret


@cache('run')
def efficiency_waste_for_heat_chp_plants():
    """
    efficiency waste for heat chp plants



    constant


    """
    
    ret = 0.33592792668422766
    #test_nans(ret, 1260)
    return ret


@cache('run')
def p_waste_change():
    """
    p waste change



    constant


    """
    
    ret = 0.04436
    #test_nans(ret, 1261)
    return ret


@cache('run')
def efficiency_waste_for_heat_plants():
    """
    efficiency waste for heat plants



    constant


    """
    
    ret = 0.7066660881268088
    #test_nans(ret, 1262)
    return ret


@cache('run')
def max_waste():
    """
    max waste



    constant


    """
    
    ret = 4
    #test_nans(ret, 1263)
    return ret


@cache('step')
def pes_solids_bioe_xandx_waste_ej():
    """
    pes solids bioe xandx waste ej



    component


    """
    
    ret = pes_solids_bioe_ej() - pes_waste_ej()
    #test_nans(ret, 1264)
    return ret


@cache('run')
def past_waste_growth():
    """
    past waste growth



    constant


    """
    
    ret = 0.0443634053842744
    #test_nans(ret, 1265)
    return ret


@cache('step')
def ch4_emissions_oil():
    """
    ch4 emissions oil



    component


    """
    
    ret = pes_oil_ej() * gch4_per_mj_oil() * mj_per_ej() / g_per_mt()
    #test_nans(ret, 1266)
    return ret


@cache('run')
def gch4_per_mj_coal():
    """
    gch4 per mj coal



    constant


    """
    
    ret = 0.094
    #test_nans(ret, 1267)
    return ret


@cache('run')
def gch4_per_mj_oil():
    """
    gch4 per mj oil



    constant


    """
    
    ret = 0.094
    #test_nans(ret, 1268)
    return ret


@cache('step')
def fe_elec_generation_from_fossil_fuels_twh():
    """
    fe elec generation from fossil fuels twh



    component


    """
    
    ret = fe_elec_generation_from_coal_twh() + fe_elec_generation_from_conv_gas_twh(
    ) + fe_elec_generation_from_unconv_gas_twh() + fe_elec_generation_from_total_oil_twh(
    ) + fes_elec_fossil_fuel_chp_plants_twh()
    #test_nans(ret, 1269)
    return ret


@cache('step')
def pe_demand_uranium_ej():
    """
    pe demand uranium ej



    component


    """
    
    ret = potential_generation_nuclear_elec_twh() * ej_per_twh(
    ) / efficiency_uranium_for_electricity()
    #test_nans(ret, 1270)
    return ret


@cache('step')
def fes_elec_fossil_fuel_chp_plants_twh():
    """
    fes elec fossil fuel chp plants twh



    component


    """
    
    ret = fes_elec_fossil_fuel_chp_plants_ej() / ej_per_twh()
    #test_nans(ret, 1271)
    return ret


@cache('step')
def fes_heatxdashxcom_fossil_fuels_chp_plants_ej():
    """
    fes heatxdashxcom fossil fuels chp plants ej



    component


    """
    
    ret = fed_heat_fossil_fuels_chp_plants_ej()
    #test_nans(ret, 1272)
    return ret


@cache('step')
def potential_generation_nuclear_elec_twh():
    """
    potential generation nuclear elec twh



    component


    """
    
    ret = np.minimum(installed_capacity_nuclear_tw() * cp_nuclear() / twe_per_twh(),
                      demand_elec_nre_twh())
    #test_nans(ret, 1273)
    return ret


@cache('run')
def threshold_remaining_potential_new_capacity():
    """
    threshold remaining potential new capacity



    constant


    """
    
    ret = 0.5
    #test_nans(ret, 1274)
    return ret


@cache('step')
def installed_capacity_res_elec_tw():
    """
    installed capacity res elec tw



    component


    """
    
    ret = integ_installed_capacity_res_elec_tw()
    #test_nans(ret, 1275)
    return ret


@cache('run')
def total_time_planxplusxconstr_res_elec():
    """
    total time planxplusxconstr res elec



    component


    """
    
    ret = np.maximum(time_construction_res_elec(), time_step()) + np.maximum(
        time_planification_res_elec(), time_step())
    #test_nans(ret, 1276)
    return ret


@cache('step')
def new_required_capacity_res_elec():
    """
    new required capacity res elec



    component


    """
    
    ret = functions.if_then_else(
        time() < (2014 - total_time_planxplusxconstr_res_elec()),
        historic_new_required_capacity_res_elec(),
        installed_capacity_res_elec_tw() * adapt_growth_res_elec_after_allocation() *
        remaining_potential_constraint_on_new_res_elec_capacity() * abundance_res_elec2())
    #test_nans(ret, 1277)
    return ret


@cache('run')
def initial_capacity_in_construction_res_elec():
    """
    initial capacity in construction res elec



    component


    """
    
    ret = initial_required_capacity_res_elec()
    #test_nans(ret, 1278)
    return ret


@cache('step')
def remaining_potential_constraint_on_new_res_elec_capacity():
    """
    remaining potential constraint on new res elec capacity



    component


    """
    
    ret = functions.if_then_else(remaining_potential_res_elec_after_intermitt() >
                                  threshold_remaining_potential_new_capacity(), 1,
                                  remaining_potential_res_elec_after_intermitt() *
                                  (1 / threshold_remaining_potential_new_capacity()))
    #test_nans(ret, 1279)
    return ret


@cache('step')
def required_capacity_res_elec_tw():
    """
    required capacity res elec tw



    component


    """
    
    ret = integ_required_capacity_res_elec_tw()
    #test_nans(ret, 1280)
    return ret


@cache('step')
def res_elec_planned_capacity_tw():
    """
    res elec planned capacity tw



    component


    """
    
    ret = integ_res_elec_planned_capacity_tw()
    #test_nans(ret, 1281)
    return ret


@cache('run')
def initial_energy_cons_per_unit_of_material_cons_xrecycledx_xdashx_data():
    """
    initial energy cons per unit of material cons xrecycledx xdashx data



    constant


    """
    
    ret = xr.DataArray(data=[0.0,29.0,0.0,0.0,0.0,0.0,0.0,16.5,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,9.4,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,10.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,258.0,60.0,3710.0,36.0,72.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1282)
    return ret


@cache('run')
def cp_phs():
    """
    cp phs



    constant


    """
    
    ret = 0.1
    #test_nans(ret, 1283)
    return ret


@cache('run')
def max_potential_phs_twe():
    """
    max potential phs twe



    constant


    """
    
    ret = 0.25
    #test_nans(ret, 1284)
    return ret


@cache('run')
def max_capacity_potential_phs():
    """
    max capacity potential phs



    component


    """
    
    ret = max_potential_phs_twe() / cp_phs()
    #test_nans(ret, 1285)
    return ret


@cache('run')
def materials_per_new_res_elec_capacity_installed_xdashx_hvdcs():
    """
    materials per new res elec capacity installed xdashx hvdcs



    constant


    """
    
    ret = xr.DataArray(data=[0.0,100.0,0.0,0.0,0.0,0.0,0.0,125.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,435.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,1160.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,112.0,0.0,15.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,200.0,7.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1286)
    return ret


@cache('run')
def materials_per_new_res_elec_capacity_installed_xdashx_material_overgrid_high_power():
    """
    materials per new res elec capacity installed xdashx material overgrid high power



    constant


    """
    
    ret = xr.DataArray(data=[0.0,7362.0,0.0,0.0,0.0,48.0,0.0,2044.0,6200.0,0.0,0.0,0.0,1140.0,0.0,0.0,562.0,0.0,0.0,0.0,29683.0,0.0,7500.0,0.0,0.0,0.0,0.0,0.0,0.0,40.0,130000.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,11.0,1390.0,970.0,190.0,0.0,0.0,160000.0,0.0,0.0,0.0,0.0,64.0,0.0,2651.0,2544.0,0.0,0.0,0.0,0.0,0.0,200.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1287)
    return ret


@cache('step')
def real_growth_gtl():
    """
    real growth gtl



    component


    """
    
    ret = functions.if_then_else(abundance_gases() >= abundance_liquids(),
                                  functions.if_then_else(abs(abundance_gases() - 1) < equality_threshold,
                                                         exogenous_growth_gtl(), 0),
                                  0) * abundance_liquids_gtl() * scarcity_conv_oil()
    #test_nans(ret, 1288)
    return ret


@cache('step')
def scarcity_conv_gas_delayed_1yr():
    """
    scarcity conv gas delayed 1yr



    component


    """
    
    ret = delay_scarcity_conv_gas_round1__time_step___time_step_02502_1__time_step()
    #test_nans(ret, 1289)
    return ret


@cache('step')
def scarcity_conv_oil_delayed_1yr():
    """
    scarcity conv oil delayed 1yr



    component


    """
    
    ret = delay_scarcity_conv_oil_round1__time_step___time_step_03989_1__time_step()
    #test_nans(ret, 1290)
    return ret


@cache('step')
def pes_natx_gas():
    """
    pes natx gas



    component


    """
    
    ret = real_extraction_conv_gas_ej() + real_extraction_unconv_gas_ej()
    #test_nans(ret, 1291)
    return ret


@cache('step')
def fe_elec_generation_from_unconv_gas_twh():
    """
    fe elec generation from unconv gas twh



    component


    """
    
    ret = real_extraction_unconv_gas_ej() * share_natx_gas_dem_for_elec(
    ) * efficiency_gas_for_electricity() / ej_per_twh()
    #test_nans(ret, 1292)
    return ret


@cache('run')
def exponent_availability_conv_oil():
    """
    exponent availability conv oil



    constant


    """
    
    ret = 1 / 4
    #test_nans(ret, 1293)
    return ret


@cache('step')
def pe_losses_uncon_gas_for_elec_ej():
    """
    pe losses uncon gas for elec ej



    component


    """
    
    ret = real_extraction_unconv_gas_ej() * share_natx_gas_dem_for_elec() * (
        1 - efficiency_gas_for_electricity())
    #test_nans(ret, 1294)
    return ret


@cache('step')
def abundance_total_natx_gas():
    """
    abundance total natx gas



    component


    """
    
    ret = functions.if_then_else(ped_natx_gas_ej() < pes_natx_gas(), 1, 1 - functions.zidz(
        (ped_natx_gas_ej() - pes_natx_gas()), ped_natx_gas_ej()))
    #test_nans(ret, 1295)
    return ret


@cache('run')
def exponent_availability_conv_gas():
    """
    exponent availability conv gas



    constant


    """
    
    ret = 1 / 4
    #test_nans(ret, 1296)
    return ret


@cache('step')
def scarcity_conv_gas():
    """
    scarcity conv gas



    component


    """
    
    ret = functions.if_then_else(
        abs(max_extraction_conv_gas_ej() - 0) < equality_threshold, 0,
        functions.if_then_else(max_extraction_conv_gas_ej() >= extraction_conv_gas_ej(), 1 -
                               ((max_extraction_conv_gas_ej() - extraction_conv_gas_ej()) /
                                max_extraction_conv_gas_ej())**exponent_availability_conv_gas(),
                               0))
    #test_nans(ret, 1297)
    return ret


@cache('step')
def year_scarcity_total_natx_gas():
    """
    year scarcity total natx gas



    component


    """
    
    ret = functions.if_then_else(abundance_total_natx_gas() > 0.95, 0, time())
    #test_nans(ret, 1298)
    return ret


@cache('run')
def share_unconv_gas_vs_tot_agg_in_x2050x():
    """
    share unconv gas vs tot agg in x2050x



    component


    """
    
    ret = functions.if_then_else(abs(choose_extraction_tot_agg_gas_curve() - 1) < equality_threshold,
                                  share_unconv_gas_vs_tot_agg_in_x2050x_laherrere2010(),
                                  functions.if_then_else(
                                      abs(choose_extraction_tot_agg_gas_curve() - 2) < equality_threshold,
                                      share_unconv_gas_vs_tot_agg_in_x2050x_mohr12_bg(),
                                      share_unconv_gas_vs_tot_agg_in_x2050x_user_defined()))
    #test_nans(ret, 1299)
    return ret


@cache('run')
def share_unconv_gas_vs_tot_agg_in_x2050x_laherrere2010():
    """
    share unconv gas vs tot agg in x2050x laherrere2010



    constant


    """
    
    ret = 0.25
    #test_nans(ret, 1300)
    return ret


@cache('run')
def share_unconv_gas_vs_tot_agg_in_x2050x_mohr12_bg():
    """
    share unconv gas vs tot agg in x2050x mohr12 bg



    constant


    """
    
    ret = 0.26666666666666666
    #test_nans(ret, 1301)
    return ret


@cache('run')
def share_unconv_gas_vs_tot_agg_in_x2050x_user_defined():
    """
    share unconv gas vs tot agg in x2050x user defined



    constant


    """
    
    ret = 0.5
    #test_nans(ret, 1302)
    return ret


@cache('run')
def cumulated_tot_agg_gas_extraction_to_x1995x():
    """
    cumulated tot agg gas extraction to x1995x



    component


    """
    
    ret = cumulated_conv_gas_extraction_to_x1995x() + cumulated_unconv_gas_extraction_to_x1995x()
    #test_nans(ret, 1303)
    return ret


@cache('step')
def share_conv_gas_vs_tot_agg():
    """
    share conv gas vs tot agg



    component


    """
    
    ret = 1 - share_unconv_gas_vs_tot_agg()
    #test_nans(ret, 1304)
    return ret


@cache('run')
def share_rurr_tot_agg_gas_to_leave_underground():
    """
    share rurr tot agg gas to leave underground



    constant


    """
    
    ret = 0
    #test_nans(ret, 1305)
    return ret


@cache('step')
def rurr_tot_agg_gas():
    """
    rurr tot agg gas



    component


    """
    
    ret = integ_rurr_tot_agg_gas()
    #test_nans(ret, 1306)
    return ret


@cache('step')
def flow_tot_agg_gas_left_in_ground():
    """
    flow tot agg gas left in ground



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_gas(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_tot_agg_gas() + 1, 0,
                                      tot_agg_gas_to_leave_underground()))
    #test_nans(ret, 1307)
    return ret


@cache('step')
def tot_rurr_tot_agg_gas():
    """
    tot rurr tot agg gas



    component


    """
    
    ret = rurr_tot_agg_gas() + total_agg_gas_left_in_ground()
    #test_nans(ret, 1308)
    return ret


@cache('run')
def urr_total_agg_gas_unlimited():
    """
    urr total agg gas unlimited



    component


    """
    
    ret = urr_total_gas_mohr12_bg() * 10000
    #test_nans(ret, 1309)
    return ret


@cache('run')
def start_policy_leave_in_ground_tot_agg_gas():
    """
    start policy leave in ground tot agg gas



    constant


    """
    
    ret = 2015
    #test_nans(ret, 1310)
    return ret


@cache('step')
def extraction_unconv_gas_xdashx_tot_agg():
    """
    extraction unconv gas xdashx tot agg



    component


    """
    
    ret = extraction_tot_agg_gas_ej() * share_unconv_gas_vs_tot_agg()
    #test_nans(ret, 1311)
    return ret


@cache('step')
def total_agg_gas_left_in_ground():
    """
    total agg gas left in ground



    component


    """
    
    ret = integ_total_agg_gas_left_in_ground()
    #test_nans(ret, 1312)
    return ret


@cache('step')
def evolution_share_unconv_gas_vs_tot_agg():
    """
    evolution share unconv gas vs tot agg



    component


    """
    
    ret = (share_unconv_gas_vs_tot_agg_in_x2050x() - 0.1268) / (2050 - 2012) * time() + (
        share_unconv_gas_vs_tot_agg_in_x2050x() -
        ((share_unconv_gas_vs_tot_agg_in_x2050x() - 0.1268) / (2050 - 2012)) * 2050)
    #test_nans(ret, 1313)
    return ret


@cache('step')
def real_extraction_conv_gas_ej():
    """
    real extraction conv gas ej



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
                                  extraction_conv_gas_ej(), extraction_conv_gas_xdashx_tot_agg())
    #test_nans(ret, 1314)
    return ret


@cache('run')
def urr_tot_agg_gas():
    """
    urr tot agg gas



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold, 0,
                                  functions.if_then_else(
                                      abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                      urr_total_agg_gas_unlimited(),
                                      functions.if_then_else(
                                          abs(unlimited_gasxquestx() - 1) < equality_threshold,
                                          urr_total_agg_gas_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_tot_agg_gas_curve() - 1) < equality_threshold,
                                              urr_total_gas_laherrere10(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_tot_agg_gas_curve() - 2) < equality_threshold,
                                                  urr_total_gas_mohr12_bg(),
                                                  urr_total_gas_user_defined())))))
    #test_nans(ret, 1315)
    return ret


@cache('step')
def cumulated_tot_agg_gas_extraction():
    """
    cumulated tot agg gas extraction



    component


    """
    
    ret = integ_cumulated_tot_agg_gas_extraction()
    #test_nans(ret, 1316)
    return ret


@cache('step')
def extraction_conv_gas_xdashx_tot_agg():
    """
    extraction conv gas xdashx tot agg



    component


    """
    
    ret = extraction_tot_agg_gas_ej() * share_conv_gas_vs_tot_agg()
    #test_nans(ret, 1317)
    return ret


@cache('step')
def real_extraction_unconv_gas_ej():
    """
    real extraction unconv gas ej



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
                                  extraction_unconv_gas_ej(),
                                  extraction_unconv_gas_xdashx_tot_agg())
    #test_nans(ret, 1318)
    return ret


@cache('step')
def extraction_tot_agg_gas_ej():
    """
    extraction tot agg gas ej



    component


    """
    
    ret = functions.if_then_else(
        rurr_tot_agg_gas() < 0, 0,
        functions.if_then_else(abs(unlimited_nrexquestx() - 1) < equality_threshold,
                               ped_natx_gas_ej(),
                               functions.if_then_else(abs(unlimited_gasxquestx() - 1) < equality_threshold,
                                                      ped_natx_gas_ej(),
                                                      np.minimum(
                                                          ped_natx_gas_ej(),
                                                          max_extraction_tot_agg_gas_ej()))))
    #test_nans(ret, 1319)
    return ret


@cache('step')
def real_extraction_conv_oil_mbxdivxd():
    """
    real extraction conv oil mbxdivxd



    component


    """
    
    ret = real_extraction_conv_oil_ej() * mbxdivxd_per_ejxdivxyear()
    #test_nans(ret, 1320)
    return ret


@cache('step')
def evolution_share_unconv_oil_vs_tot_agg():
    """
    evolution share unconv oil vs tot agg



    component


    """
    
    ret = (share_unconv_oil_vs_tot_agg_in_x2050x() - 0.059) / (2050 - 2012) * time() + (
        share_unconv_oil_vs_tot_agg_in_x2050x() -
        ((share_unconv_oil_vs_tot_agg_in_x2050x() - 0.059) / (2050 - 2012)) * 2050)
    #test_nans(ret, 1321)
    return ret


@cache('run')
def share_unconv_oil_vs_tot_agg_in_x2050x():
    """
    share unconv oil vs tot agg in x2050x



    component


    """
    
    ret = functions.if_then_else(abs(choose_extraction_curve_tot_agg_oil() - 1) < equality_threshold,
                                  share_unconv_oil_vs_tot_agg_in_x2050x_laherrere2006(),
                                  share_unconv_oil_vs_tot_agg_in_x2050x_user_defined())
    #test_nans(ret, 1322)
    return ret


@cache('run')
def share_unconv_oil_vs_tot_agg_in_x2050x_laherrere2006():
    """
    share unconv oil vs tot agg in x2050x laherrere2006



    constant


    """
    
    ret = 0.5
    #test_nans(ret, 1323)
    return ret


@cache('run')
def share_unconv_oil_vs_tot_agg_in_x2050x_user_defined():
    """
    share unconv oil vs tot agg in x2050x user defined



    constant


    """
    
    ret = 0.5
    #test_nans(ret, 1324)
    return ret


@cache('step')
def extraction_conv_oil_xdashx_tot_agg():
    """
    extraction conv oil xdashx tot agg



    component


    """
    
    ret = extraction_tot_agg_oil_ej() * share_conv_oil_vs_tot_agg()
    #test_nans(ret, 1325)
    return ret


@cache('step')
def extraction_unconv_oil_xdashx_tot_agg():
    """
    extraction unconv oil xdashx tot agg



    component


    """
    
    ret = extraction_tot_agg_oil_ej() * share_unconv_oil_vs_tot_agg()
    #test_nans(ret, 1326)
    return ret


@cache('step')
def share_conv_oil_vs_tot_agg():
    """
    share conv oil vs tot agg



    component


    """
    
    ret = 1 - share_unconv_oil_vs_tot_agg()
    #test_nans(ret, 1327)
    return ret


@cache('step')
def rurr_tot_agg_oil():
    """
    rurr tot agg oil



    component


    """
    
    ret = integ_rurr_tot_agg_oil()
    #test_nans(ret, 1328)
    return ret


@cache('step')
def pes_oil_ej():
    """
    pes oil ej



    component


    """
    
    ret = real_extraction_conv_oil_ej() + real_extraction_unconv_oil_ej()
    #test_nans(ret, 1329)
    return ret


@cache('step')
def extraction_tot_agg_oil_ej():
    """
    extraction tot agg oil ej



    component


    """
    
    ret = functions.if_then_else(
        rurr_tot_agg_oil() < 0, 0,
        functions.if_then_else(abs(unlimited_nrexquestx() - 1) < equality_threshold,
                               ped_total_oil_ej(),
                               functions.if_then_else(abs(unlimited_oilxquestx() - 1) < equality_threshold,
                                                      ped_total_oil_ej(),
                                                      np.minimum(
                                                          ped_total_oil_ej(),
                                                          max_extraction_tot_agg_oil_ej()))))
    #test_nans(ret, 1330)
    return ret


@cache('step')
def real_extraction_conv_oil_ej():
    """
    real extraction conv oil ej



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_oilxquestx() - 1) < equality_threshold,
                                  extraction_conv_oil_ej(), extraction_conv_oil_xdashx_tot_agg())
    #test_nans(ret, 1331)
    return ret


@cache('run')
def gch4_per_mj_conv_gas():
    """
    gch4 per mj conv gas



    constant


    """
    
    ret = 0.78
    #test_nans(ret, 1332)
    return ret


@cache('step')
def total_agg_oil_left_in_ground():
    """
    total agg oil left in ground



    component


    """
    
    ret = integ_total_agg_oil_left_in_ground()
    #test_nans(ret, 1333)
    return ret


@cache('step')
def cumulated_tot_agg_oil_extraction():
    """
    cumulated tot agg oil extraction



    component


    """
    
    ret = integ_cumulated_tot_agg_oil_extraction()
    #test_nans(ret, 1334)
    return ret


@cache('run')
def cumulated_tot_agg_extraction_to_x1995x():
    """
    cumulated tot agg extraction to x1995x



    component


    """
    
    ret = cumulated_conv_oil_extraction_to_x1995x() + cumulated_unconv_oil_extraction_to_x1995x()
    #test_nans(ret, 1335)
    return ret


@cache('run')
def urr_tot_agg_oil():
    """
    urr tot agg oil



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_oilxquestx() - 1) < equality_threshold, 0,
                                  functions.if_then_else(
                                      abs(unlimited_oilxquestx() - 1) < equality_threshold,
                                      urr_tot_agg_oil_unlimited(),
                                      functions.if_then_else(
                                          abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                          urr_tot_agg_oil_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_tot_agg_oil() - 1) < equality_threshold,
                                              urr_tot_agg_oil_laherrere_x2006x(),
                                              urr_tot_agg_oil_user_defined()))))
    #test_nans(ret, 1336)
    return ret


@cache('run')
def urr_tot_agg_oil_unlimited():
    """
    urr tot agg oil unlimited



    component


    """
    
    ret = urr_tot_agg_oil_laherrere_x2006x() * 10000
    #test_nans(ret, 1337)
    return ret


@cache('step')
def tot_rurr_tot_agg_oil():
    """
    tot rurr tot agg oil



    component


    """
    
    ret = rurr_tot_agg_oil() + total_agg_oil_left_in_ground()
    #test_nans(ret, 1338)
    return ret


@cache('run')
def start_policy_leave_in_ground_tot_agg_oil():
    """
    start policy leave in ground tot agg oil



    constant


    """
    
    ret = 2015
    #test_nans(ret, 1339)
    return ret


@cache('step')
def flow_tot_agg_oil_left_in_ground():
    """
    flow tot agg oil left in ground



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_tot_agg_oil(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_tot_agg_oil() + 1, 0,
                                      tot_agg_oil_to_leave_underground()))
    #test_nans(ret, 1340)
    return ret


@cache('run')
def g_per_mt():
    """
    g per mt



    constant


    """
    
    ret = 1000000000000.0
    #test_nans(ret, 1341)
    return ret


@cache('run')
def share_rurr_tot_agg_oil_to_leave_underground():
    """
    share rurr tot agg oil to leave underground



    constant


    """
    
    ret = 0
    #test_nans(ret, 1342)
    return ret


@cache('run')
def gch4_per_mj_unconv_gas():
    """
    gch4 per mj unconv gas



    constant


    """
    
    ret = 2.48
    #test_nans(ret, 1343)
    return ret


@cache('step')
def tot_rurr_unconv_gas():
    """
    tot rurr unconv gas



    component


    """
    
    ret = rurr_unconv_gas() + total_unconv_gas_left_in_ground()
    #test_nans(ret, 1344)
    return ret


@cache('run')
def start_policy_leave_in_ground_coal():
    """
    start policy leave in ground coal



    constant


    """
    
    ret = 2015
    #test_nans(ret, 1345)
    return ret


@cache('step')
def total_unconv_gas_left_in_ground():
    """
    total unconv gas left in ground



    component


    """
    
    ret = integ_total_unconv_gas_left_in_ground()
    #test_nans(ret, 1346)
    return ret


@cache('step')
def flow_coal_left_in_ground():
    """
    flow coal left in ground



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_coal(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_coal() + 1, 0,
                                      coal_to_leave_underground()))
    #test_nans(ret, 1347)
    return ret


@cache('step')
def flow_conv_gas_left_in_ground():
    """
    flow conv gas left in ground



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_conv_gas(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_conv_gas() + 1, 0,
                                      conv_gas_to_leave_underground()))
    #test_nans(ret, 1348)
    return ret


@cache('step')
def flow_conv_oil_left_in_ground():
    """
    flow conv oil left in ground



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_conv_oil(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_conv_oil() + 1, 0,
                                      conv_oil_to_leave_underground()))
    #test_nans(ret, 1349)
    return ret


@cache('step')
def flow_unconv_oil_left_in_ground():
    """
    flow unconv oil left in ground



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_unconv_oil(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_unconv_oil() + 1, 0,
                                      unconv_oil_to_leave_underground()))
    #test_nans(ret, 1350)
    return ret


@cache('run')
def share_rurr_unconv_gas_to_leave_underground():
    """
    share rurr unconv gas to leave underground



    constant


    """
    
    ret = 0
    #test_nans(ret, 1351)
    return ret


@cache('run')
def share_rurr_unconv_oil_to_leave_underground():
    """
    share rurr unconv oil to leave underground



    constant


    """
    
    ret = 0
    #test_nans(ret, 1352)
    return ret


@cache('step')
def tot_rurr_coal():
    """
    tot rurr coal



    component


    """
    
    ret = rurr_coal() + total_coal_left_in_ground()
    #test_nans(ret, 1353)
    return ret


@cache('step')
def tot_rurr_conv_gas():
    """
    tot rurr conv gas



    component


    """
    
    ret = rurr_conv_gas() + total_conv_gas_left_in_ground()
    #test_nans(ret, 1354)
    return ret


@cache('step')
def tot_rurr_conv_oil():
    """
    tot rurr conv oil



    component


    """
    
    ret = rurr_conv_oil() + total_conv_oil_left_in_ground()
    #test_nans(ret, 1355)
    return ret


@cache('run')
def share_rurr_conv_oil_to_leave_underground():
    """
    share rurr conv oil to leave underground



    constant


    """
    
    ret = 0
    #test_nans(ret, 1356)
    return ret


@cache('step')
def tot_rurr_unconv_oil():
    """
    tot rurr unconv oil



    component


    """
    
    ret = rurr_unconv_oil_ej() + total_unconv_oil_left_in_ground()
    #test_nans(ret, 1357)
    return ret


@cache('step')
def total_unconv_oil_left_in_ground():
    """
    total unconv oil left in ground



    component


    """
    
    ret = integ_total_unconv_oil_left_in_ground()
    #test_nans(ret, 1358)
    return ret


@cache('step')
def total_coal_left_in_ground():
    """
    total coal left in ground



    component


    """
    
    ret = integ_total_coal_left_in_ground()
    #test_nans(ret, 1359)
    return ret


@cache('step')
def total_conv_gas_left_in_ground():
    """
    total conv gas left in ground



    component


    """
    
    ret = integ_total_conv_gas_left_in_ground()
    #test_nans(ret, 1360)
    return ret


@cache('step')
def total_conv_oil_left_in_ground():
    """
    total conv oil left in ground



    component


    """
    
    ret = integ_total_conv_oil_left_in_ground()
    #test_nans(ret, 1361)
    return ret


@cache('step')
def extraction_conv_oil_ej():
    """
    extraction conv oil ej



    component


    """
    
    ret = functions.if_then_else(rurr_conv_oil() < 0, 0,
                                  functions.if_then_else(abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                                         demand_conv_oil_ej(),
                                                         functions.if_then_else(
                                                             abs(unlimited_oilxquestx() - 1) < equality_threshold,
                                                             demand_conv_oil_ej(),
                                                             np.minimum(
                                                                 demand_conv_oil_ej(),
                                                                 max_extraction_conv_oil_ej()))))
    #test_nans(ret, 1362)
    return ret


@cache('step')
def extraction_coal_ej():
    """
    extraction coal ej



    component


    """
    
    ret = functions.if_then_else(rurr_coal() < 0, 0,
                                  functions.if_then_else(abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                                         ped_coal_ej(),
                                                         functions.if_then_else(
                                                             abs(unlimited_coalxquestx() - 1) < equality_threshold,
                                                             ped_coal_ej(),
                                                             np.minimum(
                                                                 ped_coal_ej(),
                                                                 max_extraction_coal_ej()))))
    #test_nans(ret, 1363)
    return ret


@cache('step')
def rurr_coal():
    """
    rurr coal



    component


    """
    
    ret = integ_rurr_coal()
    #test_nans(ret, 1364)
    return ret


@cache('run')
def start_policy_leave_in_ground_unconv_gas():
    """
    start policy leave in ground unconv gas



    constant


    """
    
    ret = 2015
    #test_nans(ret, 1365)
    return ret


@cache('step')
def flow_unconv_gas_left_in_ground():
    """
    flow unconv gas left in ground



    component


    """
    
    ret = functions.if_then_else(time() < start_policy_leave_in_ground_unconv_gas(), 0,
                                  functions.if_then_else(
                                      time() >= start_policy_leave_in_ground_unconv_gas() + 1, 0,
                                      unconv_gas_to_leave_underground()))
    #test_nans(ret, 1366)
    return ret


@cache('run')
def start_policy_leave_in_ground_unconv_oil():
    """
    start policy leave in ground unconv oil



    constant


    """
    
    ret = 2015
    #test_nans(ret, 1367)
    return ret


@cache('run')
def start_policy_leave_in_ground_conv_oil():
    """
    start policy leave in ground conv oil



    constant


    """
    
    ret = 2015
    #test_nans(ret, 1368)
    return ret


@cache('run')
def share_rurr_coal_to_leave_underground():
    """
    share rurr coal to leave underground



    constant


    """
    
    ret = 0
    #test_nans(ret, 1369)
    return ret


@cache('step')
def pes_oil_mbxdivxd():
    """
    pes oil mbxdivxd



    component


    """
    
    ret = pes_oil_ej() * mbxdivxd_per_ejxdivxyear()
    #test_nans(ret, 1370)
    return ret


@cache('run')
def current_mineral_reserves_mt():
    """
    current mineral reserves mt



    constant


    """
    
    ret = xr.DataArray(data=[0.0,28000.0,0.0,0.5,0.0,0.0,480.0,720.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0052,0.0,0.0,0.0,0.011,160000.0,0.0,0.0,0.0,0.0,13.5,0.0,2400.0,570.0,0.0,0.0,11.0,0.0,0.0,0.0,81.0,0.0,0.0,0.0,87.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.53,0.0,4.8,0.0,0.0,0.0,0.01108,794.0,0.0,15.0,0.0,230.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1371)
    return ret


@cache('run')
def share_rurr_conv_gas_to_leave_underground():
    """
    share rurr conv gas to leave underground



    constant


    """
    
    ret = 0
    #test_nans(ret, 1372)
    return ret


@cache('run')
def start_policy_leave_in_ground_conv_gas():
    """
    start policy leave in ground conv gas



    constant


    """
    
    ret = 2015
    #test_nans(ret, 1373)
    return ret


@cache('step')
def extraction_conv_gas_ej():
    """
    extraction conv gas ej



    component


    """
    
    ret = functions.if_then_else(rurr_conv_gas() < 0, 0,
                                  functions.if_then_else(abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                                         demand_conv_gas(),
                                                         functions.if_then_else(
                                                             abs(unlimited_gasxquestx() - 1) < equality_threshold,
                                                             demand_conv_gas(),
                                                             np.minimum(
                                                                 demand_conv_gas(),
                                                                 max_extraction_conv_gas_ej()))))
    #test_nans(ret, 1374)
    return ret


@cache('step')
def check_gases():
    """
    check gases



    component


    """
    
    ret = functions.zidz(ped_gases() - pes_gases(), pes_gases())
    #test_nans(ret, 1375)
    return ret


@cache('step')
def abundance_gases():
    """
    abundance gases



    component


    """
    
    ret = functions.if_then_else(ped_gases() < pes_gases(), 1,
                                  1 - functions.zidz(ped_gases() - pes_gases(), ped_gases()))
    #test_nans(ret, 1376)
    return ret


@cache('step')
def demand_conv_gas():
    """
    demand conv gas



    component


    """
    
    ret = np.maximum(ped_natx_gas_ej() - extraction_unconv_gas_ej(), 0)
    #test_nans(ret, 1377)
    return ret


@cache('step')
def ped_natx_gas_without_gtl():
    """
    ped natx gas without gtl



    component


    """
    
    ret = np.maximum(0, ped_natx_gas_ej() - ped_natx_gas_for_gtl_ej())
    #test_nans(ret, 1378)
    return ret


@cache('step')
def abundance_solids():
    """
    abundance solids



    component


    """
    
    ret = functions.if_then_else(pes_solids() > ped_solids(), 1,
                                  1 - functions.zidz(ped_solids() - pes_solids(), ped_solids()))
    #test_nans(ret, 1379)
    return ret


@cache('step')
def gdp():
    """
    gdp



    component


    """
    ret = real_demand() / 1e+006
    #test_nans(ret, 1380)
    return ret


@cache('step')
def demand_elec_gas_and_coal_twh():
    """
    demand elec gas and coal twh



    component


    """
    
    ret = demand_elec_plants_fossil_fuels_twh() - fe_demand_oil_elec_plants_twh()
    #test_nans(ret, 1381)
    return ret


@cache('step')
def fe_demand_coal_elec_plants_twh():
    """
    fe demand coal elec plants twh



    component


    """
    
    ret = share_coal_for_elec() * demand_elec_gas_and_coal_twh()
    #test_nans(ret, 1382)
    return ret


@cache('step')
def fe_demand_gas_elec_plants_twh():
    """
    fe demand gas elec plants twh



    component


    """
    
    ret = share_gasxdivxxcoal_xplusxgasx_for_elec() * demand_elec_gas_and_coal_twh()
    #test_nans(ret, 1383)
    return ret


@cache('step')
def fed_heat_gasxplusxcoal_ej():
    """
    fed heat gasxplusxcoal ej



    component


    """
    
    ret = fed_heatxdashxcom_plants_fossil_fuels_ej() - fed_heat_liquids_plants_ej()
    #test_nans(ret, 1384)
    return ret


@cache('step')
def pe_demand_oil_elec_plants_ej():
    """
    pe demand oil elec plants ej



    component


    """
    
    ret = (fe_demand_oil_elec_plants_twh() / efficiency_liquids_for_electricity()) * ej_per_twh()
    #test_nans(ret, 1385)
    return ret


@cache('step')
def fe_demand_oil_elec_plants_twh():
    """
    fe demand oil elec plants twh



    component


    """
    
    ret = share_oil_for_elec() * total_fe_elec_demand_twh()
    #test_nans(ret, 1386)
    return ret


@cache('step')
def share_coal_for_elec():
    """
    share coal for elec



    component


    """
    
    ret = 1 - share_gasxdivxxcoal_xplusxgasx_for_elec()
    #test_nans(ret, 1387)
    return ret


@cache('run')
def current_mineral_resources_mt():
    """
    current mineral resources mt



    constant


    """
    
    ret = xr.DataArray(data=[0.0,75000.0,0.0,6.0,0.0,0.0,12000.0,2100.0,0.0,0.0,0.0,0.0,0.0,0.0,1.0,0.0,0.0,0.0,0.0471,800000.0,0.0,0.0,0.0,0.0,39.5,0.0,12000.0,1030.0,0.0,0.0,14.0,0.0,0.0,0.0,130.0,0.0,0.0,0.0,2000.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,1.308,0.0,76.2,0.0,0.0,0.0,0.025,2000.0,0.0,63.0,0.0,1900.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1388)
    return ret


@cache('step')
def output_elec_over_lifetime_res_elec_for_allocation2():
    """
    output elec over lifetime res elec for allocation2



    component


    """
    
    ret = xstaticx_eroigrid_res_elec() * fei_over_lifetime_res_elec_for_allocation(
    ) * quality_of_electricity()
    #test_nans(ret, 1389)
    return ret


@cache('step')
def demand_storage_capacity():
    """
    demand storage capacity



    component


    """
    
    ret = share_capacity_storagexdivxres_elec_var() * total_installed_capacity_res_elec_var()
    #test_nans(ret, 1390)
    return ret


@cache('step')
def share_capacity_storagexdivxres_elec_var():
    """
    share capacity storagexdivxres elec var



    component


    """
    
    ret = 0.099 + 0.1132 * share_elec_demand_covered_by_res()
    #test_nans(ret, 1391)
    return ret


@cache('run')
def staticxdivxdynamic_quality_of_electricityxquestx():
    """
    staticxdivxdynamic quality of electricityxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1392)
    return ret


@cache('step')
def xstaticx_eroigrid_totxdashxeffective_for_allocation_res_elec():
    """
    xstaticx eroigrid totxdashxeffective for allocation res elec



    component


    """
    
    ret = functions.zidz(
        functions.sum(output_elec_over_lifetime_res_elec_for_allocation2()),
        functions.sum(fei_over_lifetime_res_elec_for_allocation()))
    #test_nans(ret, 1393)
    return ret


@cache('step')
def ratio_eroigrid_vs_eroi_xstaticx():
    """
    ratio eroigrid vs eroi xstaticx



    component


    """
    
    ret = functions.if_then_else(xstaticx_eroi_res_elec() <= 0, 0,
                                  xstaticx_eroigrid_res_elec() / xstaticx_eroi_res_elec())
    #test_nans(ret, 1394)
    return ret


@cache('step')
def ratio_eroi_per_techn_vs_eroitot_xstaticx():
    """
    ratio eroi per techn vs eroitot xstaticx



    component


    """
    
    ret = functions.xidz(xstaticx_eroigrid_res_elec(),
                          xstaticx_eroigrid_totxdashxeffective_for_allocation_res_elec(), 0)
    #test_nans(ret, 1395)
    return ret


@cache('step')
def total_installed_capacity_res_elec_var():
    """
    total installed capacity res elec var



    component


    """
    
    ret = installed_capacity_res_elec_tw().loc['wind onshore'].values + installed_capacity_res_elec_tw().loc['wind offshore'].values + installed_capacity_res_elec_tw().loc['solar pv'].values + installed_capacity_res_elec_tw().loc['csp'].values
    #test_nans(ret, 1396)
    return ret


@cache('run')
def share_res_elec_generation_curtailedxandxstored():
    """
    share res elec generation curtailedxandxstored



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.2,0.2,0.2,0.2],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1397)
    return ret


@cache('run')
def rt_storage_efficiency_phs():
    """
    rt storage efficiency phs



    constant


    """
    
    ret = 0.75
    #test_nans(ret, 1398)
    return ret


@cache('run')
def ratio_xequalsx_x1():
    """
    ratio xequalsx x1



    constant


    """
    
    ret = 1
    #test_nans(ret, 1399)
    return ret


@cache('step')
def fei_over_lifetime_res_elec():
    """
    fei over lifetime res elec



    component


    """
    
    ret = xr.DataArray(data=[
        fei_over_lifetime_res_elec_dispatch().loc['hydro'].values,
        fei_over_lifetime_res_elec_dispatch().loc['geotxdashxelec'].values,
        fei_over_lifetime_res_elec_dispatch().loc['solid bioexdashxelec'].values,
        fei_over_lifetime_res_elec_dispatch().loc['oceanic'].values,
        fei_over_lifetime_res_elec_var().loc['wind onshore'].values,
        fei_over_lifetime_res_elec_var().loc['wind offshore'].values,
        fei_over_lifetime_res_elec_var().loc['solar pv'].values,
        fei_over_lifetime_res_elec_var().loc['csp'].values],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1400)
    return ret


@cache('step')
def real_cp_res_elec():
    """
    real cp res elec



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        cp_res_elec(),
        functions.if_then_else(
            abs(installed_capacity_res_elec_tw() - 0) < equality_threshold, 0,
            real_generation_res_elec_twh() * twe_per_twh() / installed_capacity_res_elec_tw()))
    #test_nans(ret, 1401)
    return ret


@cache('step')
def fei_over_lifetime_res_elec_for_allocation():
    """
    fei over lifetime res elec for allocation



    component


    """
    
    ret = fei_over_lifetime_res_elec() * remaining_potential_res_elec_switch()
    #test_nans(ret, 1402)
    return ret


@cache('step')
def remaining_potential_res_elec_after_intermitt():
    """
    remaining potential res elec after intermitt



    component


    """
    
    ret = functions.if_then_else(
        potential_res_elec_after_intermitt_twh() > potential_generation_res_elec_twh(),
        functions.zidz(
            potential_res_elec_after_intermitt_twh() - potential_generation_res_elec_twh(),
            potential_res_elec_after_intermitt_twh()), 0)
    #test_nans(ret, 1403)
    return ret


@cache('step')
def remaining_potential_res_elec_switch():
    """
    remaining potential res elec switch



    component


    """
    
    ret = functions.if_then_else(remaining_potential_res_elec_after_intermitt() < 0.025, 0, 1)
    #test_nans(ret, 1404)
    return ret


@cache('step')
def output_elec_over_lifetime_res_elec_for_allocation():
    """
    output elec over lifetime res elec for allocation



    component


    """
    
    ret = output_elec_over_lifetime_res_elec() * remaining_potential_res_elec_switch()
    #test_nans(ret, 1405)
    return ret


@cache('step')
def potential_res_elec_after_intermitt_twh():
    """
    potential res elec after intermitt twh



    component


    """
    
    ret = max_potential_res_elec_twe() * cp_baseload_reduction() / twe_per_twh()
    #test_nans(ret, 1406)
    return ret


@cache('step')
def replacement_rate_res_elec():
    """
    replacement rate res elec



    component


    """
    
    ret = functions.if_then_else(
        potential_generation_res_elec_twh() < potential_res_elec_after_intermitt_twh(), 1, 0.9)
    #test_nans(ret, 1407)
    return ret


@cache('step')
def xstaticx_eroitot_res_elec():
    """
    xstaticx eroitot res elec



    component


    """
    
    ret = functions.if_then_else(
        functions.sum(fei_over_lifetime_res_elec()) < 0, 0,
        functions.sum(output_elec_over_lifetime_res_elec()) /
        (functions.sum(fei_over_lifetime_res_elec())))
    #test_nans(ret, 1408)
    return ret


@cache('step')
def xstaticx_eroitotxdashxeffective_for_allocation_res_elec():
    """
    xstaticx eroitotxdashxeffective for allocation res elec



    component


    """
    
    ret = functions.zidz(
        functions.sum(output_elec_over_lifetime_res_elec_for_allocation()),
        functions.sum(fei_over_lifetime_res_elec_for_allocation()))
    #test_nans(ret, 1409)
    return ret


@cache('step')
def new_capacity_installed_onshore_wind_tw():
    """
    new capacity installed onshore wind tw



    component


    """
    
    ret = new_res_elec_capacity_under_planning().loc['wind onshore'].values
    #test_nans(ret, 1410)
    return ret


@cache('step')
def ced_oxandxm_over_lifetime_per_water_res_elec_var():
    """
    ced oxandxm over lifetime per water res elec var



    component


    """
    
    ret = res_elec_capacity_under_construction_tw() * water_for_oxandxm_xdashx_res_elec(
    ) * energy_requirements_per_unit_of_water_consumption() * lifetime_res_elec() * (
        m_per_t() / kg_per_mt()) * (kg_per_mt() / mj_per_ej())
    ret = utils.align_by_dims(ret, ('res elec', 'water0'))
    #test_nans(ret, 1411)
    return ret


@cache('run')
def selfxdashxelectricity_consumption_res_elec():
    """
    selfxdashxelectricity consumption res elec



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.02,0.02,0.005,0.0],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1412)
    return ret


@cache('run')
def share_energy_requirements_for_decom_res_elec():
    """
    share energy requirements for decom res elec



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.1,0.1,0.1,0.1],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1413)
    return ret


@cache('step')
def ced_oxandxm_over_lifetime_res_elec_var():
    """
    ced oxandxm over lifetime res elec var



    component


    """
    
    ret = (functions.sum(dim='materials', data=ced_oxandxm_over_lifetime_per_material_res_elec_var()) +
            functions.sum(dim='water0', data=ced_oxandxm_over_lifetime_per_water_res_elec_var()))
    #test_nans(ret, 1414)
    return ret


@cache('run')
def grid_correction_factor_res_elec():
    """
    grid correction factor res elec



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.06,0.06,0.06,0.075],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1415)
    return ret


@cache('run')
def materials_for_oxandxm_per_capacity_installed_xdashx_csp():
    """
    materials for oxandxm per capacity installed xdashx csp



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.78,0.0,0.0,0.0,0.0,0.0,0.0,3450.0,0.0,0.0,0.0,0.0,0.0,0.0,140.0,0.0,0.0,0.0,0.0,0.0,0.0,11.0,0.0,0.0,0.0,3.2,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,98.0,0.0,0.014,0.0,0.0,0.0,0.0,2000.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1416)
    return ret


@cache('step')
def total_water_for_oxandxm_required_by_res_elec_per_techn():
    """
    total water for oxandxm required by res elec per techn



    component


    """
    
    ret = functions.sum(dim='water0', data=water_for_oxandxm_required_for_res_elec())
    #test_nans(ret, 1417)
    return ret


@cache('step')
def water_for_oxandxm_required_for_res_elec():
    """
    water for oxandxm required for res elec



    component


    """
    
    ret = xr.DataArray(data=[
        installed_capacity_res_elec_tw().loc['hydro'].values * water_for_oxandxm_xdashx_res_elec().loc['hydro',:].drop('res elec') * m_per_t() /
        kg_per_mt(),
        installed_capacity_res_elec_tw().loc['geotxdashxelec'].values * water_for_oxandxm_xdashx_res_elec().loc['geotxdashxelec',:].drop('res elec') * m_per_t() /
        kg_per_mt(),
        installed_capacity_res_elec_tw().loc['solid bioexdashxelec'].values * water_for_oxandxm_xdashx_res_elec().loc['solid bioexdashxelec',:].drop('res elec') * m_per_t() /
        kg_per_mt(),
        installed_capacity_res_elec_tw().loc['oceanic'].values * water_for_oxandxm_xdashx_res_elec().loc['oceanic',:].drop('res elec') * m_per_t() /
        kg_per_mt(),
        installed_capacity_res_elec_tw().loc['wind onshore'].values * water_for_oxandxm_xdashx_res_elec().loc['wind onshore',:].drop('res elec') * m_per_t() /
        kg_per_mt(),
        installed_capacity_res_elec_tw().loc['wind offshore'].values * water_for_oxandxm_xdashx_res_elec().loc['wind offshore',:].drop('res elec') * m_per_t() /
        kg_per_mt(),
        installed_capacity_res_elec_tw().loc['solar pv'].values * water_for_oxandxm_xdashx_res_elec().loc['solar pv',:].drop('res elec') * m_per_t() /
        kg_per_mt(),
        installed_capacity_res_elec_tw().loc['csp'].values * water_for_oxandxm_xdashx_res_elec().loc['csp',:].drop('res elec') * m_per_t() /
        kg_per_mt()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp'], 'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']},
        dims=['res elec', 'water0'])
    ret = utils.align_by_dims(ret, ('res elec', 'water0'))
    #test_nans(ret, 1418)
    return ret


@cache('run')
def materials_for_oxandxm_per_capacity_installed_res_elec():
    """
    materials for oxandxm per capacity installed res elec



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        materials_for_oxandxm_per_capacity_installed_xdashx_wind_onshore(),
        materials_for_oxandxm_per_capacity_installed_xdashx_wind_offshore(),
        materials_for_oxandxm_per_capacity_installed_xdashx_pv(),
        materials_for_oxandxm_per_capacity_installed_xdashx_csp()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp'], 'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['res elec', 'materials'])
    ret = utils.align_by_dims(ret, ('res elec', 'materials'))
    #test_nans(ret, 1419)
    return ret


@cache('step')
def total_energy_requirements_oxandxm_for_water_consumption_res_elec():
    """
    total energy requirements oxandxm for water consumption res elec



    component


    """
    
    ret = functions.sum(dim='water0', data=energy_requirements_for_oxandxm_for_water_consumption_res_elec())
    #test_nans(ret, 1420)
    return ret


@cache('step')
def energy_requirements_for_oxandxm_for_water_consumption_res_elec():
    """
    energy requirements for oxandxm for water consumption res elec



    component


    """
    
    ret = energy_requirements_per_unit_of_water_consumption(
    ) * water_for_oxandxm_required_for_res_elec() * kg_per_mt() / mj_per_ej()
    ret = utils.align_by_dims(ret, ('res elec', 'water0'))
    #test_nans(ret, 1421)
    return ret


@cache('run')
def water_for_oxandxm_xdashx_res_elec():
    """
    water for oxandxm xdashx res elec



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0],[0.0,0.0],[0.0,0.0],[0.0,0.0],
        water_for_oxandxm_xdashx_wind_onshore(),
        water_for_oxandxm_xdashx_wind_offshore(),
        water_for_oxandxm_xdashx_pv(),
        water_for_oxandxm_xdashx_csp()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp'], 'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']},
        dims=['res elec', 'water0'])
    ret = utils.align_by_dims(ret, ('res elec', 'water0'))
    #test_nans(ret, 1422)
    return ret


@cache('run')
def energy_requirements_per_unit_of_water_consumption():
    """
    energy requirements per unit of water consumption



    constant


    """
    
    ret = xr.DataArray(data=[0.0025,0.025],
        coords={'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']},
        dims=['water0'])
    #test_nans(ret, 1423)
    return ret


@cache('run')
def unlimited_nrexquestx():
    """
    unlimited nrexquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1424)
    return ret


@cache('step')
def extraction_uranium_ej():
    """
    extraction uranium ej



    component


    """
    
    ret = functions.if_then_else(abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                  pe_demand_uranium_ej(),
                                  functions.if_then_else(abs(unlimited_uraniumxquestx() - 1) < equality_threshold,
                                                         pe_demand_uranium_ej(),
                                                         np.minimum(pe_demand_uranium_ej(),
                                                                    max_extraction_uranium_ej())))
    #test_nans(ret, 1425)
    return ret


@cache('run')
def urr_uranium():
    """
    urr uranium



    component


    """
    
    ret = functions.if_then_else(
        abs(unlimited_nrexquestx() - 1) < equality_threshold,
        urr_uranium_unlimited(),
        functions.if_then_else(abs(unlimited_uraniumxquestx() - 1) < equality_threshold,
                               urr_uranium_unlimited(),
                               functions.if_then_else(abs(choose_extraction_uranium_curve() - 1) < equality_threshold,
                                                      urr_uranium_ewg13(),
                                                      functions.if_then_else(
                                                          abs(choose_extraction_uranium_curve() - 2) < equality_threshold,
                                                          urr_uranium_zittel12(),
                                                          urr_uranium_user_defined()))))
    #test_nans(ret, 1426)
    return ret


@cache('run')
def urr_conv_oil():
    """
    urr conv oil



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_oilxquestx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                      urr_conv_oil_unlimited(),
                                      functions.if_then_else(
                                          abs(unlimited_oilxquestx() - 1) < equality_threshold,
                                          urr_conv_oil_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_conv_oil() - 1) < equality_threshold,
                                              urr_conv_oil_maggio12_middle(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_curve_conv_oil() - 2) < equality_threshold,
                                                  urr_conv_oil_maggio12_high(),
                                                  functions.if_then_else(
                                                      abs(choose_extraction_curve_conv_oil() - 3) < equality_threshold,
                                                      urr_conv_oil_maggio12_low(),
                                                      urr_conv_oil_user_defined()))))), 0)
    #test_nans(ret, 1427)
    return ret


@cache('run')
def growth_biogas():
    """
    growth biogas



    component


    """
    
    ret = biogas_evol() * past_biogas_growth()
    #test_nans(ret, 1428)
    return ret


@cache('run')
def past_biogas_growth():
    """
    past biogas growth



    constant


    """
    
    ret = 0.13194015127346587
    #test_nans(ret, 1429)
    return ret


@cache('step')
def pes_biogas_ej():
    """
    pes biogas ej



    component


    """
    
    ret = integ_pes_biogas_ej()
    #test_nans(ret, 1430)
    return ret


@cache('run')
def biogas_evol():
    """
    biogas evol



    constant


    """
    
    ret = 1
    #test_nans(ret, 1431)
    return ret


@cache('run')
def max_biogas_ej():
    """
    max biogas ej



    constant


    """
    
    ret = 5
    #test_nans(ret, 1432)
    return ret


@cache('step')
def fei_over_lifetime_res_elec_dispatch():
    """
    fei over lifetime res elec dispatch



    component


    """
    
    ret = cedtot_per_tw_over_lifetime_res_elec_dispatch(
    ) * res_elec_capacity_under_construction_tw()
    #test_nans(ret, 1433)
    return ret


@cache('run')
def activate_eroi_allocation_rule():
    """
    activate eroi allocation rule



    constant


    """
    
    ret = 1
    #test_nans(ret, 1434)
    return ret


@cache('step')
def eroi_allocation_rule_per_res_elec():
    """
    eroi allocation rule per res elec



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015, 1,
        functions.if_then_else(
            abs(ratio_eroi_per_techn_vs_eroitot_xstaticx() - 0) < equality_threshold, 0,
            functions.if_then_else(
                ratio_eroi_per_techn_vs_eroitot_xstaticx() < 0.1, 0,
                0.434294 * np.log(ratio_eroi_per_techn_vs_eroitot_xstaticx()) + 1)))
    #test_nans(ret, 1435)
    return ret


@cache('step')
def xdynamicx_eroi_res_elec_var():
    """
    xdynamicx eroi res elec var



    component


    """
    
    ret = functions.if_then_else(abs(fei_res_elec_var() - 0) < equality_threshold, 0,
                                  real_generation_res_elec_ej() / fei_res_elec_var())
    #test_nans(ret, 1436)
    return ret


@cache('run')
def eroixdashxini_res_elec_dispatch():
    """
    eroixdashxini res elec dispatch



    constant


    """
    
    ret = xr.DataArray(data=[50.0,7.0,1.5,3.25,0.0,0.0,0.0,0.0],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1437)
    return ret


@cache('step')
def res_elec_capacity_under_construction_tw():
    """
    res elec capacity under construction tw



    component


    """
    
    ret = res_elec_planned_capacity_tw() / time_construction_res_elec()
    #test_nans(ret, 1438)
    return ret


@cache('step')
def real_generation_solar_pv_ej():
    """
    real generation solar pv ej



    component


    """
    
    ret = real_generation_res_elec_ej().loc['solar pv'].values
    #test_nans(ret, 1439)
    return ret


@cache('step')
def materials_required_for_new_csp_mt():
    """
    materials required for new csp mt



    component


    """
    
    ret = materials_required_for_new_res_elec_mt().loc['csp',:].drop('res elec')
    #test_nans(ret, 1440)
    return ret


@cache('step')
def materials_required_for_new_pv_mt():
    """
    materials required for new pv mt



    component


    """
    
    ret = materials_required_for_new_res_elec_mt().loc['solar pv',:].drop('res elec')
    #test_nans(ret, 1441)
    return ret


@cache('step')
def materials_required_for_new_wind_offshore_mt():
    """
    materials required for new wind offshore mt



    component


    """
    
    ret = materials_required_for_new_res_elec_mt().loc['wind offshore',:].drop('res elec')
    #test_nans(ret, 1442)
    return ret


@cache('step')
def materials_required_for_new_wind_onshore_mt():
    """
    materials required for new wind onshore mt



    component


    """
    
    ret = materials_required_for_new_res_elec_mt().loc['wind onshore',:].drop('res elec')
    #test_nans(ret, 1443)
    return ret


@cache('step')
def materials_required_for_oxandxm_csp_mt():
    """
    materials required for oxandxm csp mt



    component


    """
    
    ret = materials_required_for_oxandxm_res_elec_mt().loc['csp',:].drop('res elec')
    #test_nans(ret, 1444)
    return ret


@cache('step')
def materials_required_for_oxandxm_pv_mt():
    """
    materials required for oxandxm pv mt



    component


    """
    
    ret = materials_required_for_oxandxm_res_elec_mt().loc['solar pv',:].drop('res elec')
    #test_nans(ret, 1445)
    return ret


@cache('step')
def materials_required_for_oxandxm_wind_offshore_mt():
    """
    materials required for oxandxm wind offshore mt



    component


    """
    
    ret = materials_required_for_oxandxm_res_elec_mt().loc['wind offshore',:].drop('res elec')
    #test_nans(ret, 1446)
    return ret


@cache('step')
def materials_required_for_oxandxm_wind_onshore_mt():
    """
    materials required for oxandxm wind onshore mt



    component


    """
    
    ret = materials_required_for_oxandxm_res_elec_mt().loc['wind onshore',:].drop('res elec')
    #test_nans(ret, 1447)
    return ret


@cache('step')
def real_generation_res_elec_ej():
    """
    real generation res elec ej



    component


    """
    
    ret = real_generation_res_elec_twh() * ej_per_twh()
    #test_nans(ret, 1448)
    return ret


@cache('run')
def energy_ownxdashxuse_constant_x830x_mtoe_in_x2013():
    """
    energy ownxdashxuse constant x830x mtoe in x2013



    constant


    """
    
    ret = 830
    #test_nans(ret, 1449)
    return ret


@cache('step')
def cedtot_solar_pv():
    """
    cedtot solar pv



    component


    """
    
    ret = fei_res_elec_var().loc['solar pv'].values
    #test_nans(ret, 1450)
    return ret


@cache('step')
def total_materials_required_for_new_res_elec_mt():
    """
    total materials required for new res elec mt



    component


    """
    
    ret = functions.sum(dim='res elec', data=materials_required_for_new_res_elec_mt())
    #test_nans(ret, 1451)
    return ret


@cache('step')
def total_materials_required_for_oxandxm_res_elec_mt():
    """
    total materials required for oxandxm res elec mt



    component


    """
    
    ret = functions.sum(dim='res elec', data=materials_required_for_oxandxm_res_elec_mt())
    #test_nans(ret, 1452)
    return ret


@cache('run')
def eroixequalsx1():
    """
    eroixequalsx1



    constant


    """
    
    ret = 1
    #test_nans(ret, 1453)
    return ret


@cache('step')
def electrical_distribution_losses_twh():
    """
    electrical distribution losses twh



    component


    """
    
    ret = total_fe_elec_demand_twh() * share_transmxandxdistr_elec_losses()
    #test_nans(ret, 1454)
    return ret


@cache('run')
def max_share_transmxandxdistr_elec_losses():
    """
    max share transmxandxdistr elec losses



    component


    """
    
    ret = share_transmxandxdistr_elec_losses_initial() * (
        1 + 0.0115 * np.exp(4.2297 * 1) - 0.00251)
    #test_nans(ret, 1455)
    return ret


@cache('step')
def heatxdashxcom_distribution_losses():
    """
    heatxdashxcom distribution losses



    component


    """
    
    ret = fed_heatxdashxcom_ej() * share_heat_distribution_losses()
    #test_nans(ret, 1456)
    return ret


@cache('step')
def potential_generation_res_elec_twh():
    """
    potential generation res elec twh



    component


    """
    
    ret = installed_capacity_res_elec_tw() * cp_res_elec() / twe_per_twh()
    #test_nans(ret, 1457)
    return ret


@cache('step')
def output_elec_over_lifetime_res_elec():
    """
    output elec over lifetime res elec



    component


    """
    
    ret = real_cp_res_elec() * res_elec_capacity_under_construction_tw() * (
        1 / twe_per_twh()) * lifetime_res_elec() * ej_per_twh()
    #test_nans(ret, 1458)
    return ret


@cache('step')
def variation_share_transmxandxdistr_elec_losses():
    """
    variation share transmxandxdistr elec losses



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  variation_share_transmxandxdistr_losses_elec() *
                                  remaining_share_transmxandxdistr_elec_losses())
    #test_nans(ret, 1459)
    return ret


@cache('step')
def remaining_share_transmxandxdistr_elec_losses():
    """
    remaining share transmxandxdistr elec losses



    component


    """
    
    ret = (max_share_transmxandxdistr_elec_losses() - share_transmxandxdistr_elec_losses()
            ) / max_share_transmxandxdistr_elec_losses()
    #test_nans(ret, 1460)
    return ret


@cache('step')
def share_transmxandxdistr_elec_losses():
    """
    share transmxandxdistr elec losses



    component


    """
    
    ret = integ_share_transmxandxdistr_elec_losses()
    #test_nans(ret, 1461)
    return ret


@cache('step')
def total_gen_losses_demand_for_elec_plants_ej():
    """
    total gen losses demand for elec plants ej



    component


    """
    
    ret = pe_demand_gas_elec_plants_ej() * (
        1 - efficiency_gas_for_electricity()) + pe_demand_coal_elec_plants_ej() * (
            1 - efficiency_coal_for_electricity()) + pe_demand_oil_elec_plants_ej() * (
                1 - efficiency_liquids_for_electricity()
            ) + pe_losses_uranium_for_elec_ej() + pe_losses_bioe_for_elec_ej()
    #test_nans(ret, 1462)
    return ret


@cache('step')
def pe_losses_bioe_for_elec_ej():
    """
    pe losses bioe for elec ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['solid bioexdashxelec'].values - fe_elec_generation_from_bioe_twh() * ej_per_twh()
    #test_nans(ret, 1463)
    return ret


@cache('run')
def water_for_oxandxm_xdashx_wind_offshore():
    """
    water for oxandxm xdashx wind offshore



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0],
        coords={'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']},
        dims=['water0'])
    #test_nans(ret, 1464)
    return ret


@cache('step')
def total_materials_required_for_res_elec_mt():
    """
    total materials required for res elec mt



    component


    """
    
    ret = total_materials_required_for_new_res_elec_mt(
    ) + total_materials_required_for_oxandxm_res_elec_mt()
    #test_nans(ret, 1465)
    return ret


@cache('step')
def share_energy_for_material_consumption_for_alt_techn_vs_tfec():
    """
    share energy for material consumption for alt techn vs tfec



    component


    """
    
    ret = tfe_required_for_total_material_consumption_for_alt_techn() / real_tfec()
    #test_nans(ret, 1466)
    return ret


@cache('run')
def water_for_oxandxm_xdashx_pv():
    """
    water for oxandxm xdashx pv



    constant


    """
    
    ret = xr.DataArray(data=[0.0,20000.0],
        coords={'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']},
        dims=['water0'])
    #test_nans(ret, 1467)
    return ret


@cache('run')
def water_for_oxandxm_xdashx_wind_onshore():
    """
    water for oxandxm xdashx wind onshore



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0],
        coords={'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']},
        dims=['water0'])
    #test_nans(ret, 1468)
    return ret


@cache('run')
def materials_for_oxandxm_per_capacity_installed_xdashx_wind_offshore():
    """
    materials for oxandxm per capacity installed xdashx wind offshore



    constant


    """
    
    ret = xr.DataArray(data=[0.0,10.8,0.0,0.0,59.6,0.0,0.0,5.8,356.0,0.0,0.0,0.0,122.0,0.0,0.0,0.0,19.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,25.6,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,9.2,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1469)
    return ret


@cache('run')
def materials_for_oxandxm_per_capacity_installed_xdashx_wind_onshore():
    """
    materials for oxandxm per capacity installed xdashx wind onshore



    constant


    """
    
    ret = xr.DataArray(data=[0.0,10.8,0.0,0.0,29.8,0.0,0.0,5.8,65.0,0.0,0.0,0.0,122.0,0.0,0.0,0.0,19.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,25.6,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,9.2,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1470)
    return ret


@cache('run')
def materials_for_oxandxm_per_capacity_installed_xdashx_pv():
    """
    materials for oxandxm per capacity installed xdashx pv



    constant


    """
    
    ret = xr.DataArray(data=[0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,1294.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,2.56,0.04,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1471)
    return ret


@cache('run')
def water_for_oxandxm_xdashx_csp():
    """
    water for oxandxm xdashx csp



    constant


    """
    
    ret = xr.DataArray(data=[12000000.0,500000.0],
        coords={'water0': ['cleanxwithx pumped water', 'distilledxwithx deionized water']},
        dims=['water0'])
    #test_nans(ret, 1472)
    return ret


@cache('run')
def materials_per_capacity_installed_xdashx_pv_x0():
    """
    materials per capacity installed xdashx pv x0



    constant


    """
    
    ret = xr.DataArray(data=[0.0,16000.0,0.0,6.1,0.0,75000.0,550.0,2200.0,15600.0,0.0,0.0,150.0,0.0,0.0,0.3,640000.0,0.0,500000.0,4.5,162500.0,0.0,0.0,0.0,0.0,0.0,0.0,53.5,500.0,100.0,0.0,50.0,0.0,0.0,0.0,235.0,0.0,0.0,0.0,21.2,5760.0,0.0,0.0,0.0,0.0,0.0,3200.0,46.7,6000000.0,463.0,0.0,2000.0,0.0,4.7,6.25,0.0,0.475,0.0,162.5],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1473)
    return ret


@cache('step')
def cum_materials_requirements_for_res_elec():
    """
    cum materials requirements for res elec



    component


    """
    
    ret = integ_cum_materials_requirements_for_res_elec()
    #test_nans(ret, 1474)
    return ret


@cache('run')
def mj_per_ej():
    """
    mj per ej



    constant


    """
    
    ret = 1000000000000.0
    #test_nans(ret, 1475)
    return ret


@cache('run')
def initial_energy_cons_per_unit_of_material_cons_xvirginx():
    """
    initial energy cons per unit of material cons xvirginx



    constant


    """
    
    ret = xr.DataArray(data=[100.0,218.0,0.0,264.0,200.0,4.5,83.0,57.0,38.5,384.2,2000.0,120.0,28.0,28.0,610000.0,15.0,100.0,0.083,3320.0,25.0,21.5,3.0,5.3,0.85,853.0,100.0,220.0,57.6,2000.0,0.95,378.0,21.5,0.0,384.2,164.0,0.0,0.0,70.0,49.0,80.5,95.4,1.0,16.6,0.081,0.1,2000.0,1580.0,0.45,250.0,0.0,56.7,207.0,589000.0,400.0,60.0,3710.0,36.0,72.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1476)
    return ret


@cache('run')
def materials_per_new_capacity_installed_xdashx_csp():
    """
    materials per new capacity installed xdashx csp



    constant


    """
    
    ret = xr.DataArray(data=[0.0,740.0,3280.0,0.0,0.0,250000.0,2200.0,3200.0,15600.0,0.0,0.0,150.0,310.0,2500.0,0.0,130000.0,0.0,500000.0,0.0,650000.0,220000.0,0.0,11000.0,170000.0,0.0,0.0,3000.0,2000.0,100.0,0.0,200.0,340000.0,340000.0,0.0,940.0,0.0,0.0,0.0,0.0,0.0,500.0,1300000.0,4700.0,1900.0,92000.0,0.0,13.0,18000000.0,0.0,18000.0,240000.0,44000.0,0.0,25.0,11.5,1.9,0.0,650.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1477)
    return ret


@cache('run')
def materials_per_new_capacity_installed_xdashx_wind_offshore():
    """
    materials per new capacity installed xdashx wind offshore



    constant


    """
    
    ret = xr.DataArray(data=[0.74,9400.0,0.0,0.0,3800.0,24000.0,0.0,22200.0,18080.88818398,14.58,450.0,0.0,3800.0,0.0,0.0,0.0,950.0,900000.0,0.0,0.0,0.0,0.0,0.0,0.0,0.0,640.0,0.0,0.0,115.90312938,0.0,0.0,0.0,0.0,183.0,111.0,0.0,0.0,670.0,0.0,9200.0,0.0,0.0,0.0,16560.0,0.0,0.0,0.0,12000000.0,0.0,0.0,400000.0,0.0,0.0,0.0,0.0,0.0,640.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1478)
    return ret


@cache('run')
def materials_per_new_capacity_installed_xdashx_wind_onshore():
    """
    materials per new capacity installed xdashx wind onshore



    constant


    """
    
    ret = xr.DataArray(data=[0.74,2030.0,0.0,0.0,1500.0,561600.0,0.0,2700.0,5700.0,4.86,450.0,0.0,6090.0,0.0,0.0,0.0,950.0,11900.0,0.0,22000.0,0.0,0.0,0.0,0.0,0.0,640.0,0.0,0.0,36.53846154,0.0,0.0,0.0,0.0,61.0,111.0,0.0,0.0,670.0,0.0,1940.0,0.0,0.0,0.0,16560.0,0.0,0.0,0.0,1500000.0,0.0,0.0,126100.0,0.0,0.0,0.0,0.0,0.0,640.0,0.0],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1479)
    return ret


@cache('run')
def initial_cumulated_material_requirements_for_res_elec_x1995x():
    """
    initial cumulated material requirements for res elec x1995x



    constant


    """
    
    ret = 0
    #test_nans(ret, 1480)
    return ret


@cache('run')
def materials_per_new_capacity_installed_xdashx_pv():
    """
    materials per new capacity installed xdashx pv



    constant


    """
    
    ret = xr.DataArray(data=[0.0,16000.0,0.0,6.1,0.0,75000.0,550.0,2200.0,15600.0,0.0,0.0,150.0,0.0,0.0,0.3,640000.0,0.0,500000.0,4.5,162500.0,0.0,0.0,0.0,0.0,0.0,0.0,53.5,500.0,100.0,0.0,50.0,0.0,0.0,0.0,235.0,0.0,0.0,0.0,21.2,5760.0,0.0,0.0,0.0,0.0,0.0,3200.0,46.7,6000000.0,463.0,0.0,2000.0,0.0,4.7,6.25,0.0,0.475,0.0,162.5],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1481)
    return ret


@cache('run')
def kg_per_mt():
    """
    kg per mt



    constant


    """
    
    ret = 1000000000
    #test_nans(ret, 1482)
    return ret


@cache('run')
def m_per_t():
    """
    m per t



    constant


    """
    
    ret = 1000000
    #test_nans(ret, 1483)
    return ret


@cache('step')
def elec_generation_variable_from_res_twh():
    """
    elec generation variable from res twh



    component


    """
    
    ret = fe_elec_generation_from_solar_pv_twh() + fe_elec_generation_from_csp_twh(
    ) + fe_elec_generation_from_onshore_wind_twh() + fe_elec_generation_from_offshore_wind_twh()
    #test_nans(ret, 1484)
    return ret


@cache('step')
def new_required_capacity_nuclear():
    """
    new required capacity nuclear



    component


    """
    
    ret = np.maximum(0,
                      functions.if_then_else(time() < 2014, 0,
                                             functions.if_then_else(abs(demand_elec_nre_twh() - 0) < equality_threshold, 0,
                                                                    installed_capacity_nuclear_tw(
                                                                    ) * p_nuclear_elec_gen()))
                      ) * effects_shortage_uranium() * cp_limit_nuclear()
    #test_nans(ret, 1485)
    return ret


@cache('run')
def replacement_res_for_heatxdashxcom():
    """
    replacement res for heatxdashxcom



    constant


    """
    
    ret = xr.DataArray(data=[1.0,1.0,1.0],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1486)
    return ret


@cache('run')
def losses_solar_for_heat():
    """
    losses solar for heat



    constant


    """
    
    ret = 0.37
    #test_nans(ret, 1487)
    return ret


@cache('run')
def cpxdashxini_res_for_heat():
    """
    cpxdashxini res for heat



    constant


    """
    
    ret = xr.DataArray(data=[1.0,0.265,0.5],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1488)
    return ret


@cache('run')
def efficiency_geothermal_for_heat():
    """
    efficiency geothermal for heat



    constant


    """
    
    ret = 0.6960561017716173
    #test_nans(ret, 1489)
    return ret


@cache('run')
def efficiency_solar_panels_for_heat():
    """
    efficiency solar panels for heat



    constant


    """
    
    ret = 0.095
    #test_nans(ret, 1490)
    return ret


@cache('step')
def max_fe_potential_res_for_heat():
    """
    max fe potential res for heat



    component


    """
    
    ret = xr.DataArray(data=[
        max_pe_potential_res_for_heat().loc['solarxdashxheat'].values + efficiency_res_heat().loc['solarxdashxheat'].values * 0,
        max_pe_potential_res_for_heat().loc['geotxdashxheat'].values * efficiency_res_heat().loc['geotxdashxheat'].values,
        max_pe_potential_res_for_heat().loc['solid bioexdashxheat'].values * efficiency_res_heat().loc['solid bioexdashxheat'].values],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1491)
    return ret


@cache('run')
def p_geothermal_for_heat():
    """
    p geothermal for heat



    constant


    """
    
    ret = 0.076
    #test_nans(ret, 1492)
    return ret


@cache('run')
def past_res_growth_for_heatxdashxcom():
    """
    past res growth for heatxdashxcom



    constant


    """
    
    ret = xr.DataArray(data=[0.2771226,0.03571556,0.06278667],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1493)
    return ret


@cache('step')
def installed_capacity_res_heatxdashxcom_tw():
    """
    installed capacity res heatxdashxcom tw



    component


    """
    
    ret = integ_installed_capacity_res_heatxdashxcom_tw()
    #test_nans(ret, 1494)
    return ret


@cache('run')
def initial_value_res_for_heatxdashxcom():
    """
    initial value res for heatxdashxcom



    constant


    """
    
    ret = xr.DataArray(data=[2.03033268e-06,0.00199927,0.01167364],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1495)
    return ret


@cache('run')
def p_solar_for_heat():
    """
    p solar for heat



    constant


    """
    
    ret = 0.127
    #test_nans(ret, 1496)
    return ret


@cache('run')
def life_time_res_for_heat():
    """
    life time res for heat



    constant


    """
    
    ret = xr.DataArray(data=[25.0,25.0,30.0],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1497)
    return ret


@cache('step')
def wear_res_capacity_for_heatxdashxcom_tw():
    """
    wear res capacity for heatxdashxcom tw



    component


    """
    
    ret = installed_capacity_res_heatxdashxcom_tw() / life_time_res_for_heat()
    #test_nans(ret, 1498)
    return ret


@cache('step')
def potential_fe_gen_elec_fossil_fuel_chp_plants_ej():
    """
    potential fe gen elec fossil fuel chp plants ej



    component


    """
    
    ret = (potential_fe_gen_elec_coal_chp_plants_ej() + potential_fe_gen_elec_gas_chp_plants_ej()
            + potential_fe_gen_elec_liquids_chp_plants_ej())
    #test_nans(ret, 1499)
    return ret


@cache('step')
def fed_heat_liquids_chp_plants_ej():
    """
    fed heat liquids chp plants ej



    component


    """
    
    ret = fed_heat_fossil_fuels_chp_plants_ej() * share_chp_plants_oil()
    #test_nans(ret, 1500)
    return ret


@cache('step')
def fed_heat_coal_chp_plants_ej():
    """
    fed heat coal chp plants ej



    component


    """
    
    ret = fed_heat_fossil_fuels_chp_plants_ej() * share_chp_plants_coal()
    #test_nans(ret, 1501)
    return ret


@cache('step')
def remaining_potential_csp():
    """
    remaining potential csp



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['csp'].values > real_generation_res_elec_twh().loc['csp'].values,
                                  (max_potential_res_elec_twh().loc['csp'].values - real_generation_res_elec_twh().loc['csp'].values) /
                                  max_potential_res_elec_twh().loc['csp'].values, 0)
    #test_nans(ret, 1502)
    return ret


@cache('step')
def pe_csp_for_elec_generation_ej():
    """
    pe csp for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['csp'].values
    #test_nans(ret, 1503)
    return ret


@cache('step')
def invest_res_for_elec():
    """
    invest res for elec



    component


    """
    
    ret = np.maximum(
        invest_biow_tdolar() + invest_geotxdashxelec_tdolar() + invest_hydro_tdolar() +
        invest_oceanic_tdolar() + invest_solar_tdolar() + invest_onshore_wind_tdolar() +
        invest_offshore_wind_tdolar() + invest_csp_tdolar(), 0)
    #test_nans(ret, 1504)
    return ret


@cache('run')
def power_density_csp():
    """
    power density csp



    component


    """
    
    ret = power_density_res_elec_twexdivxmha().loc['csp'].values
    #test_nans(ret, 1505)
    return ret


@cache('step')
def max_potential_res_elec_twe():
    """
    max potential res elec twe



    component


    """
    
    ret = xr.DataArray(data=[
        max_hydro_twe(),
        max_geotxdashxelec_twe(),
        max_bioe_twe(),
        max_oceanic_twe(),
        max_onshore_wind_twe(),
        max_offshore_wind_twe(),
        max_solar_pv_on_land_twe(),
        max_csp_twe()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1506)
    return ret


@cache('step')
def surface_csp_mha():
    """
    surface csp mha



    component


    """
    
    ret = surface_res_elec().loc['csp'].values
    #test_nans(ret, 1507)
    return ret


@cache('step')
def fe_elec_generation_from_csp_twh():
    """
    fe elec generation from csp twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['csp'].values
    #test_nans(ret, 1508)
    return ret


@cache('step')
def invest_csp_tdolar():
    """
    invest csp tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['csp'].values
    #test_nans(ret, 1509)
    return ret


@cache('step')
def cp_limit_nuclear():
    """
    cp limit nuclear



    component


    """
    
    ret = functions.if_then_else(cp_nuclear() > min_cp_nuclear(), 1, 0)
    #test_nans(ret, 1510)
    return ret


@cache('step')
def cp_nuclear():
    """
    cp nuclear



    component


    """
    
    ret = cp_nuclear_initial() * cp_exogenous_res_elec_dispatch_reduction()
    #test_nans(ret, 1511)
    return ret


@cache('step')
def installed_capacity_nuclear_tw():
    """
    installed capacity nuclear tw



    component


    """
    
    ret = integ_installed_capacity_nuclear_tw()
    #test_nans(ret, 1512)
    return ret


@cache('step')
def nuclear_capacity_phasexdashxout():
    """
    nuclear capacity phasexdashxout



    component


    """
    
    ret = functions.if_then_else(
        abs(selection_of_nuclear_scenario() - 4) < equality_threshold,
        functions.if_then_else(time() < start_year_nuclear_growth_scen3xdashx4(), 0,
                               p_nuclear_scen3xdashx4() * installed_capacity_nuclear_tw()), 0)
    #test_nans(ret, 1513)
    return ret


@cache('run')
def min_cp_nuclear():
    """
    min cp nuclear



    constant


    """
    
    ret = 0.6
    #test_nans(ret, 1514)
    return ret


@cache('step')
def replacement_nuclear_capacity():
    """
    replacement nuclear capacity



    component


    """
    
    ret = functions.if_then_else(
        time() < 2013,
        nuclear_capacity_under_construction(),
        functions.if_then_else(
            abs(selection_of_nuclear_scenario() - 2) < equality_threshold, 0,
            functions.if_then_else(abs(selection_of_nuclear_scenario() - 4) < equality_threshold, 0,
                                   replacement_rate_nuclear() * wear_nuclear() *
                                   (1 - nuclear_overcapacity())))) * cp_limit_nuclear()
    #test_nans(ret, 1515)
    return ret


@cache('run')
def min_cp_baseload_res():
    """
    min cp baseload res



    constant


    """
    
    ret = xr.DataArray(data=[0.15,0.15,0.15,0.15,0.0525,0.07875,0.0375,0.0625],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1516)
    return ret


@cache('step')
def abundance_uranium():
    """
    abundance uranium



    component


    """
    
    ret = functions.if_then_else(
        abs(pe_demand_uranium_ej() - 0) < equality_threshold, 1,
        functions.if_then_else(
            extraction_uranium_ej() > pe_demand_uranium_ej(), 1,
            1 - ((pe_demand_uranium_ej() - extraction_uranium_ej()) / pe_demand_uranium_ej())))
    #test_nans(ret, 1517)
    return ret


@cache('step')
def nuclear_overcapacity():
    """
    nuclear overcapacity



    component


    """
    
    ret = functions.if_then_else(
        abs(potential_generation_nuclear_elec_twh() - 0) < equality_threshold, 0,
        (potential_generation_nuclear_elec_twh() - fe_nuclear_elec_generation_twh()) /
        potential_generation_nuclear_elec_twh())
    #test_nans(ret, 1518)
    return ret


@cache('run')
def p_nuclear_scen3xdashx4():
    """
    p nuclear scen3xdashx4



    constant


    """
    
    ret = 0
    #test_nans(ret, 1519)
    return ret


@cache('run')
def start_year_nuclear_growth_scen3xdashx4():
    """
    start year nuclear growth scen3xdashx4



    constant


    """
    
    ret = 2020
    #test_nans(ret, 1520)
    return ret


@cache('step')
def p_nuclear_elec_gen():
    """
    p nuclear elec gen



    component


    """
    
    ret = functions.if_then_else(
        abs(selection_of_nuclear_scenario() - 1) < equality_threshold, 0,
        functions.if_then_else(abs(selection_of_nuclear_scenario() - 2) < equality_threshold, 0,
                               functions.if_then_else(
                                   abs(selection_of_nuclear_scenario() - 4) < equality_threshold, 0,
                                   functions.if_then_else(
                                       abs(selection_of_nuclear_scenario() - 3) < equality_threshold,
                                       functions.if_then_else(
                                           time() < start_year_nuclear_growth_scen3xdashx4(), 0,
                                           p_nuclear_scen3xdashx4()), 0))))
    #test_nans(ret, 1521)
    return ret


@cache('step')
def required_capacity_nuclear_tw():
    """
    required capacity nuclear tw



    component


    """
    
    ret = integ_required_capacity_nuclear_tw()
    #test_nans(ret, 1522)
    return ret


@cache('run')
def time_planification_nuclear():
    """
    time planification nuclear



    constant


    """
    
    ret = 1
    #test_nans(ret, 1523)
    return ret


@cache('run')
def time_construction_nuclear():
    """
    time construction nuclear



    constant


    """
    
    ret = 8
    #test_nans(ret, 1524)
    return ret


@cache('run')
def initial_capacity_installed_nuclear():
    """
    initial capacity installed nuclear



    component


    """
    
    ret = initial_gen_nuclear() * twe_per_twh() / cp_nuclear_initial()
    #test_nans(ret, 1525)
    return ret


@cache('step')
def new_nuclear_capacity_under_planning():
    """
    new nuclear capacity under planning



    component


    """
    
    ret = np.maximum(0, required_capacity_nuclear_tw() / time_planification_nuclear())
    #test_nans(ret, 1526)
    return ret


@cache('run')
def selection_of_nuclear_scenario():
    """
    selection of nuclear scenario



    constant


    """
    
    ret = 1
    #test_nans(ret, 1527)
    return ret


@cache('run')
def initial_gen_nuclear():
    """
    initial gen nuclear



    constant


    """
    
    ret = 2210.0
    #test_nans(ret, 1528)
    return ret


@cache('run')
def initial_capacity_in_construction_nuclear():
    """
    initial capacity in construction nuclear



    constant


    """
    
    ret = 0
    #test_nans(ret, 1529)
    return ret


@cache('run')
def initial_required_capacity_nuclear():
    """
    initial required capacity nuclear



    constant


    """
    
    ret = 0
    #test_nans(ret, 1530)
    return ret


@cache('step')
def planned_nuclear_capacity_tw():
    """
    planned nuclear capacity tw



    component


    """
    
    ret = integ_planned_nuclear_capacity_tw()
    #test_nans(ret, 1531)
    return ret


@cache('step')
def wear_nuclear():
    """
    wear nuclear



    component


    """
    
    ret = functions.if_then_else(time() < 2012, 0,
                                  installed_capacity_nuclear_tw() / life_time_nuclear())
    #test_nans(ret, 1532)
    return ret


@cache('step')
def effects_shortage_uranium():
    """
    effects shortage uranium



    component


    """
    
    ret = functions.if_then_else(abs(extraction_uranium_ej() - 0) < equality_threshold, 0,
                                  functions.if_then_else(abundance_uranium() > 0.8,
                                                         ((abundance_uranium() - 0.8) * 5)**2, 0))
    #test_nans(ret, 1533)
    return ret


@cache('step')
def tpes_intensity_ej_tdollar():
    """
    tpes intensity ej tdollar



    component


    """
    
    ret = functions.zidz(tpes_ej(), gdp())
    #test_nans(ret, 1534)
    return ret


@cache('step')
def real_ped_intensity_of_electricity():
    """
    real ped intensity of electricity



    component


    """
    
    ret = functions.zidz((total_fe_elec_demand_ej() + elec_gen_related_losses_ej()), gdp())
    #test_nans(ret, 1535)
    return ret


@cache('step')
def annual_gdp_growth_rate():
    """
    annual gdp growth rate



    component


    """
    
    ret = -1 + functions.zidz(gdp(), gdp_delayed_1yr())
    #test_nans(ret, 1536)
    return ret


@cache('step')
def gdp_delayed_1yr():
    """
    gdp delayed 1yr



    component


    """
    
    ret = delay_gdp_round1__time_step___time_step_2916_1__time_step()
    #test_nans(ret, 1537)
    return ret


@cache('step')
def share_tot_monet_invest_elec_res_vs_gdp():
    """
    share tot monet invest elec res vs gdp



    component


    """
    
    ret = functions.zidz(total_monet_invest_res_for_elec_tdolar(), gdp())
    #test_nans(ret, 1538)
    return ret


@cache('step')
def gdppc():
    """
    gdppc



    component


    """
    
    ret = gdp() * dollars_to_tdollars() / population()
    #test_nans(ret, 1539)
    return ret


@cache('run')
def unlimited_coalxquestx():
    """
    unlimited coalxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1540)
    return ret


@cache('step')
def real_tfec():
    """
    real tfec



    component


    """
    
    ret = functions.sum(real_fe_consumption_by_fuel())
    #test_nans(ret, 1541)
    return ret


@cache('run')
def unlimited_uraniumxquestx():
    """
    unlimited uraniumxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1542)
    return ret


@cache('run')
def urr_coal_unlimited():
    """
    urr coal unlimited



    component


    """
    
    ret = urr_coal_high15() * 10000
    #test_nans(ret, 1543)
    return ret


@cache('run')
def urr_uranium_unlimited():
    """
    urr uranium unlimited



    component


    """
    
    ret = urr_uranium_ewg13() * 10000
    #test_nans(ret, 1544)
    return ret


@cache('run')
def urr_conv_gas_unlimited():
    """
    urr conv gas unlimited



    component


    """
    
    ret = urr_conv_gas_high_mohr15() * 10000
    #test_nans(ret, 1545)
    return ret


@cache('run')
def unlimited_gasxquestx():
    """
    unlimited gasxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1546)
    return ret


@cache('run')
def unlimited_oilxquestx():
    """
    unlimited oilxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1547)
    return ret


@cache('run')
def urr_conv_oil_unlimited():
    """
    urr conv oil unlimited



    component


    """
    
    ret = urr_conv_oil_maggio12_high() * 10000
    #test_nans(ret, 1548)
    return ret


@cache('run')
def b_logistic():
    """
    b logistic



    constant


    """
    
    ret = 50
    #test_nans(ret, 1549)
    return ret


@cache('run')
def a_logistic():
    """
    a logistic



    constant


    """
    
    ret = 750
    #test_nans(ret, 1550)
    return ret


@cache('run')
def activate_elf_by_scenxquestx():
    """
    activate elf by scenxquestx



    constant


    """
    
    ret = 1
    #test_nans(ret, 1551)
    return ret


@cache('step')
def total_demand_liquids_mbxdivxd():
    """
    total demand liquids mbxdivxd



    component


    """
    
    ret = ped_liquids_ej() * mbxdivxd_per_ejxdivxyear()
    #test_nans(ret, 1552)
    return ret


@cache('step')
def ped_coal_without_ctl():
    """
    ped coal without ctl



    component


    """
    
    ret = ped_coal_ej() - ped_coal_for_ctl_ej()
    #test_nans(ret, 1553)
    return ret


@cache('step')
def surface_hydro_mha():
    """
    surface hydro mha



    component


    """
    
    ret = surface_res_elec().loc['hydro'].values
    #test_nans(ret, 1554)
    return ret


@cache('step')
def remaining_potential_hydro():
    """
    remaining potential hydro



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['hydro'].values > real_generation_res_elec_twh().loc['hydro'].values,
                                  (max_potential_res_elec_twh().loc['hydro'].values - real_generation_res_elec_twh().loc['hydro'].values) /
                                  max_potential_res_elec_twh().loc['hydro'].values, 0)
    #test_nans(ret, 1555)
    return ret


@cache('step')
def grid_reinforcement_costs_tdollar():
    """
    grid reinforcement costs tdollar



    component


    """
    
    ret = grid_reinforcement_costs() * new_capacity_installed_onshore_wind_tw() / g_per_t()
    #test_nans(ret, 1556)
    return ret


@cache('step')
def remaining_potential_onshore_wind():
    """
    remaining potential onshore wind



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['wind onshore'].values > real_generation_res_elec_twh().loc['wind onshore'].values,
                                  (max_potential_res_elec_twh().loc['wind onshore'].values - real_generation_res_elec_twh().loc['wind onshore'].values) /
                                  max_potential_res_elec_twh().loc['wind onshore'].values, 0)
    #test_nans(ret, 1557)
    return ret


@cache('step')
def remaining_potential_offshore_wind():
    """
    remaining potential offshore wind



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['wind offshore'].values > real_generation_res_elec_twh().loc['wind offshore'].values,
                                  (max_potential_res_elec_twh().loc['wind offshore'].values - real_generation_res_elec_twh().loc['wind offshore'].values) /
                                  max_potential_res_elec_twh().loc['wind offshore'].values, 0)
    #test_nans(ret, 1558)
    return ret


@cache('step')
def remaining_potential_bioe():
    """
    remaining potential bioe



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['solid bioexdashxelec'].values > real_generation_res_elec_twh().loc['solid bioexdashxelec'].values,
                                  (max_potential_res_elec_twh().loc['solid bioexdashxelec'].values - real_generation_res_elec_twh().loc['solid bioexdashxelec'].values) /
                                  max_potential_res_elec_twh().loc['solid bioexdashxelec'].values, 0)
    #test_nans(ret, 1559)
    return ret


@cache('step')
def remaining_potential_oceanic():
    """
    remaining potential oceanic



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['oceanic'].values > real_generation_res_elec_twh().loc['oceanic'].values,
                                  (max_potential_res_elec_twh().loc['oceanic'].values - real_generation_res_elec_twh().loc['oceanic'].values) /
                                  max_potential_res_elec_twh().loc['oceanic'].values, 0)
    #test_nans(ret, 1560)
    return ret


@cache('step')
def remaining_potential_geotxdashxelec():
    """
    remaining potential geotxdashxelec



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['geotxdashxelec'].values > real_generation_res_elec_twh().loc['geotxdashxelec'].values,
                                  (max_potential_res_elec_twh().loc['geotxdashxelec'].values - real_generation_res_elec_twh().loc['geotxdashxelec'].values) /
                                  max_potential_res_elec_twh().loc['geotxdashxelec'].values, 0)
    #test_nans(ret, 1561)
    return ret


@cache('step')
def remaining_potential_solarxdashxelec_pv():
    """
    remaining potential solarxdashxelec pv



    component


    """
    
    ret = functions.if_then_else(max_potential_res_elec_twh().loc['solar pv'].values > real_generation_res_elec_twh().loc['solar pv'].values,
                                  (max_potential_res_elec_twh().loc['solar pv'].values - real_generation_res_elec_twh().loc['solar pv'].values) /
                                  max_potential_res_elec_twh().loc['solar pv'].values, 0)
    #test_nans(ret, 1562)
    return ret


@cache('run')
def initial_nonxdashxenergy_use():
    """
    initial nonxdashxenergy use



    constant


    """
    
    ret = xr.DataArray(data=[0.001995,0.001995,17.86474451,3.74572862,0.86732782],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1563)
    return ret


@cache('step')
def real_total_output():
    """
    real total output



    component


    """
    
    ret = functions.sum(real_total_output_by_sector())
    #test_nans(ret, 1564)
    return ret


@cache('step')
def real_demand():
    """
    real demand



    component


    """
    
    ret = functions.sum(real_demand_by_sector())
    #test_nans(ret, 1565)
    return ret


@cache('run')
def max_potential_res_elec_twh():
    """
    max potential res elec twh



    component


    """
    
    ret = max_potential_res_elec_twe() / twe_per_twh()
    #test_nans(ret, 1566)
    return ret


@cache('step')
def pe_onshore_wind_for_elec_generation_ej():
    """
    pe onshore wind for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['wind onshore'].values
    #test_nans(ret, 1567)
    return ret


@cache('step')
def fe_elec_generation_from_solar_pv_twh():
    """
    fe elec generation from solar pv twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['solar pv'].values
    #test_nans(ret, 1568)
    return ret


@cache('run')
def power_density_solar_pv():
    """
    power density solar pv



    component


    """
    
    ret = power_density_res_elec_twexdivxmha().loc['solar pv'].values
    #test_nans(ret, 1569)
    return ret


@cache('step')
def pe_hydro_for_elec_generation_ej():
    """
    pe hydro for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['hydro'].values
    #test_nans(ret, 1570)
    return ret


@cache('step')
def invest_biow_tdolar():
    """
    invest biow tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['solid bioexdashxelec'].values
    #test_nans(ret, 1571)
    return ret


@cache('step')
def fe_elec_generation_from_hydro_twh():
    """
    fe elec generation from hydro twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['hydro'].values
    #test_nans(ret, 1572)
    return ret


@cache('step')
def invest_geotxdashxelec_tdolar():
    """
    invest geotxdashxelec tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['geotxdashxelec'].values
    #test_nans(ret, 1573)
    return ret


@cache('step')
def invest_hydro_tdolar():
    """
    invest hydro tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['hydro'].values
    #test_nans(ret, 1574)
    return ret


@cache('step')
def fe_elec_generation_from_onshore_wind_twh():
    """
    fe elec generation from onshore wind twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['wind onshore'].values
    #test_nans(ret, 1575)
    return ret


@cache('step')
def invest_oceanic_tdolar():
    """
    invest oceanic tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['oceanic'].values
    #test_nans(ret, 1576)
    return ret


@cache('step')
def invest_onshore_wind_tdolar():
    """
    invest onshore wind tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['wind onshore'].values
    #test_nans(ret, 1577)
    return ret


@cache('step')
def invest_offshore_wind_tdolar():
    """
    invest offshore wind tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['wind offshore'].values
    #test_nans(ret, 1578)
    return ret


@cache('step')
def fe_elec_generation_from_geotxdashxelec_twh():
    """
    fe elec generation from geotxdashxelec twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['geotxdashxelec'].values
    #test_nans(ret, 1579)
    return ret


@cache('step')
def pe_geotxdashxelec_for_elec_generation_ej():
    """
    pe geotxdashxelec for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['geotxdashxelec'].values
    #test_nans(ret, 1580)
    return ret


@cache('step')
def pe_solar_pv_for_elec_generation_ej():
    """
    pe solar pv for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['solar pv'].values
    #test_nans(ret, 1581)
    return ret


@cache('step')
def fe_elec_generation_from_bioe_twh():
    """
    fe elec generation from bioe twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['solid bioexdashxelec'].values
    #test_nans(ret, 1582)
    return ret


@cache('step')
def pe_oceanic_for_elec_generation_ej():
    """
    pe oceanic for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['oceanic'].values
    #test_nans(ret, 1583)
    return ret


@cache('step')
def pe_bioe_for_elec_generation_ej():
    """
    pe bioe for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['solid bioexdashxelec'].values
    #test_nans(ret, 1584)
    return ret


@cache('step')
def fe_elec_generation_from_offshore_wind_twh():
    """
    fe elec generation from offshore wind twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['wind offshore'].values
    #test_nans(ret, 1585)
    return ret


@cache('step')
def fe_elec_generation_from_oceanic_twh():
    """
    fe elec generation from oceanic twh



    component


    """
    
    ret = real_generation_res_elec_twh().loc['oceanic'].values
    #test_nans(ret, 1586)
    return ret


@cache('step')
def pe_offshore_wind_for_elec_generation_ej():
    """
    pe offshore wind for elec generation ej



    component


    """
    
    ret = pe_real_generation_res_elec().loc['wind offshore'].values
    #test_nans(ret, 1587)
    return ret


@cache('step')
def invest_solar_tdolar():
    """
    invest solar tdolar



    component


    """
    
    ret = invest_res_elec_tdolar().loc['solar pv'].values
    #test_nans(ret, 1588)
    return ret


@cache('run')
def initial_value_land_compet_biofuels_2gen_mha():
    """
    initial value land compet biofuels 2gen mha



    component


    """
    
    ret = initial_value_land_compet_biofuels_2gen_ktoe() * ej_per_ktoe()
    #test_nans(ret, 1589)
    return ret


@cache('run')
def initial_value_land_compet_biofuels_2gen_ktoe():
    """
    initial value land compet biofuels 2gen ktoe



    constant


    """
    
    ret = 9052.844108405054
    #test_nans(ret, 1590)
    return ret


@cache('step')
def abundance_coal():
    """
    abundance coal



    component


    """
    
    ret = functions.if_then_else(extraction_coal_ej() > ped_coal_ej(), 1, 1 - functions.zidz(
        (ped_coal_ej() - extraction_coal_ej()), ped_coal_ej()))
    #test_nans(ret, 1591)
    return ret


@cache('step')
def abundance_liquids():
    """
    abundance liquids



    component


    """
    
    ret = functions.if_then_else(
        ped_liquids_ej() < pes_liquids_ej(), 1,
        1 - functions.zidz(ped_liquids_ej() - pes_liquids_ej(), ped_liquids_ej()))
    #test_nans(ret, 1592)
    return ret


@cache('step')
def check_liquids():
    """
    check liquids



    component


    """
    
    ret = functions.zidz((ped_liquids_ej() - pes_liquids_ej()), pes_liquids_ej())
    #test_nans(ret, 1593)
    return ret


@cache('run')
def share_heat_distribution_losses():
    """
    share heat distribution losses



    constant


    """
    
    ret = 0.0615
    #test_nans(ret, 1594)
    return ret


@cache('step')
def potential_tot_res_elec_after_intermitt():
    """
    potential tot res elec after intermitt



    component


    """
    
    ret = functions.sum(potential_res_elec_after_intermitt_twh())
    #test_nans(ret, 1595)
    return ret


@cache('step')
def pe_biow_for_elec_generation_mtoe():
    """
    pe biow for elec generation mtoe



    component


    """
    
    ret = pe_real_generation_res_elec().loc['solid bioexdashxelec'].values * mtoe_per_ej()
    #test_nans(ret, 1596)
    return ret


@cache('step')
def remaining_potential_tot_res_elec_after_intermitt():
    """
    remaining potential tot res elec after intermitt



    component


    """
    
    ret = functions.if_then_else(
        potential_tot_res_elec_after_intermitt() > fe_real_tot_generation_res_elec_twh(),
        (potential_tot_res_elec_after_intermitt() - fe_real_tot_generation_res_elec_twh()) /
        potential_tot_res_elec_after_intermitt(), 0)
    #test_nans(ret, 1597)
    return ret


@cache('run')
def power_density_res_elec_twexdivxmha():
    """
    power density res elec twexdivxmha



    constant


    """
    
    ret = xr.DataArray(data=[0.04,0.5,0.0,0.0,0.025,0.01,0.033,0.033],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1598)
    return ret


@cache('step')
def time_95pc_ts_potential_res_elec():
    """
    time 95pc ts potential res elec



    component


    """
    
    ret = functions.if_then_else(remaining_potential_res_elec_after_intermitt() > 0.05, 0, time())
    #test_nans(ret, 1599)
    return ret


@cache('step')
def surface_res_elec():
    """
    surface res elec



    component


    """
    
    ret = functions.if_then_else(
        abs(power_density_res_elec_twxdivxmha() - 0) < equality_threshold, 0,
        installed_capacity_res_elec_tw() / power_density_res_elec_twxdivxmha())
    #test_nans(ret, 1600)
    return ret


@cache('run')
def max_geotxdashxelec_twe():
    """
    max geotxdashxelec twe



    component


    """
    
    ret = max_pe_geotxdashxelec_twth() * efficiency_conversion_geot_pe_to_elec()
    #test_nans(ret, 1601)
    return ret


@cache('run')
def max_pe_geotxdashxelec_twth():
    """
    max pe geotxdashxelec twth



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 1602)
    return ret


@cache('run')
def efficiency_conversion_geot_pe_to_elec():
    """
    efficiency conversion geot pe to elec



    constant


    """
    
    ret = 0.3333333333333333
    #test_nans(ret, 1603)
    return ret


@cache('run')
def geot_pe_potential_for_heat_ej():
    """
    geot pe potential for heat ej



    component


    """
    
    ret = geot_pe_potential_for_heat_twth() * ej_per_twh() / twe_per_twh()
    #test_nans(ret, 1604)
    return ret


@cache('run')
def initial_instal_cap_res_elec():
    """
    initial instal cap res elec



    constant


    """
    
    ret = xr.DataArray(data=[0.62449003,0.00738462,0.02075556,0.00020761,0.00455706,4.7771042e-06,3.92104377e-05,0.0003538],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1605)
    return ret


@cache('step')
def abundance_res_elec2():
    """
    abundance res elec2



    component


    """
    
    ret = np.sqrt(abundance_res_elec())
    #test_nans(ret, 1606)
    return ret


@cache('step')
def potential_tot_generation_res_elec_twh():
    """
    potential tot generation res elec twh



    component


    """
    
    ret = functions.sum(potential_generation_res_elec_twh())
    #test_nans(ret, 1607)
    return ret


@cache('step')
def res_elec_tot_overcapacity():
    """
    res elec tot overcapacity



    component


    """
    
    ret = functions.if_then_else(
        abs(potential_tot_generation_res_elec_twh() - 0) < equality_threshold, 0,
        (potential_tot_generation_res_elec_twh() - fe_real_tot_generation_res_elec_twh()) /
        potential_tot_generation_res_elec_twh())
    #test_nans(ret, 1608)
    return ret


@cache('step')
def wear_res_elec():
    """
    wear res elec



    component


    """
    
    ret = functions.if_then_else(time() < 2015, 0,
                                  installed_capacity_res_elec_tw() / lifetime_res_elec())
    #test_nans(ret, 1609)
    return ret


@cache('run')
def g_per_t():
    """
    g per t



    constant


    """
    
    ret = 1000
    #test_nans(ret, 1610)
    return ret


@cache('run')
def time_construction_res_elec():
    """
    time construction res elec



    constant


    """
    
    ret = xr.DataArray(data=[2.0,2.0,2.0,2.0,2.0,4.0,1.0,2.0],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1611)
    return ret


@cache('run')
def time_planification_res_elec():
    """
    time planification res elec



    constant


    """
    
    ret = xr.DataArray(data=[2.0,3.5,0.5,0.5,1.0,0.5,0.5,0.5],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1612)
    return ret


@cache('step')
def new_res_elec_capacity_under_planning():
    """
    new res elec capacity under planning



    component


    """
    
    ret = np.maximum(0, required_capacity_res_elec_tw() / time_planification_res_elec())
    #test_nans(ret, 1613)
    return ret


@cache('run')
def lifetime_res_elec():
    """
    lifetime res elec



    constant


    """
    
    ret = xr.DataArray(data=[80.0,30.0,30.0,40.0,20.0,20.0,25.0,25.0],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1614)
    return ret


@cache('run')
def past_res_elec_capacity_growth():
    """
    past res elec capacity growth



    constant


    """
    
    ret = xr.DataArray(data=[0.02816056,0.02391914,0.07241524,0.04771433,0.25131581,0.41031785,0.45342377,0.295],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1615)
    return ret


@cache('run')
def mt_per_gt():
    """
    mt per gt



    constant


    """
    
    ret = 1000
    #test_nans(ret, 1616)
    return ret


@cache('run')
def gtl_efficiency():
    """
    gtl efficiency



    constant


    """
    
    ret = 0.52
    #test_nans(ret, 1617)
    return ret


@cache('run')
def ctl_efficiency():
    """
    ctl efficiency



    constant


    """
    
    ret = 0.31
    #test_nans(ret, 1618)
    return ret


@cache('step')
def ctlxplusxgtl_gb():
    """
    ctlxplusxgtl gb



    component


    """
    
    ret = fes_ctlxplusxgtl_ej() / gboe_per_ej()
    #test_nans(ret, 1619)
    return ret


@cache('run')
def mbxdivxd_per_ejxdivxyear():
    """
    mbxdivxd per ejxdivxyear



    constant


    """
    
    ret = 0.479726
    #test_nans(ret, 1620)
    return ret


@cache('run')
def hist_growth_ctl():
    """
    hist growth ctl



    constant


    """
    
    ret = 0.021
    #test_nans(ret, 1621)
    return ret


@cache('run')
def gboe_per_ej():
    """
    gboe per ej



    constant


    """
    
    ret = 5.582
    #test_nans(ret, 1622)
    return ret


@cache('run')
def hist_growth_gtl():
    """
    hist growth gtl



    constant


    """
    
    ret = 0.163
    #test_nans(ret, 1623)
    return ret


@cache('run')
def urr_uranium_user_defined():
    """
    urr uranium user defined



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1624)
    return ret


@cache('run')
def replacement_rate_nuclear():
    """
    replacement rate nuclear



    constant


    """
    
    ret = 1
    #test_nans(ret, 1625)
    return ret


@cache('step')
def demand_gas_for_oil_refinery_gains():
    """
    demand gas for oil refinery gains



    component


    """
    
    ret = oil_refinery_gains_ej() * efficiency_gas_for_oil_refinery_gains()
    #test_nans(ret, 1626)
    return ret


@cache('step')
def share_res_vs_tpes():
    """
    share res vs tpes



    component


    """
    
    ret = tpe_from_res_ej() / tpes_ej()
    #test_nans(ret, 1627)
    return ret


@cache('step')
def tpes_mtoe():
    """
    tpes mtoe



    component


    """
    
    ret = tpes_ej() * mtoe_per_ej()
    #test_nans(ret, 1628)
    return ret


@cache('step')
def fe_elec_generation_from_nre_twh():
    """
    fe elec generation from nre twh



    component


    """
    
    ret = fe_elec_generation_from_fossil_fuels_twh() + fe_nuclear_elec_generation_twh()
    #test_nans(ret, 1629)
    return ret


@cache('run')
def pepc_consumption_people_depending_on_trad_biomass():
    """
    pepc consumption people depending on trad biomass



    component


    """
    
    ret = pe_consumption_trad_biomass_ref() / people_relying_trad_biomass_ref()
    #test_nans(ret, 1630)
    return ret


@cache('step')
def share_gas_for_oil_refinery_gains():
    """
    share gas for oil refinery gains



    component


    """
    
    ret = functions.if_then_else(ped_natx_gas_without_gtl() > 0,
                                  demand_gas_for_oil_refinery_gains() / ped_natx_gas_without_gtl(),
                                  0.5)
    #test_nans(ret, 1631)
    return ret


@cache('run')
def people_relying_trad_biomass_ref():
    """
    people relying trad biomass ref



    constant


    """
    
    ret = 2500000000
    #test_nans(ret, 1632)
    return ret


@cache('run')
def pe_consumption_trad_biomass_ref():
    """
    pe consumption trad biomass ref



    constant


    """
    
    ret = 39.626
    #test_nans(ret, 1633)
    return ret


@cache('run')
def tpefpc_threshold_high_development():
    """
    tpefpc threshold high development



    constant


    """
    
    ret = 106
    #test_nans(ret, 1634)
    return ret


@cache('run')
def tped_acceptable_standard_living():
    """
    tped acceptable standard living



    constant


    """
    
    ret = 40
    #test_nans(ret, 1635)
    return ret


@cache('step')
def pop_not_dependent_on_trad_biomass():
    """
    pop not dependent on trad biomass



    component


    """
    
    ret = population() - population_dependent_on_trad_biomass()
    #test_nans(ret, 1636)
    return ret


@cache('step')
def carbon_footprint_tco2xdivxperson():
    """
    carbon footprint tco2xdivxperson



    component


    """
    
    ret = total_co2_emissions_gtco2() * t_per_gt() / population()
    #test_nans(ret, 1637)
    return ret


@cache('step')
def average_tpespc_xwithout_trad_biomassx():
    """
    average tpespc xwithout trad biomassx



    component


    """
    
    ret = tpes_xwithout_trad_biomassx() * gj_per_ej() / pop_not_dependent_on_trad_biomass()
    #test_nans(ret, 1638)
    return ret


@cache('step')
def tpes_xwithout_trad_biomassx():
    """
    tpes xwithout trad biomassx



    component


    """
    
    ret = tpes_ej() - pe_traditional_biomass_ej_delayed_1yr()
    #test_nans(ret, 1639)
    return ret


@cache('run')
def max_npp_potential_bioe_residues_for_heat_and_elec():
    """
    max npp potential bioe residues for heat and elec



    component


    """
    
    ret = max_npp_potential_bioe_residues() * (1 - share_cellulosic_biofuels_vs_bioe_residues())
    #test_nans(ret, 1640)
    return ret


@cache('run')
def max_npp_potential_bioe_residues_for_cellulosic_biofuels():
    """
    max npp potential bioe residues for cellulosic biofuels



    component


    """
    
    ret = max_npp_potential_bioe_residues() * share_cellulosic_biofuels_vs_bioe_residues()
    #test_nans(ret, 1641)
    return ret


@cache('step')
def total_cumulative_emissions_gtco2():
    """
    total cumulative emissions gtco2



    component


    """
    
    ret = total_cumulative_emissions_gtc() / c_per_co2()
    #test_nans(ret, 1642)
    return ret


@cache('step')
def carbon_emissions_gtc():
    """
    carbon emissions gtc



    component


    """
    
    ret = total_co2_emissions_gtco2() * c_per_co2()
    #test_nans(ret, 1643)
    return ret


@cache('run')
def gco2_per_mj_unconv_gas():
    """
    gco2 per mj unconv gas



    constant


    """
    
    ret = 56.1
    #test_nans(ret, 1644)
    return ret


@cache('run')
def gco2_per_mj_unconv_oil():
    """
    gco2 per mj unconv oil



    constant


    """
    
    ret = 91.4
    #test_nans(ret, 1645)
    return ret


@cache('step')
def co2_emissions_ctl():
    """
    co2 emissions ctl



    component


    """
    
    ret = gco2_per_mj_ctl() * extraction_coal_for_ctl_ej() * mj_per_ej() / g_per_gt()
    #test_nans(ret, 1646)
    return ret


@cache('run')
def gco2_per_mj_conv_gas():
    """
    gco2 per mj conv gas



    constant


    """
    
    ret = 56.1
    #test_nans(ret, 1647)
    return ret


@cache('run')
def gco2_per_mj_ctl():
    """
    gco2 per mj ctl



    constant


    """
    
    ret = 165.2
    #test_nans(ret, 1648)
    return ret


@cache('run')
def gco2_per_mj_gtl():
    """
    gco2 per mj gtl



    constant


    """
    
    ret = 103.3
    #test_nans(ret, 1649)
    return ret


@cache('run')
def gco2_per_mj_coal():
    """
    gco2 per mj coal



    constant


    """
    
    ret = 94.6
    #test_nans(ret, 1650)
    return ret


@cache('run')
def gco2_per_mj_conv_oil():
    """
    gco2 per mj conv oil



    constant


    """
    
    ret = 73.3
    #test_nans(ret, 1651)
    return ret


@cache('run')
def gco2_per_mj_shale_oil():
    """
    gco2 per mj shale oil



    constant


    """
    
    ret = 146.1
    #test_nans(ret, 1652)
    return ret


@cache('run')
def dollars_to_tdollars():
    """
    dollars to tdollars



    constant


    """
    
    ret = 1000000000000
    #test_nans(ret, 1653)
    return ret


@cache('step')
def land_compet_2gen_vs_total_land_compet():
    """
    land compet 2gen vs total land compet



    component


    """
    
    ret = land_compet_biofuels_2gen_mha() / land_compet_required_dedicated_crops_for_biofuels()
    #test_nans(ret, 1654)
    return ret


@cache('run')
def gj_per_ej():
    """
    gj per ej



    constant


    """
    
    ret = 1000000000
    #test_nans(ret, 1655)
    return ret


@cache('step')
def land_shifted_to_biofuels_3gen():
    """
    land shifted to biofuels 3gen



    component


    """
    
    ret = functions.if_then_else(
        time() < start_year_3gen(), 0,
        functions.if_then_else(
            time() < (start_year_3gen() + 5),
            annual_shift_from_2gen_to_3gen() * land_compet_biofuels_2gen_mha() *
            biofuels_3gen_land_compet_available() * land_compet_2gen_vs_total_land_compet(),
            p_biofuels_3gen() * land_compet_biofuels_3gen_mha() *
            biofuels_3gen_land_compet_available() * land_compet_2gen_vs_total_land_compet()))
    #test_nans(ret, 1656)
    return ret


@cache('step')
def average_tpes_per_capita():
    """
    average tpes per capita



    component


    """
    
    ret = tpes_ej() * gj_per_ej() / population()
    #test_nans(ret, 1657)
    return ret


@cache('run')
def geot_pe_potential_for_heat_twth():
    """
    geot pe potential for heat twth



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 1658)
    return ret


@cache('step')
def pe_losses_uranium_for_elec_ej():
    """
    pe losses uranium for elec ej



    component


    """
    
    ret = extraction_uranium_ej() * (1 - efficiency_uranium_for_electricity())
    #test_nans(ret, 1659)
    return ret


@cache('step')
def share_res_for_elec_vs_tpe_res():
    """
    share res for elec vs tpe res



    component


    """
    
    ret = pe_elec_generation_from_res_ej() / (
        pe_supply_res_nonxdashxelec_ej() + pe_elec_generation_from_res_ej())
    #test_nans(ret, 1660)
    return ret


@cache('step')
def extraction_coal_mtoe():
    """
    extraction coal mtoe



    component


    """
    
    ret = extraction_coal_ej() * mtoe_per_ej()
    #test_nans(ret, 1661)
    return ret


@cache('step')
def max_extraction_coal_mtoe():
    """
    max extraction coal mtoe



    component


    """
    
    ret = max_extraction_coal_ej() * mtoe_per_ej()
    #test_nans(ret, 1662)
    return ret


@cache('step')
def tpe_from_res_ej():
    """
    tpe from res ej



    component


    """
    
    ret = pe_elec_generation_from_res_ej() + pe_supply_res_nonxdashxelec_ej()
    #test_nans(ret, 1663)
    return ret


@cache('step')
def tpe_from_res_mtoe():
    """
    tpe from res mtoe



    component


    """
    
    ret = tpe_from_res_ej() * mtoe_per_ej()
    #test_nans(ret, 1664)
    return ret


@cache('step')
def electrical_distribution_losses_ej():
    """
    electrical distribution losses ej



    component


    """
    
    ret = electrical_distribution_losses_twh() * ej_per_twh()
    #test_nans(ret, 1665)
    return ret


@cache('run')
def efficiency_conversion_bioe_plants_to_heat():
    """
    efficiency conversion bioe plants to heat



    constant


    """
    
    ret = 0.55
    #test_nans(ret, 1666)
    return ret


@cache('run')
def efficiency_improvement_biofuels_3gen():
    """
    efficiency improvement biofuels 3gen



    constant


    """
    
    ret = 0.15
    #test_nans(ret, 1667)
    return ret


@cache('step')
def potential_peavail_biofuels_2gen_land_compet_ej():
    """
    potential peavail biofuels 2gen land compet ej



    component


    """
    
    ret = land_compet_biofuels_2gen_mha() * land_productivity_biofuels_2gen_ej_mha()
    #test_nans(ret, 1668)
    return ret


@cache('step')
def annual_additional_historic_land_use_biofuels_2gen():
    """
    annual additional historic land use biofuels 2gen



    component


    """
    
    ret = annual_additional_historic_product_biofuels_2gen() * ej_per_ktoe(
    ) / land_productivity_biofuels_2gen_ej_mha()
    #test_nans(ret, 1669)
    return ret


@cache('run')
def max_npp_potential_bioe_residues():
    """
    max npp potential bioe residues



    constant


    """
    
    ret = 25
    #test_nans(ret, 1670)
    return ret


@cache('run')
def grid_reinforcement_costs():
    """
    grid reinforcement costs



    constant


    """
    
    ret = 238.32929015926584
    #test_nans(ret, 1671)
    return ret


@cache('run')
def historic_land_compet_available_for_biofuels_2gen():
    """
    historic land compet available for biofuels 2gen



    component


    """
    
    ret = 74847.7 * ej_per_ktoe() / land_productivity_biofuels_2gen_ej_mha()
    #test_nans(ret, 1672)
    return ret


@cache('run')
def land_productivity_biofuels_2gen_ej_mha():
    """
    land productivity biofuels 2gen ej mha



    constant


    """
    
    ret = 0.049
    #test_nans(ret, 1673)
    return ret


@cache('step')
def max_peavail_potential_biofuels_x2xdashx3gen():
    """
    max peavail potential biofuels x2xdashx3gen



    component


    """
    
    ret = functions.if_then_else(
        time() < start_year_3gen(),
        max_land_compet_biofuels_2gen() * land_productivity_biofuels_2gen_ej_mha(),
        max_land_compet_biofuels_2gen() * land_productivity_biofuels_2gen_ej_mha() *
        (1 + efficiency_improvement_biofuels_3gen()))
    #test_nans(ret, 1674)
    return ret


@cache('step')
def land_compet_biofuels_2gen_mha():
    """
    land compet biofuels 2gen mha



    component


    """
    
    ret = integ_land_compet_biofuels_2gen_mha()
    #test_nans(ret, 1675)
    return ret


@cache('run')
def efficiency_conversion_bioe_to_elec():
    """
    efficiency conversion bioe to elec



    constant


    """
    
    ret = 0.35
    #test_nans(ret, 1676)
    return ret


@cache('step')
def potential_peavail_biofuels_prod_3gen_ej():
    """
    potential peavail biofuels prod 3gen ej



    component


    """
    
    ret = land_compet_biofuels_3gen_mha() * land_productivity_biofuels_2gen_ej_mha() * (
        1 + efficiency_improvement_biofuels_3gen())
    #test_nans(ret, 1677)
    return ret


@cache('run')
def max_peavail_potential_bioe_residues_for_cellulosic_biofuels():
    """
    max peavail potential bioe residues for cellulosic biofuels



    component


    """
    
    ret = max_npp_potential_bioe_residues_for_cellulosic_biofuels(
    ) * efficiency_bioe_residues_to_cellulosic_liquids()
    #test_nans(ret, 1678)
    return ret


@cache('run')
def p_biofuels_3gen():
    """
    p biofuels 3gen



    constant


    """
    
    ret = 0.05
    #test_nans(ret, 1679)
    return ret


@cache('step')
def constrain_gas_exogenous_growthxquestx_delayed_x1yr():
    """
    constrain gas exogenous growthxquestx delayed x1yr



    component


    """
    
    ret = delay_constrain_gas_exogenous_growthxquestx_round1__time_step___time_step_1_1__time_step(
    )
    #test_nans(ret, 1680)
    return ret


@cache('step')
def constrain_gas_exogenous_growthxquestx():
    """
    constrain gas exogenous growthxquestx



    component


    """
    
    ret = functions.if_then_else(check_gases() > - equality_threshold, 1, check_gases())
    #test_nans(ret, 1681)
    return ret


@cache('step')
def constrain_liquids_exogenous_growthxquestx_delayed_x1yr():
    """
    constrain liquids exogenous growthxquestx delayed x1yr



    component


    """
    
    ret = delay_constrain_liquids_exogenous_growthxquestx_round1__time_step___time_step_1_1__time_step(
    )
    #test_nans(ret, 1682)
    return ret


@cache('step')
def check_liquids_delayed_1yr():
    """
    check liquids delayed 1yr



    component


    """
    
    ret = delay_check_liquids_round1__time_step___time_step_1_1__time_step()
    #test_nans(ret, 1683)
    return ret


@cache('step')
def check_gas_delayed_1yr():
    """
    check gas delayed 1yr



    component


    """
    
    ret = delay_check_gases_round1__time_step___time_step_1_1__time_step()
    #test_nans(ret, 1684)
    return ret


@cache('step')
def constrain_liquids_exogenous_growthxquestx():
    """
    constrain liquids exogenous growthxquestx



    component


    """
    
    ret = functions.if_then_else(check_liquids() > equality_threshold, 1, check_liquids())
    #test_nans(ret, 1685)
    return ret


@cache('step')
def share_variable_res_elec_vs_total_generation_delayed_1yr():
    """
    share variable res elec vs total generation delayed 1yr



    component


    """
    
    ret = delay_share_variable_res_elec_generation_vs_total_round1__time_step___time_step_00071_1__time_step(
    )
    #test_nans(ret, 1686)
    return ret


@cache('step')
def overcapacity_vsx_intermittent_res_penetration_x0():
    """
    overcapacity vsx intermittent res penetration x0



    component


    """
    
    ret = np.maximum(
        1, 0.9599 * np.exp(0.8938 * share_variable_res_elec_vs_total_generation_delayed_1yr()))
    #test_nans(ret, 1687)
    return ret


@cache('step')
def total_electrical_losses_ej():
    """
    total electrical losses ej



    component


    """
    
    ret = elec_gen_related_losses_ej() + electrical_distribution_losses_ej()
    #test_nans(ret, 1688)
    return ret


@cache('step')
def gen_losses_vs_pe_for_elec():
    """
    gen losses vs pe for elec



    component


    """
    
    ret = elec_gen_related_losses_ej() / total_pe_for_electricity_consumption_ej()
    #test_nans(ret, 1689)
    return ret


@cache('run')
def initial_gtl_production():
    """
    initial gtl production



    constant


    """
    
    ret = 0.064206
    #test_nans(ret, 1690)
    return ret


@cache('run')
def initial_ctl_production():
    """
    initial ctl production



    constant


    """
    
    ret = 0.26293196
    #test_nans(ret, 1691)
    return ret


@cache('step')
def pe_demand_gas_elec_plants_ej():
    """
    pe demand gas elec plants ej



    component


    """
    
    ret = (fe_demand_gas_elec_plants_twh() / efficiency_gas_for_electricity()) * ej_per_twh()
    #test_nans(ret, 1692)
    return ret


@cache('step')
def pe_demand_coal_elec_plants_ej():
    """
    pe demand coal elec plants ej



    component


    """
    
    ret = (fe_demand_coal_elec_plants_twh() / efficiency_coal_for_electricity()) * ej_per_twh()
    #test_nans(ret, 1693)
    return ret


@cache('run')
def max_efficiency_gas_power_plants():
    """
    max efficiency gas power plants



    constant


    """
    
    ret = 0.6
    #test_nans(ret, 1694)
    return ret


@cache('step')
def total_pe_for_electricity_consumption_ej():
    """
    total pe for electricity consumption ej



    component


    """
    
    ret = total_fe_elec_demand_ej() + elec_gen_related_losses_ej()
    #test_nans(ret, 1695)
    return ret


@cache('run')
def initial_efficiency_gas_for_electricity():
    """
    initial efficiency gas for electricity



    constant


    """
    
    ret = 37.813
    #test_nans(ret, 1696)
    return ret


@cache('step')
def remaining_efficiency_improv_gas_for_electricity():
    """
    remaining efficiency improv gas for electricity



    component


    """
    
    ret = (max_efficiency_gas_power_plants() - efficiency_gas_for_electricity()
            ) / max_efficiency_gas_power_plants()
    #test_nans(ret, 1697)
    return ret


@cache('run')
def efficiency_improv_gas_for_electricity():
    """
    efficiency improv gas for electricity



    constant


    """
    
    ret = 0.05
    #test_nans(ret, 1698)
    return ret


@cache('step')
def efficiency_gas_for_electricity():
    """
    efficiency gas for electricity



    component


    """
    
    ret = integ_efficiency_gas_for_electricity()
    #test_nans(ret, 1699)
    return ret


@cache('run')
def percent_to_share():
    """
    percent to share



    constant


    """
    
    ret = 0.01
    #test_nans(ret, 1700)
    return ret


@cache('step')
def additional_pe_production_of_ctlxplusxgtl_for_liquids():
    """
    additional pe production of ctlxplusxgtl for liquids



    component


    """
    
    ret = ped_coal_for_ctl_ej() + ped_natx_gas_for_gtl_ej() - fes_ctlxplusxgtl_ej()
    #test_nans(ret, 1701)
    return ret


@cache('run')
def carbon_budget():
    """
    carbon budget



    constant


    """
    
    ret = 1000
    #test_nans(ret, 1702)
    return ret


@cache('run')
def cumulative_emissions_to_x1995x():
    """
    cumulative emissions to x1995x



    constant


    """
    
    ret = 378.66
    #test_nans(ret, 1703)
    return ret


@cache('run')
def fe_solar_potential_for_heat():
    """
    fe solar potential for heat



    constant


    """
    
    ret = 22
    #test_nans(ret, 1704)
    return ret


@cache('run')
def start_year_bioe_residues_for_heatxplusxelec():
    """
    start year bioe residues for heatxplusxelec



    constant


    """
    
    ret = 2020
    #test_nans(ret, 1705)
    return ret


@cache('step')
def co2_fossil_fuel_emissions():
    """
    co2 fossil fuel emissions



    component


    """
    
    ret = co2_emissions_conv_gas_without_gtl() + co2_emissions_unconv_gas() + co2_emissions_gtl(
    ) + co2_emissions_conv_oil() + co2_emissions_unconv_oil() + co2_emissions_coal_without_ctl(
    ) + co2_emissions_ctl()
    #test_nans(ret, 1706)
    return ret


@cache('run')
def t_per_gt():
    """
    t per gt



    constant


    """
    
    ret = 1000000000
    #test_nans(ret, 1707)
    return ret


@cache('step')
def share_land_compet_biofuels():
    """
    share land compet biofuels



    component


    """
    
    ret = land_compet_required_dedicated_crops_for_biofuels() / global_arable_land()
    #test_nans(ret, 1708)
    return ret


@cache('step')
def share_land_total_res_vs_arable():
    """
    share land total res vs arable



    component


    """
    
    ret = total_land_requirements_renew_mha() / global_arable_land()
    #test_nans(ret, 1709)
    return ret


@cache('step')
def share_land_res_land_compet_vs_arable():
    """
    share land res land compet vs arable



    component


    """
    
    ret = (land_compet_required_dedicated_crops_for_biofuels() + surface_solar_pv_mha()
            ) / global_arable_land()
    #test_nans(ret, 1710)
    return ret


@cache('step')
def year_scarcity_oil():
    """
    year scarcity oil



    component


    """
    
    ret = functions.if_then_else(abundance_total_oil() > 0.95, 0, time())
    #test_nans(ret, 1711)
    return ret


@cache('run')
def future_share_gasxdivxxcoalxplusxgasx_for_elec():
    """
    future share gasxdivxxcoalxplusxgasx for elec



    constant


    """
    
    ret = 0.3
    #test_nans(ret, 1712)
    return ret


@cache('step')
def cumulated_conv_gas_extraction():
    """
    cumulated conv gas extraction



    component


    """
    
    ret = integ_cumulated_conv_gas_extraction()
    #test_nans(ret, 1713)
    return ret


@cache('run')
def cumulated_conv_gas_extraction_to_x1995x():
    """
    cumulated conv gas extraction to x1995x



    constant


    """
    
    ret = 2001.4020668330372
    #test_nans(ret, 1714)
    return ret


@cache('step')
def cumulated_unconv_gas_extraction():
    """
    cumulated unconv gas extraction



    component


    """
    
    ret = integ_cumulated_unconv_gas_extraction()
    #test_nans(ret, 1715)
    return ret


@cache('run')
def cumulated_unconv_gas_extraction_to_x1995x():
    """
    cumulated unconv gas extraction to x1995x



    constant


    """
    
    ret = 6.6975559326811815
    #test_nans(ret, 1716)
    return ret


@cache('step')
def rurr_uranium():
    """
    rurr uranium



    component


    """
    
    ret = integ_rurr_uranium()
    #test_nans(ret, 1717)
    return ret


@cache('run')
def cumulated_uranium_extraction_to_x1995x():
    """
    cumulated uranium extraction to x1995x



    constant


    """
    
    ret = 814
    #test_nans(ret, 1718)
    return ret


@cache('step')
def cumulated_conv_oil_extraction():
    """
    cumulated conv oil extraction



    component


    """
    
    ret = integ_cumulated_conv_oil_extraction()
    #test_nans(ret, 1719)
    return ret


@cache('run')
def cumulated_conv_oil_extraction_to_x1995x():
    """
    cumulated conv oil extraction to x1995x



    constant


    """
    
    ret = 4636
    #test_nans(ret, 1720)
    return ret


@cache('step')
def cumulated_unconv_oil_extraction():
    """
    cumulated unconv oil extraction



    component


    """
    
    ret = integ_cumulated_unconv_oil_extraction()
    #test_nans(ret, 1721)
    return ret


@cache('step')
def cumulated_coal_extraction():
    """
    cumulated coal extraction



    component


    """
    
    ret = integ_cumulated_coal_extraction()
    #test_nans(ret, 1722)
    return ret


@cache('run')
def cumulated_coal_extraction_to_x1995x():
    """
    cumulated coal extraction to x1995x



    constant


    """
    
    ret = 5018
    #test_nans(ret, 1723)
    return ret


@cache('run')
def cumulated_unconv_oil_extraction_to_x1995x():
    """
    cumulated unconv oil extraction to x1995x



    constant


    """
    
    ret = 13.5
    #test_nans(ret, 1724)
    return ret


@cache('step')
def cumulated_uranium_extraction():
    """
    cumulated uranium extraction



    component


    """
    
    ret = integ_cumulated_uranium_extraction()
    #test_nans(ret, 1725)
    return ret


@cache('step')
def oil_refinery_gains_ej():
    """
    oil refinery gains ej



    component


    """
    
    ret = oil_refinery_gains_share() * pes_oil_ej_delayed()
    #test_nans(ret, 1726)
    return ret


@cache('run')
def oil_refinery_gains_share():
    """
    oil refinery gains share



    constant


    """
    
    ret = 0.027
    #test_nans(ret, 1727)
    return ret


@cache('run')
def efficiency_gas_for_oil_refinery_gains():
    """
    efficiency gas for oil refinery gains



    constant


    """
    
    ret = 1
    #test_nans(ret, 1728)
    return ret


@cache('step')
def pes_liquids_ej():
    """
    pes liquids ej



    component


    """
    
    ret = pes_oil_ej() + other_liquids_supply_ej()
    #test_nans(ret, 1729)
    return ret


@cache('run')
def one_year():
    """
    one year



    constant


    """
    
    ret = 1
    #test_nans(ret, 1730)
    return ret


@cache('step')
def land_compet_required_dedicated_crops_for_biofuels():
    """
    land compet required dedicated crops for biofuels



    component


    """
    
    ret = land_compet_biofuels_2gen_mha() + land_compet_biofuels_3gen_mha()
    #test_nans(ret, 1731)
    return ret


@cache('step')
def new_biofuels_2gen_land_compet():
    """
    new biofuels 2gen land compet



    component


    """
    
    ret = functions.if_then_else(
        check_liquids() < - equality_threshold,
        constrain_liquids_exogenous_growthxquestx() * land_compet_biofuels_2gen_mha(),
        np.maximum(
            annual_additional_historic_land_use_biofuels_2gen() + adapt_growth_biofuels_2gen() *
            land_compet_biofuels_2gen_mha() * biofuels_land_compet_available(), 0))
    #test_nans(ret, 1732)
    return ret


@cache('run')
def annual_shift_from_2gen_to_3gen():
    """
    annual shift from 2gen to 3gen



    constant


    """
    
    ret = 0.1
    #test_nans(ret, 1733)
    return ret


@cache('step')
def land_compet_biofuels_3gen_mha():
    """
    land compet biofuels 3gen mha



    component


    """
    
    ret = integ_land_compet_biofuels_3gen_mha()
    #test_nans(ret, 1734)
    return ret


@cache('run')
def p_bioe_residues_for_heatxplusxelec():
    """
    p bioe residues for heatxplusxelec



    constant


    """
    
    ret = 0.05
    #test_nans(ret, 1735)
    return ret


@cache('step')
def potential_pe_cellulosic_biofuel_ej():
    """
    potential pe cellulosic biofuel ej



    component


    """
    
    ret = integ_potential_pe_cellulosic_biofuel_ej()
    #test_nans(ret, 1736)
    return ret


@cache('step')
def cellulosic_biofuels_available():
    """
    cellulosic biofuels available



    component


    """
    
    ret = functions.zidz((max_npp_potential_bioe_residues_for_cellulosic_biofuels() -
                           potential_pe_cellulosic_biofuel_ej()),
                          max_npp_potential_bioe_residues_for_cellulosic_biofuels())
    #test_nans(ret, 1737)
    return ret


@cache('step')
def bioe_residues_for_heatxplusxelec_available():
    """
    bioe residues for heatxplusxelec available



    component


    """
    
    ret = (max_npp_potential_bioe_residues_for_heat_and_elec() -
            pe_bioe_residues_for_heatxplusxelec_ej()
            ) / max_npp_potential_bioe_residues_for_heat_and_elec()
    #test_nans(ret, 1738)
    return ret


@cache('step')
def pe_bioe_residues_for_heatxplusxelec_ej():
    """
    pe bioe residues for heatxplusxelec ej



    component


    """
    
    ret = integ_pe_bioe_residues_for_heatxplusxelec_ej()
    #test_nans(ret, 1739)
    return ret


@cache('run')
def p_cellulosic_biofuels():
    """
    p cellulosic biofuels



    constant


    """
    
    ret = 0.08
    #test_nans(ret, 1740)
    return ret


@cache('run')
def share_cellulosic_biofuels_vs_bioe_residues():
    """
    share cellulosic biofuels vs bioe residues



    constant


    """
    
    ret = 0.25
    #test_nans(ret, 1741)
    return ret


@cache('run')
def start_year_cellulosic_biofuels():
    """
    start year cellulosic biofuels



    constant


    """
    
    ret = 2025
    #test_nans(ret, 1742)
    return ret


@cache('step')
def share_land_total_res_vs_urban_surface():
    """
    share land total res vs urban surface



    component


    """
    
    ret = total_land_requirements_renew_mha() / urban_surface_x2008x()
    #test_nans(ret, 1743)
    return ret


@cache('run')
def max_land_compet_biofuels_2gen():
    """
    max land compet biofuels 2gen



    component


    """
    
    ret = additional_land_compet_available_for_biofuels(
    ) + historic_land_compet_available_for_biofuels_2gen()
    #test_nans(ret, 1744)
    return ret


@cache('run')
def p_biofuels_2gen():
    """
    p biofuels 2gen



    constant


    """
    
    ret = 0.05
    #test_nans(ret, 1745)
    return ret


@cache('run')
def past_biofuels_2gen():
    """
    past biofuels 2gen



    constant


    """
    
    ret = 0.11139790590538201
    #test_nans(ret, 1746)
    return ret


@cache('run')
def ej_per_ktoe():
    """
    ej per ktoe



    constant


    """
    
    ret = 4.1868e-05
    #test_nans(ret, 1747)
    return ret


@cache('step')
def abundance_total_oil():
    """
    abundance total oil



    component


    """
    
    ret = functions.if_then_else(ped_total_oil_ej() < pes_oil_ej(), 1, 1 - functions.zidz(
        (ped_total_oil_ej() - pes_oil_ej()), ped_total_oil_ej()))
    #test_nans(ret, 1748)
    return ret


@cache('run')
def cp_nuclear_initial():
    """
    cp nuclear initial



    constant


    """
    
    ret = 0.7652
    #test_nans(ret, 1749)
    return ret


@cache('step')
def extraction_coal_without_ctl_ej():
    """
    extraction coal without ctl ej



    component


    """
    
    ret = np.maximum(extraction_coal_ej() - extraction_coal_for_ctl_ej(), 0)
    #test_nans(ret, 1750)
    return ret


@cache('step')
def extraction_coal_for_ctl_ej():
    """
    extraction coal for ctl ej



    component


    """
    
    ret = ped_coal_for_ctl_ej()
    #test_nans(ret, 1751)
    return ret


@cache('step')
def other_liquids_supply_ej():
    """
    other liquids supply ej



    component


    """
    
    ret = oil_refinery_gains_ej() + fes_ctlxplusxgtl_ej() + fes_total_biofuels_production_ej()
    #test_nans(ret, 1752)
    return ret


@cache('run')
def max_offshore_wind_twe():
    """
    max offshore wind twe



    constant


    """
    
    ret = 0.25
    #test_nans(ret, 1753)
    return ret


@cache('run')
def urr_unconv_oil_high_mohr15():
    """
    urr unconv oil high mohr15



    constant


    """
    
    ret = 22000
    #test_nans(ret, 1754)
    return ret


@cache('run')
def urr_unconv_oil_low_mohr15():
    """
    urr unconv oil low mohr15



    constant


    """
    
    ret = 6000
    #test_nans(ret, 1755)
    return ret


@cache('run')
def urr_unconv_oil_user_defined():
    """
    urr unconv oil user defined



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1756)
    return ret


@cache('run')
def urr_uranium_zittel12():
    """
    urr uranium zittel12



    constant


    """
    
    ret = 3700
    #test_nans(ret, 1757)
    return ret


@cache('run')
def urr_conv_oil_maggio12_high():
    """
    urr conv oil maggio12 high



    constant


    """
    
    ret = 16700
    #test_nans(ret, 1758)
    return ret


@cache('run')
def urr_conv_oil_maggio12_low():
    """
    urr conv oil maggio12 low



    constant


    """
    
    ret = 12500
    #test_nans(ret, 1759)
    return ret


@cache('run')
def separate_conv_and_unconv_gasxquestx():
    """
    separate conv and unconv gasxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1760)
    return ret


@cache('run')
def urr_tot_agg_oil_laherrere_x2006x():
    """
    urr tot agg oil laherrere x2006x



    constant


    """
    
    ret = 16800
    #test_nans(ret, 1761)
    return ret


@cache('run')
def urr_tot_agg_oil_user_defined():
    """
    urr tot agg oil user defined



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1762)
    return ret


@cache('run')
def choose_extraction_curve_tot_agg_oil():
    """
    choose extraction curve tot agg oil



    constant


    """
    
    ret = 1
    #test_nans(ret, 1763)
    return ret


@cache('run')
def urr_conv_oil_maggio12_middle():
    """
    urr conv oil maggio12 middle



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1764)
    return ret


@cache('run')
def urr_conv_oil_user_defined():
    """
    urr conv oil user defined



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1765)
    return ret


@cache('run')
def choose_extraction_tot_agg_gas_curve():
    """
    choose extraction tot agg gas curve



    constant


    """
    
    ret = 1
    #test_nans(ret, 1766)
    return ret


@cache('run')
def urr_unconv_oil_bg_mohr15():
    """
    urr unconv oil bg mohr15



    constant


    """
    
    ret = 10500
    #test_nans(ret, 1767)
    return ret


@cache('run')
def separate_conv_and_unconv_oilxquestx():
    """
    separate conv and unconv oilxquestx



    constant


    """
    
    ret = 0
    #test_nans(ret, 1768)
    return ret


@cache('run')
def urr_uranium_ewg13():
    """
    urr uranium ewg13



    constant


    """
    
    ret = 4000
    #test_nans(ret, 1769)
    return ret


@cache('run')
def kt_uranium_per_ej():
    """
    kt uranium per ej



    constant


    """
    
    ret = 2.386634845
    #test_nans(ret, 1770)
    return ret


@cache('run')
def life_time_nuclear():
    """
    life time nuclear



    constant


    """
    
    ret = 40
    #test_nans(ret, 1771)
    return ret


@cache('step')
def surface_solar_pv_mha():
    """
    surface solar pv mha



    component


    """
    
    ret = surface_res_elec().loc['solar pv'].values
    #test_nans(ret, 1772)
    return ret


@cache('step')
def cumulated_uranium_extraction_kt():
    """
    cumulated uranium extraction kt



    component


    """
    
    ret = cumulated_uranium_extraction() * kt_uranium_per_ej()
    #test_nans(ret, 1773)
    return ret


@cache('step')
def extraction_uranium_kt():
    """
    extraction uranium kt



    component


    """
    
    ret = extraction_uranium_ej() * kt_uranium_per_ej()
    #test_nans(ret, 1774)
    return ret


@cache('run')
def efficiency_uranium_for_electricity():
    """
    efficiency uranium for electricity



    constant


    """
    
    ret = 0.3333333333
    #test_nans(ret, 1775)
    return ret


@cache('step')
def fe_nuclear_elec_generation_twh():
    """
    fe nuclear elec generation twh



    component


    """
    
    ret = extraction_uranium_ej() * efficiency_uranium_for_electricity() / ej_per_twh()
    #test_nans(ret, 1776)
    return ret


@cache('run')
def kwh_per_twh():
    """
    kwh per twh



    constant


    """
    
    ret = 1000000000
    #test_nans(ret, 1777)
    return ret


@cache('step')
def total_fe_elec_demand_ej():
    """
    total fe elec demand ej



    component


    """
    
    ret = total_fe_elec_demand_twh() * ej_per_twh()
    #test_nans(ret, 1778)
    return ret


@cache('run')
def gco2e_per_kwh_biomass():
    """
    gco2e per kwh biomass



    constant


    """
    
    ret = 0
    #test_nans(ret, 1779)
    return ret


@cache('run')
def mtoe_per_ej():
    """
    mtoe per ej



    constant


    """
    
    ret = 23.88459
    #test_nans(ret, 1780)
    return ret


@cache('run')
def share_transmxandxdistr_elec_losses_initial():
    """
    share transmxandxdistr elec losses initial



    constant


    """
    
    ret = 0.095
    #test_nans(ret, 1781)
    return ret


@cache('run')
def ej_per_twh():
    """
    ej per twh



    constant


    """
    
    ret = 0.0036
    #test_nans(ret, 1782)
    return ret


@cache('step')
def surface_onshore_wind_mha():
    """
    surface onshore wind mha



    component


    """
    
    ret = surface_res_elec().loc['wind onshore'].values
    #test_nans(ret, 1783)
    return ret


@cache('run')
def twe_per_twh():
    """
    twe per twh



    constant


    """
    
    ret = 0.0001141552511
    #test_nans(ret, 1784)
    return ret


@cache('run')
def initial_population():
    """
    initial population



    constant


    """
    
    ret = 5709757338.0
    #test_nans(ret, 1785)
    return ret


@cache('step')
def population():
    """
    population



    component


    """
    
    ret = integ_population()
    #test_nans(ret, 1786)
    return ret


@cache('run')
def ve_objetive_ue2020_extrap():
    """
    ve objetive ue2020 extrap



    constant


    """
    
    ret = 1
    #test_nans(ret, 1787)
    return ret


@cache('run')
def ve_objetive_ue2020():
    """
    ve objetive ue2020



    constant


    """
    
    ret = 1
    #test_nans(ret, 1788)
    return ret


@cache('run')
def choose_extraction_uranium_curve():
    """
    choose extraction uranium curve



    constant


    """
    
    ret = 2
    #test_nans(ret, 1789)
    return ret


@cache('step')
def year_scarcity_tpe():
    """
    year scarcity tpe



    component


    """
    
    ret = functions.if_then_else(abundance_tpe() > 0.95, 0, time())
    #test_nans(ret, 1790)
    return ret


@cache('step')
def year_scarcity_uranium():
    """
    year scarcity uranium



    component


    """
    
    ret = functions.if_then_else(abundance_uranium() > 0.95, 0, time())
    #test_nans(ret, 1791)
    return ret


@cache('step')
def year_scarcity_coal():
    """
    year scarcity coal



    component


    """
    
    ret = functions.if_then_else(abundance_coal() > 0.95, 0, time())
    #test_nans(ret, 1792)
    return ret


@cache('step')
def year_scarcity_liquids():
    """
    year scarcity liquids



    component


    """
    
    ret = functions.if_then_else(abundance_liquids() > 0.95, 0, time())
    #test_nans(ret, 1793)
    return ret


@cache('step')
def effects_shortage_gas():
    """
    effects shortage gas



    component


    """
    
    ret = functions.if_then_else(abundance_gases() > 0.8, ((abundance_gases() - 0.8) * 5)**2, 0)
    #test_nans(ret, 1794)
    return ret


@cache('step')
def year_scarcity_gases():
    """
    year scarcity gases



    component


    """
    
    ret = functions.if_then_else(abundance_gases() > 0.95, 0, time())
    #test_nans(ret, 1795)
    return ret


@cache('run')
def max_percent_of_change():
    """
    max percent of change



    constant


    """
    
    ret = 0.448
    #test_nans(ret, 1796)
    return ret


@cache('run')
def choose_extraction_coal_curve():
    """
    choose extraction coal curve



    constant


    """
    
    ret = 3
    #test_nans(ret, 1797)
    return ret


@cache('run')
def selection_constraint_extraction_unconv_gas():
    """
    selection constraint extraction unconv gas



    constant


    """
    
    ret = 1
    #test_nans(ret, 1798)
    return ret


@cache('step')
def year_scarcity_elec():
    """
    year scarcity elec



    component


    """
    
    ret = functions.if_then_else(abundance_electricity() > 0.95, 0, time())
    #test_nans(ret, 1799)
    return ret


@cache('step')
def extraction_unconv_gas_delayed():
    """
    extraction unconv gas delayed



    component


    """
    
    ret = delay_extraction_unconv_gas_ej_roundtime_step__time_step___time_step_0_time_step__time_step(
    )
    #test_nans(ret, 1800)
    return ret


@cache('step')
def max_unconv_gas_growth_extraction_ej():
    """
    max unconv gas growth extraction ej



    component


    """
    
    ret = functions.if_then_else(
        check_gas_delayed_1yr() < - equality_threshold, (1 + constrain_gas_exogenous_growthxquestx_delayed_x1yr())
        * extraction_unconv_gas_delayed(),
        extraction_unconv_gas_delayed() * max_unconv_gas_growth_extraction())
    #test_nans(ret, 1801)
    return ret


@cache('run')
def p_constraint_growth_extraction_unconv_gas():
    """
    p constraint growth extraction unconv gas



    constant


    """
    
    ret = 0.08
    #test_nans(ret, 1802)
    return ret


@cache('run')
def choose_extraction_curve_unconv_gas():
    """
    choose extraction curve unconv gas



    constant


    """
    
    ret = 1
    #test_nans(ret, 1803)
    return ret


@cache('run')
def choose_extraction_curve_unconv_oil():
    """
    choose extraction curve unconv oil



    constant


    """
    
    ret = 1
    #test_nans(ret, 1804)
    return ret


@cache('step')
def biofuels_land_compet_available():
    """
    biofuels land compet available



    component


    """
    
    ret = (max_land_compet_biofuels_2gen() - land_compet_biofuels_2gen_mha() -
            land_compet_biofuels_3gen_mha()) / max_land_compet_biofuels_2gen()
    #test_nans(ret, 1805)
    return ret


@cache('run')
def urr_oil_aspo():
    """
    urr oil aspo



    constant


    """
    
    ret = 12800
    #test_nans(ret, 1806)
    return ret


@cache('run')
def urr_gas_mohr_high2013():
    """
    urr gas mohr high2013



    constant


    """
    
    ret = 28500
    #test_nans(ret, 1807)
    return ret


@cache('run')
def urr_gas_leherrere2010():
    """
    urr gas leherrere2010



    constant


    """
    
    ret = 13600
    #test_nans(ret, 1808)
    return ret


@cache('run')
def urr_gas_mohr_bg2012():
    """
    urr gas mohr bg2012



    constant


    """
    
    ret = 19900
    #test_nans(ret, 1809)
    return ret


@cache('run')
def res_to_fossil_accounting():
    """
    res to fossil accounting



    constant


    """
    
    ret = 1
    #test_nans(ret, 1810)
    return ret


@cache('step')
def new_c_gtc():
    """
    new c gtc



    component


    """
    
    ret = carbon_emissions_gtc()
    #test_nans(ret, 1811)
    return ret


@cache('run')
def year_adjust():
    """
    year adjust



    constant


    """
    
    ret = 1
    #test_nans(ret, 1812)
    return ret


@cache('run')
def urban_surface_x2008x():
    """
    urban surface x2008x



    constant


    """
    
    ret = 300
    #test_nans(ret, 1813)
    return ret


@cache('run')
def c_per_co2():
    """
    c per co2



    constant


    """
    
    ret = 0.272727272
    #test_nans(ret, 1814)
    return ret


@cache('run')
def activate_affores_program():
    """
    activate affores program



    constant


    """
    
    ret = 0
    #test_nans(ret, 1815)
    return ret


@cache('run')
def global_arable_land():
    """
    global arable land



    constant


    """
    
    ret = 1526
    #test_nans(ret, 1816)
    return ret


@cache('run')
def choose_extraction_curve_conv_oil():
    """
    choose extraction curve conv oil



    constant


    """
    
    ret = 1
    #test_nans(ret, 1817)
    return ret


@cache('run')
def choose_extraction_conv_gas_curve():
    """
    choose extraction conv gas curve



    constant


    """
    
    ret = 1
    #test_nans(ret, 1818)
    return ret


@cache('step')
def cumulated_total_monet_invest_res_for_elec():
    """
    cumulated total monet invest res for elec



    component


    """
    
    ret = integ_cumulated_total_monet_invest_res_for_elec()
    #test_nans(ret, 1819)
    return ret


@cache('run')
def efficiency_coal_for_electricity():
    """
    efficiency coal for electricity



    constant


    """
    
    ret = 0.353
    #test_nans(ret, 1820)
    return ret


@cache('run')
def efficiency_liquids_for_electricity():
    """
    efficiency liquids for electricity



    constant


    """
    
    ret = 0.361
    #test_nans(ret, 1821)
    return ret


@cache('run')
def additional_land_compet_available_for_biofuels():
    """
    additional land compet available for biofuels



    constant


    """
    
    ret = 100
    #test_nans(ret, 1822)
    return ret


@cache('run')
def max_hydro_twe():
    """
    max hydro twe



    constant


    """
    
    ret = 1
    #test_nans(ret, 1823)
    return ret


@cache('run')
def max_oceanic_twe():
    """
    max oceanic twe



    constant


    """
    
    ret = 0.05
    #test_nans(ret, 1824)
    return ret


@cache('run')
def max_onshore_wind_twe():
    """
    max onshore wind twe



    constant


    """
    
    ret = 1
    #test_nans(ret, 1825)
    return ret


@cache('run')
def p_ctl():
    """
    p ctl



    constant


    """
    
    ret = 0.15
    #test_nans(ret, 1826)
    return ret


@cache('run')
def p_gtl():
    """
    p gtl



    constant


    """
    
    ret = 0.2
    #test_nans(ret, 1827)
    return ret


@cache('run')
def start_year_3gen():
    """
    start year 3gen



    constant


    """
    
    ret = 2025
    #test_nans(ret, 1828)
    return ret


@cache('step')
def share_variable_res_elec_generation_vs_total():
    """
    share variable res elec generation vs total



    component


    """
    
    ret = functions.if_then_else(
        (fe_elec_generation_from_nre_twh() + elec_generation_variable_from_res_twh() +
         elec_generation_dispatch_from_res_twh()) > 0,
        elec_generation_variable_from_res_twh() /
        (fe_elec_generation_from_nre_twh() + elec_generation_variable_from_res_twh() +
         elec_generation_dispatch_from_res_twh()), 0.5)
    #test_nans(ret, 1829)
    return ret


@cache('step')
def total_cumulative_emissions_gtc():
    """
    total cumulative emissions gtc



    component


    """
    
    ret = integ_total_cumulative_emissions_gtc()
    #test_nans(ret, 1830)
    return ret


@cache('run')
def p_nuclear_scen_x1x():
    """
    p nuclear scen x1x



    constant


    """
    
    ret = 0
    #test_nans(ret, 1831)
    return ret


@cache('run')
def p_nuclear_x2xdashx3():
    """
    p nuclear x2xdashx3



    constant


    """
    
    ret = 1
    #test_nans(ret, 1832)
    return ret


@cache('run')
def gco2e_per_gtco2e_x4x():
    """
    gco2e per gtco2e x4x



    constant


    """
    
    ret = 1 / 1000
    #test_nans(ret, 1833)
    return ret


@cache('run')
def gco2e_per_gtco2e_x3x():
    """
    gco2e per gtco2e x3x



    constant


    """
    
    ret = 1 / 1000
    #test_nans(ret, 1834)
    return ret


@cache('run')
def gco2e_per_gtco2e_x0x():
    """
    gco2e per gtco2e x0x



    constant


    """
    
    ret = 1 / 1000
    #test_nans(ret, 1835)
    return ret


@cache('run')
def gco2e_per_gtco2e_x1x():
    """
    gco2e per gtco2e x1x



    constant


    """
    
    ret = 1 / 1000
    #test_nans(ret, 1836)
    return ret


@cache('run')
def twh_per_gco2e_x0x():
    """
    twh per gco2e x0x



    constant


    """
    
    ret = 0.02
    #test_nans(ret, 1837)
    return ret


@cache('step')
def fed_heatxdashxcom_ej():
    """
    fed heatxdashxcom ej



    component


    """
    
    ret = required_heatxdashxcom()
    #test_nans(ret, 1838)
    return ret


@cache('step')
def evol_final_energy_intensity_by_sector_and_fuel():
    """
    evol final energy intensity by sector and fuel



    component


    """
    
    ret = integ_evol_final_energy_intensity_by_sector_and_fuel()
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 1839)
    return ret


@cache('step')
def total_fe_elec_demand_twh():
    """
    total fe elec demand twh



    component


    """
    
    ret = (fe_demand_elec_consum_twh()) * (1 + share_transmxandxdistr_elec_losses())
    #test_nans(ret, 1840)
    return ret


@cache('step')
def real_demand_by_sector():
    """
    real demand by sector



    component


    """
    
    ret = np.maximum(
        0,
        ia_matrix().loc[:,'agriculture hunting forestry and fishing'].drop('sectors1') * real_total_output_by_sector().loc['agriculture hunting forestry and fishing'].values + ia_matrix().loc[:,'mining and quarrying'].drop('sectors1') * real_total_output_by_sector().loc['mining and quarrying'].values +
        ia_matrix().loc[:,'food beverages and tobacco'].drop('sectors1') * real_total_output_by_sector().loc['food beverages and tobacco'].values + ia_matrix().loc[:,'textiles and textile products'].drop('sectors1') * real_total_output_by_sector().loc['textiles and textile products'].values +
        ia_matrix().loc[:,'leather leather and footwear'].drop('sectors1') * real_total_output_by_sector().loc['leather leather and footwear'].values + ia_matrix().loc[:,'wood and products of woood and cork'].drop('sectors1') * real_total_output_by_sector().loc['wood and products of woood and cork'].values +
        ia_matrix().loc[:,'pulp paper printing and publishing'].drop('sectors1') * real_total_output_by_sector().loc['pulp paper printing and publishing'].values + ia_matrix().loc[:,'coke refined petroleum and nuclear fuel'].drop('sectors1') * real_total_output_by_sector().loc['coke refined petroleum and nuclear fuel'].values +
        ia_matrix().loc[:,'chemicals and chemical products'].drop('sectors1') * real_total_output_by_sector().loc['chemicals and chemical products'].values + ia_matrix().loc[:,'rubber and plastics'].drop('sectors1') * real_total_output_by_sector().loc['rubber and plastics'].values +
        ia_matrix().loc[:,'other non metalic mineral'].drop('sectors1') * real_total_output_by_sector().loc['other non metalic mineral'].values + ia_matrix().loc[:,'basic metals and fabricated metal'].drop('sectors1') * real_total_output_by_sector().loc['basic metals and fabricated metal'].values +
        ia_matrix().loc[:,'machinery nec'].drop('sectors1') * real_total_output_by_sector().loc['machinery nec'].values + ia_matrix().loc[:,'electrical and optical equipment'].drop('sectors1') * real_total_output_by_sector().loc['electrical and optical equipment'].values +
        ia_matrix().loc[:,'transport equipment'].drop('sectors1') * real_total_output_by_sector().loc['transport equipment'].values + ia_matrix().loc[:,'manufacturing nec recycling'].drop('sectors1') * real_total_output_by_sector().loc['manufacturing nec recycling'].values +
        ia_matrix().loc[:,'electricity gas and water supply'].drop('sectors1') * real_total_output_by_sector().loc['electricity gas and water supply'].values + ia_matrix().loc[:,'construction'].drop('sectors1') * real_total_output_by_sector().loc['construction'].values +
        ia_matrix().loc[:,'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].drop('sectors1') * real_total_output_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values + ia_matrix().loc[:,'wholesale trade and commissions trade except of motor vehicles and motorcycles'].drop('sectors1') * real_total_output_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values +
        ia_matrix().loc[:,'retail trade except of motor vehicles and motorcycles repair of household goods'].drop('sectors1') * real_total_output_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values + ia_matrix().loc[:,'hotels and restaurants'].drop('sectors1') * real_total_output_by_sector().loc['hotels and restaurants'].values +
        ia_matrix().loc[:,'inland transport'].drop('sectors1') * real_total_output_by_sector().loc['inland transport'].values + ia_matrix().loc[:,'water transport'].drop('sectors1') * real_total_output_by_sector().loc['water transport'].values +
        ia_matrix().loc[:,'air transport'].drop('sectors1') * real_total_output_by_sector().loc['air transport'].values + ia_matrix().loc[:,'other supporting and auxiliary transport activities activities of travel agencies'].drop('sectors1') * real_total_output_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies'].values +
        ia_matrix().loc[:,'post and telecommunications'].drop('sectors1') * real_total_output_by_sector().loc['post and telecommunications'].values + ia_matrix().loc[:,'financial intermedation'].drop('sectors1') * real_total_output_by_sector().loc['financial intermedation'].values +
        ia_matrix().loc[:,'real estate activities'].drop('sectors1') * real_total_output_by_sector().loc['real estate activities'].values + ia_matrix().loc[:,'renting od meq and other business activities'].drop('sectors1') * real_total_output_by_sector().loc['renting od meq and other business activities'].values +
        ia_matrix().loc[:,'public admin and defence compulsory social security'].drop('sectors1') * real_total_output_by_sector().loc['public admin and defence compulsory social security'].values + ia_matrix().loc[:,'education'].drop('sectors1') * real_total_output_by_sector().loc['education'].values +
        ia_matrix().loc[:,'health and social work'].drop('sectors1') * real_total_output_by_sector().loc['health and social work'].values + ia_matrix().loc[:,'other community social and persona services'].drop('sectors1') * real_total_output_by_sector().loc['other community social and persona services'].values +
        ia_matrix().loc[:,'private households with employed persons'].drop('sectors1') * real_total_output_by_sector().loc['private households with employed persons'].values)
    #test_nans(ret, 1841)
    return ret


@cache('step')
def required_total_output_by_sector():
    """
    required total output by sector



    component


    """
    
    ret = leontief_matrix().loc[:,'agriculture hunting forestry and fishing'].drop('sectors1') * demand_by_sector().loc['agriculture hunting forestry and fishing'].values + leontief_matrix().loc[:,'mining and quarrying'].drop('sectors1') * demand_by_sector().loc['mining and quarrying'].values + leontief_matrix().loc[:,'food beverages and tobacco'].drop('sectors1') * demand_by_sector().loc['food beverages and tobacco'].values + leontief_matrix().loc[:,'textiles and textile products'].drop('sectors1') * demand_by_sector().loc['textiles and textile products'].values + leontief_matrix().loc[:,'leather leather and footwear'].drop('sectors1') * demand_by_sector().loc['leather leather and footwear'].values + leontief_matrix().loc[:,'wood and products of woood and cork'].drop('sectors1') * demand_by_sector().loc['wood and products of woood and cork'].values + leontief_matrix().loc[:,'pulp paper printing and publishing'].drop('sectors1') * demand_by_sector().loc['pulp paper printing and publishing'].values + leontief_matrix().loc[:,'coke refined petroleum and nuclear fuel'].drop('sectors1') * demand_by_sector().loc['coke refined petroleum and nuclear fuel'].values + leontief_matrix().loc[:,'chemicals and chemical products'].drop('sectors1') * demand_by_sector().loc['chemicals and chemical products'].values + leontief_matrix().loc[:,'rubber and plastics'].drop('sectors1') * demand_by_sector().loc['rubber and plastics'].values + leontief_matrix().loc[:,'other non metalic mineral'].drop('sectors1') * demand_by_sector().loc['other non metalic mineral'].values + leontief_matrix().loc[:,'basic metals and fabricated metal'].drop('sectors1') * demand_by_sector().loc['basic metals and fabricated metal'].values + leontief_matrix().loc[:,'machinery nec'].drop('sectors1') * demand_by_sector().loc['machinery nec'].values + leontief_matrix().loc[:,'electrical and optical equipment'].drop('sectors1') * demand_by_sector().loc['electrical and optical equipment'].values + leontief_matrix().loc[:,'transport equipment'].drop('sectors1') * demand_by_sector().loc['transport equipment'].values + leontief_matrix().loc[:,'manufacturing nec recycling'].drop('sectors1') * demand_by_sector().loc['manufacturing nec recycling'].values + leontief_matrix().loc[:,'electricity gas and water supply'].drop('sectors1') * demand_by_sector().loc['electricity gas and water supply'].values + leontief_matrix().loc[:,'construction'].drop('sectors1') * demand_by_sector().loc['construction'].values + leontief_matrix().loc[:,'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].drop('sectors1') * demand_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values + leontief_matrix().loc[:,'wholesale trade and commissions trade except of motor vehicles and motorcycles'].drop('sectors1') * demand_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values + leontief_matrix().loc[:,'retail trade except of motor vehicles and motorcycles repair of household goods'].drop('sectors1') * demand_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values + leontief_matrix().loc[:,'hotels and restaurants'].drop('sectors1') * demand_by_sector().loc['hotels and restaurants'].values + leontief_matrix().loc[:,'inland transport'].drop('sectors1') * demand_by_sector().loc['inland transport'].values + leontief_matrix().loc[:,'water transport'].drop('sectors1') * demand_by_sector().loc['water transport'].values + leontief_matrix().loc[:,'air transport'].drop('sectors1') * demand_by_sector().loc['air transport'].values + leontief_matrix().loc[:,'other supporting and auxiliary transport activities activities of travel agencies'].drop('sectors1') * demand_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies'].values + leontief_matrix().loc[:,'post and telecommunications'].drop('sectors1') * demand_by_sector().loc['post and telecommunications'].values + leontief_matrix().loc[:,'financial intermedation'].drop('sectors1') * demand_by_sector().loc['financial intermedation'].values + leontief_matrix().loc[:,'real estate activities'].drop('sectors1') * demand_by_sector().loc['real estate activities'].values + leontief_matrix().loc[:,'renting od meq and other business activities'].drop('sectors1') * demand_by_sector().loc['renting od meq and other business activities'].values + leontief_matrix().loc[:,'public admin and defence compulsory social security'].drop('sectors1') * demand_by_sector().loc['public admin and defence compulsory social security'].values + leontief_matrix().loc[:,'education'].drop('sectors1') * demand_by_sector().loc['education'].values + leontief_matrix().loc[:,'health and social work'].drop('sectors1') * demand_by_sector().loc['health and social work'].values + leontief_matrix().loc[:,'other community social and persona services'].drop('sectors1') * demand_by_sector().loc['other community social and persona services'].values + leontief_matrix().loc[:,'private households with employed persons'].drop('sectors1') * demand_by_sector().loc['private households with employed persons'].values
    #test_nans(ret, 1842)
    return ret


@cache('step')
def rate_change_intensity_bottom_up():
    """
    rate change intensity bottom up



    constant


    """
    
    ret = xr.DataArray(data=[[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],
        functions.if_then_else((activate_bottom_up_method().loc['inland transport'].values - 1) < equality_threshold,
                               inland_transport_variation_intensity(), 0),[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0],[0.0,0.0,0.0,0.0,0.0]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 1843)
    return ret


@cache('step')
def rurr_unconv_oil_ej():
    """
    rurr unconv oil ej



    component


    """
    
    ret = integ_rurr_unconv_oil_ej()
    #test_nans(ret, 1844)
    return ret


@cache('step')
def share_liquids_for_final_energy():
    """
    share liquids for final energy



    component


    """
    
    ret = functions.zidz(required_fed_by_liquids_ej(),
                          (ped_liquids_ej() - other_liquids_required_ej()))
    #test_nans(ret, 1845)
    return ret


@cache('step')
def ped_liquids_ej():
    """
    ped liquids ej



    component


    """
    
    ret = np.maximum(0,
                      required_fed_by_liquids_ej() + other_liquids_required_ej() +
                      pe_demand_oil_elec_plants_ej() + ped_oil_for_heat_plants_ej() +
                      ped_oil_for_chp_plants_ej() + ped_liquids_heatxdashxnc())
    #test_nans(ret, 1846)
    return ret


@cache('step')
def fed_heat_liquids_plants_ej():
    """
    fed heat liquids plants ej



    component


    """
    
    ret = fed_heatxdashxcom_plants_fossil_fuels_ej() * share_liquids_fot_heat_plants()
    #test_nans(ret, 1847)
    return ret


@cache('step')
def real_final_energy_by_sector_and_fuel():
    """
    real final energy by sector and fuel



    component


    """
    
    ret = required_final_energy_by_sector_and_fuel() * energy_scarcity_feedback_shortage_coeff(
    ) * cc_impacts_feedback_shortage_coeff()
    ret = utils.align_by_dims(ret, ('final sources', 'sectors'))
    #test_nans(ret, 1848)
    return ret


@cache('run')
def hist_var_inlandt():
    """
    hist var inlandt



    constant


    """
    
    ret = xr.DataArray(
        data=[0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0., 0.],
        coords={
            'vehiclet': [
                'hv liq', 'hv hib', 'hv gas', 'lv liq', 'lv elec', 'lv hib', 'lv gas', 'bus liq',
                'bus elec', 'bus hib', 'bus gas', 'train liq', 'train elec'
            ]
        },
        dims=['vehiclet'])
    #test_nans(ret, 1849)
    return ret


@cache('step')
def inland_transport_variation_intensity():
    """
    inland transport variation intensity



    component


    """
    
    ret = xr.DataArray(data=[
        var_i_inland_elec(),0.0,
        var_i_inlandt_liq(),
        var_i_inlandt_gas(),0.0],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1850)
    return ret


@cache('step')
def fed_heatxdashxcom_nre_ej():
    """
    fed heatxdashxcom nre ej



    component


    """
    
    ret = np.maximum(
        fed_heatxdashxcom_after_priorities_ej() - total_fe_real_supply_res_for_heatxdashxcom_ej(),
        0)
    #test_nans(ret, 1851)
    return ret


@cache('step')
def real_fe_consumption_by_fuel():
    """
    real fe consumption by fuel



    component


    """
    
    ret = xr.DataArray(data=[
        total_fe_elec_consumption_ej(),
        total_fe_heat_consumption_ej(),
        real_fe_consumption_liquids_ej(),
        real_fe_consumption_gases_ej(),
        real_fe_consumption_solids_ej()],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1852)
    return ret


@cache('step')
def other_liquids_required_ej():
    """
    other liquids required ej



    component


    """
    
    ret = energy_distr_losses_ff_ej().loc['liquids'].values + transformation_ff_losses_ej().loc['liquids'].values + nonxdashxenergy_use_demand_by_final_fuel_ej().loc['liquids'].values
    #test_nans(ret, 1853)
    return ret


@cache('step')
def fe_elec_demand_consum_ej():
    """
    fe elec demand consum ej



    component


    """
    
    ret = required_fed_by_fuel().loc['electricity'].values
    #test_nans(ret, 1854)
    return ret


@cache('step')
def required_fed_by_liquids_ej():
    """
    required fed by liquids ej



    component


    """
    
    ret = required_fed_by_fuel().loc['liquids'].values
    #test_nans(ret, 1855)
    return ret


@cache('step')
def real_gfcf():
    """
    real gfcf



    component


    """
    
    ret = real_demand_by_sector() * (
        1 - share_consum_goverment_and_inventories()) * pct_gfcf_vs_gfcfxplusxhd()
    #test_nans(ret, 1856)
    return ret


@cache('step')
def real_household_demand():
    """
    real household demand



    component


    """
    
    ret = real_demand_by_sector() * (1 - share_consum_goverment_and_inventories()) * (
        1 - pct_gfcf_vs_gfcfxplusxhd())
    #test_nans(ret, 1857)
    return ret


@cache('step')
def demand_conv_oil_ej():
    """
    demand conv oil ej



    component


    """
    
    ret = np.maximum(ped_total_oil_ej() - extraction_unconv_oil_ej(), 0)
    #test_nans(ret, 1858)
    return ret


@cache('step')
def max_unconv_oil_growth_extraction_ej():
    """
    max unconv oil growth extraction ej



    component


    """
    
    ret = functions.if_then_else(
        check_liquids_delayed_1yr() < - equality_threshold,
        (1 + constrain_liquids_exogenous_growthxquestx_delayed_x1yr()) *
        extraction_unconv_oil_delayed(),
        extraction_unconv_oil_delayed() * max_unconv_oil_growth_extraction())
    #test_nans(ret, 1859)
    return ret


@cache('step')
def fe_demand_elec_consum_twh():
    """
    fe demand elec consum twh



    component


    """
    
    ret = fe_elec_demand_consum_ej() / ej_per_twh()
    #test_nans(ret, 1860)
    return ret


@cache('step')
def total_fed_heatxdashxcom_ej():
    """
    total fed heatxdashxcom ej



    component


    """
    
    ret = fed_heatxdashxcom_ej() * (1 + share_heat_distribution_losses())
    #test_nans(ret, 1861)
    return ret


@cache('step')
def real_total_output_by_fuel_and_sector():
    """
    real total output by fuel and sector



    component


    """
    
    ret = functions.xidz(real_final_energy_by_sector_and_fuel(),
                          final_energy_intensity_by_sector_and_fuel(),
                          required_total_output_by_sector() / 1e+006) * 1e+006
    ret = utils.align_by_dims(ret, ('final sources', 'sectors'))
    #test_nans(ret, 1862)
    return ret


@cache('step')
def required_final_energy_by_sector_and_fuel():
    """
    required final energy by sector and fuel



    component


    """
    
    ret = required_total_output_by_sector() * final_energy_intensity_by_sector_and_fuel() / 1e+006
    ret = utils.align_by_dims(ret, ('final sources', 'sectors'))
    #test_nans(ret, 1863)
    return ret


@cache('run')
def urr_unconv_oil():
    """
    urr unconv oil



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_oilxquestx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(choose_extraction_curve_unconv_oil() - 1) < equality_threshold,
                                      urr_unconv_oil_bg_mohr15(),
                                      functions.if_then_else(
                                          abs(choose_extraction_curve_unconv_oil() - 2) < equality_threshold,
                                          urr_unconv_oil_low_mohr15(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_unconv_oil() - 3) < equality_threshold,
                                              urr_unconv_oil_high_mohr15(),
                                              urr_unconv_oil_user_defined()))), 0)
    #test_nans(ret, 1864)
    return ret


@cache('step')
def fed_heat_coal_plants_ej():
    """
    fed heat coal plants ej



    component


    """
    
    ret = fed_heat_gasxplusxcoal_ej() * share_coalxcoalxplusxgasx_for_heat_plants()
    #test_nans(ret, 1865)
    return ret


@cache('run')
def urr_coal():
    """
    urr coal



    component


    """
    
    ret = functions.if_then_else(abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                  urr_coal_unlimited(),
                                  functions.if_then_else(
                                      abs(unlimited_coalxquestx() - 1) < equality_threshold,
                                      urr_coal_unlimited(),
                                      functions.if_then_else(
                                          abs(choose_extraction_coal_curve() - 1) < equality_threshold,
                                          urr_coal_mohr2012_ej(),
                                          functions.if_then_else(
                                              abs(choose_extraction_coal_curve() - 2) < equality_threshold,
                                              urr_coal_low_mohr15(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_coal_curve() - 3) < equality_threshold,
                                                  urr_coal_bg_mohr15(),
                                                  functions.if_then_else(
                                                      abs(choose_extraction_coal_curve() - 4) < equality_threshold,
                                                      urr_coal_high15(),
                                                      urr_coal_user_defined_ej()))))))
    #test_nans(ret, 1866)
    return ret


@cache('run')
def urr_conv_gas():
    """
    urr conv gas



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(unlimited_nrexquestx() - 1) < equality_threshold,
                                      urr_conv_gas_unlimited(),
                                      functions.if_then_else(
                                          abs(unlimited_gasxquestx() - 1) < equality_threshold,
                                          urr_conv_gas_unlimited(),
                                          functions.if_then_else(
                                              abs(choose_extraction_conv_gas_curve() - 1) < equality_threshold,
                                              urr_conv_gas_bg_mohr15(),
                                              functions.if_then_else(
                                                  abs(choose_extraction_conv_gas_curve() - 2) < equality_threshold,
                                                  urr_conv_gas_low_mohr15(),
                                                  functions.if_then_else(
                                                      abs(choose_extraction_conv_gas_curve() - 3) < equality_threshold,
                                                      urr_conv_gas_high_mohr15(),
                                                      urr_conv_gas_user_defined()))))), 0)
    #test_nans(ret, 1867)
    return ret


@cache('step')
def real_demand_by_sector_delayed():
    """
    real demand by sector delayed



    component


    """
    
    ret = xr.DataArray(data=[x() for x in delays_real_demand_by_sector_round1__time_step___time_step_10_1__time_step],
        dims=real_demand_by_sector().dims,
        coords=real_demand_by_sector().coords)
    #test_nans(ret, 1868)
    return ret


@cache('step')
def required_final_energy_other_transport():
    """
    required final energy other transport



    component


    """
    
    ret = required_final_energy_by_sector_and_fuel().loc[:,'other supporting and auxiliary transport activities activities of travel agencies'].drop('sectors')
    #test_nans(ret, 1869)
    return ret


@cache('step')
def real_total_output_by_sector():
    """
    real total output by sector



    component


    """
    
    ret = np.minimum(real_total_output_by_fuel_and_sector().loc['electricity',:].drop('final sources'),
                      np.minimum(real_total_output_by_fuel_and_sector().loc['heat',:].drop('final sources'),
                                 np.minimum(real_total_output_by_fuel_and_sector().loc['liquids',:].drop('final sources'),
                                            np.minimum(real_total_output_by_fuel_and_sector().loc['gases',:].drop('final sources'),
                                                       real_total_output_by_fuel_and_sector().loc['solids',:].drop('final sources')))))
    #test_nans(ret, 1870)
    return ret


@cache('step')
def required_final_energy_air_transport():
    """
    required final energy air transport



    component


    """
    
    ret = required_final_energy_by_sector_and_fuel().loc[:,'air transport'].drop('sectors')
    #test_nans(ret, 1871)
    return ret


@cache('step')
def required_final_energy_inland_transport():
    """
    required final energy inland transport



    component


    """
    
    ret = required_final_energy_by_sector_and_fuel().loc[:,'inland transport'].drop('sectors')
    #test_nans(ret, 1872)
    return ret


@cache('step')
def required_final_energy_water_transport():
    """
    required final energy water transport



    component


    """
    
    ret = required_final_energy_by_sector_and_fuel().loc[:,'water transport'].drop('sectors')
    #test_nans(ret, 1873)
    return ret


@cache('run')
def urr_coal_bg_mohr15():
    """
    urr coal bg mohr15



    constant


    """
    
    ret = 22500
    #test_nans(ret, 1874)
    return ret


@cache('run')
def urr_coal_user_defined_ej():
    """
    urr coal user defined ej



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1875)
    return ret


@cache('run')
def urr_conv_gas_bg_mohr15():
    """
    urr conv gas bg mohr15



    constant


    """
    
    ret = 13200
    #test_nans(ret, 1876)
    return ret


@cache('run')
def urr_conv_gas_high_mohr15():
    """
    urr conv gas high mohr15



    constant


    """
    
    ret = 22500
    #test_nans(ret, 1877)
    return ret


@cache('run')
def urr_conv_gas_low_mohr15():
    """
    urr conv gas low mohr15



    constant


    """
    
    ret = 11000
    #test_nans(ret, 1878)
    return ret


@cache('run')
def urr_conv_gas_user_defined():
    """
    urr conv gas user defined



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1879)
    return ret


@cache('run')
def urr_total_gas_laherrere10():
    """
    urr total gas laherrere10



    constant


    """
    
    ret = 13000
    #test_nans(ret, 1880)
    return ret


@cache('run')
def urr_total_gas_mohr12_bg():
    """
    urr total gas mohr12 bg



    constant


    """
    
    ret = 20000
    #test_nans(ret, 1881)
    return ret


@cache('run')
def urr_total_gas_user_defined():
    """
    urr total gas user defined



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1882)
    return ret


@cache('run')
def urr_unconv_gas():
    """
    urr unconv gas



    component


    """
    
    ret = functions.if_then_else(abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
                                  functions.if_then_else(
                                      abs(choose_extraction_curve_unconv_gas() - 1) < equality_threshold,
                                      urr_unconv_gas_bg_mohr15(),
                                      functions.if_then_else(
                                          abs(choose_extraction_curve_unconv_gas() - 2) < equality_threshold,
                                          urr_unconv_gas_low_mohr15(),
                                          functions.if_then_else(
                                              abs(choose_extraction_curve_unconv_gas() - 3) < equality_threshold,
                                              urr_unconv_gas_high_mohr15(),
                                              urr_unconv_gas_user_defined()))), 0)
    #test_nans(ret, 1883)
    return ret


@cache('run')
def urr_unconv_gas_bg_mohr15():
    """
    urr unconv gas bg mohr15



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1884)
    return ret


@cache('run')
def urr_unconv_gas_low_mohr15():
    """
    urr unconv gas low mohr15



    constant


    """
    
    ret = 2800
    #test_nans(ret, 1885)
    return ret


@cache('run')
def urr_coal_high15():
    """
    urr coal high15



    constant


    """
    
    ret = 31500
    #test_nans(ret, 1886)
    return ret


@cache('run')
def urr_coal_low_mohr15():
    """
    urr coal low mohr15



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1887)
    return ret


@cache('run')
def urr_unconv_gas_high_mohr15():
    """
    urr unconv gas high mohr15



    constant


    """
    
    ret = 24000
    #test_nans(ret, 1888)
    return ret


@cache('run')
def urr_unconv_gas_user_defined():
    """
    urr unconv gas user defined



    constant


    """
    
    ret = 14500
    #test_nans(ret, 1889)
    return ret


@cache('run')
def urr_coal_mohr2012_ej():
    """
    urr coal mohr2012 ej



    constant


    """
    
    ret = 27800
    #test_nans(ret, 1890)
    return ret


@cache('run')
def selection_constraint_extraction_unconv_oil():
    """
    selection constraint extraction unconv oil



    constant


    """
    
    ret = 1
    #test_nans(ret, 1891)
    return ret


@cache('step')
def abundance_electricity():
    """
    abundance electricity



    component


    """
    
    ret = functions.if_then_else(
        total_fe_elec_generation_twh() > total_fe_elec_demand_twh(), 1,
        1 - functions.zidz(total_fe_elec_demand_twh() - total_fe_elec_generation_twh(),
                           total_fe_elec_demand_twh()))
    #test_nans(ret, 1892)
    return ret


@cache('run')
def p_constraint_growth_extraction_unconv_oil():
    """
    p constraint growth extraction unconv oil



    constant


    """
    
    ret = 0.045
    #test_nans(ret, 1893)
    return ret


@cache('step')
def extraction_unconv_oil_delayed():
    """
    extraction unconv oil delayed



    component


    """
    
    ret = delay_extraction_unconv_oil_ej_roundtime_step__time_step___time_step_109_time_step__time_step(
    )
    #test_nans(ret, 1894)
    return ret


@cache('run')
def final_time():
    """
    final time



    constant


    """
    
    ret = 2050
    #test_nans(ret, 1895)
    return ret


@cache('run')
def initial_time():
    """
    initial time



    constant


    """
    
    ret = 1995
    #test_nans(ret, 1896)
    return ret


@cache('run')
def saveper():
    """
    saveper



    component


    """
    
    ret = time_step()
    #test_nans(ret, 1897)
    return ret


@cache('run')
def time_step():
    """
    time step



    constant


    """
    
    ret = 0.03125
    #test_nans(ret, 1898)
    return ret


@cache('step')
def new_cellulosic_biofuels():
    """
    new cellulosic biofuels



    component


    """
    
    ret = functions.if_then_else(
        time() < start_year_cellulosic_biofuels(), 0,
        functions.if_then_else(
            time() < start_year_cellulosic_biofuels() + 5,
            start_production_biofuels(time() - start_year_cellulosic_biofuels()) * ej_per_ktoe(),
            functions.if_then_else(
                check_liquids() < - equality_threshold,
                constrain_liquids_exogenous_growthxquestx() * potential_pe_cellulosic_biofuel_ej(),
                p_cellulosic_biofuels() * potential_pe_cellulosic_biofuel_ej() *
                cellulosic_biofuels_available())))
    #test_nans(ret, 1899)
    return ret


@cache('step')
def new_bioe_residues_for_heatxplusxelec():
    """
    new bioe residues for heatxplusxelec



    component


    """
    
    ret = functions.if_then_else(
        time() < start_year_bioe_residues_for_heatxplusxelec(), 0,
        functions.if_then_else(
            time() < start_year_bioe_residues_for_heatxplusxelec() + 5,
            start_production_biofuels(time() - start_year_bioe_residues_for_heatxplusxelec()) *
            ej_per_ktoe(),
            p_bioe_residues_for_heatxplusxelec() * pe_bioe_residues_for_heatxplusxelec_ej() *
            bioe_residues_for_heatxplusxelec_available()))
    #test_nans(ret, 1900)
    return ret


@cache('step')
def new_biofuels_land_marg():
    """
    new biofuels land marg



    component


    """
    
    ret = functions.if_then_else(
        time() < start_year_biofuels_land_marg(), 0,
        functions.if_then_else(
            time() < start_year_biofuels_land_marg() + 5,
            start_production_biofuels(time() - start_year_biofuels_land_marg()) * ej_per_ktoe() /
            ratio_land_productivity_2gen_vs_marg(),
            functions.if_then_else(check_liquids() < - equality_threshold,
                                   constrain_liquids_exogenous_growthxquestx() *
                                   potential_peavail_biofuels_land_marg_ej(),
                                   adapt_growth_biofuels_2gen() * bioe_gen_land_marg_available() *
                                   potential_peavail_biofuels_land_marg_ej())))
    #test_nans(ret, 1901)
    return ret


@cache('step')
def share_liquids_fot_heat_plants():
    """
    share liquids fot heat plants



    component


    """
    
    ret = functions.if_then_else(time() < 2014,
                                  historic_share_liquids_for_heat_plants(time()),
                                  functions.if_then_else(
                                      abs(phasexdashxout_oil_for_heatxdashxcomxquestx() - 0) < equality_threshold,
                                      historic_share_liquids_for_heat_plants(time()),
                                      functions.if_then_else(
                                          time() < start_year_policy_phasexdashxout_oil_for_heat(),
                                          historic_share_liquids_for_heat_plants(time()),
                                          p_share_oil_for_heat())))
    #test_nans(ret, 1902)
    return ret


@cache('step')
def a_lineal_regr_phasexdashxout_oil_for_heat():
    """
    a lineal regr phasexdashxout oil for heat



    component


    """
    
    ret = (share_in_target_year_oil_for_heat() - historic_share_liquids_for_heat_plants(time())
            ) / (target_year_policy_phasexdashxout_oil_for_heat() -
                 start_year_policy_phasexdashxout_oil_for_heat())
    #test_nans(ret, 1903)
    return ret


@cache('step')
def share_oil_for_elec():
    """
    share oil for elec



    component


    """
    
    ret = functions.if_then_else(time() < 2010,
                                  hist_share_oil_elec(time()),
                                  functions.if_then_else(
                                      abs(phasexdashxout_oil_for_electricityxquestx() - 0) < equality_threshold,
                                      hist_share_oil_elec(time()),
                                      functions.if_then_else(
                                          time() < start_year_policy_phasexdashxout_oil_for_elec(),
                                          hist_share_oil_elec(time()), p_share_oil_for_elec())))
    #test_nans(ret, 1904)
    return ret


@cache('step')
def a_lineal_regr_phasexdashxout_oil_for_elec():
    """
    a lineal regr phasexdashxout oil for elec



    component


    """
    
    ret = (share_in_target_year_oil_for_elec() - hist_share_oil_elec(time())) / (
        target_year_policy_phasexdashxout_oil_for_elec() -
        start_year_policy_phasexdashxout_oil_for_elec())
    #test_nans(ret, 1905)
    return ret


@cache('step')
def co2x_soilxandxluc_emissions():
    """
    co2x soilxandxluc emissions



    component


    """
    
    ret = co2x_landxdashxuse_change_emissions_exogenous(time())
    #test_nans(ret, 1906)
    return ret


@cache('step')
def cp_res_elec():
    """
    cp res elec



    component


    """
    
    ret = xr.DataArray(data=[
        np.maximum(min_cp_baseload_res().loc['hydro'].values,
                   cpxdashxini_res_elec_subhydro(time()) * cp_exogenous_res_elec_reduction().loc['hydro'].values),
        np.maximum(min_cp_baseload_res().loc['geotxdashxelec'].values,
                   cpxdashxini_res_elec_subgeotxdashxelec() * cp_exogenous_res_elec_reduction().loc['geotxdashxelec'].values),
        np.maximum(
            min_cp_baseload_res().loc['solid bioexdashxelec'].values,
            cpxdashxini_res_elec_subsolid_bioexdashxelec() * cp_exogenous_res_elec_reduction().loc['solid bioexdashxelec'].values),
        np.maximum(min_cp_baseload_res().loc['oceanic'].values,
                   cpxdashxini_res_elec_suboceanic() * cp_exogenous_res_elec_reduction().loc['oceanic'].values),
        np.maximum(min_cp_baseload_res().loc['wind onshore'].values,
                   cpxdashxini_res_elec_subwind_onshore() * cp_exogenous_res_elec_reduction().loc['wind onshore'].values),
        np.maximum(min_cp_baseload_res().loc['wind offshore'].values,
                   cpxdashxini_res_elec_subwind_offshore() * cp_exogenous_res_elec_reduction().loc['wind offshore'].values),
        np.maximum(min_cp_baseload_res().loc['solar pv'].values,
                   cpxdashxini_res_elec_subsolar_pv() * cp_exogenous_res_elec_reduction().loc['solar pv'].values),
        np.maximum(min_cp_baseload_res().loc['csp'].values,
                   cpxdashxini_res_elec_subcsp() * cp_exogenous_res_elec_reduction().loc['csp'].values)],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1907)
    return ret


@cache('step')
def evol_final_energy_intensity_by_sector_and_fuel_x2x():
    """
    evol final energy intensity by sector and fuel x2x



    component


    """
    
    ret = xr.DataArray(data=[[
        functions.if_then_else(
            activate_bottom_up_method().loc['agriculture hunting forestry and fishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculture_hunting_forestry_and_fishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['mining and quarrying'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submining_and_quarrying(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['food beverages and tobacco'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfood_beverages_and_tobacco(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['textiles and textile products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextiles_and_textile_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['leather leather and footwear'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleather_leather_and_footwear(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wood and products of woood and cork'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwood_and_products_of_woood_and_cork(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['pulp paper printing and publishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulp_paper_printing_and_publishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['coke refined petroleum and nuclear fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcoke_refined_petroleum_and_nuclear_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['chemicals and chemical products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicals_and_chemical_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['rubber and plastics'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubber_and_plastics(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other non metalic mineral'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_non_metalic_mineral(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['basic metals and fabricated metal'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasic_metals_and_fabricated_metal(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['machinery nec'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinery_nec(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electrical and optical equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectrical_and_optical_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['transport equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransport_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['manufacturing nec recycling'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturing_nec_recycling(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electricity gas and water supply'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricity_gas_and_water_supply(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['construction'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['construction','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['construction','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['hotels and restaurants'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotels_and_restaurants(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['inland transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinland_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['water transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwater_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['air transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subair_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other supporting and auxiliary transport activities activities of travel agencies'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['post and telecommunications'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpost_and_telecommunications(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['financial intermedation'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancial_intermedation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['real estate activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subreal_estate_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['renting od meq and other business activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrenting_od_meq_and_other_business_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['public admin and defence compulsory social security'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublic_admin_and_defence_compulsory_social_security(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['education'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['education','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['education','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['health and social work'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealth_and_social_work(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other community social and persona services'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_community_social_and_persona_services(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','electricity'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['private households with employed persons'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivate_households_with_employed_persons(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','electricity'].values))],[
        functions.if_then_else(
            activate_bottom_up_method().loc['agriculture hunting forestry and fishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculture_hunting_forestry_and_fishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['mining and quarrying'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_submining_and_quarrying(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['food beverages and tobacco'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subfood_beverages_and_tobacco(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['textiles and textile products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextiles_and_textile_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['leather leather and footwear'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subleather_leather_and_footwear(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wood and products of woood and cork'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subwood_and_products_of_woood_and_cork(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['pulp paper printing and publishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulp_paper_printing_and_publishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['coke refined petroleum and nuclear fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subcoke_refined_petroleum_and_nuclear_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['chemicals and chemical products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicals_and_chemical_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['rubber and plastics'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubber_and_plastics(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other non metalic mineral'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_non_metalic_mineral(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['basic metals and fabricated metal'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasic_metals_and_fabricated_metal(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['machinery nec'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinery_nec(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electrical and optical equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectrical_and_optical_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['transport equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransport_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['manufacturing nec recycling'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturing_nec_recycling(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electricity gas and water supply'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricity_gas_and_water_supply(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['construction'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['construction','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction(time()),
                evol_final_energy_intensity_by_sector_and_fuel().loc['construction','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['hotels and restaurants'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotels_and_restaurants(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['inland transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subinland_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['water transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subwater_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['air transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subair_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other supporting and auxiliary transport activities activities of travel agencies'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['post and telecommunications'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subpost_and_telecommunications(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['financial intermedation'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancial_intermedation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['real estate activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subreal_estate_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['renting od meq and other business activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subrenting_od_meq_and_other_business_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['public admin and defence compulsory social security'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublic_admin_and_defence_compulsory_social_security(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['education'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['education','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation(time()),
                evol_final_energy_intensity_by_sector_and_fuel().loc['education','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['health and social work'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealth_and_social_work(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other community social and persona services'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_community_social_and_persona_services(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','heat'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['private households with employed persons'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','heat'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivate_households_with_employed_persons(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','heat'].values))],[
        functions.if_then_else(
            activate_bottom_up_method().loc['agriculture hunting forestry and fishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculture_hunting_forestry_and_fishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['mining and quarrying'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_submining_and_quarrying(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['food beverages and tobacco'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfood_beverages_and_tobacco(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['textiles and textile products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextiles_and_textile_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['leather leather and footwear'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleather_leather_and_footwear(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wood and products of woood and cork'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwood_and_products_of_woood_and_cork(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['pulp paper printing and publishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulp_paper_printing_and_publishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['coke refined petroleum and nuclear fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcoke_refined_petroleum_and_nuclear_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['chemicals and chemical products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicals_and_chemical_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['rubber and plastics'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubber_and_plastics(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other non metalic mineral'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_non_metalic_mineral(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['basic metals and fabricated metal'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasic_metals_and_fabricated_metal(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['machinery nec'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinery_nec(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electrical and optical equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectrical_and_optical_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['transport equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransport_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['manufacturing nec recycling'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturing_nec_recycling(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electricity gas and water supply'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricity_gas_and_water_supply(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['construction'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['construction','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['construction','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['hotels and restaurants'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotels_and_restaurants(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['inland transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinland_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['water transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwater_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['air transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subair_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other supporting and auxiliary transport activities activities of travel agencies'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['post and telecommunications'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpost_and_telecommunications(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['financial intermedation'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancial_intermedation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['real estate activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subreal_estate_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['renting od meq and other business activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrenting_od_meq_and_other_business_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['public admin and defence compulsory social security'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublic_admin_and_defence_compulsory_social_security(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['education'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['education','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation(time()),
                evol_final_energy_intensity_by_sector_and_fuel().loc['education','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['health and social work'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealth_and_social_work(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other community social and persona services'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_community_social_and_persona_services(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','liquids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['private households with employed persons'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivate_households_with_employed_persons(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','liquids'].values))],[
        functions.if_then_else(
            activate_bottom_up_method().loc['agriculture hunting forestry and fishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculture_hunting_forestry_and_fishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['mining and quarrying'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_submining_and_quarrying(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['food beverages and tobacco'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subfood_beverages_and_tobacco(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['textiles and textile products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextiles_and_textile_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['leather leather and footwear'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subleather_leather_and_footwear(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wood and products of woood and cork'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subwood_and_products_of_woood_and_cork(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['pulp paper printing and publishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulp_paper_printing_and_publishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['coke refined petroleum and nuclear fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subcoke_refined_petroleum_and_nuclear_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['chemicals and chemical products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicals_and_chemical_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['rubber and plastics'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubber_and_plastics(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other non metalic mineral'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_non_metalic_mineral(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['basic metals and fabricated metal'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasic_metals_and_fabricated_metal(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['machinery nec'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinery_nec(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electrical and optical equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectrical_and_optical_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['transport equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransport_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['manufacturing nec recycling'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturing_nec_recycling(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electricity gas and water supply'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricity_gas_and_water_supply(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['construction'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['construction','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['construction','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['hotels and restaurants'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotels_and_restaurants(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['inland transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subinland_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['water transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subwater_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['air transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subair_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other supporting and auxiliary transport activities activities of travel agencies'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['post and telecommunications'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subpost_and_telecommunications(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['financial intermedation'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancial_intermedation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['real estate activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subreal_estate_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['renting od meq and other business activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subrenting_od_meq_and_other_business_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['public admin and defence compulsory social security'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublic_admin_and_defence_compulsory_social_security(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['education'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['education','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation(time()),
                evol_final_energy_intensity_by_sector_and_fuel().loc['education','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['health and social work'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealth_and_social_work(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other community social and persona services'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_community_social_and_persona_services(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','gases'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['private households with employed persons'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivate_households_with_employed_persons(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','gases'].values))],[
        functions.if_then_else(
            activate_bottom_up_method().loc['agriculture hunting forestry and fishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculture_hunting_forestry_and_fishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['mining and quarrying'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_submining_and_quarrying(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['food beverages and tobacco'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfood_beverages_and_tobacco(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['textiles and textile products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextiles_and_textile_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['leather leather and footwear'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleather_leather_and_footwear(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wood and products of woood and cork'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwood_and_products_of_woood_and_cork(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['pulp paper printing and publishing'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulp_paper_printing_and_publishing(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['coke refined petroleum and nuclear fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcoke_refined_petroleum_and_nuclear_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['chemicals and chemical products'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicals_and_chemical_products(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['rubber and plastics'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubber_and_plastics(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other non metalic mineral'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_non_metalic_mineral(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['basic metals and fabricated metal'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasic_metals_and_fabricated_metal(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['machinery nec'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinery_nec(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electrical and optical equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectrical_and_optical_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['transport equipment'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransport_equipment(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['manufacturing nec recycling'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturing_nec_recycling(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['electricity gas and water supply'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricity_gas_and_water_supply(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['construction'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['construction','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['construction','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['hotels and restaurants'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotels_and_restaurants(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['inland transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinland_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['water transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwater_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['air transport'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subair_transport(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other supporting and auxiliary transport activities activities of travel agencies'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['post and telecommunications'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpost_and_telecommunications(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['financial intermedation'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancial_intermedation(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['real estate activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subreal_estate_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['renting od meq and other business activities'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrenting_od_meq_and_other_business_activities(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['public admin and defence compulsory social security'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublic_admin_and_defence_compulsory_social_security(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['education'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['education','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation(time()),
                evol_final_energy_intensity_by_sector_and_fuel().loc['education','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['health and social work'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealth_and_social_work(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['other community social and persona services'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_community_social_and_persona_services(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','solids'].values)),
        functions.if_then_else(
            activate_bottom_up_method().loc['private households with employed persons'].values == 1,
            evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','solids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivate_households_with_employed_persons(
                    time()), evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','solids'].values))]],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids'], 'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['final sources', 'sectors'])
    ret = utils.align_by_dims(ret, ('final sources', 'sectors'))
    #test_nans(ret, 1908)
    return ret


@cache('step')
def energy_intensity_of_households_rest():
    """
    energy intensity of households rest



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(
            abs(activate_transport_h_bottom_up_method() - 1) < equality_threshold,
            evol_final_energy_intensity_h().loc['electricity'].values * change_total_intensity_to_rest().loc['electricity'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_h_subelectricity(time()),
                evol_final_energy_intensity_h().loc['electricity'].values)),
        functions.if_then_else(
            abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
            historic_final_energy_intensity_h_subheat(time()), evol_final_energy_intensity_h().loc['heat'].values),
        functions.if_then_else(
            abs(activate_transport_h_bottom_up_method() - 1) < equality_threshold,
            evol_final_energy_intensity_h().loc['liquids'].values * change_total_intensity_to_rest().loc['liquids'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_h_subliquids(time()),
                evol_final_energy_intensity_h().loc['liquids'].values)),
        functions.if_then_else(
            abs(activate_transport_h_bottom_up_method() - 1) < equality_threshold,
            evol_final_energy_intensity_h().loc['gases'].values * change_total_intensity_to_rest().loc['gases'].values,
            functions.if_then_else(
                abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
                historic_final_energy_intensity_h_subgases(time()),
                evol_final_energy_intensity_h().loc['gases'].values)),
        functions.if_then_else(
            abs(constant_final_energy_intensity_sectorsxandxhouseholds_tdxquestx() - 1) < equality_threshold,
            historic_final_energy_intensity_h_subsolids(time()), evol_final_energy_intensity_h().loc['solids'].values)],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1909)
    return ret


@cache('step')
def other_forcings_rcp():
    """
    other forcings rcp



    component


    """
    
    ret = functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                                  other_forcings_rcp_scenario_subrcp26(time()),
                                  functions.if_then_else(
                                      abs(choose_rcp() - 2) < equality_threshold,
                                      other_forcings_rcp_scenario_subrcp45(time()),
                                      functions.if_then_else(
                                          abs(choose_rcp() - 3) < equality_threshold,
                                          other_forcings_rcp_scenario_subrcp60(time()),
                                          other_forcings_rcp_scenario_subrcp85(time()))))
    #test_nans(ret, 1910)
    return ret


@cache('step')
def halocarbon_rf():
    """
    halocarbon rf



    component


    """
    
    ret = rf_from_f_gases() + mp_rf_total(time())
    #test_nans(ret, 1911)
    return ret


@cache('step')
def other_forcings():
    """
    other forcings



    component


    """
    
    ret = functions.if_then_else(time() <= last_historical_rf_year(),
                                  other_forcings_history(time()), other_forcings_rcp())
    #test_nans(ret, 1912)
    return ret


@cache('step')
def global_ch4_anthro_emissions():
    """
    global ch4 anthro emissions



    component


    """
    
    ret = total_ch4_emissions_fossil_fuels() + functions.if_then_else(
        abs(choose_rcp() - 1) < equality_threshold,
        global_ch4_anthro_emissions_rcp_subrcp26(time()),
        functions.if_then_else(abs(choose_rcp() - 2) < equality_threshold,
                               global_ch4_anthro_emissions_rcp_subrcp45(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 3) < equality_threshold,
                                   global_ch4_anthro_emissions_rcp_subrcp60(time()),
                                   global_ch4_anthro_emissions_rcp_subrcp85(time()))))
    #test_nans(ret, 1913)
    return ret


@cache('step')
def global_hfc_emissions():
    """
    global hfc emissions



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc134a(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc134a(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc134a(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc134a(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc23(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc23(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc23(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc23(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc32(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc32(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc32(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc32(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc125(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc125(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc125(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc125(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc143a(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc143a(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc143a(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc143a(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc152a(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc152a(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc152a(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc152a(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc227ea(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc227ea(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc227ea(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc227ea(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc245ca(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc245ca(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc245ca(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc245ca(time())))),
        functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                               global_hfc_emissions_rcp_x2x6_subhfc4310mee(time()),
                               functions.if_then_else(
                                   abs(choose_rcp() - 2) < equality_threshold,
                                   global_hfc_emissions_rcp_x4x5_subhfc4310mee(time()),
                                   functions.if_then_else(
                                       abs(choose_rcp() - 3) < equality_threshold,
                                       global_hfc_emissions_rcp_x6x0_subhfc4310mee(time()),
                                       global_hfc_emissions_rcp_x8x5_subhfc4310mee(time()))))],
        coords={'hfc type': ['hfc134a', 'hfc23', 'hfc32', 'hfc125', 'hfc143a', 'hfc152a', 'hfc227ea', 'hfc245ca', 'hfc4310mee']},
        dims=['hfc type'])
    #test_nans(ret, 1914)
    return ret


@cache('step')
def global_pfc_emissions():
    """
    global pfc emissions



    component


    """
    
    ret = functions.if_then_else(
        abs(choose_rcp() - 1) < equality_threshold,
        global_pfc_emissions_rcp_subrcp26(time()),
        functions.if_then_else(abs(choose_rcp() - 2) < equality_threshold,
                               global_pfc_emissions_rcp_subrcp45(time()),
                               functions.if_then_else(abs(choose_rcp() - 3) < equality_threshold,
                                                      global_pfc_emissions_rcp_subrcp60(time()),
                                                      global_pfc_emissions_rcp_subrcp85(time()))))
    #test_nans(ret, 1915)
    return ret


@cache('step')
def global_sf6_emissions():
    """
    global sf6 emissions



    component


    """
    
    ret = functions.if_then_else(
        abs(choose_rcp() - 1) < equality_threshold,
        global_sf6_emissions_rcp_subrcp26(time()),
        functions.if_then_else(abs(choose_rcp() - 2) < equality_threshold,
                               global_sf6_emissions_rcp_subrcp45(time()),
                               functions.if_then_else(abs(choose_rcp() - 3) < equality_threshold,
                                                      global_sf6_emissions_rcp_subrcp60(time()),
                                                      global_sf6_emissions_rcp_subrcp85(time()))))
    #test_nans(ret, 1916)
    return ret


@cache('step')
def global_n2o_anthro_emissions():
    """
    global n2o anthro emissions



    component


    """
    
    ret = functions.if_then_else(abs(choose_rcp() - 1) < equality_threshold,
                                  global_n2o_anthro_emissions_rcp_subrcp26(time()),
                                  functions.if_then_else(
                                      abs(choose_rcp() - 2) < equality_threshold,
                                      global_n2o_anthro_emissions_rcp_subrcp45(time()),
                                      functions.if_then_else(
                                          abs(choose_rcp() - 3) < equality_threshold,
                                          global_n2o_anthro_emissions_rcp_subrcp60(time()),
                                          global_n2o_anthro_emissions_rcp_subrcp85(time()))))
    #test_nans(ret, 1917)
    return ret


@cache('step')
def historic_water_by_type_intensities_by_sector():
    """
    historic water by type intensities by sector



    component


    """
    
    ret = xr.DataArray(data=[[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subblue_water(
                time()) / real_total_output_by_sector().loc['agriculture hunting forestry and fishing'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subgreen_water(
                time()) / real_total_output_by_sector().loc['agriculture hunting forestry and fishing'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subagriculture_hunting_forestry_and_fishing_subgray_water(
                time()) / real_total_output_by_sector().loc['agriculture hunting forestry and fishing'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submining_and_quarrying_subblue_water(time()) /
            real_total_output_by_sector().loc['mining and quarrying'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submining_and_quarrying_subgreen_water(time()) /
            real_total_output_by_sector().loc['mining and quarrying'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submining_and_quarrying_subgray_water(time()) /
            real_total_output_by_sector().loc['mining and quarrying'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subblue_water(time())
            / real_total_output_by_sector().loc['food beverages and tobacco'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subgreen_water(
                time()) / real_total_output_by_sector().loc['food beverages and tobacco'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subfood_beverages_and_tobacco_subgray_water(time())
            / real_total_output_by_sector().loc['food beverages and tobacco'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subtextiles_and_textile_products_subblue_water(
                time()) / real_total_output_by_sector().loc['textiles and textile products'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subtextiles_and_textile_products_subgreen_water(
                time()) / real_total_output_by_sector().loc['textiles and textile products'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subtextiles_and_textile_products_subgray_water(
                time()) / real_total_output_by_sector().loc['textiles and textile products'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subleather_leather_and_footwear_subblue_water(
                time()) / real_total_output_by_sector().loc['leather leather and footwear'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subleather_leather_and_footwear_subgreen_water(
                time()) / real_total_output_by_sector().loc['leather leather and footwear'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subleather_leather_and_footwear_subgray_water(
                time()) / real_total_output_by_sector().loc['leather leather and footwear'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subblue_water(
                time()) / real_total_output_by_sector().loc['wood and products of woood and cork'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subgreen_water(
                time()) / real_total_output_by_sector().loc['wood and products of woood and cork'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwood_and_products_of_woood_and_cork_subgray_water(
                time()) / real_total_output_by_sector().loc['wood and products of woood and cork'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subblue_water(
                time()) / real_total_output_by_sector().loc['pulp paper printing and publishing'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subgreen_water(
                time()) / real_total_output_by_sector().loc['pulp paper printing and publishing'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpulp_paper_printing_and_publishing_subgray_water(
                time()) / real_total_output_by_sector().loc['pulp paper printing and publishing'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subblue_water(
                time()) / real_total_output_by_sector().loc['coke refined petroleum and nuclear fuel'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subgreen_water(
                time()) / real_total_output_by_sector().loc['coke refined petroleum and nuclear fuel'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subcoke_refined_petroleum_and_nuclear_fuel_subgray_water(
                time()) / real_total_output_by_sector().loc['coke refined petroleum and nuclear fuel'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subblue_water(
                time()) / real_total_output_by_sector().loc['chemicals and chemical products'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subgreen_water(
                time()) / real_total_output_by_sector().loc['chemicals and chemical products'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subchemicals_and_chemical_products_subgray_water(
                time()) / real_total_output_by_sector().loc['chemicals and chemical products'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subrubber_and_plastics_subblue_water(time()) /
            real_total_output_by_sector().loc['rubber and plastics'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subrubber_and_plastics_subgreen_water(time()) /
            real_total_output_by_sector().loc['rubber and plastics'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subrubber_and_plastics_subgray_water(time()) /
            real_total_output_by_sector().loc['rubber and plastics'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_non_metalic_mineral_subblue_water(time()) /
            real_total_output_by_sector().loc['other non metalic mineral'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_non_metalic_mineral_subgreen_water(time())
            / real_total_output_by_sector().loc['other non metalic mineral'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_non_metalic_mineral_subgray_water(time()) /
            real_total_output_by_sector().loc['other non metalic mineral'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subblue_water(
                time()) / real_total_output_by_sector().loc['basic metals and fabricated metal'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subgreen_water(
                time()) / real_total_output_by_sector().loc['basic metals and fabricated metal'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subbasic_metals_and_fabricated_metal_subgray_water(
                time()) / real_total_output_by_sector().loc['basic metals and fabricated metal'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submachinery_nec_subblue_water(time()) /
            real_total_output_by_sector().loc['machinery nec'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submachinery_nec_subgreen_water(time()) /
            real_total_output_by_sector().loc['machinery nec'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submachinery_nec_subgray_water(time()) /
            real_total_output_by_sector().loc['machinery nec'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subblue_water(
                time()) / real_total_output_by_sector().loc['electrical and optical equipment'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subgreen_water(
                time()) / real_total_output_by_sector().loc['electrical and optical equipment'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subelectrical_and_optical_equipment_subgray_water(
                time()) / real_total_output_by_sector().loc['electrical and optical equipment'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subtransport_equipment_subblue_water(time()) /
            real_total_output_by_sector().loc['transport equipment'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subtransport_equipment_subgreen_water(time()) /
            real_total_output_by_sector().loc['transport equipment'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subtransport_equipment_subgray_water(time()) /
            real_total_output_by_sector().loc['transport equipment'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subblue_water(
                time()) / real_total_output_by_sector().loc['manufacturing nec recycling'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subgreen_water(
                time()) / real_total_output_by_sector().loc['manufacturing nec recycling'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_submanufacturing_nec_recycling_subgray_water(
                time()) / real_total_output_by_sector().loc['manufacturing nec recycling'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subblue_water(
                time()) / real_total_output_by_sector().loc['electricity gas and water supply'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subgreen_water(
                time()) / real_total_output_by_sector().loc['electricity gas and water supply'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subelectricity_gas_and_water_supply_subgray_water(
                time()) / real_total_output_by_sector().loc['electricity gas and water supply'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subconstruction_subblue_water(time()) /
            real_total_output_by_sector().loc['construction'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subconstruction_subgreen_water(time()) /
            real_total_output_by_sector().loc['construction'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subconstruction_subgray_water(time()) /
            real_total_output_by_sector().loc['construction'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subblue_water(
                time()) / real_total_output_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subgreen_water(
                time()) / real_total_output_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel_subgray_water(
                time()) / real_total_output_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subblue_water(
                time()) / real_total_output_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subgreen_water(
                time()) / real_total_output_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles_subgray_water(
                time()) / real_total_output_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subblue_water(
                time()) / real_total_output_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subgreen_water(
                time()) / real_total_output_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods_subgray_water(
                time()) / real_total_output_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subhotels_and_restaurants_subblue_water(time()) /
            real_total_output_by_sector().loc['hotels and restaurants'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subhotels_and_restaurants_subgreen_water(time()) /
            real_total_output_by_sector().loc['hotels and restaurants'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subhotels_and_restaurants_subgray_water(time()) /
            real_total_output_by_sector().loc['hotels and restaurants'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subinland_transport_subblue_water(time()) /
            real_total_output_by_sector().loc['inland transport'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subinland_transport_subgreen_water(time()) /
            real_total_output_by_sector().loc['inland transport'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subinland_transport_subgray_water(time()) /
            real_total_output_by_sector().loc['inland transport'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwater_transport_subblue_water(time()) /
            real_total_output_by_sector().loc['water transport'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwater_transport_subgreen_water(time()) /
            real_total_output_by_sector().loc['water transport'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subwater_transport_subgray_water(time()) /
            real_total_output_by_sector().loc['water transport'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subair_transport_subblue_water(time()) /
            real_total_output_by_sector().loc['air transport'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subair_transport_subgreen_water(time()) /
            real_total_output_by_sector().loc['air transport'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subair_transport_subgray_water(time()) /
            real_total_output_by_sector().loc['air transport'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subblue_water(
                time()) / real_total_output_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subgreen_water(
                time()) / real_total_output_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies_subgray_water(
                time()) / real_total_output_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpost_and_telecommunications_subblue_water(
                time()) / real_total_output_by_sector().loc['post and telecommunications'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpost_and_telecommunications_subgreen_water(
                time()) / real_total_output_by_sector().loc['post and telecommunications'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpost_and_telecommunications_subgray_water(
                time()) / real_total_output_by_sector().loc['post and telecommunications'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subfinancial_intermedation_subblue_water(time()) /
            real_total_output_by_sector().loc['financial intermedation'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subfinancial_intermedation_subgreen_water(time()) /
            real_total_output_by_sector().loc['financial intermedation'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subfinancial_intermedation_subgray_water(time()) /
            real_total_output_by_sector().loc['financial intermedation'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subreal_estate_activities_subblue_water(time()) /
            real_total_output_by_sector().loc['real estate activities'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subreal_estate_activities_subgreen_water(time()) /
            real_total_output_by_sector().loc['real estate activities'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subreal_estate_activities_subgray_water(time()) /
            real_total_output_by_sector().loc['real estate activities'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subblue_water(
                time()) / real_total_output_by_sector().loc['renting od meq and other business activities'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subgreen_water(
                time()) / real_total_output_by_sector().loc['renting od meq and other business activities'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subrenting_od_meq_and_other_business_activities_subgray_water(
                time()) / real_total_output_by_sector().loc['renting od meq and other business activities'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subblue_water(
                time()) / real_total_output_by_sector().loc['public admin and defence compulsory social security'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subgreen_water(
                time()) / real_total_output_by_sector().loc['public admin and defence compulsory social security'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subpublic_admin_and_defence_compulsory_social_security_subgray_water(
                time()) / real_total_output_by_sector().loc['public admin and defence compulsory social security'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subeducation_subblue_water(time()) /
            real_total_output_by_sector().loc['education'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subeducation_subgreen_water(time()) /
            real_total_output_by_sector().loc['education'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subeducation_subgray_water(time()) /
            real_total_output_by_sector().loc['education'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subhealth_and_social_work_subblue_water(time()) /
            real_total_output_by_sector().loc['health and social work'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subhealth_and_social_work_subgreen_water(time()) /
            real_total_output_by_sector().loc['health and social work'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subhealth_and_social_work_subgray_water(time()) /
            real_total_output_by_sector().loc['health and social work'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subblue_water(
                time()) / real_total_output_by_sector().loc['other community social and persona services'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subgreen_water(
                time()) / real_total_output_by_sector().loc['other community social and persona services'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subother_community_social_and_persona_services_subgray_water(
                time()) / real_total_output_by_sector().loc['other community social and persona services'].values, 0)],[
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subblue_water(
                time()) / real_total_output_by_sector().loc['private households with employed persons'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subgreen_water(
                time()) / real_total_output_by_sector().loc['private households with employed persons'].values, 0),
        functions.if_then_else(
            time() < 2009,
            historic_water_use_by_type_sectors_subprivate_households_with_employed_persons_subgray_water(
                time()) / real_total_output_by_sector().loc['private households with employed persons'].values, 0)]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'water color': ['blue water', 'green water', 'gray water']},
        dims=['sectors', 'water color'])
    ret = utils.align_by_dims(ret, ('sectors', 'water color'))
    #test_nans(ret, 1918)
    return ret


@cache('step')
def historic_water_by_type_intensities_for_households():
    """
    historic water by type intensities for households



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(time() < 2009,
                               historic_water_use_by_type_for_households_subblue_water(time()) /
                               household_demand_total(), 0),
        functions.if_then_else(time() < 2009,
                               historic_water_use_by_type_for_households_subgreen_water(time()) /
                               household_demand_total(), 0),
        functions.if_then_else(time() < 2009,
                               historic_water_use_by_type_for_households_subgray_water(time()) /
                               household_demand_total(), 0)],
        coords={'water color': ['blue water', 'green water', 'gray water']},
        dims=['water color'])
    #test_nans(ret, 1919)
    return ret


@cache('step')
def total_co2_emissions_gtco2():
    """
    total co2 emissions gtco2



    component


    """
    
    ret = co2_fossil_fuel_emissions() + co2x_soilxandxluc_emissions() + bioe_co2_emissions(
    ) + co2_emissions_peat() - afforestation_program_x2020x_gtco2()
    #test_nans(ret, 1920)
    return ret


@cache('step')
def adapt_growth_res_for_heat_x0x():
    """
    adapt growth res for heat x0x



    component


    """
    
    ret = functions.if_then_else(
        time() < 2014,
        past_res_growth_for_heatxdashxcom_x0(),
        functions.if_then_else(time() < 2019,
                               past_res_growth_for_heatxdashxcom_x0() +
                               (p_res_for_heat_x0x() - past_res_growth_for_heatxdashxcom_x0()) *
                               (time_dmnl(time()) - 2014) / 5, p_res_for_heat_x0x()))
    #test_nans(ret, 1921)
    return ret


@cache('step')
def adapt_growth_res_for_heatxdashxcom():
    """
    adapt growth res for heatxdashxcom



    component


    """
    
    ret = functions.if_then_else(
        time() < 2014,
        past_res_growth_for_heatxdashxcom(),
        functions.if_then_else(time() < start_year_p_growth_res_heat(),
                               past_res_growth_for_heatxdashxcom() +
                               (p_res_for_heat() - past_res_growth_for_heatxdashxcom()) *
                               (time_dmnl(time()) - 2014) /
                               (start_year_p_growth_res_heat() - 2014), p_res_for_heat()))
    #test_nans(ret, 1922)
    return ret


@cache('step')
def adapt_growth_res_for_heatxdashxnc():
    """
    adapt growth res for heatxdashxnc



    component


    """
    
    ret = functions.if_then_else(
        time() < 2014,
        past_res_growth_for_heatxdashxnc(),
        functions.if_then_else(time() < start_year_p_growth_res_heat(),
                               past_res_growth_for_heatxdashxnc() +
                               (p_res_for_heat() - past_res_growth_for_heatxdashxnc()) *
                               (time_dmnl(time()) - 2014) /
                               (start_year_p_growth_res_heat() - 2014), p_res_for_heat()))
    #test_nans(ret, 1923)
    return ret


@cache('step')
def adapt_growth_waste():
    """
    adapt growth waste



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        past_waste_growth(),
        functions.if_then_else(time() < 2020,
                               past_waste_growth() + (waste_change() - past_waste_growth()) *
                               (time_dmnl(time()) - 2015) / 5, waste_change()))
    #test_nans(ret, 1924)
    return ret


@cache('step')
def adapt_growth_biogas():
    """
    adapt growth biogas



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        past_biogas_growth(),
        functions.if_then_else(time() < 2020,
                               past_biogas_growth() + (growth_biogas() - past_biogas_growth()) *
                               (time_dmnl(time()) - 2015) / 5, growth_biogas()))
    #test_nans(ret, 1925)
    return ret


@cache('step')
def adapt_growth_phs():
    """
    adapt growth phs



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        past_phs_capacity_growth(),
        functions.if_then_else(time() < 2020,
                               past_phs_capacity_growth() +
                               (p_phs_growth() - past_phs_capacity_growth()) *
                               (time_dmnl(time()) - 2015) / 5, p_phs_growth()) *
        (1 + xxabundancexx_storage()))
    #test_nans(ret, 1926)
    return ret


@cache('step')
def adapt_growth_res_elec():
    """
    adapt growth res elec



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(
            time() < 2015,
            past_res_elec_capacity_growth().loc['hydro'].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc['hydro'].values +
                                   (p_res_elec_growth().loc['hydro'].values - past_res_elec_capacity_growth().loc['hydro'].values) *
                                   (time_dmnl(time()) - 2015) /
                                   (start_year_p_growth_res_elec() - 2015), p_res_elec_growth().loc['hydro'].values)),
        functions.if_then_else(
            time() < 2013,
            past_res_elec_capacity_growth().loc['geotxdashxelec'].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc['geotxdashxelec'].values +
                                   (p_res_elec_growth().loc['geotxdashxelec'].values - past_res_elec_capacity_growth().loc['geotxdashxelec'].values) *
                                   (time_dmnl(time()) - 2013) /
                                   (start_year_p_growth_res_elec() - 2013), p_res_elec_growth().loc['geotxdashxelec'].values)),
        functions.if_then_else(
            time() < 2013,
            past_res_elec_capacity_growth().loc['solid bioexdashxelec'].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc['solid bioexdashxelec'].values +
                                   (p_res_elec_growth().loc['solid bioexdashxelec'].values - past_res_elec_capacity_growth().loc['solid bioexdashxelec'].values) *
                                   (time_dmnl(time()) - 2013) /
                                   (start_year_p_growth_res_elec() - 2013), p_res_elec_growth().loc['solid bioexdashxelec'].values)),
        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc['oceanic'].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc['oceanic'].values +
                                   (p_res_elec_growth().loc['oceanic'].values - past_res_elec_capacity_growth().loc['oceanic'].values) *
                                   (time_dmnl(time()) - 2014) /
                                   (start_year_p_growth_res_elec() - 2014), p_res_elec_growth().loc['oceanic'].values)),
        functions.if_then_else(
            time() < 2015,
            past_res_elec_capacity_growth().loc['wind onshore'].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc['wind onshore'].values +
                                   (p_res_elec_growth().loc['wind onshore'].values - past_res_elec_capacity_growth().loc['wind onshore'].values) *
                                   (time_dmnl(time()) - 2015) /
                                   (start_year_p_growth_res_elec() - 2015), p_res_elec_growth().loc['wind onshore'].values)),
        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc['wind offshore'].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc['wind offshore'].values +
                                   (p_res_elec_growth().loc['wind offshore'].values - past_res_elec_capacity_growth().loc['wind offshore'].values) *
                                   (time_dmnl(time()) - 2014) /
                                   (start_year_p_growth_res_elec() - 2014), p_res_elec_growth().loc['wind offshore'].values)),
        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc['solar pv'].values,
            functions.if_then_else(time() < start_year_p_growth_res_elec(),
                                   past_res_elec_capacity_growth().loc['solar pv'].values +
                                   (p_res_elec_growth().loc['solar pv'].values - past_res_elec_capacity_growth().loc['solar pv'].values) *
                                   (time_dmnl(time()) - 2014) /
                                   (start_year_p_growth_res_elec() - 2014), p_res_elec_growth().loc['solar pv'].values)),
        functions.if_then_else(
            time() < 2014,
            past_res_elec_capacity_growth().loc['csp'].values,
            functions.if_then_else(
                time() < start_year_p_growth_res_elec(),
                past_res_elec_capacity_growth().loc['csp'].values +
                (p_res_elec_growth().loc['csp'].values - past_res_elec_capacity_growth().loc['csp'].values) *
                (time_dmnl(time()) - 2014) / (start_year_p_growth_res_elec() - 2014),
                p_res_elec_growth().loc['csp'].values))],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1927)
    return ret


@cache('step')
def adapt_co2_emissions_unconv_gas():
    """
    adapt co2 emissions unconv gas



    component


    """
    
    ret = functions.if_then_else(time() < 2050,
                                  0.01 + (0.22 - 0.01) * (time_dmnl(time()) - 2000) / 50,
                                  functions.if_then_else(time() < 2100, 0.22 + (0.6 - 0.22) *
                                                         (time_dmnl(time()) - 2050) / 50, 0.6))
    #test_nans(ret, 1928)
    return ret


@cache('step')
def adapt_emissions_shale_oil():
    """
    adapt emissions shale oil



    component


    """
    
    ret = functions.if_then_else(time() < 2050,
                                  0.001 + (0.15 - 0.001) * (time_dmnl(time()) - 2000) / 50,
                                  functions.if_then_else(time() < 2100, 0.15 + (0.72 - 0.15) *
                                                         (time_dmnl(time()) - 2050) / 50, 0.72))
    #test_nans(ret, 1929)
    return ret


@cache('step')
def adapt_growth_biofuels_2gen():
    """
    adapt growth biofuels 2gen



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015, 0,
        functions.if_then_else(time() < 2020,
                               past_biofuels_2gen() + (p_biofuels_2gen() - past_biofuels_2gen()) *
                               (time_dmnl(time()) - 2015) / 5, p_biofuels_2gen()))
    #test_nans(ret, 1930)
    return ret


@cache('step')
def annual_gdppc_growth_rate():
    """
    annual gdppc growth rate



    component


    """
    
    ret = functions.if_then_else(
        abs(select_gdppc_evolution_input() - 0) < equality_threshold,
        gdppc_annual_growth_ssp2(time()),
        functions.if_then_else(abs(select_gdppc_evolution_input() - 1) < equality_threshold,
                               p_timeseries_gdppc_growth_rate(time()),
                               functions.if_then_else(time() < p_customized_year_gdppc_evolution(),
                                                      p_timeseries_gdppc_growth_rate(time()),
                                                      p_customized_cte_gdppc_variation())))
    #test_nans(ret, 1931)
    return ret


@cache('step')
def variation_pop_ssp4():
    """
    variation pop ssp4



    component


    """
    
    ret = functions.if_then_else(time() < 2010, 0, -1 + pop_ssp4(time() + 1) / pop_ssp4(time()))
    #test_nans(ret, 1932)
    return ret


@cache('step')
def variation_pop_ssp5():
    """
    variation pop ssp5



    component


    """
    
    ret = functions.if_then_else(time() < 2010, 0, -1 + pop_ssp5(time() + 1) / pop_ssp5(time()))
    #test_nans(ret, 1933)
    return ret


@cache('step')
def variation_pop_ssp1():
    """
    variation pop ssp1



    component


    """
    
    ret = functions.if_then_else(time() < 2010, 0, -1 + pop_ssp1(time() + 1) / pop_ssp1(time()))
    #test_nans(ret, 1934)
    return ret


@cache('step')
def variation_pop_ssp2():
    """
    variation pop ssp2



    component


    """
    
    ret = functions.if_then_else(time() < 2010, 0, -1 + pop_ssp2(time() + 1) / pop_ssp2(time()))
    #test_nans(ret, 1935)
    return ret


@cache('step')
def variation_pop_ssp3():
    """
    variation pop ssp3



    component


    """
    
    ret = functions.if_then_else(time() < 2010, 0, -1 + pop_ssp3(time() + 1) / pop_ssp3(time()))
    #test_nans(ret, 1936)
    return ret


@cache('step')
def annual_population_growth_rate():
    """
    annual population growth rate



    component


    """
    
    ret = functions.if_then_else(
        abs(select_population_evolution_input() - 0) < equality_threshold,
        variation_pop_ssps(),
        functions.if_then_else(abs(select_population_evolution_input() - 1) < equality_threshold,
                               p_timeseries_pop_growth_rate(time()),
                               functions.if_then_else(time() < p_customized_year_pop_evolution(),
                                                      p_timeseries_pop_growth_rate(time()),
                                                      p_customized_cte_pop_variation())))
    #test_nans(ret, 1937)
    return ret


@cache('step')
def variation_pop_ssps():
    """
    variation pop ssps



    component


    """
    
    ret = functions.if_then_else(abs(select_pop_ssps() - 1) < equality_threshold,
                                  variation_pop_ssp1(),
                                  functions.if_then_else(abs(select_pop_ssps() - 2) < equality_threshold,
                                                         variation_pop_ssp2(),
                                                         functions.if_then_else(
                                                             abs(select_pop_ssps() - 3) < equality_threshold,
                                                             variation_pop_ssp3(),
                                                             functions.if_then_else(
                                                                 abs(select_pop_ssps() - 4) < equality_threshold,
                                                                 variation_pop_ssp4(),
                                                                 variation_pop_ssp5()))))
    #test_nans(ret, 1938)
    return ret


@cache('step')
def desired_variation_gdppc():
    """
    desired variation gdppc



    component


    """
    
    ret = functions.if_then_else(time() < 2015,
                                  variation_historic_gdppc(),
                                  desired_gdppc() * annual_gdppc_growth_rate())
    #test_nans(ret, 1939)
    return ret


@cache('step')
def pop_variation():
    """
    pop variation



    component


    """
    
    ret = functions.if_then_else(time() < 2014,
                                  variation_historic_pop(),
                                  population() * annual_population_growth_rate())
    #test_nans(ret, 1940)
    return ret


@cache('step')
def variation_historic_gdppc():
    """
    variation historic gdppc



    component


    """
    
    ret = functions.if_then_else(time() < 2015,
                                  historic_gdppc(time() + 1) - historic_gdppc(time()), 0)
    #test_nans(ret, 1941)
    return ret


@cache('step')
def growth_labour_share():
    """
    growth labour share



    component


    """
    
    ret = functions.if_then_else(time() > 2014,
                                  functions.if_then_else(
                                      time() > 2050, 0,
                                      labour_share_growth() * labor_share_ctexquestx()),
                                  historic_labour_share(time()))
    #test_nans(ret, 1942)
    return ret


@cache('step')
def historic_new_required_capacity_phs():
    """
    historic new required capacity phs



    component


    """
    
    ret = table_hist_capacity_phs(time() + total_time_planxplusxconstr_res_elec().loc['hydro'].values + 1) - table_hist_capacity_phs(time() + total_time_planxplusxconstr_res_elec().loc['hydro'].values)
    #test_nans(ret, 1943)
    return ret


@cache('run')
def initial_required_capacity_phs():
    """
    initial required capacity phs



    constant


    """
    
    ret = table_hist_capacity_phs(1996) - table_hist_capacity_phs(1995)
    #test_nans(ret, 1944)
    return ret


@cache('step')
def fed_coal_for_heatxdashxnc():
    """
    fed coal for heatxdashxnc



    component


    """
    
    ret = required_fed_by_fuel_before_heat_correction().loc['solids'].values * (
        share_feh_over_fed_by_final_fuel().loc['solids'].values - share_feh_over_fed_solid_bioe()
    ) * efficiency_coal_for_heat_plants(time()) / (1 + share_heat_distribution_losses())
    #test_nans(ret, 1945)
    return ret


@cache('step')
def historical_variation_minerals_extraction_rest():
    """
    historical variation minerals extraction rest



    component


    """
    
    ret = xr.DataArray(data=[
        historical_extraction_minerals_rest_subadhesive(time() + 1) -
        historical_extraction_minerals_rest_subadhesive(time()),
        historical_extraction_minerals_rest_subaluminium(time() + 1) -
        historical_extraction_minerals_rest_subaluminium(time()),
        historical_extraction_minerals_rest_subaluminium_mirrors(time() + 1) -
        historical_extraction_minerals_rest_subaluminium_mirrors(time()),
        historical_extraction_minerals_rest_subcadmium(time() + 1) -
        historical_extraction_minerals_rest_subcadmium(time()),
        historical_extraction_minerals_rest_subcarbon_fiber(time() + 1) -
        historical_extraction_minerals_rest_subcarbon_fiber(time()),
        historical_extraction_minerals_rest_subcement(time() + 1) -
        historical_extraction_minerals_rest_subcement(time()),
        historical_extraction_minerals_rest_subchromium(time() + 1) -
        historical_extraction_minerals_rest_subchromium(time()),
        historical_extraction_minerals_rest_subcopper(time() + 1) -
        historical_extraction_minerals_rest_subcopper(time()),
        historical_extraction_minerals_rest_subdiesel(time() + 1) -
        historical_extraction_minerals_rest_subdiesel(time()),
        historical_extraction_minerals_rest_subdy(time() + 1) -
        historical_extraction_minerals_rest_subdy(time()),
        historical_extraction_minerals_rest_subelectricxdivxelectronic_components(time() + 1) -
        historical_extraction_minerals_rest_subelectricxdivxelectronic_components(time()),
        historical_extraction_minerals_rest_subevacuation_lines(time() + 1) -
        historical_extraction_minerals_rest_subevacuation_lines(time()),
        historical_extraction_minerals_rest_subfiberglass(time() + 1) -
        historical_extraction_minerals_rest_subfiberglass(time()),
        historical_extraction_minerals_rest_subfoam_glass(time() + 1) -
        historical_extraction_minerals_rest_subfoam_glass(time()),
        historical_extraction_minerals_rest_subgalium(time() + 1) -
        historical_extraction_minerals_rest_subgalium(time()),
        historical_extraction_minerals_rest_subglass(time() + 1) -
        historical_extraction_minerals_rest_subglass(time()),
        historical_extraction_minerals_rest_subglass_reinforcing_plastic(time() + 1) -
        historical_extraction_minerals_rest_subglass_reinforcing_plastic(time()),
        historical_extraction_minerals_rest_subgravel(time() + 1) -
        historical_extraction_minerals_rest_subgravel(time()),
        historical_extraction_minerals_rest_subindium(time() + 1) -
        historical_extraction_minerals_rest_subindium(time()),
        historical_extraction_minerals_rest_subiron(time() + 1) -
        historical_extraction_minerals_rest_subiron(time()),
        historical_extraction_minerals_rest_subkno3_mined(time() + 1) -
        historical_extraction_minerals_rest_subkno3_mined(time()),
        historical_extraction_minerals_rest_subasphalt(time() + 1) -
        historical_extraction_minerals_rest_subasphalt(time()),
        historical_extraction_minerals_rest_sublime(time() + 1) -
        historical_extraction_minerals_rest_sublime(time()),
        historical_extraction_minerals_rest_sublimestone(time() + 1) -
        historical_extraction_minerals_rest_sublimestone(time()),
        historical_extraction_minerals_rest_sublithium(time() + 1) -
        historical_extraction_minerals_rest_sublithium(time()),
        historical_extraction_minerals_rest_sublubricant(time() + 1) -
        historical_extraction_minerals_rest_sublubricant(time()),
        historical_extraction_minerals_rest_submagnesium(time() + 1) -
        historical_extraction_minerals_rest_submagnesium(time()),
        historical_extraction_minerals_rest_submanganese(time() + 1) -
        historical_extraction_minerals_rest_submanganese(time()),
        historical_extraction_minerals_rest_subheavy_equipment(time() + 1) -
        historical_extraction_minerals_rest_subheavy_equipment(time()),
        historical_extraction_minerals_rest_subconcrete(time() + 1) -
        historical_extraction_minerals_rest_subconcrete(time()),
        historical_extraction_minerals_rest_submolybdenum(time() + 1) -
        historical_extraction_minerals_rest_submolybdenum(time()),
        historical_extraction_minerals_rest_subnano3_mined(time() + 1) -
        historical_extraction_minerals_rest_subnano3_mined(time()),
        historical_extraction_minerals_rest_subnano3_synthetic(time() + 1) -
        historical_extraction_minerals_rest_subnano3_synthetic(time()),
        historical_extraction_minerals_rest_subneodymium(time() + 1) -
        historical_extraction_minerals_rest_subneodymium(time()),
        historical_extraction_minerals_rest_subnickel(time() + 1) -
        historical_extraction_minerals_rest_subnickel(time()),
        historical_extraction_minerals_rest_subover_grid_x15xpercxx(time() + 1) -
        historical_extraction_minerals_rest_subover_grid_x15xpercxx(time()),
        historical_extraction_minerals_rest_subover_grid_x5xpercxx(time() + 1) -
        historical_extraction_minerals_rest_subover_grid_x5xpercxx(time()),
        historical_extraction_minerals_rest_subpaint(time() + 1) -
        historical_extraction_minerals_rest_subpaint(time()),
        historical_extraction_minerals_rest_sublead(time() + 1) -
        historical_extraction_minerals_rest_sublead(time()),
        historical_extraction_minerals_rest_subplastics(time() + 1) -
        historical_extraction_minerals_rest_subplastics(time()),
        historical_extraction_minerals_rest_subpolypropylene(time() + 1) -
        historical_extraction_minerals_rest_subpolypropylene(time()),
        historical_extraction_minerals_rest_subrock(time() + 1) -
        historical_extraction_minerals_rest_subrock(time()),
        historical_extraction_minerals_rest_subrock_wool(time() + 1) -
        historical_extraction_minerals_rest_subrock_wool(time()),
        historical_extraction_minerals_rest_subsand(time() + 1) -
        historical_extraction_minerals_rest_subsand(time()),
        historical_extraction_minerals_rest_subsilicon_sand(time() + 1) -
        historical_extraction_minerals_rest_subsilicon_sand(time()),
        historical_extraction_minerals_rest_subsilicon_wafer_modules(time() + 1) -
        historical_extraction_minerals_rest_subsilicon_wafer_modules(time()),
        historical_extraction_minerals_rest_subsilver(time() + 1) -
        historical_extraction_minerals_rest_subsilver(time()),
        historical_extraction_minerals_rest_subsite_preparation(time() + 1) -
        historical_extraction_minerals_rest_subsite_preparation(time()),
        historical_extraction_minerals_rest_subtin(time() + 1) -
        historical_extraction_minerals_rest_subtin(time()),
        historical_extraction_minerals_rest_subsoda_ash(time() + 1) -
        historical_extraction_minerals_rest_subsoda_ash(time()),
        historical_extraction_minerals_rest_substeel(time() + 1) -
        historical_extraction_minerals_rest_substeel(time()),
        historical_extraction_minerals_rest_subsynthetic_oil(time() + 1) -
        historical_extraction_minerals_rest_subsynthetic_oil(time()),
        historical_extraction_minerals_rest_subtellurium(time() + 1) -
        historical_extraction_minerals_rest_subtellurium(time()),
        historical_extraction_minerals_rest_subtitanium(time() + 1) -
        historical_extraction_minerals_rest_subtitanium(time()),
        historical_extraction_minerals_rest_subtitanium_dioxide(time() + 1) -
        historical_extraction_minerals_rest_subtitanium_dioxide(time()),
        historical_extraction_minerals_rest_subvanadium(time() + 1) -
        historical_extraction_minerals_rest_subvanadium(time()),
        historical_extraction_minerals_rest_subwires(time() + 1) -
        historical_extraction_minerals_rest_subwires(time()),
        historical_extraction_minerals_rest_subzinc(time() + 1) -
        historical_extraction_minerals_rest_subzinc(time())],
        coords={'materials': ['adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement', 'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components', 'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass', 'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt', 'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese', 'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic', 'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint', 'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand', 'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel', 'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires', 'zinc']},
        dims=['materials'])
    #test_nans(ret, 1946)
    return ret


@cache('step')
def max_unconv_gas_growth_extraction():
    """
    max unconv gas growth extraction



    component


    """
    
    ret = np.maximum(0, 1 + functions.if_then_else(
        abs(selection_constraint_extraction_unconv_gas() - 1) < equality_threshold,
        (p_constraint_growth_extraction_unconv_gas()) * time_step() * scarcity_conv_gas_stock(),
        (userxdashxdefined_extraction_growth_unconv_gas(time())) * time_step()))
    #test_nans(ret, 1947)
    return ret


@cache('step')
def new_res_capacity_for_heatxdashxnc_tw():
    """
    new res capacity for heatxdashxnc tw



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(
            time() < 2013,
            historic_res_capacity_for_heatxdashxnc_subsolarxdashxheat(time() + 1) -
            historic_res_capacity_for_heatxdashxnc_subsolarxdashxheat(time()),
            adapt_growth_res_for_heatxdashxnc().loc['solarxdashxheat'].values * installed_capacity_res_heatxdashxnc_tw().loc['solarxdashxheat'].values * remaining_potential_constraint_on_new_res_heat_capacity().loc['solarxdashxheat'].values) *
        abundance_res_heatxdashxnc2(),
        functions.if_then_else(
            time() < 2013,
            historic_res_capacity_for_heatxdashxnc_subgeotxdashxheat(time() + 1) -
            historic_res_capacity_for_heatxdashxnc_subgeotxdashxheat(time()),
            adapt_growth_res_for_heatxdashxnc().loc['geotxdashxheat'].values * installed_capacity_res_heatxdashxnc_tw().loc['geotxdashxheat'].values * remaining_potential_constraint_on_new_res_heat_capacity().loc['geotxdashxheat'].values) *
        abundance_res_heatxdashxnc2(),
        functions.if_then_else(
            time() < 2013,
            historic_res_capacity_for_heatxdashxnc_subsolid_bioexdashxheat(time() + 1) -
            historic_res_capacity_for_heatxdashxnc_subsolid_bioexdashxheat(time()),
            adapt_growth_res_for_heatxdashxnc().loc['solid bioexdashxheat'].values * installed_capacity_res_heatxdashxnc_tw().loc['solid bioexdashxheat'].values * remaining_potential_constraint_on_new_res_heat_capacity().loc['solid bioexdashxheat'].values) *
        abundance_res_heatxdashxnc2()],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1948)
    return ret


@cache('step')
def annual_variation_nonxdashxenergy_use():
    """
    annual variation nonxdashxenergy use



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(time() > 2009,
                               variation_nonxdashxenergy_use().loc['electricity'].values,
                               historic_nonxdashxenergy_use_subelectricity(time() + 1) -
                               historic_nonxdashxenergy_use_subelectricity(time())),
        functions.if_then_else(time() > 2009,
                               variation_nonxdashxenergy_use().loc['heat'].values,
                               historic_nonxdashxenergy_use_subheat(time() + 1) -
                               historic_nonxdashxenergy_use_subheat(time())),
        functions.if_then_else(time() > 2009,
                               variation_nonxdashxenergy_use().loc['liquids'].values,
                               historic_nonxdashxenergy_use_subliquids(time() + 1) -
                               historic_nonxdashxenergy_use_subliquids(time())),
        functions.if_then_else(time() > 2009,
                               variation_nonxdashxenergy_use().loc['gases'].values,
                               historic_nonxdashxenergy_use_subgases(time() + 1) -
                               historic_nonxdashxenergy_use_subgases(time())),
        functions.if_then_else(time() > 2009,
                               variation_nonxdashxenergy_use().loc['solids'].values,
                               historic_nonxdashxenergy_use_subsolids(time() + 1) -
                               historic_nonxdashxenergy_use_subsolids(time()))],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1949)
    return ret


@cache('step')
def pes_solids_bioe_ej():
    """
    pes solids bioe ej



    component


    """
    
    ret = losses_in_charcoal_plants_ej(
        time()) + pe_bioe_for_elec_generation_ej() + pe_traditional_biomass_ej_delayed_1yr(
        ) + pes_res_for_heatxdashxcom_by_techn().loc['solid bioexdashxheat'].values + pes_res_for_heatxdashxnc_by_techn().loc['solid bioexdashxheat'].values
    #test_nans(ret, 1950)
    return ret


@cache('step')
def real_fe_consumption_solids_ej():
    """
    real fe consumption solids ej



    component


    """
    
    ret = (
        extraction_coal_ej() +
        (pe_traditional_biomass_ej_delayed_1yr() + pes_waste_for_tfc() + pes_peat_ej() +
         losses_in_charcoal_plants_ej(time())) - ped_coal_for_ctl_ej() - other_solids_required()
    ) * share_solids_for_final_energy()
    #test_nans(ret, 1951)
    return ret


@cache('step')
def variation_demand_flow_fd():
    """
    variation demand flow fd



    component


    """
    
    ret = functions.if_then_else(
        time() < 2009,
        historic_variation_demand(),
        (gross_fixed_capital_formation() *
         (1 - ((1 - share_consum_goverments_and_inventories_next_step()) /
               (1 - share_consum_goverment_and_inventories()))) + household_demand() *
         (1 - ((1 - share_consum_goverments_and_inventories_next_step()) /
               (1 - share_consum_goverment_and_inventories()))) + variation_gfcf() +
         variation_household_demand()) / (1 - share_consum_goverments_and_inventories_next_step()))
    #test_nans(ret, 1952)
    return ret


@cache('step')
def historic_variation_demand():
    """
    historic variation demand



    component


    """
    
    ret = xr.DataArray(data=[
        historic_demand_by_sector_subagriculture_hunting_forestry_and_fishing(time() + 1) -
        historic_demand_by_sector_subagriculture_hunting_forestry_and_fishing(time()),
        historic_demand_by_sector_submining_and_quarrying(time() + 1) -
        historic_demand_by_sector_submining_and_quarrying(time()),
        historic_demand_by_sector_subfood_beverages_and_tobacco(time() + 1) -
        historic_demand_by_sector_subfood_beverages_and_tobacco(time()),
        historic_demand_by_sector_subtextiles_and_textile_products(time() + 1) -
        historic_demand_by_sector_subtextiles_and_textile_products(time()),
        historic_demand_by_sector_subleather_leather_and_footwear(time() + 1) -
        historic_demand_by_sector_subleather_leather_and_footwear(time()),
        historic_demand_by_sector_subwood_and_products_of_woood_and_cork(time() + 1) -
        historic_demand_by_sector_subwood_and_products_of_woood_and_cork(time()),
        historic_demand_by_sector_subpulp_paper_printing_and_publishing(time() + 1) -
        historic_demand_by_sector_subpulp_paper_printing_and_publishing(time()),
        historic_demand_by_sector_subcoke_refined_petroleum_and_nuclear_fuel(time() + 1) -
        historic_demand_by_sector_subcoke_refined_petroleum_and_nuclear_fuel(time()),
        historic_demand_by_sector_subchemicals_and_chemical_products(time() + 1) -
        historic_demand_by_sector_subchemicals_and_chemical_products(time()),
        historic_demand_by_sector_subrubber_and_plastics(time() + 1) -
        historic_demand_by_sector_subrubber_and_plastics(time()),
        historic_demand_by_sector_subother_non_metalic_mineral(time() + 1) -
        historic_demand_by_sector_subother_non_metalic_mineral(time()),
        historic_demand_by_sector_subbasic_metals_and_fabricated_metal(time() + 1) -
        historic_demand_by_sector_subbasic_metals_and_fabricated_metal(time()),
        historic_demand_by_sector_submachinery_nec(time() + 1) -
        historic_demand_by_sector_submachinery_nec(time()),
        historic_demand_by_sector_subelectrical_and_optical_equipment(time() + 1) -
        historic_demand_by_sector_subelectrical_and_optical_equipment(time()),
        historic_demand_by_sector_subtransport_equipment(time() + 1) -
        historic_demand_by_sector_subtransport_equipment(time()),
        historic_demand_by_sector_submanufacturing_nec_recycling(time() + 1) -
        historic_demand_by_sector_submanufacturing_nec_recycling(time()),
        historic_demand_by_sector_subelectricity_gas_and_water_supply(time() + 1) -
        historic_demand_by_sector_subelectricity_gas_and_water_supply(time()),
        historic_demand_by_sector_subconstruction(time() + 1) -
        historic_demand_by_sector_subconstruction(time()),
        historic_demand_by_sector_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_demand_by_sector_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        historic_demand_by_sector_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_demand_by_sector_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        historic_demand_by_sector_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_demand_by_sector_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        historic_demand_by_sector_subhotels_and_restaurants(time() + 1) -
        historic_demand_by_sector_subhotels_and_restaurants(time()),
        historic_demand_by_sector_subinland_transport(time() + 1) -
        historic_demand_by_sector_subinland_transport(time()),
        historic_demand_by_sector_subwater_transport(time() + 1) -
        historic_demand_by_sector_subwater_transport(time()),
        historic_demand_by_sector_subair_transport(time() + 1) -
        historic_demand_by_sector_subair_transport(time()),
        historic_demand_by_sector_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_demand_by_sector_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        historic_demand_by_sector_subpost_and_telecommunications(time() + 1) -
        historic_demand_by_sector_subpost_and_telecommunications(time()),
        historic_demand_by_sector_subfinancial_intermedation(time() + 1) -
        historic_demand_by_sector_subfinancial_intermedation(time()),
        historic_demand_by_sector_subreal_estate_activities(time() + 1) -
        historic_demand_by_sector_subreal_estate_activities(time()),
        historic_demand_by_sector_subrenting_od_meq_and_other_business_activities(time() + 1) -
        historic_demand_by_sector_subrenting_od_meq_and_other_business_activities(time()),
        historic_demand_by_sector_subpublic_admin_and_defence_compulsory_social_security(
            time() + 1) -
        historic_demand_by_sector_subpublic_admin_and_defence_compulsory_social_security(time()),
        historic_demand_by_sector_subeducation(time() + 1) -
        historic_demand_by_sector_subeducation(time()),
        historic_demand_by_sector_subhealth_and_social_work(time() + 1) -
        historic_demand_by_sector_subhealth_and_social_work(time()),
        historic_demand_by_sector_subother_community_social_and_persona_services(time() + 1) -
        historic_demand_by_sector_subother_community_social_and_persona_services(time()),
        historic_demand_by_sector_subprivate_households_with_employed_persons(time() + 1) -
        historic_demand_by_sector_subprivate_households_with_employed_persons(time())],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 1953)
    return ret


@cache('step')
def historic_rate_final_energy_intensity_h():
    """
    historic rate final energy intensity h



    component


    """
    
    ret = xr.DataArray(data=[
        historic_final_energy_intensity_h_subelectricity(time() + 1) -
        historic_final_energy_intensity_h_subelectricity(time()),
        historic_final_energy_intensity_h_subheat(time() + 1) -
        historic_final_energy_intensity_h_subheat(time()),
        historic_final_energy_intensity_h_subliquids(time() + 1) -
        historic_final_energy_intensity_h_subliquids(time()),
        historic_final_energy_intensity_h_subgases(time() + 1) -
        historic_final_energy_intensity_h_subgases(time()),
        historic_final_energy_intensity_h_subsolids(time() + 1) -
        historic_final_energy_intensity_h_subsolids(time())],
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 1954)
    return ret


@cache('step')
def fed_natx_gas_for_heatxdashxnc():
    """
    fed natx gas for heatxdashxnc



    component


    """
    
    ret = required_fed_by_fuel_before_heat_correction().loc['gases'].values * share_feh_over_fed_by_final_fuel().loc['gases'].values * efficiency_gases_for_heat_plants(time()) / (1 + share_heat_distribution_losses())
    #test_nans(ret, 1955)
    return ret


@cache('step')
def fed_oil_for_heatxdashxnc():
    """
    fed oil for heatxdashxnc



    component


    """
    
    ret = required_fed_by_fuel_before_heat_correction().loc['liquids'].values * share_feh_over_fed_by_final_fuel().loc['liquids'].values * efficiency_liquids_for_heat_plants(time()) / (1 + share_heat_distribution_losses())
    #test_nans(ret, 1956)
    return ret


@cache('step')
def pes_solids():
    """
    pes solids



    component


    """
    
    ret = extraction_coal_ej() + pe_traditional_biomass_ej_delayed_1yr() + pes_peat_ej(
    ) + pes_waste_for_tfc() + losses_in_charcoal_plants_ej(time())
    #test_nans(ret, 1957)
    return ret


@cache('step')
def ped_gas_heatxdashxnc():
    """
    ped gas heatxdashxnc



    component


    """
    
    ret = total_fed_nre_heatxdashxnc() * share_fed_gas_vs_nre_heatxdashxnc(
    ) / efficiency_gases_for_heat_plants(time())
    #test_nans(ret, 1958)
    return ret


@cache('step')
def ped_coal_heatxdashxnc():
    """
    ped coal heatxdashxnc



    component


    """
    
    ret = total_fed_nre_heatxdashxnc() * share_fed_coal_vs_nre_heatxdashxnc(
    ) / efficiency_coal_for_heat_plants(time())
    #test_nans(ret, 1959)
    return ret


@cache('step')
def ped_liquids_heatxdashxnc():
    """
    ped liquids heatxdashxnc



    component


    """
    
    ret = total_fed_nre_heatxdashxnc() * share_fed_liquids_vs_nre_heatxdashxnc(
    ) / efficiency_liquids_for_heat_plants(time())
    #test_nans(ret, 1960)
    return ret


@cache('step')
def ped_coal_ej():
    """
    ped coal ej



    component


    """
    
    ret = np.maximum(0,
                      ped_solids() - (pe_traditional_biomass_ej_delayed_1yr() + pes_peat_ej() +
                                      pes_waste_for_tfc() + losses_in_charcoal_plants_ej(time())))
    #test_nans(ret, 1961)
    return ret


@cache('step')
def fes_heat_from_coal():
    """
    fes heat from coal



    component


    """
    
    ret = (pes_coal_for_heatxdashxcom_plants() + pes_coal_for_heatxdashxnc_plants()
            ) * efficiency_coal_for_heat_plants(time())
    #test_nans(ret, 1962)
    return ret


@cache('step')
def fes_heat_from_natx_gas():
    """
    fes heat from natx gas



    component


    """
    
    ret = (pes_natx_gas_for_heatxdashxcom_plants() + pes_natx_gas_for_heatxdashxnc_plants()
            ) * efficiency_gases_for_heat_plants(time())
    #test_nans(ret, 1963)
    return ret


@cache('step')
def fes_heat_from_oil():
    """
    fes heat from oil



    component


    """
    
    ret = (pes_oil_for_heatxdashxcom_plants() + pes_oil_for_heatxdashxnc_plants()
            ) * efficiency_liquids_for_heat_plants(time())
    #test_nans(ret, 1964)
    return ret


@cache('step')
def new_res_capacity_for_heatxdashxcom_tw():
    """
    new res capacity for heatxdashxcom tw



    component


    """
    
    ret = xr.DataArray(data=[
        functions.if_then_else(
            time() < 2013,
            historic_res_capacity_for_heatxdashxcom_subsolarxdashxheat(time() + 1) -
            historic_res_capacity_for_heatxdashxcom_subsolarxdashxheat(time()),
            adapt_growth_res_for_heatxdashxcom().loc['solarxdashxheat'].values * installed_capacity_res_heatxdashxcom_tw().loc['solarxdashxheat'].values * remaining_potential_constraint_on_new_res_heat_capacity().loc['solarxdashxheat'].values) *
        abundance_res_heatxdashxcom2(),
        functions.if_then_else(
            time() < 2013,
            historic_res_capacity_for_heatxdashxcom_subgeotxdashxheat(time() + 1) -
            historic_res_capacity_for_heatxdashxcom_subgeotxdashxheat(time()),
            adapt_growth_res_for_heatxdashxcom().loc['geotxdashxheat'].values * installed_capacity_res_heatxdashxcom_tw().loc['geotxdashxheat'].values * remaining_potential_constraint_on_new_res_heat_capacity().loc['geotxdashxheat'].values) *
        abundance_res_heatxdashxcom2(),
        functions.if_then_else(
            time() < 2013,
            historic_res_capacity_for_heatxdashxcom_subsolid_bioexdashxheat(time() + 1) -
            historic_res_capacity_for_heatxdashxcom_subsolid_bioexdashxheat(time()),
            adapt_growth_res_for_heatxdashxcom().loc['solid bioexdashxheat'].values * installed_capacity_res_heatxdashxcom_tw().loc['solid bioexdashxheat'].values * remaining_potential_constraint_on_new_res_heat_capacity().loc['solid bioexdashxheat'].values) *
        abundance_res_heatxdashxcom2()],
        coords={'res heat': ['solarxdashxheat', 'geotxdashxheat', 'solid bioexdashxheat']},
        dims=['res heat'])
    #test_nans(ret, 1965)
    return ret


@cache('step')
def variation_historic_demand():
    """
    variation historic demand



    component


    """
    
    ret = xr.DataArray(data=[
        historic_hd_subagriculture_hunting_forestry_and_fishing(time() + 1) -
        historic_hd_subagriculture_hunting_forestry_and_fishing(time()),
        historic_hd_submining_and_quarrying(time() + 1) -
        historic_hd_submining_and_quarrying(time()),
        historic_hd_subfood_beverages_and_tobacco(time() + 1) -
        historic_hd_subfood_beverages_and_tobacco(time()),
        historic_hd_subtextiles_and_textile_products(time() + 1) -
        historic_hd_subtextiles_and_textile_products(time()),
        historic_hd_subleather_leather_and_footwear(time() + 1) -
        historic_hd_subleather_leather_and_footwear(time()),
        historic_hd_subwood_and_products_of_woood_and_cork(time() + 1) -
        historic_hd_subwood_and_products_of_woood_and_cork(time()),
        historic_hd_subpulp_paper_printing_and_publishing(time() + 1) -
        historic_hd_subpulp_paper_printing_and_publishing(time()),
        historic_hd_subcoke_refined_petroleum_and_nuclear_fuel(time() + 1) -
        historic_hd_subcoke_refined_petroleum_and_nuclear_fuel(time()),
        historic_hd_subchemicals_and_chemical_products(time() + 1) -
        historic_hd_subchemicals_and_chemical_products(time()),
        historic_hd_subrubber_and_plastics(time() + 1) -
        historic_hd_subrubber_and_plastics(time()),
        historic_hd_subother_non_metalic_mineral(time() + 1) -
        historic_hd_subother_non_metalic_mineral(time()),
        historic_hd_subbasic_metals_and_fabricated_metal(time() + 1) -
        historic_hd_subbasic_metals_and_fabricated_metal(time()),
        historic_hd_submachinery_nec(time() + 1) - historic_hd_submachinery_nec(time()),
        historic_hd_subelectrical_and_optical_equipment(time() + 1) -
        historic_hd_subelectrical_and_optical_equipment(time()),
        historic_hd_subtransport_equipment(time() + 1) -
        historic_hd_subtransport_equipment(time()),
        historic_hd_submanufacturing_nec_recycling(time() + 1) -
        historic_hd_submanufacturing_nec_recycling(time()),
        historic_hd_subelectricity_gas_and_water_supply(time() + 1) -
        historic_hd_subelectricity_gas_and_water_supply(time()),
        historic_hd_subconstruction(time() + 1) - historic_hd_subconstruction(time()),
        historic_hd_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_hd_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        historic_hd_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_hd_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        historic_hd_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_hd_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        historic_hd_subhotels_and_restaurants(time() + 1) -
        historic_hd_subhotels_and_restaurants(time()),
        historic_hd_subinland_transport(time() + 1) - historic_hd_subinland_transport(time()),
        historic_hd_subwater_transport(time() + 1) - historic_hd_subwater_transport(time()),
        historic_hd_subair_transport(time() + 1) - historic_hd_subair_transport(time()),
        historic_hd_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_hd_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        historic_hd_subpost_and_telecommunications(time() + 1) -
        historic_hd_subpost_and_telecommunications(time()),
        historic_hd_subfinancial_intermedation(time() + 1) -
        historic_hd_subfinancial_intermedation(time()),
        historic_hd_subreal_estate_activities(time() + 1) -
        historic_hd_subreal_estate_activities(time()),
        historic_hd_subrenting_od_meq_and_other_business_activities(time() + 1) -
        historic_hd_subrenting_od_meq_and_other_business_activities(time()),
        historic_hd_subpublic_admin_and_defence_compulsory_social_security(time() + 1) -
        historic_hd_subpublic_admin_and_defence_compulsory_social_security(time()),
        historic_hd_subeducation(time() + 1) - historic_hd_subeducation(time()),
        historic_hd_subhealth_and_social_work(time() + 1) -
        historic_hd_subhealth_and_social_work(time()),
        historic_hd_subother_community_social_and_persona_services(time() + 1) -
        historic_hd_subother_community_social_and_persona_services(time()),
        historic_hd_subprivate_households_with_employed_persons(time() + 1) -
        historic_hd_subprivate_households_with_employed_persons(time())],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 1966)
    return ret


@cache('step')
def variation_gfcf():
    """
    variation gfcf



    component


    """
    
    ret = functions.if_then_else(time() < 2009,
                                  variation_historic_gfcf(),
                                  np.exp(beta_x0x_cap()) * np.exp(beta_x1x_cap()) *
                                  ((cc_sectoral_next_step())**beta_x2x_cap() -
                                   (cc_sectoral())**beta_x2x_cap()))
    #test_nans(ret, 1967)
    return ret


@cache('step')
def variation_historic_gfcf():
    """
    variation historic gfcf



    component


    """
    
    ret = xr.DataArray(data=[
        historic_gfcf_subagriculture_hunting_forestry_and_fishing(time() + 1) -
        historic_gfcf_subagriculture_hunting_forestry_and_fishing(time()),
        historic_gfcf_submining_and_quarrying(time() + 1) -
        historic_gfcf_submining_and_quarrying(time()),
        historic_gfcf_subfood_beverages_and_tobacco(time() + 1) -
        historic_gfcf_subfood_beverages_and_tobacco(time()),
        historic_gfcf_subtextiles_and_textile_products(time() + 1) -
        historic_gfcf_subtextiles_and_textile_products(time()),
        historic_gfcf_subleather_leather_and_footwear(time() + 1) -
        historic_gfcf_subleather_leather_and_footwear(time()),
        historic_gfcf_subwood_and_products_of_woood_and_cork(time() + 1) -
        historic_gfcf_subwood_and_products_of_woood_and_cork(time()),
        historic_gfcf_subpulp_paper_printing_and_publishing(time() + 1) -
        historic_gfcf_subpulp_paper_printing_and_publishing(time()),
        historic_gfcf_subcoke_refined_petroleum_and_nuclear_fuel(time() + 1) -
        historic_gfcf_subcoke_refined_petroleum_and_nuclear_fuel(time()),
        historic_gfcf_subchemicals_and_chemical_products(time() + 1) -
        historic_gfcf_subchemicals_and_chemical_products(time()),
        historic_gfcf_subrubber_and_plastics(time() + 1) -
        historic_gfcf_subrubber_and_plastics(time()),
        historic_gfcf_subother_non_metalic_mineral(time() + 1) -
        historic_gfcf_subother_non_metalic_mineral(time()),
        historic_gfcf_subbasic_metals_and_fabricated_metal(time() + 1) -
        historic_gfcf_subbasic_metals_and_fabricated_metal(time()),
        historic_gfcf_submachinery_nec(time() + 1) - historic_gfcf_submachinery_nec(time()),
        historic_gfcf_subelectrical_and_optical_equipment(time() + 1) -
        historic_gfcf_subelectrical_and_optical_equipment(time()),
        historic_gfcf_subtransport_equipment(time() + 1) -
        historic_gfcf_subtransport_equipment(time()),
        historic_gfcf_submanufacturing_nec_recycling(time() + 1) -
        historic_gfcf_submanufacturing_nec_recycling(time()),
        historic_gfcf_subelectricity_gas_and_water_supply(time() + 1) -
        historic_gfcf_subelectricity_gas_and_water_supply(time()),
        historic_gfcf_subconstruction(time() + 1) - historic_gfcf_subconstruction(time()),
        historic_gfcf_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_gfcf_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        historic_gfcf_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_gfcf_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        historic_gfcf_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_gfcf_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        historic_gfcf_subhotels_and_restaurants(time() + 1) -
        historic_gfcf_subhotels_and_restaurants(time()),
        historic_gfcf_subinland_transport(time() + 1) - historic_gfcf_subinland_transport(time()),
        historic_gfcf_subwater_transport(time() + 1) - historic_gfcf_subwater_transport(time()),
        historic_gfcf_subair_transport(time() + 1) - historic_gfcf_subair_transport(time()),
        historic_gfcf_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_gfcf_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        historic_gfcf_subpost_and_telecommunications(time() + 1) -
        historic_gfcf_subpost_and_telecommunications(time()),
        historic_gfcf_subfinancial_intermedation(time() + 1) -
        historic_gfcf_subfinancial_intermedation(time()),
        historic_gfcf_subreal_estate_activities(time() + 1) -
        historic_gfcf_subreal_estate_activities(time()),
        historic_gfcf_subrenting_od_meq_and_other_business_activities(time() + 1) -
        historic_gfcf_subrenting_od_meq_and_other_business_activities(time()),
        historic_gfcf_subpublic_admin_and_defence_compulsory_social_security(time() + 1) -
        historic_gfcf_subpublic_admin_and_defence_compulsory_social_security(time()),
        historic_gfcf_subeducation(time() + 1) - historic_gfcf_subeducation(time()),
        historic_gfcf_subhealth_and_social_work(time() + 1) -
        historic_gfcf_subhealth_and_social_work(time()),
        historic_gfcf_subother_community_social_and_persona_services(time() + 1) -
        historic_gfcf_subother_community_social_and_persona_services(time()),
        historic_gfcf_subprivate_households_with_employed_persons(time() + 1) -
        historic_gfcf_subprivate_households_with_employed_persons(time())],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 1968)
    return ret


@cache('step')
def share_consum_goverment_and_inventories():
    """
    share consum goverment and inventories



    component


    """
    
    ret = xr.DataArray(data=[
        share_inventories_subagriculture_hunting_forestry_and_fishing(time()) +
        share_consum_goverments_subagriculture_hunting_forestry_and_fishing(time()),
        share_inventories_submining_and_quarrying(time()) +
        share_consum_goverments_submining_and_quarrying(time()),
        share_inventories_subfood_beverages_and_tobacco(time()) +
        share_consum_goverments_subfood_beverages_and_tobacco(time()),
        share_inventories_subtextiles_and_textile_products(time()) +
        share_consum_goverments_subtextiles_and_textile_products(time()),
        share_inventories_subleather_leather_and_footwear(time()) +
        share_consum_goverments_subleather_leather_and_footwear(time()),
        share_inventories_subwood_and_products_of_woood_and_cork(time()) +
        share_consum_goverments_subwood_and_products_of_woood_and_cork(time()),
        share_inventories_subpulp_paper_printing_and_publishing(time()) +
        share_consum_goverments_subpulp_paper_printing_and_publishing(time()),
        share_inventories_subcoke_refined_petroleum_and_nuclear_fuel(time()) +
        share_consum_goverments_subcoke_refined_petroleum_and_nuclear_fuel(time()),
        share_inventories_subchemicals_and_chemical_products(time()) +
        share_consum_goverments_subchemicals_and_chemical_products(time()),
        share_inventories_subrubber_and_plastics(time()) +
        share_consum_goverments_subrubber_and_plastics(time()),
        share_inventories_subother_non_metalic_mineral(time()) +
        share_consum_goverments_subother_non_metalic_mineral(time()),
        share_inventories_subbasic_metals_and_fabricated_metal(time()) +
        share_consum_goverments_subbasic_metals_and_fabricated_metal(time()),
        share_inventories_submachinery_nec(time()) +
        share_consum_goverments_submachinery_nec(time()),
        share_inventories_subelectrical_and_optical_equipment(time()) +
        share_consum_goverments_subelectrical_and_optical_equipment(time()),
        share_inventories_subtransport_equipment(time()) +
        share_consum_goverments_subtransport_equipment(time()),
        share_inventories_submanufacturing_nec_recycling(time()) +
        share_consum_goverments_submanufacturing_nec_recycling(time()),
        share_inventories_subelectricity_gas_and_water_supply(time()) +
        share_consum_goverments_subelectricity_gas_and_water_supply(time()),
        share_inventories_subconstruction(time()) +
        share_consum_goverments_subconstruction(time()),
        share_inventories_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()) +
        share_consum_goverments_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        share_inventories_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()) +
        share_consum_goverments_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        share_inventories_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()) +
        share_consum_goverments_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        share_inventories_subhotels_and_restaurants(time()) +
        share_consum_goverments_subhotels_and_restaurants(time()),
        share_inventories_subinland_transport(time()) +
        share_consum_goverments_subinland_transport(time()),
        share_inventories_subwater_transport(time()) +
        share_consum_goverments_subwater_transport(time()),
        share_inventories_subair_transport(time()) +
        share_consum_goverments_subair_transport(time()),
        share_inventories_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()) +
        share_consum_goverments_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        share_inventories_subpost_and_telecommunications(time()) +
        share_consum_goverments_subpost_and_telecommunications(time()),
        share_inventories_subfinancial_intermedation(time()) +
        share_consum_goverments_subfinancial_intermedation(time()),
        share_inventories_subreal_estate_activities(time()) +
        share_consum_goverments_subreal_estate_activities(time()),
        share_inventories_subrenting_od_meq_and_other_business_activities(time()) +
        share_consum_goverments_subrenting_od_meq_and_other_business_activities(time()),
        share_inventories_subpublic_admin_and_defence_compulsory_social_security(time()) +
        share_consum_goverments_subpublic_admin_and_defence_compulsory_social_security(time()),
        share_inventories_subeducation(time()) + share_consum_goverments_subeducation(time()),
        share_inventories_subhealth_and_social_work(time()) +
        share_consum_goverments_subhealth_and_social_work(time()),
        share_inventories_subother_community_social_and_persona_services(time()) +
        share_consum_goverments_subother_community_social_and_persona_services(time()),
        share_inventories_subprivate_households_with_employed_persons(time()) +
        share_consum_goverments_subprivate_households_with_employed_persons(time())],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 1969)
    return ret


@cache('step')
def share_consum_goverments_and_inventories_next_step():
    """
    share consum goverments and inventories next step



    component


    """
    
    ret = xr.DataArray(data=[
        share_consum_goverments_next_step_subagriculture_hunting_forestry_and_fishing(time()) +
        share_inventories_next_step_subagriculture_hunting_forestry_and_fishing(time()),
        share_consum_goverments_next_step_submining_and_quarrying(time()) +
        share_inventories_next_step_submining_and_quarrying(time()),
        share_consum_goverments_next_step_subfood_beverages_and_tobacco(time()) +
        share_inventories_next_step_subfood_beverages_and_tobacco(time()),
        share_consum_goverments_next_step_subtextiles_and_textile_products(time()) +
        share_inventories_next_step_subtextiles_and_textile_products(time()),
        share_consum_goverments_next_step_subleather_leather_and_footwear(time()) +
        share_inventories_next_step_subleather_leather_and_footwear(time()),
        share_consum_goverments_next_step_subwood_and_products_of_woood_and_cork(time()) +
        share_inventories_next_step_subwood_and_products_of_woood_and_cork(time()),
        share_consum_goverments_next_step_subpulp_paper_printing_and_publishing(time()) +
        share_inventories_next_step_subpulp_paper_printing_and_publishing(time()),
        share_consum_goverments_next_step_subcoke_refined_petroleum_and_nuclear_fuel(time()) +
        share_inventories_next_step_subcoke_refined_petroleum_and_nuclear_fuel(time()),
        share_consum_goverments_next_step_subchemicals_and_chemical_products(time()) +
        share_inventories_next_step_subchemicals_and_chemical_products(time()),
        share_consum_goverments_next_step_subrubber_and_plastics(time()) +
        share_inventories_next_step_subrubber_and_plastics(time()),
        share_consum_goverments_next_step_subother_non_metalic_mineral(time()) +
        share_inventories_next_step_subother_non_metalic_mineral(time()),
        share_consum_goverments_next_step_subbasic_metals_and_fabricated_metal(time()) +
        share_inventories_next_step_subbasic_metals_and_fabricated_metal(time()),
        share_consum_goverments_next_step_submachinery_nec(time()) +
        share_inventories_next_step_submachinery_nec(time()),
        share_consum_goverments_next_step_subelectrical_and_optical_equipment(time()) +
        share_inventories_next_step_subelectrical_and_optical_equipment(time()),
        share_consum_goverments_next_step_subtransport_equipment(time()) +
        share_inventories_next_step_subtransport_equipment(time()),
        share_consum_goverments_next_step_submanufacturing_nec_recycling(time()) +
        share_inventories_next_step_submanufacturing_nec_recycling(time()),
        share_consum_goverments_next_step_subelectricity_gas_and_water_supply(time()) +
        share_inventories_next_step_subelectricity_gas_and_water_supply(time()),
        share_consum_goverments_next_step_subconstruction(time()) +
        share_inventories_next_step_subconstruction(time()),
        share_consum_goverments_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()) +
        share_inventories_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        share_consum_goverments_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()) +
        share_inventories_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        share_consum_goverments_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()) +
        share_inventories_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        share_consum_goverments_next_step_subhotels_and_restaurants(time()) +
        share_inventories_next_step_subhotels_and_restaurants(time()),
        share_consum_goverments_next_step_subinland_transport(time()) +
        share_inventories_next_step_subinland_transport(time()),
        share_consum_goverments_next_step_subwater_transport(time()) +
        share_inventories_next_step_subwater_transport(time()),
        share_consum_goverments_next_step_subair_transport(time()) +
        share_inventories_next_step_subair_transport(time()),
        share_consum_goverments_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()) +
        share_inventories_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        share_consum_goverments_next_step_subpost_and_telecommunications(time()) +
        share_inventories_next_step_subpost_and_telecommunications(time()),
        share_consum_goverments_next_step_subfinancial_intermedation(time()) +
        share_inventories_next_step_subfinancial_intermedation(time()),
        share_consum_goverments_next_step_subreal_estate_activities(time()) +
        share_inventories_next_step_subreal_estate_activities(time()),
        share_consum_goverments_next_step_subrenting_od_meq_and_other_business_activities(time()) +
        share_inventories_next_step_subrenting_od_meq_and_other_business_activities(time()),
        share_consum_goverments_next_step_subpublic_admin_and_defence_compulsory_social_security(
            time()) +
        share_inventories_next_step_subpublic_admin_and_defence_compulsory_social_security(time()),
        share_consum_goverments_next_step_subeducation(time()) +
        share_inventories_next_step_subeducation(time()),
        share_consum_goverments_next_step_subhealth_and_social_work(time()) +
        share_inventories_next_step_subhealth_and_social_work(time()),
        share_consum_goverments_next_step_subother_community_social_and_persona_services(time()) +
        share_inventories_next_step_subother_community_social_and_persona_services(time()),
        share_consum_goverments_next_step_subprivate_households_with_employed_persons(time()) +
        share_inventories_next_step_subprivate_households_with_employed_persons(time())],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 1970)
    return ret


@cache('step')
def cc_sectoral():
    """
    cc sectoral



    component


    """
    
    ret = xr.DataArray(data=[
        cc_total() * share_cc_sectoral_subagriculture_hunting_forestry_and_fishing(time()),
        cc_total() * share_cc_sectoral_submining_and_quarrying(time()),
        cc_total() * share_cc_sectoral_subfood_beverages_and_tobacco(time()),
        cc_total() * share_cc_sectoral_subtextiles_and_textile_products(time()),
        cc_total() * share_cc_sectoral_subleather_leather_and_footwear(time()),
        cc_total() * share_cc_sectoral_subwood_and_products_of_woood_and_cork(time()),
        cc_total() * share_cc_sectoral_subpulp_paper_printing_and_publishing(time()),
        cc_total() * share_cc_sectoral_subcoke_refined_petroleum_and_nuclear_fuel(time()),
        cc_total() * share_cc_sectoral_subchemicals_and_chemical_products(time()),
        cc_total() * share_cc_sectoral_subrubber_and_plastics(time()),
        cc_total() * share_cc_sectoral_subother_non_metalic_mineral(time()),
        cc_total() * share_cc_sectoral_subbasic_metals_and_fabricated_metal(time()),
        cc_total() * share_cc_sectoral_submachinery_nec(time()),
        cc_total() * share_cc_sectoral_subelectrical_and_optical_equipment(time()),
        cc_total() * share_cc_sectoral_subtransport_equipment(time()),
        cc_total() * share_cc_sectoral_submanufacturing_nec_recycling(time()),
        cc_total() * share_cc_sectoral_subelectricity_gas_and_water_supply(time()),
        cc_total() * share_cc_sectoral_subconstruction(time()),
        cc_total() *
        share_cc_sectoral_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        cc_total() *
        share_cc_sectoral_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        cc_total() *
        share_cc_sectoral_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        cc_total() * share_cc_sectoral_subhotels_and_restaurants(time()),
        cc_total() * share_cc_sectoral_subinland_transport(time()),
        cc_total() * share_cc_sectoral_subwater_transport(time()),
        cc_total() * share_cc_sectoral_subair_transport(time()),
        cc_total() *
        share_cc_sectoral_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        cc_total() * share_cc_sectoral_subpost_and_telecommunications(time()),
        cc_total() * share_cc_sectoral_subfinancial_intermedation(time()),
        cc_total() * share_cc_sectoral_subreal_estate_activities(time()),
        cc_total() * share_cc_sectoral_subrenting_od_meq_and_other_business_activities(time()),
        cc_total() *
        share_cc_sectoral_subpublic_admin_and_defence_compulsory_social_security(time()),
        cc_total() * share_cc_sectoral_subeducation(time()),
        cc_total() * share_cc_sectoral_subhealth_and_social_work(time()),
        cc_total() * share_cc_sectoral_subother_community_social_and_persona_services(time()),
        cc_total() * share_cc_sectoral_subprivate_households_with_employed_persons(time())],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 1971)
    return ret


@cache('step')
def variation_cc_sectoral():
    """
    variation cc sectoral



    component


    """
    
    ret = xr.DataArray(data=[
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subagriculture_hunting_forestry_and_fishing(time())) -
        (cc_total() * share_cc_sectoral_subagriculture_hunting_forestry_and_fishing(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_submining_and_quarrying(time())) -
        (cc_total() * share_cc_sectoral_submining_and_quarrying(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subfood_beverages_and_tobacco(time()))
        - (cc_total() * share_cc_sectoral_subfood_beverages_and_tobacco(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subtextiles_and_textile_products(time())) -
        (cc_total() * share_cc_sectoral_subtextiles_and_textile_products(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subleather_leather_and_footwear(time())) -
        (cc_total() * share_cc_sectoral_subleather_leather_and_footwear(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subwood_and_products_of_woood_and_cork(time())) -
        (cc_total() * share_cc_sectoral_subwood_and_products_of_woood_and_cork(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subpulp_paper_printing_and_publishing(time())) -
        (cc_total() * share_cc_sectoral_subpulp_paper_printing_and_publishing(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subcoke_refined_petroleum_and_nuclear_fuel(time())) -
        (cc_total() * share_cc_sectoral_subcoke_refined_petroleum_and_nuclear_fuel(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subchemicals_and_chemical_products(time())) -
        (cc_total() * share_cc_sectoral_subchemicals_and_chemical_products(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subrubber_and_plastics(time())) -
        (cc_total() * share_cc_sectoral_subrubber_and_plastics(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subother_non_metalic_mineral(time())) -
        (cc_total() * share_cc_sectoral_subother_non_metalic_mineral(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subbasic_metals_and_fabricated_metal(time())) -
        (cc_total() * share_cc_sectoral_subbasic_metals_and_fabricated_metal(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_submachinery_nec(time())) -
        (cc_total() * share_cc_sectoral_submachinery_nec(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subelectrical_and_optical_equipment(time())) -
        (cc_total() * share_cc_sectoral_subelectrical_and_optical_equipment(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subtransport_equipment(time())) -
        (cc_total() * share_cc_sectoral_subtransport_equipment(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_submanufacturing_nec_recycling(time()))
        - (cc_total() * share_cc_sectoral_submanufacturing_nec_recycling(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subelectricity_gas_and_water_supply(time())) -
        (cc_total() * share_cc_sectoral_subelectricity_gas_and_water_supply(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subconstruction(time())) -
        (cc_total() * share_cc_sectoral_subconstruction(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
             time())) -
        (cc_total() *
         share_cc_sectoral_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
             time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
             time())) -
        (cc_total() *
         share_cc_sectoral_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
             time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
             time())) -
        (cc_total() *
         share_cc_sectoral_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
             time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subhotels_and_restaurants(time())) -
        (cc_total() * share_cc_sectoral_subhotels_and_restaurants(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subinland_transport(time())) -
        (cc_total() * share_cc_sectoral_subinland_transport(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subwater_transport(time())) -
        (cc_total() * share_cc_sectoral_subwater_transport(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subair_transport(time())) -
        (cc_total() * share_cc_sectoral_subair_transport(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
             time())) -
        (cc_total() *
         share_cc_sectoral_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
             time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subpost_and_telecommunications(time()))
        - (cc_total() * share_cc_sectoral_subpost_and_telecommunications(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subfinancial_intermedation(time())) -
        (cc_total() * share_cc_sectoral_subfinancial_intermedation(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subreal_estate_activities(time())) -
        (cc_total() * share_cc_sectoral_subreal_estate_activities(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subrenting_od_meq_and_other_business_activities(time())) -
        (cc_total() * share_cc_sectoral_subrenting_od_meq_and_other_business_activities(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subpublic_admin_and_defence_compulsory_social_security(time())) -
        (cc_total() *
         share_cc_sectoral_subpublic_admin_and_defence_compulsory_social_security(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subeducation(time())) -
        (cc_total() * share_cc_sectoral_subeducation(time())),
        ((cc_total() + variation_cc()) * share_cc_next_step_subhealth_and_social_work(time())) -
        (cc_total() * share_cc_sectoral_subhealth_and_social_work(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subother_community_social_and_persona_services(time())) -
        (cc_total() * share_cc_sectoral_subother_community_social_and_persona_services(time())),
        ((cc_total() + variation_cc()) *
         share_cc_next_step_subprivate_households_with_employed_persons(time())) -
        (cc_total() * share_cc_sectoral_subprivate_households_with_employed_persons(time()))],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons']},
        dims=['sectors'])
    #test_nans(ret, 1972)
    return ret


@cache('step')
def cedtot_per_tw_over_lifetime_res_elec_dispatch():
    """
    cedtot per tw over lifetime res elec dispatch



    component


    """
    
    ret = xr.DataArray(data=[
        functions.zidz((1 - res_elec_variablesxquestx().loc['hydro'].values) * (cpxdashxini_res_elec_subhydro(
            time()) * lifetime_res_elec().loc['hydro'].values * ej_per_twh() / twe_per_twh()),
                       eroixdashxini_res_elec_dispatch().loc['hydro'].values * quality_of_electricity_x2015x()),
        functions.zidz((1 - res_elec_variablesxquestx().loc['geotxdashxelec'].values) * (cpxdashxini_res_elec_subgeotxdashxelec(
        ) * lifetime_res_elec().loc['geotxdashxelec'].values * ej_per_twh() / twe_per_twh()),
                       eroixdashxini_res_elec_dispatch().loc['geotxdashxelec'].values * quality_of_electricity_x2015x()),
        functions.zidz(
            (1 - res_elec_variablesxquestx().loc['solid bioexdashxelec'].values) * (cpxdashxini_res_elec_subsolid_bioexdashxelec(
            ) * lifetime_res_elec().loc['solid bioexdashxelec'].values * ej_per_twh() / twe_per_twh()),
            eroixdashxini_res_elec_dispatch().loc['solid bioexdashxelec'].values * quality_of_electricity_x2015x()),
        functions.zidz((1 - res_elec_variablesxquestx().loc['oceanic'].values) * (cpxdashxini_res_elec_suboceanic(
        ) * lifetime_res_elec().loc['oceanic'].values * ej_per_twh() / twe_per_twh()),
                       eroixdashxini_res_elec_dispatch().loc['oceanic'].values * quality_of_electricity_x2015x()),
        functions.zidz((1 - res_elec_variablesxquestx().loc['wind onshore'].values) * (cpxdashxini_res_elec_subwind_onshore(
        ) * lifetime_res_elec().loc['wind onshore'].values * ej_per_twh() / twe_per_twh()),
                       eroixdashxini_res_elec_dispatch().loc['wind onshore'].values * quality_of_electricity_x2015x()),
        functions.zidz((1 - res_elec_variablesxquestx().loc['wind offshore'].values) * (cpxdashxini_res_elec_subwind_offshore(
        ) * lifetime_res_elec().loc['wind offshore'].values * ej_per_twh() / twe_per_twh()),
                       eroixdashxini_res_elec_dispatch().loc['wind offshore'].values * quality_of_electricity_x2015x()),
        functions.zidz((1 - res_elec_variablesxquestx().loc['solar pv'].values) * (cpxdashxini_res_elec_subsolar_pv(
        ) * lifetime_res_elec().loc['solar pv'].values * ej_per_twh() / twe_per_twh()),
                       eroixdashxini_res_elec_dispatch().loc['solar pv'].values * quality_of_electricity_x2015x()),
        functions.zidz(
            (1 - res_elec_variablesxquestx().loc['csp'].values) *
            (cpxdashxini_res_elec_subcsp() * lifetime_res_elec().loc['csp'].values * ej_per_twh() / twe_per_twh()),
            eroixdashxini_res_elec_dispatch().loc['csp'].values * quality_of_electricity_x2015x())],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1973)
    return ret


@cache('step')
def historic_rate_final_energy_intensity():
    """
    historic rate final energy intensity



    component


    """
    
    ret = xr.DataArray(data=[[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculture_hunting_forestry_and_fishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subagriculture_hunting_forestry_and_fishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculture_hunting_forestry_and_fishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subagriculture_hunting_forestry_and_fishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculture_hunting_forestry_and_fishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subagriculture_hunting_forestry_and_fishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculture_hunting_forestry_and_fishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subagriculture_hunting_forestry_and_fishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculture_hunting_forestry_and_fishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subagriculture_hunting_forestry_and_fishing(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submining_and_quarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submining_and_quarrying(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submining_and_quarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submining_and_quarrying(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submining_and_quarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submining_and_quarrying(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submining_and_quarrying(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submining_and_quarrying(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submining_and_quarrying(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_submining_and_quarrying(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfood_beverages_and_tobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfood_beverages_and_tobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfood_beverages_and_tobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfood_beverages_and_tobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfood_beverages_and_tobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfood_beverages_and_tobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subfood_beverages_and_tobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subfood_beverages_and_tobacco(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfood_beverages_and_tobacco(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfood_beverages_and_tobacco(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextiles_and_textile_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtextiles_and_textile_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextiles_and_textile_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtextiles_and_textile_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextiles_and_textile_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtextiles_and_textile_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextiles_and_textile_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtextiles_and_textile_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextiles_and_textile_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtextiles_and_textile_products(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleather_leather_and_footwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subleather_leather_and_footwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subleather_leather_and_footwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subleather_leather_and_footwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleather_leather_and_footwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subleather_leather_and_footwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subleather_leather_and_footwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subleather_leather_and_footwear(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleather_leather_and_footwear(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subleather_leather_and_footwear(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwood_and_products_of_woood_and_cork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwood_and_products_of_woood_and_cork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwood_and_products_of_woood_and_cork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwood_and_products_of_woood_and_cork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwood_and_products_of_woood_and_cork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwood_and_products_of_woood_and_cork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwood_and_products_of_woood_and_cork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwood_and_products_of_woood_and_cork(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwood_and_products_of_woood_and_cork(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwood_and_products_of_woood_and_cork(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulp_paper_printing_and_publishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpulp_paper_printing_and_publishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulp_paper_printing_and_publishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpulp_paper_printing_and_publishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulp_paper_printing_and_publishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpulp_paper_printing_and_publishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulp_paper_printing_and_publishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpulp_paper_printing_and_publishing(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulp_paper_printing_and_publishing(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpulp_paper_printing_and_publishing(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcoke_refined_petroleum_and_nuclear_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subcoke_refined_petroleum_and_nuclear_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subcoke_refined_petroleum_and_nuclear_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subcoke_refined_petroleum_and_nuclear_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcoke_refined_petroleum_and_nuclear_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subcoke_refined_petroleum_and_nuclear_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subcoke_refined_petroleum_and_nuclear_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subcoke_refined_petroleum_and_nuclear_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcoke_refined_petroleum_and_nuclear_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subcoke_refined_petroleum_and_nuclear_fuel(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicals_and_chemical_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subchemicals_and_chemical_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicals_and_chemical_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subchemicals_and_chemical_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicals_and_chemical_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subchemicals_and_chemical_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicals_and_chemical_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subchemicals_and_chemical_products(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicals_and_chemical_products(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subchemicals_and_chemical_products(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubber_and_plastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrubber_and_plastics(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubber_and_plastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrubber_and_plastics(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubber_and_plastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrubber_and_plastics(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubber_and_plastics(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrubber_and_plastics(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubber_and_plastics(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrubber_and_plastics(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_non_metalic_mineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_non_metalic_mineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_non_metalic_mineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_non_metalic_mineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_non_metalic_mineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_non_metalic_mineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_non_metalic_mineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_non_metalic_mineral(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_non_metalic_mineral(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_non_metalic_mineral(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasic_metals_and_fabricated_metal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subbasic_metals_and_fabricated_metal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasic_metals_and_fabricated_metal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subbasic_metals_and_fabricated_metal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasic_metals_and_fabricated_metal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subbasic_metals_and_fabricated_metal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasic_metals_and_fabricated_metal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subbasic_metals_and_fabricated_metal(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasic_metals_and_fabricated_metal(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subbasic_metals_and_fabricated_metal(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinery_nec(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submachinery_nec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinery_nec(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submachinery_nec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinery_nec(time() + 1)
        - historic_final_energy_intensity_by_sector_and_fuel_subliquids_submachinery_nec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinery_nec(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submachinery_nec(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinery_nec(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submachinery_nec(time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectrical_and_optical_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectrical_and_optical_equipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectrical_and_optical_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectrical_and_optical_equipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectrical_and_optical_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectrical_and_optical_equipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectrical_and_optical_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectrical_and_optical_equipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectrical_and_optical_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectrical_and_optical_equipment(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransport_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subtransport_equipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransport_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subtransport_equipment(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransport_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subtransport_equipment(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransport_equipment(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subtransport_equipment(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransport_equipment(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subtransport_equipment(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturing_nec_recycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_submanufacturing_nec_recycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturing_nec_recycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_submanufacturing_nec_recycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturing_nec_recycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_submanufacturing_nec_recycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturing_nec_recycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_submanufacturing_nec_recycling(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturing_nec_recycling(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_submanufacturing_nec_recycling(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricity_gas_and_water_supply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subelectricity_gas_and_water_supply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricity_gas_and_water_supply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subelectricity_gas_and_water_supply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricity_gas_and_water_supply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subelectricity_gas_and_water_supply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricity_gas_and_water_supply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subelectricity_gas_and_water_supply(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricity_gas_and_water_supply(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subelectricity_gas_and_water_supply(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subconstruction(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subconstruction(time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subsale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subretail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotels_and_restaurants(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhotels_and_restaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotels_and_restaurants(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhotels_and_restaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotels_and_restaurants(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhotels_and_restaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotels_and_restaurants(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subgases_subhotels_and_restaurants(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotels_and_restaurants(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhotels_and_restaurants(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinland_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subinland_transport(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subinland_transport(time() + 1)
        - historic_final_energy_intensity_by_sector_and_fuel_subheat_subinland_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinland_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subinland_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subinland_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subinland_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinland_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subinland_transport(time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwater_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subwater_transport(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwater_transport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subwater_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwater_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subwater_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subwater_transport(time() + 1)
        - historic_final_energy_intensity_by_sector_and_fuel_subgases_subwater_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwater_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subwater_transport(time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subair_transport(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subair_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subair_transport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subair_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subair_transport(time() + 1)
        - historic_final_energy_intensity_by_sector_and_fuel_subliquids_subair_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subair_transport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subair_transport(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subair_transport(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subair_transport(time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpost_and_telecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpost_and_telecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpost_and_telecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpost_and_telecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpost_and_telecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpost_and_telecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpost_and_telecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpost_and_telecommunications(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpost_and_telecommunications(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpost_and_telecommunications(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancial_intermedation(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subfinancial_intermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancial_intermedation(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subfinancial_intermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancial_intermedation(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subfinancial_intermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancial_intermedation(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subgases_subfinancial_intermedation(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancial_intermedation(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subfinancial_intermedation(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subreal_estate_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subreal_estate_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subreal_estate_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subreal_estate_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subreal_estate_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subreal_estate_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subreal_estate_activities(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subgases_subreal_estate_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subreal_estate_activities(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subreal_estate_activities(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrenting_od_meq_and_other_business_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subrenting_od_meq_and_other_business_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrenting_od_meq_and_other_business_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subrenting_od_meq_and_other_business_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrenting_od_meq_and_other_business_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subrenting_od_meq_and_other_business_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrenting_od_meq_and_other_business_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subrenting_od_meq_and_other_business_activities(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrenting_od_meq_and_other_business_activities(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subrenting_od_meq_and_other_business_activities(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublic_admin_and_defence_compulsory_social_security(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subpublic_admin_and_defence_compulsory_social_security(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublic_admin_and_defence_compulsory_social_security(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subpublic_admin_and_defence_compulsory_social_security(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublic_admin_and_defence_compulsory_social_security(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subpublic_admin_and_defence_compulsory_social_security(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublic_admin_and_defence_compulsory_social_security(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subpublic_admin_and_defence_compulsory_social_security(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublic_admin_and_defence_compulsory_social_security(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subpublic_admin_and_defence_compulsory_social_security(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(time() + 1)
        - historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subeducation(time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation(time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subeducation(time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealth_and_social_work(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subhealth_and_social_work(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealth_and_social_work(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subhealth_and_social_work(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealth_and_social_work(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subhealth_and_social_work(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealth_and_social_work(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subgases_subhealth_and_social_work(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealth_and_social_work(
            time() + 1
        ) - historic_final_energy_intensity_by_sector_and_fuel_subsolids_subhealth_and_social_work(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_community_social_and_persona_services(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subother_community_social_and_persona_services(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_community_social_and_persona_services(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subother_community_social_and_persona_services(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_community_social_and_persona_services(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subother_community_social_and_persona_services(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_community_social_and_persona_services(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subother_community_social_and_persona_services(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_community_social_and_persona_services(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subother_community_social_and_persona_services(
            time())],[
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivate_households_with_employed_persons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subelectricity_subprivate_households_with_employed_persons(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivate_households_with_employed_persons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subheat_subprivate_households_with_employed_persons(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivate_households_with_employed_persons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subliquids_subprivate_households_with_employed_persons(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivate_households_with_employed_persons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subgases_subprivate_households_with_employed_persons(
            time()),
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivate_households_with_employed_persons(
            time() + 1) -
        historic_final_energy_intensity_by_sector_and_fuel_subsolids_subprivate_households_with_employed_persons(
            time())]],
        coords={'sectors': ['agriculture hunting forestry and fishing', 'mining and quarrying', 'food beverages and tobacco', 'textiles and textile products', 'leather leather and footwear', 'wood and products of woood and cork', 'pulp paper printing and publishing', 'coke refined petroleum and nuclear fuel', 'chemicals and chemical products', 'rubber and plastics', 'other non metalic mineral', 'basic metals and fabricated metal', 'machinery nec', 'electrical and optical equipment', 'transport equipment', 'manufacturing nec recycling', 'electricity gas and water supply', 'construction', 'sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel', 'wholesale trade and commissions trade except of motor vehicles and motorcycles', 'retail trade except of motor vehicles and motorcycles repair of household goods', 'hotels and restaurants', 'inland transport', 'water transport', 'air transport', 'other supporting and auxiliary transport activities activities of travel agencies', 'post and telecommunications', 'financial intermedation', 'real estate activities', 'renting od meq and other business activities', 'public admin and defence compulsory social security', 'education', 'health and social work', 'other community social and persona services', 'private households with employed persons'], 'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['sectors', 'final sources'])
    ret = utils.align_by_dims(ret, ('sectors', 'final sources'))
    #test_nans(ret, 1974)
    return ret


@cache('step')
def fed_heatxdashxcom_by_nre_chp_plants_ej():
    """
    fed heatxdashxcom by nre chp plants ej



    component


    """
    
    ret = share_heatxdashxcom_chp_plants_nre_vs_nre_tot_heatxdashxcom_generation(
        time()) * fed_heatxdashxcom_nre_ej()
    #test_nans(ret, 1975)
    return ret


@cache('step')
def pe_supply_res_nonxdashxelec_ej():
    """
    pe supply res nonxdashxelec ej



    component


    """
    
    ret = pe_supply_from_res_nonxdashxelec_without_trad_bioe_ej(
    ) + pe_traditional_biomass_ej_delayed_1yr() + losses_in_charcoal_plants_ej(time())
    #test_nans(ret, 1976)
    return ret


@cache('step')
def new_pes_biogas():
    """
    new pes biogas



    component


    """
    
    ret = functions.if_then_else(time() < 2014,
                                  (historic_biogas_pes(time() + 1) - historic_biogas_pes(time())),
                                  ((max_biogas_ej() - pes_biogas_ej()) / max_biogas_ej()) *
                                  adapt_growth_biogas() * pes_biogas_ej())
    #test_nans(ret, 1977)
    return ret


@cache('step')
def new_waste_supply_ej():
    """
    new waste supply ej



    component


    """
    
    ret = functions.if_then_else(
        time() < 2014, (historic_pes_waste_ej(time() + 1) - historic_pes_waste_ej(time())),
        functions.if_then_else(abs(max_waste() - 0) < equality_threshold,
                               pes_waste_ej() * p_waste_change(),
                               ((max_waste() - pes_waste_ej()) / max_waste()) *
                               adapt_growth_waste() * pes_waste_ej()))
    #test_nans(ret, 1978)
    return ret


@cache('step')
def historic_new_required_capacity_res_elec():
    """
    historic new required capacity res elec



    component


    """
    
    ret = xr.DataArray(data=[
        table_hist_capacity_res_elec_subhydro(time() + total_time_planxplusxconstr_res_elec().loc['hydro'].values + 1)
        - table_hist_capacity_res_elec_subhydro(time() + total_time_planxplusxconstr_res_elec().loc['hydro'].values),
        table_hist_capacity_res_elec_subgeotxdashxelec(
            time() + total_time_planxplusxconstr_res_elec().loc['geotxdashxelec'].values + 1) -
        table_hist_capacity_res_elec_subgeotxdashxelec(time() +
                                                       total_time_planxplusxconstr_res_elec().loc['geotxdashxelec'].values),
        table_hist_capacity_res_elec_subsolid_bioexdashxelec(
            time() + total_time_planxplusxconstr_res_elec().loc['solid bioexdashxelec'].values + 1) -
        table_hist_capacity_res_elec_subsolid_bioexdashxelec(
            time() + total_time_planxplusxconstr_res_elec().loc['solid bioexdashxelec'].values),
        table_hist_capacity_res_elec_suboceanic(time() + total_time_planxplusxconstr_res_elec().loc['oceanic'].values + 1)
        - table_hist_capacity_res_elec_suboceanic(time() + total_time_planxplusxconstr_res_elec().loc['oceanic'].values),
        table_hist_capacity_res_elec_subwind_onshore(time() +
                                                     total_time_planxplusxconstr_res_elec().loc['wind onshore'].values + 1) -
        table_hist_capacity_res_elec_subwind_onshore(time() +
                                                     total_time_planxplusxconstr_res_elec().loc['wind onshore'].values),
        table_hist_capacity_res_elec_subwind_offshore(time() +
                                                      total_time_planxplusxconstr_res_elec().loc['wind offshore'].values + 1) -
        table_hist_capacity_res_elec_subwind_offshore(time() +
                                                      total_time_planxplusxconstr_res_elec().loc['wind offshore'].values),
        table_hist_capacity_res_elec_subsolar_pv(time() + total_time_planxplusxconstr_res_elec().loc['solar pv'].values +
                                                 1) -
        table_hist_capacity_res_elec_subsolar_pv(time() + total_time_planxplusxconstr_res_elec().loc['solar pv'].values),
        table_hist_capacity_res_elec_subcsp(time() + total_time_planxplusxconstr_res_elec().loc['csp'].values + 1) -
        table_hist_capacity_res_elec_subcsp(time() + total_time_planxplusxconstr_res_elec().loc['csp'].values)],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1979)
    return ret


@cache('step')
def share_unconv_gas_vs_tot_agg():
    """
    share unconv gas vs tot agg



    component


    """
    
    ret = functions.if_then_else(time() > 2012,
                                  np.minimum(evolution_share_unconv_gas_vs_tot_agg(), 1),
                                  historic_unconv_gas(time()) / ped_natx_gas_ej())
    #test_nans(ret, 1980)
    return ret


@cache('step')
def max_extraction_tot_agg_gas_ej():
    """
    max extraction tot agg gas ej



    component


    """
    
    ret = functions.if_then_else(
        abs(separate_conv_and_unconv_gasxquestx() - 0) < equality_threshold,
        functions.if_then_else(
            abs(choose_extraction_tot_agg_gas_curve() - 1) < equality_threshold,
            table_max_extraction_total_gas_laherrere10(tot_rurr_tot_agg_gas()),
            functions.if_then_else(
                abs(choose_extraction_tot_agg_gas_curve() - 2) < equality_threshold,
                table_max_extraction_total_gas_bg_mohr12(tot_rurr_tot_agg_gas()),
                table_max_extraction_total_gas_user_defined(tot_rurr_tot_agg_gas()))), 0)
    #test_nans(ret, 1981)
    return ret


@cache('step')
def share_unconv_oil_vs_tot_agg():
    """
    share unconv oil vs tot agg



    component


    """
    
    ret = functions.if_then_else(time() > 2012,
                                  np.minimum(evolution_share_unconv_oil_vs_tot_agg(), 1),
                                  historic_unconv_oil(time()) / ped_total_oil_ej())
    #test_nans(ret, 1982)
    return ret


@cache('step')
def max_extraction_tot_agg_oil_ej():
    """
    max extraction tot agg oil ej



    component


    """
    
    ret = functions.if_then_else(
        abs(separate_conv_and_unconv_oilxquestx() - 0) < equality_threshold,
        functions.if_then_else(
            abs(choose_extraction_curve_tot_agg_oil() - 1) < equality_threshold,
            table_max_extraction_tot_agg_oil_laherrere_x2006x(tot_rurr_tot_agg_oil()),
            table_max_extraction_tot_agg_oil_user_defined(tot_rurr_tot_agg_oil())), 0)
    #test_nans(ret, 1983)
    return ret


@cache('step')
def extraction_unconv_gas_ej():
    """
    extraction unconv gas ej



    component


    """
    
    ret = functions.if_then_else(rurr_unconv_gas() < 0, 0,
                                  functions.if_then_else(
                                      time() < 2013,
                                      historic_unconv_gas(time()),
                                      functions.if_then_else(
                                          abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
                                          np.minimum(max_extraction_unconv_gas(),
                                                     max_unconv_gas_growth_extraction_ej()), 0)))
    #test_nans(ret, 1984)
    return ret


@cache('step')
def max_extraction_conv_oil_ej():
    """
    max extraction conv oil ej



    component


    """
    
    ret = functions.if_then_else(
        abs(separate_conv_and_unconv_oilxquestx() - 1) < equality_threshold,
        functions.if_then_else(
            abs(choose_extraction_curve_conv_oil() - 1) < equality_threshold,
            table_max_extraction_maggio12middle_conv_oil_ej(tot_rurr_conv_oil()),
            functions.if_then_else(
                abs(choose_extraction_curve_conv_oil() - 2) < equality_threshold,
                table_max_extraction_maggio12_high_conv_oil_ej(tot_rurr_conv_oil()),
                functions.if_then_else(
                    abs(choose_extraction_curve_conv_oil() - 3) < equality_threshold,
                    table_max_extraction_maggio12_low_con_oil_ej(tot_rurr_conv_oil()),
                    table_max_conv_oil_extraction_user_defined(tot_rurr_conv_oil())))), 0)
    #test_nans(ret, 1985)
    return ret


@cache('step')
def share_chp_plants_oil():
    """
    share chp plants oil



    component


    """
    
    ret = np.maximum(
        functions.if_then_else(time() > 2014, -0.002985 * (time()) + 6.04554,
                               historic_share_chp_plants_oil(time())), 0)
    #test_nans(ret, 1986)
    return ret


@cache('step')
def share_chp_plants_coal():
    """
    share chp plants coal



    component


    """
    
    ret = 1 - historic_share_chp_plants_gas(time()) - share_chp_plants_oil()
    #test_nans(ret, 1987)
    return ret


@cache('step')
def share_coalxcoalxplusxgasx_for_heat_plants():
    """
    share coalxcoalxplusxgasx for heat plants



    component


    """
    
    ret = 1 - share_gasxdivxxcoalxplusxgasx_for_heat_plants(time())
    #test_nans(ret, 1988)
    return ret


@cache('step')
def cp_baseload_reduction():
    """
    cp baseload reduction



    component


    """
    
    ret = xr.DataArray(data=[
        cp_res_elec().loc['hydro'].values / cpxdashxini_res_elec_subhydro(time()),
        cp_res_elec().loc['geotxdashxelec'].values / cpxdashxini_res_elec_subgeotxdashxelec(),
        cp_res_elec().loc['solid bioexdashxelec'].values / cpxdashxini_res_elec_subsolid_bioexdashxelec(),
        cp_res_elec().loc['oceanic'].values / cpxdashxini_res_elec_suboceanic(),
        cp_res_elec().loc['wind onshore'].values / cpxdashxini_res_elec_subwind_onshore(),
        cp_res_elec().loc['wind offshore'].values / cpxdashxini_res_elec_subwind_offshore(),
        cp_res_elec().loc['solar pv'].values / cpxdashxini_res_elec_subsolar_pv(),
        cp_res_elec().loc['csp'].values / cpxdashxini_res_elec_subcsp()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 1989)
    return ret


@cache('step')
def pes_peat_ej():
    """
    pes peat ej



    component


    """
    
    ret = np.maximum(
        functions.if_then_else(time() > 2014, -0.0125 * (time()) + 25.3125,
                               historic_pes_peat_ej(time())), 0)
    #test_nans(ret, 1990)
    return ret


@cache('step')
def total_gen_losses_demand_for_heat_plants_ej():
    """
    total gen losses demand for heat plants ej



    component


    """
    
    ret = ped_gases_for_heat_plants_ej() * (
        1 - efficiency_gases_for_heat_plants(time())) + ped_oil_for_heat_plants_ej() * (
            1 - efficiency_liquids_for_heat_plants(time())) + ped_coal_for_heat_plants_ej() * (
                1 - efficiency_coal_for_heat_plants(time()))
    #test_nans(ret, 1991)
    return ret


@cache('step')
def total_gen_losses_demand_for_chp_plants_ej():
    """
    total gen losses demand for chp plants ej



    component


    """
    
    ret = ped_gas_for_chp_plants_ej() * (1 - efficiency_elec_gas_chp_plants(
        time()) - efficiency_heat_gas_chp_plants(time())) + ped_oil_for_chp_plants_ej() * (
            1 - efficiency_elec_oil_chp_plants(time()) - efficiency_heat_oil_chp_plants(time())
        ) + ped_coal_for_chp_plants_ej() * (
            1 - efficiency_heat_coal_chp_plants(time()) - efficiency_elec_coal_chp_plants(time()))
    #test_nans(ret, 1992)
    return ret


@cache('step')
def potential_fe_gen_elec_liquids_chp_plants_ej():
    """
    potential fe gen elec liquids chp plants ej



    component


    """
    
    ret = ped_oil_for_chp_plants_ej() * efficiency_elec_oil_chp_plants(time())
    #test_nans(ret, 1993)
    return ret


@cache('step')
def potential_fe_gen_elec_coal_chp_plants_ej():
    """
    potential fe gen elec coal chp plants ej



    component


    """
    
    ret = ped_coal_for_chp_plants_ej() * efficiency_elec_coal_chp_plants(time())
    #test_nans(ret, 1994)
    return ret


@cache('step')
def potential_fe_gen_elec_gas_chp_plants_ej():
    """
    potential fe gen elec gas chp plants ej



    component


    """
    
    ret = ped_gas_for_chp_plants_ej() * efficiency_elec_gas_chp_plants(time())
    #test_nans(ret, 1995)
    return ret


@cache('step')
def fed_heat_gas_chp_plants_ej():
    """
    fed heat gas chp plants ej



    component


    """
    
    ret = fed_heat_fossil_fuels_chp_plants_ej() * historic_share_chp_plants_gas(time())
    #test_nans(ret, 1996)
    return ret


@cache('step')
def ped_oil_for_chp_plants_ej():
    """
    ped oil for chp plants ej



    component


    """
    
    ret = fed_heat_liquids_chp_plants_ej() / efficiency_heat_oil_chp_plants(time())
    #test_nans(ret, 1997)
    return ret


@cache('step')
def ped_gas_for_chp_plants_ej():
    """
    ped gas for chp plants ej



    component


    """
    
    ret = fed_heat_gas_chp_plants_ej() / efficiency_heat_gas_chp_plants(time())
    #test_nans(ret, 1998)
    return ret


@cache('step')
def ped_coal_for_chp_plants_ej():
    """
    ped coal for chp plants ej



    component


    """
    
    ret = fed_heat_coal_chp_plants_ej() / efficiency_heat_coal_chp_plants(time())
    #test_nans(ret, 1999)
    return ret


@cache('step')
def invest_nuclear_tdolar():
    """
    invest nuclear tdolar



    component


    """
    
    ret = np.maximum(0,
                      functions.if_then_else(
                          nuclear_capacity_under_construction() < 0, 0,
                          (nuclear_capacity_under_construction() + replacement_nuclear_capacity())
                          * invest_cost_nuclear(time()) / 1000))
    #test_nans(ret, 2000)
    return ret


@cache('step')
def nuclear_capacity_under_construction():
    """
    nuclear capacity under construction



    component


    """
    
    ret = functions.if_then_else(
        time() < 2013,
        (historic_nuclear_generation_twh(time() + 1) - historic_nuclear_generation_twh(time())) *
        twe_per_twh() / cp_nuclear(), planned_nuclear_capacity_tw() / time_construction_nuclear())
    #test_nans(ret, 2001)
    return ret


@cache('step')
def invest_res_elec_tdolar():
    """
    invest res elec tdolar



    component


    """
    
    ret = xr.DataArray(data=[
        np.maximum(
            0, res_elec_capacity_under_construction_tw().loc['hydro'].values * invest_cost_res_elec_subhydro(time())),
        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc['geotxdashxelec'].values *
                   invest_cost_res_elec_subgeotxdashxelec(time())),
        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc['solid bioexdashxelec'].values *
                   invest_cost_res_elec_subsolid_bioexdashxelec(time())),
        np.maximum(
            0,
            res_elec_capacity_under_construction_tw().loc['oceanic'].values * invest_cost_res_elec_suboceanic(time())),
        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc['wind onshore'].values *
                   invest_cost_res_elec_subwind_onshore(time())),
        np.maximum(0,
                   res_elec_capacity_under_construction_tw().loc['wind offshore'].values *
                   invest_cost_res_elec_subwind_offshore(time())),
        np.maximum(
            0,
            res_elec_capacity_under_construction_tw().loc['solar pv'].values * invest_cost_res_elec_subsolar_pv(time())),
        np.maximum(
            0, res_elec_capacity_under_construction_tw().loc['csp'].values * invest_cost_res_elec_subcsp(time()))],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 2002)
    return ret


@cache('step')
def power_density_res_elec_twxdivxmha():
    """
    power density res elec twxdivxmha



    component


    """
    
    ret = xr.DataArray(data=[
        power_density_res_elec_twexdivxmha().loc['hydro'].values / cpxdashxini_res_elec_subhydro(time()),
        power_density_res_elec_twexdivxmha().loc['geotxdashxelec'].values / cpxdashxini_res_elec_subgeotxdashxelec(),
        power_density_res_elec_twexdivxmha().loc['solid bioexdashxelec'].values / cpxdashxini_res_elec_subsolid_bioexdashxelec(),
        power_density_res_elec_twexdivxmha().loc['oceanic'].values / cpxdashxini_res_elec_suboceanic(),
        power_density_res_elec_twexdivxmha().loc['wind onshore'].values / cpxdashxini_res_elec_subwind_onshore(),
        power_density_res_elec_twexdivxmha().loc['wind offshore'].values / cpxdashxini_res_elec_subwind_offshore(),
        power_density_res_elec_twexdivxmha().loc['solar pv'].values / cpxdashxini_res_elec_subsolar_pv(),
        power_density_res_elec_twexdivxmha().loc['csp'].values / cpxdashxini_res_elec_subcsp()],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 2003)
    return ret


@cache('run')
def cpxdashxini_res_elec_subgeotxdashxelec():
    """
    cpxdashxini res elec subgeotxdashxelec



    constant


    """
    
    ret = 0.65
    #test_nans(ret, 2004)
    return ret


@cache('run')
def cpxdashxini_res_elec_subsolid_bioexdashxelec():
    """
    cpxdashxini res elec subsolid bioexdashxelec



    constant


    """
    
    ret = 0.5
    #test_nans(ret, 2005)
    return ret


@cache('run')
def cpxdashxini_res_elec_suboceanic():
    """
    cpxdashxini res elec suboceanic



    constant


    """
    
    ret = 0.2
    #test_nans(ret, 2006)
    return ret


@cache('run')
def cpxdashxini_res_elec_subwind_onshore():
    """
    cpxdashxini res elec subwind onshore



    constant


    """
    
    ret = 0.21
    #test_nans(ret, 2007)
    return ret


@cache('run')
def cpxdashxini_res_elec_subwind_offshore():
    """
    cpxdashxini res elec subwind offshore



    constant


    """
    
    ret = 0.315
    #test_nans(ret, 2008)
    return ret


@cache('run')
def cpxdashxini_res_elec_subsolar_pv():
    """
    cpxdashxini res elec subsolar pv



    constant


    """
    
    ret = 0.15
    #test_nans(ret, 2009)
    return ret


@cache('run')
def cpxdashxini_res_elec_subcsp():
    """
    cpxdashxini res elec subcsp



    constant


    """
    
    ret = 0.25
    #test_nans(ret, 2010)
    return ret


@cache('run')
def initial_required_capacity_res_elec():
    """
    initial required capacity res elec



    constant


    """
    
    ret = xr.DataArray(data=[
        table_hist_capacity_res_elec_subhydro(1996) - table_hist_capacity_res_elec_subhydro(1995),
        table_hist_capacity_res_elec_subgeotxdashxelec(1996) -
        table_hist_capacity_res_elec_subgeotxdashxelec(1995),
        table_hist_capacity_res_elec_subsolid_bioexdashxelec(1996) -
        table_hist_capacity_res_elec_subsolid_bioexdashxelec(1995),
        table_hist_capacity_res_elec_suboceanic(1996) -
        table_hist_capacity_res_elec_suboceanic(1995),
        table_hist_capacity_res_elec_subwind_onshore(1996) -
        table_hist_capacity_res_elec_subwind_onshore(1995),
        table_hist_capacity_res_elec_subwind_offshore(1996) -
        table_hist_capacity_res_elec_subwind_offshore(1995),
        table_hist_capacity_res_elec_subsolar_pv(1996) -
        table_hist_capacity_res_elec_subsolar_pv(1995),
        table_hist_capacity_res_elec_subcsp(1996) - table_hist_capacity_res_elec_subcsp(1995)],
        coords={'res elec': ['hydro', 'geotxdashxelec', 'solid bioexdashxelec', 'oceanic', 'wind onshore', 'wind offshore', 'solar pv', 'csp']},
        dims=['res elec'])
    #test_nans(ret, 2011)
    return ret


@cache('step')
def afforestation_program_x2020x_gtco2():
    """
    afforestation program x2020x gtco2



    component


    """
    
    ret = afforestation_program_x2020x(time()) * activate_affores_program() / (
        c_per_co2() * mt_per_gt())
    #test_nans(ret, 2012)
    return ret


@cache('step')
def max_extraction_uranium_ej():
    """
    max extraction uranium ej



    component


    """
    
    ret = functions.if_then_else(abs(choose_extraction_uranium_curve() - 1) < equality_threshold,
                                  table_max_extraction_uranium_ewg13_ej(rurr_uranium()),
                                  functions.if_then_else(
                                      abs(choose_extraction_uranium_curve() - 2) < equality_threshold,
                                      table_max_extraction_uranium_zittel12(rurr_uranium()),
                                      table_max_extraction_uranium_user_defined(rurr_uranium())))
    #test_nans(ret, 2013)
    return ret


@cache('step')
def gtco2_historic_emissions_rcps():
    """
    gtco2 historic emissions rcps



    component


    """
    
    ret = gtc_historic_emissions_rcps(time()) / c_per_co2()
    #test_nans(ret, 2014)
    return ret


@cache('step')
def variation_historic_pop():
    """
    variation historic pop



    component


    """
    
    ret = functions.if_then_else(time() < 2014, historic_pop(time() + 1) - historic_pop(time()),
                                  0)
    #test_nans(ret, 2015)
    return ret


@cache('step')
def tpes_de_castro_phd_xdashx_scen_xmadcoalx_ej():
    """
    tpes de castro phd xdashx scen xmadcoalx ej



    component


    """
    
    ret = tpes_de_castro_phd_xdashx_scen_xmadcoalx(time()) / mtoe_per_ej()
    #test_nans(ret, 2016)
    return ret


@cache('step')
def tpes_de_castro_phd_xdashx_scen_ii_ej():
    """
    tpes de castro phd xdashx scen ii ej



    component


    """
    
    ret = tpes_de_castro_phd_xdashx_scen_ii(time()) / mtoe_per_ej()
    #test_nans(ret, 2017)
    return ret


@cache('step')
def balancing_costs():
    """
    balancing costs



    component


    """
    
    ret = (balancing_costs_ref(share_variable_res_elec_generation_vs_total())) / m_per_t()
    #test_nans(ret, 2018)
    return ret


@cache('step')
def variation_ctl():
    """
    variation ctl



    component


    """
    
    ret = functions.if_then_else(
        time() < 2013,
        historic_ctl_production(time() + 1) - historic_ctl_production(time()),
        functions.if_then_else(
            check_liquids() < - equality_threshold,
            constrain_liquids_exogenous_growthxquestx() * ctl_potential_production(),
            ctl_potential_production() * real_growth_ctl()))
    #test_nans(ret, 2019)
    return ret


@cache('step')
def variation_gtl():
    """
    variation gtl



    component


    """
    
    ret = functions.if_then_else(
        time() < 2013,
        historic_gtl_production(time() + 1) - historic_gtl_production(time()),
        functions.if_then_else(
            check_liquids() < - equality_threshold,
            constrain_liquids_exogenous_growthxquestx() * gtl_potential_production(),
            gtl_potential_production() * real_growth_gtl()))
    #test_nans(ret, 2020)
    return ret


@cache('step')
def improvement_efficiency_gas_for_electricity():
    """
    improvement efficiency gas for electricity



    component


    """
    
    ret = functions.if_then_else(
        time() < 2013, (historic_efficiency_gas_for_electricity(time() + 1) -
                        historic_efficiency_gas_for_electricity(time())) * percent_to_share(),
        efficiency_gas_for_electricity() * remaining_efficiency_improv_gas_for_electricity() *
        efficiency_improv_gas_for_electricity())
    #test_nans(ret, 2021)
    return ret


@cache('step')
def annual_additional_historic_product_biofuels_2gen():
    """
    annual additional historic product biofuels 2gen



    component


    """
    
    ret = functions.if_then_else(
        time() < 2015,
        historic_produc_biofuels_2gen(time() + 1) - historic_produc_biofuels_2gen(time()), 0)
    #test_nans(ret, 2022)
    return ret


@cache('step')
def share_gasxdivxxcoal_xplusxgasx_for_elec():
    """
    share gasxdivxxcoal xplusxgasx for elec



    component


    """
    
    ret = functions.if_then_else(time() > 2014,
                                  future_share_gasxdivxxcoalxplusxgasx_for_elec(),
                                  hist_share_gasxdivxxcoal_xplusxgasx_elec(time()))
    #test_nans(ret, 2023)
    return ret


@cache('step')
def max_unconv_oil_growth_extraction():
    """
    max unconv oil growth extraction



    component


    """
    
    ret = np.maximum(
        0, 1 + (functions.if_then_else(abs(selection_constraint_extraction_unconv_oil() - 1) < equality_threshold, (
            p_constraint_growth_extraction_unconv_oil()) * time_step() * scarcity_conv_oil_stock(),
                                       (userxdashxdefined_extraction_growth_unconv_oil(
                                           time()) * time_step()))) * abundance_unconv_oil2())
    #test_nans(ret, 2024)
    return ret


@cache('step')
def extraction_unconv_oil_ej():
    """
    extraction unconv oil ej



    component


    """
    
    ret = np.minimum(
        functions.if_then_else(rurr_unconv_oil_ej() < 0, 0,
                               functions.if_then_else(
                                   time() > 2012,
                                   functions.if_then_else(
                                       abs(separate_conv_and_unconv_oilxquestx() - 1) < equality_threshold,
                                       np.minimum(max_extraction_unconv_oil(),
                                                  max_unconv_oil_growth_extraction_ej()), 0),
                                   historic_unconv_oil(time()))), ped_total_oil_ej())
    #test_nans(ret, 2025)
    return ret


@cache('step')
def max_extraction_unconv_gas():
    """
    max extraction unconv gas



    component


    """
    
    ret = functions.if_then_else(
        abs(choose_extraction_curve_unconv_gas() - 1) < equality_threshold,
        table_max_extraction_unconv_gas_bg_mohr15(tot_rurr_unconv_gas()),
        functions.if_then_else(
            abs(choose_extraction_curve_unconv_gas() - 2) < equality_threshold,
            table_max_extraction_unconv_gas_low_mohr15(tot_rurr_unconv_gas()),
            functions.if_then_else(
                abs(choose_extraction_curve_unconv_gas() - 3) < equality_threshold,
                table_max_extraction_unconv_gas_high_mohr15(tot_rurr_unconv_gas()),
                table_max_extraction_unconv_gas_user_defined(tot_rurr_unconv_gas()))))
    #test_nans(ret, 2026)
    return ret


@cache('step')
def max_extraction_unconv_oil():
    """
    max extraction unconv oil



    component


    """
    
    ret = functions.if_then_else(
        abs(choose_extraction_curve_unconv_oil() - 1) < equality_threshold,
        table_max_extraction_unconv_oil_bg_mohr15(tot_rurr_unconv_oil()),
        functions.if_then_else(
            abs(choose_extraction_curve_unconv_oil() - 2) < equality_threshold,
            table_max_extraction_unconv_oil_low_mohr15(tot_rurr_unconv_oil()),
            functions.if_then_else(
                abs(choose_extraction_curve_unconv_oil() - 3) < equality_threshold,
                table_max_extraction_unconv_oil_high_mohr15(tot_rurr_unconv_oil()),
                table_max_extraction_unconv_oil_user_defined(tot_rurr_unconv_oil()))))
    #test_nans(ret, 2027)
    return ret


@cache('step')
def max_extraction_coal_ej():
    """
    max extraction coal ej



    component


    """
    
    ret = functions.if_then_else(
        abs(choose_extraction_coal_curve() - 1) < equality_threshold,
        table_max_extraction_coal_mohr2012_ej(tot_rurr_coal()),
        functions.if_then_else(abs(choose_extraction_coal_curve() - 2) < equality_threshold,
                               table_max_extraction_coal_low_mohr15(tot_rurr_coal()),
                               functions.if_then_else(
                                   abs(choose_extraction_coal_curve() - 3) < equality_threshold,
                                   table_max_extraction_coal_bg_mohr15(tot_rurr_coal()),
                                   functions.if_then_else(
                                       abs(choose_extraction_coal_curve() - 4) < equality_threshold,
                                       table_max_extraction_coal_high_mohr15(tot_rurr_coal()),
                                       table_max_extraction_coal_user_defined(tot_rurr_coal())))))
    #test_nans(ret, 2028)
    return ret


@cache('step')
def max_extraction_conv_gas_ej():
    """
    max extraction conv gas ej



    component


    """
    
    ret = functions.if_then_else(
        abs(separate_conv_and_unconv_gasxquestx() - 1) < equality_threshold,
        functions.if_then_else(
            abs(choose_extraction_conv_gas_curve() - 1) < equality_threshold,
            table_max_extraction_conv_gas_bg_mohr15(tot_rurr_conv_gas()),
            functions.if_then_else(
                abs(choose_extraction_conv_gas_curve() - 2) < equality_threshold,
                table_max_extraction_conv_gas_low_mohr15(tot_rurr_conv_gas()),
                functions.if_then_else(
                    abs(choose_extraction_conv_gas_curve() - 3) < equality_threshold,
                    table_max_extraction_conv_gas_high_mohr15(tot_rurr_conv_gas()),
                    table_max_extraction_conv_gas_user_defined(tot_rurr_conv_gas())))), 0)
    #test_nans(ret, 2029)
    return ret


@cache('step')
def fed_heat_gas_plants_ej():
    """
    fed heat gas plants ej



    component


    """
    
    ret = fed_heat_gasxplusxcoal_ej() * share_gasxdivxxcoalxplusxgasx_for_heat_plants(time())
    #test_nans(ret, 2030)
    return ret


@cache('step')
def ped_gases_for_heat_plants_ej():
    """
    ped gases for heat plants ej



    component


    """
    
    ret = fed_heat_gas_plants_ej() / efficiency_gases_for_heat_plants(time())
    #test_nans(ret, 2031)
    return ret


@cache('step')
def ped_oil_for_heat_plants_ej():
    """
    ped oil for heat plants ej



    component


    """
    
    ret = fed_heat_liquids_plants_ej() / efficiency_liquids_for_heat_plants(time())
    #test_nans(ret, 2032)
    return ret


@cache('step')
def ped_coal_for_heat_plants_ej():
    """
    ped coal for heat plants ej



    component


    """
    
    ret = fed_heat_coal_plants_ej() / efficiency_coal_for_heat_plants(time())
    #test_nans(ret, 2033)
    return ret



integ_cumulative_co2e_ghg_emissions = functions.Integ(lambda: total_ce_all_ghg(), lambda: 0)


def _init_scarcity_resources_counter():
    """
    Implicit

    See docs for scarcity_resources_counter

    setup

    Provides initial conditions for scarcity_resources_counter function
    """
    
    ret = xr.DataArray(
        data=np.ones([58]) * 0,
        coords={
            'materials': [
                'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
                'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components',
                'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
                'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt',
                'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese',
                'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic',
                'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint',
                'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand',
                'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel',
                'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires',
                'zinc'
            ]
        },
        dims=['materials'])
    #test_nans(ret, 2034)
    return ret


@cache('step')
def _dscarcity_resources_counter_dt():
    """
    Implicit

    See docs for scarcity_resources_counter

    component

    Provides derivative for scarcity_resources_counter function
    """
    
    ret = functions.if_then_else(abs(materials_availability_xresourcesx() - 0) < equality_threshold, 1, 0)
    #test_nans(ret, 2035)
    return ret



integ_scarcity_resources_counter = functions.Integ(lambda: _dscarcity_resources_counter_dt(),
                                                   lambda: _init_scarcity_resources_counter())


def _init_year_init_scarcity_final_fuels():
    """
    Implicit

    See docs for year_init_scarcity_final_fuels

    setup

    Provides initial conditions for year_init_scarcity_final_fuels function
    """
    
    ret = xr.DataArray(
        data=np.ones([5]) * 0,
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 2036)
    return ret


@cache('step')
def _dyear_init_scarcity_final_fuels_dt():
    """
    Implicit

    See docs for year_init_scarcity_final_fuels

    component

    Provides derivative for year_init_scarcity_final_fuels function
    """
    
    ret = functions.if_then_else(abs(scarcity_final_fuels() - 1) < equality_threshold, (functions.if_then_else(
        abs(scarcity_final_fuels_counter() - 1) < equality_threshold, (time() * 1 / time_step()) - 20, 0)), 0)
    #test_nans(ret, 2037)
    return ret



integ_year_init_scarcity_final_fuels = functions.Integ(
    lambda: _dyear_init_scarcity_final_fuels_dt(), lambda: _init_year_init_scarcity_final_fuels())


def _init_year_init_scarcity_reserves():
    """
    Implicit

    See docs for year_init_scarcity_reserves

    setup

    Provides initial conditions for year_init_scarcity_reserves function
    """
    
    ret = xr.DataArray(
        data=np.ones([58]) * 0,
        coords={
            'materials': [
                'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
                'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components',
                'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
                'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt',
                'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese',
                'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic',
                'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint',
                'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand',
                'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel',
                'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires',
                'zinc'
            ]
        },
        dims=['materials'])
    #test_nans(ret, 2038)
    return ret


@cache('step')
def _dyear_init_scarcity_reserves_dt():
    """
    Implicit

    See docs for year_init_scarcity_reserves

    component

    Provides derivative for year_init_scarcity_reserves function
    """
    
    ret = functions.if_then_else(abs(materials_availability_xreservesx() - 0) < equality_threshold,
                                  (functions.if_then_else(abs(scarcity_reserves_counter() - 1) < equality_threshold,
                                                          (time() * 1 / time_step()), 0)), 0)
    #test_nans(ret, 2039)
    return ret



integ_year_init_scarcity_reserves = functions.Integ(lambda: _dyear_init_scarcity_reserves_dt(),
                                                    lambda: _init_year_init_scarcity_reserves())


def _init_scarcity_final_fuels_counter():
    """
    Implicit

    See docs for scarcity_final_fuels_counter

    setup

    Provides initial conditions for scarcity_final_fuels_counter function
    """
    
    ret = xr.DataArray(
        data=np.ones([5]) * 0,
        coords={'final sources': ['electricity', 'heat', 'liquids', 'gases', 'solids']},
        dims=['final sources'])
    #test_nans(ret, 2040)
    return ret


@cache('step')
def _dscarcity_final_fuels_counter_dt():
    """
    Implicit

    See docs for scarcity_final_fuels_counter

    component

    Provides derivative for scarcity_final_fuels_counter function
    """
    
    ret = functions.if_then_else(abs(scarcity_final_fuels() - 1) < equality_threshold, 1, 0)
    #test_nans(ret, 2041)
    return ret



integ_scarcity_final_fuels_counter = functions.Integ(lambda: _dscarcity_final_fuels_counter_dt(),
                                                     lambda: _init_scarcity_final_fuels_counter())


def _init_scarcity_reserves_counter():
    """
    Implicit

    See docs for scarcity_reserves_counter

    setup

    Provides initial conditions for scarcity_reserves_counter function
    """
    
    ret = xr.DataArray(
        data=np.ones([58]) * 0,
        coords={
            'materials': [
                'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
                'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components',
                'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
                'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt',
                'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese',
                'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic',
                'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint',
                'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand',
                'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel',
                'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires',
                'zinc'
            ]
        },
        dims=['materials'])
    #test_nans(ret, 2042)
    return ret


@cache('step')
def _dscarcity_reserves_counter_dt():
    """
    Implicit

    See docs for scarcity_reserves_counter

    component

    Provides derivative for scarcity_reserves_counter function
    """
    
    ret = functions.if_then_else(abs(materials_availability_xreservesx() - 0) < equality_threshold, 1, 0)
    #test_nans(ret, 2043)
    return ret



integ_scarcity_reserves_counter = functions.Integ(lambda: _dscarcity_reserves_counter_dt(),
                                                  lambda: _init_scarcity_reserves_counter())


def _init_year_init_scarcity_resources():
    """
    Implicit

    See docs for year_init_scarcity_resources

    setup

    Provides initial conditions for year_init_scarcity_resources function
    """
    
    ret = xr.DataArray(
        data=np.ones([58]) * 0,
        coords={
            'materials': [
                'adhesive', 'aluminium', 'aluminium mirrors', 'cadmium', 'carbon fiber', 'cement',
                'chromium', 'copper', 'diesel', 'dy', 'electricxdivxelectronic components',
                'evacuation lines', 'fiberglass', 'foam glass', 'galium', 'glass',
                'glass reinforcing plastic', 'gravel', 'indium', 'iron', 'kno3 mined', 'asphalt',
                'lime', 'limestone', 'lithium', 'lubricant', 'magnesium', 'manganese',
                'heavy equipment', 'concrete', 'molybdenum', 'nano3 mined', 'nano3 synthetic',
                'neodymium', 'nickel', 'over grid x15xpercxx', 'over grid x5xpercxx', 'paint',
                'lead', 'plastics', 'polypropylene', 'rock', 'rock wool', 'sand', 'silicon sand',
                'silicon wafer modules', 'silver', 'site preparation', 'tin', 'soda ash', 'steel',
                'synthetic oil', 'tellurium', 'titanium', 'titanium dioxide', 'vanadium', 'wires',
                'zinc'
            ]
        },
        dims=['materials'])
    #test_nans(ret, 2044)
    return ret


@cache('step')
def _dyear_init_scarcity_resources_dt():
    """
    Implicit

    See docs for year_init_scarcity_resources

    component

    Provides derivative for year_init_scarcity_resources function
    """
    
    ret = functions.if_then_else(abs(materials_availability_xresourcesx() - 0) < equality_threshold,
                                  (functions.if_then_else(abs(scarcity_resources_counter() - 1) < equality_threshold,
                                                          (time() * 1 / time_step()), 0)), 0)
    #test_nans(ret, 2045)
    return ret



integ_year_init_scarcity_resources = functions.Integ(lambda: _dyear_init_scarcity_resources_dt(),
                                                     lambda: _init_year_init_scarcity_resources())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['agriculture hunting forestry and fishing','electricity'].values, lambda: 1 / time_step(), 1)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['mining and quarrying','electricity'].values, lambda: 1 / time_step(), 2)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['food beverages and tobacco','electricity'].values, lambda: 1 / time_step(), 3)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['textiles and textile products','electricity'].values, lambda: 1 / time_step(), 4)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['leather leather and footwear','electricity'].values, lambda: 1 / time_step(), 5)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wood and products of woood and cork','electricity'].values, lambda: 1 / time_step(), 6)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['pulp paper printing and publishing','electricity'].values, lambda: 1 / time_step(), 7)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['coke refined petroleum and nuclear fuel','electricity'].values, lambda: 1 / time_step(), 8)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['chemicals and chemical products','electricity'].values, lambda: 1 / time_step(), 9)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['rubber and plastics','electricity'].values, lambda: 1 / time_step(), 10)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other non metalic mineral','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['basic metals and fabricated metal','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['machinery nec','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electrical and optical equipment','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['transport equipment','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['manufacturing nec recycling','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electricity gas and water supply','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['construction','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['construction','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['retail trade except of motor vehicles and motorcycles repair of household goods','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['hotels and restaurants','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['inland transport','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['water transport','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['air transport','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other supporting and auxiliary transport activities activities of travel agencies','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['post and telecommunications','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['financial intermedation','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['real estate activities','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['renting od meq and other business activities','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['public admin and defence compulsory social security','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['education','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['education','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['health and social work','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other community social and persona services','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','electricity'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['private households with employed persons','electricity'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['agriculture hunting forestry and fishing','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['mining and quarrying','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['food beverages and tobacco','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['textiles and textile products','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['leather leather and footwear','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wood and products of woood and cork','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['pulp paper printing and publishing','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['coke refined petroleum and nuclear fuel','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['chemicals and chemical products','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['rubber and plastics','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other non metalic mineral','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['basic metals and fabricated metal','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['machinery nec','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electrical and optical equipment','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['transport equipment','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['manufacturing nec recycling','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electricity gas and water supply','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['construction','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['construction','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['retail trade except of motor vehicles and motorcycles repair of household goods','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['hotels and restaurants','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['inland transport','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['water transport','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['air transport','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other supporting and auxiliary transport activities activities of travel agencies','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['post and telecommunications','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['financial intermedation','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['real estate activities','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['renting od meq and other business activities','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['public admin and defence compulsory social security','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['education','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['education','heat'].values, lambda: 1 / time_step(), 11)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['health and social work','heat'].values, lambda: 1 / time_step(), 12)


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other community social and persona services','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___heat = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','heat'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['private households with employed persons','heat'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['agriculture hunting forestry and fishing','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['mining and quarrying','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['food beverages and tobacco','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['textiles and textile products','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['leather leather and footwear','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wood and products of woood and cork','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['pulp paper printing and publishing','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['coke refined petroleum and nuclear fuel','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['chemicals and chemical products','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['rubber and plastics','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other non metalic mineral','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['basic metals and fabricated metal','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['machinery nec','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electrical and optical equipment','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['transport equipment','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['manufacturing nec recycling','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electricity gas and water supply','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['construction','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['construction','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['retail trade except of motor vehicles and motorcycles repair of household goods','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['hotels and restaurants','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['inland transport','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['water transport','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['air transport','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other supporting and auxiliary transport activities activities of travel agencies','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['post and telecommunications','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['financial intermedation','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['real estate activities','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['renting od meq and other business activities','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['public admin and defence compulsory social security','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['education','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['education','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['health and social work','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other community social and persona services','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','liquids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['private households with employed persons','liquids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['agriculture hunting forestry and fishing','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['mining and quarrying','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['food beverages and tobacco','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['textiles and textile products','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['leather leather and footwear','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wood and products of woood and cork','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['pulp paper printing and publishing','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['coke refined petroleum and nuclear fuel','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['chemicals and chemical products','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['rubber and plastics','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other non metalic mineral','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['basic metals and fabricated metal','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['machinery nec','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electrical and optical equipment','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['transport equipment','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['manufacturing nec recycling','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electricity gas and water supply','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['construction','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['construction','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['retail trade except of motor vehicles and motorcycles repair of household goods','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['hotels and restaurants','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['inland transport','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['water transport','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['air transport','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other supporting and auxiliary transport activities activities of travel agencies','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['post and telecommunications','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['financial intermedation','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['real estate activities','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['renting od meq and other business activities','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['public admin and defence compulsory social security','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['education','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['education','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['health and social work','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other community social and persona services','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___gases = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','gases'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['private households with employed persons','gases'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['agriculture hunting forestry and fishing','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['agriculture hunting forestry and fishing','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['mining and quarrying','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['mining and quarrying','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['food beverages and tobacco','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['food beverages and tobacco','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['textiles and textile products','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['textiles and textile products','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['leather leather and footwear','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['leather leather and footwear','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wood and products of woood and cork','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wood and products of woood and cork','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['pulp paper printing and publishing','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['pulp paper printing and publishing','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['coke refined petroleum and nuclear fuel','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['coke refined petroleum and nuclear fuel','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['chemicals and chemical products','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['chemicals and chemical products','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['rubber and plastics','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['rubber and plastics','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other non metalic mineral','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other non metalic mineral','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['basic metals and fabricated metal','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['basic metals and fabricated metal','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['machinery nec','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['machinery nec','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electrical and optical equipment','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electrical and optical equipment','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['transport equipment','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['transport equipment','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['manufacturing nec recycling','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['manufacturing nec recycling','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['electricity gas and water supply','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['electricity gas and water supply','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['construction','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['construction','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['retail trade except of motor vehicles and motorcycles repair of household goods','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['retail trade except of motor vehicles and motorcycles repair of household goods','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['hotels and restaurants','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['hotels and restaurants','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['inland transport','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['inland transport','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['water transport','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['water transport','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['air transport','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['air transport','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other supporting and auxiliary transport activities activities of travel agencies','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other supporting and auxiliary transport activities activities of travel agencies','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['post and telecommunications','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['post and telecommunications','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['financial intermedation','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['financial intermedation','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['real estate activities','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['real estate activities','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['renting od meq and other business activities','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['renting od meq and other business activities','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['public admin and defence compulsory social security','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['public admin and defence compulsory social security','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['education','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['education','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['health and social work','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['health and social work','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['other community social and persona services','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['other community social and persona services','solids'].values, lambda: 1 / time_step())


delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___solids = functions.Delay(
    lambda: evol_final_energy_intensity_by_sector_and_fuel().loc['private households with employed persons','solids'].values,
    lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_by_fuel_and_sector_x1995x().loc['private households with employed persons','solids'].values, lambda: 1 / time_step())


delays_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step = [
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___electricity,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___heat,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___liquids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___gases,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___agriculture_hunting_forestry_and_fishing___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___mining_and_quarrying___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___food_beverages_and_tobacco___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___textiles_and_textile_products___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___leather_leather_and_footwear___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wood_and_products_of_woood_and_cork___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___pulp_paper_printing_and_publishing___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___coke_refined_petroleum_and_nuclear_fuel___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___chemicals_and_chemical_products___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___rubber_and_plastics___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_non_metalic_mineral___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___basic_metals_and_fabricated_metal___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___machinery_nec___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electrical_and_optical_equipment___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___transport_equipment___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___manufacturing_nec_recycling___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___electricity_gas_and_water_supply___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___construction___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___hotels_and_restaurants___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___inland_transport___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___water_transport___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___air_transport___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___post_and_telecommunications___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___financial_intermedation___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___real_estate_activities___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___renting_od_meq_and_other_business_activities___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___public_admin_and_defence_compulsory_social_security___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___education___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___health_and_social_work___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___other_community_social_and_persona_services___solids,
    delay_evol_final_energy_intensity_by_sector_and_fuel_round1__time_step___time_step_initial_energy_intensity_by_fuel_and_sector_x1995x_1__time_step___private_households_with_employed_persons___solids]



delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___electricity = functions.Delay(
    lambda: evol_final_energy_intensity_h().loc['electricity'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_x1995x_h().loc['electricity'].values, lambda: 1 / time_step(), 14)



delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___heat = functions.Delay(
    lambda: evol_final_energy_intensity_h().loc['heat'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_x1995x_h().loc['heat'].values, lambda: 1 / time_step(), 15)



delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___liquids = functions.Delay(
    lambda: evol_final_energy_intensity_h().loc['liquids'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_x1995x_h().loc['liquids'].values, lambda: 1 / time_step(), 16)



delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___gases = functions.Delay(
    lambda: evol_final_energy_intensity_h().loc['gases'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_x1995x_h().loc['gases'].values, lambda: 1 / time_step(), 17)



delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___solids = functions.Delay(
    lambda: evol_final_energy_intensity_h().loc['solids'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: initial_energy_intensity_x1995x_h().loc['solids'].values, lambda: 1 / time_step(), 18)



delays_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step = [
    delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___electricity,
    delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___heat,
    delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___liquids,
    delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___gases,
    delay_evol_final_energy_intensity_h_round1__time_step___time_step_initial_energy_intensity_x1995x_h_1__time_step___solids]


def _init_evol_final_energy_intensity_h():
    """
    Implicit

    See docs for evol_final_energy_intensity_h

    setup

    Provides initial conditions for evol_final_energy_intensity_h function
    """
    
    ret = initial_energy_intensity_x1995x_h()
    #test_nans(ret, 2046)
    return ret


@cache('step')
def _devol_final_energy_intensity_h_dt():
    """
    Implicit

    See docs for evol_final_energy_intensity_h

    component

    Provides derivative for evol_final_energy_intensity_h function
    """
    
    ret = increase_of_intensity_due_to_energy_a_technology_change_h_top_down(
    ) + inertial_rate_energy_intensity_h_top_down(
    ) - decrease_of_intensity_due_to_energy_a_technology_change_h_top_down()
    #test_nans(ret, 2047)
    return ret



integ_evol_final_energy_intensity_h = functions.Integ(
    lambda: _devol_final_energy_intensity_h_dt(), lambda: _init_evol_final_energy_intensity_h())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___adhesive = functions.Delay(
    lambda: by_mineral_rr_rest().loc['adhesive'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['adhesive'].values, lambda: 1 / time_step(), 20)


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['aluminium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['aluminium'].values, lambda: 1 / time_step(), 21)


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium_mirrors = functions.Delay(
    lambda: by_mineral_rr_rest().loc['aluminium mirrors'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['aluminium mirrors'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cadmium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['cadmium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['cadmium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___carbon_fiber = functions.Delay(
    lambda: by_mineral_rr_rest().loc['carbon fiber'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['carbon fiber'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cement = functions.Delay(
    lambda: by_mineral_rr_rest().loc['cement'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['cement'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___chromium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['chromium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['chromium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___copper = functions.Delay(
    lambda: by_mineral_rr_rest().loc['copper'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['copper'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___diesel = functions.Delay(
    lambda: by_mineral_rr_rest().loc['diesel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['diesel'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___dy = functions.Delay(
    lambda: by_mineral_rr_rest().loc['dy'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['dy'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___electricxdivxelectronic_components = functions.Delay(
    lambda: by_mineral_rr_rest().loc['electricxdivxelectronic components'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['electricxdivxelectronic components'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___evacuation_lines = functions.Delay(
    lambda: by_mineral_rr_rest().loc['evacuation lines'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['evacuation lines'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___fiberglass = functions.Delay(
    lambda: by_mineral_rr_rest().loc['fiberglass'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['fiberglass'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___foam_glass = functions.Delay(
    lambda: by_mineral_rr_rest().loc['foam glass'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['foam glass'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___galium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['galium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['galium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass = functions.Delay(
    lambda: by_mineral_rr_rest().loc['glass'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['glass'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass_reinforcing_plastic = functions.Delay(
    lambda: by_mineral_rr_rest().loc['glass reinforcing plastic'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['glass reinforcing plastic'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___gravel = functions.Delay(
    lambda: by_mineral_rr_rest().loc['gravel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['gravel'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___indium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['indium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['indium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___iron = functions.Delay(
    lambda: by_mineral_rr_rest().loc['iron'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['iron'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___kno3_mined = functions.Delay(
    lambda: by_mineral_rr_rest().loc['kno3 mined'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['kno3 mined'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___asphalt = functions.Delay(
    lambda: by_mineral_rr_rest().loc['asphalt'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['asphalt'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lime = functions.Delay(
    lambda: by_mineral_rr_rest().loc['lime'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lime'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___limestone = functions.Delay(
    lambda: by_mineral_rr_rest().loc['limestone'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['limestone'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lithium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['lithium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lithium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lubricant = functions.Delay(
    lambda: by_mineral_rr_rest().loc['lubricant'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lubricant'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___magnesium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['magnesium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['magnesium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___manganese = functions.Delay(
    lambda: by_mineral_rr_rest().loc['manganese'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['manganese'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___heavy_equipment = functions.Delay(
    lambda: by_mineral_rr_rest().loc['heavy equipment'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['heavy equipment'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___concrete = functions.Delay(
    lambda: by_mineral_rr_rest().loc['concrete'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['concrete'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___molybdenum = functions.Delay(
    lambda: by_mineral_rr_rest().loc['molybdenum'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['molybdenum'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_mined = functions.Delay(
    lambda: by_mineral_rr_rest().loc['nano3 mined'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['nano3 mined'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_synthetic = functions.Delay(
    lambda: by_mineral_rr_rest().loc['nano3 synthetic'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['nano3 synthetic'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___neodymium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['neodymium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['neodymium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nickel = functions.Delay(
    lambda: by_mineral_rr_rest().loc['nickel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['nickel'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x15xpercxx = functions.Delay(
    lambda: by_mineral_rr_rest().loc['over grid x15xpercxx'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['over grid x15xpercxx'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x5xpercxx = functions.Delay(
    lambda: by_mineral_rr_rest().loc['over grid x5xpercxx'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['over grid x5xpercxx'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___paint = functions.Delay(
    lambda: by_mineral_rr_rest().loc['paint'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['paint'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lead = functions.Delay(
    lambda: by_mineral_rr_rest().loc['lead'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lead'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___plastics = functions.Delay(
    lambda: by_mineral_rr_rest().loc['plastics'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['plastics'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___polypropylene = functions.Delay(
    lambda: by_mineral_rr_rest().loc['polypropylene'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['polypropylene'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock = functions.Delay(
    lambda: by_mineral_rr_rest().loc['rock'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['rock'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock_wool = functions.Delay(
    lambda: by_mineral_rr_rest().loc['rock wool'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['rock wool'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___sand = functions.Delay(
    lambda: by_mineral_rr_rest().loc['sand'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['sand'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_sand = functions.Delay(
    lambda: by_mineral_rr_rest().loc['silicon sand'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['silicon sand'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_wafer_modules = functions.Delay(
    lambda: by_mineral_rr_rest().loc['silicon wafer modules'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['silicon wafer modules'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silver = functions.Delay(
    lambda: by_mineral_rr_rest().loc['silver'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['silver'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___site_preparation = functions.Delay(
    lambda: by_mineral_rr_rest().loc['site preparation'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['site preparation'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tin = functions.Delay(
    lambda: by_mineral_rr_rest().loc['tin'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['tin'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___soda_ash = functions.Delay(
    lambda: by_mineral_rr_rest().loc['soda ash'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['soda ash'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___steel = functions.Delay(
    lambda: by_mineral_rr_rest().loc['steel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['steel'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___synthetic_oil = functions.Delay(
    lambda: by_mineral_rr_rest().loc['synthetic oil'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['synthetic oil'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tellurium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['tellurium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['tellurium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['titanium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['titanium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium_dioxide = functions.Delay(
    lambda: by_mineral_rr_rest().loc['titanium dioxide'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['titanium dioxide'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___vanadium = functions.Delay(
    lambda: by_mineral_rr_rest().loc['vanadium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['vanadium'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___wires = functions.Delay(
    lambda: by_mineral_rr_rest().loc['wires'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['wires'].values, lambda: 1 / time_step())


delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___zinc = functions.Delay(
    lambda: by_mineral_rr_rest().loc['zinc'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['zinc'].values, lambda: 1 / time_step())


delays_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step = [
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___adhesive,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium_mirrors,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cadmium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___carbon_fiber,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cement,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___chromium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___copper,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___diesel,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___dy,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___electricxdivxelectronic_components,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___evacuation_lines,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___fiberglass,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___foam_glass,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___galium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass_reinforcing_plastic,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___gravel,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___indium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___iron,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___kno3_mined,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___asphalt,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lime,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___limestone,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lithium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lubricant,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___magnesium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___manganese,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___heavy_equipment,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___concrete,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___molybdenum,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_mined,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_synthetic,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___neodymium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nickel,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x15xpercxx,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x5xpercxx,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___paint,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lead,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___plastics,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___polypropylene,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock_wool,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___sand,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_sand,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_wafer_modules,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silver,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___site_preparation,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tin,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___soda_ash,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___steel,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___synthetic_oil,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tellurium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium_dioxide,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___vanadium,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___wires,
    delay_by_mineral_rr_rest_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___zinc]



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___adhesive = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['adhesive'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['adhesive'].values, lambda: 1 / time_step(), 22)



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['aluminium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['aluminium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium_mirrors = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['aluminium mirrors'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['aluminium mirrors'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cadmium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['cadmium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['cadmium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___carbon_fiber = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['carbon fiber'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['carbon fiber'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cement = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['cement'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['cement'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___chromium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['chromium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['chromium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___copper = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['copper'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['copper'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___diesel = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['diesel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['diesel'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___dy = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['dy'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['dy'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___electricxdivxelectronic_components = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['electricxdivxelectronic components'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['electricxdivxelectronic components'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___evacuation_lines = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['evacuation lines'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['evacuation lines'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___fiberglass = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['fiberglass'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['fiberglass'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___foam_glass = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['foam glass'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['foam glass'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___galium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['galium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['galium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['glass'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['glass'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass_reinforcing_plastic = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['glass reinforcing plastic'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['glass reinforcing plastic'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___gravel = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['gravel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['gravel'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___indium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['indium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['indium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___iron = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['iron'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['iron'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___kno3_mined = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['kno3 mined'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['kno3 mined'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___asphalt = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['asphalt'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['asphalt'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lime = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['lime'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lime'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___limestone = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['limestone'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['limestone'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lithium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['lithium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lithium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lubricant = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['lubricant'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lubricant'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___magnesium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['magnesium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['magnesium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___manganese = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['manganese'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['manganese'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___heavy_equipment = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['heavy equipment'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['heavy equipment'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___concrete = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['concrete'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['concrete'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___molybdenum = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['molybdenum'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['molybdenum'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_mined = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['nano3 mined'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['nano3 mined'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_synthetic = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['nano3 synthetic'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['nano3 synthetic'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___neodymium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['neodymium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['neodymium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nickel = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['nickel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['nickel'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x15xpercxx = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['over grid x15xpercxx'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['over grid x15xpercxx'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x5xpercxx = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['over grid x5xpercxx'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['over grid x5xpercxx'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___paint = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['paint'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['paint'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lead = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['lead'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['lead'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___plastics = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['plastics'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['plastics'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___polypropylene = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['polypropylene'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['polypropylene'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['rock'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['rock'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock_wool = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['rock wool'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['rock wool'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___sand = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['sand'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['sand'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_sand = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['silicon sand'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['silicon sand'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_wafer_modules = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['silicon wafer modules'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['silicon wafer modules'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silver = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['silver'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['silver'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___site_preparation = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['site preparation'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['site preparation'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tin = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['tin'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['tin'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___soda_ash = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['soda ash'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['soda ash'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___steel = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['steel'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['steel'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___synthetic_oil = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['synthetic oil'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['synthetic oil'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tellurium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['tellurium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['tellurium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['titanium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['titanium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium_dioxide = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['titanium dioxide'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['titanium dioxide'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___vanadium = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['vanadium'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['vanadium'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___wires = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['wires'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['wires'].values, lambda: 1 / time_step())



delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___zinc = functions.Delay(
    lambda: by_mineral_rr_alt_techn().loc['zinc'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: current_recycling_rates_minerals_alt_techn().loc['zinc'].values, lambda: 1 / time_step())



delays_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step = [
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___adhesive,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___aluminium_mirrors,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cadmium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___carbon_fiber,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___cement,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___chromium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___copper,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___diesel,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___dy,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___electricxdivxelectronic_components,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___evacuation_lines,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___fiberglass,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___foam_glass,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___galium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___glass_reinforcing_plastic,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___gravel,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___indium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___iron,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___kno3_mined,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___asphalt,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lime,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___limestone,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lithium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lubricant,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___magnesium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___manganese,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___heavy_equipment,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___concrete,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___molybdenum,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_mined,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nano3_synthetic,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___neodymium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___nickel,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x15xpercxx,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___over_grid_x5xpercxx,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___paint,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___lead,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___plastics,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___polypropylene,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___rock_wool,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___sand,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_sand,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silicon_wafer_modules,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___silver,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___site_preparation,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tin,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___soda_ash,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___steel,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___synthetic_oil,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___tellurium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___titanium_dioxide,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___vanadium,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___wires,
    delay_by_mineral_rr_alt_techn_round1__time_step___time_step_current_recycling_rates_minerals_alt_techn_1__time_step___zinc]


def _init_recycling_rates_minerals_alt_techn():
    """
    Implicit

    See docs for recycling_rates_minerals_alt_techn

    setup

    Provides initial conditions for recycling_rates_minerals_alt_techn function
    """
    
    ret = current_recycling_rates_minerals_alt_techn() * all_minerals_virginxquestx()
    #test_nans(ret, 2048)
    return ret


@cache('step')
def _drecycling_rates_minerals_alt_techn_dt():
    """
    Implicit

    See docs for recycling_rates_minerals_alt_techn

    component

    Provides derivative for recycling_rates_minerals_alt_techn function
    """
    
    ret = improvement_recycling_rates_minerals_alt_techn()
    #test_nans(ret, 2049)
    return ret



integ_recycling_rates_minerals_alt_techn = functions.Integ(
    lambda: _drecycling_rates_minerals_alt_techn_dt(),
    lambda: _init_recycling_rates_minerals_alt_techn())

initial_days_per_yearsec_per_day = functions.Initial(lambda: days_per_year() * sec_per_day())

initial_area1land_area_fractionlayer_depth = functions.Initial(
    lambda: area() * (1 - land_area_fraction()) * layer_depth())

initial_mass_heat_capwatt_per_j_ssec_per_yrdensity = functions.Initial(
    lambda: mass_heat_cap() * watt_per_j_s() / sec_per_yr() * density())

initial_co2_rad_forcenplog2 = functions.Initial(lambda: co2_rad_force() * np.log(2))

initial_x2x_co2x_forcingclimate_sensitivity_to_2x_co2 = functions.Initial(
    lambda: x2x_co2x_forcing() / climate_sensitivity_to_2x_co2())


initial_heat_transfer_ratemean_depth_of_adjacent_layersheat_diffusion_covareddy_diff_coeffeddy_diff_mean1heat_diffusion_covar = functions.Initial(lambda: (heat_transfer_rate()*mean_depth_of_adjacent_layers().loc['layer1'].values)*(heat_diffusion_covar()*(eddy_diff_coeff()/eddy_diff_mean())+(1-heat_diffusion_covar())))

initial_eddy_diff_coeff_indexeddy_diff_mean = functions.Initial(
    lambda: eddy_diff_coeff_index() * eddy_diff_mean())


initial_arealand_area_fractionland_thickness1land_area_fractionmixed_depth = functions.Initial(lambda: area()*(land_area_fraction()*land_thickness()+(1-land_area_fraction())*mixed_depth()))


def _init_heat_in_deep_ocean():
    """
    Implicit

    See docs for heat_in_deep_ocean

    setup

    Provides initial conditions for heat_in_deep_ocean function
    """
    
    ret = xr.DataArray(data=[
        init_deep_ocean_temp().loc['layer1'].values * deep_ocean_heat_cap().loc['layer1'].values,
        init_deep_ocean_temp().loc['layer2'].values * deep_ocean_heat_cap().loc['layer2'].values,
        init_deep_ocean_temp().loc['layer3'].values * deep_ocean_heat_cap().loc['layer3'].values,
        init_deep_ocean_temp().loc['layer4'].values * deep_ocean_heat_cap().loc['layer4'].values],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 2050)
    return ret


@cache('step')
def _dheat_in_deep_ocean_dt():
    """
    Implicit

    See docs for heat_in_deep_ocean

    component

    Provides derivative for heat_in_deep_ocean function
    """
    
    ret = xr.DataArray(data=[
        heat_transfer().loc['layer1'].values - heat_transfer().loc['layer2'].values,
        heat_transfer().loc['layer2'].values - heat_transfer().loc['layer3'].values,
        heat_transfer().loc['layer3'].values - heat_transfer().loc['layer4'].values,
        heat_transfer().loc['layer4'].values],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 2051)
    return ret



integ_heat_in_deep_ocean = functions.Integ(lambda: _dheat_in_deep_ocean_dt(),
                                           lambda: _init_heat_in_deep_ocean())

initial_upper_layer_volume_vuvolumetric_heat_capacityarea = functions.Initial(
    lambda: upper_layer_volume_vu() * volumetric_heat_capacity() / area())

initial_lower_layer_volume_vuvolumetric_heat_capacityarea = functions.Initial(
    lambda: lower_layer_volume_vu() * volumetric_heat_capacity() / area())

integ_heat_in_atmosphere_and_upper_ocean = functions.Integ(
    lambda: effective_radiative_forcing() - feedback_cooling() - heat_transfer().loc['layer1'].values,
    lambda: init_atmos_uocean_temp() * atm_and_upper_ocean_heat_cap())

delay_total_radiative_forcing_roundtime_step__time_step___time_step_total_radiative_forcing_time_step__time_step = functions.Delay(
    lambda: total_radiative_forcing(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: 0, lambda: time_step() / time_step(), 25)

integ_sf6 = functions.Integ(lambda: global_sf6_emissions() - sf6_uptake(), lambda: initial_sf6())

initial_initial_ch4_concppb_ch4_per_mton_ch4 = functions.Initial(
    lambda: initial_ch4_conc() / ppb_ch4_per_mton_ch4())


def _init_hfc_in_atm():
    """
    Implicit

    See docs for hfc_in_atm

    setup

    Provides initial conditions for hfc_in_atm function
    """
    
    ret = initial_hfc()
    #test_nans(ret, 2052)
    return ret


@cache('step')
def _dhfc_in_atm_dt():
    """
    Implicit

    See docs for hfc_in_atm

    component

    Provides derivative for hfc_in_atm function
    """
    
    ret = global_hfc_emissions() - hfc_uptake()
    #test_nans(ret, 2053)
    return ret



integ_hfc_in_atm = functions.Integ(lambda: _dhfc_in_atm_dt(), lambda: _init_hfc_in_atm())


integ_total_c_from_permafrost = functions.Integ(lambda: flux_c_from_permafrost_release()+ch4_emissions_from_permafrost_and_clathrate()/ch4_per_c()/mtons_per_gtons(), lambda: 0)

integ_total_ch4_released = functions.Integ(
    lambda: ch4_emissions_from_permafrost_and_clathrate() / ch4_per_c() / mtons_per_gtons(),
    lambda: 0)

initial_ppt_per_molch4_molar_massg_per_tonton_per_mtonppt_per_ppb = functions.Initial(
    lambda: ppt_per_mol() / ch4_molar_mass() * g_per_ton() * ton_per_mton() / ppt_per_ppb())

initial_ppt_per_moln2oxdashxn_molar_massg_per_tonton_per_mtonppt_per_ppb = functions.Initial(
    lambda: ppt_per_mol() / n2oxdashxn_molar_mass() * g_per_ton() * ton_per_mton() / ppt_per_ppb())

initial_ppt_per_molhfc_molar_massg_per_ton = functions.Initial(
    lambda: ppt_per_mol() / hfc_molar_mass() * g_per_ton())

initial_ppt_per_molcf4_molar_massg_per_ton = functions.Initial(
    lambda: ppt_per_mol() / cf4_molar_mass() * g_per_ton())

integ_pfc_in_atm = functions.Integ(lambda: global_total_pfc_emissions() - pfc_uptake(),
                                   lambda: init_pfc_in_atm())

integ_n2o_in_atm = functions.Integ(lambda: global_n2o_emissions() - n2o_uptake(),
                                   lambda: initial_n2o())

initial_inital_hfc_conppt_hfc_per_tons_hfc = functions.Initial(
    lambda: inital_hfc_con() / ppt_hfc_per_tons_hfc())

initial_preindustrial_pfc_concppt_pfc_per_tons_pfc = functions.Initial(
    lambda: preindustrial_pfc_conc() / ppt_pfc_per_tons_pfc())

initial_ppt_per_molsf6_molar_massg_per_ton = functions.Initial(
    lambda: ppt_per_mol() / sf6_molar_mass() * g_per_ton())

initial_initial_n2o_concppb_n2o_per_mtonn = functions.Initial(
    lambda: initial_n2o_conc() / ppb_n2o_per_mtonn())


integ_ch4_in_atm = functions.Integ(lambda: ch4_emissions_from_permafrost_and_clathrate()+global_anthropogenic_ch4_emissions()+natural_ch4_emissions()-ch4_uptake(), lambda: initial_ch4())


initial_sensitivity_of_c_uptake_to_temperaturesensitivity_of_pco2_dic_to_temperature_mean = functions.Initial(lambda: sensitivity_of_c_uptake_to_temperature()*sensitivity_of_pco2_dic_to_temperature_mean())

initial_mixed_depthlayer_depth2 = functions.Initial(lambda: (mixed_depth() + layer_depth().loc['layer1'].values) / 2)

initial_biostim_coeff_indexbiostim_coeff_mean = functions.Initial(
    lambda: biostim_coeff_index() * biostim_coeff_mean())

integ_c_in_humus = functions.Integ(
    lambda: flux_biomass_to_humus() - flux_humus_to_atmosphere() - flux_humus_to_ch4(),
    lambda: init_c_in_humus())

integ_c_in_mixed_layer = functions.Integ(lambda: flux_atm_to_ocean() - diffusion_flux().loc['layer1'].values,
                                         lambda: init_c_in_mixed_ocean_per_meter() * mixed_depth())


integ_c_in_atmosphere = functions.Integ(lambda: c_from_ch4_oxidation()+flux_biomass_to_atmosphere()+flux_humus_to_atmosphere()+total_c_anthro_emissions()-flux_atm_to_biomass()-flux_atm_to_ocean()+flux_c_from_permafrost_release(), lambda: init_c_in_atmos())


initial_sensitivity_of_c_uptake_to_temperaturestrength_of_temp_effect_on_land_c_flux_mean = functions.Initial(lambda: sensitivity_of_c_uptake_to_temperature()*strength_of_temp_effect_on_land_c_flux_mean())

initial_preind_ocean_c_per_metermixed_depth = functions.Initial(
    lambda: preind_ocean_c_per_meter() * mixed_depth())


integ_c_in_biomass = functions.Integ(lambda: flux_atm_to_biomass()-flux_biomass_to_atmosphere()-flux_biomass_to_ch4()-flux_biomass_to_humus(), lambda: init_c_in_biomass())


def _init_c_in_deep_ocean():
    """
    Implicit

    See docs for c_in_deep_ocean

    setup

    Provides initial conditions for c_in_deep_ocean function
    """
    
    ret = xr.DataArray(data=[
        init_c_in_deep_ocean_per_meter().loc['layer1'].values * layer_depth().loc['layer1'].values,
        init_c_in_deep_ocean_per_meter().loc['layer2'].values * layer_depth().loc['layer2'].values,
        init_c_in_deep_ocean_per_meter().loc['layer3'].values * layer_depth().loc['layer3'].values,
        init_c_in_deep_ocean_per_meter().loc['layer4'].values * layer_depth().loc['layer4'].values],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 2054)
    return ret


@cache('step')
def _dc_in_deep_ocean_dt():
    """
    Implicit

    See docs for c_in_deep_ocean

    component

    Provides derivative for c_in_deep_ocean function
    """
    
    ret = xr.DataArray(data=[
        diffusion_flux().loc['layer1'].values - diffusion_flux().loc['layer2'].values,
        diffusion_flux().loc['layer2'].values - diffusion_flux().loc['layer3'].values,
        diffusion_flux().loc['layer3'].values - diffusion_flux().loc['layer4'].values,
        diffusion_flux().loc['layer4'].values],
        coords={'layers': ['layer1', 'layer2', 'layer3', 'layer4']},
        dims=['layers'])
    #test_nans(ret, 2055)
    return ret



integ_c_in_deep_ocean = functions.Integ(lambda: _dc_in_deep_ocean_dt(),
                                        lambda: _init_c_in_deep_ocean())

initial_layer_deptheddy_diff_coeffmean_depth_of_adjacent_layers = functions.Initial(
    lambda: layer_depth().loc['layer1'].values / (eddy_diff_coeff() / mean_depth_of_adjacent_layers().loc['layer1'].values))


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___agriculture_hunting_forestry_and_fishing___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['agriculture hunting forestry and fishing','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['agriculture hunting forestry and fishing','blue water'].values,
    lambda: 1 / time_step(), 26)


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___mining_and_quarrying___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['mining and quarrying','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['mining and quarrying','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___food_beverages_and_tobacco___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['food beverages and tobacco','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['food beverages and tobacco','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___textiles_and_textile_products___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['textiles and textile products','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['textiles and textile products','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___leather_leather_and_footwear___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['leather leather and footwear','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['leather leather and footwear','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wood_and_products_of_woood_and_cork___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['wood and products of woood and cork','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['wood and products of woood and cork','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___pulp_paper_printing_and_publishing___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['pulp paper printing and publishing','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['pulp paper printing and publishing','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___coke_refined_petroleum_and_nuclear_fuel___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['coke refined petroleum and nuclear fuel','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['coke refined petroleum and nuclear fuel','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___chemicals_and_chemical_products___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['chemicals and chemical products','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['chemicals and chemical products','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___rubber_and_plastics___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['rubber and plastics','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['rubber and plastics','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_non_metalic_mineral___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other non metalic mineral','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other non metalic mineral','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___basic_metals_and_fabricated_metal___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['basic metals and fabricated metal','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['basic metals and fabricated metal','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___machinery_nec___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['machinery nec','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['machinery nec','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electrical_and_optical_equipment___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['electrical and optical equipment','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['electrical and optical equipment','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___transport_equipment___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['transport equipment','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['transport equipment','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___manufacturing_nec_recycling___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['manufacturing nec recycling','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['manufacturing nec recycling','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electricity_gas_and_water_supply___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['electricity gas and water supply','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['electricity gas and water supply','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___construction___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['construction','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['construction','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___hotels_and_restaurants___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['hotels and restaurants','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['hotels and restaurants','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___inland_transport___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['inland transport','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['inland transport','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___water_transport___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['water transport','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['water transport','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___air_transport___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['air transport','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['air transport','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___post_and_telecommunications___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['post and telecommunications','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['post and telecommunications','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___financial_intermedation___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['financial intermedation','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['financial intermedation','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___real_estate_activities___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['real estate activities','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['real estate activities','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___renting_od_meq_and_other_business_activities___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['renting od meq and other business activities','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['renting od meq and other business activities','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___public_admin_and_defence_compulsory_social_security___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['public admin and defence compulsory social security','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['public admin and defence compulsory social security','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___education___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['education','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['education','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___health_and_social_work___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['health and social work','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['health and social work','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_community_social_and_persona_services___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other community social and persona services','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other community social and persona services','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___private_households_with_employed_persons___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['private households with employed persons','blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['private households with employed persons','blue water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___agriculture_hunting_forestry_and_fishing___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['agriculture hunting forestry and fishing','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['agriculture hunting forestry and fishing','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___mining_and_quarrying___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['mining and quarrying','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['mining and quarrying','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___food_beverages_and_tobacco___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['food beverages and tobacco','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['food beverages and tobacco','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___textiles_and_textile_products___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['textiles and textile products','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['textiles and textile products','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___leather_leather_and_footwear___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['leather leather and footwear','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['leather leather and footwear','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wood_and_products_of_woood_and_cork___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['wood and products of woood and cork','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['wood and products of woood and cork','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___pulp_paper_printing_and_publishing___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['pulp paper printing and publishing','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['pulp paper printing and publishing','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___coke_refined_petroleum_and_nuclear_fuel___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['coke refined petroleum and nuclear fuel','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['coke refined petroleum and nuclear fuel','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___chemicals_and_chemical_products___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['chemicals and chemical products','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['chemicals and chemical products','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___rubber_and_plastics___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['rubber and plastics','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['rubber and plastics','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_non_metalic_mineral___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other non metalic mineral','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other non metalic mineral','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___basic_metals_and_fabricated_metal___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['basic metals and fabricated metal','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['basic metals and fabricated metal','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___machinery_nec___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['machinery nec','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['machinery nec','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electrical_and_optical_equipment___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['electrical and optical equipment','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['electrical and optical equipment','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___transport_equipment___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['transport equipment','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['transport equipment','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___manufacturing_nec_recycling___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['manufacturing nec recycling','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['manufacturing nec recycling','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electricity_gas_and_water_supply___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['electricity gas and water supply','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['electricity gas and water supply','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___construction___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['construction','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['construction','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___hotels_and_restaurants___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['hotels and restaurants','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['hotels and restaurants','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___inland_transport___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['inland transport','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['inland transport','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___water_transport___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['water transport','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['water transport','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___air_transport___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['air transport','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['air transport','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___post_and_telecommunications___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['post and telecommunications','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['post and telecommunications','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___financial_intermedation___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['financial intermedation','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['financial intermedation','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___real_estate_activities___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['real estate activities','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['real estate activities','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___renting_od_meq_and_other_business_activities___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['renting od meq and other business activities','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['renting od meq and other business activities','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___public_admin_and_defence_compulsory_social_security___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['public admin and defence compulsory social security','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['public admin and defence compulsory social security','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___education___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['education','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['education','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___health_and_social_work___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['health and social work','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['health and social work','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_community_social_and_persona_services___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other community social and persona services','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other community social and persona services','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___private_households_with_employed_persons___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['private households with employed persons','green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['private households with employed persons','green water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___agriculture_hunting_forestry_and_fishing___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['agriculture hunting forestry and fishing','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['agriculture hunting forestry and fishing','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___mining_and_quarrying___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['mining and quarrying','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['mining and quarrying','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___food_beverages_and_tobacco___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['food beverages and tobacco','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['food beverages and tobacco','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___textiles_and_textile_products___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['textiles and textile products','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['textiles and textile products','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___leather_leather_and_footwear___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['leather leather and footwear','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['leather leather and footwear','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wood_and_products_of_woood_and_cork___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['wood and products of woood and cork','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['wood and products of woood and cork','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___pulp_paper_printing_and_publishing___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['pulp paper printing and publishing','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['pulp paper printing and publishing','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___coke_refined_petroleum_and_nuclear_fuel___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['coke refined petroleum and nuclear fuel','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['coke refined petroleum and nuclear fuel','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___chemicals_and_chemical_products___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['chemicals and chemical products','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['chemicals and chemical products','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___rubber_and_plastics___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['rubber and plastics','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['rubber and plastics','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_non_metalic_mineral___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other non metalic mineral','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other non metalic mineral','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___basic_metals_and_fabricated_metal___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['basic metals and fabricated metal','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['basic metals and fabricated metal','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___machinery_nec___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['machinery nec','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['machinery nec','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electrical_and_optical_equipment___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['electrical and optical equipment','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['electrical and optical equipment','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___transport_equipment___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['transport equipment','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['transport equipment','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___manufacturing_nec_recycling___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['manufacturing nec recycling','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['manufacturing nec recycling','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electricity_gas_and_water_supply___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['electricity gas and water supply','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['electricity gas and water supply','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___construction___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['construction','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['construction','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___hotels_and_restaurants___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['hotels and restaurants','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['hotels and restaurants','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___inland_transport___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['inland transport','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['inland transport','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___water_transport___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['water transport','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['water transport','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___air_transport___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['air transport','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['air transport','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___post_and_telecommunications___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['post and telecommunications','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['post and telecommunications','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___financial_intermedation___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['financial intermedation','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['financial intermedation','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___real_estate_activities___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['real estate activities','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['real estate activities','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___renting_od_meq_and_other_business_activities___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['renting od meq and other business activities','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['renting od meq and other business activities','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___public_admin_and_defence_compulsory_social_security___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['public admin and defence compulsory social security','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['public admin and defence compulsory social security','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___education___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['education','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['education','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___health_and_social_work___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['health and social work','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['health and social work','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_community_social_and_persona_services___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['other community social and persona services','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['other community social and persona services','gray water'].values,
    lambda: 1 / time_step())


delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___private_households_with_employed_persons___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_by_sector().loc['private households with employed persons','gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_by_sector().loc['private households with employed persons','gray water'].values,
    lambda: 1 / time_step())


delays_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step = [
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___agriculture_hunting_forestry_and_fishing___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___mining_and_quarrying___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___food_beverages_and_tobacco___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___textiles_and_textile_products___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___leather_leather_and_footwear___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wood_and_products_of_woood_and_cork___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___pulp_paper_printing_and_publishing___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___coke_refined_petroleum_and_nuclear_fuel___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___chemicals_and_chemical_products___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___rubber_and_plastics___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_non_metalic_mineral___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___basic_metals_and_fabricated_metal___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___machinery_nec___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electrical_and_optical_equipment___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___transport_equipment___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___manufacturing_nec_recycling___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electricity_gas_and_water_supply___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___construction___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___hotels_and_restaurants___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___inland_transport___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___water_transport___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___air_transport___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___post_and_telecommunications___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___financial_intermedation___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___real_estate_activities___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___renting_od_meq_and_other_business_activities___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___public_admin_and_defence_compulsory_social_security___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___education___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___health_and_social_work___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_community_social_and_persona_services___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___private_households_with_employed_persons___blue_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___agriculture_hunting_forestry_and_fishing___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___mining_and_quarrying___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___food_beverages_and_tobacco___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___textiles_and_textile_products___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___leather_leather_and_footwear___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wood_and_products_of_woood_and_cork___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___pulp_paper_printing_and_publishing___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___coke_refined_petroleum_and_nuclear_fuel___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___chemicals_and_chemical_products___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___rubber_and_plastics___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_non_metalic_mineral___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___basic_metals_and_fabricated_metal___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___machinery_nec___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electrical_and_optical_equipment___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___transport_equipment___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___manufacturing_nec_recycling___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electricity_gas_and_water_supply___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___construction___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___hotels_and_restaurants___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___inland_transport___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___water_transport___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___air_transport___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___post_and_telecommunications___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___financial_intermedation___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___real_estate_activities___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___renting_od_meq_and_other_business_activities___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___public_admin_and_defence_compulsory_social_security___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___education___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___health_and_social_work___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_community_social_and_persona_services___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___private_households_with_employed_persons___green_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___agriculture_hunting_forestry_and_fishing___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___mining_and_quarrying___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___food_beverages_and_tobacco___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___textiles_and_textile_products___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___leather_leather_and_footwear___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wood_and_products_of_woood_and_cork___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___pulp_paper_printing_and_publishing___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___coke_refined_petroleum_and_nuclear_fuel___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___chemicals_and_chemical_products___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___rubber_and_plastics___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_non_metalic_mineral___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___basic_metals_and_fabricated_metal___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___machinery_nec___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electrical_and_optical_equipment___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___transport_equipment___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___manufacturing_nec_recycling___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___electricity_gas_and_water_supply___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___construction___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___hotels_and_restaurants___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___inland_transport___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___water_transport___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___air_transport___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___post_and_telecommunications___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___financial_intermedation___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___real_estate_activities___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___renting_od_meq_and_other_business_activities___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___public_admin_and_defence_compulsory_social_security___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___education___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___health_and_social_work___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___other_community_social_and_persona_services___gray_water,
    delay_historic_water_by_type_intensities_by_sector_round1__time_step___time_step_initial_water_intensity_by_sector_1__time_step___private_households_with_employed_persons___gray_water]



delay_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step___blue_water = functions.Delay(
    lambda: historic_water_by_type_intensities_for_households().loc['blue water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_for_households().loc['blue water'].values,
    lambda: 1 / time_step(), 27)



delay_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step___green_water = functions.Delay(
    lambda: historic_water_by_type_intensities_for_households().loc['green water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_for_households().loc['green water'].values,
    lambda: 1 / time_step(), 28)



delay_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step___gray_water = functions.Delay(
    lambda: historic_water_by_type_intensities_for_households().loc['gray water'].values,
    lambda: round(1 / time_step()) * time_step(), lambda: initial_water_intensity_for_households().loc['gray water'].values,
    lambda: 1 / time_step(), 29)



delays_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step = [
    delay_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step___blue_water,
    delay_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step___green_water,
    delay_historic_water_by_type_intensities_for_households_round1__time_step___time_step_initial_water_intensity_for_households_1__time_step___gray_water]


def _init_water_intensity_for_households():
    """
    Implicit

    See docs for water_intensity_for_households

    setup

    Provides initial conditions for water_intensity_for_households function
    """
    
    ret = initial_water_intensity_for_households()
    #test_nans(ret, 2056)
    return ret


@cache('step')
def _dwater_intensity_for_households_dt():
    """
    Implicit

    See docs for water_intensity_for_households

    component

    Provides derivative for water_intensity_for_households function
    """
    
    ret = variation_water_intensity_households()
    #test_nans(ret, 2057)
    return ret



integ_water_intensity_for_households = functions.Integ(
    lambda: _dwater_intensity_for_households_dt(), lambda: _init_water_intensity_for_households())


def _init_water_intensity_by_sector():
    """
    Implicit

    See docs for water_intensity_by_sector

    setup

    Provides initial conditions for water_intensity_by_sector function
    """
    
    ret = initial_water_intensity_by_sector()
    #test_nans(ret, 2058)
    return ret


@cache('step')
def _dwater_intensity_by_sector_dt():
    """
    Implicit

    See docs for water_intensity_by_sector

    component

    Provides derivative for water_intensity_by_sector function
    """
    
    ret = variation_water_intensity_by_sector()
    #test_nans(ret, 2059)
    return ret



integ_water_intensity_by_sector = functions.Integ(lambda: _dwater_intensity_by_sector_dt(),
                                                  lambda: _init_water_intensity_by_sector())

delay_pe_traditional_biomass_consum_ej_round1__time_step___time_step_30_1__time_step = functions.Delay(
    lambda: pe_traditional_biomass_consum_ej(), lambda: round(1 / time_step()) * time_step(),
    lambda: 30, lambda: 1 / time_step(), 30)

delay_eroist_system_until_x2015x_roundtime_step__time_step___time_step_eroistx1995x_time_step__time_step = functions.Delay(
    lambda: eroist_system_until_x2015x(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: eroistx1995x(), lambda: time_step() / time_step(), 31)

delay_eroist_system_roundtime_step__time_step___time_step_eroistx1995x_time_step__time_step = functions.Delay(
    lambda: eroist_system(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: eroistx1995x(), lambda: time_step() / time_step(), 32)

delay_eroist_system_round1__time_step___time_step_eroistx1995x_1__time_step = functions.Delay(
    lambda: eroist_system(), lambda: round(1 / time_step()) * time_step(), lambda: eroistx1995x(),
    lambda: 1 / time_step())

delay_tfec_intensity_until_x2009x_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: tfec_intensity_until_x2009x(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: 0, lambda: time_step() / time_step())

delay_fe_tot_generation_all_res_elec_twh_round1__time_step___time_step_2463_1__time_step = functions.Delay(
    lambda: fe_tot_generation_all_res_elec_twh(), lambda: round(1 / time_step()) * time_step(),
    lambda: 2463, lambda: 1 / time_step())

delay_fes_res_for_heat_ej_round1__time_step___time_step_3488_1__time_step = functions.Delay(
    lambda: fes_res_for_heat_ej(), lambda: round(1 / time_step()) * time_step(), lambda: 3.488,
    lambda: 1 / time_step())

delay_desired_gdp_round1__time_step___time_step_2916_1__time_step = functions.Delay(
    lambda: desired_gdp(), lambda: round(1 / time_step()) * time_step(), lambda: 29.16,
    lambda: 1 / time_step())

integ_desired_gdppc = functions.Integ(lambda: desired_variation_gdppc(),
                                      lambda: gdppc_initial_year())

delay_tfec_intensity_until_x2009x_without_eroi_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: tfec_intensity_until_x2009x_without_eroi(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step(), 33)

delay_tpes_intensity_until_x2009x_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: tpes_intensity_until_x2009x(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: 0, lambda: time_step() / time_step(), 34)

delay_tfes_intensity_ej_tdollar_without_eroi_round1__time_step___time_step_8827_1__time_step = functions.Delay(
    lambda: tfes_intensity_ej_tdollar_without_eroi(), lambda: round(1 / time_step()) * time_step(),
    lambda: 8.827, lambda: 1 / time_step(), 35)

integ_installed_capacity_phs_tw = functions.Integ(
    lambda: phs_capacity_under_construction() - wear_phs(), lambda: initial_instal_cap_phs())


integ_phs_planned_capacity = functions.Integ(lambda: new_phs_capacity_under_planning()+replacement_capacity_phs()-phs_capacity_under_construction(), lambda: initial_capacity_in_construction_phs())

integ_required_capacity_phs = functions.Integ(
    lambda: new_required_phs_capacity() - new_phs_capacity_under_planning(),
    lambda: initial_required_capacity_phs())

integ_batteries_evxplusxhibxplusx2we = functions.Integ(
    lambda: new_batteries() + replacement_batteries() - discarded_batteries(), lambda: 1)


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___hydro = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['hydro'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___geotxdashxelec = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['geotxdashxelec'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___solid_bioexdashxelec = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['solid bioexdashxelec'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___oceanic = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['oceanic'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___wind_onshore = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['wind onshore'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___wind_offshore = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['wind offshore'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___solar_pv = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['solar pv'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___csp = functions.Delay(
    lambda: installed_capacity_res_elec_tw().loc['csp'].values, lambda: round(1 / time_step()) * time_step(),
    lambda: 0, lambda: 1 / time_step())


delays_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step = [
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___hydro,
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___geotxdashxelec,
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___solid_bioexdashxelec,
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___oceanic,
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___wind_onshore,
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___wind_offshore,
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___solar_pv,
    delay_installed_capacity_res_elec_tw_round1__time_step___time_step_0_1__time_step___csp]


delay_rurr_coal_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_coal_start_year_plg(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: 0, lambda: time_step() / time_step())

delay_rurr_tot_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_tot_gas_until_start_year_plg(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step())

delay_rurr_conv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_conv_oil_until_start_year_plg(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step())

delay_rurr_unconv_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_unconv_oil_until_start_year_plg(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step())

delay_rurr_tot_oil_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_tot_oil_until_start_year_plg(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step())

delay_rurr_conv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_conv_gas_until_start_year_plg(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step())

delay_rurr_unconv_gas_until_start_year_plg_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: rurr_unconv_gas_until_start_year_plg(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step())


integ_rurr_conv_gas = functions.Integ(lambda: -extraction_conv_gas_ej()-flow_conv_gas_left_in_ground(), lambda: urr_conv_gas()-cumulated_conv_gas_extraction_to_x1995x()*separate_conv_and_unconv_gasxquestx())


integ_rurr_unconv_gas = functions.Integ(lambda: -extraction_unconv_gas_ej()-flow_unconv_gas_left_in_ground(), lambda: urr_unconv_gas()-cumulated_unconv_gas_extraction_to_x1995x()*separate_conv_and_unconv_gasxquestx())


integ_rurr_conv_oil = functions.Integ(lambda: -extraction_conv_oil_ej()-flow_conv_oil_left_in_ground(), lambda: urr_conv_oil()-cumulated_conv_oil_extraction_to_x1995x()*separate_conv_and_unconv_oilxquestx())


def _init_recycling_rates_minerals_rest():
    """
    Implicit

    See docs for recycling_rates_minerals_rest

    setup

    Provides initial conditions for recycling_rates_minerals_rest function
    """
    
    ret = current_recycling_rates_minerals() * all_minerals_virginxquestx()
    #test_nans(ret, 2060)
    return ret


@cache('step')
def _drecycling_rates_minerals_rest_dt():
    """
    Implicit

    See docs for recycling_rates_minerals_rest

    component

    Provides derivative for recycling_rates_minerals_rest function
    """
    
    ret = improvement_recycling_rates_minerals_rest()
    #test_nans(ret, 2061)
    return ret



integ_recycling_rates_minerals_rest = functions.Integ(
    lambda: _drecycling_rates_minerals_rest_dt(), lambda: _init_recycling_rates_minerals_rest())


def _init_percents_h_vehicles():
    """
    Implicit

    See docs for percents_h_vehicles

    setup

    Provides initial conditions for percents_h_vehicles function
    """
    
    ret = xr.DataArray(data=[
        1 - initial_2w_percent(),np.ones([1]) * 0,np.ones([1]) * 0,np.ones([1]) * 0,
        initial_2w_percent(),np.ones([1]) * 0],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 2062)
    return ret


@cache('step')
def _dpercents_h_vehicles_dt():
    """
    Implicit

    See docs for percents_h_vehicles

    component

    Provides derivative for percents_h_vehicles function
    """
    
    ret = xr.DataArray(data=[
        var_percents_h_vehicles().loc['liq 4wheels'].values,
        var_percents_h_vehicles().loc['hib 4wheels'].values,
        var_percents_h_vehicles().loc['elec 4wheels'].values,
        var_percents_h_vehicles().loc['gas 4wheels'].values,
        var_percents_h_vehicles().loc['liq 2wheels'].values,
        var_percents_h_vehicles().loc['elec 2wheels'].values],
        coords={'households vehicles': ['liq 4wheels', 'hib 4wheels', 'elec 4wheels', 'gas 4wheels', 'liq 2wheels', 'elec 2wheels']},
        dims=['households vehicles'])
    #test_nans(ret, 2063)
    return ret



integ_percents_h_vehicles = functions.Integ(lambda: _dpercents_h_vehicles_dt(),
                                            lambda: _init_percents_h_vehicles())


def _init_percent_t_vehicles():
    """
    Implicit

    See docs for percent_t_vehicles

    setup

    Provides initial conditions for percent_t_vehicles function
    """
    
    ret = initial_percent_t_vehicles()
    #test_nans(ret, 2064)
    return ret


@cache('step')
def _dpercent_t_vehicles_dt():
    """
    Implicit

    See docs for percent_t_vehicles

    component

    Provides derivative for percent_t_vehicles function
    """
    
    ret = var_percent_t_vehicles()
    #test_nans(ret, 2065)
    return ret



integ_percent_t_vehicles = functions.Integ(lambda: _dpercent_t_vehicles_dt(),
                                           lambda: _init_percent_t_vehicles())


def _init_energy_intensity_of_households_transport():
    """
    Implicit

    See docs for energy_intensity_of_households_transport

    setup

    Provides initial conditions for energy_intensity_of_households_transport function
    """
    
    ret = initial_energy_intensity_of_households_transport_x2009x()
    #test_nans(ret, 2066)
    return ret


@cache('step')
def _denergy_intensity_of_households_transport_dt():
    """
    Implicit

    See docs for energy_intensity_of_households_transport

    component

    Provides derivative for energy_intensity_of_households_transport function
    """
    
    ret = variation_energy_intensity_of_households_transport()
    #test_nans(ret, 2067)
    return ret



integ_energy_intensity_of_households_transport = functions.Integ(
    lambda: _denergy_intensity_of_households_transport_dt(),
    lambda: _init_energy_intensity_of_households_transport())

integ_max_percent_x2x_wheels = functions.Integ(lambda: rate_4w_to_2w(),
                                               lambda: initial_2w_percent())

integ_max_percent_x4x_wheels = functions.Integ(lambda: -rate_4w_to_2w(),
                                               lambda: 1 - initial_2w_percent())

delay_pes_oil_ej_roundtime_step__time_step___time_step_1395_time_step__time_step = functions.Delay(
    lambda: pes_oil_ej(), lambda: round(time_step() / time_step()) * time_step(), lambda: 139.5,
    lambda: time_step() / time_step())


def _init_minerals_extraction_projection_rest_cte_rr():
    """
    Implicit

    See docs for minerals_extraction_projection_rest_cte_rr

    setup

    Provides initial conditions for minerals_extraction_projection_rest_cte_rr function
    """
    
    ret = initial_minerals_extraction_rest() * mt_per_tonne()
    #test_nans(ret, 2068)
    return ret


@cache('step')
def _dminerals_extraction_projection_rest_cte_rr_dt():
    """
    Implicit

    See docs for minerals_extraction_projection_rest_cte_rr

    component

    Provides derivative for minerals_extraction_projection_rest_cte_rr function
    """
    
    ret = variation_minerals_extraction_rest()
    #test_nans(ret, 2069)
    return ret



integ_minerals_extraction_projection_rest_cte_rr = functions.Integ(
    lambda: _dminerals_extraction_projection_rest_cte_rr_dt(),
    lambda: _init_minerals_extraction_projection_rest_cte_rr())

integ_potential_peavail_biofuels_land_marg_ej = functions.Integ(lambda: new_biofuels_land_marg(),
                                                                lambda: 0)

delay_tfec_res_ej_round1__time_step___time_step_43_1__time_step = functions.Delay(
    lambda: tfec_res_ej(), lambda: round(1 / time_step()) * time_step(), lambda: 43,
    lambda: 1 / time_step())

delay_tpe_from_res_ej_round1__time_step___time_step_4803_1__time_step = functions.Delay(
    lambda: tpe_from_res_ej(), lambda: round(1 / time_step()) * time_step(), lambda: 48.03,
    lambda: 1 / time_step())

delay_share_res_vs_tpes_round1__time_step___time_step_0123_1__time_step = functions.Delay(
    lambda: share_res_vs_tpes(), lambda: round(1 / time_step()) * time_step(), lambda: 0.123,
    lambda: 1 / time_step())

delay_share_res_vs_tfec_round1__time_step___time_step_01614_1__time_step = functions.Delay(
    lambda: share_res_vs_tfec(), lambda: round(1 / time_step()) * time_step(), lambda: 0.1614,
    lambda: 1 / time_step())


def _init_installed_capacity_res_heatxdashxnc_tw():
    """
    Implicit

    See docs for installed_capacity_res_heatxdashxnc_tw

    setup

    Provides initial conditions for installed_capacity_res_heatxdashxnc_tw function
    """
    
    ret = initial_value_res_for_heatxdashxnc()
    #test_nans(ret, 2070)
    return ret


@cache('step')
def _dinstalled_capacity_res_heatxdashxnc_tw_dt():
    """
    Implicit

    See docs for installed_capacity_res_heatxdashxnc_tw

    component

    Provides derivative for installed_capacity_res_heatxdashxnc_tw function
    """
    
    ret = new_res_capacity_for_heatxdashxnc_tw() + replacement_res_for_heatxdashxnc_tw(
    ) - wear_res_capacity_for_heatxdashxnc_tw()
    #test_nans(ret, 2071)
    return ret



integ_installed_capacity_res_heatxdashxnc_tw = functions.Integ(
    lambda: _dinstalled_capacity_res_heatxdashxnc_tw_dt(),
    lambda: _init_installed_capacity_res_heatxdashxnc_tw())


def _init_nonxdashxenergy_use_demand_by_final_fuel_ej():
    """
    Implicit

    See docs for nonxdashxenergy_use_demand_by_final_fuel_ej

    setup

    Provides initial conditions for nonxdashxenergy_use_demand_by_final_fuel_ej function
    """
    
    ret = initial_nonxdashxenergy_use()
    #test_nans(ret, 2072)
    return ret


@cache('step')
def _dnonxdashxenergy_use_demand_by_final_fuel_ej_dt():
    """
    Implicit

    See docs for nonxdashxenergy_use_demand_by_final_fuel_ej

    component

    Provides derivative for nonxdashxenergy_use_demand_by_final_fuel_ej function
    """
    
    ret = annual_variation_nonxdashxenergy_use()
    #test_nans(ret, 2073)
    return ret



integ_nonxdashxenergy_use_demand_by_final_fuel_ej = functions.Integ(
    lambda: _dnonxdashxenergy_use_demand_by_final_fuel_ej_dt(),
    lambda: _init_nonxdashxenergy_use_demand_by_final_fuel_ej())


def _init_cum_materials_to_extract_for_ev_batteries():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries

    setup

    Provides initial conditions for cum_materials_to_extract_for_ev_batteries function
    """
    
    ret = initial_cumulated_material_requirements_for_ev_batteries_x1995x()
    #test_nans(ret, 2074)
    return ret


@cache('step')
def _dcum_materials_to_extract_for_ev_batteries_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries

    component

    Provides derivative for cum_materials_to_extract_for_ev_batteries function
    """
    
    ret = total_materials_to_extract_for_ev_batteries_mt()
    #test_nans(ret, 2075)
    return ret



integ_cum_materials_to_extract_for_ev_batteries = functions.Integ(
    lambda: _dcum_materials_to_extract_for_ev_batteries_dt(),
    lambda: _init_cum_materials_to_extract_for_ev_batteries())


def _init_cum_materials_requirements_for_ev_batteries():
    """
    Implicit

    See docs for cum_materials_requirements_for_ev_batteries

    setup

    Provides initial conditions for cum_materials_requirements_for_ev_batteries function
    """
    
    ret = initial_cumulated_material_requirements_for_ev_batteries_x1995x()
    #test_nans(ret, 2076)
    return ret


@cache('step')
def _dcum_materials_requirements_for_ev_batteries_dt():
    """
    Implicit

    See docs for cum_materials_requirements_for_ev_batteries

    component

    Provides derivative for cum_materials_requirements_for_ev_batteries function
    """
    
    ret = total_materials_required_for_ev_batteries()
    #test_nans(ret, 2077)
    return ret



integ_cum_materials_requirements_for_ev_batteries = functions.Integ(
    lambda: _dcum_materials_requirements_for_ev_batteries_dt(),
    lambda: _init_cum_materials_requirements_for_ev_batteries())


def _init_cum_materials_to_extract_for_ev_batteries_from_x2015x():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries_from_x2015x

    setup

    Provides initial conditions for cum_materials_to_extract_for_ev_batteries_from_x2015x function
    """
    
    ret = initial_cumulated_material_requirements_for_ev_batteries_x1995x()
    #test_nans(ret, 2078)
    return ret


@cache('step')
def _dcum_materials_to_extract_for_ev_batteries_from_x2015x_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_ev_batteries_from_x2015x

    component

    Provides derivative for cum_materials_to_extract_for_ev_batteries_from_x2015x function
    """
    
    ret = total_materials_to_extract_for_ev_batteries_from_x2015x_mt()
    #test_nans(ret, 2079)
    return ret



integ_cum_materials_to_extract_for_ev_batteries_from_x2015x = functions.Integ(
    lambda: _dcum_materials_to_extract_for_ev_batteries_from_x2015x_dt(),
    lambda: _init_cum_materials_to_extract_for_ev_batteries_from_x2015x())

delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_139_time_step__time_step = functions.Delay(
    lambda: pes_fossil_fuel_extraction().loc['liquids'].values, lambda: round(time_step() / time_step()) * time_step(),
    lambda: 139, lambda: time_step() / time_step())

delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_101_time_step__time_step = functions.Delay(
    lambda: pes_fossil_fuel_extraction().loc['solids'].values, lambda: round(time_step() / time_step()) * time_step(),
    lambda: 101, lambda: time_step() / time_step())

delay_pes_fossil_fuel_extraction_roundtime_step__time_step___time_step_79_time_step__time_step = functions.Delay(
    lambda: pes_fossil_fuel_extraction().loc['gases'].values, lambda: round(time_step() / time_step()) * time_step(),
    lambda: 79, lambda: time_step() / time_step())

delay_tfes_intensity_ej_tdollar_round1__time_step___time_step_8827_1__time_step = functions.Delay(
    lambda: tfes_intensity_ej_tdollar(), lambda: round(1 / time_step()) * time_step(),
    lambda: 8.827, lambda: 1 / time_step())

delay_tpes_intensity_ej_tdollar_round1__time_step___time_step_1314_1__time_step = functions.Delay(
    lambda: tpes_intensity_ej_tdollar(), lambda: round(1 / time_step()) * time_step(),
    lambda: 13.14, lambda: 1 / time_step())

integ_scarcity_conv_gas_stock = functions.Integ(lambda: increase_scarcity_conv_gas(),
                                                lambda: 0.2502)

integ_ctl_potential_production = functions.Integ(
    lambda: replacement_ctl() + variation_ctl() - wear_ctl(), lambda: initial_ctl_production())

integ_gtl_potential_production = functions.Integ(
    lambda: replacement_gtl() + variation_gtl() - wear_gtl(), lambda: initial_gtl_production())

integ_abundance_unconv_oil_stock = functions.Integ(lambda: increase_abundance_unconv_oil(),
                                                   lambda: 1)

delay_abundance_unconv_oil_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: abundance_unconv_oil(), lambda: round(1 / time_step()) * time_step(), lambda: 1,
    lambda: 1 / time_step())

integ_scarcity_conv_oil_stock = functions.Integ(lambda: increase_scarcity_conv_oil(),
                                                lambda: 0.3989)

integ_share_variable_res_elec_generation_vs_total_gen = functions.Integ(
    lambda: increase_variable_res_share_elec_vs_total_generation(),
    lambda: initial_share_variable_res_elec_gen_vs_total())

integ_cumulated_invest_e_grid = functions.Integ(
    lambda: extra_monet_invest_to_cope_with_variable_elec_res(), lambda: 0)

integ_labour_share = functions.Integ(lambda: variation_labour_share(), lambda: 0.6374)


def _init_demand_by_sector_fd():
    """
    Implicit

    See docs for demand_by_sector_fd

    setup

    Provides initial conditions for demand_by_sector_fd function
    """
    
    ret = initial_demand_by_sectot()
    #test_nans(ret, 2080)
    return ret


@cache('step')
def _ddemand_by_sector_fd_dt():
    """
    Implicit

    See docs for demand_by_sector_fd

    component

    Provides derivative for demand_by_sector_fd function
    """
    
    ret = variation_demand_flow_fd() - demand_not_covered_by_sector_fd()
    #test_nans(ret, 2081)
    return ret



integ_demand_by_sector_fd = functions.Integ(lambda: _ddemand_by_sector_fd_dt(),
                                            lambda: _init_demand_by_sector_fd())


def _init_gross_fixed_capital_formation():
    """
    Implicit

    See docs for gross_fixed_capital_formation

    setup

    Provides initial conditions for gross_fixed_capital_formation function
    """
    
    ret = initial_gfcf()
    #test_nans(ret, 2082)
    return ret


@cache('step')
def _dgross_fixed_capital_formation_dt():
    """
    Implicit

    See docs for gross_fixed_capital_formation

    component

    Provides derivative for gross_fixed_capital_formation function
    """
    
    ret = variation_gfcf() - gfcf_not_covered()
    #test_nans(ret, 2083)
    return ret



integ_gross_fixed_capital_formation = functions.Integ(
    lambda: _dgross_fixed_capital_formation_dt(), lambda: _init_gross_fixed_capital_formation())


def _init_household_demand():
    """
    Implicit

    See docs for household_demand

    setup

    Provides initial conditions for household_demand function
    """
    
    ret = initial_household_demand()
    #test_nans(ret, 2084)
    return ret


@cache('step')
def _dhousehold_demand_dt():
    """
    Implicit

    See docs for household_demand

    component

    Provides derivative for household_demand function
    """
    
    ret = variation_household_demand() - household_demand_not_covered()
    #test_nans(ret, 2085)
    return ret



integ_household_demand = functions.Integ(lambda: _dhousehold_demand_dt(),
                                         lambda: _init_household_demand())

delay_quality_of_electricity_x2015x_round003125__time_step___time_step_075_003125__time_step = functions.Delay(
    lambda: quality_of_electricity_x2015x(), lambda: round(0.03125 / time_step()) * time_step(),
    lambda: 0.75, lambda: 0.03125 / time_step())

delay_sharexdashxe_losses_cc_until_x2015_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: sharexdashxe_losses_cc_until_x2015(),
    lambda: round(time_step() / time_step()) * time_step(), lambda: 0,
    lambda: time_step() / time_step())

integ_cc_total = functions.Integ(lambda: variation_cc() - cc_total_not_covered(),
                                 lambda: initial_cc_total())

integ_lc = functions.Integ(lambda: variation_lc() - lc_not_covered(), lambda: initial_lc_total())


def _init_cum_materials_to_extract_rest():
    """
    Implicit

    See docs for cum_materials_to_extract_rest

    setup

    Provides initial conditions for cum_materials_to_extract_rest function
    """
    
    ret = initial_cumulated_material_requirements_for_rest_x1995x()
    #test_nans(ret, 2086)
    return ret


@cache('step')
def _dcum_materials_to_extract_rest_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_rest

    component

    Provides derivative for cum_materials_to_extract_rest function
    """
    
    ret = materials_to_extract_rest_mt()
    #test_nans(ret, 2087)
    return ret



integ_cum_materials_to_extract_rest = functions.Integ(
    lambda: _dcum_materials_to_extract_rest_dt(), lambda: _init_cum_materials_to_extract_rest())


def _init_cum_materials_to_extract_rest_from_x2015x():
    """
    Implicit

    See docs for cum_materials_to_extract_rest_from_x2015x

    setup

    Provides initial conditions for cum_materials_to_extract_rest_from_x2015x function
    """
    
    ret = initial_cumulated_material_requirements_for_rest_x1995x()
    #test_nans(ret, 2088)
    return ret


@cache('step')
def _dcum_materials_to_extract_rest_from_x2015x_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_rest_from_x2015x

    component

    Provides derivative for cum_materials_to_extract_rest_from_x2015x function
    """
    
    ret = materials_to_extract_rest_from_x2015x_mt()
    #test_nans(ret, 2089)
    return ret



integ_cum_materials_to_extract_rest_from_x2015x = functions.Integ(
    lambda: _dcum_materials_to_extract_rest_from_x2015x_dt(),
    lambda: _init_cum_materials_to_extract_rest_from_x2015x())


def _init_cum_materials_to_extract_for_res_elec_from_x2015x():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec_from_x2015x

    setup

    Provides initial conditions for cum_materials_to_extract_for_res_elec_from_x2015x function
    """
    
    ret = initial_cumulated_material_requirements_for_res_elec_x1995x()
    #test_nans(ret, 2090)
    return ret


@cache('step')
def _dcum_materials_to_extract_for_res_elec_from_x2015x_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec_from_x2015x

    component

    Provides derivative for cum_materials_to_extract_for_res_elec_from_x2015x function
    """
    
    ret = total_materials_to_extract_for_res_elec_from_x2015x_mt()
    #test_nans(ret, 2091)
    return ret



integ_cum_materials_to_extract_for_res_elec_from_x2015x = functions.Integ(
    lambda: _dcum_materials_to_extract_for_res_elec_from_x2015x_dt(),
    lambda: _init_cum_materials_to_extract_for_res_elec_from_x2015x())


def _init_cum_materials_to_extract_for_res_elec():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec

    setup

    Provides initial conditions for cum_materials_to_extract_for_res_elec function
    """
    
    ret = initial_cumulated_material_requirements_for_res_elec_x1995x()
    #test_nans(ret, 2092)
    return ret


@cache('step')
def _dcum_materials_to_extract_for_res_elec_dt():
    """
    Implicit

    See docs for cum_materials_to_extract_for_res_elec

    component

    Provides derivative for cum_materials_to_extract_for_res_elec function
    """
    
    ret = total_materials_to_extract_for_res_elec_mt()
    #test_nans(ret, 2093)
    return ret



integ_cum_materials_to_extract_for_res_elec = functions.Integ(
    lambda: _dcum_materials_to_extract_for_res_elec_dt(),
    lambda: _init_cum_materials_to_extract_for_res_elec())

integ_pes_waste_ej = functions.Integ(lambda: new_waste_supply_ej(), lambda: initial_pes_waste())


def _init_installed_capacity_res_elec_tw():
    """
    Implicit

    See docs for installed_capacity_res_elec_tw

    setup

    Provides initial conditions for installed_capacity_res_elec_tw function
    """
    
    ret = initial_instal_cap_res_elec()
    #test_nans(ret, 2094)
    return ret


@cache('step')
def _dinstalled_capacity_res_elec_tw_dt():
    """
    Implicit

    See docs for installed_capacity_res_elec_tw

    component

    Provides derivative for installed_capacity_res_elec_tw function
    """
    
    ret = res_elec_capacity_under_construction_tw() - wear_res_elec()
    #test_nans(ret, 2095)
    return ret



integ_installed_capacity_res_elec_tw = functions.Integ(
    lambda: _dinstalled_capacity_res_elec_tw_dt(), lambda: _init_installed_capacity_res_elec_tw())


def _init_required_capacity_res_elec_tw():
    """
    Implicit

    See docs for required_capacity_res_elec_tw

    setup

    Provides initial conditions for required_capacity_res_elec_tw function
    """
    
    ret = initial_required_capacity_res_elec()
    #test_nans(ret, 2096)
    return ret


@cache('step')
def _drequired_capacity_res_elec_tw_dt():
    """
    Implicit

    See docs for required_capacity_res_elec_tw

    component

    Provides derivative for required_capacity_res_elec_tw function
    """
    
    ret = new_required_capacity_res_elec() - new_res_elec_capacity_under_planning()
    #test_nans(ret, 2097)
    return ret



integ_required_capacity_res_elec_tw = functions.Integ(
    lambda: _drequired_capacity_res_elec_tw_dt(), lambda: _init_required_capacity_res_elec_tw())


def _init_res_elec_planned_capacity_tw():
    """
    Implicit

    See docs for res_elec_planned_capacity_tw

    setup

    Provides initial conditions for res_elec_planned_capacity_tw function
    """
    
    ret = initial_capacity_in_construction_res_elec()
    #test_nans(ret, 2098)
    return ret


@cache('step')
def _dres_elec_planned_capacity_tw_dt():
    """
    Implicit

    See docs for res_elec_planned_capacity_tw

    component

    Provides derivative for res_elec_planned_capacity_tw function
    """
    
    ret = new_res_elec_capacity_under_planning() + replacement_capacity_res_elec(
    ) - res_elec_capacity_under_construction_tw()
    #test_nans(ret, 2099)
    return ret



integ_res_elec_planned_capacity_tw = functions.Integ(lambda: _dres_elec_planned_capacity_tw_dt(),
                                                     lambda: _init_res_elec_planned_capacity_tw())

delay_scarcity_conv_gas_round1__time_step___time_step_02502_1__time_step = functions.Delay(
    lambda: scarcity_conv_gas(), lambda: round(1 / time_step()) * time_step(), lambda: 0.2502,
    lambda: 1 / time_step())

delay_scarcity_conv_oil_round1__time_step___time_step_03989_1__time_step = functions.Delay(
    lambda: scarcity_conv_oil(), lambda: round(1 / time_step()) * time_step(), lambda: 0.3989,
    lambda: 1 / time_step())


integ_rurr_tot_agg_gas = functions.Integ(lambda: -extraction_tot_agg_gas_ej()-flow_tot_agg_gas_left_in_ground(), lambda: functions.if_then_else(abs(separate_conv_and_unconv_gasxquestx() - 0) < equality_threshold,urr_tot_agg_gas()-cumulated_tot_agg_gas_extraction_to_x1995x(),0))

integ_total_agg_gas_left_in_ground = functions.Integ(lambda: flow_tot_agg_gas_left_in_ground(),
                                                     lambda: 0)

integ_cumulated_tot_agg_gas_extraction = functions.Integ(
    lambda: extraction_tot_agg_gas_ej(), lambda: cumulated_tot_agg_gas_extraction_to_x1995x())


integ_rurr_tot_agg_oil = functions.Integ(lambda: -extraction_tot_agg_oil_ej()-flow_tot_agg_oil_left_in_ground(), lambda: functions.if_then_else(abs(separate_conv_and_unconv_oilxquestx() - 0) < equality_threshold,urr_tot_agg_oil()-cumulated_tot_agg_extraction_to_x1995x(),0))

integ_total_agg_oil_left_in_ground = functions.Integ(lambda: flow_tot_agg_oil_left_in_ground(),
                                                     lambda: 0)

integ_cumulated_tot_agg_oil_extraction = functions.Integ(
    lambda: extraction_tot_agg_oil_ej(), lambda: cumulated_tot_agg_extraction_to_x1995x())

integ_total_unconv_gas_left_in_ground = functions.Integ(lambda: flow_unconv_gas_left_in_ground(),
                                                        lambda: 0)

integ_total_unconv_oil_left_in_ground = functions.Integ(lambda: flow_unconv_oil_left_in_ground(),
                                                        lambda: 0)

integ_total_coal_left_in_ground = functions.Integ(lambda: flow_coal_left_in_ground(), lambda: 0)

integ_total_conv_gas_left_in_ground = functions.Integ(lambda: flow_conv_gas_left_in_ground(),
                                                      lambda: 0)

integ_total_conv_oil_left_in_ground = functions.Integ(lambda: flow_conv_oil_left_in_ground(),
                                                      lambda: 0)

integ_rurr_coal = functions.Integ(lambda: -extraction_coal_ej() - flow_coal_left_in_ground(),
                                  lambda: urr_coal() - cumulated_coal_extraction_to_x1995x())

integ_pes_biogas_ej = functions.Integ(lambda: new_pes_biogas(), lambda: 0.13135)

integ_share_transmxandxdistr_elec_losses = functions.Integ(
    lambda: variation_share_transmxandxdistr_elec_losses(),
    lambda: share_transmxandxdistr_elec_losses_initial())


def _init_cum_materials_requirements_for_res_elec():
    """
    Implicit

    See docs for cum_materials_requirements_for_res_elec

    setup

    Provides initial conditions for cum_materials_requirements_for_res_elec function
    """
    
    ret = initial_cumulated_material_requirements_for_res_elec_x1995x()
    #test_nans(ret, 2100)
    return ret


@cache('step')
def _dcum_materials_requirements_for_res_elec_dt():
    """
    Implicit

    See docs for cum_materials_requirements_for_res_elec

    component

    Provides derivative for cum_materials_requirements_for_res_elec function
    """
    
    ret = total_materials_required_for_res_elec_mt()
    #test_nans(ret, 2101)
    return ret



integ_cum_materials_requirements_for_res_elec = functions.Integ(
    lambda: _dcum_materials_requirements_for_res_elec_dt(),
    lambda: _init_cum_materials_requirements_for_res_elec())


def _init_installed_capacity_res_heatxdashxcom_tw():
    """
    Implicit

    See docs for installed_capacity_res_heatxdashxcom_tw

    setup

    Provides initial conditions for installed_capacity_res_heatxdashxcom_tw function
    """
    
    ret = initial_value_res_for_heatxdashxcom()
    #test_nans(ret, 2102)
    return ret


@cache('step')
def _dinstalled_capacity_res_heatxdashxcom_tw_dt():
    """
    Implicit

    See docs for installed_capacity_res_heatxdashxcom_tw

    component

    Provides derivative for installed_capacity_res_heatxdashxcom_tw function
    """
    
    ret = new_res_capacity_for_heatxdashxcom_tw() + replacement_res_for_heatxdashxcom_tw(
    ) - wear_res_capacity_for_heatxdashxcom_tw()
    #test_nans(ret, 2103)
    return ret



integ_installed_capacity_res_heatxdashxcom_tw = functions.Integ(
    lambda: _dinstalled_capacity_res_heatxdashxcom_tw_dt(),
    lambda: _init_installed_capacity_res_heatxdashxcom_tw())


integ_installed_capacity_nuclear_tw = functions.Integ(lambda: nuclear_capacity_under_construction()-nuclear_capacity_phasexdashxout()-wear_nuclear(), lambda: initial_capacity_installed_nuclear())

integ_required_capacity_nuclear_tw = functions.Integ(
    lambda: new_required_capacity_nuclear() - new_nuclear_capacity_under_planning(),
    lambda: initial_required_capacity_nuclear())


integ_planned_nuclear_capacity_tw = functions.Integ(lambda: new_nuclear_capacity_under_planning()+replacement_nuclear_capacity()-nuclear_capacity_under_construction(), lambda: initial_capacity_in_construction_nuclear())

delay_gdp_round1__time_step___time_step_2916_1__time_step = functions.Delay(
    lambda: gdp(), lambda: round(1 / time_step()) * time_step(), lambda: 29.16,
    lambda: 1 / time_step())


integ_land_compet_biofuels_2gen_mha = functions.Integ(lambda: new_biofuels_2gen_land_compet()-land_shifted_to_biofuels_3gen(), lambda: initial_value_land_compet_biofuels_2gen_mha()*land_productivity_biofuels_2gen_ej_mha())

delay_constrain_gas_exogenous_growthxquestx_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: constrain_gas_exogenous_growthxquestx(), lambda: round(1 / time_step()) * time_step(),
    lambda: 1, lambda: 1 / time_step())

delay_constrain_liquids_exogenous_growthxquestx_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: constrain_liquids_exogenous_growthxquestx(),
    lambda: round(1 / time_step()) * time_step(), lambda: 1, lambda: 1 / time_step())

delay_check_liquids_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: check_liquids(), lambda: round(1 / time_step()) * time_step(), lambda: 1,
    lambda: 1 / time_step())

delay_check_gases_round1__time_step___time_step_1_1__time_step = functions.Delay(
    lambda: check_gases(), lambda: round(1 / time_step()) * time_step(), lambda: 1,
    lambda: 1 / time_step())

delay_share_variable_res_elec_generation_vs_total_round1__time_step___time_step_00071_1__time_step = functions.Delay(
    lambda: share_variable_res_elec_generation_vs_total(),
    lambda: round(1 / time_step()) * time_step(), lambda: 0.0071, lambda: 1 / time_step())

integ_efficiency_gas_for_electricity = functions.Integ(
    lambda: improvement_efficiency_gas_for_electricity(),
    lambda: initial_efficiency_gas_for_electricity() * percent_to_share())

integ_cumulated_conv_gas_extraction = functions.Integ(
    lambda: extraction_conv_gas_ej(), lambda: cumulated_conv_gas_extraction_to_x1995x())

integ_cumulated_unconv_gas_extraction = functions.Integ(
    lambda: extraction_unconv_gas_ej(), lambda: cumulated_unconv_gas_extraction_to_x1995x())

integ_rurr_uranium = functions.Integ(
    lambda: -extraction_uranium_ej(),
    lambda: urr_uranium() - cumulated_uranium_extraction_to_x1995x())

integ_cumulated_conv_oil_extraction = functions.Integ(
    lambda: extraction_conv_oil_ej(), lambda: cumulated_conv_oil_extraction_to_x1995x())

integ_cumulated_unconv_oil_extraction = functions.Integ(
    lambda: extraction_unconv_oil_ej(), lambda: cumulated_unconv_oil_extraction_to_x1995x())

integ_cumulated_coal_extraction = functions.Integ(lambda: extraction_coal_ej(),
                                                  lambda: cumulated_coal_extraction_to_x1995x())

integ_cumulated_uranium_extraction = functions.Integ(
    lambda: extraction_uranium_ej(), lambda: cumulated_uranium_extraction_to_x1995x())

integ_land_compet_biofuels_3gen_mha = functions.Integ(lambda: land_shifted_to_biofuels_3gen(),
                                                      lambda: 0)

integ_potential_pe_cellulosic_biofuel_ej = functions.Integ(lambda: new_cellulosic_biofuels(),
                                                           lambda: 0)

integ_pe_bioe_residues_for_heatxplusxelec_ej = functions.Integ(
    lambda: new_bioe_residues_for_heatxplusxelec(), lambda: 0)

integ_population = functions.Integ(lambda: pop_variation(), lambda: initial_population())

delay_extraction_unconv_gas_ej_roundtime_step__time_step___time_step_0_time_step__time_step = functions.Delay(
    lambda: extraction_unconv_gas_ej(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: 0, lambda: time_step() / time_step())

integ_cumulated_total_monet_invest_res_for_elec = functions.Integ(
    lambda: total_monet_invest_res_for_elec_tdolar(), lambda: 0)

integ_total_cumulative_emissions_gtc = functions.Integ(lambda: new_c_gtc(),
                                                       lambda: cumulative_emissions_to_x1995x())


def _init_evol_final_energy_intensity_by_sector_and_fuel():
    """
    Implicit

    See docs for evol_final_energy_intensity_by_sector_and_fuel

    setup

    Provides initial conditions for evol_final_energy_intensity_by_sector_and_fuel function
    """
    
    ret = initial_energy_intensity_by_fuel_and_sector_x1995x()
    #test_nans(ret, 2104)
    return ret


@cache('step')
def _devol_final_energy_intensity_by_sector_and_fuel_dt():
    """
    Implicit

    See docs for evol_final_energy_intensity_by_sector_and_fuel

    component

    Provides derivative for evol_final_energy_intensity_by_sector_and_fuel function
    """
    
    ret = increase_of_intensity_due_to_energy_a_technology_change_top_down(
    ) + inertial_rate_energy_intensity_top_down() + rate_change_intensity_bottom_up(
    ) - decrease_of_intensity_due_to_energy_a_technology_change_top_down()
    #test_nans(ret, 2105)
    return ret



integ_evol_final_energy_intensity_by_sector_and_fuel = functions.Integ(
    lambda: _devol_final_energy_intensity_by_sector_and_fuel_dt(),
    lambda: _init_evol_final_energy_intensity_by_sector_and_fuel())


integ_rurr_unconv_oil_ej = functions.Integ(lambda: -extraction_unconv_oil_ej()-flow_unconv_oil_left_in_ground(), lambda: urr_unconv_oil()-cumulated_unconv_oil_extraction_to_x1995x()*separate_conv_and_unconv_oilxquestx())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___agriculture_hunting_forestry_and_fishing = functions.Delay(
    lambda: real_demand_by_sector().loc['agriculture hunting forestry and fishing'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___mining_and_quarrying = functions.Delay(
    lambda: real_demand_by_sector().loc['mining and quarrying'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___food_beverages_and_tobacco = functions.Delay(
    lambda: real_demand_by_sector().loc['food beverages and tobacco'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___textiles_and_textile_products = functions.Delay(
    lambda: real_demand_by_sector().loc['textiles and textile products'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___leather_leather_and_footwear = functions.Delay(
    lambda: real_demand_by_sector().loc['leather leather and footwear'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___wood_and_products_of_woood_and_cork = functions.Delay(
    lambda: real_demand_by_sector().loc['wood and products of woood and cork'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___pulp_paper_printing_and_publishing = functions.Delay(
    lambda: real_demand_by_sector().loc['pulp paper printing and publishing'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___coke_refined_petroleum_and_nuclear_fuel = functions.Delay(
    lambda: real_demand_by_sector().loc['coke refined petroleum and nuclear fuel'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___chemicals_and_chemical_products = functions.Delay(
    lambda: real_demand_by_sector().loc['chemicals and chemical products'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___rubber_and_plastics = functions.Delay(
    lambda: real_demand_by_sector().loc['rubber and plastics'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___other_non_metalic_mineral = functions.Delay(
    lambda: real_demand_by_sector().loc['other non metalic mineral'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___basic_metals_and_fabricated_metal = functions.Delay(
    lambda: real_demand_by_sector().loc['basic metals and fabricated metal'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___machinery_nec = functions.Delay(
    lambda: real_demand_by_sector().loc['machinery nec'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___electrical_and_optical_equipment = functions.Delay(
    lambda: real_demand_by_sector().loc['electrical and optical equipment'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___transport_equipment = functions.Delay(
    lambda: real_demand_by_sector().loc['transport equipment'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___manufacturing_nec_recycling = functions.Delay(
    lambda: real_demand_by_sector().loc['manufacturing nec recycling'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___electricity_gas_and_water_supply = functions.Delay(
    lambda: real_demand_by_sector().loc['electricity gas and water supply'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___construction = functions.Delay(
    lambda: real_demand_by_sector().loc['construction'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel = functions.Delay(
    lambda: real_demand_by_sector().loc['sale maintenance and repair of motor vehicles anda motorcycles retail sale of fuel'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles = functions.Delay(
    lambda: real_demand_by_sector().loc['wholesale trade and commissions trade except of motor vehicles and motorcycles'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods = functions.Delay(
    lambda: real_demand_by_sector().loc['retail trade except of motor vehicles and motorcycles repair of household goods'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___hotels_and_restaurants = functions.Delay(
    lambda: real_demand_by_sector().loc['hotels and restaurants'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___inland_transport = functions.Delay(
    lambda: real_demand_by_sector().loc['inland transport'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___water_transport = functions.Delay(
    lambda: real_demand_by_sector().loc['water transport'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___air_transport = functions.Delay(
    lambda: real_demand_by_sector().loc['air transport'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies = functions.Delay(
    lambda: real_demand_by_sector().loc['other supporting and auxiliary transport activities activities of travel agencies'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___post_and_telecommunications = functions.Delay(
    lambda: real_demand_by_sector().loc['post and telecommunications'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___financial_intermedation = functions.Delay(
    lambda: real_demand_by_sector().loc['financial intermedation'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___real_estate_activities = functions.Delay(
    lambda: real_demand_by_sector().loc['real estate activities'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___renting_od_meq_and_other_business_activities = functions.Delay(
    lambda: real_demand_by_sector().loc['renting od meq and other business activities'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___public_admin_and_defence_compulsory_social_security = functions.Delay(
    lambda: real_demand_by_sector().loc['public admin and defence compulsory social security'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___education = functions.Delay(
    lambda: real_demand_by_sector().loc['education'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___health_and_social_work = functions.Delay(
    lambda: real_demand_by_sector().loc['health and social work'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___other_community_social_and_persona_services = functions.Delay(
    lambda: real_demand_by_sector().loc['other community social and persona services'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___private_households_with_employed_persons = functions.Delay(
    lambda: real_demand_by_sector().loc['private households with employed persons'].values, lambda: round(1 / time_step()) * time_step(), lambda: 10,
    lambda: 1 / time_step())


delays_real_demand_by_sector_round1__time_step___time_step_10_1__time_step = [
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___agriculture_hunting_forestry_and_fishing,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___mining_and_quarrying,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___food_beverages_and_tobacco,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___textiles_and_textile_products,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___leather_leather_and_footwear,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___wood_and_products_of_woood_and_cork,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___pulp_paper_printing_and_publishing,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___coke_refined_petroleum_and_nuclear_fuel,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___chemicals_and_chemical_products,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___rubber_and_plastics,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___other_non_metalic_mineral,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___basic_metals_and_fabricated_metal,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___machinery_nec,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___electrical_and_optical_equipment,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___transport_equipment,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___manufacturing_nec_recycling,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___electricity_gas_and_water_supply,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___construction,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___sale_maintenance_and_repair_of_motor_vehicles_anda_motorcycles_retail_sale_of_fuel,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___wholesale_trade_and_commissions_trade_except_of_motor_vehicles_and_motorcycles,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___retail_trade_except_of_motor_vehicles_and_motorcycles_repair_of_household_goods,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___hotels_and_restaurants,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___inland_transport,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___water_transport,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___air_transport,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___other_supporting_and_auxiliary_transport_activities_activities_of_travel_agencies,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___post_and_telecommunications,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___financial_intermedation,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___real_estate_activities,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___renting_od_meq_and_other_business_activities,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___public_admin_and_defence_compulsory_social_security,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___education,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___health_and_social_work,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___other_community_social_and_persona_services,
    delay_real_demand_by_sector_round1__time_step___time_step_10_1__time_step___private_households_with_employed_persons]


delay_extraction_unconv_oil_ej_roundtime_step__time_step___time_step_109_time_step__time_step = functions.Delay(
    lambda: extraction_unconv_oil_ej(), lambda: round(time_step() / time_step()) * time_step(),
    lambda: 1.09, lambda: time_step() / time_step())
